/*
 AVKit.h
 
 Bernie Wylde 20180312
 */

#import <AppKit/AppKit.h>
#import <AVKit/AVKit.h>
#import <AVFoundation/AVFoundation.h>

#import "CocoaUI.h"

#if __LP64__
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#ifdef AVKIT_EXTERN

typedef AVAsset *AVAssetRef;
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
typedef AVAssetCache *AVAssetCacheRef;
#endif
typedef AVAssetResourceLoader *AVAssetResourceLoaderRef;
typedef AVAssetTrack *AVAssetTrackRef;
typedef AVAssetTrackGroup *AVAssetTrackGroupRef;
typedef AVAssetTrackSegment *AVAssetTrackSegmentRef;
typedef AVCompositionTrack *AVCompositionTrackRef;
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
typedef AVMediaSelection *AVMediaSelectionRef;
#endif
typedef AVMetadataItem *AVMetadataItemRef;
typedef AVPlayer *AVPlayerRef;
typedef AVPlayerItem *AVPlayerItemRef;
typedef AVPlayerLayer *AVPlayerLayerRef;
typedef AVPlayerView *AVPlayerViewRef;
typedef AVSampleCursor *AVSampleCursorRef;
typedef AVURLAsset *AVURLAssetRef;


#pragma mark - AVAsset
///////////////////////////////////////////////////
//                   AVAsset                     //
///////////////////////////////////////////////////
// Create
AVAssetRef AVAssetWithURL( CFURLRef url );

// Load data
void AVAssetCancelLoading( AVAssetRef ref );

// Inspect
CMTime AVAssetDuration( AVAssetRef ref );
Boolean AVAssetProvidesPreciseDurationAndTiming( AVAssetRef ref );
AVMetadataItemRef AVAssetCreationDate( AVAssetRef ref );
float AVAssetPreferredRate( AVAssetRef ref );
CGAffineTransform AVAssetPreferredTransform( AVAssetRef ref );
float AVAssetPreferredVolume( AVAssetRef ref );

// Usability
Boolean AVAssetIsPlayable( AVAssetRef ref );
Boolean AVAssetIsExportable( AVAssetRef ref );
Boolean AVAssetIsReadable( AVAssetRef ref );
Boolean AVAssetIsComposable( AVAssetRef ref );
Boolean AVAssetHasProtectedContent( AVAssetRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean AVAssetIsCompatibleWithAirPlayVideo( AVAssetRef ref );
#endif

// Tracks
CFArrayRef AVAssetTracks( AVAssetRef ref );
CFArrayRef AVAssetTrackGroups( AVAssetRef ref );

AVAssetTrackRef AVAssetTrackWithID( AVAssetRef ref, CMPersistentTrackID trackID );
CFArrayRef AVAssetTracksWithMediaType( AVAssetRef ref, CFStringRef type );
CFArrayRef AVAssetTracksWithMediaCharacteristic( AVAssetRef ref, CFStringRef characteristic );
CMPersistentTrackID AVAssetUnusedTrackID( AVAssetRef ref );

// Accessing metadata
CFStringRef AVAssetLyrics( AVAssetRef ref );
CFArrayRef AVAssetMetadata( AVAssetRef ref );
CFArrayRef AVAssetCommonMetadata( AVAssetRef ref );
CFArrayRef AVAssetAvailableMetadataFormats( AVAssetRef ref );
CFArrayRef AVAssetMetadataForFormat( AVAssetRef ref, CFStringRef format );

// Chapter metadata
CFArrayRef AVAssetAvailableChapterLocales( AVAssetRef ref );
CFArrayRef AVAssetChapterMetadataGroupsWithTitleLocale( AVAssetRef ref, CFLocaleRef locale, CFArrayRef commonKeys );
CFArrayRef AVAssetChapterMetadataGroupsBestMatchingPreferredLanguages( AVAssetRef ref, CFArrayRef languages );

// Find media selections
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
AVMediaSelectionRef AVAssetPreferredMediaSelection( AVAssetRef ref );
#endif
CFArrayRef AVAssetAvailableMediaCharacteristicsWithMediaSelectionOptions( AVAssetRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFArrayRef AVAssetAllMediaSelections( AVAssetRef ref );
#endif

// Reference restrictions
AVAssetReferenceRestrictions avAssetReferenceRestrictions( AVAssetRef ref );

// Fragmented movies
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean AVAssetCanContainFragments( AVAssetRef ref );
Boolean AVAssetContainsFragments( AVAssetRef ref );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
CMTime AVAssetOverallDurationHint( AVAssetRef ref );
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - AVAssetCache
///////////////////////////////////////////////////
//                AVAssetCache                   //
///////////////////////////////////////////////////
Boolean AVAssetCacheIsPlayableOffline( AVAssetCacheRef ref );
//CFArrayRef AVAssetCacheMediaSelectionOptionsInMediaSelectionGroup( AVAssetCacheRef ref );

#endif



#pragma mark - AVAssetResourceLoader
///////////////////////////////////////////////////
//           AVAssetResourceLoader               //
///////////////////////////////////////////////////



#pragma mark - AVAssetTrack
///////////////////////////////////////////////////
//                 AVAssetTrack                  //
///////////////////////////////////////////////////
// Info
AVAssetRef AVAssetTrackAsset( AVAssetTrackRef ref );
CMPersistentTrackID AVAssetTrackID( AVAssetTrackRef ref );
CFStringRef AVAssetTrackMediaType( AVAssetTrackRef ref );
Boolean AVAssetTrackHasMediaCharacteristic( AVAssetTrackRef ref, CFStringRef characteristic );
CFArrayRef AVAssetTrackFormatDescriptions( AVAssetTrackRef ref );
Boolean AVAssetTrackIsEnabled( AVAssetTrackRef ref );
Boolean AVAssetTrackIsPlayable( AVAssetTrackRef ref );
Boolean AVAssetTrackIsSelfContained( AVAssetTrackRef ref );
float AVAssetTrackEstimatedDataRate( AVAssetTrackRef ref );
long long AVAssetTrackTotalSampleDataLength( AVAssetTrackRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean AVAssetTrackIsDecodable( AVAssetTrackRef ref );
#endif

// Temporal properties
CMTimeRange AVAssetTrackTimeRange( AVAssetTrackRef ref );
CMTimeScale AVAssetTrackNaturalTimeScale( AVAssetTrackRef ref );

// Language properties
CFStringRef AVAssetTrackLanguageCode( AVAssetTrackRef ref );
CFStringRef AVAssetTrackExtendedLanguageTag( AVAssetTrackRef ref );

// Visual characteristics
CGSize AVAssetTrackNaturalSize( AVAssetTrackRef ref );
CGAffineTransform AVAssetTrackPreferredTransform( AVAssetTrackRef ref );

// Audible characteristics
float AVAssetTrackPreferredVolume( AVAssetTrackRef ref );

// Frame-based characteristics
float AVAssetTrackNominalFrameRate( AVAssetTrackRef ref );
CMTime AVAssetTrackMinFrameDuration( AVAssetTrackRef ref );
Boolean AVAssetTrackRequiresFrameReordering( AVAssetTrackRef ref );

// Track segments
CFArrayRef AVAssetTrackSegments( AVAssetTrackRef ref );
AVAssetTrackSegmentRef AVAssetTrackSegmentForTrackTime( AVAssetTrackRef ref, CMTime time );
CMTime AVAssetTrackSamplePresentationTimeForTrackTime( AVAssetTrackRef ref, CMTime time );

// Metadata
CFArrayRef AVAssetTrackMetadata( AVAssetTrackRef ref );
CFArrayRef AVAssetTrackCommonMetadata( AVAssetTrackRef ref );
CFArrayRef AVAssetTrackAvailableMetadataFormats( AVAssetTrackRef ref );
CFArrayRef AVAssetTrackMetadataForFormat( AVAssetTrackRef ref, CFStringRef format );

// Associated tracks
CFArrayRef AVAssetTrackAvailableTrackAssociationTypes( AVAssetTrackRef ref );
CFArrayRef AVAssetTrackAssociatedTracksOfType( AVAssetTrackRef ref, CFStringRef type );

// Simple cursors
Boolean AVAssetTrackCanProvideSampleCursors( AVAssetTrackRef ref );
AVSampleCursorRef AVAssetTrackMakeSampleCursorWithPresentationTimeStamp( AVAssetTrackRef ref, CMTime time );
AVSampleCursorRef AVAssetTrackMakeSampleCursorAtFirstSampleInDecodeOrder( AVAssetTrackRef ref );
AVSampleCursorRef AVAssetTrackMakeSampleCursorAtLastSampleInDecodeOrder( AVAssetTrackRef ref );




#pragma mark - AVAssetTrackGroup
///////////////////////////////////////////////////
//              AVAssetTrackGroup                //
///////////////////////////////////////////////////
// Track ID values
CFArrayRef AVAssetTrackGroupTrackIDs( AVAssetTrackGroupRef ref );



#pragma mark - AVAssetTrackSegment
///////////////////////////////////////////////////
//            AVAssetTrackSegment                //
///////////////////////////////////////////////////
// Info
CMTimeMapping AVAssetTrackSegmentTimeMapping( AVAssetTrackSegmentRef ref );
Boolean AVAssetTrackSegmentIsEmpty( AVAssetTrackSegmentRef ref );


#pragma mark - AVCompositionTrack
///////////////////////////////////////////////////
//            AVCompositionTrack                 //
///////////////////////////////////////////////////



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
#pragma mark - AVMediaSelection
///////////////////////////////////////////////////
//            AVMediaSelection                   //
///////////////////////////////////////////////////


#endif


#pragma mark - AVMetadataItem
///////////////////////////////////////////////////
//            AVMetadataItem                     //
///////////////////////////////////////////////////

#pragma mark - AVPlayer
///////////////////////////////////////////////////
//                  AVPlayer                     //
///////////////////////////////////////////////////
@interface AVPlayer (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

// Init
AVPlayerRef AVPlayerWithURL( CFURLRef url );
AVPlayerRef AVPlayerWithPlayerItem( AVPlayerItemRef item );

// Playback
void AVPlayerPlay( AVPlayerRef player );
void AVPlayerPause( AVPlayerRef player );
float AVPlayerRate( AVPlayerRef player );
void AVPlayerSetRate( AVPlayerRef player, float rate );
AVPlayerActionAtItemEnd avPlayerActionAtItemEnd( AVPlayerRef player );
void AVPlayerSetActionAtItemEnd( AVPlayerRef player, AVPlayerActionAtItemEnd action );
void AVPlayerReplaceCurrentItemWithPlayerItem( AVPlayerRef player, AVPlayerItemRef item );

// Waiting behavior
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean AVPlayerAutomaticallyWaitsToMinimizeStalling( AVPlayerRef player );
void AVPlayerSetAutomaticallyWaitsToMinimizeStalling( AVPlayerRef player, Boolean flag );
CFStringRef AVPlayerReasonForWaitingToPlay( AVPlayerRef player );
AVPlayerTimeControlStatus avPlayerTimeControlStatus( AVPlayerRef player );
void AVPlayerPlayImmediatelyAtRate( AVPlayerRef player, float rate );
#endif

// Managing time
CMTime AVPlayerCurrentTime( AVPlayerRef player );
void AVPlayerSeekToTime( AVPlayerRef player, CMTime tm );
void AVPlayerSeekToDate( AVPlayerRef player, CFDateRef dt );
void AVPlayerSeekToTimeWithTolerance( AVPlayerRef player, CMTime tm, CMTime toleranceBefore, CMTime toleranceAfter );

// Observing time

// Media selection

// External playback

// Synch playback to external source

// Audio output
Boolean AVPlayerIsMuted( AVPlayerRef player );
void AVPlayerSetMuted( AVPlayerRef player, Boolean flag );
float AVPlayerVolume( AVPlayerRef player );
void AVPlayerSetVolume( AVPlayerRef player, float volume );

// Player properties
AVPlayerStatus avPlayerStatus( AVPlayerRef player );
ErrorRef AVPlayerError( AVPlayerRef player );
AVPlayerItemRef AVPlayerCurrentItem( AVPlayerRef player );

// Instance properties

// Type properties

// Convenience
Boolean AVPlayerIsPlaying( AVPlayerRef ref );




#pragma mark - AVPlayerItem
///////////////////////////////////////////////////
//               AVPlayerItem                    //
///////////////////////////////////////////////////
// Create
AVPlayerItemRef AVPlayerItemWithURL( CFURLRef url );

// Inspecting
AVAssetRef AVPlayerItemAsset( AVPlayerItemRef item );
AVPlayerItemStatus avPlayerItemStatus( AVPlayerItemRef item );
CMTime AVPlayerItemDuration( AVPlayerItemRef item );
//CMTimebaseRef AVPlayerItemTimebase( AVPlayerItemRef item );
CGSize AVPlayerItemPresentationSize( AVPlayerItemRef item );

// Moving the playhead
void AVPlayerItemStepByCount( AVPlayerItemRef item, NSInteger count );

// Playback info
Boolean AVPlayerItemCanPlayReverse( AVPlayerItemRef item );
Boolean AVPlayerItemCanPlayFastForward( AVPlayerItemRef item );
Boolean AVPlayerItemCanPlayFastReverse( AVPlayerItemRef item );
Boolean AVPlayerItemCanPlaySlowForward( AVPlayerItemRef item );
Boolean AVPlayerItemCanPlaySlowReverse( AVPlayerItemRef item );
Boolean AVPlayerItemCanStepBackward( AVPlayerItemRef item );
Boolean AVPlayerItemCanStepForward( AVPlayerItemRef item );

// Timing info
CMTime AVPlayerItemCurrentTime( AVPlayerItemRef item );
CFDateRef AVPlayerItemCurrentDate( AVPlayerItemRef item );
CMTime AVPlayerItemForwardPlaybackEndTime( AVPlayerItemRef item );
CMTime AVPlayerItemReversePlaybackEndTime( AVPlayerItemRef item );

// Network behavior

// Settings

// Logs

// Authorization

// Media options

// Text style rules

// Outputs

// Data collectors

// Metadata

// Notifications





#pragma mark - AVPlayerLayer
///////////////////////////////////////////////////
//               AVPlayerLayer                   //
///////////////////////////////////////////////////
// Create
AVPlayerLayerRef AVPlayerLayerWithPlayer( AVPlayerRef player );

// Configure
AVPlayerRef AVPlayerLayerPlayer( AVPlayerLayerRef ref );
Boolean AVPlayerLayerIsReadyForDisplay( AVPlayerLayerRef ref );
CFStringRef AVPlayerLayerVideoGravity( AVPlayerLayerRef ref );
CGRect AVPlayerLayerVideoRect( AVPlayerLayerRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFDictionaryRef AVPlayerLayerPixelBufferAttributes( AVPlayerLayerRef ref );
#endif







#pragma mark - AVPlayerView
///////////////////////////////////////////////////
//                  AVPlayerView                 //
///////////////////////////////////////////////////
@interface AVPlayerViewSC : AVPlayerView
@end

@interface AVPlayerView (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;

- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

AVPlayerView *AVPlayerViewWithTag( NSInteger tag );

// Player
AVPlayerRef AVPlayerViewPlayer( NSInteger tag );
void AVPlayerViewSetPlayer( NSInteger tag, AVPlayerRef player );

// Customization
AVPlayerViewControlsStyle avPlayerViewControlsStyle( NSInteger tag );
void AVPlayerViewSetControlsStyle( NSInteger tag, AVPlayerViewControlsStyle style );
Boolean AVPlayerViewShowsFrameSteppingButtons( NSInteger tag );
void AVPlayerViewSetShowsFrameSteppingButtons( NSInteger tag, Boolean flag );
Boolean AVPlayerViewShowsSharingServiceButton( NSInteger tag );
void AVPlayerViewSetShowsSharingServiceButton( NSInteger tag, Boolean flag );
Boolean AVPlayerViewShowsFullScreenToggleButton( NSInteger tag );
void AVPlayerViewSetShowsFullScreenToggleButton( NSInteger tag, Boolean flag );
ViewRef AVPlayerViewContentOverlayView( NSInteger tag );
CocoaMenuRef AVPlayerViewActionPopUpButtonMenu( NSInteger tag );
void AVPlayerViewSetActionPopUpButtonMenu( NSInteger tag, CocoaMenuRef m );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean AVPlayerViewUpdatesNowPlayingInfoCenter( NSInteger tag );
void AVPlayerViewSetUpdatesNowPlayingInfoCenter( NSInteger tag, Boolean flag );
#endif

// Configure video
Boolean AVPlayerViewIsReadyForDisplay( NSInteger tag );
CGRect AVPlayerViewVideoBounds( NSInteger tag );
CFStringRef AVPlayerViewVideoGravity( NSInteger tag );
void AVPlayerViewSetVideoGravity( NSInteger tag, CFStringRef gravity );

// Trimming playback content
Boolean AVPlayerViewCanBeginTrimming( NSInteger tag );
//void AVPlayerViewBeginTrimmingWithCallback( NSInteger tag, void *callback, void *userData );

// Display chapter and title
void AVPlayerViewFlashChapterNumber( NSInteger tag, NSUInteger number, CFStringRef title );


// Convenience
Boolean AVPlayerViewPlayerIsPlaying( NSInteger tag );



// Init
AVPlayerViewRef AVPlayerViewInit( NSInteger inTag, CGRect inRect );

// Statement
void CocoaAVPlayerView( NSInteger inTag, CGRect inRect, NSInteger inControlsStyle, NSInteger inWndTag );



#pragma mark - AVSampleCursor
///////////////////////////////////////////////////
//               AVSampleCursor                  //
///////////////////////////////////////////////////




#pragma mark - AVURLAsset
///////////////////////////////////////////////////
//                  AVURLAsset                   //
///////////////////////////////////////////////////
// Create
AVURLAssetRef AVURLAssetWithURL( CFURLRef url, CFDictionaryRef options );

// URL
CFURLRef AVURLAssetURL( AVURLAssetRef ref );

// Resource loading
AVAssetResourceLoaderRef AVURLAssetResourceLoader( AVURLAssetRef ref );

// Compatible tracks
AVAssetTrackRef AVURLAssetCompatibleTrackForCompositionTrack( AVURLAssetRef ref, AVCompositionTrackRef compTrack );

// Supported media types
CFArrayRef AVURLAssetAudiovisualMIMETypes( void );
CFArrayRef AVURLAssetAudiovisualTypes( void );
Boolean AVURLAssetIsPlayableExtendedMIMEType( CFStringRef type );

// Offline AVAssets
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
AVAssetCacheRef AVURLAssetCache( AVURLAssetRef ref );
#endif

// Encrypted content
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101204
Boolean AVURLAssetMayRequireContentKeysForMediaDataProcessing( AVURLAssetRef ref );
#endif


#endif// #ifdef AVKIT_EXTERN
#endif// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#endif// #if __LP64__

//#endif  // COCOAUIAVKIT_H
