/*
 CocoaUI.m
 
 Bernie Wylde 20170712
 */

#import "CocoaUI.h"

void CocoaAppRun( void )
{
    static BOOL sCocoaAppRun = NO;
    if ( !sCocoaAppRun ) {
        NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
        sCocoaAppRun = YES;
        [NSApp run];
        [pool drain];
    }
}

BOOL IsCocoaApplication( void )
{
#if __LP64__
    return YES;
#endif
    if ( [NSApp delegate] ) return YES;
    return NO;
}



// subclass flag
static BOOL sSubclassFlag;

BOOL IsSubclassFlag( void )
{
    BOOL flag = sSubclassFlag;
    sSubclassFlag = NO;
    return flag;
}

void SetSubclassFlag( BOOL flag )
{ sSubclassFlag = flag; }


// taken from Apple documentation
CGPathRef NSBezierPathToCGPath( NSBezierPath *bezierPath )
{
    int i, numElements;

    // Need to begin a path here.
    CGPathRef cgPath = NULL;

    // Then draw the path elements.
    numElements = [bezierPath elementCount];
    if ( numElements > 0 ) {
        CGMutablePathRef    mutPath = CGPathCreateMutable();
        NSPoint             points[3];
        BOOL                didClosePath = YES;

        for ( i = 0; i < numElements; i++ ) {
            switch ( [bezierPath elementAtIndex:i associatedPoints:points] ) {
                case NSMoveToBezierPathElement:
                    CGPathMoveToPoint( mutPath, NULL, points[0].x, points[0].y );
                    break;

                case NSLineToBezierPathElement:
                    CGPathAddLineToPoint( mutPath, NULL, points[0].x, points[0].y );
                    didClosePath = NO;
                    break;

                case NSCurveToBezierPathElement:
                    CGPathAddCurveToPoint( mutPath, NULL, points[0].x, points[0].y,
                                          points[1].x, points[1].y,
                                          points[2].x, points[2].y );
                    didClosePath = NO;
                    break;

                case NSClosePathBezierPathElement:
                    CGPathCloseSubpath( mutPath );
                    didClosePath = YES;
                    break;
            }
        }

        // Be sure the path is closed or Quartz may not do valid hit detection.
        if ( !didClosePath ) CGPathCloseSubpath( mutPath );

        cgPath = CGPathCreateCopy( mutPath );
        CGPathRelease( mutPath );
    }
    return cgPath;
}



#pragma mark - utilities
///////////////////////////////////////////////////
//                  utilities                    //
///////////////////////////////////////////////////
NSImage *CocoaImageResource( NSString *path )
{
    NSImage *image = [NSImage imageNamed:path];
    if ( !image ) {
        NSString *fullPath = [[NSBundle mainBundle] pathForResource:path ofType:nil];
        if ( fullPath ) {
            image = [[[NSImage alloc] initWithContentsOfFile:fullPath] autorelease];
        }
    }
    return image;
}

NSImage *FixImageValue( void *inImage )
{
    if ( inImage ) {
        if ( CFGetTypeID( inImage ) == CFStringGetTypeID() ) {
            return CocoaImageResource( inImage );
        } else if ( [(id)inImage isKindOfClass:[NSImage class]] ) {
            return inImage;
        }
    }
    return nil;
}


NSFont *FixFont( void *inFont )
{
    return (NSFont *)inFont;
}







#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1070

void CocoaShowAlert( NSAlertStyle style, NSString *msg, NSString *info )
{
    NSAlert *alert = [[NSAlert alloc] init];
    [alert setAlertStyle:style];
    [alert setMessageText:msg];
    [alert setInformativeText:info];
    [alert runModal];
    [alert release];
}

void CocoaShowWindowDoesNotExistAlert( NSInteger tag )
{ CocoaShowAlert( NSWarningAlertStyle, @"Window error.", [NSString stringWithFormat:@"Cocoa window %ld does not exist.",(long)tag] ); }




id CocoaFindViewWithTagInAnyOpenPanelAccessoryView( NSInteger tag, Class cls )
{
    if ( sCocoaOpenPanels ) {
        for ( NSOpenPanel *op in sCocoaOpenPanels ) {
            NSView *accessoryView = [op accessoryView];
            if ( accessoryView ) {
                id view = CocoaViewSubviewWithTag( accessoryView, tag );
                if ( view ) {
                    if ( [view isKindOfClass:cls] ) {
                        return view;
                    }
                }
            }
        }
    }
    return nil;
}

id CocoaFindViewWithTagInAnySavePanelAccessoryView( NSInteger tag, Class cls )
{
    if ( sCocoaSavePanels ) {
        for ( NSSavePanel *sp in sCocoaSavePanels ) {
            NSView *accessoryView = [sp accessoryView];
            if ( accessoryView ) {
                id view = CocoaViewSubviewWithTag( accessoryView, tag );
                if ( view ) {
                    if ( [view isKindOfClass:cls] ) {
                        return view;
                    }
                }
            }
        }
    }
    return nil;
}

id CocoaFindViewWithTagInWindow( NSInteger tag, Class cls, NSWindow *w )
{
    id view = CocoaViewSubviewWithTag( [w contentView], tag );
    if ( [view isKindOfClass:cls] ) {
        return view;
    }
    return nil;
}

id CocoaFindViewWithTagInAnyWindow( NSInteger tag, Class cls )
{
    NSArray *windows = [NSApp orderedWindows];
    for ( NSWindow *w in windows ) {
        id view = CocoaFindViewWithTagInWindow( tag, cls, w );
        if ( view ) return view;
    }
    return nil;
}

id CocoaFindViewWithTag( NSInteger tag, Class cls )
{
    if ( tag > 1000000 ) return (id)tag;
    
    id w = nil;
    NSView *contentView = CocoaOutputWindowContentView();
    if ( !contentView ) {
        w = CocoaFrontWindow();
        contentView = [w contentView];
    }

    if ( contentView ) {
        w = [contentView window];
        
        id view = CocoaFindViewWithTagInWindow( tag, cls, w );
        if ( view ) return view;
        
        view = CocoaFindViewWithTagInAnyWindow( tag, cls );
        if ( view ) return view;
        
        view = CocoaFindViewWithTagInAnySavePanelAccessoryView( tag, cls );
        if ( view ) return view;

        view = CocoaFindViewWithTagInAnyOpenPanelAccessoryView( tag, cls );
        if ( view ) return view;

        view = TitlebarAccessoryViewWithTag( tag );
        if ( view ) return view;

        CocoaShowAlert( NSWarningAlertStyle, @"Tag error.", [NSString stringWithFormat:@"Can't find view %ld in window %ld.",(long)tag,(long)[w tag]] );
        
    } else {
        CocoaShowAlert( NSWarningAlertStyle, @"Error.", @"Can't find any windows." );
    }

    return nil;
}


BOOL CocoaViewWithKindOfClassExists( NSInteger tag, Class cls )
{
    id w = nil;
    NSView *contentView = CocoaOutputWindowContentView();
    if ( !contentView ) {
        w = CocoaFrontWindow();
        contentView = [w contentView];
    }

    if ( contentView ) {
        id view = CocoaViewWithTag( tag );
        if ( view ) {
            if ( [view isKindOfClass:cls] ) {
                return YES;
            }
        }
    }
    return NO;
}


id CocoaValidateViewKindOfClass( NSInteger tag, Class cls )
{
    id w = nil;
    NSView *contentView = CocoaOutputWindowContentView();
    if ( !contentView ) {
        w = CocoaFrontWindow();
        contentView = [w contentView];
    }
    
    if ( contentView ) {
        w = [contentView window];
        id view = CocoaViewWithTag( tag );
        if ( view ) {
            
            if ( [view isKindOfClass:cls] ) {
                return view;
            } else {
                CocoaShowAlert( NSWarningAlertStyle, @"Class error.", [NSString stringWithFormat:@"View %ld (%@), in window %ld, cannot respond to %@ messages.",(long)tag,[view class],(long)[w tag],cls] );
            }
            
        } else {
            CocoaShowAlert( NSWarningAlertStyle, @"Tag error.", [NSString stringWithFormat:@"Can't find view %ld in window %ld.",(long)tag,(long)[w tag]] );
        }
    } else {
        CocoaShowAlert( NSWarningAlertStyle, @"Error.", @"Can't find any windows." );
    }
    
    return nil;
}


//BOOL CocoaValidateViewClass( NSInteger wndTag, NSInteger viewTag, Class actualClass, Class requiredClass )
//{
//    if ( requiredClass != actualClass ) {
//        CocoaShowAlert( NSWarningAlertStyle, @"Class error.", [NSString stringWithFormat:@"View %ld (%@), in window %ld, should be %@ class.",(long)viewTag,actualClass,(long)wndTag,requiredClass] );
//        return NO;
//    }
//    return YES;
//}


//BOOL CocoaValidateViewClass( NSInteger wndTag, NSInteger viewTag, Class actualClass, Class requiredClass )
//{
//    id obj = CocoaViewWithTag(viewTag);
//    if ( ([obj class] != requiredClass) && ([obj superclass] != requiredClass) ) {
//        CocoaShowAlert( NSWarningAlertStyle, @"Class error.", [NSString stringWithFormat:@"View %ld (%@), in window %ld, expected %@ class.",(long)viewTag,requiredClass,(long)wndTag,actualClass] );
//        return NO;
//    }
//    return YES;
//}

BOOL CocoaValidateViewClass( NSInteger wndTag, NSInteger viewTag, id obj, Class requiredClass )
{
    if ( ![obj isKindOfClass:requiredClass] ) {
        CocoaShowAlert( NSWarningAlertStyle, @"Class error.", [NSString stringWithFormat:@"View %ld (%@), in window %ld, expected %@ class.",(long)viewTag,requiredClass,(long)wndTag,[obj class]] );
        return NO;
    }
    return YES;
}


/*
 Subclassed windows
 */
void CocoaWindowSubclassMouseDown( NSWindow *w, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    CocoaOutputWindowSetTag( [w tag] );
    CallUserDefinedOnDialogFunction( windowMouseDown, [w tag], [w tag], nil );
    CocoaEventSetEvent( nil );
}

void CocoaWindowSubclassMouseUp( NSWindow *w, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    CocoaOutputWindowSetTag( [w tag] );
    CallUserDefinedOnDialogFunction( windowMouseUp, [w tag], [w tag], nil );
    CocoaEventSetEvent( nil );
}

void CocoaWindowSubclassKeyDown( NSWindow *w, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    CocoaOutputWindowSetTag( [w tag] );
    CallUserDefinedOnDialogFunction( windowKeyDown, [w tag], [w tag], nil );
    CocoaEventSetEvent( nil );
}

void CocoaWindowSubclassKeyUp( NSWindow *w, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    CocoaOutputWindowSetTag( [w tag] );
    CallUserDefinedOnDialogFunction( windowKeyUp, [w tag], [w tag], nil );
    CocoaEventSetEvent( nil );
}

void CocoaWindowSubclassFlagsChanged( NSWindow *w, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    CocoaOutputWindowSetTag( [w tag] );
    CallUserDefinedOnDialogFunction( windowFlagsChanged, [w tag], [w tag], nil );
    CocoaEventSetEvent( nil );
}

void CocoaWindowSubclassMouseEntered( NSWindow *w, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    CocoaOutputWindowSetTag( [w tag] );
    CallUserDefinedOnDialogFunction( windowMouseEntered, [w tag], [w tag], nil );
    CocoaEventSetEvent( nil );
}

void CocoaWindowSubclassMouseExited( NSWindow *w, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    CocoaOutputWindowSetTag( [w tag] );
    CallUserDefinedOnDialogFunction( windowMouseExited, [w tag], [w tag], nil );
    CocoaEventSetEvent( nil );
}






/*
 Subclassed views
 */
void CocoaSubclassDrawRect( id view, NSRect dirtyRect )
{
    DialogEventSetBool( NO );
    DialogEventSetNSRect( dirtyRect );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewDrawRect, CocoaViewTag(view), [window tag], NULL );
}

void CocoaSubclassAcceptsFirstResponder( id view )
{
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewAcceptsFirstResponder, CocoaViewTag(view), [window tag], nil );
}

void CocoaSubclassMouseDown( NSView *view, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewMouseDown, CocoaViewTag(view), [window tag], NULL );
    CocoaEventSetEvent( nil );
}

void CocoaSubclassMouseUp( NSView *view, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewMouseUp, CocoaViewTag(view), [window tag], NULL );
    CocoaEventSetEvent( nil );
}

void CocoaSubclassMouseEntered( NSView *view, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewMouseEntered, CocoaViewTag(view), [window tag], NULL );
    CocoaEventSetEvent( nil );
}

void CocoaSubclassMouseExited( NSView *view, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewMouseExited, CocoaViewTag(view), [window tag], NULL );
    CocoaEventSetEvent( nil );
}

void CocoaSubclassMouseDragged( NSView *view, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewMouseDragged, CocoaViewTag(view), [window tag], NULL );
    CocoaEventSetEvent( nil );
}

void CocoaSubclassMouseMoved( NSView *view, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewMouseMoved, CocoaViewTag(view), [window tag], NULL );
    CocoaEventSetEvent( nil );
}

void CocoaSubclassRightMouseDown( NSView *view, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewRightMouseDown, CocoaViewTag(view), [window tag], NULL );
    CocoaEventSetEvent( nil );
}

void CocoaSubclassRightMouseUp( NSView *view, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewRightMouseUp, CocoaViewTag(view), [window tag], NULL );
    CocoaEventSetEvent( nil );
}

void CocoaSubclassRightMouseDragged( NSView *view, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewRightMouseDragged, CocoaViewTag(view), [window tag], NULL );
    CocoaEventSetEvent( nil );
}

void CocoaSubclassKeyDown( NSView *view, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewKeyDown, CocoaViewTag(view), [window tag], NULL );
    CocoaEventSetEvent( nil );
}

void CocoaSubclassKeyUp( NSView *view, NSEvent *event )
{
    DialogEventSetBool( NO );
    CocoaEventSetEvent( event );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewKeyUp, CocoaViewTag(view), [window tag], NULL );
    CocoaEventSetEvent( nil );
}

void CocoaSubclassFlagsChanged( NSView *view, NSEvent *event )
{
    DialogEventSetBool(NO);
    CocoaEventSetEvent( event );
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewFlagsChanged, CocoaViewTag(view), [window tag], NULL );
    CocoaEventSetEvent( nil );
}

void CocoaSubclassUpdateTrackingAreas( NSView *view )
{
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewUpdateTrackingAreas, CocoaViewTag(view), [window tag], NULL );
}

void CocoaSubclassResetCursorRects( NSView *view )
{
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    CallUserDefinedOnDialogFunction( viewResetCursorRects, CocoaViewTag(view), [window tag], NULL );
}

NSDragOperation CocoaSubclassDraggingEntered( NSView *view, id<NSDraggingInfo>sender )
{
    NSDragOperation dragOperation = NSDragOperationNone;

    DialogEventSetLong(dragOperation);
    NSArray *regTypes = [view registeredDraggedTypes];

    if ( [regTypes containsObject:NSFilenamesPboardType] ) {
        NSWindow *window = [view window];
        CocoaOutputWindowSetTag( [window tag] );

        NSPasteboard *pb = [sender draggingPasteboard];
        NSArray *pbTypes = [pb types];
        if ( [pbTypes containsObject:NSFilenamesPboardType] ) {
            DialogEventSetDraggingInfo(sender);

            NSArray *files = [pb propertyListForType:NSFilenamesPboardType];

            BOOL validFile = NO;

            if ( view.dragFileTypes != nil || view.dragFileExtensions != nil ) {
                for ( NSString *path in files ) {
                    validFile = NO;

                    if ( view.dragFileTypes ) {
                        NSString *utiType = [[NSWorkspace sharedWorkspace] typeOfFile:path error:nil];
                        for ( NSString *type in view.dragFileTypes ) {
                            if ( [[NSWorkspace sharedWorkspace] type:utiType conformsToType:(id)type] ) {
                                validFile = YES;
                                break;
                            }
                        }
                    }

                    if ( !validFile ) {
                        if ( view.dragFileExtensions ) {
                            NSString *extension = [path pathExtension];
                            if ( extension ) {
                                if ( [view.dragFileExtensions containsObject:extension] ) {
                                    validFile = YES;
                                }
                            }
                        }
                    }

                    if ( !validFile ) break;
                }
            } else {
                validFile = YES;
            }

            if ( validFile ) {
                dragOperation = NSDragOperationLink;
                NSMutableArray *urls = [NSMutableArray arrayWithCapacity:[files count]];
                for ( NSString *path in files ) {
                    [urls addObject:[NSURL fileURLWithPath:path]];
                }
                DialogEventSetNSArray( urls );
                DialogEventSetNSURL( [urls objectAtIndex:0] );
                view.dragInsideFlag = YES;
                DialogEventSetLong(dragOperation);
                CallUserDefinedOnDialogFunction( viewDraggingEntered, CocoaViewTag(view), [window tag], urls );
                DialogEventSetDraggingInfo(nil);
                DialogEventSetNSArray( nil );
                DialogEventSetNSURL( nil );
            }
        }
    }
    return DialogEventLong();
}

void CocoaSubclassDraggingExited( NSView *view, id <NSDraggingInfo>sender )
{
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    DialogEventSetDraggingInfo(sender);
    view.dragInsideFlag = NO;
    CallUserDefinedOnDialogFunction( viewDraggingExited, CocoaViewTag(view), [window tag], NULL );
    DialogEventSetDraggingInfo(nil);
    DialogEventSetNSArray( nil );
    DialogEventSetNSURL( nil );
}

BOOL CocoaSubclassPerformDragOperation( NSView *view, id <NSDraggingInfo>sender )
{
    [[NSCursor arrowCursor] set];
    NSWindow *window = [view window];
    CocoaOutputWindowSetTag( [window tag] );
    NSPasteboard *pb = [sender draggingPasteboard];
    NSArray *files = [pb propertyListForType:NSFilenamesPboardType];
    NSMutableArray *urls = [NSMutableArray arrayWithCapacity:[files count]];

    for ( NSString *path in files ) {
        [urls addObject:[NSURL fileURLWithPath:path]];
    }

    DialogEventSetNSArray( urls );
    DialogEventSetNSURL( [urls objectAtIndex:0] );
    DialogEventSetDraggingInfo(sender);
    DialogEventSetBool(NO);
    CallUserDefinedOnDialogFunction( viewPerformDragOperation, CocoaViewTag(view), [window tag], urls );
    CocoaSubclassDraggingExited( view, sender );
    return YES;
}

void CocoaSubclassViewDidChangeEffectiveAppearance( id view )
{
    NSWindow *window = [view window];
    CallUserDefinedOnDialogFunction( viewDidChangeEffectiveAppearance, CocoaViewTag(view), [window tag], nil );
}


/*
 NSRangeToCFRange
 NSRangeFromCFRange
 */
CFRange NSRangeToCFRange( NSRange range )
{ return CFRangeMake(range.location,range.length); }

NSRange NSRangeFromCFRange( CFRange range )
{ return NSMakeRange(range.location,range.length); }


#endif  //__MAC_OS_X_VERSION_MAX_ALLOWED >= 1070


