/*
 AffineTransform.m
 
 Bernie Wylde
 */

#import "AffineTransform.h"

AffineTransformRef AffineTransformInit( void )
{ return [NSAffineTransform transform]; }

AffineTransformRef AffineTransformWithTransform( AffineTransformRef ref )
{ return [[[NSAffineTransform alloc] initWithTransform:ref] autorelease]; }

void AffineTransformRotateByDegrees( AffineTransformRef tx, CGFloat angle )
{ [tx rotateByDegrees:angle]; }

void AffineTransformRotateByRadians( AffineTransformRef tx, CGFloat angle )
{ [tx rotateByRadians:angle]; }

void AffineTransformScale( AffineTransformRef tx, CGFloat scale )
{ [tx scaleBy:scale]; }

void AffineTransformScaleXY( AffineTransformRef tx, CGFloat xScale, CGFloat yScale )
{ [tx scaleXBy:xScale yBy:yScale]; }

void AffineTransformTranslate( AffineTransformRef tx, CGFloat x, CGFloat y )
{ [tx translateXBy:x yBy:y]; }

void AffineTransformAppend( AffineTransformRef tx1, AffineTransformRef tx2 )
{ [tx1 appendTransform:tx2]; }

void AffineTransformPrepend( AffineTransformRef tx1, AffineTransformRef tx2 )
{ [tx1 prependTransform:tx2]; }

void AffineTransformInvert( AffineTransformRef tx )
{ [tx invert]; }

CGPoint AffineTransformPoint( AffineTransformRef tx, CGPoint pt )
{ return NSPointToCGPoint([tx transformPoint:NSPointFromCGPoint(pt)]); }

CGSize AffineTransformSize( AffineTransformRef tx, CGSize size )
{ return NSSizeToCGSize([tx transformSize:NSSizeFromCGSize(size)]); }

BezierPathRef AffineTransformBezierPath( AffineTransformRef tx, BezierPathRef path )
{ return [tx transformBezierPath:path]; }

NSAffineTransformStruct AffineTransformStruct( AffineTransformRef tx )
{ return [tx transformStruct]; }

void AffineTransformSet( AffineTransformRef tx )
{ [tx set]; }

void AffineTransformConcat( AffineTransformRef tx )
{ [tx concat]; }
