/*
 Application.h
 
 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// application events
enum {
    appWillFinishLaunching = 1,
    appDidFinishLaunching,
    appShouldTerminate,
    appShouldTerminateAfterLastWindowClosed,
    appWillTerminate,
    appWillBecomeActive,
    appDidBecomeActive,
    appWillResignActive,
    appDidResignActive,
    appWillHide,
    appDidHide,
    appWillUnhide,
    appDidUnhide,
    appWillUpdate,
    appDidUpdate,
    appShouldHandleReopen,
    appDockMenu,
    appWillPresentError,
    appDidChangeScreenParameters,
    //    appOpenFile,
    appOpenFileWithoutUI,
    appOpenTempFile,
    //    appOpenFiles,
    appOpenUntitledFile,
    appShouldOpenUntitledFile,
    appPrintFile,
    appPrintFiles,
    appDidRegisterForRemoteNotificationsWithDeviceToken,
    appDidFailToRegisterForRemoteNotifications,
    appDidReceiveRemoteNotification,
    appDidDecodeRestorableState,
    appWillEncodeRestorableState,
    appWillContinueUserActivityWithType,
    appContinueUserActivity,
    appDidFailToContinueUserActivityWithType,
    appDidUpdateUserActivity,
    appDidChangeOcclusionState,
    appOpenURLs
};

typedef void (*CocoaApplicationEnumerateWindowsCallbackType)(NSInteger,BOOL*);

typedef BOOL (*CocoaApplicationTimerCallbackType)(NSTimer*);

typedef void (*CocoaAppDelegateCallbackType)(NSInteger,void*);// event,userData

@interface CocoaApplicationDelegate : NSObject <NSApplicationDelegate> {
    CocoaAppDelegateCallbackType delegateCallback;
    void *delegateUserData;
    NSMutableArray *windows;
    BOOL boolVal;
    long longVal;
    NSString *stringVal;
    NSArray *arrayVal;
    NSDictionary *dictVal;
    NSURL *urlVal;
    NSMenu *menuObj;
}
@property (assign) CocoaAppDelegateCallbackType delegateCallback;
@property (assign) void *delegateUserData;
@property (assign) NSMutableArray *windows;
@property (assign) BOOL boolVal;
@property (assign) long longVal;
@property (assign) NSString *stringVal;
@property (assign) NSArray *arrayVal;
@property (assign) NSDictionary *dictVal;
@property (assign) NSURL *urlVal;
@property (assign) NSMenu *menuObj;
@end

// events
void AppEventSetBool( Boolean value );
void AppEventSetLong( long value );
CFStringRef AppEventString( void );
CFArrayRef AppEventArray( void );
CFDictionaryRef AppEventDictionary( void );
CFURLRef AppEventURL( void );
void AppEventSetMenu( NSMenu *obj );


CocoaApplicationDelegate *CocoaApplicationDelegateGet( void );

// Shared app
ApplicationRef AppSharedApplication( void );

// aux functions
ImageRef AppIconImage( void );
void AppSetIconImage( void *inImage );
void AppTerminate( void );

CocoaEventRef AppCurrentEvent( void );

Boolean AppIsRunning( void );
void AppRun( void );
void AppStop( void );
void AppSendEvent( CocoaEventRef evnt );
void AppPostEvent( CocoaEventRef evnt, Boolean atStart );

// Remote notifications
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
void AppRegisterForRemoteNotifications( void );
#endif

CocoaMenuRef AppMainMenu( void );

CocoaMenuRef AppWindowsMenu( void );

// Appearance
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
AppearanceRef AppAppearance( void );
AppearanceRef AppEffectiveAppearance( void );
#endif

NSInteger AppKeyWindow( void );
NSInteger AppMainWindow( void );

CocoaWindowRef AppWindowWithWindowNumber( NSInteger number );
CFArrayRef AppWindows( void );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void AppEnumerateWindows( NSWindowListOptions options, CocoaApplicationEnumerateWindowsCallbackType callback );
#endif

void AppMiniaturizeAll( void );

Boolean AppIsHidden( void );
void AppHide( void );
void AppUnhide( void );
void AppUnhideWithoutActivation( void );

void AppArrangeInFront( void );

DockTileRef AppDockTile( void );

Boolean AppIsActive( void );
void AppActivateIgnoringOtherApps( Boolean flag );
void AppDeactivate( void );

void AppHideOtherApplications( void );
void AppUnhideAllApplications( void );
void AppShowHelp( void );
CocoaMenuRef AppHelpMenu( void );

NSInteger AppRequestUserAttention( NSRequestUserAttentionType type );
void AppCancelUserAttentionRequest( NSInteger request );

// Presentation options
NSApplicationPresentationOptions AppCurrentSystemPresentationOptions( void );
NSApplicationPresentationOptions AppPresentationOptions( void );
void AppSetPresentationOptions( NSApplicationPresentationOptions options );

// Activation policy
NSApplicationActivationPolicy AppActivationPolicy( void );
Boolean AppSetActivationPolicy( NSApplicationActivationPolicy policy );

// Relaunch on login
void AppDisableRelaunchOnLogin( void );
void AppEnableRelaunchOnLogin( void );

// Occlusion state
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSApplicationOcclusionState AppOcclusionState( void );
#endif

// Scripting
CFArrayRef AppOrderedDocuments( void );
CFArrayRef AppOrderedWindows( void );

NSInteger AppRunModal( NSInteger wndTag );
void AppStopModal( void );
void AppStopModalWithCode( NSInteger returnCode );
void AppAbortModal( void );
NSModalSession AppBeginModalSession( NSInteger wndTag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSModalResponse AppRunModalSession( NSModalSession session );
#endif
NSInteger AppModalWindow( void );
void AppEndModalSession( NSModalSession session );

void AppOrderFrontColorPanel( void );
void AppOrderFrontStandardAboutPanel( void );
void AppOrderFrontStandardAboutPanelWithOptions( CFDictionaryRef options );
void AppOrderFrontCharacterPalette( void );
void AppRunPageLayout( void );

NSUserInterfaceLayoutDirection AppUserInterfaceLayoutDirection( void );

void AppReplyToApplicationShouldTerminate( Boolean flag );



// custom
void CocoaInit( void );
NSInteger AppOutputWindow( void );
void AppSetOutputWindow( NSInteger tag );
void AppBringAllWindowsToFront( void );

CFRunLoopTimerRef AppSetTimer( NSTimeInterval interval, CocoaApplicationTimerCallbackType callback, BOOL repeats );
void AppSetDelegateCallback( void *callback, void *userData );


// event
long FBAppEventFunction( long val );                                          // cocoaui 20170704 bw
//void CallUserDefinedOnAppEventFunction( long appEventKind, long appEventID ); // cocoaui 20170704 bw
void CallUserDefinedOnAppEventFunction( long appEventKind, CFTypeRef obj );// cocoaui 20170704 bw    // 20180711 2nd param changed to object


// -- convenience --
ViewRef AppDockTileContentView( void );
CGSize AppDockTileSize( void );
//id AppDockTileOwner( void );
Boolean AppDockTileShowsApplicationBadge( void );
void AppDockTileSetShowsApplicationBadge( Boolean flag );
CFStringRef AppDockTileBadgeLabel( void );
void AppDockTileSetBadgeLabel( CFStringRef label );
void AppDockTileDisplay( void );

