/*
 Application.m
 
 Bernie Wylde
 */

#import "Application.h"

CocoaApplicationDelegate *sCocoaApplicationDelegate;

@implementation CocoaApplicationDelegate

@synthesize delegateCallback;
@synthesize delegateUserData;
@synthesize windows;
@synthesize boolVal;
@synthesize longVal;
@synthesize stringVal;
@synthesize arrayVal;
@synthesize dictVal;
@synthesize urlVal;
@synthesize menuObj;

- (id)init {
    if ( self = [super init] ) {
        windows = [[NSMutableArray alloc] initWithCapacity:0];
    }
    return self;
}

- (void)timerFired:(NSTimer *)timer {
    CocoaApplicationTimerCallbackType callback = [timer.userInfo pointerValue];
    if ( (*callback)(timer) ) {
        [timer invalidate];
    }
}

#pragma mark - application delegate methods
- (void)forwardApplicationEvent:(NSInteger)event object:(id)obj {
    if ( delegateCallback ) {
        (*delegateCallback)(event,delegateUserData);
    } else {
        CallUserDefinedOnAppEventFunction( event, (CFTypeRef)obj );
    }
}

- (void)applicationWillFinishLaunching:(NSNotification *)notification {
    if ( ![NSApp mainMenu] ) BuildMinimalMenu();    // make sure we have at least a quit item
    [self forwardApplicationEvent:appWillFinishLaunching object:nil];
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification {
    [self forwardApplicationEvent:appDidFinishLaunching object:nil];
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)app {
    longVal = NSTerminateNow;
    [self forwardApplicationEvent:appShouldTerminate object:nil];
    return longVal;
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)app {
    boolVal = NO;
    [self forwardApplicationEvent:appShouldTerminateAfterLastWindowClosed object:nil];
    return boolVal;
}

- (void)applicationWillTerminate:(NSNotification *)notification {
    [self forwardApplicationEvent:appWillTerminate object:nil];
}

- (void)applicationWillBecomeActive:(NSNotification *)notification {
    [self forwardApplicationEvent:appWillBecomeActive object:nil];
}

- (void)applicationDidBecomeActive:(NSNotification *)notification {
    [self forwardApplicationEvent:appDidBecomeActive object:nil];
}

- (void)applicationWillResignActive:(NSNotification *)notification {
    [self forwardApplicationEvent:appWillResignActive object:nil];
}

- (void)applicationDidResignActive:(NSNotification *)notification {
    [self forwardApplicationEvent:appDidResignActive object:nil];
}

- (void)applicationWillHide:(NSNotification *)notification {
    [self forwardApplicationEvent:appWillHide object:nil];
}

- (void)applicationDidHide:(NSNotification *)notification {
    [self forwardApplicationEvent:appDidHide object:nil];
}

- (void)applicationWillUnhide:(NSNotification *)notification {
    [self forwardApplicationEvent:appWillUnhide object:nil];
}

- (void)applicationDidUnhide:(NSNotification *)notification {
    [self forwardApplicationEvent:appDidUnhide object:nil];
}

- (void)applicationWillUpdate:(NSNotification *)notification {
    [self forwardApplicationEvent:appWillUpdate object:nil];
}

- (void)applicationDidUpdate:(NSNotification *)notification {
    [self forwardApplicationEvent:appDidUpdate object:nil];
}

- (BOOL)applicationShouldHandleReopen:(NSApplication *)app hasVisibleWindows:(BOOL)flag {
    boolVal = YES;
    [self forwardApplicationEvent:appShouldHandleReopen object:nil];
    return boolVal;
}

- (NSMenu *)applicationDockMenu:(NSApplication *)app {
    menuObj = nil;
    [self forwardApplicationEvent:appDockMenu object:menuObj];
    return menuObj;
}

- (NSError *)application:(NSApplication *)app willPresentError:(NSError *)error {
    // .....
    //[self forwardApplicationEvent:appWillPresentError object:nil];
    return error;
}

- (void)applicationDidChangeScreenParameters:(NSNotification *)notification {
    [self forwardApplicationEvent:appDidChangeScreenParameters object:nil];
}

/*
 this method doesn't get called if application:openFiles: exists
 */
//- (BOOL)application:(NSApplication *)app openFile:(NSString *)filename {
//    stringVal = filename;
//    boolVal = NO;
//    CallUserDefinedOnAppEventFunction( appOpenFile, 0 );
//    [self forwardApplicationEvent:appOpenFile];
//    return boolVal;
//}

- (BOOL)application:(id)sender openFileWithoutUI:(NSString *)filename {
    stringVal = filename;
    boolVal = NO;
    //    CallUserDefinedOnAppEventFunction( appOpenFileWithoutUI, 0 );
    [self forwardApplicationEvent:appOpenFileWithoutUI object:nil];
    return boolVal;
}

- (BOOL)application:(NSApplication *)app openTempFile:(NSString *)filename {
    stringVal = filename;
    boolVal = NO;
    [self forwardApplicationEvent:appOpenTempFile object:stringVal];
    return boolVal;
}

/*
 this method stops application:openFile: from being called so both methods can't co-exist
 */
- (void)application:(NSApplication *)app openFiles:(NSArray *)filenames {
    stringVal = [[filenames objectAtIndex:0] lastPathComponent];
    NSMutableArray *tempArray = [NSMutableArray arrayWithCapacity:0];
    for ( NSString *path in filenames ) {
        [tempArray addObject:[NSURL fileURLWithPath:path]];
    }
    urlVal = [tempArray objectAtIndex:0];
    arrayVal = (NSArray *)tempArray;

    [self forwardApplicationEvent:appOpenURLs object:arrayVal];
}

- (BOOL)applicationOpenUntitledFile:(NSApplication *)app {
    boolVal = NO;
    [self forwardApplicationEvent:appOpenUntitledFile object:nil];
    return boolVal;
}

- (BOOL)applicationShouldOpenUntitledFile:(NSApplication *)app {
    boolVal = NO;
    [self forwardApplicationEvent:appShouldOpenUntitledFile object:nil];
    return boolVal;
}

- (BOOL)application:(NSApplication *)app printFile:(NSString *)filename {
    stringVal = filename;
    boolVal = YES;
    [self forwardApplicationEvent:appPrintFile object:stringVal];
    return boolVal;
}

- (NSApplicationPrintReply)application:(NSApplication *)app printFiles:(NSArray *)filenames withSettings:(NSDictionary *)printSettings showPrintPanels:(BOOL)showPrintPanels {
    longVal = NSPrintingSuccess;
    arrayVal = filenames;
    dictVal = printSettings;
    boolVal = showPrintPanels;
    [self forwardApplicationEvent:appPrintFiles object:arrayVal];
    return longVal;
}

- (void)application:(NSApplication *)app didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken {
    // dataVal = deviceToken;
    [self forwardApplicationEvent:appDidRegisterForRemoteNotificationsWithDeviceToken object:nil];
}

- (void)application:(NSApplication *)app didFailToRegisterForRemoteNotificationsWithError:(NSError *)error {
    [self forwardApplicationEvent:appDidFailToRegisterForRemoteNotifications object:nil];
}

- (void)application:(NSApplication *)app didReceiveRemoteNotification:(NSDictionary *)userInfo {
    dictVal = userInfo;
    [self forwardApplicationEvent:appDidReceiveRemoteNotification object:nil];
}

- (void)application:(NSApplication *)app didDecodeRestorableState:(NSCoder *)coder {
    // coder
    [self forwardApplicationEvent:appDidDecodeRestorableState object:nil];
}

- (void)application:(NSApplication *)app willEncodeRestorableState:(NSCoder *)coder {
    // coder
    [self forwardApplicationEvent:appWillEncodeRestorableState object:nil];
}

- (BOOL)application:(NSApplication *)app willContinueUserActivityWithType:(NSString *)userActivityType {
    boolVal = NO;
    stringVal = userActivityType;
    [self forwardApplicationEvent:appWillContinueUserActivityWithType object:stringVal];
    return boolVal;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100

//- (BOOL)application:(NSApplication *)app continueUserActivity:(NSUserActivity *)userActivity restorationHandler:(void (^)(NSArray<id<NSUserActivityRestoring>> *restorableObjects))restorationHandler {
//    boolVal = NO;
//    //arrayVal = restorableObjects;
//    [self forwardApplicationEvent:appContinueUserActivity object:nil];
//    return boolVal;
//}

//- (BOOL)application:(NSApplication *)app continueUserActivity:(NSUserActivity *)userActivity restorationHandler:(void (^)(NSArray *restorableObjects))restorationHandler {
//    boolVal = NO;
//    //arrayVal = restorableObjects;
//    [self forwardApplicationEvent:appContinueUserActivity object:nil];
//    return boolVal;
//}
#endif

- (void)application:(NSApplication *)app didFailToContinueUserActivityWithType:(NSString *)userActivityType error:(NSError *)error {
    stringVal = userActivityType;
    // error
    [self forwardApplicationEvent:appDidFailToContinueUserActivityWithType object:stringVal];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
- (void)application:(NSApplication *)app didUpdateUserActivity:(NSUserActivity *)userActivity {
    [self forwardApplicationEvent:appDidUpdateUserActivity object:nil];
}
#endif

- (void)applicationDidChangeOcclusionState:(NSNotification *)notification {
    [self forwardApplicationEvent:appDidChangeOcclusionState object:nil];
}

/*
 if this method were to be implemented, it would prevent application:openFiles from being called
 */
//#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
//- (void)application:(NSApplication *)application openURLs:(NSArray<NSURL *> *)urls {
//    urlVal = [urls objectAtIndex:0];
//    arrayVal = urls;
//    CallUserDefinedOnAppEventFunction( appOpenURLs, 0 );
//    [self forwardApplicationEvent:appWillBecomeActive object:nil];
//}
//#endif

@end

CocoaApplicationDelegate *CocoaApplicationDelegateGet( void )
{
    CocoaApplicationDelegate *appDelegate = (CocoaApplicationDelegate *)[NSApp delegate];
    if ( !appDelegate ) {
        appDelegate = [[CocoaApplicationDelegate alloc] init];
        sCocoaApplicationDelegate = appDelegate;
    }
    return appDelegate;
}

// event
void AppEventSetBool( Boolean value )
{ [CocoaApplicationDelegateGet() setBoolVal:value]; }

void AppEventSetLong( long value )
{ [CocoaApplicationDelegateGet() setLongVal:value]; }

CFStringRef AppEventString( void )
{ return (CFStringRef)[CocoaApplicationDelegateGet() stringVal]; }

CFArrayRef AppEventArray( void )
{ return (CFArrayRef)[CocoaApplicationDelegateGet() arrayVal]; }

CFDictionaryRef AppEventDictionary( void )
{ return (CFDictionaryRef)[CocoaApplicationDelegateGet() dictVal]; }

CFURLRef AppEventURL( void )
{ return (CFURLRef)[CocoaApplicationDelegateGet() urlVal]; }

void AppEventSetMenu( NSMenu *obj )
{ [CocoaApplicationDelegateGet() setMenuObj:obj]; }



// Shared app
ApplicationRef AppSharedApplication( void )
{ return [NSApplication sharedApplication]; }


ImageRef AppIconImage( void )
{ return [NSApp applicationIconImage]; }

void AppSetIconImage( void *inImage )
{
    NSImage *image = FixImageValue( inImage );
    if ( image ) [NSApp setApplicationIconImage:image];
}

void AppTerminate( void )
{ [NSApp terminate:nil]; }

CocoaEventRef AppCurrentEvent( void )
{ return [NSApp currentEvent]; }

Boolean AppIsRunning( void )
{ return [NSApp isRunning]; }

void AppRun( void )
{ [NSApp run]; }

void AppStop( void )
{ [NSApp stop:nil]; }

void AppSendEvent( CocoaEventRef evnt )
{ [NSApp sendEvent:evnt]; }

void AppPostEvent( CocoaEventRef evnt, Boolean atStart )
{ [NSApp postEvent:evnt atStart:atStart]; }


// Remote notifications
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
void AppRegisterForRemoteNotifications( void )
{ [NSApp registerForRemoteNotifications]; }
#endif


NSInteger AppKeyWindow( void )
{
    NSWindow *w = [NSApp keyWindow];
    if ( w ) return [w tag];
    return 0;
}

NSInteger AppMainWindow( void )
{
    NSWindow *w = [NSApp mainWindow];
    if ( w ) return [w tag];
    return 0;
}

CocoaWindowRef AppWindowWithWindowNumber( NSInteger number )
{ return [NSApp windowWithWindowNumber:number]; }

CFArrayRef AppWindows( void )
{ return (CFArrayRef)[NSApp windows]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void AppEnumerateWindows( NSWindowListOptions options, CocoaApplicationEnumerateWindowsCallbackType callback )
{
    [NSApp enumerateWindowsWithOptions:options usingBlock:^(NSWindow *window, BOOL *stop) {
        BOOL stopFlag = NO;
        (*callback)([window tag],&stopFlag);
        if ( stopFlag ) *stop = YES;
    }];
}
#endif

void AppMiniaturizeAll( void )
{ [NSApp miniaturizeAll:nil]; }

Boolean AppIsHidden( void )
{ return [NSApp isHidden]; }

void AppHide( void )
{ [NSApp hide:nil]; }

void AppUnhide( void )
{ [NSApp unhide:nil]; }

void AppUnhideWithoutActivation( void )
{ [NSApp unhideWithoutActivation]; }

void AppArrangeInFront( void )
{ [NSApp arrangeInFront:nil]; }


CocoaMenuRef AppMainMenu( void )
{ return [NSApp mainMenu]; }

CocoaMenuRef AppWindowsMenu( void )
{ return [NSApp windowsMenu]; }

// Appearance
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
AppearanceRef AppAppearance( void )
{ return [NSApp appearance]; }

AppearanceRef AppEffectiveAppearance( void )
{ return [NSApp effectiveAppearance]; }
#endif

CocoaMenuRef AppHelpMenu( void )
{ return [NSApp helpMenu]; }

NSInteger AppRequestUserAttention( NSRequestUserAttentionType type )
{ return [NSApp requestUserAttention:type]; }

void AppCancelUserAttentionRequest( NSInteger request )
{ [NSApp cancelUserAttentionRequest:request]; }

// Presentation options
NSApplicationPresentationOptions AppCurrentSystemPresentationOptions( void )
{ return [NSApp currentSystemPresentationOptions]; }

NSApplicationPresentationOptions AppPresentationOptions( void )
{ return [NSApp presentationOptions]; }

void AppSetPresentationOptions( NSApplicationPresentationOptions options )
{ return [NSApp setPresentationOptions:options]; }

// Activation policy
NSApplicationActivationPolicy AppActivationPolicy( void )
{ return [NSApp activationPolicy]; }

Boolean AppSetActivationPolicy( NSApplicationActivationPolicy policy )
{ return [NSApp setActivationPolicy:policy]; }

// Relaunch on login
void AppDisableRelaunchOnLogin( void )
{ [NSApp disableRelaunchOnLogin]; }

void AppEnableRelaunchOnLogin( void )
{ [NSApp enableRelaunchOnLogin]; }

// Occlusion state
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSApplicationOcclusionState AppOcclusionState( void )
{ return [NSApp occlusionState]; }
#endif

// Scripting
CFArrayRef AppOrderedDocuments( void )
{ return (CFArrayRef)[NSApp orderedDocuments]; }

CFArrayRef AppOrderedWindows( void )
{ return (CFArrayRef)[NSApp orderedWindows]; }


DockTileRef AppDockTile( void )
{ return [NSApp dockTile]; }

Boolean AppIsActive( void )
{ return [NSApp isActive]; }

void AppDeactivate( void )
{ [NSApp deactivate]; }

void AppActivateIgnoringOtherApps( Boolean flag )
{ [NSApp activateIgnoringOtherApps:flag]; }

void AppHideOtherApplications( void )
{ [NSApp hideOtherApplications:nil]; }

void AppUnhideAllApplications( void )
{ [NSApp unhideAllApplications:nil]; }

void AppShowHelp( void )
{ [NSApp showHelp:nil]; }

NSInteger AppRunModal( NSInteger wndTag )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) return [NSApp runModalForWindow:w];
    return 0;
}

void AppStopModal( void )
{ [NSApp stopModal]; }

void AppStopModalWithCode( NSInteger returnCode )
{ [NSApp stopModalWithCode:returnCode]; }

void AppAbortModal( void )
{ [NSApp abortModal]; }

NSModalSession AppBeginModalSession( NSInteger wndTag )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) return [NSApp beginModalSessionForWindow:w];
    return nil;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSModalResponse AppRunModalSession( NSModalSession session )
{ return [NSApp runModalSession:session]; }
#endif

NSInteger AppModalWindow( void )
{
    NSWindow *w = [NSApp modalWindow];
    if ( w ) return [w tag];
    return 0;
}

void AppEndModalSession( NSModalSession session )
{ [NSApp endModalSession:session]; }

void AppOrderFrontColorPanel( void )
{
    [NSApp orderFrontColorPanel:nil];
    CocoaColorPanelSetTarget();
}

void AppOrderFrontStandardAboutPanel( void )
{ [NSApp orderFrontStandardAboutPanel:nil]; }

void AppOrderFrontStandardAboutPanelWithOptions( CFDictionaryRef options )
{
    NSMutableDictionary *dict = [(NSDictionary *)options mutableCopy];
    
    id icon = [dict objectForKey:@"ApplicationIcon"];
    if ( icon ) {
        if ( [icon isKindOfClass:[NSImage class]] ) {
            [dict setObject:icon forKey:@"ApplicationIcon"];
        } else {
            NSImage *image = CocoaImageResource( icon );
            [dict setObject:image forKey:@"ApplicationIcon"];
        }
    }
    [NSApp orderFrontStandardAboutPanelWithOptions:dict];
    [dict release];
}

void AppOrderFrontCharacterPalette( void )
{ [NSApp orderFrontCharacterPalette:nil]; }

void AppRunPageLayout( void )
{ [NSApp runPageLayout:nil]; }

NSUserInterfaceLayoutDirection AppUserInterfaceLayoutDirection( void )
{ return [NSApp userInterfaceLayoutDirection]; }

void AppReplyToApplicationShouldTerminate( Boolean flag )
{ [NSApp replyToApplicationShouldTerminate:flag]; }

CFRunLoopTimerRef AppSetTimer( NSTimeInterval interval, CocoaApplicationTimerCallbackType callback, BOOL repeats )
{
    CocoaApplicationDelegate *appDelegate = (CocoaApplicationDelegate *)[NSApp delegate];
    if ( appDelegate ) {
        return (CFRunLoopTimerRef)[NSTimer scheduledTimerWithTimeInterval:interval target:appDelegate selector:@selector(timerFired:) userInfo:[NSValue valueWithPointer:callback] repeats:repeats];
    }
    return nil;
}

void AppSetDelegateCallback( void *callback, void *userData )
{
    CocoaApplicationDelegate *appDelegate = (CocoaApplicationDelegate *)[NSApp delegate];
    if ( appDelegate ) {
        appDelegate.delegateCallback = callback;
        appDelegate.delegateUserData = userData;
    }
}

void CocoaInit( void )
{
    NSApplication *app = [NSApplication sharedApplication];
    if ( ![app delegate] ) {
        CocoaApplicationDelegate *appDelegate = [[[CocoaApplicationDelegate alloc] init] autorelease];
        [app setDelegate:appDelegate];
    }
}

NSInteger AppOutputWindow( void )
{ return CocoaOutputWindowTag(); }

void AppSetOutputWindow( NSInteger tag )
{ CocoaOutputWindowSetTag( tag ); }

void AppBringAllWindowsToFront( void )
{
    NSArray *windows = [NSApp orderedWindows];
    NSInteger count = [windows count] - 1;
    while ( count >= 0 ) {
        [[windows objectAtIndex:count] orderFrontRegardless];
        count--;
    }
}




// -- convenience --
ViewRef AppDockTileContentView( void )
{ return [[NSApp dockTile] contentView]; }

CGSize AppDockTileSize( void )
{ return NSSizeToCGSize([[NSApp dockTile] size]); }

//id AppDockTileOwner( void )
//{ return [[NSApp dockTile] owner]; }

Boolean AppDockTileShowsApplicationBadge( void )
{ return [[NSApp dockTile] showsApplicationBadge]; }

void AppDockTileSetShowsApplicationBadge( Boolean flag )
{ [[NSApp dockTile] setShowsApplicationBadge:flag]; }

CFStringRef AppDockTileBadgeLabel( void )
{ return (CFStringRef)[[NSApp dockTile] badgeLabel]; }

void AppDockTileSetBadgeLabel( CFStringRef label )
{ [[NSApp dockTile] setBadgeLabel:(NSString *)label]; }

void AppDockTileDisplay( void )
{ [[NSApp dockTile] display]; }




#if __LP64__

// application
void CallUserDefinedOnAppEventFunction( long appEventKind, CFTypeRef obj ) // cocoaui 20170704 bw    // 20180711 2nd param changed to object
{
    if ( gFBOnAppEventVector ) {
        gFBAppEventInfo.appEventKind = appEventKind;
        (*gFBOnAppEventVector)( gFBAppEventInfo.appEventKind, obj );
    }
}

long FBAppEventFunction( long val )                                         // cocoaui 20170704 bw    // 20180711 val ignored - always returns event kind
{
    return gFBAppEventInfo.appEventKind;
}

#endif  // __LP64__
