/*
 Array.h
 
 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef BOOL (*CocoaArrayEnumerateObjectsCallbackType)(CFArrayRef,CFTypeRef,NSUInteger,BOOL*,void*);
typedef BOOL (*CocoaArrayTestCallbackType)(CFTypeRef,NSUInteger,BOOL*);
typedef NSInteger (*CocoaArraySortedArrayFunctionType)(id,id,void*);

CFArrayRef ArrayWithArray( CFArrayRef array );

CFMutableArrayRef ArrayWithCapacity( NSUInteger numItems );
CFArrayRef ArrayWithContentsOfURL( CFURLRef url );
CFArrayRef ArrayWithObject( CFTypeRef obj );
CFMutableArrayRef ArrayWithObjects( CFTypeRef obj, ... );

Boolean ArrayContainsObject( CFArrayRef array, CFTypeRef obj );
NSUInteger ArrayCount( CFArrayRef array );

CFTypeRef ArrayFirstObject( CFArrayRef array );
CFTypeRef ArrayLastObject( CFArrayRef array );

CFTypeRef ArrayObjectAtIndex( CFArrayRef array, NSUInteger index );
CFArrayRef ArrayObjectsAtIndexes( CFArrayRef array, IndexSetRef indexes );
EnumeratorRef ArrayObjectEnumerator( CFArrayRef array );
EnumeratorRef ArrayReverseObjectEnumerator( CFArrayRef array );

// Find
NSUInteger ArrayIndexOfObject( CFArrayRef array, CFTypeRef obj );
NSUInteger ArrayIndexOfObjectInRange( CFArrayRef array, CFTypeRef obj, CFRange range );
NSUInteger ArrayIndexOfObjectIdenticalTo( CFArrayRef array, CFTypeRef obj );
NSUInteger ArrayIndexOfObjectPassingTest( CFArrayRef array, CocoaArrayTestCallbackType testFn );

// Sending message to elements
void ArrayMakeObjectsPerformSelector( CFArrayRef array, CFStringRef selector );
void ArrayMakeObjectsPerformSelectorWithObject( CFArrayRef array, CFStringRef selector, CFTypeRef object );
void ArrayEnumerateObjects( CFArrayRef array, CocoaArrayEnumerateObjectsCallbackType callback, void *userData );
void ArrayEnumerateObjectsWithOptions( CFArrayRef array, NSEnumerationOptions options, CocoaArrayEnumerateObjectsCallbackType callback, void *userData );
void ArrayEnumerateObjectsAtIndexes( CFArrayRef array, IndexSetRef indexSet, NSEnumerationOptions options, CocoaArrayEnumerateObjectsCallbackType callback, void *userData );

// Comparing arrays
CFTypeRef ArrayFirstObjectCommonWithArray( CFArrayRef array1, CFArrayRef array2 );
Boolean ArrayIsEqualToArray( CFArrayRef array1, CFArrayRef array2 );
CFArrayRef ArrayByAddingObject( CFArrayRef array, CFTypeRef obj );
CFArrayRef ArrayByAddingObjectsFromArray( CFArrayRef array1, CFArrayRef array2 );
CFArrayRef ArrayFilteredArrayUsingPredicate( CFArrayRef array, PredicateRef pred );
CFArrayRef ArraySubarrayWithRange( CFArrayRef array, CFRange range );

// Sort
CFDataRef ArraySortedArrayHint( CFArrayRef array );
CFArrayRef ArraySortedArrayUsingFunction( CFArrayRef array, CocoaArraySortedArrayFunctionType comparator, void *context );
CFArrayRef ArraySortedArrayUsingFunctionHint( CFArrayRef array, CocoaArraySortedArrayFunctionType comparator, void *context, CFDataRef hint );
CFArrayRef ArraySortedArrayUsingDescriptors( CFArrayRef array, CFArrayRef descriptors );
CFArrayRef ArraySortedArrayUsingSelector( CFArrayRef array, CFStringRef selector );

// Working with string elements
CFStringRef ArrayComponentsJoinedByString( CFArrayRef array, CFStringRef separator );

// Description
CFStringRef ArrayDescription( CFArrayRef array );
CFStringRef ArrayDescriptionWithLocale( CFArrayRef array, CFLocaleRef locale );
CFStringRef ArrayDescriptionWithLocaleIndent( CFArrayRef array, CFLocaleRef locale, NSUInteger indentLevel );

// Collecting paths
CFArrayRef ArrayPathsMatchingExtensions( CFArrayRef array, CFArrayRef extensions );

// Key-value coding
void ArraySetValueForKey( CFArrayRef array, CFTypeRef value, CFStringRef key );
CFTypeRef ArrayValueForKey( CFArrayRef array, CFStringRef key );

// Shuffle
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFArrayRef ArrayShuffledArray( CFArrayRef array );
#endif

Boolean ArrayWriteToURL( CFArrayRef array, CFURLRef url, Boolean atomically );

void ArrayAddObject( CFMutableArrayRef array, CFTypeRef obj );
void ArrayAddObjectsFromArray( CFMutableArrayRef array, CFArrayRef otherArray );
void ArrayInsertObjectAtIndex( CFMutableArrayRef array, CFTypeRef obj, NSUInteger index );
void ArrayInsertObjectsAtIndexes( CFMutableArrayRef array, CFArrayRef objects, IndexSetRef indexes );
void ArrayRemoveAllObjects( CFMutableArrayRef array );
void ArrayRemoveLastObject( CFMutableArrayRef array );
void ArrayRemoveObject( CFMutableArrayRef array, CFTypeRef obj );
void ArrayRemoveObjectInRange( CFMutableArrayRef array, CFTypeRef obj, CFRange range );
void ArrayRemoveObjectAtIndex( CFMutableArrayRef array, NSUInteger index );
void ArrayRemoveObjectsAtIndexes( CFMutableArrayRef array, IndexSetRef indexes );
void ArrayRemoveObjectsInArray( CFMutableArrayRef array, CFArrayRef objects );
void ArrayRemoveObjectsInRange( CFMutableArrayRef array, CFRange range );
void ArrayReplaceObjectAtIndex( CFMutableArrayRef array, CFTypeRef obj, NSUInteger index );
void ArrayReplaceObjectsAtIndexes( CFMutableArrayRef array, CFArrayRef objects, IndexSetRef indexes );
void ArraySetArray( CFMutableArrayRef array, CFArrayRef otherArray );

// Filter content
void ArrayFilterUsingPredicate( CFMutableArrayRef array, PredicateRef predicate );

// Rearrange content
void ArrayExchangeObjects( CFMutableArrayRef array, NSUInteger index1, NSUInteger index2 );
void ArraySortUsingDescriptors( CFMutableArrayRef array, CFArrayRef descriptors );
void ArraySortUsingFunction( CFMutableArrayRef array, CocoaArraySortedArrayFunctionType comparator, void *context );
void ArraySortUsingSelector( CFMutableArrayRef array, CFStringRef selector );

// - convenience -
void ArrayAddObjects( CFMutableArrayRef array, CFTypeRef obj, ... );

//#endif //ARRAY_H
