/*
 Array.m
 
 Bernie Wylde
 */

#import "Array.h"

CFArrayRef ArrayWithArray( CFArrayRef array )
{ return (CFArrayRef)[NSArray arrayWithArray:(NSArray *)array]; }

CFMutableArrayRef ArrayWithCapacity( NSUInteger numItems )
{ return (CFMutableArrayRef)[NSMutableArray arrayWithCapacity:numItems]; }

CFArrayRef ArrayWithContentsOfURL( CFURLRef url )
{ return (CFArrayRef)[NSMutableArray arrayWithContentsOfURL:(NSURL *)url]; }

CFArrayRef ArrayWithObject( CFTypeRef obj )
{ return (CFArrayRef)[NSArray arrayWithObject:(id)obj]; }

CFMutableArrayRef ArrayWithObjects( CFTypeRef obj, ... )
{
    va_list ap;
    va_start( ap, obj );
    NSMutableArray *array = [NSMutableArray arrayWithCapacity:0];
    while ( obj ) {
        [array addObject:obj];
        obj = va_arg( ap, id );
    }
    va_end( ap );
    return (CFMutableArrayRef)array;
}

Boolean ArrayContainsObject( CFArrayRef array, CFTypeRef obj )
{ return [(NSArray *)array containsObject:(id)obj]; }

NSUInteger ArrayCount( CFArrayRef array )
{ return [(NSArray *)array count]; }

CFTypeRef ArrayFirstObject( CFArrayRef array )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
    return (CFTypeRef)[(NSArray *)array firstObject];
#else
    return (CFTypeRef)[(NSArray *)array objectAtIndex:0];
#endif
}

CFTypeRef ArrayLastObject( CFArrayRef array )
{ return (CFTypeRef)[(NSArray *)array lastObject]; }

CFTypeRef ArrayObjectAtIndex( CFArrayRef array, NSUInteger index )
{ return (CFTypeRef)[(NSArray *)array objectAtIndex:index]; }

CFArrayRef ArrayObjectsAtIndexes( CFArrayRef array, IndexSetRef indexes )
{ return (CFArrayRef)[(NSArray *)array objectsAtIndexes:indexes]; }

EnumeratorRef ArrayObjectEnumerator( CFArrayRef array )
{ return [(NSArray *)array objectEnumerator]; }

EnumeratorRef ArrayReverseObjectEnumerator( CFArrayRef array )
{ return [(NSArray *)array reverseObjectEnumerator]; }

// - finding -
NSUInteger ArrayIndexOfObject( CFArrayRef array, CFTypeRef obj )
{ return [(NSArray *)array indexOfObject:(id)obj]; }

NSUInteger ArrayIndexOfObjectInRange( CFArrayRef array, CFTypeRef obj, CFRange range )
{ return [(NSArray *)array indexOfObject:(id)obj inRange:NSRangeFromCFRange(range)]; }

NSUInteger ArrayIndexOfObjectIdenticalTo( CFArrayRef array, CFTypeRef obj )
{ return [(NSArray *)array indexOfObjectIdenticalTo:(id)obj]; }

NSUInteger ArrayIndexOfObjectPassingTest( CFArrayRef array, CocoaArrayTestCallbackType testFn )
{
    return [(NSArray *)array indexOfObjectPassingTest:^(id obj, NSUInteger idx, BOOL *stop) {
        BOOL stopFlag = NO;
        BOOL result = (*testFn)((CFTypeRef)obj,idx,&stopFlag);
        if ( stopFlag ) *stop = YES;
        return result;
    }];
}

CFTypeRef ArrayFirstObjectCommonWithArray( CFArrayRef array1, CFArrayRef array2 )
{ return (CFTypeRef)[(NSArray *)array1 firstObjectCommonWithArray:(NSArray *)array2]; }

Boolean ArrayIsEqualToArray( CFArrayRef array1, CFArrayRef array2 )
{ return [(NSArray *)array1 isEqualToArray:(NSArray *)array2]; }

CFArrayRef ArrayByAddingObject( CFArrayRef array, CFTypeRef obj )
{ return (CFTypeRef)[(NSArray *)array arrayByAddingObject:(id)obj]; }

CFArrayRef ArrayByAddingObjectsFromArray( CFArrayRef array1, CFArrayRef array2 )
{ return (CFTypeRef)[(NSArray *)array1 arrayByAddingObjectsFromArray:(NSArray *)array2]; }

CFArrayRef ArrayFilteredArrayUsingPredicate( CFArrayRef array, PredicateRef pred )
{ return (CFArrayRef)[(NSArray *)array filteredArrayUsingPredicate:pred]; }

CFArrayRef ArraySubarrayWithRange( CFArrayRef array, CFRange range )
{ return (CFTypeRef)[(NSArray *)array subarrayWithRange:NSRangeFromCFRange(range)]; }

// Sending message to elements
void ArrayMakeObjectsPerformSelector( CFArrayRef array, CFStringRef selector )
{ [(NSArray *)array makeObjectsPerformSelector:NSSelectorFromString((NSString *)selector)]; }

void ArrayMakeObjectsPerformSelectorWithObject( CFArrayRef array, CFStringRef selector, CFTypeRef object )
{ [(NSArray *)array makeObjectsPerformSelector:NSSelectorFromString((NSString *)selector) withObject:(id)object]; }

void ArrayEnumerateObjectsUsingCallback( CFArrayRef array, CocoaArrayEnumerateObjectsCallbackType callback, void *userData )
{
    [(NSArray *)array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        (*callback)(array,(CFTypeRef)obj,idx,stop,userData);
    }];
}

void ArrayEnumerateObjectsWithOptionsUsingCallback( CFArrayRef array, NSEnumerationOptions options, CocoaArrayEnumerateObjectsCallbackType callback, void *userData )
{
    [(NSArray *)array enumerateObjectsWithOptions:options usingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        (*callback)(array,(CFTypeRef)obj,idx,stop,userData);
    }];
}

void ArrayEnumerateObjectsAtIndexesUsingCallback( CFArrayRef array, IndexSetRef indexSet, NSEnumerationOptions options, CocoaArrayEnumerateObjectsCallbackType callback, void *userData )
{
    [(NSArray *)array enumerateObjectsAtIndexes:indexSet options:options usingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        (*callback)(array,(CFTypeRef)obj,idx,stop,userData);
    }];
}

// Sort
CFDataRef ArraySortedArrayHint( CFArrayRef array )
{ return (CFDataRef)[(NSArray *)array sortedArrayHint]; }

CFArrayRef ArraySortedArrayUsingFunction( CFArrayRef array, CocoaArraySortedArrayFunctionType comparator, void *context )
{ return (CFArrayRef)[(NSArray *)array sortedArrayUsingFunction:(*comparator) context:context]; }

CFArrayRef ArraySortedArrayUsingFunctionHint( CFArrayRef array, CocoaArraySortedArrayFunctionType comparator, void *context, CFDataRef hint )
{ return (CFArrayRef)[(NSArray *)array sortedArrayUsingFunction:(*comparator) context:context hint:(NSData *)hint]; }

CFArrayRef ArraySortedArrayUsingDescriptors( CFArrayRef array, CFArrayRef descriptors )
{ return (CFArrayRef)[(NSArray *)array sortedArrayUsingDescriptors:(NSArray *)descriptors]; }

CFArrayRef ArraySortedArrayUsingSelector( CFArrayRef array, CFStringRef selector )
{ return (CFArrayRef)[(NSArray *)array sortedArrayUsingSelector:NSSelectorFromString((NSString *)selector)]; }

// Working with string elements -
CFStringRef ArrayComponentsJoinedByString( CFArrayRef array, CFStringRef separator )
{ return (CFStringRef)[(NSArray *)array componentsJoinedByString:(NSString *)separator]; }

// Description
CFStringRef ArrayDescription( CFArrayRef array )
{ return (CFStringRef)[(NSArray *)array description]; }

CFStringRef ArrayDescriptionWithLocale( CFArrayRef array, CFLocaleRef locale )
{ return (CFStringRef)[(NSArray *)array descriptionWithLocale:(NSLocale *)locale]; }

CFStringRef ArrayDescriptionWithLocaleIndent( CFArrayRef array, CFLocaleRef locale, NSUInteger indentLevel )
{ return (CFStringRef)[(NSArray *)array descriptionWithLocale:(NSLocale *)locale indent:indentLevel]; }

// Collecting paths
CFArrayRef ArrayPathsMatchingExtensions( CFArrayRef array, CFArrayRef extensions )
{ return (CFArrayRef)[(NSArray *)array pathsMatchingExtensions:(NSArray *)extensions]; }

// Key-value coding
void ArraySetValueForKey( CFArrayRef array, CFTypeRef value, CFStringRef key )
{ [(NSArray *)array setValue:(id)value forKey:(NSString *)key]; }

CFTypeRef ArrayValueForKey( CFArrayRef array, CFStringRef key )
{ return (CFArrayRef)[(NSArray *)array valueForKey:(NSString *)key]; }

// Shuffle
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFArrayRef ArrayShuffledArray( CFArrayRef array )
{
    if ( [(NSArray *)array respondsToSelector:@selector(shuffledArray)] ) {
        return (CFArrayRef)[(NSArray *)array shuffledArray];
    }
    return nil;
}
#endif

Boolean ArrayWriteToURL( CFArrayRef array, CFURLRef url, Boolean atomically )
{ return [(NSArray *)array writeToURL:(NSURL *)url atomically:atomically]; }

void ArrayAddObject( CFMutableArrayRef array, CFTypeRef obj )
{ [(NSMutableArray *)array addObject:(id)obj]; }

void ArrayAddObjectsFromArray( CFMutableArrayRef array, CFArrayRef otherArray )
{ [(NSMutableArray *)array addObjectsFromArray:(NSArray *)otherArray]; }

void ArrayInsertObjectAtIndex( CFMutableArrayRef array, CFTypeRef obj, NSUInteger index )
{ [(NSMutableArray *)array insertObject:(id)obj atIndex:index]; }

void ArrayInsertObjectsAtIndexes( CFMutableArrayRef array, CFArrayRef objects, IndexSetRef indexes )
{ [(NSMutableArray *)array insertObjects:(NSArray *)objects atIndexes:indexes]; }

void ArrayRemoveAllObjects( CFMutableArrayRef array )
{ [(NSMutableArray *)array removeAllObjects]; }

void ArrayRemoveLastObject( CFMutableArrayRef array )
{ [(NSMutableArray *)array removeLastObject]; }

void ArrayRemoveObject( CFMutableArrayRef array, CFTypeRef obj )
{ [(NSMutableArray *)array removeObject:(id)obj]; }

void ArrayRemoveObjectInRange( CFMutableArrayRef array, CFTypeRef obj, CFRange range )
{ [(NSMutableArray *)array removeObject:(id)obj inRange:NSRangeFromCFRange(range)]; }

void ArrayRemoveObjectAtIndex( CFMutableArrayRef array, NSUInteger index )
{ [(NSMutableArray *)array removeObjectAtIndex:index]; }

void ArrayRemoveObjectsAtIndexes( CFMutableArrayRef array, IndexSetRef indexes )
{ [(NSMutableArray *)array removeObjectsAtIndexes:indexes]; }

void ArrayRemoveObjectsInArray( CFMutableArrayRef array, CFArrayRef objects )
{ [(NSMutableArray *)array removeObjectsInArray:(NSArray *)objects]; }

void ArrayRemoveObjectsInRange( CFMutableArrayRef array, CFRange range )
{ [(NSMutableArray *)array removeObjectsInRange:NSRangeFromCFRange(range)]; }

void ArrayReplaceObjectAtIndex( CFMutableArrayRef array, CFTypeRef obj, NSUInteger index )
{ [(NSMutableArray *)array replaceObjectAtIndex:index withObject:(id)obj]; }

void ArrayReplaceObjectsAtIndexes( CFMutableArrayRef array, CFArrayRef objects, IndexSetRef indexes )
{ [(NSMutableArray *)array replaceObjectsAtIndexes:indexes withObjects:(NSArray *)objects]; }

void ArraySetArray( CFMutableArrayRef array, CFArrayRef otherArray )
{ [(NSMutableArray *)array setArray:(NSArray *)otherArray]; }

// Filter content
void ArrayFilterUsingPredicate( CFMutableArrayRef array, PredicateRef predicate )
{ [(NSMutableArray *)array filterUsingPredicate:predicate]; }

// Rearrange content
void ArrayExchangeObjects( CFMutableArrayRef array, NSUInteger index1, NSUInteger index2 )
{ [(NSMutableArray *)array exchangeObjectAtIndex:index1 withObjectAtIndex:index2]; }

void ArraySortUsingDescriptors( CFMutableArrayRef array, CFArrayRef descriptors )
{ [(NSMutableArray *)array sortUsingDescriptors:(NSArray *)descriptors]; }

void ArraySortUsingFunction( CFMutableArrayRef array, CocoaArraySortedArrayFunctionType comparator, void *context )
{ [(NSMutableArray *)array sortUsingFunction:(*comparator) context:context]; }

void ArraySortUsingSelector( CFMutableArrayRef array, CFStringRef selector )
{ [(NSMutableArray *)array sortUsingSelector:NSSelectorFromString((NSString *)selector)]; }

// - convenience -
void ArrayAddObjects( CFMutableArrayRef array, CFTypeRef obj, ... )
{
    va_list ap;
    va_start( ap, obj );
    while ( obj ) {
        [(NSMutableArray *)array addObject:obj];
        obj = va_arg( ap, id );
    }
    va_end( ap );
}

