/*
 AttributedString.h
 
 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef BOOL (*CocoaAttributedStringEnumerateCallbackType)(CFTypeRef,CFRange,void *);

// Create
CFAttributedStringRef AttributedStringWithData( CFDataRef dta, CFDictionaryRef options, CFDictionaryRef *attributes, ErrorRef *err );
CFAttributedStringRef AttributedStringWithDocFormat( CFDataRef dta, CFDictionaryRef *attributes );
CFAttributedStringRef AttributedStringWithHTML( CFDataRef dta, CFDictionaryRef *attributes );
CFAttributedStringRef AttributedStringWithHTMLBaseURL( CFDataRef dta, CFURLRef url, CFDictionaryRef *attributes );
CFAttributedStringRef AttributedStringWithHTMLOptions( CFDataRef dta, CFDictionaryRef options, CFDictionaryRef *attributes );
CFAttributedStringRef AttributedStringWithRTF( CFDataRef dta, CFDictionaryRef *attributes );
CFAttributedStringRef AttributedStringWithRTFD( CFDataRef dta, CFDictionaryRef *attributes );
CFAttributedStringRef AttributedStringWithRTFDFileWrapper( FileWrapperRef fw, CFDictionaryRef *attributes );
CFAttributedStringRef AttributedStringWithURL( CFURLRef url, CFDictionaryRef options, CFDictionaryRef *attributes, ErrorRef *err );
CFAttributedStringRef AttributedStringWithAttachment( TextAttachmentRef attachment );

// Character info
CFStringRef AttributedStringString( CFAttributedStringRef aString );
NSUInteger AttributedStringLength( CFAttributedStringRef aString );

// Attribute info
CFDictionaryRef AttributedStringAttributesAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
CFDictionaryRef AttributedStringAttributesAtIndexInRange( CFAttributedStringRef aString, NSUInteger index, CFRange *longestEffectiveRange, CFRange range );
CFTypeRef AttributedStringAttributeAtIndex( CFAttributedStringRef aString, CFStringRef attrName, NSUInteger index, CFRange *effectiveRange );
CFTypeRef AttributedStringAttributeAtIndexInRange( CFAttributedStringRef aString, CFStringRef attrName, NSUInteger index, CFRange *longestEffectiveRange, CFRange range );

// Compare
Boolean AttributedStringIsEqual( CFAttributedStringRef aString1, CFAttributedStringRef aString2 );

// Substring
CFAttributedStringRef AttributedStringAttributedSubstringFromRange( CFAttributedStringRef aString, CFRange range );

// Enumerate
void AttributedStringEnumerateAttributeInRange( CFAttributedStringRef aString, CFStringRef attrName, CFRange range, NSAttributedStringEnumerationOptions options, CocoaAttributedStringEnumerateCallbackType fnAddress, void *userData );
void AttributedStringEnumerateAttributesInRange( CFAttributedStringRef aString, CFRange range, NSAttributedStringEnumerationOptions options, CocoaAttributedStringEnumerateCallbackType fnAddress, void *userData );

// Font info
CFDictionaryRef AttributedStringFontAttributesInRange( CFAttributedStringRef aString, CFRange range );
CFDictionaryRef AttributedStringRulerAttributesInRange( CFAttributedStringRef aString, CFRange range );

// Linguistic units
CFRange AttributedStringDoubleClickAtIndex( CFAttributedStringRef aString, NSUInteger index );
NSUInteger AttributedStringLineBreakBeforeIndex( CFAttributedStringRef aString, NSUInteger index, CFRange withinRange );
NSUInteger AttributedStringLineBreakByHyphenatingBeforeIndex( CFAttributedStringRef aString, NSUInteger index, CFRange withinRange );
NSUInteger AttributedStringNextWordFromIndex( CFAttributedStringRef aString, NSUInteger index, Boolean forward );

// Ranges
NSInteger AttributedStringItemNumberInTextList( CFAttributedStringRef aString, TextListRef list, NSUInteger index );
CFRange AttributedStringRangeOfTextBlock( CFAttributedStringRef aString, TextBlockRef block, NSUInteger index );
CFRange AttributedStringRangeOfTextList( CFAttributedStringRef aString, TextListRef list, NSUInteger index );
CFRange AttributedStringRangeOfTextTable( CFAttributedStringRef aString, TextTableRef table, NSUInteger index );

// Data
CFDataRef AttributedStringDataFromRange( CFAttributedStringRef aString, CFRange range, CFDictionaryRef attributes, ErrorRef *err );
FileWrapperRef AttributedStringFileWrapperFromRange( CFAttributedStringRef aString, CFRange range, CFDictionaryRef attributes, ErrorRef *err );
CFDataRef AttributedStringDocFormatFromRange( CFAttributedStringRef aString, CFRange range, CFDictionaryRef attributes );
CFDataRef AttributedStringRTFFromRange( CFAttributedStringRef aString, CFRange range, CFDictionaryRef attributes );
CFDataRef AttributedStringRTFDFromRange( CFAttributedStringRef aString, CFRange range, CFDictionaryRef attributes );
FileWrapperRef AttributedStringRTFDFileWrapperFromRange( CFAttributedStringRef aString, CFRange range, CFDictionaryRef attributes );

// Draw
void AttributedStringDrawAtPoint( CFAttributedStringRef aString, CGPoint pt );
void AttributedStringDrawInRect( CFAttributedStringRef aString, CGRect r );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void AttributedStringDrawWithRect( CFAttributedStringRef aString, CGRect r, NSStringDrawingOptions options, StringDrawingContextRef ctx );
#endif

// Metrics
CGSize AttributedStringSize( CFAttributedStringRef aString );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGRect AttributedStringBoundingRectWithSize( CFAttributedStringRef aString, CGSize size, NSStringDrawingOptions options, StringDrawingContextRef ctx );
#endif
Boolean AttributedStringContainsAttachmentsInRange( CFAttributedStringRef aString, CFRange range );

// Data sources
CFArrayRef AttributedStringTextTypes( void );
CFArrayRef AttributedStringTextUnfilteredTypes( void );



/*
 Mutable attributed string
 */
// Create
CFMutableAttributedStringRef AttributedStringWithString( CFStringRef string );
CFMutableAttributedStringRef AttributedStringWithAttributes( CFStringRef string, CFDictionaryRef attributes );

// Character info
CFMutableStringRef AttributedStringMutableString( CFMutableAttributedStringRef aString );

// Change characters
void AttributedStringReplaceCharactersInRange( CFMutableAttributedStringRef aString, CFStringRef string, CFRange range );
void AttributedStringDeleteCharactersInRange( CFMutableAttributedStringRef aString, CFRange range );

// Change attributes
void AttributedStringSetAttributesInRange( CFMutableAttributedStringRef aString, CFDictionaryRef attrs, CFRange range );
void AttributedStringAddAttributeInRange( CFMutableAttributedStringRef aString, CFStringRef name, CFTypeRef value, CFRange range );
void AttributedStringAddAttributesInRange( CFMutableAttributedStringRef aString, CFDictionaryRef attrs, CFRange range );
void AttributedStringRemoveAttributeInRange( CFMutableAttributedStringRef aString, CFStringRef name, CFRange range );
void AttributedStringApplyFontTraitsInRange( CFMutableAttributedStringRef aString, NSFontTraitMask traitMask, CFRange range );
void AttributedStringSetAlignmentInRange( CFMutableAttributedStringRef aString, NSTextAlignment alignment, CFRange range );
void AttributedStringSetBaseWritingDirectionInRange( CFMutableAttributedStringRef aString, NSWritingDirection value, CFRange range );
void AttributedStringSubscriptRange( CFMutableAttributedStringRef aString, CFRange range );
void AttributedStringSuperscriptRange( CFMutableAttributedStringRef aString, CFRange range );
void AttributedStringUnscriptRange( CFMutableAttributedStringRef aString, CFRange range );

// Change characters and attributes
void AttributedStringAppendAttributedString( CFMutableAttributedStringRef aString, CFAttributedStringRef attrString );
void AttributedStringInsertAttributedString( CFMutableAttributedStringRef aString, CFAttributedStringRef attrString, NSUInteger index );
void AttributedStringReplaceCharactersInRangeWithAttributedString( CFMutableAttributedStringRef aString, CFRange range, CFAttributedStringRef attrString );
void AttributedStringSetAttributedString( CFMutableAttributedStringRef aString, CFAttributedStringRef attrString );

// Grouping changes
void AttributedStringBeginEditing( CFMutableAttributedStringRef aString );
void AttributedStringEndEditing( CFMutableAttributedStringRef aString );

// Update attachment contents

// Fix attributes after changes
void AttributedStringFixAttributesInRange( CFMutableAttributedStringRef aString, CFRange range );
void AttributedStringFixAttachmentAttributeInRange( CFMutableAttributedStringRef aString, CFRange range );
void AttributedStringFixFontAttributeInRange( CFMutableAttributedStringRef aString, CFRange range );
void AttributedStringFixParagraphStyleAttributeInRange( CFMutableAttributedStringRef aString, CFRange range );

// Read content
Boolean AttributedStringReadFromData( CFMutableAttributedStringRef aString, CFDataRef dta, CFDictionaryRef options, CFDictionaryRef *docAttributes, ErrorRef *err );
Boolean AttributedStringReadFromURL( CFMutableAttributedStringRef aString, CFURLRef url, CFDictionaryRef options, CFDictionaryRef *docAttributes, ErrorRef *err );


/*
 Convenience
 */
// string
void AttributedStringSetString( CFMutableAttributedStringRef as, CFStringRef string );

// append
void AttributedStringAppendCharacters( CFMutableAttributedStringRef as, CFStringRef string );

// font
FontRef AttributedStringFontAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetFontWithNameInRange( CFMutableAttributedStringRef as, CFStringRef name, CGFloat size, CFRange range );
void AttributedStringSetFontWithName( CFMutableAttributedStringRef as, CFStringRef name, CGFloat size );

// alignment
void AttributedStringSetAlignment( CFMutableAttributedStringRef as, NSTextAlignment alignment );

// stroke width
CGFloat AttributedStringStrokeWidthAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetStrokeWidthInRange( CFMutableAttributedStringRef as, CGFloat width, CFRange range );
void AttributedStringSetStrokeWidth( CFMutableAttributedStringRef as, CGFloat width );

// stroke color
ColorRef AttributedStringStrokeColorAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetStrokeColorInRange( CFMutableAttributedStringRef as, ColorRef col, CFRange range );
void AttributedStringSetStrokeColor( CFMutableAttributedStringRef as, ColorRef col );

// foreground color
ColorRef AttributedStringForegroundColorAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetForegroundColorInRange( CFMutableAttributedStringRef as, ColorRef col, CFRange range );
void AttributedStringSetForegroundColor( CFMutableAttributedStringRef as, ColorRef col );

// background color
ColorRef AttributedStringBackgroundColorAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetBackgroundColorInRange( CFMutableAttributedStringRef as, ColorRef col, CFRange range );
void AttributedStringSetBackgroundColor( CFMutableAttributedStringRef as, ColorRef col );

// shadow
ShadowRef AttributedStringShadowAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetShadowInRange( CFMutableAttributedStringRef as, CGSize offset, CGFloat blur, CFRange range );
void AttributedStringSetShadow( CFMutableAttributedStringRef as, CGSize offset, CGFloat blur );

// shadow color
ColorRef AttributedStringShadowColorAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetShadowColorInRange( CFMutableAttributedStringRef as, ColorRef col, CFRange range );
void AttributedStringSetShadowColor( CFMutableAttributedStringRef as, ColorRef col );

// underline
NSInteger AttributedStringUnderlineAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetUnderlineInRange( CFMutableAttributedStringRef as, NSInteger style, CFRange range );
void AttributedStringSetUnderline( CFMutableAttributedStringRef as, NSInteger style );

// underline color
ColorRef AttributedStringUnderlineColorAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetUnderlineColorInRange( CFMutableAttributedStringRef as, ColorRef col, CFRange range );
void AttributedStringSetUnderlineColor( CFMutableAttributedStringRef as, ColorRef col );

// strikethrough
NSInteger AttributedStringStrikethroughAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetStrikethroughInRange( CFMutableAttributedStringRef as, NSInteger style, CFRange range );
void AttributedStringSetStrikethrough( CFMutableAttributedStringRef as, NSInteger style );

// strikethrough color
ColorRef AttributedStringStrikethroughColorAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetStrikethroughColorInRange( CFMutableAttributedStringRef as, ColorRef col, CFRange range );
void AttributedStringSetStrikethroughColor( CFMutableAttributedStringRef as, ColorRef col );

// superscript
NSInteger AttributedStringSuperscriptAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetSuperscriptInRange( CFMutableAttributedStringRef as, NSInteger value, CFRange range );
void AttributedStringSetSuperscript( CFMutableAttributedStringRef as, NSInteger value );

// expansion
float AttributedStringExpansionAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetExpansionInRange( CFMutableAttributedStringRef as, float value, CFRange range );
void AttributedStringSetExpansion( CFMutableAttributedStringRef as, float value );

// obliqueness
float AttributedStringObliquenessAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetObliquenessInRange( CFMutableAttributedStringRef as, float value, CFRange range );
void AttributedStringSetObliqueness( CFMutableAttributedStringRef as, float value );

// link
CFTypeRef AttributedStringLinkAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetLinkInRange( CFMutableAttributedStringRef as, CFTypeRef value, CFRange range );
void AttributedStringSetLink( CFMutableAttributedStringRef as, CFTypeRef value );

// baseline offset
float AttributedStringBaselineOffsetAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetBaselineOffsetInRange( CFMutableAttributedStringRef as, float value, CFRange range );
void AttributedStringSetBaselineOffset( CFMutableAttributedStringRef as, float value );

// kerning
CGFloat AttributedStringKerningAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetKerningInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range );
void AttributedStringSetKerning( CFMutableAttributedStringRef as, CGFloat value );

// NSTextEffectAttributeName

// NSGlyphInfoAttributeName

// - paragraph styles -
NSMutableParagraphStyle *AttributedStringCopyParagraphStyle( CFMutableAttributedStringRef as, CFRange range );

// alignment
NSTextAlignment AttributedStringParagraphAlignmentAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetParagraphAlignmentInRange( CFMutableAttributedStringRef as, NSTextAlignment value, CFRange range );
void AttributedStringSetParagraphAlignment( CFMutableAttributedStringRef as, NSTextAlignment value );

// first line head indent
CGFloat AttributedStringFirstLineHeadIndentAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetFirstLineHeadIndentInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range );
void AttributedStringSetFirstLineHeadIndent( CFMutableAttributedStringRef as, CGFloat value );

// head indent
CGFloat AttributedStringHeadIndentAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetHeadIndentInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range );
void AttributedStringSetHeadIndent( CFMutableAttributedStringRef as, CGFloat value );

// tail indent
CGFloat AttributedStringTailIndentAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetTailIndentInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range );
void AttributedStringSetTailIndent( CFMutableAttributedStringRef as, CGFloat value );

// line break mode
NSLineBreakMode AttributedStringLineBreakModeAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetLIneBreakModeInRange( CFMutableAttributedStringRef as, NSLineBreakMode value, CFRange range );
void AttributedStringSetLIneBreakMode( CFMutableAttributedStringRef as, NSLineBreakMode value );

// maximum line height
CGFloat AttributedStringMaximumLineHeightAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetMaximumLineHeightInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range );
void AttributedStringSetMaximumLineHeight( CFMutableAttributedStringRef as, CGFloat value );

// minimum line height
CGFloat AttributedStringMinimumLineHeightAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetMinimumLineHeightInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range );
void AttributedStringSetMinimumLineHeight( CFMutableAttributedStringRef as, CGFloat value );

// line spacing
CGFloat AttributedStringLineSpacingAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetLineSpacingInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range );
void AttributedStringSetLineSpacing( CFMutableAttributedStringRef as, CGFloat value );

// paragraph spacing
CGFloat AttributedStringParagraphSpacingAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetParagraphSpacingInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range );
void AttributedStringSetParagraphSpacing( CFMutableAttributedStringRef as, CGFloat value );

// paragraph spacing before
CGFloat AttributedStringParagraphSpacingBeforeAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetParagraphSpacingBeforeInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range );
void AttributedStringSetParagraphSpacingBefore( CFMutableAttributedStringRef as, CGFloat value );

// base writing direction
//NSWritingDirection AttributedStringBaseWritingDirection( CFAttributedStringRef aString );
void AttributedStringSetBaseWritingDirectionInRange( CFMutableAttributedStringRef as, NSWritingDirection value, CFRange range );
void AttributedStringSetBaseWritingDirection( CFMutableAttributedStringRef as, NSWritingDirection value );

// line height multiple
CGFloat AttributedStringLineHeightMultipleAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetLineHeightMultipleInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range );
void AttributedStringSetLineHeightMultiple( CFMutableAttributedStringRef as, CGFloat value );

// add tab stop
void AttributedStringAddTabStopInRange( CFMutableAttributedStringRef as, NSTextAlignment alignment, CGFloat location, CFDictionaryRef options, CFRange range );
void AttributedStringAddTabStop( CFMutableAttributedStringRef as, NSTextAlignment alignment, CGFloat location, CFDictionaryRef options );

// set tab stops
CFArrayRef AttributedStringTabStopsAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetTabStopsInRange( CFMutableAttributedStringRef as, CFArrayRef tabStops, CFRange range );
void AttributedStringSetTabStops( CFMutableAttributedStringRef as, CFArrayRef tabStops );

// set default tab interval
CGFloat AttributedStringDefaultTabIntervalAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetDefaultTabIntervalInRange( CFMutableAttributedStringRef as, CGFloat interval, CFRange range );
void AttributedStringSetDefaultTabInterval( CFMutableAttributedStringRef as, CGFloat interval );

// header level
NSInteger AttributedStringHeaderLevelAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange );
void AttributedStringSetHeaderLevelInRange( CFMutableAttributedStringRef as, NSInteger value, CFRange range );
void AttributedStringSetHeaderLevel( CFMutableAttributedStringRef as, NSInteger value );

