/*
 AttributedString.M
 
 Bernie Wylde
 */

#import "AttributedString.h"

// Create
CFAttributedStringRef AttributedStringWithData( CFDataRef dta, CFDictionaryRef options, CFDictionaryRef *attributes, ErrorRef *err )
{ return (CFAttributedStringRef)[[[NSAttributedString alloc] initWithData:(NSData *)dta options:(NSDictionary *)options documentAttributes:(NSDictionary **)attributes error:err] autorelease]; }

CFAttributedStringRef AttributedStringWithDocFormat( CFDataRef dta, CFDictionaryRef *attributes )
{ return (CFAttributedStringRef)[[[NSAttributedString alloc] initWithDocFormat:(NSData *)dta documentAttributes:(NSDictionary **)attributes] autorelease]; }

CFAttributedStringRef AttributedStringWithHTML( CFDataRef dta, CFDictionaryRef *attributes )
{ return (CFAttributedStringRef)[[[NSAttributedString alloc] initWithHTML:(NSData *)dta documentAttributes:(NSDictionary **)attributes] autorelease]; }

CFAttributedStringRef AttributedStringWithHTMLBaseURL( CFDataRef dta, CFURLRef url, CFDictionaryRef *attributes )
{ return (CFAttributedStringRef)[[[NSAttributedString alloc] initWithHTML:(NSData *)dta baseURL:(NSURL *)url documentAttributes:(NSDictionary **)attributes] autorelease]; }

CFAttributedStringRef AttributedStringWithHTMLOptions( CFDataRef dta, CFDictionaryRef options, CFDictionaryRef *attributes )
{ return (CFAttributedStringRef)[[[NSAttributedString alloc] initWithHTML:(NSData *)dta options:(NSDictionary *)options documentAttributes:(NSDictionary **)attributes] autorelease]; }

CFAttributedStringRef AttributedStringWithRTF( CFDataRef dta, CFDictionaryRef *attributes )
{ return (CFAttributedStringRef)[[[NSAttributedString alloc] initWithRTF:(NSData *)dta documentAttributes:(NSDictionary **)attributes] autorelease]; }

CFAttributedStringRef AttributedStringWithRTFD( CFDataRef dta, CFDictionaryRef *attributes )
{ return (CFAttributedStringRef)[[[NSAttributedString alloc] initWithRTFD:(NSData *)dta documentAttributes:(NSDictionary **)attributes] autorelease]; }

CFAttributedStringRef AttributedStringWithRTFDFileWrapper( FileWrapperRef fw, CFDictionaryRef *attributes )
{ return (CFAttributedStringRef)[[[NSAttributedString alloc] initWithRTFDFileWrapper:fw documentAttributes:(NSDictionary **)attributes] autorelease]; }

CFAttributedStringRef AttributedStringWithURL( CFURLRef url, CFDictionaryRef options, CFDictionaryRef *attributes, ErrorRef *err )
{ return (CFAttributedStringRef)[[[NSAttributedString alloc] initWithURL:(NSURL *)url options:(NSDictionary *)options documentAttributes:(NSDictionary **)attributes error:err] autorelease]; }

CFAttributedStringRef AttributedStringWithAttachment( TextAttachmentRef attachment )
{ return (CFAttributedStringRef)[NSAttributedString attributedStringWithAttachment:attachment]; }

// Character info
CFStringRef AttributedStringString( CFAttributedStringRef aString )
{ return (CFStringRef)[(NSAttributedString *)aString string]; }

NSUInteger AttributedStringLength( CFAttributedStringRef aString )
{ return [(NSAttributedString *)aString length]; }

// Attribute info
CFDictionaryRef AttributedStringAttributesAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return (CFDictionaryRef)[(NSAttributedString *)aString attributesAtIndex:index effectiveRange:(NSRange *)effectiveRange]; }

CFDictionaryRef AttributedStringAttributesAtIndexInRange( CFAttributedStringRef aString, NSUInteger index, CFRange *longestEffectiveRange, CFRange range )
{ return (CFDictionaryRef)[(NSAttributedString *)aString attributesAtIndex:index longestEffectiveRange:(NSRange *)longestEffectiveRange inRange:NSMakeRange(range.location,range.length)]; }

CFTypeRef AttributedStringAttributeAtIndex( CFAttributedStringRef aString, CFStringRef attrName, NSUInteger index, CFRange *effectiveRange )
{ return (CFTypeRef)[(NSAttributedString *)aString attribute:(NSString *)attrName atIndex:index effectiveRange:(NSRange *)effectiveRange]; }

CFTypeRef AttributedStringAttributeAtIndexInRange( CFAttributedStringRef aString, CFStringRef attrName, NSUInteger index, CFRange *longestEffectiveRange, CFRange range )
{ return (CFTypeRef)[(NSAttributedString *)aString attribute:(NSString *)attrName atIndex:index longestEffectiveRange:(NSRange *)longestEffectiveRange inRange:NSMakeRange(range.location,range.length)]; }

// Compare
Boolean AttributedStringIsEqual( CFAttributedStringRef aString1, CFAttributedStringRef aString2 )
{ return [(NSAttributedString *)aString1 isEqualToAttributedString:(NSAttributedString *)aString2]; }

// Substring
CFAttributedStringRef AttributedStringAttributedSubstringFromRange( CFAttributedStringRef aString, CFRange range )
{ return (CFAttributedStringRef)[(NSAttributedString *)aString attributedSubstringFromRange:NSMakeRange(range.location,range.length)]; }

// Enumerate
void AttributedStringEnumerateAttributeInRange( CFAttributedStringRef aString, CFStringRef attrName, CFRange range, NSAttributedStringEnumerationOptions options, CocoaAttributedStringEnumerateCallbackType fnAddress, void *userData )
{
    [(NSAttributedString *)aString enumerateAttribute:(NSString *)attrName inRange:NSMakeRange(range.location,range.length) options:options usingBlock:^(id value, NSRange range, BOOL *stop) {
        if ( (*fnAddress)((CFTypeRef)value,CFRangeMake(range.location,range.length),userData) ) {
            *stop = YES;
        }
    }];
}

void AttributedStringEnumerateAttributesInRange( CFAttributedStringRef aString, CFRange range, NSAttributedStringEnumerationOptions options, CocoaAttributedStringEnumerateCallbackType fnAddress, void *userData )
{
    [(NSAttributedString *)aString enumerateAttributesInRange:NSMakeRange(range.location,range.length) options:options usingBlock:^(id attrs, NSRange range, BOOL *stop) {
        if ( (*fnAddress)((CFTypeRef)attrs,CFRangeMake(range.location,range.length),userData) ) {
            *stop = YES;
        }
    }];
}

// Font info
CFDictionaryRef AttributedStringFontAttributesInRange( CFAttributedStringRef aString, CFRange range )
{ return (CFDictionaryRef)[(NSAttributedString *)aString fontAttributesInRange:NSMakeRange(range.location,range.length)]; }

CFDictionaryRef AttributedStringRulerAttributesInRange( CFAttributedStringRef aString, CFRange range )
{ return (CFDictionaryRef)[(NSAttributedString *)aString rulerAttributesInRange:NSMakeRange(range.location,range.length)]; }

// Linguistic units
CFRange AttributedStringDoubleClickAtIndex( CFAttributedStringRef aString, NSUInteger index )
{
    NSRange range = [(NSAttributedString *)aString doubleClickAtIndex:index];
    return CFRangeMake(range.location,range.length);
}

NSUInteger AttributedStringLineBreakBeforeIndex( CFAttributedStringRef aString, NSUInteger index, CFRange withinRange )
{ return [(NSAttributedString *)aString lineBreakBeforeIndex:index withinRange:NSMakeRange(withinRange.location,withinRange.length)]; }

NSUInteger AttributedStringLineBreakByHyphenatingBeforeIndex( CFAttributedStringRef aString, NSUInteger index, CFRange withinRange )
{ return [(NSAttributedString *)aString lineBreakByHyphenatingBeforeIndex:index withinRange:NSMakeRange(withinRange.location,withinRange.length)]; }

NSUInteger AttributedStringNextWordFromIndex( CFAttributedStringRef aString, NSUInteger index, Boolean forward )
{ return [(NSAttributedString *)aString nextWordFromIndex:index forward:forward]; }

// Ranges
NSInteger AttributedStringItemNumberInTextList( CFAttributedStringRef aString, TextListRef list, NSUInteger index )
{ return [(NSAttributedString *)aString itemNumberInTextList:list atIndex:index]; }

CFRange AttributedStringRangeOfTextBlock( CFAttributedStringRef aString, TextBlockRef block, NSUInteger index )
{ return NSRangeToCFRange([(NSAttributedString *)aString rangeOfTextBlock:block atIndex:index]); }

CFRange AttributedStringRangeOfTextList( CFAttributedStringRef aString, TextListRef list, NSUInteger index )
{ return NSRangeToCFRange([(NSAttributedString *)aString rangeOfTextList:list atIndex:index]); }

CFRange AttributedStringRangeOfTextTable( CFAttributedStringRef aString, TextTableRef table, NSUInteger index )
{ return NSRangeToCFRange([(NSAttributedString *)aString rangeOfTextTable:table atIndex:index]); }

// Data
CFDataRef AttributedStringDataFromRange( CFAttributedStringRef aString, CFRange range, CFDictionaryRef attributes, ErrorRef *err )
{ return (CFDataRef)[(NSAttributedString *)aString dataFromRange:NSRangeFromCFRange(range) documentAttributes:(NSDictionary *)attributes error:err]; }

FileWrapperRef AttributedStringFileWrapperFromRange( CFAttributedStringRef aString, CFRange range, CFDictionaryRef attributes, ErrorRef *err )
{ return [(NSAttributedString *)aString fileWrapperFromRange:NSRangeFromCFRange(range) documentAttributes:(NSDictionary *)attributes error:err]; }

CFDataRef AttributedStringDocFormatFromRange( CFAttributedStringRef aString, CFRange range, CFDictionaryRef attributes )
{ return (CFDataRef)[(NSAttributedString *)aString docFormatFromRange:NSRangeFromCFRange(range) documentAttributes:(NSDictionary *)attributes]; }

CFDataRef AttributedStringRTFFromRange( CFAttributedStringRef aString, CFRange range, CFDictionaryRef attributes )
{ return (CFDataRef)[(NSAttributedString *)aString RTFFromRange:NSRangeFromCFRange(range) documentAttributes:(NSDictionary *)attributes]; }

CFDataRef AttributedStringRTFDFromRange( CFAttributedStringRef aString, CFRange range, CFDictionaryRef attributes )
{ return (CFDataRef)[(NSAttributedString *)aString RTFDFromRange:NSRangeFromCFRange(range) documentAttributes:(NSDictionary *)attributes]; }

FileWrapperRef AttributedStringRTFDFileWrapperFromRange( CFAttributedStringRef aString, CFRange range, CFDictionaryRef attributes )
{ return [(NSAttributedString *)aString RTFDFileWrapperFromRange:NSRangeFromCFRange(range) documentAttributes:(NSDictionary *)attributes]; }

// Draw
void AttributedStringDrawAtPoint( CFAttributedStringRef aString, CGPoint pt )
{ [(NSAttributedString *)aString drawAtPoint:NSPointFromCGPoint(pt)]; }

void AttributedStringDrawInRect( CFAttributedStringRef aString, CGRect r )
{ [(NSAttributedString *)aString drawInRect:NSRectFromCGRect(r)]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void AttributedStringDrawWithRect( CFAttributedStringRef aString, CGRect r, NSStringDrawingOptions options, StringDrawingContextRef ctx )
{ [(NSAttributedString *)aString drawWithRect:NSRectFromCGRect(r) options:options context:ctx]; }
#endif

// Metrics
CGSize AttributedStringSize( CFAttributedStringRef aString )
{ return NSSizeToCGSize( [(NSAttributedString *)aString size] ); }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGRect AttributedStringBoundingRectWithSize( CFAttributedStringRef aString, CGSize size, NSStringDrawingOptions options, StringDrawingContextRef ctx )
{ return NSRectToCGRect([(NSAttributedString *)aString boundingRectWithSize:NSSizeFromCGSize(size) options:options context:ctx]); }

Boolean AttributedStringContainsAttachmentsInRange( CFAttributedStringRef aString, CFRange range )
{ return [(NSAttributedString *)aString containsAttachmentsInRange:NSRangeFromCFRange(range)]; }
#endif

/*
 Mutable attributed string
 */
// - create -
CFMutableAttributedStringRef AttributedStringWithString( CFStringRef string )
{ return (CFMutableAttributedStringRef)[[[NSMutableAttributedString alloc] initWithString:(NSString *)string] autorelease]; }

CFMutableAttributedStringRef AttributedStringWithAttributes( CFStringRef string, CFDictionaryRef attributes )
{ return (CFMutableAttributedStringRef)[[[NSMutableAttributedString alloc] initWithString:(NSString *)string attributes:(NSDictionary *)attributes] autorelease]; }

// Character info
CFMutableStringRef AttributedStringMutableString( CFMutableAttributedStringRef aString )
{ return (CFMutableStringRef)[(NSMutableAttributedString *)aString mutableString]; }

// Change characters
void AttributedStringReplaceCharactersInRange( CFMutableAttributedStringRef aString, CFStringRef string, CFRange range )
{ [(NSMutableAttributedString *)aString replaceCharactersInRange:NSRangeFromCFRange(range) withString:(NSString *)string]; }

void AttributedStringDeleteCharactersInRange( CFMutableAttributedStringRef aString, CFRange range )
{ [(NSMutableAttributedString *)aString deleteCharactersInRange:NSMakeRange(range.location,range.length)]; }

// Change attributes
void AttributedStringSetAttributesInRange( CFMutableAttributedStringRef aString, CFDictionaryRef attrs, CFRange range )
{ [(NSMutableAttributedString *)aString setAttributes:(NSDictionary *)attrs range:NSMakeRange(range.location,range.length)]; }

void AttributedStringAddAttributeInRange( CFMutableAttributedStringRef aString, CFStringRef name, CFTypeRef value, CFRange range )
{ [(NSMutableAttributedString *)aString addAttribute:(NSString *)name value:(id)value range:NSMakeRange(range.location,range.length)]; }

void AttributedStringAddAttributesInRange( CFMutableAttributedStringRef aString, CFDictionaryRef attrs, CFRange range )
{ [(NSMutableAttributedString *)aString addAttributes:(NSDictionary *)attrs range:NSMakeRange(range.location,range.length)]; }

void AttributedStringRemoveAttributeInRange( CFMutableAttributedStringRef aString, CFStringRef name, CFRange range )
{ [(NSMutableAttributedString *)aString removeAttribute:(NSString *)name range:NSMakeRange(range.location,range.length)]; }

void AttributedStringApplyFontTraitsInRange( CFMutableAttributedStringRef aString, NSFontTraitMask traitMask, CFRange range )
{ [(NSMutableAttributedString *)aString applyFontTraits:traitMask range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetAlignmentInRange( CFMutableAttributedStringRef aString, NSTextAlignment alignment, CFRange range )
{ [(NSMutableAttributedString *)aString setAlignment:alignment range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetBaseWritingDirectionInRange( CFMutableAttributedStringRef aString, NSWritingDirection value, CFRange range )
{ [(NSMutableAttributedString *)aString setBaseWritingDirection:value range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSubscriptRange( CFMutableAttributedStringRef aString, CFRange range )
{ [(NSMutableAttributedString *)aString subscriptRange:NSMakeRange(range.location,range.length)]; }

void AttributedStringSuperscriptRange( CFMutableAttributedStringRef aString, CFRange range )
{ [(NSMutableAttributedString *)aString superscriptRange:NSMakeRange(range.location,range.length)]; }

void AttributedStringUnscriptRange( CFMutableAttributedStringRef aString, CFRange range )
{ [(NSMutableAttributedString *)aString unscriptRange:NSMakeRange(range.location,range.length)]; }

// Change characters and attributes
void AttributedStringAppendAttributedString( CFMutableAttributedStringRef aString, CFAttributedStringRef attrString )
{ [(NSMutableAttributedString *)aString appendAttributedString:(NSAttributedString *)attrString]; }

void AttributedStringInsertAttributedString( CFMutableAttributedStringRef aString, CFAttributedStringRef attrString, NSUInteger index )
{ [(NSMutableAttributedString *)aString insertAttributedString:(NSAttributedString *)attrString atIndex:index]; }

void AttributedStringReplaceCharactersInRangeWithAttributedString( CFMutableAttributedStringRef aString, CFRange range, CFAttributedStringRef attrString )
{ [(NSMutableAttributedString *)aString replaceCharactersInRange:NSMakeRange(range.location,range.length) withAttributedString:(NSAttributedString *)attrString]; }

void AttributedStringSetAttributedString( CFMutableAttributedStringRef aString, CFAttributedStringRef attrString )
{ [(NSMutableAttributedString *)aString setAttributedString:(NSAttributedString *)attrString]; }

// Grouping changes
void AttributedStringBeginEditing( CFMutableAttributedStringRef aString )
{ [(NSMutableAttributedString *)aString beginEditing]; }

void AttributedStringEndEditing( CFMutableAttributedStringRef aString )
{ [(NSMutableAttributedString *)aString endEditing]; }

// Update attachment contents

// Fix attributes after changes
void AttributedStringFixAttributesInRange( CFMutableAttributedStringRef aString, CFRange range )
{ [(NSMutableAttributedString *)aString fixAttributesInRange:NSMakeRange(range.location,range.length)]; }

void AttributedStringFixAttachmentAttributeInRange( CFMutableAttributedStringRef aString, CFRange range )
{ [(NSMutableAttributedString *)aString fixAttachmentAttributeInRange:NSMakeRange(range.location,range.length)]; }

void AttributedStringFixFontAttributeInRange( CFMutableAttributedStringRef aString, CFRange range )
{ [(NSMutableAttributedString *)aString fixFontAttributeInRange:NSMakeRange(range.location,range.length)]; }

void AttributedStringFixParagraphStyleAttributeInRange( CFMutableAttributedStringRef aString, CFRange range )
{ [(NSMutableAttributedString *)aString fixParagraphStyleAttributeInRange:NSMakeRange(range.location,range.length)]; }

// Read content
Boolean AttributedStringReadFromData( CFMutableAttributedStringRef aString, CFDataRef dta, CFDictionaryRef options, CFDictionaryRef *docAttributes, ErrorRef *err )
{ return [(NSMutableAttributedString *)aString readFromData:(NSData *)dta options:(NSDictionary *)options documentAttributes:(NSDictionary **)docAttributes error:err]; }

Boolean AttributedStringReadFromURL( CFMutableAttributedStringRef aString, CFURLRef url, CFDictionaryRef options, CFDictionaryRef *docAttributes, ErrorRef *err )
{ return [(NSMutableAttributedString *)aString readFromURL:(NSURL *)url options:(NSDictionary *)options documentAttributes:(NSDictionary **)docAttributes error:err]; }




// -- convenience --
// sring
void AttributedStringSetString( CFMutableAttributedStringRef as, CFStringRef string )
{ AttributedStringReplaceCharactersInRange( as, string, CFRangeMake(0,AttributedStringLength( as )) ); }

// append
void AttributedStringAppendCharacters( CFMutableAttributedStringRef as, CFStringRef string )
{ [(NSMutableAttributedString *)as replaceCharactersInRange:NSMakeRange([(NSMutableAttributedString *)as length],0) withString:(NSString *)string]; }

// - font -
FontRef AttributedStringFontAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [(NSMutableAttributedString *)aString attribute:NSFontAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange]; }

void AttributedStringSetFontWithNameInRange( CFMutableAttributedStringRef as, CFStringRef name, CGFloat size, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSFontAttributeName value:[NSFont fontWithName:(NSString *)name size:size] range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetFontWithName( CFMutableAttributedStringRef as, CFStringRef name, CGFloat size )
{ AttributedStringSetFontWithNameInRange( as, name, size, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// alignment
void AttributedStringSetAlignment( CFMutableAttributedStringRef as, NSTextAlignment alignment )
{ AttributedStringSetAlignmentInRange( as, alignment, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// - stroke -
CGFloat AttributedStringStrokeWidthAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [[(NSMutableAttributedString *)aString attribute:NSStrokeWidthAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange] floatValue]; }

void AttributedStringSetStrokeWidthInRange( CFMutableAttributedStringRef as, CGFloat width, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSStrokeWidthAttributeName value:[NSNumber numberWithFloat:width] range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetStrokeWidth( CFMutableAttributedStringRef as, CGFloat width )
{ AttributedStringSetStrokeWidthInRange( as, width, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

ColorRef AttributedStringStrokeColorAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [(NSMutableAttributedString *)aString attribute:NSStrokeColorAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange]; }

void AttributedStringSetStrokeColorInRange( CFMutableAttributedStringRef as, ColorRef col, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSStrokeColorAttributeName value:col range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetStrokeColor( CFMutableAttributedStringRef as, ColorRef col )
{ AttributedStringSetStrokeColorInRange( as, col, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// - foreground color -
ColorRef AttributedStringForegroundColorAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [(NSMutableAttributedString *)aString attribute:NSForegroundColorAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange]; }

void AttributedStringSetForegroundColorInRange( CFMutableAttributedStringRef as, ColorRef col, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSForegroundColorAttributeName value:col range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetForegroundColor( CFMutableAttributedStringRef as, ColorRef col )
{ AttributedStringSetForegroundColorInRange( as, col, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// - background color -
ColorRef AttributedStringBackgroundColorAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [(NSMutableAttributedString *)aString attribute:NSBackgroundColorAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange]; }

void AttributedStringSetBackgroundColorInRange( CFMutableAttributedStringRef as, ColorRef col, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSBackgroundColorAttributeName value:col range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetBackgroundColor( CFMutableAttributedStringRef as, ColorRef col )
{ AttributedStringSetBackgroundColorInRange( as, col, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// - shadow -
ShadowRef AttributedStringShadowAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [(NSMutableAttributedString *)aString attribute:NSShadowAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange]; }

void AttributedStringSetShadowInRange( CFMutableAttributedStringRef as, CGSize offset, CGFloat blur, CFRange range )
{
    NSShadow *shadow = [[NSShadow alloc] init];
    [shadow setShadowOffset:NSSizeFromCGSize(offset)];
    [shadow setShadowBlurRadius:blur];
    [(NSMutableAttributedString *)as addAttribute:NSShadowAttributeName value:shadow range:NSMakeRange(range.location,range.length)];
    [shadow release];
}

void AttributedStringSetShadow( CFMutableAttributedStringRef as, CGSize offset, CGFloat blur )
{ AttributedStringSetShadowInRange( as, offset, blur, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

ColorRef AttributedStringShadowColorAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSShadow *shadow = [(NSMutableAttributedString *)aString attribute:NSShadowAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [shadow shadowColor];
}

void AttributedStringSetShadowColorInRange( CFMutableAttributedStringRef as, ColorRef col, CFRange range )
{
    NSShadow *shadow = [(NSMutableAttributedString *)as attribute:NSShadowAttributeName atIndex:range.location effectiveRange:nil];
    if ( shadow ) {
        [shadow setShadowColor:col];
    } else {
        shadow = [[NSShadow alloc] init];
        [shadow setShadowColor:col];
        [(NSMutableAttributedString *)as addAttribute:NSShadowAttributeName value:shadow range:NSMakeRange(range.location,range.length)];
        [shadow release];
    }
}

void AttributedStringSetShadowColor( CFMutableAttributedStringRef as, ColorRef col )
{ AttributedStringSetShadowColorInRange( as, col, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// - underline -
NSInteger AttributedStringUnderlineAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [[(NSMutableAttributedString *)aString attribute:NSUnderlineStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange] integerValue]; }

void AttributedStringSetUnderlineInRange( CFMutableAttributedStringRef as, NSInteger style, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSUnderlineStyleAttributeName value:[NSNumber numberWithInteger:style] range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetUnderline( CFMutableAttributedStringRef as, NSInteger style )
{ AttributedStringSetUnderlineInRange( as, style, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

ColorRef AttributedStringUnderlineColorAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [(NSMutableAttributedString *)aString attribute:NSUnderlineColorAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange]; }

void AttributedStringSetUnderlineColorInRange( CFMutableAttributedStringRef as, ColorRef col, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSUnderlineColorAttributeName value:col range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetUnderlineColor( CFMutableAttributedStringRef as, ColorRef col )
{ AttributedStringSetUnderlineColorInRange( as, col, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// - strikethrough -
NSInteger AttributedStringStrikethroughAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [[(NSMutableAttributedString *)aString attribute:NSStrikethroughStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange] integerValue]; }

void AttributedStringSetStrikethroughInRange( CFMutableAttributedStringRef as, NSInteger style, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSStrikethroughStyleAttributeName value:[NSNumber numberWithInteger:style] range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetStrikethrough( CFMutableAttributedStringRef as, NSInteger style )
{ AttributedStringSetStrikethroughInRange( as, style, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

ColorRef AttributedStringStrikethroughColorAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [(NSMutableAttributedString *)aString attribute:NSStrikethroughColorAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange]; }

void AttributedStringSetStrikethroughColorInRange( CFMutableAttributedStringRef as, ColorRef col, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSStrikethroughColorAttributeName value:col range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetStrikethroughColor( CFMutableAttributedStringRef as, ColorRef col )
{ AttributedStringSetStrikethroughColorInRange( as, col, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// - superscript -
NSInteger AttributedStringSuperscriptAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [[(NSMutableAttributedString *)aString attribute:NSSuperscriptAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange] integerValue]; }

void AttributedStringSetSuperscriptInRange( CFMutableAttributedStringRef as, NSInteger value, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSSuperscriptAttributeName value:[NSNumber numberWithInteger:value] range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetSuperscript( CFMutableAttributedStringRef as, NSInteger value )
{ AttributedStringSetSuperscriptInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// - expansion -
float AttributedStringExpansionAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [[(NSMutableAttributedString *)aString attribute:NSExpansionAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange] floatValue]; }

void AttributedStringSetExpansionInRange( CFMutableAttributedStringRef as, float value, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSExpansionAttributeName value:[NSNumber numberWithFloat:value] range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetExpansion( CFMutableAttributedStringRef as, float value )
{ AttributedStringSetExpansionInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// - olbliqueness -
float AttributedStringObliquenessAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [[(NSMutableAttributedString *)aString attribute:NSObliquenessAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange] floatValue]; }

void AttributedStringSetObliquenessInRange( CFMutableAttributedStringRef as, float value, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSObliquenessAttributeName value:[NSNumber numberWithFloat:value] range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetObliqueness( CFMutableAttributedStringRef as, float value )
{ AttributedStringSetObliquenessInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// - link -
CFTypeRef AttributedStringLinkAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return (CFTypeRef)[(NSMutableAttributedString *)aString attribute:NSLinkAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange]; }

void AttributedStringSetLinkInRange( CFMutableAttributedStringRef as, CFTypeRef value, CFRange range )
{
    if ( CFGetTypeID( value ) == CFStringGetTypeID() ) {
        [(NSMutableAttributedString *)as addAttribute:NSLinkAttributeName value:(NSString *)value range:NSMakeRange(range.location,range.length)];
    } else if ( CFGetTypeID( value ) == CFURLGetTypeID() ) {
        [(NSMutableAttributedString *)as addAttribute:NSLinkAttributeName value:(NSURL *)value range:NSMakeRange(range.location,range.length)];
    }
}

void AttributedStringSetLink( CFMutableAttributedStringRef as, CFTypeRef value )
{ AttributedStringSetLinkInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// - baseline offset -
float AttributedStringBaselineOffsetAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [[(NSMutableAttributedString *)aString attribute:NSBaselineOffsetAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange] floatValue]; }

void AttributedStringSetBaselineOffsetInRange( CFMutableAttributedStringRef as, float value, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSBaselineOffsetAttributeName value:[NSNumber numberWithFloat:value] range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetBaselineOffset( CFMutableAttributedStringRef as, float value )
{ AttributedStringSetBaselineOffsetInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// - kerning -
CGFloat AttributedStringKerningAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [[(NSMutableAttributedString *)aString attribute:NSKernAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange] floatValue]; }

void AttributedStringSetKerningInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range )
{ [(NSMutableAttributedString *)as addAttribute:NSKernAttributeName value:[NSNumber numberWithFloat:value] range:NSMakeRange(range.location,range.length)]; }

void AttributedStringSetKerning( CFMutableAttributedStringRef as, CGFloat value )
{ AttributedStringSetKerningInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }


// - paragraph style -
NSMutableParagraphStyle *AttributedStringCopyParagraphStyle( CFMutableAttributedStringRef as, CFRange range )
{
    NSParagraphStyle *style = [(NSAttributedString *)as attribute:NSParagraphStyleAttributeName atIndex:range.location effectiveRange:nil];
    if ( !style ) style = [NSParagraphStyle defaultParagraphStyle];
    return [style mutableCopy];
}

// alignment
NSTextAlignment AttributedStringParagraphAlignmentAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{ return [[(NSMutableAttributedString *)aString attribute:NSObliquenessAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange] integerValue]; }

void AttributedStringSetParagraphAlignmentInRange( CFMutableAttributedStringRef as, NSTextAlignment value, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setAlignment:value];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetParagraphAlignment( CFMutableAttributedStringRef as, NSTextAlignment value )
{ AttributedStringSetParagraphAlignmentInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// first line head indent
CGFloat AttributedStringFirstLineHeadIndentAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [style firstLineHeadIndent];
}

void AttributedStringSetFirstLineHeadIndentInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setFirstLineHeadIndent:value];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetFirstLineHeadIndent( CFMutableAttributedStringRef as, CGFloat value )
{ AttributedStringSetFirstLineHeadIndentInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// head indent
CGFloat AttributedStringHeadIndentAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [style headIndent];
}

void AttributedStringSetHeadIndentInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setHeadIndent:value];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetHeadIndent( CFMutableAttributedStringRef as, CGFloat value )
{ AttributedStringSetHeadIndentInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// tail indent
CGFloat AttributedStringTailIndentAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [style tailIndent];
}

void AttributedStringSetTailIndentInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setTailIndent:value];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetTailIndent( CFMutableAttributedStringRef as, CGFloat value )
{ AttributedStringSetTailIndentInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// line break mode
NSLineBreakMode AttributedStringLineBreakModeAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [style lineBreakMode];
}

void AttributedStringSetLIneBreakModeInRange( CFMutableAttributedStringRef as, NSLineBreakMode value, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setLineBreakMode:value];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetLIneBreakMode( CFMutableAttributedStringRef as, NSLineBreakMode value )
{ AttributedStringSetLIneBreakModeInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// maximum line height
CGFloat AttributedStringMaximumLineHeightAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [style maximumLineHeight];
}

void AttributedStringSetMaximumLineHeightInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setMaximumLineHeight:value];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetMaximumLineHeight( CFMutableAttributedStringRef as, CGFloat value )
{ AttributedStringSetMaximumLineHeightInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// minimum line height
CGFloat AttributedStringMinimumLineHeightAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [style minimumLineHeight];
}

void AttributedStringSetMinimumLineHeightInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setMinimumLineHeight:value];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetMinimumLineHeight( CFMutableAttributedStringRef as, CGFloat value )
{ AttributedStringSetMinimumLineHeightInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// line spacing
CGFloat AttributedStringLineSpacingAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [style lineSpacing];
}

void AttributedStringSetLineSpacingInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setLineSpacing:value];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetLineSpacing( CFMutableAttributedStringRef as, CGFloat value )
{ AttributedStringSetLineSpacingInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// paragraph spacing
CGFloat AttributedStringParagraphSpacingAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [style paragraphSpacing];
}

void AttributedStringSetParagraphSpacingInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setParagraphSpacing:value];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetParagraphSpacing( CFMutableAttributedStringRef as, CGFloat value )
{ AttributedStringSetParagraphSpacingInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// paragraph spacing before
CGFloat AttributedStringParagraphSpacingBeforeAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [style paragraphSpacingBefore];
}

void AttributedStringSetParagraphSpacingBeforeInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setParagraphSpacingBefore:value];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetParagraphSpacingBefore( CFMutableAttributedStringRef as, CGFloat value )
{ AttributedStringSetParagraphSpacingBeforeInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// base writing direction
//NSWritingDirection AttributedStringBaseWritingDirection( CFAttributedStringRef aString )
//{ return [[(NSMutableAttributedString *)aString attribute:NSKernAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange] integerValue]; }

void AttributedStringSetBaseWritingDirection( CFMutableAttributedStringRef as, NSWritingDirection value )
{ AttributedStringSetBaseWritingDirectionInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// line height multiple
CGFloat AttributedStringLineHeightMultipleAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [style lineHeightMultiple];
}

void AttributedStringSetLineHeightMultipleInRange( CFMutableAttributedStringRef as, CGFloat value, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setLineHeightMultiple:value];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetLineHeightMultiple( CFMutableAttributedStringRef as, CGFloat value )
{ AttributedStringSetLineHeightMultipleInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// add tab stop
void AttributedStringAddTabStopInRange( CFMutableAttributedStringRef as, NSTextAlignment alignment, CGFloat location, CFDictionaryRef options, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    NSTextTab *tab = [[[NSTextTab alloc] initWithTextAlignment:alignment location:location options:(NSDictionary *)options] autorelease];
    [mutStyle addTabStop:tab];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringAddTabStop( CFMutableAttributedStringRef as, NSTextAlignment alignment, CGFloat location, CFDictionaryRef options )
{ AttributedStringAddTabStopInRange( as, alignment, location, options, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// set tab stops
CFArrayRef AttributedStringTabStopsAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return (CFArrayRef)[style tabStops];
}

void AttributedStringSetTabStopsInRange( CFMutableAttributedStringRef as, CFArrayRef tabStops, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setTabStops:(NSArray *)tabStops];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetTabStops( CFMutableAttributedStringRef as, CFArrayRef tabStops )
{ AttributedStringSetTabStopsInRange( as, tabStops, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// set default tab interval
CGFloat AttributedStringDefaultTabIntervalAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [style defaultTabInterval];
}

void AttributedStringSetDefaultTabIntervalInRange( CFMutableAttributedStringRef as, CGFloat interval, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setDefaultTabInterval:interval];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetDefaultTabInterval( CFMutableAttributedStringRef as, CGFloat interval )
{ AttributedStringSetDefaultTabIntervalInRange( as, interval, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }

// header level
NSInteger AttributedStringHeaderLevelAtIndex( CFAttributedStringRef aString, NSUInteger index, CFRange *effectiveRange )
{
    NSParagraphStyle *style = [(NSMutableAttributedString *)aString attribute:NSParagraphStyleAttributeName atIndex:index effectiveRange:(NSRange *)effectiveRange];
    return [style headerLevel];
}

void AttributedStringSetHeaderLevelInRange( CFMutableAttributedStringRef as, NSInteger value, CFRange range )
{
    NSMutableParagraphStyle *mutStyle = AttributedStringCopyParagraphStyle( as, range );
    [mutStyle setHeaderLevel:value];
    [(NSMutableAttributedString *)as addAttribute:NSParagraphStyleAttributeName value:mutStyle range:NSMakeRange(range.location,range.length)];
    [mutStyle release];
}

void AttributedStringSetHeaderLevel( CFMutableAttributedStringRef as, NSInteger value )
{ AttributedStringSetHeaderLevelInRange( as, value, CFRangeMake(0,CFAttributedStringGetLength(as)) ); }



