/*
 BezierPath.h
 
 Bernie Wylde
 */

#import <AppKit/AppKit.h>
#import "AffineTransform.h"

BezierPathRef BezierPathInit( void );
BezierPathRef BezierPathWithOvalInRect( CGRect r );
BezierPathRef BezierPathWithRect( CGRect r );
BezierPathRef BezierPathWithRoundedRect( CGRect r, CGFloat xRadius, CGFloat yRadius );
BezierPathRef BezierPathByFlatteningPath( BezierPathRef path );
BezierPathRef BezierPathByReversingPath( BezierPathRef path );

void BezierPathMoveToPoint( BezierPathRef path, CGPoint pt );
void BezierPathLineToPoint( BezierPathRef path, CGPoint pt );
void BezierPathCurveToPoint( BezierPathRef path, CGPoint pt, CGPoint cp1, CGPoint cp2 );
void BezierPathClose( BezierPathRef path );
void BezierPathRelativeMoveToPoint( BezierPathRef path, CGPoint pt );
void BezierPathRelativeLineToPoint( BezierPathRef path, CGPoint pt );
void BezierPathRelativeCurveToPoint( BezierPathRef path, CGPoint pt, CGPoint cp1, CGPoint cp2 );

void BezierPathAppendPath( BezierPathRef path1, BezierPathRef path2 );
void BezierPathAppendPathWithPoints( BezierPathRef path, CGPoint *points, NSInteger count );
void BezierPathAppendPathWithOvalInRect( BezierPathRef path, CGRect r );
void BezierPathAppendPathWithArcFromPoint( BezierPathRef path, CGPoint pt1, CGPoint pt2, CGFloat radius );
void BezierPathAppendPathWithArcWithCenter( BezierPathRef path, CGPoint center, CGFloat radius, CGFloat startAngle, CGFloat endAngle, Boolean clockwise );
//
// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
// void BezierPathAppendPathWithCGGlyph( BezierPathRef path, CGGlyph glyph, CTFontRef font );
// //void BezierPathAppendPathWithCGGlyphs( BezierPathRef path1, void *glyphs, NSInteger count, CTFontRef font );
// #endif
//
void BezierPathAppendPathWithRect( BezierPathRef path, CGRect r );
void BezierPathAppendPathWithRoundedRect( BezierPathRef path, CGRect r, CGFloat xRadius, CGFloat yRadius );

void BezierPathSetWindingRule( BezierPathRef path, NSWindingRule rule );
void BezierPathSetLineCapStyle( BezierPathRef path, NSLineCapStyle style );
void BezierPathSetLineJoinStyle( BezierPathRef path, NSLineJoinStyle style );
void BezierPathSetLineWidth( BezierPathRef path, CGFloat lineWidth );
void BezierPathSetMiterLimit( BezierPathRef path, CGFloat limit );
void BezierPathSetFlatness( BezierPathRef path, CGFloat flatness );
void BezierPathGetLineDash( BezierPathRef path, CGFloat *pattern, NSInteger *count, CGFloat *phase );
void BezierPathSetLineDash( BezierPathRef path, CGFloat *pattern, NSInteger count, CGFloat phase );

NSWindingRule BezierPathDefaultWindingRule( void );
void BezierPathSetDefaultWindingRule( NSWindingRule rule );
NSLineCapStyle BezierPathDefaultLineCapStyle( void );
void BezierPathSetDefaultLineCapStyle( NSLineCapStyle style );
NSLineJoinStyle BezierPathDefaultLineJoinStyle( void );
void BezierPathSetDefaultLineJoinStyle( NSLineJoinStyle style );
CGFloat BezierPathDefaultLineWidth( void );
void BezierPathSetDefaultLineWidth( CGFloat lineWidth );
CGFloat BezierPathDefaultMiterLimit( void );
void BezierPathSetDefaultMiterLimit( CGFloat limit );
CGFloat BezierPathDefaultFlatness( void );
void BezierPathSetDefaultFlatness( CGFloat flatness );

void BezierPathStroke( BezierPathRef path );
void BezierPathFill( BezierPathRef path );
//+ drawPackedGlyphs:atPoint:

void BezierPathAddClip( BezierPathRef path );
void BezierPathSetClip( BezierPathRef path );
void BezierPathClipRect( CGRect r );

Boolean BezierPathContainsPoint( BezierPathRef path, CGPoint pt );

CGRect BezierPathBounds( BezierPathRef path );
CGRect BezierPathControlPointBounds( BezierPathRef path );
CGPoint BezierPathCurrentPoint( BezierPathRef path );
Boolean BezierPathIsEmpty( BezierPathRef path );

void BezierPathTransformUsingAffineTranform( BezierPathRef path, AffineTransformRef tx );

NSInteger BezierPathElementCount( BezierPathRef path );
NSBezierPathElement BezierPathElementAtIndex( BezierPathRef path, NSInteger index );
NSBezierPathElement BezierPathElementAtIndexAssociatedPoints( BezierPathRef path, NSInteger index, CGPoint *points );
void BezierPathRemoveAllPoints( BezierPathRef path );
void BezierPathSetAssociatedPointsAtIndex( BezierPathRef path, CGPoint *points, NSInteger index );


#pragma mark - convenience
BezierPathRef BezierPathWithRotatedRect( CGRect r, CGFloat x, CGFloat y, CGFloat angle );

void BezierPathStrokeFill( BezierPathRef path, CGFloat lineWidth, ColorRef strokeCol, ColorRef fillCol );

void BezierPathStrokeFillOvalInRect( CGRect r, CGFloat lineWidth, ColorRef strokeCol, ColorRef fillCol );
void BezierPathStrokeFillRect( CGRect r, CGFloat lineWidth, ColorRef strokeCol, ColorRef fillCol );
void BezierPathStrokeFillRoundedRect( CGRect r, CGFloat xRadius, CGFloat yRadius, CGFloat lineWidth, ColorRef strokeCol, ColorRef fillCol );
void BezierPathStrokeFillPolygon( CFArrayRef pts, CGFloat lineWidth, ColorRef strokeCol, ColorRef fillCol );
//void BezierPathStrokeFillRotatedRect( CGRect r, CGFloat x, CGFloat y, CGFloat angle, CGFloat lineWidth, ColorRef strokeCol, ColorRef fillCol );

void BezierPathStrokeOvalInRect( CGRect r, CGFloat lineWidth, ColorRef strokeCol );
void BezierPathStrokeRect( CGRect r, CGFloat lineWidth, ColorRef strokeCol );
void BezierPathStrokeRoundedRect( CGRect r, CGFloat xRadius, CGFloat yRadius, CGFloat lineWidth, ColorRef strokeCol );
void BezierPathStrokeLine( CGPoint pt1, CGPoint pt2, CGFloat lineWidth, ColorRef strokeCol );
void BezierPathStrokeCurve( CGPoint pt1, CGPoint pt2, CGPoint cp1, CGPoint cp2, CGFloat lineWidth, ColorRef strokeCol );
void BezierPathStrokePolygon( CFArrayRef pts, CGFloat lineWidth, ColorRef strokeCol );
void BezierPathStrokeRotatedRect( CGRect r, CGFloat x, CGFloat y, CGFloat angle, CGFloat lineWidth, ColorRef strokeCol );

void BezierPathFillOvalInRect( CGRect r, ColorRef fillCol );
void BezierPathFillRect( CGRect r, ColorRef fillCol );
void BezierPathFillRoundedRect( CGRect r, CGFloat xRadius, CGFloat yRadius, ColorRef fillCol );
void BezierPathFillPolygon( CFArrayRef pts, ColorRef fillCol );
//void BezierPathFillRotatedRect( CGRect r, CGFloat x, CGFloat y, CGFloat angle, ColorRef fillCol );
