/*
 BezierPath.m
 
 Bernie Wylde
 */

#import "BezierPath.h"

#pragma mark - bezierpath
///////////////////////////////////////////////////
//              bezierpath                       //
///////////////////////////////////////////////////
BezierPathRef BezierPathInit( void )
{ return [NSBezierPath bezierPath]; }

BezierPathRef BezierPathWithOvalInRect( CGRect r )
{ return [NSBezierPath bezierPathWithOvalInRect:NSRectFromCGRect(r)]; }

BezierPathRef BezierPathWithRect( CGRect r )
{ return [NSBezierPath bezierPathWithRect:NSRectFromCGRect(r)]; }

BezierPathRef BezierPathWithRoundedRect( CGRect r, CGFloat xRadius, CGFloat yRadius )
{ return [NSBezierPath bezierPathWithRoundedRect:NSRectFromCGRect(r) xRadius:xRadius yRadius:yRadius]; }

BezierPathRef BezierPathByFlatteningPath( BezierPathRef path )
{ return [path bezierPathByFlatteningPath]; }

BezierPathRef BezierPathByReversingPath( BezierPathRef path )
{ return [path bezierPathByReversingPath]; }

void BezierPathMoveToPoint( BezierPathRef path, CGPoint pt )
{ [path moveToPoint:NSPointFromCGPoint(pt)]; }

void BezierPathLineToPoint( BezierPathRef path, CGPoint pt )
{ [path lineToPoint:NSPointFromCGPoint(pt)]; }

void BezierPathCurveToPoint( BezierPathRef path, CGPoint pt, CGPoint cp1, CGPoint cp2 )
{ [path curveToPoint:NSPointFromCGPoint(pt) controlPoint1:NSPointFromCGPoint(cp1) controlPoint2:NSPointFromCGPoint(cp2)]; }

void BezierPathClose( BezierPathRef path )
{ [path closePath]; }

void BezierPathRelativeMoveToPoint( BezierPathRef path, CGPoint pt )
{ [path relativeMoveToPoint:NSPointFromCGPoint(pt)]; }

void BezierPathRelativeLineToPoint( BezierPathRef path, CGPoint pt )
{ [path relativeLineToPoint:NSPointFromCGPoint(pt)]; }

void BezierPathRelativeCurveToPoint( BezierPathRef path, CGPoint pt, CGPoint cp1, CGPoint cp2 )
{ [path relativeCurveToPoint:NSPointFromCGPoint(pt) controlPoint1:NSPointFromCGPoint(cp1) controlPoint2:NSPointFromCGPoint(cp2)]; }

void BezierPathAppendPath( BezierPathRef path1, BezierPathRef path2 )
{ [path1 appendBezierPath:path2]; }

void BezierPathAppendPathWithPoints( BezierPathRef path, CGPoint *points, NSInteger count )
{ [path appendBezierPathWithPoints:(NSPointArray)points count:count]; }

void BezierPathAppendPathWithOvalInRect( BezierPathRef path, CGRect r )
{ [path appendBezierPathWithOvalInRect:NSRectFromCGRect(r)]; }

void BezierPathAppendPathWithArcFromPoint( BezierPathRef path, CGPoint pt1, CGPoint pt2, CGFloat radius )
{ [path appendBezierPathWithArcFromPoint:NSPointFromCGPoint(pt1) toPoint:NSPointFromCGPoint(pt2) radius:radius]; }

void BezierPathAppendPathWithArcWithCenter( BezierPathRef path, CGPoint center, CGFloat radius, CGFloat startAngle, CGFloat endAngle, Boolean clockwise )
{ [path appendBezierPathWithArcWithCenter:NSPointFromCGPoint(center) radius:radius startAngle:startAngle endAngle:endAngle clockwise:clockwise]; }


// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
// void BezierPathAppendPathWithCGGlyph( BezierPathRef path, CGGlyph glyph, CTFontRef font )
// { [path appendBezierPathWithCGGlyph:glyph inFont:(NSFont *)font]; }
// #endif


void BezierPathAppendPathWithRect( BezierPathRef path, CGRect r )
{ [path appendBezierPathWithRect:NSRectFromCGRect(r)]; }

void BezierPathAppendPathWithRoundedRect( BezierPathRef path, CGRect r, CGFloat xRadius, CGFloat yRadius )
{ [path appendBezierPathWithRoundedRect:NSRectFromCGRect(r) xRadius:xRadius yRadius:yRadius]; }

void BezierPathSetWindingRule( BezierPathRef path, NSWindingRule rule )
{ [path setWindingRule:rule]; }

void BezierPathSetLineCapStyle( BezierPathRef path, NSLineCapStyle style )
{ [path setLineCapStyle:style]; }

void BezierPathSetLineJoinStyle( BezierPathRef path, NSLineJoinStyle style )
{ [path setLineJoinStyle:style]; }

void BezierPathSetLineWidth( BezierPathRef path, CGFloat lineWidth )
{ [path setLineWidth:lineWidth]; }

void BezierPathSetMiterLimit( BezierPathRef path, CGFloat limit )
{ [path setMiterLimit:limit]; }

void BezierPathSetFlatness( BezierPathRef path, CGFloat flatness )
{ [path setFlatness:flatness]; }

void BezierPathGetLineDash( BezierPathRef path, CGFloat *pattern, NSInteger *count, CGFloat *phase )
{ [path getLineDash:pattern count:count phase:phase]; }

void BezierPathSetLineDash( BezierPathRef path, CGFloat *pattern, NSInteger count, CGFloat phase )
{ [path setLineDash:pattern count:count phase:phase]; }

NSWindingRule BezierPathDefaultWindingRule( void )
{ return [NSBezierPath defaultWindingRule]; }

void BezierPathSetDefaultWindingRule( NSWindingRule rule )
{ [NSBezierPath setDefaultWindingRule:rule]; }

NSLineCapStyle BezierPathDefaultLineCapStyle( void )
{ return [NSBezierPath defaultLineCapStyle]; }

void BezierPathSetDefaultLineCapStyle( NSLineCapStyle style )
{ [NSBezierPath setDefaultLineCapStyle:style]; }

NSLineJoinStyle BezierPathDefaultLineJoinStyle( void )
{ return [NSBezierPath defaultLineJoinStyle]; }

void BezierPathSetDefaultLineJoinStyle( NSLineJoinStyle style )
{ [NSBezierPath setDefaultLineJoinStyle:style]; }

CGFloat BezierPathDefaultLineWidth( void )
{ return [NSBezierPath defaultLineWidth]; }

void BezierPathSetDefaultLineWidth( CGFloat lineWidth )
{ [NSBezierPath setDefaultLineWidth:lineWidth]; }

CGFloat BezierPathDefaultMiterLimit( void )
{ return [NSBezierPath defaultMiterLimit]; }

void BezierPathSetDefaultMiterLimit( CGFloat limit )
{ [NSBezierPath setDefaultMiterLimit:limit]; }

CGFloat BezierPathDefaultFlatness( void )
{ return [NSBezierPath defaultFlatness]; }

void BezierPathSetDefaultFlatness( CGFloat flatness )
{ [NSBezierPath setDefaultFlatness:flatness]; }

void BezierPathStroke( BezierPathRef path )
{ [path stroke]; }

void BezierPathFill( BezierPathRef path )
{ [path fill]; }

void BezierPathAddClip( BezierPathRef path )
{ [path addClip]; }

void BezierPathSetClip( BezierPathRef path )
{ [path setClip]; }

void BezierPathClipRect( CGRect r )
{ [NSBezierPath clipRect:NSRectFromCGRect(r)]; }

Boolean BezierPathContainsPoint( BezierPathRef path, CGPoint pt )
{ return [path containsPoint:NSPointFromCGPoint(pt)]; }

CGRect BezierPathBounds( BezierPathRef path )
{ return NSRectToCGRect( [path bounds] ); }

CGRect BezierPathControlPointBounds( BezierPathRef path )
{ return NSRectToCGRect( [path controlPointBounds] ); }

CGPoint BezierPathCurrentPoint( BezierPathRef path )
{ return NSPointToCGPoint([path currentPoint]); }

Boolean BezierPathIsEmpty( BezierPathRef path )
{ return [path isEmpty]; }

void BezierPathTransformUsingAffineTranform( BezierPathRef path, AffineTransformRef tx )
{ [path transformUsingAffineTransform:tx]; }

NSInteger BezierPathElementCount( BezierPathRef path )
{ return [path elementCount]; }

NSBezierPathElement BezierPathElementAtIndex( BezierPathRef path, NSInteger index )
{ return [path elementAtIndex:index]; }

NSBezierPathElement BezierPathElementAtIndexAssociatedPoints( BezierPathRef path, NSInteger index, CGPoint *points )
{ return [path elementAtIndex:index associatedPoints:(NSPointArray)points]; }

void BezierPathRemoveAllPoints( BezierPathRef path )
{ [path removeAllPoints]; }

void BezierPathSetAssociatedPointsAtIndex( BezierPathRef path, CGPoint *points, NSInteger index )
{ [path setAssociatedPoints:(NSPointArray)points atIndex:index]; }


#pragma mark - convenience
BezierPathRef BezierPathWithRotatedRect( CGRect r, CGFloat x, CGFloat y, CGFloat angle )
{
    NSBezierPath *path = [NSBezierPath  bezierPathWithRect:NSRectFromCGRect(r)];
    AffineTransformRef tx = AffineTransformInit();
    AffineTransformTranslate( tx, x, y );
    AffineTransformRotateByDegrees( tx, angle );
    AffineTransformConcat( tx );
    [path transformUsingAffineTransform:tx];
    AffineTransformInvert( tx );
    AffineTransformConcat( tx );
    return path;
}



void BezierPathStrokeFill( BezierPathRef path, CGFloat lineWidth, ColorRef strokeCol, ColorRef fillCol )
{
    if ( fillCol ) {
        ColorSetFill( fillCol );
        BezierPathFill( path );
    }
    if ( lineWidth ) {
        if ( strokeCol ) {
            ColorSetStroke( strokeCol );
        } else {
            ColorSetStrokeBlack();
        }
        BezierPathSetLineWidth( path, lineWidth );
        BezierPathStroke( path );
    }
}

void BezierPathStrokeFillOvalInRect( CGRect r, CGFloat lineWidth, ColorRef strokeCol, ColorRef fillCol )
{ BezierPathStrokeFill( BezierPathWithOvalInRect(r), lineWidth, strokeCol, fillCol ); }

void BezierPathStrokeFillRect( CGRect r, CGFloat lineWidth, ColorRef strokeCol, ColorRef fillCol )
{ BezierPathStrokeFill( BezierPathWithRect(r), lineWidth, strokeCol, fillCol );}

void BezierPathStrokeFillRoundedRect( CGRect r, CGFloat xRadius, CGFloat yRadius, CGFloat lineWidth, ColorRef strokeCol, ColorRef fillCol )
{ BezierPathStrokeFill( BezierPathWithRoundedRect(r,xRadius,yRadius), lineWidth, strokeCol, fillCol ); }

void BezierPathStrokeFillPolygon( CFArrayRef pts, CGFloat lineWidth, ColorRef strokeCol, ColorRef fillCol )
{
    NSArray *array = (NSArray *)pts;
    NSUInteger count = [array count];
    BezierPathRef path = BezierPathInit();
    CGPoint pt = NSPointToCGPoint([[array objectAtIndex:0] pointValue]);
    BezierPathMoveToPoint( path, pt );
    for ( NSUInteger index = 1; index < count; index++ ) {
        pt = NSPointToCGPoint([[array objectAtIndex:index] pointValue]);
        BezierPathLineToPoint( path, pt );
    }
    BezierPathClose( path );
    BezierPathStrokeFill( path, lineWidth, strokeCol, fillCol );
}

//void BezierPathStrokeFillRotatedRect( CGRect r, CGFloat x, CGFloat y, CGFloat angle, CGFloat lineWidth, ColorRef strokeCol, ColorRef fillCol )
//{
//    AffineTransformRef tx = AffineTransformInit();
//    AffineTransformTranslate( tx, x, y );
//    AffineTransformRotateByDegrees( tx, angle );
//    AffineTransformConcat( tx );
//    BezierPathStrokeFillRect( r, lineWidth, strokeCol, fillCol );
//    AffineTransformInvert( tx );
//    AffineTransformConcat( tx );
//}

void BezierPathStrokeOvalInRect( CGRect r, CGFloat lineWidth, ColorRef strokeCol )
{ BezierPathStrokeFill( BezierPathWithOvalInRect(r), lineWidth, strokeCol, nil ); }

void BezierPathStrokeRect( CGRect r, CGFloat lineWidth, ColorRef strokeCol )
{ BezierPathStrokeFill( BezierPathWithRect(r), lineWidth, strokeCol, nil ); }

void BezierPathStrokeRoundedRect( CGRect r, CGFloat xRadius, CGFloat yRadius, CGFloat lineWidth, ColorRef strokeCol )
{ BezierPathStrokeFill(  BezierPathWithRoundedRect(r,xRadius,yRadius), lineWidth, strokeCol, nil ); }

void BezierPathStrokeRotatedRect( CGRect r, CGFloat x, CGFloat y, CGFloat angle, CGFloat lineWidth, ColorRef strokeCol )
{
    AffineTransformRef tx = AffineTransformInit();
    AffineTransformTranslate( tx, x, y );
    AffineTransformRotateByDegrees( tx, angle );
    AffineTransformConcat( tx );
    BezierPathStrokeRect( r, lineWidth, strokeCol );
    AffineTransformInvert( tx );
    AffineTransformConcat( tx );
}

void BezierPathStrokeLine( CGPoint pt1, CGPoint pt2, CGFloat lineWidth, ColorRef strokeCol )
{
    BezierPathRef path = BezierPathInit();
    BezierPathMoveToPoint( path, pt1 );
    BezierPathLineToPoint( path, pt2 );
    BezierPathStrokeFill( path, lineWidth, strokeCol, nil );
}

void BezierPathStrokeCurve( CGPoint pt1, CGPoint pt2, CGPoint cp1, CGPoint cp2, CGFloat lineWidth, ColorRef strokeCol )
{
    BezierPathRef path = BezierPathInit();
    BezierPathMoveToPoint( path, pt1 );
    BezierPathCurveToPoint( path, pt2, cp1, cp2 );
    BezierPathStrokeFill( path, lineWidth, strokeCol, nil );
}

void BezierPathStrokePolygon( CFArrayRef pts, CGFloat lineWidth, ColorRef strokeCol )
{
    NSArray *array = (NSArray *)pts;
    NSUInteger count = [array count];
    BezierPathRef path = BezierPathInit();
    CGPoint pt = NSPointToCGPoint([[array objectAtIndex:0] pointValue]);
    BezierPathMoveToPoint( path, pt );
    for ( NSUInteger index = 1; index < count; index++ ) {
        pt = NSPointToCGPoint([[array objectAtIndex:index] pointValue]);
        BezierPathLineToPoint( path, pt );
    }
    BezierPathClose( path );
    BezierPathStrokeFill( path, lineWidth, strokeCol, nil );
}

void BezierPathFillOvalInRect( CGRect r, ColorRef fillCol )
{ BezierPathStrokeFill( BezierPathWithOvalInRect(r), 0, nil, fillCol ); }

void BezierPathFillRect( CGRect r, ColorRef fillCol )
{ BezierPathStrokeFill( BezierPathWithRect(r), 0, nil, fillCol ); }

void BezierPathFillRoundedRect( CGRect r, CGFloat xRadius, CGFloat yRadius, ColorRef fillCol )
{ BezierPathStrokeFill( BezierPathWithRoundedRect(r,xRadius,yRadius), 0, nil, fillCol ); }

void BezierPathFillPolygon( CFArrayRef pts, ColorRef fillCol )
{
    NSArray *array = (NSArray *)pts;
    NSUInteger count = [array count];
    BezierPathRef path = BezierPathInit();
    CGPoint pt = NSPointToCGPoint([[array objectAtIndex:0] pointValue]);
    BezierPathMoveToPoint( path, pt );
    for ( NSUInteger index = 1; index < count; index++ ) {
        pt = NSPointToCGPoint([[array objectAtIndex:index] pointValue]);
        BezierPathLineToPoint( path, pt );
    }
    BezierPathClose( path );
    BezierPathStrokeFill( path, 0, nil, fillCol );
}

//void BezierPathFillRotatedRect( CGRect r, CGFloat x, CGFloat y, CGFloat angle, ColorRef fillCol )
//{
//    AffineTransformRef tx = AffineTransformInit();
//    AffineTransformTranslate( tx, x, y );
//    AffineTransformRotateByDegrees( tx, angle );
//    AffineTransformConcat( tx );
//    BezierPathFillRect( r, fillCol );
//    AffineTransformInvert( tx );
//    AffineTransformConcat( tx );
//}
