/*
 BitmapImageRep.h
 
 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// - create -
BitmapImageRepRef BitmapImageRepWithData( CFDataRef dta );

CFArrayRef BitmapImageRepsWithData( CFDataRef dta );
void BitmapImageRepColorizeByMapping( BitmapImageRepRef rep, CGFloat midPt, ColorRef midPtCol, ColorRef shadowCol, ColorRef lightCol );

BitmapImageRepRef BitmapImageRepWithBitmapDataPlanes( unsigned char **planes, NSInteger wide, NSInteger high, NSInteger bps, NSInteger spp, Boolean hasAlpha, Boolean isPlanar, CFStringRef csName, NSInteger rBytes, NSInteger pBits );
BitmapImageRepRef BitmapImageRepWithCGImage( CGImageRef ref );
BitmapImageRepRef BitmapImageRepWithCIImage( CIImageRef ref );
BitmapImageRepRef BitmapImageRepForIncrementalLoad( void );

// - image info -
NSBitmapFormat BitmapImageRepFormat( BitmapImageRepRef ref );
NSInteger BitmapImageRepBitsPerPixel( BitmapImageRepRef ref );
NSInteger BitmapImageRepBytesPerPlane( BitmapImageRepRef ref );
NSInteger BitmapImageRepBytesPerRow( BitmapImageRepRef ref );
Boolean BitmapImageRepIsPlanar( BitmapImageRepRef ref );
NSInteger BitmapImageRepNumberOfPlanes( BitmapImageRepRef ref );
NSInteger BitmapImageRepSamplesPerPixel( BitmapImageRepRef ref );

// - image data -
void *BitmapImageRepBitmapData( BitmapImageRepRef rep );

// - image representation -
CFDataRef BitmapImageRepTIFFRepresentationOfImageReps( CFArrayRef imageReps );
CFDataRef BitmapImageRepTIFFRepresentationOfImageRepsUsingCompression( CFArrayRef imageReps, NSTIFFCompression compression, float factor );
CFDataRef BitmapImageRepTIFFRepresentation( BitmapImageRepRef rep );
CFDataRef BitmapImageRepTIFFRepresentationUsingCompression( BitmapImageRepRef rep, NSTIFFCompression compression, float factor );
CFDataRef BitmapImageRepTIFFRepresentationOfImageRepsUsingType( CFArrayRef imageReps, NSBitmapImageFileType type, CFDictionaryRef properties );
CFDataRef BitmapImageRepRepresentationUsingType( BitmapImageRepRef rep, NSBitmapImageFileType type, CFDictionaryRef properties );

// - compression -
void BitmapImageRepGetTIFFCompressionTypes( const NSTIFFCompression **list, NSInteger *count );
CFStringRef BitmapImageRepLocalizedNameForTIFFCompressionType( NSTIFFCompression compression );
Boolean BitmapImageRepCanBeCompressed( BitmapImageRepRef rep, NSTIFFCompression compression );
void BitmapImageRepSetCompression( BitmapImageRepRef rep, NSTIFFCompression compression, float factor );
void BitmapImageRepGetCompression( BitmapImageRepRef rep, NSTIFFCompression *compression, float *factor );
void BitmapImageRepSetPropertyValue( BitmapImageRepRef rep, CFStringRef property, CFTypeRef value );
CFTypeRef BitmapImageRepValueForProperty( BitmapImageRepRef rep, CFStringRef property );

// - load image incrementally -
NSInteger BitmapImageRepIncrementalLoadFromData( BitmapImageRepRef rep, CFDataRef dta, Boolean complete );

// - pixel values -
void BitmapImageRepSetColor( BitmapImageRepRef rep, ColorRef col, NSInteger x, NSInteger y );
ColorRef BitmapImageRepColor( BitmapImageRepRef rep, NSInteger x, NSInteger y );
void BitmapImageRepSetPixel( BitmapImageRepRef rep, NSUInteger *p, NSInteger x, NSInteger y );
void BitmapImageRepGetPixel( BitmapImageRepRef rep, NSUInteger *p, NSInteger x, NSInteger y );

// - CGImage -
CGImageRef BitmapImageRepCGImage( BitmapImageRepRef rep );

// - color spaces -
BitmapImageRepRef BitmapImageRepByConvertingToColorSpace( BitmapImageRepRef rep, ColorSpaceRef cs, NSColorRenderingIntent intent );
BitmapImageRepRef BitmapImageRepByRetaggingWithColorSpace( BitmapImageRepRef rep, ColorSpaceRef cs );
ColorSpaceRef BitmapImageRepColorSpace( BitmapImageRepRef rep );
