/*
 BitmapImageRep.m
 
 Bernie Wylde
 */

#import "BitmapImageRep.h"

// - create -
BitmapImageRepRef BitmapImageRepWithData( CFDataRef dta )
{ return [NSBitmapImageRep imageRepWithData:(NSData *)dta]; }

CFArrayRef BitmapImageRepsWithData( CFDataRef dta )
{ return (CFArrayRef)[NSBitmapImageRep imageRepsWithData:(NSData *)dta]; }

void BitmapImageRepColorizeByMapping( BitmapImageRepRef rep, CGFloat midPt, ColorRef midPtCol, ColorRef shadowCol, ColorRef lightCol )
{ [rep colorizeByMappingGray:midPt toColor:midPtCol blackMapping:shadowCol whiteMapping:lightCol]; }

BitmapImageRepRef BitmapImageRepWithBitmapDataPlanes( unsigned char **planes, NSInteger wide, NSInteger high, NSInteger bps, NSInteger spp, Boolean hasAlpha, Boolean isPlanar, CFStringRef csName, NSInteger rBytes, NSInteger pBits )
{ return [[[NSBitmapImageRep alloc] initWithBitmapDataPlanes:planes pixelsWide:wide pixelsHigh:high bitsPerSample:bps samplesPerPixel:spp hasAlpha:hasAlpha isPlanar:isPlanar colorSpaceName:(NSString *)csName bytesPerRow:rBytes bitsPerPixel:pBits] autorelease]; }

BitmapImageRepRef BitmapImageRepWithCGImage( CGImageRef ref )
{ return [[[NSBitmapImageRep alloc] initWithCGImage:ref] autorelease]; }

BitmapImageRepRef BitmapImageRepWithCIImage( CIImageRef ref )
{ return [[[NSBitmapImageRep alloc] initWithCIImage:ref] autorelease]; }

BitmapImageRepRef BitmapImageRepForIncrementalLoad( void )
{ return [[[NSBitmapImageRep alloc] initForIncrementalLoad] autorelease]; }

// - image info -
NSBitmapFormat BitmapImageRepFormat( BitmapImageRepRef ref )
{ return [ref bitmapFormat]; }

NSInteger BitmapImageRepBitsPerPixel( BitmapImageRepRef ref )
{ return [ref bitsPerPixel]; }

NSInteger BitmapImageRepBytesPerPlane( BitmapImageRepRef ref )
{ return [ref bytesPerPlane]; }

NSInteger BitmapImageRepBytesPerRow( BitmapImageRepRef ref )
{ return [ref bytesPerRow]; }

Boolean BitmapImageRepIsPlanar( BitmapImageRepRef ref )
{ return [ref isPlanar]; }

NSInteger BitmapImageRepNumberOfPlanes( BitmapImageRepRef ref )
{ return [ref numberOfPlanes]; }

NSInteger BitmapImageRepSamplesPerPixel( BitmapImageRepRef ref )
{ return [ref samplesPerPixel]; }

// - image data -
void *BitmapImageRepBitmapData( BitmapImageRepRef rep )
{ return [rep bitmapData]; }

// - image representation -
CFDataRef BitmapImageRepTIFFRepresentationOfImageReps( CFArrayRef imageReps )
{ return (CFDataRef)[NSBitmapImageRep TIFFRepresentationOfImageRepsInArray:(NSArray *)imageReps]; }

CFDataRef BitmapImageRepTIFFRepresentationOfImageRepsUsingCompression( CFArrayRef imageReps, NSTIFFCompression compression, float factor )
{ return (CFDataRef)[NSBitmapImageRep TIFFRepresentationOfImageRepsInArray:(NSArray *)imageReps usingCompression:compression factor:factor]; }

CFDataRef BitmapImageRepTIFFRepresentation( BitmapImageRepRef rep )
{ return (CFDataRef)[rep TIFFRepresentation]; }

CFDataRef BitmapImageRepTIFFRepresentationUsingCompression( BitmapImageRepRef rep, NSTIFFCompression compression, float factor )
{ return (CFDataRef)[rep TIFFRepresentationUsingCompression:compression factor:factor]; }

CFDataRef BitmapImageRepTIFFRepresentationOfImageRepsUsingType( CFArrayRef imageReps, NSBitmapImageFileType type, CFDictionaryRef properties )
{ return (CFDataRef)[NSBitmapImageRep representationOfImageRepsInArray:(NSArray *)imageReps usingType:type properties:(NSDictionary *)properties]; }

CFDataRef BitmapImageRepRepresentationUsingType( BitmapImageRepRef rep, NSBitmapImageFileType type, CFDictionaryRef properties )
{ return (CFDataRef)[rep representationUsingType:type properties:(NSDictionary *)properties]; }

// - compression -
void BitmapImageRepGetTIFFCompressionTypes( const NSTIFFCompression **list, NSInteger *count )
{ [NSBitmapImageRep getTIFFCompressionTypes:list count:count]; }

CFStringRef BitmapImageRepLocalizedNameForTIFFCompressionType( NSTIFFCompression compression )
{ return (CFStringRef)[NSBitmapImageRep localizedNameForTIFFCompressionType:compression]; }

Boolean BitmapImageRepCanBeCompressed( BitmapImageRepRef rep, NSTIFFCompression compression )
{ return [rep canBeCompressedUsing:compression]; }

void BitmapImageRepSetCompression( BitmapImageRepRef rep, NSTIFFCompression compression, float factor )
{ [rep setCompression:compression factor:factor]; }

void BitmapImageRepGetCompression( BitmapImageRepRef rep, NSTIFFCompression *compression, float *factor )
{ [rep getCompression:compression factor:factor]; }

void BitmapImageRepSetPropertyValue( BitmapImageRepRef rep, CFStringRef property, CFTypeRef value )
{ [rep setProperty:(NSString *)property withValue:(id)value]; }

CFTypeRef BitmapImageRepValueForProperty( BitmapImageRepRef rep, CFStringRef property )
{ return (CFTypeRef)[rep valueForProperty:(NSString *)property]; }

// - load image incrementally -
NSInteger BitmapImageRepIncrementalLoadFromData( BitmapImageRepRef rep, CFDataRef dta, Boolean complete )
{ return [rep incrementalLoadFromData:(NSData *)dta complete:complete]; }

// - pixel values -
void BitmapImageRepSetColor( BitmapImageRepRef rep, ColorRef col, NSInteger x, NSInteger y )
{ [rep setColor:col atX:x y:y]; }

ColorRef BitmapImageRepColor( BitmapImageRepRef rep, NSInteger x, NSInteger y )
{ return [rep colorAtX:x y:y]; }

void BitmapImageRepSetPixel( BitmapImageRepRef rep, NSUInteger *p, NSInteger x, NSInteger y )
{ [rep setPixel:p atX:x y:y]; }

void BitmapImageRepGetPixel( BitmapImageRepRef rep, NSUInteger *p, NSInteger x, NSInteger y )
{ [rep getPixel:p atX:x y:y]; }

// - CGImage -
CGImageRef BitmapImageRepCGImage( BitmapImageRepRef rep )
{ return [rep CGImage]; }

// - color spaces -
BitmapImageRepRef BitmapImageRepByConvertingToColorSpace( BitmapImageRepRef rep, ColorSpaceRef cs, NSColorRenderingIntent intent )
{ return [rep bitmapImageRepByConvertingToColorSpace:cs renderingIntent:intent]; }

BitmapImageRepRef BitmapImageRepByRetaggingWithColorSpace( BitmapImageRepRef rep, ColorSpaceRef cs )
{ return [rep bitmapImageRepByRetaggingWithColorSpace:cs]; }

ColorSpaceRef BitmapImageRepColorSpace( BitmapImageRepRef rep )
{ return [rep colorSpace]; }
