/*
 Box.m
 
 Bernie Wylde
 */

#import "Box.h"

#pragma mark - subclass
@implementation BoxSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end


@implementation NSBox (Additions)
@dynamic tagNumber;
- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}
- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end


#pragma mark - functions
NSBox *BoxWithTag( NSInteger tag )
{ return (NSBox *)CocoaValidateViewKindOfClass( tag, [NSBox class] ); }

BOOL BoxExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSBox class] ); }



void BoxSetBorderType( NSInteger tag, NSBorderType type )
{
    NSBox *view = BoxWithTag( tag );
    if ( view ) [(NSBox *)view setBorderType:type];
}

void BoxSetTitlePosition( NSInteger tag, NSTitlePosition position )
{
    NSBox *view = BoxWithTag( tag );
    if ( view ) [(NSBox *)view setTitlePosition:position];
}

void BoxSetBorderColor( NSInteger tag, ColorRef col )
{
    NSBox *view = BoxWithTag( tag );
    if ( view ) [(NSBox *)view setBorderColor:col];
}

void BoxSetFillColor( NSInteger tag, ColorRef col )
{
    NSBox *view = BoxWithTag( tag );
    if ( view ) [(NSBox *)view setFillColor:col];
}

void BoxSetBorderWidth( NSInteger tag, CGFloat width )
{
    NSBox *view = BoxWithTag( tag );
    if ( view ) [(NSBox *)view setBorderWidth:width];
}

void BoxSetCornerRadius( NSInteger tag, CGFloat radius )
{
    NSBox *view = BoxWithTag( tag );
    if ( view ) [(NSBox *)view setCornerRadius:radius];
}

void BoxSetTransparent( NSInteger tag, Boolean flag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) [box setTransparent:flag];
}

void BoxSetTitleFont( NSInteger tag, CTNSFontRef inFont )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) {
        NSFont *font = FixFont( (void *)inFont );
        [box setTitleFont:font];
    }
}

void BoxSizeToFit( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) [box sizeToFit];
}

CGSize BoxContentViewMargins( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return NSSizeToCGSize([box contentViewMargins]);
    return CGSizeZero;
}

void BoxSetContentViewMargins( NSInteger tag, CGSize size )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) [box setContentViewMargins:NSSizeFromCGSize(size)];
}



CGRect BoxBorderRect( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return NSRectToCGRect([box borderRect]);
    return CGRectZero;
}

NSBoxType BoxType( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return [box boxType];
    return 0;
}

void BoxSetType( NSInteger tag, NSBoxType type )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) [box setBoxType:type];
}

NSBorderType BoxBorderType( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return [box borderType];
    return 0;
}

Boolean BoxIsTransparent( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return [box isTransparent];
    return NO;
}

CFStringRef BoxTitle( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return (CFStringRef)[box title];
    return nil;
}

void BoxSetTitle( NSInteger tag, CFStringRef title )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) [box setTitle:(NSString *)title];
}

CTNSFontRef BoxTitleFont( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return [box titleFont];
    return nil;
}

NSTitlePosition BoxTitlePosition( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return [box titlePosition];
    return 0;
}

CGRect BoxTitleRect( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return NSRectToCGRect([box titleRect]);
    return CGRectZero;
}

ColorRef BoxBorderColor( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return [box borderColor];
    return nil;
}

CGFloat BoxBorderWidth( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return [box borderWidth];
    return 0.0;
}

CGFloat BoxCornerRadius( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return [box cornerRadius];
    return 0.0;
}

ColorRef BoxFillColor( NSInteger tag )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) return [box fillColor];
    return nil;
}

void BoxSetFrameFromContentFrame( NSInteger tag, CGRect contentFrame )
{
    NSBox *box = BoxWithTag( tag );
    if ( box ) [box setFrameFromContentFrame:NSRectFromCGRect(contentFrame)];
}





// custom
void BoxSetTitleFontWithName( NSInteger tag, CFStringRef inName, CGFloat size )
{
    BOOL usePrevFont = NO;
    NSString *name = (NSString *)inName;
    NSBox *box = BoxWithTag( tag );
    if ( box ) {
        if ( name ) {
            usePrevFont = ( [name length] == 0 );
        } else {
            usePrevFont = YES;
        }
        if ( usePrevFont ) name = [[box titleFont] fontName];
        if ( (int)size == 0 ) size = [[box titleFont] pointSize];
        
        //        [box setTitleFont:[NSFont fontWithName:(NSString *)name size:size]];
        //        CTFontRef font = CTFontCreateWithName( (CFStringRef)name, size, NULL );
        //        [box setTitleFont:(NSFont *)font];
        //        CFRelease(font);
        
        [box setTitleFont:[NSFont fontWithName:name size:size]];
    }
}


// Init
BoxRef BoxInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) {
        rect = NSMakeRect( 17, 16, 177, 114 );
    }

    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSBox *box;
    if ( subclass ) {
        box = [[[BoxSC alloc] initWithFrame:rect] autorelease];
    } else {
        box = [[[NSBox alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [box setTag:inTag];
    
    return box;
}

// statement
void CocoaBox( NSInteger inTag, CFStringRef inTitle, CGRect inRect, NSInteger inType, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSString *title = (NSString *)inTitle;
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        
        NSBox *box = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !box ) {
            box = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( box ) {    // already exists
            
            // clear subclass flag
//            SetSubclassFlag(NO);
            
            // make sure it's same view class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [box class], [NSBox class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, box, [NSBox class] ) ) {
                return;
            }

            // title
            if ( title ) [box setTitle:title];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [box setFrame:rect];
            
            // type
            if ( inType != kFBParamMissing ) [box setBoxType:inType];
            
        } else {        // create
            
//            SetSubclassFlag( inSubclass );
            
            box = BoxInit( inTag, inRect );
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) {
//                rect = NSMakeRect( 17, 16, 177, 114 );
//            }
            
            // init
//            if ( inSubclass ) {
//                box = [[BoxSC alloc] initWithFrame:rect];
//            } else {
//                box = [[NSBox alloc] initWithFrame:rect];
//            }
            
            // default autoresizing
            [box setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [box setTag: inTag];
            
            // title
            if ( title ) [box setTitle:title];
            
            // type
            if ( inType != kFBParamMissing ) [box setBoxType:inType];
            
            // wndTag
            [box setWndTag:inWndTag];
            
            // add to window
            [contentView addSubview:box];
//            [box release];
        }
        
        [box setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}


