/*
 Bundle.h
 
 Bernie Wylde
 */

#import "Bundle.h"

// -- standard objects --
BundleRef BundleMain( void )
{ return [NSBundle mainBundle]; }

// -- init --
BundleRef BundleWithURL( CFURLRef url )
{ return [NSBundle bundleWithURL:(NSURL *)url]; }

// -- loading nib files --

// -- resource files --
CFURLRef BundleURLForResource( BundleRef bundle, CFStringRef name, CFStringRef extension, CFStringRef subdirectory )
{
    if ( subdirectory ) {
        return (CFURLRef)[bundle URLForResource:(NSString *)name withExtension:(NSString *)extension subdirectory:(NSString *)subdirectory];
    } else {
        return (CFURLRef)[bundle URLForResource:(NSString *)name withExtension:(NSString *)extension];
    }
}

CFArrayRef BundleURLsForResources( BundleRef bundle, CFStringRef extension, CFStringRef subdirectory )
{ return (CFArrayRef)[bundle URLsForResourcesWithExtension:(NSString *)extension subdirectory:(NSString *)subdirectory]; }

// -- image --
CFURLRef BundleURLForImageResource( BundleRef bundle, CFStringRef name )
{ return (CFURLRef)[bundle URLForImageResource:(NSString *)name]; }

ImageRef BundleImageForResource( BundleRef bundle, CFStringRef name )
{ return [bundle imageForResource:(NSString *)name]; }

// -- sound --
CFURLRef BundleURLForSoundResource( BundleRef bundle, CFStringRef name )
{ return (CFURLRef)[NSURL fileURLWithPath:[bundle pathForSoundResource:(NSString *)name]]; }

// -- localized strings --
CFStringRef BundleLocalizedString( BundleRef bundle, CFStringRef key, CFStringRef value, CFStringRef table )
{ return (CFStringRef)[bundle localizedStringForKey:(NSString *)key value:(NSString *)value table:(NSString *)table]; }

// -- context help --
CFAttributedStringRef BundleContextHelp( BundleRef bundle, CFStringRef key )
{ return (CFAttributedStringRef)[bundle contextHelpForKey:(NSString *)key]; }

// -- standard bundle directories --
CFURLRef BundleResourceURL( BundleRef bundle )
{ return (CFURLRef)[bundle resourceURL]; }

CFURLRef BundleExecutableURL( BundleRef bundle )
{ return (CFURLRef)[bundle executableURL]; }

CFURLRef BundlePrivateFrameworksURL( BundleRef bundle )
{ return (CFURLRef)[bundle privateFrameworksURL]; }

CFURLRef BundleSharedFrameworksURL( BundleRef bundle )
{ return (CFURLRef)[bundle sharedFrameworksURL]; }

CFURLRef BundleBuiltInPlugInsURL( BundleRef bundle )
{ return (CFURLRef)[bundle builtInPlugInsURL]; }

CFURLRef BundleURLForAuxiliaryExecutable( BundleRef bundle, CFStringRef name )
{ return (CFURLRef)[bundle URLForAuxiliaryExecutable:(NSString *)name]; }

CFURLRef BundleSharedSupportURL( BundleRef bundle )
{ return (CFURLRef)[bundle sharedSupportURL]; }

CFURLRef BundleAppStoreReceiptURL( BundleRef bundle )
{ return (CFURLRef)[bundle appStoreReceiptURL]; }

// -- bundle info --
CFURLRef BundleURL( BundleRef bundle )
{ return (CFURLRef)[bundle bundleURL]; }

CFStringRef BundleIdentifier( BundleRef bundle )
{ return (CFStringRef)[bundle bundleIdentifier]; }

CFDictionaryRef BundleInfoDictionary( BundleRef bundle )
{ return (CFDictionaryRef)[bundle infoDictionary]; }

CFTypeRef BundleObjectForInfoDictionaryKey( BundleRef bundle, CFStringRef key )
{ return (CFTypeRef)[bundle objectForInfoDictionaryKey:(NSString *)key]; }

// -- localization info --
CFArrayRef BundleLocalizations( BundleRef bundle )
{ return (CFArrayRef)[bundle localizations]; }

CFArrayRef BundlePreferredLocalizations( BundleRef bundle )
{ return (CFArrayRef)[bundle preferredLocalizations]; }

CFStringRef BundleDevelopmentLocalization( BundleRef bundle )
{ return (CFStringRef)[bundle developmentLocalization]; }

CFDictionaryRef BundleLocalizedInfoDictionary( BundleRef bundle )
{ return (CFDictionaryRef)[bundle localizedInfoDictionary]; }

CFArrayRef BundlePreferredLocalizationsFromArray( CFArrayRef localizationsArray )
{ return (CFArrayRef)[NSBundle preferredLocalizationsFromArray:(NSArray *)localizationsArray]; }

CFArrayRef BundlePreferredLocalizationsFromArrayForPreferences( CFArrayRef localizationsArray, CFArrayRef preferencesArray )
{ return (CFArrayRef)[NSBundle preferredLocalizationsFromArray:(NSArray *)localizationsArray forPreferences:(NSArray *)preferencesArray]; }

// -- preservation priority --

// -- classes --

// -- loading code from bundle --

// -- convenience --

