/*
 Button.m
 
 Bernie Wylde
 */

#import "Button.h"

#pragma mark - subclass
@implementation ButtonSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end

#pragma mark - functions
NSButton *ButtonWithTag( NSInteger tag )
{ return (NSButton *)CocoaValidateViewKindOfClass( tag, [NSButton class] ); }

BOOL ButtonExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSButton class] ); }

void ButtonSetType( NSInteger tag, NSButtonType type )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setButtonType:type];
}

void ButtonGetPeriodicDelay( NSInteger tag, float *delay, float *interval )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn getPeriodicDelay:delay interval:interval];
}

CFStringRef ButtonAlternateTitle( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return (CFStringRef)[btn alternateTitle];
    return nil;
}

CFAttributedStringRef ButtonAttributedTitle( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return (CFAttributedStringRef)[btn attributedTitle];
    return nil;
}


CFAttributedStringRef ButtonAttributedAlternateTitle( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return (CFAttributedStringRef)[btn attributedAlternateTitle];
    return nil;
}

void ButtonSetTitle( NSInteger tag, CFStringRef title )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setTitle:(NSString *)title];
}

SoundRef ButtonSound( NSInteger tag)
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn sound];
    return nil;
}

ImageRef ButtonImage( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn image];
    return nil;
}

ImageRef ButtonAlternateImage( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn alternateImage];
    return nil;
}

NSCellImagePosition ButtonImagePosition( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn imagePosition];
    return 0;
}

Boolean ButtonIsBordered( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn isBordered];
    return NO;
}

Boolean ButtonIsTransparent( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn isTransparent];
    return NO;
}

NSBezelStyle ButtonBezelStyle( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn bezelStyle];
    return 0;
}

void ButtonSetBezelStyle( NSInteger tag, NSBezelStyle style )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setBezelStyle:style];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101201
ColorRef ButtonBezelColor( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn bezelColor];
    return nil;
}

void ButtonSetBezelColor( NSInteger tag, ColorRef col )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setBezelColor:col];
}
#endif

Boolean ButtonShowsBorderOnlyWhileMouseInside( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn showsBorderOnlyWhileMouseInside];
    return NO;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean ButtonImageHugsTitle( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn imageHugsTitle];
    return NO;
}

void ButtonSetImageHugsTitle( NSInteger tag, Boolean flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setImageHugsTitle:flag];
}
#endif

NSImageScaling ButtonImageScaling( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [[btn cell] imageScaling];
    return 0;
}

Boolean ButtonAllowsMixedState( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn allowsMixedState];
    return NO;
}

void ButtonSetState( NSInteger tag, NSCellStateValue state )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setState:state];
}

CFStringRef ButtonKeyEquivalent( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return (CFStringRef)[btn keyEquivalent];
    return nil;
}

NSUInteger ButtonKeyEquivalentModifierMask( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn keyEquivalentModifierMask];
    return 0;
}

NSCellStateValue ButtonState( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn state];
    return 0;
}

CFStringRef ButtonTitle( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return (CFStringRef)[btn title];
    return NULL;
}

void ButtonSetSound( NSInteger tag, SoundRef snd )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setSound:snd];
}

void ButtonSetSoundNamed( NSInteger tag, CFStringRef sndName )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setSound:[NSSound soundNamed:(NSString *)sndName]];
}

void ButtonSetKeyEquivalent( NSInteger tag, CFStringRef key )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setKeyEquivalent:(NSString *)key];
}

void ButtonSetKeyEquivalentModifierMask( NSInteger tag, NSUInteger mask )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setKeyEquivalentModifierMask:mask];
}

void ButtonSetImage( NSInteger tag, void *inImage )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) {
        NSImage *image = FixImageValue( inImage );
        if ( image ) [btn setImage:image];
    }
}

void ButtonSetImageNamed( NSInteger tag, CFStringRef imageName )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) {
        NSImage *image = CocoaImageResource( (NSString *)imageName );
        if ( image ) [btn setImage:image];
    }
}

void ButtonSetImagePosition( NSInteger tag, NSCellImagePosition position )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setImagePosition:position];
}

void ButtonSetImageScaling( NSInteger tag, NSImageScaling scaling )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [[btn cell] setImageScaling:scaling];
}

void ButtonSetBordered( NSInteger tag, Boolean flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setBordered:flag];
}

void ButtonSetShowsBorderOnlyWhileMouseInside( NSInteger tag, Boolean flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setShowsBorderOnlyWhileMouseInside:flag];
}

void ButtonPerformClick( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn performClick:nil];
}

void ButtonSetAttributedTitle( NSInteger tag, CFAttributedStringRef title )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setAttributedTitle:(NSAttributedString *)title];
}

void ButtonSetAlternateTitle( NSInteger tag, CFStringRef title )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setAlternateTitle:(NSString *)title];
}

void ButtonSetPeriodicDelay( NSInteger tag, float delay, float interval )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setPeriodicDelay:delay interval:interval];
}

void ButtonSetAttributedAlternateTitle( NSInteger tag, CFAttributedStringRef title )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setAttributedAlternateTitle:(NSAttributedString *)title];
}

void ButtonSetAlternateImage( NSInteger tag, void *inImage )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) {
        NSImage *image = FixImageValue( inImage );
        if ( image ) [btn setAlternateImage:image];
    }
}

void ButtonSetAlternateImageNamed( NSInteger tag, CFStringRef imageName )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) {
        NSImage *image = CocoaImageResource( (NSString *)imageName );
        if ( image ) [btn setAlternateImage:image];
    }
}

void ButtonSetTransparent( NSInteger tag, Boolean flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setTransparent:flag];
}

void ButtonSetAllowsMixedState( NSInteger tag, Boolean flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setAllowsMixedState:flag];
}

void ButtonSetNextState( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setNextState];
}

void ButtonHighlight( NSInteger tag, Boolean flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn highlight:flag];
}

Boolean ButtonIsOpaque( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn isOpaque];
    return NO;
}

void ButtonSetHighlightsBy( NSInteger tag, NSUInteger mask )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [[btn cell] setHighlightsBy:mask];
}



// - buttoncell functions -
CTNSFontRef ButtonKeyEquivalentFont( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [[btn cell] keyEquivalentFont];
    return nil;
}

void ButtonSetKeyEquivalentFont( NSInteger tag, CTNSFontRef font )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [[btn cell] setKeyEquivalentFont:font];
}


// custom
void ButtonClose( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn removeFromSuperview];
}

void ButtonSetTitleColor( NSInteger tag, ColorRef col )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) {
        NSMutableAttributedString *as = [[[btn attributedTitle] mutableCopy] autorelease];
        [as addAttribute:NSForegroundColorAttributeName value:col range:NSMakeRange(0,[as length])];
        [btn setAttributedTitle:as];
    }
}

//void ButtonSetTitleColor( NSInteger tag, ColorRef col )
//{
//    NSButton *btn = ButtonWithTag( tag );
//    if ( btn ) {
//        NSFont *font = [btn font];
//        NSMutableAttributedString *as = [[btn attributedTitle] mutableCopy];
//        [as addAttribute:NSFontAttributeName value:font range:NSMakeRange(0,[as length])];
//        [as addAttribute:NSForegroundColorAttributeName value:col range:NSMakeRange(0,[as length])];
//        [btn setAttributedTitle:as];
//        [as release];
//    }
//}

void ButtonSetBackgroundColor( NSInteger tag, ColorRef col )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [[btn cell] setBackgroundColor:col];
}

Boolean ButtonImageDimsWhenDisabled( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [[btn cell] imageDimsWhenDisabled];
    return NO;
}

void ButtonSetImageDimsWhenDisabled( NSInteger tag, Boolean flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [[btn cell] setImageDimsWhenDisabled:flag];
}


// Init
ButtonRef ButtonInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) {
        rect = NSMakeRect( 14, 13, 81, 32 );
    }

    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSButton *btn;
    if ( subclass ) {
        btn = [[[ButtonSC alloc] initWithFrame:rect] autorelease];
    } else {
        btn = [[[NSButton alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [btn setTag:inTag];
    
    // style
    [btn setBezelStyle:NSRoundedBezelStyle];

    // type
    [btn setButtonType:NSMomentaryLightButton];

    // font
    CocoaControlFixFont( btn );

    // target/action
    [btn setTarget:CocoaControlTargetObj()];
    [btn setAction:@selector(controlAction:)];
    
    return btn;
}


#pragma mark - button statement
void CocoaButton( NSInteger inTag, NSInteger inEnabled, NSInteger inState, CFStringRef inTitle, CGRect inRect, NSInteger inType, NSInteger inStyle, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSString *title = (NSString *)inTitle;
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        
        NSButton *btn = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !btn ) {
            btn = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( btn ) {    // already exists
            
            // clear subclass flag
//            SetSubclassFlag(NO);

            // make sure it's same view class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [btn class], [NSButton class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, btn, [NSButton class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [btn setEnabled:inEnabled];
            
            // state
            if ( inState != kFBParamMissing ) {
                if ( inState == NSMixedState ) [btn setAllowsMixedState:YES];
                [btn setState:inState];
            }
            
            // title
            if ( title ) [btn setTitle:title];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [btn setFrame:rect];
            
            // type
            if ( inType != kFBParamMissing ) [btn setButtonType:inType];
            
            // style
            if ( inStyle != kFBParamMissing ) [btn setBezelStyle:inStyle];
            
        } else {        // create
            
//            SetSubclassFlag( inSubclass );

            btn = ButtonInit( inTag, inRect );
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) {
//                rect = NSMakeRect( 14, 13, 81, 32 );
//            }
            
            // init
//            if ( inSubclass ) {
//                btn = [[ButtonSC alloc] initWithFrame:rect];
//            } else {
//                btn = [[NSButton alloc] initWithFrame:rect];
//            }
            
            // default autoresizing
            [btn setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [btn setTag: inTag];
            
            // state
            if ( inState != kFBParamMissing ) {
                if ( inState == NSMixedState ) [btn setAllowsMixedState:YES];
                [btn setState:inState];
            }
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [btn setEnabled:inEnabled];
            
            // type
            if ( inType == kFBParamMissing ) inType = NSMomentaryLightButton;
            [btn setButtonType:inType];
            
            // style
            if ( inStyle == kFBParamMissing ) inStyle = NSRoundedBezelStyle;
            [btn setBezelStyle:inStyle];
            
            switch ( inStyle ) {
                case NSHelpButtonBezelStyle:
                    if ( !title ) title = @"";
                    break;
                case NSRecessedBezelStyle:
                    [btn setShowsBorderOnlyWhileMouseInside:YES];
                    break;
                default:
                    break;
            }
            
            // title
            if ( title ) {
                [btn setTitle:title];
                if ( inStyle == NSRoundedBezelStyle ) {
                    if ( inType == NSMomentaryLightButton || inType == NSMomentaryPushInButton ) {
                        if ( [title isEqualToString:@"OK"] ) [btn setKeyEquivalent:@"\r"];
                        if ( [title isEqualToString:@"Cancel"] ) [btn setKeyEquivalent:@"\e"];
                    }
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300//101003
                    if ( inType == NSButtonTypeAccelerator || inType == NSButtonTypeMultiLevelAccelerator ) {
                        if ( [title isEqualToString:@"OK"] ) [btn setKeyEquivalent:@"\r"];;
                        if ( [title isEqualToString:@"Cancel"] ) [btn setKeyEquivalent:@"\e"];
                    }
#endif
                }
            }
            
            // wndTag
            [btn setWndTag:inWndTag];
            
            // fix font
            switch ( inStyle ) {
                case NSRoundRectBezelStyle:
                    [btn setFont:[NSFont systemFontOfSize:12]];
                    break;
                case NSRecessedBezelStyle:
                    [[btn cell] setControlSize:NSSmallControlSize];
                    [btn setFont:[NSFont boldSystemFontOfSize:12]];
                    break;
                    
                case NSInlineBezelStyle:
                    [btn setFont:[NSFont boldSystemFontOfSize:11]];
                    break;
                    
                default:
                    CocoaControlFixFont( btn );
                    break;
            }
            
            // target/action
//            [btn setTarget:CocoaControlTargetObj()];
//            [btn setAction:@selector(controlAction:)];
            
            // add to window/popover
            [contentView addSubview:btn];
            //[btn release];
        }
        
        [btn setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

