/*
 CAGradientLayer.m
 
 Bernie Wylde
 */

#import "CAGradientLayer.h"

// - init -
CAGradientLayerRef CAGradientLayerInit( void )
{ return [CAGradientLayer layer]; }

// - style -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFArrayRef CAGradientLayerColors( CAGradientLayerRef layer )
{
    NSArray *cgColors = [layer colors];
    NSMutableArray *nsColors = [NSMutableArray arrayWithCapacity:0];
    for ( NSUInteger index = 0; index < [cgColors count]; index++ ) {
        CGColorRef col = (CGColorRef)[cgColors objectAtIndex:index];
        [nsColors addObject:[NSColor colorWithCGColor:col]];
    }
    return (CFArrayRef)nsColors;
}

void CAGradientLayerSetColors( CAGradientLayerRef layer, CFArrayRef cols )
{
    NSMutableArray *cgColors = [NSMutableArray arrayWithCapacity:0];
    for ( NSColor *col in (NSArray *)cols ) {
        [cgColors addObject:(id)[col CGColor]];
    }
    [layer setColors:(NSArray *)cgColors];
}
#endif

CFArrayRef CAGradientLayerLocations( CAGradientLayerRef layer )
{ return (CFArrayRef)[layer locations]; }

void CAGradientLayerSetLocations( CAGradientLayerRef layer, CFArrayRef locations )
{ [layer setLocations:(NSArray *)locations]; }

CGPoint CAGradientEndPoint( CAGradientLayerRef layer )
{ return [layer endPoint]; }

void CAGradientLayerSetEndPoint( CAGradientLayerRef layer, CGPoint pt )
{ [layer setEndPoint:pt]; }

CGPoint CAGradientStartPoint( CAGradientLayerRef layer )
{ return [layer startPoint]; }

void CAGradientLayerSetStartPoint( CAGradientLayerRef layer, CGPoint pt )
{ [layer setStartPoint:pt]; }

CFStringRef CAGradientType( CAGradientLayerRef layer )
{ return (CFStringRef)[layer type]; }

void CAGradientLayerSetType( CAGradientLayerRef layer, CFStringRef type )
{ [layer setType:(NSString *)type]; }

// - convenience -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CAGradientLayerRef CAGradientLayerWithStartEndColors( CGRect r, ColorRef startColor, CGPoint startPt, ColorRef endColor, CGPoint endPt )
{
    CAGradientLayer *layer = [CAGradientLayer layer];
    [layer setFrame:r];
    NSMutableArray *cgColors = [NSMutableArray arrayWithCapacity:2];
    [cgColors addObject:(id)[startColor CGColor]];
    [cgColors addObject:(id)[endColor CGColor]];
    [layer setColors:cgColors];
    [layer setStartPoint:startPt];
    [layer setEndPoint:endPt];
    return layer;
}
#endif
