/*
 CIContext.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// Create
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CIContextRef CIContextWithOptions( CFDictionaryRef options );
#endif

// Create for CPU-based rendering
CIContextRef CIContextWithCGContext( CGContextRef cgctx, CFDictionaryRef options );

// Render images
CGImageRef CIContextCreateCGImage( CIContextRef ctx, CIImageRef image, CGRect r );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CGImageRef CIContextCreateCGImageWithFormat( CIContextRef ctx, CIImageRef image, CGRect r, CIFormat format, CGColorSpaceRef colorSpace, Boolean deferred );
#endif
void CIContextRenderToBitmap( CIContextRef ctx, CIImageRef image, void *dta, long rowBytes, CGRect bounds, CIFormat format, CGColorSpaceRef colorSpace );

// Draw images
void CIContextDrawImage( CIContextRef ctx, CIImageRef image, CGRect inRect, CGRect fromRect );

// Manage resources
void CIContextClearCaches( CIContextRef ctx );
void CIContextReclaimResources( CIContextRef ctx );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
unsigned long CIContextOfflineGPUCount( void );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CGColorSpaceRef CIContextWorkingColorSpace( CIContextRef ctx );
CIFormat CIContextWorkingFormat( CIContextRef ctx );
#endif

// Render image for data of file export
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFDataRef CIContextTIFFRepresentationOfImage( CIContextRef ctx, CIImageRef image, CIFormat format, CGColorSpaceRef colorSpace, CFDictionaryRef options );
CFDataRef CIContextJPEGRepresentationOfImage( CIContextRef ctx, CIImageRef image, CGColorSpaceRef colorSpace, CFDictionaryRef options );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFDataRef CIContextPNGRepresentationOfImage( CIContextRef ctx, CIImageRef image, CIFormat format, CGColorSpaceRef colorSpace, CFDictionaryRef options );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101304 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101304
CFDataRef CIContextHEIFRepresentationOfImage( CIContextRef ctx, CIImageRef image, CIFormat format, CGColorSpaceRef colorSpace, CFDictionaryRef options );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean CIContextWriteTIFFRepresentationOfImageToURL( CIContextRef ctx, CIImageRef image, CFURLRef url, CIFormat format, CGColorSpaceRef colorSpace, CFDictionaryRef options, ErrorRef *err );
Boolean CIContextWriteJPEGRepresentationOfImageToURL( CIContextRef ctx, CIImageRef image, CFURLRef url, CGColorSpaceRef colorSpace, CFDictionaryRef options, ErrorRef *err );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean CIContextWritePNGRepresentationOfImageToURL( CIContextRef ctx, CIImageRef image, CFURLRef url, CIFormat format, CGColorSpaceRef colorSpace, CFDictionaryRef options, ErrorRef *err );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101304 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101304
Boolean CIContextWriteHEIFRepresentationOfImageToURL( CIContextRef ctx, CIImageRef image, CFURLRef url, CIFormat format, CGColorSpaceRef colorSpace, CFDictionaryRef options, ErrorRef *err );
#endif

// Customize render destination
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean CIContextPrepareRender( CIContextRef ctx, CIImageRef image, CGRect fromRect, CIRenderDestinationRef destination, CGPoint atPoint, ErrorRef *err );
CIRenderTaskRef CIContextStartTaskToClear( CIContextRef ctx, CIRenderDestinationRef destination, ErrorRef *err );
CIRenderTaskRef CIContextStartTaskToRenderFromRect( CIContextRef ctx, CIImageRef image, CGRect fromRect, CIRenderDestinationRef destination, CGPoint atPoint, ErrorRef *err );
CIRenderTaskRef CIContextStartTaskToRender( CIContextRef ctx, CIImageRef image, CIRenderDestinationRef destination, ErrorRef *err );
#endif

// Instance methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
CIFilterRef CIContextDepthBlurEffectFilterForImage( CIContextRef ctx, CIImageRef image, CIImageRef disparityImage, CIImageRef portraitEffectsMatte, CGImagePropertyOrientation orientation, CFDictionaryRef options );
CIFilterRef CIContextDepthBlurEffectFilterForImageData( CIContextRef ctx, CFDataRef dta, CFDictionaryRef options );
CIFilterRef CIContextDepthBlurEffectFilterForImageURL( CIContextRef ctx, CFURLRef url, CFDictionaryRef options );
#endif
