/*
 CIContext.m

 Bernie Wylde
 */

#import "CIContext.h"

// Create
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CIContextRef CIContextWithOptions( CFDictionaryRef options )
{ return [CIContext contextWithOptions:(NSDictionary *)options]; }
#endif

// Create for CPU-based rendering
CIContextRef CIContextWithCGContext( CGContextRef cgctx, CFDictionaryRef options )
{ return [CIContext contextWithCGContext:cgctx options:(NSDictionary *)options]; }

// Render images
CGImageRef CIContextCreateCGImage( CIContextRef ctx, CIImageRef image, CGRect r )
{ return [ctx createCGImage:image fromRect:r]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CGImageRef CIContextCreateCGImageWithFormat( CIContextRef ctx, CIImageRef image, CGRect r, CIFormat format, CGColorSpaceRef colorSpace, Boolean deferred )
{ return [ctx createCGImage:image fromRect:r format:format colorSpace:colorSpace deferred:deferred]; }
#endif

void CIContextRenderToBitmap( CIContextRef ctx, CIImageRef image, void *dta, long rowBytes, CGRect bounds, CIFormat format, CGColorSpaceRef colorSpace )
{ [ctx render:image toBitmap:dta rowBytes:rowBytes bounds:bounds format:format colorSpace:colorSpace]; }

// Draw images
void CIContextDrawImage( CIContextRef ctx, CIImageRef image, CGRect inRect, CGRect fromRect )
{ [ctx drawImage:image inRect:inRect fromRect:fromRect]; }

// Manage resources
void CIContextClearCaches( CIContextRef ctx )
{ [ctx clearCaches]; }

void CIContextReclaimResources( CIContextRef ctx )
{ [ctx reclaimResources]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
unsigned long CIContextOfflineGPUCount( void )
{ return [CIContext offlineGPUCount]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGColorSpaceRef CIContextWorkingColorSpace( CIContextRef ctx )
{ return [ctx workingColorSpace]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CIFormat CIContextWorkingFormat( CIContextRef ctx )
{ return [ctx workingFormat]; }
#endif

// Render image for data of file export
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFDataRef CIContextTIFFRepresentationOfImage( CIContextRef ctx, CIImageRef image, CIFormat format, CGColorSpaceRef colorSpace, CFDictionaryRef options )
{ return (CFDataRef)[ctx TIFFRepresentationOfImage:image format:format colorSpace:colorSpace options:(NSDictionary *)options]; }

CFDataRef CIContextJPEGRepresentationOfImage( CIContextRef ctx, CIImageRef image, CGColorSpaceRef colorSpace, CFDictionaryRef options )
{ return (CFDataRef)[ctx JPEGRepresentationOfImage:image colorSpace:colorSpace options:(NSDictionary *)options]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFDataRef CIContextPNGRepresentationOfImage( CIContextRef ctx, CIImageRef image, CIFormat format, CGColorSpaceRef colorSpace, CFDictionaryRef options )
{ return (CFDataRef)[ctx PNGRepresentationOfImage:image format:format colorSpace:colorSpace options:(NSDictionary *)options]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101304 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101304
CFDataRef CIContextHEIFRepresentationOfImage( CIContextRef ctx, CIImageRef image, CIFormat format, CGColorSpaceRef colorSpace, CFDictionaryRef options )
{ return (CFDataRef)[ctx HEIFRepresentationOfImage:image format:format colorSpace:colorSpace options:(NSDictionary *)options]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean CIContextWriteTIFFRepresentationOfImageToURL( CIContextRef ctx, CIImageRef image, CFURLRef url, CIFormat format, CGColorSpaceRef colorSpace, CFDictionaryRef options, ErrorRef *err )
{ return [ctx writeTIFFRepresentationOfImage:image toURL:(NSURL *)url format:format colorSpace:colorSpace options:(NSDictionary *)options error:err]; }

Boolean CIContextWriteJPEGRepresentationOfImageToURL( CIContextRef ctx, CIImageRef image, CFURLRef url, CGColorSpaceRef colorSpace, CFDictionaryRef options, ErrorRef *err )
{ return [ctx writeJPEGRepresentationOfImage:image toURL:(NSURL *)url colorSpace:colorSpace options:(NSDictionary *)options error:err]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean CIContextWritePNGRepresentationOfImageToURL( CIContextRef ctx, CIImageRef image, CFURLRef url, CIFormat format, CGColorSpaceRef colorSpace, CFDictionaryRef options, ErrorRef *err )
{ return [ctx writePNGRepresentationOfImage:image toURL:(NSURL *)url format:format colorSpace:colorSpace options:(NSDictionary *)options error:err]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101304 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101304
Boolean CIContextWriteHEIFRepresentationOfImageToURL( CIContextRef ctx, CIImageRef image, CFURLRef url, CIFormat format, CGColorSpaceRef colorSpace, CFDictionaryRef options, ErrorRef *err )
{ return [ctx writeHEIFRepresentationOfImage:image toURL:(NSURL *)url format:format colorSpace:colorSpace options:(NSDictionary *)options error:err]; }
#endif

// Customize render destination
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean CIContextPrepareRender( CIContextRef ctx, CIImageRef image, CGRect fromRect, CIRenderDestinationRef destination, CGPoint atPoint, ErrorRef *err )
{ return [ctx prepareRender:image fromRect:fromRect toDestination:destination atPoint:atPoint error:err]; }

CIRenderTaskRef CIContextStartTaskToClear( CIContextRef ctx, CIRenderDestinationRef destination, ErrorRef *err )
{ return [ctx startTaskToClear:destination error:err]; }

CIRenderTaskRef CIContextStartTaskToRenderFromRect( CIContextRef ctx, CIImageRef image, CGRect fromRect, CIRenderDestinationRef destination, CGPoint atPoint, ErrorRef *err )
{ return [ctx startTaskToRender:image fromRect:fromRect toDestination:destination atPoint:atPoint error:err]; }

CIRenderTaskRef CIContextStartTaskToRender( CIContextRef ctx, CIImageRef image, CIRenderDestinationRef destination, ErrorRef *err )
{ return [ctx startTaskToRender:image toDestination:destination error:err]; }
#endif

// Instance methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
CIFilterRef CIContextDepthBlurEffectFilterForImage( CIContextRef ctx, CIImageRef image, CIImageRef disparityImage, CIImageRef portraitEffectsMatte, CGImagePropertyOrientation orientation, CFDictionaryRef options )
{ return [ctx depthBlurEffectFilterForImage:image disparityImage:disparityImage portraitEffectsMatte:portraitEffectsMatte orientation:orientation options:(NSDictionary *)options]; }

CIFilterRef CIContextDepthBlurEffectFilterForImageData( CIContextRef ctx, CFDataRef dta, CFDictionaryRef options )
{ return [ctx depthBlurEffectFilterForImageData:(NSData *)dta options:(NSDictionary *)options]; }

CIFilterRef CIContextDepthBlurEffectFilterForImageURL( CIContextRef ctx, CFURLRef url, CFDictionaryRef options )
{ return [ctx depthBlurEffectFilterForImageURL:(NSURL *)url options:(NSDictionary *)options]; }
#endif
