/*
 CIFilter.m

 Bernie Wylde
 */

#import "CIFilter.h"

// - create -
CIFilterRef CIFilterWithName( CFStringRef name )
{ return [CIFilter filterWithName:(NSString *)name]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CIFilterRef CIFilterWithNameAndInputParamters( CFStringRef name, CFDictionaryRef params )
{ return [CIFilter filterWithName:(NSString *)name withInputParameters:(NSDictionary *)params]; }
#endif

// - create from raw image
CIFilterRef CIFilterWithImageData( CFDataRef dta, CFDictionaryRef options )
{ return [CIFilter filterWithImageData:(NSData *)dta options:(NSDictionary *)options]; }

CIFilterRef CIFilterWithImageURL( CFURLRef url, CFDictionaryRef options )
{ return [CIFilter filterWithImageURL:(NSURL *)url options:(NSDictionary *)options]; }

// - registered filters -
CFArrayRef CIFilterFilterNamesInCategories( CFArrayRef categories )
{ return (CFArrayRef)[CIFilter filterNamesInCategories:(NSArray *)categories]; }

CFArrayRef CIFilterFilterNamesInCategory( CFStringRef category )
{ return (CFArrayRef)[CIFilter filterNamesInCategory:(NSString *)category]; }

// - registering a filter -

// - parameters and attributes -
void CIFilterSetName( CIFilterRef ref, CFStringRef name )
{ [ref setName:(NSString *)name]; }

Boolean CIFilterIsEnabled( CIFilterRef ref )
{ return [ref isEnabled]; }

CFDictionaryRef CIFilterAttributes( CIFilterRef ref )
{ return (CFDictionaryRef)[ref attributes]; }

CFArrayRef CIFilterInputKeys( CIFilterRef ref )
{ return (CFArrayRef)[ref inputKeys]; }

CFArrayRef CIFilterOutputKeys( CIFilterRef ref )
{ return (CFArrayRef)[ref outputKeys]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CIImageRef CIFilterOutputImage( CIFilterRef ref )
{ return [ref outputImage]; }
#endif

// - default values -
void CIFilterSetDefaults( CIFilterRef ref )
{ [ref setDefaults]; }

// - apply -

// - localized info for registered filters -
CFStringRef CIFilterLocalizedNameForFilterName( CFStringRef filterName )
{ return (CFStringRef)[CIFilter localizedNameForFilterName:(NSString *)filterName]; }

CFStringRef CIFilterLocalizedNameForCategory( CFStringRef category )
{ return (CFStringRef)[CIFilter localizedNameForCategory:(NSString *)category]; }

CFStringRef CIFilterLocalizedDescriptionForFilterName( CFStringRef filterName )
{ return (CFStringRef)[CIFilter localizedDescriptionForFilterName:(NSString *)filterName]; }

CFURLRef CIFilterLocalizedReferenceDocumentationForFilterName( CFStringRef filterName )
{ return (CFURLRef)[CIFilter localizedReferenceDocumentationForFilterName:(NSString *)filterName]; }

// - create configuration view -

// - serialize and deserialize -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFDataRef CIFilterSerializedXMPFromFilters( CFArrayRef filters, CGRect extent )
{ return (CFDataRef)[CIFilter serializedXMPFromFilters:(NSArray *)filters inputImageExtent:extent]; }

CFArrayRef CIFilterFilterArrayFromSerializedXMP( CFDataRef xmpData, CGRect extent, ErrorRef *err )
{ return (CFArrayRef)[CIFilter filterArrayFromSerializedXMP:(NSData *)xmpData inputImageExtent:extent error:err]; }
#endif

