/*
 CIImage.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// Create -
CIImageRef CIImageEmptyImage( CFStringRef name );
CIImageRef CIImageWithColor( CIColorRef col );
CIImageRef CIImageWithBitmapData( CFDataRef dta, UInt32 bytesPerRow, CGSize size, CIFormat format, CGColorSpaceRef cs );
CIImageRef CIImageWithCGImage( CGImageRef image );
CIImageRef CIImageWithCGImageAndOptions( CGImageRef image, CFDictionaryRef options );
CIImageRef CIImageWithContentsOfURL( CFURLRef url );
CIImageRef CIImageWithContentsOfURLAndOptions( CFURLRef url, CFDictionaryRef options );
CIImageRef CIImageWithData( CFDataRef dta );
CIImageRef CIImageWithDataAndOptions( CFDataRef dta, CFDictionaryRef options );

// - create by modifying existing image -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CIImageRef CIImageByApplyingFilterWithInputParameters( CIImageRef ref, CFStringRef filterName, CFDictionaryRef params );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CIImageRef CIImageByApplyingFilter( CIImageRef ref, CFStringRef filterName );
#endif

CIImageRef CIImageByApplyingTransform( CIImageRef ref, CGAffineTransform matrix );
CIImageRef CIImageByCroppingToRect( CIImageRef ref, CGRect r );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CIImageRef CIImageByApplyingOrientation( CIImageRef ref, int orientation );
CIImageRef CIImageByClampingToExtent( CIImageRef ref );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CIImageRef CIImageByClampingToRect( CIImageRef ref, CGRect r );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CIImageRef CIImageByCompositingOverImage( CIImageRef ref, CIImageRef otherImage );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CIImageRef CIImageByPremultiplyingAlpha( CIImageRef ref );
CIImageRef CIImageByUnpremultiplyingAlpha( CIImageRef ref );
CIImageRef CIImageBySettingAlphaOneInExtent( CIImageRef ref, CGRect extent );
CIImageRef CIImageByApplyingGaussianBlurWithSigma( CIImageRef ref, double sigma );
CIImageRef CIImageBySettingProperties( CIImageRef ref, CFDictionaryRef properties );
#endif

// - info -
CGRect CIImageExtent( CIImageRef ref );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFDictionaryRef CIImageProperties( CIImageRef ref );
#endif

CFURLRef CIImageURL( CIImageRef ref );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGAffineTransform CIImageTransformForOrientation( CIImageRef ref, int orientation );
#endif

// - drawing -
void CIImageDrawAtPoint( CIImageRef ref, CGPoint pt, CGRect fromRect, NSCompositingOperation op, CGFloat fraction );
void CIImageDrawInRect( CIImageRef ref, CGRect r, CGRect fromRect, NSCompositingOperation op, CGFloat fraction );

// - autoadjustment filters -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFArrayRef CIImageAutoAdjustmentFilters( CIImageRef ref );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFArrayRef CIImageAutoAdjustmentFiltersWithOptions( CIImageRef ref, CFDictionaryRef options );
#endif

// - filter regions of interest -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGRect CIImageRegionOfInterestForImage( CIImageRef ref, CIImageRef otherImage, CGRect r );
#endif

// Orientation
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CIImageRef CIImageByApplyingCGOrientation( CIImageRef ref, CGImagePropertyOrientation orientation );
CGAffineTransform CIImageTransformForCGOrientation( CIImageRef ref, CGImagePropertyOrientation orientation );

// Sampling
CIImageRef CIImageBySamplingNearest( CIImageRef ref );
CIImageRef CIImageBySamplingLinear( CIImageRef ref );
#endif

// Original image content
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CGImageRef CIImageCGImage( CIImageRef ref );
#endif
