/*
 CIImage.m

 Bernie Wylde
 */

#import "CIImage.h"

// Create
CIImageRef CIImageEmptyImage( CFStringRef name )
{ return [CIImage emptyImage]; }

CIImageRef CIImageWithColor( CIColorRef col )
{ return [CIImage imageWithColor:col]; }

CIImageRef CIImageWithBitmapData( CFDataRef dta, UInt32 bytesPerRow, CGSize size, CIFormat format, CGColorSpaceRef cs )
{ return [CIImage imageWithBitmapData:(NSData *)dta bytesPerRow:bytesPerRow size:size format:format colorSpace:cs]; }

CIImageRef CIImageWithCGImage( CGImageRef image )
{ return [CIImage imageWithCGImage:image]; }

CIImageRef CIImageWithCGImageAndOptions( CGImageRef image, CFDictionaryRef options )
{ return [CIImage imageWithCGImage:image options:(NSDictionary *)options]; }

CIImageRef CIImageWithContentsOfURL( CFURLRef url )
{ return [CIImage imageWithContentsOfURL:(NSURL *)url]; }

CIImageRef CIImageWithContentsOfURLAndOptions( CFURLRef url, CFDictionaryRef options )
{ return [CIImage imageWithContentsOfURL:(NSURL *)url options:(NSDictionary *)options]; }

CIImageRef CIImageWithData( CFDataRef dta )
{ return [CIImage imageWithData:(NSData *)dta]; }

CIImageRef CIImageWithDataAndOptions( CFDataRef dta, CFDictionaryRef options )
{ return [CIImage imageWithData:(NSData *)dta options:(NSDictionary *)options]; }

// - create by modifying existing image -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CIImageRef CIImageByApplyingFilterWithInputParameters( CIImageRef ref, CFStringRef filterName, CFDictionaryRef params )
{ return [ref imageByApplyingFilter:(NSString *)filterName withInputParameters:(NSDictionary *)params]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CIImageRef CIImageByApplyingFilter( CIImageRef ref, CFStringRef filterName )
{ return [ref imageByApplyingFilter:(NSString *)filterName]; }
#endif

CIImageRef CIImageByApplyingTransform( CIImageRef ref, CGAffineTransform matrix )
{ return [ref imageByApplyingTransform:matrix]; }

CIImageRef CIImageByCroppingToRect( CIImageRef ref, CGRect r )
{ return [ref imageByCroppingToRect:r]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CIImageRef CIImageByApplyingOrientation( CIImageRef ref, int orientation )
{ return [ref imageByApplyingOrientation:orientation]; }

CIImageRef CIImageByClampingToExtent( CIImageRef ref )
{ return [ref imageByClampingToExtent]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CIImageRef CIImageByClampingToRect( CIImageRef ref, CGRect r )
{ return [ref imageByClampingToRect:r]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CIImageRef CIImageByCompositingOverImage( CIImageRef ref, CIImageRef otherImage )
{ return [ref imageByCompositingOverImage:otherImage]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CIImageRef CIImageByPremultiplyingAlpha( CIImageRef ref )
{ return [ref imageByPremultiplyingAlpha]; }

CIImageRef CIImageByUnpremultiplyingAlpha( CIImageRef ref )
{ return [ref imageByUnpremultiplyingAlpha]; }

CIImageRef CIImageBySettingAlphaOneInExtent( CIImageRef ref, CGRect extent )
{ return [ref imageBySettingAlphaOneInExtent:extent]; }

CIImageRef CIImageByApplyingGaussianBlurWithSigma( CIImageRef ref, double sigma )
{ return [ref imageByApplyingGaussianBlurWithSigma:sigma]; }

CIImageRef CIImageBySettingProperties( CIImageRef ref, CFDictionaryRef properties )
{ return [ref imageBySettingProperties:(NSDictionary *)properties]; }
#endif

// - info -
CGRect CIImageExtent( CIImageRef ref )
{ return [ref extent]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFDictionaryRef CIImageProperties( CIImageRef ref )
{ return (CFDictionaryRef)[ref properties]; }
#endif

CFURLRef CIImageURL( CIImageRef ref )
{ return (CFURLRef)[ref url]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGAffineTransform CIImageTransformForOrientation( CIImageRef ref, int orientation )
{ return [ref imageTransformForOrientation:orientation]; }
#endif

// - drawing -
void CIImageDrawAtPoint( CIImageRef ref, CGPoint pt, CGRect fromRect, NSCompositingOperation op, CGFloat fraction )
{ return [ref drawAtPoint:NSPointFromCGPoint(pt) fromRect:NSRectFromCGRect(fromRect) operation:op fraction:fraction]; }

void CIImageDrawInRect( CIImageRef ref, CGRect r, CGRect fromRect, NSCompositingOperation op, CGFloat fraction )
{ return [ref drawInRect:NSRectFromCGRect(r) fromRect:NSRectFromCGRect(fromRect) operation:op fraction:fraction]; }

// - autoadjustment filters -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFArrayRef CIImageAutoAdjustmentFilters( CIImageRef ref )
{ return (CFArrayRef)[ref autoAdjustmentFilters]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFArrayRef CIImageAutoAdjustmentFiltersWithOptions( CIImageRef ref, CFDictionaryRef options )
{ return (CFArrayRef)[ref autoAdjustmentFiltersWithOptions:(NSDictionary *)options]; }
#endif

// - filter regions of interest -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGRect CIImageRegionOfInterestForImage( CIImageRef ref, CIImageRef otherImage, CGRect r )
{ return [ref regionOfInterestForImage:otherImage inRect:r]; }
#endif

// Orientation
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CIImageRef CIImageByApplyingCGOrientation( CIImageRef ref, CGImagePropertyOrientation orientation )
{ return [ref imageByApplyingCGOrientation:orientation]; }

CGAffineTransform CIImageTransformForCGOrientation( CIImageRef ref, CGImagePropertyOrientation orientation )
{ return [ref imageTransformForCGOrientation:orientation]; }

// Sampling
CIImageRef CIImageBySamplingNearest( CIImageRef ref )
{ return [ref imageBySamplingNearest]; }

CIImageRef CIImageBySamplingLinear( CIImageRef ref )
{ return [ref imageBySamplingLinear]; }
#endif

// Original image content
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CGImageRef CIImageCGImage( CIImageRef ref )
{ return [ref CGImage]; }
#endif
