/*
 Calendar.h
 
 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef BOOL (*CocoaCalendarEnumerateDatesCallbackType)(CFCalendarRef,CFDateRef,BOOL,void*);

// - create -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFCalendarRef CalendarWithIdentifier( CFStringRef identifier );
#endif

// - user's calendar -
CFCalendarRef CalendarCurrent( void );
CFCalendarRef CalendarAutoupdatingCurrent( void );

// - components -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
Boolean CalendarDateMatches( CFCalendarRef cal, CFDateRef dt, DateComponentsRef comps );
NSInteger CalendarComponentFromDate( CFCalendarRef cal, NSUInteger unit, CFDateRef dt );
#endif
DateComponentsRef CalendarComponentsFromDate( CFCalendarRef cal, NSUInteger unitFlags, CFDateRef dt );
DateComponentsRef CalendarComponentsFromDateToDate( CFCalendarRef cal, NSUInteger unitFlags, CFDateRef fromDate, CFDateRef toDate, NSUInteger options );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
DateComponentsRef CalendarComponentsFromDateComponentsToDateComponents( CFCalendarRef cal, NSUInteger unitFlags, DateComponentsRef fromComps, DateComponentsRef toComps, NSCalendarOptions options );
DateComponentsRef CalendarComponentsInTimeZoneFromDate( CFCalendarRef cal, CFTimeZoneRef zone, CFDateRef dt );
void CalendarGetEraYearMonthDayFromDate( CFCalendarRef cal, NSInteger *era, NSInteger *year, NSInteger *month, NSInteger *day, CFDateRef dt );
void CalendarGetEraYearForWeekOfYear( CFCalendarRef cal, NSInteger *era, NSInteger *year, NSInteger *weekOfYear, NSInteger *weekday, CFDateRef dt );
void CalendarGetHourMinuteSecondNanosecondFromDate( CFCalendarRef cal, NSInteger *hour, NSInteger *min, NSInteger *second, NSInteger *nanosecond, CFDateRef dt );
#endif

// - info -
CFStringRef CalendarIdentifier( CFCalendarRef cal );
NSUInteger CalendarFirstWeekday( CFCalendarRef cal );
CFLocaleRef CalendarLocale( CFCalendarRef cal );
CFTimeZoneRef CalendarTimeZone( CFCalendarRef cal );
CFRange CalendarMaximumRangeOfUnit( CFCalendarRef cal, NSCalendarUnit unit );
CFRange CalendarMinimumRangeOfUnit( CFCalendarRef cal, NSCalendarUnit unit );
NSUInteger CalendarMinimumDaysInFirstWeek( CFCalendarRef cal );
NSUInteger CalendarOrdinalityOfUnitInUnit( CFCalendarRef cal, NSUInteger ofUnit, NSUInteger inUnit, CFDateRef dt );
CFRange CalendarRangeOfUnitInUnit( CFCalendarRef cal, NSUInteger ofUnit, NSUInteger inUnit, CFDateRef dt );
Boolean CalendarRangeOfUnit( CFCalendarRef cal, NSUInteger unit, CFDateRef *startDate, CFTimeInterval *interval, CFDateRef forDate );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
Boolean CalendarRangeOfWeekend( CFCalendarRef cal, CFDateRef *startDate, CFTimeInterval *interval, CFDateRef containingDate );
#endif

// - scanning dates -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFDateRef CalendarStartOfDayForDate( CFCalendarRef cal, CFDateRef dt );
void CalendarEnumerateDates( CFCalendarRef cal, CFDateRef startDate, DateComponentsRef matchingComponents, NSUInteger options, CocoaCalendarEnumerateDatesCallbackType callback, void *userData );
CFDateRef CalendarNextDateAfterDateMatchingComponents( CFCalendarRef cal, CFDateRef dt, DateComponentsRef comps, NSCalendarOptions options );
CFDateRef CalendarNextDateAfterDateMatchingHourMinuteSeconds( CFCalendarRef cal, CFDateRef dt, NSInteger hour, NSInteger minute, NSInteger second, NSCalendarOptions options );
CFDateRef CalendarNextDateAfterDateMatchingUnit( CFCalendarRef cal, CFDateRef dt, NSUInteger unit, NSInteger value, NSCalendarOptions options );
#endif

// - calculating dates -
CFDateRef CalendarDateFromComponents( CFCalendarRef cal, DateComponentsRef comps );
CFDateRef CalendarDateByAddingComponents( CFCalendarRef cal, DateComponentsRef comps, CFDateRef dt, NSUInteger options );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFDateRef CalendarDateByAddingUnit( CFCalendarRef cal, NSUInteger unit, NSInteger value, CFDateRef dt, NSCalendarOptions options );
CFDateRef CalendarDateBySettingHourMinuteSecond( CFCalendarRef cal, NSInteger hour, NSInteger minute, NSInteger second, CFDateRef dt, NSCalendarOptions options );
CFDateRef CalendarDateBySettingUnit( CFCalendarRef cal, NSUInteger unit, NSInteger value, CFDateRef dt, NSCalendarOptions options );
CFDateRef CalendarDateWithEra( CFCalendarRef cal, NSInteger era, NSInteger year, NSInteger month, NSInteger day, NSInteger hour, NSInteger minute, NSInteger second, NSInteger nanosecond );
CFDateRef CalendarDateWithEraYearForWeekOfYear( CFCalendarRef cal, NSInteger era, NSInteger year, NSInteger week, NSInteger weekday, NSInteger hour, NSInteger minute, NSInteger second, NSInteger nanosecond );
Boolean CalendarNextWeekendStartDate( CFCalendarRef cal, CFDateRef *dt, CFTimeInterval *interval, NSCalendarOptions options, CFDateRef afterDate );
#endif

// - comparing dates -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSComparisonResult CalendarCompareDateToDate( CFCalendarRef cal, CFDateRef dt1, CFDateRef dt2, NSCalendarUnit unit );
Boolean CalendarIsDateEqualToDate( CFCalendarRef cal, CFDateRef dt1, CFDateRef dt2, NSCalendarUnit unit );
Boolean CalendarIsDateInSameDayAsDate( CFCalendarRef cal, CFDateRef dt1, CFDateRef dt2 );
Boolean CalendarIsDateInToday( CFCalendarRef cal, CFDateRef dt );
Boolean CalendarIsDateInTomorrow( CFCalendarRef cal, CFDateRef dt );
Boolean CalendarIsDateInWeekend( CFCalendarRef cal, CFDateRef dt );
Boolean CalendarIsDateInYesterday( CFCalendarRef cal, CFDateRef dt );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
// - am and pm symbols
CFStringRef CalendarAMSymbol( CFCalendarRef cal );
CFStringRef CalendarPMSymbol( CFCalendarRef cal );

// - weekday symbols -
CFArrayRef CalendarWeekdaySymbols( CFCalendarRef cal );
CFArrayRef CalendarShortWeekdaySymbols( CFCalendarRef cal );
CFArrayRef CalendarVeryShortWeekdaySymbols( CFCalendarRef cal );
CFArrayRef CalendarStandaloneWeekdaySymbols( CFCalendarRef cal );
CFArrayRef CalendarShortStandaloneWeekdaySymbols( CFCalendarRef cal );
CFArrayRef CalendarVeryShortStandaloneWeekdaySymbols( CFCalendarRef cal );

// - month symbols -
CFArrayRef CalendarMonthSymbols( CFCalendarRef cal );
CFArrayRef CalendarShortMonthSymbols( CFCalendarRef cal );
CFArrayRef CalendarVeryShortMonthSymbols( CFCalendarRef cal );
CFArrayRef CalendarStandaloneMonthSymbols( CFCalendarRef cal );
CFArrayRef CalendarShortStandaloneMonthSymbols( CFCalendarRef cal );
CFArrayRef CalendarVeryShortStandaloneMonthSymbols( CFCalendarRef cal );

// - quarter symbols -
CFArrayRef CalendarQuarterSymbols( CFCalendarRef cal );
CFArrayRef CalendarShortQuarterSymbols( CFCalendarRef cal );
CFArrayRef CalendarStandaloneQuarterSymbols( CFCalendarRef cal );
CFArrayRef CalendarShortStandaloneQuarterSymbols( CFCalendarRef cal );

// - era symbols -
CFArrayRef CalendarEraSymbols( CFCalendarRef cal );
CFArrayRef CalendarLongEraSymbols( CFCalendarRef cal );
#endif

