/*
 Calendar.m
 
 Bernie Wylde
 */

#import "Calendar.h"

// - create -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFCalendarRef CalendarWithIdentifier( CFStringRef identifier )
{ return (CFCalendarRef)[[[NSCalendar alloc] initWithCalendarIdentifier:(NSString *)identifier] autorelease]; }
#endif

// - user's calendar -
CFCalendarRef CalendarCurrent( void )
{ return (CFCalendarRef)[NSCalendar currentCalendar]; }

CFCalendarRef CalendarAutoupdatingCurrent( void )
{ return (CFCalendarRef)[NSCalendar autoupdatingCurrentCalendar]; }

// - components -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
Boolean CalendarDateMatches( CFCalendarRef cal, CFDateRef dt, DateComponentsRef comps )
{ return [(NSCalendar *)cal date:(NSDate *)dt matchesComponents:comps]; }

NSInteger CalendarComponentFromDate( CFCalendarRef cal, NSUInteger unit, CFDateRef dt )
{ return [(NSCalendar *)cal component:unit fromDate:(NSDate *)dt]; }
#endif

DateComponentsRef CalendarComponentsFromDate( CFCalendarRef cal, NSUInteger unitFlags, CFDateRef dt )
{ return [(NSCalendar *)cal components:unitFlags fromDate:(NSDate *)dt]; }

DateComponentsRef CalendarComponentsFromDateToDate( CFCalendarRef cal, NSUInteger unitFlags, CFDateRef fromDate, CFDateRef toDate, NSUInteger options )
{ return [(NSCalendar *)cal components:unitFlags fromDate:(NSDate *)fromDate toDate:(NSDate *)toDate options:options]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
DateComponentsRef CalendarComponentsFromDateComponentsToDateComponents( CFCalendarRef cal, NSUInteger unitFlags, DateComponentsRef fromComps, DateComponentsRef toComps, NSCalendarOptions options )
{ return [(NSCalendar *)cal components:unitFlags fromDateComponents:fromComps toDateComponents:toComps options:options]; }

DateComponentsRef CalendarComponentsInTimeZoneFromDate( CFCalendarRef cal, CFTimeZoneRef zone, CFDateRef dt )
{ return [(NSCalendar *)cal componentsInTimeZone:(NSTimeZone *)zone fromDate:(NSDate *)dt]; }

void CalendarGetEraYearMonthDayFromDate( CFCalendarRef cal, NSInteger *era, NSInteger *year, NSInteger *month, NSInteger *day, CFDateRef dt )
{ return [(NSCalendar *)cal getEra:era year:year month:month day:day fromDate:(NSDate *)dt]; }

void CalendarGetEraYearForWeekOfYear( CFCalendarRef cal, NSInteger *era, NSInteger *year, NSInteger *weekOfYear, NSInteger *weekday, CFDateRef dt )
{ return [(NSCalendar *)cal getEra:era yearForWeekOfYear:year weekOfYear:weekOfYear weekday:weekday fromDate:(NSDate *)dt]; }

void CalendarGetHourMinuteSecondNanosecondFromDate( CFCalendarRef cal, NSInteger *hour, NSInteger *min, NSInteger *second, NSInteger *nanosecond, CFDateRef dt )
{ [(NSCalendar *)cal getHour:hour minute:min second:second nanosecond:nanosecond fromDate:(NSDate *)dt]; }
#endif

// - info -
CFStringRef CalendarIdentifier( CFCalendarRef cal )
{ return (CFStringRef)[(NSCalendar *)cal calendarIdentifier]; }

NSUInteger CalendarFirstWeekday( CFCalendarRef cal )
{ return [(NSCalendar *)cal firstWeekday]; }

CFLocaleRef CalendarLocale( CFCalendarRef cal )
{ return (CFLocaleRef)[(NSCalendar *)cal locale]; }

CFTimeZoneRef CalendarTimeZone( CFCalendarRef cal )
{ return (CFTimeZoneRef)[(NSCalendar *)cal timeZone]; }

CFRange CalendarMaximumRangeOfUnit( CFCalendarRef cal, NSCalendarUnit unit )
{
    NSRange range = [(NSCalendar *)cal maximumRangeOfUnit:unit];
    return CFRangeMake( range.location, range.length );
}

CFRange CalendarMinimumRangeOfUnit( CFCalendarRef cal, NSCalendarUnit unit )
{
    NSRange range = [(NSCalendar *)cal minimumRangeOfUnit:unit];
    return CFRangeMake( range.location, range.length );
}

NSUInteger CalendarMinimumDaysInFirstWeek( CFCalendarRef cal )
{ return [(NSCalendar *)cal minimumDaysInFirstWeek]; }

NSUInteger CalendarOrdinalityOfUnitInUnit( CFCalendarRef cal, NSUInteger ofUnit, NSUInteger inUnit, CFDateRef dt )
{ return [(NSCalendar *)cal ordinalityOfUnit:ofUnit inUnit:inUnit forDate:(NSDate *)dt]; }

CFRange CalendarRangeOfUnitInUnit( CFCalendarRef cal, NSUInteger ofUnit, NSUInteger inUnit, CFDateRef dt )
{
    NSRange range = [(NSCalendar *)cal rangeOfUnit:ofUnit inUnit:inUnit forDate:(NSDate *)dt];
    return CFRangeMake( range.location, range.length );
}

Boolean CalendarRangeOfUnit( CFCalendarRef cal, NSUInteger unit, CFDateRef *startDate, CFTimeInterval *interval, CFDateRef forDate )
{ return [(NSCalendar *)cal rangeOfUnit:unit startDate:(NSDate **)startDate interval:interval forDate:(NSDate *)forDate]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
Boolean CalendarRangeOfWeekend( CFCalendarRef cal, CFDateRef *startDate, CFTimeInterval *interval, CFDateRef containingDate )
{ return [(NSCalendar *)cal rangeOfWeekendStartDate:(NSDate **)startDate interval:interval containingDate:(NSDate *)containingDate]; }
#endif

// - scanning dates -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFDateRef CalendarStartOfDayForDate( CFCalendarRef cal, CFDateRef dt )
{ return (CFDateRef)[(NSCalendar *)cal startOfDayForDate:(NSDate *)dt]; }

void CalendarEnumerateDates( CFCalendarRef cal, CFDateRef startDate, DateComponentsRef matchingComponents, NSUInteger options, CocoaCalendarEnumerateDatesCallbackType callback, void *userData )
{
    [(NSCalendar *)cal enumerateDatesStartingAfterDate:(NSDate *)startDate matchingComponents:matchingComponents options:options usingBlock:^(NSDate *date, BOOL exactMatch, BOOL *stop) {
        BOOL flag = NO;
        if ( callback ) flag = (*callback)(cal,(CFDateRef)date,exactMatch,userData);
        if ( flag ) *stop = YES;
    }];
}

CFDateRef CalendarNextDateAfterDateMatchingComponents( CFCalendarRef cal, CFDateRef dt, DateComponentsRef comps, NSCalendarOptions options )
{ return (CFDateRef)[(NSCalendar *)cal nextDateAfterDate:(NSDate *)dt matchingComponents:comps options:options]; }

CFDateRef CalendarNextDateAfterDateMatchingHourMinuteSeconds( CFCalendarRef cal, CFDateRef dt, NSInteger hour, NSInteger minute, NSInteger second, NSCalendarOptions options )
{ return (CFDateRef)[(NSCalendar *)cal nextDateAfterDate:(NSDate *)dt matchingHour:hour minute:minute second:second options:options]; }

CFDateRef CalendarNextDateAfterDateMatchingUnit( CFCalendarRef cal, CFDateRef dt, NSUInteger unit, NSInteger value, NSCalendarOptions options )
{ return (CFDateRef)[(NSCalendar *)cal nextDateAfterDate:(NSDate *)dt matchingUnit:unit value:value options:options]; }
#endif

// - calculating dates -
CFDateRef CalendarDateFromComponents( CFCalendarRef cal, DateComponentsRef comps )
{ return (CFDateRef)[(NSCalendar *)cal dateFromComponents:comps]; }

CFDateRef CalendarDateByAddingComponents( CFCalendarRef cal, DateComponentsRef comps, CFDateRef dt, NSUInteger options )
{ return (CFDateRef)[(NSCalendar *)cal dateByAddingComponents:comps toDate:(NSDate *)dt options:options]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFDateRef CalendarDateByAddingUnit( CFCalendarRef cal, NSUInteger unit, NSInteger value, CFDateRef dt, NSCalendarOptions options )
{ return (CFDateRef)[(NSCalendar *)cal dateByAddingUnit:unit value:value toDate:(NSDate *)dt options:options]; }

CFDateRef CalendarDateBySettingHourMinuteSecond( CFCalendarRef cal, NSInteger hour, NSInteger minute, NSInteger second, CFDateRef dt, NSCalendarOptions options )
{ return (CFDateRef)[(NSCalendar *)cal dateBySettingHour:hour minute:minute second:second ofDate:(NSDate *)dt options:options]; }

CFDateRef CalendarDateBySettingUnit( CFCalendarRef cal, NSUInteger unit, NSInteger value, CFDateRef dt, NSCalendarOptions options )
{ return (CFDateRef)[(NSCalendar *)cal dateBySettingUnit:unit value:value ofDate:(NSDate *)dt options:options]; }

CFDateRef CalendarDateWithEra( CFCalendarRef cal, NSInteger era, NSInteger year, NSInteger month, NSInteger day, NSInteger hour, NSInteger minute, NSInteger second, NSInteger nanosecond )
{ return (CFDateRef)[(NSCalendar *)cal dateWithEra:era year:year month:month day:day hour:hour minute:minute second:second nanosecond:nanosecond]; }

CFDateRef CalendarDateWithEraYearForWeekOfYear( CFCalendarRef cal, NSInteger era, NSInteger year, NSInteger week, NSInteger weekday, NSInteger hour, NSInteger minute, NSInteger second, NSInteger nanosecond )
{ return (CFDateRef)[(NSCalendar *)cal dateWithEra:era yearForWeekOfYear:year weekOfYear:week weekday:weekday hour:hour minute:minute second:second nanosecond:nanosecond]; }

Boolean CalendarNextWeekendStartDate( CFCalendarRef cal, CFDateRef *dt, CFTimeInterval *interval, NSCalendarOptions options, CFDateRef afterDate )
{ return [(NSCalendar *)cal nextWeekendStartDate:(NSDate **)dt interval:interval options:options afterDate:(NSDate *)afterDate]; }
#endif

// - comparing dates -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSComparisonResult CalendarCompareDateToDate( CFCalendarRef cal, CFDateRef dt1, CFDateRef dt2, NSCalendarUnit unit )
{ return [(NSCalendar *)cal compareDate:(NSDate *)dt1 toDate:(NSDate *)dt2 toUnitGranularity:unit]; }

Boolean CalendarIsDateEqualToDate( CFCalendarRef cal, CFDateRef dt1, CFDateRef dt2, NSCalendarUnit unit )
{ return [(NSCalendar *)cal isDate:(NSDate *)dt1 equalToDate:(NSDate *)dt2 toUnitGranularity:unit]; }

Boolean CalendarIsDateInSameDayAsDate( CFCalendarRef cal, CFDateRef dt1, CFDateRef dt2 )
{ return [(NSCalendar *)cal isDate:(NSDate *)dt1 inSameDayAsDate:(NSDate *)dt2]; }

Boolean CalendarIsDateInToday( CFCalendarRef cal, CFDateRef dt )
{ return [(NSCalendar *)cal isDateInToday:(NSDate *)dt]; }

Boolean CalendarIsDateInTomorrow( CFCalendarRef cal, CFDateRef dt )
{ return [(NSCalendar *)cal isDateInTomorrow:(NSDate *)dt]; }

Boolean CalendarIsDateInWeekend( CFCalendarRef cal, CFDateRef dt )
{ return [(NSCalendar *)cal isDateInWeekend:(NSDate *)dt]; }

Boolean CalendarIsDateInYesterday( CFCalendarRef cal, CFDateRef dt )
{ return [(NSCalendar *)cal isDateInYesterday:(NSDate *)dt]; }
#endif

// - am and pm symbols
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFStringRef CalendarAMSymbol( CFCalendarRef cal )
{ return (CFStringRef)[(NSCalendar *)cal AMSymbol]; }

CFStringRef CalendarPMSymbol( CFCalendarRef cal )
{ return (CFStringRef)[(NSCalendar *)cal PMSymbol]; }

// - weekday symbols -
CFArrayRef CalendarWeekdaySymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal weekdaySymbols]; }

CFArrayRef CalendarShortWeekdaySymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal shortWeekdaySymbols]; }

CFArrayRef CalendarVeryShortWeekdaySymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal veryShortWeekdaySymbols]; }

CFArrayRef CalendarStandaloneWeekdaySymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal standaloneWeekdaySymbols]; }

CFArrayRef CalendarShortStandaloneWeekdaySymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal shortStandaloneWeekdaySymbols]; }

CFArrayRef CalendarVeryShortStandaloneWeekdaySymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal veryShortStandaloneWeekdaySymbols]; }

// - month symbols -
CFArrayRef CalendarMonthSymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal monthSymbols]; }

CFArrayRef CalendarShortMonthSymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal shortMonthSymbols]; }

CFArrayRef CalendarVeryShortMonthSymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal veryShortMonthSymbols]; }

CFArrayRef CalendarStandaloneMonthSymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal standaloneMonthSymbols]; }

CFArrayRef CalendarShortStandaloneMonthSymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal shortStandaloneMonthSymbols]; }

CFArrayRef CalendarVeryShortStandaloneMonthSymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal veryShortStandaloneMonthSymbols]; }

// - quarter symbols -
CFArrayRef CalendarQuarterSymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal quarterSymbols]; }

CFArrayRef CalendarShortQuarterSymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal shortQuarterSymbols]; }

CFArrayRef CalendarStandaloneQuarterSymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal standaloneQuarterSymbols]; }

CFArrayRef CalendarShortStandaloneQuarterSymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal shortStandaloneQuarterSymbols]; }

// - era symbols -
CFArrayRef CalendarEraSymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal eraSymbols]; }

CFArrayRef CalendarLongEraSymbols( CFCalendarRef cal )
{ return (CFArrayRef)[(NSCalendar *)cal longEraSymbols]; }
#endif

