/*
 CharacterSet.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// Standard sets
CFCharacterSetRef CharacterSetAlphanumericSet( void );
CFCharacterSetRef CharacterSetCapitalizedLetterSet( void );
CFCharacterSetRef CharacterSetControlSet( void );
CFCharacterSetRef CharacterSetDecimalDigitSet( void );
CFCharacterSetRef CharacterSetDecomposableSet( void );
CFCharacterSetRef CharacterSetIllegalSet( void );
CFCharacterSetRef CharacterSetLetterSet( void );
CFCharacterSetRef CharacterSetLowercaseLetterSet( void );
CFCharacterSetRef CharacterSetNewlineSet( void );
CFCharacterSetRef CharacterSetNonBaseSet( void );
CFCharacterSetRef CharacterSetPuntuationSet( void );
CFCharacterSetRef CharacterSetSymbolSet( void );
CFCharacterSetRef CharacterSetUppercaseLetterSet( void );
CFCharacterSetRef CharacterSetWhitespaceAndNewlineSet( void );
CFCharacterSetRef CharacterSetWhitespaceSet( void );

// URL encoding
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFCharacterSetRef CharacterSetURLFragmentAllowedSet( void );
CFCharacterSetRef CharacterSetURLHostAllowedSet( void );
CFCharacterSetRef CharacterSetURLPasswordAllowedSet( void );
CFCharacterSetRef CharacterSetURLPathAllowedSet( void );
CFCharacterSetRef CharacterSetURLQueryAllowedSet( void );
CFCharacterSetRef CharacterSetURLUserAllowedSet( void );
#endif

// Custom sets
CFCharacterSetRef CharacterSetWithCoder( CoderRef coder );
CFCharacterSetRef CharacterSetWithCharactersInString( CFStringRef string );
CFCharacterSetRef CharacterSetWithRange( CFRange range );

// Bitmap representations
CFCharacterSetRef CharacterSetWithBitmapRepresentation( CFDataRef data );
CFCharacterSetRef CharacterSetWithContentsOfURL( CFURLRef url );
CFDataRef CharacterSetBitmapRepresentation( CFCharacterSetRef ref );

// Inverting
CFCharacterSetRef CharacterSetInvertedSet( CFCharacterSetRef set );

// - testing membership -
Boolean CharacterSetCharacterIsMember( CFCharacterSetRef set, unichar chr );
Boolean CharacterSetHasMemberInPlane( CFCharacterSetRef set, UInt8 plane );
Boolean CharacterSetIsSupersetOfSet( CFCharacterSetRef set, CFCharacterSetRef otherSet );
Boolean CharacterSetLongCharacterIsMember( CFCharacterSetRef set, UInt32 longChar );


/*
 MutabeCharacterSet
 */
// Adding and removing characters
void CharacterSetAddCharactersInRange( CFMutableCharacterSetRef set, CFRange range );
void CharacterSetRemoveCharactersInRange( CFMutableCharacterSetRef set, CFRange range );
void CharacterSetAddCharactersInString( CFMutableCharacterSetRef set, CFStringRef string );
void CharacterSetRemoveCharactersInString( CFMutableCharacterSetRef set, CFStringRef string );

// Combining sets
void CharacterSetFormIntersectionWithCharacterSet( CFMutableCharacterSetRef set, CFCharacterSetRef otherSet );
void CharacterSetFormUnionWithCharacterSet( CFMutableCharacterSetRef set, CFCharacterSetRef otherSet );

// Inverting
void CharacterSetInvert( CFMutableCharacterSetRef set );
