/*
 CharacterSet.m

 Bernie Wylde
 */

#import "CharacterSet.h"

// Standard sets
CFCharacterSetRef CharacterSetAlphanumericSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet alphanumericCharacterSet]; }

CFCharacterSetRef CharacterSetCapitalizedLetterSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet capitalizedLetterCharacterSet]; }

CFCharacterSetRef CharacterSetControlSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet controlCharacterSet]; }

CFCharacterSetRef CharacterSetDecimalDigitSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet decimalDigitCharacterSet]; }

CFCharacterSetRef CharacterSetDecomposableSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet decomposableCharacterSet]; }

CFCharacterSetRef CharacterSetIllegalSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet illegalCharacterSet]; }

CFCharacterSetRef CharacterSetLetterSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet letterCharacterSet]; }

CFCharacterSetRef CharacterSetLowercaseLetterSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet lowercaseLetterCharacterSet]; }

CFCharacterSetRef CharacterSetNewlineSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet newlineCharacterSet]; }

CFCharacterSetRef CharacterSetNonBaseSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet nonBaseCharacterSet]; }

CFCharacterSetRef CharacterSetPuntuationSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet punctuationCharacterSet]; }

CFCharacterSetRef CharacterSetSymbolSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet symbolCharacterSet]; }

CFCharacterSetRef CharacterSetUppercaseLetterSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet uppercaseLetterCharacterSet]; }

CFCharacterSetRef CharacterSetWhitespaceAndNewlineSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet whitespaceAndNewlineCharacterSet]; }

CFCharacterSetRef CharacterSetWhitespaceSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet whitespaceCharacterSet]; }

// URL encoding
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFCharacterSetRef CharacterSetURLFragmentAllowedSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet URLFragmentAllowedCharacterSet]; }

CFCharacterSetRef CharacterSetURLHostAllowedSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet URLHostAllowedCharacterSet]; }

CFCharacterSetRef CharacterSetURLPasswordAllowedSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet URLPasswordAllowedCharacterSet]; }

CFCharacterSetRef CharacterSetURLPathAllowedSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet URLPathAllowedCharacterSet]; }

CFCharacterSetRef CharacterSetURLQueryAllowedSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet URLQueryAllowedCharacterSet]; }

CFCharacterSetRef CharacterSetURLUserAllowedSet( void )
{ return (CFCharacterSetRef)[NSCharacterSet URLUserAllowedCharacterSet]; }
#endif

// Custom sets
CFCharacterSetRef CharacterSetWithCoder( CoderRef coder )
{ return (CFCharacterSetRef)[[[NSCharacterSet alloc] initWithCoder:coder] autorelease]; }

CFCharacterSetRef CharacterSetWithCharactersInString( CFStringRef string )
{ return (CFCharacterSetRef)[NSCharacterSet characterSetWithCharactersInString:(NSString *)string]; }

CFCharacterSetRef CharacterSetWithRange( CFRange range )
{ return (CFCharacterSetRef)[NSCharacterSet characterSetWithRange:NSMakeRange(range.location,range.length)]; }

// Bitmap representations
CFCharacterSetRef CharacterSetWithBitmapRepresentation( CFDataRef data )
{ return (CFCharacterSetRef)[NSCharacterSet characterSetWithBitmapRepresentation:(NSData *)data]; }

CFCharacterSetRef CharacterSetWithContentsOfURL( CFURLRef url )
{ return (CFCharacterSetRef)[NSCharacterSet characterSetWithContentsOfFile:[(NSURL *)url path]]; }

CFDataRef CharacterSetBitmapRepresentation( CFCharacterSetRef ref )
{ return (CFDataRef)[(NSCharacterSet *)ref bitmapRepresentation]; }

// Inverting
CFCharacterSetRef CharacterSetInvertedSet( CFCharacterSetRef set )
{ return (CFCharacterSetRef)[(NSCharacterSet *)set invertedSet]; }

// - testing membership -
Boolean CharacterSetCharacterIsMember( CFCharacterSetRef set, unichar chr )
{ return [(NSCharacterSet *)set characterIsMember:chr]; }

Boolean CharacterSetHasMemberInPlane( CFCharacterSetRef set, UInt8 plane )
{ return [(NSCharacterSet *)set hasMemberInPlane:plane]; }

Boolean CharacterSetIsSupersetOfSet( CFCharacterSetRef set, CFCharacterSetRef otherSet )
{ return [(NSCharacterSet *)set isSupersetOfSet:(NSCharacterSet *)otherSet]; }

Boolean CharacterSetLongCharacterIsMember( CFCharacterSetRef set, UInt32 longChar )
{ return [(NSCharacterSet *)set longCharacterIsMember:longChar]; }


/*
 MutabeCharacterSet
 */
// Adding and removing characters
void CharacterSetAddCharactersInRange( CFMutableCharacterSetRef set, CFRange range )
{ [(NSMutableCharacterSet *)set addCharactersInRange:NSRangeFromCFRange(range)]; }

void CharacterSetRemoveCharactersInRange( CFMutableCharacterSetRef set, CFRange range )
{ [(NSMutableCharacterSet *)set removeCharactersInRange:NSRangeFromCFRange(range)]; }

void CharacterSetAddCharactersInString( CFMutableCharacterSetRef set, CFStringRef string )
{ [(NSMutableCharacterSet *)set addCharactersInString:(NSString *)string]; }

void CharacterSetRemoveCharactersInString( CFMutableCharacterSetRef set, CFStringRef string )
{ [(NSMutableCharacterSet *)set removeCharactersInString:(NSString *)string]; }

// Combining sets
void CharacterSetFormIntersectionWithCharacterSet( CFMutableCharacterSetRef set, CFCharacterSetRef otherSet )
{ [(NSMutableCharacterSet *)set formIntersectionWithCharacterSet:(NSCharacterSet *)otherSet]; }

void CharacterSetFormUnionWithCharacterSet( CFMutableCharacterSetRef set, CFCharacterSetRef otherSet )
{ [(NSMutableCharacterSet *)set formUnionWithCharacterSet:(NSCharacterSet *)otherSet]; }

// Inverting
void CharacterSetInvert( CFMutableCharacterSetRef set )
{ [(NSMutableCharacterSet *)set invert]; }

