/*
 Color.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// UI element
// - Label

// - Text
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400
ColorRef ColorPlaceholderText( void );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
ColorRef ColorUnemphasizedSelectedText( void );
ColorRef ColorUnemphasizedSelectedTextBackground( void );
#endif

// - Content
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400
ColorRef ColorLink( void );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
ColorRef ColorSeparator( void );
ColorRef ColorSelectedContentBackground( void );
ColorRef ColorUnemphasizedSelectedContentBackground( void );
#endif

// Table
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
CFArrayRef ColorAlternatingContentBackgrounds( void );
#endif

// - Control
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
ColorRef ColorControlAccent( void );
#endif

// - Highlights and shadows
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
ColorRef ColorFindHighlight( void );
#endif


// Creation
// - Asset catalogs
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
ColorRef ColorNamed( CFStringRef name );
ColorRef ColorNamedBundle( CFStringRef name, BundleRef bundle );
#endif

ColorRef ColorWithCatalogName( CFStringRef catalogName, CFStringRef colorName );


// - Arbitrary color space
ColorRef ColorWithColorSpace( ColorSpaceRef cs, const CGFloat *components, NSInteger count );





// Appearance
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
ColorRef ColorWithSystemEffect( ColorRef col, NSColorSystemEffect effect );
#endif

// Transforming
ColorRef ColorUsingColorSpace( ColorRef col,  ColorSpaceRef cs );
ColorRef ColorBlendedWithFractionOfColor( ColorRef col1, CGFloat fraction, ColorRef col2 );

ColorRef ColorHighlightWithLevel( ColorRef col, CGFloat value );
ColorRef ColorShadowWithLevel( ColorRef col, CGFloat value );


// Alpha support
Boolean ColorIgnoresAlpha( void );

// Copy paste color info
ColorRef ColorFromPasteboard( CocoaPasteboardRef pb );
void ColorWriteToPastebord( ColorRef col, CocoaPasteboardRef pb );



// Color space
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
NSColorType ColorType( ColorRef col );
ColorRef ColorUsingType( ColorRef col, NSColorType type );
#endif

ColorSpaceRef ColorColorSpace( ColorRef col );


// Drawing
void ColorDrawSwatchInRect( ColorRef col, CGRect r );




// system
ColorRef ColorAlternateSelectedControl( void );
ColorRef ColorAlternateSelectedControlText( void );
ColorRef ColorForControlTint( NSControlTint tint );
ColorRef ColorControlBackground( void );
ColorRef ColorControl( void );
CFArrayRef ColorControlAlternatingRowBackground( void );
ColorRef ColorControlHighlight( void );
ColorRef ColorControlShadow( void );
ColorRef ColorControlDarkShadow( void );
ColorRef ColorControlText( void );
NSControlTint ColorCurrentControlTint( void );
ColorRef ColorDisabledControlText( void );
ColorRef ColorGrid( void );
ColorRef ColorHeader( void );
ColorRef ColorHeaderText( void );
ColorRef ColorHighlight( void );
ColorRef ColorKeyboardFocusIndicator( void );
ColorRef ColorKnob( void );
ColorRef ColorScrollBar( void );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
ColorRef ColorScrubberTexturedBackground( void );
#endif

ColorRef ColorSecondarySelectedControl( void );
ColorRef ColorSelectedControl( void );
ColorRef ColorSelectedControlText( void );
ColorRef ColorSelectedMenuItem( void );
ColorRef ColorSelectedMenuItemText( void );
ColorRef ColorSelectedTextBackground( void );
ColorRef ColorSelectedText( void );
ColorRef ColorSelectedKnob( void );
ColorRef ColorShadow( void );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
ColorRef ColorSystemBlue( void );
ColorRef ColorSystemBrown( void );
ColorRef ColorSystemGray( void );
ColorRef ColorSystemGreen( void );
ColorRef ColorSystemOrange( void );
ColorRef ColorSystemPink( void );
ColorRef ColorSystemPurple( void );
ColorRef ColorSystemRed( void );
ColorRef ColorSystemYellow( void );
#endif

ColorRef ColorTextBackground( void );
ColorRef ColorText( void );
ColorRef ColorWindowBackground( void );
ColorRef ColorWindowFrame( void );
ColorRef ColorWindowFrameText( void );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
ColorRef ColorUnderPageBackground( void );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
ColorRef ColorLabel( void );
ColorRef ColorSecondaryLabel( void );
ColorRef ColorTertiaryLabel( void );
ColorRef ColorQuaternaryLabel( void );
#endif

ColorRef ColorBlack( void );
ColorRef ColorBlue( void );
ColorRef ColorBrown( void );
ColorRef ColorClear( void );
ColorRef ColorCyan( void );
ColorRef ColorDarkGray( void );
ColorRef ColorGray( void );
ColorRef ColorGreen( void );
ColorRef ColorLightGray( void );
ColorRef ColorMagenta( void );
ColorRef ColorOrange( void );
ColorRef ColorPurple( void );
ColorRef ColorRed( void );
ColorRef ColorWhite( void );
ColorRef ColorYellow( void );

ColorRef ColorWithCalibratedHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha );
ColorRef ColorWithDeviceHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
ColorRef ColorWithHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha );
#endif

ColorRef ColorWithSRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
ColorRef ColorWithDisplayP3( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
ColorRef ColorWithRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
#endif

ColorRef ColorWithCalibratedRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
ColorRef ColorWithDeviceRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
ColorRef ColorWithWhite( CGFloat white, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
ColorRef ColorWithCalibratedWhite( CGFloat white, CGFloat alpha );
#endif

ColorRef ColorWithDeviceWhite( CGFloat white, CGFloat alpha );
ColorRef ColorWithGenericGamma22White( CGFloat white, CGFloat alpha );
ColorRef ColorWithDeviceCMYK( CGFloat cyan, CGFloat magenta, CGFloat yellow, CGFloat black, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
ColorRef ColorWithCGColor( CGColorRef col );
CGColorRef ColorCGColor( ColorRef col );
#endif

ColorRef ColorWithCIColor( CIColorRef col );

ColorRef ColorWithPatternImage( void *inImage );
ImageRef ColorPatternImage( ColorRef col );

ColorRef ColorWithAlphaComponent( ColorRef col, CGFloat alpha );

void ColorGetCMYK( ColorRef col, CGFloat *c, CGFloat *m, CGFloat *y, CGFloat *k, CGFloat *a );
void ColorGetHSB( ColorRef col, CGFloat *h, CGFloat *s, CGFloat *b, CGFloat *a );
void ColorGetRGB( ColorRef col, CGFloat *r, CGFloat *g, CGFloat *b, CGFloat *a );
void ColorGetWhite( ColorRef col, CGFloat *w, CGFloat *a );
NSInteger ColorNumberOfComponents( ColorRef col );
void ColorGetComponents( ColorRef col, CGFloat *components );

CGFloat ColorAlphaComponent( ColorRef col );
CGFloat ColorWhiteComponent( ColorRef col );
CGFloat ColorRedComponent( ColorRef col );
CGFloat ColorGreenComponent( ColorRef col );
CGFloat ColorBlueComponent( ColorRef col );
CGFloat ColorCyanComponent( ColorRef col );
CGFloat ColorMagentaComponent( ColorRef col );
CGFloat ColorYellowComponent( ColorRef col );
CGFloat ColorBlackComponent( ColorRef col );
CGFloat ColorHueComponent( ColorRef col );
CGFloat ColorSaturationComponent( ColorRef col );
CGFloat ColorBrightnessComponent( ColorRef col );
CFStringRef ColorCatalogNameComponent( ColorRef col );
CFStringRef ColorLocalizedCatalogNameComponent( ColorRef col );
CFStringRef ColorNameComponent( ColorRef col );
CFStringRef ColorLocalizedNameComponent( ColorRef col );

// working with colorspace
CFStringRef ColorColorSpaceName( ColorRef col );



void ColorSet( ColorRef col );
void ColorSetFill( ColorRef col );
void ColorSetStroke( ColorRef col );


// convenience
// set
void ColorSetBlack( void );
void ColorSetBlue( void );
void ColorSetBrown( void );
void ColorSetClear( void );
void ColorSetCyan( void );
void ColorSetDarkGray( void );
void ColorSetGray( void );
void ColorSetGreen( void );
void ColorSetLightGray( void );
void ColorSetMagenta( void );
void ColorSetOrange( void );
void ColorSetPurple( void );
void ColorSetRed( void );
void ColorSetWhite( void );
void ColorSetYellow( void );

void ColorSetWithCalibratedHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha );
void ColorSetWithDeviceHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetWithHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha );
#endif

void ColorSetWithSRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void ColorSetWithDisplayP3( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetWithRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
#endif

void ColorSetWithCalibratedRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
void ColorSetWithDeviceRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetWithWhite( CGFloat white, CGFloat alpha );
#endif

void ColorSetWithCalibratedWhite( CGFloat white, CGFloat alpha );
void ColorSetWithDeviceWhite( CGFloat white, CGFloat alpha );
void ColorSetWithGenericGamma22White( CGFloat white, CGFloat alpha );
void ColorSetWithDeviceCMYK( CGFloat cyan, CGFloat magenta, CGFloat yellow, CGFloat black, CGFloat alpha );


// setFill
void ColorSetFillBlack( void );
void ColorSetFillBlue( void );
void ColorSetFillBrown( void );
void ColorSetFillClear( void );
void ColorSetFillCyan( void );
void ColorSetFillDarkGray( void );
void ColorSetFillGray( void );
void ColorSetFillGreen( void );
void ColorSetFillLightGray( void );
void ColorSetFillMagenta( void );
void ColorSetFillOrange( void );
void ColorSetFillPurple( void );
void ColorSetFillRed( void );
void ColorSetFillWhite( void );
void ColorSetFillYellow( void );

void ColorSetFillWithCalibratedHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha );
void ColorSetFillWithDeviceHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetFillWithHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha );
#endif

void ColorSetFillWithSRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void ColorSetFillWithDisplayP3( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetFillWithRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
#endif

void ColorSetFillWithCalibratedRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
void ColorSetFillWithDeviceRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetFillWithWhite( CGFloat white, CGFloat alpha );
#endif

void ColorSetFillWithCalibratedWhite( CGFloat white, CGFloat alpha );
void ColorSetFillWithDeviceWhite( CGFloat white, CGFloat alpha );
void ColorSetFillWithGenericGamma22White( CGFloat white, CGFloat alpha );
void ColorSetFillWithDeviceCMYK( CGFloat cyan, CGFloat magenta, CGFloat yellow, CGFloat black, CGFloat alpha );


// setStroke
void ColorSetStrokeBlack( void );
void ColorSetStrokeBlue( void );
void ColorSetStrokeBrown( void );
void ColorSetStrokeClear( void );
void ColorSetStrokeCyan( void );
void ColorSetStrokeDarkGray( void );
void ColorSetStrokeGray( void );
void ColorSetStrokeGreen( void );
void ColorSetStrokeLightGray( void );
void ColorSetStrokeMagenta( void );
void ColorSetStrokeOrange( void );
void ColorSetStrokePurple( void );
void ColorSetStrokeRed( void );
void ColorSetStrokeWhite( void );
void ColorSetStrokeYellow( void );

void ColorSetStrokeWithCalibratedHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha );
void ColorSetStrokeWithDeviceHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetStrokeWithHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha );
#endif

void ColorSetStrokeWithSRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void ColorSetStrokeWithDisplayP3( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetStrokeWithRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
#endif

void ColorSetStrokeWithCalibratedRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );
void ColorSetStrokeWithDeviceRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetStrokeWithWhite( CGFloat white, CGFloat alpha );
#endif

void ColorSetStrokeWithCalibratedWhite( CGFloat white, CGFloat alpha );
void ColorSetStrokeWithDeviceWhite( CGFloat white, CGFloat alpha );
void ColorSetStrokeWithGenericGamma22White( CGFloat white, CGFloat alpha );
void ColorSetStrokeWithDeviceCMYK( CGFloat cyan, CGFloat magenta, CGFloat yellow, CGFloat black, CGFloat alpha );
