/*
 Color.m

 Bernie Wylde
 */

#import "Color.h"

// UI element
// - Label

// - Text
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400
ColorRef ColorPlaceholderText( void )
{ return [NSColor placeholderTextColor]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
ColorRef ColorUnemphasizedSelectedText( void )
{ return [NSColor unemphasizedSelectedTextColor]; }

ColorRef ColorUnemphasizedSelectedTextBackground( void )
{ return [NSColor unemphasizedSelectedTextBackgroundColor]; }
#endif

// - Content
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400
ColorRef ColorLink( void )
{ return [NSColor linkColor]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
ColorRef ColorSeparator( void )
{ return [NSColor separatorColor]; }

ColorRef ColorSelectedContentBackground( void )
{ return [NSColor selectedContentBackgroundColor]; }

ColorRef ColorUnemphasizedSelectedContentBackground( void )
{ return [NSColor unemphasizedSelectedContentBackgroundColor]; }
#endif

// Table
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
CFArrayRef ColorAlternatingContentBackgrounds( void )
{ return (CFArrayRef)[NSColor alternatingContentBackgroundColors]; }
#endif

// - Control
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
ColorRef ColorControlAccent( void )
{ return [NSColor controlAccentColor]; }
#endif

// - Highlights and shadows
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
ColorRef ColorFindHighlight( void )
{ return [NSColor findHighlightColor]; }
#endif



// Creation
// - Asset catalogs
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
ColorRef ColorNamed( CFStringRef name )
{ return [NSColor colorNamed:(NSString *)name]; }

ColorRef ColorNamedBundle( CFStringRef name, BundleRef bundle )
{ return [NSColor colorNamed:(NSString *)name bundle:bundle]; }
#endif

ColorRef ColorWithCatalogName( CFStringRef catalogName, CFStringRef colorName )
{ return [NSColor colorWithCatalogName:(NSString *)catalogName colorName:(NSString *)colorName]; }



// - Arbitrary color space
ColorRef ColorWithColorSpace( ColorSpaceRef cs, const CGFloat *components, NSInteger count )
{ return [NSColor colorWithColorSpace:cs components:components count:count]; }






// Appearance
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
ColorRef ColorWithSystemEffect( ColorRef col, NSColorSystemEffect effect )
{ return [col colorWithSystemEffect:effect]; }
#endif

// Transforming
ColorRef ColorUsingColorSpace( ColorRef col,  ColorSpaceRef cs )
{ return [col colorUsingColorSpace:cs]; }

ColorRef ColorBlendedWithFractionOfColor( ColorRef col1, CGFloat fraction, ColorRef col2 )
{ return [col1 blendedColorWithFraction:fraction ofColor:col2]; }

ColorRef ColorHighlightWithLevel( ColorRef col, CGFloat value )
{ return [col highlightWithLevel:value]; }

ColorRef ColorShadowWithLevel( ColorRef col, CGFloat value )
{ return [col shadowWithLevel:value]; }


// Alpha support
Boolean ColorIgnoresAlpha( void )
{ return [NSColor ignoresAlpha]; }


// Copy paste color info
ColorRef ColorFromPasteboard( CocoaPasteboardRef pb )
{ return [NSColor colorFromPasteboard:pb]; }

void ColorWriteToPastebord( ColorRef col, CocoaPasteboardRef pb )
{ [col writeToPasteboard:pb]; }



// Color space
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
NSColorType ColorType( ColorRef col )
{ return [col type]; }

ColorRef ColorUsingType( ColorRef col, NSColorType type )
{ return [col colorUsingType:type]; }
#endif

ColorSpaceRef ColorColorSpace( ColorRef col )
{ return [col colorSpace]; }


// Drawing
void ColorDrawSwatchInRect( ColorRef col, CGRect r )
{ return [col drawSwatchInRect:NSRectFromCGRect(r)]; }










// system
ColorRef ColorAlternateSelectedControl( void )
{ return [NSColor alternateSelectedControlColor]; }

ColorRef ColorAlternateSelectedControlText( void )
{ return [NSColor alternateSelectedControlTextColor]; }

ColorRef ColorForControlTint( NSControlTint tint )
{ return [NSColor colorForControlTint:tint]; }

ColorRef ColorControlBackground( void )
{ return [NSColor controlBackgroundColor]; }

ColorRef ColorControl( void )
{ return [NSColor controlColor]; }

CFArrayRef ColorControlAlternatingRowBackground( void )
{ return (CFArrayRef)[NSColor controlAlternatingRowBackgroundColors]; }

ColorRef ColorControlHighlight( void )
{ return [NSColor controlHighlightColor]; }

ColorRef ColorControlShadow( void )
{ return [NSColor controlShadowColor]; }

ColorRef ColorControlDarkShadow( void )
{ return [NSColor controlDarkShadowColor]; }

ColorRef ColorControlText( void )
{ return [NSColor controlTextColor]; }

NSControlTint ColorCurrentControlTint( void )
{ return [NSColor currentControlTint]; }

ColorRef ColorDisabledControlText( void )
{ return [NSColor disabledControlTextColor]; }

ColorRef ColorGrid( void )
{ return [NSColor gridColor]; }

ColorRef ColorHeader( void )
{ return [NSColor headerColor]; }

ColorRef ColorHeaderText( void )
{ return [NSColor headerTextColor]; }

ColorRef ColorHighlight( void )
{ return [NSColor highlightColor]; }

ColorRef ColorKeyboardFocusIndicator( void )
{ return [NSColor keyboardFocusIndicatorColor]; }

ColorRef ColorKnob( void )
{ return [NSColor knobColor]; }

ColorRef ColorScrollBar( void )
{ return [NSColor scrollBarColor]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
ColorRef ColorScrubberTexturedBackground( void )
{ return [NSColor scrubberTexturedBackgroundColor]; }
#endif

ColorRef ColorSecondarySelectedControl( void )
{ return [NSColor secondarySelectedControlColor]; }

ColorRef ColorSelectedControl( void )
{ return [NSColor selectedControlColor]; }

ColorRef ColorSelectedControlText( void )
{ return [NSColor selectedControlTextColor]; }

ColorRef ColorSelectedMenuItem( void )
{ return [NSColor selectedMenuItemColor]; }

ColorRef ColorSelectedMenuItemText( void )
{ return [NSColor selectedMenuItemTextColor]; }

ColorRef ColorSelectedTextBackground( void )
{ return [NSColor selectedTextBackgroundColor]; }

ColorRef ColorSelectedText( void )
{ return [NSColor selectedTextColor]; }

ColorRef ColorSelectedKnob( void )
{ return [NSColor selectedKnobColor]; }

ColorRef ColorShadow( void )
{ return [NSColor shadowColor]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
ColorRef ColorSystemBlue( void )
{ return [NSColor systemBlueColor]; }

ColorRef ColorSystemBrown( void )
{ return [NSColor systemBrownColor]; }

ColorRef ColorSystemGray( void )
{ return [NSColor systemGrayColor]; }

ColorRef ColorSystemGreen( void )
{ return [NSColor systemGreenColor]; }

ColorRef ColorSystemOrange( void )
{ return [NSColor systemOrangeColor]; }

ColorRef ColorSystemPink( void )
{ return [NSColor systemPinkColor]; }

ColorRef ColorSystemPurple( void )
{ return [NSColor systemPurpleColor]; }

ColorRef ColorSystemRed( void )
{ return [NSColor systemRedColor]; }

ColorRef ColorSystemYellow( void )
{ return [NSColor systemYellowColor]; }
#endif

ColorRef ColorTextBackground( void )
{ return [NSColor textBackgroundColor]; }

ColorRef ColorText( void )
{ return [NSColor textColor]; }

ColorRef ColorWindowBackground( void )
{ return [NSColor windowBackgroundColor]; }

ColorRef ColorWindowFrame( void )
{ return [NSColor windowFrameColor]; }

ColorRef ColorWindowFrameText( void )
{ return [NSColor windowFrameTextColor]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
ColorRef ColorUnderPageBackground( void )
{ return [NSColor underPageBackgroundColor]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
ColorRef ColorLabel( void )
{ return [NSColor labelColor]; }

ColorRef ColorSecondaryLabel( void )
{ return [NSColor secondaryLabelColor]; }

ColorRef ColorTertiaryLabel( void )
{ return [NSColor tertiaryLabelColor]; }

ColorRef ColorQuaternaryLabel( void )
{ return [NSColor quaternaryLabelColor]; }
#endif

ColorRef ColorBlack( void )
{ return [NSColor blackColor]; }

ColorRef ColorBlue( void )
{ return [NSColor blueColor]; }

ColorRef ColorBrown( void )
{ return [NSColor brownColor]; }

ColorRef ColorClear( void )
{ return [NSColor clearColor]; }

ColorRef ColorCyan( void )
{ return [NSColor cyanColor]; }

ColorRef ColorDarkGray( void )
{ return [NSColor darkGrayColor]; }

ColorRef ColorGray( void )
{ return [NSColor grayColor]; }

ColorRef ColorGreen( void )
{ return [NSColor greenColor]; }

ColorRef ColorLightGray( void )
{ return [NSColor lightGrayColor]; }

ColorRef ColorMagenta( void )
{ return [NSColor magentaColor]; }

ColorRef ColorOrange( void )
{ return [NSColor orangeColor]; }

ColorRef ColorPurple( void )
{ return [NSColor purpleColor]; }

ColorRef ColorRed( void )
{ return [NSColor redColor]; }

ColorRef ColorWhite( void )
{ return [NSColor whiteColor]; }

ColorRef ColorYellow( void )
{ return [NSColor yellowColor]; }

ColorRef ColorWithCalibratedHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha )
{ return [NSColor colorWithCalibratedHue:hue saturation:saturation brightness:brightness alpha:alpha]; }

ColorRef ColorWithDeviceHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha )
{ return [NSColor colorWithDeviceHue:hue saturation:saturation brightness:brightness alpha:alpha]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
ColorRef ColorWithHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha )
{ return [NSColor colorWithHue:hue saturation:saturation brightness:brightness alpha:alpha]; }
#endif

ColorRef ColorWithSRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ return [NSColor colorWithSRGBRed:red green:green blue:blue alpha:alpha]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
ColorRef ColorWithDisplayP3( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ return [NSColor colorWithDisplayP3Red:red green:green blue:blue alpha:alpha]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
ColorRef ColorWithRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ return [NSColor colorWithRed:red green:green blue:blue alpha:alpha]; }
#endif

ColorRef ColorWithCalibratedRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ return [NSColor colorWithCalibratedRed:red green:green blue:blue alpha:alpha]; }

ColorRef ColorWithDeviceRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ return [NSColor colorWithDeviceRed:red green:green blue:blue alpha:alpha]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
ColorRef ColorWithWhite( CGFloat white, CGFloat alpha )
{ return [NSColor colorWithWhite:white alpha:alpha]; }
#endif

ColorRef ColorWithCalibratedWhite( CGFloat white, CGFloat alpha )
{ return [NSColor colorWithCalibratedWhite:white alpha:alpha]; }

ColorRef ColorWithDeviceWhite( CGFloat white, CGFloat alpha )
{ return [NSColor colorWithDeviceWhite:white alpha:alpha]; }

ColorRef ColorWithGenericGamma22White( CGFloat white, CGFloat alpha )
{ return [NSColor colorWithGenericGamma22White:white alpha:alpha]; }

ColorRef ColorWithDeviceCMYK( CGFloat cyan, CGFloat magenta, CGFloat yellow, CGFloat black, CGFloat alpha )
{ return [NSColor colorWithDeviceCyan:cyan magenta:magenta yellow:yellow black:black alpha:alpha]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
ColorRef ColorWithCGColor( CGColorRef col )
{ return [NSColor colorWithCGColor:col]; }

CGColorRef ColorCGColor( ColorRef col )
{ return [col CGColor]; }
#endif

ColorRef ColorWithCIColor( CIColorRef col )
{ return [NSColor colorWithCIColor:col]; }

ColorRef ColorWithPatternImage( void *inImage )
{
    NSImage *image = FixImageValue( inImage );
    if ( image ) return [NSColor colorWithPatternImage:image];
    return nil;
}

ImageRef ColorPatternImage( ColorRef col )
{ return [col patternImage]; }


ColorRef ColorWithAlphaComponent( ColorRef col, CGFloat alpha )
{ return [col colorWithAlphaComponent:alpha]; }


void ColorGetCMYK( ColorRef col, CGFloat *c, CGFloat *m, CGFloat *y, CGFloat *k, CGFloat *a )
{ [col getCyan:c magenta:m yellow:y black:k alpha:a]; }

void ColorGetHSB( ColorRef col, CGFloat *h, CGFloat *s, CGFloat *b, CGFloat *a )
{ [col getHue:h saturation:s brightness:b alpha:a]; }

void ColorGetRGB( ColorRef col, CGFloat *r, CGFloat *g, CGFloat *b, CGFloat *a )
{ [col getRed:r green:g blue:b alpha:a]; }

void ColorGetWhite( ColorRef col, CGFloat *w, CGFloat *a )
{ [col getWhite:w alpha:a]; }

NSInteger ColorNumberOfComponents( ColorRef col )
{ return [col numberOfComponents]; }

void ColorGetComponents( ColorRef col, CGFloat *components )
{ [col getComponents:components]; }


CGFloat ColorAlphaComponent( ColorRef col )
{ return [col alphaComponent]; }

CGFloat ColorWhiteComponent( ColorRef col )
{ return [col whiteComponent]; }

CGFloat ColorRedComponent( ColorRef col )
{ return [col redComponent]; }

CGFloat ColorGreenComponent( ColorRef col )
{ return [col greenComponent]; }

CGFloat ColorBlueComponent( ColorRef col )
{ return [col blueComponent]; }

CGFloat ColorCyanComponent( ColorRef col )
{ return [col cyanComponent]; }

CGFloat ColorMagentaComponent( ColorRef col )
{ return [col magentaComponent]; }

CGFloat ColorYellowComponent( ColorRef col )
{ return [col yellowComponent]; }

CGFloat ColorBlackComponent( ColorRef col )
{ return [col blackComponent]; }

CGFloat ColorHueComponent( ColorRef col )
{ return [col hueComponent]; }

CGFloat ColorSaturationComponent( ColorRef col )
{ return [col saturationComponent]; }

CGFloat ColorBrightnessComponent( ColorRef col )
{ return [col brightnessComponent]; }

CFStringRef ColorCatalogNameComponent( ColorRef col )
{ return (CFStringRef)[col catalogNameComponent]; }

CFStringRef ColorLocalizedCatalogNameComponent( ColorRef col )
{ return (CFStringRef)[col localizedCatalogNameComponent]; }

CFStringRef ColorNameComponent( ColorRef col )
{ return (CFStringRef)[col colorNameComponent]; }

CFStringRef ColorLocalizedNameComponent( ColorRef col )
{ return (CFStringRef)[col localizedColorNameComponent]; }

// working with colorspace
CFStringRef ColorColorSpaceName( ColorRef col )
{ return (CFStringRef)[col colorSpaceName]; }




void ColorSet( ColorRef col )
{ [col set]; }

void ColorSetFill( ColorRef col )
{ [col setFill]; }

void ColorSetStroke( ColorRef col )
{ [col setStroke]; }


// convenience
// set
void ColorSetBlack( void )
{ [[NSColor blackColor] set]; }

void ColorSetBlue( void )
{ [[NSColor blueColor] set]; }

void ColorSetBrown( void )
{ [[NSColor brownColor] set]; }

void ColorSetClear( void )
{ [[NSColor clearColor] set]; }

void ColorSetCyan( void )
{ [[NSColor cyanColor] set]; }

void ColorSetDarkGray( void )
{ [[NSColor darkGrayColor] set]; }

void ColorSetGray( void )
{ [[NSColor grayColor] set]; }

void ColorSetGreen( void )
{ [[NSColor greenColor] set]; }

void ColorSetLightGray( void )
{ [[NSColor lightGrayColor] set]; }

void ColorSetMagenta( void )
{ [[NSColor magentaColor] set]; }

void ColorSetOrange( void )
{ [[NSColor orangeColor] set]; }

void ColorSetPurple( void )
{ [[NSColor purpleColor] set]; }

void ColorSetRed( void )
{ [[NSColor redColor] set]; }

void ColorSetWhite( void )
{ [[NSColor whiteColor] set]; }

void ColorSetYellow( void )
{ [[NSColor yellowColor] set]; }

void ColorSetWithCalibratedHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha )
{ [[NSColor colorWithCalibratedHue:hue saturation:saturation brightness:brightness alpha:alpha] set]; }

void ColorSetWithDeviceHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha )
{ [[NSColor colorWithDeviceHue:hue saturation:saturation brightness:brightness alpha:alpha] set]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetWithHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha )
{ [[NSColor colorWithHue:hue saturation:saturation brightness:brightness alpha:alpha] set]; }
#endif

void ColorSetWithSRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithSRGBRed:red green:green blue:blue alpha:alpha] set]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void ColorSetWithDisplayP3( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithDisplayP3Red:red green:green blue:blue alpha:alpha] set]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetWithRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithRed:red green:green blue:blue alpha:alpha] set]; }
#endif

void ColorSetWithCalibratedRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithCalibratedRed:red green:green blue:blue alpha:alpha] set]; }

void ColorSetWithDeviceRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithDeviceRed:red green:green blue:blue alpha:alpha] set]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetWithWhite( CGFloat white, CGFloat alpha )
{ [[NSColor colorWithWhite:white alpha:alpha] set]; }
#endif

void ColorSetWithCalibratedWhite( CGFloat white, CGFloat alpha )
{ [[NSColor colorWithCalibratedWhite:white alpha:alpha] set]; }

void ColorSetWithDeviceWhite( CGFloat white, CGFloat alpha )
{ [[NSColor colorWithDeviceWhite:white alpha:alpha] set]; }

void ColorSetWithGenericGamma22White( CGFloat white, CGFloat alpha )
{ [[NSColor colorWithGenericGamma22White:white alpha:alpha] set]; }

void ColorSetWithDeviceCMYK( CGFloat cyan, CGFloat magenta, CGFloat yellow, CGFloat black, CGFloat alpha )
{ [[NSColor colorWithDeviceCyan:cyan magenta:magenta yellow:yellow black:black alpha:alpha] set]; }


// setFill
void ColorSetFillBlack( void )
{ [[NSColor blackColor] setFill]; }

void ColorSetFillBlue( void )
{ [[NSColor blueColor] setFill]; }

void ColorSetFillBrown( void )
{ [[NSColor brownColor] setFill]; }

void ColorSetFillClear( void )
{ [[NSColor clearColor] setFill]; }

void ColorSetFillCyan( void )
{ [[NSColor cyanColor] setFill]; }

void ColorSetFillDarkGray( void )
{ [[NSColor darkGrayColor] setFill]; }

void ColorSetFillGray( void )
{ [[NSColor grayColor] setFill]; }

void ColorSetFillGreen( void )
{ [[NSColor greenColor] setFill]; }

void ColorSetFillLightGray( void )
{ [[NSColor lightGrayColor] setFill]; }

void ColorSetFillMagenta( void )
{ [[NSColor magentaColor] setFill]; }

void ColorSetFillOrange( void )
{ [[NSColor orangeColor] setFill]; }

void ColorSetFillPurple( void )
{ [[NSColor purpleColor] setFill]; }

void ColorSetFillRed( void )
{ [[NSColor redColor] setFill]; }

void ColorSetFillWhite( void )
{ [[NSColor whiteColor] setFill]; }

void ColorSetFillYellow( void )
{ [[NSColor yellowColor] setFill]; }

void ColorSetFillWithCalibratedHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha )
{ [[NSColor colorWithCalibratedHue:hue saturation:saturation brightness:brightness alpha:alpha] setFill]; }

void ColorSetFillWithDeviceHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha )
{ [[NSColor colorWithDeviceHue:hue saturation:saturation brightness:brightness alpha:alpha] setFill]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetFillWithHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha )
{ [[NSColor colorWithHue:hue saturation:saturation brightness:brightness alpha:alpha] setFill]; }
#endif

void ColorSetFillWithSRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithSRGBRed:red green:green blue:blue alpha:alpha] setFill]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void ColorSetFillWithDisplayP3( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithDisplayP3Red:red green:green blue:blue alpha:alpha] setFill]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetFillWithRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithRed:red green:green blue:blue alpha:alpha] setFill]; }
#endif

void ColorSetFillWithCalibratedRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithCalibratedRed:red green:green blue:blue alpha:alpha] setFill]; }

void ColorSetFillWithDeviceRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithDeviceRed:red green:green blue:blue alpha:alpha] setFill]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetFillWithWhite( CGFloat white, CGFloat alpha )
{ [[NSColor colorWithWhite:white alpha:alpha] setFill]; }
#endif

void ColorSetFillWithCalibratedWhite( CGFloat white, CGFloat alpha )
{ [[NSColor colorWithCalibratedWhite:white alpha:alpha] setFill]; }

void ColorSetFillWithDeviceWhite( CGFloat white, CGFloat alpha )
{ [[NSColor colorWithDeviceWhite:white alpha:alpha] setFill]; }

void ColorSetFillWithGenericGamma22White( CGFloat white, CGFloat alpha )
{ [[NSColor colorWithGenericGamma22White:white alpha:alpha] setFill]; }

void ColorSetFillWithDeviceCMYK( CGFloat cyan, CGFloat magenta, CGFloat yellow, CGFloat black, CGFloat alpha )
{ [[NSColor colorWithDeviceCyan:cyan magenta:magenta yellow:yellow black:black alpha:alpha] setFill]; }


// setStroke
void ColorSetStrokeBlack( void )
{ [[NSColor blackColor] setStroke]; }

void ColorSetStrokeBlue( void )
{ [[NSColor blueColor] setStroke]; }

void ColorSetStrokeBrown( void )
{ [[NSColor brownColor] setStroke]; }

void ColorSetStrokeClear( void )
{ [[NSColor clearColor] setStroke]; }

void ColorSetStrokeCyan( void )
{ [[NSColor cyanColor] setStroke]; }

void ColorSetStrokeDarkGray( void )
{ [[NSColor darkGrayColor] setStroke]; }

void ColorSetStrokeGray( void )
{ [[NSColor grayColor] setStroke]; }

void ColorSetStrokeGreen( void )
{ [[NSColor greenColor] setStroke]; }

void ColorSetStrokeLightGray( void )
{ [[NSColor lightGrayColor] setStroke]; }

void ColorSetStrokeMagenta( void )
{ [[NSColor magentaColor] setStroke]; }

void ColorSetStrokeOrange( void )
{ [[NSColor orangeColor] setStroke]; }

void ColorSetStrokePurple( void )
{ [[NSColor purpleColor] setStroke]; }

void ColorSetStrokeRed( void )
{ [[NSColor redColor] setStroke]; }

void ColorSetStrokeWhite( void )
{ [[NSColor whiteColor] setStroke]; }

void ColorSetStrokeYellow( void )
{ [[NSColor yellowColor] setStroke]; }

void ColorSetStrokeWithCalibratedHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha )
{ [[NSColor colorWithCalibratedHue:hue saturation:saturation brightness:brightness alpha:alpha] setStroke]; }

void ColorSetStrokeWithDeviceHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha )
{ [[NSColor colorWithDeviceHue:hue saturation:saturation brightness:brightness alpha:alpha] setStroke]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetStrokeWithHSB( CGFloat hue, CGFloat saturation, CGFloat brightness, CGFloat alpha )
{ [[NSColor colorWithHue:hue saturation:saturation brightness:brightness alpha:alpha] setStroke]; }
#endif

void ColorSetStrokeWithSRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithSRGBRed:red green:green blue:blue alpha:alpha] setStroke]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void ColorSetStrokeWithDisplayP3( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithDisplayP3Red:red green:green blue:blue alpha:alpha] setStroke]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetStrokeWithRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithRed:red green:green blue:blue alpha:alpha] setStroke]; }
#endif

void ColorSetStrokeWithCalibratedRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithCalibratedRed:red green:green blue:blue alpha:alpha] setStroke]; }

void ColorSetStrokeWithDeviceRGB( CGFloat red, CGFloat green, CGFloat blue, CGFloat alpha )
{ [[NSColor colorWithDeviceRed:red green:green blue:blue alpha:alpha] setStroke]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ColorSetStrokeWithWhite( CGFloat white, CGFloat alpha )
{ [[NSColor colorWithWhite:white alpha:alpha] setStroke]; }
#endif

void ColorSetStrokeWithCalibratedWhite( CGFloat white, CGFloat alpha )
{ [[NSColor colorWithCalibratedWhite:white alpha:alpha] setStroke]; }

void ColorSetStrokeWithDeviceWhite( CGFloat white, CGFloat alpha )
{ [[NSColor colorWithDeviceWhite:white alpha:alpha] setStroke]; }

void ColorSetStrokeWithGenericGamma22White( CGFloat white, CGFloat alpha )
{ [[NSColor colorWithGenericGamma22White:white alpha:alpha] setStroke]; }

void ColorSetStrokeWithDeviceCMYK( CGFloat cyan, CGFloat magenta, CGFloat yellow, CGFloat black, CGFloat alpha )
{ [[NSColor colorWithDeviceCyan:cyan magenta:magenta yellow:yellow black:black alpha:alpha] setStroke]; }

