/*
 ColorList.m

 Bernie Wylde
 */

#import "ColorList.h"

// Init
ColorListRef ColorListWithName( CFStringRef name )
{ return [[[NSColorList alloc] initWithName:(NSString *)name] autorelease]; }

ColorListRef ColorListFromFileAtURL( CFStringRef name, CFURLRef url )
{ return [[[NSColorList alloc] initWithName:(NSString *)name fromFile:[(NSURL *)url path]] autorelease]; }

// Lists
CFArrayRef ColorListAvailableColorLists( void )
{ return (CFArrayRef)[NSColorList availableColorLists]; }

ColorListRef ColorListNamed( CFStringRef name )
{ return [NSColorList colorListNamed:(NSString *)name]; }

// Properties
CFStringRef ColorListName( ColorListRef cl )
{ return (CFStringRef)[cl name]; }

Boolean ColorListIsEditable( ColorListRef cl )
{ return [cl isEditable]; }

// Managing colors
CFArrayRef ColorListAllKeys( ColorListRef cl )
{ return (CFArrayRef)[cl allKeys]; }

ColorRef ColorListColorWithKey( ColorListRef cl, CFStringRef key )
{ return [cl colorWithKey:(NSString *)key]; }

void ColorListInsertColor( ColorListRef cl, ColorRef col, CFStringRef key, NSUInteger index )
{ [cl insertColor:col key:(NSString *)key atIndex:index]; }

void ColorListRemoveColorWithKey( ColorListRef cl, CFStringRef key )
{ [cl removeColorWithKey:(NSString *)key]; }

void ColorListSetColor( ColorListRef cl, ColorRef col, CFStringRef key )
{ [cl setColor:col forKey:(NSString *)key]; }

// Write and remove ColorList files
void ColorListRemoveFile( ColorListRef cl )
{ [cl removeFile]; }

Boolean ColorListWriteToURL( ColorListRef cl, CFURLRef url, ErrorRef *err )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
    return [cl writeToURL:(NSURL *)url error:err];
#else
    return [cl writeToFile:[(NSURL *)url path]];
#endif
}


// Custom
ColorListRef ColorListByAddingColor( ColorListRef cl, ColorRef col, CFStringRef key )
{
    if ( [cl isEditable] ) {
        [cl setColor:col forKey:(NSString *)key];
        return cl;
    } else {
        NSColorList *cl2 = [[[NSColorList alloc] initWithName:[cl name]] autorelease];
        NSArray *keys = [cl allKeys];
        for ( NSString *key1 in keys ) {
            NSColor *col1 = [cl colorWithKey:key1];
            [cl2 setColor:col1 forKey:key1];
        }
        [cl2 setColor:col forKey:(NSString *)key];
        return cl2;
    }
}

ColorListRef ColorListByAddingColorList( ColorListRef cl1, ColorListRef cl2 )
{
    NSArray *keys2 = [cl2 allKeys];
    
    if ( [cl1 isEditable] ) {
        for ( NSString *key2 in keys2 ) {
            NSColor *col2 = [cl2 colorWithKey:key2];
            [cl1 setColor:col2 forKey:key2];
        }
        return cl1;
    } else {
        NSColorList *cl3 = [[[NSColorList alloc] initWithName:[cl1 name]] autorelease];
        NSArray *keys1 = [cl1 allKeys];
        for ( NSString *key1 in keys1 ) {
            NSColor *col1 = [cl1 colorWithKey:key1];
            [cl3 setColor:col1 forKey:key1];
        }
        for ( NSString *key2 in keys2 ) {
            NSColor *col2 = [cl2 colorWithKey:key2];
            [cl3 setColor:col2 forKey:key2];
        }
        return cl3;
    }
}

