/*
 ColorPanel.m

 Bernie Wylde
 */

#import "ColorPanel.h"

CocoaColorPanelTarget *sCocoaColorPanelTarget;

@implementation CocoaColorPanelTarget
@synthesize callback;
@synthesize userData;

- (id)initWithCallback:(CocoaColorPanelCallbackType)cb userData:(void *)ud {
    if ( self = [super init] ) {
        callback = cb;
        userData = ud;
    }
    return self;
}

- (void)action:(id)sender {
    if ( callback ) {
        (*callback)([sender color],userData);
    } else {
        NSColor *col = [[NSColorPanel sharedColorPanel] color];
        DialogEventSetColor( col );
        CallUserDefinedOnDialogFunction( colorPanelChangeColor, 0, 0, col );
    }
}
@end


void CocoaColorPanelSetTarget( void )
{
    if ( !sCocoaColorPanelTarget ) {
        sCocoaColorPanelTarget = [[CocoaColorPanelTarget alloc] init];
        NSColorPanel *cp = [NSColorPanel sharedColorPanel];
        [cp setTarget:sCocoaColorPanelTarget];
        [cp setAction:@selector(action:)];
    }
}

// - shared color panel -
ColorPanelRef ColorPanelShared( void )
{ return [NSColorPanel sharedColorPanel]; }

// - configure -
ViewRef ColorPanelAccessoryView( void )
{ return [[NSColorPanel sharedColorPanel] accessoryView]; }

void ColorPanelSetAccessoryView( ViewRef ref )
{ [[NSColorPanel sharedColorPanel] setAccessoryView:ref]; }

Boolean ColorPanelIsContinuous( void )
{ return [[NSColorPanel sharedColorPanel] isContinuous]; }

void ColorPanelSetContinuous( Boolean flag )
{
    [[NSColorPanel sharedColorPanel] setContinuous:flag];
    CocoaColorPanelSetTarget();
}

NSColorPanelMode ColorPanelMode( void )
{ return [[NSColorPanel sharedColorPanel] mode]; }

void ColorPanelSetMode( NSColorPanelMode mode )
{
    [[NSColorPanel sharedColorPanel] setMode:mode];
    CocoaColorPanelSetTarget();
}

void ColorPanelSetActionCallback( void *callback, void *userData )
{
    CocoaColorPanelTarget *target = [[[CocoaColorPanelTarget alloc] initWithCallback:callback userData:userData] autorelease];
    [[NSColorPanel sharedColorPanel] setTarget:target];
    [[NSColorPanel sharedColorPanel] setAction:@selector(action:)];
}

Boolean ColorPanelShowsAlpha( void )
{ return [[NSColorPanel sharedColorPanel] showsAlpha]; }

void ColorPanelSetShowsAlpha( Boolean flag )
{
    [[NSColorPanel sharedColorPanel] setShowsAlpha:flag];
    CocoaColorPanelSetTarget();
}

// Color lists
void ColorPanelAttachColorList( ColorListRef cl )
{ [[NSColorPanel sharedColorPanel] attachColorList:cl]; }

void ColorPanelDetachColorList( ColorListRef cl )
{ [[NSColorPanel sharedColorPanel] detachColorList:cl]; }

// - picker modes
void ColorPanelSetPickerMode( NSInteger mode )
{
    [NSColorPanel setPickerMode:mode];
    CocoaColorPanelSetTarget();
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void ColorPanelSetPickerMask( NSColorPanelOptions mask )
{
    [NSColorPanel setPickerMask:mask];
    CocoaColorPanelSetTarget();
}
#endif

// - setting color -
ColorRef ColorPanelColor( void )
{ return [[NSColorPanel sharedColorPanel] color]; }

void ColorPanelSetColor( ColorRef col )
{
    [[NSColorPanel sharedColorPanel] setColor:col];
    CocoaColorPanelSetTarget();
}

// - color info -
CGFloat ColorPanelAlpha( void )
{ return [[NSColorPanel sharedColorPanel] alpha]; }

// -- custom --
void ColorPanelShow( void )
{
    [[NSColorPanel sharedColorPanel] makeKeyAndOrderFront:nil];
    CocoaColorPanelSetTarget();
}

void ColorPanelClose( void )
{
    if ( [[NSColorPanel sharedColorPanel] isVisible] ) {
        [[NSColorPanel sharedColorPanel] orderOut:nil];
    }
}

