/*
 ColorWell.m

 Bernie Wylde
 */

#import "ColorWell.h"

#pragma mark - subclass
@implementation ColorWellSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end


NSColorWell *ColorWellWithTag( NSInteger tag )
{ return (NSColorWell *)CocoaValidateViewKindOfClass( tag, [NSColorWell class] ); }

BOOL ColorWellExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSColorWell class] ); }



ColorRef ColorWellColor( NSInteger tag )
{
    NSColorWell *well = ColorWellWithTag( tag );
    if ( well ) return [well color];
    return nil;
}

void ColorWellSetColor( NSInteger tag, ColorRef col )
{
    NSColorWell *well = ColorWellWithTag( tag );
    if ( well ) [well setColor:col];
}

void ColorWellTakeColorFrom( NSInteger tag, NSInteger senderTag )
{
    NSColorWell *well = ColorWellWithTag( tag );
    if ( well ) {
        id sender = CocoaViewWithTag( senderTag );
        if ( sender ) {
            NSColor *color = [sender color];
            [well setColor:color];
        }
    }
}

void ColorWellActivate( NSInteger tag, Boolean exclusive )
{
    NSColorWell *well = ColorWellWithTag( tag );
    if ( well ) [well activate:exclusive];
}

Boolean ColorWellIsActive( NSInteger tag )
{
    NSColorWell *well = ColorWellWithTag( tag );
    if ( well ) return [well isActive];
    return false;
}

void ColorWellDeactivate( NSInteger tag )
{
    NSColorWell *well = ColorWellWithTag( tag );
    if ( well ) [well deactivate];
}

void ColorWellDrawWellInside( NSInteger tag, CGRect insideRect )
{
    NSColorWell *well = ColorWellWithTag( tag );
    if ( well ) [well drawWellInside:NSRectFromCGRect(insideRect)];
}

Boolean ColorWellIsBordered( NSInteger tag )
{
    NSColorWell *well = ColorWellWithTag( tag );
    if ( well ) return [well isBordered];
    return false;
}

void ColorWellSetBordered( NSInteger tag, Boolean flag )
{
    NSColorWell *well = ColorWellWithTag( tag );
    if ( well ) [well setBordered:flag];
}

// Init
ColorWellRef ColorWellInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) {
        rect = NSMakeRect( 20, 20, 44, 23 );
    }
    
    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSColorWell *well;
    if ( subclass ) {
        well = [[[ColorWellSC alloc] initWithFrame:rect] autorelease];
    } else {
        well = [[[NSColorWell alloc] initWithFrame:rect] autorelease];
    }

    // tag
    [well setTag:inTag];
    
    // target/action
    [well setTarget:CocoaControlTargetObj()];
    [well setAction:@selector(controlAction:)];
    
    return well;
}

// statement
void CocoaColorWell( NSInteger inTag, NSInteger inEnabled, void *inColor, CGRect inRect, NSInteger inBordered, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        
        NSColorWell *well = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !well ) {
            well = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( well ) {   // exists
            
            // make sure it's same view class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [well class], [NSColorWell class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, well, [NSColorWell class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [well setEnabled:inEnabled];
            
            // color
            if ( inColor ) [well setColor:inColor];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [well setFrame:rect];
            
            // bordered
            if ( inBordered != kFBParamMissing ) [well setBordered:inBordered];
            
        } else {    // create
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) {
//                rect = NSMakeRect( 20, 20, 44, 23 );
//            }
            
            // init
//            if ( inSubclass ) {
//                well = [[ColorWellSC alloc] initWithFrame:rect];
//            } else {
//                well = [[NSColorWell alloc] initWithFrame:rect];
//            }
            
            well = ColorWellInit( inTag, inRect );
            
            // default autoresizing
            [well setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [well setTag: inTag];
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [well setEnabled:inEnabled];
            
            // color
            if ( inColor ) [well setColor:inColor];
            
            // bordered
            if ( inBordered != kFBParamMissing ) [well setBordered:inBordered];
            
            // wndTag
            [well setWndTag:inWndTag];
            
            // target/action
//            [well setTarget:CocoaControlTargetObj()];
//            [well setAction:@selector(controlAction:)];
            
            // add to window
            [contentView addSubview:well];
//            [well release];
        }
        
        [well setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

