/*
 ComboBox.m

 Bernie Wylde
 */

#import "ComboBox.h"

@implementation CocoaComboBoxDelegate

@synthesize comboBoxData;

#pragma mark - combobox delegate methods
- (void)controlTextDidBeginEditing:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldDidBeginEditing, notification.object, NULL );
}

- (void)controlTextDidChange:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldDidChange, notification.object, NULL );
}

- (void)controlTextDidEndEditing:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldDidEndEditing, notification.object, NULL );
}

- (void)comboBoxSelectionDidChange:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( comboBoxSelectionDidChange, notification.object, NULL );
}

- (void)comboBoxSelectionIsChanging:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( comboBoxSelectionIsChanging, notification.object, NULL );
}

- (void)comboBoxWillDismiss:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( comboBoxWillDismiss, notification.object, NULL );
}

- (void)comboBoxWillPopUp:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( comboBoxWillPopUp, notification.object, NULL );
}

#pragma mark - combobox data source methods
- (NSInteger)numberOfItemsInComboBox:(NSComboBox *)aComboBox {
    return [comboBoxData count];
}

- (id)comboBox:(NSComboBox *)aComboBox objectValueForItemAtIndex:(NSInteger)index {
    return [comboBoxData objectAtIndex:index];
}
@end

CocoaComboBoxDelegate *CocoaComboBoxDelegateGet( void )
{
    static CocoaComboBoxDelegate *sCocoaComboBoxDelegate = nil;
    if ( !sCocoaComboBoxDelegate ) sCocoaComboBoxDelegate = [[CocoaComboBoxDelegate alloc] init];
    return sCocoaComboBoxDelegate;
}


#pragma mark - subclass
@implementation ComboBoxSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(YES);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end


NSComboBox *ComboBoxWithTag( NSInteger tag )
{ return (NSComboBox *)CocoaValidateViewKindOfClass( tag, [NSComboBox class] ); }

BOOL ComboBoxExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSComboBox class] ); }


// - attributes
Boolean ComboBoxHasVerticalScroller( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo hasVerticalScroller];
    return NO;
}

void ComboBoxSetHasVerticalScroller( NSInteger tag, Boolean flag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo setHasVerticalScroller:flag];
}

CGSize ComboBoxIntercellSpacing( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo intercellSpacing];
    return CGSizeZero;
}

void ComboBoxSetIntercellSpacing( NSInteger tag, CGSize size )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo setIntercellSpacing:NSSizeFromCGSize(size)];
}

Boolean ComboBoxIsBordered( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo isButtonBordered];
    return NO;
}

void ComboBoxSetBordered( NSInteger tag, Boolean flag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo setButtonBordered:flag];
}

CGFloat ComboBoxItemHeight( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo itemHeight];
    return 0.0;
}

void ComboBoxSetItemHeight( NSInteger tag, CGFloat height )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo setItemHeight:height];
}

NSInteger ComboBoxNumberOfVisibleItems( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo numberOfVisibleItems];
    return 0;
}

void ComboBoxSetNumberOfVisibleItems( NSInteger tag, NSInteger value )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo setNumberOfVisibleItems:value];
}

void ComboBoxSetUsesDataSource( NSInteger tag, Boolean flag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) {
        [combo setUsesDataSource:flag];
        if ( flag ) {
            [combo setDataSource:CocoaComboBoxDelegateGet()];
        }
    }
}

void ComboBoxSetCompletes( NSInteger tag, Boolean flag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo setCompletes:flag];
}

void ComboBoxAddItems( NSInteger tag, CFArrayRef items )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo addItemsWithObjectValues:(NSArray *)items];
}

void ComboBoxAddItem( NSInteger tag, CFTypeRef item )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo addItemWithObjectValue:(id)item];
}

void ComboBoxInsertItem( NSInteger tag, CFTypeRef item, NSInteger index )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo insertItemWithObjectValue:(id)item atIndex:index];
}

CFArrayRef ComboBoxValues( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) return (CFArrayRef)[combo objectValues];
    return NULL;
}

void ComboBoxRemoveAllItems( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo removeAllItems];
}

void ComboBoxRemoveItemAtIndex( NSInteger tag, NSInteger index )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo removeItemAtIndex:index];
}

void ComboBoxRemoveItemWithValue( NSInteger tag, CFTypeRef value )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo removeItemWithObjectValue:(id)value];
}

NSInteger ComboBoxNumberOfItems( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) return [combo numberOfItems];
    return 0;
}

NSInteger ComboBoxIndexOfItemWithValue( NSInteger tag, CFTypeRef value )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) return [combo indexOfItemWithObjectValue:(id)value];
    return 0;
}

CFTypeRef ComboBoxItemValueAtIndex( NSInteger tag, NSInteger index )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) return (id)[combo itemObjectValueAtIndex:index];
    return NULL;
}

void ComboBoxNoteNumberOfItemsChanged( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo noteNumberOfItemsChanged];
}

void ComboBoxReloadData( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo reloadData];
}

void ComboBoxScrollItemAtIndexToTop( NSInteger tag, NSInteger index )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo scrollItemAtIndexToTop:index];
}

void ComboBoxScrollItemAtIndexToVisible( NSInteger tag, NSInteger index )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo scrollItemAtIndexToVisible:index];
}

void ComboBoxDeselectItemAtIndex( NSInteger tag, NSInteger index )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo deselectItemAtIndex:index];
}

NSInteger ComboBoxIndexOfSelectedItem( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) return [combo indexOfSelectedItem];
    return 0;
}

CFTypeRef ComboBoxValueOfSelectedItem( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) return (CFTypeRef)[combo objectValueOfSelectedItem];
    return NULL;
}

void ComboBoxSelectItemAtIndex( NSInteger tag, NSInteger index )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo selectItemAtIndex:index];
}
void ComboBoxSelectItemWithValue( NSInteger tag, CFTypeRef value )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [combo selectItemWithObjectValue:(id)value];
}

// custom
CFMutableArrayRef ComboBoxData( NSInteger tag )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) return (CFMutableArrayRef)[CocoaComboBoxDelegateGet() comboBoxData];
    return nil;
}

void ComboBoxSetData( NSInteger tag, CFMutableArrayRef array )
{
    NSComboBox *combo = ComboBoxWithTag( tag );
    if ( combo ) [CocoaComboBoxDelegateGet() setComboBoxData:(NSMutableArray *)array];
}

// Init
ComboBoxRef ComboBoxInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 16, 99, 26 );

    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSComboBox *combo;
    if ( subclass ) {
        combo = [[[ComboBoxSC alloc] initWithFrame:rect] autorelease];
    } else {
        combo = [[[NSComboBox alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [combo setTag:inTag];
    
    // fix font to match I.B.
    CocoaControlFixFont( combo );
    
    // delegate
    [combo setDelegate:CocoaComboBoxDelegateGet()];
    
    return combo;
}

// statement
void CocoaComboBox( NSInteger inTag, NSInteger inEnabled, CFTypeRef inItems, CFStringRef inText, CGRect inRect, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSString *text = (NSString *)inText;
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSComboBox *combo = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !combo ) {
            combo = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( combo ) {    // already exists
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [combo class], [NSComboBox class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, combo, [NSComboBox class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [combo setEnabled:inEnabled];
            
            // items
            if ( inItems ) {
                NSArray *items = nil;
                
                if ( CFGetTypeID(inItems) == CFStringGetTypeID() ) {
                    items = [(NSString *)inItems componentsSeparatedByString:@";"];
                    
                } else if ( CFGetTypeID(inItems) == CFArrayGetTypeID() ) {
                    items = (NSArray *)inItems;
                }
                
                if ( items ) {
                    [combo removeAllItems];
                    [combo addItemsWithObjectValues:items];
                }
            }
            
            // text
            if ( text ) [combo setStringValue:text];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [combo setFrame:rect];
            
        } else {        // create
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 16, 99, 26 );
            
            // init
//            if ( inSubclass ) {
//                combo = [[ComboBoxSC alloc] initWithFrame:rect];
//            } else {
//                combo = [[NSComboBox alloc] initWithFrame:rect];
//            }
            
            combo = ComboBoxInit( inTag, inRect );
            
            // default autoresizing
            [combo setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [combo setTag:inTag];
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [combo setEnabled:inEnabled];
            
            // items
            if ( inItems ) {
                NSArray *items = nil;
                
                if ( CFGetTypeID(inItems) == CFStringGetTypeID() ) {
                    items = [(NSString *)inItems componentsSeparatedByString:@";"];
                    
                } else if ( CFGetTypeID(inItems) == CFArrayGetTypeID() ) {
                    items = (NSArray *)inItems;
                }
                
                if ( items ) {
                    [combo addItemsWithObjectValues:items];
                }
            }
            
            // text
            if ( text ) [combo setStringValue:text];
            
            // wndTag
            [combo setWndTag:inWndTag];
            
            // fix font to match I.B.
//            CocoaControlFixFont( combo );
            
            // delegate
//            [combo setDelegate:CocoaComboBoxDelegateGet()];
            
            // add to window
            [contentView addSubview:combo];
//            [combo release];
        }
        
        [combo setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}


