/*
 Compiler.m

 Bernie Wylde
 */

#import "Compiler.h"


BOOL CompilerIsRunning_priv( void )
{
    NSArray *runningApps = [[NSWorkspace sharedWorkspace] runningApplications];
    for ( NSRunningApplication *app in runningApps ) {
        if ( [[app bundleIdentifier] isEqualToString:@"com.FBtoC"] ) {
            return YES;
        }
    }
    return NO;
}

void CompilerPostNotification_priv( CFStringRef name, CFStringRef object, CFDictionaryRef userInfo )
{
    if ( CompilerIsRunning_priv() ) {
        CFNotificationCenterPostNotification( CFNotificationCenterGetDistributedCenter(), name, object, userInfo, true );
    } else {
        NSMutableArray *array = [NSMutableArray arrayWithCapacity:0];
        [array addObject:(NSString *)object];
        NSString *filePath = [(NSDictionary *)userInfo objectForKey:@"FilePath"];
        if ( filePath ) [array addObject:filePath];
        NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:array,NSWorkspaceLaunchConfigurationArguments,nil];
        NSURL *fbtocURL = [[NSWorkspace sharedWorkspace] URLForApplicationWithBundleIdentifier:@"com.FBtoC"];
        [[NSWorkspace sharedWorkspace] launchApplicationAtURL:fbtocURL options:NSWorkspaceLaunchDefault configuration:dict error:nil];
    }
}

void CompilerProcessFileAtURL( CFURLRef url, CFStringRef action )
{
    NSDictionary *userInfo = [NSDictionary dictionaryWithObject:[(NSURL *)url path] forKey:@"FilePath"];
    CompilerPostNotification_priv( (CFStringRef)@"EditorNotification", action, (CFDictionaryRef)userInfo );
}

void CompilerRunFileAtURL( CFURLRef url )
{ CompilerProcessFileAtURL( url, (CFStringRef)@"Run" ); }

void CompilerAnalyzeFileAtURL( CFURLRef url )
{ CompilerProcessFileAtURL( url, (CFStringRef)@"Analyze" ); }

void CompilerBuildFileAtURL( CFURLRef url )
{ CompilerProcessFileAtURL( url, (CFStringRef)@"Build" ); }

void CompilerAbort( void )
{ CompilerPostNotification_priv( (CFStringRef)@"EditorNotification", (CFStringRef)@"Abort", NULL ); }

void CompilerShowSettings( void )
{ CompilerPostNotification_priv( (CFStringRef)@"EditorNotification", (CFStringRef)@"ShowSettings", NULL ); }

void CompilerTerminate( void )
{
    for ( NSRunningApplication *app in [[NSWorkspace sharedWorkspace] runningApplications] ) {
        if ( [[app bundleIdentifier] isEqualToString:@"com.FBtoC"] ) {
            [app terminate];
        }
    }
}

CFURLRef CompilerHeadersURL( void )
{
    NSURL *fbtocURL = [[NSWorkspace sharedWorkspace] URLForApplicationWithBundleIdentifier:@"com.FBtoC"];
    return (CFURLRef)[[fbtocURL URLByDeletingLastPathComponent] URLByAppendingPathComponent:@"Headers"];
}

