/*
 Control.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

#pragma mark - control formatter
///////////////////////////////////////////////////
//          cocoa control formatter              //
///////////////////////////////////////////////////
// format case options
enum {
    formatCaseSensitive = 1,
    formatLowercase,
    formatUppercase,
    formatCapitalize
};


@interface CocoaControlFormatter : NSFormatter {
    NSCharacterSet *set;
    BOOL pass;
    NSUInteger length;
    NSUInteger option;
}

@property (assign) NSCharacterSet *set;
@property (assign) BOOL pass;
@property (assign) NSUInteger length;
@property (assign) NSUInteger option;

- (id)initWithCharacters:(NSString *)inCharacters pass:(BOOL)inPass length:(NSUInteger)inLength caseOption:(NSUInteger)caseOption;
@end





#pragma mark - control
///////////////////////////////////////////////////
//                  cocoa control                //
///////////////////////////////////////////////////
// dialog events
enum {
    controlTextDidBeginEditing = 1200,
    controlTextDidChange,
    controlTextDidEndEditing,
    controlTextShouldBeginEditing,
    controlTextShouldEndEditing,
    controlDoCommandBySelector
};

typedef void (*CocoaControlCallbackType)(NSInteger,NSInteger,void *);

@interface NSControl (Additions)
@property (nonatomic,retain) NSValue *callbackValue;
@property (nonatomic,retain) NSValue *userDataValue;

- (CocoaControlCallbackType)cocoaControlCallback;
- (void *)cocoaControlUserData;
- (void)setCocoaControlCallback:(CocoaControlCallbackType)cccb;
- (void)setCocoaControlUserData:(void *)userData;
@end


@interface CocoaControlTarget : NSObject {
    CocoaControlCallbackType allControlsCallback;
}
@property (assign) CocoaControlCallbackType allControlsCallback;
@end

CocoaControlTarget *CocoaControlTargetObj( void );

void CocoaControlCallUserDefinedOnDialogOrCallback( NSInteger ev, id sender, id obj  );

// private
// fix font to match I.B.
void CocoaControlFixFont( id c );


// public
NSControl *ControlWithTag( NSInteger tag );
BOOL ControlExists( NSInteger tag );

double ControlDoubleValue( NSInteger tag );
void ControlSetDoubleValue( NSInteger tag, double value );
NSInteger ControlIntegerValue( NSInteger tag );
void ControlSetIntegerValue( NSInteger tag, NSInteger value );
CFStringRef ControlStringValue( NSInteger tag );
void ControlSetStringValue( NSInteger tag, CFStringRef string );
void ControlSetSize( NSInteger tag, NSControlSize size );
void ControlSetAlignment( NSInteger tag, NSTextAlignment alignment );
void ControlSizeToFit( NSInteger tag );

NSLineBreakMode ControlLineBreakMode( NSInteger tag );
void ControlSetLineBreakMode( NSInteger tag, NSLineBreakMode mode );
Boolean ControlUsesSingleLineMode( NSInteger tag );
void ControlSetUsesSingleLineMode( NSInteger tag, Boolean flag );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void ControlDrawWithExpansionFrameInView( NSInteger ctrlTag, CGRect contentFrame, NSInteger vwTag );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
Boolean ControlAllowsExpansionToolTips( NSInteger tag );
void ControlSetAllowsExpansionToolTips( NSInteger tag, Boolean flag );
#endif

CTNSFontRef ControlFont( NSInteger tag );
void ControlSetFont( NSInteger tag, CTNSFontRef inFont );
void ControlSetAttributedStringValue( NSInteger tag, CFAttributedStringRef string );
void ControlTakeDoubleValueFrom( NSInteger tag, NSInteger otherCtrlTag );
void ControlTakeIntegerValueFrom( NSInteger tag, NSInteger otherCtrlTag );
void ControlTakeObjectValueFrom( NSInteger tag, NSInteger otherCtrlTag );
void ControlTakeStringValueFrom( NSInteger tag, NSInteger otherCtrlTag );

Boolean ControlIsEnabled( NSInteger tag );
void ControlSetEnabled( NSInteger tag, Boolean flag );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGSize ControlSizeThatFits( NSInteger tag, CGSize size );
#endif

void ControlSetContinuous( NSInteger tag, Boolean flag );
void ControlPerformClick( NSInteger tag );

void ControlSendActionOn( NSInteger tag, NSEventMask mask );

NSInteger ControlTag( CocoaControlRef ref );
void ControlSetTag( CocoaControlRef ref, NSInteger tag );

Boolean ControlRefusesFirstResponder( NSInteger tag );
void ControlSetRefusesFirstResponder( NSInteger tag, Boolean flag );

Boolean ControlIgnoresMultiClick( NSInteger tag );
void ControlSetIgnoresMultiClick( NSInteger tag, Boolean flag );

// custom
void ControlSetFontWithName( NSInteger tag, CFStringRef inName, CGFloat size );
void ControlSetTargetAction( NSInteger ctrlTag, NSInteger targetTag, CFStringRef actionName );
void ControlSetAction( NSInteger tag, CFStringRef actionName );
void ControlSetActionCallback( NSInteger tag, void *callback, void *userData );
void ControlSetFormat( NSInteger tag, CFStringRef characters, Boolean pass, NSUInteger length, NSUInteger caseOption );

