/*
 Control.m

 Bernie Wylde
 */

#import "Control.h"

#pragma mark - control formatter
///////////////////////////////////////////////////
//        cocoa control formatter              //
///////////////////////////////////////////////////
@implementation CocoaControlFormatter

@synthesize set, pass, length, option;

- (id)initWithCharacters:(NSString *)inCharacters pass:(BOOL)inPass length:(NSUInteger)inLength caseOption:(NSUInteger)caseOption {
    if ( self = [super init] ) {
        
        if ( inCharacters == NULL ) inCharacters = @"";
        NSMutableString *string = [inCharacters mutableCopy];
        
        if ( caseOption != formatCaseSensitive ) {
            [string appendString:[string uppercaseString]];
            [string appendString:[string lowercaseString]];
        }
        
        if ( inPass ) {
            self.set = [[[NSMutableCharacterSet characterSetWithCharactersInString:string] invertedSet] retain];
        } else {
            self.set = [[NSMutableCharacterSet characterSetWithCharactersInString:string] retain];
        }
        
        self.pass = inPass;
        self.length = inLength;
        self.option = caseOption;
        
        [string release];
    }
    return self;
}

- (NSString *)stringForObjectValue:(id)obj {
    if ( [obj isKindOfClass:[NSString class]] ) return [NSString stringWithString:obj];
    return nil;
}

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)error {
    if ( string != nil ) {
        *obj = [NSString stringWithString:string];
        return YES;
    }
    return NO;
}

- (BOOL)isPartialStringValid:(NSString **)partialStringPtr proposedSelectedRange:(NSRangePointer)proposedSelRangePtr originalString:(NSString *)origString originalSelectedRange:(NSRange)origSelRange errorDescription:(NSString **)error {
    
    if ( length ) {
        if ([*partialStringPtr length] > length) {
            return NO;
        }
    }
    
    NSString *resultString = [[*partialStringPtr componentsSeparatedByCharactersInSet:set] componentsJoinedByString:@""];
    
    switch ( option ) {
        case formatLowercase:
            if (![*partialStringPtr isEqual:[resultString lowercaseString]]) {
                *partialStringPtr = [resultString lowercaseString];
                return NO;
            }
            break;
        case formatUppercase:
            if (![*partialStringPtr isEqual:[resultString uppercaseString]]) {
                *partialStringPtr = [resultString uppercaseString];
                return NO;
            }
            break;
        case formatCapitalize:
            if (![*partialStringPtr isEqual:[resultString capitalizedString]]) {
                *partialStringPtr = [resultString capitalizedString];
                return NO;
            }
            
            break;
        case formatCaseSensitive:
        default:
            //*partialStringPtr = resultString;
            if ( ![resultString isEqualToString:*partialStringPtr] ) {
                *partialStringPtr = resultString;
                return NO;
            }
            break;
    }
    
    //    if ( ![resultString isEqualToString:*partialStringPtr] ) {
    //            *partialStringPtr = resultString;
    //       return NO;
    //    }
    
    return YES;
}
@end




#pragma mark - control
///////////////////////////////////////////////////
//                  cocoa control                //
///////////////////////////////////////////////////
NSControl *ControlWithTag( NSInteger tag )
{ return (NSControl *)CocoaValidateViewKindOfClass( tag, [NSControl class] ); }

BOOL ControlExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSControl class] ); }


@implementation NSControl (Additions)
@dynamic callbackValue;
@dynamic userDataValue;

- (CocoaControlCallbackType)cocoaControlCallback {
    return [objc_getAssociatedObject(self, @selector(callbackValue)) pointerValue];
}

- (void *)cocoaControlUserData {
    return [objc_getAssociatedObject(self, @selector(userDataValue)) pointerValue];
}

- (void)setCocoaControlCallback:(CocoaControlCallbackType)callback {
    objc_setAssociatedObject(self, @selector(callbackValue), [NSValue valueWithPointer:callback], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (void)setCocoaControlUserData:(void *)userData {
    objc_setAssociatedObject(self, @selector(userDataValue), [NSValue valueWithPointer:userData], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end


@implementation CocoaControlTarget

@synthesize allControlsCallback;

- (void)controlAction:(id)sender {
    
    NSInteger wndTag = [sender wndTag];// 20190609 bw
    if ( wndTag < 1 ) {
        wndTag = [[sender window] tag];
    }

    CocoaOutputWindowSetTag( wndTag );
    CocoaControlCallbackType callback = [(NSControl *)sender cocoaControlCallback];

    if ( callback ) {
        void *userData = [(NSControl *)sender cocoaControlUserData];
        (*callback)([sender tag],wndTag,userData);
    } else if ( allControlsCallback ) {
        (*allControlsCallback)([sender tag],wndTag,NULL);
    } else {
        CallUserDefinedOnDialogFunction( 1, [sender tag], wndTag, NULL );// 1 == kFBBtnClick
    }
}
@end

// private
CocoaControlTarget *CocoaControlTargetObj( void )
{
    static CocoaControlTarget *sCocoaControlTarget = nil;
    if ( !sCocoaControlTarget ) sCocoaControlTarget = [[CocoaControlTarget alloc] init];
    return sCocoaControlTarget;
}

void CocoaControlCallUserDefinedOnDialogOrCallback( NSInteger ev, id sender, id obj  )
{
    CocoaEventSetEvent( [NSApp currentEvent] );
    CocoaOutputWindowSetTag( [sender wndTag] );
    CocoaControlTarget *target = CocoaControlTargetObj();
    CocoaControlCallbackType allControlsCallback = target.allControlsCallback;
    CocoaControlCallbackType callback = [(NSControl *)sender cocoaControlCallback];
    
    NSInteger wndTag = [sender wndTag];// 20190609 bw
    if ( wndTag < 1 ) {
        wndTag = [[sender window] tag];
    }
    
    if ( callback ) {
        void *userData = [(NSControl *)sender cocoaControlUserData];
        (*callback)([sender tag],wndTag,userData);
    } else if ( allControlsCallback ) {
        (*allControlsCallback)([sender tag],wndTag,obj);
    } else {
        CallUserDefinedOnDialogFunction( ev, [sender tag], wndTag, obj );
    }
}


// private
// fix font to match IB
void CocoaControlFixFont( id c )
{ [c setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:[[c cell] controlSize]]]]; }

// public
double ControlDoubleValue( NSInteger tag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) return [c doubleValue];
    return 0.0;
}

void ControlSetDoubleValue( NSInteger tag, double value )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c setDoubleValue:value];
}

NSInteger ControlIntegerValue( NSInteger tag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) return [c integerValue];
    return 0;
}

void ControlSetIntegerValue( NSInteger tag, NSInteger value )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c setIntegerValue:value];
}

CFStringRef ControlStringValue( NSInteger tag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) return (CFStringRef)[c stringValue];
    return NULL;
}

void ControlSetStringValue( NSInteger tag, CFStringRef string )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c setStringValue:(NSString *)string];
}

void ControlSetSize( NSInteger tag, NSControlSize size )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) {
        [[c cell] setControlSize:size];
        [c setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:size]]];
    }
}

void ControlSetAlignment( NSInteger tag, NSTextAlignment alignment )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c setAlignment:alignment];
}


void ControlSizeToFit( NSInteger tag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c sizeToFit];
}

NSLineBreakMode ControlLineBreakMode( NSInteger tag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) return [[c cell] lineBreakMode];
    return 0;
}

void ControlSetLineBreakMode( NSInteger tag, NSLineBreakMode mode )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [[c cell] setLineBreakMode:mode];
}

Boolean ControlUsesSingleLineMode( NSInteger tag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) return [[c cell] usesSingleLineMode];
    return NO;
}

void ControlSetUsesSingleLineMode( NSInteger tag, Boolean flag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [[c cell] setUsesSingleLineMode:flag];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void ControlDrawWithExpansionFrameInView( NSInteger ctrlTag, CGRect contentFrame, NSInteger vwTag )
{
    NSControl *c = (NSControl *)ControlWithTag( ctrlTag );
    if ( c ) {
        NSView *view = ViewWithTag( vwTag );
        if ( view ) [c drawWithExpansionFrame:NSRectFromCGRect(contentFrame) inView:view];
    }
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
Boolean ControlAllowsExpansionToolTips( NSInteger tag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) return [c allowsExpansionToolTips];
    return NO;
}

void ControlSetAllowsExpansionToolTips( NSInteger tag, Boolean flag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c setAllowsExpansionToolTips:flag];
}
#endif

CTNSFontRef ControlFont( NSInteger tag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) return [c font];
    return nil;
}

void ControlSetFont( NSInteger tag, CTNSFontRef inFont )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) {
        NSFont *font = FixFont( (void *)inFont );
        [c setFont:font];
    }
}

void ControlSetAttributedStringValue( NSInteger tag, CFAttributedStringRef string )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c setAttributedStringValue:(NSAttributedString *)string];
}

void ControlTakeDoubleValueFrom( NSInteger tag, NSInteger otherCtrlTag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) {
        id otherCtrl = (id)CocoaViewWithTag( otherCtrlTag );
        if ( otherCtrl ) [c takeDoubleValueFrom:otherCtrl];
    }
}

void ControlTakeIntegerValueFrom( NSInteger tag, NSInteger otherCtrlTag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) {
        id otherCtrl = (id)CocoaViewWithTag( otherCtrlTag );
        if ( otherCtrl ) [c takeIntegerValueFrom:otherCtrl];
    }
}

void ControlTakeObjectValueFrom( NSInteger tag, NSInteger otherCtrlTag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) {
        id otherCtrl = (id)CocoaViewWithTag( otherCtrlTag );
        if ( otherCtrl ) [c takeIntegerValueFrom:otherCtrl];
    }
}

void ControlTakeStringValueFrom( NSInteger tag, NSInteger otherCtrlTag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) {
        id otherCtrl = (id)CocoaViewWithTag( otherCtrlTag );
        if ( otherCtrl ) [c takeStringValueFrom:otherCtrl];
    }
}

void ControlSetEnabled( NSInteger tag, Boolean flag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c setEnabled:flag];
}

Boolean ControlIsEnabled( NSInteger tag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) return [c isEnabled];
    return false;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGSize ControlSizeThatFits( NSInteger tag, CGSize size )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) return NSSizeToCGSize([c sizeThatFits:NSSizeFromCGSize(size)]);
    return CGSizeZero;
}
#endif

void ControlSetContinuous( NSInteger tag, Boolean flag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c setContinuous:flag];
}

void ControlPerformClick( NSInteger tag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c performClick:nil];
}

void ControlSendActionOn( NSInteger tag, NSEventMask mask )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c sendActionOn:mask];
}

NSInteger ControlTag( CocoaControlRef ref )
{ return [ref tag]; }

void ControlSetTag( CocoaControlRef ref, NSInteger tag )
{ [ref setTag:tag]; }

Boolean ControlRefusesFirstResponder( NSInteger tag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) return [c refusesFirstResponder];
    return NO;
}

void ControlSetRefusesFirstResponder( NSInteger tag, Boolean flag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c setRefusesFirstResponder:flag];
}

Boolean ControlIgnoresMultiClick( NSInteger tag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) return [c ignoresMultiClick];
    return NO;
}

void ControlSetIgnoresMultiClick( NSInteger tag, Boolean flag )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) [c setIgnoresMultiClick:flag];
}

// custom
void ControlSetFontWithName( NSInteger tag, CFStringRef inName, CGFloat size )
{
    BOOL usePrevFont = NO;
    NSString *name = (NSString *)inName;
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) {
        if ( name ) {
            usePrevFont = ( [name length] == 0 );
        } else {
            usePrevFont = YES;
        }
        if ( usePrevFont ) name = [[c font] fontName];
        if ( (int)size == 0 ) size = [[c font] pointSize];
        [c setFont:[NSFont fontWithName:name size:size]];// hm, this clips descenders
        
        //        CTFontRef font = CTFontCreateWithName( (CFStringRef)name, size, NULL );
        //        [c setFont:(NSFont *)font];
        //        CFRelease(font);
    }
}

void ControlSetTargetAction( NSInteger ctrlTag, NSInteger targetTag, CFStringRef actionName )
{
    NSControl *ctrl = (NSControl *)ControlWithTag( ctrlTag );
    if ( ctrl ) {
        id target = nil;
        if ( targetTag ) target = ViewWithTag( targetTag );
        [ctrl setTarget:target];
        [ctrl setAction:NSSelectorFromString((NSString *)actionName)];
    }
}

void ControlSetAction( NSInteger tag, CFStringRef actionName )
{
    NSControl *c = (NSControl *)ControlWithTag( tag );
    if ( c ) {
        [c setTarget:nil];
        [c setAction:NSSelectorFromString((NSString *)actionName)];
    }
}

void ControlSetActionCallback( NSInteger tag, void *callback, void *userData )
{
    if ( tag == 0 ) {   // all controls
        CocoaControlTarget *target = CocoaControlTargetObj();
        [target setAllControlsCallback:callback];
    } else {            // individual control
        NSControl *c = (NSControl *)ControlWithTag( tag );
        if ( c ) {
            [c setCocoaControlCallback:callback];
            [c setCocoaControlUserData:userData];
        }
    }
}

void ControlSetFormat( NSInteger tag, CFStringRef characters, Boolean pass, NSUInteger length, NSUInteger caseOption )
{
    NSControl *c = ControlWithTag( tag );
    if ( c ) {
        CocoaControlFormatter *formatter = [[CocoaControlFormatter alloc] initWithCharacters:(NSString *)characters pass:pass length:length caseOption:caseOption];
        [c setFormatter:formatter];
        [formatter release];
    }
}

