/*
 Cursor.m

 Bernie Wylde
 */

#import "Cursor.h"

CocoaUICursorAnimator *sCocoaUICursorAnimator;

@implementation CocoaUICursorAnimator

@synthesize timer, counter, name, extension, size, hotSpot, numImages;

- (void)timerFireMethod:(NSTimer *)t {
    counter++;
    if ( counter > numImages ) counter = 1;
    NSString *string = [NSString stringWithFormat:@"%@%ld.%@",name,counter,extension];
    NSImage *image = ImageNamed((CFStringRef)string);
    if ( !NSEqualSizes(size,NSZeroSize) ) [image setSize:size];
    NSCursor *cursor = [[[NSCursor alloc] initWithImage:image hotSpot:hotSpot] autorelease];
    if ( cursor ) [cursor set];
}
@end


CursorRef CursorWithImage( ImageRef image, CGPoint hotSpot )
{ return [[[NSCursor alloc] initWithImage:image hotSpot:NSPointFromCGPoint(hotSpot)] autorelease]; }


ImageRef CursorImage( CursorRef cursor )
{ return [cursor image]; }

CGPoint CursorHotSpot( CursorRef cursor )
{ return NSPointToCGPoint([cursor hotSpot]); }

void CursorHide( void )
{ [NSCursor hide]; }

void CursorUnhide( void )
{ [NSCursor unhide]; }

void CursorSetHiddenUntilMouseMoves( Boolean flag )
{ [NSCursor setHiddenUntilMouseMoves:flag]; }


void CursorPop( void )
{ [NSCursor pop]; }

void CursorPush( CursorRef cursor )
{ [cursor push]; }

void CursorSet( CursorRef cursor )
{ [cursor set]; }


CursorRef CursorCurrent( void )
{ return [NSCursor currentCursor]; }

CursorRef CursorCurrentSystem( void )
{ return [NSCursor currentSystemCursor]; }

CursorRef CursorArrow( void )
{ return [NSCursor arrowCursor]; }

CursorRef CursorContextualMenu( void )
{ return [NSCursor contextualMenuCursor]; }

CursorRef CursorClosedHand( void )
{ return [NSCursor closedHandCursor]; }

CursorRef CursorCrosshair( void )
{ return [NSCursor crosshairCursor]; }

CursorRef CursorDisappearingItem( void )
{ return [NSCursor disappearingItemCursor]; }

CursorRef CursorDragCopy( void )
{ return [NSCursor dragCopyCursor]; }

CursorRef CursorDragLink( void )
{ return [NSCursor dragLinkCursor]; }

CursorRef CursorIBeam( void )
{ return [NSCursor IBeamCursor]; }

CursorRef CursorOpenHand( void )
{ return [NSCursor openHandCursor]; }

CursorRef CursorOperationNotAllowed( void )
{ return [NSCursor operationNotAllowedCursor]; }

CursorRef CursorPointingHand( void )
{ return [NSCursor pointingHandCursor]; }

CursorRef CursorResizeDown( void )
{ return [NSCursor resizeDownCursor]; }

CursorRef CursorResizeLeft( void )
{ return [NSCursor resizeLeftCursor]; }

CursorRef CursorResizeRight( void )
{ return [NSCursor resizeRightCursor]; }

CursorRef CursorResizeUp( void )
{ return [NSCursor resizeUpCursor]; }

CursorRef CursorResizeUpDown( void )
{ return [NSCursor resizeUpDownCursor]; }

CursorRef CursorIBeamForVerticalLayout( void )
{ return [NSCursor IBeamCursorForVerticalLayout]; }


// -- convenience --
void CursorSetImage( void *inImage, CGPoint hotSpot )
{
    NSImage *image = FixImageValue( inImage );
    if ( image ) {
        NSCursor *cursor = [[[NSCursor alloc] initWithImage:image hotSpot:NSPointFromCGPoint(hotSpot)] autorelease];
        if ( cursor ) [cursor set];
    }
}

void CursorSetImageNamed( CFStringRef imageName, CGPoint hotSpot )
{
    NSImage *image = CocoaImageResource( (NSString *)imageName );
    if ( image ) {
        NSCursor *cursor = [[[NSCursor alloc] initWithImage:image hotSpot:NSPointFromCGPoint(hotSpot)] autorelease];
        if ( cursor ) {
            [cursor set];
        }
    }
}

void CursorSetArrow( void )
{ [[NSCursor arrowCursor] set]; }

void CursorSetContextualMenu( void )
{ [[NSCursor contextualMenuCursor] set]; }

void CursorSetClosedHand( void )
{ [[NSCursor closedHandCursor] set]; }

void CursorSetCrosshair( void )
{ [[NSCursor crosshairCursor] set]; }

void CursorSetDisappearingItem( void )
{ [[NSCursor disappearingItemCursor] set]; }

void CursorSetDragCopy( void )
{ [[NSCursor dragCopyCursor] set]; }

void CursorSetDragLink( void )
{ [[NSCursor dragLinkCursor] set]; }

void CursorSetIBeam( void )
{ [[NSCursor IBeamCursor] set]; }

void CursorSetOpenHand( void )
{ [[NSCursor openHandCursor] set]; }

void CursorSetOperationNotAllowed( void )
{ [[NSCursor operationNotAllowedCursor] set]; }

void CursorSetPointingHand( void )
{ [[NSCursor pointingHandCursor] set]; }

void CursorSetResizeDown( void )
{ [[NSCursor resizeDownCursor] set]; }

void CursorSetResizeLeft( void )
{ [[NSCursor resizeLeftCursor] set]; }

void CursorSetResizeRight( void )
{ [[NSCursor resizeRightCursor] set]; }

void CursorSetResizeUp( void )
{ [[NSCursor resizeUpCursor] set]; }

void CursorSetResizeUpDown( void )
{ [[NSCursor resizeUpDownCursor] set]; }

void CursorSetIBeamForVerticalLayout( void )
{ [[NSCursor IBeamCursorForVerticalLayout] set]; }


// Custom
void CursorAnimatorStart( CFTimeInterval ti, CFStringRef inFirstImageName, CGSize size, CGPoint hotSpot, long numImages )
{
    if ( !sCocoaUICursorAnimator ) {
        sCocoaUICursorAnimator = [[CocoaUICursorAnimator alloc] init];
        sCocoaUICursorAnimator.counter = 1;
        NSString *firstImageName = (NSString *)inFirstImageName;
        NSString *extension = [firstImageName pathExtension];
        NSString *name = [firstImageName stringByDeletingPathExtension];
        name = [name substringToIndex:[name length] - 1];
        sCocoaUICursorAnimator.name = [name retain];
        sCocoaUICursorAnimator.extension = [extension retain];
        sCocoaUICursorAnimator.size = NSSizeFromCGSize(size);
        sCocoaUICursorAnimator.hotSpot = NSPointFromCGPoint(hotSpot);
        sCocoaUICursorAnimator.numImages = numImages;
        NSTimer *timer = [NSTimer scheduledTimerWithTimeInterval:ti target:sCocoaUICursorAnimator selector:@selector(timerFireMethod:) userInfo:nil repeats:YES];
        sCocoaUICursorAnimator.timer = timer;
    }
}

void CursorAnimatorStop( void )
{
    if ( sCocoaUICursorAnimator ) {
        [sCocoaUICursorAnimator.timer invalidate];
        [sCocoaUICursorAnimator release];
        sCocoaUICursorAnimator = nil;
    }
    [[NSCursor arrowCursor] set];
}


