/*
 Data.m

 Bernie Wylde
 */

#import "Data.h"

// - create -
CFDataRef DataWithBytes( void *bytes, NSUInteger length )
{ return (CFDataRef)[NSData dataWithBytes:bytes length:length]; }

// - read -
CFDataRef DataWithContentsOfURL( CFURLRef url, NSDataReadingOptions options, ErrorRef *err )
{ return (CFDataRef)[NSData dataWithContentsOfURL:(NSURL *)url options:options error:err]; }

// - write -
Boolean DataWriteToURL( CFDataRef dta, CFURLRef url, NSDataWritingOptions options, ErrorRef *err )
{ return [(NSData *)dta writeToURL:(NSURL *)url options:options error:err]; }

// - encoding/decoding Base64 representation -
//CFDataRef DataBase64EncodedData( CFStringRef string, NSDataBase64EncodingOptions options );// macOS 10.9+

// - access bytes -
const void *DataBytes( CFDataRef dta )
{ return [(NSData *)dta bytes]; }

void DataGetBytes( CFDataRef dta, void *buffer, NSUInteger length )
{ [(NSData *)dta getBytes:buffer length:length]; }

void DataGetBytesInRange( CFDataRef dta, void *buffer, CFRange range )
{ [(NSData *)dta getBytes:buffer range:NSMakeRange(range.location,range.length)]; }

// - find -
CFDataRef DataSubdata( CFDataRef dta, CFRange range )
{ return (CFDataRef)[(NSData *)dta subdataWithRange:NSMakeRange(range.location,range.length)]; }

CFRange DataRangeOfData( CFDataRef dta, CFDataRef dataToFind, NSDataSearchOptions searchOptions, CFRange searchRange )
{
    NSRange range = [(NSData *)dta rangeOfData:(NSData *)dataToFind options:searchOptions range:NSMakeRange(searchRange.location,searchRange.length)];
    return CFRangeMake(range.location,range.length);
}

// - test -
Boolean DataIsEqualToData( CFDataRef dta1, CFDataRef dta2 )
{ return [(NSData *)dta1 isEqualToData:(NSData *)dta2]; }

NSUInteger DataLength( CFDataRef dta )
{ return [(NSData *)dta length]; }

// - description -
CFStringRef DataDescription( CFDataRef dta )
{ return (CFStringRef)[(NSData *)dta description]; }



// -- mutable data --
// - create -
CFMutableDataRef DataWithCapacity( NSUInteger size )
{ return (CFMutableDataRef)[NSMutableData dataWithCapacity:size]; }

CFMutableDataRef DataWithLength( NSUInteger length )
{ return (CFMutableDataRef)[NSMutableData dataWithLength:length]; }

// - access -
void *DataMutableBytes( CFMutableDataRef dta )
{ return [(NSMutableData *)dta mutableBytes]; }

// - length -
void DataSetLength( CFMutableDataRef dta, NSUInteger length )
{ [(NSMutableData *)dta setLength:length]; }

// - add -
void DataAppendBytes( CFMutableDataRef dta, void *bytes, NSUInteger length )
{ [(NSMutableData *)dta appendBytes:bytes length:length]; }

void DataAppendData( CFMutableDataRef dta1, CFDataRef dta2 )
{ [(NSMutableData *)dta1 appendData:(NSData *)dta2]; }

void DataIncreaseLengthBy( CFMutableDataRef dta, NSUInteger extraLength )
{ [(NSMutableData *)dta increaseLengthBy:extraLength]; }

// - modify -
void DataReplaceBytesInRange( CFMutableDataRef dta, void *replacementBytes, CFRange range, NSUInteger length )
{ [(NSMutableData *)dta replaceBytesInRange:NSMakeRange(range.location,range.length) withBytes:replacementBytes length:length]; }

void DataResetBytesInRange( CFMutableDataRef dta, CFRange range )
{ [(NSMutableData *)dta resetBytesInRange:NSMakeRange(range.location,range.length)]; }

void DataSetData( CFMutableDataRef dta1, CFDataRef dta2 )
{ [(NSMutableData *)dta1 setData:(NSData *)dta2]; }

