/*
 DateComponents.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// - init -
DateComponentsRef DateComponentsInit( void );

// - calendar and time zone -
CFCalendarRef DateComponentsCalendar( DateComponentsRef dc );
void DateComponentsSetCalendar( DateComponentsRef dc, CFCalendarRef cal );
CFTimeZoneRef DateComponentsTimeZone( DateComponentsRef dc );
void DateComponentsSetTimeZone( DateComponentsRef dc, CFTimeZoneRef tz );

// - validate date -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
Boolean DateComponentsIsValidDate( DateComponentsRef dc );
Boolean DateComponentsIsValidDateInCalendar( DateComponentsRef dc, CFCalendarRef cal );
#endif
CFDateRef DateComponentsDate( DateComponentsRef dc );

// - years and months -
NSInteger DateComponentsEra( DateComponentsRef dc );
void DateComponentsSetEra( DateComponentsRef dc, NSInteger era );
NSInteger DateComponentsYear( DateComponentsRef dc );
void DateComponentsSetYear( DateComponentsRef dc, NSInteger year );
NSInteger DateComponentsYearForWeekOfYear( DateComponentsRef dc );
void DateComponentsSetYearForWeekOfYear( DateComponentsRef dc, NSInteger year );
NSInteger DateComponentsQuarter( DateComponentsRef dc );
void DateComponentsSetQuarter( DateComponentsRef dc, NSInteger quarter );
NSInteger DateComponentsMonth( DateComponentsRef dc );
void DateComponentsSetMonth( DateComponentsRef dc, NSInteger month );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
Boolean DateComponentsIsLeapMonth( DateComponentsRef dc );
void DateComponentsSetLeapMonth( DateComponentsRef dc, Boolean flag );
#endif

// - weeks and days -
NSInteger DateComponentsWeekday( DateComponentsRef dc );
void DateComponentsSetWeekday( DateComponentsRef dc, NSInteger day );
NSInteger DateComponentsWeekdayOrdinal( DateComponentsRef dc );
void DateComponentsSetWeekdayOrdinal( DateComponentsRef dc, NSInteger day );
NSInteger DateComponentsWeekOfMonth( DateComponentsRef dc );
void DateComponentsSetWeekOfMonth( DateComponentsRef dc, NSInteger week );
NSInteger DateComponentsWeekOfYear( DateComponentsRef dc );
void DateComponentsSetWeekOfYear( DateComponentsRef dc, NSInteger week );
NSInteger DateComponentsDay( DateComponentsRef dc );
void DateComponentsSetDay( DateComponentsRef dc, NSInteger day );

// - hours and seconds -
NSInteger DateComponentsHour( DateComponentsRef dc );
void DateComponentsSetHour( DateComponentsRef dc, NSInteger hour );
NSInteger DateComponentsMinute( DateComponentsRef dc );
void DateComponentsSetMinute( DateComponentsRef dc, NSInteger minute );
NSInteger DateComponentsSecond( DateComponentsRef dc );
void DateComponentsSetSecond( DateComponentsRef dc, NSInteger second );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSInteger DateComponentsNanosecond( DateComponentsRef dc );
void DateComponentsSetNanosecond( DateComponentsRef dc, NSInteger nanosecond );
#endif

// - components as calendrical units -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSInteger DateComponentsValueForComponent( DateComponentsRef dc, NSCalendarUnit unit );
void DateComponentsSetValueForComponent( DateComponentsRef dc, NSInteger value, NSCalendarUnit unit );
#endif
