/*
 DateComponents.m

 Bernie Wylde
 */

#import "DateComponents.h"

// - init -
DateComponentsRef DateComponentsInit( void )
{ return [[[NSDateComponents alloc] init] autorelease]; }

// - calendar and time zone -
CFCalendarRef DateComponentsCalendar( DateComponentsRef dc )
{ return (CFCalendarRef)[dc calendar]; }

void DateComponentsSetCalendar( DateComponentsRef dc, CFCalendarRef cal )
{ [dc setCalendar:(NSCalendar *)cal]; }

CFTimeZoneRef DateComponentsTimeZone( DateComponentsRef dc )
{ return (CFTimeZoneRef)[dc timeZone]; }

void DateComponentsSetTimeZone( DateComponentsRef dc, CFTimeZoneRef tz )
{ [dc setTimeZone:(NSTimeZone *)tz]; }

// - validate date -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
Boolean DateComponentsIsValidDate( DateComponentsRef dc )
{ return [dc isValidDate]; }

Boolean DateComponentsIsValidDateInCalendar( DateComponentsRef dc, CFCalendarRef cal )
{ return [dc isValidDateInCalendar:(NSCalendar *)cal]; }
#endif

CFDateRef DateComponentsDate( DateComponentsRef dc )
{ return (CFDateRef)[dc date]; }

// - years and months -
NSInteger DateComponentsEra( DateComponentsRef dc )
{ return [dc era]; }

void DateComponentsSetEra( DateComponentsRef dc, NSInteger era )
{ [dc setEra:era]; }

NSInteger DateComponentsYear( DateComponentsRef dc )
{ return [dc year]; }

void DateComponentsSetYear( DateComponentsRef dc, NSInteger year )
{ [dc setYear:year]; }

NSInteger DateComponentsYearForWeekOfYear( DateComponentsRef dc )
{ return [dc yearForWeekOfYear]; }

void DateComponentsSetYearForWeekOfYear( DateComponentsRef dc, NSInteger year )
{ [dc setYearForWeekOfYear:year]; }

NSInteger DateComponentsQuarter( DateComponentsRef dc )
{ return [dc quarter]; }

void DateComponentsSetQuarter( DateComponentsRef dc, NSInteger quarter )
{ [dc setQuarter:quarter]; }

NSInteger DateComponentsMonth( DateComponentsRef dc )
{ return [dc month]; }

void DateComponentsSetMonth( DateComponentsRef dc, NSInteger month )
{ [dc setMonth:month]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
Boolean DateComponentsIsLeapMonth( DateComponentsRef dc )
{ return [dc isLeapMonth]; }

void DateComponentsSetLeapMonth( DateComponentsRef dc, Boolean flag )
{ [dc setLeapMonth:flag]; }
#endif

// - weeks and days -
NSInteger DateComponentsWeekday( DateComponentsRef dc )
{ return [dc weekday]; }

void DateComponentsSetWeekday( DateComponentsRef dc, NSInteger day )
{ [dc setWeekday:day]; }

NSInteger DateComponentsWeekdayOrdinal( DateComponentsRef dc )
{ return [dc weekdayOrdinal]; }

void DateComponentsSetWeekdayOrdinal( DateComponentsRef dc, NSInteger day )
{ [dc setWeekdayOrdinal:day]; }

NSInteger DateComponentsWeekOfMonth( DateComponentsRef dc )
{ return [dc weekOfMonth]; }

void DateComponentsSetWeekOfMonth( DateComponentsRef dc, NSInteger week )
{ [dc setWeekOfMonth:week]; }

NSInteger DateComponentsWeekOfYear( DateComponentsRef dc )
{ return [dc weekOfYear]; }

void DateComponentsSetWeekOfYear( DateComponentsRef dc, NSInteger week )
{ [dc setWeekOfYear:week]; }

NSInteger DateComponentsDay( DateComponentsRef dc )
{ return [dc day]; }

void DateComponentsSetDay( DateComponentsRef dc, NSInteger day )
{ [dc setDay:day]; }

// - hours and seconds -
NSInteger DateComponentsHour( DateComponentsRef dc )
{ return [dc hour]; }

void DateComponentsSetHour( DateComponentsRef dc, NSInteger hour )
{ [dc setHour:hour]; }

NSInteger DateComponentsMinute( DateComponentsRef dc )
{ return [dc minute]; }

void DateComponentsSetMinute( DateComponentsRef dc, NSInteger minute )
{ [dc setMinute:minute]; }

NSInteger DateComponentsSecond( DateComponentsRef dc )
{ return [dc second]; }

void DateComponentsSetSecond( DateComponentsRef dc, NSInteger second )
{ [dc setSecond:second]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSInteger DateComponentsNanosecond( DateComponentsRef dc )
{ return [dc nanosecond]; }

void DateComponentsSetNanosecond( DateComponentsRef dc, NSInteger nanosecond )
{ [dc setNanosecond:nanosecond]; }
#endif

// - components as calendrical units -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSInteger DateComponentsValueForComponent( DateComponentsRef dc, NSCalendarUnit unit )
{ return [dc valueForComponent:unit]; }

void DateComponentsSetValueForComponent( DateComponentsRef dc, NSInteger value, NSCalendarUnit unit )
{ [dc setValue:value forComponent:unit]; }
#endif

