/*
 DateComponentsFormatter.m

 Bernie Wylde
 */

#import "DateComponentsFormatter.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000

// Formatting values
CFStringRef DateComponentsFormatterStringFromDateComponents( DateComponentsFormatterRef formatter, DateComponentsRef dc )
{ return (CFStringRef)[formatter stringFromDateComponents:dc]; }

CFStringRef DateComponentsFormatterStringForObjectValue( DateComponentsFormatterRef formatter, CFTypeRef obj )
{ return (CFStringRef)[formatter stringForObjectValue:(id)obj]; }

CFStringRef DateComponentsFormatterStringFromDate( DateComponentsFormatterRef formatter, CFDateRef startDate, CFDateRef endDate )
{ return (CFStringRef)[formatter stringFromDate:(NSDate *)startDate toDate:(NSDate *)endDate]; }

CFStringRef DateComponentsFormatterStringFromTimeInterval( DateComponentsFormatterRef formatter, CFTimeInterval interval )
{ return (CFStringRef)[formatter stringFromTimeInterval:(NSTimeInterval)interval]; }

CFStringRef DateComponentsFormatterLocalizedStringFromDateComponents( DateComponentsRef dc, NSDateComponentsFormatterUnitsStyle style )
{ return (CFStringRef)[NSDateComponentsFormatter localizedStringFromDateComponents:dc unitsStyle:style]; }

// Configure formatter options
NSCalendarUnit DateComponentsFormatterAllowedUnits( DateComponentsFormatterRef formatter )
{ return [formatter allowedUnits]; }

void DateComponentsFormatterSetAllowedUnits( DateComponentsFormatterRef formatter, NSCalendarUnit units )
{ [formatter setAllowedUnits:units]; }

Boolean DateComponentsFormatterAllowsFractionalUnits( DateComponentsFormatterRef formatter )
{ return [formatter allowsFractionalUnits]; }

void DateComponentsFormatterSetAllowsFractionalUnits( DateComponentsFormatterRef formatter, Boolean flag )
{ [formatter setAllowsFractionalUnits:flag]; }

CFCalendarRef DateComponentsFormatterCalendar( DateComponentsFormatterRef formatter )
{ return (CFCalendarRef)[formatter calendar]; }

void DateComponentsFormatterSetCalendar( DateComponentsFormatterRef formatter, CFCalendarRef cal )
{ [formatter setCalendar:(NSCalendar *)cal]; }

Boolean DateComponentsFormatterCollapsesLargstUnit( DateComponentsFormatterRef formatter )
{ return [formatter collapsesLargestUnit]; }

void DateComponentsFormatterSetCollapsesLargstUnit( DateComponentsFormatterRef formatter, Boolean flag )
{ [formatter setCollapsesLargestUnit:flag]; }

Boolean DateComponentsFormatterIncludesApproximationPhrase( DateComponentsFormatterRef formatter )
{ return [formatter includesApproximationPhrase]; }

void DateComponentsFormatterSetIncludesApproximationPhrase( DateComponentsFormatterRef formatter, Boolean flag )
{ [formatter setIncludesApproximationPhrase:flag]; }

Boolean DateComponentsFormatterIncludesTimeRemainingPhrase( DateComponentsFormatterRef formatter )
{ return [formatter includesTimeRemainingPhrase]; }

void DateComponentsFormatterSetIncludesTimeRemainingPhrase( DateComponentsFormatterRef formatter, Boolean flag )
{ [formatter setIncludesTimeRemainingPhrase:flag]; }

NSInteger DateComponentsFormatterMaximumUnitCount( DateComponentsFormatterRef formatter )
{ return [formatter maximumUnitCount]; }

void DateComponentsFormatterSetMaximumUnitCount( DateComponentsFormatterRef formatter, NSInteger count )
{ [formatter setMaximumUnitCount:count]; }

NSDateComponentsFormatterUnitsStyle DateComponentsFormatterUnitsStyle( DateComponentsFormatterRef formatter )
{ return [formatter unitsStyle]; }

void DateComponentsFormatterSetUnitsStyle( DateComponentsFormatterRef formatter, NSDateComponentsFormatterUnitsStyle style )
{ [formatter setUnitsStyle:style]; }

NSDateComponentsFormatterZeroFormattingBehavior DateComponentsFormatterZeroFormattingBehavior( DateComponentsFormatterRef formatter )
{ return [formatter zeroFormattingBehavior]; }

void DateComponentsFormatterSetZeroFormattingBehavior( DateComponentsFormatterRef formatter, NSDateComponentsFormatterZeroFormattingBehavior behavior )
{ [formatter setZeroFormattingBehavior:behavior]; }

// Instance properties
NSFormattingContext DateComponentsFormatterFormattingContext( DateComponentsFormatterRef formatter )
{ return [formatter formattingContext]; }

void DateComponentsFormatterSetFormattingContext( DateComponentsFormatterRef formatter, NSFormattingContext ctx )
{ [formatter setFormattingContext:ctx]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFDateRef DateComponentsFormatterReferenceDate( DateComponentsFormatterRef formatter )
{ return (CFDateRef)[formatter referenceDate]; }

void DateComponentsFormatterSetReferenceDate( DateComponentsFormatterRef formatter, CFDateRef dt )
{ [formatter setReferenceDate:(NSDate *)dt]; }
#endif

// Instance methods
Boolean DateComponentsFormatterGetObjectValue( DateComponentsFormatterRef formatter, CFTypeRef *obj, CFStringRef string, CFStringRef *errorDescription )
{ return [formatter getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)errorDescription]; }

#endif
