/*
 DateFormatter.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// - init -
DateFormatterRef DateFormatterInit( void );

// - converting objects -
CFDateRef DateFormatterDateFromString( DateFormatterRef formatter, CFStringRef string );
CFStringRef DateFormatterStringFromDate( DateFormatterRef formatter, CFDateRef dt );
CFStringRef DateFormatterLocalizedStringFromDate( CFDateRef dt, NSDateFormatterStyle dateStyle, NSDateFormatterStyle timeStyle );
Boolean DateFormatterGetObjectValueForString( DateFormatterRef formatter, CFTypeRef *obj, CFStringRef string, CFRange *inOutRange, ErrorRef *err );

// - format and style
NSDateFormatterStyle DateFormatterDateStyle( DateFormatterRef formatter );
void DateFormatterSetDateStyle( DateFormatterRef formatter, NSDateFormatterStyle style );
NSDateFormatterStyle DateFormatterTimeStyle( DateFormatterRef formatter );
void DateFormatterSetTimeStyle( DateFormatterRef formatter, NSDateFormatterStyle style );
CFStringRef DateFormatterDateFormat( DateFormatterRef formatter );
void DateFormatterSetDateFormat( DateFormatterRef formatter, CFStringRef format );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void DateFormatterSetLocalizedDateFormatFromTemplate( DateFormatterRef formatter, CFStringRef template );
#endif

CFStringRef DateFormatterDateFormatFromTemplate( CFStringRef template, NSUInteger options, CFLocaleRef locale );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSFormattingContext DateFormatterFormattingContext( DateFormatterRef formatter );
void DateFormatterSetFormattingContext( DateFormatterRef formatter, NSFormattingContext ctx );
#endif

// - attributes -
CFCalendarRef DateFormatterCalendar( DateFormatterRef formatter );
void DateFormatterSetCalendar( DateFormatterRef formatter, CFCalendarRef cal );
CFDateRef DateFormatterDefaultDate( DateFormatterRef formatter );
void DateFormatterSetDefaultDate( DateFormatterRef formatter, CFDateRef dt );
CFLocaleRef DateFormatterLocale( DateFormatterRef formatter );
void DateFormatterSetLocale( DateFormatterRef formatter, CFLocaleRef locale );
CFTimeZoneRef DateFormatterTimeZone( DateFormatterRef formatter );
void DateFormatterSetTimeZone( DateFormatterRef formatter, CFTimeZoneRef zone );
CFDateRef DateFormatterTwoDigitStartDate( DateFormatterRef formatter );
void DateFormatterSetTwoDigitStartDate( DateFormatterRef formatter, CFDateRef dt );
CFDateRef DateFormatterGregorianStartDate( DateFormatterRef formatter );
void DateFormatterSetGregorianStartDate( DateFormatterRef formatter, CFDateRef dt );

// - behavior -
NSDateFormatterBehavior DateFormatterBehavior( DateFormatterRef formatter );
void DateFormatterSetBehavior( DateFormatterRef formatter, NSDateFormatterBehavior behavior );
NSDateFormatterBehavior DateFormatterDefaultBehavior( void );
void DateFormatterSetDefaultBehavior( NSDateFormatterBehavior behavior );

// - natural language -
Boolean DateFormatterIsLenient( DateFormatterRef formatter );
void DateFormatterSetLenient( DateFormatterRef formatter, Boolean flag );
Boolean DateFormatterDoesRelativeDateFormatting( DateFormatterRef formatter );
void DateFormatterSetDoesRelativeDateFormatting( DateFormatterRef formatter, Boolean flag );

// - am and pm symbols -
CFStringRef DateFormatterAMSymbol( DateFormatterRef formatter );
void DateFormatterSetAMSymbol( DateFormatterRef formatter, CFStringRef symbol );
CFStringRef DateFormatterPMSymbol( DateFormatterRef formatter );
void DateFormatterSetPMSymbol( DateFormatterRef formatter, CFStringRef symbol );

// - weekday symbols -
CFArrayRef DateFormatterWeekdaySymbols( DateFormatterRef formatter );
void DateFormatterSetWeekdaySymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterShortWeekdaySymbols( DateFormatterRef formatter );
void DateFormatterSetShortWeekdaySymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterVeryShortWeekdaySymbols( DateFormatterRef formatter );
void DateFormatterSetVeryShortWeekdaySymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterStandaloneWeekdaySymbols( DateFormatterRef formatter );
void DateFormatterSetStandaloneWeekdaySymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterShortStandaloneWeekdaySymbols( DateFormatterRef formatter );
void DateFormatterSetShortStandaloneWeekdaySymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterVeryShortStandaloneWeekdaySymbols( DateFormatterRef formatter );
void DateFormatterSetVeryShortStandaloneWeekdaySymbols( DateFormatterRef formatter, CFArrayRef symbols );

// - month symbols -
CFArrayRef DateFormatterMonthSymbols( DateFormatterRef formatter );
void DateFormatterSetMonthSymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterShortMonthSymbols( DateFormatterRef formatter );
void DateFormatterSetShortMonthSymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterVeryShortMonthSymbols( DateFormatterRef formatter );
void DateFormatterSetVeryShortMonthSymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterStandaloneMonthSymbols( DateFormatterRef formatter );
void DateFormatterSetStandaloneMonthSymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterShortStandaloneMonthSymbols( DateFormatterRef formatter );
void DateFormatterSetShortStandaloneMonthSymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterVeryShortStandaloneMonthSymbols( DateFormatterRef formatter );
void DateFormatterSetVeryShortStandaloneMonthSymbols( DateFormatterRef formatter, CFArrayRef symbols );

// - quarter symbols -
CFArrayRef DateFormatterQuarterSymbols( DateFormatterRef formatter );
void DateFormatterSetQuarterSymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterShortQuarterSymbols( DateFormatterRef formatter );
void DateFormatterSetShortQuarterSymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterStandaloneQuarterSymbols( DateFormatterRef formatter );
void DateFormatterSetStandaloneQuarterSymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterShortStandaloneQuarterSymbols( DateFormatterRef formatter );
void DateFormatterSetShortStandaloneQuarterSymbols( DateFormatterRef formatter, CFArrayRef symbols );

// - era symbols -
CFArrayRef DateFormatterEraSymbols( DateFormatterRef formatter );
void DateFormatterSetEraSymbols( DateFormatterRef formatter, CFArrayRef symbols );
CFArrayRef DateFormatterLongEraSymbols( DateFormatterRef formatter );
void DateFormatterSetLongEraSymbols( DateFormatterRef formatter, CFArrayRef symbols );

