/*
 DateFormatter.m

 Bernie Wylde
 */

#import "DateFormatter.h"

// - init -
DateFormatterRef DateFormatterInit( void )
{ return [[[NSDateFormatter alloc] init] autorelease]; }

// - converting objects -
CFDateRef DateFormatterDateFromString( DateFormatterRef formatter, CFStringRef string )
{ return (CFDateRef)[formatter dateFromString:(NSString *)string]; }

CFStringRef DateFormatterStringFromDate( DateFormatterRef formatter, CFDateRef dt )
{ return (CFStringRef)[formatter stringFromDate:(NSDate *)dt]; }

CFStringRef DateFormatterLocalizedStringFromDate( CFDateRef dt, NSDateFormatterStyle dateStyle, NSDateFormatterStyle timeStyle )
{ return (CFStringRef)[NSDateFormatter localizedStringFromDate:(NSDate *)dt dateStyle:dateStyle timeStyle:timeStyle]; }

Boolean DateFormatterGetObjectValueForString( DateFormatterRef formatter, CFTypeRef *obj, CFStringRef string, CFRange *inOutRange, ErrorRef *err )
{
    BOOL result;
    NSRange range = NSMakeRange(inOutRange->location,inOutRange->length);
    result = [formatter getObjectValue:(id *)obj forString:(NSString *)string range:&range error:err];
    inOutRange->location = range.location;
    inOutRange->length = range.length;
    return result;
}

// - format and style
NSDateFormatterStyle DateFormatterDateStyle( DateFormatterRef formatter )
{ return [formatter dateStyle]; }

void DateFormatterSetDateStyle( DateFormatterRef formatter, NSDateFormatterStyle style )
{ [formatter setDateStyle:style]; }

NSDateFormatterStyle DateFormatterTimeStyle( DateFormatterRef formatter )
{ return [formatter timeStyle]; }

void DateFormatterSetTimeStyle( DateFormatterRef formatter, NSDateFormatterStyle style )
{ [formatter setTimeStyle:style]; }

CFStringRef DateFormatterDateFormat( DateFormatterRef formatter )
{ return (CFStringRef)[formatter dateFormat]; }

void DateFormatterSetDateFormat( DateFormatterRef formatter, CFStringRef format )
{ [formatter setDateFormat:(NSString *)format]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void DateFormatterSetLocalizedDateFormatFromTemplate( DateFormatterRef formatter, CFStringRef template )
{ [formatter setLocalizedDateFormatFromTemplate:(NSString *)template]; }
#endif

CFStringRef DateFormatterDateFormatFromTemplate( CFStringRef template, NSUInteger options, CFLocaleRef locale )
{ return (CFStringRef)[NSDateFormatter dateFormatFromTemplate:(NSString *)template options:options locale:(NSLocale *)locale]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSFormattingContext DateFormatterFormattingContext( DateFormatterRef formatter )
{ return [formatter formattingContext]; }

void DateFormatterSetFormattingContext( DateFormatterRef formatter, NSFormattingContext ctx )
{ [formatter setFormattingContext:ctx]; }
#endif

// - attributes -
CFCalendarRef DateFormatterCalendar( DateFormatterRef formatter )
{ return (CFCalendarRef)[formatter calendar]; }

void DateFormatterSetCalendar( DateFormatterRef formatter, CFCalendarRef cal )
{ [formatter setCalendar:(NSCalendar *)cal]; }

CFDateRef DateFormatterDefaultDate( DateFormatterRef formatter )
{ return (CFDateRef)[formatter defaultDate]; }

void DateFormatterSetDefaultDate( DateFormatterRef formatter, CFDateRef dt )
{ [formatter setDefaultDate:(NSDate *)dt]; }

CFLocaleRef DateFormatterLocale( DateFormatterRef formatter )
{ return (CFLocaleRef)[formatter locale]; }

void DateFormatterSetLocale( DateFormatterRef formatter, CFLocaleRef locale )
{ [formatter setLocale:(NSLocale *)locale]; }

CFTimeZoneRef DateFormatterTimeZone( DateFormatterRef formatter )
{ return (CFTimeZoneRef)[formatter timeZone]; }

void DateFormatterSetTimeZone( DateFormatterRef formatter, CFTimeZoneRef zone )
{ [formatter setTimeZone:(NSTimeZone *)zone]; }

CFDateRef DateFormatterTwoDigitStartDate( DateFormatterRef formatter )
{ return (CFDateRef)[formatter twoDigitStartDate]; }

void DateFormatterSetTwoDigitStartDate( DateFormatterRef formatter, CFDateRef dt )
{ [formatter setTwoDigitStartDate:(NSDate *)dt]; }

CFDateRef DateFormatterGregorianStartDate( DateFormatterRef formatter )
{ return (CFDateRef)[formatter gregorianStartDate]; }

void DateFormatterSetGregorianStartDate( DateFormatterRef formatter, CFDateRef dt )
{ [formatter setGregorianStartDate:(NSDate *)dt]; }

// - behavior -
NSDateFormatterBehavior DateFormatterBehavior( DateFormatterRef formatter )
{ return [formatter formatterBehavior]; }

void DateFormatterSetBehavior( DateFormatterRef formatter, NSDateFormatterBehavior behavior )
{ [formatter setFormatterBehavior:behavior]; }

NSDateFormatterBehavior DateFormatterDefaultBehavior( void )
{ return [NSDateFormatter defaultFormatterBehavior]; }

void DateFormatterSetDefaultBehavior( NSDateFormatterBehavior behavior )
{ [NSDateFormatter setDefaultFormatterBehavior:behavior]; }

// - natural language -
Boolean DateFormatterIsLenient( DateFormatterRef formatter )
{ return [formatter isLenient]; }

void DateFormatterSetLenient( DateFormatterRef formatter, Boolean flag )
{ [formatter setLenient:flag]; }

Boolean DateFormatterDoesRelativeDateFormatting( DateFormatterRef formatter )
{ return [formatter doesRelativeDateFormatting]; }

void DateFormatterSetDoesRelativeDateFormatting( DateFormatterRef formatter, Boolean flag )
{ return [formatter setDoesRelativeDateFormatting:flag]; }

// - am and pm symbols -
CFStringRef DateFormatterAMSymbol( DateFormatterRef formatter )
{ return (CFStringRef)[formatter AMSymbol]; }

void DateFormatterSetAMSymbol( DateFormatterRef formatter, CFStringRef symbol )
{ [formatter setAMSymbol:(NSString *)symbol]; }

CFStringRef DateFormatterPMSymbol( DateFormatterRef formatter )
{ return (CFStringRef)[formatter PMSymbol]; }

void DateFormatterSetPMSymbol( DateFormatterRef formatter, CFStringRef symbol )
{ [formatter setPMSymbol:(NSString *)symbol]; }

// - weekday symbols -
CFArrayRef DateFormatterWeekdaySymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter weekdaySymbols]; }

void DateFormatterSetWeekdaySymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setWeekdaySymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterShortWeekdaySymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter shortWeekdaySymbols]; }

void DateFormatterSetShortWeekdaySymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setShortWeekdaySymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterVeryShortWeekdaySymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter veryShortWeekdaySymbols]; }

void DateFormatterSetVeryShortWeekdaySymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setVeryShortWeekdaySymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterStandaloneWeekdaySymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter standaloneWeekdaySymbols]; }

void DateFormatterSetStandaloneWeekdaySymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setStandaloneWeekdaySymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterShortStandaloneWeekdaySymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter shortStandaloneWeekdaySymbols]; }

void DateFormatterSetShortStandaloneWeekdaySymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setShortStandaloneWeekdaySymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterVeryShortStandaloneWeekdaySymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter veryShortStandaloneWeekdaySymbols]; }

void DateFormatterSetVeryShortStandaloneWeekdaySymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setVeryShortStandaloneWeekdaySymbols:(NSArray *)symbols]; }

// - month symbols -
CFArrayRef DateFormatterMonthSymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter monthSymbols]; }

void DateFormatterSetMonthSymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setMonthSymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterShortMonthSymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter shortMonthSymbols]; }

void DateFormatterSetShortMonthSymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setShortMonthSymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterVeryShortMonthSymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter veryShortMonthSymbols]; }

void DateFormatterSetVeryShortMonthSymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setVeryShortMonthSymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterStandaloneMonthSymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter standaloneMonthSymbols]; }

void DateFormatterSetStandaloneMonthSymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setStandaloneMonthSymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterShortStandaloneMonthSymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter shortStandaloneMonthSymbols]; }

void DateFormatterSetShortStandaloneMonthSymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setShortStandaloneMonthSymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterVeryShortStandaloneMonthSymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter veryShortStandaloneMonthSymbols]; }

void DateFormatterSetVeryShortStandaloneMonthSymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setVeryShortStandaloneMonthSymbols:(NSArray *)symbols]; }

// - quarter symbols -
CFArrayRef DateFormatterQuarterSymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter quarterSymbols]; }

void DateFormatterSetQuarterSymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setQuarterSymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterShortQuarterSymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter shortQuarterSymbols]; }

void DateFormatterSetShortQuarterSymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setShortQuarterSymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterStandaloneQuarterSymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter standaloneQuarterSymbols]; }

void DateFormatterSetStandaloneQuarterSymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setStandaloneQuarterSymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterShortStandaloneQuarterSymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter shortStandaloneQuarterSymbols]; }

void DateFormatterSetShortStandaloneQuarterSymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setShortStandaloneQuarterSymbols:(NSArray *)symbols]; }

// - era symbols -
CFArrayRef DateFormatterEraSymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter eraSymbols]; }

void DateFormatterSetEraSymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setEraSymbols:(NSArray *)symbols]; }

CFArrayRef DateFormatterLongEraSymbols( DateFormatterRef formatter )
{ return (CFArrayRef)[formatter longEraSymbols]; }

void DateFormatterSetLongEraSymbols( DateFormatterRef formatter, CFArrayRef symbols )
{ [formatter setLongEraSymbols:(NSArray *)symbols]; }

