/*
 DatePicker.m

 Bernie Wylde
 */

#import "DatePicker.h"

#pragma mark - subclass
@implementation DatePickerSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(YES);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end


NSDatePicker *DatePickerWithTag( NSInteger tag )
{ return (NSDatePicker *)CocoaValidateViewKindOfClass( tag, [NSDatePicker class] ); }

BOOL DatePickerExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSDatePicker class] ); }


// - configure -
Boolean DatePickerIsBezeled( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return [picker isBezeled];
    return NO;
}

void DatePickerSetBezeled( NSInteger tag, Boolean flag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setBezeled:flag];
}

Boolean DatePickerIsBordered( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return [picker isBordered];
    return NO;
}

void DatePickerSetBordered( NSInteger tag, Boolean flag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setBordered:flag];
}

ColorRef DatePickerBackgroundColor( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return [picker backgroundColor];
    return nil;
}

void DatePickerSetBackgroundColor( NSInteger tag, ColorRef col )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setBackgroundColor:col];
}

Boolean DatePickerDrawsBackground( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return [picker drawsBackground];
    return NO;
}

void DatePickerSetDrawsBackground( NSInteger tag, Boolean flag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setDrawsBackground:flag];
}

ColorRef DatePickerTextColor( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return [picker textColor];
    return nil;
}

void DatePickerSetTextColor( NSInteger tag, ColorRef col )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setTextColor:col];
}

NSDatePickerStyle DatePickerStyle( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return [picker datePickerStyle];
    return 0;
}

void DatePickerSetStyle( NSInteger tag, NSDatePickerStyle style )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setDatePickerStyle:style];
}

// delegate

NSDatePickerElementFlags DatePickerElements( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return [picker datePickerElements];
    return 0;
}

void DatePickerSetElements( NSInteger tag, NSDatePickerElementFlags elements )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setDatePickerElements:elements];
}

// - range and mode -
CFCalendarRef DatePickerCalendar( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return (CFCalendarRef)[picker calendar];
    return nil;
}

void DatePickerSetCalendar( NSInteger tag, CFCalendarRef cal )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setCalendar:(NSCalendar *)cal];
}

CFLocaleRef DatePickerLocale( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return (CFLocaleRef)[picker locale];
    return nil;
}

void DatePickerSetLocale( NSInteger tag, CFLocaleRef locale )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setLocale:(NSLocale *)locale];
}

NSDatePickerMode DatePickerMode( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return [picker datePickerMode];
    return 0;
}

void DatePickerSetMode( NSInteger tag, NSInteger mode )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setDatePickerMode:mode];
}

CFTimeZoneRef DatePickerTimeZone( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return (CFTimeZoneRef)[picker timeZone];
    return nil;
}

void DatePickerSetTimeZone( NSInteger tag, CFTimeZoneRef zone )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setTimeZone:(NSTimeZone *)zone];
}

// - values -
CFDateRef DatePickerDateValue( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return (CFDateRef)[picker dateValue];
    return NULL;
}

void DatePickerSetDateValue( NSInteger tag, CFDateRef dt )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setDateValue:(NSDate *)dt];
}

CFTimeInterval DatePickerTimeInterval( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return [picker timeInterval];
    return 0.0;
}

void DatePickerSetTimeInterval( NSInteger tag, CFTimeInterval ti )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setTimeInterval:ti];
}

// - constraining -
CFDateRef DatePickerMinDate( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return (CFDateRef)[picker minDate];
    return NULL;
}

void DatePickerSetMinDate( NSInteger tag, CFDateRef dt )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setMinDate:(NSDate *)dt];
}

CFDateRef DatePickerMaxDate( NSInteger tag )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) return (CFDateRef)[picker maxDate];
    return NULL;
}

void DatePickerSetMaxDate( NSInteger tag, CFDateRef dt )
{
    NSDatePicker *picker = DatePickerWithTag( tag );
    if ( picker ) [picker setMaxDate:(NSDate *)dt];
}


// Init
DatePickerRef DatePickerInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    BOOL emptyRect = NSIsEmptyRect( rect );
    if ( emptyRect ) rect = NSMakeRect( 20, 20, 99, 27 );

    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSDatePicker *picker;
    if ( subclass ) {
        picker = [[[DatePickerSC alloc] initWithFrame:rect] autorelease];
    } else {
        picker = [[[NSDatePicker alloc] initWithFrame:rect] autorelease];
    }
    
    // draws background
    [picker setDrawsBackground:YES];

    // elements
    [picker setDatePickerElements:NSYearMonthDayDatePickerElementFlag];

    // size to fit
    if ( emptyRect ) [picker sizeToFit];

    // tag
    [picker setTag:inTag];
    
    // target/action
    [picker setTarget:CocoaControlTargetObj()];
    [picker setAction:@selector(controlAction:)];
    
    return picker;
}

// Statement
void CocoaDatePicker( NSInteger inTag, NSInteger inEnabled, CFDateRef inDate, CGRect inRect, NSInteger inStyle, NSInteger inElements, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSDate *date = (NSDate *)inDate;
    NSRect rect = NSRectFromCGRect( inRect );
    BOOL emptyRect = NSIsEmptyRect( rect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSDatePicker *picker = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !picker ) {
            picker = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( picker ) {    // already exists
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [picker class], [NSDatePicker class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, picker, [NSDatePicker class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [picker setEnabled:inEnabled];
            
            // date
            if ( date ) [picker setDateValue:date];
            
            // rect
            if ( !emptyRect ) [picker setFrame:rect];
            
            // style
            if ( inStyle != kFBParamMissing ) [picker setDatePickerStyle:inStyle];
            
            // elements
            if ( inElements == kFBParamMissing ) {
                [picker setDatePickerElements:NSYearMonthDayDatePickerElementFlag];
            } else {
                [picker setDatePickerElements:inElements];
            }
            
        } else {            // create
            
            // rect
//            if ( emptyRect ) rect = NSMakeRect( 20, 20, 99, 27 );
            
            // init
//            if ( inSubclass ) {
//                picker = [[DatePickerSC alloc] initWithFrame:rect];
//            } else {
//                picker = [[NSDatePicker alloc] initWithFrame:rect];
//            }
            
            picker = DatePickerInit( inTag, inRect );
            
            // default autoresizing
            [picker setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [picker setTag:inTag];
            
            // default draws background
//            [picker setDrawsBackground:YES];
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [picker setEnabled:inEnabled];
            
            // date
            if ( date ) [picker setDateValue:date];
            
            // style
            if ( inStyle != kFBParamMissing ) [picker setDatePickerStyle:inStyle];
            
            // elements
//            if ( inElements == kFBParamMissing ) {
//                [picker setDatePickerElements:NSYearMonthDayDatePickerElementFlag];
//            } else {
//                [picker setDatePickerElements:inElements];
//            }
            if ( inElements != kFBParamMissing ) [picker setDatePickerElements:inElements];

            // size to fit
            if ( emptyRect ) [picker sizeToFit];
            
            // wndTag
            [picker setWndTag:inWndTag];
            
            // target/action
//            [picker setTarget:CocoaControlTargetObj()];
//            [picker setAction:@selector(controlAction:)];
            
            // add to window
            [contentView addSubview:picker];
//            [picker release];
        }
        
        [picker setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

