/*
 DecimalNumber.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// - create -
DecimalNumberRef DecimalNumberWithDecimal( NSDecimal dcm );
DecimalNumberRef DecimalNumberWithMantissa( UInt64 mantissa, short exponent, Boolean isNegative  );
DecimalNumberRef DecimalNumberWithString( CFStringRef string );
DecimalNumberRef DecimalNumberWithStringAndLocale( CFStringRef string, CFLocaleRef locale );
DecimalNumberRef DecimalNumberOne( void );
DecimalNumberRef DecimalNumberZero( void );
DecimalNumberRef DecimalNumberNotANumber( void );

// - perform arithmetic -
DecimalNumberRef DecimalNumberByAdding( DecimalNumberRef dn1, DecimalNumberRef dn2 );
DecimalNumberRef DecimalNumberBySubtracting( DecimalNumberRef dn1, DecimalNumberRef dn2 );
DecimalNumberRef DecimalNumberByMultiplyingBy( DecimalNumberRef dn1, DecimalNumberRef dn2 );
DecimalNumberRef DecimalNumberByDividingBy( DecimalNumberRef dn1, DecimalNumberRef dn2 );
DecimalNumberRef DecimalNumberByRaisingToPower( DecimalNumberRef dn, NSUInteger power );
DecimalNumberRef DecimalNumberByMultiplyingByPowerOf10( DecimalNumberRef dn, short power );
DecimalNumberRef DecimalNumberByAddingWithBehavior( DecimalNumberRef dn1, DecimalNumberRef dn2, DecimalNumberBehaviorsRef behavior );
DecimalNumberRef DecimalNumberBySubtractingWithBehavior( DecimalNumberRef dn1, DecimalNumberRef dn2, DecimalNumberBehaviorsRef behavior );
DecimalNumberRef DecimalNumberByMultiplyingByWithBehavior( DecimalNumberRef dn1, DecimalNumberRef dn2, DecimalNumberBehaviorsRef behavior );
DecimalNumberRef DecimalNumberByDividingByWithBehavior( DecimalNumberRef dn1, DecimalNumberRef dn2, DecimalNumberBehaviorsRef behavior );
DecimalNumberRef DecimalNumberByRaisingToPowerWithBehavior( DecimalNumberRef dn, NSUInteger power, DecimalNumberBehaviorsRef behavior );
DecimalNumberRef DecimalNumberByMultiplyingByPowerOf10WithBehavior( DecimalNumberRef dn, short power, DecimalNumberBehaviorsRef behavior );

// - rounding -
DecimalNumberRef DecimalNumberByRounding( DecimalNumberRef dn, DecimalNumberBehaviorsRef behavior );

// - behavior -
DecimalNumberBehaviorsRef DecimalNumberDefaultBehavior( void );

// - value -
NSDecimal DecimalNumberDecimalValue( DecimalNumberRef dn );
double DecimalNumberDoubleValue( DecimalNumberRef dn );
CFStringRef DecimalNumberDescription( DecimalNumberRef dn, CFLocaleRef locale );

// - compare -
NSComparisonResult DecimalNumberCompare( DecimalNumberRef dn1, DecimalNumberRef dn2 );

// - min/max values -
DecimalNumberRef DecimalNumberMaximum( void );
DecimalNumberRef DecimalNumberMinimum( void );





#pragma mark - decimalnumberbehaviors
///////////////////////////////////////////////////
//          decimalnumberbehaviors               //
///////////////////////////////////////////////////
// - rounding -
NSRoundingMode DecimalNumberBehaviorsRoundingMode( DecimalNumberBehaviorsRef ref );
short DecimalNumberBehaviorsScale( DecimalNumberBehaviorsRef ref );







#pragma mark - decimalnumberhandler
///////////////////////////////////////////////////
//           decimalnumberhandler                //
///////////////////////////////////////////////////
// - create -
DecimalNumberHandlerRef DecimalNumberHandlerDefault( void );
DecimalNumberHandlerRef DecimalNumberHandlerWithRoundingMode( NSRoundingMode mode, short scale, Boolean raiseOnExactness, Boolean raiseOnOverflow, Boolean raiseOnUnderflow, Boolean raiseOnDivideByZero );


