/*
 DecimalNumber.m

 Bernie Wylde
 */

#import "DecimalNumber.h"

// - create -
DecimalNumberRef DecimalNumberWithDecimal( NSDecimal dcm )
{ return [NSDecimalNumber decimalNumberWithDecimal:dcm]; }

DecimalNumberRef DecimalNumberWithMantissa( UInt64 mantissa, short exponent, Boolean isNegative  )
{ return [NSDecimalNumber decimalNumberWithMantissa:mantissa exponent:exponent isNegative:isNegative]; }

DecimalNumberRef DecimalNumberWithString( CFStringRef string )
{ return [NSDecimalNumber decimalNumberWithString:(NSString *)string]; }

DecimalNumberRef DecimalNumberWithStringAndLocale( CFStringRef string, CFLocaleRef locale )
{ return [NSDecimalNumber decimalNumberWithString:(NSString *)string locale:(NSLocale *)locale]; }

DecimalNumberRef DecimalNumberOne( void )
{ return [NSDecimalNumber one]; }

DecimalNumberRef DecimalNumberZero( void )
{ return [NSDecimalNumber zero]; }

DecimalNumberRef DecimalNumberNotANumber( void )
{ return [NSDecimalNumber notANumber]; }


// - perform arithmetic -
DecimalNumberRef DecimalNumberByAdding( DecimalNumberRef dn1, DecimalNumberRef dn2 )
{ return [dn1 decimalNumberByAdding:dn2]; }

DecimalNumberRef DecimalNumberBySubtracting( DecimalNumberRef dn1, DecimalNumberRef dn2 )
{ return [dn1 decimalNumberBySubtracting:dn2]; }

DecimalNumberRef DecimalNumberByMultiplyingBy( DecimalNumberRef dn1, DecimalNumberRef dn2 )
{ return [dn1 decimalNumberByMultiplyingBy:dn2]; }

DecimalNumberRef DecimalNumberByDividingBy( DecimalNumberRef dn1, DecimalNumberRef dn2 )
{ return [dn1 decimalNumberByDividingBy:dn2]; }

DecimalNumberRef DecimalNumberByRaisingToPower( DecimalNumberRef dn, NSUInteger power )
{ return [dn decimalNumberByRaisingToPower:power]; }

DecimalNumberRef DecimalNumberByMultiplyingByPowerOf10( DecimalNumberRef dn, short power )
{ return [dn decimalNumberByMultiplyingByPowerOf10:power]; }

DecimalNumberRef DecimalNumberByAddingWithBehavior( DecimalNumberRef dn1, DecimalNumberRef dn2, DecimalNumberBehaviorsRef behavior )
{ return [dn1 decimalNumberByAdding:dn2 withBehavior:behavior]; }

DecimalNumberRef DecimalNumberBySubtractingWithBehavior( DecimalNumberRef dn1, DecimalNumberRef dn2, DecimalNumberBehaviorsRef behavior )
{ return [dn1 decimalNumberBySubtracting:dn2 withBehavior:behavior]; }

DecimalNumberRef DecimalNumberByMultiplyingByWithBehavior( DecimalNumberRef dn1, DecimalNumberRef dn2, DecimalNumberBehaviorsRef behavior )
{ return [dn1 decimalNumberByMultiplyingBy:dn2 withBehavior:behavior]; }

DecimalNumberRef DecimalNumberByDividingByWithBehavior( DecimalNumberRef dn1, DecimalNumberRef dn2, DecimalNumberBehaviorsRef behavior )
{ return [dn1 decimalNumberByDividingBy:dn2 withBehavior:behavior]; }

DecimalNumberRef DecimalNumberByRaisingToPowerWithBehavior( DecimalNumberRef dn, NSUInteger power, DecimalNumberBehaviorsRef behavior )
{ return [dn decimalNumberByRaisingToPower:power withBehavior:behavior]; }

DecimalNumberRef DecimalNumberByMultiplyingByPowerOf10WithBehavior( DecimalNumberRef dn, short power, DecimalNumberBehaviorsRef behavior )
{ return [dn decimalNumberByMultiplyingByPowerOf10:power withBehavior:behavior]; }


// - rounding -
DecimalNumberRef DecimalNumberByRounding( DecimalNumberRef dn, DecimalNumberBehaviorsRef behavior )
{ return [dn decimalNumberByRoundingAccordingToBehavior:behavior]; }

// - behavior -
DecimalNumberBehaviorsRef DecimalNumberDefaultBehavior( void )
{ return [NSDecimalNumber defaultBehavior]; }

// - value -
NSDecimal DecimalNumberDecimalValue( DecimalNumberRef dn )
{ return [dn decimalValue]; }

double DecimalNumberDoubleValue( DecimalNumberRef dn )
{ return [dn doubleValue]; }

CFStringRef DecimalNumberDescription( DecimalNumberRef dn, CFLocaleRef locale )
{ return (CFStringRef)[dn descriptionWithLocale:(NSLocale *)locale]; }

// - compare -
NSComparisonResult DecimalNumberCompare( DecimalNumberRef dn1, DecimalNumberRef dn2 )
{ return [dn1 compare:dn2]; }

// - min/max values -
DecimalNumberRef DecimalNumberMaximum( void )
{ return [NSDecimalNumber maximumDecimalNumber]; }

DecimalNumberRef DecimalNumberMinimum( void )
{ return [NSDecimalNumber minimumDecimalNumber]; }






#pragma mark - decimalnumberbehaviors
///////////////////////////////////////////////////
//          decimalnumberbehaviors               //
///////////////////////////////////////////////////
// - rounding -
NSRoundingMode DecimalNumberBehaviorsRoundingMode( DecimalNumberBehaviorsRef ref )
{ return [ref roundingMode]; }

short DecimalNumberBehaviorsScale( DecimalNumberBehaviorsRef ref )
{ return [ref scale]; }







#pragma mark - decimalnumberhandler
///////////////////////////////////////////////////
//           decimalnumberhandler                //
///////////////////////////////////////////////////
// - create -
DecimalNumberHandlerRef DecimalNumberHandlerDefault( void )
{ return [NSDecimalNumberHandler defaultDecimalNumberHandler]; }

DecimalNumberHandlerRef DecimalNumberHandlerWithRoundingMode( NSRoundingMode mode, short scale, Boolean raiseOnExactness, Boolean raiseOnOverflow, Boolean raiseOnUnderflow, Boolean raiseOnDivideByZero )
{ return [NSDecimalNumberHandler decimalNumberHandlerWithRoundingMode:mode scale:scale raiseOnExactness:raiseOnExactness raiseOnOverflow:raiseOnOverflow raiseOnUnderflow:raiseOnUnderflow raiseOnDivideByZero:raiseOnDivideByZero]; }

