/*
 Dictionary.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef BOOL (*CocoaDictionaryEnumerateKeysAndObjectsCallbackType)(CFDictionaryRef,CFStringRef,CFTypeRef,BOOL*,void*);
typedef NSComparisonResult (*CocoaDictionaryKeysSortedByValueComparatorType)(CFDictionaryRef,id,id,void*);
typedef BOOL (*CocoaDictionaryKeysOfEntriesPassingTestType)(CFDictionaryRef,id,id,BOOL*,void*);

CFMutableDictionaryRef DictionaryWithCapacity( NSUInteger numItems );
CFDictionaryRef DictionaryWithContentsOfURL( CFURLRef url, ErrorRef *err );
CFDictionaryRef DictionaryWithDictionary( CFDictionaryRef dict );
CFDictionaryRef DictionaryWithObject( CFTypeRef obj, CFStringRef key );
CFDictionaryRef DictionaryWithObjects( CFTypeRef obj, ... );
Boolean DictionaryWriteToURL( CFDictionaryRef dict, CFURLRef url, ErrorRef *err );

CFDictionaryRef DictionaryWithObjectsForKeys( CFArrayRef objects, CFArrayRef keys );
NSUInteger DictionaryCount( CFDictionaryRef dict );
Boolean DictionaryIsEqual( CFDictionaryRef dict, CFDictionaryRef otherDict );
CFArrayRef DictionaryAllKeys( CFDictionaryRef dict );
CFArrayRef DictionaryAllKeysForObject( CFDictionaryRef dict, CFTypeRef obj );
CFArrayRef DictionaryAllValues( CFDictionaryRef dict );
CFTypeRef DictionaryObjectForKey( CFDictionaryRef dict, CFStringRef key );
CFArrayRef DictionaryObjectsForKeys( CFDictionaryRef dict, CFArrayRef keys, CFTypeRef notFoundMarker );

// - enumerator -
EnumeratorRef DictionaryKeyEnumerator( CFDictionaryRef dict );
EnumeratorRef DictionaryObjectEnumerator( CFDictionaryRef dict );
void DictionaryEnumerateKeysAndObjects( CFDictionaryRef dict, CocoaDictionaryEnumerateKeysAndObjectsCallbackType callback, void *userData );
void DictionaryEnumerateKeysAndObjectsWithOptions( CFDictionaryRef dict, NSEnumerationOptions options, CocoaDictionaryEnumerateKeysAndObjectsCallbackType callback, void *userData );

// sorting
CFArrayRef DictionaryKeysSortedByValueUsingSelector( CFDictionaryRef dict, CFStringRef selector );
CFArrayRef DictionaryKeysSortedByValueUsingComparator( CFDictionaryRef dict, CocoaDictionaryKeysSortedByValueComparatorType comparator, void *userData );
CFArrayRef DictionaryKeysSortedByValueWithOptionsUsingComparator( CFDictionaryRef dict, NSSortOptions options, CocoaDictionaryKeysSortedByValueComparatorType comparator, void *userData );

// Filtering
CFSetRef DictionaryKeysOfEntriesPassingTest( CFDictionaryRef dict, CocoaDictionaryKeysOfEntriesPassingTestType callback, void *userData );
CFSetRef DictionaryKeysWithOptionsOfEntriesPassingTest( CFDictionaryRef dict, NSEnumerationOptions options, CocoaDictionaryKeysOfEntriesPassingTestType callback, void *userData );

// - file attributes -
UInt64 DictionaryFileSize( CFDictionaryRef dict );
CFStringRef DictionaryFileType( CFDictionaryRef dict );
CFDateRef DictionaryFileCreationDate( CFDictionaryRef dict );
CFDateRef DictionaryFileModificationDate( CFDictionaryRef dict );
NSUInteger DictionaryFilePosixPermissions( CFDictionaryRef dict );
CFNumberRef DictionaryFileOwnerAccountID( CFDictionaryRef dict );
CFStringRef DictionaryFileOwnerAccountName( CFDictionaryRef dict );
CFNumberRef DictionaryFileGroupOwnerAccountID( CFDictionaryRef dict );
CFStringRef DictionaryFileGroupOwnerAccountName( CFDictionaryRef dict );
Boolean DictionaryFileExtensionHidden( CFDictionaryRef dict );
Boolean DictionaryFileIsImmutable( CFDictionaryRef dict );
Boolean DictionaryFileIsAppendOnly( CFDictionaryRef dict );
NSUInteger DictionaryFileSystemFileNumber( CFDictionaryRef dict );
OSType DictionaryFileHFSTypeCode( CFDictionaryRef dict );
OSType DictionaryFileHFSCreatorCode( CFDictionaryRef dict );

// Description
CFStringRef DictionaryDescription( CFDictionaryRef dict );
CFStringRef DictionaryDescriptionInStringsFileFormat( CFDictionaryRef dict );
CFStringRef DictionaryDescriptionWithLocale( CFDictionaryRef dict, CFLocaleRef locale );
CFStringRef DictionaryDescriptionWithLocaleIndent( CFDictionaryRef dict, CFLocaleRef locale, NSUInteger indent );

void DictionarySetObject( CFMutableDictionaryRef dict, CFTypeRef obj, CFStringRef key );
void DictionaryAddEntriesFromDictionary( CFMutableDictionaryRef dict, CFDictionaryRef otherDict );
void DictionarySetDictionary( CFMutableDictionaryRef dict, CFDictionaryRef otherDict );
void DictionaryRemoveObjectForKey( CFMutableDictionaryRef dict, CFStringRef key );
void DictionaryRemoveAllObjects( CFMutableDictionaryRef dict );
void DictionaryRemoveObjectsForKeys( CFMutableDictionaryRef dict, CFArrayRef keys );
