/*
 Dictionary.m

 Bernie Wylde
 */

#import "Dictionary.h"

CFMutableDictionaryRef DictionaryWithCapacity( NSUInteger numItems )
{ return (CFMutableDictionaryRef)[NSMutableDictionary dictionaryWithCapacity:numItems]; }

CFDictionaryRef DictionaryWithContentsOfURL( CFURLRef url, ErrorRef *err )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
    return (CFDictionaryRef)[NSDictionary dictionaryWithContentsOfURL:(NSURL *)url error:err];
#else
    return (CFDictionaryRef)[NSDictionary dictionaryWithContentsOfURL:(NSURL *)url];
#endif
}

CFDictionaryRef DictionaryWithDictionary( CFDictionaryRef dict )
{ return (CFDictionaryRef)[NSDictionary dictionaryWithDictionary:(NSDictionary *)dict]; }

CFDictionaryRef DictionaryWithObject( CFTypeRef obj, CFStringRef key )
{ return (CFDictionaryRef)[NSDictionary dictionaryWithObject:(id)obj forKey:(NSString *)key]; }

CFDictionaryRef DictionaryWithObjects( CFTypeRef obj, ... )
{
    va_list ap;
    va_start( ap, obj );
    NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:0];
    while ( obj ) {
        NSString *key = va_arg( ap, id );
        if ( key == NULL ) break;
        [dict setObject:obj forKey:key];
        obj = va_arg( ap, id );
    }
    va_end( ap );
    return (CFDictionaryRef)dict;
}

Boolean DictionaryWriteToURL( CFDictionaryRef dict, CFURLRef url, ErrorRef *err )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
    return [(NSDictionary *)dict writeToURL:(NSURL *)url error:err];
#else
    return [(NSDictionary *)dict writeToURL:(NSURL *)url atomically:YES];
#endif
}

CFDictionaryRef DictionaryWithObjectsForKeys( CFArrayRef objects, CFArrayRef keys )
{ return (CFDictionaryRef)[NSDictionary dictionaryWithObjects:(NSArray *)objects forKeys:(NSArray *)keys]; }

NSUInteger DictionaryCount( CFDictionaryRef dict )
{ return [(NSDictionary *)dict count]; }

Boolean DictionaryIsEqual( CFDictionaryRef dict, CFDictionaryRef otherDict )
{ return [(NSDictionary *)dict isEqualToDictionary:(NSDictionary *)otherDict]; }

CFArrayRef DictionaryAllKeys( CFDictionaryRef dict )
{ return (CFArrayRef)[(NSDictionary *)dict allKeys]; }

CFArrayRef DictionaryAllKeysForObject( CFDictionaryRef dict, CFTypeRef obj )
{ return (CFArrayRef)[(NSDictionary *)dict allKeysForObject:(id)obj]; }

CFArrayRef DictionaryAllValues( CFDictionaryRef dict )
{ return (CFArrayRef)[(NSDictionary *)dict allValues]; }

CFTypeRef DictionaryObjectForKey( CFDictionaryRef dict, CFStringRef key )
{ return (CFTypeRef)[(NSDictionary *)dict objectForKey:(NSString *)key]; }

CFArrayRef DictionaryObjectsForKeys( CFDictionaryRef dict, CFArrayRef keys, CFTypeRef notFoundMarker )
{ return (CFArrayRef)[(NSDictionary *)dict objectsForKeys:(NSArray *)keys notFoundMarker:(id)notFoundMarker]; }

// - enumerator -
EnumeratorRef DictionaryKeyEnumerator( CFDictionaryRef dict )
{ return [(NSDictionary *)dict keyEnumerator]; }

EnumeratorRef DictionaryObjectEnumerator( CFDictionaryRef dict )
{ return [(NSDictionary *)dict objectEnumerator]; }

void DictionaryEnumerateKeysAndObjects( CFDictionaryRef dict, CocoaDictionaryEnumerateKeysAndObjectsCallbackType callback, void *userData )
{
    [(NSDictionary *)dict enumerateKeysAndObjectsUsingBlock:^(NSString *key, id obj, BOOL *stop) {
        (*callback)(dict,(CFStringRef)key,(CFTypeRef)obj,stop,userData);
    }];
}

void DictionaryEnumerateKeysAndObjectsWithOptions( CFDictionaryRef dict, NSEnumerationOptions options, CocoaDictionaryEnumerateKeysAndObjectsCallbackType callback, void *userData )
{
    [(NSDictionary *)dict enumerateKeysAndObjectsWithOptions:options usingBlock:^(NSString *key, id obj, BOOL *stop) {
        (*callback)(dict,(CFStringRef)key,(CFTypeRef)obj,stop,userData);
    }];
}

// sorting
CFArrayRef DictionaryKeysSortedByValueUsingSelector( CFDictionaryRef dict, CFStringRef selector )
{ return (CFArrayRef)[(NSDictionary *)dict keysSortedByValueUsingSelector:NSSelectorFromString((NSString *)selector)]; }

CFArrayRef DictionaryKeysSortedByValueUsingComparator( CFDictionaryRef dict, CocoaDictionaryKeysSortedByValueComparatorType comparator, void *userData )
{
    return (CFArrayRef)[(NSDictionary *)dict keysSortedByValueUsingComparator:^(id obj1, id obj2) {
        return (*comparator)(dict,obj1,obj2,userData);
    }];
}

CFArrayRef DictionaryKeysSortedByValueWithOptionsUsingComparator( CFDictionaryRef dict, NSSortOptions options, CocoaDictionaryKeysSortedByValueComparatorType comparator, void *userData )
{
    return (CFArrayRef)[(NSDictionary *)dict keysSortedByValueWithOptions:options usingComparator:^(id obj1, id obj2) {
        return (*comparator)(dict,obj1,obj2,userData);
    }];
}

// Filtering
CFSetRef DictionaryKeysOfEntriesPassingTest( CFDictionaryRef dict, CocoaDictionaryKeysOfEntriesPassingTestType callback, void *userData )
{
    return (CFSetRef)[(NSDictionary *)dict keysOfEntriesPassingTest:^(id key, id obj, BOOL *stop) {
        return (*callback)(dict,key,obj,stop,userData);
    }];
}

CFSetRef DictionaryKeysWithOptionsOfEntriesPassingTest( CFDictionaryRef dict, NSEnumerationOptions options, CocoaDictionaryKeysOfEntriesPassingTestType callback, void *userData )
{
    return (CFSetRef)[(NSDictionary *)dict keysOfEntriesWithOptions:options passingTest:^(id key, id obj, BOOL *stop) {
        return (*callback)(dict,key,obj,stop,userData);
    }];
}

// - file attributes -
UInt64 DictionaryFileSize( CFDictionaryRef dict )
{ return [(NSDictionary *)dict fileSize]; }

CFStringRef DictionaryFileType( CFDictionaryRef dict )
{ return (CFStringRef)[(NSDictionary *)dict fileType]; }

CFDateRef DictionaryFileCreationDate( CFDictionaryRef dict )
{ return (CFDateRef)[(NSDictionary *)dict fileCreationDate]; }

CFDateRef DictionaryFileModificationDate( CFDictionaryRef dict )
{ return (CFDateRef)[(NSDictionary *)dict fileModificationDate]; }

NSUInteger DictionaryFilePosixPermissions( CFDictionaryRef dict )
{ return [(NSDictionary *)dict filePosixPermissions]; }

CFNumberRef DictionaryFileOwnerAccountID( CFDictionaryRef dict )
{ return (CFNumberRef)[(NSDictionary *)dict fileOwnerAccountID]; }

CFStringRef DictionaryFileOwnerAccountName( CFDictionaryRef dict )
{ return (CFStringRef)[(NSDictionary *)dict fileOwnerAccountName]; }

CFNumberRef DictionaryFileGroupOwnerAccountID( CFDictionaryRef dict )
{ return (CFNumberRef)[(NSDictionary *)dict fileGroupOwnerAccountID]; }

CFStringRef DictionaryFileGroupOwnerAccountName( CFDictionaryRef dict )
{ return (CFStringRef)[(NSDictionary *)dict fileGroupOwnerAccountName]; }

Boolean DictionaryFileExtensionHidden( CFDictionaryRef dict )
{ return [(NSDictionary *)dict fileExtensionHidden]; }

Boolean DictionaryFileIsImmutable( CFDictionaryRef dict )
{ return [(NSDictionary *)dict fileIsImmutable]; }

Boolean DictionaryFileIsAppendOnly( CFDictionaryRef dict )
{ return [(NSDictionary *)dict fileIsAppendOnly]; }

NSUInteger DictionaryFileSystemFileNumber( CFDictionaryRef dict )
{ return [(NSDictionary *)dict fileSystemFileNumber]; }

OSType DictionaryFileHFSTypeCode( CFDictionaryRef dict )
{ return [(NSDictionary *)dict fileHFSTypeCode]; }

OSType DictionaryFileHFSCreatorCode( CFDictionaryRef dict )
{ return [(NSDictionary *)dict fileHFSCreatorCode]; }

// Description
CFStringRef DictionaryDescription( CFDictionaryRef dict )
{ return (CFStringRef)[(NSDictionary *)dict description]; }

CFStringRef DictionaryDescriptionInStringsFileFormat( CFDictionaryRef dict )
{ return (CFStringRef)[(NSDictionary *)dict descriptionInStringsFileFormat]; }

CFStringRef DictionaryDescriptionWithLocale( CFDictionaryRef dict, CFLocaleRef locale )
{ return (CFStringRef)[(NSDictionary *)dict descriptionWithLocale:(id)locale]; }

CFStringRef DictionaryDescriptionWithLocaleIndent( CFDictionaryRef dict, CFLocaleRef locale, NSUInteger indent )
{ return (CFStringRef)[(NSDictionary *)dict descriptionWithLocale:(id)locale indent:indent]; }


void DictionarySetObject( CFMutableDictionaryRef dict, CFTypeRef obj, CFStringRef key )
{ [(NSMutableDictionary *)dict setObject:(id)obj forKey:(NSString *)key]; }

void DictionaryAddEntriesFromDictionary( CFMutableDictionaryRef dict, CFDictionaryRef otherDict )
{ [(NSMutableDictionary *)dict addEntriesFromDictionary:(NSDictionary *)otherDict]; }

void DictionarySetDictionary( CFMutableDictionaryRef dict, CFDictionaryRef otherDict )
{ [(NSMutableDictionary *)dict setDictionary:(NSDictionary *)otherDict]; }

void DictionaryRemoveObjectForKey( CFMutableDictionaryRef dict, CFStringRef key )
{ [(NSMutableDictionary *)dict removeObjectForKey:(NSString *)key]; }

void DictionaryRemoveAllObjects( CFMutableDictionaryRef dict )
{ [(NSMutableDictionary *)dict removeAllObjects]; }

void DictionaryRemoveObjectsForKeys( CFMutableDictionaryRef dict, CFArrayRef keys )
{ [(NSMutableDictionary *)dict removeObjectsForKeys:(NSArray *)keys]; }

