/*
 DistributedNotificationCenter.m

 Bernie Wylde
 */

#import "DistributedNotificationCenter.h"

static NSMutableArray *sCocoaDistributedNotificationCenters;

@implementation CocoaDistributedNotificationCenter : NSObject
@synthesize callback;
@synthesize name;

- (id)initWithCallback:(CocoaDistributedNotificationCenterCallbackType)cb name:(NSString *)nm {
    if ( self = [super init] ) {
        self.callback = cb;
        self.name = nm;
    }
    return self;
}

- (void)didReceiveNotification:(NSNotification *)notification {
    if ( callback ) (*callback)(notification);
}
@end

// -- private --
NSMutableArray *CocoaDistributedNotificationCenters( void )
{
    if ( !sCocoaDistributedNotificationCenters ) sCocoaDistributedNotificationCenters = [[NSMutableArray alloc] initWithCapacity:0];
    return sCocoaDistributedNotificationCenters;
}

// -- public --

// - default center -
DistributedNotificationCenterRef DistributedNotificationCenterDefaultCenter( void )
{ return [NSDistributedNotificationCenter defaultCenter]; }

DistributedNotificationCenterRef DistributedNotificationCenterForType( CFStringRef type )
{ return [NSDistributedNotificationCenter notificationCenterForType:(NSString *)type]; }

// - observers -
void DistributedNotificationCenterAddObserver( void *callback, CFStringRef name, CFTypeRef object, NSNotificationSuspensionBehavior behavior )
{
    NSMutableArray *centers = CocoaDistributedNotificationCenters();
    CocoaDistributedNotificationCenter *center = [[CocoaDistributedNotificationCenter alloc] initWithCallback:callback name:(NSString *)name];
    [centers addObject:center];
    [[NSDistributedNotificationCenter defaultCenter] addObserver:center selector:@selector(didReceiveNotification:) name:(NSString *)name object:(id)object];
}

void DistributedNotificationCenterRemoveObserver( void *callback, CFStringRef name )
{
    NSMutableArray *centers = CocoaDistributedNotificationCenters();
    for ( CocoaDistributedNotificationCenter *center in centers ) {
        if ( (center.callback == callback) && ([[center name] isEqualToString:(NSString *)name]) ) {
            [[NSDistributedNotificationCenter defaultCenter] removeObserver:center name:(NSString *)name object:nil];
            [centers removeObject:center];
            break;
        }
    }
}

// - notifications -
void DistributedNotificationCenterPostNotificationName( CFStringRef name, CFTypeRef object, CFDictionaryRef userInfo, NSUInteger options )
{ [[NSDistributedNotificationCenter defaultCenter] postNotificationName:(NSString *)name object:(id)object userInfo:(NSDictionary *)userInfo options:options]; }

// - suspend -
Boolean DistributedNotificationCenterSuspended( DistributedNotificationCenterRef ref )
{ return [ref suspended]; }

void DistributedNotificationCenterSetSuspended( DistributedNotificationCenterRef ref, Boolean flag )
{ [ref setSuspended:flag]; }


