/*
 DraggingInfo.m

 Bernie Wylde
 */

#import "DraggingInfo.h"

// Dragging session info
CocoaPasteboardRef DraggingInfoPasteboard( DraggingInfoRef info )
{ return [info draggingPasteboard]; }

NSInteger DraggingInfoSequenceNumber( DraggingInfoRef info )
{ return [info draggingSequenceNumber]; }

CFTypeRef DraggingInfoSource( DraggingInfoRef info )
{ return [info draggingSource]; }

NSDragOperation DraggingInfoSourceOperationMask( DraggingInfoRef info )
{ return [info draggingSourceOperationMask]; }

CGPoint DraggingInfoLocation( DraggingInfoRef info )
{ return NSPointToCGPoint([info draggingLocation]); }

NSInteger DraggingInfoDestinationWindow( DraggingInfoRef info )
{ return [[info draggingDestinationWindow] tag]; }

NSInteger DraggingInfoNumberOfValidItemsForDrop( DraggingInfoRef info )
{ return [info numberOfValidItemsForDrop]; }

// Image info
CGPoint DraggingInfoImageLocation( DraggingInfoRef info )
{ return NSPointToCGPoint([info draggedImageLocation]); }

// Sliding the image
void DraggingInfoSlideImageTo( DraggingInfoRef info, CGPoint screenPt )
{ [info slideDraggedImageTo:NSPointFromCGPoint(screenPt)]; }

Boolean DraggingInfoAnimatesToDestination( DraggingInfoRef info )
{ return [info animatesToDestination]; }

void DraggingInfoSetAnimatesToDestination( DraggingInfoRef info, Boolean flag )
{ [info setAnimatesToDestination:flag]; }

NSDraggingFormation DraggingInfoFormation( DraggingInfoRef info )
{ return [info draggingFormation]; }

// Enumerate
void DraggingInfoEnumerateItems( DraggingInfoRef info, NSDraggingItemEnumerationOptions options, NSInteger forViewTag, CFArrayRef classes, CFDictionaryRef searchOptions, CocoaDraggingInfoEnumerateItemsCallbackType callback, void *userData )
{
    NSView *forView = CocoaViewWithTag( forViewTag );
    [info enumerateDraggingItemsWithOptions:options forView:forView classes:(NSArray *)classes searchOptions:(NSDictionary *)searchOptions usingBlock:^(NSDraggingItem *draggingItem, NSInteger idx, BOOL *stop) {
        BOOL stopFlag = NO;
        if ( callback ) stopFlag = (*callback)( draggingItem, idx, userData );
        if ( stopFlag ) *stop = YES;
    }];
}

// Spring-loading support
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSSpringLoadingHighlight DraggingInfoSpringLoadingHighlight( DraggingInfoRef info )
{ return [info springLoadingHighlight]; }

void DraggingInfoResetSpringLoading( DraggingInfoRef info )
{ [info resetSpringLoading]; }
#endif
