/*
 Event.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

@interface CocoaEvent : NSObject {
    NSEvent *nsEvent;
}

@property (assign) NSEvent* nsEvent;
@end

CocoaEvent *CocoaEventObj( void );
void CocoaEventSetEvent( NSEvent *event );

// Create
CocoaEventRef EventKeyEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, GraphicsContextRef ctx, CFStringRef chars, CFStringRef unmodChars, Boolean isRepeat, UInt16 code );
CocoaEventRef EventMouseEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, GraphicsContextRef ctx, NSInteger evNum, NSInteger clickCount, float pressure );
CocoaEventRef EventEnterExitEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, GraphicsContextRef ctx, NSInteger evNum, NSInteger trackingNum, void *userData );
CocoaEventRef EventOtherEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, GraphicsContextRef ctx, SInt16 subtype, NSInteger data1, NSInteger data2 );
CocoaEventRef EventOtherEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, GraphicsContextRef ctx, SInt16 subtype, NSInteger data1, NSInteger data2 );

// Type
NSEventType eventType( void );// lowercase 'e' to avoid conflict with Carbon's 'EventType'

// General information
CGPoint EventLocationInWindow( void );
CFTimeInterval EventTimestamp( void );
void *EventWindow( void );

// Key info
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSEventModifierFlags EventModifierFlags( void );
#else
NSUInteger EventModifierFlags( void );
#endif
CFTimeInterval EventKeyRepeatDelay( void );
CFTimeInterval EventKeyRepeatInterval( void );
CFStringRef EventCharacters( void );
CFStringRef EventCharactersIgnoringModifiers( void );
Boolean EventIsARepeat( void );
unsigned short EventKeyCode( void );

// Mouse info
NSUInteger EventPressedMouseButtons( void );
CFTimeInterval EventDoubleClickInterval( void );
CGPoint EventMouseLocation( void );
NSInteger EventButtonNumber( CocoaEventRef evnt );
NSInteger EventClickCount( void );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
NSEventMask EventAssociatedEventsMask( CocoaEventRef evnt );
#endif

// Mouse tracking info
NSInteger EventNumber( CocoaEventRef evnt );
NSInteger EventTrackingNumber( CocoaEventRef evnt );
TrackingAreaRef EventTrackingArea( CocoaEventRef evnt );
void *EventUserData( CocoaEventRef evnt );

// Custom event info
NSInteger EventData1( CocoaEventRef evnt );
NSInteger EventData2( CocoaEventRef evnt );

// Scroll wheel info
CGFloat EventDeltaX( CocoaEventRef evnt );
CGFloat EventDeltaY( CocoaEventRef evnt );
CGFloat EventDeltaZ( CocoaEventRef evnt );

// Pressure info
float EventPressure( CocoaEventRef evnt );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
NSInteger EventStage( CocoaEventRef evnt );
CGFloat EventStageTransition( CocoaEventRef evnt );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSPressureBehavior EventPressureBehavior( CocoaEventRef evnt );
#endif

// Tablet proximity info
NSUInteger EventCapabilityMask( CocoaEventRef evnt );
NSUInteger EventDeviceID( CocoaEventRef evnt );
Boolean EventIsEnteringProximity( CocoaEventRef evnt );
NSUInteger EventPointingDeviceID( CocoaEventRef evnt );
NSUInteger EventPointingDeviceSerialNumber( CocoaEventRef evnt );
NSPointingDeviceType EventPointingDeviceType( CocoaEventRef evnt );
NSUInteger EventSystemTabletID( CocoaEventRef evnt );
NSUInteger EventTabletID( CocoaEventRef evnt );
UInt64 EventUniqueID( CocoaEventRef evnt );
NSUInteger EventVendorID( CocoaEventRef evnt );
NSUInteger EventVendorPointingDeviceType( CocoaEventRef evnt );

// Tablet pointing info
NSInteger EventAbsoluteX( CocoaEventRef evnt );
NSInteger EventAbsoluteY( CocoaEventRef evnt );
NSInteger EventAbsoluteZ( CocoaEventRef evnt );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSEventButtonMask EventButtonMask( CocoaEventRef evnt );
#endif
float EventRotation( CocoaEventRef evnt );
float EventTangentialPressure( CocoaEventRef evnt );
CGPoint EventTilt( CocoaEventRef evnt );
CFTypeRef EventVendorDefined( CocoaEventRef evnt );

// Periodic events
void EventStartPeriodicEvents( CFTimeInterval delay, CFTimeInterval period );
void EventStopPeriodicEvents( void );

// Touch and gesture info
CGFloat EventMagnification( CocoaEventRef evnt );
CFSetRef EventTouchesMatchingPhase( CocoaEventRef evnt, NSTouchPhase phase, NSInteger vwTag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFSetRef EventAllTouches( CocoaEventRef evnt );
CFSetRef EventTouchesForView( CocoaEventRef evnt, NSInteger vwTag );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
CFArrayRef EventCoalescedTouchesForTouch( CocoaEventRef evnt, TouchRef touch );
#endif
Boolean EventIsMouseCoalescingEnabled( void );
void EventSetMouseCoalescingEnabled( Boolean flag );

// Monitoring app events

// Scroll wheel and flick events
Boolean EventHasPreciseScrollingDeltas( CocoaEventRef evnt );
CGFloat EventScrollingDeltaX( CocoaEventRef evnt );
CGFloat EventScrollingDeltaY( CocoaEventRef evnt );
NSEventPhase EventMomentumPhase( CocoaEventRef evnt );
NSEventPhase EventPhase( CocoaEventRef evnt );
Boolean EventIsSwipeTrackingFromScrollEventsEnabled( void );
//void EventTrackSwipeEvent( CocoaEventRef evnt, NSEventSwipeTrackingOptions options, CGFloat dampenThresholdMin, CGFloat dampenThresholdMin, void *handler );
Boolean EventIsDirectionInvertedFromDevice( CocoaEventRef evnt );

// Convert mouse event into sprite kit node coord space
//CGPoint EventLocationInNode( SKNodeRef node );


// custom
CGPoint EventLocationInView( NSInteger tag );

