/*
 Event.m

 Bernie Wylde
 */

#import "Event.h"

@implementation CocoaEvent
@synthesize nsEvent;
@end

// private
CocoaEvent *CocoaEventObj( void )
{
    static CocoaEvent *sCocoaEvent = nil;
    if ( !sCocoaEvent ) sCocoaEvent = [[CocoaEvent alloc] init];
    return sCocoaEvent;
}

void CocoaEventSetEvent( NSEvent *event )
{ [CocoaEventObj() setNsEvent:event]; }


// public
// Create
CocoaEventRef EventKeyEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, GraphicsContextRef ctx, CFStringRef chars, CFStringRef unmodChars, Boolean isRepeat, UInt16 code )
{ return [NSEvent keyEventWithType:type location:NSPointFromCGPoint(location) modifierFlags:modifierFlags timestamp:timestamp windowNumber:wndNum context:ctx characters:(NSString *)chars charactersIgnoringModifiers:(NSString *)unmodChars isARepeat:isRepeat keyCode:code]; }

CocoaEventRef EventMouseEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, GraphicsContextRef ctx, NSInteger evNum, NSInteger clickCount, float pressure )
{ return [NSEvent mouseEventWithType:type location:NSPointFromCGPoint(location) modifierFlags:modifierFlags timestamp:timestamp windowNumber:wndNum context:ctx eventNumber:evNum clickCount:clickCount pressure:pressure]; }

CocoaEventRef EventEnterExitEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, GraphicsContextRef ctx, NSInteger evNum, NSInteger trackingNum, void *userData )
{ return [NSEvent enterExitEventWithType:type location:NSPointFromCGPoint(location) modifierFlags:modifierFlags timestamp:timestamp windowNumber:wndNum context:ctx eventNumber:evNum trackingNumber:trackingNum userData:userData]; }

CocoaEventRef EventOtherEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, GraphicsContextRef ctx, SInt16 subtype, NSInteger data1, NSInteger data2 )
{ return [NSEvent otherEventWithType:type location:NSPointFromCGPoint(location) modifierFlags:modifierFlags timestamp:timestamp windowNumber:wndNum context:ctx subtype:subtype data1:data1 data2:data2]; }

// Type
NSEventType eventType( void )// lowercase 'e' to avoid conflict with Carbon's 'EventType'
{ return [[CocoaEventObj() nsEvent] type]; }

// General information
CGPoint EventLocationInWindow( void )
{ return NSPointToCGPoint([[CocoaEventObj() nsEvent] locationInWindow]); }

CFTimeInterval EventTimestamp( void )
{ return [[CocoaEventObj() nsEvent] timestamp]; }

void *EventWindow( void )
{ return [[CocoaEventObj() nsEvent] window]; }

// Key info
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSEventModifierFlags EventModifierFlags( void )
{ return [[CocoaEventObj() nsEvent] modifierFlags]; }
#else
NSUInteger EventModifierFlags( void )
{ return [[CocoaEventObj() nsEvent] modifierFlags]; }
#endif

CFTimeInterval EventKeyRepeatDelay( void )
{ return [NSEvent keyRepeatDelay]; }

CFTimeInterval EventKeyRepeatInterval( void )
{ return [NSEvent keyRepeatInterval]; }

CFStringRef EventCharacters( void )
{ return (CFStringRef)[[CocoaEventObj() nsEvent] characters]; }

CFStringRef EventCharactersIgnoringModifiers( void )
{ return (CFStringRef)[[CocoaEventObj() nsEvent] charactersIgnoringModifiers]; }

Boolean EventIsARepeat( void )
{
    NSEvent *event = [CocoaEventObj() nsEvent];
    NSEventType type = [event type];
    switch ( type ) {
        case NSKeyDown:
        case NSKeyUp:
            return [event isARepeat];
        default:
            return NO;
    }
}

unsigned short EventKeyCode( void )
{
    NSEvent *event = [CocoaEventObj() nsEvent];
    NSEventType type = [event type];
    switch ( type ) {
        case NSKeyDown:
        case NSKeyUp:
        case NSFlagsChanged:
            return [event keyCode];
        default:
            return 0;
    }
}

// Mouse info
NSUInteger EventPressedMouseButtons( void )
{ return [NSEvent pressedMouseButtons]; }

CFTimeInterval EventDoubleClickInterval( void )
{ return [NSEvent doubleClickInterval]; }

CGPoint EventMouseLocation( void )
{ return NSPointToCGPoint([NSEvent mouseLocation]); }

NSInteger EventButtonNumber( CocoaEventRef evnt )
{ return [evnt buttonNumber]; }

NSInteger EventClickCount( void )
{ return [[CocoaEventObj() nsEvent] clickCount]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
NSEventMask EventAssociatedEventsMask( CocoaEventRef evnt )
{ return [evnt associatedEventsMask]; }
#endif

// Mouse tracking info
NSInteger EventNumber( CocoaEventRef evnt )
{ return [evnt eventNumber]; }

NSInteger EventTrackingNumber( CocoaEventRef evnt )
{ return [evnt trackingNumber]; }

TrackingAreaRef EventTrackingArea( CocoaEventRef evnt )
{ return [evnt trackingArea]; }

void *EventUserData( CocoaEventRef evnt )
{ return [evnt userData]; }

// Custom event info
NSInteger EventData1( CocoaEventRef evnt )
{ return [evnt data1]; }

NSInteger EventData2( CocoaEventRef evnt )
{ return [evnt data2]; }

// Scroll wheel info
CGFloat EventDeltaX( CocoaEventRef evnt )
{ return [evnt deltaX]; }

CGFloat EventDeltaY( CocoaEventRef evnt )
{ return [evnt deltaY]; }

CGFloat EventDeltaZ( CocoaEventRef evnt )
{ return [evnt deltaZ]; }

// Pressure info
float EventPressure( CocoaEventRef evnt )
{ return [evnt pressure]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
NSInteger EventStage( CocoaEventRef evnt )
{ return [evnt stage]; }

CGFloat EventStageTransition( CocoaEventRef evnt )
{ return [evnt stageTransition]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSPressureBehavior EventPressureBehavior( CocoaEventRef evnt )
{ return [evnt pressureBehavior]; }
#endif

// Tablet proximity info
NSUInteger EventCapabilityMask( CocoaEventRef evnt )
{ return [evnt capabilityMask]; }

NSUInteger EventDeviceID( CocoaEventRef evnt )
{ return [evnt deviceID]; }

Boolean EventIsEnteringProximity( CocoaEventRef evnt )
{ return [evnt isEnteringProximity]; }

NSUInteger EventPointingDeviceID( CocoaEventRef evnt )
{ return [evnt pointingDeviceID]; }

NSUInteger EventPointingDeviceSerialNumber( CocoaEventRef evnt )
{ return [evnt pointingDeviceSerialNumber]; }

NSPointingDeviceType EventPointingDeviceType( CocoaEventRef evnt )
{ return [evnt pointingDeviceType]; }

NSUInteger EventSystemTabletID( CocoaEventRef evnt )
{ return [evnt systemTabletID]; }

NSUInteger EventTabletID( CocoaEventRef evnt )
{ return [evnt tabletID]; }

UInt64 EventUniqueID( CocoaEventRef evnt )
{ return [evnt uniqueID]; }

NSUInteger EventVendorID( CocoaEventRef evnt )
{ return [evnt vendorID]; }

NSUInteger EventVendorPointingDeviceType( CocoaEventRef evnt )
{ return [evnt vendorPointingDeviceType]; }

// Tablet pointing info
NSInteger EventAbsoluteX( CocoaEventRef evnt )
{ return [evnt absoluteX]; }

NSInteger EventAbsoluteY( CocoaEventRef evnt )
{ return [evnt absoluteY]; }

NSInteger EventAbsoluteZ( CocoaEventRef evnt )
{ return [evnt absoluteZ]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSEventButtonMask EventButtonMask( CocoaEventRef evnt )
{ return [evnt buttonMask]; }
#endif

float EventRotation( CocoaEventRef evnt )
{ return [evnt rotation]; }

float EventTangentialPressure( CocoaEventRef evnt )
{ return [evnt pressure]; }

CGPoint EventTilt( CocoaEventRef evnt )
{ return NSPointToCGPoint([evnt tilt]); }

CFTypeRef EventVendorDefined( CocoaEventRef evnt )
{ return (CFTypeRef)[evnt vendorDefined]; }

// Periodic events
void EventStartPeriodicEvents( CFTimeInterval delay, CFTimeInterval period )
{ [NSEvent startPeriodicEventsAfterDelay:delay withPeriod:period]; }

void EventStopPeriodicEvents( void )
{ [NSEvent stopPeriodicEvents]; }

// Touch and gesture info
CGFloat EventMagnification( CocoaEventRef evnt )
{ return [evnt magnification]; }

CFSetRef EventTouchesMatchingPhase( CocoaEventRef evnt, NSTouchPhase phase, NSInteger vwTag )
{
    NSView *view = CocoaViewWithTag(vwTag);
    if ( view ) {
        return (CFSetRef)[evnt touchesMatchingPhase:phase inView:view];
    }
    return nil;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFSetRef EventAllTouches( CocoaEventRef evnt )
{ return (CFSetRef)[evnt allTouches]; }

CFSetRef EventTouchesForView( CocoaEventRef evnt, NSInteger vwTag )
{
    NSView *view = CocoaViewWithTag(vwTag);
    if ( view ) {
        return (CFSetRef)[evnt touchesForView:view];
    }
    return nil;
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
CFArrayRef EventCoalescedTouchesForTouch( CocoaEventRef evnt, TouchRef touch )
{ return (CFArrayRef)[evnt coalescedTouchesForTouch:touch]; }
#endif

Boolean EventIsMouseCoalescingEnabled( void )
{ return [NSEvent isMouseCoalescingEnabled]; }

void EventSetMouseCoalescingEnabled( Boolean flag )
{ [NSEvent setMouseCoalescingEnabled:flag]; }

// Monitoring app events

// Scroll wheel and flick events
Boolean EventHasPreciseScrollingDeltas( CocoaEventRef evnt )
{ return [evnt hasPreciseScrollingDeltas]; }

CGFloat EventScrollingDeltaX( CocoaEventRef evnt )
{ return [evnt scrollingDeltaX]; }

CGFloat EventScrollingDeltaY( CocoaEventRef evnt )
{ return [evnt scrollingDeltaX]; }

NSEventPhase EventMomentumPhase( CocoaEventRef evnt )
{ return [evnt momentumPhase]; }

NSEventPhase EventPhase( CocoaEventRef evnt )
{ return [evnt phase]; }

Boolean EventIsSwipeTrackingFromScrollEventsEnabled( void )
{ return [NSEvent isSwipeTrackingFromScrollEventsEnabled]; }

//void EventTrackSwipeEvent( CocoaEventRef evnt, NSEventSwipeTrackingOptions options, CGFloat dampenThresholdMin, CGFloat dampenThresholdMin, void *handler );

Boolean EventIsDirectionInvertedFromDevice( CocoaEventRef evnt )
{ return [evnt isDirectionInvertedFromDevice]; }

// Convert mouse event into sprite kit node coord space
//CGPoint EventLocationInNode( SKNodeRef node );











// custom
CGPoint EventLocationInView( NSInteger tag )
{
    NSView *view = (NSView *)CocoaViewWithTag( tag );
    if ( view ) {
        return NSPointToCGPoint([view convertPoint:[[CocoaEventObj() nsEvent] locationInWindow] fromView:nil]);
    }
    return CGPointZero;
}

