/*
 Expression.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef CFTypeRef (*CocoaExpressionForArgumentsCallbackType)(CFTypeRef,CFArrayRef,CFMutableDictionaryRef,void*);

// Init
ExpressionRef ExpressionWithType( NSExpressionType type );
ExpressionRef ExpressionWithFormat( CFStringRef format, ... );
ExpressionRef ExpressionWithFormatAndArgumentArray( CFStringRef format, CFArrayRef argArray );
ExpressionRef ExpressionWithFormatAndArguments( CFStringRef format, va_list argList );

// Create for value
ExpressionRef ExpressionForConstantValue( CFTypeRef obj );
ExpressionRef ExpressionForEvaluatedObject( void );
ExpressionRef ExpressionForKeyPath( CFStringRef keyPath );
ExpressionRef ExpressionForVariable( CFStringRef string );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
ExpressionRef ExpressionForAnyKey( void );
#endif

// Create collection
ExpressionRef ExpressionForAggregate( CFArrayRef subexpressions );
ExpressionRef ExpressionForUnionSet( ExpressionRef left, ExpressionRef right );
ExpressionRef ExpressionForIntersectSet( ExpressionRef left, ExpressionRef right );
ExpressionRef ExpressionForMinusSet( ExpressionRef left, ExpressionRef right );

// Create subquery
ExpressionRef ExpressionForSubquery( ExpressionRef exp, CFStringRef variable, PredicateRef predicate );

// Create using blocks
ExpressionRef ExpressionForArguments( CFArrayRef arguments, CocoaExpressionForArgumentsCallbackType callback, void *userData );

// Create for function
ExpressionRef ExpressionForFunction( CFStringRef name, CFArrayRef arguments );

// Info
CFArrayRef ExpressionArguments( ExpressionRef ref );
CFTypeRef ExpressionCollection( ExpressionRef ref );
CFTypeRef ExpressionConstantValue( ExpressionRef ref );
NSExpressionType ExpressionType( ExpressionRef ref );
CFStringRef ExpressionKeyPath( ExpressionRef ref );
ExpressionRef ExpressionOperand( ExpressionRef ref );
PredicateRef ExpressionPredicate( ExpressionRef ref );
ExpressionRef ExpressionLeftExpression( ExpressionRef ref );
ExpressionRef ExpressionRightExpression( ExpressionRef ref );
CFStringRef ExpressionVariable( ExpressionRef ref );

// Evaluate
CFTypeRef ExpressionValueWithObject( ExpressionRef ref, CFTypeRef obj, CFMutableDictionaryRef context );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ExpressionAllowEvaluation( ExpressionRef ref );
#endif

// Initializers
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ExpressionRef ExpressionForConditional( PredicateRef predicate, ExpressionRef trueExpression, ExpressionRef falseExpression );
#endif

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ExpressionRef ExpressionFalseExpression( ExpressionRef ref );
ExpressionRef ExpressionTrueExpression( ExpressionRef ref );
#endif
