/*
 Expression.m

 Bernie Wylde
 */

#import "Expression.h"

// Init
ExpressionRef ExpressionWithType( NSExpressionType type )
{ return [[[NSExpression alloc] initWithExpressionType:type] autorelease]; }

ExpressionRef ExpressionWithFormat( CFStringRef format, ... )
{
    va_list ap;
    va_start( ap, format );
    NSExpression *expression = [NSExpression expressionWithFormat:(NSString *)format arguments:ap];
    va_end(ap);
    return expression;
}

ExpressionRef ExpressionWithFormatAndArgumentArray( CFStringRef format, CFArrayRef argArray )
{ return [NSExpression expressionWithFormat:(NSString *)format argumentArray:(NSArray *)argArray]; }

ExpressionRef ExpressionWithFormatAndArguments( CFStringRef format, va_list argList )
{ return [NSExpression expressionWithFormat:(NSString *)format arguments:argList]; }

// Create for value
ExpressionRef ExpressionForConstantValue( CFTypeRef obj )
{ return [NSExpression expressionForConstantValue:(id)obj]; }

ExpressionRef ExpressionForEvaluatedObject( void )
{ return [NSExpression expressionForEvaluatedObject]; }

ExpressionRef ExpressionForKeyPath( CFStringRef keyPath )
{ return [NSExpression expressionForKeyPath:(NSString *)keyPath]; }

ExpressionRef ExpressionForVariable( CFStringRef string )
{ return [NSExpression expressionForVariable:(NSString *)string]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
ExpressionRef ExpressionForAnyKey( void )
{ return [NSExpression expressionForAnyKey]; }
#endif

// Create collection
ExpressionRef ExpressionForAggregate( CFArrayRef subexpressions )
{ return [NSExpression expressionForAggregate:(NSArray *)subexpressions]; }

ExpressionRef ExpressionForUnionSet( ExpressionRef left, ExpressionRef right )
{ return [NSExpression expressionForUnionSet:left with:right]; }

ExpressionRef ExpressionForIntersectSet( ExpressionRef left, ExpressionRef right )
{ return [NSExpression expressionForIntersectSet:left with:right]; }

ExpressionRef ExpressionForMinusSet( ExpressionRef left, ExpressionRef right )
{ return [NSExpression expressionForMinusSet:left with:right]; }

// Create subquery
ExpressionRef ExpressionForSubquery( ExpressionRef exp, CFStringRef variable, PredicateRef predicate )
{ return [NSExpression expressionForSubquery:exp usingIteratorVariable:(NSString *)variable predicate:predicate]; }

// Create using blocks
ExpressionRef ExpressionForArguments( CFArrayRef arguments, CocoaExpressionForArgumentsCallbackType callback, void *userData )
{
    return [NSExpression expressionForBlock:(id)^(id evaluatedObject, NSArray *expressions, NSMutableDictionary *context) {
        NSBeep();
        return (CFTypeRef)(*callback)((CFTypeRef)evaluatedObject,(CFArrayRef)expressions,(CFMutableDictionaryRef)context,userData);
    } arguments:(NSArray *)arguments];
}

// Create for function
ExpressionRef ExpressionForFunction( CFStringRef name, CFArrayRef arguments )
{ return [NSExpression expressionForFunction:(NSString *)name arguments:(NSArray *)arguments]; }

// Info
CFArrayRef ExpressionArguments( ExpressionRef ref )
{ return (CFArrayRef)[ref arguments]; }

CFTypeRef ExpressionCollection( ExpressionRef ref )
{ return (CFTypeRef)[ref collection]; }

CFTypeRef ExpressionConstantValue( ExpressionRef ref )
{ return (CFTypeRef)[ref constantValue]; }

NSExpressionType ExpressionType( ExpressionRef ref )
{ return [ref expressionType]; }

CFStringRef ExpressionKeyPath( ExpressionRef ref )
{ return (CFStringRef)[ref keyPath]; }

ExpressionRef ExpressionOperand( ExpressionRef ref )
{ return [ref operand]; }

PredicateRef ExpressionPredicate( ExpressionRef ref )
{ return [ref predicate]; }

ExpressionRef ExpressionLeftExpression( ExpressionRef ref )
{ return [ref leftExpression]; }

ExpressionRef ExpressionRightExpression( ExpressionRef ref )
{ return [ref rightExpression]; }

CFStringRef ExpressionVariable( ExpressionRef ref )
{ return (CFStringRef)[ref variable]; }

// Evaluate
CFTypeRef ExpressionValueWithObject( ExpressionRef ref, CFTypeRef obj, CFMutableDictionaryRef context )
{ return (CFTypeRef)[ref expressionValueWithObject:(id)obj context:(NSMutableDictionary *)context]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ExpressionAllowEvaluation( ExpressionRef ref )
{ [ref allowEvaluation]; }
#endif

// Initializers
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ExpressionRef ExpressionForConditional( PredicateRef predicate, ExpressionRef trueExpression, ExpressionRef falseExpression )
{ return [NSExpression expressionForConditional:predicate trueExpression:trueExpression falseExpression:falseExpression]; }
#endif

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ExpressionRef ExpressionFalseExpression( ExpressionRef ref )
{ return [ref falseExpression]; }

ExpressionRef ExpressionTrueExpression( ExpressionRef ref )
{ return [ref trueExpression]; }
#endif
