/*
 FileHandle.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef void (*CocoaFileHandleReadWriteHandlerType)(NSFileHandle *);

// - get -
FileHandleRef FileHandleForReadingFromURL( CFURLRef url, ErrorRef *err );
FileHandleRef FileHandleForWritingToURL( CFURLRef url, ErrorRef *err );
FileHandleRef FileHandleForUpdatingURL( CFURLRef url, ErrorRef *err );
FileHandleRef FileHandleWithStandardError( void );
FileHandleRef FileHandleWithStandardInput( void );
FileHandleRef FileHandleWithStandardOutput( void );
FileHandleRef FileHandleWithNullDevice( void );

// - create -
FileHandleRef FileHandleWithFileDescriptor( long fd );
FileHandleRef FileHandleWithFileDescriptorCloseOnDealloc( long fd, Boolean flag );

// - file descriptor -
int FileHandleFileDescriptor( FileHandleRef fh );

// - reading -
CFDataRef FileHandleAvailableData( FileHandleRef fh );
CFDataRef FileHandleReadDataToEndOfFile( FileHandleRef fh );
CFDataRef FileHandleReadDataOfLength( FileHandleRef fh, NSUInteger length );

// - writing -
void FileHandleWriteData( FileHandleRef fh, CFDataRef dta );

// - reading/writing using blocks
void FileHandleSetReadabilityHandler( FileHandleRef fh, CocoaFileHandleReadWriteHandlerType handlerFn );
void FileHandleSetWriteabilityHandler( FileHandleRef fh, CocoaFileHandleReadWriteHandlerType handlerFn );

// - communicating asynchronously -
void FileHandleAcceptConnectionInBackgroundAndNotify( FileHandleRef fh );
void FileHandleAcceptConnectionInBackgroundAndNotifyForModes( FileHandleRef fh, CFArrayRef modes );
void FileHandleReadInBackgroundAndNotify( FileHandleRef fh );
void FileHandleReadInBackgroundAndNotifyForModes( FileHandleRef fh, CFArrayRef modes );
void FileHandleReadToEndOfFileInBackgroundAndNotify( FileHandleRef fh );
void FileHandleReadToEndOfFileInBackgroundAndNotifyForModes( FileHandleRef fh, CFArrayRef modes );
void FileHandleWaitForDataInBackgroundAndNotify( FileHandleRef fh );
void FileHandleWaitForDataInBackgroundAndNotifyForModes( FileHandleRef fh, CFArrayRef modes );

// - seeking within a file -
unsigned long long FileHandleOffsetInFile( FileHandleRef fh );
unsigned long long FileHandleSeekToEndOfFile( FileHandleRef fh );
void FileHandleSeekToFileOffset( FileHandleRef fh, unsigned long long offset );

// - operating on a file -
void FileHandleCloseFile( FileHandleRef fh );
void FileHandleSynchronizeFile( FileHandleRef fh );
void FileHandleTruncateFileAtOffset( FileHandleRef fh, unsigned long long offset );

