/*
 FileHandle.m

 Bernie Wylde
 */

#import "FileHandle.h"

// - get -
FileHandleRef FileHandleForReadingFromURL( CFURLRef url, ErrorRef *err )
{ return [NSFileHandle fileHandleForReadingFromURL:(NSURL *)url error:err]; }

FileHandleRef FileHandleForWritingToURL( CFURLRef url, ErrorRef *err )
{ return [NSFileHandle fileHandleForWritingToURL:(NSURL *)url error:err]; }

FileHandleRef FileHandleForUpdatingURL( CFURLRef url, ErrorRef *err )
{ return [NSFileHandle fileHandleForUpdatingURL:(NSURL *)url error:err]; }

FileHandleRef FileHandleWithStandardError( void )
{ return [NSFileHandle fileHandleWithStandardError]; }

FileHandleRef FileHandleWithStandardInput( void )
{ return [NSFileHandle fileHandleWithStandardInput]; }

FileHandleRef FileHandleWithStandardOutput( void )
{ return [NSFileHandle fileHandleWithStandardOutput]; }

FileHandleRef FileHandleWithNullDevice( void )
{ return [NSFileHandle fileHandleWithNullDevice]; }

// - create -
FileHandleRef FileHandleWithFileDescriptor( long fd )
{ return [[[NSFileHandle alloc] initWithFileDescriptor:fd] autorelease]; }

FileHandleRef FileHandleWithFileDescriptorCloseOnDealloc( long fd, Boolean flag )
{ return [[[NSFileHandle alloc] initWithFileDescriptor:fd closeOnDealloc:flag] autorelease]; }

// - file descriptor -
int FileHandleFileDescriptor( FileHandleRef fh )
{ return [fh fileDescriptor]; }

// - reading -
CFDataRef FileHandleAvailableData( FileHandleRef fh )
{ return (CFDataRef)[fh availableData]; }

CFDataRef FileHandleReadDataToEndOfFile( FileHandleRef fh )
{ return (CFDataRef)[fh readDataToEndOfFile]; }

CFDataRef FileHandleReadDataOfLength( FileHandleRef fh, NSUInteger length )
{ return (CFDataRef)[fh readDataOfLength:length]; }

// - writing -
void FileHandleWriteData( FileHandleRef fh, CFDataRef dta )
{ [fh writeData:(NSData *)dta]; }

// - reading/writing using blocks
void FileHandleSetReadabilityHandler( FileHandleRef fh, CocoaFileHandleReadWriteHandlerType handlerFn )
{
    [fh setReadabilityHandler:^(NSFileHandle *fileHandle) {
        if ( handlerFn ) (*handlerFn)(fileHandle);
    }];
}

void FileHandleSetWriteabilityHandler( FileHandleRef fh, CocoaFileHandleReadWriteHandlerType handlerFn )
{
    [fh setWriteabilityHandler:^(NSFileHandle *fileHandle) {
        if ( handlerFn ) (*handlerFn)(fileHandle);
    }];
}

// - communicating asynchronously -
void FileHandleAcceptConnectionInBackgroundAndNotify( FileHandleRef fh )
{ [fh acceptConnectionInBackgroundAndNotify]; }

void FileHandleAcceptConnectionInBackgroundAndNotifyForModes( FileHandleRef fh, CFArrayRef modes )
{ [fh acceptConnectionInBackgroundAndNotifyForModes:(NSArray *)modes]; }

void FileHandleReadInBackgroundAndNotify( FileHandleRef fh )
{ [fh readInBackgroundAndNotify]; }

void FileHandleReadInBackgroundAndNotifyForModes( FileHandleRef fh, CFArrayRef modes )
{ [fh readInBackgroundAndNotifyForModes:(NSArray *)modes]; }

void FileHandleReadToEndOfFileInBackgroundAndNotify( FileHandleRef fh )
{ [fh readToEndOfFileInBackgroundAndNotify]; }

void FileHandleReadToEndOfFileInBackgroundAndNotifyForModes( FileHandleRef fh, CFArrayRef modes )
{ [fh readToEndOfFileInBackgroundAndNotifyForModes:(NSArray *)modes]; }

void FileHandleWaitForDataInBackgroundAndNotify( FileHandleRef fh )
{ [fh waitForDataInBackgroundAndNotify]; }

void FileHandleWaitForDataInBackgroundAndNotifyForModes( FileHandleRef fh, CFArrayRef modes )
{ [fh waitForDataInBackgroundAndNotifyForModes:(NSArray *)modes]; }

// - seeking within a file -
unsigned long long FileHandleOffsetInFile( FileHandleRef fh )
{ return [fh offsetInFile]; }

unsigned long long FileHandleSeekToEndOfFile( FileHandleRef fh )
{ return [fh seekToEndOfFile]; }

void FileHandleSeekToFileOffset( FileHandleRef fh, unsigned long long offset )
{ return [fh seekToFileOffset:offset]; }

// - operating on a file -
void FileHandleCloseFile( FileHandleRef fh )
{ [fh closeFile]; }

void FileHandleSynchronizeFile( FileHandleRef fh )
{ [fh synchronizeFile]; }

void FileHandleTruncateFileAtOffset( FileHandleRef fh, unsigned long long offset )
{ [fh truncateFileAtOffset:offset]; }

