/*
 FileManager.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// dialog events
enum {
    fileManagerShouldMoveItem = 2400,
    fileManagerShouldProceedMovingItem,
    fileManagerShouldCopyItem,
    fileManagerShouldProceedCopyingItem,
    fileManagerShouldRemoveItem,
    fileManagerShouldProceedRemovingItem,
    fileManagerShouldLinkItem,
    fileManagerShouldProceedLinkingItem
};

@interface CocoaFileManagerDelegate : NSObject <NSFileManagerDelegate>
@end

void CocoaFileManagerDelegateCreate( void );

typedef BOOL (*FileManagerEnumeratorCallbackType)(CFURLRef,ErrorRef,void *);
typedef void (*FileManagerGetFileProviderServicesHandlerType)(CFDictionaryRef,ErrorRef,void *);// macOS 10.13+
typedef void (*FileManagerUnmountVolumeHandlerType)(ErrorRef,void *);// macOS 10.11+

// Create
FileManagerRef FileManagerDefaultManager( void );

// User directories
CFURLRef FileManagerTemporaryDirectory( void );
CFURLRef FileManagerHomeDirectoryForCurrentUser( void );
CFURLRef FileManagerHomeDirectoryForUser( CFStringRef userName );

// System directories
CFURLRef FileManagerURLForDirectoryAppropriateForURL( NSSearchPathDirectory whichDirectory, NSSearchPathDomainMask domain, CFURLRef url, Boolean create, ErrorRef *err );
CFURLRef FileManagerURLForDirectory( NSSearchPathDirectory whichDirectory, NSSearchPathDomainMask domain );
CFArrayRef FileManagerURLsForDirectory( NSSearchPathDirectory whichDirectory, NSSearchPathDomainMask domain );

CFURLRef FileManagerURLForPreferencesDirectory( void );
CFURLRef FileManagerURLForApplicationDirectory( void );
CFArrayRef FileManagerContentsOfDirectoryAtURL( CFURLRef url, CFArrayRef keys, NSDirectoryEnumerationOptions options );

DirectoryEnumeratorRef FileManagerEnumeratorAtURL( CFURLRef url, CFArrayRef propertyKeys, NSDirectoryEnumerationOptions options, FileManagerEnumeratorCallbackType errorCallback, void *userData );
CFArrayRef FileManagerMountedVolumeURLs( CFArrayRef propertyKeys, NSVolumeEnumerationOptions options );

Boolean FileManagerCreateDirectoryAtURL( CFURLRef url, Boolean withIntermediateDirectories, CFDictionaryRef attributes );
Boolean FileManagerCreateFileAtURL( CFURLRef url, CFDataRef contents, CFDictionaryRef attributes );
Boolean FileManagerRemoveItemAtURL( CFURLRef url );
Boolean FileManagerReplaceItemAtURL( CFURLRef origURL, CFURLRef newURL );
void FileManagerTrashItemAtURL( CFURLRef url );
Boolean FileManagerCopyItemAtURL( CFURLRef srcURL, CFURLRef dstURL );
Boolean FileManagerMoveItemAtURL( CFURLRef srcURL, CFURLRef dstURL );

// ICloud-based items
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFTypeRef FileManagerUbiquityIdentityToken( void );
#endif
CFURLRef FileManagerURLForUbiquityContainerIdentifier( CFStringRef identifier );
Boolean FileManagerIsUbiquitousItem( CFURLRef url );
Boolean FileManagerSetUbiquitousItem( Boolean flag, CFURLRef url, CFURLRef destURL, ErrorRef *err );
Boolean FileManagerStartDownloadingUbiquitousItem( CFURLRef url, ErrorRef *err );
Boolean FileManagerEvictUbiquitousItem( CFURLRef url, ErrorRef *err );
CFURLRef FileManagerURLForPublishingUbiquitousItem( CFURLRef url, CFDateRef *expirationDate, ErrorRef *err );

// File provider services
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void FileManagerGetFileProviderServicesForItem( CFURLRef url, FileManagerGetFileProviderServicesHandlerType completionHandler, void *userData );
#endif

// Symbolic and hard links
Boolean FileManagerCreateSymbolicLinkAtURL( CFURLRef url, CFURLRef dstURL, ErrorRef *err );
Boolean FileManagerLinkItemAtURL( CFURLRef srcURL, CFURLRef dstURL, ErrorRef *err );
CFURLRef FileManagerDestinationURLOfSymbolicLinkAtURL( CFURLRef url, ErrorRef *err );

Boolean FileManagerFileExistsAtURL( CFURLRef url );
Boolean FileManagerIsReadableFileAtURL( CFURLRef url );
Boolean FileManagerIsWritableFileAtURL( CFURLRef url );
Boolean FileManagerIsExecutableFileAtURL( CFURLRef url );
Boolean FileManagerIsDeletableFileAtURL( CFURLRef url );
Boolean FileManagerIsDirectoryAtURL( CFURLRef url );
CFDictionaryRef FileManagerAttributesOfItemAtURL( CFURLRef url );
Boolean FileManagerSetAttributesOfItemAtURL( CFURLRef url, CFDictionaryRef attributes );
CFDataRef FileManagerContentsAtURL( CFURLRef url );

Boolean FileManagerContentsEqual( CFURLRef url1, CFURLRef url2 );

// Relationships between items
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean FileManagerGetRelationshipOfDirectoryAtURL( NSURLRelationship *relationship, CFURLRef dirURL, CFURLRef itemURL, ErrorRef *err );
Boolean FileManagerGetRelationshipOfDirectoryInDomain( NSURLRelationship *relationship, NSSearchPathDirectory whichDirectory, NSSearchPathDomainMask domain, CFURLRef itemURL, ErrorRef *err );
#endif

// Current directory
Boolean FileManagerChangeCurrentDirectoryURL( CFURLRef url );
CFURLRef FileManagerCurrentDirectoryURL( void );

// Unmount volumes
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void FileManagerUnmountVolumeAtURL( CFURLRef url, NSFileManagerUnmountOptions options, FileManagerUnmountVolumeHandlerType completionHandler, void *userData );
#endif
