/*
 FileManager.m

 Bernie Wylde
 */

#import "FileManager.h"

@implementation CocoaFileManagerDelegate

- (BOOL)fileManager:(NSFileManager *)fileManager shouldMoveItemAtURL:(NSURL *)srcURL toURL:(NSURL *)dstURL {
    DialogEventSetBool( YES );
    DialogEventSetNSURL( srcURL );
    DialogEventSetOtherNSURL( dstURL );
    CallUserDefinedOnDialogFunction( fileManagerShouldMoveItem, 0, 0, srcURL );
    return DialogEventBool();
}

- (BOOL)fileManager:(NSFileManager *)fileManager shouldProceedAfterError:(NSError *)error movingItemAtURL:(NSURL *)srcURL toURL:(NSURL *)dstURL {
    DialogEventSetBool( NO );
    DialogEventSetNSError( error );
    DialogEventSetNSURL( srcURL );
    DialogEventSetOtherNSURL( dstURL );
    CallUserDefinedOnDialogFunction( fileManagerShouldProceedMovingItem, 0, 0, srcURL );
    return DialogEventBool();
}

- (BOOL)fileManager:(NSFileManager *)fileManager shouldCopyItemAtURL:(NSURL *)srcURL toURL:(NSURL *)dstURL {
    DialogEventSetBool( YES );
    DialogEventSetNSURL( srcURL );
    DialogEventSetOtherNSURL( dstURL );
    CallUserDefinedOnDialogFunction( fileManagerShouldCopyItem, 0, 0, srcURL );
    return DialogEventBool();
}

- (BOOL)fileManager:(NSFileManager *)fileManager shouldProceedAfterError:(NSError *)error copyingItemAtURL:(NSURL *)srcURL toURL:(NSURL *)dstURL {
    DialogEventSetBool( NO );
    DialogEventSetNSError( error );
    DialogEventSetNSURL( srcURL );
    DialogEventSetOtherNSURL( dstURL );
    CallUserDefinedOnDialogFunction( fileManagerShouldProceedCopyingItem, 0, 0, srcURL );
    return DialogEventBool();
}

- (BOOL)fileManager:(NSFileManager *)fileManager shouldRemoveItemAtURL:(NSURL *)URL {
    DialogEventSetBool( YES );
    DialogEventSetNSURL( URL );
    CallUserDefinedOnDialogFunction( fileManagerShouldRemoveItem, 0, 0, URL );
    return DialogEventBool();
}

- (BOOL)fileManager:(NSFileManager *)fileManager shouldProceedAfterError:(NSError *)error removingItemAtURL:(NSURL *)URL {
    
    return NO;
}

- (BOOL)fileManager:(NSFileManager *)fileManager shouldLinkItemAtURL:(NSURL *)srcURL toURL:(NSURL *)dstURL {
    
    return YES;
}

- (BOOL)fileManager:(NSFileManager *)fileManager shouldProceedAfterError:(NSError *)error linkingItemAtURL:(NSURL *)srcURL toURL:(NSURL *)dstURL {
    
    return NO;
}

@end

void CocoaFileManagerDelegateCreate( void )
{
    static CocoaFileManagerDelegate *sCocoaFileManagerDelegate = nil;
    if ( !sCocoaFileManagerDelegate ) {
        sCocoaFileManagerDelegate = [[CocoaFileManagerDelegate alloc] init];
        [[NSFileManager defaultManager] setDelegate:sCocoaFileManagerDelegate];
    }
}



// Create
FileManagerRef FileManagerDefaultManager( void )
{ return [NSFileManager defaultManager]; }

// User directories
CFURLRef FileManagerTemporaryDirectory( void )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
    return (CFURLRef)[[NSFileManager defaultManager] temporaryDirectory];
#else
    return (CFURLRef)[NSURL fileURLWithPath:NSTemporaryDirectory()];
#endif
}

CFURLRef FileManagerHomeDirectoryForCurrentUser( void )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
    return (CFURLRef)[[NSFileManager defaultManager] homeDirectoryForCurrentUser];
#else
    return (CFURLRef)[NSURL fileURLWithPath:NSHomeDirectory()];
#endif
}

CFURLRef FileManagerHomeDirectoryForUser( CFStringRef userName )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
    return (CFURLRef)[[NSFileManager defaultManager] homeDirectoryForUser:(NSString *)userName];
#else
    return (CFURLRef)[NSURL fileURLWithPath:NSHomeDirectoryForUser((NSString *)userName)];
#endif
}

// System directories
CFURLRef FileManagerURLForDirectoryAppropriateForURL( NSSearchPathDirectory whichDirectory, NSSearchPathDomainMask domain, CFURLRef url, Boolean create, ErrorRef *err )
{ return (CFURLRef)[[NSFileManager defaultManager] URLForDirectory:whichDirectory inDomain:domain appropriateForURL:(NSURL *)url create:create error:err]; }

CFURLRef FileManagerURLForDirectory( NSSearchPathDirectory whichDirectory, NSSearchPathDomainMask domain )
{
    NSArray *urls = [[NSFileManager defaultManager] URLsForDirectory:whichDirectory inDomains:domain];
    if ( [urls count] ) return (CFURLRef)[urls objectAtIndex:0];
    return NULL;
}

CFArrayRef FileManagerURLsForDirectory( NSSearchPathDirectory whichDirectory, NSSearchPathDomainMask domain )
{ return (CFArrayRef)[[NSFileManager defaultManager] URLsForDirectory:whichDirectory inDomains:domain]; }

CFURLRef FileManagerURLForPreferencesDirectory( void )
{ return (CFURLRef)[NSURL fileURLWithPath:[@"~/Library/Preferences" stringByExpandingTildeInPath]]; }

CFURLRef FileManagerURLForApplicationDirectory( void )
{ return (CFURLRef)[[[NSBundle mainBundle] bundleURL] URLByDeletingLastPathComponent]; }

CFArrayRef FileManagerContentsOfDirectoryAtURL( CFURLRef url, CFArrayRef keys, NSDirectoryEnumerationOptions options )
{ return (CFArrayRef)[[NSFileManager defaultManager] contentsOfDirectoryAtURL:(NSURL *)url includingPropertiesForKeys:(NSArray *)keys options:options error:nil]; }

DirectoryEnumeratorRef FileManagerEnumeratorAtURL( CFURLRef url, CFArrayRef propertyKeys, NSDirectoryEnumerationOptions options, FileManagerEnumeratorCallbackType errorCallback, void *userData )
{
    NSDirectoryEnumerator *enumerator = [[NSFileManager defaultManager] enumeratorAtURL:(NSURL *)url
                                                             includingPropertiesForKeys:(NSArray *)propertyKeys
                                                                                options:options
                                                                           errorHandler:^(NSURL *url, NSError *error) {
                                                                               if ( errorCallback ) return (*errorCallback)((CFURLRef)url,error,userData);
                                                                               return YES;
                                                                           }];
    return enumerator;
}

CFArrayRef FileManagerMountedVolumeURLs( CFArrayRef propertyKeys, NSVolumeEnumerationOptions options )
{ return (CFArrayRef)[[NSFileManager defaultManager] mountedVolumeURLsIncludingResourceValuesForKeys:(NSArray *)propertyKeys options:options]; }

Boolean FileManagerCreateDirectoryAtURL( CFURLRef url, Boolean withIntermediateDirectories, CFDictionaryRef attributes )
{ return [[NSFileManager defaultManager] createDirectoryAtPath:[(NSURL *)url path] withIntermediateDirectories:withIntermediateDirectories attributes:(NSDictionary *)attributes error:nil]; }

Boolean FileManagerCreateFileAtURL( CFURLRef url, CFDataRef contents, CFDictionaryRef attributes )
{ return [[NSFileManager defaultManager] createFileAtPath:[(NSURL *)url path] contents:(NSData *)contents attributes:(NSDictionary *)attributes]; }

Boolean FileManagerRemoveItemAtURL( CFURLRef url )
{
    CocoaFileManagerDelegateCreate();
    return [[NSFileManager defaultManager] removeItemAtURL:(NSURL *)url error:nil];
}

Boolean FileManagerReplaceItemAtURL( CFURLRef origURL, CFURLRef newURL )
{ return [[NSFileManager defaultManager] replaceItemAtURL:(NSURL *)origURL withItemAtURL:(NSURL *)newURL backupItemName:nil options:0 resultingItemURL:nil error:nil]; }

void FileManagerTrashItemAtURL( CFURLRef url )
{ [[NSWorkspace sharedWorkspace] recycleURLs:[NSArray arrayWithObject:(NSURL *)url] completionHandler:nil]; }

Boolean FileManagerCopyItemAtURL( CFURLRef srcURL, CFURLRef dstURL )
{
    CocoaFileManagerDelegateCreate();
    return [[NSFileManager defaultManager] copyItemAtURL:(NSURL *)srcURL toURL:(NSURL *)dstURL error:nil];
}

Boolean FileManagerMoveItemAtURL( CFURLRef srcURL, CFURLRef dstURL )
{
    CocoaFileManagerDelegateCreate();
    return [[NSFileManager defaultManager] moveItemAtURL:(NSURL *)srcURL toURL:(NSURL *)dstURL error:nil];
}

// ICloud-based items
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFTypeRef FileManagerUbiquityIdentityToken( void )
{ return (CFTypeRef)[[NSFileManager defaultManager] ubiquityIdentityToken]; }
#endif

CFURLRef FileManagerURLForUbiquityContainerIdentifier( CFStringRef identifier )
{ return (CFURLRef)[[NSFileManager defaultManager] URLForUbiquityContainerIdentifier:(NSString *)identifier]; }

Boolean FileManagerIsUbiquitousItem( CFURLRef url )
{ return [[NSFileManager defaultManager] isUbiquitousItemAtURL:(NSURL *)url]; }

Boolean FileManagerSetUbiquitousItem( Boolean flag, CFURLRef url, CFURLRef destURL, ErrorRef *err )
{ return [[NSFileManager defaultManager] setUbiquitous:flag itemAtURL:(NSURL *)url destinationURL:(NSURL *)destURL error:err]; }

Boolean FileManagerStartDownloadingUbiquitousItem( CFURLRef url, ErrorRef *err )
{ return [[NSFileManager defaultManager] startDownloadingUbiquitousItemAtURL:(NSURL *)url error:err]; }

Boolean FileManagerEvictUbiquitousItem( CFURLRef url, ErrorRef *err )
{ return [[NSFileManager defaultManager] evictUbiquitousItemAtURL:(NSURL *)url error:err]; }

CFURLRef FileManagerURLForPublishingUbiquitousItem( CFURLRef url, CFDateRef *expirationDate, ErrorRef *err )
{ return (CFURLRef)[[NSFileManager defaultManager] URLForPublishingUbiquitousItemAtURL:(NSURL *)url expirationDate:(NSDate **)expirationDate error:err]; }

// File provider services
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void FileManagerGetFileProviderServicesForItem( CFURLRef url, FileManagerGetFileProviderServicesHandlerType completionHandler, void *userData )
{
    [[NSFileManager defaultManager] getFileProviderServicesForItemAtURL:(NSURL *)url completionHandler:^(NSDictionary *services,NSError *error){
        (*completionHandler)((CFDictionaryRef)services,error,userData);
    }];
}
#endif

// Symbolic and hard links
Boolean FileManagerCreateSymbolicLinkAtURL( CFURLRef url, CFURLRef dstURL, ErrorRef *err )
{ return [[NSFileManager defaultManager] createSymbolicLinkAtURL:(NSURL *)url withDestinationURL:(NSURL *)dstURL error:err]; }

Boolean FileManagerLinkItemAtURL( CFURLRef srcURL, CFURLRef dstURL, ErrorRef *err )
{ return [[NSFileManager defaultManager] linkItemAtURL:(NSURL *)srcURL toURL:(NSURL *)dstURL error:err]; }

CFURLRef FileManagerDestinationURLOfSymbolicLinkAtURL( CFURLRef url, ErrorRef *err )
{ return (CFURLRef)[NSURL fileURLWithPath:[[NSFileManager defaultManager] destinationOfSymbolicLinkAtPath:[(NSURL *)url path] error:err]]; }


Boolean FileManagerFileExistsAtURL( CFURLRef url )
{ return [[NSFileManager defaultManager] fileExistsAtPath:[(NSURL *)url path]]; }

Boolean FileManagerIsReadableFileAtURL( CFURLRef url )
{ return [[NSFileManager defaultManager] isReadableFileAtPath:[(NSURL *)url path]]; }

Boolean FileManagerIsWritableFileAtURL( CFURLRef url )
{ return [[NSFileManager defaultManager] isWritableFileAtPath:[(NSURL *)url path]]; }

Boolean FileManagerIsExecutableFileAtURL( CFURLRef url )
{ return [[NSFileManager defaultManager] isExecutableFileAtPath:[(NSURL *)url path]]; }

Boolean FileManagerIsDeletableFileAtURL( CFURLRef url )
{ return [[NSFileManager defaultManager] isDeletableFileAtPath:[(NSURL *)url path]]; }

Boolean FileManagerIsDirectoryAtURL( CFURLRef url )
{
    BOOL isDirectory = NO;
    [[NSFileManager defaultManager] fileExistsAtPath:(NSString *)[(NSURL *)url path] isDirectory:&isDirectory];
    return isDirectory;
}

CFDictionaryRef FileManagerAttributesOfItemAtURL( CFURLRef url )
{ return (CFDictionaryRef)[[NSFileManager defaultManager] attributesOfItemAtPath:[(NSURL *)url path] error:nil]; }

Boolean FileManagerSetAttributesOfItemAtURL( CFURLRef url, CFDictionaryRef attributes )
{ return [[NSFileManager defaultManager] setAttributes:(NSDictionary *)attributes ofItemAtPath:[(NSURL *)url path] error:nil]; }

CFDataRef FileManagerContentsAtURL( CFURLRef url )
{ return (CFDataRef)[[NSFileManager defaultManager] contentsAtPath:[(NSURL *)url path]]; }

Boolean FileManagerContentsEqual( CFURLRef url1, CFURLRef url2 )
{ return [[NSFileManager defaultManager] contentsEqualAtPath:[(NSURL *)url1 path] andPath:[(NSURL *)url2 path]]; }

// Relationships between items
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean FileManagerGetRelationshipOfDirectoryAtURL( NSURLRelationship *relationship, CFURLRef dirURL, CFURLRef itemURL, ErrorRef *err )
{ return [[NSFileManager defaultManager] getRelationship:relationship ofDirectoryAtURL:(NSURL *)dirURL toItemAtURL:(NSURL *)itemURL error:err]; }

Boolean FileManagerGetRelationshipOfDirectoryInDomain( NSURLRelationship *relationship, NSSearchPathDirectory whichDirectory, NSSearchPathDomainMask domain, CFURLRef itemURL, ErrorRef *err )
{ return [[NSFileManager defaultManager] getRelationship:relationship ofDirectory:whichDirectory inDomain:domain toItemAtURL:(NSURL *)itemURL error:err]; }
#endif

// Current directory
Boolean FileManagerChangeCurrentDirectoryURL( CFURLRef url )
{ return [[NSFileManager defaultManager] changeCurrentDirectoryPath:[(NSURL *)url path]]; }

CFURLRef FileManagerCurrentDirectoryURL( void )
{ return (CFURLRef)[NSURL fileURLWithPath:[[NSFileManager defaultManager] currentDirectoryPath]]; }

// Unmount volumes
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void FileManagerUnmountVolumeAtURL( CFURLRef url, NSFileManagerUnmountOptions options, FileManagerUnmountVolumeHandlerType completionHandler, void *userData )
{
    [[NSFileManager defaultManager] unmountVolumeAtURL:(NSURL *)url options:options completionHandler:^(NSError *error){
        (*completionHandler)(error,userData);
    }];
}
#endif
