/*
 FileWrapper.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// Create
FileWrapperRef FileWrapperWithURL( CFURLRef url, NSFileWrapperReadingOptions options, ErrorRef *err );
FileWrapperRef FileWrapperDirectoryWithFileWrappers( CFDictionaryRef childrenByPreferredName );
FileWrapperRef FileWrapperRegularFileWithContents( CFDataRef contents );
FileWrapperRef FileWrapperSymbolicLinkWithDestinationURL( CFURLRef url );
FileWrapperRef FileWrapperWithSerializedRepresentation( CFDataRef serializedRepresentation );

// Query
Boolean FileWrapperIsRegularFile( FileWrapperRef ref );
Boolean FileWrapperIsDirectory( FileWrapperRef ref );
Boolean FileWrapperIsSymbolicLink( FileWrapperRef ref );

// Info
CFDictionaryRef FileWrapperFileWrappers( FileWrapperRef ref );
CFStringRef FileWrapperAddFileWrapper( FileWrapperRef ref, FileWrapperRef child );
void FileWrapperRemoveFileWrapper( FileWrapperRef ref, FileWrapperRef child );
CFStringRef FileWrapperAddRegularFileWithContents( FileWrapperRef ref, CFDataRef dta, CFStringRef preferredFilename );
CFStringRef FileWrapperKeyForFileWrapper( FileWrapperRef ref, FileWrapperRef child );
CFURLRef FileWrapperSymbolicLinkDestinationURL( FileWrapperRef ref );

// Update
Boolean FileWrapperMatchesContentsOfURL( FileWrapperRef ref, CFURLRef url );
Boolean FileWrapperReadFromURL( FileWrapperRef ref, CFURLRef url, NSFileWrapperReadingOptions options, ErrorRef *err );

// Serialize
CFDataRef FileWrapperSerializedRepresentation( FileWrapperRef ref );

// Files
CFStringRef FileWrapperFilename( FileWrapperRef ref );
CFStringRef FileWrapperPreferredFilename( FileWrapperRef ref );
CFDictionaryRef FileWrapperFileAttributes( FileWrapperRef ref );
CFDataRef FileWrapperRegularFileContents( FileWrapperRef ref );

// Write files
Boolean FileWrapperWriteToURL( FileWrapperRef ref, CFURLRef url, NSFileWrapperWritingOptions options, CFURLRef origContentsURL, ErrorRef *err );

// Icons
ImageRef FileWrapperIcon( FileWrapperRef ref );


