/*
 FileWrapper.m

 Bernie Wylde
 */

#import "FileWrapper.h"

// Create
FileWrapperRef FileWrapperWithURL( CFURLRef url, NSFileWrapperReadingOptions options, ErrorRef *err )
{ return [[[NSFileWrapper alloc] initWithURL:(NSURL *)url options:options error:err] autorelease]; }

FileWrapperRef FileWrapperDirectoryWithFileWrappers( CFDictionaryRef childrenByPreferredName )
{ return [[[NSFileWrapper alloc] initDirectoryWithFileWrappers:(NSDictionary *)childrenByPreferredName] autorelease]; }

FileWrapperRef FileWrapperRegularFileWithContents( CFDataRef contents )
{ return [[[NSFileWrapper alloc] initRegularFileWithContents:(NSData *)contents] autorelease]; }

FileWrapperRef FileWrapperSymbolicLinkWithDestinationURL( CFURLRef url )
{ return [[[NSFileWrapper alloc] initSymbolicLinkWithDestinationURL:(NSURL *)url] autorelease]; }

FileWrapperRef FileWrapperWithSerializedRepresentation( CFDataRef serializedRepresentation )
{ return [[[NSFileWrapper alloc] initWithSerializedRepresentation:(NSData *)serializedRepresentation] autorelease]; }

// Query
Boolean FileWrapperIsRegularFile( FileWrapperRef ref )
{ return [ref isRegularFile]; }

Boolean FileWrapperIsDirectory( FileWrapperRef ref )
{ return [ref isDirectory]; }

Boolean FileWrapperIsSymbolicLink( FileWrapperRef ref )
{ return [ref isSymbolicLink]; }

// Info
CFDictionaryRef FileWrapperFileWrappers( FileWrapperRef ref )
{ return (CFDictionaryRef)[ref fileWrappers]; }

CFStringRef FileWrapperAddFileWrapper( FileWrapperRef ref, FileWrapperRef child )
{ return (CFStringRef)[ref addFileWrapper:child]; }

void FileWrapperRemoveFileWrapper( FileWrapperRef ref, FileWrapperRef child )
{ [ref removeFileWrapper:child]; }

CFStringRef FileWrapperAddRegularFileWithContents( FileWrapperRef ref, CFDataRef dta, CFStringRef preferredFilename )
{ return (CFStringRef)[ref addRegularFileWithContents:(NSData *)dta preferredFilename:(NSString *)preferredFilename]; }

CFStringRef FileWrapperKeyForFileWrapper( FileWrapperRef ref, FileWrapperRef child )
{ return (CFStringRef)[ref keyForFileWrapper:child]; }

CFURLRef FileWrapperSymbolicLinkDestinationURL( FileWrapperRef ref )
{ return (CFURLRef)[ref symbolicLinkDestinationURL]; }

// Update
Boolean FileWrapperMatchesContentsOfURL( FileWrapperRef ref, CFURLRef url )
{ return [ref matchesContentsOfURL:(NSURL *)url]; }

Boolean FileWrapperReadFromURL( FileWrapperRef ref, CFURLRef url, NSFileWrapperReadingOptions options, ErrorRef *err )
{ return [ref readFromURL:(NSURL *)url options:options error:err]; }

// Serialize
CFDataRef FileWrapperSerializedRepresentation( FileWrapperRef ref )
{ return (CFDataRef)[ref serializedRepresentation]; }

// Files
CFStringRef FileWrapperFilename( FileWrapperRef ref )
{ return (CFStringRef)[ref filename]; }

CFStringRef FileWrapperPreferredFilename( FileWrapperRef ref )
{ return (CFStringRef)[ref preferredFilename]; }

CFDictionaryRef FileWrapperFileAttributes( FileWrapperRef ref )
{ return (CFDictionaryRef)[ref fileAttributes]; }

CFDataRef FileWrapperRegularFileContents( FileWrapperRef ref )
{ return (CFDataRef)[ref regularFileContents]; }

// Write files
Boolean FileWrapperWriteToURL( FileWrapperRef ref, CFURLRef url, NSFileWrapperWritingOptions options, CFURLRef origContentsURL, ErrorRef *err )
{ return [ref writeToURL:(NSURL *)url options:options originalContentsURL:(NSURL *)origContentsURL error:err]; }

// Icons
ImageRef FileWrapperIcon( FileWrapperRef ref )
{ return [ref icon]; }


