/*
 Font.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// - arbitrary font -
FontRef FontWithName( CFStringRef name, CGFloat size );
FontRef FontWithDescriptor( FontDescriptorRef descriptor, CGFloat size );
FontRef FontWithDescriptorAndTextTransform( FontDescriptorRef descriptor, AffineTransformRef tx );
FontRef FontWithNameAndMatrix( CFStringRef name, CGFloat *matrix );

// - user font -
FontRef FontUserFontOfSize( CGFloat size );
FontRef FontUserFixedPitchFontOfSize( CGFloat size );

// - system font -
FontRef FontBoldSystemFontOfSize( CGFloat size );
FontRef FontControlContentFontOfSize( CGFloat size );
FontRef FontLabelFontOfSize( CGFloat size );
FontRef FontMenuFontOfSize( CGFloat size );
FontRef FontMenuBarFontOfSize( CGFloat size );
FontRef FontMessageFontOfSize( CGFloat size );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
FontRef FontMonospacedDigitSystemFontOfSize( CGFloat size, CGFloat weight );
#endif
FontRef FontPaletteFontOfSize( CGFloat size );
FontRef FontSystemFontOfSize( CGFloat size );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
FontRef FontSystemFontOfSizeAndWeight( CGFloat size, CGFloat weight );
#endif
FontRef FontTitleBarFontOfSize( CGFloat size );
FontRef FontToolTipsFontOfSize( CGFloat size );

// - draw -
void FontSet( FontRef font );

// - general information -
FontDescriptorRef FontDescriptor( FontRef font );

// - metrics information -
CGFloat FontLabelFontSize( void );
CGFloat FontSmallSystemFontSize( void );
CGFloat FontSystemFontSize( void );
CGFloat FontSystemFontSizeForControlSize( NSControlSize size );

CGFloat FontAscender( FontRef font );
CGRect FontBoundingRectForFont( FontRef font );
CGFloat FontCapHeight( FontRef font );
CGFloat FontDescender( FontRef font );

CGFloat FontItalicAngle( FontRef font );

CGFloat FontLeading( FontRef font );

const CGFloat *FontMatrix( FontRef font );

CGFloat FontSize( FontRef font );

CGFloat FontUnderlinePosition( FontRef font );
CGFloat FontUnderlineThickness( FontRef font );

CGFloat FontXHeight( FontRef font );

// - font names -
CFStringRef FontDisplayName( FontRef font );
CFStringRef FontFamilyName( FontRef font );
CFStringRef FontName( FontRef font );

// - set user font -
void FontSetUserFont( FontRef font );

// Vertical fonts
Boolean FontIsVertical( FontRef font );
FontRef FontVerticalFont( FontRef font );
