/*
 Font.m

 Bernie Wylde
 */

#import "Font.h"

// - arbitrary font -
FontRef FontWithName( CFStringRef name, CGFloat size )
{ return [NSFont fontWithName:(NSString *)name size:size]; }

FontRef FontWithDescriptor( FontDescriptorRef descriptor, CGFloat size )
{ return [NSFont fontWithDescriptor:descriptor size:size]; }

FontRef FontWithDescriptorAndTextTransform( FontDescriptorRef descriptor, AffineTransformRef tx )
{ return [NSFont fontWithDescriptor:descriptor textTransform:tx]; }

FontRef FontWithNameAndMatrix( CFStringRef name, CGFloat *matrix )
{ return [NSFont fontWithName:(NSString *)name matrix:matrix]; }

// - user font -
FontRef FontUserFontOfSize( CGFloat size )
{ return [NSFont userFontOfSize:size]; }

FontRef FontUserFixedPitchFontOfSize( CGFloat size )
{ return [NSFont userFixedPitchFontOfSize:size]; }

// - system font -
FontRef FontBoldSystemFontOfSize( CGFloat size )
{ return [NSFont boldSystemFontOfSize:size]; }

FontRef FontControlContentFontOfSize( CGFloat size )
{ return [NSFont controlContentFontOfSize:size]; }

FontRef FontLabelFontOfSize( CGFloat size )
{ return [NSFont labelFontOfSize:size]; }

FontRef FontMenuFontOfSize( CGFloat size )
{ return [NSFont menuFontOfSize:size]; }

FontRef FontMenuBarFontOfSize( CGFloat size )
{ return [NSFont menuBarFontOfSize:size]; }

FontRef FontMessageFontOfSize( CGFloat size )
{ return [NSFont messageFontOfSize:size]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
FontRef FontMonospacedDigitSystemFontOfSize( CGFloat size, CGFloat weight )
{ return [NSFont monospacedDigitSystemFontOfSize:size weight:weight]; }
#endif

FontRef FontPaletteFontOfSize( CGFloat size )
{ return [NSFont paletteFontOfSize:size]; }

FontRef FontSystemFontOfSize( CGFloat size )
{ return [NSFont systemFontOfSize:size]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
FontRef FontSystemFontOfSizeAndWeight( CGFloat size, CGFloat weight )
{ return [NSFont systemFontOfSize:size weight:weight]; }
#endif

FontRef FontTitleBarFontOfSize( CGFloat size )
{ return [NSFont titleBarFontOfSize:size]; }

FontRef FontToolTipsFontOfSize( CGFloat size )
{ return [NSFont toolTipsFontOfSize:size]; }

// - draw -
void FontSet( FontRef font )
{ [font set]; }

// - general information -
FontDescriptorRef FontDescriptor( FontRef font )
{ return [font fontDescriptor]; }

// - metrics information -
CGFloat FontLabelFontSize( void )
{ return [NSFont labelFontSize]; }

CGFloat FontSmallSystemFontSize( void )
{ return [NSFont smallSystemFontSize]; }

CGFloat FontSystemFontSize( void )
{ return [NSFont systemFontSize]; }

CGFloat FontSystemFontSizeForControlSize( NSControlSize size )
{ return [NSFont systemFontSizeForControlSize:size]; }

CGFloat FontAscender( FontRef font )
{ return [font ascender]; }

CGRect FontBoundingRectForFont( FontRef font )
{ return NSRectToCGRect([font boundingRectForFont]); }

CGFloat FontCapHeight( FontRef font )
{ return [font capHeight]; }

CGFloat FontDescender( FontRef font )
{ return [font descender]; }

CGFloat FontItalicAngle( FontRef font )
{ return [font italicAngle]; }

CGFloat FontLeading( FontRef font )
{ return [font leading]; }

const CGFloat *FontMatrix( FontRef font )
{ return [font matrix]; }

CGFloat FontSize( FontRef font )
{ return [font pointSize]; }

CGFloat FontUnderlinePosition( FontRef font )
{ return [font underlinePosition]; }

CGFloat FontUnderlineThickness( FontRef font )
{ return [font underlineThickness]; }

CGFloat FontXHeight( FontRef font )
{ return [font xHeight]; }

// - font names -
CFStringRef FontDisplayName( FontRef font )
{ return (CFStringRef)[font displayName]; }

CFStringRef FontFamilyName( FontRef font )
{ return (CFStringRef)[font familyName]; }

CFStringRef FontName( FontRef font )
{ return (CFStringRef)[font fontName]; }

// - set user font -
void FontSetUserFont( FontRef font )
{ [NSFont setUserFont:font]; }

// Vertical fonts
Boolean FontIsVertical( FontRef font )
{ return [font isVertical]; }

FontRef FontVerticalFont( FontRef font )
{ return [font verticalFont]; }

