/*
 FontManager.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// dialog events
enum {
    fontManagerChangeFont = 2200
};

@interface CocoaFontManagerDelegate : NSObject {
    NSFont *font;
}
@property (assign) NSFont *font;
@end

void CocoaFontManagerSetDelegate( void );

// Shared
FontManagerRef FontManagerShared( void );

// Default conversion
void FontManagerSetFontManagerFactory( ClassRef class );
void FontManagerSetFontPanelFactory( ClassRef class );

// Available fonts
CFArrayRef FontManagerAvailableFonts( void );
CFArrayRef FontManagerAvailableFontFamilies( void );
CFArrayRef FontManagerAvailableFontNamesWithTraits( NSFontTraitMask traits );
CFArrayRef FontManagerAvailableMembersOfFontFamily( CFStringRef fam );

// Set and examine
void FontManagerSetSelectedFont( CTNSFontRef inFont, Boolean isMultiple );
Boolean FontManagerIsMultiple( void );
FontRef FontManagerSelectedFont( void );
Boolean FontManagerSendAction( void );
CFStringRef FontManagerLocalizedNameForFamily( CFStringRef family, CFStringRef faceKey );

// Sending actions
void FontManagerAddFontTrait( void );
void FontManagerRemoveFontTrait( void );
void FontManagerModifyFont( void );
void FontManagerModifyFontViaPanel( void );
void FontManagerOrderFrontStylesPanel( void );
void FontManagerOrderFrontFontPanel( void );

// Converting automatically
FontRef FontManagerConvertFont( FontRef font );

// Convert manually
FontRef FontManagerConvertFontToFace( FontRef font, CFStringRef face );
FontRef FontManagerConvertFontToFamily( FontRef font, CFStringRef family );
FontRef FontManagerConvertFontToHaveTrait( FontRef font, NSFontTraitMask traitMask );
FontRef FontManagerConvertFontToNotHaveTrait( FontRef font, NSFontTraitMask traitMask );
FontRef FontManagerConvertFontToSize( FontRef font, CGFloat size );
FontRef FontManagerConvertFontWeight( FontRef font, Boolean upFlag );
NSFontAction FontManagerCurrentFontAction( void );
NSFontTraitMask FontManagerConvertFontTraits( NSFontTraitMask traitMask );

// Get font
FontRef FontManagerFontWithFamily( CFStringRef family, NSFontTraitMask traitMask, NSInteger weight, CGFloat size );

// Examining fonts
NSFontTraitMask FontManagerTraitsOfFont( FontRef font );
Boolean FontManagerFontNamedHasTraits( CFStringRef name, NSFontTraitMask traitMask );
NSInteger FontManagerWeightOfFont( FontRef font );

// Managing the font panel and font menu
Boolean FontManagerIsEnabled( void );
FontPanelRef FontManagerFontPanel( Boolean create );
CocoaMenuRef FontManagerFontMenu( Boolean create );
void FontManagerSetFontMenu( CocoaMenuRef m );

// Action and target
SELRef FontManagerAction( void );
CFTypeRef FontManagerTarget( void );

// Attributes
void FontManagerSetSelectedAttributes( CFDictionaryRef attrs, Boolean isMultiple );
CFDictionaryRef FontManagerConvertAttributes( CFDictionaryRef attrs );


// Custom
void FontManagerShowPanel( void );
void FontManagerClosePanel( void );

