/*
 FontManager.m

 Bernie Wylde
 */

#import "FontManager.h"

CocoaFontManagerDelegate *sCocoaFontManagerDelegate;

@implementation CocoaFontManagerDelegate

@synthesize font;

- (id)init {
    if ( self = [super init] ) {
        self.font = [NSFont fontWithName:@"Helvetica" size:12];
    }
    return self;
}

- (void)changeFont:(id)sender {
    NSFont *oldFont = [self font];
    NSFont *newFont = [sender convertFont:oldFont];
    [self setFont:newFont];
    DialogEventSetFont( newFont );
    CallUserDefinedOnDialogFunction( fontManagerChangeFont, 0, 0, newFont );
}
@end

void CocoaFontManagerSetDelegate( void )
{
    if ( !sCocoaFontManagerDelegate ) {
        sCocoaFontManagerDelegate = [[CocoaFontManagerDelegate alloc] init];
        NSFontManager *fm = [NSFontManager sharedFontManager];
        [fm setDelegate:sCocoaFontManagerDelegate];
        [fm setTarget:sCocoaFontManagerDelegate];
        [fm setSelectedFont:[NSFont fontWithName:@"Helvetica" size:12] isMultiple:NO];
    }
}



// Shared
FontManagerRef FontManagerShared( void )
{ return [NSFontManager sharedFontManager]; }

// Default conversion
void FontManagerSetFontManagerFactory( ClassRef class )
{ [NSFontManager setFontManagerFactory:class]; }

void FontManagerSetFontPanelFactory( ClassRef class )
{ [NSFontManager setFontPanelFactory:class]; }

// Available fonts
CFArrayRef FontManagerAvailableFonts( void )
{ return (CFArrayRef)[[NSFontManager sharedFontManager] availableFonts]; }

CFArrayRef FontManagerAvailableFontFamilies( void )
{ return (CFArrayRef)[[NSFontManager sharedFontManager] availableFontFamilies]; }

CFArrayRef FontManagerAvailableFontNamesWithTraits( NSFontTraitMask traits )
{ return (CFArrayRef)[[NSFontManager sharedFontManager] availableFontNamesWithTraits:traits]; }

CFArrayRef FontManagerAvailableMembersOfFontFamily( CFStringRef fam )
{ return (CFArrayRef)[[NSFontManager sharedFontManager] availableMembersOfFontFamily:(NSString *)fam]; }

// Set and examine
void FontManagerSetSelectedFont( CTNSFontRef inFont, Boolean isMultiple )
{
    NSFontManager *fm = [NSFontManager sharedFontManager];
    CocoaFontManagerSetDelegate();
    if ( inFont ) {
        NSFont *font = FixFont( (void *)inFont );
        [fm setSelectedFont:font isMultiple:isMultiple];
    }
}

Boolean FontManagerIsMultiple( void )
{ return [[NSFontManager sharedFontManager] isMultiple]; }

FontRef FontManagerSelectedFont( void )
{ return [[NSFontManager sharedFontManager] selectedFont]; }

Boolean FontManagerSendAction( void )
{ return [[NSFontManager sharedFontManager] sendAction]; }

CFStringRef FontManagerLocalizedNameForFamily( CFStringRef family, CFStringRef faceKey )
{ return (CFStringRef)[[NSFontManager sharedFontManager] localizedNameForFamily:(NSString *)family face:(NSString *)faceKey]; }

// Sending actions
void FontManagerAddFontTrait( void )
{ [[NSFontManager sharedFontManager] addFontTrait:nil]; }

void FontManagerRemoveFontTrait( void )
{ [[NSFontManager sharedFontManager] removeFontTrait:nil]; }

void FontManagerModifyFont( void )
{ [[NSFontManager sharedFontManager] modifyFont:nil]; }

void FontManagerModifyFontViaPanel( void )
{ [[NSFontManager sharedFontManager] modifyFontViaPanel:nil]; }

void FontManagerOrderFrontStylesPanel( void )
{ [[NSFontManager sharedFontManager] orderFrontStylesPanel:nil]; }

void FontManagerOrderFrontFontPanel( void )
{ [[NSFontManager sharedFontManager] orderFrontFontPanel:nil]; }

// Converting automatically
FontRef FontManagerConvertFont( FontRef font )
{ return [[NSFontManager sharedFontManager] convertFont:font]; }

// Convert manually
FontRef FontManagerConvertFontToFace( FontRef font, CFStringRef face )
{ return [[NSFontManager sharedFontManager] convertFont:font toFace:(NSString *)face]; }

FontRef FontManagerConvertFontToFamily( FontRef font, CFStringRef family )
{ return [[NSFontManager sharedFontManager] convertFont:font toFamily:(NSString *)family]; }

FontRef FontManagerConvertFontToHaveTrait( FontRef font, NSFontTraitMask traitMask )
{ return [[NSFontManager sharedFontManager] convertFont:font toHaveTrait:traitMask]; }

FontRef FontManagerConvertFontToNotHaveTrait( FontRef font, NSFontTraitMask traitMask )
{ return [[NSFontManager sharedFontManager] convertFont:font toNotHaveTrait:traitMask]; }

FontRef FontManagerConvertFontToSize( FontRef font, CGFloat size )
{ return [[NSFontManager sharedFontManager] convertFont:font toSize:size]; }

FontRef FontManagerConvertFontWeight( FontRef font, Boolean upFlag )
{ return [[NSFontManager sharedFontManager] convertWeight:upFlag ofFont:font]; }

NSFontAction FontManagerCurrentFontAction( void )
{ return [[NSFontManager sharedFontManager] currentFontAction]; }

NSFontTraitMask FontManagerConvertFontTraits( NSFontTraitMask traitMask )
{ return [[NSFontManager sharedFontManager] convertFontTraits:traitMask]; }

// Get font
FontRef FontManagerFontWithFamily( CFStringRef family, NSFontTraitMask traitMask, NSInteger weight, CGFloat size )
{ return [[NSFontManager sharedFontManager] fontWithFamily:(NSString *)family traits:traitMask weight:weight size:size]; }

// Examining fonts
NSFontTraitMask FontManagerTraitsOfFont( FontRef font )
{ return [[NSFontManager sharedFontManager] traitsOfFont:font]; }

Boolean FontManagerFontNamedHasTraits( CFStringRef name, NSFontTraitMask traitMask )
{ return [[NSFontManager sharedFontManager] fontNamed:(NSString *)name hasTraits:traitMask]; }

NSInteger FontManagerWeightOfFont( FontRef font )
{ return [[NSFontManager sharedFontManager] weightOfFont:font]; }

// Managing the font panel and font menu
Boolean FontManagerIsEnabled( void )
{ return [[NSFontManager sharedFontManager] isEnabled]; }

FontPanelRef FontManagerFontPanel( Boolean create )
{ return [[NSFontManager sharedFontManager] fontPanel:create]; }

void FontManagerSetFontMenu( CocoaMenuRef m )
{ [[NSFontManager sharedFontManager] setFontMenu:m]; }

CocoaMenuRef FontManagerFontMenu( Boolean create )
{ return [[NSFontManager sharedFontManager] fontMenu:create]; }

// Action and target
SELRef FontManagerAction( void )
{ return [[NSFontManager sharedFontManager] action]; }

CFTypeRef FontManagerTarget( void )
{ return (CFTypeRef)[[NSFontManager sharedFontManager] target]; }

// Attributes
void FontManagerSetSelectedAttributes( CFDictionaryRef attrs, Boolean isMultiple )
{ [[NSFontManager sharedFontManager] setSelectedAttributes:(NSDictionary *)attrs isMultiple:isMultiple]; }

CFDictionaryRef FontManagerConvertAttributes( CFDictionaryRef attrs )
{ return (CFDictionaryRef)[[NSFontManager sharedFontManager] convertAttributes:(NSDictionary *)attrs]; }

// Custom
void FontManagerShowPanel( void )
{
    NSFontManager *fm = [NSFontManager sharedFontManager];
    CocoaFontManagerSetDelegate();
    [fm orderFrontFontPanel:nil];
}

void FontManagerClosePanel( void )
{ [[[NSFontManager sharedFontManager] fontPanel:YES] orderOut:nil]; }

