/*
 GestureRecognizer.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000

// dialog events
enum {
    gestureRecognizerClick = 2700,
    gestureRecognizerMagnification,
    gestureRecognizerPan,
    gestureRecognizerPress,
    gestureRecognizerRotation
};

/*
 Target
 */
typedef void (*GestureRecognizerCallbackType)(id,NSInteger,void *);

@interface GestureRecognizerTarget : NSObject {
    GestureRecognizerCallbackType callback;
    void *userData;
}

@property (assign) GestureRecognizerCallbackType callback;
@property (assign) void *userData;

- (id)initWithCallback:(GestureRecognizerCallbackType)cb userData:(void *)ud;

- (void)action:(id)object;
- (void)clickAction:(id)object;
- (void)magnificationAction:(id)object;
- (void)panAction:(id)object;
- (void)pressAction:(id)object;
- (void)rotationAction:(id)object;
@end


/*
 Subclass
 */
@interface CocoaGestureRecognizer : NSGestureRecognizer
@end






// Init
//GestureRecognizerRef GestureRecognizerWithCallback( GestureRecognizerCallbackType callback, void *userData );

// - location of events -
CGPoint GestureRecognizerLocationInView( GestureRecognizerRef ref, NSInteger tag );

// - state -
NSGestureRecognizerState GestureRecognizerState( GestureRecognizerRef ref );
NSInteger GestureRecognizerView( GestureRecognizerRef ref );
Boolean GestureRecognizerIsEnabled( GestureRecognizerRef ref );
void GestureRecognizerSetEnabled( GestureRecognizerRef ref, Boolean flag );

// - delaying -
Boolean GestureRecognizerDelaysPrimaryMouseButtonEvents( GestureRecognizerRef ref );
void GestureRecognizerSetDelaysPrimaryMouseButtonEvents( GestureRecognizerRef ref, Boolean flag );
Boolean GestureRecognizerDelaysSecondaryMouseButtonEvents( GestureRecognizerRef ref );
void GestureRecognizerSetDelaysSecondaryMouseButtonEvents( GestureRecognizerRef ref, Boolean flag );
Boolean GestureRecognizerDelaysOtherMouseButtonEvents( GestureRecognizerRef ref );
void GestureRecognizerSetDelaysOtherMouseButtonEvents( GestureRecognizerRef ref, Boolean flag );
Boolean GestureRecognizerDelaysKeyEvents( GestureRecognizerRef ref );
void GestureRecognizerSetDelaysKeyEvents( GestureRecognizerRef ref, Boolean flag );
Boolean GestureRecognizerDelaysMagnificationEvents( GestureRecognizerRef ref );
void GestureRecognizerSetDelaysMagnificationEvents( GestureRecognizerRef ref, Boolean flag );
Boolean GestureRecognizerDelaysRotationEvents( GestureRecognizerRef ref );
void GestureRecognizerSetDelaysRotationEvents( GestureRecognizerRef ref, Boolean flag );

// - methods for subclasses -

// Pressure
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
PressureConfigurationRef GestureRecognizerPressureConfiguration( GestureRecognizerRef ref );
void GestureRecognizerSetPressureConfiguration( GestureRecognizerRef ref, PressureConfigurationRef config );
#endif

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
NSTouchTypeMask GestureRecognizerAllowedTouchTypes( GestureRecognizerRef ref );
void GestureRecognizerSetAllowedTouchTypes( GestureRecognizerRef ref, NSTouchTypeMask touchTypes );
#endif

// instance methods

#endif//__MAC_OS_X_VERSION_MAX_ALLOWED >= 101000

