/*
 GestureRecognizer.m

 Bernie Wylde
 */

#import "GestureRecognizer.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000

/*
 Target
 */
@implementation GestureRecognizerTarget

@synthesize callback;
@synthesize userData;

- (id)initWithCallback:(GestureRecognizerCallbackType)cb userData:(void *)ud {
    if ( [super init] ) {
        callback = cb;
        userData = ud;
    }
    return self;
}

- (void)object:(id)object forwardGestureRecognizerEvent:(NSInteger)event {
    NSView *view = [object view];
    NSInteger tag = 0;
    if ( view ) tag = CocoaViewTag(view);
    if ( callback ) {
        (*callback)(object,tag,userData);
    } else {
        CallUserDefinedOnDialogFunction( event, tag, [view wndTag], object );
    }
}



- (void)action:(id)object {
    NSView *view = [object view];
    NSInteger tag = 0;
    if ( view ) tag = CocoaViewTag(view);
    
    if ( callback ) {
        (*callback)(object,tag,userData);
    } else {
        CallUserDefinedOnDialogFunction( gestureRecognizerClick, tag, [view wndTag], NULL );
    }
}

- (void)clickAction:(id)object {
    [self object:object forwardGestureRecognizerEvent:gestureRecognizerClick];
}

- (void)magnificationAction:(id)object {
    [self object:object forwardGestureRecognizerEvent:gestureRecognizerMagnification];
}

- (void)panAction:(id)object {
    [self object:object forwardGestureRecognizerEvent:gestureRecognizerPan];
}

- (void)pressAction:(id)object {
    [self object:object forwardGestureRecognizerEvent:gestureRecognizerPress];
}

- (void)rotationAction:(id)object {
    [self object:object forwardGestureRecognizerEvent:gestureRecognizerRotation];
}

@end


/*
 Subclass
 */
@implementation CocoaGestureRecognizer
@end



// Init
//GestureRecognizerRef GestureRecognizerWithCallback( GestureRecognizerCallbackType callback, void *userData )
//{
//    
//    return [[[NSGestureRecognizer alloc] initWithCallback:callback userData:userData] autorelease];
//    
//}

// - location of events -
CGPoint GestureRecognizerLocationInView( GestureRecognizerRef ref, NSInteger tag )
{
    NSView *view = ViewWithTag(tag);
    if ( view ) return NSPointToCGPoint([ref locationInView:view]);
    return CGPointZero;
}

// - state -
NSGestureRecognizerState GestureRecognizerState( GestureRecognizerRef ref )
{ return [ref state]; }

NSInteger GestureRecognizerView( GestureRecognizerRef ref )
{
    NSView *view = [ref view];
    if ( view ) return CocoaViewTag(view);
    return 0;
}

Boolean GestureRecognizerIsEnabled( GestureRecognizerRef ref )
{ return [ref isEnabled]; }

void GestureRecognizerSetEnabled( GestureRecognizerRef ref, Boolean flag )
{ [ref setEnabled:flag]; }

// - delaying -
Boolean GestureRecognizerDelaysPrimaryMouseButtonEvents( GestureRecognizerRef ref )
{ return [ref delaysPrimaryMouseButtonEvents]; }

void GestureRecognizerSetDelaysPrimaryMouseButtonEvents( GestureRecognizerRef ref, Boolean flag )
{ return [ref setDelaysPrimaryMouseButtonEvents:flag]; }

Boolean GestureRecognizerDelaysSecondaryMouseButtonEvents( GestureRecognizerRef ref )
{ return [ref delaysSecondaryMouseButtonEvents]; }

void GestureRecognizerSetDelaysSecondaryMouseButtonEvents( GestureRecognizerRef ref, Boolean flag )
{ return [ref setDelaysSecondaryMouseButtonEvents:flag]; }

Boolean GestureRecognizerDelaysOtherMouseButtonEvents( GestureRecognizerRef ref )
{ return [ref delaysOtherMouseButtonEvents]; }

void GestureRecognizerSetDelaysOtherMouseButtonEvents( GestureRecognizerRef ref, Boolean flag )
{ return [ref setDelaysOtherMouseButtonEvents:flag]; }

Boolean GestureRecognizerDelaysKeyEvents( GestureRecognizerRef ref )
{ return [ref delaysKeyEvents]; }

void GestureRecognizerSetDelaysKeyEvents( GestureRecognizerRef ref, Boolean flag )
{ return [ref setDelaysKeyEvents:flag]; }

Boolean GestureRecognizerDelaysMagnificationEvents( GestureRecognizerRef ref )
{ return [ref delaysMagnificationEvents]; }

void GestureRecognizerSetDelaysMagnificationEvents( GestureRecognizerRef ref, Boolean flag )
{ return [ref setDelaysMagnificationEvents:flag]; }

Boolean GestureRecognizerDelaysRotationEvents( GestureRecognizerRef ref )
{ return [ref delaysRotationEvents]; }

void GestureRecognizerSetDelaysRotationEvents( GestureRecognizerRef ref, Boolean flag )
{ return [ref setDelaysRotationEvents:flag]; }

// - delegate -

// - methods for subclasses -

// Pressure -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
PressureConfigurationRef GestureRecognizerPressureConfiguration( GestureRecognizerRef ref )
{ return [ref pressureConfiguration]; }

void GestureRecognizerSetPressureConfiguration( GestureRecognizerRef ref, PressureConfigurationRef config )
{ [ref setPressureConfiguration:config]; }
#endif

// Instance properties -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
NSTouchTypeMask GestureRecognizerAllowedTouchTypes( GestureRecognizerRef ref )
{ return [ref allowedTouchTypes]; }

void GestureRecognizerSetAllowedTouchTypes( GestureRecognizerRef ref, NSTouchTypeMask touchTypes )
{ [ref setAllowedTouchTypes:touchTypes]; }
#endif

// - instance methods -

#endif// __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000


