/*
 Gradient.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// Init
GradientRef GradientWithStartEndColors( ColorRef startColor, ColorRef endColor );
GradientRef GradientWithColors( CFArrayRef colors );
GradientRef GradientWithColorsAndLocations( ColorRef firstColor, ... );
GradientRef GradientWithColorsAtLocations( CFArrayRef colors, CGFloat *locations, ColorSpaceRef cs );

// Primitive drawing methods
void GradientDrawFromPoint( GradientRef ref, CGPoint pt1, CGPoint pt2, NSGradientDrawingOptions options );
void GradientDrawFromCenter( GradientRef ref, CGPoint startCenter, CGFloat startRadius, CGPoint endCenter, CGFloat endRadius, NSGradientDrawingOptions options );

// Draw linear gradients
void GradientDrawInRect( GradientRef ref, CGRect rect, CGFloat angle );
void GradientDrawInBezierPath( GradientRef ref, BezierPathRef path, CGFloat angle );

// Draw radial gradients
void GradientDrawInRectRelativeCenterPosition( GradientRef ref, CGRect rect, CGPoint position );
void GradientDrawInBezierPathRelativeCenterPosition( GradientRef ref, BezierPathRef path, CGPoint position );

// Gradient properties
ColorSpaceRef GradientColorSpace( GradientRef ref );
NSInteger GradientNumberOfColorStops( GradientRef ref );
void GradientGetColor( GradientRef ref, ColorRef *col, CGFloat *location, NSInteger index );
ColorRef GradientInterpolatedColorAtLocation( GradientRef ref, CGFloat location );

// convenience
void GradientDrawFromPointWithStartEndColors( CGPoint pt1, CGPoint pt2, ColorRef startColor, ColorRef endColor, NSGradientDrawingOptions options );
void GradientDrawFromCenterWithStartEndColors( CGPoint startCenter, CGFloat startRadius, CGPoint endCenter, CGFloat endRadius, ColorRef startColor, ColorRef endColor, NSGradientDrawingOptions options );
void GradientDrawInRectWithStartEndColors( CGRect rect, CGFloat angle, ColorRef startColor, ColorRef endColor );
void GradientDrawInBezierPathWithStartEndColors( BezierPathRef path, CGFloat angle, ColorRef startColor, ColorRef endColor );
void GradientDrawInRectRelativeCenterPositionWithStartEndColors( CGRect rect, CGPoint position, ColorRef startColor, ColorRef endColor );
void GradientDrawInBezierPathRelativeCenterPositionWithStartEndColors( BezierPathRef path, CGPoint position, ColorRef startColor, ColorRef endColor );
