/*
 Gradient.m

 Bernie Wylde
 */

#import "Gradient.h"

// Init
GradientRef GradientWithStartEndColors( ColorRef startColor, ColorRef endColor )
{ return [[[NSGradient alloc] initWithStartingColor:startColor endingColor:endColor] autorelease]; }

GradientRef GradientWithColors( CFArrayRef colors )
{ return [[[NSGradient alloc] initWithColors:(NSArray *)colors] autorelease]; }

GradientRef GradientWithColorsAndLocations( ColorRef firstColor, ... )
{
    va_list ap;
    va_start(ap,firstColor);
    
    NSMutableArray *colors = [NSMutableArray arrayWithCapacity:0];
    NSMutableArray *locations = [NSMutableArray arrayWithCapacity:0];
    NSColor *color = firstColor;
    while ( color ) {
        [colors addObject:color];
        double location = va_arg( ap, double );
        [locations addObject:[NSNumber numberWithDouble:location]];
        color = va_arg( ap, id );
    }
    
    NSUInteger count = [locations count];
    double locs[count];
    NSUInteger index = 0;
    for ( NSNumber *number in locations ) {
        locs[index] = [number doubleValue];
        index++;
    }
    
    va_end(ap);
    
    return [[[NSGradient alloc] initWithColors:colors atLocations:(CGFloat *)locs colorSpace:[NSColorSpace genericRGBColorSpace]] autorelease];
}

GradientRef GradientWithColorsAtLocations( CFArrayRef colors, CGFloat *locations, ColorSpaceRef cs )
{ return [[[NSGradient alloc] initWithColors:(NSArray *)colors atLocations:locations colorSpace:cs] autorelease]; }

// Primitive
void GradientDrawFromPoint( GradientRef ref, CGPoint pt1, CGPoint pt2, NSGradientDrawingOptions options )
{ [ref drawFromPoint:NSPointFromCGPoint(pt1) toPoint:NSPointFromCGPoint(pt2) options:options]; }

void GradientDrawFromCenter( GradientRef ref, CGPoint startCenter, CGFloat startRadius, CGPoint endCenter, CGFloat endRadius, NSGradientDrawingOptions options )
{ [ref drawFromCenter:NSPointFromCGPoint(startCenter) radius:startRadius toCenter:NSPointFromCGPoint(endCenter) radius:endRadius options:options]; }

// Linear
void GradientDrawInRect( GradientRef ref, CGRect rect, CGFloat angle )
{ [ref drawInRect:NSRectFromCGRect(rect) angle:angle]; }

void GradientDrawInBezierPath( GradientRef ref, BezierPathRef path, CGFloat angle )
{ [ref drawInBezierPath:path angle:angle]; }

// Radial
void GradientDrawInRectRelativeCenterPosition( GradientRef ref, CGRect rect, CGPoint position )
{ [ref drawInRect:NSRectFromCGRect(rect) relativeCenterPosition:NSPointFromCGPoint(position)]; }

void GradientDrawInBezierPathRelativeCenterPosition( GradientRef ref, BezierPathRef path, CGPoint position )
{ [ref drawInBezierPath:path relativeCenterPosition:NSPointFromCGPoint(position)]; }

// Properties
ColorSpaceRef GradientColorSpace( GradientRef ref )
{ return [ref colorSpace]; }

NSInteger GradientNumberOfColorStops( GradientRef ref )
{ return [ref numberOfColorStops]; }

void GradientGetColor( GradientRef ref, ColorRef *col, CGFloat *location, NSInteger index )
{ [ref getColor:col location:location atIndex:index]; }

ColorRef GradientInterpolatedColorAtLocation( GradientRef ref, CGFloat location )
{ return [ref interpolatedColorAtLocation:location]; }

// Convenience
void GradientDrawFromPointWithStartEndColors( CGPoint pt1, CGPoint pt2, ColorRef startColor, ColorRef endColor, NSGradientDrawingOptions options )
{
    NSGradient *gradient = [[[NSGradient alloc] initWithStartingColor:startColor endingColor:endColor] autorelease];
    [gradient drawFromPoint:NSPointFromCGPoint(pt1) toPoint:NSPointFromCGPoint(pt2) options:options];
}

void GradientDrawFromCenterWithStartEndColors( CGPoint startCenter, CGFloat startRadius, CGPoint endCenter, CGFloat endRadius, ColorRef startColor, ColorRef endColor, NSGradientDrawingOptions options )
{
    NSGradient *gradient = [[[NSGradient alloc] initWithStartingColor:startColor endingColor:endColor] autorelease];
    [gradient drawFromCenter:NSPointFromCGPoint(startCenter) radius:startRadius toCenter:NSPointFromCGPoint(endCenter) radius:endRadius options:options];
}

void GradientDrawInRectWithStartEndColors( CGRect rect, CGFloat angle, ColorRef startColor, ColorRef endColor )
{
    NSGradient *gradient = [[[NSGradient alloc] initWithStartingColor:startColor endingColor:endColor] autorelease];
    [gradient drawInRect:NSRectFromCGRect(rect) angle:angle];
}

void GradientDrawInBezierPathWithStartEndColors( BezierPathRef path, CGFloat angle, ColorRef startColor, ColorRef endColor )
{
    NSGradient *gradient = [[[NSGradient alloc] initWithStartingColor:startColor endingColor:endColor] autorelease];
    [gradient drawInBezierPath:path angle:angle];
}

void GradientDrawInRectRelativeCenterPositionWithStartEndColors( CGRect rect, CGPoint position, ColorRef startColor, ColorRef endColor )
{
    NSGradient *gradient = [[[NSGradient alloc] initWithStartingColor:startColor endingColor:endColor] autorelease];
    [gradient drawInRect:NSRectFromCGRect(rect) relativeCenterPosition:NSPointFromCGPoint(position)];
}

void GradientDrawInBezierPathRelativeCenterPositionWithStartEndColors( BezierPathRef path, CGPoint position, ColorRef startColor, ColorRef endColor )
{
    NSGradient *gradient = [[[NSGradient alloc] initWithStartingColor:startColor endingColor:endColor] autorelease];
    [gradient drawInBezierPath:path relativeCenterPosition:NSPointFromCGPoint(position)];
}

