/*
 GraphicsContext.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// Create
GraphicsContextRef GraphicsContextWithAttributes( CFDictionaryRef attributes );
GraphicsContextRef GraphicsContextWithBitmapImageRep( BitmapImageRepRef rep );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
GraphicsContextRef GraphicsContextWithCGContext( CGContextRef cgCtx, Boolean flipped );
#endif
GraphicsContextRef GraphicsContextWithWindow( CocoaWindowRef window );

// Manage current context
GraphicsContextRef GraphicsContextCurrentContext( void );
void GraphicsContextSetCurrentContext( GraphicsContextRef ctx );
CGContextRef GraphicsContextCurrentCGContext( void );

// Graphics state
void GraphicsContextSaveGraphicsState( void );
void GraphicsContextRestoreGraphicsState( void );

// Testing drawing destination
Boolean GraphicsContextCurrentContextDrawingToScreen( void );
Boolean GraphicsContextDrawingToScreen( GraphicsContextRef ctx );

// Context info
CFDictionaryRef GraphicsContextAttributes( GraphicsContextRef ctx );
Boolean GraphicsContextIsFlipped( GraphicsContextRef ctx );

// Flushing
void GraphicsContextFlushGraphics( GraphicsContextRef ctx );

// Rendering options
NSCompositingOperation GraphicsContextCompositingOperation( GraphicsContextRef ctx );
void GraphicsContextSetCompositingOperation( GraphicsContextRef ctx, NSCompositingOperation operation );
NSImageInterpolation GraphicsContextImageInterpolation( GraphicsContextRef ctx );
void GraphicsContextSetImageInterpolation( GraphicsContextRef ctx, NSImageInterpolation interpolation );
Boolean GraphicsContextShouldAntialias( GraphicsContextRef ctx );
void GraphicsContextSetShouldAntialias( GraphicsContextRef ctx, Boolean flag );
CGPoint GraphicsContextPatternPhase( GraphicsContextRef ctx );
void GraphicsContextSetPatternPhase( GraphicsContextRef ctx, CGPoint phase );

// CIContext
CIContextRef GraphicsContextCIContext( GraphicsContextRef ctx );

// Color rendering intent
NSColorRenderingIntent GraphicsContextColorRenderingIntent( GraphicsContextRef ctx );
void GraphicsContextSetColorRenderingIntent( GraphicsContextRef ctx, NSColorRenderingIntent intent );


