/*
 GraphicsContext.m

 Bernie Wylde
 */

#import "GraphicsContext.h"

// Create
GraphicsContextRef GraphicsContextWithAttributes( CFDictionaryRef attributes )
{ return [NSGraphicsContext graphicsContextWithAttributes:(NSDictionary *)attributes]; }

GraphicsContextRef GraphicsContextWithBitmapImageRep( BitmapImageRepRef rep )
{ return [NSGraphicsContext graphicsContextWithBitmapImageRep:rep]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
GraphicsContextRef GraphicsContextWithCGContext( CGContextRef cgCtx, Boolean flipped )
{ return [NSGraphicsContext graphicsContextWithCGContext:cgCtx flipped:flipped]; }
#endif

GraphicsContextRef GraphicsContextWithWindow( CocoaWindowRef window )
{ return [NSGraphicsContext graphicsContextWithWindow:window]; }

// Manage current context
GraphicsContextRef GraphicsContextCurrentContext( void )
{ return [NSGraphicsContext currentContext]; }

void GraphicsContextSetCurrentContext( GraphicsContextRef ctx )
{ [NSGraphicsContext setCurrentContext:ctx]; }

CGContextRef GraphicsContextCurrentCGContext( void )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
    return [[NSGraphicsContext currentContext] CGContext];
#else
    return [[NSGraphicsContext currentContext] graphicsPort];
#endif
    return NULL;
}

// Graphics state
void GraphicsContextSaveGraphicsState( void )
{ [NSGraphicsContext saveGraphicsState]; }

void GraphicsContextRestoreGraphicsState( void )
{ [NSGraphicsContext restoreGraphicsState]; }

// Testing drawing destination
Boolean GraphicsContextCurrentContextDrawingToScreen( void )
{ return [NSGraphicsContext currentContextDrawingToScreen]; }

Boolean GraphicsContextDrawingToScreen( GraphicsContextRef ctx )
{ return [ctx isDrawingToScreen]; }

// Context info
CFDictionaryRef GraphicsContextAttributes( GraphicsContextRef ctx )
{ return (CFDictionaryRef)[ctx attributes]; }

Boolean GraphicsContextIsFlipped( GraphicsContextRef ctx )
{ return [ctx isFlipped]; }

// Flushing
void GraphicsContextFlushGraphics( GraphicsContextRef ctx )
{ [ctx flushGraphics]; }

// Rendering options
NSCompositingOperation GraphicsContextCompositingOperation( GraphicsContextRef ctx )
{ return [ctx compositingOperation]; }

void GraphicsContextSetCompositingOperation( GraphicsContextRef ctx, NSCompositingOperation operation )
{ [ctx setCompositingOperation:operation]; }

NSImageInterpolation GraphicsContextImageInterpolation( GraphicsContextRef ctx )
{ return [ctx imageInterpolation]; }

void GraphicsContextSetImageInterpolation( GraphicsContextRef ctx, NSImageInterpolation interpolation )
{ [ctx setImageInterpolation:interpolation]; }

Boolean GraphicsContextShouldAntialias( GraphicsContextRef ctx )
{ return [ctx shouldAntialias]; }

void GraphicsContextSetShouldAntialias( GraphicsContextRef ctx, Boolean flag )
{ [ctx setShouldAntialias:flag]; }

CGPoint GraphicsContextPatternPhase( GraphicsContextRef ctx )
{ return NSPointToCGPoint([ctx patternPhase]); }

void GraphicsContextSetPatternPhase( GraphicsContextRef ctx, CGPoint phase )
{ [ctx setPatternPhase:NSPointFromCGPoint(phase)]; }

// CIContext
CIContextRef GraphicsContextCIContext( GraphicsContextRef ctx )
{ return [ctx CIContext]; }

// Color rendering intent
NSColorRenderingIntent GraphicsContextColorRenderingIntent( GraphicsContextRef ctx )
{ return [ctx colorRenderingIntent]; }

void GraphicsContextSetColorRenderingIntent( GraphicsContextRef ctx, NSColorRenderingIntent intent )
{ [ctx setColorRenderingIntent:intent]; }


