/*
 GridView.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200

#pragma mark - subclass
@interface GridViewSC : NSGridView
@end

@interface NSGridView (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end


GridViewRef GridViewWithTag( NSInteger tag );
BOOL GridViewExists( NSInteger tag );

// Info
NSInteger GridViewNumberOfRows( NSInteger tag );
NSInteger GridViewNumberOfColumns( NSInteger tag );
NSInteger GridViewIndexOfColumn( NSInteger tag, GridColumnRef column );
GridRowRef GridViewRowAtIndex( NSInteger tag, NSInteger index );
GridColumnRef GridViewColumnAtIndex( NSInteger tag, NSInteger index );
NSInteger GridViewIndexOfRow( NSInteger tag, GridRowRef row );

// Add, remove, move rows
GridRowRef GridViewAddRowWithViews( NSInteger tag, CFArrayRef views );
GridRowRef GridViewInsertRowWithViews( NSInteger tag, NSInteger index, CFArrayRef views );
void GridViewRemoveRow( NSInteger tag, NSInteger index );
void GridViewMoveRow( NSInteger tag, NSInteger fromIndex, NSInteger toIndex );

// Add, remove, move columns
GridColumnRef GridViewAddColumnWithViews( NSInteger tag, CFArrayRef views );
GridColumnRef GridViewInsertColumnWithViews( NSInteger tag, NSInteger index, CFArrayRef views );
void GridViewRemoveColumn( NSInteger tag, NSInteger index );
void GridViewMoveColumn( NSInteger tag, NSInteger fromIndex, NSInteger toIndex );

// Spacing and alignment
CGFloat GridViewColumnSpacing( NSInteger tag );
void GridViewSetColumnSpacing( NSInteger tag, CGFloat spacing );
CGFloat GridViewRowSpacing( NSInteger tag );
void GridViewSetRowSpacing( NSInteger tag, CGFloat spacing );
NSGridRowAlignment GridViewRowAlignment( NSInteger tag );
void GridViewSetRowAlignment( NSInteger tag, NSGridRowAlignment alignment );
NSGridCellPlacement GridViewXPlacement( NSInteger tag );
void GridViewSetXPlacement( NSInteger tag, NSGridCellPlacement placement );
NSGridCellPlacement GridViewYPlacement( NSInteger tag );
void GridViewSetYPlacement( NSInteger tag, NSGridCellPlacement placement );

// Create and merge cells
GridCellRef GridViewCellAtColumnRow( NSInteger tag, NSInteger colIndex, NSInteger rowIndex );
GridCellRef GridViewCellForView( NSInteger tag, NSInteger vwTag );
void GridViewMergeCells( NSInteger tag, CFRange horizontalRange, CFRange verticalRange );

/*
 GridCell
 */
// Containers
GridColumnRef GridCellColumn( GridCellRef ref );
GridRowRef GridCellRow( GridCellRef ref );
ViewRef GridCellContentView( GridCellRef ref );
void GridCellSetContentView( GridCellRef ref, ViewRef vw );
ViewRef GridCellEmptyContentView( void );

// Formatting
CFArrayRef GridCellCustomPlacementConstraints( GridCellRef ref );
void GridCellSetCustomPlacementConstraints( GridCellRef ref, CFArrayRef constraints );
NSGridRowAlignment GridCellRowAlignment( GridCellRef ref );
void GridCellSetRowAlignment( GridCellRef ref, NSGridRowAlignment alignment );
NSGridCellPlacement GridCellXPlacement( GridCellRef ref );
void GridCellSetXPlacement( GridCellRef ref, NSGridCellPlacement placement );
NSGridCellPlacement GridCellYPlacement( GridCellRef ref );
void GridCellSetYPlacement( GridCellRef ref, NSGridCellPlacement placement );

/*
 GridColumn
 */
// Instance properties
GridViewRef GridColumnGridView( GridColumnRef ref );
BOOL GridColumnIsHidden( GridColumnRef ref );
void GridColumnSetHidden( GridColumnRef ref, BOOL flag );
CGFloat GridColumnLeadingPadding( GridColumnRef ref );
void GridColumnSetLeadingPadding( GridColumnRef ref, CGFloat padding );
NSInteger GridColumnNumberOfCells( GridColumnRef ref );
CGFloat GridColumnTrailingPadding( GridColumnRef ref );
void GridColumnSetTrailingPadding( GridColumnRef ref, CGFloat padding );
CGFloat GridColumnWidth( GridColumnRef ref );
void GridColumnSetWidth( GridColumnRef ref, CGFloat width );
NSGridCellPlacement GridColumnXPlacement( GridColumnRef ref );
void GridColumnSetXPlacement( GridColumnRef ref, NSGridCellPlacement placement );

// Instance methods
GridCellRef GridColumnCellAtIndex( GridColumnRef ref, NSInteger index );
void GridColumnMergeCells( GridColumnRef ref, CFRange range );

/*
 GridRow
 */
// Details
NSInteger GridRowNumberOfCells( GridRowRef ref );
BOOL GridRowIsHidden( GridRowRef ref );
void GridRowSetHidden( GridRowRef ref, BOOL flag );

// Formatting
CGFloat GridRowTopPadding( GridRowRef ref );
void GridRowSetTopPadding( GridRowRef ref, CGFloat padding );
CGFloat GridRowBottomPadding( GridRowRef ref );
void GridRowSetBottomPadding( GridRowRef ref, CGFloat padding );
CGFloat GridRowHeight( GridRowRef ref );
void GridRowSetHeight( GridRowRef ref, CGFloat height );
NSGridCellPlacement GridRowYPlacement( GridRowRef ref );
void GridRowSetYPlacement( GridRowRef ref, NSGridCellPlacement placement );

// Grid view
GridViewRef GridRowGridView( GridRowRef ref );

// Cells
GridCellRef GridRowCellAtIndex( GridRowRef ref, NSInteger index );

// Merge
void GridRowMergeCells( GridRowRef ref, CFRange range );


// Init
GridViewRef GridViewInit( NSInteger inTag, CGRect inRect );

// gridview statement
void CocoaGridView( NSInteger inTag, CGRect inRect, NSInteger inNumColumns, NSInteger inNumRows, CFTypeRef inViews, NSInteger inWndTag );

#endif// 101200
