/*
 GridView.m

 Bernie Wylde
 */

#import "GridView.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200

#pragma mark - subclass
@implementation GridViewSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end


@implementation NSGridView (Additions)
@dynamic tagNumber;
- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}
- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end


GridViewRef GridViewWithTag( NSInteger tag )
{ return (NSGridView *)CocoaValidateViewKindOfClass( tag, [NSGridView class] ); }

BOOL GridViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSGridView class] ); }

// Info
NSInteger GridViewNumberOfRows( NSInteger tag )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view numberOfRows];
    return 0;
}

NSInteger GridViewNumberOfColumns( NSInteger tag )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view numberOfColumns];
    return 0;
}

NSInteger GridViewIndexOfColumn( NSInteger tag, GridColumnRef column )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view indexOfColumn:column];
    return 0;
}

GridRowRef GridViewRowAtIndex( NSInteger tag, NSInteger index )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view rowAtIndex:index];
    return nil;
}

GridColumnRef GridViewColumnAtIndex( NSInteger tag, NSInteger index )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view columnAtIndex:index];
    return nil;
}

NSInteger GridViewIndexOfRow( NSInteger tag, GridRowRef row )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view indexOfRow:row];
    return 0;
}

// Add, remove, move rows
GridRowRef GridViewAddRowWithViews( NSInteger tag, CFArrayRef views )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view addRowWithViews:(NSArray *)views];
    return nil;
}

GridRowRef GridViewInsertRowWithViews( NSInteger tag, NSInteger index, CFArrayRef views )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view insertRowAtIndex:index withViews:(NSArray *)views];
    return nil;
}

void GridViewRemoveRow( NSInteger tag, NSInteger index )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) [view removeRowAtIndex:index];
}

void GridViewMoveRow( NSInteger tag, NSInteger fromIndex, NSInteger toIndex )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) [view moveRowAtIndex:fromIndex toIndex:toIndex];
}

// Add, remove, move columns
GridColumnRef GridViewAddColumnWithViews( NSInteger tag, CFArrayRef views )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view addColumnWithViews:(NSArray *)views];
    return nil;
}

GridColumnRef GridViewInsertColumnWithViews( NSInteger tag, NSInteger index, CFArrayRef views )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view insertColumnAtIndex:index withViews:(NSArray *)views];
    return nil;
}

void GridViewRemoveColumn( NSInteger tag, NSInteger index )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) [view removeColumnAtIndex:index];
}

void GridViewMoveColumn( NSInteger tag, NSInteger fromIndex, NSInteger toIndex )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) [view moveColumnAtIndex:fromIndex toIndex:toIndex];
}

// Spacing and alignment
CGFloat GridViewColumnSpacing( NSInteger tag )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view columnSpacing];
    return 0.0;
}

void GridViewSetColumnSpacing( NSInteger tag, CGFloat spacing )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) [view setColumnSpacing:spacing];
}

CGFloat GridViewRowSpacing( NSInteger tag )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view rowSpacing];
    return 0.0;
}

void GridViewSetRowSpacing( NSInteger tag, CGFloat spacing )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) [view setRowSpacing:spacing];
}

NSGridRowAlignment GridViewRowAlignment( NSInteger tag )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view rowAlignment];
    return 0;
}

void GridViewSetRowAlignment( NSInteger tag, NSGridRowAlignment alignment )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) [view setRowAlignment:alignment];
}

NSGridCellPlacement GridViewXPlacement( NSInteger tag )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view xPlacement];
    return 0;
}

void GridViewSetXPlacement( NSInteger tag, NSGridCellPlacement placement )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) [view setXPlacement:placement];
}

NSGridCellPlacement GridViewYPlacement( NSInteger tag )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view yPlacement];
    return 0;
}

void GridViewSetYPlacement( NSInteger tag, NSGridCellPlacement placement )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) [view setYPlacement:placement];
}

// Create and merge cells
GridCellRef GridViewCellAtColumnRow( NSInteger tag, NSInteger colIndex, NSInteger rowIndex )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) return [view cellAtColumnIndex:colIndex rowIndex:rowIndex];
    return nil;
}

GridCellRef GridViewCellForView( NSInteger tag, NSInteger vwTag )
{
    NSGridView *gridView = GridViewWithTag( tag );
    if ( gridView ) {
        NSView *view = ViewWithTag( vwTag );
        if ( view ) {
            return [gridView cellForView:view];
        }
    }
    return nil;
}

void GridViewMergeCells( NSInteger tag, CFRange horizontalRange, CFRange verticalRange )
{
    NSGridView *view = GridViewWithTag( tag );
    if ( view ) [view mergeCellsInHorizontalRange:NSRangeFromCFRange(horizontalRange) verticalRange:NSRangeFromCFRange(verticalRange)];
}


/*
 GridCell
 */
// Containers
GridColumnRef GridCellColumn( GridCellRef ref )
{ return [ref column]; }

GridRowRef GridCellRow( GridCellRef ref )
{ return [ref row]; }

ViewRef GridCellContentView( GridCellRef ref )
{ return [ref contentView]; }

void GridCellSetContentView( GridCellRef ref, ViewRef vw )
{ [ref setContentView:vw]; }

ViewRef GridCellEmptyContentView( void )
{ return [NSGridCell emptyContentView]; }

// Formatting
CFArrayRef GridCellCustomPlacementConstraints( GridCellRef ref )
{ return (CFArrayRef)[ref customPlacementConstraints]; }

void GridCellSetCustomPlacementConstraints( GridCellRef ref, CFArrayRef constraints )
{ [ref setCustomPlacementConstraints:(NSArray *)constraints]; }

NSGridRowAlignment GridCellRowAlignment( GridCellRef ref )
{ return [ref rowAlignment]; }

void GridCellSetRowAlignment( GridCellRef ref, NSGridRowAlignment alignment )
{ [ref setRowAlignment:alignment]; }

NSGridCellPlacement GridCellXPlacement( GridCellRef ref )
{ return [ref xPlacement]; }

void GridCellSetXPlacement( GridCellRef ref, NSGridCellPlacement placement )
{ [ref setXPlacement:placement]; }

NSGridCellPlacement GridCellYPlacement( GridCellRef ref )
{ return [ref yPlacement]; }

void GridCellSetYPlacement( GridCellRef ref, NSGridCellPlacement placement )
{ [ref setYPlacement:placement]; }


/*
 GridColumn
 */
// Instance properties
GridViewRef GridColumnGridView( GridColumnRef ref )
{ return [ref gridView]; }

BOOL GridColumnIsHidden( GridColumnRef ref )
{ return [ref isHidden]; }

void GridColumnSetHidden( GridColumnRef ref, BOOL flag )
{ [ref setHidden:flag]; }

CGFloat GridColumnLeadingPadding( GridColumnRef ref )
{ return [ref leadingPadding]; }

void GridColumnSetLeadingPadding( GridColumnRef ref, CGFloat padding )
{ [ref setLeadingPadding:padding]; }

NSInteger GridColumnNumberOfCells( GridColumnRef ref )
{ return [ref numberOfCells]; }

CGFloat GridColumnTrailingPadding( GridColumnRef ref )
{ return [ref trailingPadding]; }

void GridColumnSetTrailingPadding( GridColumnRef ref, CGFloat padding )
{ [ref setTrailingPadding:padding]; }

CGFloat GridColumnWidth( GridColumnRef ref )
{ return [ref width]; }

void GridColumnSetWidth( GridColumnRef ref, CGFloat width )
{ [ref setWidth:width]; }

NSGridCellPlacement GridColumnXPlacement( GridColumnRef ref )
{ return [ref xPlacement]; }

void GridColumnSetXPlacement( GridColumnRef ref, NSGridCellPlacement placement )
{ [ref setXPlacement:placement]; }

// Instance methods
GridCellRef GridColumnCellAtIndex( GridColumnRef ref, NSInteger index )
{ return [ref cellAtIndex:index]; }

void GridColumnMergeCells( GridColumnRef ref, CFRange range )
{ [ref mergeCellsInRange:NSRangeFromCFRange(range)]; }


/*
 GridRow
 */
// Details
NSInteger GridRowNumberOfCells( GridRowRef ref )
{ return [ref numberOfCells]; }

BOOL GridRowIsHidden( GridRowRef ref )
{ return [ref isHidden]; }

void GridRowSetHidden( GridRowRef ref, BOOL flag )
{ [ref setHidden:flag]; }

// Formatting
CGFloat GridRowTopPadding( GridRowRef ref )
{ return [ref topPadding]; }

void GridRowSetTopPadding( GridRowRef ref, CGFloat padding )
{ [ref setTopPadding:padding]; }

CGFloat GridRowBottomPadding( GridRowRef ref )
{ return [ref bottomPadding]; }

void GridRowSetBottomPadding( GridRowRef ref, CGFloat padding )
{ [ref setBottomPadding:padding]; }

CGFloat GridRowHeight( GridRowRef ref )
{ return [ref height]; }

void GridRowSetHeight( GridRowRef ref, CGFloat height )
{ [ref setHeight:height]; }

NSGridCellPlacement GridRowYPlacement( GridRowRef ref )
{ return [ref yPlacement]; }

void GridRowSetYPlacement( GridRowRef ref, NSGridCellPlacement placement )
{ [ref setYPlacement:placement]; }

// Grid view
GridViewRef GridRowGridView( GridRowRef ref )
{ return [ref gridView]; }

// Cells
GridCellRef GridRowCellAtIndex( GridRowRef ref, NSInteger index )
{ return [ref cellAtIndex:index]; }

// Merge
void GridRowMergeCells( GridRowRef ref, CFRange range )
{ [ref mergeCellsInRange:NSRangeFromCFRange(range)]; }



// Init
GridViewRef GridViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) {
        rect = NSMakeRect( 20, 20, 206, 102 );
    }

    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSGridView *view;
    if ( subclass ) {
        view = [[[GridViewSC alloc] initWithFrame:rect] autorelease];
    } else {
        view = [[[NSGridView alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [view setTag:inTag];
    
    return view;
}


/*
 gridview statement
 */
void CocoaGridView( NSInteger inTag, CGRect inRect, NSInteger inNumColumns, NSInteger inNumRows, CFTypeRef inViews, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSGridView *view = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !view ) {
            view = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( view ) {
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [view class], [NSGridView class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, view, [NSGridView class] ) ) {
                return;
            }

            // origin
            if ( !NSIsEmptyRect( rect ) ) {
                [view setFrame:rect];
            }
            
        } else {
            
            // subclass
            BOOL subclass = IsSubclassFlag();

            // origin
            if ( NSIsEmptyRect( rect ) ) {
                rect = NSMakeRect( 20, 20, 206, 102 );
            }
            
            // views
            NSArray *views = (NSArray *)inViews;
            
            // init
            if ( subclass ) {
                if ( inNumColumns > 0 && inNumRows > 0 ) {
                    view = [GridViewSC gridViewWithNumberOfColumns:inNumColumns rows:inNumRows];
                } else if ( views ) {
                    NSMutableArray *rows = [NSMutableArray arrayWithCapacity:0];
                    for ( NSArray *rowTags in views ) {
                        NSMutableArray *columns = [NSMutableArray arrayWithCapacity:0];
                        for ( NSNumber *tagNumber in rowTags ) {
                            NSInteger tag = [tagNumber integerValue];
                            NSView *view = CocoaViewWithTag( tag );
                            [columns addObject:view];
                        }
                        [rows addObject:columns];
                    }
                    view = [GridViewSC gridViewWithViews:rows];
                } else {
                    view = [[[GridViewSC alloc] initWithFrame:rect] autorelease];
                }
            } else {
                if ( inNumColumns > 0 && inNumRows > 0 ) {
                    view = [NSGridView gridViewWithNumberOfColumns:inNumColumns rows:inNumRows];
                } else if ( views ) {
                    NSMutableArray *rows = [NSMutableArray arrayWithCapacity:0];
                    for ( NSArray *rowTags in views ) {
                        NSMutableArray *columns = [NSMutableArray arrayWithCapacity:0];
                        for ( NSNumber *tagNumber in rowTags ) {
                            NSInteger tag = [tagNumber integerValue];
                            NSView *view;
                            if ( tag == 0 ) {
                                view = [NSGridCell emptyContentView];
                            } else {
                                view = CocoaViewWithTag( tag );
                            }
                            [columns addObject:view];
                        }
                        [rows addObject:columns];
                    }
                    
                    view = [NSGridView gridViewWithViews:rows];

                } else {
                    view = [[[NSGridView alloc] initWithFrame:rect] autorelease];
                }
            }
            
            // rect
            [view setFrame:rect];

            // default autoresizing
            [view setAutoresizingMask:NSViewMinYMargin];
            
            // tag
            [view setTag: inTag];
            
            // wndTag
            [view setWndTag:inWndTag];
            
            // add to window
            [contentView addSubview:view];
        }
        
        [view setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}


#endif// 101200

