/*
 IKImageView.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

@interface IKImageViewSC : IKImageView
@end

@interface IKImageView (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

// dialog events
enum {
    ikImageViewSelectionRectAdded = 4100,
    ikImageViewSelectionRectRemoved,
    ikImageViewSelectionRectChanged,
    ikImageViewImagePathChanged
};

@interface IKImageViewDelegate : NSObject
@end

// private
IKImageViewDelegate *IKImageViewDelegateObj( void );


// public
IKImageViewRef IKImageViewWithTag( NSInteger tag );
BOOL IKImageViewExists( NSInteger tag );

// Characteristics
CGFloat IKImageViewZoomFactor( NSInteger tag );
void IKImageViewSetZoomFactor( NSInteger tag, CGFloat value );
CGFloat IKImageViewRotationAngle( NSInteger tag );
void IKImageViewSetRotationAngle( NSInteger tag, CGFloat value );
CFStringRef IKImageViewCurrentToolMode( NSInteger tag );
void IKImageViewSetCurrentToolMode( NSInteger tag, CFStringRef value );
BOOL IKImageViewAutoresizes( NSInteger tag );
void IKImageViewSetAutoresizes( NSInteger tag, BOOL flag );
BOOL IKImageViewHasHorizontalScroller( NSInteger tag );
void IKImageViewSetHasHorizontalScroller( NSInteger tag, BOOL flag );
BOOL IKImageViewHasVerticalScroller( NSInteger tag );
void IKImageViewSetHasVerticalScroller( NSInteger tag, BOOL flag );
BOOL IKImageViewAutohidesScrollers( NSInteger tag );
void IKImageViewSetAutohidesScrollers( NSInteger tag, BOOL flag );
BOOL IKImageViewSupportsDragAndDrop( NSInteger tag );
void IKImageViewSetSupportsDragAndDrop( NSInteger tag, BOOL flag );
BOOL IKImageViewEditable( NSInteger tag );
void IKImageViewSetEditable( NSInteger tag, BOOL flag );
BOOL IKImageViewDoubleClickOpensImageEditPanel( NSInteger tag );
void IKImageViewSetDoubleClickOpensImageEditPanel( NSInteger tag, BOOL flag );
CIFilterRef IKImageViewImageCorrection( NSInteger tag );
void IKImageViewSetImageCorrection( NSInteger tag, CIFilterRef imageCorrection );
ColorRef IKImageViewBackgroundColor( NSInteger tag );
void IKImageViewSetBackgroundColor( NSInteger tag, ColorRef col );
CGSize IKImageViewImageSize( NSInteger tag );
CFDictionaryRef IKImageViewImageProperties( NSInteger tag );

// Get/set image
CGImageRef IKImageViewImage( NSInteger tag );
void IKImageViewSetImage( NSInteger tag, CGImageRef image, CFDictionaryRef properties );
void IKImageViewSetImageWithURL( NSInteger tag, CFURLRef url );

// Manipulate image
void IKImageViewSetImageRotationAngle( NSInteger tag, CGFloat angle, CGPoint centerPt );
void IKImageViewSetImageZoomFactor( NSInteger tag, CGFloat zoomFactor, CGPoint centerPt );
void IKImageViewZoomImageToFit( NSInteger tag );
void IKImageViewZoomImageToActualSize( NSInteger tag );
void IKImageViewZoomImageToRect( NSInteger tag, CGRect r );
void IKImageViewZoomIn( NSInteger tag );
void IKImageViewZoomOut( NSInteger tag );
void IKImageViewCrop( NSInteger tag );
void IKImageViewFlipImageHorzontal( NSInteger tag );
void IKImageViewFlipImageVertical( NSInteger tag );
void IKImageViewRotateImageLeft( NSInteger tag );
void IKImageViewRotateImageRight( NSInteger tag );

// Core animation
void IKImageViewSetOverlayForType( NSInteger tag, CALayerRef layer, CFStringRef type );
CALayerRef IKImageViewOverlayForType( NSInteger tag, CFStringRef type );

// Scrolling
void IKImageViewScrollToPoint( NSInteger tag, CGPoint pt );
void IKImageViewScrollToRect( NSInteger tag, CGRect r );

// Convert points and rects
CGPoint IKImageViewConvertViewPointToImagePoint( NSInteger tag, CGPoint viewPt );
CGRect IKImageViewConvertViewRectToImageRect( NSInteger tag, CGRect viewRect );
CGPoint IKImageViewConvertImagePointToViewPoint( NSInteger tag, CGPoint imagePt );
CGRect IKImageViewConvertImageRectToViewRect( NSInteger tag, CGRect imageRect );


// Init
IKImageViewRef IKImageViewInit( NSInteger tag, CGRect r );


// ikimageview statement
void CocoaIKImageView( NSInteger inTag, CGRect inRect, NSInteger inWndTag );

