/*
 IKImageView.m

 Bernie Wylde
 */

#import "IKImageView.h"

#pragma mark - subclass
@implementation IKImageViewSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end


@implementation IKImageView (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end


@implementation IKImageViewDelegate
- (void)selectionRectAdded:(IKImageView *)imageView {
    CocoaControlCallUserDefinedOnDialogOrCallback( ikImageViewSelectionRectAdded, imageView, nil );
}

- (void)selectionRectRemoved:(IKImageView *)imageView {
    CocoaControlCallUserDefinedOnDialogOrCallback( ikImageViewSelectionRectRemoved, imageView, nil );
}

- (void)selectionRectChanged:(IKImageView *)imageView {
    CocoaControlCallUserDefinedOnDialogOrCallback( ikImageViewSelectionRectChanged, imageView, nil );
}

- (void)imagePathChanged:(NSString *)imagePath {
    CocoaControlCallUserDefinedOnDialogOrCallback( ikImageViewImagePathChanged, nil, imagePath );
}
@end

IKImageViewDelegate *IKImageViewDelegateObj( void )
{
    static IKImageViewDelegate *sIKImageViewDelegate = nil;
    if ( !sIKImageViewDelegate ) sIKImageViewDelegate = [[IKImageViewDelegate alloc] init];
    return sIKImageViewDelegate;
}


IKImageViewRef IKImageViewWithTag( NSInteger tag )
{ return (IKImageView *)CocoaValidateViewKindOfClass( tag, [IKImageView class] ); }

BOOL IKImageViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [IKImageView class] ); }

// Characteristics
CGFloat IKImageViewZoomFactor( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view zoomFactor];
    return 0.0;
}

void IKImageViewSetZoomFactor( NSInteger tag, CGFloat value )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setZoomFactor:value];
}

CGFloat IKImageViewRotationAngle( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view rotationAngle];
    return 0.0;
}

void IKImageViewSetRotationAngle( NSInteger tag, CGFloat value )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setRotationAngle:value];
}

CFStringRef IKImageViewCurrentToolMode( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return (CFStringRef)[view currentToolMode];
    return nil;
}

void IKImageViewSetCurrentToolMode( NSInteger tag, CFStringRef value )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setCurrentToolMode:(NSString *)value];
}
BOOL IKImageViewAutoresizes( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view autoresizes];
    return NO;
}

void IKImageViewSetAutoresizes( NSInteger tag, BOOL flag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setAutoresizes:flag];
}

BOOL IKImageViewHasHorizontalScroller( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view hasHorizontalScroller];
    return NO;
}

void IKImageViewSetHasHorizontalScroller( NSInteger tag, BOOL flag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setHasHorizontalScroller:flag];
}

BOOL IKImageViewHasVerticalScroller( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view hasVerticalScroller];
    return NO;
}

void IKImageViewSetHasVerticalScroller( NSInteger tag, BOOL flag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setHasVerticalScroller:flag];
}

BOOL IKImageViewAutohidesScrollers( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view autohidesScrollers];
    return NO;
}

void IKImageViewSetAutohidesScrollers( NSInteger tag, BOOL flag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setAutohidesScrollers:flag];
}

BOOL IKImageViewSupportsDragAndDrop( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view supportsDragAndDrop];
    return NO;
}

void IKImageViewSetSupportsDragAndDrop( NSInteger tag, BOOL flag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setSupportsDragAndDrop:flag];
}

BOOL IKImageViewEditable( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view editable];
    return NO;
}

void IKImageViewSetEditable( NSInteger tag, BOOL flag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setEditable:flag];
}

BOOL IKImageViewDoubleClickOpensImageEditPanel( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view doubleClickOpensImageEditPanel];
    return NO;
}

void IKImageViewSetDoubleClickOpensImageEditPanel( NSInteger tag, BOOL flag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setDoubleClickOpensImageEditPanel:flag];
}

CIFilterRef IKImageViewImageCorrection( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view imageCorrection];
    return nil;
}

void IKImageViewSetImageCorrection( NSInteger tag, CIFilterRef imageCorrection )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setImageCorrection:imageCorrection];
}

ColorRef IKImageViewBackgroundColor( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view backgroundColor];
    return nil;
}

void IKImageViewSetBackgroundColor( NSInteger tag, ColorRef col )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setBackgroundColor:col];
}

CGSize IKImageViewImageSize( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return NSSizeToCGSize([view imageSize]);
    return CGSizeZero;
}

CFDictionaryRef IKImageViewImageProperties( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return (CFDictionaryRef)[view imageProperties];
    return nil;
}

// Get/set image
CGImageRef IKImageViewImage( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view image];
    return nil;
}

void IKImageViewSetImage( NSInteger tag, CGImageRef image, CFDictionaryRef properties )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setImage:image imageProperties:(NSDictionary *)properties];
}

void IKImageViewSetImageWithURL( NSInteger tag, CFURLRef url )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setImageWithURL:(NSURL *)url];
}

// Manipulate image
void IKImageViewSetImageRotationAngle( NSInteger tag, CGFloat angle, CGPoint centerPt )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setRotationAngle:angle centerPoint:NSPointFromCGPoint(centerPt)];
}

void IKImageViewSetImageZoomFactor( NSInteger tag, CGFloat zoomFactor, CGPoint centerPt )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setImageZoomFactor:zoomFactor centerPoint:NSPointFromCGPoint(centerPt)];
}

void IKImageViewZoomImageToFit( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view zoomImageToFit:nil];
}

void IKImageViewZoomImageToActualSize( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view zoomImageToActualSize:nil];
}

void IKImageViewZoomImageToRect( NSInteger tag, CGRect r )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view zoomImageToRect:NSRectFromCGRect(r)];
}

void IKImageViewZoomIn( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view zoomIn:nil];
}

void IKImageViewZoomOut( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view zoomOut:nil];
}

void IKImageViewCrop( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view crop:nil];
}

void IKImageViewFlipImageHorzontal( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view flipImageHorizontal:nil];
}

void IKImageViewFlipImageVertical( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view flipImageVertical:nil];
}

void IKImageViewRotateImageLeft( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view rotateImageLeft:nil];
}

void IKImageViewRotateImageRight( NSInteger tag )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view rotateImageRight:nil];
}

// Core animation
void IKImageViewSetOverlayForType( NSInteger tag, CALayerRef layer, CFStringRef type )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view setOverlay:layer forType:(NSString *)type];
}

CALayerRef IKImageViewOverlayForType( NSInteger tag, CFStringRef type )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return [view overlayForType:(NSString *)type];
    return nil;
}

// Scrolling
void IKImageViewScrollToPoint( NSInteger tag, CGPoint pt )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view scrollToPoint:NSPointFromCGPoint(pt)];
}

void IKImageViewScrollToRect( NSInteger tag, CGRect r )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) [view scrollToRect:NSRectFromCGRect(r)];
}

// Convert points and rects
CGPoint IKImageViewConvertViewPointToImagePoint( NSInteger tag, CGPoint viewPt )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return NSPointToCGPoint([view convertViewPointToImagePoint:NSPointFromCGPoint(viewPt)]);
    return CGPointZero;
}

CGRect IKImageViewConvertViewRectToImageRect( NSInteger tag, CGRect viewRect )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return NSRectToCGRect([view convertViewRectToImageRect:NSRectFromCGRect(viewRect)]);
    return CGRectZero;
}

CGPoint IKImageViewConvertImagePointToViewPoint( NSInteger tag, CGPoint imagePt )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return NSPointToCGPoint([view convertImagePointToViewPoint:NSPointFromCGPoint(imagePt)]);
    return CGPointZero;
}

CGRect IKImageViewConvertImageRectToViewRect( NSInteger tag, CGRect imageRect )
{
    IKImageView *view = IKImageViewWithTag( tag );
    if ( view ) return NSRectToCGRect([view convertImageRectToViewRect:NSRectFromCGRect(imageRect)]);
    return CGRectZero;
}




// Init
IKImageViewRef IKImageViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 300, 169 );

    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    IKImageView *view;
    if ( subclass ) {
        view = [[[IKImageViewSC alloc] initWithFrame:rect] autorelease];
    } else {
        view = [[[IKImageView alloc] initWithFrame:rect] autorelease];
    }
    
    // other default settings
    [view setAutoresizes:YES];
    [view setSupportsDragAndDrop:YES];
    [view setDoubleClickOpensImageEditPanel:YES];

    // tag
    [view setTag:inTag];
    
    // delegate
    [view setDelegate:IKImageViewDelegateObj()];

    return view;
}


#pragma mark - ikimageview statement
void CocoaIKImageView( NSInteger inTag, CGRect inRect, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        IKImageView *view = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !view ) {
            view = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( view ) {    // already exists
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [view class], [IKImageView class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, view, [IKImageView class] ) ) {
                return;
            }

            // rect
            if ( !NSIsEmptyRect( rect ) ) [view setFrame:rect];
            
        } else {
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 300, 169 );
            
            // init
//            if ( inSubclass ) {
//                view = [[IKImageViewSC alloc] initWithFrame:rect];
//            } else {
//                view = [[IKImageView alloc] initWithFrame:rect];
//            }
            
            view = IKImageViewInit( inTag, inRect );
            
            // default autoresizing
            [view setAutoresizingMask:NSViewMinYMargin];
            
            // other default settings
//            [view setAutoresizes:YES];
//            [view setSupportsDragAndDrop:YES];
//            [view setDoubleClickOpensImageEditPanel:YES];
            
            // tag
//            [view setTag:inTag];
            
            // wndTag
            [view setWndTag:inWndTag];
            
            // delegate
//            [view setDelegate:IKImageViewDelegateObj()];
            
            // add to window
            [contentView addSubview:view];
//            [view release];
        }
        
        [view setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}
