/*
 Image.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef BOOL (*CocoaImageDrawingHandlerType)(CGRect,void*);

// Images by name
ImageRef ImageNamed( CFStringRef name );
void ImageSetName( ImageRef ref, CFStringRef name );
CFStringRef ImageName( ImageRef ref );

// Dynamically drawn image
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
ImageRef ImageWithDrawingHandler( CGSize size, Boolean flipped, CocoaImageDrawingHandlerType handler, void *userData );
#endif

// Image from resource file
ImageRef ImageByReferencingURL( CFURLRef url );
ImageRef ImageWithContentsOfURL( CFURLRef url );

// create from existing data
ImageRef ImageWithData( CFDataRef dta );
ImageRef ImageWithDataIgnoringOrientation( CFDataRef dta );
ImageRef ImageWithCGImage( CGImageRef image, CGSize size );
ImageRef ImageWithPasteboard( CocoaPasteboardRef pb );

// Create empty image
ImageRef ImageWithSize( CGSize size );

// Attributes
CGSize ImageSize( ImageRef ref );
void ImageSetSize( ImageRef ref, CGSize size );
void ImageSetTemplate( ImageRef ref, Boolean flag );
Boolean ImageIsTemplate( ImageRef ref );

// Determine supported image types
Boolean ImageCanInitWithPasteboard( CocoaPasteboardRef pb );
CFArrayRef ImageTypes( void );
CFArrayRef ImageUnfilteredTypes( void );

// Image representation
void ImageAddRepresentation( ImageRef ref, ImageRepRef rep );
void ImageAddRepresentations( ImageRef ref, CFArrayRef reps );
CFArrayRef ImageRepresentations( ImageRef ref );
void ImageRemoveRepresentation( ImageRef ref, ImageRepRef rep );
ImageRepRef ImageBestRepresentationForRect( ImageRef ref, CGRect r, GraphicsContextRef ctx, CFDictionaryRef hints );

// Image representation selection criteria
Boolean ImagePrefersColorMatch( ImageRef ref );
void ImageSetPrefersColorMatch( ImageRef ref, Boolean flag );
Boolean ImageUsesEPSOnResolutionMismatch( ImageRef ref );
void ImageSetUsesEPSOnResolutionMismatch( ImageRef ref, Boolean flag );
Boolean ImageMatchesOnMultipleResolution( ImageRef ref );
void ImageSetMatchesOnMultipleResolution( ImageRef ref, Boolean flag );

// Drawing
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ImageDrawInRect( ImageRef ref, CGRect rect );
#endif
void ImageDrawAtPoint( ImageRef ref, CGPoint pt, CGRect fromRect, NSCompositingOperation operation, CGFloat fraction );
void ImageDrawInRectFromRect( ImageRef ref, CGRect rect, CGRect fromRect, NSCompositingOperation op, CGFloat fraction );
void ImageDrawInRectFromRectRespectFlipped( ImageRef ref, CGRect rect, CGRect fromRect, NSCompositingOperation op, CGFloat fraction, Boolean respectFlipped, CFDictionaryRef hints );
Boolean ImageDrawRepresentationInRect( ImageRef ref, ImageRepRef rep, CGRect rect );

// Drawing options
Boolean ImageIsValid( ImageRef ref );
ColorRef ImageBackgroundColor( ImageRef ref );
void ImageSetBackgroundColor( ImageRef ref, ColorRef col );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSEdgeInsets ImageCapInsets( ImageRef ref );
void ImageSetCapInsets( ImageRef ref, NSEdgeInsets insets );
NSImageResizingMode ImageResizingMode( ImageRef ref );
void ImageSetResizingMode( ImageRef ref, NSImageResizingMode mode );
#endif

// Focus
void ImageLockFocus( ImageRef ref );
void ImageLockFocusFlipped( ImageRef ref, Boolean flag );
void ImageUnlockFocus( ImageRef ref );

// Alignment
CGRect ImageAlignmentRect( ImageRef ref );
void ImageSetAlignmentRect( ImageRef ref, CGRect rect );

// Caching options
NSImageCacheMode ImageCacheMode( ImageRef ref );
void ImageSetCacheMode( ImageRef ref, NSImageCacheMode mode );
void ImageRecache( ImageRef ref );

// TIFF
CFDataRef ImageTIFFRepresentation( ImageRef ref );
CFDataRef ImageTIFFRepresentationUsingCompression( ImageRef ref, NSTIFFCompression compression, float factor );

// CGImage
CGImageRef ImageCGImageForProposedRect( ImageRef ref, CGRect *rect, GraphicsContextRef context, CFDictionaryRef hints );

// Incremental loads
void ImageCancelIncrementalLoad( ImageRef ref );

// Hit testing
Boolean ImageHitTestRect( ImageRef ref, CGRect testRectDestSpace, CGRect imageRectDestSpace, GraphicsContextRef ctx, CFDictionaryRef hints, Boolean flipped );

// Image accessibility
CFStringRef ImageAccessibilityDescription( ImageRef ref );
void ImageSetAccessibilityDescription( ImageRef ref, CFStringRef description );

// Core animation
CFTypeRef ImageLayerContentsForContentsScale( ImageRef ref, CGFloat scale );
CGFloat ImageRecommendedLayerContentsScale( ImageRef ref, CGFloat preferredScale );

// Axis matching
Boolean ImageMatchesOnlyOnBestFittingAxis( ImageRef ref );
void ImageSetMatchesOnlyOnBestFittingAxis( ImageRef ref, Boolean flag );

