/*
 ImageView.m

 Bernie Wylde
 */

#import "ImageView.h"

#pragma mark - subclass
@implementation ImageViewSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end


NSImageView *ImageViewWithTag( NSInteger tag )
{ return (NSImageView *)CocoaValidateViewKindOfClass( tag, [NSImageView class] ); }

BOOL ImageViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSImageView class] ); }


// - image
ImageRef ImageViewImage( NSInteger tag )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) return [view image];
    return nil;
}

void ImageViewSetImage( NSInteger tag, void *inImage )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) {
        NSImage *image = FixImageValue( inImage );
        if ( image ) [view setImage:image];
    }
}

void ImageViewSetImageNamed( NSInteger tag, CFStringRef imageName )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) {
        NSImage *image = CocoaImageResource( (NSString *)imageName );
        if ( image ) [view setImage:image];
    }
}

// - visual characteristics -
NSImageFrameStyle ImageViewFrameStyle( NSInteger tag )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) return [view imageFrameStyle];
    return 0;
}

void ImageViewSetFrameStyle( NSInteger tag, NSImageFrameStyle style )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) [view setImageFrameStyle:style];
}

NSImageAlignment ImageViewAlignment( NSInteger tag )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) return [view imageAlignment];
    return 0;
}

void ImageViewSetAlignment( NSInteger tag, NSImageAlignment alignment )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) [view setImageAlignment:alignment];
}

NSImageScaling ImageViewScaling( NSInteger tag )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) return [view imageScaling];
    return 0;
}

void ImageViewSetScaling( NSInteger tag, NSImageScaling scaling )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) [view setImageScaling:scaling];
}

Boolean ImageViewAnimates( NSInteger tag )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) return [view animates];
    return NO;
}

void ImageViewSetAnimates( NSInteger tag, Boolean flag )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) [view setAnimates:flag];
}

// - user events -
void ImageViewSetEditable( NSInteger tag, Boolean flag )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) [view setEditable:flag];
}

void ImageViewSetAllowsCutCopyPaste( NSInteger tag, Boolean flag )
{
    NSImageView *view = ImageViewWithTag( tag );
    if ( view ) [view setAllowsCutCopyPaste:flag];
}


// Init
ImageViewRef ImageViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 48, 48 );

    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSImageView *view;
    if ( subclass ) {
        view = [[[ImageViewSC alloc] initWithFrame:rect] autorelease];
    } else {
        view = [[[NSImageView alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [view setTag:inTag];
    
    // target/action
    [view setTarget:CocoaControlTargetObj()];
    [view setAction:@selector(controlAction:)];

    return view;
}


// Statement
void CocoaImageView( NSInteger inTag, NSInteger inEnabled, void *inImage, CGRect inRect, NSInteger inScaling, NSInteger inAlignment, NSInteger inFrameStyle, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    // imageName, imagePath, NSImage
    NSImage *image = FixImageValue( inImage );
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSImageView *view = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !view ) {
            view = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( view ) {    // already exists
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [view class], [NSImageView class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, view, [NSImageView class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [view setEnabled:inEnabled];
            
            // image
            if ( image ) [view setImage:image];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [view setFrame:rect];
            
            // scaling
            if ( inScaling != kFBParamMissing ) [view setImageScaling:inScaling];
            
            // alignment
            if ( inAlignment != kFBParamMissing ) [view setImageAlignment:inAlignment];
            
            // frame style
            if ( inFrameStyle != kFBParamMissing ) [view setImageFrameStyle:inFrameStyle];
            
        } else {
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 48, 48 );
            
            // init
//            if ( inSubclass ) {
//                view = [[ImageViewSC alloc] initWithFrame:rect];
//            } else {
//                view = [[NSImageView alloc] initWithFrame:rect];
//            }
            view = ImageViewInit( inTag, inRect );
            
            // default autoresizing
            [view setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [view setTag:inTag];
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [view setEnabled:inEnabled];
            
            // image
            if ( image ) [view setImage:image];
            
            // scaling
            if ( inScaling != kFBParamMissing ) [view setImageScaling:inScaling];
            
            // alignment
            if ( inAlignment != kFBParamMissing ) [view setImageAlignment:inAlignment];
            
            // frame style
            if ( inFrameStyle != kFBParamMissing ) [view setImageFrameStyle:inFrameStyle];
            
            // wndTag
            [view setWndTag:inWndTag];
            
            // target/action
//            [view setTarget:CocoaControlTargetObj()];
//            [view setAction:@selector(controlAction:)];
            
            // add to window
            [contentView addSubview:view];
//            [view release];
        }
        
        [view setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}
