/*
 IndexSet.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef BOOL (*CocoaIndexSetRangeCallbackType)(IndexSetRef,CFRange,BOOL*,void*);
typedef BOOL (*CocoaIndexSetIndexCallbackType)(IndexSetRef,NSUInteger,BOOL*,void*);
typedef BOOL (*CocoaIndexSetIndexPredicateType)(IndexSetRef,NSUInteger,BOOL*,void*);

IndexSetRef IndexSetWithIndex( NSUInteger value );
IndexSetRef IndexSetWithIndexesInRange( CFRange range );

Boolean IndexSetContainsIndex( IndexSetRef set, NSUInteger index );
Boolean IndexSetContainsIndexes( IndexSetRef set, IndexSetRef subset );
Boolean IndexSetContainsIndexesInRange( IndexSetRef set, CFRange range );
Boolean IndexSetIntersectsIndexesInRange( IndexSetRef set, CFRange range );
NSUInteger IndexSetCount( IndexSetRef set );
NSUInteger IndexSetCountOfIndexesInRange( IndexSetRef set, CFRange range );
NSUInteger IndexSetIndexPassingTest( IndexSetRef set, CocoaIndexSetIndexPredicateType predicate, void *userData );
IndexSetRef IndexSetIndexesPassingTest( IndexSetRef set, CocoaIndexSetIndexPredicateType predicate, void *userData );
NSUInteger IndexSetIndexWithOptionsPassingTest( IndexSetRef set, NSEnumerationOptions options, CocoaIndexSetIndexPredicateType predicate, void *userData );
IndexSetRef IndexSetIndexesWithOptionsPassingTest( IndexSetRef set, NSEnumerationOptions options, CocoaIndexSetIndexPredicateType predicate, void *userData );
NSUInteger IndexSetIndexInRangePassingTest( IndexSetRef set, CFRange range, NSEnumerationOptions options, CocoaIndexSetIndexPredicateType predicate, void *userData );
IndexSetRef IndexSetIndexesInRangePassingTest( IndexSetRef set, CFRange range, NSEnumerationOptions options, CocoaIndexSetIndexPredicateType predicate, void *userData );

// enumerate content
void IndexSetEnumerateRangesInRange( IndexSetRef set, CFRange inRange, NSEnumerationOptions options, CocoaIndexSetRangeCallbackType callback, void *userData );
void IndexSetEnumerateRanges( IndexSetRef set, CocoaIndexSetRangeCallbackType callback, void *userData );
void IndexSetEnumerateRangesWithOptions( IndexSetRef set, NSEnumerationOptions options, CocoaIndexSetRangeCallbackType callback, void *userData );

Boolean IndexSetIsEqual( IndexSetRef set1, IndexSetRef set2 );

NSUInteger IndexSetFirstIndex( IndexSetRef set );
NSUInteger IndexSetLastIndex( IndexSetRef set );
NSUInteger IndexSetIndexLessThan( IndexSetRef set, NSUInteger value );
NSUInteger IndexSetIndexLessThanOrEqualTo( IndexSetRef set, NSUInteger value );
NSUInteger IndexSetIndexGreaterThanOrEqualTo( IndexSetRef set, NSUInteger value );
NSUInteger IndexSetIndexGreaterThan( IndexSetRef set, NSUInteger value );

// enumerate indexes
void IndexSetEnumerateIndexes( IndexSetRef set, CocoaIndexSetIndexCallbackType callback, void *userData );
void IndexSetEnumerateIndexesWithOptions( IndexSetRef set, NSEnumerationOptions options, CocoaIndexSetIndexCallbackType callback, void *userData );
void IndexSetEnumerateIndexesInRange( IndexSetRef set, CFRange range, NSEnumerationOptions options, CocoaIndexSetIndexCallbackType callback, void *userData );

// --- MutableIndexSet ---
MutableIndexSetRef IndexSetInit( void );
void IndexSetAddIndex( MutableIndexSetRef set, NSUInteger index );
void IndexSetAddIndexes( MutableIndexSetRef set, IndexSetRef otherSet );
void IndexSetAddIndexesInRange( MutableIndexSetRef set, CFRange range );
void IndexSetRemoveIndex( MutableIndexSetRef set, NSUInteger index );
void IndexSetRemoveIndexes( MutableIndexSetRef set, IndexSetRef otherSet );
void IndexSetRemoveAllIndexes( MutableIndexSetRef set );
void IndexSetRemoveIndexesInRange( MutableIndexSetRef set, CFRange range );
void IndexSetShiftIndexes( MutableIndexSetRef set, NSUInteger index, NSInteger delta );

