/*
 IndexSet.m

 Bernie Wylde
 */

#import "IndexSet.h"

IndexSetRef IndexSetWithIndex( NSUInteger value )
{ return [NSIndexSet indexSetWithIndex:value]; }

IndexSetRef IndexSetWithIndexesInRange( CFRange range )
{ return [NSIndexSet indexSetWithIndexesInRange:NSRangeFromCFRange(range)]; }

Boolean IndexSetContainsIndex( IndexSetRef set, NSUInteger index )
{ return [set containsIndex:index]; }

Boolean IndexSetContainsIndexes( IndexSetRef set, IndexSetRef subset )
{ return [set containsIndexes:subset]; }

Boolean IndexSetContainsIndexesInRange( IndexSetRef set, CFRange range )
{ return [set containsIndexesInRange:NSRangeFromCFRange(range)]; }

Boolean IndexSetIntersectsIndexesInRange( IndexSetRef set, CFRange range )
{ return [set intersectsIndexesInRange:NSRangeFromCFRange(range)]; }

NSUInteger IndexSetCount( IndexSetRef set )
{ return [set count]; }

NSUInteger IndexSetCountOfIndexesInRange( IndexSetRef set, CFRange range )
{ return [set countOfIndexesInRange:NSRangeFromCFRange(range)]; }

NSUInteger IndexSetIndexPassingTest( IndexSetRef set, CocoaIndexSetIndexPredicateType predicate, void *userData )
{
    return [set indexPassingTest:^( NSUInteger idx, BOOL *stop ) {
        return (*predicate)(set,idx,stop,userData);
    }];
}

IndexSetRef IndexSetIndexesPassingTest( IndexSetRef set, CocoaIndexSetIndexPredicateType predicate, void *userData )
{
    return [set indexesPassingTest:^( NSUInteger idx, BOOL *stop ) {
        return (*predicate)(set,idx,stop,userData);
    }];
}

NSUInteger IndexSetIndexWithOptionsPassingTest( IndexSetRef set, NSEnumerationOptions options, CocoaIndexSetIndexPredicateType predicate, void *userData )
{
    return [set indexWithOptions:options passingTest:^( NSUInteger idx, BOOL *stop ) {
        return (*predicate)(set,idx,stop,userData);
    }];
}

IndexSetRef IndexSetIndexesWithOptionsPassingTest( IndexSetRef set, NSEnumerationOptions options, CocoaIndexSetIndexPredicateType predicate, void *userData )
{
    return [set indexesWithOptions:options passingTest:^( NSUInteger idx, BOOL *stop) {
        return (*predicate)(set,idx,stop,userData);
    }];
}

NSUInteger IndexSetIndexInRangePassingTest( IndexSetRef set, CFRange range, NSEnumerationOptions options, CocoaIndexSetIndexPredicateType predicate, void *userData )
{
    return [set indexInRange:NSRangeFromCFRange(range) options:options passingTest:^( NSUInteger idx, BOOL *stop) {
        return (*predicate)(set,idx,stop,userData);
    }];
}

IndexSetRef IndexSetIndexesInRangePassingTest( IndexSetRef set, CFRange range, NSEnumerationOptions options, CocoaIndexSetIndexPredicateType predicate, void *userData )
{
    return [set indexesInRange:NSRangeFromCFRange(range) options:options passingTest:^( NSUInteger idx, BOOL *stop) {
        return (*predicate)(set,idx,stop,userData);
    }];
}

// enumerate content
void IndexSetEnumerateRangesInRange( IndexSetRef set, CFRange inRange, NSEnumerationOptions options, CocoaIndexSetRangeCallbackType callback, void *userData )
{
    [set enumerateRangesInRange:NSRangeFromCFRange(inRange) options:options usingBlock:^(NSRange range, BOOL *stop) {
        (*callback)(set,NSRangeToCFRange(range),stop,userData);
    }];
}

void IndexSetEnumerateRanges( IndexSetRef set, CocoaIndexSetRangeCallbackType callback, void *userData )
{
    [set enumerateRangesUsingBlock:^(NSRange range, BOOL *stop) {
        (*callback)(set,NSRangeToCFRange(range),stop,userData);
    }];
}

void IndexSetEnumerateRangesWithOptions( IndexSetRef set, NSEnumerationOptions options, CocoaIndexSetRangeCallbackType callback, void *userData )
{
    [set enumerateRangesWithOptions:options usingBlock:^(NSRange range, BOOL *stop) {
        (*callback)(set,NSRangeToCFRange(range),stop,userData);
    }];
}

Boolean IndexSetIsEqual( IndexSetRef set1, IndexSetRef set2 )
{ return [set1 isEqualToIndexSet:set2]; }

NSUInteger IndexSetFirstIndex( IndexSetRef set )
{ return [set firstIndex]; }

NSUInteger IndexSetLastIndex( IndexSetRef set )
{ return [set lastIndex]; }

NSUInteger IndexSetIndexLessThan( IndexSetRef set, NSUInteger value )
{ return [set indexLessThanIndex:value]; }

NSUInteger IndexSetIndexLessThanOrEqualTo( IndexSetRef set, NSUInteger value )
{ return [set indexLessThanOrEqualToIndex:value]; }

NSUInteger IndexSetIndexGreaterThanOrEqualTo( IndexSetRef set, NSUInteger value )
{ return [set indexGreaterThanOrEqualToIndex:value]; }

NSUInteger IndexSetIndexGreaterThan( IndexSetRef set, NSUInteger value )
{ return [set indexGreaterThanIndex:value]; }

// enumerate indexes
void IndexSetEnumerateIndexes( IndexSetRef set, CocoaIndexSetIndexCallbackType callback, void *userData )
{
    [set enumerateIndexesUsingBlock:^(NSUInteger index, BOOL *stop) {
        (*callback)(set,index,stop,userData);
    }];
}

void IndexSetEnumerateIndexesWithOptions( IndexSetRef set, NSEnumerationOptions options, CocoaIndexSetIndexCallbackType callback, void *userData )
{
    [set enumerateIndexesWithOptions:options usingBlock:^(NSUInteger index, BOOL *stop) {
        (*callback)(set,index,stop,userData);
    }];
}

void IndexSetEnumerateIndexesInRange( IndexSetRef set, CFRange range, NSEnumerationOptions options, CocoaIndexSetIndexCallbackType callback, void *userData )
{
    [set enumerateIndexesInRange:NSRangeFromCFRange(range) options:options usingBlock:^(NSUInteger index, BOOL *stop) {
        (*callback)(set,index,stop,userData);
    }];
}

// - mutable -
MutableIndexSetRef IndexSetInit( void )
{ return [NSMutableIndexSet indexSet]; }

void IndexSetAddIndex( MutableIndexSetRef set, NSUInteger index )
{ [set addIndex:index]; }

void IndexSetAddIndexes( MutableIndexSetRef set, IndexSetRef otherSet )
{ [set addIndexes:otherSet]; }

void IndexSetAddIndexesInRange( MutableIndexSetRef set, CFRange range )
{ [set addIndexesInRange:NSRangeFromCFRange(range)]; }

void IndexSetRemoveIndex( MutableIndexSetRef set, NSUInteger index )
{ [set removeIndex:index]; }

void IndexSetRemoveIndexes( MutableIndexSetRef set, IndexSetRef otherSet )
{ [set removeIndexes:otherSet]; }

void IndexSetRemoveAllIndexes( MutableIndexSetRef set )
{ [set removeAllIndexes]; }

void IndexSetRemoveIndexesInRange( MutableIndexSetRef set, CFRange range )
{ [set removeIndexesInRange:NSRangeFromCFRange(range)]; }

void IndexSetShiftIndexes( MutableIndexSetRef set, NSUInteger index, NSInteger delta )
{ [set shiftIndexesStartingAtIndex:index by:delta]; }

