/*
 KeyedArchiver.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// Archiving
CFDataRef KeyedArchiverArchivedDataWithRootObject( CFTypeRef obj );
Boolean KeyedArchiverArchiveRootObjectToURL( CFTypeRef obj, CFURLRef url );
void KeyedArchiverFinishEncoding( KeyedArchiverRef ref );
NSPropertyListFormat KeyedArchiverOutputFormat( KeyedArchiverRef ref );
void KeyedArchiverSetOutputFormat( KeyedArchiverRef ref, NSPropertyListFormat format );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
Boolean KeyedArchiverRequiresSecureCoding( KeyedArchiverRef ref );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void KeyedArchiverSetRequiresSecureCoding( KeyedArchiverRef ref, Boolean flag );
#endif

// Encoding
void KeyedArchiverEncodeBool( KeyedArchiverRef ref, Boolean value, CFStringRef key );
void KeyedArchiverEncodeBytes( KeyedArchiverRef ref, void *bytes, NSUInteger length, CFStringRef key );
void KeyedArchiverEncodeConditionalObject( KeyedArchiverRef ref, CFTypeRef obj, CFStringRef key );
void KeyedArchiverEncodeDouble( KeyedArchiverRef ref, double value, CFStringRef key );
void KeyedArchiverEncodeFloat( KeyedArchiverRef ref, float value, CFStringRef key );
void KeyedArchiverEncodeInt32( KeyedArchiverRef ref, SInt32 value, CFStringRef key );
void KeyedArchiverEncodeInt64( KeyedArchiverRef ref, SInt64 value, CFStringRef key );
void KeyedArchiverEncodeObject( KeyedArchiverRef ref, CFTypeRef obj, CFStringRef key );

// Classes and class names
void KeyedArchiverSetClassNameForClass( CFStringRef codedName, ClassRef forClass );
CFStringRef KeyedArchiverClassNameForClass( ClassRef forClass );
void KeyedArchiverSetClassName( KeyedArchiverRef ref, CFStringRef codedName, ClassRef forClass );
CFStringRef KeyedArchiverClassName( KeyedArchiverRef ref, ClassRef forClass );

// Initializers
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
KeyedArchiverRef KeyedArchiverRequiringSecureCoding( Boolean flag );
#endif

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFDataRef KeyedArchiverEncodedData( KeyedArchiverRef ref );
#endif

// Type methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
CFDataRef KeyedArchiverArchivedDataRequiringSecureCoding( CFTypeRef obj, Boolean secureCoding, ErrorRef *err );
#endif




/*
 KeyedUnarchiver
 */

// Unarchiving
CFTypeRef KeyedUnarchiverUnarchiveObjectWithData( CFDataRef dta );
CFTypeRef KeyedUnarchiverUnarchiveObjectWithURL( CFURLRef url );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
Boolean KeyedUnarchiverRequiresSecureCoding( KeyedUnarchiverRef ref );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void KeyedUnarchiverSetRequiresSecureCoding( KeyedUnarchiverRef ref, Boolean flag );
#endif

// Decoding
Boolean KeyedUnarchiverContainsValue( KeyedUnarchiverRef ref, CFStringRef key );
Boolean KeyedUnarchiverDecodeBool( KeyedUnarchiverRef ref, CFStringRef key );
const uint8_t *KeyedUnarchiverDecodeBytes( KeyedUnarchiverRef ref, CFStringRef key, NSUInteger *returnedLength );
double KeyedUnarchiverDecodeDouble( KeyedUnarchiverRef ref, CFStringRef key );
float KeyedUnarchiverDecodeFloat( KeyedUnarchiverRef ref, CFStringRef key );
SInt32 KeyedUnarchiverDecodeInt32( KeyedUnarchiverRef ref, CFStringRef key );
SInt64 KeyedUnarchiverDecodeInt64( KeyedUnarchiverRef ref, CFStringRef key );
CFTypeRef KeyedUnarchiverDecodeObject( KeyedUnarchiverRef ref, CFStringRef key );
void KeyedUnarchiverFinishDecoding( KeyedUnarchiverRef ref );

// Classes
void KeyedUnarchiverSetClassForClassName( ClassRef class, CFStringRef codedName );
ClassRef KeyedUnarchiverClassForClassName( CFStringRef codedName );
void KeyedUnarchiverSetClass( KeyedUnarchiverRef ref, ClassRef class, CFStringRef codedName );
ClassRef KeyedUnarchiverClass( KeyedUnarchiverRef ref, CFStringRef codedName );

// Initializers
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
KeyedUnarchiverRef KeyedUnarchiverForReadingFromData( CFDataRef dta, ErrorRef *err );
#endif

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
NSUInteger KeyedUnarchiverDecodingFailurePolicy( KeyedUnarchiverRef ref );
void KeyedUnarchiverSetDecodingFailurePolicy( KeyedUnarchiverRef ref, NSUInteger policy );
#endif

// Type methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
CFTypeRef KeyedUnarchiverUnarchivedObjectOfClass( ClassRef class, CFDataRef fromData, ErrorRef *err );
CFTypeRef KeyedUnarchiverUnarchivedObjectOfClasses( CFSetRef classes, CFDataRef fromData, ErrorRef *err );
#endif

