/*
 KeyedArchiver.m

 Bernie Wylde
 */

#import "KeyedArchiver.h"

// Archiving
CFDataRef KeyedArchiverArchivedDataWithRootObject( CFTypeRef obj )
{ return (CFDataRef)[NSKeyedArchiver archivedDataWithRootObject:(id)obj]; }

Boolean KeyedArchiverArchiveRootObjectToURL( CFTypeRef obj, CFURLRef url )
{ return [NSKeyedArchiver archiveRootObject:(id)obj toFile:[(NSURL *)url path]]; }

void KeyedArchiverFinishEncoding( KeyedArchiverRef ref )
{ [ref finishEncoding]; }

NSPropertyListFormat KeyedArchiverOutputFormat( KeyedArchiverRef ref )
{ return [ref outputFormat]; }

void KeyedArchiverSetOutputFormat( KeyedArchiverRef ref, NSPropertyListFormat format )
{ [ref setOutputFormat:format]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
Boolean KeyedArchiverRequiresSecureCoding( KeyedArchiverRef ref )
{ return [ref requiresSecureCoding]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void KeyedArchiverSetRequiresSecureCoding( KeyedArchiverRef ref, Boolean flag )
{ [ref setRequiresSecureCoding:flag]; }
#endif

// Encoding
void KeyedArchiverEncodeBool( KeyedArchiverRef ref, Boolean value, CFStringRef key )
{ [ref encodeBool:value forKey:(NSString *)key]; }

void KeyedArchiverEncodeBytes( KeyedArchiverRef ref, void *bytes, NSUInteger length, CFStringRef key )
{ [ref encodeBytes:bytes length:length forKey:(NSString *)key]; }

void KeyedArchiverEncodeConditionalObject( KeyedArchiverRef ref, CFTypeRef obj, CFStringRef key )
{ [ref encodeConditionalObject:(id)obj forKey:(NSString *)key]; }

void KeyedArchiverEncodeDouble( KeyedArchiverRef ref, double value, CFStringRef key )
{ [ref encodeDouble:value forKey:(NSString *)key]; }

void KeyedArchiverEncodeFloat( KeyedArchiverRef ref, float value, CFStringRef key )
{ [ref encodeFloat:value forKey:(NSString *)key]; }

void KeyedArchiverEncodeInt32( KeyedArchiverRef ref, SInt32 value, CFStringRef key )
{ [ref encodeInt32:value forKey:(NSString *)key]; }

void KeyedArchiverEncodeInt64( KeyedArchiverRef ref, SInt64 value, CFStringRef key )
{ [ref encodeInt64:value forKey:(NSString *)key]; }

void KeyedArchiverEncodeObject( KeyedArchiverRef ref, CFTypeRef obj, CFStringRef key )
{ [ref encodeObject:(id)obj forKey:(NSString *)key]; }

// Classes and class names
void KeyedArchiverSetClassNameForClass( CFStringRef codedName, ClassRef forClass )
{ [NSKeyedArchiver setClassName:(NSString *)codedName forClass:forClass]; }

CFStringRef KeyedArchiverClassNameForClass( ClassRef forClass )
{ return (CFStringRef)[NSKeyedArchiver classNameForClass:forClass]; }

void KeyedArchiverSetClassName( KeyedArchiverRef ref, CFStringRef codedName, ClassRef forClass )
{ [ref setClassName:(NSString *)codedName forClass:forClass]; }

CFStringRef KeyedArchiverClassName( KeyedArchiverRef ref, ClassRef forClass )
{ return (CFStringRef)[ref classNameForClass:forClass]; }

// Initializers
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
KeyedArchiverRef KeyedArchiverRequiringSecureCoding( Boolean flag )
{ return [[[NSKeyedArchiver alloc] initRequiringSecureCoding:flag] autorelease]; }
#endif

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFDataRef KeyedArchiverEncodedData( KeyedArchiverRef ref )
{ return (CFDataRef)[ref encodedData]; }
#endif

// Type methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
CFDataRef KeyedArchiverArchivedDataRequiringSecureCoding( CFTypeRef obj, Boolean secureCoding, ErrorRef *err )
{ return (CFDataRef)[NSKeyedArchiver archivedDataWithRootObject:(id)obj requiringSecureCoding:secureCoding error:err]; }
#endif



/*
 KeyedUnarchiver
 */

// Unarchiving
CFTypeRef KeyedUnarchiverUnarchiveObjectWithData( CFDataRef dta )
{ return (CFTypeRef)[NSKeyedUnarchiver unarchiveObjectWithData:(NSData *)dta]; }

CFTypeRef KeyedUnarchiverUnarchiveObjectWithURL( CFURLRef url )
{ return (CFTypeRef)[NSKeyedUnarchiver unarchiveObjectWithFile:[(NSURL *)url path]]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
Boolean KeyedUnarchiverRequiresSecureCoding( KeyedUnarchiverRef ref )
{ return [ref requiresSecureCoding]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void KeyedUnarchiverSetRequiresSecureCoding( KeyedUnarchiverRef ref, Boolean flag )
{ [ref setRequiresSecureCoding:flag]; }
#endif

// Decoding
Boolean KeyedUnarchiverContainsValue( KeyedUnarchiverRef ref, CFStringRef key )
{ return [ref containsValueForKey:(NSString *)key]; }

Boolean KeyedUnarchiverDecodeBool( KeyedUnarchiverRef ref, CFStringRef key )
{ return [ref decodeBoolForKey:(NSString *)key]; }

const uint8_t *KeyedUnarchiverDecodeBytes( KeyedUnarchiverRef ref, CFStringRef key, NSUInteger *returnedLength )
{ return [ref decodeBytesForKey:(NSString *)key returnedLength:returnedLength]; }

double KeyedUnarchiverDecodeDouble( KeyedUnarchiverRef ref, CFStringRef key )
{ return [ref decodeDoubleForKey:(NSString *)key]; }

float KeyedUnarchiverDecodeFloat( KeyedUnarchiverRef ref, CFStringRef key )
{ return [ref decodeFloatForKey:(NSString *)key]; }

SInt32 KeyedUnarchiverDecodeInt32( KeyedUnarchiverRef ref, CFStringRef key )
{ return [ref decodeInt32ForKey:(NSString *)key]; }

SInt64 KeyedUnarchiverDecodeInt64( KeyedUnarchiverRef ref, CFStringRef key )
{ return [ref decodeInt64ForKey:(NSString *)key]; }

CFTypeRef KeyedUnarchiverDecodeObject( KeyedUnarchiverRef ref, CFStringRef key )
{ return (CFTypeRef)[ref decodeObjectForKey:(NSString *)key]; }

void KeyedUnarchiverFinishDecoding( KeyedUnarchiverRef ref )
{ [ref finishDecoding]; }

// Classes
void KeyedUnarchiverSetClassForClassName( ClassRef class, CFStringRef codedName )
{ [NSKeyedUnarchiver setClass:class forClassName:(NSString *)codedName]; }

ClassRef KeyedUnarchiverClassForClassName( CFStringRef codedName )
{ return [NSKeyedUnarchiver classForClassName:(NSString *)codedName]; }

void KeyedUnarchiverSetClass( KeyedUnarchiverRef ref, ClassRef class, CFStringRef codedName )
{ [ref setClass:class forClassName:(NSString *)codedName]; }

ClassRef KeyedUnarchiverClass( KeyedUnarchiverRef ref, CFStringRef codedName )
{ return [ref classForClassName:(NSString *)codedName]; }

// Initializers
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
KeyedUnarchiverRef KeyedUnarchiverForReadingFromData( CFDataRef dta, ErrorRef *err )
{ return [[[NSKeyedUnarchiver alloc] initForReadingFromData:(NSData *)dta error:err] autorelease]; }
#endif

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
NSUInteger KeyedUnarchiverDecodingFailurePolicy( KeyedUnarchiverRef ref )
{ return [ref decodingFailurePolicy]; }

void KeyedUnarchiverSetDecodingFailurePolicy( KeyedUnarchiverRef ref, NSUInteger policy )
{ [ref setDecodingFailurePolicy:policy]; }
#endif

// Type methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
CFTypeRef KeyedUnarchiverUnarchivedObjectOfClass( ClassRef class, CFDataRef fromData, ErrorRef *err )
{ return (CFTypeRef)[NSKeyedUnarchiver unarchivedObjectOfClass:class fromData:(NSData *)fromData error:err]; }

CFTypeRef KeyedUnarchiverUnarchivedObjectOfClasses( CFSetRef classes, CFDataRef fromData, ErrorRef *err )
{ return (CFTypeRef)[NSKeyedUnarchiver unarchivedObjectOfClasses:(NSSet *)classes fromData:(NSData *)fromData error:err]; }
#endif
