/*
 LayoutConstraint.m

 Bernie Wylde
 */

#import "LayoutConstraint.h"

// Create
CFArrayRef LayoutConstraintConstraintsWithVisualFormat( CFStringRef format, NSLayoutFormatOptions options, CFDictionaryRef metrics, CFDictionaryRef views )
{ return (CFArrayRef)[NSLayoutConstraint constraintsWithVisualFormat:(NSString *)format options:options metrics:(NSDictionary *)metrics views:(NSDictionary *)views]; }

LayoutConstraintRef LayoutConstraintWithItem( CFTypeRef view1, NSLayoutAttribute attr1, NSLayoutRelation relation, CFTypeRef view2, NSLayoutAttribute attr2, CGFloat multiplier, CGFloat constant )
{ return [NSLayoutConstraint constraintWithItem:(id)view1 attribute:attr1 relatedBy:relation toItem:(id)view2 attribute:attr2 multiplier:multiplier constant:constant]; }

// Activate/deactivate
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean LayoutConstraintIsActive( LayoutConstraintRef ref )
{ return [ref isActive]; }

void LayoutConstraintSetActive( LayoutConstraintRef ref, Boolean flag )
{ [ref setActive:flag]; }

void LayoutConstraintActivateConstraints( CFArrayRef constraints )
{ [NSLayoutConstraint activateConstraints:(NSArray *)constraints]; }

void LayoutConstraintDeactivateConstraints( CFArrayRef constraints )
{ [NSLayoutConstraint deactivateConstraints:(NSArray *)constraints]; }
#endif

// Data
CFTypeRef LayoutConstraintFirstItem( LayoutConstraintRef ref )
{ return (CFTypeRef)[ref firstItem]; }

NSLayoutAttribute LayoutConstraintFirstAttribute( LayoutConstraintRef ref )
{ return [ref firstAttribute]; }

NSLayoutRelation LayoutConstraintRelation( LayoutConstraintRef ref )
{ return [ref relation]; }

CFTypeRef LayoutConstraintSecondItem( LayoutConstraintRef ref )
{ return (CFTypeRef)[ref secondItem]; }

NSLayoutAttribute LayoutConstraintSecondAttribute( LayoutConstraintRef ref )
{ return [ref secondAttribute]; }

CGFloat LayoutConstraintMultiplier( LayoutConstraintRef ref )
{ return [ref multiplier]; }

CGFloat LayoutConstraintConstant( LayoutConstraintRef ref )
{ return [ref constant]; }

void LayoutConstraintSetConstant( LayoutConstraintRef ref, CGFloat value )
{ [ref setConstant:value]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
LayoutAnchorRef LayoutConstraintFirstAnchor( LayoutConstraintRef ref )
{ return [ref firstAnchor]; }

LayoutAnchorRef LayoutConstraintSecondAnchor( LayoutConstraintRef ref )
{ return [ref secondAnchor]; }
#endif

// Priority
NSLayoutPriority LayoutConstraintPriority( LayoutConstraintRef ref )
{ return [ref priority]; }

void LayoutConstraintSetPriority( LayoutConstraintRef ref, NSLayoutPriority priority )
{ [ref setPriority:priority]; }

// Identify
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFStringRef LayoutConstraintIdentifier( LayoutConstraintRef ref )
{ return (CFStringRef)[ref identifier]; }

void LayoutConstraintSetIdentifier( LayoutConstraintRef ref, CFStringRef identifier )
{ [ref setIdentifier:(NSString *)identifier]; }
#endif

// Archiving
Boolean LayoutConstraintShouldBeArchived( LayoutConstraintRef ref )
{ return [ref shouldBeArchived]; }

void LayoutConstraintSetShouldBeArchived( LayoutConstraintRef ref, Boolean flag )
{ [ref setShouldBeArchived:flag]; }

