/*
 LayoutManager.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// Init
LayoutManagerRef LayoutManagerInit( void );

// Text storage
TextStorageRef LayoutManagerTextStorage(  LayoutManagerRef ref );
void LayoutManagerReplaceTextStorage( LayoutManagerRef ref, TextStorageRef ts );

// Text containers
CFArrayRef LayoutManagerTextContainers( LayoutManagerRef ref );
void LayoutManagerAddTextContainer( LayoutManagerRef ref, TextContainerRef tc );
void LayoutManagerInsertTextContainer( LayoutManagerRef ref, TextContainerRef tc, NSUInteger index );
void LayoutManagerRemoveTextContainer( LayoutManagerRef ref, NSUInteger index );
void LayoutManagerSetTextContainer( LayoutManagerRef ref, TextContainerRef tc, CFRange glyphRange );
void LayoutManagerTextContainerChangedGeometry( LayoutManagerRef ref, TextContainerRef tc );
void LayoutManagerTextContainerChangedTextView( LayoutManagerRef ref, TextContainerRef tc );
TextContainerRef LayoutManagerTextContainerForGlyph( LayoutManagerRef ref, NSUInteger index, CFRange *effectiveRange );
TextContainerRef LayoutManagerTextContainerForGlyphWithoutAdditionalLayout( LayoutManagerRef ref, NSUInteger index, CFRange *effectiveRange, Boolean flag );
CGRect LayoutManagerUsedRectForTextContainer( LayoutManagerRef ref, TextContainerRef tc );

// Global layout manager options
Boolean LayoutManagerAllowsNonContiguousLayout( LayoutManagerRef ref );
void LayoutManagerSetAllowsNonContiguousLayout( LayoutManagerRef ref, Boolean flag );
Boolean LayoutManagerHasNonContiguousLayout( LayoutManagerRef ref );
CGFloat LayoutManagerHyphenationFactor(LayoutManagerRef ref );
void LayoutManagerSetHyphenationFactor(LayoutManagerRef ref, CGFloat factor );
Boolean LayoutManagerShowsInvisibleCharacters( LayoutManagerRef ref );
void LayoutManagerSetShowsInvisibleCharacters( LayoutManagerRef ref, Boolean flag );
Boolean LayoutManagerShowsControlCharacters( LayoutManagerRef ref );
void LayoutManagerSetShowsControlCharacters( LayoutManagerRef ref, Boolean flag );
Boolean LayoutManagerUsesFontLeading( LayoutManagerRef ref );
void LayoutManagerSetUsesFontLeading( LayoutManagerRef ref, Boolean flag );
Boolean LayoutManagerBackgroundLayoutEnabled( LayoutManagerRef ref );
void LayoutManagerSetBackgroundLayoutEnabled( LayoutManagerRef ref, Boolean flag );

// Invalidate glyphs and layout
void LayoutManagerInvalidateDisplayForCharacterRange( LayoutManagerRef ref, CFRange range );
void LayoutManagerInvalidateDisplayForGlyphRange( LayoutManagerRef ref, CFRange range );
void LayoutManagerInvalidateGlyphsForCharacterRange( LayoutManagerRef ref, CFRange range, NSInteger delta, CFRange *actualCharacterRange );
void LayoutManagerInvalidateLayoutForCharacterRange( LayoutManagerRef ref, CFRange range, CFRange *actualCharacterRange );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void LayoutManagerProcessEditingForTextStorage( LayoutManagerRef ref, TextStorageRef textStorage, NSTextStorageEditActions edited, CFRange range, NSInteger delta, CFRange invalidatedRange );
#endif

// Glyph generation and layout
void LayoutManagerEnsureGlyphsForCharacterRange( LayoutManagerRef ref, CFRange range );
void LayoutManagerEnsureGlyphsForGlyphRange( LayoutManagerRef ref, CFRange range );
void LayoutManagerEnsureLayoutForBoundingRect( LayoutManagerRef ref, CGRect r, TextContainerRef tc );
void LayoutManagerEnsureLayoutForCharacterRange( LayoutManagerRef ref, CFRange range );
void LayoutManagerEnsureLayoutForGlyphRange( LayoutManagerRef ref, CFRange range );
void LayoutManagerEnsureLayoutForTextContainer( LayoutManagerRef ref, TextContainerRef tc );
//GlyphGeneratorRef LayoutManagerGlyphGenerator( LayoutManagerRef ref );

// Accessing glyphs
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSUInteger LayoutManagerGetGlyphs( LayoutManagerRef ref, CFRange range, CGGlyph *glyphBuffer, NSGlyphProperty *props, NSUInteger *charIndexBuffer, unsigned char *bidiLevelsBuffer );
CGGlyph LayoutManagerCGGlyphAtIndex( LayoutManagerRef ref, NSUInteger glyphIndex );
CGGlyph LayoutManagerCGGlyphAtIndexIsValidIndex( LayoutManagerRef ref, NSUInteger glyphIndex, BOOL *isValidIndex );
void LayoutManagerSetCGGlyphs( LayoutManagerRef ref, CGGlyph *glyphs, NSGlyphProperty *props, NSUInteger *charIndexes, FontRef font, CFRange glyphRange );
#endif
NSUInteger LayoutManagerCharacterIndexForGlyphAtIndex( LayoutManagerRef ref, NSUInteger glyphIndex );
NSUInteger LayoutManagerGlyphIndexForCharacterAtIndex( LayoutManagerRef ref, NSUInteger charIndex );
Boolean LayoutManagerIsValidGlyphIndex( LayoutManagerRef ref, NSUInteger glyphIndex );
NSUInteger LayoutManagerNumberOfGlyphs( LayoutManagerRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSGlyphProperty LayoutManagerPropertyForGlyphAtIndex( LayoutManagerRef ref, NSUInteger glyphIndex );
#endif

// Set layout info
void LayoutManagerSetAttachmentSize( LayoutManagerRef ref, CGSize size, CFRange glyphRange );
void LayoutManagerSetDrawsOutsideLineFragment( LayoutManagerRef ref, Boolean flag, NSUInteger glyphIndex );
void LayoutManagerSetExtraLineFragmentRect( LayoutManagerRef ref, CGRect r, CGRect usedRect, TextContainerRef textContainer );
void LayoutManagerSetLineFragmentRect( LayoutManagerRef ref, CGRect r, CFRange glyphRange, CGRect usedRect );
void LayoutManagerSetLocation( LayoutManagerRef ref, CGPoint pt, CFRange startOfGlyphRange );
void LayoutManagerSetNotShownAttribute( LayoutManagerRef ref, Boolean flag, NSUInteger glyphIndex );

// Get layout info
CGSize LayoutManagerAttachmentSizeForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index );
Boolean LayoutManagerDrawsOutsideLineFragmentForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index );
CGRect LayoutManagerExtraLineFragmentRect( LayoutManagerRef ref );
TextContainerRef LayoutManagerExtraLineFragmentTextContainer( LayoutManagerRef ref );
CGRect LayoutManagerExtraLineFragmentUsedRect( LayoutManagerRef ref );
NSUInteger LayoutManagerFirstUnlaidCharacterIndex( LayoutManagerRef ref );
NSUInteger LayoutManagerFirstUnlaidGlyphIndex( LayoutManagerRef ref );
void LayoutManagerGetFirstUnlaidCharacterIndex( LayoutManagerRef ref, NSUInteger *charIndex, NSUInteger *glyphIndex );
CGRect LayoutManagerLineFragmentRectForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index, CFRange *effectiveRange );
CGRect LayoutManagerLineFragmentRectForGlyphAtIndexWithoutAdditionalLayout( LayoutManagerRef ref, NSUInteger index, CFRange *effectiveRange, Boolean flag );
CGRect LayoutManagerExtraLineFragmentUsedRectForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index, CFRange *effectiveRange );
CGRect LayoutManagerExtraLineFragmentUsedRectForGlyphAtIndexWithoutAdditionalLayout( LayoutManagerRef ref, NSUInteger index, CFRange *effectiveRange, Boolean flag );
CGPoint LayoutManagerLocationForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index );
Boolean LayoutManagerNotShownAttributeForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFRange LayoutManagerTruncatedGlyphRangeInLineFragmentForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index );
#endif

// Advanced layout queries
CGRect LayoutManagerBoundingRectForGlyphRange( LayoutManagerRef ref, CFRange glyphRange, TextContainerRef tc );
NSUInteger LayoutManagerCharacterIndexForPoint( LayoutManagerRef ref, CGPoint pt, TextContainerRef tc, CGFloat *partialFraction );
CFRange LayoutManagerCharacterRangeForGlyphRange( LayoutManagerRef ref, CFRange glyphRange, CFRange *actualGlyphRange );

CFRange LayoutManagerGlyphRangeForBoundingRect( LayoutManagerRef ref, CGRect bounds, TextContainerRef tc );

// Drawing

// Text blocks

// Attachments

// Rulers

// Responder chain

// Typesetter

// Temporary attribute support


