/*
 LayoutManager.m

 Bernie Wylde
 */

#import "LayoutManager.h"

// Init
LayoutManagerRef LayoutManagerInit( void )
{ return [[[NSLayoutManager alloc] init] autorelease]; }

// Text storage
TextStorageRef LayoutManagerTextStorage(  LayoutManagerRef ref )
{ return [ref textStorage]; }

void LayoutManagerReplaceTextStorage( LayoutManagerRef ref, TextStorageRef ts )
{ [ref replaceTextStorage:ts]; }

// Text containers
CFArrayRef LayoutManagerTextContainers( LayoutManagerRef ref )
{ return (CFArrayRef)[ref textContainers]; }

void LayoutManagerAddTextContainer( LayoutManagerRef ref, TextContainerRef tc )
{ [ref addTextContainer:tc]; }

void LayoutManagerInsertTextContainer( LayoutManagerRef ref, TextContainerRef tc, NSUInteger index )
{ [ref insertTextContainer:tc atIndex:index]; }

void LayoutManagerRemoveTextContainer( LayoutManagerRef ref, NSUInteger index )
{ [ref removeTextContainerAtIndex:index]; }

void LayoutManagerSetTextContainer( LayoutManagerRef ref, TextContainerRef tc, CFRange glyphRange )
{ [ref setTextContainer:tc forGlyphRange:NSRangeFromCFRange(glyphRange)]; }

void LayoutManagerTextContainerChangedGeometry( LayoutManagerRef ref, TextContainerRef tc )
{ [ref textContainerChangedGeometry:tc]; }

void LayoutManagerTextContainerChangedTextView( LayoutManagerRef ref, TextContainerRef tc )
{ [ref textContainerChangedTextView:tc]; }

TextContainerRef LayoutManagerTextContainerForGlyph( LayoutManagerRef ref, NSUInteger index, CFRange *effectiveRange )
{ return [ref textContainerForGlyphAtIndex:index effectiveRange:(NSRange *)effectiveRange]; }

TextContainerRef LayoutManagerTextContainerForGlyphWithoutAdditionalLayout( LayoutManagerRef ref, NSUInteger index, CFRange *effectiveRange, Boolean flag )
{ return [ref textContainerForGlyphAtIndex:index effectiveRange:(NSRange *)effectiveRange withoutAdditionalLayout:flag]; }

CGRect LayoutManagerUsedRectForTextContainer( LayoutManagerRef ref, TextContainerRef tc )
{ return NSRectToCGRect([ref usedRectForTextContainer:tc]); }

// Global layout manager options
Boolean LayoutManagerAllowsNonContiguousLayout( LayoutManagerRef ref )
{ return [ref allowsNonContiguousLayout]; }

void LayoutManagerSetAllowsNonContiguousLayout( LayoutManagerRef ref, Boolean flag )
{ [ref setAllowsNonContiguousLayout:flag]; }

Boolean LayoutManagerHasNonContiguousLayout( LayoutManagerRef ref )
{ return [ref hasNonContiguousLayout]; }

CGFloat LayoutManagerHyphenationFactor(LayoutManagerRef ref )
{ return [ref hyphenationFactor]; }

void LayoutManagerSetHyphenationFactor(LayoutManagerRef ref, CGFloat factor )
{ [ref setHyphenationFactor:factor]; }

Boolean LayoutManagerShowsInvisibleCharacters( LayoutManagerRef ref )
{ return [ref showsInvisibleCharacters]; }

void LayoutManagerSetShowsInvisibleCharacters( LayoutManagerRef ref, Boolean flag )
{ [ref setShowsInvisibleCharacters:flag]; }

Boolean LayoutManagerShowsControlCharacters( LayoutManagerRef ref )
{ return [ref showsControlCharacters]; }

void LayoutManagerSetShowsControlCharacters( LayoutManagerRef ref, Boolean flag )
{ [ref setShowsControlCharacters:flag]; }

Boolean LayoutManagerUsesFontLeading( LayoutManagerRef ref )
{ return [ref usesFontLeading]; }

void LayoutManagerSetUsesFontLeading( LayoutManagerRef ref, Boolean flag )
{ [ref setUsesFontLeading:flag]; }

Boolean LayoutManagerBackgroundLayoutEnabled( LayoutManagerRef ref )
{ return [ref backgroundLayoutEnabled]; }

void LayoutManagerSetBackgroundLayoutEnabled( LayoutManagerRef ref, Boolean flag )
{ [ref setBackgroundLayoutEnabled:flag]; }

// Invalidate glyphs and layout
void LayoutManagerInvalidateDisplayForCharacterRange( LayoutManagerRef ref, CFRange range )
{ [ref invalidateDisplayForCharacterRange:NSRangeFromCFRange(range)]; }

void LayoutManagerInvalidateDisplayForGlyphRange( LayoutManagerRef ref, CFRange range )
{ [ref invalidateDisplayForGlyphRange:NSRangeFromCFRange(range)]; }

void LayoutManagerInvalidateGlyphsForCharacterRange( LayoutManagerRef ref, CFRange range, NSInteger delta, CFRange *actualCharacterRange )
{ [ref invalidateGlyphsForCharacterRange:NSRangeFromCFRange(range) changeInLength:delta actualCharacterRange:(NSRange *)actualCharacterRange]; }

void LayoutManagerInvalidateLayoutForCharacterRange( LayoutManagerRef ref, CFRange range, CFRange *actualCharacterRange )
{ [ref invalidateLayoutForCharacterRange:NSRangeFromCFRange(range) actualCharacterRange:(NSRange *)actualCharacterRange]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void LayoutManagerProcessEditingForTextStorage( LayoutManagerRef ref, TextStorageRef textStorage, NSTextStorageEditActions edited, CFRange range, NSInteger delta, CFRange invalidatedRange )
{ [ref processEditingForTextStorage:textStorage edited:edited range:NSRangeFromCFRange(range) changeInLength:delta invalidatedRange:NSRangeFromCFRange(invalidatedRange)]; }
#endif

// Glyph generation and layout
void LayoutManagerEnsureGlyphsForCharacterRange( LayoutManagerRef ref, CFRange range )
{ [ref ensureGlyphsForCharacterRange:NSRangeFromCFRange(range)]; }

void LayoutManagerEnsureGlyphsForGlyphRange( LayoutManagerRef ref, CFRange range )
{ [ref ensureGlyphsForGlyphRange:NSRangeFromCFRange(range)]; }

void LayoutManagerEnsureLayoutForBoundingRect( LayoutManagerRef ref, CGRect r, TextContainerRef tc )
{ [ref ensureLayoutForBoundingRect:NSRectFromCGRect(r) inTextContainer:tc]; }

void LayoutManagerEnsureLayoutForCharacterRange( LayoutManagerRef ref, CFRange range )
{ [ref ensureLayoutForCharacterRange:NSRangeFromCFRange(range)]; }

void LayoutManagerEnsureLayoutForGlyphRange( LayoutManagerRef ref, CFRange range )
{ [ref ensureLayoutForGlyphRange:NSRangeFromCFRange(range)]; }

void LayoutManagerEnsureLayoutForTextContainer( LayoutManagerRef ref, TextContainerRef tc )
{ [ref ensureLayoutForTextContainer:tc]; }

//GlyphGeneratorRef LayoutManagerGlyphGenerator( LayoutManagerRef ref );

// Accessing glyphs
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSUInteger LayoutManagerGetGlyphs( LayoutManagerRef ref, CFRange range, CGGlyph *glyphBuffer, NSGlyphProperty *props, NSUInteger *charIndexBuffer, unsigned char *bidiLevelsBuffer )
{ return [ref getGlyphsInRange:NSRangeFromCFRange(range) glyphs:glyphBuffer properties:props characterIndexes:charIndexBuffer bidiLevels:bidiLevelsBuffer]; }

CGGlyph LayoutManagerCGGlyphAtIndex( LayoutManagerRef ref, NSUInteger glyphIndex )
{ return [ref CGGlyphAtIndex:glyphIndex]; }

CGGlyph LayoutManagerCGGlyphAtIndexIsValidIndex( LayoutManagerRef ref, NSUInteger glyphIndex, BOOL *isValidIndex )
{ return [ref CGGlyphAtIndex:glyphIndex isValidIndex:isValidIndex]; }

void LayoutManagerSetCGGlyphs( LayoutManagerRef ref, CGGlyph *glyphs, NSGlyphProperty *props, NSUInteger *charIndexes, FontRef font, CFRange glyphRange )
{ [ref setGlyphs:glyphs properties:props characterIndexes:charIndexes font:font forGlyphRange:NSRangeFromCFRange(glyphRange)]; }
#endif

NSUInteger LayoutManagerCharacterIndexForGlyphAtIndex( LayoutManagerRef ref, NSUInteger glyphIndex )
{ return [ref characterIndexForGlyphAtIndex:glyphIndex]; }

NSUInteger LayoutManagerGlyphIndexForCharacterAtIndex( LayoutManagerRef ref, NSUInteger charIndex )
{ return [ref glyphIndexForCharacterAtIndex:charIndex]; }

Boolean LayoutManagerIsValidGlyphIndex( LayoutManagerRef ref, NSUInteger glyphIndex )
{ return [ref isValidGlyphIndex:glyphIndex]; }

NSUInteger LayoutManagerNumberOfGlyphs( LayoutManagerRef ref )
{ return [ref numberOfGlyphs]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSGlyphProperty LayoutManagerPropertyForGlyphAtIndex( LayoutManagerRef ref, NSUInteger glyphIndex )
{ return [ref propertyForGlyphAtIndex:glyphIndex]; }
#endif

// Set layout info
void LayoutManagerSetAttachmentSize( LayoutManagerRef ref, CGSize size, CFRange glyphRange )
{ [ref setAttachmentSize:NSSizeFromCGSize(size) forGlyphRange:NSRangeFromCFRange(glyphRange)]; }

void LayoutManagerSetDrawsOutsideLineFragment( LayoutManagerRef ref, Boolean flag, NSUInteger glyphIndex )
{ [ref setDrawsOutsideLineFragment:flag forGlyphAtIndex:glyphIndex]; }

void LayoutManagerSetExtraLineFragmentRect( LayoutManagerRef ref, CGRect r, CGRect usedRect, TextContainerRef textContainer )
{ [ref setExtraLineFragmentRect:NSRectFromCGRect(r) usedRect:NSRectFromCGRect(usedRect) textContainer:textContainer]; }

void LayoutManagerSetLineFragmentRect( LayoutManagerRef ref, CGRect r, CFRange glyphRange, CGRect usedRect )
{ [ref setLineFragmentRect:NSRectFromCGRect(r) forGlyphRange:NSRangeFromCFRange(glyphRange) usedRect:NSRectFromCGRect(usedRect)]; }

void LayoutManagerSetLocation( LayoutManagerRef ref, CGPoint pt, CFRange startOfGlyphRange )
{ [ref setLocation:NSPointFromCGPoint(pt) forStartOfGlyphRange:NSRangeFromCFRange(startOfGlyphRange)]; }

void LayoutManagerSetNotShownAttribute( LayoutManagerRef ref, Boolean flag, NSUInteger glyphIndex )
{ [ref setNotShownAttribute:flag forGlyphAtIndex:glyphIndex]; }

// Get layout info
CGSize LayoutManagerAttachmentSizeForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index )
{ return NSSizeToCGSize([ref attachmentSizeForGlyphAtIndex:index]); }

Boolean LayoutManagerDrawsOutsideLineFragmentForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index )
{ return [ref drawsOutsideLineFragmentForGlyphAtIndex:index]; }

CGRect LayoutManagerExtraLineFragmentRect( LayoutManagerRef ref )
{ return NSRectToCGRect([ref extraLineFragmentRect]); }

TextContainerRef LayoutManagerExtraLineFragmentTextContainer( LayoutManagerRef ref )
{ return [ref extraLineFragmentTextContainer]; }

CGRect LayoutManagerExtraLineFragmentUsedRect( LayoutManagerRef ref )
{ return NSRectToCGRect([ref extraLineFragmentUsedRect]); }

NSUInteger LayoutManagerFirstUnlaidCharacterIndex( LayoutManagerRef ref )
{ return [ref firstUnlaidCharacterIndex]; }

NSUInteger LayoutManagerFirstUnlaidGlyphIndex( LayoutManagerRef ref )
{ return [ref firstUnlaidGlyphIndex]; }

void LayoutManagerGetFirstUnlaidCharacterIndex( LayoutManagerRef ref, NSUInteger *charIndex, NSUInteger *glyphIndex )
{ [ref getFirstUnlaidCharacterIndex:charIndex glyphIndex:glyphIndex]; }

CGRect LayoutManagerLineFragmentRectForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index, CFRange *effectiveRange )
{ return NSRectToCGRect([ref lineFragmentRectForGlyphAtIndex:index effectiveRange:(NSRange *)effectiveRange]); }

CGRect LayoutManagerLineFragmentRectForGlyphAtIndexWithoutAdditionalLayout( LayoutManagerRef ref, NSUInteger index, CFRange *effectiveRange, Boolean flag )
{ return NSRectToCGRect([ref lineFragmentRectForGlyphAtIndex:index effectiveRange:(NSRange *)effectiveRange withoutAdditionalLayout:flag]); }

CGRect LayoutManagerExtraLineFragmentUsedRectForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index, CFRange *effectiveRange )
{ return NSRectToCGRect([ref lineFragmentUsedRectForGlyphAtIndex:index effectiveRange:(NSRange *)effectiveRange]); }

CGRect LayoutManagerExtraLineFragmentUsedRectForGlyphAtIndexWithoutAdditionalLayout( LayoutManagerRef ref, NSUInteger index, CFRange *effectiveRange, Boolean flag )
{ return NSRectToCGRect([ref lineFragmentUsedRectForGlyphAtIndex:index effectiveRange:(NSRange *)effectiveRange withoutAdditionalLayout:flag]); }

CGPoint LayoutManagerLocationForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index )
{ return NSPointToCGPoint([ref locationForGlyphAtIndex:index]); }

Boolean LayoutManagerNotShownAttributeForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index )
{ return [ref notShownAttributeForGlyphAtIndex:index]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFRange LayoutManagerTruncatedGlyphRangeInLineFragmentForGlyphAtIndex( LayoutManagerRef ref, NSUInteger index )
{ return NSRangeToCFRange([ref truncatedGlyphRangeInLineFragmentForGlyphAtIndex:index]); }
#endif

// Advanced layout queries
CGRect LayoutManagerBoundingRectForGlyphRange( LayoutManagerRef ref, CFRange glyphRange, TextContainerRef tc )
{ return NSRectToCGRect([ref boundingRectForGlyphRange:NSRangeFromCFRange(glyphRange) inTextContainer:tc]); }

NSUInteger LayoutManagerCharacterIndexForPoint( LayoutManagerRef ref, CGPoint pt, TextContainerRef tc, CGFloat *partialFraction )
{ return [ref characterIndexForPoint:NSPointFromCGPoint(pt) inTextContainer:tc fractionOfDistanceBetweenInsertionPoints:partialFraction]; }

CFRange LayoutManagerCharacterRangeForGlyphRange( LayoutManagerRef ref, CFRange glyphRange, CFRange *actualGlyphRange )
{ return NSRangeToCFRange([ref characterRangeForGlyphRange:NSRangeFromCFRange(glyphRange) actualGlyphRange:(NSRange *)actualGlyphRange]); }


CFRange LayoutManagerGlyphRangeForBoundingRect( LayoutManagerRef ref, CGRect bounds, TextContainerRef tc )
{ return NSRangeToCFRange([ref glyphRangeForBoundingRect:NSRectFromCGRect(bounds) inTextContainer:tc]); }

// Drawing

// Text blocks

// Attachments

// Rulers

// Responder chain

// Typesetter

// Temporary attribute support

