/*
 LevelIndicator.m

 Bernie Wylde
 */

#import "LevelIndicator.h"

#pragma mark - subclass
@implementation LevelIndicatorSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end


NSLevelIndicator *LevelIndicatorWithTag( NSInteger tag )
{ return (NSLevelIndicator *)CocoaValidateViewKindOfClass( tag, [NSLevelIndicator class] ); }

BOOL LevelIndicatorExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSLevelIndicator class] ); }


// - range -
double LevelIndicatorMinValue( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind minValue];
    return 0.0;
}

void LevelIndicatorSetMinValue( NSInteger tag, double value )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setMinValue:value];
    
}

double LevelIndicatorMaxValue( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind maxValue];
    return 0.0;
}

void LevelIndicatorSetMaxValue( NSInteger tag, double value )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setMaxValue:value];
    
}

double LevelIndicatorWarningValue( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind warningValue];
    return 0.0;
}

void LevelIndicatorSetWarningValue( NSInteger tag, double value )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setWarningValue:value];
    
}

double LevelIndicatorCriticalValue( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind criticalValue];
    return 0.0;
}

void LevelIndicatorSetCriticalValue( NSInteger tag, double value )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setCriticalValue:value];
    
}

// - tick marks and style -
NSTickMarkPosition LevelIndicatorTickMarkPosition( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind tickMarkPosition];
    return 0;
}

void LevelIndicatorSetTickMarkPosition( NSInteger tag, NSTickMarkPosition position )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setTickMarkPosition:position];
}

NSInteger LevelIndicatorNumberOfTickMarks( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind numberOfTickMarks];
    return 0;
}

void LevelIndicatorSetNumberOfTickMarks( NSInteger tag, NSInteger tickMarks )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setNumberOfTickMarks:tickMarks];
}

NSInteger LevelIndicatorNumberOfMajorTickMarks( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind numberOfMajorTickMarks];
    return 0;
}
void LevelIndicatorSetNumberOfMajorTickMarks( NSInteger tag, NSInteger tickMarks )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setNumberOfMajorTickMarks:tickMarks];
}

CGRect LevelIndicatorRectOfTickMarkAtIndex( NSInteger tag, NSInteger index )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return NSRectToCGRect([ind rectOfTickMarkAtIndex:index]);
    return CGRectZero;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSLevelIndicatorStyle LevelIndicatorStyle( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind levelIndicatorStyle];
    return 0;
}

void LevelIndicatorSetStyle( NSInteger tag, NSLevelIndicatorStyle style )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setLevelIndicatorStyle:style];
}
#endif


// - instance properties -
Boolean LevelIndicatorIsEditable( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [[ind cell] isEditable];
    return NO;
}

void LevelIndicatorSetEditable( NSInteger tag, Boolean flag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) {
        [[ind cell] setEditable:flag];
        if ( flag ) {
            [ind setTarget:CocoaControlTargetObj()];
            [ind setAction:@selector(controlAction:)];
        } else {
            [ind setTarget:nil];
            [ind setAction:nil];
        }
    }
}

ImageRef LevelIndicatorImage( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [[ind cell] image];
    return nil;
}

void LevelIndicatorSetImage( NSInteger tag, void *inImage )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) {
        NSImage *image = FixImageValue( inImage );
        if ( image ) [[ind cell] setImage:image];
    }
}

void LevelIndicatorSetImageNamed( NSInteger tag, CFStringRef imageName )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) {
        if ( imageName ) {
            NSImage *image = CocoaImageResource( (NSString *)imageName );
            if ( image ) [[ind cell] setImage:image];
        }
    }
}



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
ColorRef LevelIndicatorCriticalFillColor( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind criticalFillColor];
    return nil;
}

void LevelIndicatorSetCriticalFillColor( NSInteger tag, ColorRef col )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setCriticalFillColor:col];
}

Boolean LevelIndicatorDrawsTieredCapacityLevels( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind drawsTieredCapacityLevels];
    return NO;
}

void LevelIndicatorSetDrawsTieredCapacityLevels( NSInteger tag, Boolean flag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setDrawsTieredCapacityLevels:flag];
}

ColorRef LevelIndicatorFillColor( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind fillColor];
    return nil;
}

void LevelIndicatorSetFillColor( NSInteger tag, ColorRef col )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setFillColor:col];
}

NSLevelIndicatorPlaceholderVisibility LevelIndicatorPlaceholderVisibility( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind placeholderVisibility];
    return 0;
}

void LevelIndicatorSetPlaceholderVisibility( NSInteger tag, NSLevelIndicatorPlaceholderVisibility visibility )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setPlaceholderVisibility:visibility];
}

ImageRef LevelIndicatorRatingImage( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind ratingImage];
    return nil;
}

void LevelIndicatorSetRatingImage( NSInteger tag, void *inImage )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) {
        NSImage *image = FixImageValue( inImage );
        if ( image ) [ind setRatingImage:image];
    }
}

ImageRef LevelIndicatorRatingPlaceholderImage( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind ratingPlaceholderImage];
    return nil;
}

void LevelIndicatorSetRatingPlaceholderImage( NSInteger tag, void *inImage )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) {
        NSImage *image = FixImageValue( inImage );
        if ( image ) [ind setRatingPlaceholderImage:image];
    }
}

void LevelIndicatorSetRatingPlaceholderImageNamed( NSInteger tag, CFStringRef imageName )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) {
        if ( imageName ) {
            NSImage *image = CocoaImageResource( (NSString *)imageName );
            if ( image ) [ind setRatingPlaceholderImage:image];
        }
    }
}

ColorRef LevelIndicatorWarningFillColor( NSInteger tag )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) return [ind warningFillColor];
    return nil;
}

void LevelIndicatorSetWarningFillColor( NSInteger tag, ColorRef col )
{
    NSLevelIndicator *ind = LevelIndicatorWithTag( tag );
    if ( ind ) [ind setWarningFillColor:col];
}
#endif


// Init
LevelIndicatorRef LevelIndicatorInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) {
        rect = NSMakeRect( 20, 18, 96, 18 );
    }
    
    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSLevelIndicator *ind;
    if ( subclass ) {
        ind = [[[LevelIndicatorSC alloc] initWithFrame:rect] autorelease];
    } else {
        ind = [[[NSLevelIndicator alloc] initWithFrame:rect] autorelease];
    }
    
    // min
    [ind setMinValue:0.0];

    // max
    [ind setMaxValue:2.0];

    // value
    [ind setDoubleValue:2.0];

    // tag
    [ind setTag:inTag];
    
    return ind;
}


void CocoaLevelIndicator( NSInteger inTag, NSInteger inEnabled, double inValue, CGRect inRect, double inMinValue, double inMaxValue, double inWarningValue, double inCriticalValue, NSInteger inTickMarks, NSInteger inMajorTickMarks, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSLevelIndicator *ind = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !ind ) {
            ind = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( ind ) {
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [ind class], [NSLevelIndicator class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, ind, [NSLevelIndicator class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [ind setEnabled:inEnabled];
            
            // min
            if ( (int)inMinValue != kFBParamMissing ) [ind setMinValue:inMinValue];
            double minValue = [ind minValue];

            // max
            if ( (int)inMaxValue != kFBParamMissing ) [ind setMaxValue:inMaxValue];
            double maxValue = [ind maxValue];
            
            // value
            if ( (int)inValue != kFBParamMissing ) {
                if ( inValue >= minValue && inValue <= maxValue ) [ind setDoubleValue:inValue];
            }
            
            // warning
            if ( (int)inWarningValue != kFBParamMissing ) [ind setWarningValue:inWarningValue];
            
            // critical
            if ( (int)inCriticalValue != kFBParamMissing ) [ind setCriticalValue:inCriticalValue];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [ind setFrame:rect];
            
            // tick marks
            if ( inTickMarks ) {
                [ind setNumberOfTickMarks:inTickMarks];
                [ind setNumberOfMajorTickMarks:inMajorTickMarks];
            }
            
        } else {
            
            // tick marks
            if ( inTickMarks == kFBParamMissing ) inTickMarks = 0;
            if ( inTickMarks > 0 ) {
                if ( inMajorTickMarks == kFBParamMissing ) inMajorTickMarks = 0;
            }
            
            // rect
            if ( NSIsEmptyRect( rect ) ) {
                if ( inTickMarks ) {
                    rect = NSMakeRect( 20, 18, 96, 25 );
                } else {
                    rect = NSMakeRect( 20, 18, 96, 18 );
                }
            }
            
            // init
//            if ( inSubclass ) {
//                ind = [[LevelIndicatorSC alloc] initWithFrame:rect];
//            } else {
//                ind = [[NSLevelIndicator alloc] initWithFrame:rect];
//            }
            ind = LevelIndicatorInit( inTag, NSRectToCGRect(rect) );

            // default autoresizing
            [ind setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [ind setTag:inTag];
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [ind setEnabled:inEnabled];
            
            // min
            //if ( (int)inMinValue == kFBParamMissing ) inMinValue = 0.0;
            //[ind setMinValue:inMinValue];
            if ( (int)inMinValue != kFBParamMissing ) [ind setMinValue:inMinValue];
            
            // max
//            if ( (int)inMaxValue == kFBParamMissing ) inMaxValue = 2.0;
//            [ind setMaxValue:inMaxValue];
            if ( (int)inMaxValue != kFBParamMissing ) [ind setMaxValue:inMaxValue];

            // value
//            if ( (int)inValue == kFBParamMissing ) inValue = 2;
//            if ( inValue <= inMaxValue ) [ind setDoubleValue:inValue];
            if ( (int)inValue != kFBParamMissing ) [ind setDoubleValue:inValue];

            // warning
            if ( (int)inWarningValue != kFBParamMissing ) [ind setWarningValue:inWarningValue];
            
            // critical
            if ( (int)inCriticalValue != kFBParamMissing ) [ind setCriticalValue:inCriticalValue];
            
            // tick marks
            if ( inTickMarks ) {
                [ind setNumberOfTickMarks:inTickMarks];
                [ind setNumberOfMajorTickMarks:inMajorTickMarks];
            }
            
            // wndTag
            [ind setWndTag:inWndTag];
            
            // add to window
            [contentView addSubview:ind];
//            [ind release];
        }
        
        [ind setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

