/*
 LinguisticTagger.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef BOOL (*CocoaLinguisticTaggerCallbackType1)(NSLinguisticTagger*,CFStringRef,CFRange,CFRange,void*);
typedef BOOL (*CocoaLinguisticTaggerCallbackType2)(NSLinguisticTagger*,CFStringRef,CFRange,void*);

//@interface CocoaLinguisticTaggerTarget : NSObject {
//    CocoaLinguisticTaggerCallbackType callback;
//    void *userData;
//}
//@property (assign) CocoaLinguisticTaggerCallbackType callback;
//@property (assign) void *userData;
//@end

// - init -
LinguisticTaggerRef LinguisticTaggerInit( CFArrayRef tagSchemes, NSUInteger options );

// - string -
CFStringRef LinguisticTaggerString( LinguisticTaggerRef ref );
void LinguisticTaggerSetString( LinguisticTaggerRef ref, CFStringRef string );

// - tag schemes -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFArrayRef LinguisticTaggerAvailableTagSchemesForUnit( NSLinguisticTaggerUnit unit, CFStringRef language );
#endif
CFArrayRef LinguisticTaggerAvailableTagSchemesForLanguage( CFStringRef language );
CFArrayRef LinguisticTaggerTagSchemes( LinguisticTaggerRef ref );

// - determine dominant language and orthography -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFStringRef LinguisticTaggerDominantLanguageForString( CFStringRef string );
CFStringRef LinguisticTaggerDominantLanguage( LinguisticTaggerRef ref );
#endif
OrthographyRef LinguisticTaggerOrthographyAtIndex( LinguisticTaggerRef ref, NSUInteger index, CFRange *effectiveRange );
void LinguisticTaggerSetOrthography( LinguisticTaggerRef ref, OrthographyRef orthRef, CFRange range );

// - enumerate -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void LinguisticTaggerEnumerateTagsForUnit( LinguisticTaggerRef ref, CFRange range, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSLinguisticTaggerOptions options, CocoaLinguisticTaggerCallbackType2 callback, void *userData );
#endif
void LinguisticTaggerEnumerateTagsForScheme( LinguisticTaggerRef ref, CFRange range, CFStringRef scheme, NSLinguisticTaggerOptions options, CocoaLinguisticTaggerCallbackType1 callback, void *userData );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void LinguisticTaggerEnumerateTagsForString( CFStringRef string, CFRange range, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSLinguisticTaggerOptions options, OrthographyRef orthRef, CocoaLinguisticTaggerCallbackType2 callback, void *userData );
#endif

// - get -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFStringRef LinguisticTaggerTagForUnit( LinguisticTaggerRef ref, NSUInteger index, NSLinguisticTaggerUnit unit, CFStringRef scheme, CFRange *tokenRange );
#endif
CFStringRef LinguisticTaggerTagForScheme( LinguisticTaggerRef ref, NSUInteger index, CFStringRef scheme, CFRange *tokenRange, CFRange *sentenceRange );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFStringRef LinguisticTaggerTagForString( CFStringRef string, NSUInteger index, NSLinguisticTaggerUnit unit, CFStringRef scheme, OrthographyRef orthRef, CFRange *tokenRange );
CFArrayRef LinguisticTaggerTagsForUnit( LinguisticTaggerRef ref, CFRange range, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSLinguisticTaggerOptions options, CFArrayRef *tokenRanges );
#endif
CFArrayRef LinguisticTaggerTagsForScheme( LinguisticTaggerRef ref, CFRange range, CFStringRef scheme, NSLinguisticTaggerOptions options, CFArrayRef *tokenRanges );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFArrayRef LinguisticTaggerTagsForString( CFStringRef string, CFRange range, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSLinguisticTaggerOptions options, OrthographyRef orthRef, CFArrayRef *tokenRanges );
#endif

// - range of unit token -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFRange LinguisticTaggerTokenRangeAtIndex( LinguisticTaggerRef ref, NSUInteger index, NSLinguisticTaggerUnit unit );
#endif
CFRange LinguisticTaggerSentenceRangeForRange( LinguisticTaggerRef ref, CFRange range );

// - possible tags -
CFArrayRef LinguisticTaggerPossibleTagsAtIndex( LinguisticTaggerRef ref, NSUInteger index, CFStringRef scheme, CFRange *tokenRange, CFRange *sentenceRange, CFArrayRef *scores );

// Notify changes to analyzed string
void LinguisticTaggerStringEditedInRange( LinguisticTaggerRef ref, CFRange newRange, NSInteger delta );

