/*
 LinguisticTagger.m

 Bernie Wylde
 */

#import "LinguisticTagger.h"

//@implementation CocoaLinguisticTaggerTarget
//@synthesize callback;
//@synthesize userData;
//
//- (void)action:(id)sender {
//    if ( callback ) (*callback)(sender,userData);
//}
//@end

// - init -
LinguisticTaggerRef LinguisticTaggerInit( CFArrayRef tagSchemes, NSUInteger options )
{ return [[[NSLinguisticTagger alloc] initWithTagSchemes:(NSArray *)tagSchemes options:options] autorelease]; }

// - string -
CFStringRef LinguisticTaggerString( LinguisticTaggerRef ref )
{ return (CFStringRef)[ref string]; }

void LinguisticTaggerSetString( LinguisticTaggerRef ref, CFStringRef string )
{ [ref setString:(NSString *)string]; }

// - tag schemes -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFArrayRef LinguisticTaggerAvailableTagSchemesForUnit( NSLinguisticTaggerUnit unit, CFStringRef language )
{ return (CFArrayRef)[NSLinguisticTagger availableTagSchemesForUnit:unit language:(NSString *)language]; }
#endif

CFArrayRef LinguisticTaggerAvailableTagSchemesForLanguage( CFStringRef language )
{ return (CFArrayRef)[NSLinguisticTagger availableTagSchemesForLanguage:(NSString *)language]; }

CFArrayRef LinguisticTaggerTagSchemes( LinguisticTaggerRef ref )
{ return (CFArrayRef)[ref tagSchemes]; }

// - determine dominant language and orthography -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFStringRef LinguisticTaggerDominantLanguageForString( CFStringRef string )
{ return (CFStringRef)[NSLinguisticTagger dominantLanguageForString:(NSString *)string]; }

CFStringRef LinguisticTaggerDominantLanguage( LinguisticTaggerRef ref )
{ return (CFStringRef)[ref dominantLanguage]; }
#endif

OrthographyRef LinguisticTaggerOrthographyAtIndex( LinguisticTaggerRef ref, NSUInteger index, CFRange *effectiveRange )
{ return [ref orthographyAtIndex:index effectiveRange:(NSRangePointer)effectiveRange]; }

void LinguisticTaggerSetOrthography( LinguisticTaggerRef ref, OrthographyRef orthRef, CFRange range )
{ [ref setOrthography:orthRef range:NSMakeRange(range.location,range.length)]; }

// - enumerate -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void LinguisticTaggerEnumerateTagsForUnit( LinguisticTaggerRef ref, CFRange range, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSLinguisticTaggerOptions options, CocoaLinguisticTaggerCallbackType2 callback, void *userData )
{
    [ref enumerateTagsInRange:NSMakeRange(range.location,range.length)
                         unit:unit
                       scheme:(NSString *)scheme
                      options:options
                   usingBlock:^(NSString *tag, NSRange tokenRange, BOOL *stop) {
                       BOOL stopFlag = NO;
                       if ( callback ) stopFlag = (*callback)(ref,(CFStringRef)tag, CFRangeMake(tokenRange.location,tokenRange.length), userData );
                       if ( stopFlag ) *stop = YES;
                   }];
}
#endif

void LinguisticTaggerEnumerateTagsForScheme( LinguisticTaggerRef ref, CFRange range, CFStringRef scheme, NSLinguisticTaggerOptions options, CocoaLinguisticTaggerCallbackType1 callback, void *userData )
{
    [ref enumerateTagsInRange:NSMakeRange(range.location,range.length)
                       scheme:(NSString *)scheme
                      options:options
                   usingBlock:^(NSString *tag, NSRange tokenRange, NSRange sentenceRange, BOOL *stop) {
                       BOOL stopFlag = NO;
                       if ( callback ) stopFlag = (*callback)(ref,(CFStringRef)tag, CFRangeMake(tokenRange.location,tokenRange.length), CFRangeMake(sentenceRange.location,sentenceRange.length), userData );
                       if ( stopFlag ) *stop = YES;
                   }];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void LinguisticTaggerEnumerateTagsForString( CFStringRef string, CFRange range, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSLinguisticTaggerOptions options, OrthographyRef orthRef, CocoaLinguisticTaggerCallbackType2 callback, void *userData )
{
    [NSLinguisticTagger enumerateTagsForString:(NSString *)string
                                         range:NSMakeRange(range.location,range.length)
                                          unit:unit
                                        scheme:(NSString *)scheme
                                       options:options
                                   orthography:orthRef
                                    usingBlock:^(NSString *tag, NSRange tokenRange, BOOL *stop) {
                                        BOOL stopFlag = NO;
                                        if ( callback ) stopFlag = (*callback)(nil, (CFStringRef)tag, CFRangeMake(tokenRange.location,tokenRange.length), userData );
                                        if ( stopFlag ) *stop = YES;
                                    }];
}
#endif

// - get -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFStringRef LinguisticTaggerTagForUnit( LinguisticTaggerRef ref, NSUInteger index, NSLinguisticTaggerUnit unit, CFStringRef scheme, CFRange *tokenRange )
{ return (CFStringRef)[ref tagAtIndex:index unit:unit scheme:(NSString *)scheme tokenRange:(NSRangePointer)tokenRange]; }
#endif

CFStringRef LinguisticTaggerTagForScheme( LinguisticTaggerRef ref, NSUInteger index, CFStringRef scheme, CFRange *tokenRange, CFRange *sentenceRange )
{ return (CFStringRef)[ref tagAtIndex:index scheme:(NSString *)scheme tokenRange:(NSRangePointer)tokenRange sentenceRange:(NSRangePointer)sentenceRange]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFStringRef LinguisticTaggerTagForString( CFStringRef string, NSUInteger index, NSLinguisticTaggerUnit unit, CFStringRef scheme, OrthographyRef orthRef, CFRange *tokenRange )
{ return (CFStringRef)[NSLinguisticTagger tagForString:(NSString *)string atIndex:index unit:unit scheme:(NSString *)scheme orthography:orthRef tokenRange:(NSRangePointer)tokenRange]; }
CFArrayRef LinguisticTaggerTagsForUnit( LinguisticTaggerRef ref, CFRange range, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSLinguisticTaggerOptions options, CFArrayRef *tokenRanges )
{ return (CFArrayRef)[ref tagsInRange:NSMakeRange(range.location,range.length) unit:unit scheme:(NSString *)scheme options:options tokenRanges:(NSArray **)tokenRanges]; }
#endif
CFArrayRef LinguisticTaggerTagsForScheme( LinguisticTaggerRef ref, CFRange range, CFStringRef scheme, NSLinguisticTaggerOptions options, CFArrayRef *tokenRanges )
{ return (CFArrayRef)[ref tagsInRange:NSMakeRange(range.location,range.length) scheme:(NSString *)scheme options:options tokenRanges:(NSArray **)tokenRanges]; }
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFArrayRef LinguisticTaggerTagsForString( CFStringRef string, CFRange range, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSLinguisticTaggerOptions options, OrthographyRef orthRef, CFArrayRef *tokenRanges )
{ return (CFArrayRef)[NSLinguisticTagger tagsForString:(NSString *)string range:NSMakeRange(range.location,range.length) unit:unit scheme:(NSString *)scheme options:options orthography:orthRef tokenRanges:(NSArray **)tokenRanges]; }
#endif

// - range of unit token -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFRange LinguisticTaggerTokenRangeAtIndex( LinguisticTaggerRef ref, NSUInteger index, NSLinguisticTaggerUnit unit )
{
    NSRange range = [ref tokenRangeAtIndex:index unit:unit];
    return CFRangeMake(range.location,range.length);
}
#endif
CFRange LinguisticTaggerSentenceRangeForRange( LinguisticTaggerRef ref, CFRange range )
{
    NSRange outRange = [ref sentenceRangeForRange:(NSMakeRange(range.location,range.length))];
    return CFRangeMake(outRange.location,outRange.length);
}

// - possible tags -
CFArrayRef LinguisticTaggerPossibleTagsAtIndex( LinguisticTaggerRef ref, NSUInteger index, CFStringRef scheme, CFRange *tokenRange, CFRange *sentenceRange, CFArrayRef *scores )
{ return (CFArrayRef)[ref possibleTagsAtIndex:index scheme:(NSString *)scheme tokenRange:(NSRangePointer)tokenRange sentenceRange:(NSRangePointer)sentenceRange scores:(NSArray **)scores]; }

// Notify changes to analyzed string
void LinguisticTaggerStringEditedInRange( LinguisticTaggerRef ref, CFRange newRange, NSInteger delta )
{ [ref stringEditedInRange:NSRangeFromCFRange(newRange) changeInLength:delta]; }

