/*
 Locale.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// Init
CFLocaleRef LocaleWithIdentifier( CFStringRef identifier );

// User's locale
CFLocaleRef LocaleAutoupdatingCurrent( void );
CFLocaleRef LocaleCurrent( void );
CFLocaleRef LocaleSystem( void );

// Known identifiers and codes
CFArrayRef LocaleAvailableIdentifiers( void );
CFArrayRef LocaleISOCountryCodes( void );
CFArrayRef LocaleISOLanguageCodes( void );
CFArrayRef LocaleISOCurrencyCodes( void );
CFArrayRef LocaleCommonISOCurrencyCodes( void );

// Converting between identifiers
CFStringRef LocaleCanonicalIdentifierFromString( CFStringRef string );
CFDictionaryRef LocaleComponentsFromIdentifier( CFStringRef identifier );
CFStringRef LocaleIdentifierFromComponents( CFDictionaryRef components );
CFStringRef LocaleCanonicalLanguageIdentifierFromString( CFStringRef string );
CFStringRef LocaleIdentifierFromWindowsLocaleCode( UInt32 lcid );
UInt32 LocaleWindowsLocaleCodeFromIdentifier( CFStringRef identifier );

// Locale info
CFStringRef LocaleIdentifier( CFLocaleRef locale );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFStringRef LocaleCountryCode( CFLocaleRef locale );
CFStringRef LocaleLanguageCode( CFLocaleRef locale );
CFStringRef LocaleScriptCode( CFLocaleRef locale );
CFStringRef LocaleVariantCode( CFLocaleRef locale );
CFCharacterSetRef LocaleExemplarCharacterSet( CFLocaleRef locale );
CFStringRef LocaleCollationIdentifier( CFLocaleRef locale );
CFStringRef LocaleCollatorIdentifier( CFLocaleRef locale );
Boolean LocaleUsesMetricSystem( CFLocaleRef locale );
CFStringRef LocaleDecimalSeparator( CFLocaleRef locale );
CFStringRef LocaleGroupingSeparator( CFLocaleRef locale );
CFStringRef LocaleCurrencyCode( CFLocaleRef locale );
CFStringRef LocaleCurrencySymbol( CFLocaleRef locale );
CFStringRef LocaleCalendarIdentifier( CFLocaleRef locale );
CFStringRef LocaleQuotationBeginDelimiter( CFLocaleRef locale );
CFStringRef LocaleQuotationEndDelimiter( CFLocaleRef locale );
CFStringRef LocaleAlternateQuotationBeginDelimiter( CFLocaleRef locale );
CFStringRef LocaleAlternateQuotationEndDelimiter( CFLocaleRef locale );
#endif

// Display info
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFStringRef LocaleLocalizedStringForIdentifier( CFLocaleRef locale, CFStringRef identifier );
CFStringRef LocaleLocalizedStringForCountryCode( CFLocaleRef locale, CFStringRef code );
CFStringRef LocaleLocalizedStringForLanguageCode( CFLocaleRef locale, CFStringRef code );
CFStringRef LocaleLocalizedStringForScriptCode( CFLocaleRef locale, CFStringRef code );
CFStringRef LocaleLocalizedStringForVariantCode( CFLocaleRef locale, CFStringRef code );
CFStringRef LocaleLocalizedStringForCollationIdentifier( CFLocaleRef locale, CFStringRef identifier );
CFStringRef LocaleLocalizedStringForCollatorIdentifier( CFLocaleRef locale, CFStringRef identifier );
CFStringRef LocaleLocalizedStringForCurrencyCode( CFLocaleRef locale, CFStringRef code );
CFStringRef LocaleLocalizedStringForCalendarIdentifier( CFLocaleRef locale, CFStringRef identifier );
#endif

// Info by key
CFTypeRef LocaleObjectForKey( CFLocaleRef locale, CFStringRef key );
CFStringRef LocaleDisplayNameForKey( CFLocaleRef locale, CFStringRef key, CFTypeRef value );

// Preferred languages
CFArrayRef LocalePreferredLanguages( void );

// Line and character direction
NSLocaleLanguageDirection LocaleCharacterDirectionForLanguage( CFStringRef isoLangCode );
NSLocaleLanguageDirection LocaleLineDirectionForLanguage( CFStringRef isoLangCode );
