/*
 Locale.m

 Bernie Wylde
 */

#import "Locale.h"

// Init
CFLocaleRef LocaleWithIdentifier( CFStringRef identifier )
{ return (CFLocaleRef)[[[NSLocale alloc] initWithLocaleIdentifier:(NSString *)identifier] autorelease]; }

// User's locale
CFLocaleRef LocaleAutoupdatingCurrent( void )
{ return (CFLocaleRef)[NSLocale autoupdatingCurrentLocale]; }

CFLocaleRef LocaleCurrent( void )
{ return (CFLocaleRef)[NSLocale currentLocale]; }

CFLocaleRef LocaleSystem( void )
{ return (CFLocaleRef)[NSLocale systemLocale]; }

// Known identifiers and codes
CFArrayRef LocaleAvailableIdentifiers( void )
{ return (CFArrayRef)[NSLocale availableLocaleIdentifiers]; }

CFArrayRef LocaleISOCountryCodes( void )
{ return (CFArrayRef)[NSLocale ISOCountryCodes]; }

CFArrayRef LocaleISOLanguageCodes( void )
{ return (CFArrayRef)[NSLocale ISOLanguageCodes]; }

CFArrayRef LocaleISOCurrencyCodes( void )
{ return (CFArrayRef)[NSLocale ISOCurrencyCodes]; }

CFArrayRef LocaleCommonISOCurrencyCodes( void )
{ return (CFArrayRef)[NSLocale commonISOCurrencyCodes]; }

// Converting between identifiers
CFStringRef LocaleCanonicalIdentifierFromString( CFStringRef string )
{ return (CFStringRef)[NSLocale canonicalLocaleIdentifierFromString:(NSString *)string]; }

CFDictionaryRef LocaleComponentsFromIdentifier( CFStringRef identifier )
{ return (CFDictionaryRef)[NSLocale componentsFromLocaleIdentifier:(NSString *)identifier]; }

CFStringRef LocaleIdentifierFromComponents( CFDictionaryRef components )
{ return (CFStringRef)[NSLocale localeIdentifierFromComponents:(NSDictionary *)components]; }

CFStringRef LocaleCanonicalLanguageIdentifierFromString( CFStringRef string )
{ return (CFStringRef)[NSLocale canonicalLanguageIdentifierFromString:(NSString *)string]; }

CFStringRef LocaleIdentifierFromWindowsLocaleCode( UInt32 lcid )
{ return (CFStringRef)[NSLocale localeIdentifierFromWindowsLocaleCode:lcid]; }

UInt32 LocaleWindowsLocaleCodeFromIdentifier( CFStringRef identifier )
{ return [NSLocale windowsLocaleCodeFromLocaleIdentifier:(NSString *)identifier]; }

// Locale info
CFStringRef LocaleIdentifier( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale localeIdentifier]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFStringRef LocaleCountryCode( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale countryCode]; }

CFStringRef LocaleLanguageCode( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale languageCode]; }

CFStringRef LocaleScriptCode( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale scriptCode]; }

CFStringRef LocaleVariantCode( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale variantCode]; }

CFCharacterSetRef LocaleExemplarCharacterSet( CFLocaleRef locale )
{ return (CFCharacterSetRef)[(NSLocale *)locale exemplarCharacterSet]; }

CFStringRef LocaleCollationIdentifier( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale collationIdentifier]; }

CFStringRef LocaleCollatorIdentifier( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale collatorIdentifier]; }

Boolean LocaleUsesMetricSystem( CFLocaleRef locale )
{ return [(NSLocale *)locale usesMetricSystem]; }

CFStringRef LocaleDecimalSeparator( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale decimalSeparator]; }

CFStringRef LocaleGroupingSeparator( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale groupingSeparator]; }

CFStringRef LocaleCurrencyCode( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale currencyCode]; }

CFStringRef LocaleCurrencySymbol( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale currencySymbol]; }

CFStringRef LocaleCalendarIdentifier( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale calendarIdentifier]; }

CFStringRef LocaleQuotationBeginDelimiter( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale quotationBeginDelimiter]; }

CFStringRef LocaleQuotationEndDelimiter( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale quotationEndDelimiter]; }

CFStringRef LocaleAlternateQuotationBeginDelimiter( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale alternateQuotationBeginDelimiter]; }

CFStringRef LocaleAlternateQuotationEndDelimiter( CFLocaleRef locale )
{ return (CFStringRef)[(NSLocale *)locale alternateQuotationEndDelimiter]; }
#endif

// Display info
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFStringRef LocaleLocalizedStringForIdentifier( CFLocaleRef locale, CFStringRef identifier )
{ return (CFStringRef)[(NSLocale *)locale localizedStringForLocaleIdentifier:(NSString *)identifier]; }

CFStringRef LocaleLocalizedStringForCountryCode( CFLocaleRef locale, CFStringRef code )
{ return (CFStringRef)[(NSLocale *)locale localizedStringForCountryCode:(NSString *)code]; }

CFStringRef LocaleLocalizedStringForLanguageCode( CFLocaleRef locale, CFStringRef code )
{ return (CFStringRef)[(NSLocale *)locale localizedStringForLanguageCode:(NSString *)code]; }

CFStringRef LocaleLocalizedStringForScriptCode( CFLocaleRef locale, CFStringRef code )
{ return (CFStringRef)[(NSLocale *)locale localizedStringForScriptCode:(NSString *)code]; }

CFStringRef LocaleLocalizedStringForVariantCode( CFLocaleRef locale, CFStringRef code )
{ return (CFStringRef)[(NSLocale *)locale localizedStringForVariantCode:(NSString *)code]; }

CFStringRef LocaleLocalizedStringForCollationIdentifier( CFLocaleRef locale, CFStringRef identifier )
{ return (CFStringRef)[(NSLocale *)locale localizedStringForCollationIdentifier:(NSString *)identifier]; }

CFStringRef LocaleLocalizedStringForCollatorIdentifier( CFLocaleRef locale, CFStringRef identifier )
{ return (CFStringRef)[(NSLocale *)locale localizedStringForCollatorIdentifier:(NSString *)identifier]; }

CFStringRef LocaleLocalizedStringForCurrencyCode( CFLocaleRef locale, CFStringRef code )
{ return (CFStringRef)[(NSLocale *)locale localizedStringForCurrencyCode:(NSString *)code]; }

CFStringRef LocaleLocalizedStringForCalendarIdentifier( CFLocaleRef locale, CFStringRef identifier )
{ return (CFStringRef)[(NSLocale *)locale localizedStringForCalendarIdentifier:(NSString *)identifier]; }
#endif

// Info by key
CFTypeRef LocaleObjectForKey( CFLocaleRef locale, CFStringRef key )
{ return (CFTypeRef)[(NSLocale *)locale objectForKey:(NSString *)key]; }

CFStringRef LocaleDisplayNameForKey( CFLocaleRef locale, CFStringRef key, CFTypeRef value )
{ return (CFStringRef)[(NSLocale *)locale displayNameForKey:(NSString *)key value:(id)value]; }

// Preferred languages
CFArrayRef LocalePreferredLanguages( void )
{ return (CFArrayRef)[NSLocale preferredLanguages]; }

// Line and character direction
NSLocaleLanguageDirection LocaleCharacterDirectionForLanguage( CFStringRef isoLangCode )
{ return [NSLocale characterDirectionForLanguage:(NSString *)isoLangCode]; }

NSLocaleLanguageDirection LocaleLineDirectionForLanguage( CFStringRef isoLangCode )
{ return [NSLocale lineDirectionForLanguage:(NSString *)isoLangCode]; }

