/*
 Menu.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef void (*CocoaMenuItemCallbackType)(NSInteger,NSInteger,void *);
typedef BOOL (*CocoaValidateMenuItemCallbackType)(NSInteger,NSInteger);

void BuildMinimalMenu( void );

@interface CocoaMenuItemTarget : NSObject {
    CocoaValidateMenuItemCallbackType validateCallback;
    BOOL boolVal;
}
@property (assign) CocoaValidateMenuItemCallbackType validateCallback;
@property (assign) BOOL boolVal;
@end

void CocoaNibMenuSetup( NSMenu *menu );

CocoaMenuItemTarget *CocoaMenuItemTargetGet( void );

void CocoaOtherMenuAdd( NSMenu *menu );
NSMenu *CocoaOtherMenuAtIndex( NSInteger index );
NSInteger CocoaMenuIndex( NSMenu *menu );

@interface NSMenu (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

// private
void CocoaMenuPadItemsToIndex( NSMenu *menu, NSInteger index );




@interface NSMenuItem (Additions)
@property (nonatomic,retain) NSValue *callbackValue;
@property (nonatomic,retain) NSValue *userDataValue;

- (CocoaMenuItemCallbackType)cocoaMenuItemCallback;
- (void *)cocoaMenuItemUserData;
- (void)setCocoaMenuItemCallback:(CocoaMenuItemCallbackType)cb;
- (void)setCocoaMenuItemUserData:(void *)userData;
@end



void MenuEventSetBool( Boolean flag );

NSMenu *MenuAtIndex( NSInteger index );
BOOL MenuExists( NSInteger index );
NSInteger MenuIndex( CocoaMenuRef m );


// MenuBar
Boolean MenuBarVisible( void );
void MenuBarSetVisible( Boolean flag );
CGFloat MenuBarHeight( void );

// Init
CocoaMenuRef MenuWithTitle( CFStringRef title );

// Add remove items
void MenuInsertItem( NSInteger menuIndex, NSInteger itemIndex, CFStringRef title, CFStringRef keyEquivalent, NSUInteger keyEquivalentModifierMask );
void MenuAddItem( NSInteger menuIndex, CFStringRef title, CFStringRef keyEquivalent, NSUInteger keyEquivalentModifierMask );
void MenuRemoveItem( NSInteger menuIndex, NSInteger itemIndex );
void MenuRemoveAllItems( NSInteger menuIndex );

// Find items
NSInteger MenuItemWithTitle( NSInteger menuIndex, CFStringRef title );
NSInteger MenuNumberOfItems( NSInteger menuIndex );
CFArrayRef MenuItemArray( NSInteger menuIndex );

// Find indices of items
NSInteger MenuIndexOfItemWithTitle( NSInteger menuIndex, CFStringRef title );
NSInteger MenuIndexOfItemWithTag( NSInteger menuIndex, NSInteger tag );

// Submenus
void MenuSetSubmenu( NSInteger parMenuIndex, NSInteger parItemIndex, NSInteger submenuIndex );

// Enable/disable items
void MenuSetAutoenablesItems( NSInteger menuIndex, Boolean flag );

// Font
CTNSFontRef MenuFont( NSInteger menuIndex );
void MenuSetFont( NSInteger menuIndex, CTNSFontRef inFont );
void MenuSetFontWithName( NSInteger menuIndex, CFStringRef inName, CGFloat size );// convenience

// Title
CFStringRef MenuTitle( NSInteger index );

// Size
CGFloat MenuMinimumWidth( NSInteger menuIndex );
void MenuSetMinimumWidth( NSInteger menuIndex, CGFloat width );
CGSize MenuSize( NSInteger menuIndex );

// Context-sensitive help
Boolean MenuPopUp( NSInteger menuIndex, NSInteger itemIndex, CGPoint location, NSInteger viewTag );

// State column
Boolean MenuShowsStateColumn( NSInteger menuIndex );
void MenuSetShowsStateColumn( NSInteger menuIndex, Boolean flag );

// Highlighting
NSInteger MenuHighlightedItem( NSInteger menuIndex );

// Custom
void MenuSetTag( NSInteger menuIndex, NSInteger tag );
void MenuSetOneItemOnState( NSInteger menuIndex, NSInteger itemIndex );
void MenuSetValidateItemsCallback( CocoaValidateMenuItemCallbackType callback );
Boolean MenuIsHidden( NSInteger menuIndex );
void MenuSetHidden( NSInteger menuIndex, Boolean flag );
Boolean MenuIsHiddenOrHasHiddenAncestor( NSInteger menuIndex );

CFTypeRef MenuProperty( NSInteger menuIndex, CFStringRef key );
void MenuSetProperty( NSInteger menuIndex, CFStringRef key, CFTypeRef value );
void MenuRemoveProperty( NSInteger menuIndex, CFStringRef key );
void MenuRemoveAllProperties( NSInteger menuIndex );

// FB keywords
void CocoaMenu( NSInteger inMenuIndex, NSInteger inItemIndex, NSInteger inEnabled, CFStringRef inTitle, CFStringRef inKeyEquivalent, NSInteger inKeyEquivalentModifier );

void CocoaEditMenu( NSInteger inMenuIndex );
void CocoaWindowMenu( NSInteger inMenuIndex );
void CocoaFileMenu( NSInteger inMenuIndex );
void CocoaFindMenu( NSInteger inMenuIndex );
void CocoaFormatMenu( NSInteger inMenuIndex );
void CocoaFontMenu( NSInteger inMenuIndex );
void CocoaTextMenu( NSInteger inMenuIndex );
void CocoaViewMenu( NSInteger inMenuIndex );
void CocoaHelpMenu( NSInteger inMenuIndex );

#pragma mark - nibmenu
void CocoaNibMenu( CFStringRef inNibName );

