/*
 Menu.m

 Bernie Wylde
 */

#import "Menu.h"

void BuildMinimalMenu( void )
{
    NSString *appName = [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleName"];
    
    NSMenu *mainMenu = [NSApp mainMenu];
    if ( !mainMenu ) mainMenu = [[[NSMenu alloc] initWithTitle:@"MainMenu"] autorelease];
    
    NSMenuItem *item = [mainMenu addItemWithTitle:appName action:nil keyEquivalent:@""];
    NSMenu *mApp = [[[NSMenu alloc] initWithTitle:appName] autorelease];
    [mainMenu setSubmenu:mApp forItem:item];
    
    [mApp addItemWithTitle:[NSString stringWithFormat:@"About %@", appName] action:@selector(orderFrontStandardAboutPanel:) keyEquivalent:@""];
    [mApp addItem:[NSMenuItem separatorItem]];
    [mApp addItemWithTitle:[NSString stringWithFormat:@"Quit %@", appName] action:@selector(terminate:) keyEquivalent:@"q"];
    [NSApp setMainMenu:mainMenu];
}

@implementation CocoaMenuItemTarget

@synthesize validateCallback;
@synthesize boolVal;


- (void)menuItemAction:(id)sender {
    NSMenu *menu = [sender menu];
    NSInteger menuIndex = [menu tag];
    if ( menuIndex <= 0 ) {
        NSMenu *supermenu = [menu supermenu];
        menuIndex = [supermenu indexOfItemWithSubmenu:menu];
    }
    NSInteger itemIndex = [menu indexOfItem:sender];
    
    CocoaMenuItemCallbackType callback = [sender cocoaMenuItemCallback];
    if ( callback ) {
        void *userData = [sender cocoaMenuItemUserData];
        (*callback)(menuIndex,itemIndex,userData);
    } else {
        if ( gFBOnMenuVector ) {
            gFBMenuEventInfo.menu = menuIndex;
            gFBMenuEventInfo.item = itemIndex;
            (*gFBOnMenuVector)( gFBMenuEventInfo.menu, gFBMenuEventInfo.item ); // call user's DoMenu function
        }
    }
}

//- (BOOL)validateUserInterfaceItem:(id<NSValidatedUserInterfaceItem>)item {
//    return YES;
//}
//
//- (BOOL)validateMenuItem:(NSMenuItem *)menuItem {
//    boolVal = YES;
//    if ( gFBOnMenuVector ) {
//        NSMenu *menu = [menuItem menu];
//        if ( [menu autoenablesItems] ) {
//            NSInteger menuIndex = [menu tag];
//            if ( menuIndex <= 0 ) {
//                NSMenu *supermenu = [menu supermenu];
//                menuIndex = [supermenu indexOfItemWithSubmenu:menu];
//            }
//            NSInteger itemIndex = [menu indexOfItem:menuItem];
//            gFBMenuEventInfo.menu = -menuIndex;
//            gFBMenuEventInfo.item = itemIndex;
//            (*gFBOnMenuVector)(); // call user's DoMenu function
//        }
//    }
//    return boolVal;
//}
//

- (BOOL)validateMenuItem:(NSMenuItem *)menuItem {
    BOOL flag = [menuItem isEnabled];
    if ( validateCallback ) {
        NSMenu *menu = [menuItem menu];
        NSInteger menuIndex = [menu tag];
        if ( menuIndex <= 0 ) {
            NSMenu *supermenu = [menu supermenu];
            menuIndex = [supermenu indexOfItemWithSubmenu:menu];
        }
        NSInteger itemIndex = [menu indexOfItem:menuItem];
        return (*validateCallback)(menuIndex,itemIndex);
    }
    return flag;
}

@end


@implementation NSMenuItem (Additions)
@dynamic callbackValue;
@dynamic userDataValue;

- (CocoaMenuItemCallbackType)cocoaMenuItemCallback {
    return [objc_getAssociatedObject(self, @selector(callbackValue)) pointerValue];
}

- (void *)cocoaMenuItemUserData {
    return [objc_getAssociatedObject(self, @selector(userDataValue)) pointerValue];
}

- (void)setCocoaMenuItemCallback:(CocoaMenuItemCallbackType)callback {
    objc_setAssociatedObject(self, @selector(callbackValue), [NSValue valueWithPointer:callback], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (void)setCocoaMenuItemUserData:(void *)userData {
    objc_setAssociatedObject(self, @selector(userDataValue), [NSValue valueWithPointer:userData], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end



// private
CocoaMenuItemTarget *CocoaMenuItemTargetGet( void )
{
    static CocoaMenuItemTarget *sCocoaMenuItemTarget = nil;
    if ( !sCocoaMenuItemTarget ) sCocoaMenuItemTarget = [[CocoaMenuItemTarget alloc] init];
    return sCocoaMenuItemTarget;
}

void MenuEventSetBool( Boolean flag )
{ [CocoaMenuItemTargetGet() setBoolVal:flag]; }

static NSMutableArray *sCocoaOtherMenus;

void CocoaOtherMenuAdd( NSMenu *menu )
{
    if ( sCocoaOtherMenus == nil ) sCocoaOtherMenus = [[NSMutableArray alloc] initWithCapacity:0];
    [sCocoaOtherMenus addObject:menu];
}

NSMenu *CocoaOtherMenuAtIndex( NSInteger index )
{
    if ( sCocoaOtherMenus ) {
        for ( NSMenu *menu in sCocoaOtherMenus ) {
            if ( [menu tag] == index ) return menu;
        }
    }
    return nil;
}


NSInteger CocoaMenuIndex( NSMenu *menu ) {
    NSMenu *mainMenu = [NSApp mainMenu];
    NSArray *items = [mainMenu itemArray];
    NSInteger mainMenuItemCount = [items count];
    if ( mainMenuItemCount ) {
        for ( NSInteger index = 0; index < mainMenuItemCount; index++ ) {
            NSMenuItem *item = [mainMenu itemAtIndex:index];
            NSMenu *submenu = [item submenu];
            if ( submenu ) {
                if ( submenu == menu ) return index;
            }
        }
    }
    return [menu tag];
}



@implementation NSMenu (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

// - private -
void CocoaMenuPadItemsToIndex( NSMenu *menu, NSInteger index )
{
    NSInteger itemCount = [menu numberOfItems];
    if ( index > 1 ) {
        while ( itemCount < index ) {
            NSMenuItem *newItem = [menu addItemWithTitle:@"" action:nil keyEquivalent:@""];
            NSMenu *newMenu = [[NSMenu alloc] initWithTitle:@""];
            [menu setSubmenu:newMenu forItem:newItem];
            itemCount++;
        }
    }
}


// - public -
NSMenu *MenuAtIndex( NSInteger index )
{
    if ( index > 1000000 ) return (NSMenu *)index;
    
    if ( index > 100 ) {
        return CocoaOtherMenuAtIndex( index );
    } else {
        NSMenu *mainMenu = [NSApp mainMenu];
        if ( mainMenu ) {
            NSInteger count = [mainMenu numberOfItems];
            if ( index < count ) {
                NSMenuItem *item = [mainMenu itemAtIndex:index];
                return [item submenu];
            }
        }
    }
    return nil;
}

BOOL MenuExists( NSInteger index )
{
    if ( index > 100 ) {
        return ( CocoaOtherMenuAtIndex( index ) != nil );
    } else {
        NSMenu *mainMenu = [NSApp mainMenu];
        if ( mainMenu ) {
            NSInteger count = [mainMenu numberOfItems];
            if ( index < count ) {
                return ( [mainMenu itemAtIndex:index] != nil );
            }
        }
    }
    return NO;
}

NSInteger MenuIndex( CocoaMenuRef m )
{ return CocoaMenuIndex( m ); }


// MenuBar
Boolean MenuBarVisible( void )
{ return [NSMenu menuBarVisible]; }

void MenuBarSetVisible( Boolean flag )
{ [NSMenu setMenuBarVisible:flag]; }

CGFloat MenuBarHeight( void )
{ return [[NSApp mainMenu] menuBarHeight]; }

// Init
CocoaMenuRef MenuWithTitle( CFStringRef title )
{ return [[[NSMenu alloc] initWithTitle:(NSString *)title] autorelease]; }

// Add/remove items
void MenuInsertItem( NSInteger menuIndex, NSInteger itemIndex, CFStringRef title, CFStringRef keyEquivalent, NSUInteger keyEquivalentModifierMask )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSInteger count = [menu numberOfItems];
        if ( itemIndex < count ) {
            NSMenuItem *item = [menu insertItemWithTitle:(NSString *)title action:@selector(menuItemAction:) keyEquivalent:(NSString *)keyEquivalent atIndex:(NSInteger)itemIndex];
            if ( keyEquivalentModifierMask > 0 ) [item setKeyEquivalentModifierMask:keyEquivalentModifierMask];
            [item setTarget:CocoaMenuItemTargetGet()];
        }
    }
}

void MenuAddItem( NSInteger menuIndex, CFStringRef title, CFStringRef keyEquivalent, NSUInteger keyEquivalentModifierMask )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = [menu addItemWithTitle:(NSString *)title action:@selector(menuItemAction:) keyEquivalent:(NSString *)keyEquivalent];
        if ( keyEquivalentModifierMask > 0 ) [item setKeyEquivalentModifierMask:keyEquivalentModifierMask];
        [item setTarget:CocoaMenuItemTargetGet()];
    }
}

void MenuRemoveItem( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        [menu removeItemAtIndex:itemIndex];
    }
}

void MenuRemoveAllItems( NSInteger menuIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        [menu removeAllItems];
    }
}

// Find menu items
NSInteger MenuItemWithTitle( NSInteger menuIndex, CFStringRef title )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        return [menu indexOfItemWithTitle:(NSString *)title];
    }
    return 0;
}

NSInteger MenuNumberOfItems( NSInteger menuIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) return [menu numberOfItems];
    return 0;
}

CFArrayRef MenuItemArray( NSInteger menuIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) return (CFArrayRef)[menu itemArray];
    return NULL;
}

// Find indices of items
NSInteger MenuIndexOfItemWithTitle( NSInteger menuIndex, CFStringRef title )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) return [menu indexOfItemWithTitle:(NSString *)title];
    return -1;
}

NSInteger MenuIndexOfItemWithTag( NSInteger menuIndex, NSInteger tag )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) return [menu indexOfItemWithTag:tag];
    return -1;
}

// Submenus
void MenuSetSubmenu( NSInteger parMenuIndex, NSInteger parItemIndex, NSInteger submenuIndex )
{
    NSMenu *parMenu = MenuAtIndex( parMenuIndex );
    if ( parMenu ) {
        NSMenuItem *parItem = [parMenu itemAtIndex:parItemIndex];
        if ( parItem ) {
            NSMenu *submenu = MenuAtIndex( submenuIndex );
            if ( submenu ) {
                [parMenu setSubmenu:submenu forItem:parItem];
            }
        }
    }
}

// Enable/disable menu items
void MenuSetAutoenablesItems( NSInteger menuIndex, Boolean flag )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        [menu setAutoenablesItems:flag];
    }
}

// Font
CTNSFontRef MenuFont( NSInteger menuIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        return [menu font];
    }
    return nil;
}


void MenuSetFont( NSInteger menuIndex, CTNSFontRef inFont )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSFont *font = FixFont( (void *)inFont );
        [menu setFont:(NSFont *)font];
    }
}

void MenuSetFontWithName( NSInteger menuIndex, CFStringRef inName, CGFloat size )// convenience
{
    BOOL usePrevFont = NO;
    NSString *name = (NSString *)inName;
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        if ( name ) {
            usePrevFont = ( [name length] == 0 );
        } else {
            usePrevFont = YES;
        }
        if ( usePrevFont ) {
            name = [[menu font] fontName];
        }
        if ( (int)size == 0 ) size = [[menu font] pointSize];
        //        [menu setFont:[NSFont fontWithName:(NSString *)name size:size]];
        CTFontRef font = CTFontCreateWithName( (CFStringRef)name, size, NULL );
        [menu setFont:(NSFont *)font];
        CFRelease(font);
    }
}

// Title
CFStringRef MenuTitle( NSInteger index )
{
    NSMenu *menu = MenuAtIndex( index );
    if ( menu ) {
        NSString *title = [menu title];
        if ( title != nil ) {
            if ( [title length] == 0 ) {
                NSMenu *supermenu = [menu supermenu];
                if ( supermenu ) {
                    for ( NSMenuItem *item in [supermenu itemArray] ) {
                        if ( [item submenu] == menu ) {
                            title = [item title];
                            break;
                        }
                    }
                }
            }
        }
        return (CFStringRef)title;
    }
    //    NSMenu *mainMenu = [NSApp mainMenu];
    //    if ( mainMenu ) {
    //        NSMenuItem *item = CocoaMenuItemAtIndex( mainMenu, index );
    //        if ( item ) return (CFStringRef)[item title];
    //    }
    return NULL;
}

// Size
CGFloat MenuMinimumWidth( NSInteger menuIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) return [menu minimumWidth];
    return 0.0;
}

void MenuSetMinimumWidth( NSInteger menuIndex, CGFloat width )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) [menu setMinimumWidth:width];
}

CGSize MenuSize( NSInteger menuIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) return NSSizeToCGSize([menu size]);
    return CGSizeZero;
}

// Context-sensitive help
Boolean MenuPopUp( NSInteger menuIndex, NSInteger itemIndex, CGPoint location, NSInteger viewTag )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = [menu itemAtIndex:itemIndex];
        if ( item ) {
            NSView *view = ViewWithTag( viewTag );
            if ( view ) {
                return [menu popUpMenuPositioningItem:item atLocation:NSPointFromCGPoint(location) inView:view];
            }
        }
    }
    return NO;
}

// State column
Boolean MenuShowsStateColumn( NSInteger menuIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) return [menu showsStateColumn];
    return NO;
}

void MenuSetShowsStateColumn( NSInteger menuIndex, Boolean flag )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        [menu setShowsStateColumn:flag];
    }
}

// Highlighting
NSInteger MenuHighlightedItem( NSInteger menuIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = [menu highlightedItem];
        if ( item ) {
            return [menu indexOfItem:item];
        }
    }
    return -1;
}

// Custom
void MenuSetTag( NSInteger menuIndex, NSInteger tag )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = [[menu supermenu] itemAtIndex:menuIndex];
        if ( item ) [item setTag:tag];
    }
}

void MenuSetOneItemOnState( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSInteger count = [menu numberOfItems];
        for ( NSInteger index = 0; index < count; index++ ) {
            NSMenuItem *item = [menu itemAtIndex:index];
            [item setState:(index == itemIndex )];
        }
    }
}

void MenuSetValidateItemsCallback( CocoaValidateMenuItemCallbackType callback )
{
    CocoaMenuItemTarget *target = CocoaMenuItemTargetGet();
    if ( target ) {
        target.validateCallback = callback;
    }
}

Boolean MenuIsHidden( NSInteger menuIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = [[menu supermenu] itemAtIndex:menuIndex];
        if ( item ) return [item isHidden];
    }
    return NO;
}

void MenuSetHidden( NSInteger menuIndex, Boolean flag )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = [[menu supermenu] itemAtIndex:menuIndex];
        if ( item ) [item setHidden:flag];
    }
}

Boolean MenuIsHiddenOrHasHiddenAncestor( NSInteger menuIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = [[menu supermenu] itemAtIndex:menuIndex];
        if ( item ) return [item isHiddenOrHasHiddenAncestor];
    }
    return NO;
}



CFTypeRef MenuProperty( NSInteger menuIndex, CFStringRef key )
{
    id property = nil;
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        property = ObjectProperty( menu, key );
    }
    return (CFTypeRef)property;
}

void MenuSetProperty( NSInteger menuIndex, CFStringRef key, CFTypeRef value )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        ObjectSetProperty( menu, key, value );
    }
}

void MenuRemoveProperty( NSInteger menuIndex, CFStringRef key )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        ObjectRemoveProperty( menu, key );
    }
}

void MenuRemoveAllProperties( NSInteger menuIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        ObjectRemoveAllProperties( menu );
    }
}



// FB keywords
void CocoaMenu( NSInteger inMenuIndex, NSInteger inItemIndex, NSInteger inEnabled, CFStringRef inTitle, CFStringRef inKeyEquivalent, NSInteger inKeyEquivalentModifier )
{
    if ( !IsCocoaApplication() ) {
        CocoaShowAlert( NSCriticalAlertStyle, @"Runtime error.", @"Cocoa Menu can only be used with Cocoa runtime." );
        return;
    }
    
    // title
    NSString *title = (NSString *)inTitle;
        
    // keyEquivalent
//     NSString *keyEquivalent = nil;
//     if ( inKeyEquivalent ) {
//         keyEquivalent = [NSString stringWithString:(NSString *)inKeyEquivalent];
//     } else {
//         keyEquivalent = @"";
//     }
    
    NSString *keyEquivalent = (NSString *)inKeyEquivalent;// 20180726 bw
    if ( !keyEquivalent ) keyEquivalent = @"";// 20180726 bw
    
    // make sure we have at least a minimal menu
    if ( ![NSApp mainMenu] ) {
        BuildMinimalMenu();
    }
    
    // get the main menu
    NSMenu *mainMenu = [NSApp mainMenu];
    if ( mainMenu ) {
        
        if ( inMenuIndex == 0 ) { // application menu
            NSMenu *appMenu = [[mainMenu itemAtIndex:0] submenu];
            
            NSMenuItem *item = [appMenu itemAtIndex:inItemIndex];
            
            BOOL userItemExists = NO;
            if ( item ) {
                if ( [item tag] ) {
                    userItemExists = YES;
                }
            }
            
            if ( userItemExists ) {   // item exists
                
                // enabled
                if ( inEnabled != kFBParamMissing ) [item setEnabled:inEnabled];
                
                // title
                if ( title ) [item setTitle:title];
                
                // keyEquivalent
                if ( [keyEquivalent length] ) [item setKeyEquivalent:keyEquivalent];// 20180726 bw
//                if ( inKeyEquivalent ) [item setKeyEquivalent:keyEquivalent];

                // keyEquivalentModifierMask
                if ( inKeyEquivalentModifier != kFBParamMissing ) [item setKeyEquivalentModifierMask:inKeyEquivalentModifier];
                
                // target
                if ( ![item target] ) [item setTarget:CocoaMenuItemTargetGet()];// 20180726 bw

                // action
                if ( ![item action] ) [item setAction:@selector(menuItemAction:)];// 20180726 bw
                
            } else {        // create item
                
                NSInteger index = 1;
                while ( [[appMenu itemAtIndex:index] tag] != 0 ) {
                    index++;
                }
                
                if ( index ) {
                    BOOL separator = ( title == NULL || [title isEqualToString:@"-"] );
                    
                    NSMenuItem *newAppItem;
                    
                    if ( separator ) {
                        newAppItem = [NSMenuItem separatorItem];
                    } else {
                        newAppItem = [[NSMenuItem alloc] initWithTitle:title action:@selector(menuItemAction:) keyEquivalent:keyEquivalent];
                        
                        // key equivalent modifier
                        if ( inKeyEquivalentModifier != kFBParamMissing ) [newAppItem setKeyEquivalentModifierMask:inKeyEquivalentModifier];
                        [newAppItem setTarget:CocoaMenuItemTargetGet()];// 20180726 bw

                        // target
                        //[newAppItem setTarget:CocoaMenuItemTargetGet()];// 20180726 bw
                        
                        // action
                        //[item setAction:@selector(menuItemAction:)];// 20180726 bw
                    }
                    [newAppItem setTag:-123];
                    [appMenu insertItem:newAppItem atIndex:index];
                }
            }
            
        } else {        // menu other than the application menu
            
            BOOL otherMenuFlag = ( inMenuIndex > 100 ); // non-menubar menu
            
            NSInteger mainMenuItemCount = [mainMenu numberOfItems];
            NSInteger itemCount = 0;
            NSMenu *menu = nil;
            NSMenuItem *menuItem = nil;
            
            if ( otherMenuFlag ) {           // other menu
                menu = CocoaOtherMenuAtIndex( inMenuIndex );
            } else {                         // menu bar menu
                
                itemCount = mainMenuItemCount;
                if ( inMenuIndex < itemCount ) {
                    menuItem = [mainMenu itemAtIndex:inMenuIndex];
                    menu = [menuItem submenu];
                }
            }
            
            if ( menu ) {
                if ( inItemIndex < 0 ) {    // we're changing the menu itself
                    
                    if ( menuItem ) {   // main menu item
                        // enabled
                        if ( inEnabled != kFBParamMissing ) [menuItem setEnabled:inEnabled];
                    }
                    
                    // title
                    if ( title ) [menu setTitle:title];
                    
                } else {    // add/change item
                    
                    NSMenuItem *item = nil;
                    
                    NSInteger itemCount = [menu numberOfItems];
                    
                    if ( inItemIndex < itemCount ) {   // item exists
                        
                        item = [menu itemAtIndex:inItemIndex];
                        
                        // enabled
                        if ( inEnabled != kFBParamMissing ) [item setEnabled:inEnabled];
                        
                        // title
                        if ( title ) [item setTitle:title];
                        
                        // keyEquivalent
                        if ( [keyEquivalent length] ) [item setKeyEquivalent:keyEquivalent];
//                        if ( inKeyEquivalent ) [item setKeyEquivalent:keyEquivalent];// 20180726 bw

                        // keyEquivalentModifierMask
                        if ( inKeyEquivalentModifier != kFBParamMissing ) [item setKeyEquivalentModifierMask:inKeyEquivalentModifier];
                        
                        // target
                        if ( ![item target] ) [item setTarget:CocoaMenuItemTargetGet()];// 20180726 bw
                        
                        // action
                        if ( ![item action] ) [item setAction:@selector(menuItemAction:)];// 20180726 bw
                        
                    } else {        // create item
                        
                        while ( inItemIndex > itemCount ) {
                            [menu addItem:[NSMenuItem separatorItem]];
                            itemCount = [menu numberOfItems];
                        }
                        
                        BOOL separator = ( title == NULL || [title isEqualToString:@"-"] );
                        
                        if ( separator ) {
                            item = [NSMenuItem separatorItem];
                        } else {
                            item = [[NSMenuItem alloc] initWithTitle:title action:@selector(menuItemAction:) keyEquivalent:keyEquivalent];
                            
                            // key equivalent modifier
                            if ( inKeyEquivalentModifier != kFBParamMissing ) [item setKeyEquivalentModifierMask:inKeyEquivalentModifier];
                            
                            // enabled
                            if ( inEnabled != kFBParamMissing ) [item setEnabled:inEnabled];
                            
                            // target
                            [item setTarget:CocoaMenuItemTargetGet()];
                            
                            // action
//                            [item setAction:@selector(menuItemAction:)];// 20180726 bw
                        }
                        [menu insertItem:item atIndex:inItemIndex];
                    }
                    
                }
                
            } else {
                
                if ( inItemIndex >= 0 ) return;    // we're creating a new menu here, so item index must be negative
                
                if ( !title ) title = @"";
                
                NSMenu *menu = [[NSMenu alloc] initWithTitle:title];
                
                if ( otherMenuFlag ) {
                    [menu setTag:inMenuIndex];
                    CocoaOtherMenuAdd( menu );
                } else {
                    
                    CocoaMenuPadItemsToIndex( mainMenu, inMenuIndex );// fill unused items
                    
                    //NSMenuItem *menuItem = [mainMenu addItemWithTitle:title action:nil keyEquivalent:@""];
                    
                    NSMenuItem *menuItem = [mainMenu insertItemWithTitle:title action:nil keyEquivalent:@"" atIndex:inMenuIndex];
                    
                    [mainMenu setSubmenu:menu forItem:menuItem];
                }
                
                [menu setAutoenablesItems:NO];
            }
            
        }
    }
}

#pragma mark - filemenu
///////////////////////////////////////////////////
//               cocoa filemenu                  //
///////////////////////////////////////////////////
void CocoaFileMenu( NSInteger inMenuIndex )
{
    if ( !IsCocoaApplication() ) {
        CocoaShowAlert( NSCriticalAlertStyle, @"Runtime error.", @"FileMenu can only be used with Cocoa runtime." );
        return;
    }
    
    // make sure we have at least a minimal menu
    if ( ![NSApp mainMenu] ) BuildMinimalMenu();
    
    NSMenu *mainMenu = [NSApp mainMenu];
    if ( mainMenu ) {
        
        NSInteger itemCount = [mainMenu numberOfItems];
        if ( inMenuIndex < itemCount ) {
            
            
        } else {
            // fill unused items
            CocoaMenuPadItemsToIndex( mainMenu, inMenuIndex );
            
            NSMenuItem *menuItem = [mainMenu addItemWithTitle:@"File" action:nil keyEquivalent:@""];
            NSMenu *menu = [[NSMenu alloc] initWithTitle:@"File"];
            [mainMenu setSubmenu:menu forItem:menuItem];
            [menu setAutoenablesItems:NO];
            
            NSMenuItem *item;
            
            item = [menu addItemWithTitle:@"New" action:@selector(menuItemAction:) keyEquivalent:@"n"];
            [item setTarget:CocoaMenuItemTargetGet()];
            
            item = [menu addItemWithTitle:@"Open…" action:@selector(menuItemAction:) keyEquivalent:@"o"];
            [item setTarget:CocoaMenuItemTargetGet()];
            
            [menu addItem:[NSMenuItem separatorItem]];
            
            [menu addItemWithTitle:@"Close" action:@selector(performClose:) keyEquivalent:@"w"];
            
            item = [menu addItemWithTitle:@"Close All" action:@selector(performClose:) keyEquivalent:@"W"];
            [item setAlternate:YES];
            
            item = [menu addItemWithTitle:@"Save" action:@selector(performClose:) keyEquivalent:@"s"];
            [item setTarget:CocoaMenuItemTargetGet()];
            
            item = [menu addItemWithTitle:@"Save As…" action:@selector(performClose:) keyEquivalent:@"S"];
            [item setTarget:CocoaMenuItemTargetGet()];
        }
    }
}


#pragma mark - editmenu
///////////////////////////////////////////////////
//                  cocoa editmenu               //
///////////////////////////////////////////////////
void CocoaEditMenu( NSInteger inMenuIndex )
{
    if ( !IsCocoaApplication() ) {
        CocoaShowAlert( NSCriticalAlertStyle, @"Runtime error.", @"EditMenu can only be used with Cocoa runtime." );
        return;
    }
    
    // make sure we have at least a minimal menu
    if ( ![NSApp mainMenu] ) BuildMinimalMenu();
    
    // get the main menu
    NSMenu *mainMenu = [NSApp mainMenu];
    if ( mainMenu ) {
        NSInteger itemCount = [mainMenu numberOfItems];
        if ( inMenuIndex < itemCount ) {

            
        } else {
            CocoaMenuPadItemsToIndex( mainMenu, inMenuIndex );// fill unused items
            
            NSMenuItem *editMenuItem = [mainMenu addItemWithTitle:@"Edit" action:nil keyEquivalent:@""];
            
            NSMenu *mEdit = [[NSMenu alloc] initWithTitle:@"Edit"];
            [mainMenu setSubmenu:mEdit forItem:editMenuItem];
            
            [mEdit addItemWithTitle:@"Undo" action:@selector(undo:) keyEquivalent:@"z"];
            [mEdit addItemWithTitle:@"Redo" action:@selector(redo:) keyEquivalent:@"Z"];
            [mEdit addItem:[NSMenuItem separatorItem]];
            [mEdit addItemWithTitle:@"Cut" action:@selector(cut:) keyEquivalent:@"x"];
            [mEdit addItemWithTitle:@"Copy" action:@selector(copy:) keyEquivalent:@"c"];
            [mEdit addItemWithTitle:@"Paste" action:@selector(paste:) keyEquivalent:@"v"];
            [mEdit addItemWithTitle:@"Delete" action:@selector(delete:) keyEquivalent:@""];
            [mEdit addItemWithTitle:@"Select All" action:@selector(selectAll:) keyEquivalent:@"a"];
            
        }
    }
}


#pragma mark - findmenu
///////////////////////////////////////////////////
//               cocoa findmenu                  //
///////////////////////////////////////////////////
void CocoaFindMenu( NSInteger inMenuIndex )
{
    if ( !IsCocoaApplication() ) {
        CocoaShowAlert( NSCriticalAlertStyle, @"Runtime error.", @"FindMenu can only be used with Cocoa runtime." );
        return;
    }
    
    // make sure we have at least a minimal menu
    if ( ![NSApp mainMenu] ) BuildMinimalMenu();
    
    NSMenu *mainMenu = [NSApp mainMenu];
    
    BOOL otherMenuFlag = ( inMenuIndex > 100 ); // non-menubar menu or hierarchical menu
    
    NSInteger mainMenuItemCount = [mainMenu numberOfItems];
    NSInteger itemCount = 0;
    NSMenu *menu = nil;
    NSMenuItem *menuItem = nil;
    
    if ( otherMenuFlag ) {           // other menu
        menu = CocoaOtherMenuAtIndex( inMenuIndex );
    } else {                         // menu bar menu
        itemCount = mainMenuItemCount;
        if ( inMenuIndex < itemCount ) {
            menuItem = [mainMenu itemAtIndex:inMenuIndex];
            menu = [menuItem submenu];
        }
    }
    
    if ( !menu ) {
        NSMenu *menu = [[NSMenu alloc] initWithTitle:@"Find"];
        if ( otherMenuFlag ) {
            [menu setTag:inMenuIndex];
            CocoaOtherMenuAdd( menu );
        } else {
            NSMenu *mainMenu = [NSApp mainMenu];
            CocoaMenuPadItemsToIndex( mainMenu, inMenuIndex );// fill unused items
            NSMenuItem *menuItem = [mainMenu insertItemWithTitle:@"Find" action:nil keyEquivalent:@"" atIndex:inMenuIndex];
            [mainMenu setSubmenu:menu forItem:menuItem];
        }
        NSMenuItem *item;
        
        item = [menu addItemWithTitle:@"Find…" action:@selector(performFindPanelAction:) keyEquivalent:@"f"];
        [item setTag:NSTextFinderActionShowFindInterface];
        item = [menu addItemWithTitle:@"Find and Replace…" action:@selector(performFindPanelAction:) keyEquivalent:@"f"];
        [item setKeyEquivalentModifierMask:NSCommandKeyMask + NSAlternateKeyMask];
        [item setTag:NSTextFinderActionShowReplaceInterface];
        item = [menu addItemWithTitle:@"Find Next" action:@selector(performFindPanelAction:) keyEquivalent:@"g"];
        [item setTag:NSTextFinderActionNextMatch];
        item = [menu addItemWithTitle:@"Find Previous" action:@selector(performFindPanelAction:) keyEquivalent:@"G"];
        [item setTag:NSTextFinderActionPreviousMatch];
    }
}


#pragma mark - fontmenu
///////////////////////////////////////////////////
//               cocoa fontmenu                  //
///////////////////////////////////////////////////
void CocoaFontMenu( NSInteger inMenuIndex )
{
    if ( !IsCocoaApplication() ) {
        CocoaShowAlert( NSCriticalAlertStyle, @"Runtime error.", @"FontMenu can only be used with Cocoa runtime." );
        return;
    }
    
    // make sure we have at least a minimal menu
    if ( ![NSApp mainMenu] ) BuildMinimalMenu();
    
    NSMenu *mainMenu = [NSApp mainMenu];
    
    BOOL otherMenuFlag = ( inMenuIndex > 100 ); // non-menubar menu or hierarchical menu
    
    NSInteger mainMenuItemCount = [mainMenu numberOfItems];
    NSInteger itemCount = 0;
    NSMenu *menu = nil;
    NSMenuItem *menuItem = nil;
    
    if ( otherMenuFlag ) {           // other menu
        menu = CocoaOtherMenuAtIndex( inMenuIndex );
    } else {                         // menu bar menu
        itemCount = mainMenuItemCount;
        if ( inMenuIndex < itemCount ) {
            menuItem = [mainMenu itemAtIndex:inMenuIndex];
            menu = [menuItem submenu];
        }
    }
    
    if ( !menu ) {
        NSMenu *menu = [[NSFontManager sharedFontManager] fontMenu:YES];
        
        if ( otherMenuFlag ) {
            [menu setTag:inMenuIndex];
            CocoaOtherMenuAdd( menu );
        } else {
            NSMenu *mainMenu = [NSApp mainMenu];
            CocoaMenuPadItemsToIndex( mainMenu, inMenuIndex );// fill unused items
            NSMenuItem *menuItem = [mainMenu insertItemWithTitle:@"Font" action:nil keyEquivalent:@"" atIndex:inMenuIndex];
            [mainMenu setSubmenu:menu forItem:menuItem];
        }
    }
}


#pragma mark - formatmenu
///////////////////////////////////////////////////
//               cocoa formatmenu                //
///////////////////////////////////////////////////
void CocoaFormatMenu( NSInteger inMenuIndex )
{
    if ( !IsCocoaApplication() ) {
        CocoaShowAlert( NSCriticalAlertStyle, @"Runtime error.", @"FormatMenu can only be used with Cocoa runtime." );
        return;
    }
    
    // make sure we have at least a minimal menu
    if ( ![NSApp mainMenu] ) BuildMinimalMenu();
    
    NSMenu *mainMenu = [NSApp mainMenu];
    
    BOOL otherMenuFlag = ( inMenuIndex > 100 ); // non-menubar menu or hierarchical menu
    
    NSInteger mainMenuItemCount = [mainMenu numberOfItems];
    NSInteger itemCount = 0;
    NSMenu *formatMenu = nil;
    NSMenuItem *formatMenuItem = nil;
    
    if ( otherMenuFlag ) {           // other menu
        formatMenu = CocoaOtherMenuAtIndex( inMenuIndex );
    } else {                         // menu bar menu
        itemCount = mainMenuItemCount;
        if ( inMenuIndex < itemCount ) {
            formatMenuItem = [mainMenu itemAtIndex:inMenuIndex];
            formatMenu = [formatMenuItem submenu];
        }
    }
    
    if ( !formatMenu ) {
        NSMenu *formatMenu = [[NSMenu alloc] initWithTitle:@"Format"];
        if ( otherMenuFlag ) {
            [formatMenu setTag:inMenuIndex];
            CocoaOtherMenuAdd( formatMenu );
        } else {
            NSMenu *mainMenu = [NSApp mainMenu];
            CocoaMenuPadItemsToIndex( mainMenu, inMenuIndex );// fill unused items
            NSMenuItem *formatMenuItem = [mainMenu insertItemWithTitle:@"Format" action:nil keyEquivalent:@"" atIndex:inMenuIndex];
            [mainMenu setSubmenu:formatMenu forItem:formatMenuItem];
        }
        
        // Font menu
        NSMenuItem *fontMenuItem = [formatMenu addItemWithTitle:@"Font" action:nil keyEquivalent:@""];
        NSMenu *fontMenu = [[NSFontManager sharedFontManager] fontMenu:YES];
        [formatMenu setSubmenu:fontMenu forItem:fontMenuItem];
        
        
        // Text menu
        NSMenuItem *item;
        
        NSMenuItem *textMenuItem = [formatMenu addItemWithTitle:@"Text" action:nil keyEquivalent:@""];
        NSMenu *textMenu = [[NSMenu alloc] initWithTitle:@"Text"];
        [formatMenu setSubmenu:textMenu forItem:textMenuItem];
        
        // align
        [textMenu addItemWithTitle:@"Align Left" action:@selector(alignLeft:) keyEquivalent:@"{"];
        [textMenu addItemWithTitle:@"Align Center" action:@selector(alignCenter:) keyEquivalent:@"|"];
        [textMenu addItemWithTitle:@"Justify" action:@selector(alignJustified:) keyEquivalent:@""];
        [textMenu addItemWithTitle:@"Align Right" action:@selector(alignRight:) keyEquivalent:@"}"];
        
        [textMenu addItem:[NSMenuItem separatorItem]];
        
        // writing direction
        NSMenuItem *writingDirectionMenuItem = [textMenu addItemWithTitle:@"Writing Direction" action:nil keyEquivalent:@""];
        NSMenu *writingDirectionMenu = [[NSMenu alloc] initWithTitle:@"Writing Direction"];
        [textMenu setSubmenu:writingDirectionMenu forItem:writingDirectionMenuItem];
        
        [writingDirectionMenu addItemWithTitle:@"Paragraph" action:nil keyEquivalent:@"{"];
        item = [writingDirectionMenu addItemWithTitle:@"Default" action:@selector(makeBaseWritingDirectionNatural:) keyEquivalent:@""];
        [item setIndentationLevel:2];
        
        item = [writingDirectionMenu addItemWithTitle:@"Left to Right" action:@selector(makeBaseWritingDirectionLeftToRight:) keyEquivalent:@""];
        [item setIndentationLevel:2];
        
        item = [writingDirectionMenu addItemWithTitle:@"Right to Left" action:@selector(makeBaseWritingDirectionRightToLeft:) keyEquivalent:@""];
        [item setIndentationLevel:2];
        
        [writingDirectionMenu addItem:[NSMenuItem separatorItem]];
        
        [writingDirectionMenu addItemWithTitle:@"Selection" action:nil keyEquivalent:@"{"];
        item = [writingDirectionMenu addItemWithTitle:@"Default" action:@selector(makeTextWritingDirectionNatural:) keyEquivalent:@""];
        [item setIndentationLevel:2];
        
        item = [writingDirectionMenu addItemWithTitle:@"Left to Right" action:@selector(makeTextWritingDirectionLeftToRight:) keyEquivalent:@""];
        [item setIndentationLevel:2];
        
        item = [writingDirectionMenu addItemWithTitle:@"Right to Left" action:@selector(makeTextWritingDirectionRightToLeft:) keyEquivalent:@""];
        [item setIndentationLevel:2];
        
        [textMenu addItem:[NSMenuItem separatorItem]];
        
        // ruler
        [textMenu addItemWithTitle:@"Show Ruler" action:@selector(toggleRuler:) keyEquivalent:@""];
        item = [textMenu addItemWithTitle:@"Copy Ruler" action:@selector(copyRuler:) keyEquivalent:@"c"];
        [item setKeyEquivalentModifierMask:NSCommandKeyMask + NSControlKeyMask];
        item = [textMenu addItemWithTitle:@"Paste Ruler" action:@selector(pasteRuler:) keyEquivalent:@"v"];
        [item setKeyEquivalentModifierMask:NSCommandKeyMask + NSControlKeyMask];
    }
}


#pragma mark - textmenu
///////////////////////////////////////////////////
//               cocoa textmenu                  //
///////////////////////////////////////////////////
void CocoaTextMenu( NSInteger inMenuIndex )
{
    if ( !IsCocoaApplication() ) {
        CocoaShowAlert( NSCriticalAlertStyle, @"Runtime error.", @"FormatMenu can only be used with Cocoa runtime." );
        return;
    }
    
    // make sure we have at least a minimal menu
    if ( ![NSApp mainMenu] ) BuildMinimalMenu();
    
    NSMenu *mainMenu = [NSApp mainMenu];
    
    BOOL otherMenuFlag = ( inMenuIndex > 100 ); // non-menubar menu or hierarchical menu
    
    NSInteger mainMenuItemCount = [mainMenu numberOfItems];
    NSInteger itemCount = 0;
    NSMenu *menu = nil;
    NSMenuItem *menuItem = nil;
    
    if ( otherMenuFlag ) {           // other menu
        menu = CocoaOtherMenuAtIndex( inMenuIndex );
    } else {                         // menu bar menu
        itemCount = mainMenuItemCount;
        if ( inMenuIndex < itemCount ) {
            menuItem = [mainMenu itemAtIndex:inMenuIndex];
            menu = [menuItem submenu];
        }
    }
    
    if ( !menu ) {
        NSMenu *menu = [[NSMenu alloc] initWithTitle:@"Text"];
        if ( otherMenuFlag ) {
            [menu setTag:inMenuIndex];
            CocoaOtherMenuAdd( menu );
        } else {
            NSMenu *mainMenu = [NSApp mainMenu];
            CocoaMenuPadItemsToIndex( mainMenu, inMenuIndex );// fill unused items
            NSMenuItem *menuItem = [mainMenu insertItemWithTitle:@"Text" action:nil keyEquivalent:@"" atIndex:inMenuIndex];
            [mainMenu setSubmenu:menu forItem:menuItem];
        }
        
        NSMenuItem *item;
        
        // align
        [menu addItemWithTitle:@"Align Left" action:@selector(alignLeft:) keyEquivalent:@"{"];
        [menu addItemWithTitle:@"Align Center" action:@selector(alignCenter:) keyEquivalent:@"|"];
        [menu addItemWithTitle:@"Justify" action:@selector(alignJustified:) keyEquivalent:@""];
        [menu addItemWithTitle:@"Align Right" action:@selector(alignRight:) keyEquivalent:@"}"];
        
        [menu addItem:[NSMenuItem separatorItem]];
        
        // writing direction
        NSMenuItem *writingDirectionMenuItem = [menu addItemWithTitle:@"Writing Direction" action:nil keyEquivalent:@""];
        NSMenu *writingDirectionMenu = [[NSMenu alloc] initWithTitle:@"Writing Direction"];
        [menu setSubmenu:writingDirectionMenu forItem:writingDirectionMenuItem];
        
        [writingDirectionMenu addItemWithTitle:@"Paragraph" action:nil keyEquivalent:@"{"];
        item = [writingDirectionMenu addItemWithTitle:@"Default" action:@selector(makeBaseWritingDirectionNatural:) keyEquivalent:@""];
        [item setIndentationLevel:2];
        
        item = [writingDirectionMenu addItemWithTitle:@"Left to Right" action:@selector(makeBaseWritingDirectionLeftToRight:) keyEquivalent:@""];
        [item setIndentationLevel:2];
        
        item = [writingDirectionMenu addItemWithTitle:@"Right to Left" action:@selector(makeBaseWritingDirectionRightToLeft:) keyEquivalent:@""];
        [item setIndentationLevel:2];
        
        [writingDirectionMenu addItem:[NSMenuItem separatorItem]];
        
        [writingDirectionMenu addItemWithTitle:@"Selection" action:nil keyEquivalent:@"{"];
        item = [writingDirectionMenu addItemWithTitle:@"Default" action:@selector(makeTextWritingDirectionNatural:) keyEquivalent:@""];
        [item setIndentationLevel:2];
        
        item = [writingDirectionMenu addItemWithTitle:@"Left to Right" action:@selector(makeTextWritingDirectionLeftToRight:) keyEquivalent:@""];
        [item setIndentationLevel:2];
        
        item = [writingDirectionMenu addItemWithTitle:@"Right to Left" action:@selector(makeTextWritingDirectionRightToLeft:) keyEquivalent:@""];
        [item setIndentationLevel:2];
        
        [menu addItem:[NSMenuItem separatorItem]];
        
        // ruler
        [menu addItemWithTitle:@"Show Ruler" action:@selector(toggleRuler:) keyEquivalent:@""];
        item = [menu addItemWithTitle:@"Copy Ruler" action:@selector(copyRuler:) keyEquivalent:@"c"];
        [item setKeyEquivalentModifierMask:NSCommandKeyMask + NSControlKeyMask];
        item = [menu addItemWithTitle:@"Paste Ruler" action:@selector(pasteRuler:) keyEquivalent:@"v"];
        [item setKeyEquivalentModifierMask:NSCommandKeyMask + NSControlKeyMask];
    }
}


#pragma mark - viewmenu
///////////////////////////////////////////////////
//               cocoa viewmenu                  //
///////////////////////////////////////////////////
void CocoaViewMenu( NSInteger inMenuIndex )
{
    if ( !IsCocoaApplication() ) {
        CocoaShowAlert( NSCriticalAlertStyle, @"Runtime error.", @"ViewMenu can only be used with Cocoa runtime." );
        return;
    }
    
    // make sure we have at least a minimal menu
    if ( ![NSApp mainMenu] ) BuildMinimalMenu();
    
    NSMenu *mainMenu = [NSApp mainMenu];
    
    BOOL otherMenuFlag = ( inMenuIndex > 100 ); // non-menubar menu or hierarchical menu
    
    NSInteger mainMenuItemCount = [mainMenu numberOfItems];
    NSInteger itemCount = 0;
    NSMenu *menu = nil;
    NSMenuItem *menuItem = nil;
    
    if ( otherMenuFlag ) {           // other menu
        menu = CocoaOtherMenuAtIndex( inMenuIndex );
    } else {                         // menu bar menu
        itemCount = mainMenuItemCount;
        if ( inMenuIndex < itemCount ) {
            menuItem = [mainMenu itemAtIndex:inMenuIndex];
            menu = [menuItem submenu];
        }
    }
    
    if ( !menu ) {
        NSMenu *menu = [[NSMenu alloc] initWithTitle:@"View"];
        if ( otherMenuFlag ) {
            [menu setTag:inMenuIndex];
            CocoaOtherMenuAdd( menu );
        } else {
            NSMenu *mainMenu = [NSApp mainMenu];
            CocoaMenuPadItemsToIndex( mainMenu, inMenuIndex );// fill unused items
            NSMenuItem *menuItem = [mainMenu insertItemWithTitle:@"View" action:nil keyEquivalent:@"" atIndex:inMenuIndex];
            [mainMenu setSubmenu:menu forItem:menuItem];
        }
        
        NSMenuItem *item;
        
        [menu addItem:[NSMenuItem separatorItem]];
        
        item = [menu addItemWithTitle:@"Show Toolbar" action:@selector(toggleToolbarShown:) keyEquivalent:@"t"];
        [item setKeyEquivalentModifierMask:NSCommandKeyMask + NSAlternateKeyMask];
        
        [menu addItemWithTitle:@"Customize Toolbar…" action:@selector(runToolbarCustomizationPalette:) keyEquivalent:@""];
    }
}

#pragma mark - windowmenu
///////////////////////////////////////////////////
//               cocoa windowmenu               //
///////////////////////////////////////////////////
void CocoaWindowMenu( NSInteger inMenuIndex )
{
    if ( !IsCocoaApplication() ) {
        CocoaShowAlert( NSCriticalAlertStyle, @"Runtime error.", @"WindowMenu can only be used with Cocoa runtime." );
        return;
    }
    
    // make sure we have at least a minimal menu
    if ( ![NSApp mainMenu] ) BuildMinimalMenu();
    
    NSMenu *mainMenu = [NSApp mainMenu];
    if ( mainMenu ) {
        NSInteger itemCount = [mainMenu numberOfItems];
        if ( inMenuIndex < itemCount ) {
            
            
        } else {
            // fill unused items
            CocoaMenuPadItemsToIndex( mainMenu, inMenuIndex );
            
            NSMenuItem *windowMenuItem = [mainMenu addItemWithTitle:@"Window" action:nil keyEquivalent:@""];
            NSMenu *windowMenu = [[NSMenu alloc] initWithTitle:@"Window"];
            [mainMenu setSubmenu:windowMenu forItem:windowMenuItem];
            [windowMenu addItemWithTitle:@"Minimize" action:@selector(performMiniaturize:) keyEquivalent:@"m"];
            [NSApp setWindowsMenu:windowMenu];
        }
    }
}

#pragma mark - helpmenu
///////////////////////////////////////////////////
//               cocoa helpmenu                  //
///////////////////////////////////////////////////
void CocoaHelpMenu( NSInteger inMenuIndex )
{
    if ( !IsCocoaApplication() ) {
        CocoaShowAlert( NSCriticalAlertStyle, @"Runtime error.", @"HelpMenu can only be used with Cocoa runtime." );
        return;
    }
    
    // make sure we have at least a minimal menu
    if ( ![NSApp mainMenu] ) BuildMinimalMenu();
    
    // get the main menu
    NSMenu *mainMenu = [NSApp mainMenu];
    if ( mainMenu ) {
        NSInteger itemCount = [mainMenu numberOfItems];
        if ( inMenuIndex < itemCount ) {
            
            
        } else {
            CocoaMenuPadItemsToIndex( mainMenu, inMenuIndex );// fill unused items
            
            NSMenuItem *helpMenuItem = [mainMenu addItemWithTitle:@"Help" action:nil keyEquivalent:@""];
            NSMenu *mHelp = [[NSMenu alloc] initWithTitle:@"Help"];
            [mainMenu setSubmenu:mHelp forItem:helpMenuItem];
            
            NSString *name = [[NSBundle mainBundle] objectForInfoDictionaryKey:(NSString *)kCFBundleNameKey];
            [mHelp addItemWithTitle:[NSString stringWithFormat:@"%@ Help",name] action:@selector(showHelp:) keyEquivalent:@"?"];
        }
    }
}


#pragma mark - nibmenu
///////////////////////////////////////////////////
//                  cocoa nib menu               //
///////////////////////////////////////////////////
void CocoaNibMenuSetup( NSMenu *menu )
{
    NSMenu *supermenu = [menu supermenu];
    if ( supermenu ) {
        NSInteger index = [supermenu indexOfItemWithSubmenu:menu];
        NSMenuItem *submenuItem = [supermenu itemAtIndex:index];
        NSInteger tag = [submenuItem tag];
        if ( tag > 100 ) {
            [menu setTag:tag];
            CocoaOtherMenuAdd( menu );
        }
    }
    
    NSArray *items = [menu itemArray];
    for ( NSMenuItem *item in items ) {
        if ( ![item action] ) {
            [item setTarget:CocoaMenuItemTargetGet()];
            [item setAction:@selector(menuItemAction:)];
        }
        NSMenu *submenu = [item submenu];
        if ( submenu ) CocoaNibMenuSetup( submenu );
    }
}

void CocoaNibMenu( CFStringRef inNibName )
{
    if ( !IsCocoaApplication() ) {
        CocoaShowAlert( NSWarningAlertStyle, @"Runtime error.", @"NibMenu can only be used with Cocoa runtime." );
        return;
    }
    
    NSString *nibName = (NSString *)inNibName;
    
    CocoaNibObject *nibObj = CocoaNibObjectWithName( nibName );
    if ( !nibObj ) {
        NSNib *nib = [[NSNib alloc] initWithNibNamed:nibName bundle:nil];
        if ( nib ) {
            NSArray *objects = nil;
            
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
            if ( [nib respondsToSelector:@selector(instantiateWithOwner:topLevelObjects:)]) {
                [nib instantiateWithOwner:nil topLevelObjects:&objects];
            } else {
                [nib instantiateNibWithOwner:nil topLevelObjects:&objects];
            }
#else
            [nib instantiateNibWithOwner:nil topLevelObjects:&objects];
#endif
            
            CocoaNibObjectAdd( nibName, objects );
            CocoaNibMenuSetup( [NSApp mainMenu] );
        }
    }
}



