/*
 MenuItem.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

NSMenuItem *CocoaMenuItemAtIndex( NSMenu *menu, NSInteger index );

NSMenuItem *MenuItemAtIndex( NSInteger menuIndex, NSInteger itemIndex );
BOOL MenuItemExists( NSInteger menuIndex, NSInteger itemIndex );

// Enable
Boolean MenuItemIsEnabled( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetEnabled( NSInteger menuIndex, NSInteger itemIndex, Boolean flag );

// Hidden
Boolean MenuItemIsHidden( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetHidden( NSInteger menuIndex, NSInteger itemIndex, Boolean flag );
Boolean MenuItemIsHiddenOrHasHiddenAncestor( NSInteger menuIndex, NSInteger itemIndex );

// Target and action
void MenuItemSetAction( NSInteger menuIndex, NSInteger itemIndex, CFStringRef actionName );

// Title
CFStringRef MenuItemTitle( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetTitle( NSInteger menuIndex, NSInteger itemIndex, CFStringRef title );
CFAttributedStringRef MenuItemAttributedTitle( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetAttributedTitle( NSInteger menuIndex, NSInteger itemIndex, CFAttributedStringRef title );

// Tag
NSInteger MenuItemTag( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetTag( NSInteger menuIndex, NSInteger itemIndex, NSInteger tag );

// State
NSCellStateValue MenuItemState( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetState( NSInteger menuIndex, NSInteger itemIndex, NSCellStateValue state );

// Image
ImageRef MenuItemImage( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetImage( NSInteger menuIndex, NSInteger itemIndex, ImageRef image );
ImageRef MenuItemOnStateImage( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetOnStateImage( NSInteger menuIndex, NSInteger itemIndex, ImageRef image );
ImageRef MenuItemOffStateImage( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetOffStateImage( NSInteger menuIndex, NSInteger itemIndex, ImageRef image );
ImageRef MenuItemMixedStateImage( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetMixedStateImage( NSInteger menuIndex, NSInteger itemIndex, ImageRef image );

// Submenu
Boolean MenuItemHasSubmenu( NSInteger menuIndex, NSInteger itemIndex );

// Separator
Boolean MenuItemIsSeparator( NSInteger menuIndex, NSInteger itemIndex );

// KeyEquivalent
CFStringRef MenuItemKeyEquivalent( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetKeyEquivalent( NSInteger menuIndex, NSInteger itemIndex, CFStringRef string );
NSUInteger MenuItemKeyEquivalentModifierMask( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetKeyEquivalentModifierMask( NSInteger menuIndex, NSInteger itemIndex, NSUInteger mask );

// User key equivalents
Boolean MenuItemUsesUserKeyEquivalents( void );
CFStringRef MenuItemUserKeyEquivalent( NSInteger menuIndex, NSInteger itemIndex );

// Alternate
Boolean MenuItemIsAlternate( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetAlternate( NSInteger menuIndex, NSInteger itemIndex, Boolean flag );

// Indentation level
NSInteger MenuItemIndentationLevel( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetIndentationLevel( NSInteger menuIndex, NSInteger itemIndex, NSInteger level );

// ToolTip
CFStringRef MenuItemToolTip( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetToolTip( NSInteger menuIndex, NSInteger itemIndex, CFStringRef string );

// Highlighted
Boolean MenuItemIsHighlighted( NSInteger menuIndex, NSInteger itemIndex );

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean MenuItemAllowsKeyEquivalentWhenHidden( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetAllowsKeyEquivalentWhenHidden( NSInteger menuIndex, NSInteger itemIndex, Boolean flag );
#endif

// Custom
void MenuItemSetOnMenuAction( NSInteger menuIndex, NSInteger itemIndex );
void MenuItemSetActionCallback( NSInteger menuIndex, NSInteger itemIndex, void *callback, void *userData );

CFTypeRef MenuItemProperty( NSInteger menuIndex, NSInteger itemIndex, CFStringRef key );
void MenuItemSetProperty( NSInteger menuIndex, NSInteger itemIndex, CFStringRef key, CFTypeRef value );
void MenuItemRemoveProperty( NSInteger menuIndex, NSInteger itemIndex, CFStringRef key );
void MenuItemRemoveAllProperties( NSInteger menuIndex, NSInteger itemIndex );

