/*
 MenuItem.m

 Bernie Wylde
 */

#import "MenuItem.h"

NSMenuItem *CocoaMenuItemAtIndex( NSMenu *menu, NSInteger index )
{
    NSInteger count = [menu numberOfItems];
    if ( index < count ) return [menu itemAtIndex:index];
    return nil;
}


NSMenuItem *MenuItemAtIndex( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) return [menu itemAtIndex:itemIndex];
    return nil;
}

BOOL MenuItemExists( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) return ( [menu itemAtIndex:itemIndex] != nil );
    return NO;
}

// Enable
Boolean MenuItemIsEnabled( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item isEnabled];
    }
    return NO;
}

void MenuItemSetEnabled( NSInteger menuIndex, NSInteger itemIndex, Boolean flag )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setEnabled:flag];
    }
}

// Hidden
Boolean MenuItemIsHidden( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item isHidden];
    }
    return NO;
}

void MenuItemSetHidden( NSInteger menuIndex, NSInteger itemIndex, Boolean flag )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setHidden:flag];
    }
}

Boolean MenuItemIsHiddenOrHasHiddenAncestor( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item isHiddenOrHasHiddenAncestor];
    }
    return NO;
}

// Target and action
void MenuItemSetAction( NSInteger menuIndex, NSInteger itemIndex, CFStringRef actionName )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) {
            if ( [(NSString *)actionName isEqualToString:@"menuItemAction:"]) {
                [item setTarget:CocoaMenuItemTargetGet()];
            } else {
                [item setTarget:nil];
            }
            [item setAction:NSSelectorFromString((NSString *)actionName)];
        }
    }
}

// Title
CFStringRef MenuItemTitle( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return (CFStringRef)[item title];
    }
    return NULL;
}

void MenuItemSetTitle( NSInteger menuIndex, NSInteger itemIndex, CFStringRef title )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setTitle:(NSString *)title];
    }
}

CFAttributedStringRef MenuItemAttributedTitle( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return (CFAttributedStringRef)[item attributedTitle];
    }
    return NULL;
}

void MenuItemSetAttributedTitle( NSInteger menuIndex, NSInteger itemIndex, CFAttributedStringRef title )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setAttributedTitle:(NSAttributedString *)title];
    }
}

// Tag
NSInteger MenuItemTag( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item tag];
    }
    return 0;
}

void MenuItemSetTag( NSInteger menuIndex, NSInteger itemIndex, NSInteger tag )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setTag:tag];
    }
}

// State
NSCellStateValue MenuItemState( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item state];
    }
    return NSOffState;
}

void MenuItemSetState( NSInteger menuIndex, NSInteger itemIndex, NSCellStateValue state )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setState:state];
    }
}

// Image
ImageRef MenuItemImage( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item image];
    }
    return nil;
}

void MenuItemSetImage( NSInteger menuIndex, NSInteger itemIndex, ImageRef image )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setImage:image];
    }
}

ImageRef MenuItemOnStateImage( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item onStateImage];
    }
    return nil;
}

void MenuItemSetOnStateImage( NSInteger menuIndex, NSInteger itemIndex, ImageRef image )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setOnStateImage:image];
    }
}

ImageRef MenuItemOffStateImage( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item offStateImage];
    }
    return nil;
}

void MenuItemSetOffStateImage( NSInteger menuIndex, NSInteger itemIndex, ImageRef image )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setOffStateImage:image];
    }
}

ImageRef MenuItemMixedStateImage( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item mixedStateImage];
    }
    return nil;
}

void MenuItemSetMixedStateImage( NSInteger menuIndex, NSInteger itemIndex, ImageRef image )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setMixedStateImage:image];
    }
}

// Submenu
Boolean MenuItemHasSubmenu( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item hasSubmenu];
    }
    return NO;
}

// Separator
Boolean MenuItemIsSeparator( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item isSeparatorItem];
    }
    return NO;
}

// KeyEquivalent
CFStringRef MenuItemKeyEquivalent( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return (CFStringRef)[item keyEquivalent];
    }
    return nil;
}

void MenuItemSetKeyEquivalent( NSInteger menuIndex, NSInteger itemIndex, CFStringRef string )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setKeyEquivalent:(NSString *)string];
    }
}

NSUInteger MenuItemKeyEquivalentModifierMask( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item keyEquivalentModifierMask];
    }
    return 0;
}

void MenuItemSetKeyEquivalentModifierMask( NSInteger menuIndex, NSInteger itemIndex, NSUInteger mask )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setKeyEquivalentModifierMask:mask];
    }
}

// User key equivalents
Boolean MenuItemUsesUserKeyEquivalents( void )
{ return [NSMenuItem usesUserKeyEquivalents]; }

CFStringRef MenuItemUserKeyEquivalent( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return (CFStringRef)[item userKeyEquivalent];
    }
    return nil;
}

// Alternate
Boolean MenuItemIsAlternate( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item isAlternate];
    }
    return NO;
}

void MenuItemSetAlternate( NSInteger menuIndex, NSInteger itemIndex, Boolean flag )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setAlternate:flag];
    }
}

// Indentation level
NSInteger MenuItemIndentationLevel( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item indentationLevel];
    }
    return 0;
}

void MenuItemSetIndentationLevel( NSInteger menuIndex, NSInteger itemIndex, NSInteger level )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setIndentationLevel:level];
    }
}

// ToolTip
CFStringRef MenuItemToolTip( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return (CFStringRef)[item toolTip];
    }
    return nil;
}

void MenuItemSetToolTip( NSInteger menuIndex, NSInteger itemIndex, CFStringRef string )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setToolTip:(NSString *)string];
    }
}

// Highlighted
Boolean MenuItemIsHighlighted( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item isHighlighted];
    }
    return NO;
}

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean MenuItemAllowsKeyEquivalentWhenHidden( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) return [item allowsKeyEquivalentWhenHidden];
    }
    return NO;
}

void MenuItemSetAllowsKeyEquivalentWhenHidden( NSInteger menuIndex, NSInteger itemIndex, Boolean flag )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) [item setAllowsKeyEquivalentWhenHidden:flag];
    }
}
#endif

// Custom
void MenuItemSetOnMenuAction( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *mainMenu = [NSApp mainMenu];
    if ( !mainMenu ) BuildMinimalMenu();
    
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) {
            [item setTarget:CocoaMenuItemTargetGet()];
            [item setAction:@selector(menuItemAction:)];
        }
    }
}

void MenuItemSetActionCallback( NSInteger menuIndex, NSInteger itemIndex, void *callback, void *userData )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        if ( itemIndex >= 0 ) {
            NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
            if ( item ) {
                [item setTarget:CocoaMenuItemTargetGet()];
                [item setAction:@selector(menuItemAction:)];

                [item setCocoaMenuItemCallback:callback];
                [item setCocoaMenuItemUserData:userData];
            }
        } else {
            NSArray *items = [menu itemArray];
            for ( NSMenuItem *item in items ) {
                [item setTarget:CocoaMenuItemTargetGet()];
                [item setAction:@selector(menuItemAction:)];

                [item setCocoaMenuItemCallback:callback];
                [item setCocoaMenuItemUserData:userData];
            }
        }
    }
}


CFTypeRef MenuItemProperty( NSInteger menuIndex, NSInteger itemIndex, CFStringRef key )
{
    id property = nil;
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) {
            property = ObjectProperty( item, key );
        }
    }
    return (CFTypeRef)property;
}

void MenuItemSetProperty( NSInteger menuIndex, NSInteger itemIndex, CFStringRef key, CFTypeRef value )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) {
            ObjectSetProperty( item, key, value );
        }
    }
}

void MenuItemRemoveProperty( NSInteger menuIndex, NSInteger itemIndex, CFStringRef key )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) {
            ObjectRemoveProperty( item, key );
        }
    }
}

void MenuItemRemoveAllProperties( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *menu = MenuAtIndex( menuIndex );
    if ( menu ) {
        NSMenuItem *item = CocoaMenuItemAtIndex( menu, itemIndex );
        if ( item ) {
            ObjectRemoveAllProperties( item );
        }
    }
}


