/*
 Nib.m

 Bernie Wylde
 */

#import "Nib.h"

#pragma mark - cocoa nibs
static NSMutableArray *sCocoaNibObjects;

@implementation CocoaNibObject
- (id)initWithName:(NSString *)name objects:(NSArray *)tlObjects {
    if ( self = [super init] ) {
        nibName = [name retain];// 20190401 bw
        topLevelObjects = [tlObjects retain];// 20190401 Paolo
    }
    return self;
}

@synthesize nibName;
@synthesize topLevelObjects;
@end

CocoaNibObject *CocoaNibObjectWithName( NSString *name )
{
    if ( sCocoaNibObjects ) {
        for ( CocoaNibObject *nibObj in sCocoaNibObjects ) {
            if ( [[nibObj nibName] isEqualToString:name] ) return nibObj;
        }
    }
    return nil;
}

void CocoaNibObjectAdd( NSString *name, NSArray *objects )
{
    CocoaNibObject *nibObj = CocoaNibObjectWithName( name );
    if ( !nibObj ) {
        nibObj = [[CocoaNibObject alloc] initWithName:name objects:objects];
        if ( !sCocoaNibObjects ) sCocoaNibObjects = [[NSMutableArray alloc] initWithCapacity:0];
        [sCocoaNibObjects addObject:nibObj];
        [nibObj release];
    }
}

#pragma mark - nibviewsetup
void CocoaNibViewSetup( NSInteger wndTag, NSView *superview )
{
    NSArray *subviews = [superview subviews];
    for ( id subview in subviews ) {
        // wndTag
        [subview setWndTag:wndTag];
        
        // get view tag
        NSInteger tag = 0;
        if ( [subview respondsToSelector:@selector(tag)] ) tag = [subview tag];
        
        if ( tag <= 0 ) {   // try to get tag from identifier
            NSString *identifier = [subview identifier];
            if ( identifier ) {
                if ( [subview respondsToSelector:@selector(setTag:)] ) {
                    tag = [identifier integerValue];
                    [subview setTag:tag];
                }
            }
        }
        
#if __LP64__
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
        
        if ( [subview isKindOfClass:NSClassFromString(@"AVPlayerView")] ) {
            continue;
        }
        
        if ( [subview isKindOfClass:NSClassFromString(@"WKWebView")] ) {
            continue;
        }

#endif// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
        
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
        if ( [subview isKindOfClass:NSClassFromString(@"MKMapView")] ) {
            continue;
        }
#endif
        
        
#endif// #if __LP64__
        
        if ( [subview isKindOfClass:[NSButton class]] ) {
            if ( [subview action] == 0 ) {
                if ( [(NSButton *)subview target] == nil ) [(NSButton *)subview setTarget:CocoaControlTargetObj()];
                if ( [subview action] == 0 ) [subview setAction:@selector(controlAction:)];
            }
            continue;
        }
        
        if ( [subview isKindOfClass:[NSColorWell class]] ) {
            if ( [subview action] == 0 ) {
                if ( [(NSColorWell *)subview target] == nil ) [(NSColorWell *)subview setTarget:CocoaControlTargetObj()];
                if ( [subview action] == 0 ) [subview setAction:@selector(controlAction:)];
            }
            continue;
        }
        
        if ( [subview isKindOfClass:[NSComboBox class]] ) {
            if ( [subview delegate] == nil ) [subview setDelegate:CocoaComboBoxDelegateGet()];
            continue;
        }
        
        if ( [subview isKindOfClass:[NSDatePicker class]] ) {
            if ( [subview action] == 0 ) {
                [(NSDatePicker *)subview setTarget:CocoaControlTargetObj()];
                [subview setAction:@selector(controlAction:)];
            }
            continue;
        }
        
        if ( [subview isKindOfClass:[NSLevelIndicator class]] ) {
            if ( [(NSLevelIndicator *)subview target] == nil ) [(NSLevelIndicator *)subview setTarget:CocoaControlTargetObj()];
            if ( [subview action] == 0 ) [subview setAction:@selector(controlAction:)];
            continue;
        }
        
        if ( [subview isKindOfClass:[NSOutlineView class]] ) {
            if ( [subview delegate] == nil || [subview dataSource] == nil ) {
                BOOL flag = YES;
                NSArray *columns = [subview tableColumns];
                for ( NSTableColumn *col in columns ) {
                    if ( [col infoForBinding:NSValueBinding] ) {
                        [subview setDelegate:nil];
                        [subview setDataSource:nil];
                        flag = NO;
                        break;
                    }
                }
                
                if ( flag ) {
                    CocoaOutlineViewDelegate *delegate = [[CocoaOutlineViewDelegate alloc] init];
                    if ( [subview delegate] == nil ) {
                        [subview setDelegate:delegate];
                        [delegate setTable:subview];
                    }
                    if ( [subview action] == 0 ) {
                        [(NSOutlineView *)subview setTarget:delegate];
                        if ( ![subview action] ) [subview setAction:@selector(singleAction:)];
                        [subview setDoubleAction:@selector(doubleAction:)];
                    }
                    if ( [subview dataSource] == nil ) [subview setDataSource:delegate];
                }
                
            }
            continue;
        }
        
        if ( [subview isKindOfClass:NSClassFromString(@"PDFView")] ) {
            continue;
        }
        
        if ( [subview isKindOfClass:NSClassFromString(@"PDFThumbnailView")] ) {
            continue;
        }
        
        if ( [subview isKindOfClass:[NSPopUpButton class]] ) {
            if ( [subview action] == 0 ) {
                if ( [(NSPopUpButton *)subview target] == nil ) [(NSPopUpButton *)subview setTarget:CocoaControlTargetObj()];
                if ( [subview action] == 0 ) [subview setAction:@selector(controlAction:)];
            }
            continue;
        }
        
        if ( [subview isKindOfClass:[NSSearchField class]] ) {
            if ( [subview delegate] == nil ) [subview setDelegate:CocoaSearchFieldDelegateGet()];
            continue;
        }
        
        if ( [subview isKindOfClass:[NSSecureTextField class]] ) {
            if ( [subview delegate] == nil ) [subview setDelegate:CocoaTextFieldDelegateGet()];
            continue;
        }
        
        if ( [subview isKindOfClass:[NSSegmentedControl class]] ) {
            if ( [subview action] == 0 ) {
                if ( [(NSSegmentedControl *)subview target] == nil ) [(NSSegmentedControl *)subview setTarget:CocoaControlTargetObj()];
                if ( [subview action] == 0 ) [subview setAction:@selector(controlAction:)];
            }
            continue;
        }
        
        if ( [subview isKindOfClass:[NSSlider class]] ) {
            if ( [subview action] == 0 ) {
                if ( [(NSSlider *)subview target] == nil ) [(NSSlider *)subview setTarget:CocoaControlTargetObj()];
                if ( [subview action] == 0 ) [subview setAction:@selector(controlAction:)];
            }
            continue;
        }
        
        if ( [subview isKindOfClass:[NSSplitView class]] ) {
            if ( [subview delegate] == nil ) [subview setDelegate:CocoaSplitViewDelegateGet()];
            //continue;
            goto Done;
        }
        
        if ( [subview isKindOfClass:[NSStepper class]] ) {
            if ( [subview action] == 0 ) {
                if ( [(NSStepper *)subview target] == nil ) [(NSStepper *)subview setTarget:CocoaControlTargetObj()];
                if ( [subview action] == 0 ) [subview setAction:@selector(controlAction:)];
            }
            continue;
        }
        
        if ( [subview isKindOfClass:[NSTableView class]] ) {
            if ( [subview delegate] == nil || [subview dataSource] == nil ) {
                CocoaTableViewDelegate *delegate = [[CocoaTableViewDelegate alloc] init];
                if ( [subview delegate] == nil ) {
                    [subview setDelegate:delegate];
                    [delegate setTable:subview];
                }
                if ( [subview action] == 0 ) {
                    [(NSTableView *)subview setTarget:delegate];
                    if ( ![subview action] ) [subview setAction:@selector(singleAction:)];
                    [subview setDoubleAction:@selector(doubleAction:)];
                }
                if ( [subview dataSource] == nil ) [subview setDataSource:delegate];
            }
            continue;
        }
        
        if ( [subview isKindOfClass:[NSTabView class]] ) {
            if ( [subview delegate] == nil ) [subview setDelegate:CocoaTabViewDelegateGet()];
            // special handling for tab view items
            for ( NSTabViewItem *item in [subview tabViewItems] ) {
                CocoaNibViewSetup( wndTag, [item view] );
            }
            goto Done;
        }
        
        if ( [subview isKindOfClass:[NSTextField class]] ) {
            if ( [subview isEditable] ) [subview setDelegate:CocoaTextFieldDelegateGet()];
            continue;
        }
        
        if ( [subview isKindOfClass:[NSTextView class]] ) {
            if ( [subview isEditable] ) [subview setDelegate:CocoaTextViewDelegateGet()];
            continue;
        }
        
        if ( [subview isKindOfClass:[NSTokenField class]] ) {
            if ( [subview delegate] == nil ) [subview setDelegate:CocoaTokenFieldDelegateGet()];
            continue;
        }
        
        if ( [subview isKindOfClass:[NSView class]] ) {
            goto Done;  // container view - check subviews
        }
        
    Done:
        CocoaNibViewSetup( wndTag, subview );
    }
}


void CocoaNibWindowToolbarItemSetup( NSInteger wndTag, NSToolbar *toolbar )
{
    NSArray *toolbarItems = [toolbar items];
    for ( NSToolbarItem *item in toolbarItems ) {
        
        id view = [item view];
        if ( view ) {
            
            // tag
            NSInteger viewTag = 0;
            if ( [view respondsToSelector:@selector(tag)] ) viewTag = [view tag];
            
            if ( viewTag <= 0 ) {   // try to get tag from identifier
                NSString *identifier = [view identifier];
                if ( identifier ) {
                    if ( [view respondsToSelector:@selector(setTag:)] ) {
                        viewTag = [identifier integerValue];
                        [view setTag:viewTag];
                    }
                }
            }
            
            // button
            if ( [view isKindOfClass:[NSButton class]] ) {
                if ( [view action] == 0 ) {
                    //                    if ( [view target] == nil ) [view setTarget:CocoaControlTargetObj()];
                    //                    if ( [view action] == 0 ) [view setAction:@selector(controlAction:)];
                    [item setTarget:CocoaToolbarItemTargetGet()];
                    [item setAction:@selector(toolbarItemAction:)];
                }
                continue;
            }
            
            // colorwell
            if ( [view isKindOfClass:[NSColorWell class]] ) {
                if ( [view action] == 0 ) {
                    //                    if ( [view target] == nil ) [view setTarget:CocoaControlTargetObj()];
                    //                    if ( [view action] == 0 ) [view setAction:@selector(controlAction:)];
                    [item setTarget:CocoaToolbarItemTargetGet()];
                    [item setAction:@selector(toolbarItemAction:)];
                }
                continue;
            }
            
            // combobox
            if ( [view isKindOfClass:[NSComboBox class]] ) {
                if ( [view delegate] == nil ) [view setDelegate:CocoaComboBoxDelegateGet()];
                continue;
            }
            
            // datepicker
            if ( [view isKindOfClass:[NSDatePicker class]] ) {
                if ( [view action] == 0 ) {
                    //                    [view setTarget:CocoaControlTargetObj()];
                    //                    [view setAction:@selector(controlAction:)];
                    [item setTarget:CocoaToolbarItemTargetGet()];
                    [item setAction:@selector(toolbarItemAction:)];
                }
                continue;
            }
            
            // levelindicator
            if ( [view isKindOfClass:[NSLevelIndicator class]] ) {
                //                if ( [view target] == nil ) [view setTarget:CocoaControlTargetObj()];
                //                if ( [view action] == 0 ) [view setAction:@selector(controlAction:)];
                [item setTarget:CocoaToolbarItemTargetGet()];
                [item setAction:@selector(toolbarItemAction:)];
                continue;
            }
            
            if ( [view isKindOfClass:[NSPopUpButton class]] ) {
                if ( [view action] == 0 ) {
                    //                    if ( [view target] == nil ) [view setTarget:CocoaControlTargetObj()];
                    //                    if ( [view action] == 0 ) [view setAction:@selector(controlAction:)];
                    [item setTarget:CocoaToolbarItemTargetGet()];
                    [item setAction:@selector(toolbarItemAction:)];
                }
                continue;
            }
            
            // searchfield
            if ( [view isKindOfClass:[NSSearchField class]] ) {
                if ( [view delegate] == nil ) [view setDelegate:CocoaSearchFieldDelegateGet()];
                continue;
            }
            
            // securetextfield
            if ( [view isKindOfClass:[NSSecureTextField class]] ) {
                if ( [view delegate] == nil ) [view setDelegate:CocoaTextFieldDelegateGet()];
                continue;
            }
            
            // segmentedcontrol
            if ( [view isKindOfClass:[NSSegmentedControl class]] ) {
                if ( [view action] == 0 ) {
                    //                    if ( [view target] == nil ) [view setTarget:CocoaControlTargetObj()];
                    //                    if ( [view action] == 0 ) [view setAction:@selector(controlAction:)];
                    [item setTarget:CocoaToolbarItemTargetGet()];
                    [item setAction:@selector(toolbarItemAction:)];
                }
                continue;
            }
            
            // slider
            if ( [view isKindOfClass:[NSSlider class]] ) {
                if ( [view action] == 0 ) {
                    //                    if ( [view target] == nil ) [view setTarget:CocoaControlTargetObj()];
                    //                    if ( [view action] == 0 ) [view setAction:@selector(controlAction:)];
                    [item setTarget:CocoaToolbarItemTargetGet()];
                    [item setAction:@selector(toolbarItemAction:)];
                }
                continue;
            }
            
            // stepper
            if ( [view isKindOfClass:[NSStepper class]] ) {
                if ( [view action] == 0 ) {
                    //                    if ( [view target] == nil ) [view setTarget:CocoaControlTargetObj()];
                    //                    if ( [view action] == 0 ) [view setAction:@selector(controlAction:)];
                    [item setTarget:CocoaToolbarItemTargetGet()];
                    [item setAction:@selector(toolbarItemAction:)];
                }
                continue;
            }
            
            // textfield
            if ( [view isKindOfClass:[NSTextField class]] ) {
                if ( [view isEditable] ) [view setDelegate:CocoaTextFieldDelegateGet()];
                continue;
            }
            
            // tokenfield
            if ( [view isKindOfClass:[NSTokenField class]] ) {
                if ( [view delegate] == nil ) [view setDelegate:CocoaTokenFieldDelegateGet()];
                continue;
            }
            
        }
    }
}

void CocoaNibWindowToolbarSetup( NSInteger wndTag )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) {
        NSToolbar *toolbar = [w toolbar];
        if ( toolbar ) {
            CocoaToolbarObjAdd( toolbar );
            CocoaNibWindowToolbarItemSetup( wndTag, toolbar );
        }
    }
}


id CocoaWindowFromNibObjects( NSInteger tag, NSString *nibName, NSString *wndIdentifier )
{
    id w = nil;
    
    CocoaNibObject *nibObj = CocoaNibObjectWithName( nibName );
    if ( nibObj ) {
        
        NSArray *objects = [nibObj topLevelObjects];
        
        for ( id obj in objects ) {
            w = nil;
            if ( [obj isKindOfClass:[NSWindow class]] || [obj isKindOfClass:[NSPanel class]] ) {
                
                if ( wndIdentifier ) {
                    if ( class_respondsToSelector( [obj class], NSSelectorFromString( @"identifier" ) ) ) {
                        if ( [obj identifier] != nil ) {
                            if ( [(NSString *)[obj identifier] isEqualToString:(NSString *)wndIdentifier] ) {
                                w = obj;
                            }
                        }
                    }
                } else {
                    w = obj;
                }
                
                if ( w ) {
                    if ( [obj tag] ) return w;
                    
                    /* create content view subclass for:
                     1. drawing focus rect around text controls (Carbon apps only).
                     2. intercepting content view drawRect: method to issue dialog viewDrawRect event.
                     */
                    // copy views to new content view
                    NSView *contentView = [obj contentView];
                    NSArray *subviews = [[contentView subviews] copy];
                    ContentViewSC *newContentView = [[ContentViewSC alloc] initWithFrame:[[obj contentView] frame]];
                    [newContentView setSubviews:subviews];
                    [obj setContentView:newContentView];
                    
                    [newContentView setIdentifier:[NSString stringWithFormat:@"%d",windowContentViewTag]];
                    
                    [subviews release];
                    
                    // tag
                    [obj setTag:tag];
                    
                    // window delegate
                    [obj setDelegate:CocoaWindowDelegateGet()];
                    
                    // app delegate
                    CocoaApplicationDelegate *appDelegate = CocoaApplicationDelegateGet();
                    if ( appDelegate ) {
                        [[appDelegate windows] addObject:obj];
                    }
                    
                    // set subviews action, target, etc.
                    CocoaNibViewSetup( tag, [obj contentView] );
                    
                    // set toolbar and custom items
                    CocoaNibWindowToolbarSetup( tag );
                    
                    // workaround for cosmetic in Carbon (black marks in top left and top right window border)
                    if ( !IsCocoaApplication() ) {
                        CocoaWindowCarbonRuntimeCosmeticFix( obj );
                    }
                    
                    // focus first editable view
                    CocoaWindowMakeFirstTextViewResponder( obj );
                    
#if __MAC_OS_X_VERSION_MAX_ALLOWED < 101100
                    [[obj contentView] setNeedsDisplay:YES];  // force drawRect:
#endif
                    
                    break;
                }
            }
        }
    }
    return w;
}


void CocoaViewFromNibObjects( NSInteger tag, NSString *nibName, NSString *viewIdentifier, NSPoint origin, NSView *superview )
{
    NSView *view = nil;
    
    CocoaNibObject *nibObj = CocoaNibObjectWithName( nibName );
    if ( nibObj ) {
        NSArray *objects = [nibObj topLevelObjects];
        
        for ( id obj in objects ) {
            view = nil;
            
            if ( [obj class] == [NSView class] || [obj class] == [ViewSC class] ) {
                if ( viewIdentifier ) {
                    if ( class_respondsToSelector( [obj class], NSSelectorFromString( @"identifier" ) ) ) {
                        if ( [obj identifier] != nil ) {
                            if ( [(NSString *)[obj identifier] isEqualToString:viewIdentifier] ) {
                                view = obj;
                            }
                        }
                    }
                } else {
                    view = obj;
                }
                
                if ( view ) {
                    
                    // origin
                    if ( origin.x != 0.0 || origin.y != 0.0 ) [view setFrameOrigin:origin];
                    [superview addSubview:view];
                    
                    // tag
                    if ( tag ) {
                        [view setIdentifier:[NSString stringWithFormat:@"%ld",(long)tag]];
                    }
                    
                    // set subviews action, target, etc.
                    CocoaNibViewSetup( [[superview window] tag], view );
                    
                    break;
                }
                
            }
        }
        
    }
}



void CocoaNibView( NSInteger inTag, CFStringRef inNibName, CFStringRef inViewIdentifier, CGPoint inOrigin, NSInteger inSuperviewTag )
{
    inTag = ABS(inTag);
    
    NSPoint origin = NSPointFromCGPoint( inOrigin );
    NSView *superview = nil;
    
    NSInteger dummyTag = 0;
    id w = CocoaTargetWindow( &dummyTag );
    if ( w ) {
        superview = CocoaTargetContentView( w );
        if ( inSuperviewTag ) {
            superview = CocoaViewSubviewWithTag( superview, inSuperviewTag );
        }
    }
    
    if ( superview ) {
        
        NSString *nibName = (NSString *)inNibName;
        NSString *viewIdentifier = (NSString *)inViewIdentifier;
        
        CocoaNibObject *nibObj = CocoaNibObjectWithName( nibName );
        if ( !nibObj ) {
            NSNib *nib = [[NSNib alloc] initWithNibNamed:nibName bundle:nil];
            if ( nib ) {
                NSArray *objects = nil;
                
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
                if ( [nib respondsToSelector:@selector(instantiateWithOwner:topLevelObjects:)]) {
                    [nib instantiateWithOwner:nil topLevelObjects:&objects];
                } else {
                    [nib instantiateNibWithOwner:nil topLevelObjects:&objects];
                }
#else
                [nib instantiateNibWithOwner:nil topLevelObjects:&objects];
#endif
                
                CocoaNibObjectAdd( nibName, objects );
            }
        }
        
        CocoaViewFromNibObjects( inTag, nibName, viewIdentifier, origin, superview );
    }
}



#pragma mark - nibpopover
///////////////////////////////////////////////////
//                  cocoa nibpopover             //
///////////////////////////////////////////////////
void CocoaPopoverFromNibObjects( NSInteger tag, NSString *nibName )
{
    CocoaNibObject *nibObj = CocoaNibObjectWithName( nibName );
    if ( nibObj ) {
        NSArray *objects = [nibObj topLevelObjects];
        
        for ( id obj in objects ) {
            if ( [obj isKindOfClass:[NSPopover class]] ) {
                
                // tag
                if ( tag ) [obj setTag:tag];
                
                // add popover
                CocoaPopoverAdd( obj, tag );
                
                // set subviews action, target, etc.
                NSView *view = [[obj contentViewController] view];
                CocoaNibViewSetup( tag, view );
                
                break;
            }
        }
        
    }
}

void CocoaNibPopover( NSInteger inTag, CFStringRef inNibName )
{
    inTag = ABS(inTag);
    
    if ( CocoaObjectIsWindow( inTag ) ) return;
    
#if !__LP64__
    if ( CocoaWindowIsFBWindow( inTag ) ) return;// if an FB Carbon window exists with same id/tag, alert and abort
#endif  //__LP64__
    
    NSPopover *popover = CocoaPopoverWithTag( inTag );
    if ( !popover ) {
        
        NSString *nibName = (NSString *)inNibName;
        
        CocoaNibObject *nibObj = CocoaNibObjectWithName( nibName );
        if ( !nibObj ) {
            NSNib *nib = [[NSNib alloc] initWithNibNamed:nibName bundle:nil];
            if ( nib ) {
                NSArray *objects = nil;
                
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
                if ( [nib respondsToSelector:@selector(instantiateWithOwner:topLevelObjects:)]) {
                    [nib instantiateWithOwner:nil topLevelObjects:&objects];
                } else {
                    [nib instantiateNibWithOwner:nil topLevelObjects:&objects];
                }
#else
                [nib instantiateNibWithOwner:nil topLevelObjects:&objects];
#endif
                
                CocoaNibObjectAdd( nibName, objects );
            }
        }
        
        CocoaPopoverFromNibObjects( inTag, nibName );
        
    }
}


#pragma mark - nibtoolbar
///////////////////////////////////////////////////
//                  cocoa nibtoolbar             //
///////////////////////////////////////////////////
void CocoaToolbarFromNibObjects( NSInteger tag, NSString *nibName )
{
    CocoaNibObject *nibObj = CocoaNibObjectWithName( nibName );
    if ( nibObj ) {
        NSArray *objects = [nibObj topLevelObjects];
        
        for ( id obj in objects ) {
            if ( [obj isKindOfClass:[NSToolbar class]] ) {
                
                // set item action, target, etc.
                NSArray *items = [obj items];
                for ( NSToolbarItem *item in items ) {
                    if ( [item tag] > 0 ) {
                        [item setTarget:CocoaToolbarItemTargetGet()];
                        [item setAction:@selector(toolbarItemAction:)];
                    }
                }
                
                // add to toolbars list
                CocoaToolbarObjAdd( obj );
                
                break;
            }
        }
        
    }
}

void CocoaNibToolbar( NSInteger inTag, CFStringRef inNibName )
{
    inTag = ABS(inTag);
    
    NSToolbar *toolbar = CocoaToolbarWithTag( inTag );
    if ( !toolbar ) {
        
        NSString *nibName = (NSString *)inNibName;
        
        CocoaNibObject *nibObj = CocoaNibObjectWithName( nibName );
        if ( !nibObj ) {
            NSNib *nib = [[NSNib alloc] initWithNibNamed:nibName bundle:nil];
            if ( nib ) {
                NSArray *objects = nil;
                
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
                if ( [nib respondsToSelector:@selector(instantiateWithOwner:topLevelObjects:)]) {
                    [nib instantiateWithOwner:nil topLevelObjects:&objects];
                } else {
                    [nib instantiateNibWithOwner:nil topLevelObjects:&objects];
                }
#else
                [nib instantiateNibWithOwner:nil topLevelObjects:&objects];
#endif
                
                CocoaNibObjectAdd( nibName, objects );
            }
        }
        
        CocoaToolbarFromNibObjects( inTag, nibName );
    }
}

//#endif // __MAC_OS_X_VERSION_MAX_ALLOWED > 1060



#pragma mark - nib <everything>
void CocoaNib( CFStringRef inNibName )
{
    NSString *nibName = (NSString *)inNibName;
    
    CocoaNibObject *nibObj = CocoaNibObjectWithName( nibName );
    if ( !nibObj ) {
        NSNib *nib = [[NSNib alloc] initWithNibNamed:nibName bundle:nil];
        if ( nib ) {
            NSArray *objects = nil;
            
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
            if ( [nib respondsToSelector:@selector(instantiateWithOwner:topLevelObjects:)]) {
                [nib instantiateWithOwner:nil topLevelObjects:&objects];
            } else {
                [nib instantiateNibWithOwner:nil topLevelObjects:&objects];
            }
#else
            [nib instantiateNibWithOwner:nil topLevelObjects:&objects];
#endif
            
            CocoaNibObjectAdd( nibName, objects );
            
            // menus
            
            // windows
            
            // popovers
            
            // toolbars
            
            // views
            
        }
    }
}


