/*
 Notification.m

 Bernie Wylde
 */

#import "Notification.h"

// - create -
NotificationRef NotificationWithName( CFStringRef name, CFTypeRef object, CFDictionaryRef userInfo )
{ return [NSNotification notificationWithName:(NSString *)name object:(id)object userInfo:(NSDictionary *)userInfo]; }

// - info -
CFStringRef NotificationName( NotificationRef ref )
{ return (CFStringRef)[ref name]; }

CFTypeRef NotificationObject( NotificationRef ref )
{ return (CFTypeRef)[ref object]; }

CFDictionaryRef NotificationUserInfo( NotificationRef ref )
{ return (CFDictionaryRef)[ref userInfo]; }



/*
 NotificationCenter.m
 */
static NSMutableArray *sCocoaNotificationCenters;

@implementation CocoaNotificationCenter : NSObject
@synthesize callback;
@synthesize name;

- (id)initWithCallback:(CocoaNotificationCenterCallbackType)cb name:(NSString *)nm {
    if ( self = [super init] ) {
        self.callback = cb;
        self.name = nm;
    }
    return self;
}

- (void)didReceiveNotification:(NSNotification *)notification {
    if ( callback ) (*callback)(notification);
}
@end

// -- private --
NSMutableArray *CocoaNotificationCenters( void )
{
    if ( !sCocoaNotificationCenters ) sCocoaNotificationCenters = [[NSMutableArray alloc] initWithCapacity:0];
    return sCocoaNotificationCenters;
}

// -- public --

// - default center -
NotificationCenterRef NotificationCenterDefault( void )
{ return [NSNotificationCenter defaultCenter]; }

// - adding and removing observers -
void NotificationCenterAddObserver( void *callback, CFStringRef name, CFTypeRef object )
{
    NSMutableArray *centers = CocoaNotificationCenters();
    CocoaNotificationCenter *center = [[CocoaNotificationCenter alloc] initWithCallback:callback name:(NSString *)name];
    [centers addObject:center];
    [[NSNotificationCenter defaultCenter] addObserver:center selector:@selector(didReceiveNotification:) name:(NSString *)name object:(id)object];
}

void NotificationCenterRemoveObserver( void *callback, CFStringRef name )
{
    NSMutableArray *centers = CocoaNotificationCenters();
    for ( CocoaNotificationCenter *center in centers ) {
        if ( (center.callback == callback) && ([[center name] isEqualToString:(NSString *)name]) ) {
            [[NSNotificationCenter defaultCenter] removeObserver:center name:(NSString *)name object:nil];
            [centers removeObject:center];
            break;
        }
    }
}

// - post notification -
void NotificationCenterPostNotification( NotificationRef note )
{ [[NSNotificationCenter defaultCenter] postNotification:note]; }

void NotificationCenterPostNotificationName( CFStringRef name, CFTypeRef object, CFDictionaryRef userInfo )
{ [[NSNotificationCenter defaultCenter] postNotificationName:(NSString *)name object:object userInfo:(NSDictionary *)userInfo]; }


