/*
 NumberFormatter.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// - init -
NumberFormatterRef NumberFormatterInit( void );

// - configure -
NSNumberFormatterBehavior NumberFormatterBehavior( NumberFormatterRef formatter );
void NumberFormatterSetBehavior( NumberFormatterRef formatter, NSNumberFormatterBehavior behavior );
NSNumberFormatterBehavior NumberFormatterDefaultBehavior( void );
void NumberFormatterSetDefaultBehavior( NSNumberFormatterBehavior behavior );
NSNumberFormatterStyle NumberFormatterNumberStyle( NumberFormatterRef formatter );
void NumberFormatterSetNumberStyle( NumberFormatterRef formatter, NSNumberFormatterStyle style );
Boolean NumberFormatterGeneratesDecimalNumbers( NumberFormatterRef formatter );
void NumberFormatterSetGeneratesDecimalNumbers( NumberFormatterRef formatter, Boolean flag );

// - convert between numbers and strings -
Boolean NumberFormatterGetObjectValue( NumberFormatterRef formatter, CFTypeRef *obj, CFStringRef string, CFRange *range, ErrorRef *err  );
CFNumberRef NumberFormatterNumberFromString( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterStringFromNumber( NumberFormatterRef formatter, CFNumberRef number );
CFStringRef NumberFormatterLocalizedStringFromNumber( CFNumberRef number, NSNumberFormatterStyle style );

// - localization -
Boolean NumberFormatterLocalizesFormat( NumberFormatterRef formatter );
void NumberFormatterSetLocalizesFormat( NumberFormatterRef formatter, Boolean flag );
CFLocaleRef NumberFormatterLocale( NumberFormatterRef formatter );
void NumberFormatterSetLocale( NumberFormatterRef formatter, CFLocaleRef locale );

// - rounding -
DecimalNumberHandlerRef NumberFormatterRoundingBehavior( NumberFormatterRef formatter );
void NumberFormatterSetRoundingBehavior( NumberFormatterRef formatter, DecimalNumberHandlerRef behavior );
CFNumberRef NumberFormatterRoundingIncrement( NumberFormatterRef formatter );
void NumberFormatterSetRoundingIncrement( NumberFormatterRef formatter, CFNumberRef number );
NSNumberFormatterRoundingMode NumberFormatterRoundingMode( NumberFormatterRef formatter );
void NumberFormatterSetRoundingMode( NumberFormatterRef formatter, NSNumberFormatterRoundingMode mode );

// - formats -
CFStringRef NumberFormatterFormat( NumberFormatterRef formatter );
void NumberFormatterSetFormat( NumberFormatterRef formatter, CFStringRef string );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSFormattingContext NumberFormatterFormattingContext( NumberFormatterRef formatter );
void NumberFormatterSetFormattingContext( NumberFormatterRef formatter, NSFormattingContext context );
#endif

NSUInteger NumberFormatterFormatWidth( NumberFormatterRef formatter );
void NumberFormatterSetFormatWidth( NumberFormatterRef formatter, NSUInteger value );
CFStringRef NumberFormatterNegativeFormat( NumberFormatterRef formatter );
void NumberFormatterSetNegativeWidth( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterPositiveFormat( NumberFormatterRef formatter );
void NumberFormatterSetPositiveFormat( NumberFormatterRef formatter, CFStringRef string );
CFNumberRef NumberFormatterMultiplier( NumberFormatterRef formatter );
void NumberFormatterSetMultiplier( NumberFormatterRef formatter, CFNumberRef number );

// - symbols -
CFStringRef NumberFormatterPercentSymbol( NumberFormatterRef formatter );
void NumberFormatterSetPercentSymbol( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterPerMillSymbol( NumberFormatterRef formatter );
void NumberFormatterSetPerMillSymbol( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterMinusSign( NumberFormatterRef formatter );
void NumberFormatterSetMinusSign( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterPlusSign( NumberFormatterRef formatter );
void NumberFormatterSetPlusSign( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterExponentSymbol( NumberFormatterRef formatter );
void NumberFormatterSetExponentSymbol( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterZeroSymbol( NumberFormatterRef formatter );
void NumberFormatterSetZeroSymbol( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterNilSymbol( NumberFormatterRef formatter );
void NumberFormatterSetNilSymbol( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterNotANumberSymbol( NumberFormatterRef formatter );
void NumberFormatterSetNotANumberSymbol( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterNegativeInfinitySymbol( NumberFormatterRef formatter );
void NumberFormatterSetNegativeInfinitySymbol( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterPositiveInfinitySymbol( NumberFormatterRef formatter );
void NumberFormatterSetPositiveInfinitySymbol( NumberFormatterRef formatter, CFStringRef string );

// - currency -
CFStringRef NumberFormatterCurrencySymbol( NumberFormatterRef formatter );
void NumberFormatterSetCurrencySymbol( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterCurrencyCode( NumberFormatterRef formatter );
void NumberFormatterSetCurrencyCode( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterInternationalCurrencySymbol( NumberFormatterRef formatter );
void NumberFormatterSetInternationalCurrencySymbol( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterCurrencyGroupingSeparator( NumberFormatterRef formatter );
void NumberFormatterSetCurrencyGroupingSeparator( NumberFormatterRef formatter, CFStringRef string );

// - prefix and suffix -
CFStringRef NumberFormatterPositivePrefix( NumberFormatterRef formatter );
void NumberFormatterSetPositivePrefix( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterPositiveSuffix( NumberFormatterRef formatter );
void NumberFormatterSetPositiveSuffix( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterNegativePrefix( NumberFormatterRef formatter );
void NumberFormatterSetNegativePrefix( NumberFormatterRef formatter, CFStringRef string );
CFStringRef NumberFormatterNegativeSuffix( NumberFormatterRef formatter );
void NumberFormatterSetNegativeSuffix( NumberFormatterRef formatter, CFStringRef string );

// - display -
CFDictionaryRef NumberFormatterTextAttributesForNegativeValues( NumberFormatterRef formatter );
void NumberFormatterSetTextAttributesForNegativeValues( NumberFormatterRef formatter, CFDictionaryRef attributes );
CFDictionaryRef NumberFormatterTextAttributesForPositiveValues( NumberFormatterRef formatter );
void NumberFormatterSetTextAttributesForPositiveValues( NumberFormatterRef formatter, CFDictionaryRef attributes );
CFAttributedStringRef NumberFormatterAttributedStringForZero( NumberFormatterRef formatter );
void NumberFormatterSetAttributedStringForZero( NumberFormatterRef formatter, CFAttributedStringRef aString );
CFDictionaryRef NumberFormatterTextAttributesForZero( NumberFormatterRef formatter );
void NumberFormatterSetTextAttributesForZero( NumberFormatterRef formatter, CFDictionaryRef attributes );
CFAttributedStringRef NumberFormatterAttributedStringForNil( NumberFormatterRef formatter );
void NumberFormatterSetAttributedStringForNil( NumberFormatterRef formatter, CFAttributedStringRef aString );
CFDictionaryRef NumberFormatterTextAttributesForNil( NumberFormatterRef formatter );
void NumberFormatterSetTextAttributesForNil( NumberFormatterRef formatter, CFDictionaryRef attributes );
CFAttributedStringRef NumberFormatterAttributedStringForNotANumber( NumberFormatterRef formatter );
void NumberFormatterSetAttributedStringForNotANumber( NumberFormatterRef formatter, CFAttributedStringRef aString );
CFDictionaryRef NumberFormatterTextAttributesForNotANumber( NumberFormatterRef formatter );
void NumberFormatterSetTextAttributesForNotANumber( NumberFormatterRef formatter, CFDictionaryRef attributes );
CFDictionaryRef NumberFormatterTextAttributesForPositiveInfinity( NumberFormatterRef formatter );
void NumberFormatterSetTextAttributesForPositiveInfinity( NumberFormatterRef formatter, CFDictionaryRef attributes );
CFDictionaryRef NumberFormatterTextAttributesForNegativeInfinity( NumberFormatterRef formatter );
void NumberFormatterSetTextAttributesForNegativeInfinity( NumberFormatterRef formatter, CFDictionaryRef attributes );

// - separators and grouping -
CFStringRef NumberFormatterGroupingSeparator( NumberFormatterRef formatter );
void NumberFormatterSetGroupingSeparator( NumberFormatterRef formatter, CFStringRef string );
Boolean NumberFormatterUsesGroupingSeparator( NumberFormatterRef formatter );
void NumberFormatterSetUsesGroupingSeparator( NumberFormatterRef formatter, Boolean flag );
CFStringRef NumberFormatterThousandSeparator( NumberFormatterRef formatter );
void NumberFormatterSetThousandSeparator( NumberFormatterRef formatter, CFStringRef string );
Boolean NumberFormatterHasThousandSeparators( NumberFormatterRef formatter );
void NumberFormatterSetHasThousandSeparators( NumberFormatterRef formatter, Boolean flag );
CFStringRef NumberFormatterDecimalSeparator( NumberFormatterRef formatter );
void NumberFormatterSetDecimalSeparator( NumberFormatterRef formatter, CFStringRef string );
Boolean NumberFormatterAlwaysShowsDecimalSeparator( NumberFormatterRef formatter );
void NumberFormatterSetAlwaysShowsDecimalSeparator( NumberFormatterRef formatter, Boolean flag );
CFStringRef NumberFormatterCurrencyDecimalSeparator( NumberFormatterRef formatter );
void NumberFormatterSetCurrencyDecimalSeparator( NumberFormatterRef formatter, CFStringRef string );
NSUInteger NumberFormatterGroupingSize( NumberFormatterRef formatter );
void NumberFormatterSetGroupingSize( NumberFormatterRef formatter, NSUInteger value );
NSUInteger NumberFormatterSecondaryGroupingSize( NumberFormatterRef formatter );
void NumberFormatterSetSecondaryGroupingSize( NumberFormatterRef formatter, NSUInteger value );

// - padding -
CFStringRef NumberFormatterPaddingCharacter( NumberFormatterRef formatter );
void NumberFormatterSetPaddingCharacter( NumberFormatterRef formatter, CFStringRef string );
NSNumberFormatterPadPosition NumberFormatterPaddingPosition( NumberFormatterRef formatter );
void NumberFormatterSetPaddingPosition( NumberFormatterRef formatter, NSNumberFormatterPadPosition position );

// - input and output attributes -
Boolean NumberFormatterAllowsFloats( NumberFormatterRef formatter );
void NumberFormatterSetAllowsFloats( NumberFormatterRef formatter, Boolean flag );
CFNumberRef NumberFormatterMinimum( NumberFormatterRef formatter );
void NumberFormatterSetMinimum( NumberFormatterRef formatter, CFNumberRef number );
CFNumberRef NumberFormatterMaximum( NumberFormatterRef formatter );
void NumberFormatterSetMaximum( NumberFormatterRef formatter, CFNumberRef number );

// - integer and fraction digits -
NSUInteger NumberFormatterMinimumIntegerDigits( NumberFormatterRef formatter );
void NumberFormatterSetMinimumIntegerDigits( NumberFormatterRef formatter, NSUInteger value );
NSUInteger NumberFormatterMaximumIntegerDigits( NumberFormatterRef formatter );
void NumberFormatterSetMaximumIntegerDigits( NumberFormatterRef formatter, NSUInteger value );
NSUInteger NumberFormatterMinimumFractionDigits( NumberFormatterRef formatter );
void NumberFormatterSetMinimumFractionDigits( NumberFormatterRef formatter, NSUInteger value );
NSUInteger NumberFormatterMaximumFractionDigits( NumberFormatterRef formatter );
void NumberFormatterSetMaximumFractionDigits( NumberFormatterRef formatter, NSUInteger value );

// - significant digits -
Boolean NumberFormatterUsesSignificantDigits( NumberFormatterRef formatter );
void NumberFormatterSetUsesSignificantDigits( NumberFormatterRef formatter, Boolean flag );
NSUInteger NumberFormatterMinimumSignificantDigits( NumberFormatterRef formatter );
void NumberFormatterSetMinimumSignificantDigits( NumberFormatterRef formatter, NSUInteger value );
NSUInteger NumberFormatterMaximumSignificantDigits( NumberFormatterRef formatter );
void NumberFormatterSetMaximumSignificantDigits( NumberFormatterRef formatter, NSUInteger value );

// - leniency behavior -
Boolean NumberFormatterIsLenient( NumberFormatterRef formatter );
void NumberFormatterSetLenient( NumberFormatterRef formatter, Boolean flag );

// - validation of partial numeric -
Boolean NumberFormatterIsPartialStringValidationEnabled( NumberFormatterRef formatter );
void NumberFormatterSetPartialStringValidationEnabled( NumberFormatterRef formatter, Boolean flag );

