/*
 NumberFormatter.m

 Bernie Wylde
 */

#import "NumberFormatter.h"

// - init -
NumberFormatterRef NumberFormatterInit( void )
{ return [[[NSNumberFormatter alloc] init] autorelease]; }

// - configure -
NSNumberFormatterBehavior NumberFormatterBehavior( NumberFormatterRef formatter )
{ return [formatter formatterBehavior]; }

void NumberFormatterSetBehavior( NumberFormatterRef formatter, NSNumberFormatterBehavior behavior )
{ [formatter setFormatterBehavior:behavior]; }

NSNumberFormatterBehavior NumberFormatterDefaultBehavior( void )
{ return [NSNumberFormatter defaultFormatterBehavior]; }

void NumberFormatterSetDefaultBehavior( NSNumberFormatterBehavior behavior )
{ [NSNumberFormatter setDefaultFormatterBehavior:behavior]; }

NSNumberFormatterStyle NumberFormatterNumberStyle( NumberFormatterRef formatter )
{ return [formatter numberStyle]; }

void NumberFormatterSetNumberStyle( NumberFormatterRef formatter, NSNumberFormatterStyle style )
{ [formatter setNumberStyle:style]; }

Boolean NumberFormatterGeneratesDecimalNumbers( NumberFormatterRef formatter )
{ return [formatter generatesDecimalNumbers]; }

void NumberFormatterSetGeneratesDecimalNumbers( NumberFormatterRef formatter, Boolean flag )
{ [formatter setGeneratesDecimalNumbers:flag]; }

// - convert between numbers and strings -
Boolean NumberFormatterGetObjectValue( NumberFormatterRef formatter, CFTypeRef *obj, CFStringRef string, CFRange *range, ErrorRef *err  )
{
    NSRange nsRange;
    Boolean result = [formatter getObjectValue:(id *)obj forString:(NSString *)string range:&nsRange error:err];
    if ( result ) {
        range->location = nsRange.location;
        range->length = nsRange.length;
    }
    return result;
}

CFNumberRef NumberFormatterNumberFromString( NumberFormatterRef formatter, CFStringRef string )
{ return (CFNumberRef)[formatter numberFromString:(NSString *)string]; }

CFStringRef NumberFormatterStringFromNumber( NumberFormatterRef formatter, CFNumberRef number )
{ return (CFStringRef)[formatter stringFromNumber:(NSNumber *)number]; }

CFStringRef NumberFormatterLocalizedStringFromNumber( CFNumberRef number, NSNumberFormatterStyle style )
{ return (CFStringRef)[NSNumberFormatter localizedStringFromNumber:(NSNumber *)number numberStyle:style]; }

// - localization -
Boolean NumberFormatterLocalizesFormat( NumberFormatterRef formatter )
{ return [formatter localizesFormat]; }

void NumberFormatterSetLocalizesFormat( NumberFormatterRef formatter, Boolean flag )
{ [formatter setLocalizesFormat:flag]; }

CFLocaleRef NumberFormatterLocale( NumberFormatterRef formatter )
{ return (CFLocaleRef)[formatter locale]; }

void NumberFormatterSetLocale( NumberFormatterRef formatter, CFLocaleRef locale )
{ [formatter setLocale:(NSLocale *)locale]; }

// - rounding -
DecimalNumberHandlerRef NumberFormatterRoundingBehavior( NumberFormatterRef formatter )
{ return [formatter roundingBehavior]; }

void NumberFormatterSetRoundingBehavior( NumberFormatterRef formatter, DecimalNumberHandlerRef behavior )
{ [formatter setRoundingBehavior:behavior]; }

CFNumberRef NumberFormatterRoundingIncrement( NumberFormatterRef formatter )
{ return (CFNumberRef)[formatter roundingIncrement]; }

void NumberFormatterSetRoundingIncrement( NumberFormatterRef formatter, CFNumberRef number )
{ [formatter setRoundingIncrement:(NSNumber *)number]; }

NSNumberFormatterRoundingMode NumberFormatterRoundingMode( NumberFormatterRef formatter )
{ return [formatter roundingMode]; }

void NumberFormatterSetRoundingMode( NumberFormatterRef formatter, NSNumberFormatterRoundingMode mode )
{ [formatter setRoundingMode:mode]; }

// - formats -
CFStringRef NumberFormatterFormat( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter format]; }

void NumberFormatterSetFormat( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setFormat:(NSString *)string]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSFormattingContext NumberFormatterFormattingContext( NumberFormatterRef formatter )
{ return [formatter formattingContext]; }

void NumberFormatterSetFormattingContext( NumberFormatterRef formatter, NSFormattingContext context )
{ [formatter setFormattingContext:context]; }
#endif

NSUInteger NumberFormatterFormatWidth( NumberFormatterRef formatter )
{ return [formatter formatWidth]; }

void NumberFormatterSetFormatWidth( NumberFormatterRef formatter, NSUInteger value )
{ [formatter setFormatWidth:value]; }

CFStringRef NumberFormatterNegativeFormat( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter negativeFormat]; }

void NumberFormatterSetNegativeWidth( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setNegativeFormat:(NSString *)string]; }

CFStringRef NumberFormatterPositiveFormat( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter positiveFormat]; }

void NumberFormatterSetPositiveFormat( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setPositiveFormat:(NSString *)string]; }

CFNumberRef NumberFormatterMultiplier( NumberFormatterRef formatter )
{ return (CFNumberRef)[formatter multiplier]; }

void NumberFormatterSetMultiplier( NumberFormatterRef formatter, CFNumberRef number )
{ [formatter setMultiplier:(NSNumber *)number]; }

// - symbols -
CFStringRef NumberFormatterPercentSymbol( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter percentSymbol]; }

void NumberFormatterSetPercentSymbol( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setPercentSymbol:(NSString *)string]; }

CFStringRef NumberFormatterPerMillSymbol( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter perMillSymbol]; }

void NumberFormatterSetPerMillSymbol( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setPerMillSymbol:(NSString *)string]; }

CFStringRef NumberFormatterMinusSign( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter minusSign]; }

void NumberFormatterSetMinusSign( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setMinusSign:(NSString *)string]; }

CFStringRef NumberFormatterPlusSign( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter plusSign]; }

void NumberFormatterSetPlusSign( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setPlusSign:(NSString *)string]; }

CFStringRef NumberFormatterExponentSymbol( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter exponentSymbol]; }

void NumberFormatterSetExponentSymbol( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setExponentSymbol:(NSString *)string]; }

CFStringRef NumberFormatterZeroSymbol( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter zeroSymbol]; }

void NumberFormatterSetZeroSymbol( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setZeroSymbol:(NSString *)string]; }

CFStringRef NumberFormatterNilSymbol( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter nilSymbol]; }

void NumberFormatterSetNilSymbol( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setNilSymbol:(NSString *)string]; }

CFStringRef NumberFormatterNotANumberSymbol( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter notANumberSymbol]; }

void NumberFormatterSetNotANumberSymbol( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setNotANumberSymbol:(NSString *)string]; }

CFStringRef NumberFormatterNegativeInfinitySymbol( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter negativeInfinitySymbol]; }

void NumberFormatterSetNegativeInfinitySymbol( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setNegativeInfinitySymbol:(NSString *)string]; }

CFStringRef NumberFormatterPositiveInfinitySymbol( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter positiveInfinitySymbol]; }

void NumberFormatterSetPositiveInfinitySymbol( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setPositiveInfinitySymbol:(NSString *)string]; }

// - currency -
CFStringRef NumberFormatterCurrencySymbol( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter currencySymbol]; }

void NumberFormatterSetCurrencySymbol( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setCurrencySymbol:(NSString *)string]; }

CFStringRef NumberFormatterCurrencyCode( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter currencyCode]; }

void NumberFormatterSetCurrencyCode( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setCurrencyCode:(NSString *)string]; }

CFStringRef NumberFormatterInternationalCurrencySymbol( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter internationalCurrencySymbol]; }

void NumberFormatterSetInternationalCurrencySymbol( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setInternationalCurrencySymbol:(NSString *)string]; }

CFStringRef NumberFormatterCurrencyGroupingSeparator( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter currencyGroupingSeparator]; }

void NumberFormatterSetCurrencyGroupingSeparator( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setCurrencyGroupingSeparator:(NSString *)string]; }

// - prefix and suffix -
CFStringRef NumberFormatterPositivePrefix( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter positivePrefix]; }

void NumberFormatterSetPositivePrefix( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setPositivePrefix:(NSString *)string]; }

CFStringRef NumberFormatterPositiveSuffix( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter positiveSuffix]; }

void NumberFormatterSetPositiveSuffix( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setPositiveSuffix:(NSString *)string]; }

CFStringRef NumberFormatterNegativePrefix( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter negativePrefix]; }

void NumberFormatterSetNegativePrefix( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setNegativePrefix:(NSString *)string]; }

CFStringRef NumberFormatterNegativeSuffix( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter negativeSuffix]; }

void NumberFormatterSetNegativeSuffix( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setNegativeSuffix:(NSString *)string]; }

// - display -
CFDictionaryRef NumberFormatterTextAttributesForNegativeValues( NumberFormatterRef formatter )
{ return (CFDictionaryRef)[formatter textAttributesForNegativeValues]; }

void NumberFormatterSetTextAttributesForNegativeValues( NumberFormatterRef formatter, CFDictionaryRef attributes )
{ [formatter setTextAttributesForNegativeValues:(NSDictionary *)attributes]; }

CFDictionaryRef NumberFormatterTextAttributesForPositiveValues( NumberFormatterRef formatter )
{ return (CFDictionaryRef)[formatter textAttributesForPositiveValues]; }

void NumberFormatterSetTextAttributesForPositiveValues( NumberFormatterRef formatter, CFDictionaryRef attributes )
{ [formatter setTextAttributesForPositiveValues:(NSDictionary *)attributes]; }

CFAttributedStringRef NumberFormatterAttributedStringForZero( NumberFormatterRef formatter )
{ return (CFAttributedStringRef)[formatter attributedStringForZero]; }

void NumberFormatterSetAttributedStringForZero( NumberFormatterRef formatter, CFAttributedStringRef aString )
{ [formatter setAttributedStringForZero:(NSAttributedString *)aString]; }

CFDictionaryRef NumberFormatterTextAttributesForZero( NumberFormatterRef formatter )
{ return (CFDictionaryRef)[formatter textAttributesForZero]; }

void NumberFormatterSetTextAttributesForZero( NumberFormatterRef formatter, CFDictionaryRef attributes )
{ [formatter setTextAttributesForZero:(NSDictionary *)attributes]; }

CFAttributedStringRef NumberFormatterAttributedStringForNil( NumberFormatterRef formatter )
{ return (CFAttributedStringRef)[formatter attributedStringForNil]; }

void NumberFormatterSetAttributedStringForNil( NumberFormatterRef formatter, CFAttributedStringRef aString )
{ [formatter setAttributedStringForNil:(NSAttributedString *)aString]; }

CFDictionaryRef NumberFormatterTextAttributesForNil( NumberFormatterRef formatter )
{ return (CFDictionaryRef)[formatter textAttributesForNil]; }

void NumberFormatterSetTextAttributesForNil( NumberFormatterRef formatter, CFDictionaryRef attributes )
{ [formatter setTextAttributesForNil:(NSDictionary *)attributes]; }

CFAttributedStringRef NumberFormatterAttributedStringForNotANumber( NumberFormatterRef formatter )
{ return (CFAttributedStringRef)[formatter attributedStringForNotANumber]; }

void NumberFormatterSetAttributedStringForNotANumber( NumberFormatterRef formatter, CFAttributedStringRef aString )
{ [formatter setAttributedStringForNotANumber:(NSAttributedString *)aString]; }

CFDictionaryRef NumberFormatterTextAttributesForNotANumber( NumberFormatterRef formatter )
{ return (CFDictionaryRef)[formatter textAttributesForNotANumber]; }

void NumberFormatterSetTextAttributesForNotANumber( NumberFormatterRef formatter, CFDictionaryRef attributes )
{ [formatter setTextAttributesForNotANumber:(NSDictionary *)attributes]; }

CFDictionaryRef NumberFormatterTextAttributesForPositiveInfinity( NumberFormatterRef formatter )
{ return (CFDictionaryRef)[formatter textAttributesForPositiveInfinity]; }

void NumberFormatterSetTextAttributesForPositiveInfinity( NumberFormatterRef formatter, CFDictionaryRef attributes )
{ [formatter setTextAttributesForPositiveInfinity:(NSDictionary *)attributes]; }

CFDictionaryRef NumberFormatterTextAttributesForNegativeInfinity( NumberFormatterRef formatter )
{ return (CFDictionaryRef)[formatter textAttributesForNegativeInfinity]; }

void NumberFormatterSetTextAttributesForNegativeInfinity( NumberFormatterRef formatter, CFDictionaryRef attributes )
{ [formatter setTextAttributesForNegativeInfinity:(NSDictionary *)attributes]; }

// - separators and grouping -
CFStringRef NumberFormatterGroupingSeparator( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter groupingSeparator]; }

void NumberFormatterSetGroupingSeparator( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setGroupingSeparator:(NSString *)string]; }

Boolean NumberFormatterUsesGroupingSeparator( NumberFormatterRef formatter )
{ return [formatter usesGroupingSeparator]; }

void NumberFormatterSetUsesGroupingSeparator( NumberFormatterRef formatter, Boolean flag )
{ [formatter setUsesGroupingSeparator:flag]; }

CFStringRef NumberFormatterThousandSeparator( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter thousandSeparator]; }

void NumberFormatterSetThousandSeparator( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setThousandSeparator:(NSString *)string]; }

Boolean NumberFormatterHasThousandSeparators( NumberFormatterRef formatter )
{ return [formatter hasThousandSeparators]; }

void NumberFormatterSetHasThousandSeparators( NumberFormatterRef formatter, Boolean flag )
{ [formatter setHasThousandSeparators:flag]; }

CFStringRef NumberFormatterDecimalSeparator( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter decimalSeparator]; }

void NumberFormatterSetDecimalSeparator( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setDecimalSeparator:(NSString *)string]; }

Boolean NumberFormatterAlwaysShowsDecimalSeparator( NumberFormatterRef formatter )
{ return [formatter alwaysShowsDecimalSeparator]; }

void NumberFormatterSetAlwaysShowsDecimalSeparator( NumberFormatterRef formatter, Boolean flag )
{ [formatter setAlwaysShowsDecimalSeparator:flag]; }

CFStringRef NumberFormatterCurrencyDecimalSeparator( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter currencyDecimalSeparator]; }

void NumberFormatterSetCurrencyDecimalSeparator( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setCurrencyDecimalSeparator:(NSString *)string]; }

NSUInteger NumberFormatterGroupingSize( NumberFormatterRef formatter )
{ return [formatter groupingSize]; }

void NumberFormatterSetGroupingSize( NumberFormatterRef formatter, NSUInteger value )
{ [formatter setGroupingSize:value]; }

NSUInteger NumberFormatterSecondaryGroupingSize( NumberFormatterRef formatter )
{ return [formatter secondaryGroupingSize]; }

void NumberFormatterSetSecondaryGroupingSize( NumberFormatterRef formatter, NSUInteger value )
{ [formatter setSecondaryGroupingSize:value]; }

// - padding -
CFStringRef NumberFormatterPaddingCharacter( NumberFormatterRef formatter )
{ return (CFStringRef)[formatter paddingCharacter]; }

void NumberFormatterSetPaddingCharacter( NumberFormatterRef formatter, CFStringRef string )
{ [formatter setPaddingCharacter:(NSString *)string]; }

NSNumberFormatterPadPosition NumberFormatterPaddingPosition( NumberFormatterRef formatter )
{ return [formatter paddingPosition]; }

void NumberFormatterSetPaddingPosition( NumberFormatterRef formatter, NSNumberFormatterPadPosition position )
{ [formatter setPaddingPosition:position]; }

// - input and output attributes -
Boolean NumberFormatterAllowsFloats( NumberFormatterRef formatter )
{ return [formatter allowsFloats]; }

void NumberFormatterSetAllowsFloats( NumberFormatterRef formatter, Boolean flag )
{ [formatter setAllowsFloats:flag]; }

CFNumberRef NumberFormatterMinimum( NumberFormatterRef formatter )
{ return (CFNumberRef)[formatter minimum]; }

void NumberFormatterSetMinimum( NumberFormatterRef formatter, CFNumberRef number )
{ [formatter setMinimum:(NSNumber *)number]; }

CFNumberRef NumberFormatterMaximum( NumberFormatterRef formatter )
{ return (CFNumberRef)[formatter maximum]; }

void NumberFormatterSetMaximum( NumberFormatterRef formatter, CFNumberRef number )
{ [formatter setMaximum:(NSNumber *)number]; }

// - integer and fraction digits -
NSUInteger NumberFormatterMinimumIntegerDigits( NumberFormatterRef formatter )
{ return [formatter minimumIntegerDigits]; }

void NumberFormatterSetMinimumIntegerDigits( NumberFormatterRef formatter, NSUInteger value )
{ [formatter setMinimumIntegerDigits:value]; }

NSUInteger NumberFormatterMaximumIntegerDigits( NumberFormatterRef formatter )
{ return [formatter maximumIntegerDigits]; }

void NumberFormatterSetMaximumIntegerDigits( NumberFormatterRef formatter, NSUInteger value )
{ [formatter setMaximumIntegerDigits:value]; }

NSUInteger NumberFormatterMinimumFractionDigits( NumberFormatterRef formatter )
{ return [formatter minimumFractionDigits]; }

void NumberFormatterSetMinimumFractionDigits( NumberFormatterRef formatter, NSUInteger value )
{ [formatter setMinimumFractionDigits:value]; }

NSUInteger NumberFormatterMaximumFractionDigits( NumberFormatterRef formatter )
{ return [formatter maximumFractionDigits]; }

void NumberFormatterSetMaximumFractionDigits( NumberFormatterRef formatter, NSUInteger value )
{ [formatter setMaximumFractionDigits:value]; }

// - significant digits -
Boolean NumberFormatterUsesSignificantDigits( NumberFormatterRef formatter )
{ return [formatter usesSignificantDigits]; }

void NumberFormatterSetUsesSignificantDigits( NumberFormatterRef formatter, Boolean flag )
{ [formatter setUsesSignificantDigits:flag]; }

NSUInteger NumberFormatterMinimumSignificantDigits( NumberFormatterRef formatter )
{ return [formatter minimumSignificantDigits]; }

void NumberFormatterSetMinimumSignificantDigits( NumberFormatterRef formatter, NSUInteger value )
{ [formatter setMinimumSignificantDigits:value]; }

NSUInteger NumberFormatterMaximumSignificantDigits( NumberFormatterRef formatter )
{ return [formatter maximumSignificantDigits]; }

void NumberFormatterSetMaximumSignificantDigits( NumberFormatterRef formatter, NSUInteger value )
{ [formatter setMaximumSignificantDigits:value]; }

// - leniency behavior -
Boolean NumberFormatterIsLenient( NumberFormatterRef formatter )
{ return [formatter isLenient]; }

void NumberFormatterSetLenient( NumberFormatterRef formatter, Boolean flag )
{ [formatter setLenient:flag]; }

// - validation of partial numeric -
Boolean NumberFormatterIsPartialStringValidationEnabled( NumberFormatterRef formatter )
{ return [formatter isPartialStringValidationEnabled]; }

void NumberFormatterSetPartialStringValidationEnabled( NumberFormatterRef formatter, Boolean flag )
{ [formatter setPartialStringValidationEnabled:flag]; }

