/*
 Object.m

 Bernie Wylde
 */

#import "Object.h"

@implementation CocoaObjectTarget

@synthesize callback;

- (void)objectSelector:(id)object {
    (*callback)(object);
}
@end

@implementation NSObject (Additions)
@dynamic wndTagNumber;
@dynamic objProperties;

- (NSInteger)wndTag {
    return [objc_getAssociatedObject(self, @selector(wndTagNumber)) integerValue];
}

- (void)setWndTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(wndTagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (NSMutableDictionary *)objectProperties {
    return objc_getAssociatedObject(self, @selector(objProperties));
}

- (void)setObjectProperties:(NSMutableDictionary *)properties {
    objc_setAssociatedObject(self, @selector(objProperties), properties, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

//NSMutableArray *CocoaObjects( void )
//{
//    static NSMutableArray *sCocoaObjectss;
//    if ( !sCocoaObjects ) sCocoaObjects = [[NSMutableArray alloc] initWithCapacity:0];
//    return sCocoaObjects;
//}
//
//ObjectRef ObjectWithID( NSInteger objID )
//{
//    NSMutableArray *objects = CocoaObjects();
//    for ( id obj in objects ) {
//        if ( [obj ID] ) return obj;
//    }
//    return nil;
//}
//
//void ObjectSetID( ObjectRef obj, NSInteger objID )
//{
//    NSMutableArray *objects = CocoaObjects();
//    NSUInteger index = [objects indexOfObject:obj];
//    [obj setID:objID];
//    if ( index == NSNotFound ) {
//        [objects addObject:obj];
//    } else {
//        [objects replaceObjectAtIndex:index withObject:obj];
//    }
//}



// - create, copy, dealloc -
CFTypeRef ObjectCopy( CFTypeRef obj )
{ return (CFTypeRef)[[(id)obj copy] autorelease]; }

CFTypeRef ObjectMutableCopy( CFTypeRef obj )
{ return (CFTypeRef)[[(id)obj mutableCopy] autorelease]; }


//id ObjectNSClassFromCF( CFTypeRef obj )
//{
//    CFTypeID type = CFGetTypeID(obj);
//    switch ( type ) {
//        case CFArrayGetTypeID():
//            return [NSArray class];
//        case CFDictionaryGetTypeID():
//            return [NSDictionary class];
//        case CFStringGetTypeID():
//            return [NSString class];
//        default:
//            break;
//    }
//    return [(id)obj class]
//}


ClassRef ObjectClass( CFTypeRef obj )
{
    // should check for CF type here and return NS equivalent
    return [(id)obj class];
}

ClassRef ObjectSuperclass( CFTypeRef obj )
{ return [(id)obj superclass]; }


// - identify class -
Boolean ObjectIsSubclassOfClass( CFTypeRef obj, ClassRef class )
{ return [(id)obj isSubclassOfClass:class]; }

// Sending messages
void ObjectPerformSelector( CFTypeRef obj, CFStringRef selector, CFTypeRef withObj, CFTimeInterval afterDelay )
{ [(id)obj performSelector:NSSelectorFromString((NSString *)selector) withObject:(id)withObj afterDelay:afterDelay]; }

void ObjectPerformSelectorOnMainThread( CFTypeRef obj, CFStringRef selector, CFTypeRef withObj, Boolean waitUntilDone )
{ [(id)obj performSelectorOnMainThread:NSSelectorFromString((NSString *)selector) withObject:(id)withObj waitUntilDone:waitUntilDone]; }

void ObjectPerformSelectorInBackground( CFTypeRef obj, CFStringRef selector, CFTypeRef withObj )
{ [(id)obj performSelectorInBackground:NSSelectorFromString((NSString *)selector) withObject:(id)withObj]; }


// - instance methods -
Boolean ObjectIsEqual( CFTypeRef obj1, CFTypeRef obj2 )
{ return [(id)obj1 isEqual:(id)obj2]; }

void ObjectSetValueForKey( CFTypeRef obj, CFTypeRef value, CFStringRef key )
{ [(NSObject *)obj setValue:(id)value forKey:(NSString *)key]; }

CFTypeRef ObjectValueForKey( CFTypeRef obj, CFStringRef key )
{ return (CFTypeRef)[(id)obj valueForKey:(NSString *)key]; }


// -- NSObject protocol methods --
Boolean ObjectIsKindOfClass( CFTypeRef obj, ClassRef class )
{ return [(id)obj isKindOfClass:class]; }





// -- custom --
void ObjectCallFunction( void *fnAddress, CFTypeRef withObj, CFTimeInterval afterDelay )
{
    CocoaObjectTarget *target = [[CocoaObjectTarget alloc] init];
    [target setCallback:fnAddress];
    [target performSelector:@selector(objectSelector:) withObject:withObj afterDelay:afterDelay];
}

void ObjectCallFunctionOnMainThread( void *fnAddress, CFTypeRef withObj, Boolean waitUntilDone )
{
    CocoaObjectTarget *target = [[CocoaObjectTarget alloc] init];
    [target setCallback:fnAddress];
    [target performSelectorOnMainThread:@selector(objectSelector:) withObject:withObj waitUntilDone:waitUntilDone];
}

void ObjectCallFunctionInBackground( void *fnAddress, CFTypeRef withObj )
{
    CocoaObjectTarget *target = [[CocoaObjectTarget alloc] init];
    [target setCallback:fnAddress];
    [target performSelectorInBackground:@selector(objectSelector:) withObject:withObj];
}


CFTypeRef ObjectProperty( CFTypeRef obj, CFStringRef key )
{
    id property = nil;
    NSMutableDictionary *properties = [(id)obj objectProperties];
    if ( properties ) {
        property = [properties objectForKey:(NSString *)key];
    }
    return (CFTypeRef)property;
}

void ObjectSetProperty( CFTypeRef obj, CFStringRef key, CFTypeRef value )
{
    NSMutableDictionary *properties = [(id)obj objectProperties];
    if ( !properties ) {
        properties = [[NSMutableDictionary alloc] initWithCapacity:0];
        [(id)obj setObjectProperties:properties];
    }
    if ( value == NULL ) {
        [properties removeObjectForKey:(NSString *)key];
        if ( [properties count] == 0 ) {
            [properties release];
            [(id)obj setObjectProperties:nil];
        }
    } else {
        [properties setObject:(id)value forKey:(NSString *)key];
    }
}

void ObjectRemoveProperty( CFTypeRef obj, CFStringRef key )
{
    NSMutableDictionary *properties = [(id)obj objectProperties];
    if ( properties ) {
        [properties removeObjectForKey:(NSString *)key];
        if ( [properties count] == 0 ) {
            [properties release];
            [(id)obj setObjectProperties:nil];
        }
    }
}

void ObjectRemoveAllProperties( CFTypeRef obj )
{
    NSMutableDictionary *properties = [(id)obj objectProperties];
    if ( properties ) {
        [properties release];
        [(id)obj setObjectProperties:nil];
    }
}

