/*
 OpenPanel.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

static NSMutableArray *sCocoaOpenPanels;

// dialog events
enum {
    openPanelDidEnd = 2800
};

@interface NSOpenPanel (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
@property (nonatomic,retain) NSNumber *sheetNumber;

- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
- (BOOL)sheet;
- (void)setSheet:(BOOL)sheet;
@end

typedef void (*CocoaOpenSavePanelCallbackType)(NSInteger,void*);

NSOpenPanel *CocoaOpenPanelWithTag( NSInteger tag );
void CocoaOpenPanelAdd( NSOpenPanel *op );
void CocoaOpenPanelRemove( NSOpenPanel *op );


OpenPanelRef OpenPanelWithTag( NSInteger tag );
BOOL OpenPanelExists( NSInteger tag );

// Configure
Boolean OpenPanelCanChooseFiles( NSInteger tag );
void OpenPanelSetCanChooseFiles( NSInteger tag, Boolean flag );
Boolean OpenPanelCanChooseDirectories( NSInteger tag );
void OpenPanelSetCanChooseDirectories( NSInteger tag, Boolean flag );
Boolean OpenPanelResolvesAliases( NSInteger tag );
void OpenPanelSetResolvesAliases( NSInteger tag, Boolean flag );
Boolean OpenPanelAllowsMultipleSelection( NSInteger tag );
void OpenPanelSetAllowsMultipleSelection( NSInteger tag, Boolean flag );

// User selection
CFArrayRef OpenPanelURLs( NSInteger tag );

// Ubiquitous documents
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean OpenPanelCanDownloadUbiquitousContents( NSInteger tag );
void OpenPanelSetCanDownloadUbiquitousContents( NSInteger tag, Boolean flag );
Boolean OpenPanelCanResolveUbiquitousConflicts( NSInteger tag );
void OpenPanelSetCanResolveUbiquitousConflicts( NSInteger tag, Boolean flag );
#endif

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean OpenPanelIsAccessoryViewDisclosed( NSInteger tag );
void OpenPanelSetAccessoryViewDisclosed( NSInteger tag, Boolean flag );
#endif


// Convenience - SavePanel duplicates -
// Configure
ViewRef OpenPanelAccessoryView( NSInteger tag );
void OpenPanelSetAccessoryView( NSInteger tag, ViewRef ref );
CFStringRef OpenPanelTitle( NSInteger tag );
void OpenPanelSetTitle( NSInteger tag, CFStringRef title );
CFStringRef OpenPanelPrompt( NSInteger tag );
void OpenPanelSetPrompt( NSInteger tag, CFStringRef prompt );
CFStringRef OpenPanelMessage( NSInteger tag );
void OpenPanelSetMessage( NSInteger tag, CFStringRef message );
Boolean OpenPanelCanCreateDirectories( NSInteger tag );
void OpenPanelSetCanCreateDirectories( NSInteger tag, Boolean flag );
Boolean OpenPanelShowsHiddenFiles( NSInteger tag );
void OpenPanelSetShowsHiddenFiles( NSInteger tag, Boolean flag );

// Panel content
CFURLRef OpenPanelDirectoryURL( NSInteger tag );
void OpenPanelSetDirectoryURL( NSInteger tag, CFURLRef url );
CFArrayRef OpenPanelAllowedFileTypes( NSInteger tag );
void OpenPanelSetAllowedFileTypes( NSInteger tag, CFArrayRef fileTypes );
Boolean OpenPanelAllowsOtherFileTypes( NSInteger tag );
void OpenPanelSetAllowsOtherFileTypes( NSInteger tag, Boolean flag );
Boolean OpenPanelTreatsFilePackagesAsDirectories( NSInteger tag );
void OpenPanelSetTreatsFilePackagesAsDirectories( NSInteger tag, Boolean flag );

// Running
void OpenPanelBeginSheetModal( NSInteger tag, NSInteger wndTag, CocoaOpenSavePanelCallbackType callback, void *userData );
void OpenPanelBegin( NSInteger tag, CocoaOpenSavePanelCallbackType callback, void *userData );
NSInteger OpenPanelRunModal( NSInteger tag );
void OpenPanelValidateVisibleColumns( NSInteger tag );

// User selection
CFURLRef OpenPanelURL( NSInteger tag );

// Actions
void OpenPanelOK( NSInteger tag );
void OpenPanelCancel( NSInteger tag );

// Custom
void OpenPanelRemove( NSInteger tag );


//NSInteger CocoaOpenPanel( NSInteger inTag, CFStringRef inMsg, CFTypeRef inFileTypes, CFStringRef inPrompt, CFURLRef inDirURL, NSInteger inSheetFlag );

id CocoaOpenPanelSetDialogEventObjects( NSOpenPanel *op, NSInteger response );


// Init
OpenPanelRef OpenPanelInit( NSInteger inTag );

// statement/function
CFTypeRef CocoaOpenPanel( NSInteger inTag, CFStringRef inMsg, CFTypeRef inFileTypes, CFStringRef inPrompt, CFURLRef inDirURL, NSInteger inSheetFlag );

