/*
 OpenPanel.m

 Bernie Wylde
 */

#import "OpenPanel.h"

NSOpenPanel *CocoaOpenPanelWithTag( NSInteger tag )
{
    if ( tag ) {
        if ( tag > 1000000 ) return (NSOpenPanel *)tag;// it's an OpenPanelRef

        if ( sCocoaOpenPanels ) {
            for ( NSOpenPanel *op in sCocoaOpenPanels ) {
                if ( [op tag] == tag ) return op;
            }
        }
    }
    return nil;
}

void CocoaOpenPanelAdd( NSOpenPanel *op )
{
    if ( !sCocoaOpenPanels ) sCocoaOpenPanels = [[NSMutableArray alloc] initWithCapacity:0];
    [sCocoaOpenPanels addObject:op];
}

void CocoaOpenPanelRemove( NSOpenPanel *op )
{ [sCocoaOpenPanels removeObject:op]; }

@implementation NSOpenPanel (Additions)
@dynamic tagNumber;
@dynamic sheetNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (BOOL)sheet {
    return [objc_getAssociatedObject(self, @selector(sheetNumber)) boolValue];
}

- (void)setSheet:(BOOL)sheet {
    objc_setAssociatedObject(self, @selector(sheetNumber), [NSNumber numberWithBool:sheet], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

OpenPanelRef OpenPanelWithTag( NSInteger tag )
{ return CocoaOpenPanelWithTag( tag ); }

BOOL OpenPanelExists( NSInteger tag )
{
    if ( sCocoaOpenPanels ) {
        for ( NSOpenPanel *op in sCocoaOpenPanels ) {
            if ( [op tag] == tag ) return YES;
        }
    }
    return NO;
}



// Configure
Boolean OpenPanelCanChooseFiles( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op canChooseFiles];
    return NO;
}

void OpenPanelSetCanChooseFiles( NSInteger tag, Boolean flag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setCanChooseFiles:flag];
}

Boolean OpenPanelCanChooseDirectories( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op canChooseDirectories];
    return NO;
}


void OpenPanelSetCanChooseDirectories( NSInteger tag, Boolean flag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setCanChooseDirectories:flag];
}

Boolean OpenPanelResolvesAliases( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op resolvesAliases];
    return NO;
}

void OpenPanelSetResolvesAliases( NSInteger tag, Boolean flag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setResolvesAliases:flag];
}

Boolean OpenPanelAllowsMultipleSelection( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op allowsMultipleSelection];
    return NO;
}

void OpenPanelSetAllowsMultipleSelection( NSInteger tag, Boolean flag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setAllowsMultipleSelection:flag];
}

// User selection
CFArrayRef OpenPanelURLs( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return (CFArrayRef)[op URLs];
    return nil;
}

// Ubiquitous documents
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean OpenPanelCanDownloadUbiquitousContents( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op canDownloadUbiquitousContents];
    return NO;
}

void OpenPanelSetCanDownloadUbiquitousContents( NSInteger tag, Boolean flag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setCanDownloadUbiquitousContents:flag];
}

Boolean OpenPanelCanResolveUbiquitousConflicts( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op canResolveUbiquitousConflicts];
    return NO;
}

void OpenPanelSetCanResolveUbiquitousConflicts( NSInteger tag, Boolean flag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setCanResolveUbiquitousConflicts:flag];
}
#endif

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean OpenPanelIsAccessoryViewDisclosed( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op isAccessoryViewDisclosed];
    return NO;
}

void OpenPanelSetAccessoryViewDisclosed( NSInteger tag, Boolean flag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setAccessoryViewDisclosed:flag];
}
#endif



// Convenience - SavePanel functions -
// Configure
ViewRef OpenPanelAccessoryView( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op accessoryView];
    return nil;
}

void OpenPanelSetAccessoryView( NSInteger tag, ViewRef ref )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setAccessoryView:ref];
}

CFStringRef OpenPanelTitle( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return (CFStringRef)[op title];
    return nil;
}

void OpenPanelSetTitle( NSInteger tag, CFStringRef title )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setTitle:(NSString *)title];
}

CFStringRef OpenPanelPrompt( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return (CFStringRef)[op prompt];
    return nil;
}

void OpenPanelSetPrompt( NSInteger tag, CFStringRef prompt )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setPrompt:(NSString *)prompt];
}

CFStringRef OpenPanelMessage( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return (CFStringRef)[op message];
    return nil;
}

void OpenPanelSetMessage( NSInteger tag, CFStringRef message )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setMessage:(NSString *)message];
}

Boolean OpenPanelCanCreateDirectories( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op canCreateDirectories];
    return NO;
}

void OpenPanelSetCanCreateDirectories( NSInteger tag, Boolean flag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setCanCreateDirectories:flag];
}

Boolean OpenPanelShowsHiddenFiles( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op showsHiddenFiles];
    return NO;
}

void OpenPanelSetShowsHiddenFiles( NSInteger tag, Boolean flag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setShowsHiddenFiles:flag];
}

// Panel content
CFURLRef OpenPanelDirectoryURL( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return (CFURLRef)[op directoryURL];
    return nil;
}

void OpenPanelSetDirectoryURL( NSInteger tag, CFURLRef url )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setDirectoryURL:(NSURL *)url];
}

CFArrayRef OpenPanelAllowedFileTypes( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return (CFArrayRef)[op allowedFileTypes];
    return nil;
}

void OpenPanelSetAllowedFileTypes( NSInteger tag, CFArrayRef fileTypes )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setAllowedFileTypes:(NSArray *)fileTypes];
}

Boolean OpenPanelAllowsOtherFileTypes( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op allowsOtherFileTypes];
    return NO;
}

void OpenPanelSetAllowsOtherFileTypes( NSInteger tag, Boolean flag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setAllowsOtherFileTypes:flag];
}

Boolean OpenPanelTreatsFilePackagesAsDirectories( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op treatsFilePackagesAsDirectories];
    return NO;
}

void OpenPanelSetTreatsFilePackagesAsDirectories( NSInteger tag, Boolean flag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op setTreatsFilePackagesAsDirectories:flag];
}

// Running
void OpenPanelBeginSheetModal( NSInteger tag, NSInteger wndTag, CocoaOpenSavePanelCallbackType callback, void *userData )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) {
        NSWindow *window = WindowWithTag( wndTag );
        if ( window ) {
            [op beginSheetModalForWindow: window completionHandler:^(NSInteger returnCode) {
                if ( callback ) (*callback)(returnCode,userData);
            }];
        }
    }
}

void OpenPanelBegin( NSInteger tag, CocoaOpenSavePanelCallbackType callback, void *userData )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) {
        [op beginWithCompletionHandler:^(NSInteger returnCode) {
            if ( callback ) (*callback)(returnCode,userData);
        }];
    }
}

NSInteger OpenPanelRunModal( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return [op runModal];
    return 0;
}

void OpenPanelValidateVisibleColumns( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op validateVisibleColumns];
}

// User selection
CFURLRef OpenPanelURL( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) return (CFURLRef)[op URL];
    return nil;
}

// Actions
void OpenPanelOK( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op ok:nil];
}

void OpenPanelCancel( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) [op cancel:nil];
}

// -- custom --
void OpenPanelRemove( NSInteger tag )
{
    NSOpenPanel *op = CocoaOpenPanelWithTag( tag );
    if ( op ) {
        [sCocoaOpenPanels removeObject:op];
    }
}

//
//NSInteger CocoaOpenPanel( NSInteger inTag, CFStringRef inMsg, CFTypeRef inFileTypes, CFStringRef inPrompt, CFURLRef inDirURL, NSInteger inSheetFlag )
//{
//    NSInteger response = 0;
//
//    // tag
//    BOOL show = ( inTag > 0 );
//    inTag = ABS(inTag);
//
//    // msg
//    NSString *msg = (NSString *)inMsg;
//
//    // prompt
//    NSString *prompt = (NSString *)inPrompt;
//
//    // dirURL
//    NSURL *dirURL = (NSURL *)inDirURL;
//
//    NSOpenPanel *op = CocoaOpenPanelWithTag( inTag );
//
//    if ( op ) {     // exists
//
//        // sheet
//        //        if ( inSheetFlag != kFBParamMissing ) [op setSheet:inSheetFlag];
//
//    } else {
//
//        // init
//        op = [NSOpenPanel openPanel];
//
//        // sheet
//        //        if ( inSheetFlag != kFBParamMissing ) [op setSheet:inSheetFlag];
//
//        // tag
//        [op setTag:inTag];
//
//        // add to open panels array
//        CocoaOpenPanelAdd( op );
//
//    }
//
//    // msg
//    if ( msg ) [op setMessage:msg];
//
//    // allowed file types
//    if ( inFileTypes ) {
//        NSArray *fileTypes = nil;
//
//        if ( CFGetTypeID(inFileTypes) == CFStringGetTypeID() ) {
//            fileTypes = [(NSString *)inFileTypes componentsSeparatedByString:@";"];
//
//        } else if ( CFGetTypeID(inFileTypes) == CFArrayGetTypeID() ) {
//            fileTypes = (NSArray *)inFileTypes;
//        }
//
//        if ( fileTypes ) {
//            [op setAllowedFileTypes:fileTypes];
//        }
//    }
//
//    // prompt
//    if ( prompt ) [op setPrompt:prompt];
//
//    // dirURL
//    if ( dirURL ) [op setDirectoryURL:dirURL];
//
//    // show
//    if ( ![op isVisible] ) {
//
//        // sheet
//        if ( inSheetFlag != kFBParamMissing ) [op setSheet:inSheetFlag];
//
//        if ( show ) {
//
//            // get front window
//            NSInteger wndTag = 0;
//            //            NSWindow *window = CocoaTargetWindow( &wndTag );
//            NSWindow *window = CocoaVisibleTargetWindow( &wndTag );
//
//            if ( [op sheet] && window ) {  // sheet
//
//                // beginSheet
//                [op beginSheetModalForWindow:window completionHandler:^(NSInteger result) {
//                    CallUserDefinedOnDialogFunction( openPanelDidEnd, result, [op tag], NULL );
//                }];
//
//            } else {
//
//                // runModal
//                response = [op runModal];
//                CallUserDefinedOnDialogFunction( openPanelDidEnd, response, [op tag], NULL );
//
//            }
//
//        }
//    }
//
//    return response;
//}


id CocoaOpenPanelSetDialogEventObjects( NSOpenPanel *op, NSInteger response )
{
    id obj = nil;
    if ( response == NSOKButton ) {
        if ( [op allowsMultipleSelection] ) {
            obj = [op URLs];
            DialogEventSetNSArray( obj );
            DialogEventSetNSURL( [obj objectAtIndex:0] );
        } else {
            obj = [op URL];
            DialogEventSetNSArray( [op URLs] );
            DialogEventSetNSURL( obj );
        }
    } else {
        DialogEventSetNSArray( nil );
        DialogEventSetNSURL( nil );
    }
    return obj;
}


// Init
OpenPanelRef OpenPanelInit( NSInteger inTag )
{
    // init
    NSOpenPanel *op = [NSOpenPanel openPanel];

    // tag
    [op setTag:inTag];

    // add to open panels array
    CocoaOpenPanelAdd( op );

    return op;
}


// statement/function
CFTypeRef CocoaOpenPanel( NSInteger inTag, CFStringRef inMsg, CFTypeRef inFileTypes, CFStringRef inPrompt, CFURLRef inDirURL, NSInteger inSheetFlag )
{
    id obj = nil;
    
    // tag
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    // msg
    NSString *msg = (NSString *)inMsg;
    
    // prompt
    NSString *prompt = (NSString *)inPrompt;
    
    // dirURL
    NSURL *dirURL = (NSURL *)inDirURL;
    
    NSOpenPanel *op = CocoaOpenPanelWithTag( inTag );
    
    if ( op ) {     // exists
        
        // sheet
        //        if ( inSheetFlag != kFBParamMissing ) [op setSheet:inSheetFlag];
        
    } else {
        
        // init
        op = OpenPanelInit( inTag );
        
        // tag
//        [op setTag:inTag];
        
        // add to open panels array
//        CocoaOpenPanelAdd( op );
        
    }
    
    // msg
    if ( msg ) [op setMessage:msg];
    
    // allowed file types
    if ( inFileTypes ) {
        NSArray *fileTypes = nil;
        
        if ( CFGetTypeID(inFileTypes) == CFStringGetTypeID() ) {
            fileTypes = [(NSString *)inFileTypes componentsSeparatedByString:@";"];
            
        } else if ( CFGetTypeID(inFileTypes) == CFArrayGetTypeID() ) {
            fileTypes = (NSArray *)inFileTypes;
        }
        
        if ( fileTypes ) {
            [op setAllowedFileTypes:fileTypes];
        }
    }
    
    // prompt
    if ( prompt ) [op setPrompt:prompt];
    
    // dirURL
    if ( dirURL ) [op setDirectoryURL:dirURL];
    
    // show
    if ( ![op isVisible] ) {
        
        // sheet
        if ( inSheetFlag != kFBParamMissing ) [op setSheet:inSheetFlag];
        
        if ( show ) {
            
            // get front window
            NSInteger wndTag = 0;
            //            NSWindow *window = CocoaTargetWindow( &wndTag );
            NSWindow *window = CocoaVisibleTargetWindow( &wndTag );
            
            if ( [op sheet] && window ) {  // sheet
                
                // beginSheet
                [op beginSheetModalForWindow:window completionHandler:^(NSInteger result) {
                    
                    id obj = nil;

//                    if ( result == NSOKButton ) { // NSModalResponseOK
                        obj = CocoaOpenPanelSetDialogEventObjects( op, result );
//                    }
                    
                    CallUserDefinedOnDialogFunction( openPanelDidEnd, result, [op tag], (CFTypeRef)obj );
                    
                    CocoaOpenPanelRemove( op );

                }];
                
            } else {
                
                // runModal
                NSInteger response = [op runModal];
//                if ( response == NSOKButton ) { // NSModalResponseOK
                    
                    obj = CocoaOpenPanelSetDialogEventObjects( op, response );
                    
//                }
                CallUserDefinedOnDialogFunction( openPanelDidEnd, response, [op tag], (CFTypeRef)obj );
                
                CocoaOpenPanelRemove( op );
            }
        }
    }
    
    return (CFTypeRef)obj;
}
