/*
 Operation.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef void (*CocoaOperationFunctionCallbackType)(void);

// Init
OperationRef OperationInit( void );

// Executing
void OperationStart( OperationRef op );
void OperationMain( OperationRef op );
void OperationSetCompletionFunction( OperationRef op, CocoaOperationFunctionCallbackType fnAddress );

// Cancelling
void OperationCancel( OperationRef op );

// Status
Boolean OperationIsCancelled( OperationRef op );
Boolean OperationIsExecuting( OperationRef op );
Boolean OperationIsFinished( OperationRef op );
Boolean OperationIsConcurrent( OperationRef op );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean OperationIsAsynchronous( OperationRef op );
#endif

Boolean OperationIsReady( OperationRef op );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CFStringRef OperationName( OperationRef op );
void OperationSetName( OperationRef op, CFStringRef name );
#endif

// Dependencies
void OperationAddDependency( OperationRef op1, OperationRef op2 );
void OperationRemoveDependency( OperationRef op1, OperationRef op2 );
CFArrayRef OperationDependencies( OperationRef op );

// Execution priority
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSQualityOfService OperationQualityOfService( OperationRef op );
void OperationSetQualityOfService( OperationRef op, NSQualityOfService qos );
#endif

NSOperationQueuePriority OperationQueuePriority( OperationRef op );
void OperationSetQueuePriority( OperationRef op, NSOperationQueuePriority priority );

// Waiting
void OperationWaitUntilFinished( OperationRef op );
