/*
 OperationQueue.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef void (*CocoaOperationQueueFunctionCallbackType)(OperationQueueRef,CFTypeRef);

// Init
OperationQueueRef OperationQueueInit( void );

// Get specific queues
OperationQueueRef OperationQueueMain( void );
OperationQueueRef OperationQueueCurrent( void );

// Managing operations
void OperationQueueAddOperation( OperationQueueRef queue, OperationRef op );
void OperationQueueAddOperations( OperationQueueRef queue, CFArrayRef ops, Boolean waitUntilFinished );
void OperationQueueAddOperationWithFunction( OperationQueueRef queue, CocoaOperationQueueFunctionCallbackType fnAddress, CFTypeRef obj );
CFArrayRef OperationQueueOperations( OperationQueueRef queue );
NSUInteger OperationQueueOperationCount( OperationQueueRef queue );
void OperationQueueCancelAllOperations( OperationQueueRef queue );
void OperationQueueWaitUntilAllOperationsAreFinished( OperationQueueRef queue );

// Managing execution
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSQualityOfService OperationQueueQualityOfService( OperationQueueRef queue );
void OperationQueueSetQualityOfService( OperationQueueRef queue, NSQualityOfService qos );
#endif

NSInteger OperationQueueMaxConcurrentOperationCount( OperationQueueRef queue );
void OperationQueueSetMaxConcurrentOperationCount( OperationQueueRef queue, NSInteger count );

// Suspending
Boolean OperationQueueIsSuspended( OperationQueueRef queue );
void OperationQueueSetSuspended( OperationQueueRef queue, Boolean flag );

// Configure
CFStringRef OperationQueueName( OperationQueueRef queue );
void OperationQueueSetName( OperationQueueRef queue, CFStringRef name );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
dispatch_queue_t OperationQueueUnderlyingQueue( OperationQueueRef queue );
void OperationQueueSetUnderlyingQueue( OperationQueueRef queue, dispatch_queue_t dq );
#endif

