/*
 OperationQueue.m

 Bernie Wylde
 */

#import "OperationQueue.h"

// Init
OperationQueueRef OperationQueueInit( void )
{ return [[[NSOperationQueue alloc] init] autorelease]; }

// Get specific queues
OperationQueueRef OperationQueueMain( void )
{ return [NSOperationQueue mainQueue]; }

OperationQueueRef OperationQueueCurrent( void )
{ return [NSOperationQueue currentQueue]; }

// Managing operations
void OperationQueueAddOperation( OperationQueueRef queue, OperationRef op )
{ [queue addOperation:op]; }

void OperationQueueAddOperations( OperationQueueRef queue, CFArrayRef ops, Boolean waitUntilFinished )
{ [queue addOperations:(NSArray *)ops waitUntilFinished:waitUntilFinished]; }

void OperationQueueAddOperationWithFunction( OperationQueueRef queue, CocoaOperationQueueFunctionCallbackType fnAddress, CFTypeRef obj )
{
    [queue addOperationWithBlock:^(void) {
        (*fnAddress)( queue, obj );
    }];
}

CFArrayRef OperationQueueOperations( OperationQueueRef queue )
{ return (CFArrayRef)[queue operations]; }

NSUInteger OperationQueueOperationCount( OperationQueueRef queue )
{ return [queue operationCount]; }

void OperationQueueCancelAllOperations( OperationQueueRef queue )
{ [queue cancelAllOperations]; }

void OperationQueueWaitUntilAllOperationsAreFinished( OperationQueueRef queue )
{ [queue waitUntilAllOperationsAreFinished]; }

// Managing execution
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSQualityOfService OperationQueueQualityOfService( OperationQueueRef queue )
{ return [queue qualityOfService]; }

void OperationQueueSetQualityOfService( OperationQueueRef queue, NSQualityOfService qos )
{ [queue setQualityOfService:qos]; }
#endif

NSInteger OperationQueueMaxConcurrentOperationCount( OperationQueueRef queue )
{ return [queue maxConcurrentOperationCount]; }

void OperationQueueSetMaxConcurrentOperationCount( OperationQueueRef queue, NSInteger count )
{ [queue setMaxConcurrentOperationCount:count]; }

// Suspending
Boolean OperationQueueIsSuspended( OperationQueueRef queue )
{ return [queue isSuspended]; }

void OperationQueueSetSuspended( OperationQueueRef queue, Boolean flag )
{ [queue setSuspended:flag]; }

// Configure
CFStringRef OperationQueueName( OperationQueueRef queue )
{ return (CFStringRef)[queue name]; }

void OperationQueueSetName( OperationQueueRef queue, CFStringRef name )
{ return [queue setName:(NSString *)name]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
dispatch_queue_t OperationQueueUnderlyingQueue( OperationQueueRef queue )
{ return [queue underlyingQueue]; }

void OperationQueueSetUnderlyingQueue( OperationQueueRef queue, dispatch_queue_t dq )
{ [queue setUnderlyingQueue:dq]; }
#endif

