/*
 OrderedSet.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// - create -
OrderedSetRef OrderedSetWithArray( CFArrayRef array );
OrderedSetRef OrderedSetWithArrayRange( CFArrayRef array, CFRange range, Boolean copyItems );
OrderedSetRef OrderedSetWithObject( CFTypeRef obj );
OrderedSetRef OrderedSetWithObjects( CFTypeRef obj, ... );
OrderedSetRef OrderedSetWithSet( CFSetRef set );

// - init -

// - count -
NSUInteger OrderedSetCount( OrderedSetRef os );

// - accessing set members -
Boolean OrderedSetContainsObject( OrderedSetRef os, CFTypeRef obj );
// ...
CFTypeRef OrderedSetFirstObject( OrderedSetRef os );
CFTypeRef OrderedSetLastObject( OrderedSetRef os );
CFTypeRef OrderedSetObjectAtIndex( OrderedSetRef os, NSUInteger index );
CFArrayRef OrderedSetObjectsAtIndexes( OrderedSetRef os, IndexSetRef indexes );
NSUInteger OrderedSetIndexOfObject( OrderedSetRef os, CFTypeRef obj );

// - key-value coding -

// - key-value observing -

// - observing -
Boolean OrderedSetIsEqualToOrderedSet( OrderedSetRef os1, OrderedSetRef os2 );
Boolean OrderedSetIntersectsOrderedSet( OrderedSetRef os1, OrderedSetRef os2 );
Boolean OrderedSetIntersectsSet( OrderedSetRef os, CFSetRef set );
Boolean OrderedSetIsSubsetOfOrderedSet( OrderedSetRef os1, OrderedSetRef os2 );
Boolean OrderedSetIsSubsetOfSet( OrderedSetRef os, CFSetRef set );

// - sorted array -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFArrayRef OrderedSetSortedArrayUsingDescriptors( OrderedSetRef os, CFArrayRef descriptors );
#endif

// - filtering -

// - description -
CFStringRef OrderedSetDescription( OrderedSetRef os );

// - converting -
CFArrayRef OrderedSetArray( OrderedSetRef os );
CFSetRef OrderedSetSet( OrderedSetRef os );




// -- mutable ordered set --
// - create -
MutableOrderedSetRef OrderedSetWithCapacity( NSUInteger size );

// - add, remove, reorder -
void OrderedSetAddObject( MutableOrderedSetRef os, CFTypeRef obj );
void OrderedSetAddObjectsFromArray( MutableOrderedSetRef os, CFArrayRef array );
void OrderedSetInsertObjectAtIndex( MutableOrderedSetRef os, CFTypeRef obj, NSUInteger index );
void OrderedSetInsertObjectsAtIndexes( MutableOrderedSetRef os, CFArrayRef objects, IndexSetRef indexes );
void OrderedSetRemoveObject( MutableOrderedSetRef os, CFTypeRef obj );
void OrderedSetRemoveObjectAtIndex( MutableOrderedSetRef os, NSUInteger index );
void OrderedSetRemoveObjectsAtIndexes( MutableOrderedSetRef os, IndexSetRef indexes );
void OrderedSetRemoveObjectsInArray( MutableOrderedSetRef os, CFArrayRef array );
void OrderedSetRemoveObjectsInRange( MutableOrderedSetRef os, CFRange range );
void OrderedSetRemoveAllObjects( MutableOrderedSetRef os );
void OrderedSetReplaceObjectAtIndex( MutableOrderedSetRef os, CFTypeRef obj, NSUInteger index );
void OrderedSetReplaceObjectsAtIndexes( MutableOrderedSetRef os, CFArrayRef objects, IndexSetRef indexes );
void OrderedSetSetObjectAtIndex( MutableOrderedSetRef os, CFTypeRef obj, NSUInteger index );
void OrderedSetMoveObjectsAtIndexesToIndex( MutableOrderedSetRef os, IndexSetRef indexes, NSUInteger index );
void OrderedSetExchangeObjectAtIndex( MutableOrderedSetRef os, NSUInteger index1, NSUInteger index2 );

// - sorting -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void OrderedSetSortUsingDescriptors( MutableOrderedSetRef os, CFArrayRef descriptors );
#endif

// - combining and recombining -
void OrderedSetIntersectOrderedSet( MutableOrderedSetRef os1, OrderedSetRef os2 );
void OrderedSetIntersectSet( MutableOrderedSetRef os, CFSetRef set );
void OrderedSetMinusOrderedSet( MutableOrderedSetRef os1, OrderedSetRef os2 );
void OrderedSetMinusSet( MutableOrderedSetRef os, CFSetRef set );
void OrderedSetUnionOrderedSet( MutableOrderedSetRef os1, OrderedSetRef os2 );
void OrderedSetUnionSet( MutableOrderedSetRef os, CFSetRef set );

