/*
 OrderedSet.m

 Bernie Wylde
 */

#import "OrderedSet.h"

// - create -
OrderedSetRef OrderedSetWithArray( CFArrayRef array )
{ return [NSOrderedSet orderedSetWithArray:(NSArray *)array]; }

OrderedSetRef OrderedSetWithArrayRange( CFArrayRef array, CFRange range, Boolean copyItems )
{ return [NSOrderedSet orderedSetWithArray:(NSArray *)array range:NSMakeRange(range.location,range.length) copyItems:copyItems]; }

OrderedSetRef OrderedSetWithObject( CFTypeRef obj )
{ return [NSOrderedSet orderedSetWithObject:(id)obj]; }

OrderedSetRef OrderedSetWithObjects( CFTypeRef obj, ... )
{
    va_list ap;
    va_start( ap, obj );
    NSMutableOrderedSet *os = [NSMutableOrderedSet orderedSetWithCapacity:0];
    while ( obj ) {
        [os addObject:obj];
        obj = va_arg( ap, id );
    }
    va_end( ap );
    return (OrderedSetRef)os;
}

OrderedSetRef OrderedSetWithSet( CFSetRef set )
{ return [NSOrderedSet orderedSetWithSet:(NSSet *)set]; }

// - init -

// - count -
NSUInteger OrderedSetCount( OrderedSetRef os )
{ return [os count]; }

// - accessing set members -
Boolean OrderedSetContainsObject( OrderedSetRef os, CFTypeRef obj )
{ return [os containsObject:(id)obj]; }

CFTypeRef OrderedSetFirstObject( OrderedSetRef os )
{ return (CFTypeRef)[os firstObject]; }

CFTypeRef OrderedSetLastObject( OrderedSetRef os )
{ return (CFTypeRef)[os lastObject]; }

CFTypeRef OrderedSetObjectAtIndex( OrderedSetRef os, NSUInteger index )
{ return (CFTypeRef)[os objectAtIndex:index]; }

CFArrayRef OrderedSetObjectsAtIndexes( OrderedSetRef os, IndexSetRef indexes )
{ return (CFTypeRef)[os objectsAtIndexes:indexes]; }

NSUInteger OrderedSetIndexOfObject( OrderedSetRef os, CFTypeRef obj )
{ return [os indexOfObject:(id)obj]; }

// - key-value coding -

// - key-value observing -

// - observing -
Boolean OrderedSetIsEqualToOrderedSet( OrderedSetRef os1, OrderedSetRef os2 )
{ return [os1 isEqualToOrderedSet:os2]; }

Boolean OrderedSetIntersectsOrderedSet( OrderedSetRef os1, OrderedSetRef os2 )
{ return [os1 intersectsOrderedSet:os2]; }

Boolean OrderedSetIntersectsSet( OrderedSetRef os, CFSetRef set )
{ return [os intersectsSet:(NSSet *)set]; }

Boolean OrderedSetIsSubsetOfOrderedSet( OrderedSetRef os1, OrderedSetRef os2 )
{ return [os1 isSubsetOfOrderedSet:os2]; }

Boolean OrderedSetIsSubsetOfSet( OrderedSetRef os, CFSetRef set )
{ return [os isSubsetOfSet:(NSSet *)set]; }

// - sorted array -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFArrayRef OrderedSetSortedArrayUsingDescriptors( OrderedSetRef os, CFArrayRef descriptors )
{ return (CFArrayRef)[os sortedArrayUsingDescriptors:(NSArray *)descriptors]; }
#endif

// - filtering -

// - description -
CFStringRef OrderedSetDescription( OrderedSetRef os )
{ return (CFStringRef)[os description]; }

// ...

// - converting -
CFArrayRef OrderedSetArray( OrderedSetRef os )
{ return (CFArrayRef)[os array]; }

CFSetRef OrderedSetSet( OrderedSetRef os )
{ return (CFSetRef)[os set]; }



// -- mutable ordered set --
// - create -
MutableOrderedSetRef OrderedSetWithCapacity( NSUInteger size )
{ return [NSMutableOrderedSet orderedSetWithCapacity:size]; }

// - add, remove, reorder -
void OrderedSetAddObject( MutableOrderedSetRef os, CFTypeRef obj )
{ [os addObject:(id)obj]; }

void OrderedSetAddObjectsFromArray( MutableOrderedSetRef os, CFArrayRef array )
{ [os addObjectsFromArray:(NSArray *)array]; }

void OrderedSetInsertObjectAtIndex( MutableOrderedSetRef os, CFTypeRef obj, NSUInteger index )
{ [os insertObject:(id)obj atIndex:index]; }

void OrderedSetInsertObjectsAtIndexes( MutableOrderedSetRef os, CFArrayRef objects, IndexSetRef indexes )
{ [os insertObjects:(NSArray *)objects atIndexes:indexes]; }

void OrderedSetRemoveObject( MutableOrderedSetRef os, CFTypeRef obj )
{ [os removeObject:(id)obj]; }

void OrderedSetRemoveObjectAtIndex( MutableOrderedSetRef os, NSUInteger index )
{ [os removeObjectAtIndex:index]; }

void OrderedSetRemoveObjectsAtIndexes( MutableOrderedSetRef os, IndexSetRef indexes )
{ [os removeObjectsAtIndexes:indexes]; }

void OrderedSetRemoveObjectsInArray( MutableOrderedSetRef os, CFArrayRef array )
{ [os removeObjectsInArray:(NSArray *)array]; }

void OrderedSetRemoveObjectsInRange( MutableOrderedSetRef os, CFRange range )
{ [os removeObjectsInRange:NSMakeRange(range.location,range.length)]; }

void OrderedSetRemoveAllObjects( MutableOrderedSetRef os )
{ [os removeAllObjects]; }

void OrderedSetReplaceObjectAtIndex( MutableOrderedSetRef os, CFTypeRef obj, NSUInteger index )
{ [os replaceObjectAtIndex:index withObject:(id)obj]; }

void OrderedSetReplaceObjectsAtIndexes( MutableOrderedSetRef os, CFArrayRef objects, IndexSetRef indexes )
{ [os replaceObjectsAtIndexes:indexes withObjects:(NSArray *)objects]; }

void OrderedSetSetObjectAtIndex( MutableOrderedSetRef os, CFTypeRef obj, NSUInteger index )
{ [os setObject:(id)obj atIndex:index]; }

void OrderedSetMoveObjectsAtIndexesToIndex( MutableOrderedSetRef os, IndexSetRef indexes, NSUInteger index )
{ [os moveObjectsAtIndexes:indexes toIndex:index]; }

void OrderedSetExchangeObjectAtIndex( MutableOrderedSetRef os, NSUInteger index1, NSUInteger index2 )
{ [os exchangeObjectAtIndex:index1 withObjectAtIndex:index2]; }

// - sorting -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void OrderedSetSortUsingDescriptors( MutableOrderedSetRef os, CFArrayRef descriptors )
{ [os sortUsingDescriptors:(NSArray *)descriptors]; }
#endif

// - combining and recombining -
void OrderedSetIntersectOrderedSet( MutableOrderedSetRef os1, OrderedSetRef os2 )
{ [os1 intersectOrderedSet:os2]; }

void OrderedSetIntersectSet( MutableOrderedSetRef os, CFSetRef set )
{ [os intersectSet:(NSSet *)set]; }

void OrderedSetMinusOrderedSet( MutableOrderedSetRef os1, OrderedSetRef os2 )
{ [os1 minusOrderedSet:os2]; }

void OrderedSetMinusSet( MutableOrderedSetRef os, CFSetRef set )
{ [os minusSet:(NSSet *)set]; }

void OrderedSetUnionOrderedSet( MutableOrderedSetRef os1, OrderedSetRef os2 )
{ [os1 unionOrderedSet:os2]; }

void OrderedSetUnionSet( MutableOrderedSetRef os, CFSetRef set )
{ [os unionSet:(NSSet *)set]; }

