/*
 OutlineView.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

#define OUTLINEVIEW_REORDER_PASTEBOARD_TYPE @"com.fb.outlineview.reorder.item"

#pragma mark - outlineitem
///////////////////////////////////////////////////
//                  outlineitem                  //
///////////////////////////////////////////////////
@interface OutlineItem : NSObject {
    NSString *string;
    NSImage *image;
    BOOL isGroup;
    NSMutableArray *children;
    NSDictionary *columns;
    NSMutableDictionary *attributes;
}

@property (assign) NSString *string;
@property (assign) NSImage *image;
@property (assign) BOOL isGroup;
@property (assign) NSMutableArray *children;
@property (assign) NSDictionary *columns;
@property (assign) NSMutableDictionary *attributes;

- (id)initWithString:(NSString *)inString image:(NSImage *)inImage isGroup:(BOOL)inIsGroup;
- (id)initWithColumns:(NSDictionary *)inColumns isGroup:(BOOL)inIsGroup;
@end

// Create
OutlineItem *OutlineItemWithString( CFStringRef string, ImageRef image, Boolean isGroup );
OutlineItem *OutlineItemWithColumns( CFDictionaryRef columns, Boolean isGroup );

// Info
CFStringRef OutlineItemString( OutlineItemRef item );
ImageRef OutlineItemImage( OutlineItemRef item );
Boolean OutlineItemIsGroup( OutlineItemRef item );
CFMutableArrayRef OutlineItemChildren( OutlineItemRef item );
CFDictionaryRef OutlineItemColumns( OutlineItemRef item );

// Add
OutlineItem *OutlineItemAddChildWithString( OutlineItemRef parItem, CFStringRef string, ImageRef image, Boolean isGroup );
void OutlineItemAddChild( OutlineItemRef parItem, OutlineItemRef childItem );
void OutlineItemAddChildColumns( OutlineItemRef parItem, CFDictionaryRef columns, Boolean isGroup );

// Insert
OutlineItemRef OutlineItemInsertChildWithString( OutlineItemRef parItem, CFStringRef string, ImageRef image, Boolean isGroup, NSUInteger index );
void OutlineItemInsertChild( OutlineItemRef parItem, OutlineItemRef childItem, NSUInteger index );
void OutlineItemInsertChildColumns( OutlineItemRef parItem, CFDictionaryRef columns, Boolean isGroup, NSUInteger index );

// Remove
void OutlineItemRemoveChildAtIndex( OutlineItemRef parItem, NSUInteger index );
void OutlineItemRemoveChild( OutlineItemRef parItem, OutlineItemRef childItem );



#pragma mark - outlineview
///////////////////////////////////////////////////
//                  outlineview                  //
///////////////////////////////////////////////////
@interface OutlineViewSC : NSOutlineView
@end

typedef void (*CocoaOutlineViewCallbackType)(NSInteger,NSInteger);

@interface NSOutlineView (Additions)
@property (nonatomic,retain) NSValue *callbackValue;
- (CocoaOutlineViewCallbackType)cocoaOutlineViewCallback;
- (void)setCocoaOutlineViewCallback:(CocoaOutlineViewCallbackType)cb;
@end


/*
 dialog events
 */
enum {
    // delegate
    outlineViewShouldExpandItem = 3900,
    outlineViewShouldCollapseItem,
    outlineViewTypeSelectStringForTableColumn,
    outlineViewNextTypeSelectMatchFromItem,
    outlineViewShouldTypeSelectForEvent,
    outlineViewToolTipForCell,
    outlineViewShouldSelectTableColumn,
    outlineViewShouldSelectItem,
    outlineViewSelectionIndexesForProposedSelection,
    outlineViewSelectionShouldChange,
    outlineViewSelectionIsChanging,
    outlineViewSelectionDidChange,
    outlineViewWillDisplayCell,
    outlineViewWillDisplayOutlineCell,
    outlineViewDataCellForTableColumn,
    outlineViewShouldShowOutlineCellForItem,
    outlineViewShouldShowCellExpansionForTableColumn,
    outlineViewShouldReorderColumn,
    outlineViewColumnDidMove,
    outlineViewColumnDidResize,
    outlineViewItemWillExpand,
    outlineViewItemDidExpand,
    outlineViewItemWillCollapse,
    outlineViewItemDidCollapse,
    outlineViewShouldEditTableColumn,
    outlineViewMouseDownInHeaderOfTableColumn,
    outlineViewDidClickTableColumn,
    outlineViewDidDragTableColumn,
    outlineViewHeightOfRowByItem,
    outlineViewSizeToFitWidthOfColumn,
    outlineViewShouldTrackCell,
    outlineViewIsGroupItem,
    outlineViewDidAddRowView,
    outlineViewDidRemoveRowView,
    outlineViewRowViewForItem,
    outlineViewViewForTableColumn,
    
    // data source
    outlineViewAcceptDrop,
    outlineViewChildOfItem,
    outlineViewDraggingSessionEnded,
    outlineViewDraggingSessionWillBegin,
    outlineViewIsItemExpandable,
    outlineViewItemForPersistentObject,
    outlineViewNumberOfChildrenOfItem,
    outlineViewObjectValueForTableColumn,
    outlineViewPasteboardWriterForItem,
    outlineViewPersistentObjectForItem,
    outlineViewSetObjectValueForTableColumn,
    outlineViewSortDescriptorsDidChange,
    outlineViewUpdateDraggingItemsForDrag,
    outlineViewValidateDrop,
    outlineViewWriteItemsToPasteboard,
    
    // other actions
    outlineViewDoubleAction = tableViewDoubleAction
};


@interface CocoaOutlineViewDelegate : NSObject <NSOutlineViewDelegate,NSOutlineViewDataSource,NSTextFieldDelegate,NSPasteboardItemDataProvider> {
    NSOutlineView *table;
    NSMutableArray *tableData;
    OutlineItem *draggedItem;
}
@property (assign) NSOutlineView *table;
@property (assign) NSMutableArray *tableData;
@property (assign) OutlineItem *draggedItem;

- (void)doubleAction:(id)sender;
- (void)singleAction:(id)sender;
@end

NSOutlineView *OutlineViewWithTag( NSInteger tag );
BOOL OutlineViewExists( NSInteger tag );

// Data source
CFTypeRef OutlineViewDataSource( NSInteger tag );
void OutlineViewSetDataSource( NSInteger tag, CFTypeRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean OutlineViewStronglyReferencesItems( NSInteger tag );
void OutlineViewSetStronglyReferencesItems( NSInteger tag, Boolean flag );
#endif// 101200

// -- expandability --
Boolean OutlineViewIsItemExpandable( NSInteger tag, OutlineItemRef item );
Boolean OutlineViewIsItemExpanded( NSInteger tag, OutlineItemRef item );

// -- expand/collapse --
void OutlineViewExpandItem( NSInteger tag, OutlineItemRef item );
void OutlineViewExpandItemAndChildren( NSInteger tag, OutlineItemRef item, Boolean expandChildren );
void OutlineViewCollapseItem( NSInteger tag, OutlineItemRef item );
void OutlineViewCollapseItemAndChildren( NSInteger tag, OutlineItemRef item, Boolean collapseChildren );

// -- redisplay --
void OutlineViewReloadItem( NSInteger tag, OutlineItemRef item );
void OutlineViewReloadItemAndChildren( NSInteger tag, OutlineItemRef item, Boolean reloadChildren );

// -- convert between items and rows --
OutlineItemRef OutlineViewItemAtRow( NSInteger tag, NSInteger row );
NSInteger OutlineViewRowForItem( NSInteger tag, OutlineItemRef item );

// Outline column
TableColumnRef OutlineViewOutlineTableColumn( NSInteger tag );
void OutlineViewSetOutlineTableColumn( NSInteger tag, TableColumnRef columnRef );
Boolean OutlineViewAutoresizesOutlineColumn( NSInteger tag );
void OutlineViewSetAutoresizesOutlineColumn( NSInteger tag, Boolean flag );

// -- indentation --
NSInteger OutlineViewLevelForItem( NSInteger tag, OutlineItemRef item );
NSInteger OutlineViewLevelForRow( NSInteger tag, NSInteger row );
CGFloat OutlineViewIndentationPerLevel( NSInteger tag );
void OutlineViewSetIndentationPerLevel( NSInteger tag, CGFloat indent );
Boolean OutlineViewIndentationMarkerFollowsCell( NSInteger tag );
void OutlineViewSetIndentationMarkerFollowsCell( NSInteger tag, Boolean flag );

// Persistence
Boolean OutlineViewAutosaveExpandedItems( NSInteger tag );
void OutlineViewSetAutosaveExpandedItems( NSInteger tag, Boolean flag );

// Drag drop
void OutlineViewSetDropItem( NSInteger tag, OutlineItemRef item, NSInteger dropChildIndex );

// -- get related items --
OutlineItemRef OutlineViewParentForItem( NSInteger tag, OutlineItemRef item );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSInteger OutlineViewChildIndexForItem( NSInteger tag, OutlineItemRef item );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
OutlineItemRef OutlineViewChildOfItem( NSInteger tag, NSInteger index, OutlineItemRef item );
NSInteger OutlineViewNumberOfChildrenOfItem( NSInteger tag, OutlineItemRef item );
#endif

// -- cell frame --

// Delegate
CFTypeRef OutlineViewDelegate( NSInteger tag );
void OutlineViewSetDelegate( NSInteger tag, CFTypeRef ref );

// -- manipulate items --

// User interface layout direction
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSUserInterfaceLayoutDirection OutlineViewUserInterfaceLayoutDirection( NSInteger tag );
void OutlineViewSetUserInterfaceLayoutDirection( NSInteger tag, NSUserInterfaceLayoutDirection direction );
#endif// 1090

// Custom
CFMutableArrayRef OutlineViewData( NSInteger tag );
void OutlineViewSetData( NSInteger tag, CFMutableArrayRef array );
void OutlineViewReloadData( NSInteger tag );

