/*
 OutlineView.m

 Bernie Wylde
 */

#import "OutlineView.h"

#pragma mark - OutlineItem
///////////////////////////////////////////////////
//                  outlineitem                  //
///////////////////////////////////////////////////
@implementation OutlineItem

@synthesize string;
@synthesize image;
@synthesize isGroup;
@synthesize children;
@synthesize columns;
@synthesize attributes;

- (id)initWithString:(NSString *)inString image:(NSImage *)inImage isGroup:(BOOL)inIsGroup {
    if ( self = [super init] ) {
        string = [inString copy];
        image = [inImage copy];
        isGroup = inIsGroup;
        children = [[NSMutableArray alloc] initWithCapacity:0];
    }
    return self;
}

- (id)initWithColumns:(NSMutableDictionary *)inColumns isGroup:(BOOL)inIsGroup {
    if ( self = [super init] ) {
        columns = [inColumns copy];
        isGroup = inIsGroup;
        children = [[NSMutableArray alloc] initWithCapacity:0];
    }
    return self;
}

@end

// Create
OutlineItem *OutlineItemWithString( CFStringRef string, ImageRef image, Boolean isGroup )
{ return [[[OutlineItem alloc] initWithString:(NSString *)string image:image isGroup:isGroup] autorelease]; }

OutlineItem *OutlineItemWithColumns( CFDictionaryRef columns, Boolean isGroup )
{ return [[[OutlineItem alloc] initWithColumns:(NSDictionary *)columns isGroup:isGroup] autorelease]; }

// Info
CFStringRef OutlineItemString( OutlineItemRef item )
{ return (CFStringRef)[item string]; }

ImageRef OutlineItemImage( OutlineItemRef item )
{ return [item image]; }

Boolean OutlineItemIsGroup( OutlineItemRef item )
{ return [item isGroup]; }

CFMutableArrayRef OutlineItemChildren( OutlineItemRef item )
{ return (CFMutableArrayRef)[item children]; }

CFDictionaryRef OutlineItemColumns( OutlineItemRef item )
{ return (CFDictionaryRef)[item columns]; }

// Add
OutlineItem *OutlineItemAddChildWithString( OutlineItemRef parItem, CFStringRef string, ImageRef image, Boolean isGroup )
{
    OutlineItem *childItem = OutlineItemWithString( string, image, isGroup );
    [[parItem children] addObject:childItem];
    return childItem;
}

void OutlineItemAddChild( OutlineItemRef parItem, OutlineItemRef childItem )
{ [[parItem children] addObject:childItem]; }

void OutlineItemAddChildColumns( OutlineItemRef parItem, CFDictionaryRef columns, Boolean isGroup )
{
    OutlineItem *childItem = OutlineItemWithColumns( columns, isGroup );
    [[parItem children] addObject:childItem];
}

// Insert
OutlineItemRef OutlineItemInsertChildWithString( OutlineItemRef parItem, CFStringRef string, ImageRef image, Boolean isGroup, NSUInteger index )
{
    OutlineItem *childItem = OutlineItemWithString( string, image, isGroup );
    [[parItem children] insertObject:childItem atIndex:index];
    return childItem;
}

// Remove
void OutlineItemInsertChild( OutlineItemRef parItem, OutlineItemRef childItem, NSUInteger index )
{ [[parItem children] insertObject:childItem atIndex:index]; }

void OutlineItemInsertChildColumns( OutlineItemRef parItem, CFDictionaryRef columns, Boolean isGroup, NSUInteger index )
{
    OutlineItem *childItem = OutlineItemWithColumns( columns, isGroup );
    [[parItem children] insertObject:childItem atIndex:index];
}




#pragma mark - outlineview
///////////////////////////////////////////////////
//                   outlineview                 //
///////////////////////////////////////////////////
@implementation OutlineViewSC
@end

@implementation CocoaOutlineViewDelegate

@synthesize table;
@synthesize tableData;
@synthesize draggedItem;

- (id)init {
    if ( self = [super init] ) {
        self.tableData = [[NSMutableArray alloc] initWithCapacity:0];
    }
    return self;
}

- (void)outlineView:(NSOutlineView *)ov forwardEvent:(NSInteger)event object:(id)obj {
    if ( [ov cocoaOutlineViewCallback] ) {
        CocoaOutlineViewCallbackType callback = [ov cocoaOutlineViewCallback];
        (*callback)([ov tag],event);
    } else {
        CallUserDefinedOnDialogFunction( event, [ov tag], [ov wndTag], obj );
    }
}


// delegate methods
- (NSView *)outlineView:(NSOutlineView *)outlineView viewForTableColumn:(NSTableColumn *)tableColumn item:(id)item {
    
    NSString *identifier = [tableColumn identifier];
    NSTableCellView *cellView = [outlineView makeViewWithIdentifier:identifier owner:self];
    
    if ( item ) {
        if ( cellView ) {
            if ( cellView.textField ) {
                if ( [cellView.textField isEditable] ) {
                    [cellView.textField setDelegate:self];
                }
            }
            
            NSString *string = [item string];
            NSImage *image = [item image];
            
            NSDictionary *columns = [item columns];
            if ( columns ) {
                OutlineItem *colItem = [columns objectForKey:identifier];
                string = [colItem string];
                image = [colItem image];
            }
            
            if ( cellView.textField ) {
                if ( !string ) string = @" ";
                cellView.textField.stringValue = string;
            }
            if ( image ) cellView.imageView.image = image;
        }
    }
    
    return cellView;
}


- (void)outlineViewSelectionDidChange:(NSNotification *)notification {
    CocoaOutputWindowSetTag( [notification.object wndTag] );
    [self outlineView:notification.object forwardEvent:outlineViewSelectionDidChange object:nil];
}


// data source methods
- (BOOL)outlineView:(NSOutlineView *)outlineView acceptDrop:(id<NSDraggingInfo>)info item:(id)item childIndex:(NSInteger)index {
    DialogEventSetBool(YES);
    DialogEventSetLong(index);
    
    if ( draggedItem == nil ) {    // Finder drop
        NSPasteboard *pb = info.draggingPasteboard;
        NSDictionary *options = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] forKey:NSPasteboardURLReadingFileURLsOnlyKey];
        NSArray *urls = [pb readObjectsForClasses:[NSArray arrayWithObject:[NSURL class]] options:options];
        DialogEventSetNSArray( urls );
        DialogEventSetNSURL( [urls objectAtIndex:0] );
        
        [self outlineView:outlineView forwardEvent:outlineViewAcceptDrop object:item];

    } else {    // drag within outline view
        
        DialogEventSetNSArray( nil );
        DialogEventSetNSURL( nil );

#if 1
        
        OutlineItem *srcItem = draggedItem;
        OutlineItem *destItem = item;
        
        NSInteger oldIndex;// = [outlineView childIndexForItem:srcItem];
        
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
        oldIndex = [outlineView childIndexForItem:srcItem];
#else
        if ( [srcItem isGroup] ) {
            oldIndex = [tableData indexOfObject:srcItem];
        } else {
            OutlineItem *parItem = [outlineView parentForItem:srcItem];
            NSArray *children = [parItem children];
            oldIndex = [children indexOfObject:srcItem];
        }
#endif

        NSInteger newIndex = index;
        NSInteger destGroupIndex = [tableData indexOfObject:destItem];
        
        BOOL srcItemIsGroup = [srcItem isGroup];
        BOOL destItemIsGroup = [destItem isGroup];
        
        if ( srcItemIsGroup && destItemIsGroup ) {
            
            [outlineView moveItemAtIndex:oldIndex inParent:nil toIndex:destGroupIndex inParent:nil];
            
            if ( destGroupIndex < oldIndex ) {
                [tableData insertObject:srcItem atIndex:destGroupIndex];
                [tableData removeObjectAtIndex:oldIndex+1];
            } else {
                [tableData insertObject:srcItem atIndex:destGroupIndex+1];
                [tableData removeObjectAtIndex:oldIndex];
            }
            return YES;
        }
        
        if ( srcItemIsGroup ) {
            
            [outlineView moveItemAtIndex:oldIndex inParent:nil toIndex:newIndex inParent:nil];
            
            if ( newIndex < oldIndex ) {
                [tableData insertObject:srcItem atIndex:newIndex];
                [tableData removeObjectAtIndex:oldIndex+1];
            } else {
                [tableData insertObject:srcItem atIndex:newIndex];
                [tableData removeObjectAtIndex:oldIndex];
            }
            return YES;
        } else {
            
            if ( destGroupIndex != NSNotFound ) {
                
                OutlineItem *srcGroupItem = [outlineView parentForItem:srcItem];
                NSInteger srcGroupIndex = [tableData indexOfObject:srcGroupItem];
                
                NSMutableArray *srcChildren = [srcGroupItem children];
                NSMutableArray *destChildren = [destItem children];
                
                if ( newIndex < 0 ) newIndex = 0;
                
                if ( srcGroupIndex == destGroupIndex ) {
                    if ( newIndex != oldIndex ) {
                        if ( newIndex < oldIndex ) {
                            [outlineView moveItemAtIndex:oldIndex inParent:srcGroupItem toIndex:newIndex inParent:srcGroupItem];
                            [srcChildren insertObject:srcItem atIndex:newIndex];
                            [srcChildren removeObjectAtIndex:oldIndex+1];
                        } else {
                            [outlineView moveItemAtIndex:oldIndex inParent:srcGroupItem toIndex:newIndex-1 inParent:srcGroupItem];
                            [srcChildren insertObject:srcItem atIndex:newIndex];
                            [srcChildren removeObjectAtIndex:oldIndex];
                        }
                    }
                } else {
                    [outlineView moveItemAtIndex:oldIndex inParent:srcGroupItem toIndex:newIndex inParent:destItem];
                    [destChildren insertObject:srcItem atIndex:newIndex];
                    [srcChildren removeObject:srcItem];
                }
                
            } else {
                
                OutlineItem *srcGroupItem = [outlineView parentForItem:srcItem];
                OutlineItem *destGroupItem = [outlineView parentForItem:destItem];
                
                NSInteger srcGroupIndex = [tableData indexOfObject:[outlineView parentForItem:srcItem]];
                destGroupIndex = [tableData indexOfObject:[outlineView parentForItem:destItem]];
                
                NSMutableArray *srcChildren = [srcGroupItem children];
                NSMutableArray *destChildren = [destGroupItem children];
                
                if ( newIndex < 0 ) {
                    newIndex = [destChildren indexOfObject:destItem];
                }
                
                if ( srcGroupIndex == destGroupIndex ) {
                    [outlineView moveItemAtIndex:oldIndex inParent:srcGroupItem toIndex:newIndex inParent:destGroupItem];
                    if ( newIndex != oldIndex ) {
                        if ( newIndex < oldIndex ) {
                            [srcChildren insertObject:srcItem atIndex:newIndex];
                            [srcChildren removeObjectAtIndex:oldIndex+1];
                        } else {
                            [srcChildren insertObject:srcItem atIndex:newIndex+1];
                            [srcChildren removeObjectAtIndex:oldIndex];
                        }
                    }
                } else {
                    [outlineView moveItemAtIndex:oldIndex inParent:srcGroupItem toIndex:newIndex inParent:destGroupItem];
                    [destChildren insertObject:srcItem atIndex:newIndex];
                    [srcChildren removeObject:srcItem];
                }
                
            }
        }
        
        
#else
        
        DialogEventSetNSArray( nil );
        DialogEventSetNSURL( nil );

        OutlineItem *srcItem = draggedItem;
        OutlineItem *destItem = item;
        
        NSInteger oldIndex;
        
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
        oldIndex = [outlineView childIndexForItem:srcItem];
#else
        if ( [srcItem isGroup] ) {
            oldIndex = [tableData indexOfObject:srcItem];
        } else {
            OutlineItem *parItem = [outlineView parentForItem];
            NSArray *children = [parItem children];
            oldIndex = [children indexOfObject:srcItem];
        }
#endif
        
        NSInteger newIndex = index;
        NSInteger destGroupIndex = [tableData indexOfObject:destItem];
        
        BOOL srcItemIsGroup = [srcItem isGroup];
        BOOL destItemIsGroup = [destItem isGroup];
        
        if ( srcItemIsGroup && destItemIsGroup ) {
            
            [outlineView moveItemAtIndex:oldIndex inParent:nil toIndex:destGroupIndex inParent:nil];
            
            if ( destGroupIndex < oldIndex ) {
                [tableData insertObject:srcItem atIndex:destGroupIndex];
                [tableData removeObjectAtIndex:oldIndex+1];
            } else {
                [tableData insertObject:srcItem atIndex:destGroupIndex+1];
                [tableData removeObjectAtIndex:oldIndex];
            }
            return YES;
        }
        
        if ( srcItemIsGroup ) {
            [outlineView moveItemAtIndex:oldIndex inParent:nil toIndex:newIndex inParent:nil];
            
            if ( newIndex < oldIndex ) {
                [tableData insertObject:srcItem atIndex:newIndex];
                [tableData removeObjectAtIndex:oldIndex+1];
            } else {
                [tableData insertObject:srcItem atIndex:newIndex];
                [tableData removeObjectAtIndex:oldIndex];
            }
            return YES;
        } else {
            
            if ( destGroupIndex != NSNotFound ) {
                
                OutlineItem *srcGroupItem = [outlineView parentForItem:srcItem];
                NSInteger srcGroupIndex = [tableData indexOfObject:srcGroupItem];
                
                NSMutableArray *srcChildren = [srcGroupItem children];
                NSMutableArray *destChildren = [destItem children];
                
                if ( newIndex < 0 ) newIndex = 0;
                
                if ( srcGroupIndex == destGroupIndex ) {
                    if ( newIndex != oldIndex ) {
                        if ( newIndex < oldIndex ) {
                            [outlineView moveItemAtIndex:oldIndex inParent:srcGroupItem toIndex:newIndex inParent:srcGroupItem];
                            [srcChildren insertObject:srcItem atIndex:newIndex];
                            [srcChildren removeObjectAtIndex:oldIndex+1];
                        } else {
                            [outlineView moveItemAtIndex:oldIndex inParent:srcGroupItem toIndex:newIndex-1 inParent:srcGroupItem];
                            [srcChildren insertObject:srcItem atIndex:newIndex];
                            [srcChildren removeObjectAtIndex:oldIndex];
                        }
                    }
                } else {
                    [outlineView moveItemAtIndex:oldIndex inParent:srcGroupItem toIndex:newIndex inParent:destItem];
                    [destChildren insertObject:srcItem atIndex:newIndex];
                    [srcChildren removeObject:srcItem];
                }
                
            } else {
                
                OutlineItem *srcGroupItem = [outlineView parentForItem:srcItem];
                OutlineItem *destGroupItem = [outlineView parentForItem:destItem];
                
                NSInteger srcGroupIndex = [tableData indexOfObject:[outlineView parentForItem:srcItem]];
                destGroupIndex = [tableData indexOfObject:[outlineView parentForItem:destItem]];
                
                NSMutableArray *srcChildren = [srcGroupItem children];
                NSMutableArray *destChildren = [destGroupItem children];
                
                if ( newIndex < 0 ) {
                    newIndex = [destChildren indexOfObject:destItem];
                }
                
                if ( srcGroupIndex == destGroupIndex ) {
                    [outlineView moveItemAtIndex:oldIndex inParent:srcGroupItem toIndex:newIndex inParent:destGroupItem];
                    if ( newIndex != oldIndex ) {
                        if ( newIndex < oldIndex ) {
                            [srcChildren insertObject:srcItem atIndex:newIndex];
                            [srcChildren removeObjectAtIndex:oldIndex+1];
                        } else {
                            [srcChildren insertObject:srcItem atIndex:newIndex+1];
                            [srcChildren removeObjectAtIndex:oldIndex];
                        }
                    }
                } else {
                    [outlineView moveItemAtIndex:oldIndex inParent:srcGroupItem toIndex:newIndex inParent:destGroupItem];
                    [destChildren insertObject:srcItem atIndex:newIndex];
                    [srcChildren removeObject:srcItem];
                }
                
            }
        }
        
#endif
        
    }
    
    //[self outlineView:outlineView forwardEvent:outlineViewAcceptDrop object:item];

    return YES;//DialogEventBool();
}

- (id)outlineView:(NSOutlineView *)outlineView child:(NSInteger)index ofItem:(id)item {
    return !item ? [tableData objectAtIndex:index] : [[item children] objectAtIndex:index];
}

- (void)outlineView:(NSOutlineView *)outlineView draggingSession:(NSDraggingSession *)session endedAtPoint:(NSPoint)screenPoint operation:(NSDragOperation)operation {
    [outlineView reloadData];

    //[self outlineView:outlineView forwardEvent:outlineViewDraggingSessionEnded object:session];

    draggedItem = nil;
}

- (void)outlineView:(NSOutlineView *)outlineView draggingSession:(NSDraggingSession *)session willBeginAtPoint:(NSPoint)screenPoint forItems:(NSArray *)draggedItems {
    draggedItem = [draggedItems objectAtIndex:0];
    [session.draggingPasteboard setData:nil forType:OUTLINEVIEW_REORDER_PASTEBOARD_TYPE];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item {
    return [item isGroup];
}

//- (id)outlineView:(NSOutlineView *)outlineView itemForPersistentObject:(id)object {
//    return [NSKeyedUnarchiver unarchiveObjectWithData:object];
//}

- (NSInteger)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item {
    return !item ? [tableData count] : [[item children] count];
}

- (id<NSPasteboardWriting>)outlineView:(NSOutlineView *)outlineView pasteboardWriterForItem:(id)item {
    NSPasteboardItem *pbItem = [[NSPasteboardItem alloc] init];
    [pbItem setDataProvider:self forTypes:[NSArray arrayWithObjects:OUTLINEVIEW_REORDER_PASTEBOARD_TYPE,nil]];
    return pbItem;
}

//- (id)outlineView:(NSOutlineView *)outlineView persistentObjectForItem:(id)item {
//    return [NSKeyedArchiver archivedDataWithRootObject:item];
//}

//- (void)outlineView:(NSOutlineView *)outlineView sortDescriptorsDidChange:(NSArray<NSSortDescriptor *> *)oldDescriptors {
//
//}

//- (void)outlineView:(NSOutlineView *)outlineView updateDraggingItemsForDrag:(id<NSDraggingInfo>)draggingInfo {
//
//}

- (NSDragOperation)outlineView:(NSOutlineView *)outlineView validateDrop:(id<NSDraggingInfo>)info proposedItem:(id)item proposedChildIndex:(NSInteger)index {
    DialogEventSetLong(NSDragOperationLink);

    //[self outlineView:outlineView forwardEvent:outlineViewValidateDrop object:info];

    return DialogEventLong();
}

//- (BOOL)outlineView:(NSOutlineView *)outlineView writeItems:(NSArray *)items toPasteboard:(NSPasteboard *)pasteboard {
//    return YES;
//}

- (void)pasteboard:(NSPasteboard *)pasteboard item:(NSPasteboardItem *)item provideDataForType:(NSString *)type {
}



// action methods
- (void)doubleAction:(id)sender {
    CocoaOutputWindowSetTag( [sender wndTag] );
    [self outlineView:sender forwardEvent:outlineViewDoubleAction object:nil];
}

- (void)singleAction:(id)sender {
    CocoaOutputWindowSetTag( [sender wndTag] );
    [self outlineView:sender forwardEvent:1 object:nil];
}

// text field delegate methods
- (void)controlTextDidEndEditing:(NSNotification *)notification {
    NSTextField *textField = [notification object];
    NSInteger row = [table rowForView:textField];
    if ( row != -1 ) {
        NSString *string = [textField stringValue];
        if ( string ) {
            OutlineItem *rowObj = [table itemAtRow:row];
            if ( rowObj ) [rowObj setString:[string retain]];
        }
        [table reloadData];
    }
}
@end


@implementation NSOutlineView (Additions)
@dynamic callbackValue;

- (CocoaOutlineViewCallbackType)cocoaOutlineViewCallback {
    return [objc_getAssociatedObject(self, @selector(callbackValue)) pointerValue];
}

- (void)setCocoaOutlineViewCallback:(CocoaOutlineViewCallbackType)callback {
    objc_setAssociatedObject(self, @selector(callbackValue), [NSValue valueWithPointer:callback], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end


NSOutlineView *OutlineViewWithTag( NSInteger tag )
{
    NSOutlineView *ov = (NSOutlineView *)CocoaViewWithTag( tag );
    if ( ov ) {
        if ( [ov isKindOfClass:[NSOutlineView class]] ) return ov;
    }
    CocoaShowAlert( NSWarningAlertStyle, @"Error.", [NSString stringWithFormat:@"Can't find outlineview with tag: %ld.",(long)tag] );
    return nil;
}

BOOL OutlineViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSOutlineView class] ); }


// Data source
CFTypeRef OutlineViewDataSource( NSInteger tag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov dataSource];
    return nil;
}

void OutlineViewSetDataSource( NSInteger tag, CFTypeRef ref )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov setDataSource:ref];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean OutlineViewStronglyReferencesItems( NSInteger tag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov stronglyReferencesItems];
    return YES;
}

void OutlineViewSetStronglyReferencesItems( NSInteger tag, Boolean flag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov setStronglyReferencesItems:flag];
}
#endif// 101200

// -- expandability --
Boolean OutlineViewIsItemExpandable( NSInteger tag, OutlineItemRef item )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov isExpandable:item];
    return NO;
}

Boolean OutlineViewIsItemExpanded( NSInteger tag, OutlineItemRef item )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov isItemExpanded:item];
    return NO;
}

// -- expand/collapse --
void OutlineViewExpandItem( NSInteger tag, OutlineItemRef item )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov expandItem:item];
}

void OutlineViewExpandItemAndChildren( NSInteger tag, OutlineItemRef item, Boolean expandChildren )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov expandItem:item expandChildren:expandChildren];
}

void OutlineViewCollapseItem( NSInteger tag, OutlineItemRef item )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov collapseItem:item];
}

void OutlineViewCollapseItemAndChildren( NSInteger tag, OutlineItemRef item, Boolean collapseChildren )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov collapseItem:item collapseChildren:collapseChildren];
}

// -- redisplay --
void OutlineViewReloadItem( NSInteger tag, OutlineItemRef item )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov reloadItem:item];
}

void OutlineViewReloadItemAndChildren( NSInteger tag, OutlineItemRef item, Boolean reloadChildren )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov reloadItem:item reloadChildren:reloadChildren];
}

// -- convert between items and rows --
OutlineItemRef OutlineViewItemAtRow( NSInteger tag, NSInteger row )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov itemAtRow:row];
    return nil;
}

NSInteger OutlineViewRowForItem( NSInteger tag, OutlineItemRef item )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov rowForItem:item];
    return -1;
}

// Outline column
TableColumnRef OutlineViewOutlineTableColumn( NSInteger tag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov outlineTableColumn];
    return nil;
}

void OutlineViewSetOutlineTableColumn( NSInteger tag, TableColumnRef columnRef )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov setOutlineTableColumn:columnRef];
}

Boolean OutlineViewAutoresizesOutlineColumn( NSInteger tag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov autoresizesOutlineColumn];
    return NO;
}

void OutlineViewSetAutoresizesOutlineColumn( NSInteger tag, Boolean flag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov setAutoresizesOutlineColumn:flag];
}

// -- indentation --
NSInteger OutlineViewLevelForItem( NSInteger tag, OutlineItemRef item )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov levelForItem:item];
    return -1;
}

NSInteger OutlineViewLevelForRow( NSInteger tag, NSInteger row )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov levelForRow:row];
    return -1;
}

CGFloat OutlineViewIndentationPerLevel( NSInteger tag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov indentationPerLevel];
    return 0.0;
}

void OutlineViewSetIndentationPerLevel( NSInteger tag, CGFloat indent )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov setIndentationPerLevel:indent];
}

Boolean OutlineViewIndentationMarkerFollowsCell( NSInteger tag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov indentationMarkerFollowsCell];
    return NO;
}

void OutlineViewSetIndentationMarkerFollowsCell( NSInteger tag, Boolean flag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov setIndentationMarkerFollowsCell:flag];
}

// Persistence
Boolean OutlineViewAutosaveExpandedItems( NSInteger tag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov autosaveExpandedItems];
    return NO;
}

void OutlineViewSetAutosaveExpandedItems( NSInteger tag, Boolean flag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov setAutosaveExpandedItems:flag];
}

// Drag drop
void OutlineViewSetDropItem( NSInteger tag, OutlineItemRef item, NSInteger dropChildIndex )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov setDropItem:item dropChildIndex:dropChildIndex];
}

// -- persistence --

// -- drag drop --

// -- get related items --
OutlineItemRef OutlineViewParentForItem( NSInteger tag, OutlineItemRef item )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov parentForItem:item];
    return nil;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSInteger OutlineViewChildIndexForItem( NSInteger tag, OutlineItemRef item )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov childIndexForItem:item];
    return 0;
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
OutlineItemRef OutlineViewChildOfItem( NSInteger tag, NSInteger index, OutlineItemRef item )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov child:index ofItem:item];
    return nil;
}

NSInteger OutlineViewNumberOfChildrenOfItem( NSInteger tag, OutlineItemRef item )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov numberOfChildrenOfItem:item];
    return 0;
}
#endif

// -- cell frame --

// Delegate
CFTypeRef OutlineViewDelegate( NSInteger tag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov delegate];
    return nil;
}

void OutlineViewSetDelegate( NSInteger tag, CFTypeRef ref )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov setDelegate:ref];
}

// -- manipulate items --

// User interface layout direction
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSUserInterfaceLayoutDirection OutlineViewUserInterfaceLayoutDirection( NSInteger tag )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) return [ov userInterfaceLayoutDirection];
    return 0;
}

void OutlineViewSetUserInterfaceLayoutDirection( NSInteger tag, NSUserInterfaceLayoutDirection direction )
{
    NSOutlineView *ov = OutlineViewWithTag( tag );
    if ( ov ) [ov setUserInterfaceLayoutDirection:direction];
}
#endif

// Custom
CFMutableArrayRef OutlineViewData( NSInteger tag )
{
    NSOutlineView *outlineView = OutlineViewWithTag( tag );
    if ( outlineView ) {
        if ( [outlineView isKindOfClass:[NSOutlineView class]] ) {
            CocoaOutlineViewDelegate *delegate = (CocoaOutlineViewDelegate *)[outlineView delegate];
            return (CFMutableArrayRef)delegate.tableData;
        }
    }
    return nil;
}

void OutlineViewSetData( NSInteger tag, CFMutableArrayRef array )
{
    NSOutlineView *outlineView = OutlineViewWithTag( tag );
    if ( outlineView ) {
        if ( [outlineView isKindOfClass:[NSOutlineView class]] ) {
            CocoaOutlineViewDelegate *delegate = (CocoaOutlineViewDelegate *)[outlineView delegate];
            NSArray *oldArray = [delegate tableData];
            if ( oldArray ) [oldArray release];
            [delegate setTableData:[(NSMutableArray *)array retain]];
            [outlineView reloadData];
        }
    }
}

void OutlineViewReloadData( NSInteger tag )
{
    NSOutlineView *outlineView = OutlineViewWithTag( tag );
    if ( outlineView ) [outlineView reloadData];
}

