/*
 Panel.m

 Bernie Wylde
 */

#import "Panel.h"

#pragma mark - subclass
@implementation PanelSC
@end


@implementation NSPanel (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

@end


NSPanel *PanelWithTag( NSInteger tag )
{
    NSPanel *panel = (NSPanel *)CocoaWindowWithTag( tag );
    if ( panel ) {
        if ( [panel isKindOfClass:[NSPanel class]] ) return panel;
    }
    return nil;
}

BOOL PanelExists( NSInteger tag )
{
    NSPanel *panel = (NSPanel *)CocoaWindowWithTag( tag );
    if ( panel ) {
        if ( [panel isKindOfClass:[NSPanel class]] ) return YES;
    }
    return NO;
}



void PanelSetFloatingPanel( NSInteger tag, Boolean flag )
{
    NSPanel *w = PanelWithTag( tag );
    if ( w ) [w setFloatingPanel:flag];
}

void PanelSetBecomesKeyOnlyIfNeeded( NSInteger tag, Boolean flag )
{
    NSPanel *w = PanelWithTag( tag );
    if ( w ) [w setBecomesKeyOnlyIfNeeded:flag];
}

void PanelSetWorksWhenModal( NSInteger tag, Boolean flag )
{
    NSPanel *w = PanelWithTag( tag );
    if ( w ) [w setWorksWhenModal:flag];
}


// Init
PanelRef PanelInit( NSInteger inTag, CGRect inRect )
{
    BOOL center = NO;

    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) {
        rect = NSMakeRect( 0, 0, 276, 378 );
    } else {
        center = ( rect.origin.x == 0.0 && rect.origin.y == 0.0 );
    }

    // style
    NSUInteger style = NSTitledWindowMask | NSClosableWindowMask | NSResizableWindowMask | NSUtilityWindowMask;

    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSPanel *panel;
    if ( subclass ) {
        panel = [[PanelSC alloc] initWithContentRect:rect styleMask:style backing:NSBackingStoreBuffered defer:NO];
    } else {
        panel = [[NSPanel alloc] initWithContentRect:rect styleMask:style backing:NSBackingStoreBuffered defer:NO];
    }
    
    // floating panel
    [panel setFloatingPanel:YES];

    // content view
    /* create content view subclass for:
     1. drawing focus rect around text controls (Carbon apps only).
     2. intercepting content view drawRect: method to issue dialog viewDrawRect event.
     */
    ContentViewSC *contentView = [[ContentViewSC alloc] initWithFrame:[[panel contentView] frame]];
    [panel setContentView:contentView];
    [contentView setIdentifier:[NSString stringWithFormat:@"%d",windowContentViewTag]];

    // key view loop
    [panel setAutorecalculatesKeyViewLoop:YES];
    
    // tag
    [panel setTag:inTag];

    // center
    if ( center ) [(NSWindow *)panel center];
    
    // title
    [panel setTitle:@"Untitled"];
    
    // delegate
    [panel setDelegate:CocoaWindowDelegateGet()];
    
    // app delegate
    CocoaApplicationDelegate *appDelegate = CocoaApplicationDelegateGet();
    if ( appDelegate ) {
        [[appDelegate windows] addObject:panel];
    }

    return panel;
}

// Statement
void CocoaPanel( NSInteger inTag, CFStringRef inTitle, CGRect inRect, NSInteger inStyle )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    if ( CocoaObjectIsPopover( inTag ) ) return;
    
    if ( !IsCocoaApplication() ) {
        NSApplicationLoad();
        
#if !__LP64__
        if ( CocoaWindowIsFBWindow( inTag ) ) return;
#endif  //__LP64__
        
    }
    
    NSString *title = (NSString *)inTitle;
    NSRect rect = NSRectFromCGRect( inRect );
    
    id w = CocoaWindowWithTag( inTag );
    
    if ( w ) {      // exists
        
        // title
        if ( title ) {
            [w setTitle:title];
        }
        
        // rect
        if ( !NSIsEmptyRect( rect ) ) {
            [w setFrame:[w frameRectForContentRect:rect] display:YES];
        }
        
        // style
        if ( (int)inStyle != kFBParamMissing ) [w setStyleMask:inStyle];
        
    } else {    // create
        
//        BOOL center = NO;
        
        
        // rect
//        if ( NSIsEmptyRect( rect ) ) {
//            rect = NSMakeRect( 0, 0, 276, 378 );
//        } else {
//            center = ( rect.origin.x == 0.0 && rect.origin.y == 0.0 );
//        }
        
        // style
//        if ( (int)inStyle == kFBParamMissing ) {
//            inStyle = NSTitledWindowMask | NSClosableWindowMask | NSResizableWindowMask | NSUtilityWindowMask;
//        }
        
        // init
//        if ( inSubclass ) {
//            w = [[PanelSC alloc] initWithContentRect:rect styleMask:inStyle backing:NSBackingStoreBuffered defer:NO];
//        } else {
//            w = [[NSPanel alloc] initWithContentRect:rect styleMask:inStyle backing:NSBackingStoreBuffered defer:NO];
//        }
        
        w = PanelInit( inTag, inRect );

        // style
        if ( (int)inStyle != kFBParamMissing ) [w setStyleMask:inStyle];
        
        // floating panel
//        [w setFloatingPanel:YES];
        
        // content view
        /* create content view subclass for:
         1. drawing focus rect around text controls (Carbon apps only).
         2. intercepting content view drawRect: method to issue dialog viewDrawRect event.
         */
//        ContentViewSC *contentView = [[ContentViewSC alloc] initWithFrame:[[w contentView] frame]];
//        [w setContentView:contentView];
//
//        [contentView setIdentifier:[NSString stringWithFormat:@"%d",windowContentViewTag]];
        
        // key view loop
//        [w setAutorecalculatesKeyViewLoop:YES];
        
        // tag
//        [w setTag:inTag];
        
        // title
        if ( title ) [w setTitle:title];
        
        // center
//        if ( center ) [(NSWindow *)w center];
        
        // delegate
//        [w setDelegate:CocoaWindowDelegateGet()];
        
        // app delegate
//        CocoaApplicationDelegate *appDelegate = CocoaApplicationDelegateGet();
//        if ( appDelegate ) {
//            [[appDelegate windows] addObject:w];
//        }
    }
    
    // show
    if ( show ) {
        [w makeKeyAndOrderFront:nil];
        
#if __MAC_OS_X_VERSION_MAX_ALLOWED < 101100
        [[w contentView] setNeedsDisplay:YES];  // force drawRect:
#endif
        
    } else {
        [w orderOut:nil];
    }
    
    CocoaOutputWindowSetTag( inTag );
}

