/*
 ParagraphStyle.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

// - create -
ParagraphStyleRef ParagraphStyleDefaultStyle( void );

// - style -
NSTextAlignment ParagraphStyleAlignment( ParagraphStyleRef ref );
CGFloat ParagraphStyleFirstLineHeadIndent( ParagraphStyleRef ref );
CGFloat ParagraphStyleHeadIndent( ParagraphStyleRef ref );
CGFloat ParagraphStyleTailIndent( ParagraphStyleRef ref );
CGFloat ParagraphStyleLineHeightMultiple( ParagraphStyleRef ref );
CGFloat ParagraphStyleMaximumLineHeight( ParagraphStyleRef ref );
CGFloat ParagraphStyleMinimumLineHeight( ParagraphStyleRef ref );
CGFloat ParagraphStyleLineSpacing( ParagraphStyleRef ref );
CGFloat ParagraphStyleParagraphSpacing( ParagraphStyleRef ref );
CGFloat ParagraphStyleParagraphSpacingBefore( ParagraphStyleRef ref );

// - tab info -
CFArrayRef ParagraphStyleTabStops( ParagraphStyleRef ref );
CGFloat ParagraphStyleDefaultTabInterval( ParagraphStyleRef ref );

// - text block and list info -
CFArrayRef ParagraphStyleTextBlocks( ParagraphStyleRef ref );
CFArrayRef ParagraphStyleTextLists( ParagraphStyleRef ref );

// - line breaking info -
NSLineBreakMode ParagraphStyleLineBreakMode( ParagraphStyleRef ref );
float ParagraphStyleHyphenationFactor( ParagraphStyleRef ref );
float ParagraphStyleTighteningFactorForTruncation( ParagraphStyleRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean ParagraphStyleAllowsDefaultTighteningForTruncation( ParagraphStyleRef ref );
#endif
// - HTML header level -
NSInteger ParagraphStyleHeaderLevel( ParagraphStyleRef ref );

// - writing direction -
NSWritingDirection ParagraphStyleDefaultWritingDrectionForLanguage( CFStringRef languageName );
NSWritingDirection ParagraphStyleBaseWritingDirection( ParagraphStyleRef ref );


// -- mutable paragraph style --
// - init -
MutableParagraphStyleRef ParagraphStyleInit( void );

// - style info -
void ParagraphStyleSet( MutableParagraphStyleRef ref, ParagraphStyleRef style );
void ParagraphStyleSetAlignment( MutableParagraphStyleRef ref, NSTextAlignment alignment );
void ParagraphStyleSetFirstLineHeadIndent( MutableParagraphStyleRef ref, CGFloat indent );
void ParagraphStyleSetHeadIndent( MutableParagraphStyleRef ref, CGFloat indent );
void ParagraphStyleSetTailIndent( MutableParagraphStyleRef ref, CGFloat indent );
void ParagraphStyleSetLineBreakMode( MutableParagraphStyleRef ref, NSLineBreakMode lineBreakMode );
void ParagraphStyleSetMaximumLineHeight( MutableParagraphStyleRef ref, CGFloat height );
void ParagraphStyleSetMinimumLineHeight( MutableParagraphStyleRef ref, CGFloat height );
void ParagraphStyleSetLineSpacing( MutableParagraphStyleRef ref, CGFloat spacing );
void ParagraphStyleSetParagraphSpacing( MutableParagraphStyleRef ref, CGFloat spacing );
void ParagraphStyleSetParagraphSpacingBefore( MutableParagraphStyleRef ref, CGFloat spacing );
void ParagraphStyleSetBaseWritingDirection( MutableParagraphStyleRef ref, NSWritingDirection direction );
void ParagraphStyleSetLineHeightMultiple( MutableParagraphStyleRef ref, CGFloat height );

// - tab info -
void ParagraphStyleAddTabStop( MutableParagraphStyleRef ref, TextTabRef textTab );
void ParagraphStyleRemoveTabStop( MutableParagraphStyleRef ref, TextTabRef textTab );
void ParagraphStyleSetTabStops( MutableParagraphStyleRef ref, CFArrayRef tabStops );
void ParagraphStyleSetDefaultTabInterval( MutableParagraphStyleRef ref, CGFloat interval );

// - text blocks and lists
void ParagraphStyleSetTextBlocks( MutableParagraphStyleRef ref, CFArrayRef blocks );
void ParagraphStyleSetTextLists( MutableParagraphStyleRef ref, CFArrayRef lists );

// - hyphenation factor -
void ParagraphStyleSetHyphenationFactor( MutableParagraphStyleRef ref, float factor );
void ParagraphStyleSetTighteningFactorForTruncation( MutableParagraphStyleRef ref, float factor );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void ParagraphStyleSetAllowsDefaultTighteningForTruncation( MutableParagraphStyleRef ref, Boolean flag );
#endif

// - HTML header level -
void ParagraphStyleSetHeaderLevel( MutableParagraphStyleRef ref, NSInteger level );

