/*
 ParagraphStyle.m

 Bernie Wylde
 */

#import "ParagraphStyle.h"

// - create -
ParagraphStyleRef ParagraphStyleDefaultStyle( void )
{ return [NSParagraphStyle defaultParagraphStyle]; }

// - style -
NSTextAlignment ParagraphStyleAlignment( ParagraphStyleRef ref )
{ return [ref alignment]; }

CGFloat ParagraphStyleFirstLineHeadIndent( ParagraphStyleRef ref )
{ return [ref firstLineHeadIndent]; }

CGFloat ParagraphStyleHeadIndent( ParagraphStyleRef ref )
{ return [ref headIndent]; }

CGFloat ParagraphStyleTailIndent( ParagraphStyleRef ref )
{ return [ref tailIndent]; }

CGFloat ParagraphStyleLineHeightMultiple( ParagraphStyleRef ref )
{ return [ref lineHeightMultiple]; }

CGFloat ParagraphStyleMaximumLineHeight( ParagraphStyleRef ref )
{ return [ref maximumLineHeight]; }

CGFloat ParagraphStyleMinimumLineHeight( ParagraphStyleRef ref )
{ return [ref minimumLineHeight]; }

CGFloat ParagraphStyleLineSpacing( ParagraphStyleRef ref )
{ return [ref lineSpacing]; }

CGFloat ParagraphStyleParagraphSpacing( ParagraphStyleRef ref )
{ return [ref paragraphSpacing]; }

CGFloat ParagraphStyleParagraphSpacingBefore( ParagraphStyleRef ref )
{ return [ref paragraphSpacingBefore]; }

// - tab info -
CFArrayRef ParagraphStyleTabStops( ParagraphStyleRef ref )
{ return (CFArrayRef)[ref tabStops]; }

CGFloat ParagraphStyleDefaultTabInterval( ParagraphStyleRef ref )
{ return [ref defaultTabInterval]; }

// - text block and list info -
CFArrayRef ParagraphStyleTextBlocks( ParagraphStyleRef ref )
{ return (CFArrayRef)[ref textBlocks]; }

CFArrayRef ParagraphStyleTextLists( ParagraphStyleRef ref )
{ return (CFArrayRef)[ref textLists]; }

// - line breaking info -
NSLineBreakMode ParagraphStyleLineBreakMode( ParagraphStyleRef ref )
{ return [ref lineBreakMode]; }

float ParagraphStyleHyphenationFactor( ParagraphStyleRef ref )
{ return [ref hyphenationFactor]; }

float ParagraphStyleTighteningFactorForTruncation( ParagraphStyleRef ref )
{ return [ref tighteningFactorForTruncation]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean ParagraphStyleAllowsDefaultTighteningForTruncation( ParagraphStyleRef ref )
{ return [ref allowsDefaultTighteningForTruncation]; }
#endif

// - HTML header level -
NSInteger ParagraphStyleHeaderLevel( ParagraphStyleRef ref )
{ return [ref headerLevel]; }

// - writing direction -
NSWritingDirection ParagraphStyleDefaultWritingDrectionForLanguage( CFStringRef languageName )
{ return [NSParagraphStyle defaultWritingDirectionForLanguage:(NSString *)languageName]; }

NSWritingDirection ParagraphStyleBaseWritingDirection( ParagraphStyleRef ref )
{ return [ref baseWritingDirection]; }


// -- mutable paragraph style --
// - init -
MutableParagraphStyleRef ParagraphStyleInit( void )
{ return [[[NSMutableParagraphStyle alloc] init] autorelease]; }

// - style info -
void ParagraphStyleSet( MutableParagraphStyleRef ref, ParagraphStyleRef style )
{ [ref setParagraphStyle:style]; }

void ParagraphStyleSetAlignment( MutableParagraphStyleRef ref, NSTextAlignment alignment )
{ [ref setAlignment:alignment]; }

void ParagraphStyleSetFirstLineHeadIndent( MutableParagraphStyleRef ref, CGFloat indent )
{ [ref setFirstLineHeadIndent:indent]; }

void ParagraphStyleSetHeadIndent( MutableParagraphStyleRef ref, CGFloat indent )
{ [ref setHeadIndent:indent]; }

void ParagraphStyleSetTailIndent( MutableParagraphStyleRef ref, CGFloat indent )
{ [ref setTailIndent:indent]; }

void ParagraphStyleSetLineBreakMode( MutableParagraphStyleRef ref, NSLineBreakMode lineBreakMode )
{ [ref setLineBreakMode:lineBreakMode]; }

void ParagraphStyleSetMaximumLineHeight( MutableParagraphStyleRef ref, CGFloat height )
{ [ref setMaximumLineHeight:height]; }

void ParagraphStyleSetMinimumLineHeight( MutableParagraphStyleRef ref, CGFloat height )
{ [ref setMinimumLineHeight:height]; }

void ParagraphStyleSetLineSpacing( MutableParagraphStyleRef ref, CGFloat spacing )
{ [ref setLineSpacing:spacing]; }

void ParagraphStyleSetParagraphSpacing( MutableParagraphStyleRef ref, CGFloat spacing )
{ [ref setParagraphSpacing:spacing]; }

void ParagraphStyleSetParagraphSpacingBefore( MutableParagraphStyleRef ref, CGFloat spacing )
{ [ref setParagraphSpacingBefore:spacing]; }

void ParagraphStyleSetBaseWritingDirection( MutableParagraphStyleRef ref, NSWritingDirection direction )
{ [ref setBaseWritingDirection:direction]; }

void ParagraphStyleSetLineHeightMultiple( MutableParagraphStyleRef ref, CGFloat height )
{ [ref setLineHeightMultiple:height]; }

// - tab info -
void ParagraphStyleAddTabStop( MutableParagraphStyleRef ref, TextTabRef textTab )
{ [ref addTabStop:textTab]; }

void ParagraphStyleRemoveTabStop( MutableParagraphStyleRef ref, TextTabRef textTab )
{ [ref removeTabStop:textTab]; }

void ParagraphStyleSetTabStops( MutableParagraphStyleRef ref, CFArrayRef tabStops )
{ [ref setTabStops:(NSArray *)tabStops]; }

void ParagraphStyleSetDefaultTabInterval( MutableParagraphStyleRef ref, CGFloat interval )
{ [ref setDefaultTabInterval:interval]; }

// - text blocks and lists
void ParagraphStyleSetTextBlocks( MutableParagraphStyleRef ref, CFArrayRef blocks )
{ [ref setTextBlocks:(NSArray *)blocks]; }

void ParagraphStyleSetTextLists( MutableParagraphStyleRef ref, CFArrayRef lists )
{ [ref setTextLists:(NSArray *)lists]; }

// - hyphenation factor -
void ParagraphStyleSetHyphenationFactor( MutableParagraphStyleRef ref, float factor )
{ [ref setHyphenationFactor:factor]; }

void ParagraphStyleSetTighteningFactorForTruncation( MutableParagraphStyleRef ref, float factor )
{ [ref setTighteningFactorForTruncation:factor]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void ParagraphStyleSetAllowsDefaultTighteningForTruncation( MutableParagraphStyleRef ref, Boolean flag )
{ [ref setAllowsDefaultTighteningForTruncation:flag]; }
#endif

// - HTML header level -
void ParagraphStyleSetHeaderLevel( MutableParagraphStyleRef ref, NSInteger level )
{ [ref setHeaderLevel:level]; }

