/*
 Popover.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

@interface PopoverSC : NSPopover
@end

static NSMutableArray *sCocoaPopovers;

// dialog events
enum {
    popoverDetachableWindow = 1300,
    popoverShouldClose,
    popoverWillShow,
    popoverDidShow,
    popoverWillClose,
    popoverDidClose,
    popoverDidDetach,// macOS 10.10
    popoverShouldDetach// macOS 10.10
};

@interface CocoaPopoverDelegate : NSObject <NSPopoverDelegate,NSWindowDelegate> {
    NSWindow *detachableWindow;
}
@property (assign) NSWindow *detachableWindow;
@end

@interface NSPopover (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

void CocoaPopoverAdd( NSPopover *popover, NSInteger tag );

NSPopover *CocoaPopoverForView( id view );
NSPopover *CocoaPopoverWithTag( NSInteger tag );

PopoverRef PopoverWithTag( NSInteger tag );
BOOL PopoverExists( NSInteger tag );

NSPopoverBehavior PopoverBehavior( NSInteger tag );
void PopoverSetBehavior( NSInteger tag, NSPopoverBehavior behavior );
void PopoverShow( NSInteger tag, CGRect inRect, NSInteger viewTag, CGRectEdge preferredEdge );
CGRect PopoverPositioningRect( NSInteger tag );
void PopoverSetPositioningRect( NSInteger tag, CGRect rect );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101000
AppearanceRef PopoverAppearance( NSInteger tag );
void PopoverSetAppearance( NSInteger tag, AppearanceRef app );
AppearanceRef PopoverEffectiveAppearance( NSInteger tag );
#endif

Boolean PopoverAnimates( NSInteger tag );
void PopoverSetAnimates( NSInteger tag, Boolean flag );
CGSize PopoverContentSize( NSInteger tag );
void PopoverSetContentSize( NSInteger tag, CGSize size );
Boolean PopoverIsShown( NSInteger tag );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean PopoverIsDetached( NSInteger tag );
#endif

void PopoverSetDetachableWindow( NSInteger popoverTag, NSInteger wndTag );

void PopoverPerformClose( NSInteger tag );
void PopoverClose( NSInteger tag );

// -- convenience --
void PopoverAddSubview( NSInteger tag, NSInteger subviewTag );


// Init
PopoverRef PopoverInit( NSInteger inTag, CGRect inRect );

// Statement
void CocoaPopover( NSInteger inTag, CGRect inRect, NSInteger inBehavior, NSInteger inAnimates );

