/*
 Popover.m

 Bernie Wylde
 */

#import "Popover.h"

@implementation PopoverSC

@end


@implementation CocoaPopoverDelegate

@synthesize detachableWindow;

- (BOOL)windowShouldClose:(NSWindow *)window {
    if ( window == detachableWindow ) {
        [window orderOut:nil];
        return NO;
    }
    return YES;
}

- (NSWindow *)detachableWindowForPopover:(NSPopover *)popover {
    NSWindow *window;
    NSInteger detachableWndTag = 0;
    
    DialogEventSetLong(detachableWndTag);
    if ( detachableWindow ) DialogEventSetLong([detachableWindow tag]);
    
    CallUserDefinedOnDialogFunction( popoverDetachableWindow, [popover tag], [popover tag], NULL );
    
    detachableWndTag = DialogEventLong();
    window = CocoaWindowWithTag(detachableWndTag);
    CocoaOutputWindowSetTag( detachableWndTag );
    
    return window;
}

- (BOOL)popoverShouldClose:(NSPopover *)popover {
    DialogEventSetBool( YES );
    CallUserDefinedOnDialogFunction( popoverShouldClose, [popover tag], [popover tag], NULL );
    return DialogEventBool();
}

- (void)popoverWillShow:(NSNotification *)notification {
    CallUserDefinedOnDialogFunction( popoverWillShow, [notification.object tag], [notification.object tag], NULL );
}

- (void)popoverDidShow:(NSNotification *)notification {
    CallUserDefinedOnDialogFunction( popoverDidShow, [notification.object tag], [notification.object tag], NULL );
}

- (void)popoverWillClose:(NSNotification *)notification {
    CallUserDefinedOnDialogFunction( popoverWillClose, [notification.object tag], [notification.object tag], NULL );
}

- (void)popoverDidClose:(NSNotification *)notification {
    CallUserDefinedOnDialogFunction( popoverDidClose, [notification.object tag], [notification.object tag], NULL );
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
- (void)popoverDidDetach:(NSPopover *)popover {
    CallUserDefinedOnDialogFunction( popoverDidDetach, [popover tag], [popover tag], NULL );
}

- (BOOL)popoverShouldDetach:(NSPopover *)popover {
    DialogEventSetBool( YES );
    CallUserDefinedOnDialogFunction( popoverShouldDetach, [popover tag], [popover tag], NULL );
    return DialogEventBool();
}
#endif

@end

@implementation NSPopover (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

NSPopover *CocoaPopoverForView( id view )
{
    NSPopover *popover = nil;
    if ( [view class] == [NSView class] ) {
        NSResponder *viewNextResponder = [view nextResponder];
        if ( viewNextResponder ) {
            if ( [viewNextResponder class] == [NSViewController class] ) {
                NSResponder *viewControllerNextResponder = [viewNextResponder nextResponder];
                if ( viewControllerNextResponder ) {
                    if ( [viewControllerNextResponder class] == [NSPopover class] ) {
                        popover = (NSPopover *)viewControllerNextResponder;
                    }
                }
            }
        }
    } else {
        id superview = [view superview];
        if ( superview ) popover = CocoaPopoverForView( superview );
    }
    return popover;
}

void CocoaPopoverAdd( NSPopover *popover, NSInteger tag )
{
    if ( !sCocoaPopovers ) sCocoaPopovers = [[NSMutableArray alloc] initWithCapacity:0];
    NSDictionary *poDict = [NSDictionary dictionaryWithObject:popover forKey:[NSString stringWithFormat:@"%ld",(long)tag]];
    [sCocoaPopovers addObject:poDict];
    CocoaPopoverDelegate *delegate = [[CocoaPopoverDelegate alloc] init];
    [popover setDelegate:delegate];
}

NSPopover *CocoaPopoverWithTag( NSInteger tag )
{
    if ( tag ) {
        if ( tag > 1000000 ) return (NSPopover *)tag;// it's a PopoverRef

        if ( sCocoaPopovers ) {
            for ( NSDictionary *poDict in sCocoaPopovers ) {
                NSPopover *popover = [poDict objectForKey:[NSString stringWithFormat:@"%ld",(long)tag]];
                if ( popover ) return popover;
            }
        }
    }
    
    return nil;
}


void PopoverAddSubview( NSInteger tag, NSInteger subviewTag )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) {
        NSViewController *controller = [popover contentViewController];
        if ( controller ) {
            NSView *view = [controller view];
            if ( view ) {
                NSView *subview = CocoaViewWithTag( subviewTag );
                if ( subview ) {
                    [view addSubview:subview];
                    [(NSButton *)subview setWndTag:tag];
                }
            }
        }
    }
}

PopoverRef PopoverWithTag( NSInteger tag )
{ return CocoaPopoverWithTag( tag ); }

BOOL PopoverExists( NSInteger tag )
{
    if ( sCocoaPopovers ) {
        for ( NSDictionary *poDict in sCocoaPopovers ) {
            NSPopover *popover = [poDict objectForKey:[NSString stringWithFormat:@"%ld",(long)tag]];
            if ( popover ) return YES;
        }
    }
    return NO;
}

NSPopoverBehavior PopoverBehavior( NSInteger tag )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) return [popover behavior];
    return 0;
}

void PopoverSetBehavior( NSInteger tag, NSPopoverBehavior behavior )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) [popover setBehavior:behavior];
}

void PopoverShow( NSInteger tag, CGRect inRect, NSInteger viewTag, CGRectEdge preferredEdge )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) {
        NSView *view = CocoaViewWithTag( viewTag );
        if ( view ) {
            NSRect rect = NSRectFromCGRect( inRect );
            [popover showRelativeToRect:rect ofView:view preferredEdge:(NSRectEdge)preferredEdge];
        }
    }
}

CGRect PopoverPositioningRect( NSInteger tag )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) return NSRectToCGRect([popover positioningRect]);
    return CGRectZero;
}

void PopoverSetPositioningRect( NSInteger tag, CGRect rect )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) [popover setPositioningRect:NSRectFromCGRect(rect)];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101000
AppearanceRef PopoverAppearance( NSInteger tag )// macOS 10.10+
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) return [popover appearance];
    return nil;
}

void PopoverSetAppearance( NSInteger tag, AppearanceRef app )// macOS 10.10+
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) [popover setAppearance:app];
}

AppearanceRef PopoverEffectiveAppearance( NSInteger tag )// macOS 10.10+
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) return [popover effectiveAppearance];
    return nil;
}
#endif


Boolean PopoverAnimates( NSInteger tag )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) return [popover animates];
    return NO;
}

void PopoverSetAnimates( NSInteger tag, Boolean flag )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) [popover setAnimates:flag];
}

CGSize PopoverContentSize( NSInteger tag )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) return NSSizeToCGSize([popover contentSize]);
    return CGSizeZero;
}

void PopoverSetContentSize( NSInteger tag, CGSize size )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) [popover setContentSize:NSSizeFromCGSize(size)];
}

Boolean PopoverIsShown( NSInteger tag )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) return [popover isShown];
    return false;
}

void PopoverPerformClose( NSInteger tag )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) [popover performClose:nil];
}

void PopoverClose( NSInteger tag )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) [popover close];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean PopoverIsDetached( NSInteger tag )
{
    NSPopover *popover = CocoaPopoverWithTag( tag );
    if ( popover ) {
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
        return [popover isDetached];
#else                                           // silence 'may not respond to selector' compiler warning
        BOOL flag;
        SEL selector = NSSelectorFromString( @"isDetached" );
        NSMethodSignature *signature = [popover methodSignatureForSelector:selector];
        NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:signature];
        [invocation setTarget:popover];
        [invocation setSelector:selector];
        [invocation invoke];
        [invocation getReturnValue:&flag];
        return flag;
#endif
    }
    return NO;
}
#endif

void PopoverSetDetachableWindow( NSInteger popoverTag, NSInteger wndTag )
{
    NSPopover *popover = CocoaPopoverWithTag( popoverTag );
    if ( popover ) {
        NSWindow *w = CocoaWindowWithTag( wndTag );
        if ( w ) {
            CocoaPopoverDelegate *delegate = (CocoaPopoverDelegate *)[popover delegate];
            delegate.detachableWindow = w;
            [w setDelegate:delegate];
        }
    }
}



// Init
PopoverRef PopoverInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) {
        rect = NSMakeRect( 0, 0, 163, 96 );
    }
    
    // create popover view
    id view;
    if ( IsCocoaApplication() ) {
        view = [[NSView alloc] initWithFrame:rect];
    } else {
        view = [[ContentViewSC alloc] initWithFrame:rect];
        [view setIdentifier:[NSString stringWithFormat:@"%d",windowContentViewTag]];
    }
    
    // create popover view controller
    NSViewController *controller = [[NSViewController alloc] init];
    [controller setView:view];
    [view release];
    
    // subclass
    BOOL subclass = IsSubclassFlag();

    // create popover
    NSPopover *popover;
    if ( subclass ) {
        popover = [[[PopoverSC alloc] init] autorelease];
    } else {
        popover = [[[NSPopover alloc] init] autorelease];
    }
    
    // set controller
    [popover setContentViewController:controller];
    [controller release];
    
    // add to list of popovers
    CocoaPopoverAdd( popover, inTag );
    
    // tag
    [popover setTag:inTag];

    return popover;
}

// Statement
void CocoaPopover( NSInteger inTag, CGRect inRect, NSInteger inBehavior, NSInteger inAnimates )
{
    if ( CocoaObjectIsWindow( inTag ) ) return;
    
#if !__LP64__
    if ( CocoaWindowIsFBWindow( inTag ) ) return;// if an FB Carbon window exists with same id/tag, alert and abort
#endif  //__LP64__
    
    NSPopover *popover = CocoaPopoverWithTag( inTag );
    if ( !popover ) {
        
        // rect
//        NSRect rect = NSRectFromCGRect( inRect );
//        if ( NSIsEmptyRect( rect ) ) {
//            rect = NSMakeRect( 0, 0, 163, 96 );
//        }
        
        // create popover view
//        id view;
//        if ( IsCocoaApplication() ) {
//            view = [[NSView alloc] initWithFrame:rect];
//        } else {
//            view = [[ContentViewSC alloc] initWithFrame:rect];
//            [view setIdentifier:[NSString stringWithFormat:@"%d",windowContentViewTag]];
//        }
        
        // create popover view controller
//        NSViewController *controller = [[NSViewController alloc] init];
//        [controller setView:view];
//        [view release];
        
        // create popover
//        if ( inSubclass ) {
//            popover = [[PopoverSC alloc] init];
//        } else {
//            popover = [[NSPopover alloc] init];
//        }
        
        popover = PopoverInit( inTag, inRect );
        
        // behavior
        if ( inBehavior != kFBParamMissing ) [popover setBehavior:inBehavior];
        
        // animates
        if ( inAnimates != kFBParamMissing ) [popover setAnimates:inAnimates];
        
        // set controller
//        [popover setContentViewController:controller];
//        [controller release];
        
        // add to list of popovers
//        CocoaPopoverAdd( popover, inTag );
//        [popover release];
        
        // tag
//        [popover setTag:inTag];
    }
    CocoaOutputWindowSetTag( inTag );
}
