/*
 Predicate.m
 
 Bernie Wylde
 */

#import "Predicate.h"

// Create
PredicateRef PredicateWithFormat( CFStringRef format, ... )
{
    va_list ap;
    va_start( ap, format );
    NSPredicate *pred = [NSPredicate predicateWithFormat:(NSString *)format arguments:ap];
    va_end( ap );
    return pred;
}

PredicateRef PredicateWithFormatAndArgumentArray( CFStringRef format, CFArrayRef arguments )
{ return [NSPredicate predicateWithFormat:(NSString *)format argumentArray:(NSArray *)arguments]; }

PredicateRef PredicateWithSubstitionVariables( PredicateRef pred, CFDictionaryRef variables )
{ return [pred predicateWithSubstitutionVariables:(NSDictionary *)variables]; }

PredicateRef PredicateWithValue( Boolean value )
{ return [NSPredicate predicateWithValue:value]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
PredicateRef PredicateFromMetadataQueryString( CFStringRef string )
{ return [NSPredicate predicateFromMetadataQueryString:(NSString *)string]; }
#endif

// Evaluate
Boolean PredicateEvaluateWithObject( PredicateRef pred, CFTypeRef obj )
{ return [pred evaluateWithObject:(id)obj]; }

Boolean PredicateEvaluateWithObjectAndSubstitutionVariables( PredicateRef pred, CFTypeRef obj, CFDictionaryRef variables )
{ return [pred evaluateWithObject:(id)obj substitutionVariables:(NSDictionary *)variables]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void PredicateAllowEvaluation( PredicateRef pred )
{ [pred allowEvaluation]; }
#endif

// String representation
CFStringRef PredicateFormat( PredicateRef pred )
{ return(CFStringRef)[pred predicateFormat]; }

