/*
 PrintOperation.m

 Bernie Wylde
 */

#import "PrintOperation.h"

CocoaPrintOperationDelegate *sCocoaPrintOperationDelegate;

@implementation CocoaPrintOperationDelegate

@synthesize callback;

- (void)printOperationDidRun:(NSPrintOperation *)printOperation success:(BOOL)success contextInfo:(void *)contextInfo {
    if ( callback ) (*callback)(printOperation,success,contextInfo);
}
@end


PrintOperationRef PrintOperationEPSWithViewToData( NSInteger viewTag, CGRect rect, CFMutableDataRef data )
{
    NSView *view = ViewWithTag( viewTag );
    if ( view ) return [NSPrintOperation EPSOperationWithView:view insideRect:NSRectFromCGRect(rect) toData:(NSMutableData *)data];
    return nil;
}

PrintOperationRef PrintOperationEPSWithViewToDataPrintInfo( NSInteger viewTag, CGRect rect, CFMutableDataRef data, PrintInfoRef printInfo )
{
    NSView *view = ViewWithTag( viewTag );
    if ( view ) return [NSPrintOperation EPSOperationWithView:view insideRect:NSRectFromCGRect(rect) toData:(NSMutableData *)data printInfo:printInfo];
    return nil;
}

PrintOperationRef PrintOperationEPSWithViewToPath( NSInteger viewTag, CGRect rect, CFStringRef path, PrintInfoRef printInfo )
{
    NSView *view = ViewWithTag( viewTag );
    if ( view ) return [NSPrintOperation EPSOperationWithView:view insideRect:NSRectFromCGRect(rect) toPath:(NSString *)path printInfo:printInfo];
    return nil;
}

PrintOperationRef PrintOperationPDFWithViewToData( NSInteger viewTag, CGRect rect, CFMutableDataRef data )
{
    NSView *view = ViewWithTag( viewTag );
    if ( view ) return [NSPrintOperation PDFOperationWithView:view insideRect:NSRectFromCGRect(rect) toData:(NSMutableData *)data];
    return nil;
}

PrintOperationRef PrintOperationPDFWithViewToDataPrintInfo( NSInteger viewTag, CGRect rect, CFMutableDataRef data, PrintInfoRef printInfo )
{
    NSView *view = ViewWithTag( viewTag );
    if ( view ) return [NSPrintOperation PDFOperationWithView:view insideRect:NSRectFromCGRect(rect) toData:(NSMutableData *)data printInfo:printInfo];
    return nil;
}

PrintOperationRef PrintOperationPDFWithViewToPath( NSInteger viewTag, CGRect rect, CFStringRef path, PrintInfoRef printInfo )
{
    NSView *view = ViewWithTag( viewTag );
    if ( view ) return [NSPrintOperation PDFOperationWithView:view insideRect:NSRectFromCGRect(rect) toPath:(NSString *)path printInfo:printInfo];
    return nil;
}

PrintOperationRef PrintOperationWithView( NSInteger viewTag )
{
    NSView *view = ViewWithTag( viewTag );
    if ( view ) return [NSPrintOperation printOperationWithView:view];
    return nil;
}

PrintOperationRef PrintOperationWithViewPrintInfo( NSInteger viewTag, PrintInfoRef printInfo )
{
    NSView *view = ViewWithTag( viewTag );
    if ( view ) return [NSPrintOperation printOperationWithView:view printInfo:printInfo];
    return nil;
}

PrintOperationRef PrintOperationCurrent( void )
{ return [NSPrintOperation currentOperation]; }

Boolean PrintOperationIsCopying( PrintOperationRef printOperation )
{ return [printOperation isCopyingOperation]; }

PrintInfoRef PrintOperationPrintInfo( PrintOperationRef printOperation )
{ return [printOperation printInfo]; }

void PrintOperationSetPrintInfo( PrintOperationRef printOperation, PrintInfoRef printInfo )
{ [printOperation setPrintInfo:printInfo]; }

NSInteger PrintOperationView( PrintOperationRef printOperation )
{
    NSView *view = [printOperation view];
    if ( view ) return [view tag];
    return 0;
}

NSPrintRenderingQuality PrintOperationPreferredRenderingQuality( PrintOperationRef printOperation )
{ return [printOperation preferredRenderingQuality]; }

Boolean PrintOperationRun( PrintOperationRef printOperation )
{ return [printOperation runOperation]; }

void PrintOperationRunModal( PrintOperationRef printOperation, NSInteger wndTag, void *callback, void *userData )
{
    NSWindow *window = WindowWithTag( wndTag );
    if ( window ) {
        if ( !sCocoaPrintOperationDelegate ) sCocoaPrintOperationDelegate = [[CocoaPrintOperationDelegate alloc] init];
        sCocoaPrintOperationDelegate.callback = callback;
        [printOperation runOperationModalForWindow:window delegate:sCocoaPrintOperationDelegate didRunSelector:@selector(printOperationDidRun:success:contextInfo:) contextInfo:userData];
    }
}

Boolean PrintOperationDeliverResult( PrintOperationRef printOperation )
{ return [printOperation deliverResult]; }

Boolean PrintOperationShowsPrintPanel( PrintOperationRef printOperation )
{ return [printOperation showsPrintPanel]; }

void PrintOperationSetShowsPrintPanel( PrintOperationRef printOperation, Boolean flag )
{ [printOperation setShowsPrintPanel:flag]; }

Boolean PrintOperationShowsProgressPanel( PrintOperationRef printOperation )
{ return [printOperation showsProgressPanel]; }

void PrintOperationSetShowsProgressPanel( PrintOperationRef printOperation, Boolean flag )
{ [printOperation setShowsProgressPanel:flag]; }

CFStringRef PrintOperationJobTitle( PrintOperationRef printOperation )
{ return (CFStringRef)[printOperation jobTitle]; }

void PrintOperationSetJobTitle( PrintOperationRef printOperation, CFStringRef title )
{ [printOperation setJobTitle:(NSString *)title]; }

PrintPanelRef PrintOperationPrintPanel( PrintOperationRef printOperation )
{ return [printOperation printPanel]; }

void PrintOperationSetPrintPanel( PrintOperationRef printOperation, PrintPanelRef printPanel )
{ [printOperation setPrintPanel:printPanel]; }

GraphicsContextRef PrintOperationContext( PrintOperationRef printOperation )
{ return [printOperation context]; }

GraphicsContextRef PrintOperationCreateContext( PrintOperationRef printOperation )
{ return [printOperation createContext]; }

NSInteger PrintOperationCurrentPage( PrintOperationRef printOperation )
{ return [printOperation currentPage]; }

CFRange PrintOperationPageRange( PrintOperationRef printOperation )
{
    NSRange range = [printOperation pageRange];
    return CFRangeMake(range.location,range.length);
}

NSPrintingPageOrder PrintOperationPageOrder( PrintOperationRef printOperation )
{ return [printOperation pageOrder]; }

void PrintOperationSetPageOrder( PrintOperationRef printOperation, NSPrintingPageOrder pageOrder )
{ [printOperation setPageOrder:pageOrder]; }

Boolean PrintOperationCanSpawnSeparateThread( PrintOperationRef printOperation )
{ return [printOperation canSpawnSeparateThread]; }

void PrintOperationSetCanSpawnSeparateThread( PrintOperationRef printOperation, Boolean flag )
{ [printOperation setCanSpawnSeparateThread:flag]; }

