/*
 ProcessInfo.m

 Bernie Wylde
 */

#import "ProcessInfo.h"

// - get process info agent
ProcessInfoRef ProcessInfoInit( void )
{ return [NSProcessInfo processInfo]; }

// - accessing process info -
CFArrayRef ProcessInfoArguments( void )
{ return (CFArrayRef)[[NSProcessInfo processInfo] arguments]; }

CFDictionaryRef ProcessInfoEnvironment( void )
{ return (CFDictionaryRef)[[NSProcessInfo processInfo] environment]; }

CFStringRef ProcessInfoGloballyUniqueString( void )
{ return (CFStringRef)[[NSProcessInfo processInfo] globallyUniqueString]; }

int ProcessInfoProcessIdentifier( void )
{ return [[NSProcessInfo processInfo] processIdentifier]; }

CFStringRef ProcessInfoProcessName( void )
{ return (CFStringRef)[[NSProcessInfo processInfo] processName]; }

// - user info -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFStringRef ProcessInfoUserName( void )
{ return (CFStringRef)[[NSProcessInfo processInfo] userName]; }

CFStringRef ProcessInfoFullUserName( void )
{ return (CFStringRef)[[NSProcessInfo processInfo] fullUserName]; }
#endif

// - sudden app termination -
void ProcessInfoDisableSuddenTermination( void )
{ [[NSProcessInfo processInfo] disableSuddenTermination]; }

void ProcessInfoEnableSuddenTermination( void )
{ [[NSProcessInfo processInfo] enableSuddenTermination]; }

// - control auto termination
void ProcessInfoDisableAutomaticTermination( CFStringRef reason )
{ [[NSProcessInfo processInfo] disableAutomaticTermination:(NSString *)reason]; }

void ProcessInfoEnableAutomaticTermination( CFStringRef reason )
{ [[NSProcessInfo processInfo] enableAutomaticTermination:(NSString *)reason]; }

Boolean ProcessInfoAutomaticTerminationSupportEnabled( void )
{ return [[NSProcessInfo processInfo] automaticTerminationSupportEnabled]; }

// - host info -
CFStringRef ProcessInfoHostName( void )
{ return (CFStringRef)[[NSProcessInfo processInfo] hostName]; }

CFStringRef ProcessInfoOperatingSystemVersionString( void )
{ return (CFStringRef)[[NSProcessInfo processInfo] operatingSystemVersionString]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSOperatingSystemVersion ProcessInfoOperatingSystemVersion( void )
{ return [[NSProcessInfo processInfo] operatingSystemVersion]; }

Boolean ProcessInfoIsOperatingSystemAtLeastVersion( NSOperatingSystemVersion version)
{ return [[NSProcessInfo processInfo] isOperatingSystemAtLeastVersion:version]; }
#endif

// - computer info -
NSUInteger ProcessInfoProcessorCount( void )
{ return [[NSProcessInfo processInfo] processorCount]; }

NSUInteger ProcessInfoActiveProcessorCount( void )
{ return [[NSProcessInfo processInfo] activeProcessorCount]; }

unsigned long long ProcessInfoPhysicalMemory( void )
{ return [[NSProcessInfo processInfo] physicalMemory]; }

CFTimeInterval ProcessInfoSystemUptime( void )
{ return [[NSProcessInfo processInfo] systemUptime]; }

// - activities -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void *ProcessInfoBeginActivity( NSActivityOptions options, CFStringRef reason )
{ return (void *)[[NSProcessInfo processInfo] beginActivityWithOptions:options reason:(NSString *)reason]; }

void ProcessInfoEndActivity( void *obj )
{ [[NSProcessInfo processInfo] endActivity:(id)obj]; }
#endif

// - thermal state -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
NSProcessInfoThermalState ProcessInfoThermalState( void )
{ return [[NSProcessInfo processInfo] thermalState]; }
#endif

