/*
 Progress.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090

typedef void (*CocoaProgressHandlerType)(NSProgress*,void *);

// Create
ProgressRef ProgressWithParent( ProgressRef parent, CFDictionaryRef userInfo );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ProgressRef ProgressDiscreteWithTotalUnitCount( SInt64 totalUnitCount );
#endif
ProgressRef ProgressWithTotalUnitCount( SInt64 totalUnitCount );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ProgressRef ProgressWithPendingUnitCount( SInt64 totalUnitCount, ProgressRef parent, SInt64 pendingUnitCount );
#endif

// Current
ProgressRef ProgressCurrent( void );
void ProgressBecomeCurrent( ProgressRef ref, SInt64 pendingUnitCount );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void ProgressAddChild( ProgressRef ref, ProgressRef child, SInt64 pendingUnitCount );
#endif
void ProgressResignCurrent( ProgressRef ref );

// Reporting
SInt64 ProgressTotalUnitCount( ProgressRef ref );
SInt64 ProgressCompletedUnitCount( ProgressRef ref );
CFStringRef ProgressLocalizedDescription( ProgressRef ref );
CFStringRef ProgressLocalizedAdditionalDescription( ProgressRef ref );

// Observing
double ProgressFractionCompleted( ProgressRef ref );

// Controlling
Boolean ProgressIsCancellable( ProgressRef ref );
void ProgressIsSetCancellable( ProgressRef ref, Boolean flag );
Boolean ProgressIsCancelled( ProgressRef ref );
void ProgressCancel( ProgressRef ref );
void ProgressSetCancellationHandler( ProgressRef ref, CocoaProgressHandlerType handler, void *userData );
Boolean ProgressIsPausable( ProgressRef ref );
void ProgressIsSetPausable( ProgressRef ref, Boolean flag );
Boolean ProgressIsPaused( ProgressRef ref );
void ProgressPause( ProgressRef ref );
void ProgressSetPausingHandler( ProgressRef ref, CocoaProgressHandlerType handler, void *userData );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void ProgressResume( ProgressRef ref );
void ProgressSetResumingHandler( ProgressRef ref, CocoaProgressHandlerType handler, void *userData );
#endif

// Info
Boolean ProgressIsIndeterminate( ProgressRef ref );
CFStringRef ProgressKind( ProgressRef ref );
void ProgressSetKind( ProgressRef ref, CFStringRef kind );
void ProgressSetUserInfoObject( ProgressRef ref, CFTypeRef obj, CFStringRef key );
CFDictionaryRef ProgressUserInfo( ProgressRef ref );

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFStringRef ProgressFileOperationKind( ProgressRef ref );
void ProgressSetFileOperationKind( ProgressRef ref, CFStringRef kind );
CFURLRef ProgressFileURL( ProgressRef ref );
void ProgressSetFileURL( ProgressRef ref, CFURLRef url );
Boolean ProgressIsFinished( ProgressRef ref );
#endif
Boolean ProgressIsOld( ProgressRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFNumberRef ProgressEstimatedTimeRemaining( ProgressRef ref );
void ProgressSetEstimatedTimeRemaining( ProgressRef ref, CFNumberRef number );
CFNumberRef ProgressFileCompletedCount( ProgressRef ref );
void ProgressSetFileCompletedCount( ProgressRef ref, CFNumberRef number );
CFNumberRef ProgressFileTotalCount( ProgressRef ref );
void ProgressSetFileTotalCount( ProgressRef ref, CFNumberRef number );
CFNumberRef ProgressThroughput( ProgressRef ref );
void ProgressSetThroughput( ProgressRef ref, CFNumberRef number );
#endif

// Instance methods
void ProgressPublish( ProgressRef ref );
void ProgressUnpublish( ProgressRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void ProgressPerformAsCurrent( ProgressRef ref, SInt64 pendingUnitCount, CocoaProgressHandlerType callback, void *userData );
#endif

// Type methods
CFTypeRef ProgressAddSubscriberForFileURL( CFURLRef url, CocoaProgressHandlerType publishingHandler, void *userData );
void ProgressRemoveSubscriber( CFTypeRef subscriber );

#endif
