/*
 Progress.m

 Bernie Wylde
 */

#import "Progress.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090

// Create
ProgressRef ProgressWithParent( ProgressRef parent, CFDictionaryRef userInfo )
{ return [[[NSProgress alloc] initWithParent:parent userInfo:(NSDictionary *)userInfo] autorelease]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ProgressRef ProgressDiscreteWithTotalUnitCount( SInt64 totalUnitCount )
{ return [NSProgress discreteProgressWithTotalUnitCount:totalUnitCount]; }
#endif

ProgressRef ProgressWithTotalUnitCount( SInt64 totalUnitCount )
{ return [NSProgress progressWithTotalUnitCount:totalUnitCount]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ProgressRef ProgressWithPendingUnitCount( SInt64 totalUnitCount, ProgressRef parent, SInt64 pendingUnitCount )
{ return [NSProgress progressWithTotalUnitCount:totalUnitCount parent:parent pendingUnitCount:pendingUnitCount]; }
#endif

// Current
ProgressRef ProgressCurrent( void )
{ return [NSProgress currentProgress]; }

void ProgressBecomeCurrent( ProgressRef ref, SInt64 pendingUnitCount )
{ [ref becomeCurrentWithPendingUnitCount:pendingUnitCount]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void ProgressAddChild( ProgressRef ref, ProgressRef child, SInt64 pendingUnitCount )
{ [ref addChild:child withPendingUnitCount:pendingUnitCount]; }
#endif

void ProgressResignCurrent( ProgressRef ref )
{ [ref resignCurrent]; }

// Reporting
SInt64 ProgressTotalUnitCount( ProgressRef ref )
{ return [ref totalUnitCount]; }

SInt64 ProgressCompletedUnitCount( ProgressRef ref )
{ return [ref completedUnitCount]; }

CFStringRef ProgressLocalizedDescription( ProgressRef ref )
{ return (CFStringRef)[ref localizedDescription]; }

CFStringRef ProgressLocalizedAdditionalDescription( ProgressRef ref )
{ return (CFStringRef)[ref localizedAdditionalDescription]; }

// Observing
double ProgressFractionCompleted( ProgressRef ref )
{ return [ref fractionCompleted]; }

// Controlling
Boolean ProgressIsCancellable( ProgressRef ref )
{ return [ref isCancellable]; }

void ProgressIsSetCancellable( ProgressRef ref, Boolean flag )
{ [ref setCancellable:flag]; }

Boolean ProgressIsCancelled( ProgressRef ref )
{ return [ref isCancelled]; }

void ProgressCancel( ProgressRef ref )
{ [ref cancel]; }

void ProgressSetCancellationHandler( ProgressRef ref, CocoaProgressHandlerType handler, void *userData )
{
    [ref setCancellationHandler:^(void){
        (*handler)(ref,userData);
    }];
}

Boolean ProgressIsPausable( ProgressRef ref )
{ return [ref isPausable]; }

void ProgressIsSetPausable( ProgressRef ref, Boolean flag )
{ [ref setPausable:flag]; }

Boolean ProgressIsPaused( ProgressRef ref )
{ return [ref isPaused]; }

void ProgressPause( ProgressRef ref )
{ [ref pause]; }

void ProgressSetPausingHandler( ProgressRef ref, CocoaProgressHandlerType handler, void *userData )
{
    [ref setPausingHandler:^(void){
        (*handler)(ref,userData);
    }];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void ProgressResume( ProgressRef ref )
{ [ref resume]; }

void ProgressSetResumingHandler( ProgressRef ref, CocoaProgressHandlerType handler, void *userData )
{
    [ref setResumingHandler:^(void){
        (*handler)(ref,userData);
    }];
}
#endif

// Info
Boolean ProgressIsIndeterminate( ProgressRef ref )
{ return [ref isIndeterminate]; }

CFStringRef ProgressKind( ProgressRef ref )
{ return (CFStringRef)[ref kind]; }

void ProgressSetKind( ProgressRef ref, CFStringRef kind )
{ [ref setKind:(NSString *)kind]; }

void ProgressSetUserInfoObject( ProgressRef ref, CFTypeRef obj, CFStringRef key )
{ [ref setUserInfoObject:(id)obj forKey:(NSString *)key]; }

CFDictionaryRef ProgressUserInfo( ProgressRef ref )
{ return (CFDictionaryRef)[ref userInfo]; }

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFStringRef ProgressFileOperationKind( ProgressRef ref )
{ return (CFStringRef)[ref fileOperationKind]; }

void ProgressSetFileOperationKind( ProgressRef ref, CFStringRef kind )
{ [ref setFileOperationKind:(NSString *)kind]; }

CFURLRef ProgressFileURL( ProgressRef ref )
{ return (CFURLRef)[ref fileURL]; }

void ProgressSetFileURL( ProgressRef ref, CFURLRef url )
{ [ref setFileURL:(NSURL *)url]; }

Boolean ProgressIsFinished( ProgressRef ref )
{ return [ref isFinished]; }
#endif

Boolean ProgressIsOld( ProgressRef ref )
{ return [ref isOld]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFNumberRef ProgressEstimatedTimeRemaining( ProgressRef ref )
{ return (CFNumberRef)[ref estimatedTimeRemaining]; }

void ProgressSetEstimatedTimeRemaining( ProgressRef ref, CFNumberRef number )
{ [ref setEstimatedTimeRemaining:(NSNumber *)number]; }

CFNumberRef ProgressFileCompletedCount( ProgressRef ref )
{ return (CFNumberRef)[ref fileCompletedCount]; }

void ProgressSetFileCompletedCount( ProgressRef ref, CFNumberRef number )
{ [ref setFileCompletedCount:(NSNumber *)number]; }

CFNumberRef ProgressFileTotalCount( ProgressRef ref )
{ return (CFNumberRef)[ref fileTotalCount]; }

void ProgressSetFileTotalCount( ProgressRef ref, CFNumberRef number )
{ [ref setFileTotalCount:(NSNumber *)number]; }

CFNumberRef ProgressThroughput( ProgressRef ref )
{ return (CFNumberRef)[ref throughput]; }

void ProgressSetThroughput( ProgressRef ref, CFNumberRef number )
{ [ref setThroughput:(NSNumber *)number]; }
#endif

// Instance methods
void ProgressPublish( ProgressRef ref )
{ [ref publish]; }

void ProgressUnpublish( ProgressRef ref )
{ [ref unpublish]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void ProgressPerformAsCurrent( ProgressRef ref, SInt64 pendingUnitCount, CocoaProgressHandlerType callback, void *userData )
{
    [ref performAsCurrentWithPendingUnitCount:pendingUnitCount usingBlock:^(void){
        (*callback)(ref,userData);
    }];
}
#endif

// Type methods
CFTypeRef ProgressAddSubscriberForFileURL( CFURLRef url, CocoaProgressHandlerType publishingHandler, void *userData )
{
    return (CFTypeRef)[NSProgress addSubscriberForFileURL:(NSURL *)url withPublishingHandler:^NSProgressUnpublishingHandler(NSProgress *progress) {
        (*publishingHandler)(progress,userData);
        return nil;
    }];
}

void ProgressRemoveSubscriber( CFTypeRef subscriber )
{ [NSProgress removeSubscriber:(id)subscriber]; }

#endif
