/*
 ProgressIndicator.m

 Bernie Wylde
 */

#import "ProgressIndicator.h"

#pragma mark - subclass
@implementation ProgressIndicatorSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end


@implementation NSProgressIndicator (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

NSProgressIndicator *ProgressIndicatorWithTag( NSInteger tag )
{ return (NSProgressIndicator *)CocoaValidateViewKindOfClass( tag, [NSProgressIndicator class] ); }

BOOL ProgressIndicatorExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSProgressIndicator class] ); }


// - animating -
void ProgressIndicatorStartAnimation( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) [c startAnimation:nil];
}

void ProgressIndicatorStopAnimation( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) [c stopAnimation:nil];
}

Boolean ProgressIndicatorUsesThreadedAnimation( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) return [c usesThreadedAnimation];
    return NO;
}

void ProgressIndicatorSetUsesThreadedAnimation( NSInteger tag, Boolean flag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) [c setUsesThreadedAnimation:flag];
}

// - advancing -
void ProgressIndicatorIncrementBy( NSInteger tag, double delta )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) [c incrementBy:delta];
}

double ProgressIndicatorDoubleValue( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) return [c doubleValue];
    return 0.0;
}

void ProgressIndicatorSetDoubleValue( NSInteger tag, double value )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    [c setDoubleValue:value];
}

double ProgressIndicatorMinValue( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) return [c minValue];
    return 0.0;
}

void ProgressIndicatorSetMinValue( NSInteger tag, double value )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    [c setMinValue:value];
}

double ProgressIndicatorMaxValue( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) return [c maxValue];
    return 0.0;
}

void ProgressIndicatorSetMaxValue( NSInteger tag, double value )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    [c setMaxValue:value];
}

// - appearance -
NSControlSize ProgressIndicatorControlSize( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) return [c controlSize];
    return 0;
}

void ProgressIndicatorSetControlSize( NSInteger tag, NSControlSize size )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) [c setControlSize:size];
}

NSControlTint ProgressIndicatorControlTint( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) return [c controlTint];
    return 0;
}

void ProgressIndicatorSetControlTint( NSInteger tag, NSControlTint tint )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) [c setControlTint:tint];
}

Boolean ProgressIndicatorIsBezeled( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) return [c isBezeled];
    return NO;
}

void ProgressIndicatorSetBezeled( NSInteger tag, Boolean flag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) [c setBezeled:flag];
}

Boolean ProgressIndicatorIsIndeterminate( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) return [c isIndeterminate];
    return NO;
}

void ProgressIndicatorSetIndeterminate( NSInteger tag, Boolean flag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) [c setIndeterminate:flag];
}

NSUInteger ProgressIndicatorStyle( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) return [c style];
    return 0;
}

void ProgressIndicatorSetStyle( NSInteger tag, NSUInteger style )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) [c setStyle:style];
}

void ProgressIndicatorSizeToFit( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) [c sizeToFit];
}

Boolean ProgressIndicatorIsDisplayWhenStopped( NSInteger tag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) return [c isDisplayedWhenStopped];
    return NO;
}

void ProgressIndicatorSetDisplayWhenStopped( NSInteger tag, Boolean flag )
{
    NSProgressIndicator *c = ProgressIndicatorWithTag( tag );
    if ( c ) [c setDisplayedWhenStopped:flag];
}




// Init
ProgressIndicatorRef ProgressIndicatorInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) {
        rect = NSMakeRect( 20, 19, 100, 20 );
    }
    
    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    NSProgressIndicator *ind;
    if ( subclass ) {
        ind = [[[ProgressIndicatorSC alloc] initWithFrame:rect] autorelease];
    } else {
        ind = [[[NSProgressIndicator alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [ind setTag:inTag];

    // min
    [ind setMinValue:0.0];

    // max
    [ind setMaxValue:100.0];

    // value
    [ind setDoubleValue:0.0];
    
    // indeterminate
    [ind setIndeterminate:NO];
    
    // displayWhenStopped
    [ind setDisplayedWhenStopped:YES];
    
    return ind;
}



void CocoaProgressIndicator( NSInteger inTag, double inValue, CGRect inRect, double inMinValue, double inMaxValue, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSProgressIndicator *ind = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !ind ) {
            ind = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( ind ) {
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [ind class], [NSProgressIndicator class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, ind, [NSProgressIndicator class] ) ) {
                return;
            }

            // min
            if ( (int)inMinValue != kFBParamMissing ) [ind setMinValue:inMinValue];
            double minValue = [ind minValue];

            // max
            if ( (int)inMaxValue != kFBParamMissing ) [ind setMaxValue:inMaxValue];
            double maxValue = [ind maxValue];
            
            // value
            if ( (int)inValue != kFBParamMissing ) {
                if ( inValue >= minValue && inValue <= maxValue ) [ind setDoubleValue:inValue];
            }
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [ind setFrame:rect];
            
        } else {
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) {
//                rect = NSMakeRect( 20, 19, 100, 20 );
//            }
            
            // init
//            if ( inSubclass ) {
//                ind = [[ProgressIndicatorSC alloc] initWithFrame:rect];
//            } else {
//                ind = [[NSProgressIndicator alloc] initWithFrame:rect];
//            }
            
            ind = ProgressIndicatorInit( inTag, inRect );
            
            // default autoresizing
            [ind setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [ind setTag:inTag];
            
            // min
            if ( (int)inMinValue != kFBParamMissing ) [ind setMinValue:inMinValue];
            
            // max
            if ( (int)inMaxValue != kFBParamMissing ) [ind setMaxValue:inMaxValue];
            
            // value
            if ( (int)inValue != kFBParamMissing ) {
                if ( inValue <= [ind maxValue] ) [ind setDoubleValue:inValue];
            }
            
            // indeterminate
//            [ind setIndeterminate:NO];
            
            // displayWhenStopped
//            [ind setDisplayedWhenStopped:YES];
            
            // wndTag
            [ind setWndTag:inWndTag];
            
            // add to window
            [contentView addSubview:ind];
//            [ind release];
        }
        
        [ind setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

