/*
 RegularExpression.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef BOOL (*CocoaRegularExpressionCallbackType)(TextCheckingResultRef,NSMatchingFlags,void *);

// Create
RegularExpressionRef RegularExpressionWithPattern( CFStringRef pattern, NSRegularExpressionOptions options, ErrorRef *err );

// Getting the regex and options
CFStringRef RegularExpressionPattern( RegularExpressionRef ref );
NSRegularExpressionOptions RegularExpressionOptions( RegularExpressionRef ref );
NSUInteger RegularExpressionNumberOfCaptureGroups( RegularExpressionRef ref );

// Searching
NSUInteger RegularExpressionNumberOfMatches( RegularExpressionRef ref, CFStringRef string, NSMatchingOptions options, CFRange range );
void RegularExpressionEnumerateMatches( RegularExpressionRef ref, CFStringRef string, NSMatchingOptions options, CFRange range, CocoaRegularExpressionCallbackType callback, void *userData );
CFArrayRef RegularExpressionMatches( RegularExpressionRef ref, CFStringRef string, NSMatchingOptions options, CFRange range );
TextCheckingResultRef RegularExpressionFirstMatch( RegularExpressionRef ref, CFStringRef string, NSMatchingOptions options, CFRange range );
CFRange RegularExpressionRangeOfFirstMatch( RegularExpressionRef ref, CFStringRef string, NSMatchingOptions options, CFRange range );

// Replacing
NSUInteger RegularExpressionReplaceMatches( RegularExpressionRef ref, CFMutableStringRef string, NSMatchingOptions options, CFRange range, CFStringRef templ );
CFStringRef RegularExpressionStringByReplacingMatches( RegularExpressionRef ref, CFStringRef string, NSMatchingOptions options, CFRange range, CFStringRef templ );

// Escaping
CFStringRef RegularExpressionEscapedTemplate( CFStringRef string );
CFStringRef RegularExpressionEscapedPattern( CFStringRef string );

// Custom replace
CFStringRef RegularExpressionReplacementStringForResult( RegularExpressionRef ref, TextCheckingResultRef result, CFStringRef string, NSInteger offset, CFStringRef templ );



/*
 DataDetector.h
 */

// Create
DataDetectorRef DataDetectorWithTypes( NSTextCheckingTypes types, ErrorRef *err );

// Checking types
NSTextCheckingTypes DataDetectorCheckingTypes( DataDetectorRef ref );
