/*
 RegularExpression.m

 Bernie Wylde
 */

#import "RegularExpression.h"

// Create
RegularExpressionRef RegularExpressionWithPattern( CFStringRef pattern, NSRegularExpressionOptions options, ErrorRef *err )
{ return [NSRegularExpression regularExpressionWithPattern:(NSString *)pattern options:options error:err]; }

// Getting the regex and options
CFStringRef RegularExpressionPattern( RegularExpressionRef ref )
{ return (CFStringRef)[ref pattern]; }

NSRegularExpressionOptions RegularExpressionOptions( RegularExpressionRef ref )
{ return [ref options]; }

NSUInteger RegularExpressionNumberOfCaptureGroups( RegularExpressionRef ref )
{ return [ref numberOfCaptureGroups]; }

// Searching
NSUInteger RegularExpressionNumberOfMatches( RegularExpressionRef ref, CFStringRef string, NSMatchingOptions options, CFRange range )
{ return [ref numberOfMatchesInString:(NSString *)string options:options range:NSRangeFromCFRange(range)]; }

void RegularExpressionEnumerateMatches( RegularExpressionRef ref, CFStringRef string, NSMatchingOptions options, CFRange range, CocoaRegularExpressionCallbackType callback, void *userData )
{
    [ref enumerateMatchesInString:(NSString *)string options:options range:NSRangeFromCFRange(range) usingBlock:^(NSTextCheckingResult *result, NSMatchingFlags flags, BOOL *stop) {
        BOOL flag = (*callback)(result,flags,userData);
        if ( flag ) *stop = flag;
    }];
}

CFArrayRef RegularExpressionMatches( RegularExpressionRef ref, CFStringRef string, NSMatchingOptions options, CFRange range )
{ return (CFArrayRef)[ref matchesInString:(NSString *)string options:options range:NSRangeFromCFRange(range)]; }

TextCheckingResultRef RegularExpressionFirstMatch( RegularExpressionRef ref, CFStringRef string, NSMatchingOptions options, CFRange range )
{ return [ref firstMatchInString:(NSString *)string options:options range:NSRangeFromCFRange(range)]; }

CFRange RegularExpressionRangeOfFirstMatch( RegularExpressionRef ref, CFStringRef string, NSMatchingOptions options, CFRange range )
{ return NSRangeToCFRange([ref rangeOfFirstMatchInString:(NSString *)string options:options range:NSRangeFromCFRange(range)]); }

// Replacing
NSUInteger RegularExpressionReplaceMatches( RegularExpressionRef ref, CFMutableStringRef string, NSMatchingOptions options, CFRange range, CFStringRef templ )
{ return [ref replaceMatchesInString:(NSMutableString *)string options:options range:NSRangeFromCFRange(range) withTemplate:(NSString *)templ]; }

CFStringRef RegularExpressionStringByReplacingMatches( RegularExpressionRef ref, CFStringRef string, NSMatchingOptions options, CFRange range, CFStringRef templ )
{ return (CFStringRef)[ref stringByReplacingMatchesInString:(NSString *)string options:options range:NSRangeFromCFRange(range) withTemplate:(NSString *)templ]; }

// Escaping
CFStringRef RegularExpressionEscapedTemplate( CFStringRef string )
{ return (CFStringRef)[NSRegularExpression escapedTemplateForString:(NSString *)string]; }

CFStringRef RegularExpressionEscapedPattern( CFStringRef string )
{ return (CFStringRef)[NSRegularExpression escapedPatternForString:(NSString *)string]; }

// Custom replace
CFStringRef RegularExpressionReplacementStringForResult( RegularExpressionRef ref, TextCheckingResultRef result, CFStringRef string, NSInteger offset, CFStringRef templ )
{ return (CFStringRef)[ref replacementStringForResult:result inString:(NSString *)string offset:offset template:(NSString *)templ]; }



/*
 DataDetector.m
 */

// Create
DataDetectorRef DataDetectorWithTypes( NSTextCheckingTypes types, ErrorRef *err )
{ return [NSDataDetector dataDetectorWithTypes:types error:err]; }

// Checking types
NSTextCheckingTypes DataDetectorCheckingTypes( DataDetectorRef ref )
{ return [ref checkingTypes]; }
