/*
 RulerView.m

 Bernie Wylde
 */

#import "RulerView.h"

// Create
RulerViewRef RulerViewWithScrollView( NSInteger scrollTag, NSRulerOrientation orientation )
{
    NSScrollView *scrollView = ScrollViewWithTag( scrollTag );
    if ( scrollView ) return [[[NSRulerView alloc] initWithScrollView:scrollView orientation:orientation] autorelease];
    return nil;
}

// Measurement units
void RulerViewRegisterUnit( CFStringRef name, CFStringRef abbreviation, CGFloat unitToPointsFactor, CFArrayRef stepUpCycle, CFArrayRef stepDownCycle )
{ [NSRulerView registerUnitWithName:(NSString *)name abbreviation:(NSString *)abbreviation unitToPointsConversionFactor:unitToPointsFactor stepUpCycle:(NSArray *)stepUpCycle stepDownCycle:(NSArray *)stepDownCycle]; }

CFStringRef RulerViewMeasurementUnits( RulerViewRef ref )
{ return (CFStringRef)[ref measurementUnits]; }

void RulerViewSetMeasurementUnits( RulerViewRef ref, CFStringRef unitName )
{ [ref setMeasurementUnits:(NSString *)unitName]; }

// Client view
ViewRef RulerViewClientView( RulerViewRef ref )
{ return [ref clientView]; }

void RulerViewSetClientView( RulerViewRef ref, ViewRef clientView )
{ [ref setClientView:clientView]; }

// Accessory view
ViewRef RulerViewAccessoryView( RulerViewRef ref )
{ return [ref accessoryView]; }

void RulerViewSetAccessoryView( RulerViewRef ref, ViewRef accessoryView )
{ [ref setAccessoryView:accessoryView]; }

// Zero mark position
CGFloat RulerViewOriginOffset( RulerViewRef ref )
{ return [ref originOffset]; }

void RulerViewSetOriginOffset( RulerViewRef ref, CGFloat offset )
{ [ref setOriginOffset:offset]; }

// Markers
CFArrayRef RulerViewMarkers( RulerViewRef ref )
{ return (CFArrayRef)[ref markers]; }

void RulerViewSetMarkers( RulerViewRef ref, CFArrayRef markers )
{ [ref setMarkers:(NSArray *)markers]; }

void RulerViewAddMarker( RulerViewRef ref, RulerMarkerRef marker )
{ [ref addMarker:marker]; }

void RulerViewRemoveMarker( RulerViewRef ref, RulerMarkerRef marker )
{ [ref removeMarker:marker]; }

// Temporary ruler lines
void RulerViewMoveRulerline( RulerViewRef ref, CGFloat fromLocation, CGFloat toLocation )
{ [ref moveRulerlineFromLocation:fromLocation toLocation:toLocation]; }

// Drawing
void RulerViewDrawHashMarksAndLabelsInRect( RulerViewRef ref, CGRect r )
{ [ref drawHashMarksAndLabelsInRect:NSRectFromCGRect(r)]; }

void RulerViewDrawMarkersInRect( RulerViewRef ref, CGRect r )
{ [ref drawMarkersInRect:NSRectFromCGRect(r)]; }

void RulerViewInvalidateHashMarks( RulerViewRef ref )
{ [ref invalidateHashMarks]; }

// Layout
NSInteger RulerViewScrollView( RulerViewRef ref )
{
    NSScrollView *scrollView = [ref scrollView];
    if ( scrollView ) return [scrollView tag];
    return 0;
}

void RulerViewSetScrollView( RulerViewRef ref, NSInteger scrollTag )
{
    NSScrollView *scrollView = ScrollViewWithTag( scrollTag );
    if ( scrollView ) [ref setScrollView:scrollView];
}

NSRulerOrientation RulerViewOrientation( RulerViewRef ref )
{ return [ref orientation]; }

void RulerViewSetOrientation( RulerViewRef ref, NSRulerOrientation orientation )
{ [ref setOrientation:orientation]; }

CGFloat RulerViewReservedThicknessForAccessoryView( RulerViewRef ref )
{ return [ref reservedThicknessForAccessoryView]; }

void RulerViewSetReservedThicknessForAccessoryView( RulerViewRef ref, CGFloat thickness )
{ [ref setReservedThicknessForAccessoryView:thickness]; }

CGFloat RulerViewReservedThicknessForMarkers( RulerViewRef ref )
{ return [ref reservedThicknessForMarkers]; }

void RulerViewSetReservedThicknessForMarkers( RulerViewRef ref, CGFloat thickness )
{ [ref setReservedThicknessForMarkers:thickness]; }

CGFloat RulerViewRuleThickness( RulerViewRef ref )
{ return [ref ruleThickness]; }

void RulerViewSetRuleThickness( RulerViewRef ref, CGFloat thickness )
{ [ref setRuleThickness:thickness]; }

CGFloat RulerViewRequiredThickness( RulerViewRef ref )
{ return [ref requiredThickness]; }

CGFloat RulerViewBaselineLocation( RulerViewRef ref )
{ return [ref baselineLocation]; }

Boolean RulerViewIsFlipped( RulerViewRef ref )
{ return [ref isFlipped]; }

