/*
 SavePanel.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

static NSMutableArray *sCocoaSavePanels;

// dialog events
enum {
    savePanelDidEnd = 2900
};

@interface NSSavePanel (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
@property (nonatomic,retain) NSNumber *sheetNumber;

- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
- (BOOL)sheet;
- (void)setSheet:(BOOL)sheet;
@end

void CocoaSavePanelAdd( NSSavePanel *sp );
void CocoaSavePanelRemove( NSSavePanel *sp );
SavePanelRef CocoaSavePanelWithTag( NSInteger tag );

SavePanelRef SavePanelWithTag( NSInteger tag );
BOOL SavePanelExists( NSInteger tag );

// Configure
ViewRef SavePanelAccessoryView( NSInteger tag );
void SavePanelSetAccessoryView( NSInteger tag, ViewRef ref );
CFStringRef SavePanelTitle( NSInteger tag );
void SavePanelSetTitle( NSInteger tag, CFStringRef title );
CFStringRef SavePanelPrompt( NSInteger tag );
void SavePanelSetPrompt( NSInteger tag, CFStringRef prompt );
CFStringRef SavePanelNameFieldLabel( NSInteger tag );
void SavePanelSetNameFieldLabel( NSInteger tag, CFStringRef label );
CFStringRef SavePanelMessage( NSInteger tag );
void SavePanelSetMessage( NSInteger tag, CFStringRef message );
Boolean SavePanelCanCreateDirectories( NSInteger tag );
void SavePanelSetCanCreateDirectories( NSInteger tag, Boolean flag );
Boolean SavePanelShowsHiddenFiles( NSInteger tag );
void SavePanelSetShowsHiddenFiles( NSInteger tag, Boolean flag );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
Boolean SavePanelShowsTagField( NSInteger tag );
void SavePanelSetShowsTagField( NSInteger tag, Boolean flag );
CFArrayRef SavePanelTagNames( NSInteger tag );
void SavePanelSetTagNames( NSInteger tag, CFArrayRef tagNames );
#endif

// Panel content
Boolean SavePanelIsExtensionHidden( NSInteger tag );
void SavePanelSetExtensionHidden( NSInteger tag, Boolean flag );
CFURLRef SavePanelDirectoryURL( NSInteger tag );
void SavePanelSetDirectoryURL( NSInteger tag, CFURLRef url );
Boolean SavePanelCanSelectHiddenExtension( NSInteger tag );
void SavePanelSetCanSelectHiddenExtension( NSInteger tag, Boolean flag );
CFArrayRef SavePanelAllowedFileTypes( NSInteger tag );
void SavePanelSetAllowedFileTypes( NSInteger tag, CFArrayRef fileTypes );
Boolean SavePanelAllowsOtherFileTypes( NSInteger tag );
void SavePanelSetAllowsOtherFileTypes( NSInteger tag, Boolean flag );
Boolean SavePanelTreatsFilePackagesAsDirectories( NSInteger tag );
void SavePanelSetTreatsFilePackagesAsDirectories( NSInteger tag, Boolean flag );

// Running
void SavePanelBeginSheetModal( NSInteger tag, NSInteger wndTag, CocoaOpenSavePanelCallbackType callback, void *userData );
void SavePanelBegin( NSInteger tag, CocoaOpenSavePanelCallbackType callback, void *userData );
NSInteger SavePanelRunModal( NSInteger tag );
void SavePanelValidateVisibleColumns( NSInteger tag );

// User selection
CFURLRef SavePanelURL( NSInteger tag );
Boolean SavePanelIsExpanded( NSInteger tag );
CFStringRef SavePanelNameFieldStringValue( NSInteger tag );
void SavePanelSetNameFieldStringValue( NSInteger tag, CFStringRef string );

// Actions
void SavePanelOK( NSInteger tag );
void SavePanelCancel( NSInteger tag );

// Custom
void SavePanelRemove( NSInteger tag );

//NSInteger CocoaSavePanel( NSInteger inTag, CFStringRef inMsg, CFTypeRef inFileTypes, CFStringRef inNameFldString, CFStringRef inPrompt, CFURLRef inDirURL, NSInteger inSheetFlag );

id CocoaSavePanelSetDialogEventObjects( NSSavePanel *sp, NSInteger response );


// Init
SavePanelRef SavePanelInit( NSInteger inTag );

// Statement
CFTypeRef CocoaSavePanel( NSInteger inTag, CFStringRef inMsg, CFTypeRef inFileTypes, CFStringRef inNameFieldString, CFStringRef inPrompt, CFURLRef inDirURL, NSInteger inSheetFlag );

