/*
 SavePanel.m

 Bernie Wylde
 */

#import "SavePanel.h"

SavePanelRef CocoaSavePanelWithTag( NSInteger tag )
{
    if ( tag ) {
        if ( tag > 1000000 ) return (NSSavePanel *)tag;// it's a SavePanelRef
        
        if ( sCocoaSavePanels ) {
            for ( NSSavePanel *sp in sCocoaSavePanels ) {
                if ( [sp tag] == tag ) return sp;
            }
        }
    }
    return nil;
}

void CocoaSavePanelAdd( NSSavePanel *sp )
{
    if ( !sCocoaSavePanels ) sCocoaSavePanels = [[NSMutableArray alloc] initWithCapacity:0];
    [sCocoaSavePanels addObject:sp];
}

void CocoaSavePanelRemove( NSSavePanel *sp )
{ [sCocoaSavePanels removeObject:sp]; }


@implementation NSSavePanel (Additions)
@dynamic tagNumber;
@dynamic sheetNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (BOOL)sheet {
    return [objc_getAssociatedObject(self, @selector(sheetNumber)) boolValue];
}

- (void)setSheet:(BOOL)sheet {
    objc_setAssociatedObject(self, @selector(sheetNumber), [NSNumber numberWithBool:sheet], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

SavePanelRef SavePanelWithTag( NSInteger tag )
{ return CocoaSavePanelWithTag( tag ); }

BOOL SavePanelExists( NSInteger tag )
{
    if ( sCocoaSavePanels ) {
        for ( NSSavePanel *sp in sCocoaSavePanels ) {
            if ( [sp tag] == tag ) return YES;
        }
    }
    return NO;
}


// Configure
ViewRef SavePanelAccessoryView( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return [sp accessoryView];
    return nil;
}

void SavePanelSetAccessoryView( NSInteger tag, ViewRef ref )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setAccessoryView:ref];
}

CFStringRef SavePanelTitle( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return (CFStringRef)[sp title];
    return nil;
}

void SavePanelSetTitle( NSInteger tag, CFStringRef title )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setTitle:(NSString *)title];
}

CFStringRef SavePanelPrompt( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return (CFStringRef)[sp prompt];
    return nil;
}

void SavePanelSetPrompt( NSInteger tag, CFStringRef prompt )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setPrompt:(NSString *)prompt];
}

CFStringRef SavePanelNameFieldLabel( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return (CFStringRef)[sp nameFieldLabel];
    return nil;
}

void SavePanelSetNameFieldLabel( NSInteger tag, CFStringRef label )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setNameFieldLabel:(NSString *)label];
}

CFStringRef SavePanelMessage( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return (CFStringRef)[sp message];
    return nil;
}

void SavePanelSetMessage( NSInteger tag, CFStringRef message )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setMessage:(NSString *)message];
}

Boolean SavePanelCanCreateDirectories( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return [sp canCreateDirectories];
    return NO;
}

void SavePanelSetCanCreateDirectories( NSInteger tag, Boolean flag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setCanCreateDirectories:flag];
}

Boolean SavePanelShowsHiddenFiles( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return [sp showsHiddenFiles];
    return NO;
}

void SavePanelSetShowsHiddenFiles( NSInteger tag, Boolean flag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setShowsHiddenFiles:flag];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
Boolean SavePanelShowsTagField( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return [sp showsTagField];
    return NO;
}

void SavePanelSetShowsTagField( NSInteger tag, Boolean flag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setShowsTagField:flag];
}

CFArrayRef SavePanelTagNames( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return (CFArrayRef)[sp tagNames];
    return nil;
}

void SavePanelSetTagNames( NSInteger tag, CFArrayRef tagNames )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setTagNames:(NSArray *)tagNames];
}
#endif

// Panel content
Boolean SavePanelIsExtensionHidden( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return [sp isExtensionHidden];
    return NO;
}

void SavePanelSetExtensionHidden( NSInteger tag, Boolean flag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setExtensionHidden:flag];
}

CFURLRef SavePanelDirectoryURL( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return (CFURLRef)[sp directoryURL];
    return nil;
}

void SavePanelSetDirectoryURL( NSInteger tag, CFURLRef url )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setDirectoryURL:(NSURL *)url];
}

Boolean SavePanelCanSelectHiddenExtension( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return [sp canSelectHiddenExtension];
    return NO;
}

void SavePanelSetCanSelectHiddenExtension( NSInteger tag, Boolean flag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setCanSelectHiddenExtension:flag];
}

CFArrayRef SavePanelAllowedFileTypes( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return (CFArrayRef)[sp allowedFileTypes];
    return nil;
}

void SavePanelSetAllowedFileTypes( NSInteger tag, CFArrayRef fileTypes )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setAllowedFileTypes:(NSArray *)fileTypes];
}

Boolean SavePanelAllowsOtherFileTypes( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return [sp allowsOtherFileTypes];
    return NO;
}

void SavePanelSetAllowsOtherFileTypes( NSInteger tag, Boolean flag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setAllowsOtherFileTypes:flag];
}

Boolean SavePanelTreatsFilePackagesAsDirectories( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return [sp treatsFilePackagesAsDirectories];
    return NO;
}

void SavePanelSetTreatsFilePackagesAsDirectories( NSInteger tag, Boolean flag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setTreatsFilePackagesAsDirectories:flag];
}

// Running
void SavePanelBeginSheetModal( NSInteger tag, NSInteger wndTag, CocoaOpenSavePanelCallbackType callback, void *userData )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) {
        NSWindow *window = WindowWithTag( wndTag );
        if ( window ) {
            [sp beginSheetModalForWindow: window completionHandler:^(NSInteger returnCode) {
                if ( callback ) (*callback)(returnCode,userData);
            }];
        }
    }
}

void SavePanelBegin( NSInteger tag, CocoaOpenSavePanelCallbackType callback, void *userData )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) {
        [sp beginWithCompletionHandler:^(NSInteger returnCode) {
            if ( callback ) (*callback)(returnCode,userData);
        }];
    }
}

NSInteger SavePanelRunModal( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return [sp runModal];
    return 0;
}

void SavePanelValidateVisibleColumns( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp validateVisibleColumns];
}

// User selection
CFURLRef SavePanelURL( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return (CFURLRef)[sp URL];
    return nil;
}

Boolean SavePanelIsExpanded( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return [sp isExpanded];
    return NO;
}

CFStringRef SavePanelNameFieldStringValue( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) return (CFStringRef)[sp nameFieldStringValue];
    return nil;
}

void SavePanelSetNameFieldStringValue( NSInteger tag, CFStringRef string )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp setNameFieldStringValue:(NSString *)string];
}

// Actions
void SavePanelOK( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp ok:nil];
}

void SavePanelCancel( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) [sp cancel:nil];
}

// -- custom --
void SavePanelRemove( NSInteger tag )
{
    NSSavePanel *sp = CocoaSavePanelWithTag( tag );
    if ( sp ) {
        [sCocoaSavePanels removeObject:sp];
    }
}



//NSInteger CocoaSavePanel( NSInteger inTag, CFStringRef inMsg, CFTypeRef inFileTypes, CFStringRef inNameFieldString, CFStringRef inPrompt, CFURLRef inDirURL, NSInteger inSheetFlag )
//{
//    NSInteger response = 0;
//
//    // tag
//    BOOL show = ( inTag > 0 );
//    inTag = ABS(inTag);
//
//    // msg
//    NSString *msg = (NSString *)inMsg;
//
//    // nameFldString
//    NSString *nameFieldString = (NSString *)inNameFieldString;
//
//    // prompt
//    NSString *prompt = (NSString *)inPrompt;
//
//    // dirURL
//    NSURL *dirURL = (NSURL *)inDirURL;
//
//    NSSavePanel *sp = CocoaSavePanelWithTag( inTag );
//
//    if ( sp ) {     // exists
//
//        // sheet
//        //        if ( inSheetFlag != kFBParamMissing ) [sp setSheet:inSheetFlag];
//
//    } else {
//
//        // init
//        sp = [NSSavePanel savePanel];
//
//        // sheet
//        //        if ( inSheetFlag != kFBParamMissing ) [sp setSheet:inSheetFlag];
//
//        // tag
//        [sp setTag:inTag];
//
//        // add to open panels array
//        CocoaSavePanelAdd( sp );
//
//    }
//
//    // msg
//    if ( msg ) [sp setMessage:msg];
//
//    // allowed file types
//    if ( inFileTypes ) {
//        NSArray *fileTypes = nil;
//
//        if ( CFGetTypeID(inFileTypes) == CFStringGetTypeID() ) {
//            fileTypes = [(NSString *)inFileTypes componentsSeparatedByString:@";"];
//
//        } else if ( CFGetTypeID(inFileTypes) == CFArrayGetTypeID() ) {
//            fileTypes = (NSArray *)inFileTypes;
//        }
//
//        if ( fileTypes ) {
//            [sp setAllowedFileTypes:fileTypes];
//        }
//    }
//
//    // nameFieldSring
//    if ( nameFieldString ) [sp setNameFieldStringValue:nameFieldString];
//
//    // prompt
//    if ( prompt ) [sp setPrompt:prompt];
//
//    // dirURL
//    if ( dirURL ) [sp setDirectoryURL:dirURL];
//
//    // show
//    if ( ![sp isVisible] ) {
//
//        // sheet
//        if ( inSheetFlag != kFBParamMissing ) [sp setSheet:inSheetFlag];
//
//        if ( show ) {
//            // get front window
//            NSInteger wndTag = 0;
//
//            // NSWindow *window = CocoaTargetWindow( &wndTag );
//            NSWindow *window = CocoaVisibleTargetWindow( &wndTag );
//
//            if ( [sp sheet] && window ) {  // sheet
//
//                // beginSheet
//                [sp beginSheetModalForWindow:window completionHandler:^(NSInteger result) {
//
//                    CallUserDefinedOnDialogFunction( savePanelDidEnd, result, [sp tag], NULL );
//
//                }];
//
//            } else {
//
//                response = [sp runModal];
//                CallUserDefinedOnDialogFunction( savePanelDidEnd, response, [sp tag], NULL );
//
//            }
//
//        }
//    }
//
//    return response;
//}


id CocoaSavePanelSetDialogEventObjects( NSSavePanel *sp, NSInteger response )
{
    id obj = nil;
    if ( response == NSOKButton ) {
        obj = [sp URL];
        DialogEventSetNSURL( obj );
    } else {
        DialogEventSetNSURL( nil );
    }
    return obj;
}


// Init
SavePanelRef SavePanelInit( NSInteger inTag )
{
    // init
    NSSavePanel *sp = [NSSavePanel savePanel];
    
    // tag
    [sp setTag:inTag];
    
    // add to save panels array
    CocoaSavePanelAdd( sp );

    return sp;
}


// Statement
CFTypeRef CocoaSavePanel( NSInteger inTag, CFStringRef inMsg, CFTypeRef inFileTypes, CFStringRef inNameFieldString, CFStringRef inPrompt, CFURLRef inDirURL, NSInteger inSheetFlag )
{
    id obj = nil;
    
    // tag
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    // msg
    NSString *msg = (NSString *)inMsg;
    
    // nameFldString
    NSString *nameFieldString = (NSString *)inNameFieldString;
    
    // prompt
    NSString *prompt = (NSString *)inPrompt;
    
    // dirURL
    NSURL *dirURL = (NSURL *)inDirURL;
    
    NSSavePanel *sp = CocoaSavePanelWithTag( inTag );
    
    if ( sp ) {     // exists
        
        // sheet
        //        if ( inSheetFlag != kFBParamMissing ) [sp setSheet:inSheetFlag];
        
    } else {
        
        // init
        sp = SavePanelInit( inTag );
        
        // sheet
        //        if ( inSheetFlag != kFBParamMissing ) [sp setSheet:inSheetFlag];
        
        // tag
//        [sp setTag:inTag];
        
        // add to save panels array
 //       CocoaSavePanelAdd( sp );
        
    }
    
    // msg
    if ( msg ) [sp setMessage:msg];
    
    // allowed file types
    if ( inFileTypes ) {
        NSArray *fileTypes = nil;
        
        if ( CFGetTypeID(inFileTypes) == CFStringGetTypeID() ) {
            fileTypes = [(NSString *)inFileTypes componentsSeparatedByString:@";"];
            
        } else if ( CFGetTypeID(inFileTypes) == CFArrayGetTypeID() ) {
            fileTypes = (NSArray *)inFileTypes;
        }
        
        if ( fileTypes ) {
            [sp setAllowedFileTypes:fileTypes];
        }
    }
    
    // nameFieldSring
    if ( nameFieldString ) [sp setNameFieldStringValue:nameFieldString];
    
    // prompt
    if ( prompt ) [sp setPrompt:prompt];
    
    // dirURL
    if ( dirURL ) [sp setDirectoryURL:dirURL];
    
    // show
    if ( ![sp isVisible] ) {
        
        // sheet
        if ( inSheetFlag != kFBParamMissing ) [sp setSheet:inSheetFlag];
        
        if ( show ) {
            // get front window
            NSInteger wndTag = 0;
            
            // NSWindow *window = CocoaTargetWindow( &wndTag );
            NSWindow *window = CocoaVisibleTargetWindow( &wndTag );
            
            if ( [sp sheet] && window ) {  // sheet
                
                // beginSheet
                [sp beginSheetModalForWindow:window completionHandler:^(NSInteger result) {
                    
                    id obj = nil;
//                    if ( result == NSOKButton ) { // NSModalResponseOK
                        obj = CocoaSavePanelSetDialogEventObjects( sp, result );
//                    }

                    CallUserDefinedOnDialogFunction( savePanelDidEnd, result, [sp tag], (CFTypeRef)obj );
                    
                    CocoaSavePanelRemove( sp );
                }];
                
            } else {
                
                NSInteger response = [sp runModal];
                
//                if ( response == NSOKButton ) { // NSModalResponseOK
                    obj = CocoaSavePanelSetDialogEventObjects( sp, response );
//                }

                CallUserDefinedOnDialogFunction( savePanelDidEnd, response, [sp tag], (CFTypeRef)obj );
                
                CocoaSavePanelRemove( sp );

            }
            
        }
    }
    
    return (CFTypeRef)obj;
}
