/*
 ScrollView.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

@interface ScrollViewSC : NSScrollView {
    BOOL scrollingDisabled;
}
@property (assign) BOOL scrollingDisabled;
@end

@interface NSScrollView (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;

- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

NSScrollView *ScrollViewWithTag( NSInteger tag );
BOOL ScrollViewExists( NSInteger tag );

ScrollerRef ScrollViewHorizontalScroller( NSInteger tag );
Boolean ScrollViewHasHorizontalScroller( NSInteger tag );
void ScrollViewSetHasHorizontalScroller( NSInteger tag, Boolean flag );
ScrollerRef ScrollViewVerticalScroller( NSInteger tag );
Boolean ScrollViewHasVerticalScroller( NSInteger tag );
void ScrollViewSetHasVerticalScroller( NSInteger tag, Boolean flag );
Boolean ScrollViewAutohidesScrollers( NSInteger tag );
void ScrollViewSetAutohidesScrollers( NSInteger tag, Boolean flag );
void ScrollViewFlashScrollers( NSInteger tag );
ClipViewRef ScrollViewContentView( NSInteger tag );
void ScrollViewSetContentView( NSInteger tag, ClipViewRef clip );
void ScrollViewSetDocumentView( NSInteger scrollTag, NSInteger docTag );
CGSize ScrollViewContentSize( NSInteger tag );
CGRect ScrollViewDocumentVisibleRect( NSInteger tag );
ColorRef ScrollViewBackgroundColor( NSInteger tag );
void ScrollViewSetBackgroundColor( NSInteger tag, ColorRef col );
Boolean ScrollViewDrawsBackground( NSInteger tag );
void ScrollViewSetDrawsBackground( NSInteger tag, Boolean flag );
NSBorderType ScrollViewBorderType( NSInteger tag );
void ScrollViewSetBorderType( NSInteger tag, NSBorderType type );
CursorRef ScrollViewDocumentCursor( NSInteger tag );
void ScrollViewSetDocumentCursor( NSInteger tag, CursorRef ref );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ScrollViewAddFloatingSubview( NSInteger scrollTag, NSInteger subviewTag, NSEventGestureAxis axis );
#endif

Boolean ScrollViewHasHorizontalRuler( NSInteger tag );
void ScrollViewSetHasHorizontalRuler( NSInteger tag, Boolean flag );

RulerViewRef ScrollViewHorizontalRulerView( NSInteger tag );
void ScrollViewSetHorizontalRulerView( NSInteger tag, RulerViewRef ruler );

Boolean ScrollViewHasVerticalRuler( NSInteger tag );
void ScrollViewSetHasVerticalRuler( NSInteger tag, Boolean flag );

RulerViewRef ScrollViewVerticalRulerView( NSInteger tag );
void ScrollViewSetVerticalRulerView( NSInteger tag, RulerViewRef ruler );

Boolean ScrollViewRulersVisible( NSInteger tag );
void ScrollViewSetRulersVisible( NSInteger tag, Boolean flag );

// Insets
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean ScrollViewAutomaticallyAdjustsContentInsets( NSInteger tag );
void ScrollViewSetAutomaticallyAdjustsContentInsets( NSInteger tag, Boolean flag );
NSEdgeInsets ScrollViewContentInsets( NSInteger tag );
void ScrollViewSetContentInsets( NSInteger tag, NSEdgeInsets insets );
NSEdgeInsets ScrollViewScrollerInsets( NSInteger tag );
void ScrollViewSetScrollerInsets( NSInteger tag, NSEdgeInsets insets );
#endif

// Style
NSScrollerKnobStyle ScrollViewScrollerKnobStyle( NSInteger tag );
void ScrollViewSetScrollerKnobStyle( NSInteger tag, NSScrollerKnobStyle style );
NSScrollerStyle ScrollViewScrollerStyle( NSInteger tag );
void ScrollViewSetScrollerStyle( NSInteger tag, NSScrollerStyle style );

// Behavior
CGFloat ScrollViewLineScroll( NSInteger tag );
void ScrollViewSetLineScroll( NSInteger tag, CGFloat value );
CGFloat ScrollViewHorizontalLineScroll( NSInteger tag );
void ScrollViewSetHorizontalineScroll( NSInteger tag, CGFloat value );
CGFloat ScrollViewVerticalLineScroll( NSInteger tag );
void ScrollViewSetVerticalLineScroll( NSInteger tag, CGFloat value );
CGFloat ScrollViewPageScroll( NSInteger tag );
void ScrollViewSetPageScroll( NSInteger tag, CGFloat value );
CGFloat ScrollViewHorizontalPageScroll( NSInteger tag );
void ScrollViewSetHorizontalPageScroll( NSInteger tag, CGFloat value );
CGFloat ScrollViewVerticalPageScroll( NSInteger tag );
void ScrollViewSetVerticalPageScroll( NSInteger tag, CGFloat value );
Boolean ScrollViewScrollsDynamically( NSInteger tag );
void ScrollViewSetScrollsDynamically( NSInteger tag, Boolean flag );
void ScrollViewScrollWheel( NSInteger tag, CocoaEventRef evnt );


void ScrollViewReflectScrolledClipView( NSInteger tag );

NSScrollViewFindBarPosition ScrollViewFindBarPosition( NSInteger tag );
void ScrollViewSetFindBarPosition( NSInteger tag, NSScrollViewFindBarPosition position );

// Predominant behavior
Boolean ScrollViewUsesPredominantAxisScrolling( NSInteger tag );
void ScrollViewSetUsesPredominantAxisScrolling( NSInteger tag, Boolean flag );

// Elasticity -
NSInteger ScrollViewHorizontalScrollElasticity( NSInteger tag );
void ScrollViewSetHorizontalScrollElasticity( NSInteger tag, NSInteger elasticity );
NSInteger ScrollViewVerticalScrollElasticity( NSInteger tag );
void ScrollViewSetVerticalScrollElasticity( NSInteger tag, NSInteger elasticity );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
Boolean ScrollViewAllowsMagnification( NSInteger tag );
void ScrollViewSetAllowsMagnification( NSInteger tag, Boolean flag );
CGFloat ScrollViewMagnification( NSInteger tag );
void ScrollViewSetMagnification( NSInteger tag, CGFloat value );
void ScrollViewMagnifyToFitRect( NSInteger tag, CGRect r );
CGFloat ScrollViewMaxMagnification( NSInteger tag );
void ScrollViewSetMaxMagnification( NSInteger tag, CGFloat value );
CGFloat ScrollViewMinMagnification( NSInteger tag );
void ScrollViewSetMinMagnification( NSInteger tag, CGFloat value );
void ScrollViewSetMagnificationCenteredAtPoint( NSInteger tag, CGFloat value, CGPoint pt );
#endif

// -- custom --
void ScrollViewScrollToPoint( NSInteger tag, CGPoint pt );
//void ScrollViewScrollToTop( NSInteger tag );

// Subclass functions
void ScrollViewSetScrollingDisabled( NSInteger tag, Boolean flag );
Boolean ScrollViewIsScrollingDisabled( NSInteger tag );


// Init
ScrollViewRef ScrollViewInit( NSInteger inTag, CGRect inRect );

// Statement
void CocoaScrollView( NSInteger inTag, CGRect inRect, NSInteger inBorderType, NSInteger inDocTag, NSInteger inWndTag );

