/*
 ScrollView.m

 Bernie Wylde
 */

#import "ScrollView.h"


#pragma mark - subclass
@implementation ScrollViewSC

@synthesize scrollingDisabled;

- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (void)scrollWheel:(NSEvent *)theEvent {
    if ( !scrollingDisabled ) {
        [super scrollWheel:theEvent];
    }
}
@end


@implementation NSScrollView (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

NSScrollView *ScrollViewWithTag( NSInteger tag )
{ return (NSScrollView *)CocoaValidateViewKindOfClass( tag, [NSScrollView class] ); }

BOOL ScrollViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSScrollView class] ); }


ScrollerRef ScrollViewHorizontalScroller( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView horizontalScroller];
    return nil;
}

Boolean ScrollViewHasHorizontalScroller( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView hasHorizontalScroller];
    return NO;
}

void ScrollViewSetHasHorizontalScroller( NSInteger tag, Boolean flag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) {
        [scrollView setHasHorizontalScroller:flag];
    }
}

ScrollerRef ScrollViewVerticalScroller( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView verticalScroller];
    return nil;
}

Boolean ScrollViewHasVerticalScroller( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView hasVerticalScroller];
    return NO;
}

void ScrollViewSetHasVerticalScroller( NSInteger tag, Boolean flag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) {
        [scrollView setHasVerticalScroller:flag];
    }
}

Boolean ScrollViewAutohidesScrollers( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView autohidesScrollers];
    return NO;
}

void ScrollViewSetAutohidesScrollers( NSInteger tag, Boolean flag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) {
        [scrollView setAutohidesScrollers:flag];
    }
}

void ScrollViewFlashScrollers( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) {
        [scrollView flashScrollers];
    }
}

ClipViewRef ScrollViewContentView( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView contentView];
    return nil;
}
void ScrollViewSetContentView( NSInteger tag, ClipViewRef clip )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setContentView:clip];
}

void ScrollViewSetDocumentView( NSInteger scrollTag, NSInteger docTag )
{
    NSScrollView *scrollView = ScrollViewWithTag( scrollTag );
    if ( scrollView ) {
        NSView *docView = CocoaViewWithTag( docTag );
        if ( docView ) [scrollView setDocumentView:docView];
    }
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ScrollViewAddFloatingSubview( NSInteger scrollTag, NSInteger subviewTag, NSEventGestureAxis axis )
{
    NSScrollView *scrollView = ScrollViewWithTag( scrollTag );
    if ( scrollView ) {
        NSView *subview = ViewWithTag( subviewTag );
        if ( subview ) [scrollView addFloatingSubview:subview forAxis:axis];
    }
}
#endif

CGSize ScrollViewContentSize( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return NSSizeToCGSize([scrollView contentSize]);
    return CGSizeZero;
}

CGRect ScrollViewDocumentVisibleRect( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return NSRectToCGRect([scrollView documentVisibleRect]);
    return CGRectZero;
}

ColorRef ScrollViewBackgroundColor( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView backgroundColor];
    return NULL;
}

void ScrollViewSetBackgroundColor( NSInteger tag, ColorRef col )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setBackgroundColor:col];
}

Boolean ScrollViewDrawsBackground( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView drawsBackground];
    return NO;
}

void ScrollViewSetDrawsBackground( NSInteger tag, Boolean flag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setDrawsBackground:flag];
}

NSBorderType ScrollViewBorderType( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView borderType];
    return 0;
}

void ScrollViewSetBorderType( NSInteger tag, NSBorderType type )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setBorderType:type];
}

CursorRef ScrollViewDocumentCursor( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView documentCursor];
    return NULL;
}

void ScrollViewSetDocumentCursor( NSInteger tag, CursorRef ref )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setDocumentCursor:ref];
}

Boolean ScrollViewHasHorizontalRuler( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView hasHorizontalRuler];
    return NO;
}

void ScrollViewSetHasHorizontalRuler( NSInteger tag, Boolean flag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setHasHorizontalRuler:flag];
}

RulerViewRef ScrollViewHorizontalRulerView( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView horizontalRulerView];
    return nil;
}

void ScrollViewSetHorizontalRulerView( NSInteger tag, RulerViewRef ruler )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setHorizontalRulerView:ruler];
}

Boolean ScrollViewHasVerticalRuler( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView hasVerticalRuler];
    return NO;
}

void ScrollViewSetHasVerticalRuler( NSInteger tag, Boolean flag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setHasVerticalRuler:flag];
}

RulerViewRef ScrollViewVerticalRulerView( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView verticalRulerView];
    return nil;
}

void ScrollViewSetVerticalRulerView( NSInteger tag, RulerViewRef ruler )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setVerticalRulerView:ruler];
}

Boolean ScrollViewRulersVisible( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView rulersVisible];
    return NO;
}

void ScrollViewSetRulersVisible( NSInteger tag, Boolean flag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setRulersVisible:flag];
}

// Insets
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean ScrollViewAutomaticallyAdjustsContentInsets( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView automaticallyAdjustsContentInsets];
    return NO;
}

void ScrollViewSetAutomaticallyAdjustsContentInsets( NSInteger tag, Boolean flag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setAutomaticallyAdjustsContentInsets:flag];
}

NSEdgeInsets ScrollViewContentInsets( NSInteger tag )
{
    NSEdgeInsets insets = {0,0,0,0};
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView contentInsets];
    return insets;
}

void ScrollViewSetContentInsets( NSInteger tag, NSEdgeInsets insets )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setContentInsets:insets];
}

NSEdgeInsets ScrollViewScrollerInsets( NSInteger tag )
{
    NSEdgeInsets insets = {0,0,0,0};
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView scrollerInsets];
    return insets;
}

void ScrollViewSetScrollerInsets( NSInteger tag, NSEdgeInsets insets )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setScrollerInsets:insets];
}
#endif

// Style
NSScrollerKnobStyle ScrollViewScrollerKnobStyle( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView scrollerKnobStyle];
    return 0;
}

void ScrollViewSetScrollerKnobStyle( NSInteger tag, NSScrollerKnobStyle style )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setScrollerKnobStyle:style];
}

NSScrollerStyle ScrollViewScrollerStyle( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView scrollerStyle];
    return 0;
}

void ScrollViewSetScrollerStyle( NSInteger tag, NSScrollerStyle style )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setScrollerStyle:style];
}

CGFloat ScrollViewLineScroll( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView lineScroll];
    return 0.0;
}

void ScrollViewSetLineScroll( NSInteger tag, CGFloat value )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setLineScroll:value];
}

CGFloat ScrollViewHorizontalLineScroll( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView horizontalLineScroll];
    return 0.0;
}

void ScrollViewSetHorizontalineScroll( NSInteger tag, CGFloat value )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setHorizontalLineScroll:value];
}

CGFloat ScrollViewVerticalLineScroll( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView verticalLineScroll];
    return 0.0;
}

void ScrollViewSetVerticalLineScroll( NSInteger tag, CGFloat value )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setVerticalLineScroll:value];
}

CGFloat ScrollViewPageScroll( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView pageScroll];
    return 0.0;
}

void ScrollViewSetPageScroll( NSInteger tag, CGFloat value )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setPageScroll:value];
}

CGFloat ScrollViewHorizontalPageScroll( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView horizontalPageScroll];
    return 0.0;
}

void ScrollViewSetHorizontalPageScroll( NSInteger tag, CGFloat value )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setHorizontalPageScroll:value];
}

CGFloat ScrollViewVerticalPageScroll( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView verticalPageScroll];
    return 0.0;
}

void ScrollViewSetVerticalPageScroll( NSInteger tag, CGFloat value )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setVerticalPageScroll:value];
}

Boolean ScrollViewScrollsDynamically( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView scrollsDynamically];
    return NO;
}

void ScrollViewScrollWheel( NSInteger tag, CocoaEventRef evnt )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView scrollWheel:evnt];
}

void ScrollViewSetScrollsDynamically( NSInteger tag, Boolean flag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setScrollsDynamically:flag];
}

NSScrollViewFindBarPosition ScrollViewFindBarPosition( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView findBarPosition];
    return 0;
}

void ScrollViewSetFindBarPosition( NSInteger tag, NSScrollViewFindBarPosition position )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setFindBarPosition:position];
}

// Predominant behavior
Boolean ScrollViewUsesPredominantAxisScrolling( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView usesPredominantAxisScrolling];
    return NO;
}

void ScrollViewSetUsesPredominantAxisScrolling( NSInteger tag, Boolean flag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setUsesPredominantAxisScrolling:flag];
}

// Elasticity -
NSInteger ScrollViewHorizontalScrollElasticity( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView horizontalScrollElasticity];
    return 0;
}

void ScrollViewSetHorizontalScrollElasticity( NSInteger tag, NSInteger elasticity )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setHorizontalScrollElasticity:elasticity];
}

NSInteger ScrollViewVerticalScrollElasticity( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView verticalScrollElasticity];
    return 0;
}

void ScrollViewSetVerticalScrollElasticity( NSInteger tag, NSInteger elasticity )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setVerticalScrollElasticity:elasticity];
}


void ScrollViewReflectScrolledClipView( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView reflectScrolledClipView:[scrollView contentView]];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
Boolean ScrollViewAllowsMagnification( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView allowsMagnification];
    return NO;
}

void ScrollViewSetAllowsMagnification( NSInteger tag, Boolean flag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setAllowsMagnification:flag];
}

CGFloat ScrollViewMagnification( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView magnification];
    return 0.0;
}

void ScrollViewSetMagnification( NSInteger tag, CGFloat value )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setMagnification:value];
}

void ScrollViewMagnifyToFitRect( NSInteger tag, CGRect r )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView magnifyToFitRect:NSRectFromCGRect(r)];
}

CGFloat ScrollViewMaxMagnification( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView maxMagnification];
    return 0.0;
}

void ScrollViewSetMaxMagnification( NSInteger tag, CGFloat value )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setMaxMagnification:value];
}

CGFloat ScrollViewMinMagnification( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) return [scrollView minMagnification];
    return 0.0;
}

void ScrollViewSetMinMagnification( NSInteger tag, CGFloat value )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setMinMagnification:value];
}

void ScrollViewSetMagnificationCenteredAtPoint( NSInteger tag, CGFloat value, CGPoint pt )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [scrollView setMagnification:value centeredAtPoint:NSPointFromCGPoint(pt)];
}
#endif


// -- convenience --
void ScrollViewScrollToPoint( NSInteger tag, CGPoint pt )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) [[scrollView contentView] scrollToPoint:NSPointFromCGPoint(pt)];
}

//void ScrollViewScrollToTop( NSInteger tag )
//{
//    NSScrollView *scrollView = ScrollViewWithTag( tag );
//    if ( scrollView ) {
//        NSPoint newOrigin = NSMakePoint(0, NSMaxY([scrollView documentView].frame) - scrollView.bounds.size.height);
//        [[scrollView contentView] scrollToPoint:newOrigin];
//    }
//}


// Subclass functions
void ScrollViewSetScrollingDisabled( NSInteger tag, Boolean flag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) {
        if ( [scrollView isKindOfClass:[ScrollViewSC class]] ) {
            [(ScrollViewSC *)scrollView setScrollingDisabled:flag];
        }
    }
}

Boolean ScrollViewIsScrollingDisabled( NSInteger tag )
{
    NSScrollView *scrollView = ScrollViewWithTag( tag );
    if ( scrollView ) {
        if ( [scrollView isKindOfClass:[ScrollViewSC class]] ) {
            return [(ScrollViewSC *)scrollView scrollingDisabled];
        }
    }
    return NO;
}


// Init
ScrollViewRef ScrollViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 240, 135 );

    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    NSScrollView *view;
    if ( subclass ) {
        view = [[[ScrollViewSC alloc] initWithFrame:rect] autorelease];
    } else {
        view = [[[NSScrollView alloc] initWithFrame:rect] autorelease];
    }

    // vertical scroller
    [view setHasVerticalScroller:YES];

    // tag
    [view setTag:inTag];
    
    return view;
}


// Statement
void CocoaScrollView( NSInteger inTag, CGRect inRect, NSInteger inBorderType, NSInteger inDocTag, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSScrollView *scrollView = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !scrollView ) {
            scrollView = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( scrollView ) { // already exists
            
            // clear subclass flag
//            SetSubclassFlag( NO );
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [scrollView class], [NSScrollView class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, scrollView, [NSScrollView class] ) ) {
                return;
            }

            // rect
            if ( !NSIsEmptyRect( rect ) ) [scrollView setFrame:rect];
            
            // border type
            if ( inBorderType != kFBParamMissing) [scrollView setBorderType:inBorderType];
            
        } else {            // create
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 240, 135 );
            
            // init scroll view
//            if ( inSubclass ) {
//                scrollView = [[ScrollViewSC alloc] initWithFrame:rect];
//            } else {
//                scrollView = [[NSScrollView alloc] initWithFrame:rect];
//            }
            
            scrollView = ScrollViewInit( inTag, inRect );
            
            // vertical scroller
//            [scrollView setHasVerticalScroller:YES];
            
            // border type
            if ( inBorderType != kFBParamMissing) [scrollView setBorderType:inBorderType];
            
            // document view
            if ( inDocTag != 0 ) {
                NSView *docView = CocoaViewSubviewWithTag( contentView, inDocTag );
                
                if ( !docView ) {
                    docView = TitlebarAccessoryViewWithTag( inTag );
                }
                
                if ( docView ) {
                    [docView retain];
                    [docView removeFromSuperview];
                    [scrollView setDocumentView:docView];
                    [docView release];
                }
            }
            
            // default autoresizing
            [scrollView setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [scrollView setTag:inTag];
            
            // wndTag
            [scrollView setWndTag:inWndTag];
            
            // add to window
            [contentView addSubview:scrollView];
//            [scrollView release];
        }
        
        [scrollView setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}
