/*
 SearchField.m

 Bernie Wylde
 */

#import "SearchField.h"

@implementation SearchFieldSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end

@implementation CocoaSearchFieldDelegate

- (void)controlTextDidBeginEditing:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldDidBeginEditing, notification.object, NULL );
}

- (void)controlTextDidChange:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldDidChange, notification.object, NULL );
}

- (void)controlTextDidEndEditing:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldDidEndEditing, notification.object, NULL );
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
- (void)searchFieldDidStartSearching:(NSSearchField *)sender {
    CocoaControlCallUserDefinedOnDialogOrCallback( searchFieldDidStartSearching, sender, NULL );
}

- (void)searchFieldDidEndSearching:(NSSearchField *)sender {
    CocoaControlCallUserDefinedOnDialogOrCallback( searchFieldDidEndSearching, sender, NULL );
}
#endif  //__MAC_OS_X_VERSION_MAX_ALLOWED >= 101100

@end

CocoaSearchFieldDelegate *CocoaSearchFieldDelegateGet( void )
{
    static CocoaSearchFieldDelegate *sCocoaSearchFieldDelegate = nil;
    if ( !sCocoaSearchFieldDelegate ) sCocoaSearchFieldDelegate = [[CocoaSearchFieldDelegate alloc] init];
    return sCocoaSearchFieldDelegate;
}

NSSearchField *SearchFieldWithTag( NSInteger tag )
{ return (NSSearchField *)CocoaValidateViewKindOfClass( tag, [NSSearchField class] ); }

BOOL SearchFieldExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSSearchField class] ); }


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean SearchFieldSendsSearchStringImmediately( NSInteger tag )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) {
        return [fld sendsSearchStringImmediately];
    }
    return NO;
}

void SearchFieldSetSendsSearchStringImmediately( NSInteger tag, Boolean flag )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) {
        [fld setSendsSearchStringImmediately:flag];
    }
}

Boolean SearchFieldSendsWholeSearchString( NSInteger tag )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) {
        return [fld sendsWholeSearchString];
    }
    return NO;
}

void SearchFieldSetSendsWholeSearchString( NSInteger tag, Boolean flag )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) {
        [fld setSendsWholeSearchString:flag];
    }
}
#endif

CFArrayRef SearchFieldRecentSearches( NSInteger tag )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) return (CFArrayRef)[fld recentSearches];
    return NULL;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean SearchFieldCentersPlaceholder( NSInteger tag )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) {
        return [fld centersPlaceholder];
    }
    return NO;
}

void SearchFieldSetCentersPlaceholder( NSInteger tag, Boolean flag )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) {
        [fld setCentersPlaceholder:flag];
    }
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSInteger SearchFieldMaximumRecents( NSInteger tag )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) {
        return [fld maximumRecents];
    }
    return 0;
}

void SearchFieldSetMaximumRecents( NSInteger tag, NSInteger value )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) {
        [fld setMaximumRecents:value];
    }
}
#endif

CFStringRef SearchFieldRecentsAutosaveName( NSInteger tag )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) {
        return (CFStringRef)[fld recentsAutosaveName];
    }
    return nil;
}

void SearchFieldSetRecentsAutosaveName( NSInteger tag, CFStringRef name )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) {
        [fld setRecentsAutosaveName:(NSString *)name];
    }
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSInteger SearchFieldSearchMenuTemplate( NSInteger tag )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) {
        NSMenu *menu = [fld searchMenuTemplate];
        if ( menu ) {
            // we need to get the menu index here
            // ...
            
            
        }
    }
    return -1;
}

void SearchFieldSetSearchMenuTemplate( NSInteger tag, NSInteger menuIndex )
{
    NSSearchField *fld = SearchFieldWithTag( tag );
    if ( fld ) {
        NSMenu *menu = MenuAtIndex( menuIndex );
        if ( menu ) {
            [fld setSearchMenuTemplate:menu];
            NSArray *items = [menu itemArray];
            for ( NSMenuItem *item in items ) {
                [item setTarget:nil];
                [item setAction:nil];
                
                // set search field tags here:
                // NSSearchFieldClearRecentsMenuItemTag
                // NSSearchFieldRecentsTitleMenuItemTag
                // NSSearchFieldRecentsMenuItemTag
            }
        }
    }
}
#endif


// Init
SearchFieldRef SearchFieldInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 96, 22 );

    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    NSSearchField *fld;
    if ( subclass ) {
        fld = [[[SearchFieldSC alloc] initWithFrame:rect] autorelease];
    } else {
        fld = [[[NSSearchField alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [fld setTag: inTag];

    // fix font to match I.B.
    CocoaControlFixFont( fld );
    
    // delegate
    [fld setDelegate:CocoaSearchFieldDelegateGet()];

    return fld;
}

void CocoaSearchField( NSInteger inTag, NSInteger inEnabled, CFStringRef inText, CGRect inRect, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSString *text = (NSString *)inText;
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSSearchField *fld = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !fld ) {
            fld = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( fld ) {    // already exists
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [fld class], [NSSearchField class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, fld, [NSSearchField class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [fld setEnabled:inEnabled];
            
            // text
            if ( text ) [fld setStringValue:text];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [fld setFrame:rect];
            
        } else {        // create
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 96, 22 );
            
            // init
//            if ( inSubclass ) {
//                fld = [[SearchFieldSC alloc] initWithFrame:rect];
//            } else {
//                fld = [[NSSearchField alloc] initWithFrame:rect];
//            }
            
            fld = SearchFieldInit( inTag, inRect );
            
            // default autoresizing
            [fld setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [fld setTag: inTag];
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [fld setEnabled:inEnabled];
            
            // text
            if ( text ) [fld setStringValue:text];
            
            // wndTag
            [fld setWndTag:inWndTag];
            
            // fix font to match I.B.
//            CocoaControlFixFont( fld );
            
            // delegate
//            [fld setDelegate:CocoaSearchFieldDelegateGet()];
            
            // add to window
            [contentView addSubview:fld];
//            [fld release];
        }
        
        [fld setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

