/*
 SecureTextField.m

 Bernie Wylde
 */

#import "SecureTextField.h"

@implementation SecureTextFieldSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end

NSSecureTextField *SecureTextFieldWithTag( NSInteger tag )
{ return (NSSecureTextField *)CocoaValidateViewKindOfClass( tag, [NSSecureTextField class] ); }

BOOL SecureTextFieldExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSSecureTextField class] ); }


Boolean SecureTextFieldEchosBullets( NSInteger tag )
{
    NSSecureTextField *fld = SecureTextFieldWithTag( tag );
    if ( fld ) [[fld cell] echosBullets];
    return NO;
}

void SecureTextFieldSetEchosBullets( NSInteger tag, Boolean flag )
{
    NSSecureTextField *fld = SecureTextFieldWithTag( tag );
    if ( fld ) [[fld cell] setEchosBullets:flag];
}


// Init
SecureTextFieldRef SecureTextFieldInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 96, 22 );
    
    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSSecureTextField *fld;
    if ( subclass ) {
        fld = [[[SecureTextFieldSC alloc] initWithFrame:rect] autorelease];
    } else {
        fld = [[[NSSecureTextField alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [fld setTag: inTag];
    
    // fix font to match I.B.
    CocoaControlFixFont( fld );
    
    // delegate
    [fld setDelegate:CocoaTextFieldDelegateGet()];

    return fld;
}



void CocoaSecureTextField( NSInteger inTag, NSInteger inEnabled, CFStringRef inText, CGRect inRect, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSString *text = (NSString *)inText;
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSSecureTextField *fld = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !fld ) {
            fld = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( fld ) {    // already exists
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [fld class], [NSSecureTextField class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, fld, [NSSecureTextField class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [fld setEnabled:inEnabled];
            
            // text
            if ( text ) [fld setStringValue:text];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [fld setFrame:rect];
            
        } else {        // create
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 96, 22 );
            
            // init
//            if ( inSubclass ) {
//                fld = [[SecureTextFieldSC alloc] initWithFrame:rect];
//            } else {
//                fld = [[NSSecureTextField alloc] initWithFrame:rect];
//            }
            fld = SecureTextFieldInit( inTag, inRect );
            
            // default autoresizing
            [fld setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [fld setTag: inTag];
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [fld setEnabled:inEnabled];
            
            // text
            if ( text ) [fld setStringValue:text];
            
            // wndTag
            [fld setWndTag:inWndTag];
            
            // fix font to match I.B.
//            CocoaControlFixFont( fld );
            
            // delegate
//            [fld setDelegate:CocoaTextFieldDelegateGet()];
            
            // add to window
            [contentView addSubview:fld];
//            [fld release];
        }
        
        [fld setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

