/*
 SegmentedControl.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

@interface SegmentedControlSC : NSSegmentedControl
@end

@interface CocoaSegmentedControlTarget : NSObject
@end

NSSegmentedControl *SegmentedControlWithTag( NSInteger tag );
BOOL SegmentedControlExists( NSInteger tag );

// Behavior
#if __MAC_OS_X_VERSION_MAX_ALLOWED > 101003
NSSegmentSwitchTracking SegmentedControlTrackingMode( NSInteger tag );
void SegmentedControlSetTrackingMode( NSInteger tag, NSSegmentSwitchTracking mode );
#endif
NSSegmentStyle SegmentedControlStyle( NSInteger tag );
void SegmentedControlSetStyle( NSInteger tag, NSSegmentStyle style );

// Number of segments
NSInteger SegmentedControlSegmentCount( NSInteger tag );
void SegmentedControlSetSegmentCount( NSInteger tag, NSInteger count );

// Text
CFStringRef SegmentedControlLabelForSegment( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentLabel( NSInteger tag, NSInteger index, CFStringRef string );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
NSTextAlignment SegmentedControlSegmentAlignment( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentAlignment( NSInteger tag, NSInteger index, NSTextAlignment value );
#endif

// Image
ImageRef SegmentedControlImageForSegment( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentImage( NSInteger tag, NSInteger index, ImageRef image );
NSImageScaling SegmentedControlImageScaling( NSInteger tag, NSInteger index );
void SegmentedControlSetImageScaling( NSInteger tag, NSInteger index, NSImageScaling scaling );

// Menu
CocoaMenuRef SegmentedControlMenuForSegment( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentMenu( NSInteger tag, NSInteger index, CocoaMenuRef menu );
void SegmentedControlSetSegmentMenuIndex( NSInteger tag, NSInteger index, NSInteger menuIndex );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean SegmentedControlSegmentShowsMenuIndicator( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentShowsMenuIndicator( NSInteger tag, NSInteger index, Boolean flag );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
Boolean SegmentedControlIsSpringLoaded( NSInteger tag );
void SegmentedControlSetSpringLoaded( NSInteger tag, Boolean flag );
#endif

// Selected segment
NSInteger SegmentedControlSelectedSegment( NSInteger tag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
NSInteger SegmentedControlIndexOfSelectedItem( NSInteger tag );
#endif
Boolean SegmentedControlSelectSegmentWithTag( NSInteger ctrlTag, NSInteger segTag );
void SegmentedControlSetSegmentSelected( NSInteger tag, NSInteger index, Boolean flag );
Boolean SegmentedControlIsSegmentSelected( NSInteger tag, NSInteger index );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
ColorRef SegmentedControlSelectedSegmentBezelColor( NSInteger tag );
void SegmentedControlSetSelectedSegmentBezelColor( NSInteger tag, ColorRef col );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
double SegmentedControlSelectedSegmentDoubleValue( NSInteger tag, NSInteger index );
#endif

// Spacing
CGFloat SegmentedControlSegmentWidth( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentWidth( NSInteger tag, NSInteger index, CGFloat value );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void SegmentedControlSetSegmentDistribution( NSInteger tag, NSSegmentDistribution value );
#endif

// Enable
void SegmentedControlSetSegmentEnabled( NSInteger tag, NSInteger index, Boolean flag );
Boolean SegmentedControlIsSegmentEnabled( NSInteger tag, NSInteger index );

// Tags and tooltips
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
NSInteger SegmentedControlSegmentTag( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentTag( NSInteger tag, NSInteger index, NSInteger segTag );
CFStringRef SegmentedControlSegmentToolTip( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentToolTip( NSInteger tag, NSInteger index, CFStringRef string );
#endif

// Convenience
void SegmentedControlSetSegment( NSInteger tag, NSInteger index, NSInteger enabled, CFStringRef label, ImageRef image, NSUInteger imageScaling, NSInteger width );



// Init
SegmentedControlRef SegmentedControlInit( NSInteger inTag, CGRect inRect );

// Statement
void CocoaSegmentedControl( NSInteger inTag, NSInteger inEnabled, NSInteger inIndex, CGRect inRect, NSInteger inSegments, NSInteger inWndTag );

