/*
 SegmentedControl.m

 Bernie Wylde
 */

#import "SegmentedControl.h"

@implementation SegmentedControlSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}
@end

NSSegmentedControl *SegmentedControlWithTag( NSInteger tag )
{ return (NSSegmentedControl *)CocoaValidateViewKindOfClass( tag, [NSSegmentedControl class] ); }

BOOL SegmentedControlExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSSegmentedControl class] ); }


// Behavior
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
NSSegmentSwitchTracking SegmentedControlTrackingMode( NSInteger tag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c trackingMode];
    return 0;
}

void SegmentedControlSetTrackingMode( NSInteger tag, NSSegmentSwitchTracking mode )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setTrackingMode:mode];
}
#endif

NSSegmentStyle SegmentedControlStyle( NSInteger tag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c segmentStyle];
    return 0;
}

void SegmentedControlSetStyle( NSInteger tag, NSSegmentStyle style )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setSegmentStyle:style];
}

// Number of segments
NSInteger SegmentedControlSegmentCount( NSInteger tag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c segmentCount];
    return 0;
}

void SegmentedControlSetSegmentCount( NSInteger tag, NSInteger count )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setSegmentCount:count];
}

// Text
CFStringRef SegmentedControlLabelForSegment( NSInteger tag, NSInteger index )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return (CFStringRef)[c labelForSegment:index];
    return nil;
}

void SegmentedControlSetSegmentLabel( NSInteger tag, NSInteger index, CFStringRef string )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setLabel:(NSString *)string forSegment:index];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
NSTextAlignment SegmentedControlSegmentAlignment( NSInteger tag, NSInteger index )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c alignmentForSegment:index];
    return 0;
}

void SegmentedControlSetSegmentAlignment( NSInteger tag, NSInteger index, NSTextAlignment value )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setAlignment:value forSegment:index];
}
#endif

// Image
ImageRef SegmentedControlImageForSegment( NSInteger tag, NSInteger index )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c imageForSegment:index];
    return nil;
}

void SegmentedControlSetSegmentImage( NSInteger tag, NSInteger index, ImageRef image )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setImage:image forSegment:index];
}

NSImageScaling SegmentedControlImageScaling( NSInteger tag, NSInteger index )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c imageScalingForSegment:index];
    return 0;
}

void SegmentedControlSetImageScaling( NSInteger tag, NSInteger index, NSImageScaling scaling )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setImageScaling:scaling forSegment:index];
}

// Menu
CocoaMenuRef SegmentedControlMenuForSegment( NSInteger tag, NSInteger index )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c menuForSegment:index];
    return nil;
}

void SegmentedControlSetSegmentMenu( NSInteger tag, NSInteger index, CocoaMenuRef menu )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setMenu:menu forSegment:index];
}

void SegmentedControlSetSegmentMenuIndex( NSInteger tag, NSInteger index, NSInteger menuIndex )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) {
        NSMenu *menu = MenuAtIndex( menuIndex );
        if ( menu ) {
            [c setMenu:menu forSegment:index];
        }
    }
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean SegmentedControlSegmentShowsMenuIndicator( NSInteger tag, NSInteger index )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c showsMenuIndicatorForSegment:index];
    return NO;
}

void SegmentedControlSetSegmentShowsMenuIndicator( NSInteger tag, NSInteger index, Boolean flag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setShowsMenuIndicator:flag forSegment:index];
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
Boolean SegmentedControlIsSpringLoaded( NSInteger tag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c isSpringLoaded];
    return NO;
}

void SegmentedControlSetSpringLoaded( NSInteger tag, Boolean flag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setSpringLoaded:flag];
}
#endif

// Selected segment
NSInteger SegmentedControlSelectedSegment( NSInteger tag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c selectedSegment];
    return -1;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
NSInteger SegmentedControlIndexOfSelectedItem( NSInteger tag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c indexOfSelectedItem];
    return 0;
}
#endif

Boolean SegmentedControlSelectSegmentWithTag( NSInteger ctrlTag, NSInteger segTag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( ctrlTag );
    if ( c ) return [c selectSegmentWithTag:segTag];
    return NO;
}

void SegmentedControlSetSegmentSelected( NSInteger tag, NSInteger index, Boolean flag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setSelected:flag forSegment:index];
}

Boolean SegmentedControlIsSegmentSelected( NSInteger tag, NSInteger index )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c isSelectedForSegment:index];
    return NO;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
ColorRef SegmentedControlSelectedSegmentBezelColor( NSInteger tag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c selectedSegmentBezelColor];
    return nil;
}

void SegmentedControlSetSelectedSegmentBezelColor( NSInteger tag, ColorRef col )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setSelectedSegmentBezelColor:col];
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
double SegmentedControlSelectedSegmentDoubleValue( NSInteger tag, NSInteger index )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c doubleValueForSelectedSegment];
    return 0.0;
}
#endif

// Spacing
CGFloat SegmentedControlSegmentWidth( NSInteger tag, NSInteger index )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c widthForSegment:index];
    return 0.0;
}

void SegmentedControlSetSegmentWidth( NSInteger tag, NSInteger index, CGFloat value )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setWidth:value forSegment:index];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void SegmentedControlSetSegmentDistribution( NSInteger tag, NSSegmentDistribution value )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setSegmentDistribution:value];
}
#endif

// Enable
void SegmentedControlSetSegmentEnabled( NSInteger tag, NSInteger index, Boolean flag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setEnabled:flag forSegment:index];
}

Boolean SegmentedControlIsSegmentEnabled( NSInteger tag, NSInteger index )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c isEnabledForSegment:index];
    return NO;
}

// Tags and tooltips
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
NSInteger SegmentedControlSegmentTag( NSInteger tag, NSInteger index )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return [c tagForSegment:index];
    return 0;
}

void SegmentedControlSetSegmentTag( NSInteger tag, NSInteger index, NSInteger segTag )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setTag:segTag forSegment:index];
}

CFStringRef SegmentedControlSegmentToolTip( NSInteger tag, NSInteger index )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) return (CFStringRef)[c toolTipForSegment:index];
    return nil;
}

void SegmentedControlSetSegmentToolTip( NSInteger tag, NSInteger index, CFStringRef string )
{
    NSSegmentedControl *c = SegmentedControlWithTag( tag );
    if ( c ) [c setToolTip:(NSString *)string forSegment:index];
}
#endif

// Convenience
void SegmentedControlSetSegment( NSInteger tag, NSInteger index, NSInteger enabled, CFStringRef label, ImageRef image, NSUInteger imageScaling, NSInteger width )
{
    // target window
    id w = CocoaOutputWindow();
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSSegmentedControl *c = CocoaViewSubviewWithTag( contentView, tag );
        
        if ( !c ) {
            c = TitlebarAccessoryViewWithTag( tag );
        }
        
        if ( c ) {
            NSSegmentedCell *cell = (NSSegmentedCell *)[c cell];
            
            // label
            if ( label ) [cell setLabel:(NSString *)label forSegment:index];
            
            // image
            if ( image ) {
                [cell setImage:image forSegment:index];
                
                // image scaling
                [cell setImageScaling:imageScaling forSegment:index];
            }
            
            // width
            if ( width ) [cell setWidth:width forSegment:index];
            
            // enabled
            [cell setEnabled:enabled forSegment:index];
            
        }
    } else {
        CocoaShowAlert( NSWarningAlertStyle, @"Window error.", @"Cocoa window does not exist." );
    }
}

// Init
SegmentedControlRef SegmentedControlInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 18, 18, 104, 24 );

    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    NSSegmentedControl *seg;
    if ( subclass ) {
        seg = [[[SegmentedControlSC alloc] initWithFrame:rect] autorelease];
    } else {
        seg = [[[NSSegmentedControl alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [seg setTag:inTag];

    // segment count
    [seg setSegmentCount:3];

    // target/action
    [seg setTarget:CocoaControlTargetObj()];
    [seg setAction:@selector(controlAction:)];
    
    return seg;
}

void CocoaSegmentedControl( NSInteger inTag, NSInteger inEnabled, NSInteger inIndex, CGRect inRect, NSInteger inSegments, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSSegmentedControl *seg = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !seg ) {
            seg = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( seg ) {
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [seg class], [NSSegmentedControl class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, seg, [NSSegmentedControl class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [seg setEnabled:inEnabled];
            
            // index
            if ( inIndex != kFBParamMissing ) [seg setSelected:YES forSegment:inIndex];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [seg setFrame:rect];
            
        } else {
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 18, 18, 104, 24 );
            
            // init
//            if ( inSubclass ) {
//                seg = [[SegmentedControlSC alloc] initWithFrame:rect];
//            } else {
//                seg = [[NSSegmentedControl alloc] initWithFrame:rect];
//            }
            seg = SegmentedControlInit( inTag, inRect );
            
            // default autoresizing
            [seg setAutoresizingMask:NSViewMinYMargin];
            
            // tag
 //           [seg setTag:inTag];
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [seg setEnabled:inEnabled];
            
            // segment count
            if ( inSegments != kFBParamMissing ) [seg setSegmentCount:inSegments];
            
            NSInteger count = [seg segmentCount];
            for ( NSInteger index = 0; index < count; index++ ) {
                [seg setWidth:rect.size.width/count forSegment:index];
            }
            
            // wndTag
            [seg setWndTag:inWndTag];
            
            // index
            if ( inIndex != kFBParamMissing ) [seg setSelected:YES forSegment:inIndex];
            
            // target/action
//            [seg setTarget:CocoaControlTargetObj()];
//            [seg setAction:@selector(controlAction:)];
            
            // add to window
            [contentView addSubview:seg];
//            [seg release];
        }
        
        [seg setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

