/*
 Sound.m

 Bernie Wylde
 */

#import "Sound.h"

@implementation CocoaSoundDelegate
- (void)sound:(NSSound *)sound didFinishPlaying:(BOOL)flag {
    DialogEventSetBool(flag);
    CallUserDefinedOnDialogFunction( soundDidFinishPlaying, 0, 0, sound );
}
@end

// private
CocoaSoundDelegate *CocoaSoundDelegateObj( void )
{
    static CocoaSoundDelegate *sCocoaSoundDelegate = nil;
    if ( !sCocoaSoundDelegate ) sCocoaSoundDelegate = [[CocoaSoundDelegate alloc] init];
    return sCocoaSoundDelegate;
}

// -- create --
Boolean SoundCanInitWithPasteboard( CocoaPasteboardRef pb )
{ return [NSSound canInitWithPasteboard:pb]; }

SoundRef SoundWithContentsOfURL( CFURLRef url, Boolean byReference )
{ return [[[NSSound alloc] initWithContentsOfURL:(NSURL *)url byReference:byReference] autorelease]; }

SoundRef SoundWithData( CFDataRef data )
{ return [[[NSSound alloc] initWithData:(NSData *)data] autorelease]; }

SoundRef SoundWithPasteboard( CocoaPasteboardRef pb )
{ return [[[NSSound alloc] initWithPasteboard:pb] autorelease]; }

// -- configure --
CFStringRef SoundName( SoundRef snd )
{ return (CFStringRef)[snd name]; }

Boolean SoundSetName( SoundRef snd, CFStringRef name )
{ return [snd setName:(NSString *)name]; }

float SoundVolume( SoundRef snd )
{ return [snd volume]; }

void SoundSetVolume( SoundRef snd, float value )
{ return [snd setVolume:value]; }

CFTimeInterval SoundCurrentTime( SoundRef snd )
{ return [snd currentTime]; }

void SoundSetCurrentTime( SoundRef snd, CFTimeInterval ti )
{ [snd setCurrentTime:ti]; }

Boolean SoundLoops( SoundRef snd )
{ return [snd loops]; }

void SoundSetLoops( SoundRef snd, Boolean flag )
{ [snd setLoops:flag]; }

void SoundSetPlaybackDeviceIdentifier( SoundRef snd, CFStringRef identifier )
{ [snd setPlaybackDeviceIdentifier:(NSString *)identifier]; }

// -- info --
CFArrayRef SoundUnfilteredTypes( void )
{ return (CFArrayRef)[NSSound soundUnfilteredTypes]; }

SoundRef SoundNamed( CFStringRef name )
{ return [NSSound soundNamed:(NSString *)name]; }

CFTimeInterval SoundDuration( SoundRef snd )
{ return [snd duration]; }

// -- play --
Boolean SoundIsPlaying( SoundRef snd )
{ return [snd isPlaying]; }

Boolean SoundPause( SoundRef snd )
{ return [snd pause]; }

Boolean SoundPlay( SoundRef snd )
{
    [snd setDelegate:CocoaSoundDelegateObj()];
    return [snd play];
}

Boolean SoundResume( SoundRef snd )
{ return [snd resume]; }

Boolean SoundStop( SoundRef snd )
{ return [snd stop]; }

// -- write --
void SoundWriteToPasteboard( SoundRef snd, CocoaPasteboardRef pb )
{ [snd writeToPasteboard:pb]; }

