/*
 SplitView.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

@interface SplitViewSC : NSSplitView
@end

// dialog events
enum {
    splitViewResizeSubviewsWithOldSize = 1500,
    splitViewWillResizeSubviews,
    splitViewDidResizeSubviews,
    splitViewCanCollapseSubview,
    splitViewShouldCollapseSubview,
    splitViewShouldAdjustSizeOfSubview,
    splitViewEffectiveRect,
    splitViewShouldHideDividerAtIndex,
    splitViewAdditionalEffectiveRectOfDividerAtIndex,
    splitViewConstrainMaxCoordinate,
    splitViewConstrainMinCoordinate,
    splitViewConstrainSplitPosition,
    
    // subclass events
    splitViewDividerColor,
    splitViewDividerThickness,
    splitViewDrawDividerInRect
};

@interface NSSplitView (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;

- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

@interface CocoaSplitViewDelegate : NSObject <NSSplitViewDelegate>
@end

CocoaSplitViewDelegate *CocoaSplitViewDelegateGet( void );

NSSplitView *SplitViewWithTag( NSInteger tag );
BOOL SplitViewExists( NSInteger tag );


// Arranging subviews
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFArrayRef SplitViewArrangedSubviews( NSInteger tag );
Boolean SplitViewArrangesAllSubviews( NSInteger tag );
void SplitViewAddArrangedSubview( NSInteger tag, NSInteger vwTag );
void SplitViewInsertArrangedSubview( NSInteger tag, NSInteger vwTag, NSInteger index );
void SplitViewRemoveArrangedSubview( NSInteger tag, NSInteger vwTag );
#endif// 101100

// Managing subviews
void SplitViewAdjustSubviews( NSInteger tag );
BOOL SplitViewIsPaneCollapsed( NSInteger tag, NSInteger index );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
float SplitViewHoldingPriority( NSInteger tag, NSInteger paneIndex );
void SplitViewSetHoldingPriority( NSInteger tag, NSInteger index, float priority );
#endif// 1080

// Divider orientation
Boolean SplitViewIsVertical( NSInteger tag );
void SplitViewSetVertical( NSInteger tag, Boolean flag );

// Configure and draw dividers
NSSplitViewDividerStyle SplitViewDividerStyle( NSInteger tag );
void SplitViewSetDividerStyle( NSInteger tag, NSSplitViewDividerStyle style );
CGFloat SplitViewDividerThickness( NSInteger tag );

// Saving subview positions
CFStringRef SplitViewAutosaveName( NSInteger tag );
void SplitViewSetAutosaveName( NSInteger tag, CFStringRef name );

// Constraining split position
CGFloat SplitViewMinPossiblePositionOfDivider( NSInteger tag, NSInteger index );
CGFloat SplitViewMaxPossiblePositionOfDivider( NSInteger tag, NSInteger index );
void SplitViewSetPositionOfDivider( NSInteger tag, NSInteger index, CGFloat position );

// Convenience
void SplitViewAddSubview( NSInteger splitViewTag, NSInteger index, NSInteger subviewTag );
void SplitViewAddSubviewSizeToFit( NSInteger splitViewTag, NSInteger paneIndex, NSInteger subviewTag );

ViewRef SplitViewPaneView( NSInteger tag, NSInteger paneIndex );
CGSize SplitViewPaneSize( NSInteger tag, NSInteger paneIndex );


// Init
SplitViewRef SplitViewInit( NSInteger inTag, CGRect inRect );

// Statement
void CocoaSplitView( NSInteger inTag, CGRect inRect, NSInteger inVertical, NSInteger inDividerStyle, NSInteger inWndTag );
