/*
 SplitView.m

 Bernie Wylde
 */

#import "SplitView.h"

@implementation SplitViewSC
//- (void)drawRect:(NSRect)dirtyRect {
//    CocoaSubclassDrawRect( self, dirtyRect );
//    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
//}

- (NSColor *)dividerColor {
    DialogEventSetColor( [NSColor clearColor] );
    CocoaOutputWindowSetTag( [self wndTag] );
    CallUserDefinedOnDialogFunction( splitViewDividerColor, [self tag], [self wndTag], nil );
    return DialogEventColor();
}

- (CGFloat)dividerThickness {
    CGFloat thickness;
    switch ( [self dividerStyle] ) {
        case NSSplitViewDividerStyleThin:
            thickness = 1.0;
            break;
        case NSSplitViewDividerStyleThick:
            thickness = 9.0;
            break;
        case NSSplitViewDividerStylePaneSplitter:
        default:
            thickness = 10.0;
            break;
    }
    DialogEventSetDouble( thickness );
    CocoaOutputWindowSetTag( [self wndTag] );
    CallUserDefinedOnDialogFunction( splitViewDividerThickness, [self tag], [self wndTag], nil );
    return DialogEventDouble();
}

- (void)drawDividerInRect:(NSRect)rect {
    DialogEventSetBool(NO);
    CocoaOutputWindowSetTag( [self wndTag] );
    DialogEventSetNSRect( rect );
    CallUserDefinedOnDialogFunction( splitViewDrawDividerInRect, [self tag], [self wndTag], nil );
    if ( !DialogEventBool() ) [super drawDividerInRect:rect];
}
@end

@implementation NSSplitView (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

@implementation CocoaSplitViewDelegate

//- (void)splitView:(NSSplitView *)splitView resizeSubviewsWithOldSize:(NSSize)oldSize {
//    DialogEventSetNSSize( oldSize );
////    CocoaOutputWindowSetTag( [splitView wndTag] );
//    CocoaOutputWindowSetTag( [[splitView window] tag] );
//    CallUserDefinedOnDialogFunction( splitViewResizeSubviewsWithOldSize, [splitView tag], [[splitView window] tag], NULL );
//}

- (void)splitViewWillResizeSubviews:(NSNotification *)notification {
    CocoaOutputWindowSetTag( [notification.object wndTag] );
    CallUserDefinedOnDialogFunction( splitViewWillResizeSubviews, [notification.object tag], [notification.object wndTag], NULL );
}

- (void)splitViewDidResizeSubviews:(NSNotification *)notification {
    CocoaOutputWindowSetTag( [notification.object wndTag] );
    CallUserDefinedOnDialogFunction( splitViewDidResizeSubviews, [notification.object tag], [notification.object wndTag], NULL );
}

- (BOOL)splitView:(NSSplitView *)splitView canCollapseSubview:(NSView *)subview {
    DialogEventSetBool( NO );
    DialogEventSetViewTag( [subview tag] );
    CocoaOutputWindowSetTag( [splitView wndTag] );
    CallUserDefinedOnDialogFunction( splitViewCanCollapseSubview, [splitView tag], [splitView wndTag], NULL );
    return DialogEventBool();
}

- (BOOL)splitView:(NSSplitView *)splitView shouldCollapseSubview:(NSView *)subview forDoubleClickOnDividerAtIndex:(NSInteger)dividerIndex {
    DialogEventSetBool( NO );
    DialogEventSetViewTag( [subview tag] );
    DialogEventSetLong( dividerIndex );
    CocoaOutputWindowSetTag( [splitView wndTag] );
    CallUserDefinedOnDialogFunction( splitViewShouldCollapseSubview, [splitView tag], [splitView wndTag], NULL );
    return DialogEventBool();
}

- (BOOL)splitView:(NSSplitView *)splitView shouldAdjustSizeOfSubview:(NSView *)view {
    DialogEventSetBool( YES );
    DialogEventSetViewTag( [view tag] );
    CocoaOutputWindowSetTag( [splitView wndTag] );
    CallUserDefinedOnDialogFunction( splitViewShouldAdjustSizeOfSubview, [splitView tag], [splitView wndTag], NULL );
    return DialogEventBool();
}

- (NSRect)splitView:(NSSplitView *)splitView effectiveRect:(NSRect)proposedEffectiveRect forDrawnRect:(NSRect)drawnRect ofDividerAtIndex:(NSInteger)dividerIndex {
    
    return proposedEffectiveRect;
}

- (BOOL)splitView:(NSSplitView *)splitView shouldHideDividerAtIndex:(NSInteger)dividerIndex {
    DialogEventSetBool( NO );
    DialogEventSetLong( dividerIndex );
    CocoaOutputWindowSetTag( [splitView wndTag] );
    CallUserDefinedOnDialogFunction( splitViewShouldHideDividerAtIndex, [splitView tag], [splitView wndTag], NULL );
    return DialogEventBool();
}

//- (NSRect)splitView:(NSSplitView *)splitView additionalEffectiveRectOfDividerAtIndex:(NSInteger)dividerIndex {
//
//    return rect;
//}

- (CGFloat)splitView:(NSSplitView *)splitView constrainMaxCoordinate:(CGFloat)proposedMaximumPosition ofSubviewAt:(NSInteger)dividerIndex {
    DialogEventSetFloat( proposedMaximumPosition );
    DialogEventSetLong( dividerIndex );
    CocoaOutputWindowSetTag( [splitView wndTag] );
    CallUserDefinedOnDialogFunction( splitViewConstrainMaxCoordinate, [splitView tag], [splitView wndTag], NULL );
    return DialogEventFloat();
}

- (CGFloat)splitView:(NSSplitView *)splitView constrainMinCoordinate:(CGFloat)proposedMinimumPosition ofSubviewAt:(NSInteger)dividerIndex {
    DialogEventSetFloat( proposedMinimumPosition );
    DialogEventSetLong( dividerIndex );
    CocoaOutputWindowSetTag( [splitView wndTag] );
    CallUserDefinedOnDialogFunction( splitViewConstrainMinCoordinate, [splitView tag], [splitView wndTag], NULL );
    return DialogEventFloat();
}

- (CGFloat)splitView:(NSSplitView *)splitView constrainSplitPosition:(CGFloat)proposedPosition ofSubviewAt:(NSInteger)dividerIndex {
    // .....
    return proposedPosition;
}
@end

CocoaSplitViewDelegate *CocoaSplitViewDelegateGet( void )
{
    static CocoaSplitViewDelegate *sCocoaSplitViewDelegate = nil;
    if ( !sCocoaSplitViewDelegate ) sCocoaSplitViewDelegate = [[CocoaSplitViewDelegate alloc] init];
    return sCocoaSplitViewDelegate;
}

NSSplitView *SplitViewWithTag( NSInteger tag )
{ return (NSSplitView *)CocoaValidateViewKindOfClass( tag, [NSSplitView class] ); }

BOOL SplitViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSSplitView class] ); }

// Arranging subviews
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFArrayRef SplitViewArrangedSubviews( NSInteger tag )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) return (CFArrayRef)[splitView arrangedSubviews];
    return nil;
}

Boolean SplitViewArrangesAllSubviews( NSInteger tag )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) return [splitView arrangesAllSubviews];
    return NO;
}

void SplitViewAddArrangedSubview( NSInteger tag, NSInteger vwTag )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) {
        NSView *vwRef = CocoaViewWithTag( vwTag );
        if ( vwRef ) [splitView addArrangedSubview:vwRef];
    }
}

void SplitViewInsertArrangedSubview( NSInteger tag, NSInteger vwTag, NSInteger index )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) {
        NSView *vwRef = CocoaViewWithTag( vwTag );
        if ( vwRef ) [splitView insertArrangedSubview:vwRef atIndex:index];
    }
}

void SplitViewRemoveArrangedSubview( NSInteger tag, NSInteger vwTag )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) {
        NSView *vwRef = CocoaViewWithTag( vwTag );
        if ( vwRef ) [splitView removeArrangedSubview:vwRef];
    }
}
#endif// 101100

// Managing subviews
void SplitViewAdjustSubviews( NSInteger tag )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) [splitView adjustSubviews];
}

BOOL SplitViewIsPaneCollapsed( NSInteger tag, NSInteger index )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) {
        NSArray *views = [splitView subviews];
        if ( views ) {
            NSView *view = [views objectAtIndex:index];
            return [splitView isSubviewCollapsed:view];
        }
    }
    return NO;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
float SplitViewHoldingPriority( NSInteger tag, NSInteger paneIndex )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) return [splitView holdingPriorityForSubviewAtIndex:paneIndex];
    return 0.0;
}

void SplitViewSetHoldingPriority( NSInteger tag, NSInteger index, float priority )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) {
        if ( [splitView respondsToSelector:@selector(setHoldingPriority:forSubviewAtIndex:)] ) {
            [splitView setHoldingPriority:priority forSubviewAtIndex:index];
        }
    }
}
#endif// 1080

// Divider orientation
Boolean SplitViewIsVertical( NSInteger tag )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) return [splitView isVertical];
    return NO;
}

void SplitViewSetVertical( NSInteger tag, Boolean flag )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) [splitView setVertical:flag];
}

// Configure and draw dividers
NSSplitViewDividerStyle SplitViewDividerStyle( NSInteger tag )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) return [splitView dividerStyle];
    return 0;
}

void SplitViewSetDividerStyle( NSInteger tag, NSSplitViewDividerStyle style )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) [splitView setDividerStyle:style];
}

CGFloat SplitViewDividerThickness( NSInteger tag )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) {
        return [splitView dividerThickness];
    }
    return 0.0;
}

// Saving subview positions
CFStringRef SplitViewAutosaveName( NSInteger tag )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) return (CFStringRef)[splitView autosaveName];
    return nil;
}

void SplitViewSetAutosaveName( NSInteger tag, CFStringRef name )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) [splitView setAutosaveName:(NSString *)name];
}

// Constraining split position
CGFloat SplitViewMinPossiblePositionOfDivider( NSInteger tag, NSInteger index )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) {
        return [splitView minPossiblePositionOfDividerAtIndex:index];
    }
    return 0.0;
}

CGFloat SplitViewMaxPossiblePositionOfDivider( NSInteger tag, NSInteger index )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) {
        return [splitView maxPossiblePositionOfDividerAtIndex:index];
    }
    return 0.0;
}

void SplitViewSetPositionOfDivider( NSInteger tag, NSInteger index, CGFloat position )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) {
        [splitView setPosition:position ofDividerAtIndex:index];
    }
}

// Convenience
void SplitViewAddSubview( NSInteger splitViewTag, NSInteger index, NSInteger subviewTag )
{
    NSSplitView *splitView = SplitViewWithTag( splitViewTag );
    if ( splitView ) {
        NSArray *views = [splitView subviews];
        NSView *view = [views objectAtIndex:index];
        id subview = CocoaViewWithTag( subviewTag );
        if ( subview ) {
            [view addSubview:subview];
        }
    }
}

void SplitViewAddSubviewSizeToFit( NSInteger splitViewTag, NSInteger index, NSInteger subviewTag )
{
    NSSplitView *splitView = SplitViewWithTag( splitViewTag );
    if ( splitView ) {
        NSArray *views = [splitView subviews];
        NSView *view = [views objectAtIndex:index];
        id subview = CocoaViewWithTag( subviewTag );
        if ( subview ) {
            [subview setFrame:[view bounds]];
            [view addSubview:subview];
        }
    }
}

ViewRef SplitViewPaneView( NSInteger tag, NSInteger paneIndex )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) {
        NSArray *views = [splitView subviews];
        return [views objectAtIndex:paneIndex];
    }
    return nil;
}

CGSize SplitViewPaneSize( NSInteger tag, NSInteger paneIndex )
{
    NSSplitView *splitView = SplitViewWithTag( tag );
    if ( splitView ) {
        NSArray *views = [splitView subviews];
        return NSSizeToCGSize([[views objectAtIndex:paneIndex] bounds].size);
    }
    return CGSizeZero;
}


// Init
SplitViewRef SplitViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 163, 96 );

    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    NSSplitView *splitView;
    if ( subclass ) {
        splitView = [[[SplitViewSC alloc] initWithFrame:rect] autorelease];
    } else {
        splitView = [[[NSSplitView alloc] initWithFrame:rect] autorelease];
    }

    // tag
    [splitView setTag:inTag];

    // subviews
//    CGFloat dividerThickness = [splitView dividerThickness];
//
//    CGFloat view1x = 0.0;
//    CGFloat view1y = 0.0;
//    
//    CGFloat viewWidth = rect.size.width;
//    CGFloat viewHeight = (rect.size.height - dividerThickness) / 2;
//    view1y = viewHeight+dividerThickness;
//    
//    NSView *view0 = [[[NSView alloc] initWithFrame:NSMakeRect(0,0,viewWidth,viewHeight)] autorelease];
//    NSView *view1 = [[[NSView alloc] initWithFrame:NSMakeRect(view1x,view1y,viewWidth,viewHeight)] autorelease];
//    
//    [splitView addSubview:view0];
//    [splitView addSubview:view1];

    // delegate
    [splitView setDelegate:CocoaSplitViewDelegateGet()];

    return splitView;
}

// Statement
void CocoaSplitView( NSInteger inTag, CGRect inRect, NSInteger inVertical, NSInteger inDividerStyle, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSSplitView *splitView = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !splitView ) {
            splitView = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( splitView ) {    // exists
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [splitView class], [NSSplitView class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, splitView, [NSSplitView class] ) ) {
                return;
            }

            // rect
            if ( !NSIsEmptyRect( rect ) ) [splitView setFrame:rect];
            
            // vertical
            if ( inVertical != kFBParamMissing ) [splitView setVertical:inVertical];
            
            // divider style
            if ( inDividerStyle != kFBParamMissing ) [splitView setDividerStyle:inDividerStyle];
            
        } else {        // create
            
            BOOL vertical = NO;
            if ( inVertical != kFBParamMissing ) vertical = inVertical;
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 163, 96 );
            
            // init
            splitView = SplitViewInit( inTag, inRect );
//            if ( inSubclass ) {
//                splitView = [[SplitViewSC alloc] initWithFrame:rect];
//            } else {
//                splitView = [[NSSplitView alloc] initWithFrame:rect];
//            }
            
            // default autoresizing
            [splitView setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [splitView setTag:inTag];
            
            // vertical
            [splitView setVertical:vertical];
            
            // divider style
            if ( inDividerStyle == kFBParamMissing ) {
                [splitView setDividerStyle:NSSplitViewDividerStylePaneSplitter];
            } else {
                [splitView setDividerStyle:inDividerStyle];
            }
            
            // subviews
            CGFloat dividerThickness = [splitView dividerThickness];
            CGFloat viewWidth, viewHeight;
            CGFloat view1x = 0.0;
            CGFloat view1y = 0.0;

            if ( vertical ) {
                viewWidth = (rect.size.width - dividerThickness) / 2;
                viewHeight = rect.size.height;
                view1x = viewWidth+dividerThickness;
            } else {
                viewWidth = rect.size.width;
                viewHeight = (rect.size.height - dividerThickness) / 2;
                view1y = viewHeight+dividerThickness;
            }

            NSView *view0 = [[NSView alloc] initWithFrame:NSMakeRect(0,0,viewWidth,viewHeight)];
            NSView *view1 = [[NSView alloc] initWithFrame:NSMakeRect(view1x,view1y,viewWidth,viewHeight)];

            [splitView addSubview:view0];
            [splitView addSubview:view1];
            
            // wndTag
            [splitView setWndTag:inWndTag];
            
            // delegate
 //           [splitView setDelegate:CocoaSplitViewDelegateGet()];
            
            // add to window
            [contentView addSubview:splitView];
//            [splitView release];
            
            [splitView adjustSubviews];
        }
        
        [splitView setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

