/*
 Stepper.m

 Bernie Wylde
 */

#import "Stepper.h"

@implementation StepperSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}
@end

NSStepper *StepperWithTag( NSInteger tag )
{ return (NSStepper *)CocoaValidateViewKindOfClass( tag, [NSStepper class] ); }

BOOL StepperExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSStepper class] ); }


double StepperMaxValue( NSInteger tag )
{
    NSStepper *stepper = StepperWithTag( tag );
    if ( stepper ) return [stepper maxValue];
    return 0.0;
}

void StepperSetMaxValue( NSInteger tag, double value )
{
    NSStepper *stepper = StepperWithTag( tag );
    if ( stepper ) [stepper setMaxValue:value];
}

double StepperMinValue( NSInteger tag )
{
    NSStepper *stepper = StepperWithTag( tag );
    if ( stepper ) return [stepper minValue];
    return 0.0;
}

void StepperSetMinValue( NSInteger tag, double value )
{
    NSStepper *stepper = StepperWithTag( tag );
    if ( stepper ) [stepper setMinValue:value];
}

double StepperIncrement( NSInteger tag )
{
    NSStepper *stepper = StepperWithTag( tag );
    if ( stepper ) return [stepper increment];
    return 0.0;
}

void StepperSetIncrement( NSInteger tag, double increment )
{
    NSStepper *stepper = StepperWithTag( tag );
    if ( stepper ) [stepper setIncrement:increment];
}

Boolean StepperAutorepeat( NSInteger tag )
{
    NSStepper *stepper = StepperWithTag( tag );
    if ( stepper ) return [stepper autorepeat];
    return NO;
}

void StepperSetAutorepeat( NSInteger tag, Boolean flag )
{
    NSStepper *stepper = StepperWithTag( tag );
    if ( stepper ) [stepper setAutorepeat:flag];
}

Boolean StepperValueWraps( NSInteger tag )
{
    NSStepper *stepper = StepperWithTag( tag );
    if ( stepper ) return [stepper valueWraps];
    return NO;
}

void StepperSetValueWraps( NSInteger tag, Boolean flag )
{
    NSStepper *stepper = StepperWithTag( tag );
    if ( stepper ) [stepper setValueWraps:flag];
}


// Init
StepperRef StepperInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 17, 18, 19, 27 );
    
    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSStepper *stepper;
    if ( subclass ) {
        stepper = [[[StepperSC alloc] initWithFrame:rect] autorelease];
    } else {
        stepper = [[[NSStepper alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [stepper setTag:inTag];
    
    // min
    [stepper setMinValue:0.0];
    
    // max
    [stepper setMaxValue:100.0];
    
    // value
    [stepper setDoubleValue:0.0];
    
    // value wraps
    [stepper setValueWraps:NO];
    
    // target/action
    [stepper setTarget:CocoaControlTargetObj()];
    [stepper setAction:@selector(controlAction:)];
    
    return stepper;
}


void CocoaStepper( NSInteger inTag, NSInteger inEnabled, double inValue, CGRect inRect, double inMinValue, double inMaxValue, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSStepper *stepper = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !stepper ) {
            stepper = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( stepper ) {
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [stepper class], [NSStepper class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, stepper, [NSStepper class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [stepper setEnabled:inEnabled];
            
            // min
            if ( (int)inMinValue != kFBParamMissing ) [stepper setMinValue:inMinValue];
            double minValue = [stepper minValue];
            
            // max
            if ( (int)inMaxValue != kFBParamMissing ) [stepper setMaxValue:inMaxValue];
            double maxValue = [stepper maxValue];

            // value
            if ( (int)inValue != kFBParamMissing ) {
                if ( inValue >= minValue && inValue <= maxValue ) [stepper setDoubleValue:inValue];
            }
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [stepper setFrame:rect];
            
        } else {
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) {
//                rect = NSMakeRect( 17, 18, 19, 27 );
//            }
            
            // init
//            if ( inSubclass ) {
//                stepper = [[StepperSC alloc] initWithFrame:rect];
//            } else {
//                stepper = [[NSStepper alloc] initWithFrame:rect];
//            }
            
            stepper = StepperInit( inTag, inRect );
            
            // default autoresizing
            [stepper setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [stepper setTag:inTag];
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [stepper setEnabled:inEnabled];
            
            // min
//            if ( (int)inMinValue == kFBParamMissing ) inMinValue = 0.0;
//            [stepper setMinValue:inMinValue];
//
//            // max
//            if ( (int)inMaxValue == kFBParamMissing ) inMaxValue = 100.0;
//            [stepper setMaxValue:inMaxValue];
//
//            // value
//            if ( (int)inValue == kFBParamMissing ) inValue = inMinValue;
//            if ( inValue <= inMaxValue ) [stepper setDoubleValue:inValue];
            
            
            // min
            if ( (int)inMinValue != kFBParamMissing ) [stepper setMinValue:inMinValue];
            
            // max
            if ( (int)inMaxValue != kFBParamMissing ) [stepper setMaxValue:inMaxValue];
            
            // value
            if ( (int)inValue != kFBParamMissing ) {
                if ( inValue <= [stepper maxValue] ) [stepper setDoubleValue:inValue];
            }

            // value wraps
//            [stepper setValueWraps:NO];
            
            // wndTag
            [stepper setWndTag:inWndTag];
            
            // target/action
//            [stepper setTarget:CocoaControlTargetObj()];
//            [stepper setAction:@selector(controlAction:)];
            
            // add to window
            [contentView addSubview:stepper];
//            [stepper release];
        }
        
        [stepper setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

