/*
 String.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef void (*CocoaStringEnumerateLinesCallbackType)(CFStringRef,BOOL*);
typedef void (*CocoaStringEnumerateSubstringsCallbackType)(CFStringRef,CFRange,CFRange,BOOL*);
typedef BOOL (*CocoaStringLinguisticTaggerCallbackType)(CFStringRef,CFRange,CFRange,void*);

// Init
CFStringRef StringWithBytes( void *bytes, NSUInteger length, NSStringEncoding encoding );
CFStringRef StringWithCharacters( const unichar *characters, NSUInteger length );
CFStringRef StringWithData( CFDataRef dta, NSStringEncoding encoding );
CFStringRef StringWithFormat( CFStringRef format, ... );
CFStringRef StringWithCString( const char *cString, NSStringEncoding encoding );
CFStringRef StringWithUTF8String( const char *cString );

// Init from URL
CFStringRef StringWithContentsOfURL( CFURLRef url, NSStringEncoding enc, ErrorRef *err );
CFStringRef StringWithContentsOfURLUsedEncoding( CFURLRef url, NSStringEncoding *enc, ErrorRef *err );

// Length
NSUInteger StringLength( CFStringRef string );

// Characters and bytes
unichar StringCharacterAtIndex( CFStringRef string, NSUInteger index );

// Getting c strings
void *StringCStringUsingEncoding( CFStringRef string, NSStringEncoding enc );
void *StringUTF8String( CFStringRef string );

// Identify and compare
NSComparisonResult StringCaseInsensitiveCompare( CFStringRef string, CFStringRef otherString );
NSComparisonResult StringLocalizedCaseInsensitiveCompare( CFStringRef string, CFStringRef otherString );
NSComparisonResult StringCompare( CFStringRef string, CFStringRef otherString );
NSComparisonResult StringLocalizedCompare( CFStringRef string, CFStringRef otherString );
NSComparisonResult StringCompareWithOptions( CFStringRef string, CFStringRef otherString, NSStringCompareOptions options );
NSComparisonResult StringCompareWithOptionsInRange( CFStringRef string, CFStringRef otherString, NSStringCompareOptions options, CFRange range );
NSComparisonResult StringCompareWithOptionsInRangeWithLocale( CFStringRef string, CFStringRef otherString, NSStringCompareOptions options, CFRange range, CFLocaleRef locale );
NSComparisonResult StringLocalizedStandardCompare( CFStringRef string, CFStringRef otherString );
Boolean StringHasPrefix( CFStringRef string, CFStringRef prefix );
Boolean StringHasSuffix( CFStringRef string, CFStringRef suffix );
Boolean StringIsEqualToString( CFStringRef string, CFStringRef otherString );

// Combine
CFStringRef StringByAppendingFormat( CFStringRef string, CFStringRef format, ... );
CFStringRef StringByAppendingString( CFStringRef string1, CFStringRef string2 );
CFStringRef StringByPaddingToLength( CFStringRef string, NSUInteger newLength, CFStringRef padString, NSUInteger padIndex );

// Case
CFStringRef StringLowercaseString( CFStringRef string );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFStringRef StringLocalizedLowercaseString( CFStringRef string );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFStringRef StringLowercaseStringWithLocale( CFStringRef string, CFLocaleRef locale );
#endif// 1080

CFStringRef StringUppercaseString( CFStringRef string );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFStringRef StringLocalizedUppercaseString( CFStringRef string );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFStringRef StringUppercaseStringWithLocale( CFStringRef string, CFLocaleRef locale );
#endif// 1080

CFStringRef StringCapitalizedString( CFStringRef string );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFStringRef StringLocalizedCapitalizedString( CFStringRef string );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFStringRef StringCapitalizedStringWithLocale( CFStringRef string, CFLocaleRef locale );
#endif// 1080

// Divide
CFArrayRef StringComponentsSeparatedByString( CFStringRef string, CFStringRef separator );
CFArrayRef StringComponentsSeparatedByCharactersInSet( CFStringRef string, CFCharacterSetRef set );
CFStringRef StringByTrimmingCharactersInSet( CFStringRef string, CFCharacterSetRef set );
CFStringRef StringSubstringFromIndex( CFStringRef string, NSUInteger index );
CFStringRef StringSubstringWithRange( CFStringRef string, CFRange range );
CFStringRef StringSubstringToIndex( CFStringRef string, NSUInteger index );

// Normalize

// Fold
CFStringRef StringByFolding( CFStringRef string, NSStringCompareOptions options, CFLocaleRef locale );

// Transform
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFStringRef StringByApplyingTransform( CFStringRef string, CFStringRef transform, Boolean reverse );
#endif// 101100

// Find
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean StringContainsString( CFStringRef string1, CFStringRef string2 );
Boolean StringLocalizedCaseInsensitiveContainsString( CFStringRef string1, CFStringRef string2 );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
//Boolean StringLocalizedStandardContainsString( CFStringRef string1, CFStringRef string2 );
#endif

CFRange StringRangeOfCharacterFromSet( CFStringRef string, CFCharacterSetRef set );
CFRange StringRangeOfCharacterFromSetWithOptions( CFStringRef string, CFCharacterSetRef set, NSStringCompareOptions options );
CFRange StringRangeOfCharacterFromSetWithOptionsInRange( CFStringRef string, CFCharacterSetRef set, NSStringCompareOptions options, CFRange inRange );
CFRange StringRangeOfString( CFStringRef string, CFStringRef searchString );
CFRange StringRangeOfStringWithOptions( CFStringRef string, CFStringRef searchString, NSStringCompareOptions options );
CFRange StringRangeOfStringWithOptionsInRange( CFStringRef string, CFStringRef searchString, NSStringCompareOptions options, CFRange inRange );
CFRange StringRangeOfStringWithOptionsInRangeAndLocale( CFStringRef string, CFStringRef searchString, NSStringCompareOptions options, CFRange inRange, CFLocaleRef locale );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFRange StringLocalizedStandardRangeOfString( CFStringRef string1, CFStringRef string2 );
#endif

void StringEnumerateLines( CFStringRef string, CocoaStringEnumerateLinesCallbackType callback );
void StringEnumerateSubstringsInRange( CFStringRef string, CFRange range, NSStringEnumerationOptions options, CocoaStringEnumerateSubstringsCallbackType callback );

// Replace substrings
CFStringRef StringByReplacingOccurencesOfString( CFStringRef string, CFStringRef target, CFStringRef replacement );
CFStringRef StringByReplacingOccurencesOfStringWithOptions( CFStringRef string, CFStringRef target, CFStringRef replacement, NSStringCompareOptions options, CFRange range );
CFStringRef StringByReplacingCharactersInRange( CFStringRef string, CFRange range, CFStringRef replacement );

// Shared prefix
CFStringRef StringCommonPrefixWithString( CFStringRef string1, CFStringRef string2, NSStringCompareOptions options );

// Linguistic analysis
void StringEnumerateLinguisticTags( CFStringRef string, CFRange range, CFStringRef scheme, NSLinguisticTaggerOptions options, OrthographyRef orthRef, CocoaStringLinguisticTaggerCallbackType callback );
CFArrayRef StringLinguisticTagsInRange( CFStringRef string, CFRange range, CFStringRef scheme, NSLinguisticTaggerOptions options, OrthographyRef orthRef, CFArrayRef *tokenRanges );

// Line and paragraph ranges
void StringGetLineStartEnd( CFStringRef string, NSUInteger *startPtr, NSUInteger *lineEndPtr, NSUInteger *contentsEndPtr, CFRange range );
CFRange StringLineRange( CFStringRef string, CFRange range );
void StringGetParagraphStartEnd( CFStringRef string, NSUInteger *startPtr, NSUInteger *parEndPtr, NSUInteger *contentsEndPtr, CFRange range );
CFRange StringParagraphRange( CFStringRef string, CFRange range );

// Composed char sequences
CFRange StringRangeOfComposedCharacterSequenceAtIndex( CFStringRef string, NSUInteger index );
CFRange StringRangeOfComposedCharacterSequencesForRange( CFStringRef string, CFRange range );

// Write to URL
Boolean StringWriteToURL( CFStringRef string, CFURLRef url, Boolean atomically, NSStringEncoding encoding, ErrorRef *err );

// Property list
CFTypeRef StringPropertyList( CFStringRef string );
CFDictionaryRef StringPropertyListFromStringsFileFormat( CFStringRef string );

// Sizing and drawing
void StringDrawAtPoint( CFStringRef string, CGPoint pt, CFDictionaryRef attributes );
void StringDrawInRect( CFStringRef string, CGRect rect, CFDictionaryRef attributes );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void StringDrawWithRect( CFStringRef string, CGRect rect, NSStringDrawingOptions options, CFDictionaryRef attributes, StringDrawingContextRef context );
CGRect StringBoundingRectWithSize( CFStringRef string, CGSize size, NSStringDrawingOptions options, CFDictionaryRef attributes, StringDrawingContextRef context );
#endif// 101100

CGSize StringSizeWithAttributes( CFStringRef string, CFDictionaryRef attributes );

// Numeric values
double StringDoubleValue( CFStringRef string );
float StringFloatValue( CFStringRef string );
SInt32 StringIntValue( CFStringRef string );
NSInteger StringIntegerValue( CFStringRef string );
SInt64 StringLongLongValue( CFStringRef string );
Boolean StringBoolValue( CFStringRef string );

// Working with encodings
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSStringEncoding StringEncodingForData( CFDataRef dta, CFDictionaryRef encodingOptions, CFStringRef *convertedString, Boolean *usedLossyConversion );
#endif
CFStringRef StringLocalizedNameOfStringEncoding( NSStringEncoding encoding );
Boolean StringCanBeConvertedToEncoding( CFStringRef string, NSStringEncoding encoding );
CFDataRef StringData( CFStringRef string, NSStringEncoding encoding );
CFDataRef StringDataAllowLossyConversion( CFStringRef string, NSStringEncoding encoding, Boolean lossy );
NSStringEncoding StringFastestEncoding( CFStringRef string );
NSStringEncoding StringSmallestEncoding( CFStringRef string );

// Paths
CFStringRef StringPathWithComponents( CFArrayRef array );
CFArrayRef StringPathComponents( CFStringRef string );
Boolean StringIsAbsolutePath( CFStringRef string );
CFStringRef StringLastPathComponent( CFStringRef string );
CFStringRef StringPathExtension( CFStringRef string );
CFStringRef StringByAbbreviatingWithTildeInPath( CFStringRef string );
CFStringRef StringByAppendingPathComponent( CFStringRef string, CFStringRef component );
CFStringRef StringByAppendingPathExtension( CFStringRef string, CFStringRef extension );
CFStringRef StringByDeletingLastPathComponent( CFStringRef string );
CFStringRef StringByDeletingPathExtension( CFStringRef string );
CFStringRef StringByExpandingTildeInPath( CFStringRef string );
CFStringRef StringByResolvingSymlinksInPath( CFStringRef string );
CFStringRef StringByStandardizingPath( CFStringRef string );
CFArrayRef StringStringsByAppendingPaths( CFStringRef string, CFArrayRef paths );

// Working with URL strings
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFStringRef StringByAddingPercentEncodingWithAllowedCharacters( CFStringRef string, CFCharacterSetRef allowedCharacters );
CFStringRef StringByRemovingPercentEncoding( CFStringRef string );
#endif

/*
 mutable string
 */
// Create
CFMutableStringRef StringWithCapacity( NSUInteger capacity );

// Modify
void StringAppendFormat( CFMutableStringRef string, CFStringRef format, ... );
void StringAppendString( CFMutableStringRef string, CFStringRef otherString );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void StringApplyTransform( CFMutableStringRef string, CFStringRef transform, Boolean reverse, CFRange range, CFRange *resultingRange );
#endif

void StringDeleteCharacters( CFMutableStringRef string, CFRange range );
void StringInsertString( CFMutableStringRef string, CFStringRef otherString, NSUInteger index );
void StringReplaceCharacters( CFMutableStringRef string, CFStringRef otherString, CFRange range );
void StringReplaceOccurrencesOfString( CFMutableStringRef string, CFStringRef targetString, CFStringRef replacementString, NSStringCompareOptions options, CFRange range );
void StringSetString( CFMutableStringRef string, CFStringRef otherString );

