/*
 String.m

 Bernie Wylde
 */

#import "String.h"

// Init
CFStringRef StringWithBytes( void *bytes, NSUInteger length, NSStringEncoding encoding )
{ return (CFStringRef)[[[NSString alloc] initWithBytes:bytes length:length encoding:encoding] autorelease]; }

CFStringRef StringWithCharacters( const unichar *characters, NSUInteger length )
{ return (CFStringRef)[[[NSString alloc] initWithCharacters:characters length:length] autorelease]; }

CFStringRef StringWithData( CFDataRef dta, NSStringEncoding encoding )
{ return (CFStringRef)[[[NSString alloc] initWithData:(NSData *)dta encoding:encoding] autorelease]; }

CFStringRef StringWithFormat( CFStringRef format, ... )
{
    va_list ap;
    va_start( ap, format );
    NSString *string = [[[NSString alloc] initWithFormat:(NSString *)format arguments:ap] autorelease];
    va_end( ap );
    return (CFStringRef)string;
}

CFStringRef StringWithCString( const char *cString, NSStringEncoding encoding )
{ return (CFStringRef)[NSString stringWithCString:cString encoding:encoding]; }

CFStringRef StringWithUTF8String( const char *cString )
{ return (CFStringRef)[NSString stringWithUTF8String:cString]; }

// Init from URL
CFStringRef StringWithContentsOfURL( CFURLRef url, NSStringEncoding enc, ErrorRef *err )
{ return (CFStringRef)[NSString stringWithContentsOfURL:(NSURL *)url encoding:enc error:err]; }

CFStringRef StringWithContentsOfURLUsedEncoding( CFURLRef url, NSStringEncoding *enc, ErrorRef *err )
{ return (CFStringRef)[NSString stringWithContentsOfURL:(NSURL *)url usedEncoding:enc error:err]; }

// Length
NSUInteger StringLength( CFStringRef string )
{ return [(NSString *)string length]; }

// Characters and bytes
unichar StringCharacterAtIndex( CFStringRef string, NSUInteger index )
{ return [(NSString *)string characterAtIndex:index]; }

// Getting c strings
void *StringCStringUsingEncoding( CFStringRef string, NSStringEncoding enc )
{ return (void *)[(NSString *)string cStringUsingEncoding:enc]; }

void *StringUTF8String( CFStringRef string )
{ return (void *)[(NSString *)string UTF8String]; }

// Identify and compare
NSComparisonResult StringCaseInsensitiveCompare( CFStringRef string, CFStringRef otherString )
{ return [(NSString *)string caseInsensitiveCompare:(NSString *)otherString]; }

NSComparisonResult StringLocalizedCaseInsensitiveCompare( CFStringRef string, CFStringRef otherString )
{ return [(NSString *)string localizedCaseInsensitiveCompare:(NSString *)otherString]; }

NSComparisonResult StringCompare( CFStringRef string, CFStringRef otherString )
{ return [(NSString *)string compare:(NSString *)otherString]; }

NSComparisonResult StringLocalizedCompare( CFStringRef string, CFStringRef otherString )
{ return [(NSString *)string localizedCompare:(NSString *)otherString]; }

NSComparisonResult StringCompareWithOptions( CFStringRef string, CFStringRef otherString, NSStringCompareOptions options )
{ return [(NSString *)string compare:(NSString *)otherString options:options]; }

NSComparisonResult StringCompareWithOptionsInRange( CFStringRef string, CFStringRef otherString, NSStringCompareOptions options, CFRange range )
{ return [(NSString *)string compare:(NSString *)otherString options:options range:NSRangeFromCFRange(range)]; }

NSComparisonResult StringCompareWithOptionsInRangeWithLocale( CFStringRef string, CFStringRef otherString, NSStringCompareOptions options, CFRange range, CFLocaleRef locale )
{ return [(NSString *)string compare:(NSString *)otherString options:options range:NSRangeFromCFRange(range) locale:(NSLocale *)locale]; }

NSComparisonResult StringLocalizedStandardCompare( CFStringRef string, CFStringRef otherString )
{ return [(NSString *)string localizedStandardCompare:(NSString *)otherString]; }

Boolean StringHasPrefix( CFStringRef string, CFStringRef prefix )
{ return [(NSString *)string hasPrefix:(NSString *)prefix]; }

Boolean StringHasSuffix( CFStringRef string, CFStringRef suffix )
{ return [(NSString *)string hasSuffix:(NSString *)suffix]; }

Boolean StringIsEqualToString( CFStringRef string, CFStringRef otherString )
{ return [(NSString *)string isEqualToString:(NSString *)otherString]; }

// Combine
CFStringRef StringByAppendingFormat( CFStringRef string, CFStringRef format, ... )
{
    va_list ap;
    va_start( ap, format );
    NSString *formattedString = [[NSString alloc] initWithFormat:(NSString *)format arguments:ap];
    NSString *outString = [(NSString *)string stringByAppendingString:formattedString];
    [formattedString release];
    va_end( ap );
    return (CFStringRef)outString;
}

CFStringRef StringByAppendingString( CFStringRef string1, CFStringRef string2 )
{ return (CFStringRef)[(NSString *)string1 stringByAppendingString:(NSString *)string2]; }

CFStringRef StringByPaddingToLength( CFStringRef string, NSUInteger newLength, CFStringRef padString, NSUInteger padIndex )
{ return (CFStringRef)[(NSString *)string stringByPaddingToLength:newLength withString:(NSString *)padString startingAtIndex:padIndex]; }

// Case
CFStringRef StringLowercaseString( CFStringRef string )
{ return (CFStringRef)[(NSString *)string lowercaseString]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFStringRef StringLocalizedLowercaseString( CFStringRef string )
{ return (CFStringRef)[(NSString *)string localizedLowercaseString]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFStringRef StringLowercaseStringWithLocale( CFStringRef string, CFLocaleRef locale )
{ return (CFStringRef)[(NSString *)string lowercaseStringWithLocale:(NSLocale *)locale]; }
#endif// 1080

CFStringRef StringUppercaseString( CFStringRef string )
{ return (CFStringRef)[(NSString *)string uppercaseString]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFStringRef StringLocalizedUppercaseString( CFStringRef string )
{ return (CFStringRef)[(NSString *)string localizedUppercaseString]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFStringRef StringUppercaseStringWithLocale( CFStringRef string, CFLocaleRef locale )
{ return (CFStringRef)[(NSString *)string uppercaseStringWithLocale:(NSLocale *)locale]; }
#endif// 1080

CFStringRef StringCapitalizedString( CFStringRef string )
{ return (CFStringRef)[(NSString *)string capitalizedString]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFStringRef StringLocalizedCapitalizedString( CFStringRef string )
{ return (CFStringRef)[(NSString *)string localizedCapitalizedString]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CFStringRef StringCapitalizedStringWithLocale( CFStringRef string, CFLocaleRef locale )
{ return (CFStringRef)[(NSString *)string capitalizedStringWithLocale:(NSLocale *)locale]; }
#endif// 1080

// Divide
CFArrayRef StringComponentsSeparatedByString( CFStringRef string, CFStringRef separator )
{ return (CFArrayRef)[(NSString *)string componentsSeparatedByString:(NSString *)separator]; }

CFArrayRef StringComponentsSeparatedByCharactersInSet( CFStringRef string, CFCharacterSetRef set )
{ return (CFArrayRef)[(NSString *)string componentsSeparatedByCharactersInSet:(NSCharacterSet *)set]; }

CFStringRef StringByTrimmingCharactersInSet( CFStringRef string, CFCharacterSetRef set )
{ return (CFStringRef)[(NSString *)string stringByTrimmingCharactersInSet:(NSCharacterSet *)set]; }

CFStringRef StringSubstringFromIndex( CFStringRef string, NSUInteger index )
{ return (CFStringRef)[(NSString *)string substringFromIndex:index]; }

CFStringRef StringSubstringWithRange( CFStringRef string, CFRange range )
{ return (CFStringRef)[(NSString *)string substringWithRange:NSRangeFromCFRange(range)]; }

CFStringRef StringSubstringToIndex( CFStringRef string, NSUInteger index )
{ return (CFStringRef)[(NSString *)string substringToIndex:index]; }

// Normalize

// Fold
CFStringRef StringByFolding( CFStringRef string, NSStringCompareOptions options, CFLocaleRef locale )
{ return (CFStringRef)[(NSString *)string stringByFoldingWithOptions:options locale:(NSLocale *)locale]; }

// Transform
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFStringRef StringByApplyingTransform( CFStringRef string, CFStringRef transform, Boolean reverse )
{ return (CFStringRef)[(NSString *)string stringByApplyingTransform:(NSString *)transform reverse:reverse]; }
#endif// 101100

// Find
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean StringContainsString( CFStringRef string1, CFStringRef string2 )
{ return [(NSString *)string1 containsString:(NSString *)string2]; }

Boolean StringLocalizedCaseInsensitiveContainsString( CFStringRef string1, CFStringRef string2 )
{ return [(NSString *)string1 localizedCaseInsensitiveContainsString:(NSString *)string2]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean StringLocalizedStandardContainsString( CFStringRef string1, CFStringRef string2 )
{ return [(NSString *)string1 localizedStandardContainsString:(NSString *)string2]; }
#endif

CFRange StringRangeOfCharacterFromSet( CFStringRef string, CFCharacterSetRef set )
{
    NSRange range = [(NSString *)string rangeOfCharacterFromSet:(NSCharacterSet *)set];
    return NSRangeToCFRange( range );
}

CFRange StringRangeOfCharacterFromSetWithOptions( CFStringRef string, CFCharacterSetRef set, NSStringCompareOptions options )
{
    NSRange range = [(NSString *)string rangeOfCharacterFromSet:(NSCharacterSet *)set options:options];
    return NSRangeToCFRange( range );
}

CFRange StringRangeOfCharacterFromSetWithOptionsInRange( CFStringRef string, CFCharacterSetRef set, NSStringCompareOptions options, CFRange inRange )
{
    NSRange range = [(NSString *)string rangeOfCharacterFromSet:(NSCharacterSet *)set options:options range:NSRangeFromCFRange(inRange)];
    return NSRangeToCFRange( range );
}

CFRange StringRangeOfString( CFStringRef string, CFStringRef searchString )
{
    NSRange range = [(NSString *)string rangeOfString:(NSString *)searchString];
    return NSRangeToCFRange( range );
}

CFRange StringRangeOfStringWithOptions( CFStringRef string, CFStringRef searchString, NSStringCompareOptions options )
{
    NSRange range = [(NSString *)string rangeOfString:(NSString *)searchString options:options];
    return NSRangeToCFRange( range );
}

CFRange StringRangeOfStringWithOptionsInRange( CFStringRef string, CFStringRef searchString, NSStringCompareOptions options, CFRange inRange )
{
    NSRange range = [(NSString *)string rangeOfString:(NSString *)searchString options:options range:NSRangeFromCFRange(inRange)];
    return NSRangeToCFRange( range );
}

CFRange StringRangeOfStringWithOptionsInRangeAndLocale( CFStringRef string, CFStringRef searchString, NSStringCompareOptions options, CFRange inRange, CFLocaleRef locale )
{
    NSRange range = [(NSString *)string rangeOfString:(NSString *)searchString options:options range:NSRangeFromCFRange(inRange) locale:(NSLocale *)locale];
    return NSRangeToCFRange( range );
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFRange StringLocalizedStandardRangeOfString( CFStringRef string1, CFStringRef string2 )
{
    NSRange range = [(NSString *)string1 localizedStandardRangeOfString:(NSString *)string2];
    return NSRangeToCFRange( range );
}
#endif

void StringEnumerateLines( CFStringRef string, CocoaStringEnumerateLinesCallbackType callback )
{
    [(NSString *)string enumerateLinesUsingBlock:^(NSString *line, BOOL *stop) {
        BOOL stopFlag = NO;
        (*callback)( (CFStringRef)line, &stopFlag );
        if ( stopFlag ) *stop = YES;
    }];
}

void StringEnumerateSubstringsInRange( CFStringRef string, CFRange range, NSStringEnumerationOptions options, CocoaStringEnumerateSubstringsCallbackType callback )
{
    [(NSString *)string enumerateSubstringsInRange:NSRangeFromCFRange(range) options:options usingBlock:^(NSString *substring, NSRange substringRange, NSRange enclosingRange, BOOL *stop) {
        BOOL stopFlag = NO;
        (*callback)( (CFStringRef)substring, NSRangeToCFRange(substringRange), NSRangeToCFRange(enclosingRange), &stopFlag );
        if ( stopFlag ) *stop = YES;
    }];
}

// Replace substrings
CFStringRef StringByReplacingOccurencesOfString( CFStringRef string, CFStringRef target, CFStringRef replacement )
{ return (CFStringRef)[(NSString *)string stringByReplacingOccurrencesOfString:(NSString *)target withString:(NSString *)replacement]; }

CFStringRef StringByReplacingOccurencesOfStringWithOptions( CFStringRef string, CFStringRef target, CFStringRef replacement, NSStringCompareOptions options, CFRange range )
{ return (CFStringRef)[(NSString *)string stringByReplacingOccurrencesOfString:(NSString *)target withString:(NSString *)replacement options:options range:NSRangeFromCFRange(range)]; }

CFStringRef StringByReplacingCharactersInRange( CFStringRef string, CFRange range, CFStringRef replacement )
{ return (CFStringRef)[(NSString *)string stringByReplacingCharactersInRange:NSRangeFromCFRange(range) withString:(NSString *)replacement]; }

// Shared prefix
CFStringRef StringCommonPrefixWithString( CFStringRef string1, CFStringRef string2, NSStringCompareOptions options )
{ return (CFStringRef)[(NSString *)string1 commonPrefixWithString:(NSString *)string2 options:options]; }

// Linguistic analysis
void StringEnumerateLinguisticTags( CFStringRef string, CFRange range, CFStringRef scheme, NSLinguisticTaggerOptions options, OrthographyRef orthRef, CocoaStringLinguisticTaggerCallbackType callback )
{
    [(NSString *)string enumerateLinguisticTagsInRange:NSRangeFromCFRange(range)
                                                scheme:(NSString *)scheme
                                               options:options
                                           orthography:orthRef
                                            usingBlock:^(NSString *tag, NSRange tokenRange, NSRange sentenceRange, BOOL *stop) {
                                                BOOL stopFlag = NO;
                                                (*callback)( (CFStringRef)tag, NSRangeToCFRange(tokenRange), NSRangeToCFRange(sentenceRange), &stopFlag );
                                                if ( stopFlag ) *stop = YES;
                                            }];
}

CFArrayRef StringLinguisticTagsInRange( CFStringRef string, CFRange range, CFStringRef scheme, NSLinguisticTaggerOptions options, OrthographyRef orthRef, CFArrayRef *tokenRanges )
{ return (CFArrayRef)[(NSString *)string linguisticTagsInRange:NSRangeFromCFRange(range) scheme:(NSString *)scheme options:options orthography:orthRef tokenRanges:(NSArray **)tokenRanges]; }

// Line and paragraph ranges
void StringGetLineStartEnd( CFStringRef string, NSUInteger *startPtr, NSUInteger *lineEndPtr, NSUInteger *contentsEndPtr, CFRange range )
{ [(NSString *)string getLineStart:startPtr end:lineEndPtr contentsEnd:contentsEndPtr forRange:NSRangeFromCFRange(range)]; }

CFRange StringLineRange( CFStringRef string, CFRange range )
{ return NSRangeToCFRange([(NSString *)string lineRangeForRange:NSRangeFromCFRange(range)]); }

void StringGetParagraphStartEnd( CFStringRef string, NSUInteger *startPtr, NSUInteger *parEndPtr, NSUInteger *contentsEndPtr, CFRange range )
{ [(NSString *)string getParagraphStart:startPtr end:parEndPtr contentsEnd:contentsEndPtr forRange:NSRangeFromCFRange(range)]; }

CFRange StringParagraphRange( CFStringRef string, CFRange range )
{ return NSRangeToCFRange([(NSString *)string paragraphRangeForRange:NSRangeFromCFRange(range)]); }

// Composed char sequences
CFRange StringRangeOfComposedCharacterSequenceAtIndex( CFStringRef string, NSUInteger index )
{ return NSRangeToCFRange([(NSString *)string rangeOfComposedCharacterSequenceAtIndex:index]); }

CFRange StringRangeOfComposedCharacterSequencesForRange( CFStringRef string, CFRange range )
{ return NSRangeToCFRange([(NSString *)string rangeOfComposedCharacterSequencesForRange:NSRangeFromCFRange(range)]); }

// Write to URL
Boolean StringWriteToURL( CFStringRef string, CFURLRef url, Boolean atomically, NSStringEncoding encoding, ErrorRef *err )
{ return [(NSString *)string writeToURL:(NSURL *)url atomically:atomically encoding:encoding error:err]; }

// Property list
CFTypeRef StringPropertyList( CFStringRef string )
{ return (CFTypeRef)[(NSString *)string propertyList]; }

CFDictionaryRef StringPropertyListFromStringsFileFormat( CFStringRef string )
{ return (CFDictionaryRef)[(NSString *)string propertyListFromStringsFileFormat]; }

// Sizing and drawing
void StringDrawAtPoint( CFStringRef string, CGPoint pt, CFDictionaryRef attributes )
{ [(NSString *)string drawAtPoint:NSPointFromCGPoint(pt) withAttributes:(NSDictionary *)attributes]; }

void StringDrawInRect( CFStringRef string, CGRect rect, CFDictionaryRef attributes )
{ [(NSString *)string drawInRect:NSRectFromCGRect(rect) withAttributes:(NSDictionary *)attributes]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void StringDrawWithRect( CFStringRef string, CGRect rect, NSStringDrawingOptions options, CFDictionaryRef attributes, StringDrawingContextRef context )
{ [(NSString *)string drawWithRect:NSRectFromCGRect(rect) options:options attributes:(NSDictionary *)attributes context:context]; }

CGRect StringBoundingRectWithSize( CFStringRef string, CGSize size, NSStringDrawingOptions options, CFDictionaryRef attributes, StringDrawingContextRef context )
{ return NSRectToCGRect([(NSString *)string boundingRectWithSize:NSSizeFromCGSize(size) options:options attributes:(NSDictionary *)attributes context:context]); }
#endif// 101100

CGSize StringSizeWithAttributes( CFStringRef string, CFDictionaryRef attributes )
{ return NSSizeToCGSize([(NSString *)string sizeWithAttributes:(NSDictionary *)attributes]); }

// Numeric values
double StringDoubleValue( CFStringRef string )
{ return [(NSString *)string doubleValue]; }

float StringFloatValue( CFStringRef string )
{ return [(NSString *)string floatValue]; }

SInt32 StringIntValue( CFStringRef string )
{ return [(NSString *)string intValue]; }

NSInteger StringIntegerValue( CFStringRef string )
{ return [(NSString *)string integerValue]; }

SInt64 StringLongLongValue( CFStringRef string )
{ return [(NSString *)string longLongValue]; }

Boolean StringBoolValue( CFStringRef string )
{ return [(NSString *)string boolValue]; }

// Working with encodings
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSStringEncoding StringEncodingForData( CFDataRef dta, CFDictionaryRef encodingOptions, CFStringRef *convertedString, Boolean *usedLossyConversion )
{ return [NSString stringEncodingForData:(NSData *)dta encodingOptions:(NSDictionary *)encodingOptions convertedString:(NSString **)convertedString usedLossyConversion:(BOOL *)usedLossyConversion]; }
#endif

CFStringRef StringLocalizedNameOfStringEncoding( NSStringEncoding encoding )
{ return (CFStringRef)[NSString localizedNameOfStringEncoding:encoding]; }

Boolean StringCanBeConvertedToEncoding( CFStringRef string, NSStringEncoding encoding )
{ return [(NSString *)string canBeConvertedToEncoding:encoding]; }

CFDataRef StringData( CFStringRef string, NSStringEncoding encoding )
{ return (CFDataRef)[(NSString *)string dataUsingEncoding:encoding]; }

CFDataRef StringDataAllowLossyConversion( CFStringRef string, NSStringEncoding encoding, Boolean lossy )
{ return (CFDataRef)[(NSString *)string dataUsingEncoding:encoding allowLossyConversion:lossy]; }

NSStringEncoding StringFastestEncoding( CFStringRef string )
{ return [(NSString *)string fastestEncoding]; }

NSStringEncoding StringSmallestEncoding( CFStringRef string )
{ return [(NSString *)string smallestEncoding]; }

// Paths
CFStringRef StringPathWithComponents( CFArrayRef array )
{ return (CFStringRef)[NSString pathWithComponents:(NSArray *)array]; }

CFArrayRef StringPathComponents( CFStringRef string )
{ return (CFArrayRef)[(NSString *)string pathComponents]; }

Boolean StringIsAbsolutePath( CFStringRef string )
{ return [(NSString *)string isAbsolutePath]; }

CFStringRef StringLastPathComponent( CFStringRef string )
{ return (CFStringRef)[(NSString *)string lastPathComponent]; }

CFStringRef StringPathExtension( CFStringRef string )
{ return (CFStringRef)[(NSString *)string pathExtension]; }

CFStringRef StringByAbbreviatingWithTildeInPath( CFStringRef string )
{ return (CFStringRef)[(NSString *)string stringByAbbreviatingWithTildeInPath]; }

CFStringRef StringByAppendingPathComponent( CFStringRef string, CFStringRef component )
{ return (CFStringRef)[(NSString *)string stringByAppendingPathComponent:(NSString *)component]; }

CFStringRef StringByAppendingPathExtension( CFStringRef string, CFStringRef extension )
{ return (CFStringRef)[(NSString *)string stringByAppendingPathExtension:(NSString *)extension]; }

CFStringRef StringByDeletingLastPathComponent( CFStringRef string )
{ return (CFStringRef)[(NSString *)string stringByDeletingLastPathComponent]; }

CFStringRef StringByDeletingPathExtension( CFStringRef string )
{ return (CFStringRef)[(NSString *)string stringByDeletingPathExtension]; }

CFStringRef StringByExpandingTildeInPath( CFStringRef string )
{ return (CFStringRef)[(NSString *)string stringByExpandingTildeInPath]; }

CFStringRef StringByResolvingSymlinksInPath( CFStringRef string )
{ return (CFStringRef)[(NSString *)string stringByResolvingSymlinksInPath]; }

CFStringRef StringByStandardizingPath( CFStringRef string )
{ return (CFStringRef)[(NSString *)string stringByStandardizingPath]; }

CFArrayRef StringStringsByAppendingPaths( CFStringRef string, CFArrayRef paths )
{ return (CFArrayRef)[(NSString *)string stringsByAppendingPaths:(NSArray *)paths]; }

// Working with URL strings
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFStringRef StringByAddingPercentEncodingWithAllowedCharacters( CFStringRef string, CFCharacterSetRef allowedCharacters )
{ return (CFStringRef)[(NSString *)string stringByAddingPercentEncodingWithAllowedCharacters:(NSCharacterSet *)allowedCharacters]; }

CFStringRef StringByRemovingPercentEncoding( CFStringRef string )
{ return (CFStringRef)[(NSString *)string stringByRemovingPercentEncoding]; }
#endif

/*
 mutable string
 */
// Create
CFMutableStringRef StringWithCapacity( NSUInteger capacity )
{ return (CFMutableStringRef)[NSMutableString stringWithCapacity:capacity]; }

// Modify
void StringAppendFormat( CFMutableStringRef string, CFStringRef format, ... )
{
    va_list ap;
    va_start( ap, format );
    NSString *formattedString = [[NSString alloc] initWithFormat:(NSString *)format arguments:ap];
    [(NSMutableString *)string appendString:formattedString];
    [formattedString release];
    va_end( ap );
}

void StringAppendString( CFMutableStringRef string, CFStringRef otherString )
{ [(NSMutableString *)string appendString:(NSString *)otherString]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void StringApplyTransform( CFMutableStringRef string, CFStringRef transform, Boolean reverse, CFRange range, CFRange *resultingRange )
{ [(NSMutableString *)string applyTransform:(NSString *)transform reverse:reverse range:NSRangeFromCFRange(range) updatedRange:(NSRangePointer)resultingRange]; }
#endif

void StringDeleteCharacters( CFMutableStringRef string, CFRange range )
{ [(NSMutableString *)string deleteCharactersInRange:NSRangeFromCFRange(range)]; }

void StringInsertString( CFMutableStringRef string, CFStringRef otherString, NSUInteger index )
{ [(NSMutableString *)string insertString:(NSString *)otherString atIndex:index]; }

void StringReplaceCharacters( CFMutableStringRef string, CFStringRef otherString, CFRange range )
{ [(NSMutableString *)string replaceCharactersInRange:NSRangeFromCFRange(range) withString:(NSString *)otherString]; }

void StringReplaceOccurrencesOfString( CFMutableStringRef string, CFStringRef targetString, CFStringRef replacementString, NSStringCompareOptions options, CFRange range )
{ [(NSMutableString *)string replaceOccurrencesOfString:(NSString *)targetString withString:(NSString *)replacementString options:options range:NSRangeFromCFRange(range)]; }

void StringSetString( CFMutableStringRef string, CFStringRef otherString )
{ [(NSMutableString *)string setString:(NSString *)otherString]; }

