/*
 TabView.m

 Bernie Wylde
 */

#import "TabView.h"

@implementation TabViewSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}
@end

@implementation NSTabView (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

@implementation CocoaTabViewDelegate

@synthesize boolVal;
@synthesize indexVal;

- (void)tabView:(NSTabView *)tabView callUserDefinedOnDialogFunctionWithEvent:(NSInteger)event object:(id)obj {
    NSInteger wndTag = [tabView wndTag];
    CocoaOutputWindowSetTag( wndTag );
    CallUserDefinedOnDialogFunction( event, [tabView tag], wndTag, obj );
}

- (void)tabViewDidChangeNumberOfTabViewItems:(NSTabView *)tabView {
    [self tabView:tabView callUserDefinedOnDialogFunctionWithEvent:tabViewDidChangeNumberOfTabViewItems object:nil];
}

- (BOOL)tabView:(NSTabView *)tabView shouldSelectTabViewItem:(NSTabViewItem *)tabViewItem {
    DialogEventSetBool( YES );
    [self tabView:tabView callUserDefinedOnDialogFunctionWithEvent:tabViewShouldSelectTabViewItem object:nil];
    return DialogEventBool();
}

- (void)tabView:(NSTabView *)tabView willSelectTabViewItem:(NSTabViewItem *)tabViewItem {
    DialogEventSetLong( [tabView indexOfTabViewItem:tabViewItem] );
    [self tabView:tabView callUserDefinedOnDialogFunctionWithEvent:tabViewWillSelectTabViewItem object:nil];
}

- (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem {
    DialogEventSetLong( [tabView indexOfTabViewItem:tabViewItem] );
    [self tabView:tabView callUserDefinedOnDialogFunctionWithEvent:1 object:nil];
}
@end

CocoaTabViewDelegate *CocoaTabViewDelegateGet( void )
{
    static CocoaTabViewDelegate *sCocoaTabViewDelegate = nil;
    if ( !sCocoaTabViewDelegate ) sCocoaTabViewDelegate = [[CocoaTabViewDelegate alloc] init];
    return sCocoaTabViewDelegate;
}

NSTabView *TabViewWithTag( NSInteger tag )
{ return (NSTabView *)CocoaValidateViewKindOfClass( tag, [NSTabView class] ); }

BOOL TabViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSTabView class] ); }


void TabViewSetType( NSInteger tag, NSTabViewType type )
{
    NSTabView *c = TabViewWithTag( tag );
    if ( c ) [c setTabViewType:type];
}

NSInteger TabViewIndex( NSInteger tag )
{
    NSTabView *c = TabViewWithTag( tag );
    if ( c ) {
        NSTabViewItem *item = [c selectedTabViewItem];
        if ( item ) return [c indexOfTabViewItem:item];
    }
    return 0;
}

void TabViewSelectFirstItem( NSInteger tag )
{
    NSTabView *tab = TabViewWithTag( tag );
    if ( tab ) [tab selectFirstTabViewItem:nil];
}

void TabViewSelectLastItem( NSInteger tag )
{
    NSTabView *tab = TabViewWithTag( tag );
    if ( tab ) [tab selectLastTabViewItem:nil];
}

void TabViewSelectNextItem( NSInteger tag )
{
    NSTabView *tab = TabViewWithTag( tag );
    if ( tab ) [tab selectNextTabViewItem:nil];
}

void TabViewSelectPreviousItem( NSInteger tag )
{
    NSTabView *tab = TabViewWithTag( tag );
    if ( tab ) [tab selectPreviousTabViewItem:nil];
}

void TabViewSetFont( NSInteger tag, CTNSFontRef inFont )
{
    NSTabView *tab = TabViewWithTag( tag );
    if ( tab ) {
        NSFont *font = FixFont( (void *)inFont );
        [tab setFont:font];
    }
}

void TabViewSetControlTint( NSInteger tag, NSControlTint tint )
{
    NSTabView *tab = TabViewWithTag( tag );
    if ( tab ) [tab setControlTint:tint];
}

void TabViewSetDrawsBackground( NSInteger tag, Boolean flag )
{
    NSTabView *tab = TabViewWithTag( tag );
    if ( tab ) [tab setDrawsBackground:flag];
}

CGRect TabViewContentRect( NSInteger tag )
{
    NSTabView *tab = TabViewWithTag( tag );
    if ( tab ) return NSRectToCGRect([tab contentRect]);
    return CGRectZero;
}

// custom
void TabViewAddSubview( NSInteger tabTag, NSInteger index, NSInteger subviewTag )
{
    NSTabView *tabView = TabViewWithTag( tabTag );
    if ( tabView ) {
        NSTabViewItem *item = [tabView tabViewItemAtIndex:index];
        if ( item ) {
            id subview = CocoaViewWithTag( subviewTag );
            if ( subview ) {
                [[item view] addSubview:subview];
            }
        }
    }
}

// Init
TabViewRef TabViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 13, 2, 177, 124 );

    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    NSTabView *tab;
    if ( subclass ) {
        tab = [[[TabViewSC alloc] initWithFrame:rect] autorelease];
    } else {
        tab = [[[NSTabView alloc] initWithFrame:rect] autorelease];
    }

    // tag
    [tab setTag:inTag];

    // target
    [tab setDelegate:CocoaTabViewDelegateGet()];

    return tab;
}


// Statement
void CocoaTabView( NSInteger inTag, NSInteger inIndex, CFTypeRef inItems, CGRect inRect, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSTabView *tab = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !tab ) {
            tab = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( tab ) {    // exists
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [tab class], [NSTabView class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, tab, [NSTabView class] ) ) {
                return;
            }

            // index
            if ( inIndex != kFBParamMissing ) {
                if ( inIndex < [tab numberOfTabViewItems] ) [tab selectTabViewItemAtIndex:inIndex];
            }
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [tab setFrame:rect];
            
        } else {        // create
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 13, 2, 177, 124 );
            
            // init
//            if ( inSubclass ) {
//                tab = [[TabViewSC alloc] initWithFrame:rect];
//            } else {
//                tab = [[NSTabView alloc] initWithFrame:rect];
//            }
            
            tab = TabViewInit( inTag, inRect );
            
            // default autoresizing
            [tab setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [tab setTag:inTag];
            
            // items
            NSArray *items = nil;
            
            if ( inItems ) {
                if ( CFGetTypeID(inItems) == CFStringGetTypeID() ) {
                    items = [(NSString *)inItems componentsSeparatedByString:@";"];
                    
                } else if ( CFGetTypeID(inItems) == CFArrayGetTypeID() ) {
                    items = (NSArray *)inItems;
                }
            } else {
                items = [NSArray arrayWithObjects:@"Tab 1",@"Tab 2",nil];
            }
            
            NSInteger count = [items count];
            for ( NSInteger index = 0; index < count; index++ ) {
                NSTabViewItem *item = [[NSTabViewItem alloc] initWithIdentifier:[NSString stringWithFormat:@"%ld",(long)index+1]];
                [item setLabel:[items objectAtIndex:index]];
                [[item view] setFrame:[tab contentRect]];
                [tab addTabViewItem:item];
            }
            
            // index
            if ( inIndex != kFBParamMissing ) {
                if ( inIndex < [tab numberOfTabViewItems] ) [tab selectTabViewItemAtIndex:inIndex];
            }
            
            // wndTag
            [tab setWndTag:inWndTag];
            
            // fix font
            // CocoaControlFixFont( tab );
            
            // target
//            [tab setDelegate:CocoaTabViewDelegateGet()];
            
            // add to window
            [contentView addSubview:tab];
//            [tab release];
        }
        
        [tab setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

