/*
 TableView.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

#define TABLEVIEW_REORDER_PASTEBOARD_TYPE @"com.fb.tableview.reorder.item"

@interface TableViewSC : NSTableView
@end

// dialog events
enum {
    // delegate
    tableViewView = 1600,
    tableViewRowView,
    tableViewDidAddRowView,
    tableViewDidRemoveRowView,
    tableViewIsGroupRow,
    tableViewDataCell,
    tableViewShouldShowCellExpansion,
    tableViewToolTipForCell,
    tableViewShouldEdit,
    tableViewHeightOfRow,
    tableViewSizeToFitWidthOfColumn,
    tableViewSelectionShouldChange,
    tableViewShouldSelectRow,
    tableViewSelectionIndexes,
    tableViewShouldSelectTableColumn,
    tableViewSelectionIsChanging,
    tableViewSelectionDidChange,
    tableViewShouldTypeSelect,
    tableViewTypeSelectString,
    tableViewNextTypeSelectMatch,
    tableViewShouldReorderColumn,
    tableViewDidDragTableColumn,
    tableViewColumnDidMove,
    tableViewColumnDidResize,
    tableViewDidClickTableColumn,
    tableViewMouseDownInHeader,
    tableViewShouldTrackCell,
    tableViewRowActions,

    // data source
    tableViewNumberOfRows,
    tableViewPasteboardWriterForRow,
    tableViewAcceptDrop,
    tableViewValidateDrop,
    tableViewWriteRows,
    tableViewDraggingSessionWillBegin,
    tableViewUpdateDraggingItems,
    tableViewDraggingSessionEnded,
    tableViewSortDescriptorsDidChange,
    
    // other actions
    tableViewDoubleAction
};


@interface CocoaTableViewDelegate : NSObject <NSTableViewDelegate,NSTableViewDataSource,NSTextFieldDelegate> {
    NSTableView *table;
    NSMutableArray *tableData;
    NSInteger dragSourceIndex;
    
    // dialog event objects
    NSInteger eventNumberOfRows;
    NSView *eventCellView;
    NSString *eventCellIdentifier;
    NSInteger eventCellRow;
    NSString *eventCellString;
    NSImage *eventCellImage;
}
@property (assign) NSTableView *table;
@property (assign) NSMutableArray *tableData;
@property (assign) NSInteger dragSourceIndex;

// dialog event values
@property (assign) NSInteger eventNumberOfRows;
@property (assign) NSView *eventCellView;
@property (assign) NSString *eventCellIdentifier;
@property (assign) NSInteger eventCellRow;
@property (assign) NSString *eventCellString;
@property (assign) NSImage *eventCellImage;

- (void)doubleAction:(id)sender;
@end


typedef void (*CocoaTableViewCallbackType)(NSInteger,NSInteger);

// dialog events
void TableViewEventSetNumberOfRows( NSInteger tag, NSInteger value );
void TableViewEventSetCellView( NSInteger tag, ViewRef vw );
CFStringRef TableViewEventCellIdentifier( NSInteger tag );
NSInteger TableViewEventCellRow( NSInteger tag );

void TableViewEventSetCellString( NSInteger tag, CFStringRef string );
void TableViewEventSetCellImage( NSInteger tag, ImageRef image );


@interface NSTableView (Additions)
@property (nonatomic,retain) NSValue *callbackValue;
- (CocoaTableViewCallbackType)cocoaTableViewCallback;
- (void)setCocoaTableViewCallback:(CocoaTableViewCallbackType)cb;
@end

NSTableView *TableViewWithTag( NSInteger tag );
BOOL TableViewExists( NSInteger tag );

// Data source
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean TableViewUsesStaticContents( NSInteger tag );
void TableViewSetUsesStaticContents( NSInteger tag, Boolean flag );
#endif// 101000
void TableViewSetData( NSInteger tag, CFMutableArrayRef array );
CFMutableArrayRef TableViewData( NSInteger tag );
void TableViewReloadData( NSInteger tag );
void TableViewReloadDataForRowsColumns( NSInteger tag, IndexSetRef rows, IndexSetRef columns );

// Create views
TableCellViewRef TableViewMakeViewWithIdentifier( NSInteger tag, CFStringRef identifier );

// Update
void TableViewBeginUpdates( NSInteger tag );
void TableViewEndUpdates( NSInteger tag );
void TableViewMoveRow( NSInteger tag, NSInteger atIndex, NSInteger toIndex );

// Target-action
NSInteger TableViewClickedColumn( NSInteger tag );
NSInteger TableViewClickedRow( NSInteger tag );

// Behavior
Boolean TableViewAllowsColumnReordering( NSInteger tag );
void TableViewSetAllowsColumnReordering( NSInteger tag, Boolean flag );
Boolean TableViewAllowsColumnResizing( NSInteger tag );
void TableViewSetAllowsColumnResizing( NSInteger tag, Boolean flag );
Boolean TableViewAllowsMultipleSelection( NSInteger tag );
void TableViewSetAllowsMultipleSelection( NSInteger tag, Boolean flag );
Boolean TableViewAllowsEmptySelection( NSInteger tag );
void TableViewSetAllowsEmptySelection( NSInteger tag, Boolean flag );
Boolean TableViewAllowsColumnSelection( NSInteger tag );
void TableViewSetAllowsColumnSelection( NSInteger tag, Boolean flag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean TableViewUsesAutomaticRowHeights( NSInteger tag );
void TableViewSetUsesAutomaticRowHeights( NSInteger tag, Boolean flag );
#endif// 101300

// Display attributes
CGSize TableViewIntercellSpacing( NSInteger tag );
void TableViewSetIntercellSpacing( NSInteger tag, CGSize spacing );
CGFloat TableViewRowHeight( NSInteger tag );
void TableViewSetRowHeight( NSInteger tag, CGFloat height );
ColorRef TableViewBackgroundColor( NSInteger tag );
void TableViewSetBackgroundColor( NSInteger tag, ColorRef col );
Boolean TableViewUsesAlternatingRowBackgroundColors( NSInteger tag );
void TableViewSetUsesAlternatingRowBackgroundColors( NSInteger tag, Boolean flag );
NSTableViewSelectionHighlightStyle TableViewSelectionHighlightStyle( NSInteger tag );
void TableViewSetSelectionHighlightStyle( NSInteger tag, NSTableViewSelectionHighlightStyle style );
ColorRef TableViewGridColor( NSInteger tag );
void TableViewSetGridColor( NSInteger tag, ColorRef col );
NSTableViewGridLineStyle TableViewGridStyleMask( NSInteger tag );
void TableViewSetGridStyleMask( NSInteger tag, NSTableViewGridLineStyle style );
ImageRef TableViewIndicatorImageInTableColumn( NSInteger tag, TableColumnRef columnRef );
void TableViewSetIndicatorImageInTableColumn( NSInteger tag, ImageRef image, TableColumnRef columnRef );

// Row size styles
NSTableViewRowSizeStyle TableViewEffectiveRowSizeStyle( NSInteger tag );
NSTableViewRowSizeStyle TableViewRowSizeStyle( NSInteger tag );
void TableViewSetRowSizeStyle( NSInteger tag, NSTableViewRowSizeStyle style );

// Column management
void TableViewAddTableColumn( NSInteger tag, TableColumnRef columnRef );
void TableViewRemoveTableColumn( NSInteger tag, TableColumnRef columnRef );
void TableViewMoveColumn( NSInteger tag, NSInteger atIndex, NSInteger toIndex );
CFArrayRef TableViewTableColumns( NSInteger tag );
NSInteger TableViewColumnWithIdentifier( NSInteger tag, CFStringRef identifier );
NSInteger TableViewColumnWithIdentifier( NSInteger tag, CFStringRef identifier );
TableColumnRef TableViewTableColumnWithIdentifier( NSInteger tag, CFStringRef identifier );

// Selecting columns and rows
void TableViewSelectColumns( NSInteger tag, IndexSetRef columnIndexes, Boolean extendSelection );
NSInteger TableViewSelectedColumn( NSInteger tag );
IndexSetRef TableViewSelectedColumns( NSInteger tag );
void TableViewDeselectColumn( NSInteger tag, NSInteger columnIndex );
NSInteger TableViewNumberOfSelectedColumns( NSInteger tag );
Boolean TableViewIsColumnSelected( NSInteger tag, NSInteger columnIndex );
void TableViewSelectRows( NSInteger tag, IndexSetRef rowIndexes, Boolean extendSelection );
void TableViewSelectRow( NSInteger tag, NSInteger row );
NSInteger TableViewSelectedRow( NSInteger tag );
CFArrayRef TableViewSelectedRowIndexes( NSInteger tag );
void TableViewDeselectRow( NSInteger tag, NSInteger row );
NSInteger TableViewNumberOfSelectedRows( NSInteger tag );
Boolean TableViewIsRowSelected( NSInteger tag, NSInteger row );
void TableViewSelectAll( NSInteger tag );
void TableViewDeselectAll( NSInteger tag );

// Type select
Boolean TableViewAllowsTypeSelect( NSInteger tag );
void TableViewSetAllowsTypeSelect( NSInteger tag, Boolean flag );

// Dimensions
NSInteger TableViewNumberOfColumns( NSInteger tag );

// Floating rows
Boolean TableViewFloatsGroupRows( NSInteger tag );
void TableViewSetFloatsGroupRows( NSInteger tag, Boolean flag );

// Editing cells
void TableViewEditColumnRow( NSInteger tag, NSInteger columnIndex, NSInteger row, Boolean selectFlag );

// Auxiliary views
TableHeaderViewRef TableViewHeaderView( NSInteger tag );
void TableViewSetHeaderView( NSInteger tag, TableHeaderViewRef headerView );
ViewRef TableViewCornerView( NSInteger tag );
void TableViewSetCornerView( NSInteger tag, ViewRef cornerView );

// Layout support
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
NSUserInterfaceLayoutDirection TableViewUserInterfaceLayoutDirection( NSInteger tag );
void TableViewSetUserInterfaceLayoutDirection( NSInteger tag, NSUserInterfaceLayoutDirection direction );
#endif// 101200
CGRect TableViewRectOfColumn( NSInteger tag, NSInteger col );
CGRect TableViewRectOfRow( NSInteger tag, NSInteger row );
CFRange TableViewRowsInRect( NSInteger tag, CGRect r );
IndexSetRef TableViewColumnIndexesInRect( NSInteger tag, CGRect r );
NSInteger TableViewColumnAtPoint( NSInteger tag, CGPoint pt );
NSInteger TableViewRowAtPoint( NSInteger tag, CGPoint pt );
CGRect TableViewFrameOfCell( NSInteger tag, NSInteger col, NSInteger row );
NSTableViewColumnAutoresizingStyle TableViewColumnAutoresizingStyle( NSInteger tag );
void TableViewSetColumnAutoresizingStyle( NSInteger tag, NSTableViewColumnAutoresizingStyle style );
void TableViewSizeLastColumnToFit( NSInteger tag );
void TableViewTile( NSInteger tag );
void TableViewSizeToFit( NSInteger tag );
void TableViewNoteHeightOfRowsWithIndexesChanged( NSInteger tag, IndexSetRef indexSet );

// Scrolling
void TableViewScrollRowToVisible( NSInteger tag, NSInteger row );
void TableViewScrollColumnToVisible( NSInteger tag, NSInteger col );

// Column state persistence
Boolean TableViewAutosaveColumns( NSInteger tag );
void TableViewSetAutosaveColumns( NSInteger tag, Boolean flag );
CFStringRef TableViewAutosaveName( NSInteger tag );
void TableViewSetAutosaveName( NSInteger tag, CFStringRef name );

// Highlightable column headers
TableColumnRef TableViewHighlightedTableColumn( NSInteger tag );
void TableViewSetHighlightedTableColumn( NSInteger tag, TableColumnRef columnRef );

// Dragging
Boolean TableViewCanDragRows( NSInteger tag, IndexSetRef rowIndexes, CGPoint atPoint );
void TableViewSetDraggingSourceOperationMask( NSInteger tag, NSDragOperation dragOperation, Boolean forLocal );
Boolean TableViewVerticalMotionCanBeginDrag( NSInteger tag );
void TableViewSetVerticalMotionCanBeginDrag( NSInteger tag, Boolean flag );
NSTableViewDraggingDestinationFeedbackStyle TableViewDraggingDestinationFeedbackStyle( NSInteger tag );
void TableViewSetDraggingDestinationFeedbackStyle( NSInteger tag, NSTableViewDraggingDestinationFeedbackStyle style );
void TableViewSetDropRow( NSInteger tag, NSInteger row, NSTableViewDropOperation operation );

// Sorting
CFArrayRef TableViewSortDescriptors( NSInteger tag );
void TableViewSetSortDescriptors( NSInteger tag, CFArrayRef descriptors );

// Row actions
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean TableViewRowActionsVisible( NSInteger tag );
void TableViewSetRowActionsVisible( NSInteger tag, Boolean flag );
#endif// 101100

// Hiding and showing rows
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void TableViewHideRow( NSInteger tag, NSInteger rowIndex, NSTableViewAnimationOptions animation );
void TableViewHideRows( NSInteger tag, IndexSetRef indexes, NSTableViewAnimationOptions animation );
void TableViewUnhideRow( NSInteger tag, NSInteger rowIndex, NSTableViewAnimationOptions animation );
void TableViewUnhideRows( NSInteger tag, IndexSetRef indexes, NSTableViewAnimationOptions animation );
IndexSetRef TableViewHiddenRowIndexes( NSInteger tag );
#endif

// custom
void *TableViewView( NSInteger tag, NSInteger row, NSInteger col );
void TableViewSetDelegateCallback( NSInteger tag, void *callback );
void TableViewCellMakeFirstResponder( NSInteger tag, NSInteger row, NSInteger col );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void TableViewSetHeaderFont( NSInteger tag, FontRef font );
void TableViewSetHeaderTextColor( NSInteger tag, ColorRef textColor );
#endif


// Init
TableViewRef TableViewInit( NSInteger inTag, CGRect inRect );

// Statement
void CocoaTableView( NSInteger inTag, NSInteger inEnabled, CGRect inRect, CFTypeRef inColumnIdentifiers, NSInteger inScrollTag, NSInteger inWndTag );
