/*
 TableView.m

 Bernie Wylde
 */

#import "TableView.h"

@implementation TableViewSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end


@implementation CocoaTableViewDelegate
@synthesize table;
@synthesize tableData;
@synthesize dragSourceIndex;

// dialog event values
@synthesize eventNumberOfRows;
@synthesize eventCellView;
@synthesize eventCellIdentifier;
@synthesize eventCellRow;
@synthesize eventCellString;
@synthesize eventCellImage;

- (id)init {
    if ( self = [super init] ) {
        self.tableData = [[NSMutableArray alloc] initWithCapacity:0];
        self.dragSourceIndex = -1;
    }
    return self;
}

- (void)tableView:(NSTableView *)tableView forwardEvent:(NSInteger)event object:(id)obj {
    CocoaOutputWindowSetTag( [tableView wndTag] );
    if ( [tableView cocoaTableViewCallback] ) {
        CocoaTableViewCallbackType callback = [tableView cocoaTableViewCallback];
        (*callback)([tableView tag],event);
    } else {
        CallUserDefinedOnDialogFunction( event, [tableView tag], [tableView wndTag], obj );
    }
}

- (void)moveItemAtIndex:(NSInteger)srcIndex toIndex:(NSInteger)destIndex {
    id obj = [tableData objectAtIndex:srcIndex];
    [tableData insertObject:obj atIndex:destIndex];
    if ( srcIndex < destIndex ) {
        [tableData removeObjectAtIndex:srcIndex];
    } else {
        [tableData removeObjectAtIndex:srcIndex+1];
    }
    [table reloadData];
}

#pragma mark - tableview delegate
- (NSView *)tableView:(NSTableView *)tableView viewForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
    NSString *identifier = [tableColumn identifier];
    
    eventCellView = nil;
    eventCellString = nil;
    eventCellImage = nil;
    eventCellIdentifier = identifier;
    eventCellRow = row;
    
    [self tableView:tableView forwardEvent:tableViewView object:nil];
    
    if ( eventCellView ) {
        return eventCellView;
    }
    
    if ( eventCellString || eventCellImage  ) {
        NSTableCellView *cellView = [tableView makeViewWithIdentifier:identifier owner:self];
        if ( eventCellString ) {
            cellView.textField.stringValue = eventCellString;
        }
        if ( eventCellImage ) {
            cellView.imageView.image = eventCellImage;
        }
        return cellView;
    }

    NSTableCellView *cellView = [tableView makeViewWithIdentifier:identifier owner:self];
    
    if ( !cellView ) {
        cellView = [[NSTableCellView alloc] init];
        CGFloat rowHeight = [tableView rowHeight];
        CGFloat colWidth = [tableColumn width];
        NSTextField *textField = [[[NSTextField alloc] initWithFrame:NSMakeRect(0,0,colWidth,rowHeight)] autorelease];
        [textField setDrawsBackground:NO];
        [textField setBezeled:NO];
        [cellView addSubview:textField];
        cellView.textField = textField;
    }
    
    if ( [cellView.textField isEditable] ) {
        [cellView.textField setDelegate:self];
    }
    
    id rowObj = [tableData objectAtIndex:row];
    
    if ( [rowObj isKindOfClass:[NSAttributedString class]] ) {
        [cellView.textField setAttributedStringValue:rowObj];           // single column attributed string
        
    } else if ( [rowObj isKindOfClass:[NSString class]] ) {              // single-column plain text
        [cellView.textField setStringValue:rowObj];
        
    } else if ( [rowObj isKindOfClass:[NSImage class]] ) {               // single-column image
        if ( cellView.imageView ) {
            [cellView.imageView setImage:rowObj];
        }
        
    } else if ( [rowObj isKindOfClass:[NSDictionary class]] ) {         // multi-column or single-column
        id colObj = [rowObj objectForKey:identifier];
        
        if ( colObj ) {
            if ( [colObj isKindOfClass:[NSAttributedString class]] ) {      // attributed string
                [cellView.textField setAttributedStringValue:colObj];
                
            } else if ( [colObj isKindOfClass:[NSString class]] ) {         // plain text
                [cellView.textField setStringValue:colObj];
                
            } else if ( [colObj isKindOfClass:[NSImage class]] ) {          // image
                if ( cellView.imageView ) {
                    [cellView.imageView setImage:colObj];
                }
            } else if ( [colObj isKindOfClass:[NSDictionary class]] ) {     // dictionary
                
                NSArray *allValues = [colObj allValues];
                
                for ( id obj in allValues ) {
                    if ( [obj isKindOfClass:[NSString class]] ) {
                        [cellView.textField setStringValue:obj];
                    } else if ( [obj isKindOfClass:[NSImage class]] ) {
                        [cellView.imageView setImage:obj];
                    }
                }
                
            }
            
        } else {
            
            NSArray *allValues = [rowObj allValues];
            
            for ( id obj in allValues ) {
                if ( [obj isKindOfClass:[NSString class]] ) {
                    [cellView.textField setStringValue:obj];
                } else if ( [obj isKindOfClass:[NSImage class]] ) {
                    [cellView.imageView setImage:obj];
                }
            }
            
        }
        
    }
    
    return cellView;
}

- (NSTableRowView *)tableView:(NSTableView *)tableView rowViewForRow:(NSInteger)row {
    DialogEventSetLong(row);
    DialogEventSetView(nil);
    [self tableView:tableView forwardEvent:tableViewRowView object:nil];
    return (NSTableRowView *)DialogEventView();
}

- (void)tableView:(NSTableView *)tableView didAddRowView:(NSTableRowView *)rowView forRow:(NSInteger)row {
    [self tableView:tableView forwardEvent:tableViewDidAddRowView object:nil];
}

- (void)tableView:(NSTableView *)tableView didRemoveRowView:(NSTableRowView *)rowView forRow:(NSInteger)row {
    [self tableView:tableView forwardEvent:tableViewDidRemoveRowView object:nil];
}

- (BOOL)tableView:(NSTableView *)tableView isGroupRow:(NSInteger)row {
    DialogEventSetBool(NO);
    DialogEventSetLong(row);
    [self tableView:tableView forwardEvent:tableViewIsGroupRow object:nil];
    return DialogEventBool();
}

//- (void)tableView:(NSTableView *)tableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
//    [self tableView:tableView forwardEvent:tableViewWillDisplayCell object:nil];
//}

//- (NSCell *)tableView:(NSTableView *)tableView dataCellForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
//[self tableView:sender forwardEvent:tableViewDataCell object:nil];
//    return nil;
//}

//- (BOOL)tableView:(NSTableView *)tableView shouldShowCellExpansionForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
//[self tableView:tableView forwardEvent:tableViewShouldShowCellExpansion object:nil];
//    return NO;
//}

//- (NSString *)tableView:(NSTableView *)tableView toolTipForCell:(NSCell *)cell rect:(NSRectPointer)rect tableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row mouseLocation:(NSPoint)mouseLocation {
//[self tableView:tableView forwardEvent:tableViewToolTipForCell object:nil];
//    return nil;
//}

//- (BOOL)tableView:(NSTableView *)tableView shouldEditTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
//[self tableView:tableView forwardEvent:tableViewShouldEdit object:nil];
//    return YES;
//}

//- (CGFloat)tableView:(NSTableView *)tableView heightOfRow:(NSInteger)row {
//[self tableView:tableView forwardEvent:tableViewHeightOfRow object:nil];
//    return 0.0;
//}
//
//- (CGFloat)tableView:(NSTableView *)tableView sizeToFitWidthOfColumn:(NSInteger)column {
//[self tableView:tableView forwardEvent:tableViewSizeToFitWidthOfColumn object:nil];
//    return 0.0;
//}

- (BOOL)selectionShouldChangeInTableView:(NSTableView *)tableView {
    [self tableView:tableView forwardEvent:tableViewSelectionShouldChange object:nil];
    return YES;
}

- (BOOL)tableView:(NSTableView *)tableView shouldSelectRow:(NSInteger)row {
    [self tableView:tableView forwardEvent:tableViewShouldSelectRow object:nil];
    return YES;
}

//- (NSIndexSet *)tableView:(NSTableView *)tableView selectionIndexesForProposedSelection:(NSIndexSet *)proposedSelectionIndexes {
//[self tableView:tableView forwardEvent:tableViewSelectionIndexes object:nil];
//    return nil;
//}

- (BOOL)tableView:(NSTableView *)tableView shouldSelectTableColumn:(NSTableColumn *)tableColumn {
    [self tableView:tableView forwardEvent:tableViewShouldSelectTableColumn object:nil];
    return YES;
}

- (void)tableViewSelectionIsChanging:(NSNotification *)notification {
    [self tableView:notification.object forwardEvent:tableViewSelectionIsChanging object:nil];
}

- (void)tableViewSelectionDidChange:(NSNotification *)notification {
    //    CocoaOutputWindowSetTag( [notification.object wndTag] );
    [self tableView:notification.object forwardEvent:tableViewSelectionDidChange object:nil];
}

- (BOOL)tableView:(NSTableView *)tableView shouldTypeSelectForEvent:(NSEvent *)event withCurrentSearchString:(NSString *)searchString {
    [self tableView:tableView forwardEvent:tableViewShouldTypeSelect object:nil];
    return YES;
}

//- (NSString *)tableView:(NSTableView *)tableView typeSelectStringForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
//[self tableView:tableView forwardEvent:tableViewTypeSelectString object:nil];
//    return nil;
//}

//- (NSInteger)tableView:(NSTableView *)tableView nextTypeSelectMatchFromRow:(NSInteger)startRow toRow:(NSInteger)endRow forString:(NSString *)searchString {
//[self tableView:tableView forwardEvent:tableViewNextTypeSelectMatch object:nil];
//    return -1;
//}

- (BOOL)tableView:(NSTableView *)tableView shouldReorderColumn:(NSInteger)columnIndex toColumn:(NSInteger)newColumnIndex {
    [self tableView:tableView forwardEvent:tableViewShouldReorderColumn object:nil];
    return YES;
}

- (void)tableView:(NSTableView *)tableView didDragTableColumn:(NSTableColumn *)tableColumn {
    [self tableView:tableView forwardEvent:tableViewDidDragTableColumn object:nil];
}

- (void)tableViewColumnDidMove:(NSNotification *)notification {
    [self tableView:notification.object forwardEvent:tableViewColumnDidMove object:nil];
}

- (void)tableViewColumnDidResize:(NSNotification *)notification {
    [self tableView:notification.object forwardEvent:tableViewColumnDidResize object:nil];
}

- (void)tableView:(NSTableView *)tableView didClickTableColumn:(NSTableColumn *)tableColumn {
    [self tableView:tableView forwardEvent:tableViewDidClickTableColumn object:nil];
}

- (void)tableView:(NSTableView *)tableView mouseDownInHeaderOfTableColumn:(NSTableColumn *)tableColumn {
    [self tableView:tableView forwardEvent:tableViewMouseDownInHeader object:nil];
}

//- (BOOL)tableView:(NSTableView *)tableView shouldTrackCell:(NSCell *)cell forTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
//[self tableView:tableView forwardEvent:tableViewShouldTrackCell object:nil];
//    return YES;
//}

//- (NSArray<NSTableViewRowAction *> *)tableView:(NSTableView *)tableView rowActionsForRow:(NSInteger)row edge:(NSTableRowActionEdge)edge {
//[self tableView:tableView forwardEvent:tableViewRowActions object:nil];
//    return nil;
//}

- (void)doubleAction:(id)sender {
    [self tableView:sender forwardEvent:tableViewDoubleAction object:nil];
}

- (void)singleAction:(id)sender {
    [self tableView:sender forwardEvent:1 object:nil];// 1 = _btnClick
}

- (void)controlTextDidEndEditing:(NSNotification *)notification {
    NSTextField *textField = [notification object];
    NSInteger row = [table rowForView:textField];
    NSInteger col = [table columnForView:textField];
    NSTableColumn *column = [[table tableColumns] objectAtIndex:col];
    NSString *identifier = [column identifier];
    NSString *string = [textField stringValue];
    
    id rowObj = [tableData objectAtIndex:row];
    if ( [rowObj isKindOfClass:[NSAttributedString class]] ) {  // single column attributed string
        if ( [string length] == 0 ) string = @ " ";
        [rowObj replaceCharactersInRange:NSMakeRange(0,[rowObj length]) withString:string];
    } else if ( [rowObj isKindOfClass:[NSString class]] ) {     // single column plain text
        [tableData replaceObjectAtIndex:row withObject:string];
        
        // dictionary
    } else if ( [rowObj isKindOfClass:[NSMutableDictionary class]] ) { // single or multi column
        id colObj = [rowObj objectForKey:identifier];
        if ( colObj ) {
            if ( [colObj isKindOfClass:[NSAttributedString class]] ) {  // attributed string
                if ( [string length] == 0 ) string = @ " ";
                [colObj replaceCharactersInRange:NSMakeRange(0,[colObj length]) withString:string];
            } else if ( [colObj isKindOfClass:[NSString class]] ) {     // plain text
                [rowObj setObject:string forKey:identifier];
            }
        }
    } else {
        // not a valid row obj type (probably due to CFDictionaryRef instead of CFMutableDictionaryRef)
        [table reloadData];
    }
}

#pragma mark - tableview data source
- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView {
    eventNumberOfRows = -1;
    [self tableView:aTableView forwardEvent:tableViewNumberOfRows object:nil];
    if ( eventNumberOfRows != -1 ) return eventNumberOfRows;
    return [tableData count];
}

//- (id<NSPasteboardWriting>)tableView:(NSTableView *)tableView pasteboardWriterForRow:(NSInteger)row {
//    return nil;
//}

- (NSDragOperation)tableView:(NSTableView *)tableView validateDrop:(id<NSDraggingInfo>)info proposedRow:(NSInteger)row proposedDropOperation:(NSTableViewDropOperation)dropOperation {
    DialogEventSetLong(NSDragOperationLink);
    [self tableView:tableView forwardEvent:tableViewValidateDrop object:info];
    return DialogEventLong();
}

- (BOOL)tableView:(NSTableView *)tableView acceptDrop:(id<NSDraggingInfo>)info row:(NSInteger)row dropOperation:(NSTableViewDropOperation)dropOperation {
    DialogEventSetBool(YES);
    DialogEventSetLong(row);
    if ( dragSourceIndex == -1 ) {  // Finder drop
        
        NSPasteboard *pb = info.draggingPasteboard;
        NSDictionary *options = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] forKey:NSPasteboardURLReadingFileURLsOnlyKey];
        NSArray *urls = [pb readObjectsForClasses:[NSArray arrayWithObject:[NSURL class]] options:options];

        DialogEventSetNSArray(urls);
        DialogEventSetNSURL([urls objectAtIndex:0]);
        
        [self tableView:tableView forwardEvent:tableViewAcceptDrop object:info];
        
    } else {                        // item move
        
        //[self tableView:tableView forwardEvent:tableViewAcceptDrop object:info];
        
        [self moveItemAtIndex:dragSourceIndex toIndex:row];

    }
    return YES;//DialogEventBool();
}

- (BOOL)tableView:(NSTableView *)tableView writeRowsWithIndexes:(NSIndexSet *)rowIndexes toPasteboard:(NSPasteboard *)pboard {
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:rowIndexes];
    [pboard declareTypes:[NSArray arrayWithObject:TABLEVIEW_REORDER_PASTEBOARD_TYPE] owner:self];
    [pboard setData:data forType:TABLEVIEW_REORDER_PASTEBOARD_TYPE];
    dragSourceIndex = [rowIndexes firstIndex];
    return YES;
}

//- (void)tableView:(NSTableView *)tableView draggingSession:(NSDraggingSession *)session willBeginAtPoint:(NSPoint)screenPoint forRowIndexes:(NSIndexSet *)rowIndexes {
//    
//}
//
//- (void)tableView:(NSTableView *)tableView updateDraggingItemsForDrag:(id<NSDraggingInfo>)draggingInfo {
//    
//}

- (void)tableView:(NSTableView *)tableView draggingSession:(NSDraggingSession *)session endedAtPoint:(NSPoint)screenPoint operation:(NSDragOperation)operation {
    DialogEventSetNSPoint( screenPoint );
    
    [self tableView:tableView forwardEvent:tableViewDraggingSessionEnded object:session];

    dragSourceIndex = -1;
}

- (void)tableView:(NSTableView *)tableView sortDescriptorsDidChange:(NSArray *)oldDescriptors {
    [tableData sortUsingDescriptors:[tableView sortDescriptors]];
    [table reloadData];
}
@end


@implementation NSTableView (Additions)
@dynamic callbackValue;

- (CocoaTableViewCallbackType)cocoaTableViewCallback {
    return [objc_getAssociatedObject(self, @selector(callbackValue)) pointerValue];
}

- (void)setCocoaTableViewCallback:(CocoaTableViewCallbackType)callback {
    objc_setAssociatedObject(self, @selector(callbackValue), [NSValue valueWithPointer:callback], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

// dialog events
void TableViewEventSetNumberOfRows( NSInteger tag, NSInteger value )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        CocoaTableViewDelegate *delegate = (CocoaTableViewDelegate *)[tableView delegate];
        [delegate setEventNumberOfRows:value];
    }
}

void TableViewEventSetCellView( NSInteger tag, ViewRef vw )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        CocoaTableViewDelegate *delegate = (CocoaTableViewDelegate *)[tableView delegate];
        [delegate setEventCellView:vw];
    }
}

CFStringRef TableViewEventCellIdentifier( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        CocoaTableViewDelegate *delegate = (CocoaTableViewDelegate *)[tableView delegate];
        return (CFStringRef)[delegate eventCellIdentifier];
    }
    return nil;
}

NSInteger TableViewEventCellRow( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        CocoaTableViewDelegate *delegate = (CocoaTableViewDelegate *)[tableView delegate];
        return [delegate eventCellRow];
    }
    return -1;
}

void TableViewEventSetCellString( NSInteger tag, CFStringRef string )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        CocoaTableViewDelegate *delegate = (CocoaTableViewDelegate *)[tableView delegate];
        [delegate setEventCellString:[(NSString *)string retain]];
    }
}

void TableViewEventSetCellImage( NSInteger tag, ImageRef image )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        CocoaTableViewDelegate *delegate = (CocoaTableViewDelegate *)[tableView delegate];
        [delegate setEventCellImage:[image retain]];
    }
}


NSTableView *TableViewWithTag( NSInteger tag )
{ return (NSTableView *)CocoaValidateViewKindOfClass( tag, [NSTableView class] ); }

BOOL TableViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSTableView class] ); }

// Data source
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean TableViewUsesStaticContents( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView usesStaticContents];
    return NO;
}

void TableViewSetUsesStaticContents( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setUsesStaticContents:flag];
}
#endif// 101000

void TableViewSetData( NSInteger tag, CFMutableArrayRef array )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        if ( [tableView isKindOfClass:[NSOutlineView class]] ) {
            CocoaOutlineViewDelegate *delegate = (CocoaOutlineViewDelegate *)[tableView delegate];
            NSArray *oldArray = [delegate tableData];
            if ( oldArray ) [oldArray release];
            [delegate setTableData:[(NSMutableArray *)array retain]];
            [tableView reloadData];
        } else {
            CocoaTableViewDelegate *delegate = (CocoaTableViewDelegate *)[tableView delegate];
            NSArray *oldArray = [delegate tableData];
            if ( oldArray ) [oldArray release];
            [delegate setTableData:[(NSMutableArray *)array retain]];// retain added 20171211 bw
            [tableView reloadData];
        }
    }
}

CFMutableArrayRef TableViewData( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        if ( [tableView isKindOfClass:[NSOutlineView class]] ) {
            CocoaOutlineViewDelegate *delegate = (CocoaOutlineViewDelegate *)[tableView delegate];
            return (CFMutableArrayRef)delegate.tableData;
        } else {
            CocoaTableViewDelegate *delegate = (CocoaTableViewDelegate *)[tableView delegate];
            return (CFMutableArrayRef)delegate.tableData;
        }
    }
    return nil;
}

void TableViewReloadData( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView reloadData];
}

void TableViewReloadDataForRowsColumns( NSInteger tag, IndexSetRef rows, IndexSetRef columns )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView reloadDataForRowIndexes:rows columnIndexes:columns];
}

// Create views
TableCellViewRef TableViewMakeViewWithIdentifier( NSInteger tag, CFStringRef identifier )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        CocoaTableViewDelegate *delegate = (CocoaTableViewDelegate *)[tableView delegate];
        return [tableView makeViewWithIdentifier:(NSString *)identifier owner:delegate];
    }
    return nil;
}

// Update
void TableViewBeginUpdates( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView beginUpdates];
}

void TableViewEndUpdates( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView endUpdates];
}

void TableViewMoveRow( NSInteger tag, NSInteger atIndex, NSInteger toIndex )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView moveRowAtIndex:atIndex toIndex:toIndex];
}

// Target-action
NSInteger TableViewClickedColumn( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView clickedColumn];
    return 0;
}

NSInteger TableViewClickedRow( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView clickedRow];
    return 0;
}

// Behavior
Boolean TableViewAllowsColumnReordering( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView allowsColumnReordering];
    return NO;
}

void TableViewSetAllowsColumnReordering( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setAllowsColumnReordering:flag];
}

Boolean TableViewAllowsColumnResizing( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView allowsColumnResizing];
    return NO;
}

void TableViewSetAllowsColumnResizing( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setAllowsColumnResizing:flag];
}

Boolean TableViewAllowsMultipleSelection( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView allowsMultipleSelection];
    return NO;
}

void TableViewSetAllowsMultipleSelection( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setAllowsMultipleSelection:flag];
}

Boolean TableViewAllowsEmptySelection( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView allowsEmptySelection];
    return NO;
}

void TableViewSetAllowsEmptySelection( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setAllowsEmptySelection:flag];
}

Boolean TableViewAllowsColumnSelection( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView allowsColumnSelection];
    return NO;
}

void TableViewSetAllowsColumnSelection( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setAllowsColumnSelection:flag];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean TableViewUsesAutomaticRowHeights( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView usesAutomaticRowHeights];
    return NO;
}

void TableViewSetUsesAutomaticRowHeights( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setUsesAutomaticRowHeights:flag];
}
#endif// 101300

// Display attributes
CGSize TableViewIntercellSpacing( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return NSSizeToCGSize([tableView intercellSpacing]);
    return CGSizeZero;
}

void TableViewSetIntercellSpacing( NSInteger tag, CGSize spacing )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setIntercellSpacing:NSSizeFromCGSize(spacing)];
}

CGFloat TableViewRowHeight( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView rowHeight];
    return 0.0;
}

void TableViewSetRowHeight( NSInteger tag, CGFloat height )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setRowHeight:height];
}

ColorRef TableViewBackgroundColor( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView backgroundColor];
    return nil;
}

void TableViewSetBackgroundColor( NSInteger tag, ColorRef col )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setBackgroundColor:col];
}

Boolean TableViewUsesAlternatingRowBackgroundColors( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView usesAlternatingRowBackgroundColors];
    return NO;
}

void TableViewSetUsesAlternatingRowBackgroundColors( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setUsesAlternatingRowBackgroundColors:flag];
}

NSTableViewSelectionHighlightStyle TableViewSelectionHighlightStyle( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView selectionHighlightStyle];
    return 0;
}

void TableViewSetSelectionHighlightStyle( NSInteger tag, NSTableViewSelectionHighlightStyle style )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setSelectionHighlightStyle:style];
}

ColorRef TableViewGridColor( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView gridColor];
    return nil;
}

void TableViewSetGridColor( NSInteger tag, ColorRef col )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setGridColor:col];
}

NSTableViewGridLineStyle TableViewGridStyleMask( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView gridStyleMask];
    return 0;
}

void TableViewSetGridStyleMask( NSInteger tag, NSTableViewGridLineStyle style )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setGridStyleMask:style];
}

ImageRef TableViewIndicatorImageInTableColumn( NSInteger tag, TableColumnRef columnRef )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView indicatorImageInTableColumn:columnRef];
    return nil;
}

void TableViewSetIndicatorImageInTableColumn( NSInteger tag, ImageRef image, TableColumnRef columnRef )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setIndicatorImage:image inTableColumn:columnRef];
}

// Row size styles
NSTableViewRowSizeStyle TableViewEffectiveRowSizeStyle( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView effectiveRowSizeStyle];
    return 0;
}

NSTableViewRowSizeStyle TableViewRowSizeStyle( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView rowSizeStyle];
    return 0;
}

void TableViewSetRowSizeStyle( NSInteger tag, NSTableViewRowSizeStyle style )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setRowSizeStyle:style];
}

// Column management
void TableViewAddTableColumn( NSInteger tag, TableColumnRef columnRef )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView addTableColumn:columnRef];
}

void TableViewRemoveTableColumn( NSInteger tag, TableColumnRef columnRef )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView removeTableColumn:columnRef];
}

void TableViewMoveColumn( NSInteger tag, NSInteger atIndex, NSInteger toIndex )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView moveColumn:atIndex toColumn:toIndex];
}

CFArrayRef TableViewTableColumns( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return (CFArrayRef)[tableView tableColumns];
    return nil;
}

NSInteger TableViewColumnWithIdentifier( NSInteger tag, CFStringRef identifier )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView columnWithIdentifier:(NSString *)identifier];
    return -1;
}

TableColumnRef TableViewTableColumnWithIdentifier( NSInteger tag, CFStringRef identifier )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView tableColumnWithIdentifier:(NSString *)identifier];
    return nil;
}

// Selecting columns and rows
void TableViewSelectColumns( NSInteger tag, IndexSetRef columnIndexes, Boolean extendSelection )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView selectColumnIndexes:columnIndexes byExtendingSelection:extendSelection];
}

NSInteger TableViewSelectedColumn( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView selectedColumn];
    return -1;
}

IndexSetRef TableViewSelectedColumns( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView selectedColumnIndexes];
    return nil;
}

void TableViewDeselectColumn( NSInteger tag, NSInteger columnIndex )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView deselectColumn:columnIndex];
}

NSInteger TableViewNumberOfSelectedColumns( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView numberOfSelectedColumns];
    return 0;
}

Boolean TableViewIsColumnSelected( NSInteger tag, NSInteger columnIndex )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView isColumnSelected:columnIndex];
    return NO;
}

void TableViewSelectRows( NSInteger tag, IndexSetRef rowIndexes, Boolean extendSelection )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView selectRowIndexes:rowIndexes byExtendingSelection:extendSelection];
}

void TableViewSelectRow( NSInteger tag, NSInteger row )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        [tableView selectRowIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];
    }
}

NSInteger TableViewSelectedRow( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView selectedRow];
    return -1;
}


CFArrayRef TableViewSelectedRowIndexes( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        NSIndexSet *set = [tableView selectedRowIndexes];
        if ( set ) {
            NSMutableArray *array = [NSMutableArray arrayWithCapacity:0];
            [set enumerateIndexesUsingBlock:^(NSUInteger index, BOOL *stop) {
                [array addObject:[NSNumber numberWithInteger:index]];
            }];
            return (CFArrayRef)array;
        }
    }
    return nil;
}

void TableViewDeselectRow( NSInteger tag, NSInteger row )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView deselectRow:row];
}

NSInteger TableViewNumberOfSelectedRows( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView numberOfSelectedRows];
    return 0;
}

Boolean TableViewIsRowSelected( NSInteger tag, NSInteger row )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView isRowSelected:row];
    return false;
}

void TableViewSelectAll( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView selectAll:nil];
}

void TableViewDeselectAll( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView deselectAll:nil];
}

// Type select
Boolean TableViewAllowsTypeSelect( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView allowsTypeSelect];
    return NO;
}

void TableViewSetAllowsTypeSelect( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setAllowsTypeSelect:flag];
}

// Dimensions
NSInteger TableViewNumberOfColumns( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView numberOfColumns];
    return 0;
}

// Floating rows
Boolean TableViewFloatsGroupRows( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView floatsGroupRows];
    return NO;
}

void TableViewSetFloatsGroupRows( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setFloatsGroupRows:flag];
}

// Editing cells
void TableViewEditColumnRow( NSInteger tag, NSInteger columnIndex, NSInteger row, Boolean selectFlag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView editColumn:columnIndex row:row withEvent:nil select:selectFlag];
}

// Auxiliary views
TableHeaderViewRef TableViewHeaderView( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView headerView];
    return nil;
}

void TableViewSetHeaderView( NSInteger tag, TableHeaderViewRef headerView )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setHeaderView:headerView];
}

ViewRef TableViewCornerView( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView cornerView];
    return nil;
}

void TableViewSetCornerView( NSInteger tag, ViewRef cornerView )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setCornerView:cornerView];
}

// Layout support
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
NSUserInterfaceLayoutDirection TableViewUserInterfaceLayoutDirection( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView userInterfaceLayoutDirection];
    return 0;
}

void TableViewSetUserInterfaceLayoutDirection( NSInteger tag, NSUserInterfaceLayoutDirection direction )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setUserInterfaceLayoutDirection:direction];
}
#endif// 101200

CGRect TableViewRectOfColumn( NSInteger tag, NSInteger col )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return NSRectToCGRect([tableView rectOfColumn:col]);
    return CGRectZero;
}

CGRect TableViewRectOfRow( NSInteger tag, NSInteger row )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return NSRectToCGRect([tableView rectOfRow:row]);
    return CGRectZero;
}

CFRange TableViewRowsInRect( NSInteger tag, CGRect r )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return NSRangeToCFRange([tableView rowsInRect:NSRectFromCGRect(r)]);
    return CFRangeMake(NSNotFound,0);
}

IndexSetRef TableViewColumnIndexesInRect( NSInteger tag, CGRect r )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView columnIndexesInRect:NSRectFromCGRect(r)];
    return nil;
}

NSInteger TableViewColumnAtPoint( NSInteger tag, CGPoint pt )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView columnAtPoint:NSPointFromCGPoint(pt)];
    return -1;
}

NSInteger TableViewRowAtPoint( NSInteger tag, CGPoint pt )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView rowAtPoint:NSPointFromCGPoint(pt)];
    return -1;
}

CGRect TableViewFrameOfCell( NSInteger tag, NSInteger col, NSInteger row )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return NSRectToCGRect([tableView frameOfCellAtColumn:col row:row]);
    return CGRectZero;
}

NSTableViewColumnAutoresizingStyle TableViewColumnAutoresizingStyle( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView columnAutoresizingStyle];
    return 0;
}

void TableViewSetColumnAutoresizingStyle( NSInteger tag, NSTableViewColumnAutoresizingStyle style )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setColumnAutoresizingStyle:style];
}

void TableViewSizeLastColumnToFit( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView sizeLastColumnToFit];
}

void TableViewTile( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView tile];
}
void TableViewSizeToFit( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView sizeToFit];
}
void TableViewNoteHeightOfRowsWithIndexesChanged( NSInteger tag, IndexSetRef indexSet )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView noteHeightOfRowsWithIndexesChanged:indexSet];
}

// Scrolling
void TableViewScrollRowToVisible( NSInteger tag, NSInteger row )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView scrollRowToVisible:row];
}

void TableViewScrollColumnToVisible( NSInteger tag, NSInteger col )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView scrollColumnToVisible:col];
}

// Column state persistence
Boolean TableViewAutosaveColumns( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView autosaveTableColumns];
    return NO;
}

void TableViewSetAutosaveColumns( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setAutosaveTableColumns:flag];
}

CFStringRef TableViewAutosaveName( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return (CFStringRef)[tableView autosaveName];
    return nil;
}

void TableViewSetAutosaveName( NSInteger tag, CFStringRef name )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setAutosaveName:(NSString *)name];
}

// Highlightable column headers
TableColumnRef TableViewHighlightedTableColumn( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView highlightedTableColumn];
    return nil;
}

void TableViewSetHighlightedTableColumn( NSInteger tag, TableColumnRef columnRef )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setHighlightedTableColumn:columnRef];
}

// Dragging
Boolean TableViewCanDragRows( NSInteger tag, IndexSetRef rowIndexes, CGPoint atPoint )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView canDragRowsWithIndexes:rowIndexes atPoint:NSPointFromCGPoint(atPoint)];
    return NO;
}

void TableViewSetDraggingSourceOperationMask( NSInteger tag, NSDragOperation operation, Boolean forLocal )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setDraggingSourceOperationMask:operation forLocal:forLocal];
}

Boolean TableViewVerticalMotionCanBeginDrag( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView verticalMotionCanBeginDrag];
    return NO;
}

void TableViewSetVerticalMotionCanBeginDrag( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setVerticalMotionCanBeginDrag:flag];
}

NSTableViewDraggingDestinationFeedbackStyle TableViewDraggingDestinationFeedbackStyle( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView draggingDestinationFeedbackStyle];
    return 0;
}

void TableViewSetDraggingDestinationFeedbackStyle( NSInteger tag, NSTableViewDraggingDestinationFeedbackStyle style )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setDraggingDestinationFeedbackStyle:style];
}

void TableViewSetDropRow( NSInteger tag, NSInteger row, NSTableViewDropOperation operation )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setDropRow:row dropOperation:operation];
}

// Sorting
CFArrayRef TableViewSortDescriptors( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return (CFArrayRef)[tableView sortDescriptors];
    return nil;
}

void TableViewSetSortDescriptors( NSInteger tag, CFArrayRef descriptors )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setSortDescriptors:(NSArray *)descriptors];
}

// Row actions
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean TableViewRowActionsVisible( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView rowActionsVisible];
    return NO;
}

void TableViewSetRowActionsVisible( NSInteger tag, Boolean flag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setRowActionsVisible:flag];
}
#endif// 101100

// Hiding and showing rows
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void TableViewHideRow( NSInteger tag, NSInteger rowIndex, NSTableViewAnimationOptions animation )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView hideRowsAtIndexes:[NSIndexSet indexSetWithIndex:rowIndex] withAnimation:animation];
}

void TableViewHideRows( NSInteger tag, IndexSetRef indexes, NSTableViewAnimationOptions animation )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView hideRowsAtIndexes:indexes withAnimation:animation];
}

void TableViewUnhideRow( NSInteger tag, NSInteger rowIndex, NSTableViewAnimationOptions animation )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView unhideRowsAtIndexes:[NSIndexSet indexSetWithIndex:rowIndex] withAnimation:animation];
}

void TableViewUnhideRows( NSInteger tag, IndexSetRef indexes, NSTableViewAnimationOptions animation )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView unhideRowsAtIndexes:indexes withAnimation:animation];
}

IndexSetRef TableViewHiddenRowIndexes( NSInteger tag )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView hiddenRowIndexes];
    return [NSIndexSet indexSet];
}
#endif



// custom
void *TableViewView( NSInteger tag, NSInteger row, NSInteger col )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) return [tableView viewAtColumn:col row:row makeIfNecessary:YES];
    return nil;
}

void TableViewSetDelegateCallback( NSInteger tag, void *callback )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) [tableView setCocoaTableViewCallback:callback];
}

void TableViewCellMakeFirstResponder( NSInteger tag, NSInteger row, NSInteger col )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        NSTableCellView *view = [tableView viewAtColumn:col row:row makeIfNecessary:YES];
        [[view window] makeFirstResponder:[view textField]];
    }
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void TableViewSetHeaderFont( NSInteger tag, FontRef font )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        NSArray *columns = [tableView tableColumns];
        for ( NSTableColumn *column in columns ) {
            TableHeaderCellSC *cell = [column headerCell];
            if ( ![cell isKindOfClass:[TableHeaderCellSC class]] ) {
                NSString *title = [column title];
                cell = [[[TableHeaderCellSC alloc] init] autorelease];
                [column setHeaderCell:cell];
                [column setTitle:title];
            }
            [cell setFontProp:font];
        }
    }
}

void TableViewSetHeaderTextColor( NSInteger tag, ColorRef textColor )
{
    NSTableView *tableView = TableViewWithTag( tag );
    if ( tableView ) {
        NSArray *columns = [tableView tableColumns];
        for ( NSTableColumn *column in columns ) {
            TableHeaderCellSC *cell = [column headerCell];
            if ( ![cell isKindOfClass:[TableHeaderCellSC class]] ) {
                NSString *title = [column title];
                cell = [[[TableHeaderCellSC alloc] init] autorelease];
                [column setHeaderCell:cell];
                [column setTitle:title];
            }
            [cell setTextColorProp:textColor];
        }
    }
}
#endif


// Init
TableViewRef TableViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 240, 135 );

    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    NSTableView *view;
    if ( subclass ) {
        view = [[[TableViewSC alloc] initWithFrame:rect] autorelease];
    } else {
        view = [[[NSTableView alloc] initWithFrame:rect] autorelease];
    }

    // tag
    [view setTag:inTag];
    
    return view;
}


// Statement
void CocoaTableView( NSInteger inTag, NSInteger inEnabled, CGRect inRect, CFTypeRef inColumnIdentifiers, NSInteger inScrollTag, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSTableView *tableView = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !tableView ) {
            tableView = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( tableView ) {    // already exists
            
            // make sure it's same view class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [tableView class], [NSTableView class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, tableView, [NSTableView class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [tableView setEnabled:inEnabled];

            // rect
            if ( !NSIsEmptyRect( rect ) ) [tableView setFrame:rect];
            
        } else {            // create
            
            NSScrollView *scrollView = nil;
            if ( inScrollTag ) {
                scrollView = CocoaViewSubviewWithTag( contentView, inScrollTag );
                if ( !scrollView ) {
                    scrollView = TitlebarAccessoryViewWithTag( inTag );
                }
            }
            
            // rect
            if ( NSIsEmptyRect( rect ) ) {
                if ( scrollView ) {
                    NSSize size = [scrollView contentSize];
                    rect = NSMakeRect( 0, 0, size.width, size.height );
                } else {
                    rect = NSMakeRect( 20, 20, 240, 135 );
                }
            }
            
            // init
//            if ( inSubclass ) {
//                tableView = [[TableViewSC alloc] initWithFrame:rect];
//            } else {
//                tableView = [[NSTableView alloc] initWithFrame:rect];
//            }
            tableView = TableViewInit( inTag, NSRectToCGRect(rect) );
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [tableView setEnabled:inEnabled];

            // autoresizing
            [tableView setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
            
            // tag
//            [tableView setTag:inTag];
            
            // columnIdentifiers
            if ( inColumnIdentifiers ) {
                NSArray *identifiers = nil;
                if ( CFGetTypeID(inColumnIdentifiers) == CFStringGetTypeID() ) {
                    identifiers = [(NSString *)inColumnIdentifiers componentsSeparatedByString:@";"];
                } else if ( CFGetTypeID(inColumnIdentifiers) == CFArrayGetTypeID() ) {
                    identifiers = (NSArray *)inColumnIdentifiers;
                }
                if ( identifiers ) {
                    for ( NSString *identifier in identifiers ) {
                        NSTableColumn *column = [[[NSTableColumn alloc] initWithIdentifier:identifier] autorelease];
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
                        [column setTitle:identifier];
#else
                        [[column headerCell] setStringValue:identifier];
#endif
                        [tableView addTableColumn:column];
                        [column sizeToFit];
                    }
                }
            }

            // delegate & data source
            CocoaTableViewDelegate *delegate = [[CocoaTableViewDelegate alloc] init];
            [tableView setDelegate:delegate];
            [tableView setDataSource:delegate];
            [tableView setTarget:delegate];
            [tableView setAction:@selector(singleAction:)];
            [tableView setDoubleAction:@selector(doubleAction:)];

            // add to superview
            if ( scrollView ) {
                [scrollView setDocumentView:tableView];
            } else {
                [contentView addSubview:tableView];
            }
            
            // wndTag
            [tableView setWndTag:inWndTag];
            
//            [tableView release];
        }
        [tableView setHidden:!show];
    } else {
        CocoaShowAlert( NSWarningAlertStyle, @"Window error.", [NSString stringWithFormat:@"Cocoa window %ld does not exist.",(long)inWndTag] );
    }
}

