/*
 Task.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef void (*CocoaTaskTerminationHandlerType)(NSTask *);

// - init -
TaskRef TaskInit( void );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
TaskRef TaskLaunchedTaskWithExecutableURL( CFURLRef url, CFArrayRef arguments, ErrorRef *err, CocoaTaskTerminationHandlerType handlerFn );
#endif

// - info + configure-
CFArrayRef TaskArguments( TaskRef tsk );
void TaskSetArguments( TaskRef tsk, CFArrayRef arguments );

CFURLRef TaskCurrentDirectoryURL( TaskRef tsk );
void TaskSetCurrentDirectoryURL( TaskRef tsk, CFURLRef url );
CFURLRef TaskExecutableURL( TaskRef tsk );
void TaskSetExecutableURL( TaskRef tsk, CFURLRef url );

CFDictionaryRef TaskEnvironment( TaskRef tsk );
void TaskSetEnvironment( TaskRef tsk, CFDictionaryRef environment );
int TaskProcessIdentifier( TaskRef tsk );
void *TaskStandardError( TaskRef tsk );
void TaskSetStandardError( TaskRef tsk, void *obj );
void *TaskStandardInput( TaskRef tsk );
void TaskSetStandardInput( TaskRef tsk, void *obj );
void *TaskStandardOutput( TaskRef tsk );
void TaskSetStandardOutput( TaskRef tsk, void *obj );

// - running and stopping -
void TaskInterrupt( TaskRef tsk );
Boolean TaskLaunch( TaskRef tsk, ErrorRef *err );
Boolean TaskResume( TaskRef tsk );
Boolean TaskSuspend( TaskRef tsk );
void TaskTerminate( TaskRef tsk );
void TaskWaitUntilExit( TaskRef tsk );

// - query state -
Boolean TaskIsRunning( TaskRef tsk );
int TaskTerminationStatus( TaskRef tsk );
NSTaskTerminationReason TaskTerminationReason( TaskRef tsk );

// - termination handler -
void TaskSetTerminationHandler( TaskRef tsk, CocoaTaskTerminationHandlerType handlerFn );

