/*
 Task.m

 Bernie Wylde
 */

#import "Task.h"

// - init -
TaskRef TaskInit( void )
{ return [[[NSTask alloc] init] autorelease]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
TaskRef TaskLaunchedTaskWithExecutableURL( CFURLRef url, CFArrayRef arguments, ErrorRef *err, CocoaTaskTerminationHandlerType handlerFn )
{
    return [NSTask launchedTaskWithExecutableURL:(NSURL *)url arguments:(NSArray *)arguments error:err terminationHandler:^(NSTask *task) {
        if ( handlerFn ) (*handlerFn)(task);
    }];
}
#endif

// - info + configure-
CFArrayRef TaskArguments( TaskRef tsk )
{ return (CFArrayRef)[tsk arguments]; }

void TaskSetArguments( TaskRef tsk, CFArrayRef arguments )
{ [tsk setArguments:(NSArray *)arguments]; }

CFURLRef TaskCurrentDirectoryURL( TaskRef tsk )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
    return (CFURLRef)[tsk currentDirectoryURL];
#else
    return (CFURLRef)[NSURL fileURLWithPath:[tsk currentDirectoryPath]];
#endif
}

void TaskSetCurrentDirectoryURL( TaskRef tsk, CFURLRef url )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
    [tsk setCurrentDirectoryURL:(NSURL *)url];
#else
    [tsk setCurrentDirectoryPath:[(NSURL *)url path]];
#endif
}

CFURLRef TaskExecutableURL( TaskRef tsk )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
    return (CFURLRef)[tsk executableURL];
#else
    return (CFURLRef)[NSURL fileURLWithPath:[tsk launchPath]];
#endif
}

void TaskSetExecutableURL( TaskRef tsk, CFURLRef url )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
    [tsk setExecutableURL:(NSURL *)url];
#else
    [tsk setLaunchPath:[(NSURL *)url path]];
#endif
}

CFDictionaryRef TaskEnvironment( TaskRef tsk )
{ return (CFDictionaryRef)[tsk environment]; }

void TaskSetEnvironment( TaskRef tsk, CFDictionaryRef environment )
{ [tsk setEnvironment:(NSDictionary *)environment]; }

int TaskProcessIdentifier( TaskRef tsk )
{ return [tsk processIdentifier]; }

void *TaskStandardError( TaskRef tsk )
{ return (void *)[tsk standardError]; }

void TaskSetStandardError( TaskRef tsk, void *obj )
{ [tsk setStandardError:(id)obj]; }

void *TaskStandardInput( TaskRef tsk )
{ return (void *)[tsk standardInput]; }

void TaskSetStandardInput( TaskRef tsk, void *obj )
{ [tsk setStandardInput:(id)obj]; }

void *TaskStandardOutput( TaskRef tsk )
{ return (void *)[tsk standardOutput]; }

void TaskSetStandardOutput( TaskRef tsk, void *obj )
{ [tsk setStandardOutput:(id)obj]; }

// - running and stopping -
void TaskInterrupt( TaskRef tsk )
{ [tsk interrupt]; }

Boolean TaskLaunch( TaskRef tsk, ErrorRef *err )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
    return [tsk launchAndReturnError:err];
#else
    [tsk launch];
    return YES;
#endif
}

Boolean TaskResume( TaskRef tsk )
{ return [tsk resume]; }

Boolean TaskSuspend( TaskRef tsk )
{ return [tsk suspend]; }

void TaskTerminate( TaskRef tsk )
{ [tsk terminate]; }

void TaskWaitUntilExit( TaskRef tsk )
{ [tsk waitUntilExit]; }

// - query state -
Boolean TaskIsRunning( TaskRef tsk )
{ return [tsk isRunning]; }

int TaskTerminationStatus( TaskRef tsk )
{ return [tsk terminationStatus]; }

NSTaskTerminationReason TaskTerminationReason( TaskRef tsk )
{ return [tsk terminationReason]; }

// - termination handler -
void TaskSetTerminationHandler( TaskRef tsk, CocoaTaskTerminationHandlerType handlerFn )
{
    [tsk setTerminationHandler:^(NSTask *task) {
        if ( handlerFn ) (*handlerFn)(task);
    }];
}

