/*
 TextAttachment.m

 Bernie Wylde
 */

#import "TextAttachment.h"

// Init
TextAttachmentRef TextAttachmentInit( void )
{ return [[[NSTextAttachment alloc] init] autorelease]; }

// Contents
#if __LP64__
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGRect TextAttachmentBounds( TextAttachmentRef ref )
{ return NSRectToCGRect([ref bounds]); }

void TextAttachmentSetBounds( TextAttachmentRef ref, CGRect r )
{ [ref setBounds:NSRectFromCGRect(r)]; }

CFDataRef TextAttachmentContents( TextAttachmentRef ref )
{ return (CFDataRef)[ref contents]; }

void TextAttachmentSetContents( TextAttachmentRef ref, CFDataRef dta )
{ [ref setContents:(NSData *)dta]; }

CFStringRef TextAttachmentFileType( TextAttachmentRef ref )
{ return (CFStringRef)[ref fileType]; }

void TextAttachmentSetFileType( TextAttachmentRef ref, CFStringRef type )
{ [ref setFileType:(NSString *)type]; }

ImageRef TextAttachmentImage( TextAttachmentRef ref )
{ return [ref image]; }

void TextAttachmentSetImage( TextAttachmentRef ref, ImageRef image )
{ [ref setImage:image]; }
#endif
#endif

FileWrapperRef TextAttachmentFileWrapper( TextAttachmentRef ref )
{ return [ref fileWrapper]; }

void TextAttachmentSetFileWrapper( TextAttachmentRef ref, FileWrapperRef wrapper )
{ [ref setFileWrapper:wrapper]; }

// Cell
TextAttachmentCellRef TextAttachmentCell( TextAttachmentRef ref )
{ return [ref attachmentCell]; }

void TextAttachmentSetCell( TextAttachmentRef ref, TextAttachmentCellRef cell )
{ [ref setAttachmentCell:cell]; }




/*
 TextAttachmentCell
 */
// Init
TextAttachmentCellRef TextAttachmentCellWithImage( ImageRef image )
{ return [[[NSTextAttachmentCell alloc] initImageCell:image] autorelease]; }

// Draw
void TextAttachmentCellDraw( TextAttachmentCellRef ref, CGRect frame, NSInteger controlViewTag )
{
    NSView *view = ViewWithTag( controlViewTag );
    if ( view ) {
        [ref drawWithFrame:NSRectFromCGRect(frame) inView:view];
    }
}

void TextAttachmentCellDrawCharacterIndex( TextAttachmentCellRef ref, CGRect frame, NSInteger controlViewTag, NSUInteger charIndex )
{
    NSView *view = ViewWithTag( controlViewTag );
    if ( view ) {
        [ref drawWithFrame:NSRectFromCGRect(frame) inView:view characterIndex:charIndex];
    }
}

void TextAttachmentCellDrawCharacterIndexLayoutManager( TextAttachmentCellRef ref, CGRect frame, NSInteger controlViewTag, NSUInteger charIndex, LayoutManagerRef layoutManager )
{
    NSView *view = ViewWithTag( controlViewTag );
    if ( view ) {
        [ref drawWithFrame:NSRectFromCGRect(frame) inView:view characterIndex:charIndex layoutManager:layoutManager];
    }
}

void TextAttachmentCellHighlight( TextAttachmentCellRef ref, Boolean highlight, CGRect frame, NSInteger controlViewTag )
{
    NSView *view = ViewWithTag( controlViewTag );
    if ( view ) {
        [ref highlight:highlight withFrame:NSRectFromCGRect(frame) inView:view];
    }
}

// Cell size and position
CGSize TextAttachmentCellSize( TextAttachmentCellRef ref )
{ return NSSizeToCGSize([ref cellSize]); }

CGPoint TextAttachmentCellBaselineOffset( TextAttachmentCellRef ref )
{ return NSPointToCGPoint([ref cellBaselineOffset]); }

CGRect TextAttachmentCellFrameForTextContainer( TextAttachmentCellRef ref, TextContainerRef tc, CGRect proposedLineFragment, CGPoint glyphPosition, NSUInteger charIndex )
{ return NSRectToCGRect([ref cellFrameForTextContainer:tc proposedLineFragment:NSRectFromCGRect(proposedLineFragment) glyphPosition:NSPointFromCGPoint(glyphPosition) characterIndex:charIndex]); }

// Set attachment
void TextAttachmentCellSetAttachment( TextAttachmentCellRef ref, TextAttachmentRef attachment )
{ [ref setAttachment:attachment]; }


