/*
 TextContainer.m

 Bernie Wylde
 */

#import "TextContainer.h"

// - create -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
TextContainerRef TextContainerWithSize( CGSize size )
{ return [[[NSTextContainer alloc] initWithSize:NSSizeFromCGSize(size)] autorelease]; }
#endif

// - components -
LayoutManagerRef TextContainerLayoutManager( TextContainerRef ref )
{ return [ref layoutManager]; }

void TextContainerSetLayoutManager( TextContainerRef ref, LayoutManagerRef lm )
{ [ref setLayoutManager:lm]; }

void TextContainerReplaceLayoutManager( TextContainerRef ref, LayoutManagerRef lm )
{ [ref replaceLayoutManager:lm]; }

NSInteger TextContainerTextView( TextContainerRef tc )
{
    NSTextView *view = [tc textView];
    if ( view ) return [view tag];
    return 0;
}

// - shape -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGSize TextContainerSize( TextContainerRef tc )
{ return NSSizeToCGSize([tc size]); }

void TextContainerSetSize( TextContainerRef tc, CGSize size )
{ [tc setSize:NSSizeFromCGSize(size)]; }

CFArrayRef TextContainerExclusionPaths( TextContainerRef tc )
{ return (CFArrayRef)[tc exclusionPaths]; }

void TextContainerSetExclusionPaths( TextContainerRef tc, CFArrayRef paths )
{ [tc setExclusionPaths:(NSArray *)paths]; }

NSLineBreakMode TextContainerLineBreakMode( TextContainerRef tc )
{ return [tc lineBreakMode]; }

void TextContainerSetLineBreakMode( TextContainerRef tc, NSLineBreakMode lineBreakMode )
{ [tc setLineBreakMode:lineBreakMode]; }
#endif

Boolean TextContainerWidthTracksTextView( TextContainerRef tc )
{ return [tc widthTracksTextView]; }

void TextContainerSetWidthTracksTextView( TextContainerRef tc, Boolean flag )
{ [tc setWidthTracksTextView:flag]; }

Boolean TextContainerHeightTracksTextView( TextContainerRef tc )
{ return [tc heightTracksTextView]; }

void TextContainerSetHeightTracksTextView( TextContainerRef tc, Boolean flag )
{ [tc setHeightTracksTextView:flag]; }

// - constrain -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSUInteger TextContainerMaximumNumberOfLines( TextContainerRef tc )
{ return [tc maximumNumberOfLines]; }

void TextContainerSetMaximumNumberOfLines( TextContainerRef tc, NSUInteger lines )
{ return [tc setMaximumNumberOfLines:lines]; }
#endif

CGFloat TextContainerLineFragmentPadding( TextContainerRef tc )
{ return [tc lineFragmentPadding]; }

void TextContainerSetLineFragmentPadding( TextContainerRef tc, CGFloat padding )
{ return [tc setLineFragmentPadding:padding]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGRect TextContainerLineFragmentRectForProposedRect( TextContainerRef tc, CGRect proposedRect, NSUInteger index, NSWritingDirection direction, CGRect *remainingRect )
{ return NSRectToCGRect([tc lineFragmentRectForProposedRect:NSRectFromCGRect(proposedRect) atIndex:index writingDirection:direction remainingRect:(NSRect *)remainingRect] );
}
#endif

Boolean TextContainerIsSimpleRectangularTextContainer( TextContainerRef tc )
{ return [tc isSimpleRectangularTextContainer]; }
