/*
 TextField.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

@interface TextFieldSC : NSTextField
@end

// dialog events
enum {
    textFieldDidBeginEditing    = controlTextDidBeginEditing,
    textFieldDidChange          = controlTextDidChange,
    textFieldDidEndEditing      = controlTextDidEndEditing,
    textFieldShouldBeginEditing = controlTextShouldBeginEditing,
    textFieldShouldEndEditing   = controlTextShouldEndEditing
};

@interface CocoaTextFieldDelegate : NSObject <NSTextFieldDelegate>
@end

CocoaTextFieldDelegate *CocoaTextFieldDelegateGet( void );

NSTextField *TextFieldWithTag( NSInteger tag );
BOOL TextFieldExists( NSInteger tag );

// Selection and editing
Boolean TextFieldIsEditable( NSInteger tag );
void TextFieldSetEditable( NSInteger tag, Boolean flag );
Boolean TextFieldIsSelectable( NSInteger tag );
void TextFieldSetSelectable( NSInteger tag, Boolean flag );

// Rich text behavior
Boolean TextFieldAllowsEditingTextAttributes( NSInteger tag );
void TextFieldSetAllowsEditingTextAttributes( NSInteger tag, Boolean flag );
Boolean TextFieldImportsGraphics( NSInteger tag );
void TextFieldSetImportsGraphics( NSInteger tag, Boolean flag );

// Text color
ColorRef TextFieldTextColor( NSInteger tag );
void TextFieldSetTextColor( NSInteger tag, ColorRef col );

// Autolayout sizing
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CGFloat TextFieldPreferredMaxLayoutWidth( NSInteger tag );
void TextFieldSetPreferredMaxLayoutWidth( NSInteger tag, CGFloat width );
#endif

// Background
ColorRef TextFieldBackgroundColor( NSInteger tag );
void TextFieldSetBackgroundColor( NSInteger tag, ColorRef col );
Boolean TextFieldDrawsBackground( NSInteger tag );
void TextFieldSetDrawsBackground( NSInteger tag, Boolean flag );

// Border
Boolean TextFieldIsBezeled( NSInteger tag );
void TextFieldSetBezeled( NSInteger tag, Boolean flag );
NSTextFieldBezelStyle TextFieldBezelStyle( NSInteger tag );
void TextFieldSetBezelStyle( NSInteger tag, NSTextFieldBezelStyle style );
Boolean TextFieldIsBordered( NSInteger tag );
void TextFieldSetBordered( NSInteger tag, Boolean flag );

// Selecting text
void TextFieldSelectText( NSInteger tag );

// Responder
Boolean TextFieldAcceptsFirstResponder( NSInteger tag );

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean TextFieldAllowsDefaultTighteningForTruncation( NSInteger tag );
void TextFieldSetAllowsDefaultTighteningForTruncation( NSInteger tag, Boolean flag );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean TextFieldIsAutomaticTextCompletionEnabled( NSInteger tag );
void TextFieldSetAutomaticTextCompletionEnabled( NSInteger tag, Boolean flag );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSInteger TextFieldMaximumNumberOfLines( NSInteger tag );
void TextFieldSetMaximumNumberOfLines( NSInteger tag, NSInteger lines );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CFAttributedStringRef TextFieldPlaceholderAttributedString( NSInteger tag );
void TextFieldSetPlaceholderAttributedString( NSInteger tag, CFAttributedStringRef string );
CFStringRef TextFieldPlaceholderString( NSInteger tag );
void TextFieldSetPlaceholderString( NSInteger tag, CFStringRef string );
#endif

// Cell methods
Boolean TextFieldSendsActionOnEndEditing( NSInteger tag );
void TextFieldSetSendsActionOnEndEditing( NSInteger tag, Boolean flag );
Boolean TextFieldAllowsUndo( NSInteger tag );
void TextFieldSetAllowsUndo( NSInteger tag, Boolean flag );

// Convenience
// Selected ranges
void TextFieldSetSelectedRange( NSInteger tag, CFRange range );
CFRange TextFieldSelectedRange( NSInteger tag );



// Init
TextFieldRef TextFieldInit( NSInteger inTag, CGRect inRect );

// Statement
void CocoaTextField( NSInteger inTag, NSInteger inEnabled, CFStringRef inText, CGRect inRect, NSInteger inWndTag );

