/*
 TextField.m

 Bernie Wylde
 */

#import "TextField.h"

#pragma mark - subclass
@implementation TextFieldSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end

@implementation CocoaTextFieldDelegate
- (void)textFieldAction:(id)sender {
    CocoaControlCallUserDefinedOnDialogOrCallback( 1, sender, NULL );// 1 = _btnClick
}

- (void)controlTextDidBeginEditing:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldDidBeginEditing, notification.object, notification );
}

- (void)controlTextDidChange:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldDidChange, notification.object, notification );
}

- (void)controlTextDidEndEditing:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldDidEndEditing, notification.object, notification );
}

- (BOOL)control:(NSControl *)control textShouldBeginEditing:(NSText *)fieldEditor {
    DialogEventSetBool(YES);
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldShouldBeginEditing, control, fieldEditor );
    return DialogEventBool();
}

- (BOOL)control:(NSControl *)control textShouldEndEditing:(NSText *)fieldEditor {
    DialogEventSetBool(YES);
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldShouldEndEditing, control, fieldEditor );
    return DialogEventBool();
}

- (BOOL)control:(NSControl *)control textView:(NSTextView *)fieldEditor doCommandBySelector:(SEL)commandSelector {
    DialogEventSetBool( NO );
    NSString *string = NSStringFromSelector(commandSelector);
    DialogEventSetNSString( string );
    CocoaControlCallUserDefinedOnDialogOrCallback( controlDoCommandBySelector, control, string );
    return DialogEventBool();
}
@end

CocoaTextFieldDelegate *CocoaTextFieldDelegateGet( void )
{
    static CocoaTextFieldDelegate *sCocoaTextFieldDelegate = nil;
    if ( !sCocoaTextFieldDelegate ) sCocoaTextFieldDelegate = [[CocoaTextFieldDelegate alloc] init];
    return sCocoaTextFieldDelegate;
}

NSTextField *TextFieldWithTag( NSInteger tag )
{ return (NSTextField *)CocoaValidateViewKindOfClass( tag, [NSTextField class] ); }

BOOL TextFieldExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSTextField class] ); }

// Selection and editing
Boolean TextFieldIsEditable( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld isEditable];
    return NO;
}

void TextFieldSetEditable( NSInteger tag, Boolean flag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setEditable:flag];
}

Boolean TextFieldIsSelectable( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld isSelectable];
    return NO;
}

void TextFieldSetSelectable( NSInteger tag, Boolean flag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setSelectable:flag];
}

// Rich text behavior
Boolean TextFieldAllowsEditingTextAttributes( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld allowsEditingTextAttributes];
    return NO;
}

void TextFieldSetAllowsEditingTextAttributes( NSInteger tag, Boolean flag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setAllowsEditingTextAttributes:flag];
}

Boolean TextFieldImportsGraphics( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld importsGraphics];
    return NO;
}

void TextFieldSetImportsGraphics( NSInteger tag, Boolean flag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setImportsGraphics:flag];
}

// Text color
ColorRef TextFieldTextColor( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld textColor];
    return nil;
}

void TextFieldSetTextColor( NSInteger tag, ColorRef col )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setTextColor:col];
}

// Autolayout sizing
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
CGFloat TextFieldPreferredMaxLayoutWidth( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld preferredMaxLayoutWidth];
    return 0.0;
}

void TextFieldSetPreferredMaxLayoutWidth( NSInteger tag, CGFloat width )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setPreferredMaxLayoutWidth:width];
}
#endif

// Background
ColorRef TextFieldBackgroundColor( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld backgroundColor];
    return nil;
}

void TextFieldSetBackgroundColor( NSInteger tag, ColorRef col )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setBackgroundColor:col];
}

Boolean TextFieldDrawsBackground( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld drawsBackground];
    return NO;
}

void TextFieldSetDrawsBackground( NSInteger tag, Boolean flag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setDrawsBackground:flag];
}

// Border
Boolean TextFieldIsBezeled( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld isBezeled];
    return NO;
}

void TextFieldSetBezeled( NSInteger tag, Boolean flag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setBezeled:flag];
}

NSTextFieldBezelStyle TextFieldBezelStyle( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld bezelStyle];
    return NO;
}

void TextFieldSetBezelStyle( NSInteger tag, NSTextFieldBezelStyle style )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setBezelStyle:style];
}

Boolean TextFieldIsBordered( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld isBordered];
    return NO;
}

void TextFieldSetBordered( NSInteger tag, Boolean flag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setBordered:flag];
}

// Selecting text
void TextFieldSelectText( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld selectText:nil];
}

// Responder
Boolean TextFieldAcceptsFirstResponder( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld acceptsFirstResponder];
    return NO;
}

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean TextFieldAllowsDefaultTighteningForTruncation( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld allowsDefaultTighteningForTruncation];
    return NO;
}

void TextFieldSetAllowsDefaultTighteningForTruncation( NSInteger tag, Boolean flag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setAllowsDefaultTighteningForTruncation:flag];
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean TextFieldIsAutomaticTextCompletionEnabled( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld isAutomaticTextCompletionEnabled];
    return NO;
}

void TextFieldSetAutomaticTextCompletionEnabled( NSInteger tag, Boolean flag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setAutomaticTextCompletionEnabled:flag];
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSInteger TextFieldMaximumNumberOfLines( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [fld maximumNumberOfLines];
    return 0;
}

void TextFieldSetMaximumNumberOfLines( NSInteger tag, NSInteger lines )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setMaximumNumberOfLines:lines];
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000

CFAttributedStringRef TextFieldPlaceholderAttributedString( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return (CFAttributedStringRef)[fld placeholderAttributedString];
    return nil;
}

void TextFieldSetPlaceholderAttributedString( NSInteger tag, CFAttributedStringRef string )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [fld setPlaceholderAttributedString:(NSAttributedString *)string];
}

CFStringRef TextFieldPlaceholderString( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return (CFStringRef)[fld placeholderString];
    return nil;
}

void TextFieldSetPlaceholderString( NSInteger tag, CFStringRef string )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) {
        if ( [fld respondsToSelector:@selector(setPlaceholderString:)] ) {
            [fld setPlaceholderString:(NSString *)string];   // macOS 10.10
        }
    }
}
#endif

// Cell methods
Boolean TextFieldSendsActionOnEndEditing( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [[fld cell] sendsActionOnEndEditing];
    return NO;
}

void TextFieldSetSendsActionOnEndEditing( NSInteger tag, Boolean flag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [[fld cell] setSendsActionOnEndEditing:flag];
}

Boolean TextFieldAllowsUndo( NSInteger tag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) return [[fld cell] allowsUndo];
    return NO;
}

void TextFieldSetAllowsUndo( NSInteger tag, Boolean flag )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) [[fld cell] setAllowsUndo:flag];
}

// Convenience
CFRange TextFieldSelectedRange( NSInteger tag )
{
    CFRange range = {0,0};
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) {
        NSText *fldEditor = [fld currentEditor];
        NSRange nsRange = [fldEditor selectedRange];
        range.location = nsRange.location;
        range.length = nsRange.length;
    }
    return range;
}

void TextFieldSetSelectedRange( NSInteger tag, CFRange range )
{
    NSTextField *fld = TextFieldWithTag( tag );
    if ( fld ) {
        NSText *fldEditor = [fld currentEditor];
        [fldEditor setSelectedRange:NSMakeRange(range.location,range.length)];
    }
}


// Init
TextFieldRef TextFieldInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 96, 22 );

    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    NSTextField *fld;
    if ( subclass ) {
        fld = [[[TextFieldSC alloc] initWithFrame:rect] autorelease];
    } else {
        fld = [[[NSTextField alloc] initWithFrame:rect] autorelease];
    }

    // tag
    [fld setTag: inTag];

    // fix font to match I.B.
    CocoaControlFixFont( fld );
    
    // delegate
    [fld setDelegate:CocoaTextFieldDelegateGet()];
    
    // target/action (to intercept mouse clicks)
    [fld setTarget:CocoaTextFieldDelegateGet()];
    [fld setAction:@selector(textFieldAction:)];
    [[fld cell] setSendsActionOnEndEditing:YES];
    
    return fld;
}


// Statement
void CocoaTextField( NSInteger inTag, NSInteger inEnabled, CFStringRef inText, CGRect inRect, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSString *text = (NSString *)inText;
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSTextField *fld = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !fld ) {
            fld = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( fld ) {    // text field already exists
            
            //                if ( [fld class] != [NSTextField class] ) return;
            
            // make sure it's same view class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [fld class], [NSTextField class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, fld, [NSTextField class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [fld setEnabled:inEnabled];
            
            // text
            if ( text ) [fld setStringValue:text];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [fld setFrame:rect];
            
        } else {        // create text field
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 96, 22 );
            
            // init
//            if ( inSubclass ) {
//                fld = [[TextFieldSC alloc] initWithFrame:rect];
//            } else {
//                fld = [[NSTextField alloc] initWithFrame:rect];
//            }
            fld = TextFieldInit( inTag, inRect );
            
            // default autoresizing
            [fld setAutoresizingMask:NSViewMinYMargin];
            
            // tag
 //           [fld setTag: inTag];
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [fld setEnabled:inEnabled];
            
            // text
            if ( text ) [fld setStringValue:text];
            
            // wndTag
            [fld setWndTag:inWndTag];
            
//            // fix font to match I.B.
//            CocoaControlFixFont( fld );
//
//            // delegate
//            [fld setDelegate:CocoaTextFieldDelegateGet()];
//
//            // target/action (to intercept mouse clicks)
//            [fld setTarget:CocoaTextFieldDelegateGet()];
//            [fld setAction:@selector(textFieldAction:)];
//            [[fld cell] setSendsActionOnEndEditing:YES];
            
            // add to window
            [contentView addSubview:fld];
//            [fld release];
        }
        
        [fld setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

