/*
 TextLabel.m

 Bernie Wylde
 */

#import "TextLabel.h"

TextLabelRef TextLabelWithTag( NSInteger tag )
{ return TextFieldWithTag( tag ); }

BOOL TextLabelExists( NSInteger tag )
{ return TextFieldExists( tag ); }


// Init
TextLabelRef TextLabelInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 18, 20, 37, 17 );

    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSTextField *fld;
    if ( subclass ) {
        fld = [[[TextFieldSC alloc] initWithFrame:rect] autorelease];
    } else {
        fld = [[[NSTextField alloc] initWithFrame:rect] autorelease];
    }
    
    [fld setDrawsBackground:NO];
    [fld setEditable:NO];
    [fld setSelectable:NO];
    [fld setBezeled:NO];

    // tag
    [fld setTag: inTag];
    
    // fix font to match I.B.
    CocoaControlFixFont( fld );
    
    return fld;
}


// Statement
void CocoaTextLabel( NSInteger inTag, CFStringRef inText, CGRect inRect, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSString *text = (NSString *)inText;
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSTextField *fld = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !fld ) {
            fld = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( fld ) {    // text field already exists
            
            // make sure it's correct class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [fld class], [NSTextField class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, fld, [NSTextField class] ) ) {
                return;
            }

            // text
            if ( text ) [fld setStringValue:text];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [fld setFrame:rect];
            
        } else {        // create text field
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 18, 20, 37, 17 );
            
            // init
//            if ( inSubclass ) {
//                fld = [[TextFieldSC alloc] initWithFrame:rect];
//            } else {
//                fld = [[NSTextField alloc] initWithFrame:rect];
//            }
            fld = TextLabelInit( inTag, inRect );
            
//            [fld setDrawsBackground:NO];
//            [fld setEditable:NO];
//            [fld setSelectable:NO];
//            [fld setBezeled:NO];
            
            // default autoresizing
            [fld setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [fld setTag: inTag];
            
            // text
            if ( text ) [fld setStringValue:text];
            
            // wndTag
            [fld setWndTag:inWndTag];
            
            // fix font to match I.B.
//            CocoaControlFixFont( fld );
            
            // add to window
            [contentView addSubview:fld];
//            [fld release];
        }
        
        [fld setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

