/*
 TextStorage.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef BOOL (*CocoaTextStorageCallbackType)(TextStorageRef,NSUInteger,CFRange,NSInteger,void *);

@interface NSTextStorage (Additions)
@property (nonatomic,retain) NSValue *willProcessEditingCallbackValue;
@property (nonatomic,retain) NSValue *willProcessEditingUserDataValue;

@property (nonatomic,retain) NSValue *didProcessEditingCallbackValue;
@property (nonatomic,retain) NSValue *didProcessEditingUserDataValue;

- (CocoaTextStorageCallbackType)willProcessEditingCallback;
- (void *)willProcessEditingUserData;
- (void)setCocoaTextStorageWillProcessEditingCallback:(CocoaTextStorageCallbackType)cb;
- (void)setCocoaTextStorageWillProcessEditingUserData:(void *)ud;

- (CocoaTextStorageCallbackType)didProcessEditingCallback;
- (void *)didProcessEditingUserData;
- (void)setCocoaTextStorageDidProcessEditingCallback:(CocoaTextStorageCallbackType)cb;
- (void)setCocoaTextStorageDidProcessEditingUserData:(void *)ud;
@end

@interface CocoaTextStorageDelegate : NSObject <NSTextStorageDelegate>
@end

// private
CocoaTextStorageDelegate *CocoaTextStorageDelegateObj( void );

// Layout managers
CFArrayRef TextStorageLayoutManagers( TextStorageRef ref );
void TextStorageAddLayoutManager( TextStorageRef ref, LayoutManagerRef lm );
void TextStorageRemoveLayoutManager( TextStorageRef ref, LayoutManagerRef lm );

// Delegate
void TextStorageSetWillProcessEditingCallback( TextStorageRef ref, CocoaTextStorageCallbackType callback, void *userData );
void TextStorageSetDidProcessEditingCallback( TextStorageRef ref, CocoaTextStorageCallbackType callback, void *userData );

// Convenience
TextStorageRef TextStorageWithURL( CFURLRef url, CFDictionaryRef options, CFDictionaryRef *attributes, ErrorRef *err );


