/*
 TextStorage.m

 Bernie Wylde
 */

#import "TextStorage.h"

@implementation NSTextStorage (Additions)
@dynamic willProcessEditingCallbackValue;
@dynamic willProcessEditingUserDataValue;

@dynamic didProcessEditingCallbackValue;
@dynamic didProcessEditingUserDataValue;

- (CocoaTextStorageCallbackType)willProcessEditingCallback {
    return [objc_getAssociatedObject(self, @selector(willProcessEditingCallbackValue)) pointerValue];
}

- (void *)willProcessEditingUserData {
    return [objc_getAssociatedObject(self, @selector(willProcessEditingUserDataValue)) pointerValue];
}

- (void)setCocoaTextStorageWillProcessEditingCallback:(CocoaTextStorageCallbackType)callback {
    objc_setAssociatedObject(self, @selector(willProcessEditingCallbackValue), [NSValue valueWithPointer:callback], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (void)setCocoaTextStorageWillProcessEditingUserData:(void *)userData {
    objc_setAssociatedObject(self, @selector(willProcessEditingUserDataValue), [NSValue valueWithPointer:userData], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (CocoaTextStorageCallbackType)didProcessEditingCallback {
    return [objc_getAssociatedObject(self, @selector(didProcessEditingCallbackValue)) pointerValue];
}

- (void *)didProcessEditingUserData {
    return [objc_getAssociatedObject(self, @selector(didProcessEditingUserDataValue)) pointerValue];
}

- (void)setCocoaTextStorageDidProcessEditingCallback:(CocoaTextStorageCallbackType)callback {
    objc_setAssociatedObject(self, @selector(didProcessEditingCallbackValue), [NSValue valueWithPointer:callback], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (void)setCocoaTextStorageDidProcessEditingUserData:(void *)userData {
    objc_setAssociatedObject(self, @selector(didProcessEditingUserDataValue), [NSValue valueWithPointer:userData], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end


@implementation CocoaTextStorageDelegate

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100

- (void)textStorage:(NSTextStorage *)textStorage willProcessEditing:(NSTextStorageEditActions)editedMask range:(NSRange)editedRange changeInLength:(NSInteger)delta {
    CocoaTextStorageCallbackType callback = [textStorage willProcessEditingCallback];
    if ( callback ) {
        void *userData = [textStorage willProcessEditingUserData];
        (*callback)(textStorage,editedMask,NSRangeToCFRange(editedRange),delta,userData);
    }
}

- (void)textStorage:(NSTextStorage *)textStorage didProcessEditing:(NSTextStorageEditActions)editedMask range:(NSRange)editedRange changeInLength:(NSInteger)delta {
    CocoaTextStorageCallbackType callback = [textStorage didProcessEditingCallback];
    if ( callback ) {
        void *userData = [textStorage didProcessEditingUserData];
        (*callback)(textStorage,editedMask,NSRangeToCFRange(editedRange),delta,userData);
    }
}

#else   // __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100

- (void)textStorageWillProcessEditing:(NSNotification *)aNotification {
    CocoaTextStorageCallbackType callback = [aNotification.object willProcessEditingCallback];
    if ( callback ) {
        void *userData = [aNotification.object willProcessEditingUserData];
        (*callback)(aNotification.object,0,CFRangeMake(NSNotFound,0),0,userData);
    }
}

- (void)textStorageDidProcessEditing:(NSNotification *)aNotification {
    CocoaTextStorageCallbackType callback = [aNotification.object didProcessEditingCallback];
    if ( callback ) {
        void *userData = [aNotification.object didProcessEditingUserData];
        (*callback)(aNotification.object,0,CFRangeMake(NSNotFound,0),0,userData);
    }
}

#endif  // __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100

@end



// - private -
CocoaTextStorageDelegate *CocoaTextStorageDelegateObj( void )
{
    static CocoaTextStorageDelegate *sCocoaTextStorageDelegate = nil;
    if ( !sCocoaTextStorageDelegate ) sCocoaTextStorageDelegate = [[CocoaTextStorageDelegate alloc] init];
    return sCocoaTextStorageDelegate;
}


// - public -
// Layout managers
CFArrayRef TextStorageLayoutManagers( TextStorageRef ref )
{ return (CFArrayRef)[ref layoutManagers]; }

void TextStorageAddLayoutManager( TextStorageRef ref, LayoutManagerRef lm )
{ [ref addLayoutManager:lm]; }

void TextStorageRemoveLayoutManager( TextStorageRef ref, LayoutManagerRef lm )
{ [ref removeLayoutManager:lm]; }

// Delegate
void TextStorageSetWillProcessEditingCallback( TextStorageRef ref, CocoaTextStorageCallbackType callback, void *userData )
{
    CocoaTextStorageDelegate *delegate = CocoaTextStorageDelegateObj();
    [ref setDelegate:delegate];
    [ref setCocoaTextStorageWillProcessEditingCallback:callback];
    [ref setCocoaTextStorageWillProcessEditingUserData:userData];
}

void TextStorageSetDidProcessEditingCallback( TextStorageRef ref, CocoaTextStorageCallbackType callback, void *userData )
{
    CocoaTextStorageDelegate *delegate = CocoaTextStorageDelegateObj();
    [ref setDelegate:delegate];
    [ref setCocoaTextStorageDidProcessEditingCallback:callback];
    [ref setCocoaTextStorageDidProcessEditingUserData:userData];
}


// Convenience
TextStorageRef TextStorageWithURL( CFURLRef url, CFDictionaryRef options, CFDictionaryRef *attributes, ErrorRef *err )
{ return [[[NSTextStorage alloc] initWithURL:(NSURL *)url options:(NSDictionary *)options documentAttributes:(NSDictionary **)attributes error:err] autorelease]; }

