/*
 TextTable.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

TextTableRef TextTableInit( void );

NSUInteger TextTableNumberOfColumns( TextTableRef ref );
void TextTableSetNumberOfColumns( TextTableRef ref, NSUInteger cols );

NSTextTableLayoutAlgorithm TextTableLayoutAlgorithm( TextTableRef ref );
void TextTableSetLayoutAlgorithm( TextTableRef ref, NSTextTableLayoutAlgorithm algorithm );

Boolean TextTableCollapsesBorders( TextTableRef ref );
void TextTableSetCollapsesBorders( TextTableRef ref, Boolean flag );

Boolean TextTableHidesEmptyCells( TextTableRef ref );
void TextTableSetHidesEmptyCells( TextTableRef ref, Boolean flag );

CGRect TextTableRectForBlock( TextTableRef ref, TextTableBlockRef block, CGPoint startingPt, CGRect inRect, TextContainerRef textContainer, CFRange charRange );
CGRect TextTableBoundsRectForBlock( TextTableRef ref, TextTableBlockRef block, CGRect contentRect, CGRect inRect, TextContainerRef textContainer, CFRange charRange );

void TextTableDrawBackgroundForBlock( TextTableRef ref, TextTableBlockRef block, CGRect frame, ViewRef controlView, CFRange charRange, LayoutManagerRef layoutManager );


/*
 TextTableBlock
 */
// Create
TextTableBlockRef TextTableBlockWithTable( TextTableRef table, NSInteger startingRow, NSInteger rowSpan, NSInteger startingColumn, NSInteger columnSpan );

// Table
TextTableRef TextTableBlockTable( TextTableBlockRef ref );

// Info
NSInteger TextTableBlockStartingRow( TextTableBlockRef ref );
NSInteger TextTableBlockRowSpan( TextTableBlockRef ref );
NSInteger TextTableBlockStartingColumn( TextTableBlockRef ref );
NSInteger TextTableBlockColumnSpan( TextTableBlockRef ref );



/*
 TextBlock
 */
// Create
TextBlockRef TextBlockInit( void );

// Dimensions
void TextBlockSetValue( TextBlockRef ref, CGFloat value, NSTextBlockValueType type, NSTextBlockDimension dimension );
CGFloat TextBlockValueForDimension( TextBlockRef ref, NSTextBlockDimension dimension );
NSTextBlockValueType TextBlockValueTypeForDimension( TextBlockRef ref, NSTextBlockDimension dimension );
CGFloat TextBlockContentWidth( TextBlockRef ref );
NSTextBlockValueType TextBlockContentWidthValueType( TextBlockRef ref );

// Margins, borders and padding
void TextBlockSetWidthEdge( TextBlockRef ref, CGFloat value, NSTextBlockValueType type, NSTextBlockLayer layer, CGRectEdge edge );
void TextBlockSetWidth( TextBlockRef ref, CGFloat value, NSTextBlockValueType type, NSTextBlockLayer layer );
CGFloat TextBlockWidthForLayer( TextBlockRef ref, NSTextBlockLayer layer, CGRectEdge edge );
NSTextBlockValueType TextBlockWidthValueTypeForLayer( TextBlockRef ref, NSTextBlockLayer layer, CGRectEdge edge );

// Alignment
NSTextBlockVerticalAlignment TextBlockVerticalAlignment( TextBlockRef ref );
void TextBlockSetVerticalAlignment( TextBlockRef ref, NSTextBlockVerticalAlignment alignment );

// Color
ColorRef TextBlockBackgroundColor( TextBlockRef ref );
void TextBlockSetBackgroundColor( TextBlockRef ref, ColorRef col );
void TextBlockSetBorderColorForEdge( TextBlockRef ref, ColorRef col, CGRectEdge edge );
void TextBlockSetBorderColor( TextBlockRef ref, ColorRef col );
ColorRef TextBlockBorderColorForEdge( TextBlockRef ref, CGRectEdge edge );

// Size and position
CGRect TextBlockRectForLayoutAtPoint( TextBlockRef ref, CGPoint pt, CGRect inRect, TextContainerRef textContainer, CFRange charRange );
CGRect TextBlockBoundsRectForContentRect( TextBlockRef ref, CGRect contentRect, CGRect inRect, TextContainerRef textContainer, CFRange charRange );

// Drawing
void TextBlockDrawBackgroundWithFrame( TextBlockRef ref, CGRect frame, ViewRef controlView, CFRange charRange, LayoutManagerRef layoutManager );

// Instance methods
void TextBlockSetContentWidth( TextBlockRef ref, CGFloat width, NSTextBlockValueType type );

