/*
 TextTable.m

 Bernie Wylde
 */

#import "TextTable.h"

TextTableRef TextTableInit( void )
{ return [[[NSTextTable alloc] init] autorelease]; }

NSUInteger TextTableNumberOfColumns( TextTableRef ref )
{ return [ref numberOfColumns]; }

void TextTableSetNumberOfColumns( TextTableRef ref, NSUInteger cols )
{ [ref setNumberOfColumns:cols]; }

NSTextTableLayoutAlgorithm TextTableLayoutAlgorithm( TextTableRef ref )
{ return [ref layoutAlgorithm]; }

void TextTableSetLayoutAlgorithm( TextTableRef ref, NSTextTableLayoutAlgorithm algorithm )
{ [ref setLayoutAlgorithm:algorithm]; }

Boolean TextTableCollapsesBorders( TextTableRef ref )
{ return [ref collapsesBorders]; }

void TextTableSetCollapsesBorders( TextTableRef ref, Boolean flag )
{ [ref setCollapsesBorders:flag]; }

Boolean TextTableHidesEmptyCells( TextTableRef ref )
{ return [ref hidesEmptyCells]; }

void TextTableSetHidesEmptyCells( TextTableRef ref, Boolean flag )
{ [ref setHidesEmptyCells:flag]; }

CGRect TextTableRectForBlock( TextTableRef ref, TextTableBlockRef block, CGPoint startingPt, CGRect inRect, TextContainerRef textContainer, CFRange charRange )
{ return NSRectToCGRect([ref rectForBlock:block layoutAtPoint:NSPointFromCGPoint(startingPt) inRect:NSRectFromCGRect(inRect) textContainer:textContainer characterRange:NSRangeFromCFRange(charRange)]); }

CGRect TextTableBoundsRectForBlock( TextTableRef ref, TextTableBlockRef block, CGRect contentRect, CGRect inRect, TextContainerRef textContainer, CFRange charRange )
{ return NSRectToCGRect([ref boundsRectForBlock:block contentRect:NSRectFromCGRect(contentRect) inRect:NSRectFromCGRect(inRect) textContainer:textContainer characterRange:NSRangeFromCFRange(charRange)]); }

void TextTableDrawBackgroundForBlock( TextTableRef ref, TextTableBlockRef block, CGRect frame, ViewRef controlView, CFRange charRange, LayoutManagerRef layoutManager )
{ [ref drawBackgroundForBlock:block withFrame:NSRectFromCGRect(frame) inView:controlView characterRange:NSRangeFromCFRange(charRange) layoutManager:layoutManager]; }


/*
 TextTableBlock
 */
// Create
TextTableBlockRef TextTableBlockWithTable( TextTableRef table, NSInteger startingRow, NSInteger rowSpan, NSInteger startingColumn, NSInteger columnSpan )
{ return [[[NSTextTableBlock alloc] initWithTable:table startingRow:startingRow rowSpan:rowSpan startingColumn:startingColumn columnSpan:columnSpan] autorelease]; }

// Table
TextTableRef TextTableBlockTable( TextTableBlockRef ref )
{ return [ref table]; }

// Info
NSInteger TextTableBlockStartingRow( TextTableBlockRef ref )
{ return [ref startingRow]; }

NSInteger TextTableBlockRowSpan( TextTableBlockRef ref )
{ return [ref rowSpan]; }

NSInteger TextTableBlockStartingColumn( TextTableBlockRef ref )
{ return [ref startingColumn]; }

NSInteger TextTableBlockColumnSpan( TextTableBlockRef ref )
{ return [ref columnSpan]; }



/*
 TextBlock
 */
// Create
TextBlockRef TextBlockInit( void )
{ return [[[NSTextBlock alloc] init] autorelease]; }

// Dimensions
void TextBlockSetValue( TextBlockRef ref, CGFloat value, NSTextBlockValueType type, NSTextBlockDimension dimension )
{ [ref setValue:value type:type forDimension:dimension]; }

CGFloat TextBlockValueForDimension( TextBlockRef ref, NSTextBlockDimension dimension )
{ return [ref valueForDimension:dimension]; }

NSTextBlockValueType TextBlockValueTypeForDimension( TextBlockRef ref, NSTextBlockDimension dimension )
{ return [ref valueTypeForDimension:dimension]; }

CGFloat TextBlockContentWidth( TextBlockRef ref )
{ return [ref contentWidth]; }

NSTextBlockValueType TextBlockContentWidthValueType( TextBlockRef ref )
{ return [ref contentWidthValueType]; }

// Margins, borders and padding
void TextBlockSetWidthEdge( TextBlockRef ref, CGFloat value, NSTextBlockValueType type, NSTextBlockLayer layer, CGRectEdge edge )
{ [ref setWidth:value type:type forLayer:layer edge:(NSRectEdge)edge]; }

void TextBlockSetWidth( TextBlockRef ref, CGFloat value, NSTextBlockValueType type, NSTextBlockLayer layer )
{ [ref setWidth:value type:type forLayer:layer]; }

CGFloat TextBlockWidthForLayer( TextBlockRef ref, NSTextBlockLayer layer, CGRectEdge edge )
{ return [ref widthForLayer:layer edge:(NSRectEdge)edge]; }

NSTextBlockValueType TextBlockWidthValueTypeForLayer( TextBlockRef ref, NSTextBlockLayer layer, CGRectEdge edge )
{ return [ref widthValueTypeForLayer:layer edge:(NSRectEdge)edge]; }

// Alignment
NSTextBlockVerticalAlignment TextBlockVerticalAlignment( TextBlockRef ref )
{ return [ref verticalAlignment]; }

void TextBlockSetVerticalAlignment( TextBlockRef ref, NSTextBlockVerticalAlignment alignment )
{ [ref setVerticalAlignment:alignment]; }

// Color
ColorRef TextBlockBackgroundColor( TextBlockRef ref )
{ return [ref backgroundColor]; }

void TextBlockSetBackgroundColor( TextBlockRef ref, ColorRef col )
{ [ref setBackgroundColor:col]; }

void TextBlockSetBorderColorForEdge( TextBlockRef ref, ColorRef col, CGRectEdge edge )
{ [ref setBorderColor:col forEdge:(NSRectEdge)edge]; }

void TextBlockSetBorderColor( TextBlockRef ref, ColorRef col )
{ [ref setBorderColor:col]; }

ColorRef TextBlockBorderColorForEdge( TextBlockRef ref, CGRectEdge edge )
{ return [ref borderColorForEdge:(NSRectEdge)edge]; }

// Size and position
CGRect TextBlockRectForLayoutAtPoint( TextBlockRef ref, CGPoint pt, CGRect inRect, TextContainerRef textContainer, CFRange charRange )
{ return NSRectToCGRect([ref rectForLayoutAtPoint:NSPointFromCGPoint(pt) inRect:NSRectFromCGRect(inRect) textContainer:textContainer characterRange:NSRangeFromCFRange(charRange)]); }

CGRect TextBlockBoundsRectForContentRect( TextBlockRef ref, CGRect contentRect, CGRect inRect, TextContainerRef textContainer, CFRange charRange )
{ return NSRectToCGRect([ref boundsRectForContentRect:NSRectFromCGRect(contentRect) inRect:NSRectFromCGRect(inRect) textContainer:textContainer characterRange:NSRangeFromCFRange(charRange)]); }

// Drawing
void TextBlockDrawBackgroundWithFrame( TextBlockRef ref, CGRect frame, ViewRef controlView, CFRange charRange, LayoutManagerRef layoutManager )
{ [ref drawBackgroundWithFrame:NSRectFromCGRect(frame) inView:controlView characterRange:NSRangeFromCFRange(charRange) layoutManager:layoutManager]; }

// Instance methods
void TextBlockSetContentWidth( TextBlockRef ref, CGFloat width, NSTextBlockValueType type )
{ [ref setContentWidth:width type:type]; }

