/*
 TextView.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

#pragma mark - subclass
@interface TextViewSC : NSTextView
@end

// dialog events
enum {
    textViewWillChangeSelectionFromCharacterRange = 2300,
    textViewWillChangeSelectionFromCharacterRanges,
    textViewDidChangeSelection,
    textViewWillShowSharingServicePicker,
    textViewDoCommandBySelector
};

#pragma mark - text view delegate
@interface CocoaTextViewDelegate : NSObject <NSTextViewDelegate> {
    NSRange fromRange;
    NSRange toRange;
    NSArray *fromRanges;
    NSArray *toRanges;
}

@property (assign) NSRange fromRange;
@property (assign) NSRange toRange;
@property (assign) NSArray *fromRanges;
@property (assign) NSArray *toRanges;

@end

CocoaTextViewDelegate *CocoaTextViewDelegateGet( void );


#pragma mark - text view additions
@interface NSTextView (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end


#pragma mark - text view events
// textViewWillChangeSelectionFromCharacterRange
CFRange TextViewEventFromRange( void );
CFRange TextViewEventToRange( void );
void TextViewEventSetToRange( CFRange range );

// textViewWillChangeSelectionFromCharacterRanges
CFArrayRef TextViewEventFromRanges( void );
CFArrayRef TextViewEventToRanges( void );
void TextViewEventSetToRanges( CFArrayRef ranges );


#pragma mark - text view functions
NSTextView *TextViewWithTag( NSInteger tag );
BOOL TextViewExists( NSInteger tag );

// Accessing text system objects
TextContainerRef TextViewTextContainer( NSInteger tag );
void TextViewSetTextContainer( NSInteger tag, TextContainerRef tc );
void TextViewReplaceTextContainer( NSInteger tag, TextContainerRef tc );
CGSize TextViewTextContainerInset( NSInteger tag );
void TextViewSetTextContainerInset( NSInteger tag, CGSize inset );
CGPoint TextViewTextContainerOrigin( NSInteger tag );
void TextViewInvalidateTextContainerOrigin( NSInteger tag );
LayoutManagerRef TextViewLayoutManager( NSInteger tag );
//CFMutableAttributedStringRef TextViewTextStorage( NSInteger tag );
TextStorageRef TextViewTextStorage( NSInteger tag );

// Graphics attributes
ColorRef TextViewBackgroundColor( NSInteger tag );
void TextViewSetBackgroundColor( NSInteger tag, ColorRef col );
Boolean TextViewDrawsBackground( NSInteger tag );
void TextViewSetDrawsBackground( NSInteger tag, Boolean flag );
Boolean TextViewAllowsDocumentBackgroundColorChange( NSInteger tag );
void TextViewSetAllowsDocumentBackgroundColorChange( NSInteger tag, Boolean flag );
void TextViewChangeDocumentBackgroundColor( NSInteger tag );

// Controlling display
void TextViewShowFindIndicatorForRange( NSInteger tag, CFRange range );

// Behavioral attributes
Boolean TextViewAllowsUndo( NSInteger tag );
void TextViewSetAllowsUndo( NSInteger tag, Boolean flag );
Boolean TextViewIsEditable( NSInteger tag );
void TextViewSetEditable( NSInteger tag, Boolean flag );
Boolean TextViewIsSelectable( NSInteger tag );
void TextViewSetSelectable( NSInteger tag, Boolean flag );
Boolean TextViewIsRichText( NSInteger tag );
void TextViewSetRichText( NSInteger tag, Boolean flag );
Boolean TextViewImportsGraphics( NSInteger tag );
void TextViewSetImportsGraphics( NSInteger tag, Boolean flag );
void TextViewSetBaseWritingDirection( NSInteger tag, NSWritingDirection direction );
ParagraphStyleRef TextViewDefaultParagraphStyle( NSInteger tag );
void TextViewSetDefaultParagraphStyle( NSInteger tag, ParagraphStyleRef style );
Boolean TextViewAllowsImageEditing( NSInteger tag );
void TextViewSetAllowsImageEditing( NSInteger tag, Boolean flag );
Boolean TextViewIsAutomaticQuoteSubstitutionEnabled( NSInteger tag );
void TextViewSetAutomaticQuoteSubstitutionEnabled( NSInteger tag, Boolean flag );
void TextViewToggleAutomaticQuoteSubstitution( NSInteger tag );
Boolean TextViewIsAutomaticLinkDetectionEnabled( NSInteger tag );
void TextViewSetAutomaticLinkDetectionEnabled( NSInteger tag, Boolean flag );
void TextViewToggleAutomaticLinkDetection( NSInteger tag );
Boolean TextViewDisplaysLinkToolTips( NSInteger tag );
void TextViewSetDisplaysLinkToolTips( NSInteger tag, Boolean flag );

// Formatting controls
Boolean TextViewUsesRuler( NSInteger tag );
void TextViewSetUsesRuler( NSInteger tag, Boolean flag );
Boolean TextViewIsRulerVisible( NSInteger tag );
void TextViewSetRulerVisible( NSInteger tag, Boolean flag );
Boolean TextViewUsesInspectorBar( NSInteger tag );
void TextViewSetUsesInspectorBar( NSInteger tag, Boolean flag );

// Selection
void TextViewSetSelectedRange( NSInteger tag, CFRange range );
void TextViewSetSelectedRangeAffinity( NSInteger tag, CFRange range, NSSelectionAffinity affinity, Boolean stillSelecting );
NSSelectionAffinity TextViewSelectionAffinity( NSInteger tag );
NSSelectionGranularity TextViewSelectionGranularity( NSInteger tag );
void TextViewSetSelectionGranularity( NSInteger tag, NSSelectionGranularity granularity );
ColorRef TextViewInsertionPointColor( NSInteger tag );
void TextViewSetInsertionPointColor( NSInteger tag, ColorRef col );
CFDictionaryRef TextViewSelectedTextAttributes( NSInteger tag );
void TextViewSetSelectedTextAttributes( NSInteger tag, CFDictionaryRef attributes );
CFDictionaryRef TextViewMarkedTextAttributes( NSInteger tag );
void TextViewSetMarkedTextAttributes( NSInteger tag, CFDictionaryRef attributes );
CFDictionaryRef TextViewLinkTextAttributes( NSInteger tag );
void TextViewSetLinkTextAttributes( NSInteger tag, CFDictionaryRef attributes );
NSUInteger TextViewCharacterIndexForInsertionAtPoint( NSInteger tag, CGPoint pt );

// Managing the pasteboard
CFArrayRef TextViewReadablePasteboardTypes( NSInteger tag );
CFArrayRef TextViewWritablePasteboardTypes( NSInteger tag );

// Text attributes
void TextViewAlignJustified( NSInteger tag );
void TextViewChangeAttributes( NSInteger tag );
void TextViewChangeColor( NSInteger tag );
void TextViewSetAlignment( NSInteger tag, NSTextAlignment alignment, CFRange range );
CFDictionaryRef TextViewTypingAttributes( NSInteger tag );
void TextViewSetTypingAttributes( NSInteger tag, CFDictionaryRef attributes );
void TextViewUseStandardKerning( NSInteger tag );
void TextViewLowerBaseline( NSInteger tag );
void TextViewRaiseBaseline( NSInteger tag );
void TextViewTurnOffKerning( NSInteger tag );
void TextViewLoosenKerning( NSInteger tag );
void TextViewTightenKerning( NSInteger tag );
void TextViewUseStandardLigatures( NSInteger tag );
void TextViewTurnOffLigatures( NSInteger tag );
void TextViewUseAllLigatures( NSInteger tag );

// Clicking and pasting
// ...
void TextViewPasteAsPlainText( NSInteger tag );
void TextViewPasteAsRichText( NSInteger tag );

// Undo support
void TextViewBreakUndoCoalescing( NSInteger tag );
Boolean TextViewIsCoalescingUndo( NSInteger tag );

// Subclasses to override
CFRange TextViewRangeForUserTextChange( NSInteger tag );

// Spell checker
Boolean TextViewIsContinuousSpellCheckingEnabled( NSInteger tag );
void TextViewSetContinuousSpellCheckingEnabled( NSInteger tag, Boolean flag );
void TextViewToggleContinuousSpellChecking( NSInteger tag );
Boolean TextViewIsGrammarCheckingEnabled( NSInteger tag );
void TextViewSetGrammarCheckingEnabled( NSInteger tag, Boolean flag );
void TextViewToggleGrammarChecking( NSInteger tag );
void TextViewSetSpellingState( NSInteger tag, NSInteger value, CFRange range );

// Service picker
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
void TextViewOrderFrontSharingServicePicker( NSInteger tag );
#endif

// Speaking
void TextViewStartSpeaking( NSInteger tag );
void TextViewStopSpeaking( NSInteger tag );

// Working with panels
Boolean TextViewUsesFontPanel( NSInteger tag );
void TextViewSetUsesFontPanel( NSInteger tag, Boolean flag );
Boolean TextViewUsesFindPanel( NSInteger tag );
void TextViewSetUsesFindPanel( NSInteger tag, Boolean flag );
void TextViewOrderFrontLinkPanel( NSInteger tag );
void TextViewOrderFrontListPanel( NSInteger tag );
void TextViewOrderFrontSpacingPanel( NSInteger tag );
void TextViewOrderFrontTablePanel( NSInteger tag );
void TextViewOrderFrontSubstitutionsPanel( NSInteger tag );

// Changing layout orientation
void TextViewChangeLayoutOrientation( NSInteger tag );
void TextViewSetLayoutOrientation( NSInteger tag, NSTextLayoutOrientation orientation );

// Find bar
Boolean TextViewUsesFindBar( NSInteger tag );
void TextViewSetUsesFindBar( NSInteger tag, Boolean flag );
Boolean TextViewIsIncrementalSearchingEnabled( NSInteger tag );
void TextViewSetIncrementalSearchingEnabled( NSInteger tag, Boolean flag );

// custom
void TextViewPerformFindPanelAction( NSInteger tag, NSTextFinderAction action );
void TextViewSetWordWrap( NSInteger tag, Boolean flag );


// Init
TextViewRef TextViewInit( NSInteger inTag, CGRect inRect );

// Statement
void CocoaTextView( NSInteger inTag, CGRect inRect, NSInteger inScrollTag, NSTextContainer *tc, NSInteger subclass, NSInteger inWndTag );
