/*
 TextView.m

 Bernie Wylde
 */

#import "TextView.h"

#pragma mark - subclass
@implementation TextViewSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

// required for key events
//- (BOOL)acceptsFirstResponder {
//    return YES;//self.responderFlag;
//}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(YES);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}

- (void)resetCursorRects {
    CocoaSubclassResetCursorRects( self );
}

#pragma mark - dragging
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender {
    return CocoaSubclassDraggingEntered( self, sender );
}

- (void)draggingExited:(id <NSDraggingInfo>)sender {
    CocoaSubclassDraggingExited( self, sender );
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender {
    return CocoaSubclassPerformDragOperation( self, sender );
}
@end


@implementation CocoaTextViewDelegate

@synthesize fromRange;
@synthesize toRange;
@synthesize fromRanges;
@synthesize toRanges;

- (void)textView:(NSTextView *)textView callUserDefinedOnDialogFunctionWithEvent:(NSInteger)event object:obj {
    NSInteger wndTag = [textView wndTag];
    CocoaOutputWindowSetTag( wndTag );
    CallUserDefinedOnDialogFunction( event, [textView tag], wndTag, obj );
}


- (NSRange)textView:(NSTextView *)textView willChangeSelectionFromCharacterRange:(NSRange)oldSelectedCharRange toCharacterRange:(NSRange)newSelectedCharRange {
    fromRange = oldSelectedCharRange;
    toRange = newSelectedCharRange;
    [self textView:textView callUserDefinedOnDialogFunctionWithEvent:textViewWillChangeSelectionFromCharacterRange object:nil];
    return toRange;
}

// we can't have both this method and the one above
//- (NSArray<NSValue *> *)textView:(NSTextView *)textView willChangeSelectionFromCharacterRanges:(NSArray<NSValue *> *)oldSelectedCharRanges toCharacterRanges:(NSArray<NSValue *> *)newSelectedCharRanges {
//    fromRanges = oldSelectedCharRanges;
//    toRanges = newSelectedCharRanges;
//    [self textView:textView callUserDefinedOnDialogFunctionWithEvent:textViewWillChangeSelectionFromCharacterRanges object:nil];
//    return toRanges;
//}

- (void)textDidBeginEditing:(NSNotification *)notification {

    [self textView:notification.object callUserDefinedOnDialogFunctionWithEvent:textDidBeginEditing object:nil];
}

- (void)textDidChange:(NSNotification *)notification {
    [self textView:notification.object callUserDefinedOnDialogFunctionWithEvent:textDidChange object:nil];
}

- (void)textDidEndEditing:(NSNotification *)notification {
    [self textView:notification.object callUserDefinedOnDialogFunctionWithEvent:textDidEndEditing object:nil];
}

- (void)textViewDidChangeSelection:(NSNotification *)notification {
    [self textView:notification.object callUserDefinedOnDialogFunctionWithEvent:textViewDidChangeSelection object:nil];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
- (NSSharingServicePicker *)textView:(NSTextView *)textView willShowSharingServicePicker:(NSSharingServicePicker *)servicePicker forItems:(NSArray *)items {
    DialogEventSetNSArray( items );
    DialogEventSetNSObject( servicePicker );
    [self textView:textView callUserDefinedOnDialogFunctionWithEvent:textViewWillShowSharingServicePicker object:items];
    return DialogEventNSObject();
}
#endif

- (BOOL)textView:(NSTextView *)textView doCommandBySelector:(SEL)commandSelector {
    DialogEventSetBool( NO );
    NSString *string = NSStringFromSelector(commandSelector);
    DialogEventSetNSString( string );
    [self textView:textView callUserDefinedOnDialogFunctionWithEvent:textViewDoCommandBySelector object:string];
    return DialogEventBool();
}

// ...
@end

CocoaTextViewDelegate *CocoaTextViewDelegateGet( void )
{
    static CocoaTextViewDelegate *sCocoaTextViewDelegate = nil;
    if ( !sCocoaTextViewDelegate ) sCocoaTextViewDelegate = [[CocoaTextViewDelegate alloc] init];
    return sCocoaTextViewDelegate;
}


// textViewWillChangeSelectionFromCharacterRange

CFRange TextViewEventFromRange( void )
{
    NSRange range = [CocoaTextViewDelegateGet() fromRange];
    return CFRangeMake(range.location,range.length);
}

CFRange TextViewEventToRange( void )
{
    NSRange range = [CocoaTextViewDelegateGet() toRange];
    return CFRangeMake(range.location,range.length);
}

void TextViewEventSetToRange( CFRange range )
{ [CocoaTextViewDelegateGet() setToRange:NSMakeRange(range.location,range.length)]; }

// textViewWillChangeSelectionFromCharacterRanges
CFArrayRef TextViewEventFromRanges( void )
{ return (CFArrayRef)[CocoaTextViewDelegateGet() fromRanges]; }

CFArrayRef TextViewEventToRanges( void )
{ return (CFArrayRef)[CocoaTextViewDelegateGet() toRanges]; }

void TextViewEventSetToRanges( CFArrayRef ranges )
{ [CocoaTextViewDelegateGet() setToRanges:(NSArray *)ranges]; }


@implementation NSTextView (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end


// Functions
NSTextView *TextViewWithTag( NSInteger tag )
{ return (NSTextView *)CocoaValidateViewKindOfClass( tag, [NSTextView class] ); }

BOOL TextViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSTextView class] ); }

// Accessing text system objects
TextContainerRef TextViewTextContainer( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView textContainer];
    return nil;
}

void TextViewSetTextContainer( NSInteger tag, TextContainerRef tc )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setTextContainer:tc];
}

void TextViewReplaceTextContainer( NSInteger tag, TextContainerRef tc )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView replaceTextContainer:tc];
}

CGSize TextViewTextContainerInset( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return NSSizeToCGSize([textView textContainerInset]);
    return CGSizeZero;
}

void TextViewSetTextContainerInset( NSInteger tag, CGSize inset )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setTextContainerInset:NSSizeFromCGSize(inset)];
}

CGPoint TextViewTextContainerOrigin( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return NSPointToCGPoint([textView textContainerOrigin]);
    return CGPointZero;
}

void TextViewInvalidateTextContainerOrigin( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView invalidateTextContainerOrigin];
}

ColorRef TextViewBackgroundColor( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView backgroundColor];
    return nil;
}

void TextViewSetBackgroundColor( NSInteger tag, ColorRef col )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setBackgroundColor:col];
}

Boolean TextViewDrawsBackground( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView drawsBackground];
    return NO;
}

void TextViewSetDrawsBackground( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setDrawsBackground:flag];
}

Boolean TextViewAllowsDocumentBackgroundColorChange( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView allowsDocumentBackgroundColorChange];
    return NO;
}

void TextViewSetAllowsDocumentBackgroundColorChange( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setAllowsDocumentBackgroundColorChange:flag];
}

void TextViewChangeDocumentBackgroundColor( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView changeDocumentBackgroundColor:nil];
}

// Controlling display
void TextViewShowFindIndicatorForRange( NSInteger tag, CFRange range )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView showFindIndicatorForRange:NSRangeFromCFRange(range)];
}

Boolean TextViewIsEditable( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView isEditable];
    return NO;
}

void TextViewSetEditable( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setEditable:flag];
}

Boolean TextViewIsSelectable( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView isSelectable];
    return NO;
}

void TextViewSetSelectable( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setSelectable:flag];
}

Boolean TextViewUsesRuler( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView usesRuler];
    return NO;
}

void TextViewSetUsesRuler( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setUsesRuler:flag];
}

Boolean TextViewIsRulerVisible( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView isRulerVisible];
    return NO;
}

void TextViewSetRulerVisible( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setRulerVisible:flag];
}

Boolean TextViewUsesInspectorBar( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView usesInspectorBar];
    return NO;
}

void TextViewSetUsesInspectorBar( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setUsesInspectorBar:flag];
}

void TextViewSetSelectedRange( NSInteger tag, CFRange range )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        [textView setSelectedRange:NSMakeRange(range.location,range.length)];
    }
}

void TextViewSetSelectedRangeAffinity( NSInteger tag, CFRange range, NSSelectionAffinity affinity, Boolean stillSelecting )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        [textView setSelectedRange:NSMakeRange(range.location,range.length) affinity:affinity stillSelecting:stillSelecting];
    }
}

NSSelectionAffinity TextViewSelectionAffinity( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        return [textView selectionAffinity];
    }
    return 0;
}

NSSelectionGranularity TextViewSelectionGranularity( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        return [textView selectionGranularity];
    }
    return 0;
}

void TextViewSetSelectionGranularity( NSInteger tag, NSSelectionGranularity granularity )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        [textView setSelectionGranularity:granularity];
    }
}

ColorRef TextViewInsertionPointColor( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        return [textView insertionPointColor];
    }
    return nil;
}

void TextViewSetInsertionPointColor( NSInteger tag, ColorRef col )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        [textView setInsertionPointColor:col];
    }
}

CFDictionaryRef TextViewSelectedTextAttributes( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        return (CFDictionaryRef)[textView selectedTextAttributes];
    }
    return nil;
}

void TextViewSetSelectedTextAttributes( NSInteger tag, CFDictionaryRef attributes )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        [textView setSelectedTextAttributes:(NSDictionary *)attributes];
    }
}

CFDictionaryRef TextViewMarkedTextAttributes( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        return (CFDictionaryRef)[textView markedTextAttributes];
    }
    return nil;
}

void TextViewSetMarkedTextAttributes( NSInteger tag, CFDictionaryRef attributes )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        [textView setMarkedTextAttributes:(NSDictionary *)attributes];
    }
}

CFDictionaryRef TextViewLinkTextAttributes( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        return (CFDictionaryRef)[textView linkTextAttributes];
    }
    return nil;
}

void TextViewSetLinkTextAttributes( NSInteger tag, CFDictionaryRef attributes )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        [textView setLinkTextAttributes:(NSDictionary *)attributes];
    }
}

NSUInteger TextViewCharacterIndexForInsertionAtPoint( NSInteger tag, CGPoint pt )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        return [textView characterIndexForInsertionAtPoint:NSPointFromCGPoint(pt)];
    }
    return 0;
}

// Managing the pasteboard
CFArrayRef TextViewReadablePasteboardTypes( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        return (CFArrayRef)[textView readablePasteboardTypes];
    }
    return nil;
}

CFArrayRef TextViewWritablePasteboardTypes( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        return (CFArrayRef)[textView writablePasteboardTypes];
    }
    return nil;
}

// Text attributes
void TextViewAlignJustified( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView alignJustified:nil];
}

void TextViewChangeAttributes( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView changeAttributes:nil];
}

void TextViewChangeColor( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView changeColor:nil];
}

void TextViewSetAlignment( NSInteger tag, NSTextAlignment alignment, CFRange range )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setAlignment:alignment range:NSRangeFromCFRange(range)];
}

CFDictionaryRef TextViewTypingAttributes( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return (CFDictionaryRef)[textView typingAttributes];
    return nil;
}

void TextViewSetTypingAttributes( NSInteger tag, CFDictionaryRef attributes )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setTypingAttributes:(NSDictionary *)attributes];
}

void TextViewUseStandardKerning( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView useStandardKerning:nil];
}

void TextViewLowerBaseline( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView lowerBaseline:nil];
}

void TextViewRaiseBaseline( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView raiseBaseline:nil];
}

void TextViewTurnOffKerning( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView turnOffKerning:nil];
}

void TextViewLoosenKerning( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView loosenKerning:nil];
}

void TextViewTightenKerning( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView tightenKerning:nil];
}

void TextViewUseStandardLigatures( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView useStandardLigatures:nil];
}

void TextViewTurnOffLigatures( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView turnOffLigatures:nil];
}

void TextViewUseAllLigatures( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView useAllLigatures:nil];
}

// Clicking and pasting
void TextViewPasteAsPlainText( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView pasteAsPlainText:nil];
}

void TextViewPasteAsRichText( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView pasteAsRichText:nil];
}

// Undo support
void TextViewBreakUndoCoalescing( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView breakUndoCoalescing];
}

Boolean TextViewIsCoalescingUndo( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView isCoalescingUndo];
    return NO;
}

CFRange TextViewRangeForUserTextChange( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        return NSRangeToCFRange([textView rangeForUserTextChange]);
    }
    return CFRangeMake(NSNotFound,0);
}

// Spell checker
Boolean TextViewIsContinuousSpellCheckingEnabled( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        return [textView isContinuousSpellCheckingEnabled];
    }
    return NO;
}

void TextViewSetContinuousSpellCheckingEnabled( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setContinuousSpellCheckingEnabled:flag];
}

void TextViewToggleContinuousSpellChecking( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        [textView toggleContinuousSpellChecking:nil];
    }
}

Boolean TextViewIsGrammarCheckingEnabled( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        return [textView isGrammarCheckingEnabled];
    }
    return NO;
}

void TextViewSetGrammarCheckingEnabled( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setGrammarCheckingEnabled:flag];
}

void TextViewToggleGrammarChecking( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        [textView toggleGrammarChecking:nil];
    }
}

void TextViewSetSpellingState( NSInteger tag, NSInteger value, CFRange range )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        [textView setSpellingState:value range:NSRangeFromCFRange(range)];
    }
}

// Sharing service picker
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
void TextViewOrderFrontSharingServicePicker( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView orderFrontSharingServicePicker:nil];
}
#endif

void TextViewStartSpeaking( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView startSpeaking:nil];
}

void TextViewStopSpeaking( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView stopSpeaking:nil];
}

void TextViewSetUsesFontPanel( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setUsesFontPanel:flag];
}

void TextViewSetUsesFindPanel( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setUsesFindPanel:flag];
}

void TextViewSetUsesFindBar( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setUsesFindBar:flag];
}

Boolean TextViewIsIncrementalSearchingEnabled( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView isIncrementalSearchingEnabled];
    return NO;
}

void TextViewSetIncrementalSearchingEnabled( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setIncrementalSearchingEnabled:flag];
}

void TextViewOrderFrontLinkPanel( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView orderFrontLinkPanel:nil];
}

void TextViewOrderFrontListPanel( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView orderFrontListPanel:nil];
}

void TextViewOrderFrontSpacingPanel( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView orderFrontSpacingPanel:nil];
}

void TextViewOrderFrontTablePanel( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView orderFrontTablePanel:nil];
}

void TextViewOrderFrontSubstitutionsPanel( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView orderFrontSubstitutionsPanel:nil];
}

LayoutManagerRef TextViewLayoutManager( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView layoutManager];
    return nil;
}

//CFMutableAttributedStringRef TextViewTextStorage( NSInteger tag )
//{
//    NSTextView *textView = TextViewWithTag( tag );
//    if ( textView ) return (CFMutableAttributedStringRef)[textView textStorage];
//    return NULL;
//}

TextStorageRef TextViewTextStorage( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView textStorage];
    return nil;
}

Boolean TextViewIsRichText( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView isRichText];
    return NO;
}

void TextViewSetRichText( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setRichText:flag];
}

Boolean TextViewAllowsUndo( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView allowsUndo];
    return NO;
}

void TextViewSetAllowsUndo( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setAllowsUndo:flag];
}

Boolean TextViewImportsGraphics( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView importsGraphics];
    return NO;
}

void TextViewSetImportsGraphics( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setImportsGraphics:flag];
}

void TextViewSetBaseWritingDirection( NSInteger tag, NSWritingDirection direction )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView setBaseWritingDirection:direction];
}

ParagraphStyleRef TextViewDefaultParagraphStyle( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView defaultParagraphStyle];
    return nil;
}

void TextViewSetDefaultParagraphStyle( NSInteger tag, ParagraphStyleRef style )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setDefaultParagraphStyle:style];
}

Boolean TextViewAllowsImageEditing( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView allowsImageEditing];
    return NO;
}

void TextViewSetAllowsImageEditing( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setAllowsImageEditing:flag];
}

Boolean TextViewIsAutomaticLinkDetectionEnabled( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView isAutomaticLinkDetectionEnabled];
    return NO;
}

void TextViewSetAutomaticLinkDetectionEnabled( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setAutomaticLinkDetectionEnabled:flag];
}

void TextViewToggleAutomaticLinkDetection( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView toggleAutomaticLinkDetection:nil];
}

Boolean TextViewDisplaysLinkToolTips( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView displaysLinkToolTips];
    return NO;
}

void TextViewSetDisplaysLinkToolTips( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setDisplaysLinkToolTips:flag];
}

Boolean TextViewIsAutomaticQuoteSubstitutionEnabled( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView isAutomaticQuoteSubstitutionEnabled];
    return NO;
}

void TextViewSetAutomaticQuoteSubstitutionEnabled( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setAutomaticQuoteSubstitutionEnabled:flag];
}

void TextViewToggleAutomaticQuoteSubstitution( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView toggleAutomaticQuoteSubstitution:nil];
}



Boolean TextViewUsesFontPanel( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView usesFontPanel];
    return NO;
}

// Changing layout orientation
void TextViewChangeLayoutOrientation( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView changeLayoutOrientation:nil];
}

void TextViewSetLayoutOrientation( NSInteger tag, NSTextLayoutOrientation orientation )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) [textView setLayoutOrientation:orientation];
}



Boolean TextViewUsesFindPanel( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView usesFindPanel];
    return NO;
}

Boolean TextViewUsesFindBar( NSInteger tag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) return [textView usesFindBar];
    return NO;
}





// custom
void TextViewPerformFindPanelAction( NSInteger tag, NSTextFinderAction action )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        NSControl *c = [[NSControl alloc] init];
        [c setTag:action];
        [textView performFindPanelAction:c];
        [c release];
    }
}

void TextViewSetWordWrap( NSInteger tag, Boolean flag )
{
    NSTextView *textView = TextViewWithTag( tag );
    if ( textView ) {
        NSTextContainer *tc = [textView textContainer];
        if ( flag ) {
            NSScrollView *scrollView = [textView enclosingScrollView];
            if ( scrollView ) {
                NSSize size = [scrollView contentSize];
                [tc setWidthTracksTextView:YES];
                [tc setContainerSize:NSMakeSize(size.width, FLT_MAX)];
                [textView setHorizontallyResizable:NO];
                [textView setFrame:NSMakeRect(0,0,size.width,size.height)];
            }
        } else {
            [tc setWidthTracksTextView:NO];
            [tc setContainerSize:NSMakeSize(FLT_MAX,FLT_MAX)];
            [textView setMaxSize:NSMakeSize(FLT_MAX,FLT_MAX)];
            [textView setHorizontallyResizable:YES];
        }
    }
}



// Init
TextViewRef TextViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 240, 135 );
    
    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    NSTextView *textView;
    if ( subclass ) {
        textView = [[[TextViewSC alloc] initWithFrame:rect] autorelease];
    } else {
        textView = [[[NSTextView alloc] initWithFrame:rect] autorelease];
    }

    // vertically resizable
    [textView setVerticallyResizable:YES];

    // max size
    [textView setMaxSize:NSMakeSize(FLT_MAX, FLT_MAX)];

    // default to allows undo
    [textView setAllowsUndo:YES];

    // tag
    [textView setTag:inTag];
    
    // delegate
    [textView setDelegate:CocoaTextViewDelegateGet()];
    
    return textView;
}

// Statement
void CocoaTextView( NSInteger inTag, CGRect inRect, NSInteger inScrollTag, NSTextContainer *tc, NSInteger subclass, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSTextView *textView = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !textView ) {
            textView = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( textView ) {    // already exists
            
            // make sure it's same view class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [textView class], [NSTextView class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, textView, [NSTextView class] ) ) {
                return;
            }

            // rect
            if ( !NSIsEmptyRect( rect ) ) [textView setFrame:rect];
            
        } else {            // create
            
            NSScrollView *scrollView = nil;
            if ( inScrollTag ) {
                scrollView = CocoaViewSubviewWithTag( contentView, inScrollTag );
                
                if ( !scrollView ) {
                    scrollView = TitlebarAccessoryViewWithTag( inTag );
                }
                
            }
            
            // rect
            if ( NSIsEmptyRect( rect ) ) {
                if ( scrollView ) {
                    NSSize size = [scrollView contentSize];
                    rect = NSMakeRect( 0, 0, size.width, size.height );
                } else {
                    rect = NSMakeRect( 20, 20, 240, 135 );
                }
            }
            
            // init
 //           if ( subclass != kFBParamMissing ) inSubclass = subclass;

//            if ( inSubclass ) {
//                if ( tc ) {
//                    textView = [[TextViewSC alloc] initWithFrame:rect textContainer:tc];
//                } else {
//                    textView = [[TextViewSC alloc] initWithFrame:rect];
//
//                    // vertically resizable
//                    [textView setVerticallyResizable:YES];
//                }
//            } else {
//                if ( tc ) {
//                    textView = [[NSTextView alloc] initWithFrame:rect textContainer:tc];
//                } else {
//                    textView = [[NSTextView alloc] initWithFrame:rect];
//
//                    // vertically resizable
//                    [textView setVerticallyResizable:YES];
//                }
//            }
            
            if ( tc ) {
                if ( IsSubclassFlag() ) {
                    textView = [[[TextViewSC alloc] initWithFrame:rect textContainer:tc] autorelease];
                } else {
                    textView = [[[NSTextView alloc] initWithFrame:rect textContainer:tc] autorelease];
                }
                [textView setAllowsUndo:YES];
                [textView setMaxSize:NSMakeSize(FLT_MAX, FLT_MAX)];
                [textView setTag:inTag];
                [textView setDelegate:CocoaTextViewDelegateGet()];
            } else {
                textView = TextViewInit( inTag, NSRectToCGRect(rect) );
            }
            
            // default to allows undo
//            [textView setAllowsUndo:YES];
            
            // max size
//            [textView setMaxSize:NSMakeSize(FLT_MAX, FLT_MAX)];
            
            // vertically resizable
            // [textView setVerticallyResizable:YES];
            
            // autoresizing
            [textView setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
            
//            // tag
//            [textView setTag:inTag];
//
//            // delegate
//            [textView setDelegate:CocoaTextViewDelegateGet()];
            
            // add to superview
            if ( scrollView ) {
                [scrollView setDocumentView:textView];
            } else {
                [contentView addSubview:textView];
            }
            
            // wndTag
            [textView setWndTag:inWndTag];
            
//            [textView release];
        }
        [textView setHidden:!show];
    } else {
        CocoaShowAlert( NSWarningAlertStyle, @"Window error.", [NSString stringWithFormat:@"Cocoa window %ld does not exist.",(long)inWndTag] );
    }
}

