/*
 Thread.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef void (*CocoaThreadCallbackType)(id);

@interface CocoaThreadTarget : NSObject {
    CocoaThreadCallbackType callback;
}
@property (assign) CocoaThreadCallbackType callback;
- (void)doThread:(id)object;
@end

// -- start --
void ThreadStart( ThreadRef thread );

// -- stop --
void ThreadSleepUntilDate( CFDateRef date );
void ThreadSleepForTimeInterval( CFTimeInterval ti );
void ThreadExit( void );
void ThreadCancel( ThreadRef thread );

// -- execution state --
Boolean ThreadIsExecuting( ThreadRef thread );
Boolean ThreadIsFinished( ThreadRef thread );
Boolean ThreadIsCancelled( ThreadRef thread );

// Main thread
Boolean ThreadIsMainThread( ThreadRef thread );
ThreadRef ThreadMainThread( void );

// Query
Boolean ThreadIsMultiThreaded( void );
ThreadRef ThreadCurrentThread( void );
CFArrayRef ThreadCallStackReturnAddresses( void );
CFArrayRef ThreadCallStackSymbols( void );

// -- properties --
CFMutableDictionaryRef ThreadDictionary( ThreadRef thread );
CFStringRef ThreadName( ThreadRef thread );
void ThreadSetName( ThreadRef thread, CFStringRef name );
NSUInteger ThreadStackSize( ThreadRef thread );

// -- prioritizing --
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSQualityOfService ThreadQualityOfService( ThreadRef thread );
void ThreadSetQualityOfService( ThreadRef thread, NSQualityOfService value );
#endif
double ThreadPriority( ThreadRef thread );
void ThreadSetPriority( ThreadRef thread, double priority );

// -- custom --
void ThreadDetachNewThreadFunction( void *fnAddress, CFTypeRef object );
ThreadRef ThreadWithFunction( void *fnAddress, CFTypeRef object );

