/*
 Thread.m

 Bernie Wylde
 */

#import "Thread.h"

@implementation CocoaThreadTarget

@synthesize callback;

- (void)doThread:(id)object {
    if ( callback ) (*callback)(object);
}
@end

// -- start --
void ThreadStart( ThreadRef thread )
{ [thread start]; }

// -- stop --
void ThreadSleepUntilDate( CFDateRef date )
{ [NSThread sleepUntilDate:(NSDate *)date]; }

void ThreadSleepForTimeInterval( CFTimeInterval ti )
{ [NSThread sleepForTimeInterval:ti]; }

void ThreadExit( void )
{ [NSThread exit]; }

void ThreadCancel( ThreadRef thread )
{ [thread cancel]; }

// -- execution state --
Boolean ThreadIsExecuting( ThreadRef thread )
{ return [thread isExecuting]; }

Boolean ThreadIsFinished( ThreadRef thread )
{ return [thread isFinished]; }

Boolean ThreadIsCancelled( ThreadRef thread )
{ return [thread isCancelled]; }

// Main thread
Boolean ThreadIsMainThread( ThreadRef thread )
{
    if ( !thread ) return [NSThread isMainThread];
    return [thread isMainThread];
}

ThreadRef ThreadMainThread( void )
{ return [NSThread mainThread]; }

// Query
Boolean ThreadIsMultiThreaded( void )
{ return [NSThread isMultiThreaded]; }

ThreadRef ThreadCurrentThread( void )
{ return [NSThread currentThread]; }

CFArrayRef ThreadCallStackReturnAddresses( void )
{ return (CFArrayRef)[NSThread callStackReturnAddresses]; }

CFArrayRef ThreadCallStackSymbols( void )
{ return (CFArrayRef)[NSThread callStackSymbols]; }

// -- properties --
CFMutableDictionaryRef ThreadDictionary( ThreadRef thread )
{ return (CFMutableDictionaryRef)[thread threadDictionary]; }

CFStringRef ThreadName( ThreadRef thread )
{ return (CFStringRef)[thread name]; }

void ThreadSetName( ThreadRef thread, CFStringRef name )
{ [thread setName:(NSString *)name]; }

NSUInteger ThreadStackSize( ThreadRef thread )
{ return [thread stackSize]; }

// -- prioritizing --
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSQualityOfService ThreadQualityOfService( ThreadRef thread )
{ return [thread qualityOfService]; }

void ThreadSetQualityOfService( ThreadRef thread, NSQualityOfService value )
{ [thread setQualityOfService:value]; }
#endif

double ThreadPriority( ThreadRef thread )
{
    if ( !thread ) return [NSThread threadPriority];
    return [thread threadPriority];
}

void ThreadSetPriority( ThreadRef thread, double priority )
{
    if ( !thread ) {
        [NSThread setThreadPriority:priority];
    } else {
        [thread setThreadPriority:priority];
    }
}

// -- custom --
void ThreadDetachNewThreadFunction( void *fnAddress, CFTypeRef object )
{
    CocoaThreadTarget *target = [[[CocoaThreadTarget alloc] init] autorelease];
    [target setCallback:fnAddress];
    [NSThread detachNewThreadSelector:@selector(doThread:) toTarget:target withObject:(id)object];
}

ThreadRef ThreadWithFunction( void *fnAddress, CFTypeRef object )
{
    CocoaThreadTarget *target = [[[CocoaThreadTarget alloc] init] autorelease];
    [target setCallback:fnAddress];
    NSThread *thread = [[[NSThread alloc] initWithTarget:target selector:@selector(doThread:) object:(id)object] autorelease];
    return thread;
}







