/*
 TimeZone.m

 Bernie Wylde
 */

#import "TimeZone.h"

// System time zones
CFTimeZoneRef TimeZoneLocal( void )
{ return (CFTimeZoneRef)[NSTimeZone localTimeZone]; }

CFTimeZoneRef TimeZoneSystem( void )
{ return (CFTimeZoneRef)[NSTimeZone systemTimeZone]; }

void TimeZoneResetSystem( void )
{ [NSTimeZone resetSystemTimeZone]; }

CFTimeZoneRef TimeZoneDefault( void )
{ return (CFTimeZoneRef)[NSTimeZone defaultTimeZone]; }

// Create
CFTimeZoneRef TimeZoneWithName( CFStringRef name )
{ return (CFTimeZoneRef)[NSTimeZone timeZoneWithName:(NSString *)name]; }

CFTimeZoneRef TimeZoneWithNameAndData( CFStringRef name, CFDataRef dta )
{ return (CFTimeZoneRef)[NSTimeZone timeZoneWithName:(NSString *)name data:(NSData *)dta]; }

CFTimeZoneRef TimeZoneWithAbbreviation( CFStringRef abbreviation )
{ return (CFTimeZoneRef)[NSTimeZone timeZoneWithAbbreviation:(NSString *)abbreviation]; }

CFTimeZoneRef TimeZoneForSecondsFromGMT( NSInteger seconds )
{ return (CFTimeZoneRef)[NSTimeZone timeZoneForSecondsFromGMT:seconds]; }

CFArrayRef TimeZoneKnownNames( void )
{ return (CFArrayRef)[NSTimeZone knownTimeZoneNames]; }

CFDictionaryRef TimeZoneAbbreviationDictionary( void )
{ return (CFDictionaryRef)[NSTimeZone abbreviationDictionary]; }

// Info
CFStringRef TimeZoneName( CFTimeZoneRef ref )
{ return (CFStringRef)[(NSTimeZone *)ref name]; }

CFStringRef TimeZoneAbbreviation( CFTimeZoneRef ref )
{ return (CFStringRef)[(NSTimeZone *)ref abbreviation]; }

CFStringRef TimeZoneAbbreviationForDate( CFTimeZoneRef ref, CFDateRef dt )
{ return (CFStringRef)[(NSTimeZone *)ref abbreviationForDate:(NSDate *)dt]; }

NSInteger TimeZoneSecondsFromGMT( CFTimeZoneRef ref )
{ return [(NSTimeZone *)ref secondsFromGMT]; }

NSInteger TimeZoneSecondsFromGMTForDate( CFTimeZoneRef ref, CFDateRef dt )
{ return [(NSTimeZone *)ref secondsFromGMTForDate:(NSDate *)dt]; }

CFDataRef TimeZoneData( CFTimeZoneRef ref )
{ return (CFDataRef)[(NSTimeZone *)ref data]; }

CFStringRef TimeZoneDataVersion( void )
{ return (CFStringRef)[NSTimeZone timeZoneDataVersion]; }

// Daylight savings
Boolean TimeZoneIsDaylightSavingTime( CFTimeZoneRef ref )
{ return [(NSTimeZone *)ref isDaylightSavingTime]; }

Boolean TimeZoneIsDaylightSavingTimeForDate( CFTimeZoneRef ref, CFDateRef dt )
{ return [(NSTimeZone *)ref isDaylightSavingTimeForDate:(NSDate *)dt]; }

CFTimeInterval TimeZoneDaylightSavingTimeOffset( CFTimeZoneRef ref )
{ return [(NSTimeZone *)ref daylightSavingTimeOffset]; }

CFTimeInterval TimeZoneDaylightSavingTimeOffsetForDate( CFTimeZoneRef ref, CFDateRef dt )
{ return [(NSTimeZone *)ref daylightSavingTimeOffsetForDate:(NSDate *)dt]; }

CFDateRef TimeZoneNextDaylightSavingTimeTransition( CFTimeZoneRef ref )
{ return (CFDateRef)[(NSTimeZone *)ref nextDaylightSavingTimeTransition]; }

CFDateRef TimeZoneNextDaylightSavingTimeTransitionAfterDate( CFTimeZoneRef ref, CFDateRef dt )
{ return (CFDateRef)[(NSTimeZone *)ref nextDaylightSavingTimeTransitionAfterDate:(NSDate *)dt]; }

// Compare
Boolean TimeZoneIsEqual( CFTimeZoneRef ref, CFTimeZoneRef otherZone )
{ return [(NSTimeZone *)ref isEqualToTimeZone:(NSTimeZone *)otherZone]; }

// Describe
CFStringRef TimeZoneLocalizedName( CFTimeZoneRef ref, NSTimeZoneNameStyle nameStyle, CFLocaleRef locale )
{ return (CFStringRef)[(NSTimeZone *)ref localizedName:nameStyle locale:(NSLocale *)locale]; }

CFStringRef TimeZoneDescription( CFTimeZoneRef ref )
{ return (CFStringRef)[(NSTimeZone *)ref description]; }

