/*
 Timer.m

 Bernie Wylde
 */

#import "Timer.h"

@implementation CocoaTimerTarget

@synthesize callback;

- (id)initWithCallback:(CocoaTimerCallbackType)cb {
    if ( self = [super init] ) {
        callback = cb;
    }
    return self;
}

- (void)timerFireMethod:(NSTimer *)timer {
    if ( callback ) (*callback)( (CFRunLoopTimerRef)timer, (CFTypeRef)[timer userInfo] );
}
@end

// - create -
CFRunLoopTimerRef TimerScheduledWithInterval( CFTimeInterval ti, CocoaTimerCallbackType callback, CFTypeRef userInfo, Boolean repeats )
{
    CocoaTimerTarget *target = [[[CocoaTimerTarget alloc] initWithCallback:callback] autorelease];
    return (CFRunLoopTimerRef)[NSTimer scheduledTimerWithTimeInterval:ti target:target selector:@selector(timerFireMethod:) userInfo:(id)userInfo repeats:repeats];
}

CFRunLoopTimerRef TimerWithInterval( CFTimeInterval ti, CocoaTimerCallbackType callback, CFTypeRef userInfo, Boolean repeats )
{
    CocoaTimerTarget *target = [[[CocoaTimerTarget alloc] initWithCallback:callback] autorelease];
    NSTimer *timer = [NSTimer timerWithTimeInterval:ti target:target selector:@selector(timerFireMethod:) userInfo:(id)userInfo repeats:repeats];
    [[NSRunLoop currentRunLoop] addTimer:timer forMode:NSDefaultRunLoopMode];
    return (CFRunLoopTimerRef)timer;
}

CFRunLoopTimerRef TimerWithFireDate( CFDateRef dt, CFTimeInterval ti, CocoaTimerCallbackType callback, CFTypeRef userInfo, Boolean repeats )
{
    CocoaTimerTarget *target = [[[CocoaTimerTarget alloc] initWithCallback:callback] autorelease];
    NSTimer *timer = [[[NSTimer alloc] initWithFireDate:(NSDate *)dt interval:ti target:target selector:@selector(timerFireMethod:) userInfo:(id)userInfo repeats:repeats] autorelease];
    [[NSRunLoop currentRunLoop] addTimer:timer forMode:NSDefaultRunLoopMode];
    return (CFRunLoopTimerRef)timer;
}

// - fire -
void TimerFire( CFRunLoopTimerRef t )
{ [(NSTimer *)t fire]; }

// - stop -
void TimerInvalidate( CFRunLoopTimerRef t )
{ [(NSTimer *)t invalidate]; }

// - info -
Boolean TimerIsValid( CFRunLoopTimerRef t )
{ return [(NSTimer *)t isValid]; }

CFDateRef TimerFireDate( CFRunLoopTimerRef t )
{ return (CFDateRef)[(NSTimer *)t fireDate]; }

CFTimeInterval TimerTimeInterval( CFRunLoopTimerRef t )
{ return [(NSTimer *)t timeInterval]; }

CFTypeRef TimerUserInfo( CFRunLoopTimerRef t )
{ return (CFTypeRef)[(NSTimer *)t userInfo]; }

// - tolerance -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
CFTimeInterval TimerTolerance( CFRunLoopTimerRef t )
{ return [(NSTimer *)t tolerance]; }
#endif
