/*
 TokenField.m
 
 Bernie Wylde
 */

#import "TokenField.h"

#pragma mark - subclass
@implementation TokenFieldSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end


@implementation CocoaTokenFieldDelegate

@synthesize boolVal;

- (void)tokenField:(NSTokenField *)fld forwardEvent:(NSInteger)ev object:(id)obj {
    CocoaOutputWindowSetTag( [fld wndTag] );
    CallUserDefinedOnDialogFunction( ev, [fld tag], [fld wndTag], obj );
}

- (NSString *)tokenField:(NSTokenField *)tokenField displayStringForRepresentedObject:(id)representedObject {
    DialogEventSetNSString( nil );
    [self tokenField:tokenField forwardEvent:tokenFieldDisplayStringForRepresentedObject object:representedObject];
    return DialogEventNSString();
}

- (NSTokenStyle)tokenField:(NSTokenField *)tokenField styleForRepresentedObject:(id)representedObject {
    DialogEventSetLong( 0 );
    [self tokenField:tokenField forwardEvent:tokenFieldStyleForRepresentedObject object:representedObject];
    return DialogEventLong();
}

- (NSArray *)tokenField:(NSTokenField *)tokenField completionsForSubstring:(NSString *)substring indexOfToken:(NSInteger)tokenIndex indexOfSelectedItem:(NSInteger *)selectedIndex {
    DialogEventSetNSArray(nil);
    DialogEventSetLong(tokenIndex);
    
    // should set dialog event selectedIndex value here
    
    [self tokenField:tokenField forwardEvent:tokenFieldCompletionsForSubstring object:substring];
    return DialogEventNSArray();
}

- (NSString *)tokenField:(NSTokenField *)tokenField editingStringForRepresentedObject:(id)representedObject {
    DialogEventSetNSString( nil );
    [self tokenField:tokenField forwardEvent:tokenFieldEditingStringForRepresentedObject object:representedObject];
    return DialogEventNSString();
}

- (id)tokenField:(NSTokenField *)tokenField representedObjectForEditingString:(NSString *)editingString {
    DialogEventSetNSObject( nil );
    [self tokenField:tokenField forwardEvent:tokenFieldRepresentedObjectForEditingString object:editingString];
    return DialogEventNSObject();
}

- (NSArray *)tokenField:(NSTokenField *)tokenField shouldAddObjects:(NSArray *)tokens atIndex:(NSUInteger)index {
    DialogEventSetLong( index );
    DialogEventSetNSArray( nil );
    [self tokenField:tokenField forwardEvent:tokenFieldShouldAddObjects object:tokens];
    return DialogEventNSArray();
}

- (NSArray *)tokenField:(NSTokenField *)tokenField readFromPasteboard:(NSPasteboard *)pboard {
    DialogEventSetNSArray( nil );
    [self tokenField:tokenField forwardEvent:tokenFieldReadFromPasteboard object:pboard];
    return DialogEventNSArray();
}

- (BOOL)tokenField:(NSTokenField *)tokenField writeRepresentedObjects:(NSArray *)objects toPasteboard:(NSPasteboard *)pboard {
    DialogEventSetBool(NO);
    DialogEventSetNSArray( objects );
    [self tokenField:tokenField forwardEvent:tokenFieldWriteRepresentedObjectToPasteboard object:pboard];
    return DialogEventBool();
}

- (BOOL)tokenField:(NSTokenField *)tokenField hasMenuForRepresentedObject:(id)representedObject {
    DialogEventSetBool(NO);
    [self tokenField:tokenField forwardEvent:tokenFieldHasMenuForRepresentedObject object:representedObject];
    return DialogEventBool();
}

- (NSMenu *)tokenField:(NSTokenField *)tokenField menuForRepresentedObject:(id)representedObject {
    DialogEventSetNSMenu(nil);
    [self tokenField:tokenField forwardEvent:tokenFieldMenuForRepresentedObject object:representedObject];
    return DialogEventNSMenu();
}

#pragma mark - control text delegate methods
- (void)controlTextDidBeginEditing:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldDidBeginEditing, notification.object, NULL );
}

- (void)controlTextDidChange:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldDidChange, notification.object, NULL );
}

- (void)controlTextDidEndEditing:(NSNotification *)notification {
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldDidEndEditing, notification.object, NULL );
}

- (BOOL)control:(NSControl *)control textShouldBeginEditing:(NSText *)fieldEditor {
    DialogEventSetBool(YES);
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldShouldBeginEditing, control, fieldEditor );
    return DialogEventBool();
}

- (BOOL)control:(NSControl *)control textShouldEndEditing:(NSText *)fieldEditor {
    DialogEventSetBool(YES);
    CocoaControlCallUserDefinedOnDialogOrCallback( textFieldShouldEndEditing, control, fieldEditor );
    return DialogEventBool();
}
@end

CocoaTokenFieldDelegate *CocoaTokenFieldDelegateGet( void )
{
    static CocoaTokenFieldDelegate *sCocoaTokenFieldDelegate = nil;
    if ( !sCocoaTokenFieldDelegate ) sCocoaTokenFieldDelegate = [[CocoaTokenFieldDelegate alloc] init];
    return sCocoaTokenFieldDelegate;
}

NSTokenField *TokenFieldWithTag( NSInteger tag )
{ return (NSTokenField *)CocoaValidateViewKindOfClass( tag, [NSTokenField class] ); }

BOOL TokenFieldExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSTokenField class] ); }


// Style
NSUInteger TokenFieldStyle( NSInteger tag )
{
    NSTokenField *fld = TokenFieldWithTag( tag );
    if ( fld ) return [fld tokenStyle];
    return 0;
}

void TokenFieldSetStyle( NSInteger tag, NSUInteger style )
{
    NSTokenField *fld = TokenFieldWithTag( tag );
    if ( fld ) [fld setTokenStyle:style];
}

// Character set
CFCharacterSetRef TokenFieldTokenizingCharacterSet( NSInteger tag )
{
    NSTokenField *fld = TokenFieldWithTag( tag );
    if ( fld ) return (CFCharacterSetRef)[fld tokenizingCharacterSet];
    return nil;
}

void TokenFieldSetTokenizingCharacterSet( NSInteger tag, CFCharacterSetRef set )
{
    NSTokenField *fld = TokenFieldWithTag( tag );
    if ( fld ) [fld setTokenizingCharacterSet:(NSCharacterSet *)set];
}

CFCharacterSetRef TokenFieldDefaultTokenizingCharacterSet( void )
{ return (CFCharacterSetRef)[NSTokenField defaultTokenizingCharacterSet]; }

// CompletionDelay
CFTimeInterval TokenFieldCompletionDelay( NSInteger tag )
{
    NSTokenField *fld = TokenFieldWithTag( tag );
    if ( fld ) return [fld completionDelay];
    return 0.0;
}

void TokenFieldSetCompletionDelay( NSInteger tag, CFTimeInterval ti )
{
    NSTokenField *fld = TokenFieldWithTag( tag );
    if ( fld ) [fld setCompletionDelay:ti];
}

CFTimeInterval TokenFieldDefaultCompletionDelay( void )
{ return [NSTokenField defaultCompletionDelay]; }

// Custom
CFArrayRef TokenFieldRangesOfSelections( NSInteger tag )
{
    NSTokenField *fld = TokenFieldWithTag( tag );
    if ( fld ) {
        NSTextView *textView = [[fld cell] fieldEditorForView:fld];
        return (CFArrayRef)[textView selectedRanges];
    }
    return nil;
}


// Init
TokenFieldRef TokenFieldInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 96, 22 );
    
    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    NSTokenField *fld;
    if ( subclass ) {
        fld = [[[TokenFieldSC alloc] initWithFrame:rect] autorelease];
    } else {
        fld = [[[NSTokenField alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [fld setTag: inTag];
    
    // fix font to match I.B.
    CocoaControlFixFont( fld );
    
    // delegate
    [fld setDelegate:CocoaTokenFieldDelegateGet()];
    
    return fld;
}

// Statement
void CocoaTokenField( NSInteger inTag, NSInteger inEnabled, CFStringRef inText, CGRect inRect, NSInteger inStyle, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSString *text = (NSString *)inText;
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSTokenField *fld = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !fld ) {
            fld = TitlebarAccessoryViewWithTag( inTag );
        }
        
        
        if ( fld ) {    // already exists
            
            // make sure it's same view class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [fld class], [NSTokenField class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, fld, [NSTokenField class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [fld setEnabled:inEnabled];
            
            // text
            if ( text ) [fld setStringValue:text];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [fld setFrame:rect];
            
            // style
            if ( inStyle != kFBParamMissing ) [fld setTokenStyle:inStyle];
            
        } else {        // create
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 96, 22 );
            
            // init
//            if ( inSubclass ) {
//                fld = [[TokenFieldSC alloc] initWithFrame:rect];
//            } else {
//                fld = [[NSTokenField alloc] initWithFrame:rect];
//            }
            fld = TokenFieldInit( inTag, inRect );
            
            // default autoresizing
            [fld setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [fld setTag: inTag];
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [fld setEnabled:inEnabled];
            
            // text
            if ( text ) [fld setStringValue:text];
            
            // style
            if ( inStyle != kFBParamMissing ) [fld setTokenStyle:inStyle];
            
            // wndTag
            [fld setWndTag:inWndTag];
            
            // fix font to match I.B.
            CocoaControlFixFont( fld );
            
            // delegate
//            [fld setDelegate:CocoaTokenFieldDelegateGet()];
            
            // add to window
            [contentView addSubview:fld];
//            [fld release];
        }
        
        [fld setHidden:!show];
    } else {
        CocoaShowWindowDoesNotExistAlert( inWndTag );
    }
}

