/*
 Toolbar.m

 Bernie Wylde
 */

#import "Toolbar.h"

@implementation ToolbarSC
@end

@implementation CocoaToolbarItemTarget
- (void)toolbarItemAction:(id)sender {
    CocoaOutputWindowSetTag( AppKeyWindow() );// 20171216
    CallUserDefinedOnDialogFunction( toolbarItemClick, [sender tag], CocoaOutputWindowTag(), NULL );
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)item {
    CocoaOutputWindowSetTag( AppKeyWindow() );
    DialogEventSetBool( [item isEnabled] );
    CallUserDefinedOnDialogFunction( toolbarItemValidate, [item tag], CocoaOutputWindowTag(), NULL );
    return DialogEventBool();
}

//- (BOOL)validateToolbarItem:(NSToolbarItem *)item {
//    return [item isEnabled];
//}
@end

// private
CocoaToolbarItemTarget *CocoaToolbarItemTargetGet( void )
{
    static CocoaToolbarItemTarget *sCocoaToolbarItemTarget = nil;
    if ( !sCocoaToolbarItemTarget ) sCocoaToolbarItemTarget = [[CocoaToolbarItemTarget alloc] init];
    return sCocoaToolbarItemTarget;
}


@implementation CocoaToolbarDelegate

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar *)toolbar {
    NSInteger tag = [[toolbar identifier] integerValue];
    CocoaToolbarObj *obj = CocoaToolbarObjWithTag( tag );
    if ( obj ) {
        return [obj allowedIdentifiers];
    }
    return nil;
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar *)toolbar {
    NSInteger tag = [[toolbar identifier] integerValue];
    CocoaToolbarObj *obj = CocoaToolbarObjWithTag( tag );
    if ( obj ) {
        return [obj defaultIdentifiers];
    }
    return nil;
}

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag {
    NSToolbarItem *item = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
    NSInteger tag = [[toolbar identifier] integerValue];
    CocoaToolbarObj *obj = CocoaToolbarObjWithTag( tag );
    if ( obj ) {
        NSDictionary *itemDict = [[obj items] objectForKey:itemIdentifier];
        if ( itemDict ) {
            
            NSInteger itemTag = [[itemDict objectForKey:@"Tag"] integerValue];
            NSString *label = [itemDict objectForKey:@"Label"];
            NSImage *image = [itemDict objectForKey:@"Image"];
            
            [item setTag:itemTag];
            if ( label ) [item setLabel:label];
            
            if ( image ) [item setImage:image];
            
            // target
            [item setTarget:CocoaToolbarItemTargetGet()];
            
            // action
            [item setAction:@selector(toolbarItemAction:)];
        }
    }
    return item;
}

@end

CocoaToolbarDelegate *CocoaToolbarDelegateGet( void )
{
    static CocoaToolbarDelegate *sCocoaToolbarDelegate = nil;
    if ( !sCocoaToolbarDelegate ) sCocoaToolbarDelegate = [[CocoaToolbarDelegate alloc] init];
    return sCocoaToolbarDelegate;
}

static NSMutableArray *sCocoaToolbars;

@implementation CocoaToolbarObj

@synthesize toolbar;
@synthesize items;
@synthesize allowedIdentifiers;
@synthesize defaultIdentifiers;

- (id)initWithToolbar:(NSToolbar *)tb {
    if ( self = [super init] ) {
        self.toolbar = tb;
        self.items = [[NSMutableDictionary alloc] initWithCapacity:0];
        self.allowedIdentifiers = [[NSMutableArray alloc] initWithCapacity:0];
        self.defaultIdentifiers = [[NSMutableArray alloc] initWithCapacity:0];
    }
    return self;
}
@end

CocoaToolbarObj *CocoaToolbarObjWithTag( NSInteger tag )
{
    if ( sCocoaToolbars ) {
        NSString *identifier = [NSString stringWithFormat:@"%ld",(long)tag];
        for ( CocoaToolbarObj *obj in sCocoaToolbars ) {
            NSToolbar *toolbar = [obj toolbar];
            if ( [[toolbar identifier] isEqualToString:identifier] ) return obj;
        }
    }
    return nil;
}

void CocoaToolbarObjAdd( NSToolbar *tb )
{
    if ( !sCocoaToolbars ) sCocoaToolbars = [[NSMutableArray alloc] initWithCapacity:0];
    CocoaToolbarObj *obj = [[CocoaToolbarObj alloc] initWithToolbar:tb];
    [sCocoaToolbars addObject:obj];
    //[obj release];
}

void CocoaToolbarObjAddItem( CocoaToolbarObj *obj, NSString *identifier, NSInteger tag, NSString *label, NSImage *image, Boolean allowed, Boolean dflt )
{
    NSDictionary *itemDict = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInteger:tag], @"Tag", label, @"Label", image, @"Image", nil];
    [[obj items] setObject:itemDict forKey:identifier];
    if ( allowed ) [[obj allowedIdentifiers] addObject:identifier];
    if ( dflt ) [[obj defaultIdentifiers] addObject:identifier];
}

NSInteger CocoaToolbarTag( NSToolbar *tb )
{
    if ( sCocoaToolbars ) {
        for ( CocoaToolbarObj *obj in sCocoaToolbars ) {
            NSToolbar *toolbar = [obj toolbar];
            if ( toolbar == tb ) {
                return [[toolbar identifier] integerValue];
            }
        }
    }
    return 0;
}

NSInteger CocoaToolbarLastTag( void )
{
    if ( sCocoaToolbars ) {
        CocoaToolbarObj *obj = [sCocoaToolbars lastObject];
        if ( obj ) {
            return [[[obj toolbar] identifier] integerValue];
        }
    }
    return 0;
}

NSToolbar *CocoaToolbarWithTag( NSInteger tag )
{
    if ( tag ) {
        if ( tag > 1000000 ) return (NSToolbar *)tag;// it's a ToolbarRef

        CocoaToolbarObj *obj = CocoaToolbarObjWithTag( tag );
        if ( obj ) return [obj toolbar];
    }
    return nil;
}

NSToolbar *ToolbarWithTag( NSInteger tag )
{ return CocoaToolbarWithTag( tag ); }

BOOL ToolbarExists( NSInteger tag )
{ return ( CocoaToolbarWithTag( tag ) != nil ); }


//void ToolbarInsertItem( NSInteger toolbarTag, NSInteger itemTag, CFStringRef inIdentifier, CFStringRef inLabel, CFStringRef inImageName, Boolean allowed, Boolean dflt )
//{
//    CocoaToolbarObj *obj = CocoaToolbarObjWithTag( toolbarTag );
//    if ( obj ) {
//        NSToolbar *toolbar = [obj toolbar];
//        if ( toolbar ) {
//            if ( inIdentifier ) {
//                NSString *identifier = (NSString *)inIdentifier;
//                NSString *label = (NSString *)inLabel;
//                NSString *imageName = (NSString *)inImageName;
//
//                [toolbar insertItemWithItemIdentifier:identifier atIndex:0];
//
//                CocoaToolbarObjAddItem( obj, identifier, itemTag, label, imageName, allowed, dflt );
//            }
//        }
//    }
//}


// Attributes
NSToolbarDisplayMode ToolbarDisplayMode( NSInteger tag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) return [toolbar displayMode];
    return 0;
}

void ToolbarSetDisplayMode( NSInteger tag, NSToolbarDisplayMode mode )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) [toolbar setDisplayMode:mode];
}

Boolean ToolbarShowsBaselineSeparator( NSInteger tag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) return [toolbar showsBaselineSeparator];
    return NO;
}

void ToolbarSetShowsBaselineSeparator( NSInteger tag, Boolean flag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) [toolbar setShowsBaselineSeparator:flag];
}

Boolean ToolbarAllowsUserCustomization( NSInteger tag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) return [toolbar allowsUserCustomization];
    return NO;
}

void ToolbarSetAllowsUserCustomization( NSInteger tag, Boolean flag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) [toolbar setAllowsUserCustomization:flag];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean ToolbarAllowsExtensionItems( NSInteger tag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) return [toolbar allowsExtensionItems];
    return NO;
}

void ToolbarSetAllowsExtensionItems( NSInteger tag, Boolean flag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) [toolbar setAllowsExtensionItems:flag];
}
#endif

NSToolbarSizeMode ToolbarSizeMode( NSInteger tag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) return [toolbar sizeMode];
    return 0;
}

void ToolbarSetSizeMode( NSInteger tag, NSToolbarSizeMode size )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) {
        [toolbar setSizeMode:size];
    }
}

// Display
Boolean ToolbarIsVisible( NSInteger tag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) return [toolbar isVisible];
    return NO;
}

void ToolbarSetVisible( NSInteger tag, Boolean flag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) [toolbar setVisible:flag];
}

// Customization
void ToolbarRunCustomizationPalette( NSInteger tag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) [toolbar runCustomizationPalette:nil];
}

Boolean ToolbarCustomizationPaletteIsRunning( NSInteger tag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) return [toolbar customizationPaletteIsRunning];
    return NO;
}

// Autosaving
Boolean ToolbarAutosavesConfiguration( NSInteger tag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) return [toolbar autosavesConfiguration];
    return NO;
}

void ToolbarSetAutosavesConfiguration( NSInteger tag, Boolean flag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) [toolbar setAutosavesConfiguration:flag];
}

CFDictionaryRef ToolbarConfigurationDictionary( NSInteger tag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) return (CFDictionaryRef)[toolbar configurationDictionary];
    return nil;
}

void ToolbarSetConfigurationFromDictionary( NSInteger tag, CFDictionaryRef dict )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( tag );
    if ( toolbar ) [toolbar setConfigurationFromDictionary:(NSDictionary *)dict];
}


// Init
ToolbarRef ToolbarInit( NSInteger inTag, CFStringRef identifier )
{
    // subclass
    BOOL subclass = IsSubclassFlag();

    NSToolbar *toolbar;
    if ( subclass ) {
        toolbar = [[[ToolbarSC alloc] initWithIdentifier:(NSString *)identifier] autorelease];
    } else {
        toolbar = [[[NSToolbar alloc] initWithIdentifier:(NSString *)identifier] autorelease];
    }

    // delegate
    [toolbar setDelegate:CocoaToolbarDelegateGet()];
    
    // add to toolbars array
    CocoaToolbarObjAdd( toolbar );
    
    return toolbar;
}


// Statement
void CocoaToolbar( NSInteger inTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSString *identifier = [NSString stringWithFormat:@"%ld",(long)inTag];
    
    NSToolbar *toolbar = CocoaToolbarWithTag( inTag );
    
    if ( toolbar ) {    // exists
        
        // ... don't think there's anything we can do here
        
    } else {        // create
        
        // init
//        if ( inSubclass ) {
//            toolbar = [[ToolbarSC alloc] initWithIdentifier:identifier];
//        } else {
//            toolbar = [[NSToolbar alloc] initWithIdentifier:identifier];
//        }
        toolbar = ToolbarInit( inTag, (CFStringRef)identifier );
        
        // delegate
 //       [toolbar setDelegate:CocoaToolbarDelegateGet()];
        
        // add to toolbars array
//        CocoaToolbarObjAdd( toolbar );
    }
    
    [toolbar setVisible:show];
    
    // workaround for cosmetic in Carbon
    if ( !IsCocoaApplication() ) {
        CocoaWindowCarbonRuntimeCosmeticFix( CocoaOutputWindow() );
    }
}

