/*
 ToolbarItem.m

 Bernie Wylde
 */

#import "ToolbarItem.h"

@implementation ToolbarItemSC
@end

NSToolbarItem *ToolbarItemWithTag( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbar *toolbar = CocoaToolbarWithTag( toolbarTag );
    if ( toolbar ) {
        NSArray *items = [toolbar items];
        for ( NSToolbarItem *item in items ) {
            if ( [item tag] == itemTag ) {
                return item;
            }
        }
    }
    return nil;
}

//BOOL ToolbarItemExists( NSInteger toolbarTag, NSInteger itemTag )
//{
//    NSToolbar *toolbar = CocoaToolbarWithTag( toolbarTag );
//    if ( toolbar ) {
//        NSArray *items = [toolbar items];
//        for ( NSToolbarItem *item in items ) {
//            if ( [item tag] == itemTag ) {
//                return YES;
//            }
//        }
//    }
//    return NO;
//}


// Attributes
CFStringRef ToolbarItemIdentifier( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) return (CFStringRef)[item itemIdentifier];
    return nil;
}

CFStringRef ToolbarItemLabel( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) return (CFStringRef)[item label];
    return nil;
}

//void ToolbarItemSetLabel( NSInteger toolbarTag, NSInteger itemTag, CFStringRef label )
//{
//    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
//    if ( item ) [item setLabel:(NSString *)label];
//}

CFStringRef ToolbarItemPaletteLabel( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) return (CFStringRef)[item paletteLabel];
    return nil;
}

void ToolbarItemSetPaletteLabel( NSInteger toolbarTag, NSInteger itemTag, CFStringRef label )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) [item setPaletteLabel:(NSString *)label];
}

CFStringRef ToolbarItemToolTip( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) return (CFStringRef)[item toolTip];
    return nil;
}

void ToolbarItemSetToolTip( NSInteger toolbarTag, NSInteger itemTag, CFStringRef toolTip )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) [item setToolTip:(NSString *)toolTip];
}

MenuItemRef ToolbarItemMenuFormRepresentation( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) return [item menuFormRepresentation];
    return nil;
}

void ToolbarItemSetMenuFormRepresentation( NSInteger toolbarTag, NSInteger itemTag, MenuItemRef menuItem )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) [item setMenuFormRepresentation:menuItem];
}

Boolean ToolbarItemIsEnabled( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) return [item isEnabled];
    return NO;
}

void ToolbarItemSetEnabled( NSInteger toolbarTag, NSInteger itemTag, Boolean flag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) [item setEnabled:flag];
}

ImageRef ToolbarItemImage( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) return [item image];
    return nil;
}

//void ToolbarItemSetImage( NSInteger toolbarTag, NSInteger itemTag, ImageRef image )
//{
//    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
//    if ( item ) [item setImage:image];
//}

ViewRef ToolbarItemView( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) return [item view];
    return nil;
}

void ToolbarItemSetView( NSInteger toolbarTag, NSInteger itemTag, NSInteger vwTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) {
        NSView *view = CocoaViewWithTag( vwTag );
        if ( view ) [item setView:view];
    }
}

CGSize ToolbarItemMinSize( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) NSSizeToCGSize([item minSize]);
    return CGSizeZero;
}

void ToolbarItemSetMinSize( NSInteger toolbarTag, NSInteger itemTag, CGSize size )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) [item setMinSize:NSSizeFromCGSize( size )];
}

CGSize ToolbarItemMaxSize( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) NSSizeToCGSize([item maxSize]);
    return CGSizeZero;
}

void ToolbarItemSetMaxSize( NSInteger toolbarTag, NSInteger itemTag, CGSize size )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) [item setMaxSize:NSSizeFromCGSize( size )];
}

// VisibilityPriority
NSInteger ToolbarItemVisibilityPriority( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) return [item visibilityPriority];
    return 0;
}

void ToolbarItemSetVisibilityPriority( NSInteger toolbarTag, NSInteger itemTag, NSInteger priority )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) [item setVisibilityPriority:priority];
}

// Validation
void ToolbarItemValidate( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) [item validate];
}

Boolean ToolbarItemAutovalidates( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) return [item autovalidates];
    return NO;
}

void ToolbarItemSetAutovalidates( NSInteger toolbarTag, NSInteger itemTag, Boolean flag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) [item setAutovalidates:flag];
}

// Duplicates
Boolean ToolbarItemAllowsDuplicatesInToolbar( NSInteger toolbarTag, NSInteger itemTag )
{
    NSToolbarItem *item = ToolbarItemWithTag( toolbarTag, itemTag );
    if ( item ) return [item allowsDuplicatesInToolbar];
    return NO;
}


// Init
//ToolbarItemRef ToolbarItemInit( NSInteger inTags, CFStringRef inIdentifier )
//{
//    // subclass
//    BOOL subclass = IsSubclassFlag();
//
//    // init
//    NSToolbarItem *item;
//    if ( subclass ) {
//        item = [[[ToolbarItemSC alloc] init] autorelease];
//    } else {
//        item = [[[NSToolbarItem alloc] init] autorelease];
//    }
//    
//    return item;
//}


// Statement
void CocoaToolbarItem( NSInteger inTag, NSString *inIdentifier, CFStringRef inLabel, void *inImage, NSInteger inAllowed, NSInteger inDefault, NSInteger inToolbarTag )
{
    // get target toolbar
    if ( inToolbarTag == 0 ) inToolbarTag = CocoaToolbarLastTag();
    
    if ( inToolbarTag ) {
        CocoaToolbarObj *obj = CocoaToolbarObjWithTag( inToolbarTag );
        if ( obj ) {
            
            // subclass
            IsSubclassFlag();// clear the subclass flag

            NSToolbar *toolbar = [obj toolbar];
            
            NSString *identifier = (NSString *)inIdentifier;
            NSString *label = (NSString *)inLabel;
            
            // imageName, imagePath, CGImage, NSImage
            NSImage *image = FixImageValue( inImage );
            
            BOOL allowed = YES;
            BOOL dflt = YES;
            if ( inAllowed != kFBParamMissing ) allowed = inAllowed;
            if ( inDefault != kFBParamMissing ) dflt = inDefault;
            
            // insert item
            [toolbar insertItemWithItemIdentifier:identifier atIndex:0];
            
            CocoaToolbarObjAddItem( obj, identifier, inTag, label, image, allowed, dflt );
        }
    } else {
        // show warning - can't find toolbar
    }
}

