/*
 URLSession.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
typedef void (*URLSessionDataTaskCompletionHandlerType)(NSURLSession*,CFDataRef,NSURLResponse*,NSError*,void*);
typedef void (*URLSessionDownloadTaskCompletionHandlerType)(NSURLSession*,CFURLRef,NSURLResponse*,NSError*,void*);
typedef void (*URLSessionUploadTaskCompletionHandlerType)(NSURLSession*,CFDataRef,NSURLResponse*,NSError*,void*);
typedef void (*URLSessionFlushCompletionHandlerType)(NSURLSession*,void*);
typedef void (*URLSessionGetTasksCompletionHandlerType)(NSURLSession*,CFArrayRef,CFArrayRef,CFArrayRef,void*);
typedef void (*URLSessionGetAllTasksCompletionHandlerType)(NSURLSession*,CFArrayRef,void*);
typedef void (*URLSessionResetCompletionHandlerType)(NSURLSession*,void*);

// dialog events
enum {
    // NSURLSessionDelegate
    URLSessionDidBecomeInvalid = 3600,
    URLSessionDidFinishEvents,
    URLSessionDidReceiveChallenge,
    
    // NSURLSessionDownloadDelegate
    URLSessionDownloadTaskDidFinishDownloading,
    URLSessionDownloadTaskDidResume,
    URLSessionDownloadTaskDidWriteData,

    // NSURLSessionTaskDelegate
    URLSessionTaskDidComplete,
    URLSessionTaskWillPerformHTTPRedirection,
    URLSessionTaskDidSendBodyData,
    URLSessionTaskNeedNewBodyStream,
    URLSessionTaskDidReceiveChallenge,
    URLSessionTaskWillBeginDelayedRequest,
    URLSessionTaskIsWaitingForConnectivity,
    URLSessionTaskDidFinishCollectingMetrics,
    
    // NSURLSessionDataDelegate
    URLSessionDataTaskDidReceiveResponse,
    URLSessionDataTaskDidBecomeDownloadTask,
    URLSessionDataTaskDidBecomeStreamTask,
    URLSessionDataTaskDidReceiveData,
    URLSessionDataTaskWillCacheResponse,
    
    // NSURLSessionStreamDelegate
    URLSessionStreamTaskBetterRouteDiscovered,// macOS 10.11+
    URLSessionStreamTaskDidBecomeInputStream,// macOS 10.11+
    URLSessionStreamTaskReadClosed,// macOS 10.11+
    URLSessionStreamTaskWriteClosed// macOS 10.11+
};

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
@interface CocoaURLSessionDelegate : NSObject <NSURLSessionDelegate,NSURLSessionDownloadDelegate,NSURLSessionTaskDelegate,NSURLSessionDataDelegate,NSURLSessionStreamDelegate>
#else
@interface CocoaURLSessionDelegate : NSObject <NSURLSessionDelegate,NSURLSessionDownloadDelegate,NSURLSessionTaskDelegate,NSURLSessionDataDelegate>
#endif
@end

// - Private -
CocoaURLSessionDelegate *CocoaURLSessionDelegateObj( void );

// - Public -
// Create
URLSessionRef URLSessionWithConfiguration( URLSessionConfigurationRef config );
URLSessionRef URLSessionWithDelegateQueue( URLSessionConfigurationRef config, OperationQueueRef queue );
URLSessionRef URLSessionSharedSession( void );

// Configure
URLSessionConfigurationRef URLSessionConfiguration( URLSessionRef ref );
OperationQueueRef URLSessionDelegateQueue( URLSessionRef ref );
CFStringRef URLSessionDescription( URLSessionRef ref );

// Add data tasks
URLSessionDataTaskRef URLSessionDataTaskWithURL( URLSessionRef ref, CFURLRef url );
URLSessionDataTaskRef URLSessionDataTaskWithURLCompletionHandler( URLSessionRef ref, CFURLRef url, URLSessionDataTaskCompletionHandlerType handlerAddress, void *userData );
URLSessionDataTaskRef URLSessionDataTaskWithRequest( URLSessionRef ref, URLRequestRef request );
URLSessionDataTaskRef URLSessionDataTaskWithRequestCompletionHandler( URLSessionRef ref, URLRequestRef request, URLSessionDataTaskCompletionHandlerType handlerAddress, void *userData );

// Add download tasks
URLSessionDownloadTaskRef URLSessionDownloadTaskWithURL( URLSessionRef ref, CFURLRef url );
URLSessionDownloadTaskRef URLSessionDownloadTaskWithURLCompletionHandler( URLSessionRef ref, CFURLRef url, URLSessionDownloadTaskCompletionHandlerType handlerAddress, void *userData );
URLSessionDownloadTaskRef URLSessionDownloadTaskWithRequest( URLSessionRef ref, URLRequestRef request );
URLSessionDownloadTaskRef URLSessionDownloadTaskWithRequestCompletionHandler( URLSessionRef ref, URLRequestRef request, URLSessionDownloadTaskCompletionHandlerType handlerAddress, void *userData );
URLSessionDownloadTaskRef URLSessionDownloadTaskWithResumeData( URLSessionRef ref, CFDataRef dta );
URLSessionDownloadTaskRef URLSessionDownloadTaskWithResumeDataCompletionHandler( URLSessionRef ref, CFDataRef dta, URLSessionDownloadTaskCompletionHandlerType handlerAddress, void *userData );

// Add upload tasks
URLSessionUploadTaskRef URLSessionUploadTaskWithRequestFromData( URLSessionRef ref, URLRequestRef request, CFDataRef dta );
URLSessionUploadTaskRef URLSessionUploadTaskWithRequestFromDataCompletionHandler( URLSessionRef ref, URLRequestRef request, CFDataRef dta, URLSessionUploadTaskCompletionHandlerType handlerAddress, void *userData );
URLSessionUploadTaskRef URLSessionUploadTaskWithRequestFromFile( URLSessionRef ref, URLRequestRef request, CFURLRef url );
URLSessionUploadTaskRef URLSessionUploadTaskWithRequestFromFileCompletionHandler( URLSessionRef ref, URLRequestRef request, CFURLRef url, URLSessionUploadTaskCompletionHandlerType handlerAddress, void *userData );
URLSessionUploadTaskRef URLSessionUploadTaskWithStreamedRequest( URLSessionRef ref, URLRequestRef request );

// Add stream tasks
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
URLSessionStreamTaskRef URLSessionStreamTaskWithHostName( URLSessionRef ref, CFStringRef hostName, NSInteger port );
URLSessionStreamTaskRef URLSessionStreamTaskWithNetService( URLSessionRef ref, NetServiceRef service );
#endif

// Managing session
void URLSessionFinishTasksAndInvalidate( URLSessionRef ref );
void URLSessionFlushWithCompletionHandler( URLSessionRef ref, URLSessionFlushCompletionHandlerType handlerAddress, void *userData );
void URLSessionGetTasksWithCompletionHandler( URLSessionRef ref, URLSessionGetTasksCompletionHandlerType handlerAddress, void *userData );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void URLSessionGetAllTasksWithCompletionHandler( URLSessionRef ref, URLSessionGetAllTasksCompletionHandlerType handlerAddress, void *userData );
#endif
void URLSessionInvalidateAndCancel( URLSessionRef ref );
void URLSessionResetWithCompletionHandler( URLSessionRef ref, URLSessionResetCompletionHandlerType handlerAddress, void *userData );


/*
 URLSessionConfiguration
 */
// Create
URLSessionConfigurationRef URLSessionConfigurationDefault( void );
URLSessionConfigurationRef URLSessionConfigurationEphemeral( void );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
URLSessionConfigurationRef URLSessionConfigurationBackground( CFStringRef identifier );
#endif

// General properties
CFStringRef URLSessionConfigurationIdentifier( URLSessionConfigurationRef ref );
CFDictionaryRef URLSessionConfigurationHTTPAdditionalHeaders( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetHTTPAdditionalHeaders( URLSessionConfigurationRef ref, CFDictionaryRef headers );
NSURLRequestNetworkServiceType URLSessionConfigurationNetworkServiceType( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetNetworkServiceType( URLSessionConfigurationRef ref, NSURLRequestNetworkServiceType type );
Boolean URLSessionConfigurationAllowsCellularAccess( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetAllowsCellularAccess( URLSessionConfigurationRef ref, Boolean flag );
CFTimeInterval URLSessionConfigurationTimeoutIntervalForRequest( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetTimeoutIntervalForRequest( URLSessionConfigurationRef ref, CFTimeInterval ti );
CFTimeInterval URLSessionConfigurationTimeoutIntervalForResource( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetTimeoutIntervalForResource( URLSessionConfigurationRef ref, CFTimeInterval ti );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CFStringRef URLSessionConfigurationSharedContainerIdentifier( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetSharedContainerIdentifier( URLSessionConfigurationRef ref, CFStringRef identifier );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean URLSessionConfigurationWaitsForConnectivity( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetWaitsForConnectivity( URLSessionConfigurationRef ref, Boolean flag );
#endif

// Cookie policies
NSHTTPCookieAcceptPolicy URLSessionConfigurationHTTPCookieAcceptPolicy( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetHTTPCookieAcceptPolicy( URLSessionConfigurationRef ref, NSHTTPCookieAcceptPolicy policy );
Boolean URLSessionConfigurationHTTPShouldSetCookies( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetHTTPShouldSetCookies( URLSessionConfigurationRef ref, Boolean flag );
HTTPCookieStorageRef URLSessionConfigurationHTTPCookieStorage( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetHTTPCookieStorage( URLSessionConfigurationRef ref, HTTPCookieStorageRef storage );

// Security policies
SSLProtocol URLSessionConfigurationTLSMaximumSupportedProtocol( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetTLSMaximumSupportedProtocol( URLSessionConfigurationRef ref, SSLProtocol protocol );
SSLProtocol URLSessionConfigurationTLSMinimumSupportedProtocol( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetTLSMinimumSupportedProtocol( URLSessionConfigurationRef ref, SSLProtocol protocol );
URLCredentialStorageRef URLSessionConfigurationURLCredentialStorage( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetURLCredentialStorage( URLSessionConfigurationRef ref, URLCredentialStorageRef storage );

// Caching policies
URLCacheRef URLSessionConfigurationURLCache( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetURLCache( URLSessionConfigurationRef ref, URLCacheRef cache );
NSURLRequestCachePolicy URLSessionConfigurationRequestCachePolicy( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetRequestCachePolicy( URLSessionConfigurationRef ref, NSURLRequestCachePolicy policy );

// Background transfers
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean URLSessionConfigurationIsDiscretionary( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetDiscretionary( URLSessionConfigurationRef ref, Boolean flag );
#endif
Boolean URLSessionConfigurationShouldUseExtendedBackgroundIdleMode( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetShouldUseExtendedBackgroundIdleMode( URLSessionConfigurationRef ref, Boolean flag );

// Custom protocols
CFArrayRef URLSessionConfigurationProtocoClasses( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetProtocoClasses( URLSessionConfigurationRef ref, CFArrayRef classes );

// HTTP policy and proxy properties
NSInteger URLSessionConfigurationHTTPMaximumConnectionsPerHost( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetHTTPMaximumConnectionsPerHost( URLSessionConfigurationRef ref, NSInteger connections );
Boolean URLSessionConfigurationHTTPShouldUsePipelining( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetHTTPShouldUsePipelining( URLSessionConfigurationRef ref, Boolean flag );
CFDictionaryRef URLSessionConfigurationConnectionProxyDictionary( URLSessionConfigurationRef ref );
void URLSessionConfigurationSetConnectionProxyDictionary( URLSessionConfigurationRef ref, CFDictionaryRef proxy );

/*
 URLSessionTask
 */
// Task state
void URLSessionTaskCancel( URLSessionTaskRef ref );
void URLSessionTaskResume( URLSessionTaskRef ref );
void URLSessionTaskSuspend( URLSessionTaskRef ref );
NSURLSessionTaskState URLSessionTaskState( URLSessionTaskRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
float URLSessionTaskPriority( URLSessionTaskRef ref );
void URLSessionTaskSetPriority( URLSessionTaskRef ref, float priority );
#endif

// Task progress
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
ProgressRef URLSessionTaskProgress( URLSessionTaskRef ref );
#endif
SInt64 URLSessionTaskCountOfBytesExpectedToReceive( URLSessionTaskRef ref );
SInt64 URLSessionTaskCountOfBytesReceived( URLSessionTaskRef ref );
SInt64 URLSessionTaskCountOfBytesExpectedToSend( URLSessionTaskRef ref );
SInt64 URLSessionTaskCountOfBytesSent( URLSessionTaskRef ref );

// General task info
URLRequestRef URLSessionTaskCurrentRequest( URLSessionTaskRef ref );
URLRequestRef URLSessionTaskOriginalRequest( URLSessionTaskRef ref );
URLResponseRef URLSessionTaskResponse( URLSessionTaskRef ref );
CFStringRef URLSessionTaskDescription( URLSessionTaskRef ref );
void URLSessionTaskSetDescription( URLSessionTaskRef ref, CFStringRef description );
NSUInteger URLSessionTaskIdentifier( URLSessionTaskRef ref );
ErrorRef URLSessionTaskError( URLSessionTaskRef ref );

// Scheduling tasks
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
SInt64 URLSessionTaskCountOfBytesClientExpectsToReceive( URLSessionTaskRef ref );
SInt64 URLSessionTaskCountOfBytesClientExpectsToSend( URLSessionTaskRef ref );
CFDateRef URLSessionTaskEarliestBeginDate( URLSessionTaskRef ref );
#endif

/*
 URLSessionDataTask
 */


/*
 URLSessionDownloadTask
 */
typedef void (*URLSessionDownloadTaskCancelCompletionHandlerType)(NSURLSessionDownloadTask*,CFDataRef,void*);

// Cancel download
void URLSessionDownloadTaskCancelByProducingResumeData( URLSessionDownloadTaskRef ref, URLSessionDownloadTaskCancelCompletionHandlerType handlerAddress, void *userData );


/*
 URLSessionStreamTask
 */
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
typedef void (*URLSessionStreamTaskReadCompletionHandlerType)(NSURLSessionStreamTask*,CFDataRef,BOOL,NSError*,void*);
typedef void (*URLSessionStreamTaskWriteCompletionHandlerType)(NSURLSessionStreamTask*,NSError*,void*);

// Reading and writing
void URLSessionStreamTaskReadData( URLSessionStreamTaskRef ref, NSUInteger minLength, NSUInteger maxLength, CFTimeInterval timeout, URLSessionStreamTaskReadCompletionHandlerType handlerAddress, void *userData );
void URLSessionStreamTaskWriteData( URLSessionStreamTaskRef ref, CFDataRef dta, CFTimeInterval timeout, URLSessionStreamTaskWriteCompletionHandlerType handlerAddress, void *userData );

// Capture streams
void URLSessionStreamCaptureStreams( URLSessionStreamTaskRef ref );
void URLSessionStreamCloseWrite( URLSessionStreamTaskRef ref );

// Secure connections
void URLSessionStreamStartSecureConnection( URLSessionStreamTaskRef ref );
void URLSessionStreamStopSecureConnection( URLSessionStreamTaskRef ref );
#endif// __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100



#endif// __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
