/*
 URLSession.m

 Bernie Wylde
 */

#import "URLSession.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000

typedef void (*URLSessionDataTaskCompletionHandlerType)(NSURLSession*,CFDataRef,NSURLResponse*,NSError*,void*);
typedef void (*URLSessionDownloadTaskCompletionHandlerType)(NSURLSession*,CFURLRef,NSURLResponse*,NSError*,void*);
typedef void (*URLSessionUploadTaskCompletionHandlerType)(NSURLSession*,CFDataRef,NSURLResponse*,NSError*,void*);
typedef void (*URLSessionFlushCompletionHandlerType)(NSURLSession*,void*);
typedef void (*URLSessionGetTasksCompletionHandlerType)(NSURLSession*,CFArrayRef,CFArrayRef,CFArrayRef,void*);
typedef void (*URLSessionGetAllTasksCompletionHandlerType)(NSURLSession*,CFArrayRef,void*);
typedef void (*URLSessionResetCompletionHandlerType)(NSURLSession*,void*);

@implementation CocoaURLSessionDelegate
#pragma mark - NSURLSessionDelegate
- (void)URLSession:(NSURLSession *)session didBecomeInvalidWithError:(NSError *)error {
    CallUserDefinedOnDialogFunction( URLSessionDidBecomeInvalid, 0, 0, session );
}

- (void)URLSessionDidFinishEventsForBackgroundURLSession:(NSURLSession *)session {
    CallUserDefinedOnDialogFunction( URLSessionDidFinishEvents, 0, 0, session );
}

//- (void)URLSession:(NSURLSession *)session didReceiveChallenge:(NSURLAuthenticationChallenge *)challenge completionHandler:(void (^)(NSURLSessionAuthChallengeDisposition disposition, NSURLCredential *credential))completionHandler {
//    CallUserDefinedOnDialogFunction( URLSessionDidReceiveChallenge, session, 0, error );
//}

#pragma mark - NSURLSessionDownloadDelegate
- (void)URLSession:(NSURLSession *)session downloadTask:(NSURLSessionDownloadTask *)downloadTask didFinishDownloadingToURL:(NSURL *)location {
    DialogEventSetNSURL( location );
    CallUserDefinedOnDialogFunction( URLSessionDownloadTaskDidFinishDownloading, 0, 0, session );
}

- (void)URLSession:(NSURLSession *)session downloadTask:(NSURLSessionDownloadTask *)downloadTask didResumeAtOffset:(int64_t)fileOffset expectedTotalBytes:(int64_t)expectedTotalBytes {
    CallUserDefinedOnDialogFunction( URLSessionDownloadTaskDidResume, 0, 0, session );
}

- (void)URLSession:(NSURLSession *)session downloadTask:(NSURLSessionDownloadTask *)downloadTask didWriteData:(int64_t)bytesWritten totalBytesWritten:(int64_t)totalBytesWritten totalBytesExpectedToWrite:(int64_t)totalBytesExpectedToWrite {
    CallUserDefinedOnDialogFunction( URLSessionDownloadTaskDidWriteData, 0, 0, session );
}

#pragma mark - NSURLSessionTaskDelegate
- (void)URLSession:(NSURLSession *)session task:(NSURLSessionTask *)task didCompleteWithError:(NSError *)error {
    CallUserDefinedOnDialogFunction( URLSessionTaskDidComplete, 0, 0, session );
}

//- (void)URLSession:(NSURLSession *)session task:(NSURLSessionTask *)task willPerformHTTPRedirection:(NSHTTPURLResponse *)response newRequest:(NSURLRequest *)request completionHandler:(void (^)(NSURLRequest *))completionHandler {
//
//}

- (void)URLSession:(NSURLSession *)session task:(NSURLSessionTask *)task didSendBodyData:(int64_t)bytesSent totalBytesSent:(int64_t)totalBytesSent totalBytesExpectedToSend:(int64_t)totalBytesExpectedToSend {
    CallUserDefinedOnDialogFunction( URLSessionTaskDidSendBodyData, 0, 0, session );
}

//- (void)URLSession:(NSURLSession *)session task:(NSURLSessionTask *)task needNewBodyStream:(void (^)(NSInputStream *bodyStream))completionHandler {
//
//}

//- (void)URLSession:(NSURLSession *)session task:(NSURLSessionTask *)task didReceiveChallenge:(NSURLAuthenticationChallenge *)challenge completionHandler:(void (^)(NSURLSessionAuthChallengeDisposition disposition, NSURLCredential *credential))completionHandler {
//    
//}

//- (void)URLSession:(NSURLSession *)session task:(NSURLSessionTask *)task willBeginDelayedRequest:(NSURLRequest *)request completionHandler:(void (^)(NSURLSessionDelayedRequestDisposition disposition, NSURLRequest *newRequest))completionHandler {
//
//}

- (void)URLSession:(NSURLSession *)session taskIsWaitingForConnectivity:(NSURLSessionTask *)task {
    CallUserDefinedOnDialogFunction( URLSessionTaskIsWaitingForConnectivity, 0, 0, session );
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
- (void)URLSession:(NSURLSession *)session task:(NSURLSessionTask *)task didFinishCollectingMetrics:(NSURLSessionTaskMetrics *)metrics {
    CallUserDefinedOnDialogFunction( URLSessionTaskDidFinishCollectingMetrics, 0, 0, session );
}
#endif

// NSURLSessionDataDelegate
//- (void)URLSession:(NSURLSession *)session dataTask:(NSURLSessionDataTask *)dataTask didReceiveResponse:(NSURLResponse *)response completionHandler:(void (^)(NSURLSessionResponseDisposition disposition))completionHandler {
//    
//}

- (void)URLSession:(NSURLSession *)session dataTask:(NSURLSessionDataTask *)dataTask didBecomeDownloadTask:(NSURLSessionDownloadTask *)downloadTask {
    
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
- (void)URLSession:(NSURLSession *)session dataTask:(NSURLSessionDataTask *)dataTask didBecomeStreamTask:(NSURLSessionStreamTask *)streamTask {
    
}
#endif

- (void)URLSession:(NSURLSession *)session dataTask:(NSURLSessionDataTask *)dataTask didReceiveData:(NSData *)data {
    
}

//- (void)URLSession:(NSURLSession *)session dataTask:(NSURLSessionDataTask *)dataTask willCacheResponse:(NSCachedURLResponse *)proposedResponse completionHandler:(void (^)(NSCachedURLResponse *cachedResponse))completionHandler {
//
//}


// NSURLSessionStreamDelegate
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
- (void)URLSession:(NSURLSession *)session betterRouteDiscoveredForStreamTask:(NSURLSessionStreamTask *)streamTask {
    CallUserDefinedOnDialogFunction( URLSessionStreamTaskBetterRouteDiscovered, 0, 0, session );
}

- (void)URLSession:(NSURLSession *)session streamTask:(NSURLSessionStreamTask *)streamTask didBecomeInputStream:(NSInputStream *)inputStream outputStream:(NSOutputStream *)outputStream {
    CallUserDefinedOnDialogFunction( URLSessionStreamTaskDidBecomeInputStream, 0, 0, session );
}

- (void)URLSession:(NSURLSession *)session readClosedForStreamTask:(NSURLSessionStreamTask *)streamTask {
    CallUserDefinedOnDialogFunction( URLSessionStreamTaskReadClosed, 0, 0, session );
}

- (void)URLSession:(NSURLSession *)session writeClosedForStreamTask:(NSURLSessionStreamTask *)streamTask {
    CallUserDefinedOnDialogFunction( URLSessionStreamTaskWriteClosed, 0, 0, session );
}
#endif
@end

// - Private -
CocoaURLSessionDelegate *CocoaURLSessionDelegateObj( void )
{
    static CocoaURLSessionDelegate *sCocoaURLSessionDelegate = nil;
    if ( !sCocoaURLSessionDelegate ) sCocoaURLSessionDelegate = [[CocoaURLSessionDelegate alloc] init];
    return sCocoaURLSessionDelegate;
}

// Create
URLSessionRef URLSessionWithConfiguration( URLSessionConfigurationRef config )
{ return [NSURLSession sessionWithConfiguration:config]; }

URLSessionRef URLSessionWithDelegateQueue( URLSessionConfigurationRef config, OperationQueueRef queue )
{ return [NSURLSession sessionWithConfiguration:config delegate:CocoaURLSessionDelegateObj() delegateQueue:queue]; }

URLSessionRef URLSessionSharedSession( void )
{ return [NSURLSession sharedSession]; }

// Configure
URLSessionConfigurationRef URLSessionConfiguration( URLSessionRef ref )
{ return [ref configuration]; }

OperationQueueRef URLSessionDelegateQueue( URLSessionRef ref )
{ return [ref delegateQueue]; }

CFStringRef URLSessionDescription( URLSessionRef ref )
{ return (CFStringRef)[ref sessionDescription]; }

// Add data tasks
URLSessionDataTaskRef URLSessionDataTaskWithURL( URLSessionRef ref, CFURLRef url )
{ return [ref dataTaskWithURL:(NSURL *)url]; }

URLSessionDataTaskRef URLSessionDataTaskWithURLCompletionHandler( URLSessionRef ref, CFURLRef url, URLSessionDataTaskCompletionHandlerType handlerAddress, void *userData )
{
    return [ref dataTaskWithURL:(NSURL *)url completionHandler:^(NSData *data, NSURLResponse *response, NSError *error) {
        (*handlerAddress)( ref, (CFDataRef)data, response, error, userData );
    }];
}

URLSessionDataTaskRef URLSessionDataTaskWithRequest( URLSessionRef ref, URLRequestRef request )
{ return [ref dataTaskWithRequest:request]; }

URLSessionDataTaskRef URLSessionDataTaskWithRequestCompletionHandler( URLSessionRef ref, URLRequestRef request, URLSessionDataTaskCompletionHandlerType handlerAddress, void *userData )
{
    return [ref dataTaskWithRequest:request completionHandler:^(NSData *data, NSURLResponse *response, NSError *error) {
        (*handlerAddress)( ref, (CFDataRef)data, response, error, userData );
    }];
}

// Add download tasks
URLSessionDownloadTaskRef URLSessionDownloadTaskWithURL( URLSessionRef ref, CFURLRef url )
{ return [ref downloadTaskWithURL:(NSURL *)url]; }

URLSessionDownloadTaskRef URLSessionDownloadTaskWithURLCompletionHandler( URLSessionRef ref, CFURLRef url, URLSessionDownloadTaskCompletionHandlerType handlerAddress, void *userData )
{
    return [ref downloadTaskWithURL:(NSURL *)url completionHandler:^(NSURL *location, NSURLResponse *response, NSError *error) {
        (*handlerAddress)( ref, (CFURLRef)location, response, error, userData );
    }];
}

URLSessionDownloadTaskRef URLSessionDownloadTaskWithRequest( URLSessionRef ref, URLRequestRef request )
{ return [ref downloadTaskWithRequest:request]; }

URLSessionDownloadTaskRef URLSessionDownloadTaskWithRequestCompletionHandler( URLSessionRef ref, URLRequestRef request, URLSessionDownloadTaskCompletionHandlerType handlerAddress, void *userData )
{
    return [ref downloadTaskWithRequest:request completionHandler:^(NSURL *location, NSURLResponse *response, NSError *error) {
        (*handlerAddress)( ref, (CFURLRef)location, response, error, userData );
    }];
}

URLSessionDownloadTaskRef URLSessionDownloadTaskWithResumeData( URLSessionRef ref, CFDataRef dta )
{ return [ref downloadTaskWithResumeData:(NSData *)dta]; }

URLSessionDownloadTaskRef URLSessionDownloadTaskWithResumeDataCompletionHandler( URLSessionRef ref, CFDataRef dta, URLSessionDownloadTaskCompletionHandlerType handlerAddress, void *userData )
{
    return [ref downloadTaskWithResumeData:(NSData *)dta completionHandler:^(NSURL *location, NSURLResponse *response, NSError *error) {
        (*handlerAddress)( ref, (CFURLRef)location, response, error, userData );
    }];
}

// Add upload tasks
URLSessionUploadTaskRef URLSessionUploadTaskWithRequestFromData( URLSessionRef ref, URLRequestRef request, CFDataRef dta )
{ return [ref uploadTaskWithRequest:request fromData:(NSData *)dta]; }

URLSessionUploadTaskRef URLSessionUploadTaskWithRequestFromDataCompletionHandler( URLSessionRef ref, URLRequestRef request, CFDataRef dta, URLSessionUploadTaskCompletionHandlerType handlerAddress, void *userData )
{
    return [ref uploadTaskWithRequest:request fromData:(NSData *)dta completionHandler:^(NSData *data, NSURLResponse *response, NSError *error) {
        (*handlerAddress)( ref, (CFDataRef)data, response, error, userData );
    }];
}

URLSessionUploadTaskRef URLSessionUploadTaskWithRequestFromFile( URLSessionRef ref, URLRequestRef request, CFURLRef url )
{ return [ref uploadTaskWithRequest:request fromFile:(NSURL *)url]; }

URLSessionUploadTaskRef URLSessionUploadTaskWithRequestFromFileCompletionHandler( URLSessionRef ref, URLRequestRef request, CFURLRef url, URLSessionUploadTaskCompletionHandlerType handlerAddress, void *userData )
{
    return [ref uploadTaskWithRequest:request fromFile:(NSURL *)url completionHandler:^(NSData *data, NSURLResponse *response, NSError *error) {
        (*handlerAddress)( ref, (CFDataRef)data, response, error, userData );
    }];
}

URLSessionUploadTaskRef URLSessionUploadTaskWithStreamedRequest( URLSessionRef ref, URLRequestRef request )
{ return [ref uploadTaskWithStreamedRequest:request]; }

// Add stream tasks
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
URLSessionStreamTaskRef URLSessionStreamTaskWithHostName( URLSessionRef ref, CFStringRef hostName, NSInteger port )
{ return [ref streamTaskWithHostName:(NSString *)hostName port:port]; }

URLSessionStreamTaskRef URLSessionStreamTaskWithNetService( URLSessionRef ref, NetServiceRef service )
{ return [ref streamTaskWithNetService:service]; }
#endif

// Managing session
void URLSessionFinishTasksAndInvalidate( URLSessionRef ref )
{ [ref finishTasksAndInvalidate]; }

void URLSessionFlushWithCompletionHandler( URLSessionRef ref, URLSessionFlushCompletionHandlerType handlerAddress, void *userData )
{
    [ref flushWithCompletionHandler:^(void) {
        (*handlerAddress)( ref, userData );
    }];
}

void URLSessionGetTasksWithCompletionHandler( URLSessionRef ref, URLSessionGetTasksCompletionHandlerType handlerAddress, void *userData )
{
    [ref getTasksWithCompletionHandler:^(NSArray *dataTasks, NSArray *uploadTasks, NSArray *downloadTasks) {
        (*handlerAddress)( ref, (CFArrayRef)dataTasks, (CFArrayRef)uploadTasks, (CFArrayRef)downloadTasks, userData );
    }];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void URLSessionGetAllTasksWithCompletionHandler( URLSessionRef ref, URLSessionGetAllTasksCompletionHandlerType handlerAddress, void *userData )
{
    [ref getAllTasksWithCompletionHandler:^(NSArray *tasks) {
        (*handlerAddress)( ref, (CFArrayRef)tasks, userData );
    }];
}
#endif

void URLSessionInvalidateAndCancel( URLSessionRef ref )
{ [ref invalidateAndCancel]; }

void URLSessionResetWithCompletionHandler( URLSessionRef ref, URLSessionResetCompletionHandlerType handlerAddress, void *userData )
{
    [ref resetWithCompletionHandler:^(void) {
        (*handlerAddress)( ref, userData );
    }];
}


/*
 URLSessionConfiguration
 */
// Create
URLSessionConfigurationRef URLSessionConfigurationDefault( void )
{ return [NSURLSessionConfiguration defaultSessionConfiguration]; }

URLSessionConfigurationRef URLSessionConfigurationEphemeral( void )
{ return [NSURLSessionConfiguration ephemeralSessionConfiguration]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
URLSessionConfigurationRef URLSessionConfigurationBackground( CFStringRef identifier )
{ return [NSURLSessionConfiguration backgroundSessionConfigurationWithIdentifier:(NSString *)identifier]; }
#endif

// General properties
CFStringRef URLSessionConfigurationIdentifier( URLSessionConfigurationRef ref )
{ return (CFStringRef)[ref identifier]; }

CFDictionaryRef URLSessionConfigurationHTTPAdditionalHeaders( URLSessionConfigurationRef ref )
{ return (CFDictionaryRef)[ref HTTPAdditionalHeaders]; }

void URLSessionConfigurationSetHTTPAdditionalHeaders( URLSessionConfigurationRef ref, CFDictionaryRef headers )
{ [ref setHTTPAdditionalHeaders:(NSDictionary *)headers]; }

NSURLRequestNetworkServiceType URLSessionConfigurationNetworkServiceType( URLSessionConfigurationRef ref )
{ return [ref networkServiceType]; }

void URLSessionConfigurationSetNetworkServiceType( URLSessionConfigurationRef ref, NSURLRequestNetworkServiceType type )
{ [ref setNetworkServiceType:type]; }

Boolean URLSessionConfigurationAllowsCellularAccess( URLSessionConfigurationRef ref )
{ return [ref allowsCellularAccess]; }

void URLSessionConfigurationSetAllowsCellularAccess( URLSessionConfigurationRef ref, Boolean flag )
{ [ref setAllowsCellularAccess:flag]; }

CFTimeInterval URLSessionConfigurationTimeoutIntervalForRequest( URLSessionConfigurationRef ref )
{ return [ref timeoutIntervalForRequest]; }

void URLSessionConfigurationSetTimeoutIntervalForRequest( URLSessionConfigurationRef ref, CFTimeInterval ti )
{ [ref setTimeoutIntervalForRequest:ti]; }

CFTimeInterval URLSessionConfigurationTimeoutIntervalForResource( URLSessionConfigurationRef ref )
{ return [ref timeoutIntervalForResource]; }

void URLSessionConfigurationSetTimeoutIntervalForResource( URLSessionConfigurationRef ref, CFTimeInterval ti )
{ [ref setTimeoutIntervalForResource:ti]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CFStringRef URLSessionConfigurationSharedContainerIdentifier( URLSessionConfigurationRef ref )
{ return (CFStringRef)[ref sharedContainerIdentifier]; }

void URLSessionConfigurationSetSharedContainerIdentifier( URLSessionConfigurationRef ref, CFStringRef identifier )
{ [ref setSharedContainerIdentifier:(NSString *)identifier]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean URLSessionConfigurationWaitsForConnectivity( URLSessionConfigurationRef ref )
{ return [ref waitsForConnectivity]; }

void URLSessionConfigurationSetWaitsForConnectivity( URLSessionConfigurationRef ref, Boolean flag )
{ [ref setWaitsForConnectivity:flag]; }
#endif

// Cookie policies
NSHTTPCookieAcceptPolicy URLSessionConfigurationHTTPCookieAcceptPolicy( URLSessionConfigurationRef ref )
{ return [ref HTTPCookieAcceptPolicy]; }

void URLSessionConfigurationSetHTTPCookieAcceptPolicy( URLSessionConfigurationRef ref, NSHTTPCookieAcceptPolicy policy )
{ [ref setHTTPCookieAcceptPolicy:policy]; }

Boolean URLSessionConfigurationHTTPShouldSetCookies( URLSessionConfigurationRef ref )
{ return [ref HTTPShouldSetCookies]; }

void URLSessionConfigurationSetHTTPShouldSetCookies( URLSessionConfigurationRef ref, Boolean flag )
{ [ref setHTTPShouldSetCookies:flag]; }

HTTPCookieStorageRef URLSessionConfigurationHTTPCookieStorage( URLSessionConfigurationRef ref )
{ return [ref HTTPCookieStorage]; }

void URLSessionConfigurationSetHTTPCookieStorage( URLSessionConfigurationRef ref, HTTPCookieStorageRef storage )
{ [ref setHTTPCookieStorage:storage]; }

// Security policies
SSLProtocol URLSessionConfigurationTLSMaximumSupportedProtocol( URLSessionConfigurationRef ref )
{ return [ref TLSMaximumSupportedProtocol]; }

void URLSessionConfigurationSetTLSMaximumSupportedProtocol( URLSessionConfigurationRef ref, SSLProtocol protocol )
{ [ref setTLSMaximumSupportedProtocol:protocol]; }

SSLProtocol URLSessionConfigurationTLSMinimumSupportedProtocol( URLSessionConfigurationRef ref )
{ return [ref TLSMinimumSupportedProtocol]; }

void URLSessionConfigurationSetTLSMinimumSupportedProtocol( URLSessionConfigurationRef ref, SSLProtocol protocol )
{ [ref setTLSMinimumSupportedProtocol:protocol]; }

URLCredentialStorageRef URLSessionConfigurationURLCredentialStorage( URLSessionConfigurationRef ref )
{ return [ref URLCredentialStorage]; }

void URLSessionConfigurationSetURLCredentialStorage( URLSessionConfigurationRef ref, URLCredentialStorageRef storage )
{ [ref setURLCredentialStorage:storage]; }

// Caching policies
URLCacheRef URLSessionConfigurationURLCache( URLSessionConfigurationRef ref )
{ return [ref URLCache]; }

void URLSessionConfigurationSetURLCache( URLSessionConfigurationRef ref, URLCacheRef cache )
{ [ref setURLCache:cache]; }

NSURLRequestCachePolicy URLSessionConfigurationRequestCachePolicy( URLSessionConfigurationRef ref )
{ return [ref requestCachePolicy]; }

void URLSessionConfigurationSetRequestCachePolicy( URLSessionConfigurationRef ref, NSURLRequestCachePolicy policy )
{ [ref setRequestCachePolicy:policy]; }

// Background transfers
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean URLSessionConfigurationIsDiscretionary( URLSessionConfigurationRef ref )
{ return [ref isDiscretionary]; }

void URLSessionConfigurationSetDiscretionary( URLSessionConfigurationRef ref, Boolean flag )
{ [ref setDiscretionary:flag]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean URLSessionConfigurationShouldUseExtendedBackgroundIdleMode( URLSessionConfigurationRef ref )
{ return [ref shouldUseExtendedBackgroundIdleMode]; }

void URLSessionConfigurationSetShouldUseExtendedBackgroundIdleMode( URLSessionConfigurationRef ref, Boolean flag )
{ [ref setShouldUseExtendedBackgroundIdleMode:flag]; }
#endif

// Custom protocols
CFArrayRef URLSessionConfigurationProtocoClasses( URLSessionConfigurationRef ref )
{ return (CFArrayRef)[ref protocolClasses]; }

void URLSessionConfigurationSetProtocoClasses( URLSessionConfigurationRef ref, CFArrayRef classes )
{ [ref setProtocolClasses:(NSArray *)classes]; }

// HTTP policy and proxy properties
NSInteger URLSessionConfigurationHTTPMaximumConnectionsPerHost( URLSessionConfigurationRef ref )
{ return [ref HTTPMaximumConnectionsPerHost]; }

void URLSessionConfigurationSetHTTPMaximumConnectionsPerHost( URLSessionConfigurationRef ref, NSInteger connections )
{ [ref setHTTPMaximumConnectionsPerHost:connections]; }

Boolean URLSessionConfigurationHTTPShouldUsePipelining( URLSessionConfigurationRef ref )
{ return [ref HTTPShouldUsePipelining]; }

void URLSessionConfigurationSetHTTPShouldUsePipelining( URLSessionConfigurationRef ref, Boolean flag )
{ [ref setHTTPShouldUsePipelining:flag]; }

CFDictionaryRef URLSessionConfigurationConnectionProxyDictionary( URLSessionConfigurationRef ref )
{ return (CFDictionaryRef)[ref connectionProxyDictionary]; }

void URLSessionConfigurationSetConnectionProxyDictionary( URLSessionConfigurationRef ref, CFDictionaryRef proxy )
{ [ref setConnectionProxyDictionary:(NSDictionary *)proxy]; }


/*
 URLSessionTask
 */
// Task state
void URLSessionTaskCancel( URLSessionTaskRef ref )
{ [ref cancel]; }

void URLSessionTaskResume( URLSessionTaskRef ref )
{ [ref resume]; }

void URLSessionTaskSuspend( URLSessionTaskRef ref )
{ [ref suspend]; }

NSURLSessionTaskState URLSessionTaskState( URLSessionTaskRef ref )
{ return [ref state]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
float URLSessionTaskPriority( URLSessionTaskRef ref )
{ return [ref priority]; }

void URLSessionTaskSetPriority( URLSessionTaskRef ref, float priority )
{ [ref setPriority:priority]; }
#endif

// Task progress
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
ProgressRef URLSessionTaskProgress( URLSessionTaskRef ref )
{ return [ref progress]; }
#endif

SInt64 URLSessionTaskCountOfBytesExpectedToReceive( URLSessionTaskRef ref )
{ return [ref countOfBytesExpectedToReceive]; }

SInt64 URLSessionTaskCountOfBytesReceived( URLSessionTaskRef ref )
{ return [ref countOfBytesReceived]; }

SInt64 URLSessionTaskCountOfBytesExpectedToSend( URLSessionTaskRef ref )
{ return [ref countOfBytesExpectedToSend]; }

SInt64 URLSessionTaskCountOfBytesSent( URLSessionTaskRef ref )
{ return [ref countOfBytesSent]; }

// General task info
URLRequestRef URLSessionTaskCurrentRequest( URLSessionTaskRef ref )
{ return [ref currentRequest]; }

URLRequestRef URLSessionTaskOriginalRequest( URLSessionTaskRef ref )
{ return [ref originalRequest]; }

URLResponseRef URLSessionTaskResponse( URLSessionTaskRef ref )
{ return [ref response]; }

CFStringRef URLSessionTaskDescription( URLSessionTaskRef ref )
{ return (CFStringRef)[ref taskDescription]; }

void URLSessionTaskSetDescription( URLSessionTaskRef ref, CFStringRef description )
{ [ref setTaskDescription:(NSString *)description]; }

NSUInteger URLSessionTaskIdentifier( URLSessionTaskRef ref )
{ return [ref taskIdentifier]; }

ErrorRef URLSessionTaskError( URLSessionTaskRef ref )
{ return [ref error]; }

// Scheduling tasks
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
SInt64 URLSessionTaskCountOfBytesClientExpectsToReceive( URLSessionTaskRef ref )
{ return [ref countOfBytesClientExpectsToReceive]; }

SInt64 URLSessionTaskCountOfBytesClientExpectsToSend( URLSessionTaskRef ref )
{ return [ref countOfBytesClientExpectsToSend]; }

CFDateRef URLSessionTaskEarliestBeginDate( URLSessionTaskRef ref )
{ return (CFDateRef)[ref earliestBeginDate]; }
#endif

/*
 URLSessionDataTask
 */


/*
 URLSessionDownloadTask
 */
typedef void (*URLSessionDownloadTaskCancelCompletionHandlerType)(NSURLSessionDownloadTask*,CFDataRef,void*);

// Cancel download
void URLSessionDownloadTaskCancelByProducingResumeData( URLSessionDownloadTaskRef ref, URLSessionDownloadTaskCancelCompletionHandlerType handlerAddress, void *userData )
{
    [ref cancelByProducingResumeData:^(NSData *resumeData) {
        (*handlerAddress)( ref, (CFDataRef)resumeData, userData );
    }];
}


/*
 URLSessionStreamTask
 */
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
typedef void (*URLSessionStreamTaskReadCompletionHandlerType)(NSURLSessionStreamTask*,CFDataRef,BOOL,NSError*,void*);
typedef void (*URLSessionStreamTaskWriteCompletionHandlerType)(NSURLSessionStreamTask*,NSError*,void*);

// Reading and writing
void URLSessionStreamTaskReadData( URLSessionStreamTaskRef ref, NSUInteger minLength, NSUInteger maxLength, CFTimeInterval timeout, URLSessionStreamTaskReadCompletionHandlerType handlerAddress, void *userData )
{
    [ref readDataOfMinLength:minLength maxLength:maxLength timeout:timeout completionHandler:^(NSData *data, BOOL atEOF, NSError *error) {
        (*handlerAddress)( ref, (CFDataRef)data, atEOF, error, userData );
    }];
}

void URLSessionStreamTaskWriteData( URLSessionStreamTaskRef ref, CFDataRef dta, CFTimeInterval timeout, URLSessionStreamTaskWriteCompletionHandlerType handlerAddress, void *userData )
{
    [ref writeData:(NSData *)dta timeout:timeout completionHandler:^(NSError *error) {
        (*handlerAddress)( ref, error, userData );
    }];
}

// Capture streams
void URLSessionStreamCaptureStreams( URLSessionStreamTaskRef ref )
{ [ref captureStreams]; }

void URLSessionStreamCloseWrite( URLSessionStreamTaskRef ref )
{ [ref closeWrite]; }

// Secure connections
void URLSessionStreamStartSecureConnection( URLSessionStreamTaskRef ref )
{ [ref startSecureConnection]; }

void URLSessionStreamStopSecureConnection( URLSessionStreamTaskRef ref )
{ [ref stopSecureConnection]; }
#endif// __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100


#endif//__MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
