/*
 UndoManager.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef void (*CocoaUndoManagerTargetCallbackType)(CFTypeRef);

@interface CocoaUndoManagerTarget : NSObject {
    CocoaUndoManagerTargetCallbackType callback;
}
- (id)initWithCallback:(CocoaUndoManagerTargetCallbackType)cb;
@property (assign) CocoaUndoManagerTargetCallbackType callback;
@end

// private
NSMutableArray *CocoaUndoManagerTargetsArray( void );
void CocoaUndoManagerAddTarget( CocoaUndoManagerTarget *target );


// Registering
void UndoManagerRegisterUndo( UndoManagerRef ref, void *fnAddress, CFTypeRef object );

// Ability
Boolean UndoManagerCanUndo( UndoManagerRef ref );
Boolean UndoManagerCanRedo( UndoManagerRef ref );

// Perform
void UndoManagerUndo( UndoManagerRef ref );
void UndoManagerUndoNestedGroup( UndoManagerRef ref );
void UndoManagerRedo( UndoManagerRef ref );

// Limiting
NSUInteger UndoManagerLevelsOfUndo( UndoManagerRef ref );
void UndoManagerSetLevelsOfUndo( UndoManagerRef ref, NSUInteger levels );

// Groups
void UndoManagerBeginUndoGrouping( UndoManagerRef ref );
void UndoManagerEndUndoGrouping( UndoManagerRef ref );
Boolean UndoManagerGroupsByEvent( UndoManagerRef ref );
void UndoManagerSetGroupsByEvent( UndoManagerRef ref, Boolean flag );
NSInteger UndoManagerGroupingLevel( UndoManagerRef ref );

// Enable
void UndoManagerDisableUndoRegistration( UndoManagerRef ref );
void UndoManagerEnableUndoRegistration( UndoManagerRef ref );
Boolean UndoManagerIsUndoRegistrationEnabled( UndoManagerRef ref );

// Performed
Boolean UndoManagerIsUndoing( UndoManagerRef ref );
Boolean UndoManagerIsRedoing( UndoManagerRef ref );

// Clearing
void UndoManagerRemoveAllActions( UndoManagerRef ref );

// Action name
CFStringRef UndoManagerUndoActionName( UndoManagerRef ref );
CFStringRef UndoManagerRedoActionName( UndoManagerRef ref );
void UndoManagerSetActionName( UndoManagerRef ref, CFStringRef name );

// Menu item title
CFStringRef UndoManagerUndoMenuItemTitle( UndoManagerRef ref );
CFStringRef UndoManagerRedoMenuItemTitle( UndoManagerRef ref );
CFStringRef UndoManagerUndoMenuTitleForUndoActionName( UndoManagerRef ref, CFStringRef name );
CFStringRef UndoManagerRedoMenuTitleForUndoActionName( UndoManagerRef ref, CFStringRef name );

// Loops
CFArrayRef UndoManagerRunLoopModes( UndoManagerRef ref );

// Discardable
void UndoManagerSetActionIsDiscardable( UndoManagerRef ref, Boolean flag );
Boolean UndoManagerUndoActionIsDiscardable( UndoManagerRef ref );
Boolean UndoManagerRedoActionIsDiscardable( UndoManagerRef ref );

