/*
 UndoManager.m

 Bernie Wylde
 */

#import "UndoManager.h"

@implementation CocoaUndoManagerTarget

@synthesize callback;

static CocoaUndoManagerTarget *sCocoaUndoManagerTarget;

static NSMutableArray *sCocoaUndoManagerTargets;

- (id)initWithCallback:(CocoaUndoManagerTargetCallbackType)cb {
    if ( self = [super init] ) {
        callback = cb;
    }
    return self;
}

- (void)undoAction:(id) obj {
    if ( callback ) (*callback)( (CFTypeRef)obj );}
@end

NSMutableArray *CocoaUndoManagerTargetsArray( void )
{
    if ( !sCocoaUndoManagerTargets ) sCocoaUndoManagerTargets = [[NSMutableArray alloc] initWithCapacity:0];
    return sCocoaUndoManagerTargets;
}

void CocoaUndoManagerAddTarget( CocoaUndoManagerTarget *target )
{
    BOOL flag = NO;
    NSMutableArray *targets = CocoaUndoManagerTargetsArray();
    for ( CocoaUndoManagerTarget *t in targets ) {
        if ( [t callback] == [target callback] ) {
            flag = YES;
            break;
        }
    }
    if ( !flag ) [targets addObject:target];
}






// Registering
void UndoManagerRegisterUndo( UndoManagerRef ref, void *fnAddress, CFTypeRef object )
{
//    if ( !sCocoaUndoManagerTarget ) sCocoaUndoManagerTarget = [[CocoaUndoManagerTarget alloc] initWithCallback:fnAddress];
//    [ref registerUndoWithTarget:sCocoaUndoManagerTarget selector:@selector(undoAction:) object:(id)object];

    CocoaUndoManagerTarget *target = [[CocoaUndoManagerTarget alloc] initWithCallback:fnAddress];
    CocoaUndoManagerAddTarget( target );
    
    [ref registerUndoWithTarget:target selector:@selector(undoAction:) object:(id)object];
}

// Ability
Boolean UndoManagerCanUndo( UndoManagerRef ref )
{ return [ref canUndo]; }

Boolean UndoManagerCanRedo( UndoManagerRef ref )
{ return [ref canRedo]; }

// Perform
void UndoManagerUndo( UndoManagerRef ref )
{ [ref undo]; }

void UndoManagerUndoNestedGroup( UndoManagerRef ref )
{ [ref undoNestedGroup]; }

void UndoManagerRedo( UndoManagerRef ref )
{ [ref redo]; }

// Limiting
NSUInteger UndoManagerLevelsOfUndo( UndoManagerRef ref )
{ return [ref levelsOfUndo]; }

void UndoManagerSetLevelsOfUndo( UndoManagerRef ref, NSUInteger levels )
{ [ref setLevelsOfUndo:levels]; }

// Groups
void UndoManagerBeginUndoGrouping( UndoManagerRef ref )
{ [ref beginUndoGrouping]; }

void UndoManagerEndUndoGrouping( UndoManagerRef ref )
{ [ref endUndoGrouping]; }

Boolean UndoManagerGroupsByEvent( UndoManagerRef ref )
{ return [ref groupsByEvent]; }

void UndoManagerSetGroupsByEvent( UndoManagerRef ref, Boolean flag )
{ [ref setGroupsByEvent:flag]; }

NSInteger UndoManagerGroupingLevel( UndoManagerRef ref )
{ return [ref groupingLevel]; }

// Enable
void UndoManagerDisableUndoRegistration( UndoManagerRef ref )
{ [ref disableUndoRegistration]; }

void UndoManagerEnableUndoRegistration( UndoManagerRef ref )
{ [ref enableUndoRegistration]; }

Boolean UndoManagerIsUndoRegistrationEnabled( UndoManagerRef ref )
{ return [ref isUndoRegistrationEnabled]; }


// Performed
Boolean UndoManagerIsUndoing( UndoManagerRef ref )
{ return [ref isUndoing]; }

Boolean UndoManagerIsRedoing( UndoManagerRef ref )
{ return [ref isRedoing]; }

// Clearing
void UndoManagerRemoveAllActions( UndoManagerRef ref )
{ [ref removeAllActions]; }

// Action name
CFStringRef UndoManagerUndoActionName( UndoManagerRef ref )
{ return (CFStringRef)[ref undoActionName]; }

CFStringRef UndoManagerRedoActionName( UndoManagerRef ref )
{ return (CFStringRef)[ref redoActionName]; }

void UndoManagerSetActionName( UndoManagerRef ref, CFStringRef name )
{ [ref setActionName:(NSString *)name]; }

// Menu item title
CFStringRef UndoManagerUndoMenuItemTitle( UndoManagerRef ref )
{ return (CFStringRef)[ref undoMenuItemTitle]; }

CFStringRef UndoManagerRedoMenuItemTitle( UndoManagerRef ref )
{ return (CFStringRef)[ref redoMenuItemTitle]; }

CFStringRef UndoManagerUndoMenuTitleForUndoActionName( UndoManagerRef ref, CFStringRef name )
{ return (CFStringRef)[ref undoMenuTitleForUndoActionName:(NSString *)name]; }

CFStringRef UndoManagerRedoMenuTitleForUndoActionName( UndoManagerRef ref, CFStringRef name )
{ return (CFStringRef)[ref redoMenuTitleForUndoActionName:(NSString *)name]; }

// Loops
CFArrayRef UndoManagerRunLoopModes( UndoManagerRef ref )
{ return (CFArrayRef)[ref runLoopModes]; }

// Discardable
void UndoManagerSetActionIsDiscardable( UndoManagerRef ref, Boolean flag )
{ [ref setActionIsDiscardable:flag]; }

Boolean UndoManagerUndoActionIsDiscardable( UndoManagerRef ref )
{ return [ref undoActionIsDiscardable]; }

Boolean UndoManagerRedoActionIsDiscardable( UndoManagerRef ref )
{ return [ref redoActionIsDiscardable]; }

