/*
 UserDefaults.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

#pragma mark - private functions
//NSDictionary *UserDefaultsProcessDictionary( NSDictionary *inDict );



#pragma mark - public functions
// Get values
CFTypeRef UserDefaultsObject( CFStringRef key );
Boolean UserDefaultsBool( CFStringRef key );
long UserDefaultsInteger( CFStringRef key );
double UserDefaultsDouble( CFStringRef key );
float UserDefaultsFloat( CFStringRef key );
CFURLRef UserDefaultsURL( CFStringRef key );
CFArrayRef UserDefaultsArray( CFStringRef key );
CFDictionaryRef UserDefaultsDictionary( CFStringRef key );
CFStringRef UserDefaultsString( CFStringRef key );
CFDataRef UserDefaultsData( CFStringRef key );
CFDictionaryRef UserDefaultsDictionaryRepresentation( void );

// Set values
void UserDefaultsSetObject( CFStringRef key, CFTypeRef obj );
void UserDefaultsSetBool( CFStringRef key, Boolean value );
void UserDefaultsSetInteger( CFStringRef key, long value );
void UserDefaultsSetDouble( CFStringRef key, double value );
void UserDefaultsSetFloat( CFStringRef key, float value );
void UserDefaultsSetURL( CFStringRef key, CFURLRef value );

// Remove
void UserDefaultsRemoveObject( CFStringRef key );

// Suites
void UserDefaultsAddSuiteNamed( CFStringRef name );
void UserDefaultsRemoveSuiteNamed( CFStringRef name );

// Register
void UserDefaultsRegisterDefaults( CFDictionaryRef defaults );

// Persistent domains
CFDictionaryRef UserDefaultsPersistentDomainForName( CFStringRef name );
void UserDefaultsSetPersistentDomainForName( CFDictionaryRef domain, CFStringRef name );
void UserDefaultsRemovePersistentDomainForName( CFStringRef name );

// Volatile domains
CFArrayRef UserDefaultsVolatileDomainNames( void );
CFDictionaryRef UserDefaultsVolatileDomainForName( CFStringRef name );
void UserDefaultsSetVolatileDomainForName( CFDictionaryRef domain, CFStringRef name );
void UserDefaultsRemoveVolatileDomainForName( CFStringRef name );

// Managed environment keys
Boolean UserDefaultsObjectIsForcedForKey( CFStringRef key );
Boolean UserDefaultsObjectIsForcedForKeyInDomain( CFStringRef key, CFStringRef domain );

// Legacy
Boolean UserDefaultsSynchronize( void );


// custom - private
void CocoaUserDefaultsAddValueToDictionary( id view, NSMutableDictionary *dict );
void CocoaUserDefaultsAddSubviewValuesToDictionary( id superview, NSMutableDictionary *dict );
void CocoaUserDefaultsStoreWindowViewValues( NSWindow *w, NSString *key );
void CocoaUserDefaultsRestoreValueFromDictionary( id view, NSDictionary *dict );
void CocoaUserDefaultsRestoreSubviewValuesFromDictionary( id superview, NSDictionary *dict );
void CocoaUserDefaultsRestoreWindowViewValues( NSWindow *w, NSString *key );
void CocoaUserDefaultsRemoveWindowViewValues( NSWindow *w, NSString *key );

// custom - public

// color
ColorRef UserDefaultsColor( CFStringRef key );
void UserDefaultsSetColor( CFStringRef key, ColorRef col );

// font
CTNSFontRef UserDefaultsFont( CFStringRef key );
void UserDefaultsSetFont( CFStringRef key, CTNSFontRef font );

// Dictation edit menu item
void UserDefaultsDictationMenuItemSetDisabled( Boolean flag );

// Character palette edit menu item
void UserDefaultsCharacterPaletteMenuItemSetDisabled( Boolean flag );

// window view values
void UserDefaultsStoreWindowViewValues( NSInteger wndTag, CFStringRef key );
void UserDefaultsRestoreWindowViewValues( NSInteger wndTag, CFStringRef key );
void UserDefaultsRemoveWindowViewValues( NSInteger wndTag, CFStringRef key );
