/*
 UserDefaults.m

 Bernie Wylde
 */

#import "UserDefaults.h"

#pragma mark - private functions
//NSDictionary *UserDefaultsProcessDictionary( NSDictionary *inDict )
//{
//    NSMutableDictionary *dict = [inDict mutableCopy];
//    NSArray *keys = [dict allKeys];
//    for ( NSString *key in keys ) {
//        id obj = [dict objectForKey:key];
//        if ( [obj isKindOfClass:[NSColor class]] ) {
//            NSData *data = [NSKeyedArchiver archivedDataWithRootObject:obj];
//            [dict setObject:data forKey:key];
//        } else if ( [obj isKindOfClass:[NSFont class]] ) {
//            NSData *data = [NSKeyedArchiver archivedDataWithRootObject:obj];
//            [dict setObject:data forKey:key];
//        } else if ( [obj isKindOfClass:[NSDictionary class]] ) {
//            NSDictionary *newDict = UserDefaultsProcessDictionary( obj );
//            [dict setObject:newDict forKey:key];
//        }
//    }
//    return (NSDictionary *)dict;
//}


#pragma mark - public functions
// Get values
CFTypeRef UserDefaultsObject( CFStringRef key )
{ return [[NSUserDefaults standardUserDefaults] objectForKey:(NSString *)key]; }

Boolean UserDefaultsBool( CFStringRef key )
{ return [[NSUserDefaults standardUserDefaults] boolForKey:(NSString *)key]; }

long UserDefaultsInteger( CFStringRef key )
{ return [[NSUserDefaults standardUserDefaults] integerForKey:(NSString *)key]; }

double UserDefaultsDouble( CFStringRef key )
{ return [[NSUserDefaults standardUserDefaults] doubleForKey:(NSString *)key]; }

float UserDefaultsFloat( CFStringRef key )
{ return [[NSUserDefaults standardUserDefaults] floatForKey:(NSString *)key]; }

CFURLRef UserDefaultsURL( CFStringRef key )
{ return (CFURLRef)[[NSUserDefaults standardUserDefaults] URLForKey:(NSString *)key]; }

CFArrayRef UserDefaultsArray( CFStringRef key )
{ return (CFArrayRef)[[NSUserDefaults standardUserDefaults] arrayForKey:(NSString *)key]; }

CFDictionaryRef UserDefaultsDictionary( CFStringRef key )
{ return (CFDictionaryRef)[[NSUserDefaults standardUserDefaults] dictionaryForKey:(NSString *)key]; }

CFStringRef UserDefaultsString( CFStringRef key )
{ return (CFStringRef)[[NSUserDefaults standardUserDefaults] stringForKey:(NSString *)key]; }

CFDataRef UserDefaultsData( CFStringRef key )
{ return (CFDataRef)[[NSUserDefaults standardUserDefaults] dataForKey:(NSString *)key]; }

CFDictionaryRef UserDefaultsDictionaryRepresentation( void )
{ return (CFDictionaryRef)[[NSUserDefaults standardUserDefaults] dictionaryRepresentation]; }

// Set values
void UserDefaultsSetObject( CFStringRef key, CFTypeRef obj )
{ [[NSUserDefaults standardUserDefaults] setObject:(id)obj forKey:(NSString *)key]; }

void UserDefaultsSetBool( CFStringRef key, Boolean value )
{ [[NSUserDefaults standardUserDefaults] setBool:value forKey:(NSString *)key]; }

void UserDefaultsSetInteger( CFStringRef key, long value )
{ [[NSUserDefaults standardUserDefaults] setInteger:value forKey:(NSString *)key]; }

void UserDefaultsSetDouble( CFStringRef key, double value )
{ [[NSUserDefaults standardUserDefaults] setDouble:value forKey:(NSString *)key]; }

void UserDefaultsSetFloat( CFStringRef key, float value )
{ [[NSUserDefaults standardUserDefaults] setFloat:value forKey:(NSString *)key]; }

void UserDefaultsSetURL( CFStringRef key, CFURLRef value )
{ [[NSUserDefaults standardUserDefaults] setURL:(NSURL *)value forKey:(NSString *)key]; }

// Remove
void UserDefaultsRemoveObject( CFStringRef key )
{ [[NSUserDefaults standardUserDefaults] removeObjectForKey:(NSString *)key]; }

// Suites
void UserDefaultsAddSuiteNamed( CFStringRef name )
{ [[NSUserDefaults standardUserDefaults] addSuiteNamed:(NSString *)name]; }

void UserDefaultsRemoveSuiteNamed( CFStringRef name )
{ [[NSUserDefaults standardUserDefaults] removeSuiteNamed:(NSString *)name]; }

// Register
void UserDefaultsRegisterDefaults( CFDictionaryRef defaults )
{
    NSMutableDictionary *dict = [(NSDictionary *)defaults mutableCopy];
    NSArray *keys = [dict allKeys];
    for ( NSString *key in keys ) {
        id obj = [dict objectForKey:key];
        if ( [obj isKindOfClass:[NSColor class]] ) {
            NSData *data = [NSKeyedArchiver archivedDataWithRootObject:obj];
            [dict setObject:data forKey:key];
        } else if ( [obj isKindOfClass:[NSFont class]] ) {
            NSData *data = [NSKeyedArchiver archivedDataWithRootObject:obj];
            [dict setObject:data forKey:key];
        }
    }
    [[NSUserDefaults standardUserDefaults] registerDefaults:dict];
}

// Persistent domains
CFDictionaryRef UserDefaultsPersistentDomainForName( CFStringRef name )
{ return (CFDictionaryRef)[[NSUserDefaults standardUserDefaults] persistentDomainForName:(NSString *)name]; }

void UserDefaultsSetPersistentDomainForName( CFDictionaryRef domain, CFStringRef name )
{ [[NSUserDefaults standardUserDefaults] setPersistentDomain:(NSDictionary *)domain forName:(NSString *)name]; }

void UserDefaultsRemovePersistentDomainForName( CFStringRef name )
{ [[NSUserDefaults standardUserDefaults] removePersistentDomainForName:(NSString *)name]; }

// Volatile domains
CFArrayRef UserDefaultsVolatileDomainNames( void )
{ return (CFArrayRef)[[NSUserDefaults standardUserDefaults] volatileDomainNames]; }

CFDictionaryRef UserDefaultsVolatileDomainForName( CFStringRef name )
{ return (CFDictionaryRef)[[NSUserDefaults standardUserDefaults] volatileDomainForName:(NSString *)name]; }

void UserDefaultsSetVolatileDomainForName( CFDictionaryRef domain, CFStringRef name )
{ [[NSUserDefaults standardUserDefaults] setVolatileDomain:(NSDictionary *)domain forName:(NSString *)name]; }

void UserDefaultsRemoveVolatileDomainForName( CFStringRef name )
{ [[NSUserDefaults standardUserDefaults] removeVolatileDomainForName:(NSString *)name]; }

// Managed environment keys
Boolean UserDefaultsObjectIsForcedForKey( CFStringRef key )
{ return [[NSUserDefaults standardUserDefaults] objectIsForcedForKey:(NSString *)key]; }

Boolean UserDefaultsObjectIsForcedForKeyInDomain( CFStringRef key, CFStringRef domain )
{ return [[NSUserDefaults standardUserDefaults] objectIsForcedForKey:(NSString *)key inDomain:(NSString *)domain]; }

// Legacy
Boolean UserDefaultsSynchronize( void )
{ return [[NSUserDefaults standardUserDefaults] synchronize]; }


// custom
// private
void CocoaUserDefaultsAddValueToDictionary( id view, NSMutableDictionary *dict )
{
    NSInteger tag = [view tag];
    
    if ( tag > 0 ) {
        // button
        if ( [view class] == [NSButton class] ) {
            NSString *key = [NSString stringWithFormat:@"Button%ldState",(long)[view tag]];
            [dict setObject:[NSNumber numberWithInteger:[(NSButton *)view state]] forKey:key];
            return;
        }
        
        // colorwell
        if ( [view class] == [NSColorWell class] ) {
            NSString *key = [NSString stringWithFormat:@"ColorWell%ldColor",(long)[view tag]];
            [dict setObject:[NSArchiver archivedDataWithRootObject:[view color]] forKey:key];
            return;
        }
        
        // combobox
        if ( [view class] == [NSComboBox class] ) {
            NSString *key = [NSString stringWithFormat:@"ComboBox%ldStringValue",(long)[view tag]];
            [dict setObject:[view stringValue] forKey:key];
            key = [NSString stringWithFormat:@"ComboBox%ldObjectValues",(long)[view tag]];
            [dict setObject:[view objectValues] forKey:key];
            return;
        }
        
        // datepicker
        if ( [view class] == [NSDatePicker class] ) {
            NSString *key = [NSString stringWithFormat:@"DatePicker%ldDateValue",(long)[view tag]];
            [dict setObject:[view dateValue] forKey:key];
            return;
        }
        
        // popupbutton
        if ( [view class] == [NSPopUpButton class] ) {
            NSString *key = [NSString stringWithFormat:@"PopUpButton%ldTitleOfSelectedItem",(long)[view tag]];
            [dict setObject:[view titleOfSelectedItem] forKey:key];
            return;
        }
        
        // searchfield
        if ( [view class] == [NSSearchField class] ) {
            NSString *key = [NSString stringWithFormat:@"SearchField%ldStringValue",(long)[view tag]];
            [dict setObject:[view stringValue] forKey:key];
            return;
        }
        
        // segmentedcontrol
        if ( [view class] == [NSSegmentedControl class] ) {
            NSString *key = [NSString stringWithFormat:@"SegmentedControl%ldSelectedSegment",(long)[view tag]];
            [dict setObject:[NSNumber numberWithInteger:[view selectedSegment]] forKey:key];
            return;
        }
        
        // slider
        if ( [view class] == [NSSlider class] ) {
            NSString *key = [NSString stringWithFormat:@"Slider%ldDoubleValue",(long)[view tag]];
            [dict setObject:[NSNumber numberWithDouble:[view doubleValue]] forKey:key];
            return;
        }
        
        // stepper
        if ( [view class] == [NSStepper class]) {
            NSString *key = [NSString stringWithFormat:@"Stepper%ldDoubleValue",(long)[view tag]];
            [dict setObject:[NSNumber numberWithDouble:[view doubleValue]] forKey:key];
            return;
        }
        
        // textfield
        if ( [view class] == [NSTextField class] ) {
            NSString *key = [NSString stringWithFormat:@"TextField%ldStringValue",(long)[view tag]];
            [dict setObject:[view stringValue] forKey:key];
            return;
        }
        
        // tokenfield
        if ( [view class] == [NSTokenField class] ) {
            NSString *key = [NSString stringWithFormat:@"TokenField%ldStringValue",(long)[view tag]];
            [dict setObject:[view stringValue] forKey:key];
            return;
        }
        
    } else {
        
        // tabview - store tabview item views
        if ( [view class] == [NSTabView class] ) {
            NSArray *items = [view tabViewItems];
            for ( NSTabViewItem *item in items ) {
                NSView *itemView = [item view];
                CocoaUserDefaultsAddSubviewValuesToDictionary( itemView, dict );
            }
            return;
        }
        
    }
}

void CocoaUserDefaultsAddSubviewValuesToDictionary( id superview, NSMutableDictionary *dict )
{
    NSArray *subviews = [superview subviews];
    for ( id obj in subviews ) {
        if ( [obj respondsToSelector:@selector(tag)] ) {
            CocoaUserDefaultsAddValueToDictionary( obj, dict );
        }
        CocoaUserDefaultsAddSubviewValuesToDictionary( obj, dict );
    }
}

void CocoaUserDefaultsStoreWindowViewValues( NSWindow *w, NSString *key )
{
    NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:0];
    CocoaUserDefaultsAddSubviewValuesToDictionary( [w contentView], dict );
    if ( !key ) key = [w title];
    [[NSUserDefaults standardUserDefaults] setObject:dict forKey:key];
}



void CocoaUserDefaultsRestoreValueFromDictionary( id view, NSDictionary *dict )
{
    NSInteger tag = [view tag];
    if ( tag > 0 ) {
        // button
        if ( [view class] == [NSButton class] ) {
            NSString *key = [NSString stringWithFormat:@"Button%ldState",(long)[view tag]];
            NSNumber *number = [dict objectForKey:key];
            if ( number ) [view setState:[number integerValue]];
            return;
        }
        
        // colorwell
        if ( [view class] == [NSColorWell class] ) {
            NSString *key = [NSString stringWithFormat:@"ColorWell%ldColor",(long)[view tag]];
            NSData *data = [dict objectForKey:key];
            if ( data ) [view setColor:[NSUnarchiver unarchiveObjectWithData:data]];
            return;
        }
        
        // combobox
        if ( [view class] == [NSComboBox class] ) {
            NSString *key = [NSString stringWithFormat:@"ComboBox%ldStringValue",(long)[view tag]];
            NSString *string = [dict objectForKey:key];
            if ( string ) [view setStringValue:string];
            key = [NSString stringWithFormat:@"ComboBox%ldObjectValues",(long)[view tag]];
            NSArray *array = [dict objectForKey:key];
            [view removeAllItems];
            if ( key ) [view addItemsWithObjectValues:array];
            return;
        }
        
        // datepicker
        if ( [view class] == [NSDatePicker class] ) {
            NSString *key = [NSString stringWithFormat:@"DatePicker%ldDateValue",(long)[view tag]];
            NSDate *date = [dict objectForKey:key];
            if ( date ) [view setDateValue:date];
            return;
        }
        
        // popupbutton
        if ( [view class] == [NSPopUpButton class] ) {
            NSString *key = [NSString stringWithFormat:@"PopUpButton%ldTitleOfSelectedItem",(long)[view tag]];
            NSString *string = [dict objectForKey:key];
            if ( string ) [view selectItemWithTitle:string];
            return;
        }
        
        // searchfield
        if ( [view class] == [NSSearchField class] ) {
            NSString *key = [NSString stringWithFormat:@"SearchField%ldStringValue",(long)[view tag]];
            NSString *string = [dict objectForKey:key];
            if ( string ) [view setStringValue:string];
            return;
        }
        
        // segmentedcontrol
        if ( [view class] == [NSSegmentedControl class] ) {
            NSString *key = [NSString stringWithFormat:@"SegmentedControl%ldSelectedSegment",(long)[view tag]];
            NSNumber *number = [dict objectForKey:key];
            if ( number ) [view setSelectedSegment:[number integerValue]];
            return;
        }
        
        // slider
        if ( [view class] == [NSSlider class] ) {
            NSString *key = [NSString stringWithFormat:@"Slider%ldDoubleValue",(long)[view tag]];
            NSNumber *number = [dict objectForKey:key];
            if ( number ) [view setDoubleValue:[number doubleValue]];
            return;
        }
        
        // stepper
        if ( [view class] == [NSStepper class] ) {
            NSString *key = [NSString stringWithFormat:@"Stepper%ldDoubleValue",(long)[view tag]];
            NSNumber *number = [dict objectForKey:key];
            if ( number ) [view setDoubleValue:[number doubleValue]];
            return;
        }
        
        // textfield
        if ( [view class] == [NSTextField class] ) {
            NSString *key = [NSString stringWithFormat:@"TextField%ldStringValue",(long)[view tag]];
            NSString *string = [dict objectForKey:key];
            if ( string ) [view setStringValue:string];
            return;
        }
        
        // tokenfield
        if ( [view class] == [NSTokenField class] ) {
            NSString *key = [NSString stringWithFormat:@"TokenField%ldStringValue",(long)[view tag]];
            NSString *string = [dict objectForKey:key];
            if ( string ) [view setStringValue:string];
            return;
        }
        
    } else {
        
        // tabview - restore tabview item views
        if ( [view class] == [NSTabView class] ) {
            NSArray *items = [view tabViewItems];
            for ( NSTabViewItem *item in items ) {
                NSView *itemView = [item view];
                CocoaUserDefaultsRestoreSubviewValuesFromDictionary( itemView, dict );
            }
            return;
        }
        
    }
}

void CocoaUserDefaultsRestoreSubviewValuesFromDictionary( id superview, NSDictionary *dict )
{
    NSArray *subviews = [superview subviews];
    for ( id obj in subviews ) {
        if ( [obj respondsToSelector:@selector(tag)] ) {
            CocoaUserDefaultsRestoreValueFromDictionary( obj, dict );
        }
        CocoaUserDefaultsRestoreSubviewValuesFromDictionary( obj, dict );
    }
}

void CocoaUserDefaultsRestoreWindowViewValues( NSWindow *w, NSString *key )
{
    if ( !key ) key = [w title];
    NSDictionary *dict = [[NSUserDefaults standardUserDefaults] objectForKey:key];
    if ( dict ) CocoaUserDefaultsRestoreSubviewValuesFromDictionary( [w contentView], dict );
}

void CocoaUserDefaultsRemoveWindowViewValues( NSWindow *w, NSString *key )
{
    if ( !key ) key = [w title];
    [[NSUserDefaults standardUserDefaults] setObject:nil forKey:key];
}


// custom

// color
ColorRef UserDefaultsColor( CFStringRef key )
{
    NSData *data = [[NSUserDefaults standardUserDefaults] objectForKey:(NSString *)key];
    if ( data ) return [NSKeyedUnarchiver unarchiveObjectWithData:data];
    return nil;
}

void UserDefaultsSetColor( CFStringRef key, ColorRef col )
{
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:col];
    if ( data ) [[NSUserDefaults standardUserDefaults] setObject:data forKey:(NSString *)key];
}

// font
CTNSFontRef UserDefaultsFont( CFStringRef key )
{
    NSData *data = [[NSUserDefaults standardUserDefaults] objectForKey:(NSString *)key];
    if ( data ) return (CTNSFontRef)[NSKeyedUnarchiver unarchiveObjectWithData:data];
    return nil;
}

void UserDefaultsSetFont( CFStringRef key, CTNSFontRef font )
{
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:(id)font];
    if ( data ) [[NSUserDefaults standardUserDefaults] setObject:data forKey:(NSString *)key];
}

// Dictation edit menu item
void UserDefaultsDictationMenuItemSetDisabled( Boolean flag )
{ [[NSUserDefaults standardUserDefaults] setBool:flag forKey:@"NSDisabledDictationMenuItem"]; }

// Character palette edit menu item
void UserDefaultsCharacterPaletteMenuItemSetDisabled( Boolean flag )
{ [[NSUserDefaults standardUserDefaults] setBool:flag forKey:@"NSDisabledCharacterPaletteMenuItem"]; }


// window view values
void UserDefaultsStoreWindowViewValues( NSInteger wndTag, CFStringRef key )
{
    NSWindow *w = WindowWithTag( wndTag );
    if ( w ) CocoaUserDefaultsStoreWindowViewValues( w, (NSString *)key );
}

void UserDefaultsRestoreWindowViewValues( NSInteger wndTag, CFStringRef key )
{
    NSWindow *w = WindowWithTag( wndTag );
    if ( w ) CocoaUserDefaultsRestoreWindowViewValues( w, (NSString *)key );
}

void UserDefaultsRemoveWindowViewValues( NSInteger wndTag, CFStringRef key )
{
    NSWindow *w = WindowWithTag( wndTag );
    if ( w ) CocoaUserDefaultsRemoveWindowViewValues( w, (NSString *)key );
}

