/*
 UserNotification.m

 Bernie Wylde
 */

#import "UserNotification.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080

// - init -
UserNotificationRef UserNotificationInit( void )
{ return [[[NSUserNotification alloc] init] autorelease]; }

// - dislay info -
CFStringRef UserNotificationTitle( UserNotificationRef ref )
{ return (CFStringRef)[ref title]; }

void UserNotificationSetTitle( UserNotificationRef ref, CFStringRef title )
{ [ref setTitle:(NSString *)title]; }

CFStringRef UserNotificationSubtitle( UserNotificationRef ref )
{ return (CFStringRef)[ref subtitle]; }

void UserNotificationSetSubtitle( UserNotificationRef ref, CFStringRef subtitle )
{ [ref setSubtitle:(NSString *)subtitle]; }

CFStringRef UserNotificationInformativeText( UserNotificationRef ref )
{ return (CFStringRef)[ref informativeText]; }

void UserNotificationSetInformativeText( UserNotificationRef ref, CFStringRef infoText )
{ [ref setInformativeText:(NSString *)infoText]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
ImageRef UserNotificationContentImage( UserNotificationRef ref )
{ return [ref contentImage]; }

void UserNotificationSetContentImage( UserNotificationRef ref, ImageRef image )
{ [ref setContentImage:image]; }

CFStringRef UserNotificationIdentifier( UserNotificationRef ref )
{ return (CFStringRef)[ref identifier]; }

void UserNotificationSetIdentifier( UserNotificationRef ref, CFStringRef identifier )
{ [ref setIdentifier:(NSString *)identifier]; }

CFAttributedStringRef UserNotificationResponse( UserNotificationRef ref )
{ return (CFAttributedStringRef)[ref response]; }

CFStringRef UserNotificationResponsePlaceholder( UserNotificationRef ref )
{ return (CFStringRef)[ref responsePlaceholder]; }
#endif

// - displayed buttons -
Boolean UserNotificationHasActionButton( UserNotificationRef ref )
{ return [ref hasActionButton]; }

void UserNotificationSetHasActionButton( UserNotificationRef ref, Boolean flag )
{ [ref setHasActionButton:flag]; }

CFStringRef UserNotificationActionButtonTitle( UserNotificationRef ref )
{ return (CFStringRef)[ref actionButtonTitle]; }

void UserNotificationSetActionButtonTitle( UserNotificationRef ref, CFStringRef title )
{ [ref setActionButtonTitle:(NSString *)title]; }

CFStringRef UserNotificationOtherButtonTitle( UserNotificationRef ref )
{ return (CFStringRef)[ref otherButtonTitle]; }

void UserNotificationSetOtherButtonTitle( UserNotificationRef ref, CFStringRef title )
{ [ref setOtherButtonTitle:(NSString *)title]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
Boolean UserNotificationHasReplyButton( UserNotificationRef ref )
{ return [ref hasReplyButton]; }

void UserNotificationSetHasReplyButton( UserNotificationRef ref, Boolean flag )
{ [ref setHasReplyButton:flag]; }
#endif

// - delivery timing -
CFDateRef UserNotificationDeliveryDate( UserNotificationRef ref )
{ return (CFDateRef)[ref deliveryDate]; }

void UserNotificationSetDeliveryDate( UserNotificationRef ref, CFDateRef dt )
{ [ref setDeliveryDate:(NSDate *)dt]; }

CFDateRef UserNotificationActualDeliveryDate( UserNotificationRef ref )
{ return (CFDateRef)[ref actualDeliveryDate]; }

DateComponentsRef UserNotificationDeliveryRepeatInterval( UserNotificationRef ref )
{ return [ref deliveryRepeatInterval]; }

void UserNotificationSetDeliveryRepeatInterval( UserNotificationRef ref, DateComponentsRef components )
{ [ref setDeliveryRepeatInterval:components]; }

CFTimeZoneRef UserNotificationDeliveryTimeZone( UserNotificationRef ref )
{ return (CFTimeZoneRef)[ref deliveryTimeZone]; }

void UserNotificationSetDeliveryTimeZone( UserNotificationRef ref, CFTimeZoneRef zone )
{ [ref setDeliveryTimeZone:(NSTimeZone *)zone]; }

// - delivery info -
Boolean UserNotificationIsPresented( UserNotificationRef ref )
{ return [ref isPresented]; }

Boolean UserNotificationIsRemote( UserNotificationRef ref )
{ return [ref isRemote]; }

CFStringRef UserNotificationSoundName( UserNotificationRef ref )
{ return (CFStringRef)[ref soundName]; }

void UserNotificationSetSoundName( UserNotificationRef ref, CFStringRef name )
{ [ref setSoundName:(NSString *)name]; }

// - activation method -
NSUserNotificationActivationType UserNotificationActivationType( UserNotificationRef ref )
{ return [ref activationType]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
UserNotificationActionRef UserNotificationAdditionalActivationAction( UserNotificationRef ref )
{ return [ref additionalActivationAction]; }

CFArrayRef UserNotificationAdditionalActions( UserNotificationRef ref )
{ return (CFArrayRef)[ref additionalActions]; }

void UserNotificationSetAdditionalActions( UserNotificationRef ref, CFArrayRef actions )
{ [ref setAdditionalActions:(NSArray *)actions]; }
#endif

// - user info -
CFDictionaryRef UserNotificationUserInfo( UserNotificationRef ref )
{ return (CFDictionaryRef)[ref userInfo]; }

void UserNotificationSetUserInfo( UserNotificationRef ref, CFDictionaryRef userInfo )
{ [ref setUserInfo:(NSDictionary *)userInfo]; }

// - convenience -
UserNotificationRef UserNotificationWithTitle( CFStringRef title, CFStringRef subtitle,  CFStringRef infoText, CFStringRef sndName )
{
    NSUserNotification *note = [[[NSUserNotification alloc] init] autorelease];
    if ( title ) [note setTitle:(NSString *)title];
    if ( subtitle ) [note setSubtitle:(NSString *)subtitle];
    if ( infoText ) [note setInformativeText:(NSString *)infoText];
    if ( sndName ) [note setSoundName:(NSString *)sndName];
    return note;
}
#endif

