/*
 View.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

@interface ViewSC : NSView
@end

typedef CGPoint (*CocoaViewShowDefinitionBaselineOriginProviderType)(CFRange,void*);

@interface NSView (Additions)
// suspendDrawRect
@property (nonatomic,retain) NSNumber *suspendDrawRectNumber;
- (BOOL)suspendDrawRect;
- (void)setSuspendDrawRect:(BOOL)flag;

// flipFlag
@property (nonatomic,retain) NSNumber *flipFlagNumber;
- (BOOL)flipFlag;
- (void)setFlipFlag:(BOOL)flag;

// responderFlag
@property (nonatomic,retain) NSNumber *responderFlagNumber;
- (BOOL)responderFlag;
- (void)setResponderFlag:(BOOL)flag;

// dragFileTypes
@property (nonatomic,retain) NSArray *dragFileTypesArray;
- (NSArray *)dragFileTypes;
- (void)setDragFileTypes:(NSArray *)types;

// dragFileExtensions
@property (nonatomic,retain) NSArray *dragFileExtensionsArray;
- (NSArray *)dragFileExtensions;
- (void)setDragFileExtensions:(NSArray *)extensions;

// dragInsideFlag
@property (nonatomic,retain) NSNumber *dragInsideFlagNumber;
- (BOOL)dragInsideFlag;
- (void)setDragInsideFlag:(BOOL)flag;

@end


// dialog events
enum {
    viewDrawRect = 2000,
    
    // first responder
    viewAcceptsFirstResponder,
    
    viewMouseDown,
    viewMouseDragged,
    viewMouseUp,
    viewMouseMoved,
    viewMouseEntered,
    viewMouseExited,
    viewRightMouseDown,
    viewRightMouseDragged,
    viewRightMouseUp,
    viewKeyDown,
    viewKeyUp,
    viewFlagsChanged,
    viewUpdateTrackingAreas,
    viewResetCursorRects,
    viewDraggingEntered,
    viewWantsPeriodicDraggingUpdates,
    viewDraggingUpdated,
    viewDraggingEnded,
    viewDraggingExited,
    viewPrepareForDragOperation,
    viewPerformDragOperation,
    viewConcludeDragOperation,
    viewUpdateDraggingItemsForDrag,
    viewDidChangeEffectiveAppearance
};



typedef NSComparisonResult (*CocoaSortSubviewsFunctionType)(NSInteger,NSInteger,void*);
NSComparisonResult CocoaSortSubviewsFunction( NSView *view1, NSView *view2, void *context );

id ViewSubviewWithIdentifier( id superview, CFStringRef identifier );

NSInteger CocoaViewTag( id view );
id CocoaViewSubviewWithTag( NSView *superview, NSInteger tag );
NSView *CocoaViewWithTag( NSInteger tag );
NSInteger ViewSuperview( NSInteger tag );
CFArrayRef ViewSubviews( NSInteger tag );
NSInteger ViewWindow( NSInteger tag );
void ViewSetFrameOrigin( NSInteger tag, CGPoint origin );
void ViewSetFrameSize( NSInteger tag, CGSize size );
void ViewSetFrameRotation( NSInteger tag, CGFloat rotation );
CGRect ViewBounds( NSInteger tag );
void ViewSetBounds( NSInteger tag, CGRect bounds );
void ViewSetBoundsOrigin( NSInteger tag, CGPoint origin );
void ViewSetBoundsSize( NSInteger tag, CGSize size );
void ViewSetBoundsRotation( NSInteger tag, CGFloat rotation );
CGFloat ViewFrameRotation( NSInteger tag );
CGFloat ViewBoundsRotation( NSInteger tag );


// Layer
void ViewSetWantsLayer( NSInteger tag, Boolean flag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
Boolean ViewWantsUpdateLayer( NSInteger tag );
#endif
CALayerRef ViewLayer( NSInteger tag );
void ViewSetLayer( NSInteger tag, CALayerRef layer );
CALayerRef ViewMakeBackingLayer( NSInteger tag );
NSViewLayerContentsPlacement ViewLayerContentsPlacement( NSInteger tag );
void ViewSetLayerContentsPlacement( NSInteger tag, NSViewLayerContentsPlacement placement );
NSViewLayerContentsRedrawPolicy ViewLayerContentsRedrawPolicy( NSInteger tag );
void ViewSetLayerContentsRedrawPolicy( NSInteger tag, NSViewLayerContentsRedrawPolicy policy );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
Boolean ViewCanDrawSubviewsIntoLayer( NSInteger tag );
void ViewSetCanDrawSubviewsIntoLayer( NSInteger tag, Boolean flag );
Boolean ViewLayerUsesCoreImageFilters( NSInteger tag );
void ViewSetLayerUsesCoreImageFilters( NSInteger tag, Boolean flag );
#endif// 1090


CGFloat ViewAlphaValue( NSInteger tag );
void ViewSetAlphaValue( NSInteger tag, CGFloat value );
CGFloat ViewFrameCenterRotation( NSInteger tag );
void ViewSetFrameCenterRotation( NSInteger tag, CGFloat rotation );
CFArrayRef ViewBackgroundFilters( NSInteger tag );
void ViewSetBackgroundFilters( NSInteger tag, CFArrayRef filters );
CIFilterRef ViewCompositingFilter( NSInteger tag );
void ViewSetCompositingFilter( NSInteger tag, CIFilterRef filter );
CFArrayRef ViewContentFilters( NSInteger tag );
void ViewSetContentFilters( NSInteger tag, CFArrayRef filters );
ShadowRef ViewShadow( NSInteger tag );
void ViewSetShadow( NSInteger tag, ShadowRef shadow );

// Drawing
Boolean ViewCanDrawConcurrently( NSInteger tag );
void ViewSetCanDrawConcurrently( NSInteger tag, Boolean flag );
CGRect ViewVisibleRect( NSInteger tag );
Boolean ViewNeedsToDrawRect( NSInteger tag, CGRect r );
Boolean ViewWantsDefaultClipping( NSInteger tag );
BitmapImageRepRef ViewBitmapImageRepForCachingDisplayInRect( NSInteger tag, CGRect r );
void ViewCacheDisplayInRectToBitmapImageRep( NSInteger tag, CGRect r, BitmapImageRepRef repRef );

// Print
void ViewPrint( NSInteger tag );
void ViewSetNeedsDisplay( NSInteger tag );
Boolean ViewIsOpaque( NSInteger tag );

// Converting coordinate values
CGPoint ViewConvertPointFromView( NSInteger tag1, CGPoint pt, NSInteger tag2 );

void ViewRotateByAngle( NSInteger tag, CGFloat angle );

void ViewAddSubview( NSInteger viewTag, NSInteger subviewTag );
void ViewAddSubviewPositioned( NSInteger superviewTag, NSInteger subviewTag, NSWindowOrderingMode position, NSInteger otherViewTag );
void ViewRemoveFromSuperview( NSInteger tag );

//void ViewReplaceSubview( NSInteger tag, NSInteger subview1, NSInteger subview2 );
Boolean ViewIsDescendantOf( NSInteger tag, NSInteger superTag );
NSInteger ViewOpaqueAncestor( NSInteger tag );
NSInteger ViewAncestorSharedWithView( NSInteger tag, NSInteger otherTag );
void ViewSortSubviewsUsingFunction( NSInteger tag, void *fnAddress, void *context );

Boolean ViewAutoresizesSubviews( NSInteger tag );
void ViewSetAutoresizesSubviews( NSInteger tag, Boolean flag );
NSUInteger ViewAutoresizingMask( NSInteger tag );
void ViewSetAutoresizingMask( NSInteger tag, NSUInteger mask );


// Layout anchors
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
LayoutYAxisAnchorRef ViewBottomAnchor( NSInteger tag );
LayoutXAxisAnchorRef ViewCenterXAnchor( NSInteger tag );
LayoutYAxisAnchorRef ViewCenterYAnchor( NSInteger tag );
LayoutYAxisAnchorRef ViewFirstBaselineAnchor( NSInteger tag );
LayoutDimensionRef ViewHeightAnchor( NSInteger tag );
LayoutYAxisAnchorRef ViewLastBaselineAnchor( NSInteger tag );
LayoutXAxisAnchorRef ViewLeadingAnchor( NSInteger tag );
LayoutXAxisAnchorRef ViewLeftAnchor( NSInteger tag );
LayoutXAxisAnchorRef ViewRightAnchor( NSInteger tag );
LayoutYAxisAnchorRef ViewTopAnchor( NSInteger tag );
LayoutXAxisAnchorRef ViewTrailingAnchor( NSInteger tag );
LayoutDimensionRef ViewWidthAnchor( NSInteger tag );
#endif// 101100

// Constraints
CFArrayRef ViewConstraints( NSInteger tag );
void ViewAddConstraint( NSInteger tag, LayoutConstraintRef constraint );
void ViewAddConstraints( NSInteger tag, CFArrayRef constraints );
void ViewRemoveConstraint( NSInteger tag, LayoutConstraintRef constraint );
void ViewRemoveConstraints( NSInteger tag, CFArrayRef constraints );

// Layout guides
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void ViewAddLayoutGuide( NSInteger tag, LayoutGuideRef guide );
CFArrayRef ViewLayoutGuides( NSInteger tag );
void ViewRemoveLayoutGuide( NSInteger tag, LayoutGuideRef guide );
#endif// 101100

// Measuring in auto layout
CGSize ViewFittingSize( NSInteger tag );
CGSize ViewIntrinsicContentSize( NSInteger tag );
void ViewInvalidateIntrinsicContentSize( NSInteger tag );


void ViewLockFocus( NSInteger tag );
void ViewUnlockFocus( NSInteger tag );
NSInteger ViewFocusView( void );

// Vibrancy
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean ViewAllowsVibrancy( NSInteger tag );
#endif

CGRect ViewFrame( NSInteger tag );
void ViewSetFrame( NSInteger tag, CGRect r );
NSInteger ViewTag( ViewRef view );
void ViewSetToolTip( NSInteger tag, CFStringRef string );

// Tracking rect
NSTrackingRectTag ViewAddTrackingRect( NSInteger tag, CGRect r, id owner, void *userData, Boolean assumeInside );
void ViewRemoveTrackingRect( NSInteger tag, NSTrackingRectTag rectTag );

// Tracking area
void ViewAddTrackingArea( NSInteger tag, TrackingAreaRef ta );
void ViewRemoveTrackingArea( NSInteger tag, TrackingAreaRef ta );
CFArrayRef ViewTrackingAreas( NSInteger tag );
//void ViewUpdateTrackingAreas( NSInteger tag );

// Cursor tracking
void ViewAddCursorRect( NSInteger tag, CGRect r, CursorRef curs );
void ViewRemoveCursorRect( NSInteger tag, CGRect r, CursorRef curs );
void ViewDiscardCursorRects( NSInteger tag );

// Definition windows
void ViewShowDefinition( NSInteger tag, CFAttributedStringRef aString, CGPoint pt );
void ViewShowDefinitionBaselineOriginProvider( NSInteger tag, CFAttributedStringRef aString, CFRange targetRange, CFDictionaryRef options, CocoaViewShowDefinitionBaselineOriginProviderType originProvider, void *userData );

Boolean ViewIsHidden( NSInteger tag );
void ViewSetHidden( NSInteger tag, Boolean flag );
Boolean ViewIsHiddenOrHasHiddenAncestor( NSInteger tag );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
// Gesture recognizer
CFArrayRef ViewGestureRecognizers( NSInteger tag );
void ViewAddGestureRecognizer( NSInteger tag, GestureRecognizerRef ref );
void ViewRemoveGestureRecognizer( NSInteger tag, GestureRecognizerRef ref );
#endif

// Event handling
NSInteger ViewHitTest( NSInteger tag, CGPoint pt );


NSInteger ViewNextKeyView( NSInteger tag );
void ViewSetNextKeyView( NSInteger tag, NSInteger nextViewTag );
NSInteger ViewNextValidKeyView( NSInteger tag );
NSInteger ViewPreviousKeyView( NSInteger tag );
NSInteger ViewPreviousValidKeyView( NSInteger tag );
void ViewScrollRect( NSInteger tag, CGRect r, CGSize size );

NSInteger ViewEnclosingScrollView( NSInteger tag );

void ViewRegisterForDraggedTypes( NSInteger tag, CFArrayRef types );
void ViewSetNeedsDisplayInRect( NSInteger tag, CGRect r );
void ViewSetAcceptsFirstResponder( NSInteger tag, Boolean flag );

// Notifications
Boolean ViewPostsFrameChangedNotifications( NSInteger tag );
void ViewSetPostsFrameChangedNotifications( NSInteger tag, Boolean flag );
Boolean ViewPostsBoundsChangedNotifications( NSInteger tag );
void ViewSetPostsBoundsChangedNotifications( NSInteger tag, Boolean flag );

// custom
void ViewRegisterForDraggedFileTypes( NSInteger tag, CFTypeRef inFileTypes );
void ViewRegisterForDraggedFileExtensions( NSInteger tag, CFTypeRef inFileExtensions );
BOOL ViewDragInside( NSInteger tag );
void ViewSetDragInside( NSInteger tag, BOOL flag );

// generic functions
//CFStringRef ViewText( NSInteger tag );

// animator
void *ViewAnimator( NSInteger tag );
void ViewAnimatorSetFrame( NSInteger tag, CGRect r );
void ViewAnimatorSetFrameOrigin( NSInteger tag, CGPoint origin );
void ViewAnimatorSetFrameSize( NSInteger tag, CGSize size );
void ViewAnimatorSetFrameRotation( NSInteger tag, CGFloat rot );
void ViewAnimatorSetBounds( NSInteger tag, CGRect r );
void ViewAnimatorSetBoundsOrigin( NSInteger tag, CGPoint origin );
void ViewAnimatorSetBoundsSize( NSInteger tag, CGSize size );
void ViewAnimatorSetBoundsRotation( NSInteger tag, CGFloat rot );
void ViewAnimatorSetFrameCenterRotation( NSInteger tag, CGFloat rot );
void ViewAnimatorSetAlphaValue( NSInteger tag, CGFloat value );

//void ViewSetTrackingArea( NSInteger tag, CGRect r, NSTrackingAreaOptions options );

// custom
id ViewWithTag( NSInteger tag );

Boolean ViewIsFlipped( NSInteger tag );
void ViewSetFlipped( NSInteger tag, Boolean flag );
void ViewsEmbedInView( NSInteger tag, ... );


// Create
ViewRef ViewInit( NSInteger inTag, CGRect inRect );

// Statement
void CocoaView( NSInteger inTag, CGRect inRect, NSInteger subclass, NSInteger inWndTag );


// gesture recognizer
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000

// - user dialog -
void ViewSetClickGestureRecognizer( NSInteger tag );
void ViewSetMagnificationGestureRecognizer( NSInteger tag );
void ViewSetPanGestureRecognizer( NSInteger tag );
void ViewSetPressGestureRecognizer( NSInteger tag );
void ViewSetRotationGestureRecognizer( NSInteger tag );

// - callback function -
void ViewSetClickGestureRecognizerCallback( NSInteger tag, void *callback, void *userData );
void ViewSetMagnificationGestureRecognizerCallback( NSInteger tag, void *callback, void *userData );
void ViewSetPanGestureRecognizerCallback( NSInteger tag, void *callback, void *userData );
void ViewSetPressGestureRecognizerCallback( NSInteger tag, void *callback, void *userData );
void ViewSetRotationGestureRecognizerCallback( NSInteger tag, void *callback, void *userData );
#endif

// Appearance
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ViewSetAppearance( NSInteger tag, AppearanceRef appearance );
void ViewSetAppearanceNamed( NSInteger tag, CFStringRef name );
#endif

/*
 NSAppearanceCustomization
 */
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
AppearanceRef ViewAppearance( NSInteger tag );
AppearanceRef ViewEffectiveAppearance( NSInteger tag );
#endif

// - custom -
Boolean ViewExists( NSInteger tag );

CFTypeRef ViewProperty( NSInteger tag, CFStringRef key );
void ViewSetProperty( NSInteger tag, CFStringRef key, CFTypeRef value );
void ViewRemoveProperty( NSInteger tag, CFStringRef key );
void ViewRemoveAllProperties( NSInteger tag );

// Suspend drawRect: method
void ViewSuspendDrawRect( NSInteger tag, Boolean flag );
Boolean ViewDrawRectIsSuspended( NSInteger tag );
