/*
 VisualEffectView.m

 Bernie Wylde
 */

#import "VisualEffectView.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000

@implementation VisualEffectViewSC

//- (void)drawLayer:(CALayer *)theLayer inContext:(CGContextRef)theContext {
//    CallUserDefinedOnDialogFunction( CALayerDraw, 0, 0, NULL );
//}

//- (void)setTrackingAreaRect:(NSRect)rect options:(NSTrackingAreaOptions)options {
//    NSTrackingArea *trackingArea = [[NSTrackingArea alloc] initWithRect:rect options:options owner:self userInfo:nil];
//    [self addTrackingArea:trackingArea];
//    [trackingArea release];
//}

//- (void)viewWillMoveToWindow:(NSWindow *)newWindow {
//    NSTrackingAreaOptions options = NSTrackingMouseEnteredAndExited | NSTrackingMouseMoved | NSTrackingActiveAlways;
//    NSTrackingArea *trackingArea = [[NSTrackingArea alloc] initWithRect:[self bounds] options:options owner:self userInfo:nil];
//    [self addTrackingArea:trackingArea];
//    //[trackingArea release];
//}

- (void)updateTrackingAreas {
    NSArray *areas = [self trackingAreas];
    if ( [areas count] > 0 ) {
        NSTrackingArea *trackingArea = [areas objectAtIndex:0];
        [self removeTrackingArea:trackingArea];
        [trackingArea release];
        NSTrackingAreaOptions options = NSTrackingMouseEnteredAndExited | NSTrackingMouseMoved | NSTrackingActiveAlways;
        trackingArea = [[NSTrackingArea alloc] initWithRect:[self bounds] options:options owner:self userInfo:nil];
        [self addTrackingArea:trackingArea];
        //[trackingArea release];
    }
    [super updateTrackingAreas];
}

- (void)drawRect:(NSRect)dirtyRect {
    if ( ![self suspendDrawRect] ) {
        CocoaSubclassDrawRect( self, dirtyRect );
        if ( !DialogEventBool() ) [super drawRect:dirtyRect];
    }
}

// this is required for key events
//- (BOOL)acceptsFirstResponder {
//    return self.responderFlag;
//}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
    //    if ( !DialogEventBool() ) [super keyUp:event];
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
    //    if ( !DialogEventBool() ) [super flagsChanged:event];
}

//- (void)updateTrackingAreas {
//    CocoaSubclassUpdateTrackingAreas( self );
//}

- (void)resetCursorRects {
    //[super resetCursorRects];
    CocoaSubclassResetCursorRects( self );
}

#pragma mark - dragging
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender {
    return CocoaSubclassDraggingEntered( self, sender );
}

//- (BOOL)wantsPeriodicDraggingUpdates {
//    return NO;
//}

//- (NSDragOperation)draggingUpdated:(id<NSDraggingInfo>)sender {
//
//    return NSDragOperationNone;
//}

- (void)draggingEnded:(id<NSDraggingInfo>)sender {
    
}

- (void)draggingExited:(id <NSDraggingInfo>)sender {
    CocoaSubclassDraggingExited( self, sender );
}

//- (BOOL)prepareForDragOperation:(id<NSDraggingInfo>)sender {
//    return NO;
//}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender {
    return CocoaSubclassPerformDragOperation( self, sender );
}

- (void)concludeDragOperation:(id<NSDraggingInfo>)sender {
    
}

- (void)updateDraggingItemsForDrag:(id<NSDraggingInfo>)sender {
    
}

- (void)viewDidChangeEffectiveAppearance {
    CocoaSubclassViewDidChangeEffectiveAppearance( self );
}

#pragma mark - flipped
- (BOOL)isFlipped {
    //return flip;
    return self.flipFlag;
}
@end

VisualEffectViewRef VisualEffectViewWithTag( NSInteger tag )
{ return (NSVisualEffectView *)CocoaValidateViewKindOfClass( tag, [NSVisualEffectView class] ); }

Boolean VisualEffectViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [NSVisualEffectView class] ); }

// Functions
NSVisualEffectMaterial VisualEffectViewMaterial( NSInteger tag )
{
    NSVisualEffectView *view = VisualEffectViewWithTag( tag );
    if ( view ) return [view material];
    return 0;
}

void VisualEffectViewSetMaterial( NSInteger tag, NSVisualEffectMaterial material )
{
    NSVisualEffectView *view = VisualEffectViewWithTag( tag );
    if ( view ) [view setMaterial:material];
}

NSBackgroundStyle VisualEffectViewInteriorBackgroundStyle( NSInteger tag )
{
    NSVisualEffectView *view = VisualEffectViewWithTag( tag );
    if ( view ) return [view interiorBackgroundStyle];
    return 0;
}

NSVisualEffectBlendingMode VisualEffectViewBlendingMode( NSInteger tag )
{
    NSVisualEffectView *view = VisualEffectViewWithTag( tag );
    if ( view ) return [view blendingMode];
    return 0;
}

void VisualEffectViewSetBlendingMode( NSInteger tag, NSVisualEffectBlendingMode mode )
{
    NSVisualEffectView *view = VisualEffectViewWithTag( tag );
    if ( view ) [view setBlendingMode:mode];
}

NSVisualEffectState VisualEffectViewState( NSInteger tag )
{
    NSVisualEffectView *view = VisualEffectViewWithTag( tag );
    if ( view ) return [view state];
    return 0;
}

void VisualEffectViewSetState( NSInteger tag, NSVisualEffectState state )
{
    NSVisualEffectView *view = VisualEffectViewWithTag( tag );
    if ( view ) [view setState:state];
}

ImageRef VisualEffectViewMaskImage( NSInteger tag )
{
    NSVisualEffectView *view = VisualEffectViewWithTag( tag );
    if ( view ) return [view maskImage];
    return nil;
}

void VisualEffectViewSetMaskImage( NSInteger tag, ImageRef image )
{
    NSVisualEffectView *view = VisualEffectViewWithTag( tag );
    if ( view ) [view setMaskImage:image];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean VisualEffectViewIsEmphasized( NSInteger tag )
{
    NSVisualEffectView *view = VisualEffectViewWithTag( tag );
    if ( view ) return [view isEmphasized];
    return NO;
}

void VisualEffectViewSetEmphasized( NSInteger tag, Boolean flag )
{
    NSVisualEffectView *view = VisualEffectViewWithTag( tag );
    if ( view ) [view setEmphasized:flag];
}
#endif// >= 10.12

// Init
VisualEffectViewRef VisualEffectViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 128, 128 );

    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    NSVisualEffectView *view;
    if ( subclass ) {
        view = [[[VisualEffectViewSC alloc] initWithFrame:rect] autorelease];
    } else {
        view = [[[NSVisualEffectView alloc] initWithFrame:rect] autorelease];
    }

    // material
    [view setMaterial:NSVisualEffectMaterialAppearanceBased];
    
    // mode
    [view setBlendingMode:NSVisualEffectBlendingModeBehindWindow];
    
    // appearance
    [view setAppearance:[NSAppearance appearanceNamed:NSAppearanceNameVibrantDark]];

    // tag
    [view setIdentifier:[NSString stringWithFormat:@"%ld",(long)inTag]];

    return view;
}

// Statement
void CocoaVisualEffectView( NSInteger inTag, CGRect inRect, NSInteger inMaterial, NSInteger inMode, NSInteger inState, void *inImage, NSInteger inEmphasized, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // imageName, imagePath, NSImage
    NSImage *image = FixImageValue( inImage );

    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        NSVisualEffectView *view = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !view ) {
            view = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( view ) {    // already exists
            
            // make sure it's same view class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [view class], [NSVisualEffectView class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, view, [NSVisualEffectView class] ) ) {
                return;
            }

            // rect
            if ( !NSIsEmptyRect( rect ) ) [view setFrame:rect];
            
            // material
            if ( inMaterial != kFBParamMissing ) [view setMaterial:inMaterial];

            // mode
            if ( inMode != kFBParamMissing ) [view setBlendingMode:inMode];

            // state
            if ( inState != kFBParamMissing ) [view setState:inState];

            // image
            if ( image ) [view setMaskImage:image];

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
            // emphasized
            if ( inEmphasized != kFBParamMissing ) [view setEmphasized:inEmphasized];
#endif

        } else {            // create
            // rect
//            if ( NSIsEmptyRect( rect ) ) {
//                rect = NSMakeRect( 20, 20, 128, 128 );
//            }
            
            // init
//            if ( inSubclass ) {
//                view = [[VisualEffectViewSC alloc] initWithFrame:rect];
//            } else {
//                view = [[NSVisualEffectView alloc] initWithFrame:rect];
//            }
            view = VisualEffectViewInit( inTag, inRect );
            
            // material
//            if ( inMaterial == kFBParamMissing ) {
//                inMaterial = NSVisualEffectMaterialAppearanceBased;
//            }
            if ( inMaterial != kFBParamMissing ) [view setMaterial:inMaterial];

            // mode
            if ( inMode != kFBParamMissing ) [view setBlendingMode:inMode];

            // state
            if ( inState != kFBParamMissing ) [view setState:inState];

            // image
            if ( image ) [view setMaskImage:image];

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
            // emphasized
            if ( inEmphasized != kFBParamMissing ) [view setEmphasized:inEmphasized];
#endif
            
            // appearance
//            [view setAppearance:[NSAppearance appearanceNamed:NSAppearanceNameVibrantDark]];

            // autoresizing
            [view setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
            
            // tag
//            [view setIdentifier:[NSString stringWithFormat:@"%ld",(long)inTag]];

            // add to superview
            [contentView addSubview:view];
            
            // wndTag
            [view setWndTag:inWndTag];
            
            //[view release];
        }
        [view setHidden:!show];
    } else {
        CocoaShowAlert( NSWarningAlertStyle, @"Window error.", [NSString stringWithFormat:@"Cocoa window %ld does not exist.",(long)inWndTag] );
    }
}

#endif// >= 10.10

