/*
 Window.m

 Bernie Wylde
 */

#import "Window.h"

#pragma mark - titlebar accessory view
///////////////////////////////////////////////////
//          titlebar accessory view              //
///////////////////////////////////////////////////
id TitlebarAccessoryViewWithTag( NSInteger tag )
{
    id view = nil;
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
    id w = CocoaOutputWindow();
    if ( [w isKindOfClass:[NSWindow class]] ) {
        if ( [w hasTitleBar] ) {
            NSArray *controllers = [w titlebarAccessoryViewControllers];
            if ( controllers ) {
                for ( NSTitlebarAccessoryViewController *c in controllers ) {
                    NSView *view = [c view];
                    NSInteger viewTag = [[view identifier] integerValue];
                    if ( viewTag == tag ) return view;
                    NSView *subview = CocoaViewSubviewWithTag( view, tag );
                    if ( subview ) return subview;
                }
            }
        }
    }
#endif
    return view;
}

#pragma mark - window
///////////////////////////////////////////////////
//                  cocoa window                 //
///////////////////////////////////////////////////
#pragma mark - subclass
@implementation WindowSC

- (void)mouseDown:(NSEvent *)event {
    CocoaWindowSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaWindowSubclassMouseUp( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaWindowSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaWindowSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaWindowSubclassFlagsChanged( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaWindowSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaWindowSubclassMouseExited( self, event );
}

@end


#pragma mark - output window
static NSInteger sCocoaOutputWindowTag;

NSInteger CocoaOutputWindowTag( void )
{ return sCocoaOutputWindowTag; }

void CocoaOutputWindowSetTag( NSInteger tag )
{ sCocoaOutputWindowTag = tag; }

id CocoaVisibleOutputWindow( void )
{
    id w = nil;
    NSInteger tag = CocoaOutputWindowTag();
    if ( tag ) {
        w = CocoaVisibleWindowWithTag( tag );
        if ( !w ) {
            w = CocoaPopoverWithTag( tag );
        }
    }
    return w;
}

id CocoaOutputWindow( void )
{
    id w = nil;
    NSInteger tag = CocoaOutputWindowTag();
    if ( tag ) {
        w = CocoaWindowWithTag( tag );
        
        if ( !w ) {
            w = CocoaPopoverWithTag( tag );
        }
        
    }
    return w;
}

NSView *CocoaOutputWindowContentView( void )
{
    NSView *contentView = nil;

    id w = CocoaOutputWindow();
    if ( w ) {
        
        if ( [w isKindOfClass:[NSWindow class]] ) {
            contentView = [w contentView];
        } else if ( [w isKindOfClass:[NSPopover class]] ) {
            contentView = [[w contentViewController] view];
        } else {
            // ... it's not a window, panel or popover
        }
    }
    
    return contentView;
}

NSInteger CocoaParentTagForView( id view )
{
    NSPopover *popover = CocoaPopoverForView( [view superview] );
    if ( popover ) {
        return [popover tag];
    } else {
        return [[view window] tag];
    }
}



id CocoaTargetWithTag( NSInteger tag )
{
    id target = CocoaWindowWithTag( tag );
    if ( !target ) {
        target = CocoaPopoverWithTag( tag );
    }
    return target;
}






id CocoaVisibleTargetWithTag( NSInteger tag )
{
    id target = CocoaVisibleWindowWithTag( tag );
    if ( !target ) {
        target = CocoaPopoverWithTag( tag );
    }
    return target;
}

id CocoaVisibleTargetWindow( NSInteger *inWndTag )
{
    id w = nil;
    if ( *inWndTag ) {
        w = CocoaVisibleTargetWithTag( *inWndTag ); // get window or popover
    } else {
        w = CocoaOutputWindow();
        if ( w ) {
            if ( ![w isVisible] ) {
                w = CocoaVisibleFrontWindow();
            }
        } else {
            w = CocoaVisibleFrontWindow();
        }
        if ( w ) *inWndTag = [w tag];
    }
    return w;
}







id CocoaTargetWindow( NSInteger *inWndTag )
{
    id w = nil;
    if ( *inWndTag ) {
        w = CocoaTargetWithTag( *inWndTag ); // get window or popover
    } else {
        w = CocoaOutputWindow();
        if ( !w ) {
            w = CocoaFrontWindow();
        }
        if ( w ) *inWndTag = [w tag];
    }
    return w;
}

NSView *CocoaTargetContentView( id w )
{
    NSView *contentView = nil;
    
    if ( [w isKindOfClass:[NSWindow class]] ) {
        contentView = [w contentView];
    } else if ( [w isKindOfClass:[NSPopover class]] ) {
        contentView = [[w contentViewController] view];
    } else {
        // ... it isn't a window, panel or popover
    }
    return contentView;
}


#pragma mark - ContentViewSC
@implementation ContentViewSC

@synthesize flip;
@synthesize subclassFlag;
@synthesize responderFlag;
@synthesize dragFileTypes;
@synthesize dragFileExtensions;
@synthesize dragInside;

- (void)viewWillMoveToWindow:(NSWindow *)newWindow {
    NSTrackingAreaOptions options = NSTrackingMouseEnteredAndExited | NSTrackingMouseMoved | NSTrackingActiveAlways;
    NSTrackingArea *trackingArea = [[NSTrackingArea alloc] initWithRect:[self bounds] options:options owner:self userInfo:nil];
    [self addTrackingArea:trackingArea];
    //[trackingArea release];
}

- (void)updateTrackingAreas {
    NSArray *areas = [self trackingAreas];
    if ( [areas count] > 0 ) {
        NSTrackingArea *trackingArea = [areas objectAtIndex:0];
        [self removeTrackingArea:trackingArea];
        [trackingArea release];
        NSTrackingAreaOptions options = NSTrackingMouseEnteredAndExited | NSTrackingMouseMoved | NSTrackingActiveAlways;
        trackingArea = [[NSTrackingArea alloc] initWithRect:[self bounds] options:options owner:self userInfo:nil];
        [self addTrackingArea:trackingArea];
        //[trackingArea release];
    }
    [super updateTrackingAreas];
}

- (void)drawRect:(NSRect)dirtyRect {
    if ( [self suspendDrawRect] ) return;
    
    [super drawRect:dirtyRect];
    NSWindow *window = [self window];
    
    if ( subclassFlag ) {
        DialogEventSetNSRect( dirtyRect );
        CocoaOutputWindowSetTag( [window tag] );
        CallUserDefinedOnDialogFunction( viewDrawRect, windowContentViewTag, [window tag], NULL );
    }
    
    // fix focus ring when running in Carbon
    if ( !IsCocoaApplication() ) {
        if ( ![window isKeyWindow] ) return;
        id responder = [window firstResponder];
        if ( responder ) {
            if ( [responder respondsToSelector:@selector(superview)] ) {
                id focusView = [[responder superview] superview];// ugh! I don't like this - bw
                if ( focusView ) {
                    if ( [focusView respondsToSelector:@selector(focusRingType)] ) {
                        if ( [focusView focusRingType] != NSFocusRingTypeNone ) {
                            NSRect rect = [focusView frame];
                            
                            BOOL superIsFlipped = [[focusView superview] isFlipped];
                            
                            // fix focus ring rect for embedded views
                            NSPoint pt = [[focusView superview] convertPoint:[focusView frame].origin toView:nil];
                            rect.origin = pt;
                            
                            // fix focus view origin when embedded in flipped parent view - 20171104 bw
                            if ( superIsFlipped ) {
                                NSRect superFrame = [[focusView superview] frame];
                                rect.origin.y = superFrame.size.height - rect.origin.y;
                            }
                            
                            if ( [focusView class] == [NSTextField class] || [focusView class] == [NSTokenField class] || [focusView class] == [NSSecureTextField class] ) {
                                NSSetFocusRingStyle(NSFocusRingOnly);
                                NSRectFill(rect);
                            } else {
                                NSBezierPath *path = nil;
                                if ( [focusView class] == [NSSearchField class] ) {
                                    rect = NSMakeRect( rect.origin.x-1,rect.origin.y-1,rect.size.width+2,rect.size.height+2);
                                    path = [NSBezierPath bezierPathWithRoundedRect:rect xRadius:5.0 yRadius:5.0];
                                    [path setLineWidth:3];
                                } else if ( [focusView class] == [NSComboBox class] ) {
                                    if ( superIsFlipped ) {
                                        rect = NSMakeRect( rect.origin.x-0.5,rect.origin.y+1,rect.size.width-2,rect.size.height-4);
                                    } else {
                                        rect = NSMakeRect( rect.origin.x-0.5,rect.origin.y+3,rect.size.width-2,rect.size.height-4);
                                    }
                                    path = [NSBezierPath bezierPathWithRoundedRect:rect xRadius:1.5 yRadius:1.5];
                                    [path setLineWidth:4];
                                } else {
                                    // more text type views... (maybe)
                                }
                                if ( path ) {
                                    NSColor *c = [NSColor colorWithCalibratedRed:(131.0/256) green:(180.0/256) blue:(240.0/256) alpha:1.0];
                                    [c set];
                                    [path stroke];
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}

- (void)mouseDown:(NSEvent *)event {
    [super mouseDown:event];
    
    NSWindow *window = [self window];
    if ( [window isKindOfClass:[NSWindow class]] ) {
        // close any transient or semitransient popovers when running in Carbon
        if ( !IsCocoaApplication() ) {
            if ( sCocoaPopovers ) {
                for ( NSDictionary *poDict in sCocoaPopovers ) {
                    NSArray *popovers = [poDict allValues];
                    NSPopover *popover = [popovers objectAtIndex:0];
                    if ( [popover behavior] != NSPopoverBehaviorApplicationDefined ) {
                        if ( [popover isShown] ) [popover close];
                    }
                }
            }
        }
    }
    
    if ( subclassFlag ) {
        CocoaEventSetEvent( event );
        CocoaOutputWindowSetTag( [window tag] );
        CallUserDefinedOnDialogFunction( viewMouseDown, windowContentViewTag, [window tag], NULL );
        CocoaEventSetEvent( nil );
    }
}

- (void)mouseUp:(NSEvent *)event {
    if ( subclassFlag ) {
        NSWindow *window = [self window];
        CocoaEventSetEvent( event );
        CocoaOutputWindowSetTag( [window tag] );
        CallUserDefinedOnDialogFunction( viewMouseUp, windowContentViewTag, [window tag], NULL );
        CocoaEventSetEvent( nil );
    } else {
        [super mouseUp:event];
    }
}

- (void)mouseEntered:(NSEvent *)event {
    if ( subclassFlag ) {
        NSWindow *window = [self window];
        CocoaEventSetEvent( event );
        CocoaOutputWindowSetTag( [window tag] );
        CallUserDefinedOnDialogFunction( viewMouseEntered, windowContentViewTag, [window tag], NULL );
        CocoaEventSetEvent( nil );
    } else {
        [super mouseEntered:event];
    }
}

- (void)mouseExited:(NSEvent *)event {
    if ( subclassFlag ) {
        NSWindow *window = [self window];
        CocoaEventSetEvent( event );
        CocoaOutputWindowSetTag( [window tag] );
        CallUserDefinedOnDialogFunction( viewMouseExited, windowContentViewTag, [window tag], NULL );
        CocoaEventSetEvent( nil );
    } else {
        [super mouseExited:event];
    }
}

- (void)mouseDragged:(NSEvent *)event {
    if ( subclassFlag ) {
        NSWindow *window = [self window];
        CocoaEventSetEvent( event );
        CocoaOutputWindowSetTag( [window tag] );
        CallUserDefinedOnDialogFunction( viewMouseDragged, windowContentViewTag, [window tag], NULL );
        CocoaEventSetEvent( nil );
    } else {
        [super mouseDragged:event];
    }
}

- (void)mouseMoved:(NSEvent *)event {
    if ( subclassFlag ) {
        NSWindow *window = [self window];
        CocoaEventSetEvent( event );
        CocoaOutputWindowSetTag( [window tag] );
        CallUserDefinedOnDialogFunction( viewMouseMoved, windowContentViewTag, [window tag], NULL );
        CocoaEventSetEvent( nil );
    } else {
        [super mouseMoved:event];
    }
}

- (void)rightMouseDown:(NSEvent *)event {
    if ( subclassFlag ) {
        NSWindow *window = [self window];
        CocoaEventSetEvent( event );
        CocoaOutputWindowSetTag( [window tag] );
        CallUserDefinedOnDialogFunction( viewRightMouseDown, windowContentViewTag, [window tag], NULL );
        CocoaEventSetEvent( nil );
    } else {
        [super rightMouseDown:event];
    }
}

- (void)rightMouseUp:(NSEvent *)event {
    if ( subclassFlag ) {
        NSWindow *window = [self window];
        CocoaEventSetEvent( event );
        CocoaOutputWindowSetTag( [window tag] );
        CallUserDefinedOnDialogFunction( viewRightMouseUp, windowContentViewTag, [window tag], NULL );
        CocoaEventSetEvent( nil );
    } else {
        [super rightMouseUp:event];
    }
}

- (void)rightMouseDragged:(NSEvent *)event {
    if ( subclassFlag ) {
        NSWindow *window = [self window];
        CocoaEventSetEvent( event );
        CocoaOutputWindowSetTag( [window tag] );
        CallUserDefinedOnDialogFunction( viewRightMouseDragged, windowContentViewTag, [window tag], NULL );
        CocoaEventSetEvent( nil );
    } else {
        [super rightMouseDragged:event];
    }
}

// this is required for key events
- (BOOL)acceptsFirstResponder {
    return responderFlag;
}

- (void)keyDown:(NSEvent *)event {
    if ( subclassFlag ) {
        NSWindow *window = [self window];
        CocoaEventSetEvent( event );
        CocoaOutputWindowSetTag( [window tag] );
        DialogEventSetBool(NO);
        CallUserDefinedOnDialogFunction( viewKeyDown, windowContentViewTag, [window tag], NULL );
        if ( !DialogEventBool() ) [super keyDown:event];
        CocoaEventSetEvent( nil );
    } else {
        [super keyDown:event];
    }
}

- (void)keyUp:(NSEvent *)event {
    if ( subclassFlag ) {
        NSWindow *window = [self window];
        CocoaEventSetEvent( event );
        CocoaOutputWindowSetTag( [window tag] );
        DialogEventSetBool(NO);
        CallUserDefinedOnDialogFunction( viewKeyUp, windowContentViewTag, [window tag], NULL );
        if ( !DialogEventBool() ) [super keyUp:event];
        CocoaEventSetEvent( nil );
    } else {
        [super keyUp:event];
    }
}

- (void)flagsChanged:(NSEvent *)event {
    if ( subclassFlag ) {
        NSWindow *window = [self window];
        CocoaEventSetEvent( event );
        CocoaOutputWindowSetTag( [window tag] );
        DialogEventSetBool(NO);
        CallUserDefinedOnDialogFunction( viewFlagsChanged, windowContentViewTag, [window tag], NULL );
        if ( !DialogEventBool() ) [super flagsChanged:event];
        CocoaEventSetEvent( nil );
    } else {
        [super flagsChanged:event];
    }
}

- (void)resetCursorRects {
    CocoaSubclassResetCursorRects( self );
}

#pragma mark - dragging
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender {
    NSDragOperation dragOperation = NSDragOperationNone;
        
    DialogEventSetLong(dragOperation);
    NSArray *regTypes = [self registeredDraggedTypes];
    
    if ( [regTypes containsObject:NSFilenamesPboardType] ) {
        NSWindow *window = [self window];
        CocoaOutputWindowSetTag( [window tag] );
        
        NSPasteboard *pb = [sender draggingPasteboard];
        NSArray *pbTypes = [pb types];
        if ( [pbTypes containsObject:NSFilenamesPboardType] ) {
            DialogEventSetDraggingInfo(sender);
            
            NSArray *files = [pb propertyListForType:NSFilenamesPboardType];
            
            BOOL validFile = NO;
            
            if ( dragFileTypes != nil || dragFileExtensions != nil ) {
                for ( NSString *path in files ) {
                    validFile = NO;
                    
                    if ( dragFileTypes ) {
                        NSString *utiType = [[NSWorkspace sharedWorkspace] typeOfFile:path error:nil];
                        for ( NSString *type in dragFileTypes ) {
                            if ( [[NSWorkspace sharedWorkspace] type:utiType conformsToType:(id)type] ) {
                                validFile = YES;
                                break;
                            }
                        }
                    }
                    
                    if ( !validFile ) {
                        if ( dragFileExtensions ) {
                            NSString *extension = [path pathExtension];
                            if ( extension ) {
                                if ( [dragFileExtensions containsObject:extension] ) {
                                    validFile = YES;
                                }
                            }
                        }
                    }
                    
                    if ( !validFile ) break;
                }
            } else {
                validFile = YES;
            }
            
            if ( validFile ) {
                dragOperation = NSDragOperationLink;
                NSMutableArray *urls = [NSMutableArray arrayWithCapacity:[files count]];
                for ( NSString *path in files ) {
                    [urls addObject:[NSURL fileURLWithPath:path]];
                }
                DialogEventSetNSArray( urls );
                DialogEventSetNSURL( [urls objectAtIndex:0] );
                dragInside = YES;
                DialogEventSetLong(dragOperation);
                CallUserDefinedOnDialogFunction( viewDraggingEntered, windowContentViewTag, [window tag], NULL );
                DialogEventSetDraggingInfo(nil);
                DialogEventSetNSArray( nil );
                DialogEventSetNSURL( nil );
            }
        }
    }
    return DialogEventLong();
}

//- (BOOL)wantsPeriodicDraggingUpdates {
//    return NO;
//}

//- (NSDragOperation)draggingUpdated:(id<NSDraggingInfo>)sender {
//
//    return NSDragOperationNone;
//}

- (void)draggingEnded:(id<NSDraggingInfo>)sender {
    
}

- (void)draggingExited:(id <NSDraggingInfo>)sender {
    NSWindow *window = [self window];
    CocoaOutputWindowSetTag( [window tag] );
    DialogEventSetDraggingInfo(sender);
    dragInside = NO;
    CallUserDefinedOnDialogFunction( viewDraggingExited, windowContentViewTag, [window tag], NULL );
    DialogEventSetDraggingInfo(nil);
    DialogEventSetNSArray( nil );
    DialogEventSetNSURL( nil );
}

//- (BOOL)prepareForDragOperation:(id<NSDraggingInfo>)sender {
//    return NO;
//}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender {
    [[NSCursor arrowCursor] set];
    
    NSWindow *window = [self window];
    CocoaOutputWindowSetTag( [window tag] );
    
    NSPasteboard *pb = [sender draggingPasteboard];
    
    NSArray *files = [pb propertyListForType:NSFilenamesPboardType];
    NSMutableArray *urls = [NSMutableArray arrayWithCapacity:[files count]];
    
    for ( NSString *path in files ) {
        [urls addObject:[NSURL fileURLWithPath:path]];
    }
    
    DialogEventSetNSArray( urls );
    DialogEventSetNSURL( [urls objectAtIndex:0] );
    DialogEventSetDraggingInfo(sender);
    DialogEventSetBool(NO);
    
    CallUserDefinedOnDialogFunction( viewPerformDragOperation, windowContentViewTag, [window tag], NULL );
    
    [self draggingExited:sender];
    //    return DialogEventBool();
    return YES;
}

- (void)concludeDragOperation:(id<NSDraggingInfo>)sender {
    
}

- (void)updateDraggingItemsForDrag:(id<NSDraggingInfo>)sender {
    
}

- (BOOL)isFlipped {
    return flip;
}

- (void)viewDidChangeEffectiveAppearance {
    CocoaSubclassViewDidChangeEffectiveAppearance( self );
}
@end




@implementation CocoaWindowDelegate

@synthesize boolVal;
@synthesize sizeVal;
@synthesize rectVal;
@synthesize allWindowsCallback;
@synthesize dragInside;
@synthesize dragFileTypes;
@synthesize dragFileExtensions;

#pragma mark - window delegate methods

- (void)window:(NSWindow *)w forwardEvent:(NSInteger)event tag:(NSInteger)tag obj:(id)obj {
    CocoaWindowCallbackType callback = [w cocoaWindowCallback];
    if ( callback ) {
        void *userData = [w cocoaWindowUserData];
        (*callback)([w tag],event,userData);
    } else if ( allWindowsCallback ) {
        (*allWindowsCallback)([w tag],event,NULL);
    } else {
        CallUserDefinedOnDialogFunction( event, tag, [w tag], obj );
    }
}

//- (NSRect)window:(NSWindow *)window willPositionSheet:(NSWindow *)sheet usingRect:(NSRect)rect;

- (void)windowWillBeginSheet:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    CocoaOutputWindowSetTag( tag );
    [self window:notification.object forwardEvent:windowWillBeginSheet tag:tag obj:nil];
}

- (void)windowDidEndSheet:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    CocoaOutputWindowSetTag( tag );
    [self window:notification.object forwardEvent:windowDidEndSheet tag:tag obj:nil];
}

- (NSSize)windowWillResize:(NSWindow *)sender toSize:(NSSize)frameSize {
    NSInteger tag = [sender tag];
    CocoaOutputWindowSetTag( tag );
    DialogEventSetNSSize( frameSize );
    [self window:sender forwardEvent:windowWillResize tag:tag obj:nil];
    return DialogEventNSSize();
}

- (void)windowDidResize:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    [self window:notification.object forwardEvent:windowDidResize tag:tag obj:nil];
}

- (void)windowWillStartLiveResize:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    [self window:notification.object forwardEvent:windowWillStartLiveResize tag:tag obj:nil];
}

- (void)windowDidEndLiveResize:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    CocoaOutputWindowSetTag( tag );
    [self window:notification.object forwardEvent:windowDidEndLiveResize tag:tag obj:nil];
}

- (void)windowWillMiniaturize:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    [self window:notification.object forwardEvent:windowWillMiniaturize tag:tag obj:nil];
}

- (void)windowDidMiniaturize:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    [self window:notification.object forwardEvent:windowDidMiniaturize tag:tag obj:nil];
}

- (void)windowDidDeminiaturize:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    CocoaOutputWindowSetTag( tag );
    [self window:notification.object forwardEvent:windowDidDeminiaturize tag:tag obj:nil];
}

// - (NSRect)windowWillUseStandardFrame:(NSWindow *)window defaultFrame:(NSRect)newFrame;

- (BOOL)windowShouldZoom:(NSWindow *)window toFrame:(NSRect)newFrame {
    NSInteger tag = [window tag];
    DialogEventSetBool( YES );
    DialogEventSetNSRect( newFrame );
    [self window:window forwardEvent:windowShouldZoom tag:tag obj:nil];
    return DialogEventBool();
}

//- (NSSize)window:(NSWindow *)window willUseFullScreenContentSize:(NSSize)proposedSize;

//- (NSApplicationPresentationOptions)window:(NSWindow *)window willUseFullScreenPresentationOptions:(NSApplicationPresentationOptions)proposedOptions;

- (void)windowWillEnterFullScreen:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    [self window:notification.object forwardEvent:windowWillEnterFullScreen tag:tag obj:nil];
}

- (void)windowDidEnterFullScreen:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    CocoaOutputWindowSetTag( tag );
    [self window:notification.object forwardEvent:windowDidEnterFullScreen tag:tag obj:nil];
}

- (void)windowWillExitFullScreen:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    [self window:notification.object forwardEvent:windowWillExitFullScreen tag:tag obj:nil];
}

- (void)windowDidExitFullScreen:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    CocoaOutputWindowSetTag( tag );
    [self window:notification.object forwardEvent:windowDidExitFullScreen tag:tag obj:nil];
}

//- (NSArray<NSWindow *> *)customWindowsToEnterFullScreenForWindow:(NSWindow *)window;

//- (NSArray<NSWindow *> *)customWindowsToEnterFullScreenForWindow:(NSWindow *)window onScreen:(NSScreen *)screen;

//- (void)window:(NSWindow *)window startCustomAnimationToEnterFullScreenWithDuration:(NSTimeInterval)duration;

//- (void)window:(NSWindow *)window startCustomAnimationToEnterFullScreenOnScreen:(NSScreen *)screen withDuration:(NSTimeInterval)duration;

- (void)windowDidFailToEnterFullScreen:(NSWindow *)window {
    NSInteger tag = [window tag];
    [self window:window forwardEvent:windowDidFailToEnterFullScreen tag:tag obj:nil];
}

//- (NSArray<NSWindow *> *)customWindowsToExitFullScreenForWindow:(NSWindow *)window;

//- (void)window:(NSWindow *)window startCustomAnimationToExitFullScreenWithDuration:(NSTimeInterval)duration;

- (void)windowDidFailToExitFullScreen:(NSWindow *)window {
    NSInteger tag = [window tag];
    [self window:window forwardEvent:windowDidFailToExitFullScreen tag:tag obj:nil];
}

- (void)windowWillMove:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    CocoaOutputWindowSetTag( tag );
    [self window:notification.object forwardEvent:windowWillMove tag:tag obj:nil];
}

- (void)windowDidMove:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    CocoaOutputWindowSetTag( tag );
    [self window:notification.object forwardEvent:windowDidMove tag:tag obj:nil];
}

- (void)windowDidChangeScreen:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    CocoaOutputWindowSetTag( tag );
    [self window:notification.object forwardEvent:windowDidChangeScreen tag:tag obj:nil];
}

//- (void)windowDidChangeScreenProfile:(NSNotification *)notification;

//- (void)windowDidChangeBackingProperties:(NSNotification *)notification;

- (BOOL)windowShouldClose:(NSWindow *)window {
    NSInteger tag = [window tag];
    DialogEventSetBool( YES );
    [self window:window forwardEvent:windowShouldClose tag:tag obj:nil];
    return DialogEventBool();
}

- (void)windowWillClose:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    [self window:notification.object forwardEvent:windowWillClose tag:tag obj:nil];
}

- (void)windowDidBecomeKey:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    CocoaOutputWindowSetTag( tag );
    [self window:notification.object forwardEvent:windowDidBecomeKey tag:tag obj:nil];
}

- (void)windowDidResignKey:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    [self window:notification.object forwardEvent:windowDidResignKey tag:tag obj:nil];
}

- (void)windowDidBecomeMain:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    CocoaOutputWindowSetTag( tag );
    [self window:notification.object forwardEvent:windowDidBecomeMain tag:tag obj:nil];
}

- (void)windowDidResignMain:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    [self window:notification.object forwardEvent:windowDidResignMain tag:tag obj:nil];
}

//- (id)windowWillReturnFieldEditor:(NSWindow *)sender toObject:(id)client;

- (void)windowDidUpdate:(NSNotification *)notification {
    NSInteger tag = [notification.object tag];
    [self window:notification.object forwardEvent:windowDidUpdate tag:tag obj:nil];
}

//- (void)windowDidExpose:(NSNotification *)notification;

//- (void)windowDidChangeOcclusionState:(NSNotification *)notification;

//- (BOOL)window:(NSWindow *)window shouldDragDocumentWithEvent:(NSEvent *)event from:(NSPoint)dragImageLocation withPasteboard:(NSPasteboard *)pasteboard;

//- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)window;

//- (BOOL)window:(NSWindow *)window shouldPopUpDocumentPathMenu:(NSMenu *)menu;

//- (void)window:(NSWindow *)window willEncodeRestorableState:(NSCoder *)state;

//- (void)window:(NSWindow *)window didDecodeRestorableState:(NSCoder *)state;

//- (NSSize)window:(NSWindow *)window willResizeForVersionBrowserWithMaxPreferredSize:(NSSize)maxPreferredFrameSize maxAllowedSize:(NSSize)maxAllowedFrameSize;

//- (void)windowWillEnterVersionBrowser:(NSNotification *)notification;

//- (void)windowDidEnterVersionBrowser:(NSNotification *)notification;

//- (void)windowWillExitVersionBrowser:(NSNotification *)notification;

//- (void)windowDidExitVersionBrowser:(NSNotification *)notification;


#pragma mark - dragging
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender {
    NSDragOperation dragOperation = NSDragOperationNone;
        
    DialogEventSetLong(dragOperation);
    
    NSWindow *window = [sender draggingDestinationWindow];
    
    CocoaOutputWindowSetTag( [window tag] );
    
    
    NSPasteboard *pb = [sender draggingPasteboard];
    NSArray *pbTypes = [pb types];
    if ( [pbTypes containsObject:NSFilenamesPboardType] ) {
        
        DialogEventSetDraggingInfo(sender);
        
        NSArray *files = [pb propertyListForType:NSFilenamesPboardType];
        
        BOOL validFile = NO;
        
        for ( NSString *path in files ) {
            validFile = NO;
            
            if ( dragFileTypes ) {
            NSString *utiType = [[NSWorkspace sharedWorkspace] typeOfFile:path error:nil];
            for ( NSString *type in dragFileTypes ) {
                if ( [[NSWorkspace sharedWorkspace] type:utiType conformsToType:(id)type] ) {
                    validFile = YES;
                    break;
                }
            }
            }
            
            if ( !validFile ) {
                if ( dragFileExtensions ) {
                    NSString *extension = [path pathExtension];
                    if ( extension ) {
                        if ( [dragFileExtensions containsObject:extension] ) {
                            validFile = YES;
                        }
                    }
                }
            }
            
            if ( !validFile ) break;
        }
        
        if ( validFile ) {
            dragOperation = NSDragOperationLink;
            NSMutableArray *urls = [NSMutableArray arrayWithCapacity:[files count]];
            for ( NSString *path in files ) {
                [urls addObject:[NSURL fileURLWithPath:path]];
            }
            DialogEventSetNSArray( urls );
            DialogEventSetNSURL( [urls objectAtIndex:0] );
            dragInside = YES;
            DialogEventSetLong(dragOperation);
            CallUserDefinedOnDialogFunction( windowDraggingEntered, [window tag], [window tag], NULL );
            DialogEventSetDraggingInfo(nil);
            DialogEventSetNSArray( nil );
            DialogEventSetNSURL( nil );
        }
    }
    return DialogEventLong();
}

- (void)draggingExited:(id <NSDraggingInfo>)sender {
    NSWindow *window = [sender draggingDestinationWindow];
    CocoaOutputWindowSetTag( [window tag] );
    DialogEventSetDraggingInfo(sender);
    dragInside = NO;
    CallUserDefinedOnDialogFunction( windowDraggingExited, [window tag], [window tag], NULL );
    DialogEventSetDraggingInfo(nil);
    DialogEventSetNSArray( nil );
    DialogEventSetNSURL( nil );
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender {
    [[NSCursor arrowCursor] set];

    NSWindow *window = [sender draggingDestinationWindow];

    CocoaOutputWindowSetTag( [window tag] );

    NSPasteboard *pb = [sender draggingPasteboard];

    NSArray *files = [pb propertyListForType:NSFilenamesPboardType];
    NSMutableArray *urls = [NSMutableArray arrayWithCapacity:[files count]];

    for ( NSString *path in files ) {
        [urls addObject:[NSURL fileURLWithPath:path]];
    }

    DialogEventSetNSArray( urls );
    DialogEventSetNSURL( [urls objectAtIndex:0] );
    DialogEventSetDraggingInfo(sender);
    DialogEventSetBool(NO);

    CallUserDefinedOnDialogFunction( windowPerformDragOperation, [window tag], [window tag], NULL );

    [self draggingExited:sender];
    return YES;
}

@end

CocoaWindowDelegate *CocoaWindowDelegateGet( void )
{
    static CocoaWindowDelegate *sCocoaWindowDelegate = nil;
    if ( !sCocoaWindowDelegate ) sCocoaWindowDelegate = [[CocoaWindowDelegate alloc] init];
    return sCocoaWindowDelegate;
}


@implementation NSWindow (Additions)
@dynamic tagNumber;
@dynamic callbackValue;
@dynamic userDataValue;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (CocoaWindowCallbackType)cocoaWindowCallback {
    return [objc_getAssociatedObject(self, @selector(callbackValue)) pointerValue];
}

- (void *)cocoaWindowUserData {
    return [objc_getAssociatedObject(self, @selector(userDataValue)) pointerValue];
}

- (void)setCocoaWindowCallback:(CocoaWindowCallbackType)callback {
    objc_setAssociatedObject(self, @selector(callbackValue), [NSValue valueWithPointer:callback], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (void)setCocoaWindowUserData:(void *)userData {
    objc_setAssociatedObject(self, @selector(userDataValue), [NSValue valueWithPointer:userData], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end


// private

#if !__LP64__

BOOL CocoaWindowIsFBWindow( NSInteger tag )
{
    WindowRef fbw = FBWindowFindByID( tag );
    if ( fbw ) {
        CocoaShowAlert( NSWarningAlertStyle, @"Window tag error.", @"FB and Cocoa windows cannot have same tag value." );
        return YES;
    }
    return NO;
}

BOOL FBWindowIsCocoaWindow( NSInteger tag )
{
    NSWindow *w = CocoaTargetWithTag( tag );
    if ( w ) {
        CocoaShowAlert( NSWarningAlertStyle, @"Window tag error.", @"Cocoa and FB windows cannot have same tag value." );
        return YES;
    }
    return NO;
}

#endif  //!__LP64__

BOOL CocoaObjectIsPopover( NSInteger tag )
{
    if ( CocoaPopoverWithTag( tag ) ) {
        CocoaShowAlert( NSWarningAlertStyle, @"Window tag error.", @"Windows and Popovers cannot have same tag value." );
        return YES;
    }
    return NO;
}

BOOL CocoaObjectIsWindow( NSInteger tag )
{
    if ( CocoaWindowWithTag( tag ) ) {
        CocoaShowAlert( NSWarningAlertStyle, @"Popover tag error.", @"Popovers and Windows cannot have same tag value." );
        return YES;
    }
    return NO;
}

id CocoaVisibleWindowWithTag( NSInteger tag )
{
    id w = nil;
    if ( tag ) {
        if ( tag > 1000000 ) return (id)tag;// it's a CocoaWindowRef

        NSArray *windows = [NSApp windows];
        if ( windows ) {
            for ( id obj in windows ) {
                if ( [obj isKindOfClass:[NSWindow class]] ) {
                    if ( [obj tag] == tag ) {
                        if ( [obj isVisible] ) {
                            w = obj;
                            break;
                        }
                    }
                }
            }
        }
    }
    return w;
}

id CocoaWindowWithTag( NSInteger tag )
{
    id w = nil;
    if ( tag ) {
        if ( tag > 1000000 ) return (id)tag;// it's a CocoaWindowRef
        NSArray *windows = [NSApp windows];
        if ( windows ) {
            for ( id obj in windows ) {
                if ( [obj isKindOfClass:[NSWindow class]] ) {
                    if ( [obj tag] == tag ) {
                        w = obj;
                        break;
                    }
                }
            }
        }
    }
    return w;
}

NSWindow *CocoaWindowWithIdentifier( NSString *identifier )
{
    NSArray *windows = [NSApp windows];
    for ( NSWindow *w in windows ) {
        if ( [[w identifier] isEqualToString:identifier] ) return w;
    }
    return nil;
}

NSWindow *CocoaVisibleFrontWindow( void )
{
    NSWindow *w = nil;
    NSArray *windows = [NSApp orderedWindows];
    for ( id obj in windows ) {
        if ( [obj isKindOfClass:[NSWindow class]] ) {
            if ( [obj isVisible] ) {
                w = obj;
                break;
            }
        }
    }
    return w;
}


NSWindow *CocoaFrontWindow( void )
{
    NSWindow *w = nil;
    NSArray *windows = [NSApp orderedWindows];
    for ( id obj in windows ) {
        if ( [obj isKindOfClass:[NSWindow class]] ) {
            w = obj;
            break;
        }
    }
    return w;
}

BOOL WindowIsCocoa( NSInteger tag )
{
    if ( CocoaWindowWithTag( tag ) ) return YES;
    if ( CocoaPopoverWithTag( tag ) ) return YES;
    return NO;
}

void CocoaWindowSetTitle( NSInteger tag, CFStringRef title )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setTitle:(NSString *)title];
}

void CocoaWindowMakeFirstTextViewResponder( NSWindow *w )
{
    NSArray *views = [[w contentView] subviews];
    for ( NSView *view in views ) {
        if ( [view acceptsFirstResponder] ) {
            Class cls = [view class];
            if ( cls == [NSTextField class] || cls == [NSSearchField class] || cls == [NSComboBox class] || cls == [NSTokenField class]  || cls == [NSSecureTextField class] ) {
                [w makeFirstResponder:view];
                break;
            }
        }
    }
}

/*
 kludge to fix cosmetics in Carbon
 - black marks in top left and top right window border
 - black window background when attaching toolbar
 */
void CocoaWindowCarbonRuntimeCosmeticFix( NSWindow *w )
{
    if ( !IsCocoaApplication() ) {
        NSRect r = [w frame];
        NSRect r2 = r;
        r2.size.width++;
        [w setFrame:r2 display:YES];
        [w setFrame:r display:YES];
    }
}

// === public ===
// Configure
Boolean WindowWorksWhenModal( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w worksWhenModal];
    return NO;
}

CGFloat WindowAlphaValue( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w alphaValue];
    return 0.0;
}

void WindowSetAlphaValue( NSInteger tag, CGFloat value )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setAlphaValue:value];
}

ColorRef WindowBackgroundColor( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w backgroundColor];
    return nil;
}

void WindowSetBackgroundColor( NSInteger tag, ColorRef col )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setBackgroundColor:col];
}

ColorSpaceRef WindowColorSpace( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w colorSpace];
    return nil;
}

void WindowSetColorSpace( NSInteger tag, ColorSpaceRef cs )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setColorSpace:cs];
}

ViewRef WindowContentView( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w contentView];
    return nil;
}

void WindowInvalidateShadow( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w invalidateShadow];
}

Boolean WindowAutorecalculatesContentBorderThickness( NSInteger tag, CGRectEdge edge )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        return [w autorecalculatesContentBorderThicknessForEdge:(NSRectEdge)edge];
    }
    return NO;
}

void WindowSetAutorecalculatesContentBorderThickness( NSInteger tag, Boolean flag, CGRectEdge edge )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        [w setAutorecalculatesContentBorderThickness:flag forEdge:(NSRectEdge)edge];
    }
}

CGFloat WindowContentBorderThickness( NSInteger tag, CGRectEdge edge )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        switch ( edge ) {
            case CGRectMaxYEdge: {
                NSUInteger style = [w styleMask];
                if ( (style & NSTexturedBackgroundWindowMask) == 0 ) break;
            }
            case CGRectMinYEdge:
                return [w contentBorderThicknessForEdge:(NSRectEdge)edge];
                break;
            default:
                break;
        }
    }
    return 0.0;
}

void WindowSetContentBorderThickness( NSInteger tag, CGFloat thickness, CGRectEdge edge )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        switch ( edge ) {
            case CGRectMaxYEdge: {
                NSUInteger style = [w styleMask];
                if ( (style & NSTexturedBackgroundWindowMask) == 0 ) break;
            }
            case CGRectMinYEdge:
                [w setContentBorderThickness:thickness forEdge:(NSRectEdge)edge];
                break;
            default:
                break;
        }
    }
}

Boolean WindowPreventsApplicationTerminationWhenModal( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w preventsApplicationTerminationWhenModal];
    return NO;
}

void WindowSetPreventsApplicationTerminationWhenModal( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setPreventsApplicationTerminationWhenModal:flag];
}

NSWindowDepth WindowDefaultDepthLimit( void )
{ return [NSWindow defaultDepthLimit]; }

NSInteger WindowNumber( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w windowNumber];
    return 0;
}

CFArrayRef WindowNumbersWithOptions( NSWindowNumberListOptions options )
{ return (CFArrayRef)[NSWindow windowNumbersWithOptions:options]; }

NSInteger WindowAttachedSheet( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        NSWindow *sheet = [w attachedSheet];
        if ( sheet ) {
            return [sheet tag];
        }
    }
    return 0;
}

CFDictionaryRef WindowDeviceDescription( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return (CFDictionaryRef)[w deviceDescription];
    return nil;
}

Boolean WindowCanBecomeVisibleWithoutLogin( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w canBecomeVisibleWithoutLogin];
    return NO;
}

void WindowSetCanBecomeVisibleWithoutLogin( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setCanBecomeVisibleWithoutLogin:flag];
}

NSWindowSharingType WindowSharingType( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w sharingType];
    return 0;
}

void WindowSetSharingType( NSInteger tag, NSWindowSharingType type )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setSharingType:type];
}

NSBackingStoreType WindowBackingType( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w backingType];
    return 0;
}

void WindowSetBackingType( NSInteger tag, NSBackingStoreType type )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setBackingType:type];
}

NSWindowDepth WindowDepthLimit( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w depthLimit];
    return 0;
}

void WindowSetDepthLimit( NSInteger tag, NSWindowDepth depth )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setDepthLimit:depth];
}

Boolean WindowHasDynamicDepthLimit( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w hasDynamicDepthLimit];
    return NO;
}

Boolean WindowIsSheet( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isSheet];
    return NO;
}

CGSize WindowAspectRatio( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSSizeToCGSize([w aspectRatio]);
    return CGSizeZero;
}

void WindowSetAspectRatio( NSInteger tag, CGSize size )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setAspectRatio:NSSizeFromCGSize( size )];
}

CFStringRef WindowTitle( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return (CFStringRef)[w title];
    return NULL;
}

CFURLRef WindowRepresentedURL( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return (CFURLRef)[w representedURL];
    return NULL;
}

void WindowSetRepresentedURL( NSInteger tag, CFURLRef url )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setRepresentedURL:(NSURL *)url];
}

ScreenRef WindowScreen( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w screen];
    return nil;
}

ScreenRef WindowDeepestScreen( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w deepestScreen];
    return nil;
}

Boolean WindowDisplaysWhenScreenProfileChanges( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w displaysWhenScreenProfileChanges];
    return NO;
}

void WindowSetDisplaysWhenScreenProfileChanges( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setDisplaysWhenScreenProfileChanges:flag];
}

Boolean WindowIsMovableByWindowBackground( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isMovableByWindowBackground];
    return NO;
}

void WindowSetMovableByWindowBackground( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setMovableByWindowBackground:flag];
}

Boolean WindowIsMovable( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isMovable];
    return YES;
}

void WindowSetMovable( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setMovable:flag];
}

void WindowCenter( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w center];
}

void WindowPerformClose( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w performClose:nil];
}

void WindowClose( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w close];
}

Boolean WindowIsReleasedWhenClosed( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isReleasedWhenClosed];
    return NO;
}

void WindowSetReleasedWhenClosed( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setReleasedWhenClosed:flag];
}

Boolean WindowIsMiniaturized( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isMiniaturized];
    return NO;
}

void WindowPerformMiniaturize( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w performMiniaturize:nil];
}

void WindowMiniaturize( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w miniaturize:nil];
}

void WindowDeminiaturize( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w deminiaturize:nil];
}

ImageRef WindowMiniwindowImage( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w miniwindowImage];
    return nil;
}

void WindowSetMiniwindowImage( NSInteger tag, void *inImage )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        NSImage *image = FixImageValue( inImage );
        if ( image ) [w setMiniwindowImage:image];
    }
}

void WindowSetMiniwindowImageNamed( NSInteger tag, CFStringRef imageName )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setMiniwindowImage:CocoaImageResource( (NSString *)imageName )];
}

CFStringRef WindowMiniwindowTitle( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return (CFStringRef)[w miniwindowTitle];
    return nil;
}

void WindowSetMiniwindowTitle( NSInteger tag, CFStringRef title )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setMiniwindowTitle:(NSString *)title];
}

DockTileRef WindowDockTile( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w dockTile];
    return nil;
}

void WindowPrint( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w print:nil];
}

CFDataRef WindowDataWithEPSInsideRect( NSInteger tag, CGRect r )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return (CFDataRef)[w dataWithEPSInsideRect:NSRectFromCGRect(r)];
    return nil;
}

CFDataRef WindowDataWithPDFInsideRect( NSInteger tag, CGRect r )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return (CFDataRef)[w dataWithPDFInsideRect:NSRectFromCGRect(r)];
    return nil;
}

// Scripting attributes
Boolean WindowHasCloseBox( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w hasCloseBox];
    return NO;
}

Boolean WindowHasTitleBar( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w hasTitleBar];
    return NO;
}

void WindowSetIsMiniaturized( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setIsMiniaturized:flag];
}

void WindowSetIsVisible( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setIsVisible:flag];
}

void WindowSetIsZoomed( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setIsZoomed:flag];
}

// Ordered indices
NSInteger WindowOrderedIndex( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w orderedIndex];
    return 0;
}
void WindowSetOrderedIndex( NSInteger tag, NSInteger index )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setOrderedIndex:index];
}


CGRect WindowFrame( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSRectToCGRect([w frame]);
    return CGRectZero;
}

CGRect WindowContentRect( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSRectToCGRect([w contentRectForFrameRect:[w frame]]);
    return CGRectZero;
}

void WindowSetFrameTopLeftPoint( NSInteger tag, CGPoint pt )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setFrameTopLeftPoint:NSPointFromCGPoint( pt )];
}

void WindowConstrainFrameRect( NSInteger tag, CGRect frame, ScreenRef screen )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w constrainFrameRect:NSRectFromCGRect(frame) toScreen:screen];
}

void WindowSetMinSize( NSInteger tag, CGSize size )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setMinSize:NSSizeFromCGSize(size)];
}

void WindowSetMaxSize( NSInteger tag, CGSize size )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setMaxSize:NSSizeFromCGSize(size)];
}

void WindowSetContentAspectRatio( NSInteger tag, CGSize size )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setContentAspectRatio:NSSizeFromCGSize( size )];
}

CGSize WindowContentMinSize( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSSizeToCGSize([w contentMinSize]);
    return CGSizeZero;
}

void WindowSetContentMinSize( NSInteger tag, CGSize size )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setContentMinSize:NSSizeFromCGSize(size)];
}

CGSize WindowContentMaxSize( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSSizeToCGSize([w contentMaxSize]);
    return CGSizeZero;
}

void WindowSetContentMaxSize( NSInteger tag, CGSize size )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setContentMaxSize:NSSizeFromCGSize(size)];
}

void WindowOrderOut( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w orderOut:nil];
}

void WindowOrderBack( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w orderBack:nil];
}

void WindowOrderFront( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w orderFront:nil];
}

void WindowOrderFrontRegardless( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w orderFrontRegardless];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSWindowOcclusionState WindowOcclusionState( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w occlusionState];
    return 0;
}
#endif

Boolean WindowIsKey( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isKeyWindow];
    return NO;
}

Boolean WindowCanBecomeKey( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w canBecomeKeyWindow];
    return NO;
}

void WindowMakeKey( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w makeKeyWindow];
}

void WindowMakeKeyAndOrderFront( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w makeKeyAndOrderFront:nil];
}

Boolean WindowIsMain( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isMainWindow];
    return NO;
}

Boolean WindowCanBecomeMain( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w canBecomeMainWindow];
    return NO;
}

void WindowMakeMain( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w makeMainWindow];
}

void WindowSetFrameUsingName( NSInteger tag, CFStringRef name )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setFrameUsingName:(NSString *)name];
}

void WindowSetFrameUsingNameForce( NSInteger tag, CFStringRef name, Boolean force )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setFrameUsingName:(NSString *)name force:force];
}

void WindowSaveFrameUsingName( NSInteger tag, CFStringRef name )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setFrameUsingName:(NSString *)name];
}

CFStringRef WindowFrameAutosaveName( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return (CFStringRef)[w frameAutosaveName];
    return nil;
}

CFStringRef WindowStringWithSavedFrame( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return (CFStringRef)[w stringWithSavedFrame];
    return nil;
}

void WindowSetFrameFromString( NSInteger tag, CFStringRef string )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setFrameFromString:(NSString *)string];
}


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void WindowBeginSheet( NSInteger tag, NSInteger sheetTag )
{
    if ( tag != sheetTag ) {
        NSWindow *w = CocoaWindowWithTag( tag );
        if ( w ) {
            NSWindow *sheet = CocoaWindowWithTag( sheetTag );
            if ( sheet ) {
                [w beginSheet:sheet completionHandler:^(NSModalResponse returnCode) {
                }];
            }
        }
    }
}

void WindowBeginCriticalSheet( NSInteger tag, NSInteger sheetTag )
{
    if ( tag != sheetTag ) {
        NSWindow *w = CocoaWindowWithTag( tag );
        if ( w ) {
            NSWindow *sheet = CocoaWindowWithTag( sheetTag );
            if ( sheet ) {
                [w beginCriticalSheet:sheet completionHandler:^(NSModalResponse returnCode) {

                }];
            }
        }
    }
}

void WindowEndSheet( NSInteger tag, NSInteger sheetTag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        NSWindow *sheet = CocoaWindowWithTag( sheetTag );
        if ( sheet ) [w endSheet:sheet];
    }
}

void WindowEndSheetWithReturnCode( NSInteger tag, NSInteger sheetTag, NSInteger returnCode )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        NSWindow *sheet = CocoaWindowWithTag( sheetTag );
        if ( sheet ) {
            [w endSheet:sheet returnCode:returnCode];
        }
    }
}

#endif

void WindowFlush( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w flushWindow];
}

// Default buttons
void WindowEnableKeyEquivalentForDefaultButtonCell( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w enableKeyEquivalentForDefaultButtonCell];
}

void WindowDisableKeyEquivalentForDefaultButtonCell( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w disableKeyEquivalentForDefaultButtonCell];
}

NSInteger WindowInitialFirstResponder( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        NSView *view = [w initialFirstResponder];
        if ( view ) return [view tag];
    }
    return 0;
}

void WindowSetInitialFirstResponder( NSInteger wndTag, NSInteger responderTag )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) {
        NSView *view = CocoaViewWithTag( responderTag );
        if ( view ) [w setInitialFirstResponder:view];
    }
}

NSInteger WindowFirstResponder( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        NSText *fe = [w fieldEditor:NO forObject:nil];
        if ( fe ) {
            return [(NSView *)[fe delegate] tag];
        } else {
            return [(NSView *)[w firstResponder] tag];
        }
    }
    return 0;
}

void WindowMakeFirstResponder( NSInteger wndTag, NSInteger viewTag )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) {
        if ( viewTag ) {
            NSView *view = CocoaViewWithTag( viewTag );
            if ( view ) [w makeFirstResponder:view];
        } else {
            [w makeFirstResponder:nil];
        }
    }
}

void WindowSelectKeyViewPrecedingView( NSInteger wndTag, NSInteger viewTag )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) {
        NSView *view = CocoaViewWithTag( viewTag );
        if ( view ) [w selectKeyViewPrecedingView:view];
    }
}

void WindowSelectKeyViewFollowingView( NSInteger wndTag, NSInteger viewTag )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) {
        NSView *view = CocoaViewWithTag( viewTag );
        if ( view ) [w selectKeyViewFollowingView:view];
    }
}

void WindowSelectPreviousKeyView( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w selectPreviousKeyView:nil];
}

void WindowSelectNextKeyView( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w selectNextKeyView:nil];
}

NSSelectionDirection WindowKeyViewSelectionDirection( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w keyViewSelectionDirection];
    return 0;
}

Boolean WindowAutorecalculatesKeyViewLoop( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w autorecalculatesKeyViewLoop];
    return NO;
}

void WindowSetAutorecalculatesKeyViewLoop( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setAutorecalculatesKeyViewLoop:flag];
}

void WindowRecalculateKeyViewLoop( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w recalculateKeyViewLoop];
}

Boolean WindowAcceptsMouseMovedEvents( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w acceptsMouseMovedEvents];
    return NO;
}

void WindowSetAcceptsMouseMovedEvents( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setAcceptsMouseMovedEvents:flag];
}

Boolean WindowIgnoresMouseEvents( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w ignoresMouseEvents];
    return NO;
}

void WindowSetIgnoresMouseEvents( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setIgnoresMouseEvents:flag];
}

CGPoint WindowMouseLocationOutsideOfEventStream( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSPointToCGPoint([w mouseLocationOutsideOfEventStream]);
    return CGPointZero;
}

NSInteger WindowNumberAtPoint( CGPoint pt, NSInteger belowWindowNumber )
{ return [NSWindow windowNumberAtPoint:NSPointFromCGPoint(pt) belowWindowWithWindowNumber:belowWindowNumber]; }

void WindowSetToolbar( NSInteger wndTag, NSInteger toolbarTag )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) {
        NSToolbar *toolbar = CocoaToolbarWithTag( toolbarTag );
        if ( toolbar ) {
            [w setToolbar:toolbar];
            // workaround for cosmetic in Carbon
            if ( !IsCocoaApplication() ) {
                CocoaWindowCarbonRuntimeCosmeticFix( w );
            }
        }
    }
}

NSInteger WindowToolbar( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        NSToolbar *toolbar = [w toolbar];
        if ( toolbar ) return [[toolbar identifier] integerValue];
    }
    return 0;
}

void WindowToggleToolbarShown( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w toggleToolbarShown:nil];
}

void WindowRunToolbarCustomizationPalette( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w runToolbarCustomizationPalette:nil];
}

void WindowSetDocumentEdited( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setDocumentEdited:flag];
}

Boolean WindowDocumentEdited( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isDocumentEdited];
    return NO;
}

// Converting coordinates
CGFloat WindowBackingScaleFactor( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w backingScaleFactor];
    return 0.0;
}

CGRect WindowBackingAlignedRect( NSInteger tag, CGRect r, NSAlignmentOptions options )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSRectToCGRect([w backingAlignedRect:NSRectFromCGRect(r) options:options]);
    return CGRectZero;
}

CGRect WindowConvertRectFromBacking( NSInteger tag, CGRect r )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSRectToCGRect([w convertRectFromBacking:NSRectFromCGRect(r)]);
    return CGRectZero;
}

CGRect WindowConvertRectToBacking( NSInteger tag, CGRect r )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSRectToCGRect([w convertRectToBacking:NSRectFromCGRect(r)]);
    return CGRectZero;
}

CGRect WindowConvertRectToScreen( NSInteger tag, CGRect r )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSRectToCGRect([w convertRectToScreen:NSRectFromCGRect(r)]);
    return CGRectZero;
}

CGRect WindowConvertRectFromScreen( NSInteger tag, CGRect r )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSRectToCGRect([w convertRectFromScreen:NSRectFromCGRect(r)]);
    return CGRectZero;
}

void WindowToggleFullScreen( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w toggleFullScreen:nil];
}

Boolean WindowCanHide( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w canHide];
    return NO;
}

void WindowSetCanHide( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setCanHide:flag];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean WindowTitlebarAppearsTransparent( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w titlebarAppearsTransparent];
    return NO;
}

void WindowSetTitlebarAppearsTransparent( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setTitlebarAppearsTransparent:flag];
}

void WindowAddTitlebarAccessoryView( NSInteger wndTag, NSInteger viewTag, NSLayoutAttribute layout )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) {
        NSView *view = CocoaViewWithTag( viewTag );
        if ( view ) {
            NSTitlebarAccessoryViewController *controller = [[NSTitlebarAccessoryViewController alloc] init];
            [controller setView:view];
            [controller setLayoutAttribute:layout];
            [w addTitlebarAccessoryViewController:controller];
            [controller release];
        }
    }
}

void WindowInsertTitlebarAccessoryView( NSInteger wndTag, NSInteger viewTag, NSInteger index, NSLayoutAttribute layout )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) {
        NSView *view = CocoaViewWithTag( viewTag );
        if ( view ) {
            NSTitlebarAccessoryViewController *controller = [[NSTitlebarAccessoryViewController alloc] init];
            [controller setView:view];
            [controller setLayoutAttribute:layout];
            [w insertTitlebarAccessoryViewController:controller atIndex:index];
            [controller release];
        }
    }
}

void WindowRemoveTitlebarAccessoryView( NSInteger wndTag, NSInteger index )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) {
        [w removeTitlebarAccessoryViewControllerAtIndex:index];
    }
}
#endif


// Tabs
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean WindowAllowsAutomaticWindowTabbing( void )
{ return [NSWindow allowsAutomaticWindowTabbing]; }

NSWindowUserTabbingPreference WindowUserTabbingPreference( void )
{ return [NSWindow userTabbingPreference]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
WindowTabRef WindowTab( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w tab];
    return nil;
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFStringRef WindowTabbingIdentifier( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return (CFStringRef)[w tabbingIdentifier];
    return nil;
}

void WindowAddTabbedWindow( NSInteger tag, NSInteger otherTag, NSWindowOrderingMode orderingMode )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        NSWindow *otherWnd = CocoaWindowWithTag( otherTag );
        if ( otherWnd ) {
            [w addTabbedWindow:otherWnd ordered:orderingMode];
        }
    }
}

NSWindowTabbingMode WindowTabbingMode( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w tabbingMode];
    return 0;
}

CFArrayRef WindowTabbedWindows( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        NSArray *windows = [w tabbedWindows];
        if ( windows ) {
            NSMutableArray *tagsArray = [NSMutableArray arrayWithCapacity:0];
            for ( NSWindow *wnd in windows ) {
                NSInteger wndTag = [wnd tag];
                if ( wndTag ) {
                    [tagsArray addObject:[NSNumber numberWithInteger:wndTag]];
                }
            }
            return (CFArrayRef)tagsArray;
        }
    }
    return nil;
}

void WindowSelectNextTab( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w selectNextTab:nil];
}

void WindowSelectPreviousTab( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w selectPreviousTab:nil];
}

void WindowMoveTabToNewWindow( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w moveTabToNewWindow:nil];
}

void WindowToggleTabBar( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w toggleTabBar:nil];
}

#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void WindowToggleTabOverview( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w toggleTabOverview:nil];
}

WindowTabGroupRef WindowTabGroup( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w tabGroup];
    return nil;
}
#endif


void WindowSetAllowsToolTipsWhenApplicationIsInactive( NSInteger tag, Boolean flag)
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setAllowsToolTipsWhenApplicationIsInactive:flag];
}

void WindowSetTitleWithRepresentedFilename( NSInteger tag, CFStringRef filename )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setTitleWithRepresentedFilename:(NSString *)filename];
}

CFArrayRef WindowChildWindows( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        NSArray *children = [w childWindows];
        if ( children ) {
            NSMutableArray *childTags = [NSMutableArray arrayWithCapacity:0];
            for ( NSWindow *child in children ) {
                NSInteger childTag = [child tag];
                if ( childTag ) [childTags addObject:[NSNumber numberWithInteger:childTag]];
            }
            return (CFArrayRef)childTags;
        }
    }
    return nil;
}

void WindowAddChildWindow( NSInteger parTag, NSInteger childTag, NSWindowOrderingMode ordered )
{
    NSWindow *parW = CocoaWindowWithTag( parTag );
    if ( parW ) {
        NSWindow *childW = CocoaWindowWithTag( childTag );
        if ( childW ) {
            [parW addChildWindow:childW ordered:ordered];
        }
    }
}

void WindowRemoveChildWindow( NSInteger parTag, NSInteger childTag )
{
    NSWindow *parW = CocoaWindowWithTag( parTag );
    if ( parW ) {
        NSWindow *childW = CocoaWindowWithTag( childTag );
        if ( childW ) {
            [parW removeChildWindow:childW];
        }
    }
}

NSInteger WindowParentWindow( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        return [[w parentWindow] tag];
    }
    return 0;
}

Boolean WindowHidesOnDeactivate( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w hidesOnDeactivate];
    return NO;
}

void WindowSetHidesOnDeactivate( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setHidesOnDeactivate:flag];
}

Boolean WindowIsOnActiveSpace( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isOnActiveSpace];
    return NO;
}

NSWindowCollectionBehavior WindowCollectionBehavior( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w collectionBehavior];
    return 0;
}

void WindowSetCollectionBehavior( NSInteger tag, NSWindowCollectionBehavior behavior )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setCollectionBehavior:behavior];
}

Boolean WindowIsOpaque( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isOpaque];
    return NO;
}

void WindowSetOpaque( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setOpaque:flag];
}

Boolean WindowHasShadow( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w hasShadow];
    return NO;
}

void WindowSetHasShadow( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setHasShadow:flag];
}

void WindowSetFrameOrigin( NSInteger tag, CGPoint pt )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setFrameOrigin:NSPointFromCGPoint(pt)];
}

void WindowSetFrame( NSInteger tag, CGRect frame, Boolean display, Boolean animate )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setFrame:NSRectFromCGRect(frame) display:display animate:animate];
}

CGPoint WindowCascadeTopLeftFromPoint( NSInteger tag, CGPoint pt )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSPointToCGPoint([w cascadeTopLeftFromPoint:NSPointFromCGPoint(pt)]);
    return CGPointZero;
}

Boolean WindowIsZoomed( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isZoomed];
    return NO;
}

void WindowPerformZoom( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w performZoom:nil];
}

void WindowZoom( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w zoom:nil];
}

Boolean WindowShowsResizeIndicator( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w showsResizeIndicator];
    return YES;
}

void WindowSetShowsResizeIndicator( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setShowsResizeIndicator:flag];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGSize WindowMaxFullScreenContentSize( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSSizeToCGSize([w maxFullScreenContentSize]);
    return CGSizeZero;
}

void WindowSetMaxFullScreenContentSize( NSInteger tag, CGSize size )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setMaxFullScreenContentSize:NSSizeFromCGSize(size)];
}

CGSize WindowMinFullScreenContentSize( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSSizeToCGSize([w minFullScreenContentSize]);
    return CGSizeZero;
}

void WindowSetMinFullScreenContentSize( NSInteger tag, CGSize size )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setMinFullScreenContentSize:NSSizeFromCGSize(size)];
}
#endif

Boolean WindowIsVisible( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isVisible];
    return NO;
}

void WindowRemoveFrameUsingName( CFStringRef name )
{ [NSWindow removeFrameUsingName:(NSString *)name]; }

Boolean WindowIsExcludedFromWindowsMenu( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isExcludedFromWindowsMenu];
    return NO;
}

void WindowSetExcludedFromWindowsMenu( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setExcludedFromWindowsMenu:flag];
}

// Drawing
void WindowDisplay( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w display];
}

void WindowDisplayIfNeeded( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w displayIfNeeded];
}

Boolean WindowViewsNeedDisplay( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w viewsNeedDisplay];
    return NO;
}

void WindowSetViewsNeedDisplay( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setViewsNeedDisplay:flag];
}

CGContextRef WindowGraphicsContext( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
        return [[w graphicsContext] CGContext];
#else
        return [[w graphicsContext] graphicsPort];
#endif
    }
    return NULL;
}

Boolean WindowAllowsConcurrentViewDrawing( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w allowsConcurrentViewDrawing];
    return NO;
}

void WindowSetAllowsConcurrentViewDrawing( NSInteger tag, Boolean flag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setAllowsConcurrentViewDrawing:flag];
}

Boolean WindowAreCursorRectsEnabled( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w areCursorRectsEnabled];
    return NO;
}

void WindowEnableCursorRects( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w enableCursorRects];
}

void WindowDisableCursorRects( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w disableCursorRects];
}

void WindowDiscardCursorRects( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w discardCursorRects];
}

void WindowInvalidateCursorRectsForView( NSInteger tag, NSInteger vwTag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        NSView *view = CocoaViewWithTag( vwTag );
        if ( view ) [w invalidateCursorRectsForView:view];
    }
}

void WindowResetCursorRects( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w resetCursorRects];
}

// Animation
NSWindowAnimationBehavior WindowAnimationBehavior( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w animationBehavior];
    return 0;
}

void WindowSetAnimationBehavior( NSInteger tag, NSWindowAnimationBehavior behavior )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setAnimationBehavior:behavior];
}

// Dragging items
void WindowRegisterForDraggedTypes( NSInteger tag, CFArrayRef types )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w registerForDraggedTypes:(NSArray *)types];
}

void WindowUnregisterDraggedTypes( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w unregisterDraggedTypes];
}

CGRect WindowContentRectForFrameRectAndStyle( CGRect r, NSInteger styleMask )
{ return NSRectToCGRect([NSWindow contentRectForFrameRect:NSRectFromCGRect(r) styleMask:styleMask]); }

CGRect WindowFrameRectForContentRectAndStyle( CGRect r, NSInteger styleMask )
{ return NSRectToCGRect([NSWindow frameRectForContentRect:NSRectFromCGRect(r) styleMask:styleMask]); }

CGFloat WindowMinFrameWidthWithTitle( CFStringRef title, NSInteger styleMask )
{ return [NSWindow minFrameWidthWithTitle:(NSString *)title styleMask:styleMask]; }

CGRect WindowContentRectForFrameRect( NSInteger tag, CGRect r )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSRectToCGRect([w contentRectForFrameRect:NSRectFromCGRect(r)]);
    return CGRectZero;
}

CGRect WindowFrameRectForContentRect( NSInteger tag, CGRect r )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSRectToCGRect([w frameRectForContentRect:NSRectFromCGRect(r)]);
    return CGRectZero;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
NSInteger WindowSheetParent( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [[w sheetParent] tag];
    return 0;
}

CFArrayRef WindowSheets( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        NSArray *sheets = [w sheets];
        if ( sheets ) {
            NSMutableArray *tagsArray = [NSMutableArray arrayWithCapacity:0];
            for ( NSWindow *sheet in sheets ) {
                NSInteger sheetTag = [sheet tag];
                if ( sheetTag ) [tagsArray addObject:[NSNumber numberWithInteger:sheetTag]];
            }
            return (CFArrayRef)tagsArray;
        }
    }
    return nil;
}
#endif

CGSize WindowResizeIncrements( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return NSSizeToCGSize([w resizeIncrements]);
    return CGSizeZero;
}

void WindowSetResizeIncrements( NSInteger tag, CGSize size )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setResizeIncrements:NSSizeFromCGSize(size)];
}

Boolean WindowInLiveResize( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w inLiveResize];
    return NO;
}

void WindowSetContentSize( NSInteger tag, CGSize size )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setContentSize:NSSizeFromCGSize(size)];
}

CGSize WindowContentResizeIncrements( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) NSSizeToCGSize([w contentResizeIncrements]);
    return CGSizeZero;
}

void WindowSetContentResizeIncrements( NSInteger tag, CGSize size )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setContentResizeIncrements:NSSizeFromCGSize(size)];
}

void WindowOrderRelativeTo( NSInteger wndTag, NSWindowOrderingMode order, NSInteger otherWndTag )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) {
        if ( otherWndTag ) {
            NSWindow *otherWnd = CocoaWindowWithTag( otherWndTag );
            if ( otherWnd ) {
                [w orderWindow:order relativeTo:[otherWnd windowNumber]];
            }
        } else {
            [w orderWindow:order relativeTo:0];
        }
    }
}

NSInteger WindowLevel( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w level];
    return 0;
}

void WindowSetLevel( NSInteger tag, NSInteger level )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setLevel:level];
}

void *WindowStandardWindowButtonForStyleMask( NSWindowButton btn, NSUInteger styleMask )
{ return [NSWindow standardWindowButton:btn forStyleMask:styleMask]; }

void *WindowStandardWindowButton( NSInteger tag, NSWindowButton btn )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w standardWindowButton:btn];
    return nil;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSWindowTitleVisibility WindowTitleVisibility( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w titleVisibility];
    return 0;
}

void WindowSetTitleVisibility( NSInteger tag, NSWindowTitleVisibility visibility )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setTitleVisibility:visibility];
}
#endif

NSUInteger WindowResizeFlags( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w resizeFlags];
    return 0;
}

// Instance properties
Boolean WindowIsFloatingPanel( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isFloatingPanel];
    return NO;
}

Boolean WindowIsMiniaturizable( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isMiniaturizable];
    return NO;
}

Boolean WindowIsModalPanel( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isModalPanel];
    return NO;
}

Boolean WindowIsResizable( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isResizable];
    return NO;
}

NSUInteger WindowStyleMask( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w styleMask];
    return 0;
}

Boolean WindowIsZoomable( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w isZoomable];
    return NO;
}

// Instance methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void WindowMergeAllWindows( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w mergeAllWindows:nil];
}
#endif

void WindowSetFrameAutosaveName( NSInteger tag, CFStringRef name )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setFrameAutosaveName:(NSString *)name];
}


/*
 NSAppearanceCustomization
 */
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
AppearanceRef WindowAppearance( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w appearance];
    return nil;
}

AppearanceRef WindowEffectiveAppearance( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) return [w effectiveAppearance];
    return nil;
}
#endif


// custom
void WindowSetOutput( NSInteger tag )
{ CocoaOutputWindowSetTag( tag ); }

void WindowAddSubview( NSInteger tag, ViewRef vwRef )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        [[w contentView] addSubview:vwRef];
    }
}

void WindowSetTitleWithRepresentedURL( NSInteger tag, CFURLRef url )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) [w setTitleWithRepresentedFilename:[(NSURL *)url path]];
}

Boolean WindowExists( NSInteger tag )
{ return CocoaTargetWithTag( tag ) != nil; }

id WindowWithTag( NSInteger tag )
{ return (id)CocoaWindowWithTag( tag ); }

id WindowViewWithTag( NSInteger wndTag, NSInteger viewTag )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) {
        NSView *contentView = [w contentView];
        if ( viewTag == windowContentViewTag ) return contentView;
        return CocoaViewSubviewWithTag( contentView, viewTag );
    }
    return nil;
}

void WindowSetDelegateCallback( NSInteger tag, void *callback, void *userData )
{
    if ( tag == 0 ) {   // all windows
        CocoaWindowDelegate *delegate = CocoaWindowDelegateGet();
        [delegate setAllWindowsCallback:callback];
    } else {            // one window
        NSWindow *w = CocoaWindowWithTag( tag );
        if ( w ) {
            [w setCocoaWindowCallback:callback];
            [w setCocoaWindowUserData:userData];
        }
    }
}

void WindowSubclassContentView( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        ContentViewSC *view = (ContentViewSC *)[w contentView];
        [(ContentViewSC *)view setSubclassFlag:YES];
    }
}

void WindowRecalculateKeyViewLoopByTagOrder( NSInteger wndTag )
{
    NSWindow *w = CocoaWindowWithTag( wndTag );
    if ( w ) {
        NSArray *subviews = [[w contentView] subviews];
        if ( subviews ) {
            [w setAutorecalculatesKeyViewLoop:NO];
            NSSortDescriptor *ascendingSort = [[NSSortDescriptor alloc] initWithKey:@"tag" ascending:YES];
            NSArray *sortedSubviews = [subviews sortedArrayUsingDescriptors:[NSArray arrayWithObject:ascendingSort]];
            id firstView = [sortedSubviews objectAtIndex:0];
            [w setInitialFirstResponder:firstView];
            NSUInteger count = [sortedSubviews count];
            for ( NSUInteger index = 0; index < count; index++ ) {
                id view = [sortedSubviews objectAtIndex:index];
                if ( index == count - 1 ) {
                    [view setNextKeyView:firstView];
                } else {
                    [view setNextKeyView:[sortedSubviews objectAtIndex:index+1]];
                }
            }
        }
    }
}


CFTypeRef WindowProperty( NSInteger tag, CFStringRef key )
{
    id property = nil;
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        property = ObjectProperty( w, key );
    }
    return (CFTypeRef)property;
}

void WindowSetProperty( NSInteger tag, CFStringRef key, CFTypeRef value )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        ObjectSetProperty( w, key, value );
    }
}

void WindowRemoveProperty( NSInteger tag, CFStringRef key )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        ObjectRemoveProperty( w, key );
    }
}

void WindowRemoveAllProperties( NSInteger tag )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        ObjectRemoveAllProperties( w );
    }
}


// Cascade
void WindowCascadeTopLeftFromWindow( NSInteger tag, NSInteger otherTag )
{
    NSWindow *w1 = CocoaWindowWithTag( tag );
    if ( w1 ) {
        NSWindow *w2 = CocoaWindowWithTag( otherTag );
        if ( w2 ) {
            NSRect r = [w2 frame];
            NSPoint pt = NSMakePoint(r.origin.x+21,r.origin.y+r.size.height-23);
            [w1 cascadeTopLeftFromPoint:pt];
        }
    }
}

void WindowCascadeTopLeftFromNextWindow( NSInteger tag )
{
    NSWindow *w1 = CocoaWindowWithTag( tag );
    if ( w1 ) {
        NSWindow *w2 = nil;
        NSArray *windows = [NSApp orderedWindows];
        NSUInteger count = [windows count];
        if ( count > 1 ) {
            for ( NSUInteger index = 0; index < count - 1; index++ ) {
                if ( [[windows objectAtIndex:index] isEqualTo:w1] ) {
                    w2 = [windows objectAtIndex:index+1];
                    break;
                }
            }
        }
        NSPoint pt = NSMakePoint(0,1);
        if ( w2 ) {
            NSRect r = [w2 frame];
            pt = NSMakePoint(r.origin.x+21,r.origin.y+r.size.height-23);
        }
        [w1 cascadeTopLeftFromPoint:pt];
    }
}

// SheetAlert
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void WindowEndSheetAlert( NSInteger tag, NSInteger alertTag )
{
    NSWindow *w = WindowWithTag( tag );
    if ( w ) {
        NSAlert *alert = AlertWithTag( alertTag );
        if ( alert ) {
            [w endSheet:[alert window]];
        }
    }
}
#endif


void WindowRegisterForDraggedFileTypes( NSInteger tag, CFTypeRef inFileTypes )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        
        NSArray *fileTypes = nil;
        if ( inFileTypes ) {
            if ( CFGetTypeID( inFileTypes ) == CFStringGetTypeID() ) {
                fileTypes = [(NSString *)inFileTypes componentsSeparatedByString:@";"];
            } else if ( CFGetTypeID( inFileTypes ) == CFArrayGetTypeID() ) {
                fileTypes = (NSArray *)inFileTypes;
            }
        }
        
        [(CocoaWindowDelegate *)[w delegate] setDragFileTypes:[fileTypes retain]];
        [w registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];

    }
}

void WindowRegisterForDraggedFileExtensions( NSInteger tag, CFTypeRef inFileExtensions )
{
    NSWindow *w = CocoaWindowWithTag( tag );
    if ( w ) {
        
        NSArray *fileExtensions = nil;
        if ( inFileExtensions ) {
            if ( CFGetTypeID( inFileExtensions ) == CFStringGetTypeID() ) {
                fileExtensions = [(NSString *)inFileExtensions componentsSeparatedByString:@";"];
            } else if ( CFGetTypeID( inFileExtensions ) == CFArrayGetTypeID() ) {
                fileExtensions = (NSArray *)inFileExtensions;
            }
        }
        
        [(CocoaWindowDelegate *)[w delegate] setDragFileExtensions:[fileExtensions retain]];
        [w registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
    }
}


// Init
CocoaWindowRef WindowInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    BOOL center = NO;
    if ( NSIsEmptyRect( rect ) ) {
        rect = NSMakeRect( 0, 0, 550, 400 );
        //center = YES;
    } else {
        center = ( rect.origin.x == 0.0 && rect.origin.y == 0.0 );
    }

    // style
    NSUInteger style = NSTitledWindowMask | NSClosableWindowMask | NSMiniaturizableWindowMask | NSResizableWindowMask;
    
    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    NSWindow *w;
    if ( subclass ) {
        w = [[WindowSC alloc] initWithContentRect:rect styleMask:style backing:NSBackingStoreBuffered defer:NO];
    } else {
        w = [[NSWindow alloc] initWithContentRect:rect styleMask:style backing:NSBackingStoreBuffered defer:NO];
    }

    // content view
    /* create content view subclass for:
     1. drawing focus rect around text controls (Carbon apps only).
     2. intercepting content view drawRect: method to issue dialog viewDrawRect event.
     */
    ContentViewSC *contentView = [[ContentViewSC alloc] initWithFrame:[[w contentView] frame]];
    [w setContentView:contentView];
    [contentView setIdentifier:[NSString stringWithFormat:@"%d",windowContentViewTag]];

    // key view loop
    [w setAutorecalculatesKeyViewLoop:YES];
    
    // tag
    [w setTag:inTag];
    
    // title
    [w setTitle:@"Untitled"];

    // center
    if ( center ) [(NSWindow *)w center];
    
    // delegate
    [w setDelegate:CocoaWindowDelegateGet()];
    
    // app delegate
    CocoaApplicationDelegate *appDelegate = CocoaApplicationDelegateGet();
    if ( appDelegate ) {
        [[appDelegate windows] addObject:w];
    }
    
    return w;
}

// Statement
void CocoaWindow( NSInteger inTag, CFStringRef inTitle, CGRect inRect, NSInteger inStyle )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    if ( CocoaObjectIsPopover( inTag ) ) return;
    
    if ( !IsCocoaApplication() ) {
        NSApplicationLoad();
        
#if !__LP64__
        if ( CocoaWindowIsFBWindow( inTag ) ) return;
#endif  //__LP64__
        
    }
    
    NSString *title = (NSString *)inTitle;
    NSRect rect = NSRectFromCGRect( inRect );
    
    id w = CocoaWindowWithTag( inTag );
    
    if ( w ) {      // exists
        
        // title
        if ( title ) {
            [w setTitle:title];
        }
        
        // rect
        if ( !NSIsEmptyRect( rect ) ) {
            [w setFrame:[w frameRectForContentRect:rect] display:YES];
        }
        
        if ( (int)inStyle != kFBParamMissing ) [w setStyleMask:inStyle];
        
    } else {    // create
        
//        BOOL center = NO;
        
        // rect
//        if ( NSIsEmptyRect( rect ) ) {
//            rect = NSMakeRect( 0, 0, 550, 400 );
//        } else {
//            center = ( rect.origin.x == 0.0 && rect.origin.y == 0.0 );
//        }
        

        
        // init
//        if ( inSubclass ) {
//            w = [[WindowSC alloc] initWithContentRect:rect styleMask:inStyle backing:NSBackingStoreBuffered defer:NO];
//        } else {
//            w = [[NSWindow alloc] initWithContentRect:rect styleMask:inStyle backing:NSBackingStoreBuffered defer:NO];
//        }
        w = WindowInit( inTag, inRect );
        
//        {
//            inStyle = NSTitledWindowMask | NSClosableWindowMask | NSMiniaturizableWindowMask | NSResizableWindowMask;
//        }

        if ( (int)inStyle != kFBParamMissing ) [w setStyleMask:inStyle];
        
        
        /* create content view subclass for:
         1. drawing focus rect around text controls (Carbon apps only).
         2. intercepting content view drawRect: method to issue dialog viewDrawRect event.
         */
        
//        ContentViewSC *contentView = [[ContentViewSC alloc] initWithFrame:[[w contentView] frame]];
//        [w setContentView:contentView];
//
//        [contentView setIdentifier:[NSString stringWithFormat:@"%d",windowContentViewTag]];
        
        // print view
        //        CocoaWindowAddPrintView( w );
        
        // key view loop
//        [w setAutorecalculatesKeyViewLoop:YES];
        
        // tag
//        [w setTag:inTag];
        
        // title
        if ( title ) [w setTitle:title];
        
        // center
//        if ( center ) [(NSWindow *)w center];
        
        // delegate
//        [w setDelegate:CocoaWindowDelegateGet()];
        
        // app delegate
//        CocoaApplicationDelegate *appDelegate = CocoaApplicationDelegateGet();
//        if ( appDelegate ) {
//            [[appDelegate windows] addObject:w];
//        }
    }
    
    // show
    if ( show ) {
        [w makeKeyAndOrderFront:nil];
        
#if __MAC_OS_X_VERSION_MAX_ALLOWED < 101100
        [[w contentView] setNeedsDisplay:YES];  // force drawRect:
#endif
        
    } else {
        [w orderOut:nil];
    }
    
    CocoaOutputWindowSetTag( inTag );
}





#pragma mark - nib window
void CocoaNibWindow( NSInteger inTag, CFStringRef inNibName, CFStringRef inWndIdentifier )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    if ( CocoaObjectIsPopover( inTag ) ) return;
    
    if ( !IsCocoaApplication() ) {
        NSApplicationLoad();
        
#if !__LP64__
        if ( CocoaWindowIsFBWindow( inTag ) ) return;
#endif  // __LP64__
    }
    
    NSWindow *w = CocoaWindowWithTag( inTag );
    if ( !w ) {             // ensure another cocoa window doesn't already exist
        
        NSString *nibName = (NSString *)inNibName;
        NSString *wndIdentifier = (NSString *)inWndIdentifier;
        
        CocoaNibObject *nibObj = CocoaNibObjectWithName( nibName );
        
        if ( !nibObj ) {
            
            NSNib *nib = [[NSNib alloc] initWithNibNamed:nibName bundle:nil];
            if ( nib ) {
                NSArray *objects = nil;
                
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
                if ( [nib respondsToSelector:@selector(instantiateWithOwner:topLevelObjects:)]) {
                    [nib instantiateWithOwner:nil topLevelObjects:&objects];
                } else {
                    [nib instantiateNibWithOwner:nil topLevelObjects:&objects];
                }
#else
                [nib instantiateNibWithOwner:nil topLevelObjects:&objects];
#endif
                
                CocoaNibObjectAdd( nibName, objects );
            }
        }
        
        w = CocoaWindowFromNibObjects( inTag, nibName, wndIdentifier );
        
//        if ( ![w isVisible] ) {
//            show = NO;
//        }
        
        // print view
        //        w = CocoaWindowWithTag( inTag );
        //        CocoaWindowAddPrintView( w );
    }
    CocoaOutputWindowSetTag( inTag );
    
    // show
    if ( show ) {
        [w makeKeyAndOrderFront:nil];
        
#if __MAC_OS_X_VERSION_MAX_ALLOWED < 101100
        [[w contentView] setNeedsDisplay:YES];  // force drawRect:
#endif
        
    } else {
        [w orderOut:nil];
    }
    
}

