/*
 WindowTabGroup.m

 Bernie Wylde
 */

#import "WindowTabGroup.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300

CFStringRef WindowTabGroupIdentifier( WindowTabGroupRef ref )
{ return (CFStringRef)[ref identifier]; }

Boolean WindowTabGroupIsOverviewVisible( WindowTabGroupRef ref )
{ return [ref isOverviewVisible]; }

void WindowTabGroupSetOverviewVisible( WindowTabGroupRef ref, Boolean flag )
{ [ref setOverviewVisible:flag]; }

Boolean WindowTabGroupIsTabBarVisible( WindowTabGroupRef ref )
{ return [ref isTabBarVisible]; }

NSInteger WindowTabGroupSelectedWindow( WindowTabGroupRef ref )
{
    NSWindow *w = [ref selectedWindow];
    if ( w ) return [w tag];
    return 0;
}

void WindowTabGroupSetSelectedWindow( WindowTabGroupRef ref, NSInteger tag )
{
    NSWindow *w = WindowWithTag( tag );
    if ( w ) [ref setSelectedWindow:w];
}

CFArrayRef WindowTabGroupWindows( WindowTabGroupRef ref )
{
    NSArray *windows = [ref windows];
    if ( windows ) {
        NSMutableArray *tagsArray = [NSMutableArray arrayWithCapacity:0];
        for ( NSWindow *w in windows ) {
            NSInteger tag = [w tag];
            if ( tag ) [tagsArray addObject:[NSNumber numberWithInteger:tag]];
        }
        return (CFArrayRef)tagsArray;
    }
    return nil;
}

void WindowTabGroupAddWindow( WindowTabGroupRef ref, NSInteger tag )
{
    NSWindow *w = WindowWithTag( tag );
    if ( w ) {
        [ref addWindow:w];
    }
}

void WindowTabGroupInsertWindow( WindowTabGroupRef ref, NSInteger tag, NSInteger index )
{
    NSWindow *w = WindowWithTag( tag );
    if ( w ) {
        [ref insertWindow:w atIndex:index];
    }
}

void WindowTabGroupRemoveWindow( WindowTabGroupRef ref, NSInteger tag )
{
    NSWindow *w = WindowWithTag( tag );
    if ( w ) {
        [ref removeWindow:w];
    }
}

#endif
