/*
 Workspace.h

 Bernie Wylde
 */

#import <AppKit/AppKit.h>

typedef void (*CocoaWorkspaceNotificationCenterCallbackType)(NSNotification *);

@interface CocoaWorkspaceNotificationCenter : NSObject {
    CocoaWorkspaceNotificationCenterCallbackType callback;
    NSString *name;
}
@property (assign) CocoaWorkspaceNotificationCenterCallbackType callback;
@property (assign) NSString *name;

- (id)initWithCallback:(CocoaWorkspaceNotificationCenterCallbackType)cb name:(NSString *)nm;

@end

// -- private --
NSMutableArray *CocoaWorkspaceNotificationCenters( void );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
typedef void (*WorkspaceRequestAuthorizationCompletionHandlerType)(NSWorkspaceAuthorization*,NSError*,void*);
#endif

// Shared
WorkspaceRef WorkspaceShared( void );

// Notification center
NotificationCenterRef WorkspaceNotificationCenter( void );
void WorkspaceNotificationCenterAddObserver( void *callback, CFStringRef name, CFTypeRef object );
void WorkspaceNotificationCenterRemoveObserver( void *callback, CFStringRef name );
void WorkspaceNotificationCenterPostNotification( NotificationRef note );
void WorkspaceNotificationCenterPostNotificationName( CFStringRef name, CFTypeRef object, CFDictionaryRef userInfo );

// Opening files
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
RunningApplicationRef WorkspaceOpenURLWithOptions( CFURLRef url, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration, ErrorRef *err );
#endif
Boolean WorkspaceOpenURL( CFURLRef url );
Boolean WorkspaceOpenURLWithApplication( CFURLRef url, CFStringRef appName, Boolean deactivate );
Boolean WorkspaceOpenURLsWithApplicationAtURL( CFArrayRef urls, CFURLRef appURL, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration );
Boolean WorkspaceOpenURLWithApplicationAtURL( CFURLRef url, CFURLRef appURL, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration );
Boolean WorkspaceLaunchApplication( CFStringRef appName );

RunningApplicationRef WorkspaceLaunchApplicationAtURL( CFURLRef url, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration, ErrorRef *err );

void WorkspaceHideOtherApplications( void );
void WorkspaceDuplicateURLs( CFArrayRef urls );
void WorkspaceRecycleURL( CFURLRef url);
void WorkspaceRecycleURLs( CFArrayRef urls );

void WorkspaceActivateFileViewerSelectingURLs( CFArrayRef fileURLs );

// UTI
CFStringRef WorkspaceTypeOfFileAtURL( CFURLRef url );
CFStringRef WorkspaceLocalizedDescriptionForType( CFStringRef type );
CFStringRef WorkspacePreferredFilenameExtensionForType( CFStringRef typeName );
Boolean WorkspaceFilenameExtensionIsValidForType( CFStringRef filenameExtension, CFStringRef typeName );
Boolean WorkspaceFileTypeConformsToType( CFStringRef firstTypeName, CFStringRef secondTypeName );
Boolean WorkspaceGetInfoForFileAtURL( CFURLRef url, CFStringRef *appName, CFStringRef *type );
CFURLRef WorkspaceURLForApplicationToOpenURL( CFURLRef url );
CFURLRef WorkspaceURLForApplication( CFStringRef name );

Boolean WorkspaceGetFileSystemInfoForURL( CFURLRef url, Boolean *isRemovable, Boolean *isWritable, Boolean *isUnmountable, CFStringRef *description, CFStringRef *type );

Boolean WorkspaceIsFilePackageAtURL( CFURLRef url );
RunningApplicationRef WorkspaceFrontmostApplication( void );
CFArrayRef WorkspaceRunningApplications( void );
RunningApplicationRef WorkspaceMenuBarOwningApplication( void );
ImageRef WorkspaceIconForFileAtURL( CFURLRef url );
ImageRef WorkspaceIconForFileType( CFStringRef type );
void WorkspaceSetIconForFileAtURL( ImageRef icon, CFURLRef url, NSWorkspaceIconCreationOptions options );

// Unmount
Boolean WorkspaceUnmountAndEjectDeviceAtURL( CFURLRef url, ErrorRef *err );

Boolean WorkspaceShowSearchResultsForQueryString( CFStringRef string );

CFArrayRef WorkspaceFileLabelColors( void );
CFArrayRef WorkspaceFileLabels( void );

CFURLRef WorkspaceURLForApplicationWithBundleIdentifier( CFStringRef identifier );
void WorkspaceNoteFileSystemChangedAtURL( CFURLRef url );

// - desktop image -
CFURLRef WorkspaceDesktopImageURLForScreen( ScreenRef screen );
void WorkspaceSetDesktopImageURLForScreen( CFURLRef url, ScreenRef screen, CFDictionaryRef options, ErrorRef *err );
CFDictionaryRef WorkspaceDesktopImageOptionsForScreen( ScreenRef screen );

// Instance methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
void WorkspaceRequestAuthorizationOfType( NSWorkspaceAuthorizationType type, WorkspaceRequestAuthorizationCompletionHandlerType completionHandler, void *userData );
#endif
