/*
 Contacts.h
 
 Bernie Wylde 20190222
 */

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100

#import <Contacts/Contacts.h>
#import "CocoaUI.h"

#if __LP64__
#ifdef CONTACTS_EXTERN

typedef CNContact *CNContactRef;
typedef CNContactFetchRequest *CNContactFetchRequestRef;
typedef CNContactFormatter *CNContactFormatterRef;
typedef CNContactProperty *CNContactPropertyRef;
typedef CNContactRelation *CNContactRelationRef;
typedef CNContactStore *CNContactStoreRef;
typedef CNContactsUserDefaults *CNContactsUserDefaultsRef;
typedef CNContactVCardSerialization *CNContactVCardSerializationRef;
typedef CNContainer *CNContainerRef;

typedef CNGroup *CNGroupRef;

typedef CNInstantMessageAddress *CNInstantMessageAddressRef;

typedef id<CNKeyDescriptor> CNKeyDescriptorRef;

typedef CNLabeledValue *CNLabeledValueRef;

typedef CNMutableContact *CNMutableContactRef;
typedef CNMutableGroup *CNMutableGroupRef;
typedef CNMutablePostalAddress *CNMutablePostalAddressRef;

typedef CNPhoneNumber *CNPhoneNumberRef;
typedef CNPostalAddress *CNPostalAddressRef;

typedef CNSaveRequest *CNSaveRequestRef;
typedef CNSocialProfile *CNSocialProfileRef;


#pragma mark - CNContact
///////////////////////////////////////////////////
//                     CNContact                 //
///////////////////////////////////////////////////
// Identification
CFStringRef CNContactIdentifier( CNContactRef ref );
CNContactType cnContactType( CNContactRef ref );

// Date properties
DateComponentsRef CNContactBirthday( CNContactRef ref );
DateComponentsRef CNContactNonGregorianBirthday( CNContactRef ref );
CFArrayRef CNContactDates( CNContactRef ref );

// Name properties
CFStringRef CNContactNamePrefix( CNContactRef ref );
CFStringRef CNContactGivenName( CNContactRef ref );
CFStringRef CNContactMiddleName( CNContactRef ref );
CFStringRef CNContactFamilyName( CNContactRef ref );
CFStringRef CNContactPreviousFamilyName( CNContactRef ref );
CFStringRef CNContactNameSuffix( CNContactRef ref );
CFStringRef CNContactNickname( CNContactRef ref );
CFStringRef CNContactPhoneticGivenName( CNContactRef ref );
CFStringRef CNContactPhoneticMiddleName( CNContactRef ref );
CFStringRef CNContactPhoneticFamilyName( CNContactRef ref );
CFStringRef CNContactOrganizationName( CNContactRef ref );
CFStringRef CNContactDepartmentName( CNContactRef ref );
CFStringRef CNContactJobTitle( CNContactRef ref );

// Localization
CFStringRef CNContactLocalizedStringForKey( CFStringRef key );

// Contact comparator
CNKeyDescriptorRef CNContactDescriptorForAllComparatorKeys( void );

// Social properties
CFArrayRef CNContactSocialProfiles( CNContactRef ref );

// Phone numbers
CFArrayRef CNContactPhoneNumbers( CNContactRef ref );

// URL addresses
CFArrayRef CNContactURLAddresses( CNContactRef ref );

// Postal addresses
CFArrayRef CNContactPostalAddresses( CNContactRef ref );

// Email addresses
CFArrayRef CNContactEmailAddresses( CNContactRef ref );

// Note property
CFStringRef CNContactNote( CNContactRef ref );

// Image properties
CFDataRef CNContactImageData( CNContactRef ref );
CFDataRef CNContactThumbnailImageData( CNContactRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean CNContactImageDataAvailable( CNContactRef ref );
#endif

// Labeled value properties
CFArrayRef CNContactRelations( CNContactRef ref );
CFArrayRef CNContactInstantMessageAddresses( CNContactRef ref );

// Partial contact property
Boolean CNContactIsKeyAvailable( CNContactRef ref, CFStringRef key );
Boolean CNContactAreKeysAvailable( CNContactRef ref, CFArrayRef keys );

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFStringRef CNContactPhoneticOrganizationName( CNContactRef ref );
#endif


#pragma mark - CNContactFetchRequest
///////////////////////////////////////////////////
//           CNContactFetchRequest               //
///////////////////////////////////////////////////
// Fetching options
Boolean CNContactFetchRequestMutableObjects( CNContactFetchRequestRef ref );
void CNContactFetchRequestSetMutableObjects( CNContactFetchRequestRef ref, Boolean flag );
Boolean CNContactFetchRequestUnifyResults( CNContactFetchRequestRef ref );
void CNContactFetchRequestSetUnifyResults( CNContactFetchRequestRef ref, Boolean flag );
CNContactSortOrder CNContactFetchRequestSortOrder( CNContactFetchRequestRef ref );
void CNContactFetchRequestSetSortOrder( CNContactFetchRequestRef ref, CNContactSortOrder order );

// Keys
CNContactFetchRequestRef CNContactFetchRequestWithKeysToFetch( CFArrayRef keys );
CFArrayRef CNContactFetchRequestKeysToFetch( CNContactFetchRequestRef ref );
void CNContactFetchRequestSetKeysToFetch( CNContactFetchRequestRef ref, CFArrayRef keys );
PredicateRef CNContactFetchPredicate( CNContactFetchRequestRef ref );
void CNContactFetchSetPredicate( CNContactFetchRequestRef ref, PredicateRef predicate );


#pragma mark - CNContactFormatter
///////////////////////////////////////////////////
//              CNContactFormatter               //
///////////////////////////////////////////////////
// Init
CNContactFormatterRef CNContactFormatterInit( void );

// Class
CFAttributedStringRef CNContactFormatterAttributedStringFromContactStyle( CNContactRef contact, CNContactFormatterStyle style, CFDictionaryRef defaultAttributes );
CNKeyDescriptorRef CNContactFormatterDescriptorForRequiredKeysForStyle( CNContactFormatterStyle style );
CFStringRef CNContactFormatterStringFromContactStyle( CNContactRef contact, CNContactFormatterStyle style );

// Manual formatting
CFStringRef CNContactFormatterDelimiterForContact( CNContactRef contact );
CNContactDisplayNameOrder CNContactFormatterNameOrderForContact( CNContactRef contact );

// Style
CNContactFormatterStyle cnContactFormatterStyle( CNContactFormatterRef ref );
void CNContactFormatterSetStyle( CNContactFormatterRef ref, CNContactFormatterStyle style );

// Instance
CFAttributedStringRef CNContactFormatterAttributedStringFromContact( CNContactFormatterRef ref, CNContactRef contact, CFDictionaryRef defaultAttributes );
CFStringRef CNContactFormatterStringFromContact( CNContactFormatterRef ref, CNContactRef contact );


#pragma mark - CNContactProperty
///////////////////////////////////////////////////
//              CNContactProperty                //
///////////////////////////////////////////////////
// Properties
CNContactRef CNContactPropertyContact( CNContactPropertyRef ref );
CFStringRef CNContactPropertyKey( CNContactPropertyRef ref );
CFTypeRef CNContactPropertyValue( CNContactPropertyRef ref );
CFStringRef CNContactPropertyLabel( CNContactPropertyRef ref );

// Identifier
CFStringRef CNContactPropertyIdentifier( CNContactPropertyRef ref );


#pragma mark - CNContactRelation
///////////////////////////////////////////////////
//              CNContactRelation                //
///////////////////////////////////////////////////
CNContactRelationRef CNContactRelationWithName( CFStringRef name );
CFStringRef CNContactRelationName( CNContactRelationRef ref );


#pragma mark - CNContactStore
///////////////////////////////////////////////////
//                CNContactStore                 //
///////////////////////////////////////////////////
// callbacks
typedef void (*CNContactStoreRequestAccessForEntityTypeCompletionHandlerType)(CNContactStore*,BOOL,NSError*,void*);
typedef BOOL (*CNContactStoreEnumerateContactsWithFetchRequestCompletionHandlerType)(CNContact*,void*);

// Init
CNContactStoreRef CNContactStoreInit( void );

// Privacy access
CNAuthorizationStatus CNContactStoreAuthorizationStatusForEntityType( CNEntityType type );
void CNContactStoreRequestAccessForEntityType( CNContactStoreRef ref, CNEntityType type, CNContactStoreRequestAccessForEntityTypeCompletionHandlerType fnAddress, void *userData );

// Fetching and saving
// ...
Boolean CNContactStoreEnumerateContactsWithFetchRequest( CNContactStoreRef ref, CNContactFetchRequestRef request, ErrorRef *err, CNContactStoreEnumerateContactsWithFetchRequestCompletionHandlerType fnAddress, void *userData );


#pragma mark - CNContactsUserDefaults
///////////////////////////////////////////////////
//             CNContactsUserDefaults             //
///////////////////////////////////////////////////
// Country code
CFStringRef CNContactsUserDefaultsCountryCode( void );

// Sort order
CNContactSortOrder CNContactsUserDefaultsSortOrder( void );

// Shared defaults
CNContactsUserDefaultsRef CNContactsUserDefaultsSharedDefaults( void );


#pragma mark - CNContactVCardSerialization
///////////////////////////////////////////////////
//          CNContactVCardSerialization          //
///////////////////////////////////////////////////
CFDataRef CNContactVCardSerializationDataWithContacts( CFArrayRef contacts, ErrorRef *err );
CFArrayRef CNContactVCardSerializationContactsWithData( CFDataRef dta, ErrorRef *err );
CNKeyDescriptorRef CNContactVCardSerializationDescriptorForRequiredKeys( void );


#pragma mark - CNContainer
///////////////////////////////////////////////////
//                 CNContainer                   //
///////////////////////////////////////////////////
// Properties
CFStringRef CNContainerIdentifier( CNContainerRef ref );
CFStringRef CNContainerName( CNContainerRef ref );
CNContainerType cnContainerType( CNContainerRef ref );

// Predicate
PredicateRef CNContainerPredicateForContainerOfContact( CFStringRef contactIdentifier );
PredicateRef CNContainerPredicateForContainers( CFArrayRef identifiers );
PredicateRef CNContainerPredicateForContainerOfGroup( CFStringRef groupIdentifier );


#pragma mark - CNGroup
///////////////////////////////////////////////////
//                   CNGroup                     //
///////////////////////////////////////////////////
// Properties
CFStringRef CNGroupIdentifier( CNGroupRef ref );
CFStringRef CNGroupName( CNGroupRef ref );

// Predicate
PredicateRef CNGroupPredicateForGroups( CFArrayRef groupIdentifiers );
PredicateRef CNGroupPredicateForGroupsInContainer( CFStringRef containerIdentifier );


#pragma mark - CNInstantMessageAddress
///////////////////////////////////////////////////
//          CNInstantMessageAddress              //
///////////////////////////////////////////////////
// Message service
CNInstantMessageAddressRef CNInstantMessageAddressWithUsername( CFStringRef username, CFStringRef service );
CFStringRef CNInstantMessageAddressService( CNInstantMessageAddressRef ref );
CFStringRef CNInstantMessageAddressUsername( CNInstantMessageAddressRef ref );

// Localization
CFStringRef CNInstantMessageAddressLocalizedStringForKey( CFStringRef key );
CFStringRef CNInstantMessageAddressLocalizedStringForService( CFStringRef service );


#pragma mark - CNLabeledValue
///////////////////////////////////////////////////
//                CNLabeledValue                 //
///////////////////////////////////////////////////
// Ideentifier
CFStringRef CNLabeledValueIdentifier( CNLabeledValueRef ref );

// Settinglabels and values
CNLabeledValueRef CNLabeledValueWithLabel( CFStringRef label, CFTypeRef value );
CNLabeledValueRef CNLabeledValueBySettingLabel( CNLabeledValueRef ref, CFStringRef label );
CNLabeledValueRef CNLabeledValueBySettingLabelAndValue( CNLabeledValueRef ref, CFStringRef label, CFTypeRef value );
CNLabeledValueRef CNLabeledValueBySettingValue( CNLabeledValueRef ref, CFTypeRef value );

// Labels and values
CFStringRef CNLabeledValueLabel( CNLabeledValueRef ref );
CFTypeRef CNLabeledValueValue( CNLabeledValueRef ref );

// Localization
CFStringRef CNLabeledValueLocalizedStringForLabel( CFStringRef label );


#pragma mark - CNMutableContact
///////////////////////////////////////////////////
//                 CNMutableContact              //
///////////////////////////////////////////////////
// Type
CNContactType CNMutableContactType( CNMutableContactRef ref );
void CNMutableContactSetType( CNMutableContactRef ref, CNContactType type );

// Dates
CFArrayRef CNMutableContactDates( CNMutableContactRef ref );
void CNMutableContactSetDates( CNMutableContactRef ref, CFArrayRef dates );
DateComponentsRef CNMutableContactNonGregorianBirthday( CNMutableContactRef ref );
void CNMutableContactSetNonGregorianBirthday( CNMutableContactRef ref, DateComponentsRef components );
DateComponentsRef CNMutableContactBirthday( CNMutableContactRef ref );
void CNMutableContactSetBirthday( CNMutableContactRef ref, DateComponentsRef components );

// Name
CFStringRef CNMutableContactNamePrefix( CNMutableContactRef ref );
void CNMutableContactSetNamePrefix( CNMutableContactRef ref, CFStringRef prefix );
CFStringRef CNMutableContactGivenName( CNMutableContactRef ref );
void CNMutableContactSetGivenName( CNMutableContactRef ref, CFStringRef name );
CFStringRef CNMutableContactMiddleName( CNMutableContactRef ref );
void CNMutableContactSetMiddleName( CNMutableContactRef ref, CFStringRef name );
CFStringRef CNMutableContactFamilyName( CNMutableContactRef ref );
void CNMutableContactSetFamilyName( CNMutableContactRef ref, CFStringRef name );
CFStringRef CNMutableContactPreviousFamilyName( CNMutableContactRef ref );
void CNMutableContactSetPreviousFamilyName( CNMutableContactRef ref, CFStringRef name );
CFStringRef CNMutableContactNameSuffix( CNMutableContactRef ref );
void CNMutableContactSetNameSuffix( CNMutableContactRef ref, CFStringRef suffix );
CFStringRef CNMutableContactNickname( CNMutableContactRef ref );
void CNMutableContactSetNickname( CNMutableContactRef ref, CFStringRef name );
CFStringRef CNMutableContactPhoneticGivenName( CNMutableContactRef ref );
void CNMutableContactSetPhoneticGivenName( CNMutableContactRef ref, CFStringRef name );
CFStringRef CNMutableContactPhoneticMiddleName( CNMutableContactRef ref );
void CNMutableContactSetPhoneticMiddleName( CNMutableContactRef ref, CFStringRef name );
CFStringRef CNMutableContactPhoneticFamilyName( CNMutableContactRef ref );
void CNMutableContactSetPhoneticFamilyName( CNMutableContactRef ref, CFStringRef name );
CFStringRef CNMutableContactOrganizationName( CNMutableContactRef ref );
void CNMutableContactSetOrganizationName( CNMutableContactRef ref, CFStringRef name );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFStringRef CNMutableContactPhoneticOrganizationName( CNMutableContactRef ref );
void CNMutableContactSetPhoneticOrganizationName( CNMutableContactRef ref, CFStringRef name );
#endif
CFStringRef CNMutableContactDepartmentName( CNMutableContactRef ref );
void CNMutableContactSetDepartmentName( CNMutableContactRef ref, CFStringRef name );
CFStringRef CNMutableContactJobTitle( CNMutableContactRef ref );
void CNMutableContactSetJobTitle( CNMutableContactRef ref, CFStringRef title );

// Social
CFArrayRef CNMutableContactSocialProfiles( CNMutableContactRef ref );
void CNMutableContactSetSocialProfiles( CNMutableContactRef ref, CFArrayRef profiles );

// Phone numbers
CFArrayRef CNMutableContactPhoneNumbers( CNMutableContactRef ref );
void CNMutableContactSetPhoneNumbers( CNMutableContactRef ref, CFArrayRef numbers );

// Online addresses
CFArrayRef CNMutableContactEmailAddresses( CNMutableContactRef ref );
void CNMutableContactSetEmailAddresses( CNMutableContactRef ref, CFArrayRef addresses );
CFArrayRef CNMutableContactURLAddresses( CNMutableContactRef ref );
void CNMutableContactSetURLAddresses( CNMutableContactRef ref, CFArrayRef addresses );

// Postal addresses
CFArrayRef CNMutableContactPostalAddresses( CNMutableContactRef ref );
void CNMutableContactSetPostalAddresses( CNMutableContactRef ref, CFArrayRef addresses );

// Image
CFDataRef CNMutableContactImageData( CNMutableContactRef ref );
void CNMutableContactSetImageData( CNMutableContactRef ref, CFDataRef imageData );

// Labeled value properties
CFArrayRef CNMutableContactRelations( CNMutableContactRef ref );
void CNMutableContactSetRelations( CNMutableContactRef ref, CFArrayRef relations );
CFArrayRef CNMutableContactInstantMessageAddresses( CNMutableContactRef ref );
void CNMutableContactSetInstantMessageAddresses( CNMutableContactRef ref, CFArrayRef addresses );


#pragma mark - CNMutableGroup
///////////////////////////////////////////////////
//                 CNMutableGroup                //
///////////////////////////////////////////////////
// zTaskGroup
CFStringRef CNMutableGroupName( CNMutableGroupRef ref );
void CNMutableGroupSetName( CNMutableGroupRef ref, CFStringRef name );


#pragma mark - CNMutablePostalAddress
///////////////////////////////////////////////////
//            CNMutablePostalAddress             //
///////////////////////////////////////////////////
// Init
CNMutablePostalAddressRef CNMutablePostalAddressInit( void );

// Parts
CFStringRef CNMutablePostalAddressStreet( CNMutablePostalAddressRef ref );
void CNMutablePostalAddressSetStreet( CNMutablePostalAddressRef ref, CFStringRef string );
CFStringRef CNMutablePostalAddressCity( CNMutablePostalAddressRef ref );
void CNMutablePostalAddressSetCity( CNMutablePostalAddressRef ref, CFStringRef string );
CFStringRef CNMutablePostalAddressState( CNMutablePostalAddressRef ref );
void CNMutablePostalAddressSetState( CNMutablePostalAddressRef ref, CFStringRef string );
CFStringRef CNMutablePostalAddressPostalCode( CNMutablePostalAddressRef ref );
void CNMutablePostalAddressSetPostalCode( CNMutablePostalAddressRef ref, CFStringRef string );
CFStringRef CNMutablePostalAddressCountry( CNMutablePostalAddressRef ref );
void CNMutablePostalAddressSetCountry( CNMutablePostalAddressRef ref, CFStringRef string );
CFStringRef CNMutablePostalAddressISOCountryCode( CNMutablePostalAddressRef ref );
void CNMutablePostalAddressSetISOCountryCode( CNMutablePostalAddressRef ref, CFStringRef string );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101204
CFStringRef CNMutablePostalAddressSubAdministrativeArea( CNMutablePostalAddressRef ref );
void CNMutablePostalAddressSetSubAdministrativeArea( CNMutablePostalAddressRef ref, CFStringRef string );
CFStringRef CNMutablePostalAddressSubLocality( CNMutablePostalAddressRef ref );
void CNMutablePostalAddressSetSubLocality( CNMutablePostalAddressRef ref, CFStringRef string );
#endif


#pragma mark - CNPhoneNumber
///////////////////////////////////////////////////
//                 CNPhoneNumber                 //
///////////////////////////////////////////////////
// Methods
CNPhoneNumberRef CNPhoneNumberWithStringValue( CFStringRef string );
CFStringRef CNPhoneNumberStringValue( CNPhoneNumberRef ref );


#pragma mark - CNPostalAddress
///////////////////////////////////////////////////
//               CNPostalAddress                 //
///////////////////////////////////////////////////
// Parts
CFStringRef CNPostalAddressStreet( CNPostalAddressRef ref );
CFStringRef CNPostalAddressCity( CNPostalAddressRef ref );
CFStringRef CNPostalAddressState( CNPostalAddressRef ref );
CFStringRef CNPostalAddressPostalCode( CNPostalAddressRef ref );
CFStringRef CNPostalAddressCountry( CNPostalAddressRef ref );
CFStringRef CNPostalAddressISOCountryCode( CNPostalAddressRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101204
CFStringRef CNPostalAddressSubAdministrativeArea( CNPostalAddressRef ref );
CFStringRef CNPostalAddressSubLocality( CNPostalAddressRef ref );
#endif

// Localized info
CFStringRef CNPostalAddressLocalizedStringForKey( CFStringRef key );


#pragma mark - CNSaveRequest
///////////////////////////////////////////////////
//                 CNSaveRequest                 //
///////////////////////////////////////////////////
// Saving contact changes
void CNSaveRequestAddContact( CNSaveRequestRef ref, CNMutableContactRef contact, CFStringRef containerIdentifier );
void CNSaveRequestUpdateContact( CNSaveRequestRef ref, CNMutableContactRef contact );
void CNSaveRequestDeleteContact( CNSaveRequestRef ref, CNMutableContactRef contact );

// Saving group changes
void CNSaveRequestAddGroup( CNSaveRequestRef ref, CNMutableGroupRef group, CFStringRef containerIdentifier );
void CNSaveRequestUpdateGroup( CNSaveRequestRef ref, CNMutableGroupRef group );
void CNSaveRequestDeleteGroup( CNSaveRequestRef ref, CNMutableGroupRef group );
void CNSaveRequestAddMemberToGroup( CNSaveRequestRef ref, CNContactRef contact, CNGroupRef group );
void CNSaveRequestRemoveMemberFromGroup( CNSaveRequestRef ref, CNContactRef contact, CNGroupRef group );

// Instance
void CNSaveRequestAddSubgroupToGroup( CNSaveRequestRef ref, CNGroupRef subgroup, CNGroupRef group );
void CNSaveRequestRemoveSubgroupFromGroup( CNSaveRequestRef ref, CNGroupRef subgroup, CNGroupRef group );


#pragma mark - CNSocialProfile
///////////////////////////////////////////////////
//               CNSocialProfile                 //
///////////////////////////////////////////////////
// Init
CNSocialProfileRef CNSocialProfileWithURLString( CFStringRef urlString, CFStringRef username, CFStringRef userIdentifier, CFStringRef service );

// Properties
CFStringRef CNSocialProfileService( CNSocialProfileRef ref );
CFStringRef CNSocialProfileURLString( CNSocialProfileRef ref );
CFStringRef CNSocialProfileUserIdentifier( CNSocialProfileRef ref );
CFStringRef CNSocialProfileUsername( CNSocialProfileRef ref );

// Localization
CFStringRef CNSocialProfileLocalizedStringForKey( CFStringRef key );
CFStringRef CNSocialProfileLocalizedStringForService( CFStringRef service );


#endif// #ifdef CONTACTS_EXTERN
#endif// #if __LP64__
#endif// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
