/*
 CocoaUI_CoreLocation.h
 
 Bernie Wylde 20190222
 */

#import <CoreLocation/CoreLocation.h>
#import "CocoaUI.h"
#import "Contacts.h"

#if __LP64__
#ifdef __CORELOCATION__

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
typedef CLCircularRegion *CLCircularRegionRef;
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
typedef CLGeocoder *CLGeocoderRef;
#endif

typedef CLHeading *CLHeadingRef;

typedef CLLocation *CLLocationRef;
typedef CLLocationManager *CLLocationManagerRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
typedef CLPlacemark *CLPlacemarkRef;
#endif

typedef CLRegion *CLRegionRef;


#pragma mark - CLCircularRegion
///////////////////////////////////////////////////
//            CLCircularRegion                   //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
// Init
CLCircularRegionRef CLCircularRegionWithCenter( CLLocationCoordinate2D center, CLLocationDistance radius, CFStringRef identifier );

// Attributes
CLLocationCoordinate2D CLCircularRegionCenter( CLCircularRegionRef ref );
CLLocationDistance CLCircularRegionRadius( CLCircularRegionRef ref );

// Hit testing
Boolean CLCircularRegionContainsCoordinate( CLCircularRegionRef ref, CLLocationCoordinate2D coordinate );
#endif


#pragma mark - CLGeocoder
///////////////////////////////////////////////////
//                  CLGeocoder                   //
///////////////////////////////////////////////////
// callbacks
typedef void (*CLGeocoderCompletionHandlerType)(CLGeocoder*,CFArrayRef,NSError*,void*);

// Init
CLGeocoderRef CLGeocoderInit( void );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
// Reverse geocoding a location
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void CLGeocoderReverseGeocodeLocationPreferredLocale( CLGeocoderRef ref, CLLocationRef location, CFLocaleRef preferredLocale, CLGeocoderCompletionHandlerType fnAddress, void *userData );
#endif
void CLGeocoderReverseGeocodeLocation( CLGeocoderRef ref, CLLocationRef location, CLGeocoderCompletionHandlerType fnAddress, void *userData );

// Geocoding an address
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void CLGeocoderGeocodeAddressStringPreferredLocale( CLGeocoderRef ref, CFStringRef addressString, CLRegionRef region, CFLocaleRef preferredLocale, CLGeocoderCompletionHandlerType fnAddress, void *userData );
#endif
void CLGeocoderGeococdeAddressString( CLGeocoderRef ref, CFStringRef addressString, CLGeocoderCompletionHandlerType fnAddress, void *userData );
void CLGeocoderGeocodeAddressInRegion( CLGeocoderRef ref, CFStringRef addressString, CLRegionRef region, CLGeocoderCompletionHandlerType fnAddress, void *userData );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void CLGeocoderGeocodePostalAddress( CLGeocoderRef ref, CNPostalAddressRef postalAddress, CLGeocoderCompletionHandlerType fnAddress, void *userData );
void CLGeocoderGeocodePostalAddressPreferredLocale( CLGeocoderRef ref, CNPostalAddressRef postalAddress, CFLocaleRef preferredLocale, CLGeocoderCompletionHandlerType fnAddress, void *userData );
#endif

// Geocode requests
void CLGeocoderCancelGeocode( CLGeocoderRef ref );
Boolean CLGeocoderIsGeocoding( CLGeocoderRef ref );
#endif


#pragma mark - CLHeading
///////////////////////////////////////////////////
//                  CLHeading                    //
///////////////////////////////////////////////////
// Attributes
CLLocationDirection CLHeadingMagneticHeading( CLHeadingRef ref );
CLLocationDirection CLHeadingTrueHeading( CLHeadingRef ref );
CLLocationDirection CLHeadingAccuracy( CLHeadingRef ref );
CFDateRef CLHeadingTimestamp( CLHeadingRef ref );

// Raw data
CLHeadingComponentValue CLHeadingX( CLHeadingRef ref );
CLHeadingComponentValue CLHeadingY( CLHeadingRef ref );
CLHeadingComponentValue CLHeadingZ( CLHeadingRef ref );


#pragma mark - CLLocation
///////////////////////////////////////////////////
//                  CLLocation                   //
///////////////////////////////////////////////////
// Init
CLLocationRef CLLocationWithLatitudeLongitude( CLLocationDegrees latitude, CLLocationDegrees longitude );
CLLocationRef CLLocationWithCoordinate( CLLocationCoordinate2D coordinate, CLLocationDistance altitude, CLLocationAccuracy hAccuracy, CLLocationAccuracy vAccuracy, CFDateRef timestamp );
CLLocationRef CLLocationWithCoordinateSpeed( CLLocationCoordinate2D coordinate, CLLocationDistance altitude, CLLocationAccuracy hAcc, CLLocationAccuracy vAcc, CLLocationDirection course, CLLocationSpeed speed, CFDateRef timestamp );

// Attributes
CLLocationCoordinate2D CLLocationCoordinate( CLLocationRef ref );
CLLocationDistance CLLocationAltitude( CLLocationRef ref );
CLLocationAccuracy CLLocationHorizontalAccuracy( CLLocationRef ref );
CLLocationAccuracy CLLocationVerticalAccuracy( CLLocationRef ref );
CFDateRef CLLocationTimestamp( CLLocationRef ref );

// Distance
CLLocationDistance CLLocationDistanceFromLocation( CLLocationRef ref, CLLocationRef otherRef );

// Speed and course
CLLocationSpeed clLocationSpeed( CLLocationRef ref );
CLLocationDirection CLLocationCourse( CLLocationRef ref );


#pragma mark - CLLocationManager
///////////////////////////////////////////////////
//              CLLocationManager                //
///////////////////////////////////////////////////
// dialog events
enum {
    locationManagerDidUpdateLocations = 3300,
    locationManagerDidFail,
    locationManagerDidFinishDeferredUpdates,
    locationManagerDidEnterRegion,
    locationManagerDidExitRegion,
    locationManagerDidDetermineStateForRegion,
    locationManagerMonitoringDidFailForRegion,
    locationManagerDidStartMonitoringForRegion,
    locationManagerDidChangeAuthorizationStatus
};

@interface CocoaCLLocationManagerDelegate : NSObject <CLLocationManagerDelegate>
@end

CocoaCLLocationManagerDelegate *CocoaCLLocationManagerDelegateObj( void );

// Init
CLLocationManagerRef CLLocationManagerInit( void );

// Availability of services
CLAuthorizationStatus CLLocationManagerAuthorizationStatus( void );
Boolean CLLocationManagerLocationServicesEnabled( void );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
Boolean CLLocationManagerDeferredLocationUpdatesAvailable( void );
#endif
Boolean CLLocationManagerSignificantLocationChangeMonitoringAvailable( void );
Boolean CLLocationManagerHeadingAvailable( void );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean CLLocationManagerIsMonitoringAvailableForClass( ClassRef regionClass );
#endif

// Receiving data
void CLLocationManagerInstallDelegate( CLLocationManagerRef ref );

// Standard location updates
void CLLocationManagerStartUpdatingLocation( CLLocationManagerRef ref );
void CLLocationManagerStopUpdatingLocation( CLLocationManagerRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
void CLLocationManagerRequestLocation( CLLocationManagerRef ref );
#endif
CLLocationDistance CLLocationManagerDistanceFilter( CLLocationManagerRef ref );
void CLLocationManagerSetDistanceFilter( CLLocationManagerRef ref, CLLocationDistance value );
CLLocationAccuracy CLLocationManagerDesiredAccuracy( CLLocationManagerRef ref );
void CLLocationManagerSetDesiredAccuracy( CLLocationManagerRef ref, CLLocationAccuracy value );

// Signification location updates
void CLLocationManagerStartMonitoringSignificantLocationChanges( CLLocationManagerRef ref );
void CLLocationManagerStopMonitoringSignificantLocationChanges( CLLocationManagerRef ref );

// Region monitoring
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
void CLLocationManagerStartMonitoringForRegion( CLLocationManagerRef ref, CLRegionRef region );
void CLLocationManagerStopMonitoringForRegion( CLLocationManagerRef ref, CLRegionRef region );
CFSetRef CLLocationManagerMonitoredRegions( CLLocationManagerRef ref );
CLLocationDistance CLLocationManagerMaximumRegionMonitoringDistance( CLLocationManagerRef ref );
#endif

// Beacon ranging requests
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void CLLocationManagerRequestStateForRegion( CLLocationManagerRef ref, CLRegionRef region );
#endif

// Recently retrieved data
CLLocationRef CLLocationManagerLocation( CLLocationManagerRef ref );


#pragma mark - CLPlacemark
///////////////////////////////////////////////////
//                 CLPlacemark                   //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
// Init
CLPlacemarkRef CLPlacemarkWithPlacemark( CLPlacemarkRef placemark );

//#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
//CLPlacemarkRef CLPlacemarkWithLocation( CLLocationRef location, CFStringRef name, CNPostalAddressRef postalAddress );// macOS 10.12.2+
//#endif// __MAC_OS_X_VERSION_MAX_ALLOWED >= 101202

// Location data
CLLocationRef CLPlacemarkLocation( CLPlacemarkRef ref );

// Attributes
CFStringRef CLPlacemarkName( CLPlacemarkRef ref );
CFStringRef CLPlacemarkISOCountryCode( CLPlacemarkRef ref );
CFStringRef CLPlacemarkCountry( CLPlacemarkRef ref );
CFStringRef CLPlacemarkPostalCode( CLPlacemarkRef ref );
CFStringRef CLPlacemarkAdministrativeArea( CLPlacemarkRef ref );
CFStringRef CLPlacemarkSubAdministrativeArea( CLPlacemarkRef ref );
CFStringRef CLPlacemarkLocality( CLPlacemarkRef ref );
CFStringRef CLPlacemarkSubLocality( CLPlacemarkRef ref );
CFStringRef CLPlacemarkThoroughfare( CLPlacemarkRef ref );
CFStringRef CLPlacemarkSubThoroughfare( CLPlacemarkRef ref );
CLRegionRef CLPlacemarkRegion( CLPlacemarkRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFTimeZoneRef CLPlacemarkTimeZone( CLPlacemarkRef ref );
#endif

// Address info
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CNPostalAddressRef CLPlacemarkPostalAddress( CLPlacemarkRef ref );
#endif

// Geographic info
CFStringRef CLPlacemarkInlandWater( CLPlacemarkRef ref );
CFStringRef CLPlacemarkOcean( CLPlacemarkRef ref );

// Landmark info
CFArrayRef CLPlacemarkAreasOfInterest( CLPlacemarkRef ref );
#endif


#pragma mark - CLRegion
///////////////////////////////////////////////////
//                   CLRegion                    //
///////////////////////////////////////////////////
// Init
CLRegionRef CLRegionInit( void );

// Attributes
CFStringRef CLRegionIdentifier( CLRegionRef ref );

// Notification delivery
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean CLRegionNotifyOnEntry( CLRegionRef ref );
void CLRegionSetNotifyOnEntry( CLRegionRef ref, Boolean flag );
Boolean CLRegionNotifyOnExit( CLRegionRef ref );
void CLRegionSetNotifyOnExit( CLRegionRef ref, Boolean flag );
#endif


#endif// #ifdef __CORELOCATION__
#endif// #if __LP64__
