/*
 CocoaUI_CoreLocation.m
 
 Bernie Wylde 20190222
 */

#import "CoreLocation.h"

#if __LP64__
#ifdef __CORELOCATION__

#pragma mark - CLCircularRegion
///////////////////////////////////////////////////
//            CLCircularRegion                   //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
// Init
CLCircularRegionRef CLCircularRegionWithCenter( CLLocationCoordinate2D center, CLLocationDistance radius, CFStringRef identifier )
{ return [[[CLCircularRegion alloc] initWithCenter:center radius:radius identifier:(NSString *)identifier] autorelease]; }

// Attributes
CLLocationCoordinate2D CLCircularRegionCenter( CLCircularRegionRef ref )
{ return [ref center]; }

CLLocationDistance CLCircularRegionRadius( CLCircularRegionRef ref )
{ return [ref radius]; }

// Hit testing
Boolean CLCircularRegionContainsCoordinate( CLCircularRegionRef ref, CLLocationCoordinate2D coordinate )
{ return [ref containsCoordinate:coordinate]; }
#endif


#pragma mark - CLGeocoder
///////////////////////////////////////////////////
//                  CLGeocoder                   //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
// Init
CLGeocoderRef CLGeocoderInit( void )
{ return [[[CLGeocoder alloc] init] autorelease]; }

// Reverse geocoding a location
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void CLGeocoderReverseGeocodeLocationPreferredLocale( CLGeocoderRef ref, CLLocationRef location, CFLocaleRef preferredLocale, CLGeocoderCompletionHandlerType fnAddress, void *userData )
{
    [ref reverseGeocodeLocation:location preferredLocale:(NSLocale *)preferredLocale completionHandler:^(NSArray<CLPlacemark *> *placemarks, NSError *error) {
        (*fnAddress)( ref, (CFArrayRef)placemarks, error, userData );
    }];
}
#endif

void CLGeocoderReverseGeocodeLocation( CLGeocoderRef ref, CLLocationRef location, CLGeocoderCompletionHandlerType fnAddress, void *userData )
{
    [ref reverseGeocodeLocation:location completionHandler:^(NSArray<CLPlacemark *> *placemarks, NSError *error) {
        (*fnAddress)( ref, (CFArrayRef)placemarks, error, userData );
    }];
}

// Geocoding an address
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void CLGeocoderGeocodeAddressStringPreferredLocale( CLGeocoderRef ref, CFStringRef addressString, CLRegionRef region, CFLocaleRef preferredLocale, CLGeocoderCompletionHandlerType fnAddress, void *userData )
{
    [ref geocodeAddressString:(NSString *)addressString inRegion:region preferredLocale:(NSLocale *)preferredLocale completionHandler:^(NSArray<CLPlacemark *> *placemarks, NSError *error) {
        (*fnAddress)( ref, (CFArrayRef)placemarks, error, userData );
    }];
}
#endif

void CLGeocoderGeococdeAddressString( CLGeocoderRef ref, CFStringRef addressString, CLGeocoderCompletionHandlerType fnAddress, void *userData )
{
    [ref geocodeAddressString:(NSString *)addressString completionHandler:^(NSArray<CLPlacemark *> *placemarks, NSError *error) {
        (*fnAddress)( ref, (CFArrayRef)placemarks, error, userData );
    }];
}

void CLGeocoderGeocodeAddressInRegion( CLGeocoderRef ref, CFStringRef addressString, CLRegionRef region, CLGeocoderCompletionHandlerType fnAddress, void *userData )
{
    [ref geocodeAddressString:(NSString *)addressString inRegion:region completionHandler:^(NSArray<CLPlacemark *> *placemarks, NSError *error) {
        (*fnAddress)( ref, (CFArrayRef)placemarks, error, userData );
    }];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void CLGeocoderGeocodePostalAddress( CLGeocoderRef ref, CNPostalAddressRef postalAddress, CLGeocoderCompletionHandlerType fnAddress, void *userData )
{
    [ref geocodePostalAddress:postalAddress completionHandler:^(NSArray<CLPlacemark *> *placemarks, NSError *error) {
        (*fnAddress)( ref, (CFArrayRef)placemarks, error, userData );
    }];
}

void CLGeocoderGeocodePostalAddressPreferredLocale( CLGeocoderRef ref, CNPostalAddressRef postalAddress, CFLocaleRef preferredLocale, CLGeocoderCompletionHandlerType fnAddress, void *userData )
{
    [ref geocodePostalAddress:postalAddress preferredLocale:(NSLocale *)preferredLocale completionHandler:^(NSArray<CLPlacemark *> *placemarks, NSError *error) {
        (*fnAddress)( ref, (CFArrayRef)placemarks, error, userData );
    }];
}
#endif

// Geocode requests
void CLGeocoderCancelGeocode( CLGeocoderRef ref )
{ [ref cancelGeocode]; }

Boolean CLGeocoderIsGeocoding( CLGeocoderRef ref )
{ return [ref isGeocoding]; }
#endif


#pragma mark - CLHeading
///////////////////////////////////////////////////
//                  CLHeading                    //
///////////////////////////////////////////////////
// Attributes
CLLocationDirection CLHeadingMagneticHeading( CLHeadingRef ref )
{ return [ref magneticHeading]; }

CLLocationDirection CLHeadingTrueHeading( CLHeadingRef ref )
{ return [ref trueHeading]; }

CLLocationDirection CLHeadingAccuracy( CLHeadingRef ref )
{ return [ref headingAccuracy]; }

CFDateRef CLHeadingTimestamp( CLHeadingRef ref )
{ return (CFDateRef)[ref timestamp]; }

// Raw data
CLHeadingComponentValue CLHeadingX( CLHeadingRef ref )
{ return [ref x]; }

CLHeadingComponentValue CLHeadingY( CLHeadingRef ref )
{ return [ref y]; }

CLHeadingComponentValue CLHeadingZ( CLHeadingRef ref )
{ return [ref z]; }


#pragma mark - CLLocation
///////////////////////////////////////////////////
//                  CLLocation                   //
///////////////////////////////////////////////////
// Init
CLLocationRef CLLocationWithLatitudeLongitude( CLLocationDegrees latitude, CLLocationDegrees longitude )
{ return [[[CLLocation alloc] initWithLatitude:latitude longitude:longitude] autorelease]; }

CLLocationRef CLLocationWithCoordinate( CLLocationCoordinate2D coordinate, CLLocationDistance altitude, CLLocationAccuracy hAccuracy, CLLocationAccuracy vAccuracy, CFDateRef timestamp )
{ return [[[CLLocation alloc] initWithCoordinate:coordinate altitude:altitude horizontalAccuracy:hAccuracy verticalAccuracy:vAccuracy timestamp:(NSDate *)timestamp] autorelease]; }

CLLocationRef CLLocationWithCoordinateSpeed( CLLocationCoordinate2D coordinate, CLLocationDistance altitude, CLLocationAccuracy hAcc, CLLocationAccuracy vAcc, CLLocationDirection course, CLLocationSpeed speed, CFDateRef timestamp )
{ return [[[CLLocation alloc] initWithCoordinate:coordinate altitude:altitude horizontalAccuracy:hAcc verticalAccuracy:vAcc course:course speed:speed timestamp:(NSDate *)timestamp] autorelease]; }

// Attributes
CLLocationCoordinate2D CLLocationCoordinate( CLLocationRef ref )
{ return [ref coordinate]; }

CLLocationDistance CLLocationAltitude( CLLocationRef ref )
{ return [ref altitude]; }

CLLocationAccuracy CLLocationHorizontalAccuracy( CLLocationRef ref )
{ return [ref horizontalAccuracy]; }

CLLocationAccuracy CLLocationVerticalAccuracy( CLLocationRef ref )
{ return [ref verticalAccuracy]; }

CFDateRef CLLocationTimestamp( CLLocationRef ref )
{ return (CFDateRef)[ref timestamp]; }


// Distance
CLLocationDistance CLLocationDistanceFromLocation( CLLocationRef ref, CLLocationRef otherRef )
{ return [ref distanceFromLocation:otherRef]; }

// Speed and course
CLLocationSpeed clLocationSpeed( CLLocationRef ref )
{ return [ref speed]; }

CLLocationDirection CLLocationCourse( CLLocationRef ref )
{ return [ref course]; }


#pragma mark - CLLocationManager
///////////////////////////////////////////////////
//              CLLocationManager                //
///////////////////////////////////////////////////
@implementation CocoaCLLocationManagerDelegate
- (void)locationManager:(CLLocationManager *)manager didUpdateLocations:(NSArray<CLLocation *> *)locations {
    CallUserDefinedOnDialogFunction( locationManagerDidUpdateLocations, 0, 0, (CFTypeRef)locations );
}

- (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error {
    CallUserDefinedOnDialogFunction( locationManagerDidFail, 0, 0, (CFTypeRef)error );
}

- (void)locationManager:(CLLocationManager *)manager didFinishDeferredUpdatesWithError:(NSError *)error {
    CallUserDefinedOnDialogFunction( locationManagerDidFinishDeferredUpdates, 0, 0, (CFTypeRef)error );
}

- (void)locationManager:(CLLocationManager *)manager didEnterRegion:(CLRegion *)region {
    CallUserDefinedOnDialogFunction( locationManagerDidEnterRegion, 0, 0, (CFTypeRef)region );
}

- (void)locationManager:(CLLocationManager *)manager didExitRegion:(CLRegion *)region {
    CallUserDefinedOnDialogFunction( locationManagerDidExitRegion, 0, 0, (CFTypeRef)region );
}

- (void)locationManager:(CLLocationManager *)manager didDetermineState:(CLRegionState)state forRegion:(CLRegion *)region {
    CallUserDefinedOnDialogFunction( locationManagerDidDetermineStateForRegion, state, 0, (CFTypeRef)region );
}

- (void)locationManager:(CLLocationManager *)manager monitoringDidFailForRegion:(CLRegion *)region withError:(NSError *)error {
    CallUserDefinedOnDialogFunction( locationManagerMonitoringDidFailForRegion, 0, 0, (CFTypeRef)region );
}

- (void)locationManager:(CLLocationManager *)manager didStartMonitoringForRegion:(CLRegion *)region {
    CallUserDefinedOnDialogFunction( locationManagerDidStartMonitoringForRegion, 0, 0, (CFTypeRef)region );
}

- (void)locationManager:(CLLocationManager *)manager didChangeAuthorizationStatus:(CLAuthorizationStatus)status {
    CallUserDefinedOnDialogFunction( locationManagerDidChangeAuthorizationStatus, status, 0, NULL );
}
@end

CocoaCLLocationManagerDelegate *CocoaCLLocationManagerDelegateObj( void )
{
    static CocoaCLLocationManagerDelegate *sCocoaCLLocationManagerDelegate = nil;
    if ( !sCocoaCLLocationManagerDelegate ) sCocoaCLLocationManagerDelegate = [[CocoaCLLocationManagerDelegate alloc] init];
    return sCocoaCLLocationManagerDelegate;
}

// Init
CLLocationManagerRef CLLocationManagerInit( void )
{ return [[[CLLocationManager alloc] init] autorelease]; }

// Availability of services
CLAuthorizationStatus CLLocationManagerAuthorizationStatus( void )
{ return [CLLocationManager authorizationStatus]; }

Boolean CLLocationManagerLocationServicesEnabled( void )
{ return [CLLocationManager locationServicesEnabled]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
Boolean CLLocationManagerDeferredLocationUpdatesAvailable( void )
{ return [CLLocationManager deferredLocationUpdatesAvailable]; }
#endif

Boolean CLLocationManagerSignificantLocationChangeMonitoringAvailable( void )
{ return [CLLocationManager significantLocationChangeMonitoringAvailable]; }

Boolean CLLocationManagerHeadingAvailable( void )
{ return [CLLocationManager headingAvailable]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean CLLocationManagerIsMonitoringAvailableForClass( ClassRef regionClass )
{ return [CLLocationManager isMonitoringAvailableForClass:regionClass]; }
#endif

// Receiving data
void CLLocationManagerInstallDelegate( CLLocationManagerRef ref )
{ [ref setDelegate:CocoaCLLocationManagerDelegateObj()]; }

// Standard location updates
void CLLocationManagerStartUpdatingLocation( CLLocationManagerRef ref )
{ [ref startUpdatingLocation]; }

void CLLocationManagerStopUpdatingLocation( CLLocationManagerRef ref )
{ [ref stopUpdatingLocation]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
void CLLocationManagerRequestLocation( CLLocationManagerRef ref )
{ [ref requestLocation]; }
#endif

CLLocationDistance CLLocationManagerDistanceFilter( CLLocationManagerRef ref )
{ return [ref distanceFilter]; }

void CLLocationManagerSetDistanceFilter( CLLocationManagerRef ref, CLLocationDistance value )
{ [ref setDistanceFilter:value]; }

CLLocationAccuracy CLLocationManagerDesiredAccuracy( CLLocationManagerRef ref )
{ return [ref desiredAccuracy]; }

void CLLocationManagerSetDesiredAccuracy( CLLocationManagerRef ref, CLLocationAccuracy value )
{ [ref setDesiredAccuracy:value]; }

// Signification location updates
void CLLocationManagerStartMonitoringSignificantLocationChanges( CLLocationManagerRef ref )
{ [ref startMonitoringSignificantLocationChanges]; }

void CLLocationManagerStopMonitoringSignificantLocationChanges( CLLocationManagerRef ref )
{ [ref stopMonitoringSignificantLocationChanges]; }

// Region monitoring
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
void CLLocationManagerStartMonitoringForRegion( CLLocationManagerRef ref, CLRegionRef region )
{ [ref startMonitoringForRegion:region]; }

void CLLocationManagerStopMonitoringForRegion( CLLocationManagerRef ref, CLRegionRef region )
{ [ref stopMonitoringForRegion:region]; }

CFSetRef CLLocationManagerMonitoredRegions( CLLocationManagerRef ref )
{ return (CFSetRef)[ref monitoredRegions]; }

CLLocationDistance CLLocationManagerMaximumRegionMonitoringDistance( CLLocationManagerRef ref )
{ return [ref maximumRegionMonitoringDistance]; }
#endif

// Ranging requests
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void CLLocationManagerRequestStateForRegion( CLLocationManagerRef ref, CLRegionRef region )
{ [ref requestStateForRegion:region]; }
#endif

// Recently retrieved data
CLLocationRef CLLocationManagerLocation( CLLocationManagerRef ref )
{ return [ref location]; }


#pragma mark - CLPlacemark
///////////////////////////////////////////////////
//                 CLPlacemark                   //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
// Init
CLPlacemarkRef CLPlacemarkWithPlacemark( CLPlacemarkRef placemark )
{ return [[[CLPlacemark alloc] initWithPlacemark:placemark] autorelease]; }

//#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
//CLPlacemarkRef CLPlacemarkWithLocation( CLLocationRef location, CFStringRef name, CNPostalAddressRef postalAddress )
//{ return [CLPlacemark placemarkWithLocation:location name:(NSString *)name postalAddress:postalAddress]; }
//#endif// __MAC_OS_X_VERSION_MAX_ALLOWED >= 101202

// Location data
CLLocationRef CLPlacemarkLocation( CLPlacemarkRef ref )
{ return [ref location]; }

// Attributes
CFStringRef CLPlacemarkName( CLPlacemarkRef ref )
{ return (CFStringRef)[ref name]; }

CFStringRef CLPlacemarkISOCountryCode( CLPlacemarkRef ref )
{ return (CFStringRef)[ref ISOcountryCode]; }

CFStringRef CLPlacemarkCountry( CLPlacemarkRef ref )
{ return (CFStringRef)[ref country]; }

CFStringRef CLPlacemarkPostalCode( CLPlacemarkRef ref )
{ return (CFStringRef)[ref postalCode]; }

CFStringRef CLPlacemarkAdministrativeArea( CLPlacemarkRef ref )
{ return (CFStringRef)[ref administrativeArea]; }

CFStringRef CLPlacemarkSubAdministrativeArea( CLPlacemarkRef ref )
{ return (CFStringRef)[ref subAdministrativeArea]; }

CFStringRef CLPlacemarkLocality( CLPlacemarkRef ref )
{ return (CFStringRef)[ref locality]; }

CFStringRef CLPlacemarkSubLocality( CLPlacemarkRef ref )
{ return (CFStringRef)[ref subLocality]; }

CFStringRef CLPlacemarkThoroughfare( CLPlacemarkRef ref )
{ return (CFStringRef)[ref thoroughfare]; }

CFStringRef CLPlacemarkSubThoroughfare( CLPlacemarkRef ref )
{ return (CFStringRef)[ref subThoroughfare]; }

CLRegionRef CLPlacemarkRegion( CLPlacemarkRef ref )
{ return [ref region]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFTimeZoneRef CLPlacemarkTimeZone( CLPlacemarkRef ref )
{ return (CFTimeZoneRef)[ref timeZone]; }
#endif

// Address info
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CNPostalAddressRef CLPlacemarkPostalAddress( CLPlacemarkRef ref )
{ return [ref postalAddress]; }
#endif

// Geographic info
CFStringRef CLPlacemarkInlandWater( CLPlacemarkRef ref )
{ return (CFStringRef)[ref inlandWater]; }

CFStringRef CLPlacemarkOcean( CLPlacemarkRef ref )
{ return (CFStringRef)[ref ocean]; }

// Landmark info
CFArrayRef CLPlacemarkAreasOfInterest( CLPlacemarkRef ref )
{ return (CFArrayRef)[ref areasOfInterest]; }
#endif


#pragma mark - CLRegion
///////////////////////////////////////////////////
//                   CLRegion                    //
///////////////////////////////////////////////////
// Init
CLRegionRef CLRegionInit( void )
{ return [[[CLRegion alloc] init] autorelease]; }

// Attributes
CFStringRef CLRegionIdentifier( CLRegionRef ref )
{ return (CFStringRef)[ref identifier]; }

// Notification delivery
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean CLRegionNotifyOnEntry( CLRegionRef ref )
{ return [ref notifyOnEntry]; }

void CLRegionSetNotifyOnEntry( CLRegionRef ref, Boolean flag )
{ [ref setNotifyOnEntry:flag]; }

Boolean CLRegionNotifyOnExit( CLRegionRef ref )
{ return [ref notifyOnExit]; }

void CLRegionSetNotifyOnExit( CLRegionRef ref, Boolean flag )
{ [ref setNotifyOnExit:flag]; }
#endif


#endif// #ifdef __CORELOCATION__
#endif// #if __LP64__

