#include "FileHandlingUtils.h"
#include "ArrayIndices.h"

///////////////////////////////////////////////////
//       translated globals and fns              //
///////////////////////////////////////////////////
#pragma mark - Utility functions
CFStringRef FH_URLGetPath( CFURLRef url ) {
    return (CFStringRef)[(NSURL *)url path];
}

FBBoolean  FH_FileExistsAtPath( CFStringRef path ) {
    FBBoolean  exists = false;
    if( path ) exists = [[NSFileManager defaultManager] fileExistsAtPath:(NSString *)path];
    return exists;
}


FBBoolean  FH_LastFileExists( CFURLRef url ) {
    FBBoolean  exists = false;
    if( url ) exists = FH_FileExistsAtPath((CFStringRef)[(NSURL *)url path] );
    return exists;
}

#pragma mark - URL functions
CFURLRef  FH_AppendComponentToURL( CFURLRef baseURL, CFStringRef componentToAppend ) {
    return(CFURLRef)[(NSURL *)baseURL URLByAppendingPathComponent:(NSString *)componentToAppend];
    return NULL;
}


CFStringRef  FH_AppendComponentToString( CFStringRef basePath, CFStringRef componentToAppend ) {
    return(CFStringRef)[(NSString *)basePath stringByAppendingPathComponent:(NSString *)componentToAppend];
    return NULL;
}


CFURLRef  FH_GetBundleResourceDirectoryURL() {
    return(CFURLRef)[[NSBundle mainBundle] resourceURL];
    return NULL;
}

/*
 CFURLRef  FH_GetSysDirAndPathURL( UInt32 whichDirectory, UInt32 whichDomain, CFStringRef whichAppendComponent ) {
 CFURLRef          url1;
 CFURLRef          url2;
 url1 = SystemDirectoryCopyURL( whichDirectory, whichDomain );
 url2 = FH_AppendComponentToURL( url1, whichAppendComponent );
 CFRelease( url1 );
 return url2;
 }
 */
#pragma mark - Creator/Type functions
long  FH_GetFileType( CFURLRef url, OSType * type ) {
    NSDictionary *attributes = [[NSFileManager defaultManager] attributesOfItemAtPath:[(NSURL *)url path] error:nil];
    *type = [[attributes objectForKey:NSFileHFSTypeCode] unsignedLongValue];
    //*creator = [[attributes objectForKey:NSFileHFSCreatorCode] unsignedLongValue];
    return 0;
}


void  FH_SetTypeCreator( CFURLRef fileURL, OSType type, OSType creator ) {
    NSString *filePath = [(NSURL *)fileURL path];
    NSNumber *typeNum = [NSNumber numberWithInt:type];
    NSNumber *creatorNum = [NSNumber numberWithInt:creator];
    NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
    [attributes setObject:typeNum forKey:NSFileHFSTypeCode];
    [attributes setObject:creatorNum forKey:NSFileHFSCreatorCode];
    [[NSFileManager defaultManager] setAttributes:attributes ofItemAtPath:filePath error:nil];
}

UInt64  FH_FileLength( NSURL *url ) {
    UInt64 returnLength = 0;
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc]init];
    NSFileHandle *fh = [NSFileHandle fileHandleForReadingFromURL:url error:NULL];
    if ( fh ) {
        UInt64    offSet = [fh offsetInFile];     // save current offset
        UInt64    length = [fh seekToEndOfFile];
        [fh seekToFileOffset:offSet];             // restore offset in file
                                                  //[fh release];
        returnLength = length;
    }
    [pool drain];
    return returnLength;
}
#pragma mark - Read functions
NSData * FH_ReadCopyData( NSUInteger length, SInt64 *offset, NSURL *url ) {
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc]init];
    NSFileHandle *fh = [NSFileHandle fileHandleForReadingFromURL:url error:NULL];
    [fh seekToFileOffset:*offset];
    NSData *data = [[fh readDataOfLength:length] retain]; // required because 'data' in the local pool is drained
    *offset = [fh offsetInFile];
    [pool drain];
    return data;
}

void  FH_PrintFile( CFURLRef url, CFStringRef string ) {
    [(NSString *)string writeToURL:(NSURL *)url atomically:NO encoding:NSUTF8StringEncoding error:nil];
}


CFStringRef  FH_InputFile( CFURLRef url, CFStringRef string ) {
    return (CFStringRef)[NSString stringWithContentsOfURL:(NSURL *)url encoding:NSUTF8StringEncoding error:nil];
}

OSStatus  FH_ReadChar( SInt64 * offset, UInt8 *value, NSURL *url ) {
    OSStatus err = 0;
    NSData *data = FH_ReadCopyData( sizeof(char), offset, url );
    if( data != nil ) {
        [data getBytes:value length:[data length]];
        [data release];
    } else {
        *value = '\0';
        //err = ioErr;
    }
    return err;     // always returns zero for now. Just testing. Brian 20171130
}

void  FH_ReadFile( void *p, SInt64 length, SInt64 *offset, NSURL *url ) {
    NSData *data = FH_ReadCopyData( length, offset, url ); // allow for file already being read and this is a subsequent read.
    if ( data != nil ) {
        [data getBytes:p length: [data length]];
        [data release];
    }
}


OSStatus FH_CheckFileLockStatus( int fd, int inStatusType, int *outFileLockStat ) {
    OSStatus err = noErr;
    struct flock fileLockRec = { 0, 0, 0, 0, 0 };
    
    //fileLockRec.l_type = F_WRLCK;
    //fileLockRec.l_type = F_RDLCK;
    fileLockRec.l_type = inStatusType;
    err = fcntl( fd, F_GETLK, &fileLockRec );
    if ( err >= 0 ) {
        *outFileLockStat = (int)fileLockRec.l_type;     // if no lock found to prevent a F_WRLCK( or F_RDLCK ), the lock type is set to F_UNLCK.
    } else {
        err = errno;
    }
    return err;
}

// 'Updateable' means available for reading and writing
// 'Writable'   means available only for writing.
OSStatus FH_IsFileAvailable( CFURLRef url, char *modeRequested, int *fd, int fileID ) {
    OSStatus err  = noErr;
    int openFileMode, localFD, fileLockStat = 0;
    
    const char *pathName = [[(NSURL *)url path] UTF8String];
    //
    // Does another process have this file open?  Note: the answer should be to try it with the unix 'open' below.
    // Instead this code creates a file handle which leaves open a crash possibility should Apple change their reaction.
    // Creating this file handle runs the risk the OS will deallocate and fail to do its own close of the file descriptor
    // because this code already does it. Since FH_IsFileAvailable() is called only on FB OPENs, there is no risk of exhausting file descriptors.
    // This is the error message the system generates because our 'close' usurped its close:
    // "BUG in libdispatch client: kevent[EVFILT_MACHPORT] monitored resource vanished before the source cancel handler was invoked"
    //
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc]init];
    NSFileHandle *fh = [NSFileHandle fileHandleForReadingFromURL:(NSURL *)url error:NULL];
    localFD = [fh fileDescriptor];  // file handle creation does not install a lock on the file
    err = FH_CheckFileLockStatus( localFD, F_WRLCK, &fileLockStat );
    close( localFD );

    switch ( *modeRequested ) {
        case 'N': openFileMode = O_RDWR   | O_SHLOCK   | O_NONBLOCK;
            if ( err == noErr ) {
                switch ( fileLockStat ) {
                    case F_RDLCK:                            // at least one process has a shared lock
                        *modeRequested = 'I';                // need to do an open "I" since other process has a shared lock
                        err = EACCES;                        // tell caller write permission denied
                        openFileMode = O_RDONLY   | O_SHLOCK   | O_NONBLOCK;
                    case F_UNLCK:                            // means file is unlocked and available.
                        localFD = open(pathName, openFileMode);
                        *fd = localFD;
                        //gFBOpenFiles[fileID].fh = [[NSFileHandle alloc]initWithFileDescriptor:localFD closeOnDealloc:YES];
                        break;
                    case F_WRLCK:                            // one process has an exclusive or write lock
                        err = EAGAIN;                        // resource temporarily not available
                        break;
                }
            }
            break;
        case 'I': openFileMode = O_RDONLY | O_SHLOCK   | O_NONBLOCK;
            if ( err == noErr ) {
                switch ( fileLockStat ) {
                    case F_RDLCK:                            // at least one process has a shared lock
                        //err = EACCES;                        // tell caller write permission denied
                                                             // 'break' intentionally omitted
                    case F_UNLCK:                            // means file is unlocked and available.
                        localFD = open(pathName, openFileMode);
                        *fd = localFD;
                        //gFBOpenFiles[fileID].fh = [[NSFileHandle alloc]initWithFileDescriptor:localFD closeOnDealloc:YES];
                        break;
                    case F_WRLCK:                            // one process has an exclusive or write lock
                        err = EAGAIN;                        // resource temporarily not available
                        break;
                }
            }

            break;
        case 'O':
        case 'A':
        case 'R': openFileMode = O_RDWR   | O_EXLOCK   | O_NONBLOCK;   // modes 'O','A','R' are all exclusive
            if ( err == noErr ) {
                switch ( fileLockStat ) {
                    case F_RDLCK:                            // at least one process has a shared lock
                    case F_WRLCK:                            // one process has an exclusive or write lock
                        err = EAGAIN;                        // resource temporarily not available
                        break;
                    case F_UNLCK:                            // means file is unlocked and available.
                        localFD = open(pathName, openFileMode);
                        *fd = localFD;
                        //gFBOpenFiles[fileID].fh = [[NSFileHandle alloc]initWithFileDescriptor:localFD closeOnDealloc:YES];
                        break;
                }
            }
            break;
        default:
            err = 999; // invalid modeRequested
            break;
            
    }
    [pool drain];
    return err;
}

#pragma mark - Write functions
//UInt64  FH_WriteData( CFURLRef url, CFDataRef inData, UInt64 inOffset, NSFileHandle *fh ) {
UInt64  FH_WriteData( CFDataRef inData, UInt64 inOffset, NSURL *url ) {
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc]init];
    NSFileHandle *fh = [NSFileHandle fileHandleForWritingToURL:url error:NULL];
    [fh seekToFileOffset:inOffset];
    [fh writeData:(NSData *)inData];
    UInt64 outOffset = [fh offsetInFile];
    [pool drain];
    return outOffset;
}


//UInt64  FH_WriteFile( CFURLRef url, void *p, UInt64 length, UInt64 offset, NSFileHandle *fh ) {
UInt64  FH_WriteFile( void *p, UInt64 length, UInt64 offset, NSURL *url ) {
    return FH_WriteData( (CFDataRef)[NSData dataWithBytes:p length:length], offset, url );
}


UInt64  FH_WriteChar( CFURLRef url, char theChar, SInt64 offset) {
    return FH_WriteData( (CFDataRef)[NSData dataWithBytes:&theChar length:sizeof(theChar)], offset, (NSURL *)url );;
}


BOOL  FH_BuildEmptyFile( CFURLRef url ) {
    NSFileManager *fm = [NSFileManager defaultManager];
    return [fm createFileAtPath: (NSString *)FH_URLGetPath( url ) contents:nil attributes:nil];
}

long  FH_FSpTrashObject( CFStringRef sourceObj ) {
    OSErr             err;
    err = noErr;
    if ( sourceObj ) {
        //NSAutoreleasePool *pool = [[NSAutoreleasePool alloc]init];
        NSString *src = [(NSString *)sourceObj stringByDeletingLastPathComponent];
        NSString *file = [(NSString *)sourceObj lastPathComponent];
        [[NSWorkspace sharedWorkspace] performFileOperation:NSWorkspaceRecycleOperation source:src destination:@"" files:[NSArray arrayWithObject:file] tag:nil];
        NSSound *systemSound = [[NSSound alloc] initWithContentsOfFile:@"/System/Library/Components/CoreAudio.component/Contents/SharedSupport/SystemSounds/dock/drag to trash.aif" byReference:YES];
        if(systemSound) {
            [systemSound play];
            [systemSound release];
        }
        //[pool drain];
    } else {
        err = 999;
    }
    return err;
}

OSStatus FH_SetFilePointerToEndOfFile( NSURL *url, SInt64 *filePos ) {
    OSStatus err = noErr;
    
    UInt64 length = FH_FileLength( url );
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc]init];
    NSFileHandle *fh = [NSFileHandle fileHandleForReadingFromURL:url error:NULL];
    @try {
        [fh seekToFileOffset: length];
        
    } @catch (NSException *theException) {
        if (theException.name == NSFileHandleOperationException) {
            err = fnOpnErr;
        }
    }
    
    //@finally {
    
    //}
    [pool drain];
    *filePos = length;
    return err;
}


UInt64 FH_TruncateFileAtOffset( NSFileHandle *fh,  UInt64 inOffset ) {
    [fh seekToFileOffset:inOffset];
    [fh truncateFileAtOffset: inOffset];
    return inOffset;
}
