/*
 MapKit.h
 
 Bernie Wylde 20190222
 */

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090

#import <MapKit/MapKit.h>
#import "CocoaUI.h"
#import "CoreLocation.h"

#if __LP64__
#ifdef MK_EXTERN

typedef id<MKAnnotation> MKAnnotationRef;
typedef MKAnnotationView *MKAnnotationViewRef;

typedef MKCircle *MKCircleRef;
typedef MKCircleRenderer *MKCircleRendererRef;
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
typedef MKClusterAnnotation *MKClusterAnnotationRef;
#endif

typedef MKDirections *MKDirectionsRef;
typedef MKDirectionsRequest *MKDirectionsRequestRef;
typedef MKDirectionsResponse *MKDirectionsResponseRef;
typedef MKDistanceFormatter *MKDistanceFormatterRef;

typedef MKETAResponse *MKETAResponseRef;

typedef MKGeodesicPolyline *MKGeodesicPolylineRef;

typedef MKLocalSearch *MKLocalSearchRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101104
typedef MKLocalSearchCompleter *MKLocalSearchCompleterRef;
typedef MKLocalSearchCompletion *MKLocalSearchCompletionRef;
#endif

typedef MKLocalSearchRequest *MKLocalSearchRequestRef;
typedef MKLocalSearchResponse *MKLocalSearchResponseRef;

typedef MKMapCamera *MKMapCameraRef;
typedef MKMapItem *MKMapItemRef;
typedef MKMapSnapshot *MKMapSnapshotRef;
typedef MKMapSnapshotter *MKMapSnapshotterRef;
typedef MKMapSnapshotOptions *MKMapSnapshotOptionsRef;
typedef MKMapView *MKMapViewRef;
typedef MKMultiPoint *MKMultiPointRef;

typedef id<MKOverlay> MKOverlayRef;
typedef MKOverlayRenderer *MKOverlayRendererRef;
typedef MKOverlayPathRenderer *MKOverlayPathRendererRef;

typedef MKPinAnnotationView *MKPinAnnotationViewRef;
typedef MKPlacemark *MKPlacemarkRef;
typedef MKPointAnnotation *MKPointAnnotationRef;
typedef MKPolygon *MKPolygonRef;
typedef MKPolygonRenderer *MKPolygonRendererRef;
typedef MKPolyline *MKPolylineRef;
typedef MKPolylineRenderer *MKPolylineRendererRef;

typedef MKRoute *MKRouteRef;
typedef MKRouteStep *MKRouteStepRef;

typedef MKShape *MKShapeRef;

typedef MKTileOverlay *MKTileOverlayRef;
typedef MKTileOverlayRenderer *MKTileOverlayRendererRef;

typedef MKUserLocation *MKUserLocationRef;


#pragma mark - MKAnnotation
///////////////////////////////////////////////////
//               MKAnnotation                    //
///////////////////////////////////////////////////
// Position
CLLocationCoordinate2D MKAnnotationCoordinate( MKAnnotationRef ref );
void MKAnnotationSetCoordinate( MKAnnotationRef ref, CLLocationCoordinate2D coordinate );

// Title
CFStringRef MKAnnotationTitle( MKAnnotationRef ref );
CFStringRef MKAnnotationSubtitle( MKAnnotationRef ref );


#pragma mark - MKAnnotationView
///////////////////////////////////////////////////
//           MKAnnotationView                    //
///////////////////////////////////////////////////
// Create and prepare
MKAnnotationViewRef MKAnnotationViewWithAnnotation( MKAnnotationRef annotation, CFStringRef reuseIdentifier );
void MKAnnotationViewPrepareForReuse( MKAnnotationViewRef ref );
void MKAnnotationViewPrepareForDisplay( MKAnnotationViewRef ref );

// Priority
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
MKFeatureDisplayPriority MKAnnotationViewDisplayPriority( MKAnnotationViewRef ref );
#endif

// Attributes
Boolean MKAnnotationViewIsEnabled( MKAnnotationViewRef ref );
void MKAnnotationViewSetEnabled( MKAnnotationViewRef ref, Boolean flag );
ImageRef MKAnnotationViewImage( MKAnnotationViewRef ref );
void MKAnnotationViewSetImage( MKAnnotationViewRef ref, ImageRef image );
Boolean MKAnnotationViewIsHighlighted( MKAnnotationViewRef ref );
void MKAnnotationViewSetHighlighted( MKAnnotationViewRef ref, Boolean flag );
MKAnnotationRef MKAnnotationViewAnnotation( MKAnnotationViewRef ref );
void MKAnnotationViewSetAnnotation( MKAnnotationViewRef ref, MKAnnotationRef annotation );
CGPoint MKAnnotationViewCenterOffset( MKAnnotationViewRef ref );
void MKAnnotationViewSetCenterOffset( MKAnnotationViewRef ref, CGPoint offset );
CGPoint MKAnnotationViewCalloutOffset( MKAnnotationViewRef ref );
void MKAnnotationViewSetCalloutOffset( MKAnnotationViewRef ref, CGPoint offset );
CFStringRef MKAnnotationViewReuseIdentifier( MKAnnotationViewRef ref );

// Selection state
void MKAnnotationViewSetSelected( MKAnnotationViewRef ref, Boolean selected, Boolean animated );
Boolean MKAnnotationViewIsSelected( MKAnnotationViewRef ref );

// Callout views
Boolean MKAnnotationViewCanShowCallout( MKAnnotationViewRef ref );
void MKAnnotationViewSetCanShowCallout( MKAnnotationViewRef ref, Boolean flag );
ViewRef MKAnnotationViewLeftCalloutAccessoryView( MKAnnotationViewRef ref );
void MKAnnotationViewSetLeftCalloutAccessoryView( MKAnnotationViewRef ref, ViewRef vw );
ViewRef MKAnnotationViewRightCalloutAccessoryView( MKAnnotationViewRef ref );
void MKAnnotationViewSetRightCalloutAccessoryView( MKAnnotationViewRef ref, ViewRef vw );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ViewRef MKAnnotationViewDetailCalloutAccessoryView( MKAnnotationViewRef ref );
void MKAnnotationViewSetDetailCalloutAccessoryView( MKAnnotationViewRef ref, ViewRef vw );
#endif
CGPoint MKAnnotationViewLeftCalloutOffset( MKAnnotationViewRef ref );
void MKAnnotationViewSetLeftCalloutOffset( MKAnnotationViewRef ref, CGPoint offset );
CGPoint MKAnnotationViewRightCalloutOffset( MKAnnotationViewRef ref );
void MKAnnotationViewSetRightCalloutOffset( MKAnnotationViewRef ref, CGPoint offset );

// Drag operations
Boolean MKAnnotationViewIsDraggable( MKAnnotationViewRef ref );
void MKAnnotationViewSetDraggable( MKAnnotationViewRef ref, Boolean flag );
void MKAnnotationViewSetDragState( MKAnnotationViewRef ref, MKAnnotationViewDragState state, Boolean animated );
MKAnnotationViewDragState mkAnnotationViewDragState( MKAnnotationViewRef ref );

// Collisions
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
MKAnnotationViewCollisionMode mkAnnotationViewCollisionMode( MKAnnotationViewRef ref );
void MKAnnotationViewSetCollisionMode( MKAnnotationViewRef ref, MKAnnotationViewCollisionMode mode );
#endif

// Clustering
CFStringRef MKAnnotationViewClusteringIdentifier( MKAnnotationViewRef ref );
void MKAnnotationViewSetClusteringIdentifier( MKAnnotationViewRef ref, CFStringRef identifier );
MKAnnotationViewRef MKAnnotationViewClusterAnnotationView( MKAnnotationViewRef ref );


#pragma mark - MKCircle
///////////////////////////////////////////////////
//                   MKCircle                    //
///////////////////////////////////////////////////
// Create
MKCircleRef MKCircleWithCenterCoordinate( CLLocationCoordinate2D coord, CLLocationDistance radius );
MKCircleRef MKCircleWithMapRect( MKMapRect r );

// Attributes
CLLocationCoordinate2D MKCircleCoordinate( MKCircleRef ref );
CLLocationDistance MKCircleRadius( MKCircleRef ref );
MKMapRect MKCircleBoundingMapRect( MKCircleRef ref );


#pragma mark - MKCircleRenderer
///////////////////////////////////////////////////
//              MKCircleRenderer                 //
///////////////////////////////////////////////////
// Create
MKCircleRendererRef MKCircleRendererWithCircle( MKCircleRef circle );

// Overlay object
MKCircleRef MKCircleRendererCircle( MKCircleRendererRef ref );


#pragma mark - MKClusterAnnotation
///////////////////////////////////////////////////
//            MKClusterAnnotation                //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
// Create
MKClusterAnnotationRef MKClusterAnnotationWithMemberAnnotations( CFArrayRef annotations );

// Attributes
CFStringRef MKClusterAnnotationTitle( MKClusterAnnotationRef ref );
void MKClusterAnnotationSetTitle( MKClusterAnnotationRef ref, CFStringRef title );
CFStringRef MKClusterAnnotationSubTitle( MKClusterAnnotationRef ref );
void MKClusterAnnotationSetSubtitle( MKClusterAnnotationRef ref, CFStringRef subtitle );

// Annotations
CFArrayRef MKClusterAnnotationMemberAnnotations( MKClusterAnnotationRef ref );
#endif


#pragma mark - MKDirections
///////////////////////////////////////////////////
//               MKDirections                    //
///////////////////////////////////////////////////
// callbacks
typedef void (*DirectionsCompletionHandlerType)(MKDirectionsResponse*,NSError*,void*);
typedef void (*ETACompletionHandlerType)(MKETAResponse*,NSError*,void*);

// Create
MKDirectionsRef MKDirectionsWithRequest( MKDirectionsRequestRef request );

// Directions
void MKDirectionsCalculateDirections( MKDirectionsRef ref, DirectionsCompletionHandlerType fnAddress, void *userData );

// ETA
void MKDirectionsCalculateETA( MKDirectionsRef ref, ETACompletionHandlerType fnAddress, void *userData );

// Request
void MKDirectionsCancel( MKDirectionsRef ref );
Boolean MKDirectionsIsCalculating( MKDirectionsRef ref );


#pragma mark - MKDirectionsRequest
///////////////////////////////////////////////////
//           MKDirectionsRequest                 //
///////////////////////////////////////////////////
// Create
MKDirectionsRequestRef MKDirectionsRequestInit( void );
Boolean MKDirectionsRequestIsDirectionsRequestURL( CFURLRef url );
MKDirectionsRequestRef MKDirectionsRequestWithContentsOfURL( CFURLRef url );

// Start and end points
MKMapItemRef MKDirectionsRequestSource( MKDirectionsRequestRef ref );
void MKDirectionsRequestSetSource( MKDirectionsRequestRef ref, MKMapItemRef source );
MKMapItemRef MKDirectionsRequestDestination( MKDirectionsRequestRef ref );
void MKDirectionsRequestSetDestination( MKDirectionsRequestRef ref, MKMapItemRef destination );

// Transportation options
MKDirectionsTransportType mkDirectionsRequestTransportType( MKDirectionsRequestRef ref );
void MKDirectionsRequestSetTransportType( MKDirectionsRequestRef ref, MKDirectionsTransportType type );
Boolean MKDirectionsRequestRequestsAlternateRoutes( MKDirectionsRequestRef ref );
void MKDirectionsRequestSetRequestsAlternateRoutes( MKDirectionsRequestRef ref, Boolean flag );
CFDateRef MKDirectionsRequestDepartureDate( MKDirectionsRequestRef ref );
void MKDirectionsRequestSetDepartureDate( MKDirectionsRequestRef ref, CFDateRef dt );
CFDateRef MKDirectionsRequestArrivalDate( MKDirectionsRequestRef ref );
void MKDirectionsRequestSetArrivalDate( MKDirectionsRequestRef ref, CFDateRef dt );


#pragma mark - MKDirectionsResponse
///////////////////////////////////////////////////
//           MKDirectionsResponse                //
///////////////////////////////////////////////////
// End points
MKMapItemRef MKDirectionsResponseSource( MKDirectionsResponseRef ref );
MKMapItemRef MKDirectionsResponseDestination( MKDirectionsResponseRef ref );

// Route info
CFArrayRef MKDirectionsResponseRoutes( MKDirectionsResponseRef ref );


#pragma mark - MKDistanceFormatter
///////////////////////////////////////////////////
//           MKDistanceFormatter                 //
///////////////////////////////////////////////////
// Converting distances
CFStringRef MKDistanceFormatterStringFromDistance( MKDistanceFormatterRef ref, CLLocationDistance distance );
CLLocationDistance MKDistanceFormatterDistanceFromString( MKDistanceFormatterRef ref, CFStringRef string );

// Format
CFLocaleRef MKDistanceFormatterLocale( MKDistanceFormatterRef ref );
void MKDistanceFormatterSetLocale( MKDistanceFormatterRef ref, CFLocaleRef locale );
MKDistanceFormatterUnits mkDistanceFormatterUnits( MKDistanceFormatterRef ref );
void MKDistanceFormatterSetUnits( MKDistanceFormatterRef ref, MKDistanceFormatterUnits units );
MKDistanceFormatterUnitStyle mkDistanceFormatterUnitStyle( MKDistanceFormatterRef ref );
void MKDistanceFormatterSetUnitStyle( MKDistanceFormatterRef ref, MKDistanceFormatterUnitStyle style );


#pragma mark - MKETAResponse
///////////////////////////////////////////////////
//                 MKETAResponse                 //
///////////////////////////////////////////////////
// End points
MKMapItemRef MKETAResponseSource( MKETAResponseRef ref );
MKMapItemRef MKETAResponseDestination( MKETAResponseRef ref );

// Info
CFTimeInterval MKETAResponseExpectedTravelTime( MKETAResponseRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFDateRef MKETAResponseExpectedDepartureDate( MKETAResponseRef ref );
CFDateRef MKETAResponseExpectedArrivalDate( MKETAResponseRef ref );
CLLocationDistance MKETAResponseDistance( MKETAResponseRef ref );
MKDirectionsTransportType MKETAResponseTransportType( MKETAResponseRef ref );
#endif


#pragma mark - MKGeodesicPolyline
///////////////////////////////////////////////////
//              MKGeodesicPolyline               //
///////////////////////////////////////////////////
// Create
MKGeodesicPolylineRef MKGeodesicPolylineWithPoints( MKMapPoint *points, NSUInteger count );
MKGeodesicPolylineRef MKGeodesicPolylineWithCoordinates( CLLocationCoordinate2D *coords, NSUInteger count );


#pragma mark - MKLocalSearch
///////////////////////////////////////////////////
//                 MKLocalSearch                 //
///////////////////////////////////////////////////
// callback
typedef void (*LocalSearchCompletionHandlerType)(MKLocalSearchResponse*,NSError*,void*);

// Create
MKLocalSearchRef MKLocalSearchWithRequest( MKLocalSearchRequestRef request );

// Perform search
void MKLocalSearchStart( MKLocalSearchRef ref, LocalSearchCompletionHandlerType fnAddress, void *userData );
Boolean MKLocalSearchIsSearching( MKLocalSearchRef ref );
void MKLocalSearchCancel( MKLocalSearchRef ref );


#pragma mark - MKLocalSearchCompleter
///////////////////////////////////////////////////
//           MKLocalSearchCompleter              //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101104
// Query attributew
CFStringRef MKLocalSearchCompleterQueryFragment( MKLocalSearchCompleterRef ref );
void MKLocalSearchCompleterSetQueryFragment( MKLocalSearchCompleterRef ref, CFStringRef fragment );
MKCoordinateRegion MKLocalSearchCompleterRegion( MKLocalSearchCompleterRef ref );
void MKLocalSearchCompleterSetRegion( MKLocalSearchCompleterRef ref, MKCoordinateRegion region );
MKSearchCompletionFilterType MKLocalSearchCompleterFilterType( MKLocalSearchCompleterRef ref );
void MKLocalSearchCompleterSetFilterType( MKLocalSearchCompleterRef ref, MKSearchCompletionFilterType type );

// Cancelling
void MKLocalSearchCompleterCancel( MKLocalSearchCompleterRef ref );
Boolean MKLocalSearchCompleterIsSearching( MKLocalSearchCompleterRef ref );

// Query results
CFArrayRef MKLocalSearchCompleterResults( MKLocalSearchCompleterRef ref );
#endif

#pragma mark - MKLocalSearchCompletion
///////////////////////////////////////////////////
//           MKLocalSearchCompletion             //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101104
// Get completions
CFStringRef MKLocalSearchCompletionTitle( MKLocalSearchCompletionRef ref );
CFStringRef MKLocalSearchCompletionSubtitle( MKLocalSearchCompletionRef ref );
CFArrayRef MKLocalSearchCompletionTitleHighlightRanges( MKLocalSearchCompletionRef ref );
CFArrayRef MKLocalSearchCompletionSubtitleHighlightRanges( MKLocalSearchCompletionRef ref );
#endif


#pragma mark - MKLocalSearchRequest
///////////////////////////////////////////////////
//             MKLocalSearchRequest              //
///////////////////////////////////////////////////
// Init
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101104
MKLocalSearchRequestRef MKLocalSearchRequestWithCompletion( MKLocalSearchCompletionRef completion );
#endif

// Search parameters
CFStringRef MKLocalSearchRequestNaturalLanguageQuery( MKLocalSearchRequestRef ref );
void MKLocalSearchRequestSetNaturalLanguageQuery( MKLocalSearchRequestRef ref, CFStringRef string );
MKCoordinateRegion MKLocalSearchRequestRegion( MKLocalSearchRequestRef ref );
void MKLocalSearchRequestSetRegion( MKLocalSearchRequestRef ref, MKCoordinateRegion region );


#pragma mark - MKLocalSearchResponse
///////////////////////////////////////////////////
//            MKLocalSearchResponse              //
///////////////////////////////////////////////////
// Search results
CFArrayRef MKLocalSearchResponseMapItems( MKLocalSearchResponseRef ref );
MKCoordinateRegion MKLocalSearchResponseBoundingRegion( MKLocalSearchResponseRef ref );


#pragma mark - MKMapCamera
///////////////////////////////////////////////////
//                  MKMapCamera                  //
///////////////////////////////////////////////////
// Init
MKMapCameraRef MKMapCameraInit( void );
MKMapCameraRef MKMapCameraFromEyeCoordinate( CLLocationCoordinate2D centerCoordinate, CLLocationCoordinate2D eyeCoordinate, CLLocationDistance eyeAltitude );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
MKMapCameraRef MKMapCameraFromDistance( CLLocationCoordinate2D centerCoordinate, CLLocationDistance distance, CGFloat pitch, CLLocationDirection heading );
#endif

// Viewing angle
CLLocationCoordinate2D MKMapCameraCenterCoordinate( MKMapCameraRef ref );
void MKMapCameraSetCenterCoordinate( MKMapCameraRef ref, CLLocationCoordinate2D coordinate );
CLLocationDirection MKMapCameraHeading( MKMapCameraRef ref );
void MKMapCameraSetHeading( MKMapCameraRef ref, CLLocationDirection heading );
CGFloat MKMapCameraPitch( MKMapCameraRef ref );
void MKMapCameraSetPitch( MKMapCameraRef ref, CGFloat pitch );
CLLocationDistance MKMapCameraAltitude( MKMapCameraRef ref );
void MKMapCameraSetAltitude( MKMapCameraRef ref, CLLocationDistance altitude );


#pragma mark - MKMapItem
///////////////////////////////////////////////////
//                  MKMapItem                    //
///////////////////////////////////////////////////
// Create
MKMapItemRef MKMapItemForCurrentLocation( void );
MKMapItemRef MKMapItemWithPlacemark( MKPlacemarkRef placemark );

// Attributes
MKPlacemarkRef MKMapItemPlacemark( MKMapItemRef ref );
Boolean MKMapItemIsCurrentLocation( MKMapItemRef ref );
CFStringRef MKMapItemName( MKMapItemRef ref );
void MKMapItemSetName( MKMapItemRef ref, CFStringRef name );
CFStringRef MKMapItemPhoneNumber( MKMapItemRef ref );
void MKMapItemSetPhoneNumbewr( MKMapItemRef ref, CFStringRef number );
CFURLRef MKMapItemURL( MKMapItemRef ref );
void MKMapItemSetURL( MKMapItemRef ref, CFURLRef url );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFTimeZoneRef MKMapItemTimeZone( MKMapItemRef ref );
void MKMapItemSetTimeZone( MKMapItemRef ref, CFTimeZoneRef zone );
#endif

// Launch Maps app
Boolean MKMapItemOpenMapsWithItems( CFArrayRef items, CFDictionaryRef options );
Boolean MKMapItemOpenInMaps( MKMapItemRef ref, CFDictionaryRef options );


#pragma mark - MKMapSnapshot
///////////////////////////////////////////////////
//              MKMapSnapshot                    //
///////////////////////////////////////////////////
// Image
ImageRef MKMapSnapshotImage( MKMapSnapshotRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
AppearanceRef MKMapSnapshotAppearance( MKMapSnapshotRef ref );
#endif

// Points on image
CGPoint MKMapSnapshotPointForCoordinate( MKMapSnapshotRef ref, CLLocationCoordinate2D coordinate );


#pragma mark - MKMapSnapshotOptions
///////////////////////////////////////////////////
//           MKMapSnapshotOptions                //
///////////////////////////////////////////////////
// Configure map data
MKMapCameraRef MKMapSnapshotOptionsCamera( MKMapSnapshotOptionsRef ref );
void MKMapSnapshotOptionsSetCamera( MKMapSnapshotOptionsRef ref, MKMapCameraRef camera );
MKCoordinateRegion MKMapSnapshotOptionsRegion( MKMapSnapshotOptionsRef ref );
void MKMapSnapshotOptionsSetRegion( MKMapSnapshotOptionsRef ref, MKCoordinateRegion region );
MKMapRect MKMapSnapshotOptionsMapRect( MKMapSnapshotOptionsRef ref );
void MKMapSnapshotOptionsSetMapRect( MKMapSnapshotOptionsRef ref, MKMapRect rect );
MKMapType MKMapSnapshotOptionsMapType( MKMapSnapshotOptionsRef ref );
void MKMapSnapshotOptionsSetMapType( MKMapSnapshotOptionsRef ref, MKMapType type );
Boolean MKMapSnapshotOptionsShowsPointsOfInterest( MKMapSnapshotOptionsRef ref );
void MKMapSnapshotOptionsSetShowsPointsOfInterest( MKMapSnapshotOptionsRef ref, Boolean flag );
Boolean MKMapSnapshotOptionsShowsBuildings( MKMapSnapshotOptionsRef ref );
void MKMapSnapshotOptionsSetShowsBuildings( MKMapSnapshotOptionsRef ref, Boolean flag );

// Image output
CGSize MKMapSnapshotOptionsSize( MKMapSnapshotOptionsRef ref );
void MKMapSnapshotOptionsSetSize( MKMapSnapshotOptionsRef ref, CGSize size );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
AppearanceRef MKMapSnapshotOptionsAppearance( MKMapSnapshotOptionsRef ref );
void MKMapSnapshotOptionsSetAppearance( MKMapSnapshotOptionsRef ref, AppearanceRef appearnce );
#endif


#pragma mark - MKMapSnapshotter
///////////////////////////////////////////////////
//               MKMapSnapshotter                //
///////////////////////////////////////////////////
// callback
typedef void (*MapSnapshotterCompletionHandlerType)(MKMapSnapshot*,NSError*,void*);

// Create
MKMapSnapshotterRef MKMapSnapshotterWithOptions( MKMapSnapshotOptionsRef options );

// Generate snapshot
void MKMapSnapshotterStart( MKMapSnapshotterRef ref, MapSnapshotterCompletionHandlerType fnAddress, void *userData );
void MKMapSnapshotterCancel( MKMapSnapshotterRef ref );
Boolean MKMapSnapshotterIsLoading( MKMapSnapshotterRef ref );


#pragma mark - MKMapView
///////////////////////////////////////////////////
//                  MKMapView                    //
///////////////////////////////////////////////////
@interface MapViewSC : MKMapView
@end

// dialog events
enum {
    mapViewRegionWillChange = 3100,
    mapViewDidChangeVisibleRegion,
    mapViewRegionDidChange,
    mapViewWillStartLoadingMap,
    mapViewDidFinishLoadingMap,
    mapViewDidFailLoadingMap,
    mapViewWillStartRenderingMap,
    mapViewDidFinishRenderingMap,
    mapViewWillStartLocatingUser,
    mapViewDidStopLocatingUser,
    mapViewDidUpdateUserLocation,
    mapViewDidFailToLocateUser,
    mapViewViewForAnnotation,
    mapViewDidAddAnnotationViews,
    mapViewClusterAnnotationForMemberAnnotations,
    mapViewAnnotationViewDidChangeDragState,
    mapViewDidSelectAnnotationView,
    mapViewDidDeselectAnnotationView,
    mapViewRendererForOverlay,
    mapViewDidAddOverlayRenderers
};

@interface MKMapView (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

@interface CocoaMKMapViewDelegate : NSObject <MKMapViewDelegate>
@end

CocoaMKMapViewDelegate *CocoaMKMapViewDelegateObj( void );

MKMapView *MKMapViewWithTag( NSInteger tag );
BOOL MKMapViewExists( NSInteger tag );

// Properties
MKMapType MKMapViewMapType( NSInteger tag );
void MKMapViewSetMapType( NSInteger tag, MKMapType type );
Boolean MKMapViewIsZoomEnabled( NSInteger tag );
void MKMapViewSetZoomEnabled( NSInteger tag, Boolean flag );
Boolean MKMapViewIsScrollEnabled( NSInteger tag );
void MKMapViewSetScrollEnabled( NSInteger tag, Boolean flag );
Boolean MKMapViewIsPitchEnabled( NSInteger tag );
void MKMapViewSetPitchEnabled( NSInteger tag, Boolean flag );
Boolean MKMapViewIsRotateEnabled( NSInteger tag );
void MKMapViewSetRotateEnabled( NSInteger tag, Boolean flag );

// Visible portion
MKCoordinateRegion MKMapViewRegion( NSInteger tag );

void MKMapViewSetRegion( NSInteger tag, MKCoordinateRegion region, Boolean animated );

CLLocationCoordinate2D MKMapViewCenterCoordinate( NSInteger tag );
void MKMapViewSetCenterCoordinate( NSInteger tag, CLLocationCoordinate2D coordinate, Boolean animated );
void MKMapViewShowAnnotations( NSInteger tag, CFArrayRef annotations, Boolean animated );
MKMapRect MKMapViewVisibleMapRect( NSInteger tag );
void MKMapViewSetVisibleMapRect( NSInteger tag, MKMapRect rect, Boolean animated );
void MKMapViewSetVisibleMapRectEdgePadding( NSInteger tag, MKMapRect rect, NSEdgeInsets insets, Boolean animated );

// Appearance
MKMapCameraRef MKMapViewCamera( NSInteger tag );
void MKMapViewSetCamera( NSInteger tag, MKMapCameraRef camera, Boolean animated );
Boolean MKMapViewShowsPointsOfInterest( NSInteger tag );
void MKMapViewSetShowsPointsOfInterest( NSInteger tag, Boolean flag );
Boolean MKMapViewShowsBuildings( NSInteger tag );
void MKMapViewSetShowsBuildings( NSInteger tag, Boolean flag );
Boolean MKMapViewShowsCompass( NSInteger tag );
void MKMapViewSetShowsCompass( NSInteger tag, Boolean flag );
Boolean MKMapViewShowsZoomControls( NSInteger tag );
void MKMapViewSetShowsZoomControls( NSInteger tag, Boolean flag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean MKMapViewShowsScale( NSInteger tag );
void MKMapViewSetShowsScale( NSInteger tag, Boolean flag );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean MKMapViewShowsTraffic( NSInteger tag );
void MKMapViewSetShowsTraffic( NSInteger tag, Boolean flag );
#endif

// User location
Boolean MKMapViewShowsUserLocation( NSInteger tag );
void MKMapViewSetShowsUserLocation( NSInteger tag, Boolean flag );
Boolean MKMapViewIsUserLocationVisible( NSInteger tag );
MKUserLocationRef MKMapViewUserLocation( NSInteger tag );

// Annotating
CFArrayRef MKMapViewAnnotations( NSInteger tag );
void MKMapViewAddAnnotation( NSInteger tag, MKAnnotationRef annotation );
void MKMapViewAddAnnotations( NSInteger tag, CFArrayRef annotations );
void MKMapViewRemoveAnnotation( NSInteger tag, MKAnnotationRef annotation );
void MKMapViewRemoveAnnotations( NSInteger tag, CFArrayRef annotations );
CFSetRef MKMapViewAnnotationsInMapRect( NSInteger tag, MKMapRect rect );

// Annotation selections
CGRect MKMapViewAnnotationVisibleRect( NSInteger tag );
CFArrayRef MKMapViewSelectedAnnotations( NSInteger tag );
void MKMapViewSelectAnnotation( NSInteger tag, MKAnnotationRef annotation, Boolean animated );
void MKMapViewDeselectAnnotation( NSInteger tag, MKAnnotationRef annotation, Boolean animated );

// Annotation views
// ...
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
MKAnnotationViewRef MKMapViewDequeueReusableAnnotationViewForAnnotation( NSInteger tag, CFStringRef identifier, MKAnnotationRef annotation );
#endif
MKAnnotationViewRef MKMapViewDequeueReusableAnnotationView( NSInteger tag, CFStringRef identifier );
MKAnnotationViewRef MKMapViewViewForAnnotation( NSInteger tag, MKAnnotationRef annotation );

// Accessing overlays
CFArrayRef MKMapViewOverlays( NSInteger tag );
CFArrayRef MKMapViewOverlaysInLevel( NSInteger tag, MKOverlayLevel level );
MKOverlayRendererRef MKMapViewRendererForOverlay( NSInteger tag, MKOverlayRef overlay );

// Add overlays
void MKMapViewAddOverlayAtLevel( NSInteger tag, MKOverlayRef overlay, MKOverlayLevel level );
void MKMapViewAddOverlaysAtLevel( NSInteger tag, CFArrayRef overlays, MKOverlayLevel level );
void MKMapViewAddOverlay( NSInteger tag, MKOverlayRef overlay );
void MKMapViewAddOverlays( NSInteger tag, CFArrayRef overlays );
void MKMapViewInsertOverlayAtLevel( NSInteger tag, MKOverlayRef overlay, NSUInteger index, MKOverlayLevel level );
void MKMapViewInsertOverlay( NSInteger tag, MKOverlayRef overlay, NSUInteger index );
void MKMapViewInsertOverlayAboveOverlay( NSInteger tag, MKOverlayRef overlay1, MKOverlayRef overlay2 );
void MKMapViewInsertOverlayBelowOverlay( NSInteger tag, MKOverlayRef overlay1, MKOverlayRef overlay2 );
void MKMapViewExchangeOverlay( NSInteger tag, MKOverlayRef overlay1, MKOverlayRef overlay2 );
void MKMapViewExchangeOverlayAtIndex( NSInteger tag, NSUInteger index1, NSUInteger index2 );

// Remove overlays
void MKMapViewRemoveOverlay( NSInteger tag, MKOverlayRef overlay );
void MKMapViewRemoveOverlays( NSInteger tag, CFArrayRef overlays );

// Convert map coordinates

// Regions and rectangles
MKCoordinateRegion MKMapViewRegionThatFits( NSInteger tag, MKCoordinateRegion region );
MKMapRect MKMapViewMapRectThatFits( NSInteger tag, MKMapRect rect );
MKMapRect MKMapViewMapRectThatFitsEdgePadding( NSInteger tag, MKMapRect rect, NSEdgeInsets insets );

// Custom
void MKMapViewInstallDelegate( NSInteger tag );


// Init
MKMapViewRef MKMapViewInit( NSInteger tag, CGRect r );

// mapview statement
void CocoaMapView( NSInteger inTag, CGRect inRect, NSInteger inWndTag );


#pragma mark - MKMultiPoint
///////////////////////////////////////////////////
//                MKMultiPoint                   //
///////////////////////////////////////////////////
// Points
MKMapPoint *MKMultiPointPoints( MKMultiPointRef ref );
NSUInteger MKMultiPointCount( MKMultiPointRef ref );

// Coordinate values
void MKMultiPointGetCoordinates( MKMultiPointRef ref, CLLocationCoordinate2D *coords, CFRange range );


#pragma mark - MKOverlay
///////////////////////////////////////////////////
//                  MKOverlay                    //
///////////////////////////////////////////////////
// Geometry
CLLocationCoordinate2D MKOverlayCoordinate( MKOverlayRef ref );
MKMapRect MKOverlayBoundingMapRect( MKOverlayRef ref );

// Intersections
Boolean MKOverlayIntersectsMapRect( MKOverlayRef ref, MKMapRect rect );

// Rendering
Boolean MKOverlayCanReplaceMapContent( MKOverlayRef ref );


#pragma mark - MKOverlayPathRenderer
///////////////////////////////////////////////////
//         MKOverlayPathRenderer                 //
///////////////////////////////////////////////////
// Path
CGPathRef MKOverlayPathRendererPath( MKOverlayPathRendererRef ref );
void MKOverlayPathRendererSetPath( MKOverlayPathRendererRef ref, CGPathRef path );
void MKOverlayPathRendererCreatePath( MKOverlayPathRendererRef ref );
void MKOverlayPathRendererInvalidatePath( MKOverlayPathRendererRef ref );

// Drawing attributes
ColorRef MKOverlayPathRendererFillColor( MKOverlayPathRendererRef ref );
void MKOverlayPathRendererSetFillColor( MKOverlayPathRendererRef ref, ColorRef col );
ColorRef MKOverlayPathRendererStrokeColor( MKOverlayPathRendererRef ref );
void MKOverlayPathRendererSetStrokeColor( MKOverlayPathRendererRef ref, ColorRef col );
CGFloat MKOverlayPathRendererLineWidth( MKOverlayPathRendererRef ref );
void MKOverlayPathRendererSetLineWidth( MKOverlayPathRendererRef ref, CGFloat lineWidth );
CGLineJoin MKOverlayPathRendererLineJoin( MKOverlayPathRendererRef ref );
void MKOverlayPathRendererSetLineJoin( MKOverlayPathRendererRef ref, CGLineJoin lineJoin );
CGLineCap MKOverlayPathRendererLineCap( MKOverlayPathRendererRef ref );
void MKOverlayPathRendererSetLineCap( MKOverlayPathRendererRef ref, CGLineCap lineCap );
CGFloat MKOverlayPathRendererMiterLimit( MKOverlayPathRendererRef ref );
void MKOverlayPathRendererSetMiterLimit( MKOverlayPathRendererRef ref, CGFloat limit );
CGFloat MKOverlayPathRendererLineDashPhase( MKOverlayPathRendererRef ref );
void MKOverlayPathRendererSetLineDashPhase( MKOverlayPathRendererRef ref, CGFloat phase );
CFArrayRef MKOverlayPathRendererLineDashPattern( MKOverlayPathRendererRef ref );
void MKOverlayPathRendererSetLineDashPattern( MKOverlayPathRendererRef ref, CFArrayRef pattern );

// Drawing the path
void MKOverlayPathRendererApplyStrokeProperties( MKOverlayPathRendererRef ref, CGContextRef ctx, MKZoomScale zoomScale );
void MKOverlayPathRendererApplyFillProperties( MKOverlayPathRendererRef ref, CGContextRef ctx, MKZoomScale zoomScale );
void MKOverlayPathRendererStrokePath( MKOverlayPathRendererRef ref, CGPathRef path, CGContextRef ctx );
void MKOverlayPathRendererFillPath( MKOverlayPathRendererRef ref, CGPathRef path, CGContextRef ctx );


#pragma mark - MKOverlayRenderer
///////////////////////////////////////////////////
//              MKOverlayRenderer                //
///////////////////////////////////////////////////
// Create
MKOverlayRendererRef MKOverlayRendererWithOverlay( MKOverlayRef overlay );

// Attributes
MKOverlayRef MKOverlayRendererOverlay( MKOverlayRendererRef ref );
CGFloat MKOverlayRendererAlpha( MKOverlayRendererRef ref );
void MKOverlayRendererSetAlpha( MKOverlayRendererRef ref, CGFloat alpha );
CGFloat MKOverlayRendererContentScaleFactor( MKOverlayRendererRef ref );

// Converting points
CGPoint MKOverlayRendererPointForMapPoint( MKOverlayRendererRef ref, MKMapPoint pt );
MKMapPoint MKOverlayRendererMapPointForPoint( MKOverlayRendererRef ref, CGPoint pt );
CGRect MKOverlayRendererRectForMapRect( MKOverlayRendererRef ref, MKMapRect rect );
MKMapRect MKOverlayRendererMapRectForRect( MKOverlayRendererRef ref, CGRect rect );

// Drawing
Boolean MKOverlayRendererCanDrawMapRect( MKOverlayRendererRef ref, MKMapRect rect, MKZoomScale zoomScale );
void MKOverlayRendererDrawMapRect( MKOverlayRendererRef ref, MKMapRect rect, MKZoomScale zoomScale, CGContextRef ctx );
void MKOverlayRendererSetNeedsDisplay( MKOverlayRendererRef ref );
void MKOverlayRendererSetNeedsDisplayInMapRect( MKOverlayRendererRef ref, MKMapRect rect );
void MKOverlayRendererSetNeedsDisplayZoomScale( MKOverlayRendererRef ref, MKMapRect rect, MKZoomScale zoomScale );


#pragma mark - MKPinAnnotationView
///////////////////////////////////////////////////
//             MKPinAnnotationView               //
///////////////////////////////////////////////////
// Standard pin colors
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ColorRef MKPinAnnotationViewRedPinColor( void );
ColorRef MKPinAnnotationViewGreenPinColor( void );
ColorRef MKPinAnnotationViewPurplePinColor( void );
#endif

// Attributes
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ColorRef MKPinAnnotationViewPinTintColor( MKPinAnnotationViewRef ref );
void MKPinAnnotationViewSetPinTintColor( MKPinAnnotationViewRef ref, ColorRef col );
#endif
Boolean MKPinAnnotationViewAnimatesDrop( MKPinAnnotationViewRef ref );
void MKPinAnnotationViewSetAnimatesDrop( MKPinAnnotationViewRef ref, Boolean flag );


#pragma mark - MKPlacemark
///////////////////////////////////////////////////
//                  MKPlacemark                  //
///////////////////////////////////////////////////
// Create
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
MKPlacemarkRef MKPlacemarkWithCoordinate( CLLocationCoordinate2D coordinate );
#ifdef CNPostalAddressRef
MKPlacemarkRef MKPlacemarkWithCoordinatePostalAddress( CLLocationCoordinate2D coordinate, CNPostalAddressRef postalAddress );
#endif // CNPostalAddressRef
#endif // __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
MKPlacemarkRef MKPlacemarkWithCoordinateAddressDictionary( CLLocationCoordinate2D coordinate, CFDictionaryRef addressDict );

// Attributes
CFStringRef MKPlacemarkCountryCode( MKPlacemarkRef ref );


#pragma mark - MKPointAnnotation
///////////////////////////////////////////////////
//             MKPointAnnotation                 //
///////////////////////////////////////////////////
// Init
MKPointAnnotationRef MKPointAnnotationWithCoordinate( CLLocationCoordinate2D coordinate, CFStringRef title, CFStringRef subtitle );

// Location
CLLocationCoordinate2D MKPointAnnotationCoordinate( MKPointAnnotationRef ref );
void MKPointAnnotationSetCoordinate( MKPointAnnotationRef ref, CLLocationCoordinate2D coordinate );


#pragma mark - MKPolygon
///////////////////////////////////////////////////
//                   MKPolygon                   //
///////////////////////////////////////////////////
// Create
MKPolygonRef MKPolygonWithPoints( MKMapPoint *points, NSUInteger count );
MKPolygonRef MKPolygonWithPointsInteriorPolygons( MKMapPoint *points, NSUInteger count, CFArrayRef interiorPolygons );
MKPolygonRef MKPolygonWithCoordinates( CLLocationCoordinate2D *coords, NSUInteger count );
MKPolygonRef MKPolygonWithCoordinatesInteriorPolygons( CLLocationCoordinate2D *coords, NSUInteger count, CFArrayRef interiorPolygons );

// Interior polygons
CFArrayRef MKPolygonInteriorPolygons( MKPolygonRef ref );


#pragma mark - MKPolygonRenderer
///////////////////////////////////////////////////
//               MKPolygonRenderer               //
///////////////////////////////////////////////////
// Create
MKPolygonRendererRef MKPolygonRendererWithPolygon( MKPolygonRef polygon );

// Polygon
MKPolygonRef MKPolygonRendererPolygon( MKPolygonRendererRef ref );


#pragma mark - MKPolyline
///////////////////////////////////////////////////
//                  MKPolyline                   //
///////////////////////////////////////////////////
// Create
MKPolylineRef MKPolylineWithPoints( MKMapPoint *points, NSUInteger count );
MKPolylineRef MKPolylineWithCoordinates( CLLocationCoordinate2D *coords, NSUInteger count );


#pragma mark - MKPolylineRenderer
///////////////////////////////////////////////////
//             MKPolylineRenderer                //
///////////////////////////////////////////////////
// Create
MKPolylineRendererRef MKPolylineRendererWithPolyline( MKPolylineRef polyline );

// Accessing the polyline overlay
MKPolylineRef MKPolylineRendererPolyline( MKPolylineRendererRef ref );


#pragma mark - MKRoute
///////////////////////////////////////////////////
//                     MKRoute                   //
///////////////////////////////////////////////////
// Geometry
MKPolylineRef MKRoutePolyline( MKRouteRef ref );
CFArrayRef MKRouteSteps( MKRouteRef ref );

// Details
CFStringRef MKRouteName( MKRouteRef ref );
CFArrayRef MKRouteAdvisoryNotices( MKRouteRef ref );
CLLocationDistance MKRouteDistance( MKRouteRef ref );
CFTimeInterval MKRouteExpectedTravelTime( MKRouteRef ref );
MKDirectionsTransportType MKRouteTransportType( MKRouteRef ref );


#pragma mark - MKRouteStep
///////////////////////////////////////////////////
//                 MKRouteStep                   //
///////////////////////////////////////////////////
// Geometry
MKPolylineRef MKRouteStepPolyline( MKRouteStepRef ref );

// Details
CFStringRef MKRouteStepInstructions( MKRouteStepRef ref );
CFStringRef MKRouteStepNotice( MKRouteStepRef ref );
CLLocationDistance MKRouteStepDistance( MKRouteStepRef ref );
MKDirectionsTransportType MKRouteStepTransportType( MKRouteStepRef ref );


#pragma mark - MKShape
///////////////////////////////////////////////////
//                     MKShape                   //
///////////////////////////////////////////////////
// Attributes
CFStringRef MKShapeTitle( MKShapeRef ref );
void MKShapeSetTitle( MKShapeRef ref, CFStringRef title );
CFStringRef MKShapeSubtitle( MKShapeRef ref );
void MKShapeSetSubtitle( MKShapeRef ref, CFStringRef title );


#pragma mark - MKTileOverlay
///////////////////////////////////////////////////
//                  MKTileOverlay                //
///////////////////////////////////////////////////
// callback
typedef void (*MKTileOverlayCompletionHandlerType)(CFDataRef,NSError*,void*);

// Create
MKTileOverlayRef MKTileOverlayWithURLTemplate( CFStringRef urlTemplate );

// Attributes
CGSize MKTileOverlayTileSize( MKTileOverlayRef ref );
void MKTileOverlaySetTileSize( MKTileOverlayRef ref, CGSize size );
Boolean MKTileOverlayIsGeometryFlipped( MKTileOverlayRef ref );
void MKTileOverlaySetGeometryFlipped( MKTileOverlayRef ref, Boolean flag );
NSInteger MKTileOverlayMinimumZ( MKTileOverlayRef ref );
void MKTileOverlaySetMinimumZ( MKTileOverlayRef ref, NSInteger value );
NSInteger MKTileOverlayMaximumZ( MKTileOverlayRef ref );
void MKTileOverlaySetMaximumZ( MKTileOverlayRef ref, NSInteger value );
Boolean MKTileOverlayCanReplaceMapContent( MKTileOverlayRef ref );
void MKTileOverlaySetCanReplaceMapContent( MKTileOverlayRef ref, Boolean flag );

// Loading
CFStringRef MKTileOverlayURLTemplate( MKTileOverlayRef ref );
CFURLRef MKTileOverlayURLForTilePath( MKTileOverlayRef ref, MKTileOverlayPath path );
void MKTileOverlayLoadTileAtPath( MKTileOverlayRef ref, MKTileOverlayPath path, MKTileOverlayCompletionHandlerType fnAddress, void *userData );


#pragma mark - MKTileOverlayRenderer
///////////////////////////////////////////////////
//              MKTileOverlayRenderer            //
///////////////////////////////////////////////////
// Create
MKTileOverlayRendererRef MKTileOverlayRendererWithTileOverlay( MKTileOverlayRef tileOverlay );

// Reload
void MKTileOverlayRendererReloadData( MKTileOverlayRendererRef ref );


#pragma mark - MKUserLocation
///////////////////////////////////////////////////
//               MKUserLocation                  //
///////////////////////////////////////////////////
// Position
CLLocationRef MKUserLocationLocation( MKUserLocationRef ref );
Boolean MKUserLocationIsUpdating( MKUserLocationRef ref );
CLHeadingRef MKUserLocationHeading( MKUserLocationRef ref );

// Annotation text
CFStringRef MKUserLocationTitle( MKUserLocationRef ref );
void MKUserLocationSetTitle( MKUserLocationRef ref, CFStringRef title );
CFStringRef MKUserLocationSubtitle( MKUserLocationRef ref );
void MKUserLocationSetSubtitle( MKUserLocationRef ref, CFStringRef subtitle );

#endif// #ifdef MK_EXTERN
#endif// #if __LP64__
#endif// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
