/*
 MapKit.m
 
 Bernie Wylde 20190222
 */

#import "MapKit.h"

#if __LP64__
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
#ifdef MK_EXTERN

#pragma mark - MKAnnotation
///////////////////////////////////////////////////
//               MKAnnotation                    //
///////////////////////////////////////////////////
// Position
CLLocationCoordinate2D MKAnnotationCoordinate( MKAnnotationRef ref )
{ return [ref coordinate]; }

void MKAnnotationSetCoordinate( MKAnnotationRef ref, CLLocationCoordinate2D coordinate )
{ [ref setCoordinate:coordinate]; }

// Title
CFStringRef MKAnnotationTitle( MKAnnotationRef ref )
{ return (CFStringRef)[ref title]; }

CFStringRef MKAnnotationSubtitle( MKAnnotationRef ref )
{ return (CFStringRef)[ref subtitle]; }


#pragma mark - MKAnnotationView
///////////////////////////////////////////////////
//           MKAnnotationView                    //
///////////////////////////////////////////////////
// Create and prepare
MKAnnotationViewRef MKAnnotationViewWithAnnotation( MKAnnotationRef annotation, CFStringRef reuseIdentifier )
{ return [[[MKAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:(NSString *)reuseIdentifier] autorelease]; }

void MKAnnotationViewPrepareForReuse( MKAnnotationViewRef ref )
{ [ref prepareForReuse]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void MKAnnotationViewPrepareForDisplay( MKAnnotationViewRef ref )
{ [ref prepareForDisplay]; }
#endif

// Priority
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
MKFeatureDisplayPriority MKAnnotationViewDisplayPriority( MKAnnotationViewRef ref )
{ return [ref displayPriority]; }
#endif

// Attributes
Boolean MKAnnotationViewIsEnabled( MKAnnotationViewRef ref )
{ return [ref isEnabled]; }

void MKAnnotationViewSetEnabled( MKAnnotationViewRef ref, Boolean flag )
{ [ref setEnabled:flag]; }

ImageRef MKAnnotationViewImage( MKAnnotationViewRef ref )
{ return [ref image]; }

void MKAnnotationViewSetImage( MKAnnotationViewRef ref, ImageRef image )
{ [ref setImage:image]; }

Boolean MKAnnotationViewIsHighlighted( MKAnnotationViewRef ref )
{ return [ref isHighlighted]; }

void MKAnnotationViewSetHighlighted( MKAnnotationViewRef ref, Boolean flag )
{ [ref setHighlighted:flag]; }

MKAnnotationRef MKAnnotationViewAnnotation( MKAnnotationViewRef ref )
{ return [ref annotation]; }

void MKAnnotationViewSetAnnotation( MKAnnotationViewRef ref, MKAnnotationRef annotation )
{ [ref setAnnotation:annotation]; }

CGPoint MKAnnotationViewCenterOffset( MKAnnotationViewRef ref )
{ return [ref centerOffset]; }

void MKAnnotationViewSetCenterOffset( MKAnnotationViewRef ref, CGPoint offset )
{ [ref setCenterOffset:offset]; }

CGPoint MKAnnotationViewCalloutOffset( MKAnnotationViewRef ref )
{ return [ref calloutOffset]; }

void MKAnnotationViewSetCalloutOffset( MKAnnotationViewRef ref, CGPoint offset )
{ [ref setCalloutOffset:offset]; }

CFStringRef MKAnnotationViewReuseIdentifier( MKAnnotationViewRef ref )
{ return (CFStringRef)[ref reuseIdentifier]; }

// Selection state
void MKAnnotationViewSetSelected( MKAnnotationViewRef ref, Boolean selected, Boolean animated )
{ [ref setSelected:selected animated:animated]; }

Boolean MKAnnotationViewIsSelected( MKAnnotationViewRef ref )
{ return [ref isSelected]; }

// Callout views
Boolean MKAnnotationViewCanShowCallout( MKAnnotationViewRef ref )
{ return [ref canShowCallout]; }

void MKAnnotationViewSetCanShowCallout( MKAnnotationViewRef ref, Boolean flag )
{ [ref setCanShowCallout:flag]; }

ViewRef MKAnnotationViewLeftCalloutAccessoryView( MKAnnotationViewRef ref )
{ return [ref leftCalloutAccessoryView]; }

void MKAnnotationViewSetLeftCalloutAccessoryView( MKAnnotationViewRef ref, ViewRef vw )
{ [ref setLeftCalloutAccessoryView:vw]; }

ViewRef MKAnnotationViewRightCalloutAccessoryView( MKAnnotationViewRef ref )
{ return [ref rightCalloutAccessoryView]; }

void MKAnnotationViewSetRightCalloutAccessoryView( MKAnnotationViewRef ref, ViewRef vw )
{ [ref setRightCalloutAccessoryView:vw]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ViewRef MKAnnotationViewDetailCalloutAccessoryView( MKAnnotationViewRef ref )
{ return [ref detailCalloutAccessoryView]; }

void MKAnnotationViewSetDetailCalloutAccessoryView( MKAnnotationViewRef ref, ViewRef vw )
{ [ref setDetailCalloutAccessoryView:vw]; }
#endif

CGPoint MKAnnotationViewLeftCalloutOffset( MKAnnotationViewRef ref )
{ return [ref leftCalloutOffset]; }

void MKAnnotationViewSetLeftCalloutOffset( MKAnnotationViewRef ref, CGPoint offset )
{ [ref setLeftCalloutOffset:offset]; }

CGPoint MKAnnotationViewRightCalloutOffset( MKAnnotationViewRef ref )
{ return [ref rightCalloutOffset]; }

void MKAnnotationViewSetRightCalloutOffset( MKAnnotationViewRef ref, CGPoint offset )
{ [ref setRightCalloutOffset:offset]; }

// Drag operations
Boolean MKAnnotationViewIsDraggable( MKAnnotationViewRef ref )
{ return [ref isDraggable]; }

void MKAnnotationViewSetDraggable( MKAnnotationViewRef ref, Boolean flag )
{ [ref setDraggable:flag]; }

void MKAnnotationViewSetDragState( MKAnnotationViewRef ref, MKAnnotationViewDragState state, Boolean animated )
{ [ref setDragState:state animated:animated]; }

MKAnnotationViewDragState mkAnnotationViewDragState( MKAnnotationViewRef ref )
{ return [ref dragState]; }

// Collisions
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
MKAnnotationViewCollisionMode mkAnnotationViewCollisionMode( MKAnnotationViewRef ref )
{ return [ref collisionMode]; }

void MKAnnotationViewSetCollisionMode( MKAnnotationViewRef ref, MKAnnotationViewCollisionMode mode )
{ [ref setCollisionMode:mode]; }
#endif

// Clustering
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFStringRef MKAnnotationViewClusteringIdentifier( MKAnnotationViewRef ref )
{ return (CFStringRef)[ref clusteringIdentifier]; }

void MKAnnotationViewSetClusteringIdentifier( MKAnnotationViewRef ref, CFStringRef identifier )
{ [ref setClusteringIdentifier:(NSString *)identifier]; }

MKAnnotationViewRef MKAnnotationViewClusterAnnotationView( MKAnnotationViewRef ref )
{ return [ref clusterAnnotationView]; }
#endif


#pragma mark - MKCircle
///////////////////////////////////////////////////
//                   MKCircle                    //
///////////////////////////////////////////////////
// Create
MKCircleRef MKCircleWithCenterCoordinate( CLLocationCoordinate2D coord, CLLocationDistance radius )
{ return [MKCircle circleWithCenterCoordinate:coord radius:radius]; }

MKCircleRef MKCircleWithMapRect( MKMapRect r )
{ return [MKCircle circleWithMapRect:r]; }

// Attributes
CLLocationCoordinate2D MKCircleCoordinate( MKCircleRef ref )
{ return [ref coordinate]; }

CLLocationDistance MKCircleRadius( MKCircleRef ref )
{ return [ref radius]; }

MKMapRect MKCircleBoundingMapRect( MKCircleRef ref )
{ return [ref boundingMapRect]; }


#pragma mark - MKCircleRenderer
///////////////////////////////////////////////////
//              MKCircleRenderer                 //
///////////////////////////////////////////////////
// Create
MKCircleRendererRef MKCircleRendererWithCircle( MKCircleRef circle )
{ return [[[MKCircleRenderer alloc] initWithCircle:circle] autorelease]; }

// Overlay object
MKCircleRef MKCircleRendererCircle( MKCircleRendererRef ref )
{ return [ref circle]; }


#pragma mark - MKClusterAnnotation
///////////////////////////////////////////////////
//            MKClusterAnnotation                //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
// Create
MKClusterAnnotationRef MKClusterAnnotationWithMemberAnnotations( CFArrayRef annotations )
{ return [[[MKClusterAnnotation alloc] initWithMemberAnnotations:(NSArray *)annotations] autorelease]; }

// Attributes
CFStringRef MKClusterAnnotationTitle( MKClusterAnnotationRef ref )
{ return (CFStringRef)[ref title]; }

void MKClusterAnnotationSetTitle( MKClusterAnnotationRef ref, CFStringRef title )
{ [ref setTitle:(NSString *)title]; }

CFStringRef MKClusterAnnotationSubTitle( MKClusterAnnotationRef ref )
{ return (CFStringRef)[ref subtitle]; }

void MKClusterAnnotationSetSubtitle( MKClusterAnnotationRef ref, CFStringRef subtitle )
{ [ref setSubtitle:(NSString *)subtitle]; }

// Annotations
CFArrayRef MKClusterAnnotationMemberAnnotations( MKClusterAnnotationRef ref )
{ return (CFArrayRef)[ref memberAnnotations]; }
#endif


#pragma mark - MKDirections
///////////////////////////////////////////////////
//               MKDirections                    //
///////////////////////////////////////////////////
// Create
MKDirectionsRef MKDirectionsWithRequest( MKDirectionsRequestRef request )
{ return [[[MKDirections alloc] initWithRequest:request] autorelease]; }

// Directions
void MKDirectionsCalculateDirections( MKDirectionsRef ref, DirectionsCompletionHandlerType fnAddress, void *userData )
{
    [ref calculateDirectionsWithCompletionHandler:^(MKDirectionsResponse *response, NSError *error) {
        (*fnAddress)( response, error, userData );
    }];
}

// ETA
void MKDirectionsCalculateETA( MKDirectionsRef ref, ETACompletionHandlerType fnAddress, void *userData )
{
    [ref calculateETAWithCompletionHandler:^(MKETAResponse *response, NSError *error) {
        (*fnAddress)( response, error, userData );
    }];
}

// Request
void MKDirectionsCancel( MKDirectionsRef ref )
{ [ref cancel]; }

Boolean MKDirectionsIsCalculating( MKDirectionsRef ref )
{ return [ref isCalculating]; }


#pragma mark - MKDirectionsRequest
///////////////////////////////////////////////////
//           MKDirectionsRequest                 //
///////////////////////////////////////////////////
// Create
MKDirectionsRequestRef MKDirectionsRequestInit( void )
{ return [[[MKDirectionsRequest alloc] init] autorelease]; }

Boolean MKDirectionsRequestIsDirectionsRequestURL( CFURLRef url )
{ return [MKDirectionsRequest isDirectionsRequestURL:(NSURL *)url]; }

MKDirectionsRequestRef MKDirectionsRequestWithContentsOfURL( CFURLRef url )
{ return [[[MKDirectionsRequest alloc] initWithContentsOfURL:(NSURL *)url] autorelease]; }

// Start and end points
MKMapItemRef MKDirectionsRequestSource( MKDirectionsRequestRef ref )
{ return [ref source]; }

void MKDirectionsRequestSetSource( MKDirectionsRequestRef ref, MKMapItemRef source )
{ [ref setSource:source]; }

MKMapItemRef MKDirectionsRequestDestination( MKDirectionsRequestRef ref )
{ return [ref destination]; }

void MKDirectionsRequestSetDestination( MKDirectionsRequestRef ref, MKMapItemRef destination )
{ [ref setDestination:destination]; }

// Transportation options
MKDirectionsTransportType mkDirectionsRequestTransportType( MKDirectionsRequestRef ref )
{ return [ref transportType]; }

void MKDirectionsRequestSetTransportType( MKDirectionsRequestRef ref, MKDirectionsTransportType type )
{ [ref setTransportType:type]; }

Boolean MKDirectionsRequestRequestsAlternateRoutes( MKDirectionsRequestRef ref )
{ return [ref requestsAlternateRoutes]; }

void MKDirectionsRequestSetRequestsAlternateRoutes( MKDirectionsRequestRef ref, Boolean flag )
{ [ref setRequestsAlternateRoutes:flag]; }

CFDateRef MKDirectionsRequestDepartureDate( MKDirectionsRequestRef ref )
{ return (CFDateRef)[ref departureDate]; }

void MKDirectionsRequestSetDepartureDate( MKDirectionsRequestRef ref, CFDateRef dt )
{ [ref setDepartureDate:(NSDate *)dt]; }

CFDateRef MKDirectionsRequestArrivalDate( MKDirectionsRequestRef ref )
{ return (CFDateRef)[ref arrivalDate]; }

void MKDirectionsRequestSetArrivalDate( MKDirectionsRequestRef ref, CFDateRef dt )
{ [ref setArrivalDate:(NSDate *)dt]; }


#pragma mark - MKDirectionsResponse
///////////////////////////////////////////////////
//           MKDirectionsResponse                //
///////////////////////////////////////////////////
// End points
MKMapItemRef MKDirectionsResponseSource( MKDirectionsResponseRef ref )
{ return [ref source]; }

MKMapItemRef MKDirectionsResponseDestination( MKDirectionsResponseRef ref )
{ return [ref destination]; }

// Route info
CFArrayRef MKDirectionsResponseRoutes( MKDirectionsResponseRef ref )
{ return (CFArrayRef)[ref routes]; }


#pragma mark - MKDistanceFormatter
///////////////////////////////////////////////////
//           MKDistanceFormatter                 //
///////////////////////////////////////////////////
// Converting distances
CFStringRef MKDistanceFormatterStringFromDistance( MKDistanceFormatterRef ref, CLLocationDistance distance )
{ return (CFStringRef)[ref stringFromDistance:distance]; }

CLLocationDistance MKDistanceFormatterDistanceFromString( MKDistanceFormatterRef ref, CFStringRef string )
{ return [ref distanceFromString:(NSString *)string]; }

// Format
CFLocaleRef MKDistanceFormatterLocale( MKDistanceFormatterRef ref )
{ return (CFLocaleRef)[ref locale]; }

void MKDistanceFormatterSetLocale( MKDistanceFormatterRef ref, CFLocaleRef locale )
{ [ref setLocale:(NSLocale *)locale]; }

MKDistanceFormatterUnits mkDistanceFormatterUnits( MKDistanceFormatterRef ref )
{ return [ref units]; }

void MKDistanceFormatterSetUnits( MKDistanceFormatterRef ref, MKDistanceFormatterUnits units )
{ [ref setUnits:units]; }

MKDistanceFormatterUnitStyle mkDistanceFormatterUnitStyle( MKDistanceFormatterRef ref )
{ return [ref unitStyle]; }

void MKDistanceFormatterSetUnitStyle( MKDistanceFormatterRef ref, MKDistanceFormatterUnitStyle style )
{ [ref setUnitStyle:style]; }


#pragma mark - MKETAResponse
///////////////////////////////////////////////////
//                 MKETAResponse                 //
///////////////////////////////////////////////////
// End points
MKMapItemRef MKETAResponseSource( MKETAResponseRef ref )
{ return [ref source]; }

MKMapItemRef MKETAResponseDestination( MKETAResponseRef ref )
{ return [ref destination]; }

// Info
CFTimeInterval MKETAResponseExpectedTravelTime( MKETAResponseRef ref )
{ return (CFTimeInterval)[ref expectedTravelTime]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFDateRef MKETAResponseExpectedDepartureDate( MKETAResponseRef ref )
{ return (CFDateRef)[ref expectedDepartureDate]; }

CFDateRef MKETAResponseExpectedArrivalDate( MKETAResponseRef ref )
{ return (CFDateRef)[ref expectedArrivalDate]; }

CLLocationDistance MKETAResponseDistance( MKETAResponseRef ref )
{ return [ref distance]; }

MKDirectionsTransportType MKETAResponseTransportType( MKETAResponseRef ref )
{ return [ref transportType]; }
#endif


#pragma mark - MKGeodesicPolyline
///////////////////////////////////////////////////
//              MKGeodesicPolyline               //
///////////////////////////////////////////////////
// Create
MKGeodesicPolylineRef MKGeodesicPolylineWithPoints( MKMapPoint *points, NSUInteger count )
{ return [MKGeodesicPolyline polylineWithPoints:points count:count]; }

MKGeodesicPolylineRef MKGeodesicPolylineWithCoordinates( CLLocationCoordinate2D *coords, NSUInteger count )
{ return [MKGeodesicPolyline polylineWithCoordinates:coords count:count]; }


#pragma mark - MKLocalSearch
///////////////////////////////////////////////////
//                 MKLocalSearch                 //
///////////////////////////////////////////////////
// Create
MKLocalSearchRef MKLocalSearchWithRequest( MKLocalSearchRequestRef request )
{ return [[[MKLocalSearch alloc] initWithRequest:request] autorelease]; }

// Perform search
void MKLocalSearchStart( MKLocalSearchRef ref, LocalSearchCompletionHandlerType fnAddress, void *userData )
{
    [ref startWithCompletionHandler:^(MKLocalSearchResponse *response, NSError *error) {
        (*fnAddress)( response, error, userData );
    }];
}

Boolean MKLocalSearchIsSearching( MKLocalSearchRef ref )
{ return [ref isSearching]; }

void MKLocalSearchCancel( MKLocalSearchRef ref )
{ [ref cancel]; }


#pragma mark - MKLocalSearchCompleter
///////////////////////////////////////////////////
//           MKLocalSearchCompleter              //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101104
// Query attributew
CFStringRef MKLocalSearchCompleterQueryFragment( MKLocalSearchCompleterRef ref )
{ return (CFStringRef)[ref queryFragment]; }

void MKLocalSearchCompleterSetQueryFragment( MKLocalSearchCompleterRef ref, CFStringRef fragment )
{ [ref setQueryFragment:(NSString *)fragment]; }

MKCoordinateRegion MKLocalSearchCompleterRegion( MKLocalSearchCompleterRef ref )
{ return [ref region]; }

void MKLocalSearchCompleterSetRegion( MKLocalSearchCompleterRef ref, MKCoordinateRegion region )
{ [ref setRegion:region]; }

MKSearchCompletionFilterType MKLocalSearchCompleterFilterType( MKLocalSearchCompleterRef ref )
{ return [ref filterType]; }

void MKLocalSearchCompleterSetFilterType( MKLocalSearchCompleterRef ref, MKSearchCompletionFilterType type )
{ [ref setFilterType:type]; }

// Cancelling
void MKLocalSearchCompleterCancel( MKLocalSearchCompleterRef ref )
{ [ref cancel]; }

Boolean MKLocalSearchCompleterIsSearching( MKLocalSearchCompleterRef ref )
{ return [ref isSearching]; }

// Query results
CFArrayRef MKLocalSearchCompleterResults( MKLocalSearchCompleterRef ref )
{ return (CFArrayRef)[ref results]; }
#endif


#pragma mark - MKLocalSearchCompletion
///////////////////////////////////////////////////
//           MKLocalSearchCompletion             //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101104
// Get completions
CFStringRef MKLocalSearchCompletionTitle( MKLocalSearchCompletionRef ref )
{ return (CFStringRef)[ref title]; }

CFStringRef MKLocalSearchCompletionSubtitle( MKLocalSearchCompletionRef ref )
{ return (CFStringRef)[ref subtitle]; }

CFArrayRef MKLocalSearchCompletionTitleHighlightRanges( MKLocalSearchCompletionRef ref )
{ return (CFArrayRef)[ref titleHighlightRanges]; }

CFArrayRef MKLocalSearchCompletionSubtitleHighlightRanges( MKLocalSearchCompletionRef ref )
{ return (CFArrayRef)[ref subtitleHighlightRanges]; }
#endif


#pragma mark - MKLocalSearchRequest
///////////////////////////////////////////////////
//             MKLocalSearchRequest              //
///////////////////////////////////////////////////
// Init
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101104
MKLocalSearchRequestRef MKLocalSearchRequestWithCompletion( MKLocalSearchCompletionRef completion )
{ return [[[MKLocalSearchRequest alloc] initWithCompletion:completion] autorelease]; }
#endif

// Search parameters
CFStringRef MKLocalSearchRequestNaturalLanguageQuery( MKLocalSearchRequestRef ref )
{ return (CFStringRef)[ref naturalLanguageQuery]; }

void MKLocalSearchRequestSetNaturalLanguageQuery( MKLocalSearchRequestRef ref, CFStringRef string )
{ [ref setNaturalLanguageQuery:(NSString *)string]; }

MKCoordinateRegion MKLocalSearchRequestRegion( MKLocalSearchRequestRef ref )
{ return [ref region]; }

void MKLocalSearchRequestSetRegion( MKLocalSearchRequestRef ref, MKCoordinateRegion region )
{ [ref setRegion:region]; }


#pragma mark - MKLocalSearchResponse
///////////////////////////////////////////////////
//            MKLocalSearchResponse              //
///////////////////////////////////////////////////
// Search results
CFArrayRef MKLocalSearchResponseMapItems( MKLocalSearchResponseRef ref )
{ return (CFArrayRef)[ref mapItems]; }

MKCoordinateRegion MKLocalSearchResponseBoundingRegion( MKLocalSearchResponseRef ref )
{ return [ref boundingRegion]; }


#pragma mark - MKMapCamera
///////////////////////////////////////////////////
//                  MKMapCamera                  //
///////////////////////////////////////////////////
// Init
MKMapCameraRef MKMapCameraInit( void )
{ return [MKMapCamera camera]; }

MKMapCameraRef MKMapCameraFromEyeCoordinate( CLLocationCoordinate2D centerCoordinate, CLLocationCoordinate2D eyeCoordinate, CLLocationDistance eyeAltitude )
{ return [MKMapCamera cameraLookingAtCenterCoordinate:centerCoordinate fromEyeCoordinate:eyeCoordinate eyeAltitude:eyeAltitude]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
MKMapCameraRef MKMapCameraFromDistance( CLLocationCoordinate2D centerCoordinate, CLLocationDistance distance, CGFloat pitch, CLLocationDirection heading )
{ return [MKMapCamera cameraLookingAtCenterCoordinate:centerCoordinate fromDistance:distance pitch:pitch heading:heading]; }
#endif

// Viewing angle
CLLocationCoordinate2D MKMapCameraCenterCoordinate( MKMapCameraRef ref )
{ return [ref centerCoordinate]; }

void MKMapCameraSetCenterCoordinate( MKMapCameraRef ref, CLLocationCoordinate2D coordinate )
{ [ref setCenterCoordinate:coordinate]; }

CLLocationDirection MKMapCameraHeading( MKMapCameraRef ref )
{ return [ref heading]; }

void MKMapCameraSetHeading( MKMapCameraRef ref, CLLocationDirection heading )
{ [ref setHeading:heading]; }

CGFloat MKMapCameraPitch( MKMapCameraRef ref )
{ return [ref pitch]; }

void MKMapCameraSetPitch( MKMapCameraRef ref, CGFloat pitch )
{ [ref setPitch:pitch]; }

CLLocationDistance MKMapCameraAltitude( MKMapCameraRef ref )
{ return [ref altitude]; }

void MKMapCameraSetAltitude( MKMapCameraRef ref, CLLocationDistance altitude )
{ [ref setAltitude:altitude]; }


#pragma mark - MKMapItem
///////////////////////////////////////////////////
//                  MKMapItem                    //
///////////////////////////////////////////////////
// Create
MKMapItemRef MKMapItemForCurrentLocation( void )
{ return [MKMapItem mapItemForCurrentLocation]; }

MKMapItemRef MKMapItemWithPlacemark( MKPlacemarkRef placemark )
{ return [[[MKMapItem alloc] initWithPlacemark:placemark] autorelease]; }

// Attributes
MKPlacemarkRef MKMapItemPlacemark( MKMapItemRef ref )
{ return [ref placemark]; }

Boolean MKMapItemIsCurrentLocation( MKMapItemRef ref )
{ return [ref isCurrentLocation]; }

CFStringRef MKMapItemName( MKMapItemRef ref )
{ return (CFStringRef)[ref name]; }

void MKMapItemSetName( MKMapItemRef ref, CFStringRef name )
{ [ref setName:(NSString *)name]; }

CFStringRef MKMapItemPhoneNumber( MKMapItemRef ref )
{ return (CFStringRef)[ref phoneNumber]; }

void MKMapItemSetPhoneNumbewr( MKMapItemRef ref, CFStringRef number )
{ [ref setPhoneNumber:(NSString *)number]; }

CFURLRef MKMapItemURL( MKMapItemRef ref )
{ return (CFURLRef)[ref url]; }

void MKMapItemSetURL( MKMapItemRef ref, CFURLRef url )
{ [ref setUrl:(NSURL *)url]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFTimeZoneRef MKMapItemTimeZone( MKMapItemRef ref )
{ return (CFTimeZoneRef)[ref timeZone]; }

void MKMapItemSetTimeZone( MKMapItemRef ref, CFTimeZoneRef zone )
{ [ref setTimeZone:(NSTimeZone *)zone]; }
#endif

// Launch Maps app
Boolean MKMapItemOpenMapsWithItems( CFArrayRef items, CFDictionaryRef options )
{ return [MKMapItem openMapsWithItems:(NSArray *)items launchOptions:(NSDictionary *)options]; }

Boolean MKMapItemOpenInMaps( MKMapItemRef ref, CFDictionaryRef options )
{ return [ref openInMapsWithLaunchOptions:(NSDictionary *)options]; }


#pragma mark - MKMapSnapshot
///////////////////////////////////////////////////
//              MKMapSnapshot                    //
///////////////////////////////////////////////////
// Image
ImageRef MKMapSnapshotImage( MKMapSnapshotRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
AppearanceRef MKMapSnapshotAppearance( MKMapSnapshotRef ref );
#endif

// Points on image
CGPoint MKMapSnapshotPointForCoordinate( MKMapSnapshotRef ref, CLLocationCoordinate2D coordinate );


#pragma mark - MKMapSnapshotOptions
///////////////////////////////////////////////////
//           MKMapSnapshotOptions                //
///////////////////////////////////////////////////
// Configure map data
MKMapCameraRef MKMapSnapshotOptionsCamera( MKMapSnapshotOptionsRef ref )
{ return [ref camera]; }

void MKMapSnapshotOptionsSetCamera( MKMapSnapshotOptionsRef ref, MKMapCameraRef camera )
{ [ref setCamera:camera]; }

MKCoordinateRegion MKMapSnapshotOptionsRegion( MKMapSnapshotOptionsRef ref )
{ return [ref region]; }

void MKMapSnapshotOptionsSetRegion( MKMapSnapshotOptionsRef ref, MKCoordinateRegion region )
{ [ref setRegion:region]; }

MKMapRect MKMapSnapshotOptionsMapRect( MKMapSnapshotOptionsRef ref )
{ return [ref mapRect]; }

void MKMapSnapshotOptionsSetMapRect( MKMapSnapshotOptionsRef ref, MKMapRect rect )
{ [ref setMapRect:rect]; }

MKMapType MKMapSnapshotOptionsMapType( MKMapSnapshotOptionsRef ref )
{ return [ref mapType]; }

void MKMapSnapshotOptionsSetMapType( MKMapSnapshotOptionsRef ref, MKMapType type )
{ [ref setMapType:type]; }

Boolean MKMapSnapshotOptionsShowsPointsOfInterest( MKMapSnapshotOptionsRef ref )
{ return [ref showsPointsOfInterest]; }

void MKMapSnapshotOptionsSetShowsPointsOfInterest( MKMapSnapshotOptionsRef ref, Boolean flag )
{ [ref setShowsPointsOfInterest:flag]; }

Boolean MKMapSnapshotOptionsShowsBuildings( MKMapSnapshotOptionsRef ref )
{ return [ref showsBuildings]; }

void MKMapSnapshotOptionsSetShowsBuildings( MKMapSnapshotOptionsRef ref, Boolean flag )
{ [ref setShowsBuildings:flag]; }

// Image output
CGSize MKMapSnapshotOptionsSize( MKMapSnapshotOptionsRef ref )
{ return NSSizeToCGSize([ref size]); }

void MKMapSnapshotOptionsSetSize( MKMapSnapshotOptionsRef ref, CGSize size )
{ [ref setSize:NSSizeFromCGSize(size)]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
AppearanceRef MKMapSnapshotOptionsAppearance( MKMapSnapshotOptionsRef ref )
{ return [ref appearance]; }

void MKMapSnapshotOptionsSetAppearance( MKMapSnapshotOptionsRef ref, AppearanceRef appearnce )
{ [ref setAppearance:appearnce]; }
#endif


#pragma mark - MKMapSnapshotter
///////////////////////////////////////////////////
//               MKMapSnapshotter                //
///////////////////////////////////////////////////
// Create
MKMapSnapshotterRef MKMapSnapshotterWithOptions( MKMapSnapshotOptionsRef options )
{ return [[[MKMapSnapshotter alloc] initWithOptions:options] autorelease]; }

// Generate snapshot
void MKMapSnapshotterStart( MKMapSnapshotterRef ref, MapSnapshotterCompletionHandlerType fnAddress, void *userData )
{
    [ref startWithCompletionHandler:^(MKMapSnapshot *snapshot, NSError *error) {
        (*fnAddress)( snapshot, error, userData );
    }];
}

void MKMapSnapshotterCancel( MKMapSnapshotterRef ref )
{ [ref cancel]; }

Boolean MKMapSnapshotterIsLoading( MKMapSnapshotterRef ref )
{ return [ref isLoading]; }


#pragma mark - MKMapView
///////////////////////////////////////////////////
//                  MKMapView                    //
///////////////////////////////////////////////////
#pragma mark - subclass
@implementation MapViewSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end


@implementation MKMapView (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end


@implementation CocoaMKMapViewDelegate

- (void)mapView:(MKMapView *)mapView forwardEvent:(NSInteger)ev object:(id)obj {
    CallUserDefinedOnDialogFunction( ev, [mapView tag], [mapView wndTag], obj );
}


- (void)mapView:(MKMapView *)mapView regionWillChangeAnimated:(BOOL)animated {
    DialogEventSetBool(animated);
    [self mapView:mapView forwardEvent:mapViewRegionWillChange object:nil];
}

- (void)mapViewDidChangeVisibleRegion:(MKMapView *)mapView {
    [self mapView:mapView forwardEvent:mapViewDidChangeVisibleRegion object:nil];
}

- (void)mapView:(MKMapView *)mapView regionDidChangeAnimated:(BOOL)animated {
    DialogEventSetBool(animated);
    [self mapView:mapView forwardEvent:mapViewRegionDidChange object:nil];
}

- (void)mapViewWillStartLoadingMap:(MKMapView *)mapView {
    [self mapView:mapView forwardEvent:mapViewWillStartLoadingMap object:nil];
}

- (void)mapViewDidFinishLoadingMap:(MKMapView *)mapView {
    [self mapView:mapView forwardEvent:mapViewDidFinishLoadingMap object:nil];
}

- (void)mapViewDidFailLoadingMap:(MKMapView *)mapView withError:(NSError *)error {
    [self mapView:mapView forwardEvent:mapViewDidFailLoadingMap object:nil];
}

- (void)mapViewWillStartRenderingMap:(MKMapView *)mapView {
    [self mapView:mapView forwardEvent:mapViewWillStartRenderingMap object:nil];
}

- (void)mapViewDidFinishRenderingMap:(MKMapView *)mapView fullyRendered:(BOOL)fullyRendered {
    DialogEventSetBool(fullyRendered);
    [self mapView:mapView forwardEvent:mapViewDidFinishRenderingMap object:nil];
}

- (void)mapViewWillStartLocatingUser:(MKMapView *)mapView {
    [self mapView:mapView forwardEvent:mapViewWillStartLocatingUser object:nil];
}

- (void)mapViewDidStopLocatingUser:(MKMapView *)mapView {
    [self mapView:mapView forwardEvent:mapViewDidStopLocatingUser object:nil];
}

- (void)mapView:(MKMapView *)mapView didUpdateUserLocation:(MKUserLocation *)userLocation {
    [self mapView:mapView forwardEvent:mapViewDidUpdateUserLocation object:userLocation];
}

- (void)mapView:(MKMapView *)mapView didFailToLocateUserWithError:(NSError *)error {
    [self mapView:mapView forwardEvent:mapViewDidFailToLocateUser object:error];
}

- (MKAnnotationView *)mapView:(MKMapView *)mapView viewForAnnotation:(id<MKAnnotation>)annotation {
    DialogEventSetView( nil );
    [self mapView:mapView forwardEvent:mapViewViewForAnnotation object:annotation];
    return (MKAnnotationView *)DialogEventView();
}

//- (void)mapView:(MKMapView *)mapView didAddAnnotationViews:(NSArray<MKAnnotationView *> *)views {
//    [self mapView:mapView forwardEvent:mapViewDidAddAnnotationViews object:views];
//}

- (void)mapView:(MKMapView *)mapView didAddAnnotationViews:(NSArray *)views {
    [self mapView:mapView forwardEvent:mapViewDidAddAnnotationViews object:views];
}

//- (MKClusterAnnotation *)mapView:(MKMapView *)mapView clusterAnnotationForMemberAnnotations:(NSArray<id<MKAnnotation>> *)memberAnnotations {
//
//    [self mapView:mapView forwardEvent:mapViewDidAddAnnotationViews object:memberAnnotations];
//
//    return nil;
//}

- (void)mapView:(MKMapView *)mapView annotationView:(MKAnnotationView *)view didChangeDragState:(MKAnnotationViewDragState)newState fromOldState:(MKAnnotationViewDragState)oldState {
    DialogEventSetLong( oldState );
    [self mapView:mapView forwardEvent:mapViewAnnotationViewDidChangeDragState object:nil];
}

- (void)mapView:(MKMapView *)mapView didSelectAnnotationView:(MKAnnotationView *)view {
    [self mapView:mapView forwardEvent:mapViewDidSelectAnnotationView object:view];
}

- (void)mapView:(MKMapView *)mapView didDeselectAnnotationView:(MKAnnotationView *)view {
    [self mapView:mapView forwardEvent:mapViewDidDeselectAnnotationView object:view];
}

- (MKOverlayRenderer *)mapView:(MKMapView *)mapView rendererForOverlay:(id<MKOverlay>)overlay {
    DialogEventSetObject( nil );
    [self mapView:mapView forwardEvent:mapViewRendererForOverlay object:overlay];
    return DialogEventObject();
}

//- (void)mapView:(MKMapView *)mapView didAddOverlayRenderers:(NSArray<MKOverlayRenderer *> *)renderers {
//    [self mapView:mapView forwardEvent:mapViewDidAddOverlayRenderers object:renderers];
//}

- (void)mapView:(MKMapView *)mapView didAddOverlayRenderers:(NSArray *)renderers {
    [self mapView:mapView forwardEvent:mapViewDidAddOverlayRenderers object:renderers];
}
@end

CocoaMKMapViewDelegate *CocoaMKMapViewDelegateObj( void )
{
    static CocoaMKMapViewDelegate *sCocoaMKMapViewDelegate = nil;
    if ( !sCocoaMKMapViewDelegate ) sCocoaMKMapViewDelegate = [[CocoaMKMapViewDelegate alloc] init];
    return sCocoaMKMapViewDelegate;
}

MKMapView *MKMapViewWithTag( NSInteger tag )
{
    MKMapView *view = (MKMapView *)CocoaViewWithTag( tag );
    if ( [view isKindOfClass:[MKMapView class]] ) return view;
    CocoaShowAlert( NSWarningAlertStyle, @"Error.", [NSString stringWithFormat:@"Can't find mapview with tag: %ld.",(long)tag] );
    return nil;
}

BOOL MKMapViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [MKMapView class] ); }

// Properties
MKMapType MKMapViewMapType( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view mapType];
    return 0;
}

void MKMapViewSetMapType( NSInteger tag, MKMapType type )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setMapType:type];
}

Boolean MKMapViewIsZoomEnabled( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view isZoomEnabled];
    return NO;
}

void MKMapViewSetZoomEnabled( NSInteger tag, Boolean flag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setZoomEnabled:flag];
}

Boolean MKMapViewIsScrollEnabled( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view isScrollEnabled];
    return NO;
}

void MKMapViewSetScrollEnabled( NSInteger tag, Boolean flag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setScrollEnabled:flag];
}

Boolean MKMapViewIsPitchEnabled( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view isPitchEnabled];
    return NO;
}

void MKMapViewSetPitchEnabled( NSInteger tag, Boolean flag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setPitchEnabled:flag];
}

Boolean MKMapViewIsRotateEnabled( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view isRotateEnabled];
    return NO;
}

void MKMapViewSetRotateEnabled( NSInteger tag, Boolean flag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setRotateEnabled:flag];
}

// Visible portion
MKCoordinateRegion MKMapViewRegion( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view region];
    return MKCoordinateRegionMake( CLLocationCoordinate2DMake(0,0), MKCoordinateSpanMake(0,0) );
}

void MKMapViewSetRegion( NSInteger tag, MKCoordinateRegion region, Boolean animated )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setRegion:region animated:animated];
}

CLLocationCoordinate2D MKMapViewCenterCoordinate( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view centerCoordinate];
    return CLLocationCoordinate2DMake(0,0);
}

void MKMapViewSetCenterCoordinate( NSInteger tag, CLLocationCoordinate2D coordinate, Boolean animated )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setCenterCoordinate:coordinate animated:animated];
}

void MKMapViewShowAnnotations( NSInteger tag, CFArrayRef annotations, Boolean animated )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view showAnnotations:(NSArray *)annotations animated:animated];
}

MKMapRect MKMapViewVisibleMapRect( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view visibleMapRect];
    return MKMapRectNull;
}

void MKMapViewSetVisibleMapRect( NSInteger tag, MKMapRect rect, Boolean animated )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setVisibleMapRect:rect animated:animated];
}

void MKMapViewSetVisibleMapRectEdgePadding( NSInteger tag, MKMapRect rect, NSEdgeInsets insets, Boolean animated )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setVisibleMapRect:rect edgePadding:insets animated:animated];
}

// Appearance
MKMapCameraRef MKMapViewCamera( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view camera];
    return nil;
}

void MKMapViewSetCamera( NSInteger tag, MKMapCameraRef camera, Boolean animated )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setCamera:camera animated:animated];
}

Boolean MKMapViewShowsPointsOfInterest( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view showsPointsOfInterest];
    return NO;
}

void MKMapViewSetShowsPointsOfInterest( NSInteger tag, Boolean flag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setShowsPointsOfInterest:flag];
}

Boolean MKMapViewShowsBuildings( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view showsBuildings];
    return NO;
}

void MKMapViewSetShowsBuildings( NSInteger tag, Boolean flag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setShowsBuildings:flag];
}

Boolean MKMapViewShowsCompass( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view showsCompass];
    return NO;
}

void MKMapViewSetShowsCompass( NSInteger tag, Boolean flag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setShowsCompass:flag];
}

Boolean MKMapViewShowsZoomControls( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view showsZoomControls];
    return NO;
}

void MKMapViewSetShowsZoomControls( NSInteger tag, Boolean flag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setShowsZoomControls:flag];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean MKMapViewShowsScale( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view showsScale];
    return NO;
}

void MKMapViewSetShowsScale( NSInteger tag, Boolean flag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setShowsScale:flag];
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean MKMapViewShowsTraffic( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view showsTraffic];
    return NO;
}

void MKMapViewSetShowsTraffic( NSInteger tag, Boolean flag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setShowsTraffic:flag];
}
#endif

// User location
Boolean MKMapViewShowsUserLocation( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view showsUserLocation];
    return NO;
}

void MKMapViewSetShowsUserLocation( NSInteger tag, Boolean flag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view setShowsUserLocation:flag];
}

Boolean MKMapViewIsUserLocationVisible( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view isUserLocationVisible];
    return NO;
}

MKUserLocationRef MKMapViewUserLocation( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view userLocation];
    return nil;
}

// Annotating
CFArrayRef MKMapViewAnnotations( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return (CFArrayRef)[view annotations];
    return nil;
}

void MKMapViewAddAnnotation( NSInteger tag, MKAnnotationRef annotation )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view addAnnotation:annotation];
}

void MKMapViewAddAnnotations( NSInteger tag, CFArrayRef annotations )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view addAnnotations:(NSArray *)annotations];
}

void MKMapViewRemoveAnnotation( NSInteger tag, MKAnnotationRef annotation )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view removeAnnotation:annotation];
}

void MKMapViewRemoveAnnotations( NSInteger tag, CFArrayRef annotations )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view removeAnnotations:(NSArray *)annotations];
}

CFSetRef MKMapViewAnnotationsInMapRect( NSInteger tag, MKMapRect rect )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return (CFSetRef)[view annotationsInMapRect:rect];
    return nil;
}

// Annotation views
// ...
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
MKAnnotationViewRef MKMapViewDequeueReusableAnnotationViewForAnnotation( NSInteger tag, CFStringRef identifier, MKAnnotationRef annotation )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view dequeueReusableAnnotationViewWithIdentifier:(NSString *)identifier forAnnotation:annotation];
    return nil;
}
#endif

MKAnnotationViewRef MKMapViewDequeueReusableAnnotationView( NSInteger tag, CFStringRef identifier )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view dequeueReusableAnnotationViewWithIdentifier:(NSString *)identifier];
    return nil;
}

MKAnnotationViewRef MKMapViewViewForAnnotation( NSInteger tag, MKAnnotationRef annotation )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view viewForAnnotation:annotation];
    return nil;
}

// Annotation selections
CGRect MKMapViewAnnotationVisibleRect( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view annotationVisibleRect];
    return CGRectZero;
}

CFArrayRef MKMapViewSelectedAnnotations( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return (CFArrayRef)[view selectedAnnotations];
    return nil;
}

void MKMapViewSelectAnnotation( NSInteger tag, MKAnnotationRef annotation, Boolean animated )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view selectAnnotation:annotation animated:animated];
}

void MKMapViewDeselectAnnotation( NSInteger tag, MKAnnotationRef annotation, Boolean animated )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view deselectAnnotation:annotation animated:animated];
}

// Annotation views

// Accessing overlays
CFArrayRef MKMapViewOverlays( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return (CFArrayRef)[view overlays];
    return nil;
}

CFArrayRef MKMapViewOverlaysInLevel( NSInteger tag, MKOverlayLevel level )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return (CFArrayRef)[view overlaysInLevel:level];
    return nil;
}

MKOverlayRendererRef MKMapViewRendererForOverlay( NSInteger tag, MKOverlayRef overlay )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view rendererForOverlay:overlay];
    return nil;
}

// Add overlays
void MKMapViewAddOverlayAtLevel( NSInteger tag, MKOverlayRef overlay, MKOverlayLevel level )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view addOverlay:overlay level:level];
}

void MKMapViewAddOverlaysAtLevel( NSInteger tag, CFArrayRef overlays, MKOverlayLevel level )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view addOverlays:(NSArray *)overlays level:level];
}

void MKMapViewAddOverlay( NSInteger tag, MKOverlayRef overlay )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view addOverlay:overlay];
}

void MKMapViewAddOverlays( NSInteger tag, CFArrayRef overlays )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view addOverlays:(NSArray *)overlays];
}

void MKMapViewInsertOverlayAtLevel( NSInteger tag, MKOverlayRef overlay, NSUInteger index, MKOverlayLevel level )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view insertOverlay:overlay atIndex:index level:level];
}

void MKMapViewInsertOverlay( NSInteger tag, MKOverlayRef overlay, NSUInteger index )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view insertOverlay:overlay atIndex:index];
}

void MKMapViewInsertOverlayAboveOverlay( NSInteger tag, MKOverlayRef overlay1, MKOverlayRef overlay2 )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view insertOverlay:overlay1 aboveOverlay:overlay2];
}

void MKMapViewInsertOverlayBelowOverlay( NSInteger tag, MKOverlayRef overlay1, MKOverlayRef overlay2 )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view insertOverlay:overlay1 belowOverlay:overlay2];
}

void MKMapViewExchangeOverlay( NSInteger tag, MKOverlayRef overlay1, MKOverlayRef overlay2 )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view exchangeOverlay:overlay1 withOverlay:overlay2];
}

void MKMapViewExchangeOverlayAtIndex( NSInteger tag, NSUInteger index1, NSUInteger index2 )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view exchangeOverlayAtIndex:index1 withOverlayAtIndex:index2];
}

// Remove overlays
void MKMapViewRemoveOverlay( NSInteger tag, MKOverlayRef overlay )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view removeOverlay:overlay];
}

void MKMapViewRemoveOverlays( NSInteger tag, CFArrayRef overlays )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) [view removeOverlays:(NSArray *)overlays];
}

// Convert map coordinates

// Regions and rectangles
MKCoordinateRegion MKMapViewRegionThatFits( NSInteger tag, MKCoordinateRegion region )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view regionThatFits:region];
    return MKCoordinateRegionMake(CLLocationCoordinate2DMake(0,0),MKCoordinateSpanMake(0,0));
}

MKMapRect MKMapViewMapRectThatFits( NSInteger tag, MKMapRect rect )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view mapRectThatFits:rect];
    return MKMapRectNull;
}

MKMapRect MKMapViewMapRectThatFitsEdgePadding( NSInteger tag, MKMapRect rect, NSEdgeInsets insets )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) return [view mapRectThatFits:rect edgePadding:insets];
    return MKMapRectNull;
}

// Custom
void MKMapViewInstallDelegate( NSInteger tag )
{
    MKMapView *view = MKMapViewWithTag( tag );
    if ( view ) {
        [view setDelegate:CocoaMKMapViewDelegateObj()];
    }
}


// Init
MKMapViewRef MKMapViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 128, 128 );
    
    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    MKMapView *view;
    if ( subclass ) {
        view = [[MapViewSC alloc] initWithFrame:rect];
    } else {
        view = [[MKMapView alloc] initWithFrame:rect];
    }

    // tag
    [view setTag:inTag];
        
    return view;
}


// mapview statement
void CocoaMapView( NSInteger inTag, CGRect inRect, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        MKMapView *view = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !view ) {
            view = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( view ) {    // already exists
            
            // make sure it's same view class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [view class], [MKMapView class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, view, [MKMapView class] ) ) {
                return;
            }

            // rect
            if ( !NSIsEmptyRect( rect ) ) [view setFrame:rect];
            
        } else {            // create
            // rect
//            if ( NSIsEmptyRect( rect ) ) {
//                rect = NSMakeRect( 20, 20, 128, 128 );
//            }
            
            // init
//            if ( inSubclass ) {
//                view = [[MapViewSC alloc] initWithFrame:rect];
//            } else {
//                view = [[MKMapView alloc] initWithFrame:rect];
//            }
            view = MKMapViewInit( inTag, inRect );
            
            // default autoresizing
            [(NSView *)view setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [view setTag:inTag];
            
            // add to superview
            [contentView addSubview:view];
            
            // wndTag
            [view setWndTag:inWndTag];
            
//            [view release];
        }
        [view setHidden:!show];
    } else {
        CocoaShowAlert( NSWarningAlertStyle, @"Window error.", [NSString stringWithFormat:@"Cocoa window %ld does not exist.",(long)inWndTag] );
    }
}

#pragma mark - MKMultiPoint
///////////////////////////////////////////////////
//                MKMultiPoint                   //
///////////////////////////////////////////////////
// Points
MKMapPoint *MKMultiPointPoints( MKMultiPointRef ref )
{ return [ref points]; }

NSUInteger MKMultiPointCount( MKMultiPointRef ref )
{ return [ref pointCount]; }

// Coordinate values
void MKMultiPointGetCoordinates( MKMultiPointRef ref, CLLocationCoordinate2D *coords, CFRange range )
{ [ref getCoordinates:coords range:NSRangeFromCFRange(range)]; }


#pragma mark - MKOverlay
///////////////////////////////////////////////////
//                  MKOverlay                    //
///////////////////////////////////////////////////
// Geometry
CLLocationCoordinate2D MKOverlayCoordinate( MKOverlayRef ref )
{ return [ref coordinate]; }

MKMapRect MKOverlayBoundingMapRect( MKOverlayRef ref )
{ return [ref boundingMapRect]; }

// Intersections
Boolean MKOverlayIntersectsMapRect( MKOverlayRef ref, MKMapRect rect )
{ return [ref intersectsMapRect:rect]; }

// Rendering
Boolean MKOverlayCanReplaceMapContent( MKOverlayRef ref )
{ return [ref canReplaceMapContent]; }


#pragma mark - MKOverlayPathRenderer
///////////////////////////////////////////////////
//         MKOverlayPathRenderer                 //
///////////////////////////////////////////////////

// Path
CGPathRef MKOverlayPathRendererPath( MKOverlayPathRendererRef ref )
{ return [ref path]; }

void MKOverlayPathRendererSetPath( MKOverlayPathRendererRef ref, CGPathRef path )
{ [ref setPath:path]; }

void MKOverlayPathRendererCreatePath( MKOverlayPathRendererRef ref )
{ [ref createPath]; }

void MKOverlayPathRendererInvalidatePath( MKOverlayPathRendererRef ref )
{ [ref invalidatePath]; }

// Drawing attributes
ColorRef MKOverlayPathRendererFillColor( MKOverlayPathRendererRef ref )
{ return [ref fillColor]; }

void MKOverlayPathRendererSetFillColor( MKOverlayPathRendererRef ref, ColorRef col )
{ [ref setFillColor:col]; }

ColorRef MKOverlayPathRendererStrokeColor( MKOverlayPathRendererRef ref )
{ return [ref strokeColor]; }

void MKOverlayPathRendererSetStrokeColor( MKOverlayPathRendererRef ref, ColorRef col )
{ [ref setStrokeColor:col]; }

CGFloat MKOverlayPathRendererLineWidth( MKOverlayPathRendererRef ref )
{ return [ref lineWidth]; }

void MKOverlayPathRendererSetLineWidth( MKOverlayPathRendererRef ref, CGFloat lineWidth )
{ [ref setLineWidth:lineWidth]; }

CGLineJoin MKOverlayPathRendererLineJoin( MKOverlayPathRendererRef ref )
{ return [ref lineJoin]; }

void MKOverlayPathRendererSetLineJoin( MKOverlayPathRendererRef ref, CGLineJoin lineJoin )
{ [ref setLineJoin:lineJoin]; }

CGLineCap MKOverlayPathRendererLineCap( MKOverlayPathRendererRef ref )
{ return [ref lineCap]; }

void MKOverlayPathRendererSetLineCap( MKOverlayPathRendererRef ref, CGLineCap lineCap )
{ [ref setLineCap:lineCap]; }

CGFloat MKOverlayPathRendererMiterLimit( MKOverlayPathRendererRef ref )
{ return [ref miterLimit]; }

void MKOverlayPathRendererSetMiterLimit( MKOverlayPathRendererRef ref, CGFloat limit )
{ [ref setMiterLimit:limit]; }

CGFloat MKOverlayPathRendererLineDashPhase( MKOverlayPathRendererRef ref )
{ return [ref lineDashPhase]; }

void MKOverlayPathRendererSetLineDashPhase( MKOverlayPathRendererRef ref, CGFloat phase )
{ [ref setLineDashPhase:phase]; }

CFArrayRef MKOverlayPathRendererLineDashPattern( MKOverlayPathRendererRef ref )
{ return (CFArrayRef)[ref lineDashPattern]; }

void MKOverlayPathRendererSetLineDashPattern( MKOverlayPathRendererRef ref, CFArrayRef pattern )
{ [ref setLineDashPattern:(NSArray *)pattern]; }

// Drawing the path
void MKOverlayPathRendererApplyStrokeProperties( MKOverlayPathRendererRef ref, CGContextRef ctx, MKZoomScale zoomScale )
{ [ref applyStrokePropertiesToContext:ctx atZoomScale:zoomScale]; }

void MKOverlayPathRendererApplyFillProperties( MKOverlayPathRendererRef ref, CGContextRef ctx, MKZoomScale zoomScale )
{ [ref applyFillPropertiesToContext:ctx atZoomScale:zoomScale]; }

void MKOverlayPathRendererStrokePath( MKOverlayPathRendererRef ref, CGPathRef path, CGContextRef ctx )
{ [ref strokePath:path inContext:ctx]; }

void MKOverlayPathRendererFillPath( MKOverlayPathRendererRef ref, CGPathRef path, CGContextRef ctx )
{ [ref fillPath:path inContext:ctx]; }


#pragma mark - MKOverlayRenderer
///////////////////////////////////////////////////
//              MKOverlayRenderer                //
///////////////////////////////////////////////////
// Create
MKOverlayRendererRef MKOverlayRendererWithOverlay( MKOverlayRef overlay )
{ return [[[MKOverlayRenderer alloc] initWithOverlay:overlay] autorelease]; }

// Attributes
MKOverlayRef MKOverlayRendererOverlay( MKOverlayRendererRef ref )
{ return [ref overlay]; }

CGFloat MKOverlayRendererAlpha( MKOverlayRendererRef ref )
{ return [ref alpha]; }

void MKOverlayRendererSetAlpha( MKOverlayRendererRef ref, CGFloat alpha )
{ [ref setAlpha:alpha]; }

CGFloat MKOverlayRendererContentScaleFactor( MKOverlayRendererRef ref )
{ return [ref contentScaleFactor]; }

// Converting points
CGPoint MKOverlayRendererPointForMapPoint( MKOverlayRendererRef ref, MKMapPoint pt )
{ return [ref pointForMapPoint:pt]; }

MKMapPoint MKOverlayRendererMapPointForPoint( MKOverlayRendererRef ref, CGPoint pt )
{ return [ref mapPointForPoint:pt]; }

CGRect MKOverlayRendererRectForMapRect( MKOverlayRendererRef ref, MKMapRect rect )
{ return [ref rectForMapRect:rect]; }

MKMapRect MKOverlayRendererMapRectForRect( MKOverlayRendererRef ref, CGRect rect )
{ return [ref mapRectForRect:rect]; }

// Drawing
Boolean MKOverlayRendererCanDrawMapRect( MKOverlayRendererRef ref, MKMapRect rect, MKZoomScale zoomScale )
{ return [ref canDrawMapRect:rect zoomScale:zoomScale]; }

void MKOverlayRendererDrawMapRect( MKOverlayRendererRef ref, MKMapRect rect, MKZoomScale zoomScale, CGContextRef ctx )
{ [ref drawMapRect:rect zoomScale:zoomScale inContext:ctx]; }

void MKOverlayRendererSetNeedsDisplay( MKOverlayRendererRef ref )
{ [ref setNeedsDisplay]; }

void MKOverlayRendererSetNeedsDisplayInMapRect( MKOverlayRendererRef ref, MKMapRect rect )
{ [ref setNeedsDisplayInMapRect:rect]; }

void MKOverlayRendererSetNeedsDisplayZoomScale( MKOverlayRendererRef ref, MKMapRect rect, MKZoomScale zoomScale )
{ [ref setNeedsDisplayInMapRect:rect zoomScale:zoomScale]; }


#pragma mark - MKPinAnnotationView
///////////////////////////////////////////////////
//             MKPinAnnotationView               //
///////////////////////////////////////////////////
// Standard pin colors
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ColorRef MKPinAnnotationViewRedPinColor( void )
{ return [MKPinAnnotationView redPinColor]; }

ColorRef MKPinAnnotationViewGreenPinColor( void )
{ return [MKPinAnnotationView greenPinColor]; }

ColorRef MKPinAnnotationViewPurplePinColor( void )
{ return [MKPinAnnotationView purplePinColor]; }
#endif

// Attributes
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
ColorRef MKPinAnnotationViewPinTintColor( MKPinAnnotationViewRef ref )
{ return [ref pinTintColor]; }

void MKPinAnnotationViewSetPinTintColor( MKPinAnnotationViewRef ref, ColorRef col )
{ [ref setPinTintColor:col]; }
#endif

Boolean MKPinAnnotationViewAnimatesDrop( MKPinAnnotationViewRef ref )
{ return [ref animatesDrop]; }

void MKPinAnnotationViewSetAnimatesDrop( MKPinAnnotationViewRef ref, Boolean flag )
{ [ref setAnimatesDrop:flag]; }


#pragma mark - MKPlacemark
///////////////////////////////////////////////////
//                  MKPlacemark                  //
///////////////////////////////////////////////////
// Create
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200

MKPlacemarkRef MKPlacemarkWithCoordinate( CLLocationCoordinate2D coordinate )
{ return [[[MKPlacemark alloc] initWithCoordinate:coordinate] autorelease]; }

#ifdef CNPostalAddressRef
MKPlacemarkRef MKPlacemarkWithCoordinatePostalAddress( CLLocationCoordinate2D coordinate, CNPostalAddressRef postalAddress )
{ return [[[MKPlacemark alloc] initWithCoordinate:coordinate postalAddress:postalAddress] autorelease]; }
#endif// CNPostalAddressRef

#endif// __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200

MKPlacemarkRef MKPlacemarkWithCoordinateAddressDictionary( CLLocationCoordinate2D coordinate, CFDictionaryRef addressDict )
{ return [[[MKPlacemark alloc] initWithCoordinate:coordinate addressDictionary:(NSDictionary *)addressDict] autorelease]; }

// Attributes
CFStringRef MKPlacemarkCountryCode( MKPlacemarkRef ref )
{ return (CFStringRef)[ref countryCode]; }


#pragma mark - MKPointAnnotation
///////////////////////////////////////////////////
//             MKPointAnnotation                 //
///////////////////////////////////////////////////
// Init
MKPointAnnotationRef MKPointAnnotationWithCoordinate( CLLocationCoordinate2D coordinate, CFStringRef title, CFStringRef subtitle )
{
    MKPointAnnotation *annotation = [[[MKPointAnnotation alloc] init] autorelease];
    [annotation setCoordinate:coordinate];
    if ( title ) [annotation setTitle:(NSString *)title];
    if ( subtitle ) [annotation setSubtitle:(NSString *)subtitle];
    return annotation;
}

// Location
CLLocationCoordinate2D MKPointAnnotationCoordinate( MKPointAnnotationRef ref )
{ return [ref coordinate]; }

void MKPointAnnotationSetCoordinate( MKPointAnnotationRef ref, CLLocationCoordinate2D coordinate )
{ [ref setCoordinate:coordinate]; }


#pragma mark - MKPolygon
///////////////////////////////////////////////////
//                   MKPolygon                   //
///////////////////////////////////////////////////
// Create
MKPolygonRef MKPolygonWithPoints( MKMapPoint *points, NSUInteger count )
{ return [MKPolygon polygonWithPoints:points count:count]; }

MKPolygonRef MKPolygonWithPointsInteriorPolygons( MKMapPoint *points, NSUInteger count, CFArrayRef interiorPolygons )
{ return [MKPolygon polygonWithPoints:points count:count interiorPolygons:(NSArray *)interiorPolygons]; }

MKPolygonRef MKPolygonWithCoordinates( CLLocationCoordinate2D *coords, NSUInteger count )
{ return [MKPolygon polygonWithCoordinates:coords count:count]; }

MKPolygonRef MKPolygonWithCoordinatesInteriorPolygons( CLLocationCoordinate2D *coords, NSUInteger count, CFArrayRef interiorPolygons )
{ return [MKPolygon polygonWithCoordinates:coords count:count interiorPolygons:(NSArray *)interiorPolygons]; }

// Interior polygons
CFArrayRef MKPolygonInteriorPolygons( MKPolygonRef ref )
{ return (CFArrayRef)[ref interiorPolygons]; } 


#pragma mark - MKPolygonRenderer
///////////////////////////////////////////////////
//               MKPolygonRenderer               //
///////////////////////////////////////////////////
// Create
MKPolygonRendererRef MKPolygonRendererWithPolygon( MKPolygonRef polygon )
{ return [[[MKPolygonRenderer alloc] initWithPolygon:polygon] autorelease]; }

// Polygon
MKPolygonRef MKPolygonRendererPolygon( MKPolygonRendererRef ref )
{ return [ref polygon]; }


#pragma mark - MKPolyline
///////////////////////////////////////////////////
//                  MKPolyline                   //
///////////////////////////////////////////////////
// Create
MKPolylineRef MKPolylineWithPoints( MKMapPoint *points, NSUInteger count )
{ return [MKPolyline polylineWithPoints:points count:count]; }

MKPolylineRef MKPolylineWithCoordinates( CLLocationCoordinate2D *coords, NSUInteger count )
{ return [MKPolyline polylineWithCoordinates:coords count:count]; }


#pragma mark - MKPolylineRenderer
///////////////////////////////////////////////////
//             MKPolylineRenderer                //
///////////////////////////////////////////////////
// Create
MKPolylineRendererRef MKPolylineRendererWithPolyline( MKPolylineRef polyline )
{ return [[[MKPolylineRenderer alloc] initWithPolyline:polyline] autorelease]; }

// Accessing the polyline overlay
MKPolylineRef MKPolylineRendererPolyline( MKPolylineRendererRef ref )
{ return [ref polyline]; }


#pragma mark - MKRoute
///////////////////////////////////////////////////
//                     MKRoute                   //
///////////////////////////////////////////////////
// Geometry
MKPolylineRef MKRoutePolyline( MKRouteRef ref )
{ return [ref polyline]; }

CFArrayRef MKRouteSteps( MKRouteRef ref )
{ return (CFArrayRef)[ref steps]; }

// Details
CFStringRef MKRouteName( MKRouteRef ref )
{ return (CFStringRef)[ref name]; }

CFArrayRef MKRouteAdvisoryNotices( MKRouteRef ref )
{ return (CFArrayRef)[ref advisoryNotices]; }

CLLocationDistance MKRouteDistance( MKRouteRef ref )
{ return [ref distance]; }

CFTimeInterval MKRouteExpectedTravelTime( MKRouteRef ref )
{ return (CFTimeInterval)[ref expectedTravelTime]; }

MKDirectionsTransportType MKRouteTransportType( MKRouteRef ref )
{ return [ref transportType]; }


#pragma mark - MKRouteStep
///////////////////////////////////////////////////
//                 MKRouteStep                   //
///////////////////////////////////////////////////
// Geometry
MKPolylineRef MKRouteStepPolyline( MKRouteStepRef ref )
{ return [ref polyline]; }

// Details
CFStringRef MKRouteStepInstructions( MKRouteStepRef ref )
{ return (CFStringRef)[ref instructions]; }

CFStringRef MKRouteStepNotice( MKRouteStepRef ref )
{ return (CFStringRef)[ref notice]; }

CLLocationDistance MKRouteStepDistance( MKRouteStepRef ref )
{ return [ref distance]; }

MKDirectionsTransportType MKRouteStepTransportType( MKRouteStepRef ref )
{ return [ref transportType]; }


#pragma mark - MKShape
///////////////////////////////////////////////////
//                     MKShape                   //
///////////////////////////////////////////////////
// Attributes
CFStringRef MKShapeTitle( MKShapeRef ref )
{ return (CFStringRef)[ref title]; }

void MKShapeSetTitle( MKShapeRef ref, CFStringRef title )
{ [ref setTitle:(NSString *)title]; }

CFStringRef MKShapeSubtitle( MKShapeRef ref )
{ return (CFStringRef)[ref subtitle]; }

void MKShapeSetSubtitle( MKShapeRef ref, CFStringRef title )
{ [ref setSubtitle:(NSString *)title]; }


#pragma mark - MKTileOverlay
///////////////////////////////////////////////////
//                  MKTileOverlay                //
///////////////////////////////////////////////////
// Create
MKTileOverlayRef MKTileOverlayWithURLTemplate( CFStringRef urlTemplate )
{ return [[[MKTileOverlay alloc] initWithURLTemplate:(NSString *)urlTemplate] autorelease]; }

// Attributes
CGSize MKTileOverlayTileSize( MKTileOverlayRef ref )
{ return [ref tileSize]; }

void MKTileOverlaySetTileSize( MKTileOverlayRef ref, CGSize size )
{ [ref setTileSize:size]; }

Boolean MKTileOverlayIsGeometryFlipped( MKTileOverlayRef ref )
{ return [ref isGeometryFlipped]; }

void MKTileOverlaySetGeometryFlipped( MKTileOverlayRef ref, Boolean flag )
{ [ref setGeometryFlipped:flag]; }

NSInteger MKTileOverlayMinimumZ( MKTileOverlayRef ref )
{ return [ref minimumZ]; }

void MKTileOverlaySetMinimumZ( MKTileOverlayRef ref, NSInteger value )
{ [ref setMinimumZ:value]; }

NSInteger MKTileOverlayMaximumZ( MKTileOverlayRef ref )
{ return [ref maximumZ]; }

void MKTileOverlaySetMaximumZ( MKTileOverlayRef ref, NSInteger value )
{ [ref setMaximumZ:value]; }

Boolean MKTileOverlayCanReplaceMapContent( MKTileOverlayRef ref )
{ return [ref canReplaceMapContent]; }

void MKTileOverlaySetCanReplaceMapContent( MKTileOverlayRef ref, Boolean flag )
{ [ref setCanReplaceMapContent:flag]; }


// Loading
CFStringRef MKTileOverlayURLTemplate( MKTileOverlayRef ref )
{ return (CFStringRef)[ref URLTemplate]; }

CFURLRef MKTileOverlayURLForTilePath( MKTileOverlayRef ref, MKTileOverlayPath path )
{ return (CFURLRef)[ref URLForTilePath:path]; }

void MKTileOverlayLoadTileAtPath( MKTileOverlayRef ref, MKTileOverlayPath path, MKTileOverlayCompletionHandlerType fnAddress, void *userData )
{
    [ref loadTileAtPath:path result:^(NSData *tileData, NSError *error) {
        (*fnAddress)( (CFDataRef)tileData, error, userData );
    }];
}


#pragma mark - MKTileOverlayRenderer
///////////////////////////////////////////////////
//              MKTileOverlayRenderer            //
///////////////////////////////////////////////////
// Create
MKTileOverlayRendererRef MKTileOverlayRendererWithTileOverlay( MKTileOverlayRef tileOverlay )
{ return [[[MKTileOverlayRenderer alloc] initWithTileOverlay:tileOverlay] autorelease]; }

// Reload
void MKTileOverlayRendererReloadData( MKTileOverlayRendererRef ref )
{ [ref reloadData]; }


#pragma mark - MKUserLocation
///////////////////////////////////////////////////
//               MKUserLocation                  //
///////////////////////////////////////////////////
// Position
CLLocationRef MKUserLocationLocation( MKUserLocationRef ref )
{ return [ref location]; }

Boolean MKUserLocationIsUpdating( MKUserLocationRef ref )
{ return [ref isUpdating]; }

CLHeadingRef MKUserLocationHeading( MKUserLocationRef ref )
{ return [ref heading]; }

// Annotation text
CFStringRef MKUserLocationTitle( MKUserLocationRef ref )
{ return (CFStringRef)[ref title]; }

void MKUserLocationSetTitle( MKUserLocationRef ref, CFStringRef title )
{ [ref setTitle:(NSString *)title]; }

CFStringRef MKUserLocationSubtitle( MKUserLocationRef ref )
{ return (CFStringRef)[ref subtitle]; }

void MKUserLocationSetSubtitle( MKUserLocationRef ref, CFStringRef subtitle )
{ [ref setSubtitle:(NSString *)subtitle]; }


#endif// #ifdef MK_EXTERN
#endif// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#endif// #if __LP64__

