/*
 Metal.h
 
 Bernie Wylde 20180606
 */

#import <AppKit/AppKit.h>
#import <MetalKit/MetalKit.h>
#import <Metal/Metal.h>
#import <AVFoundation/AVFoundation.h>

#import "CocoaUI.h"

#if __LP64__
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
#ifdef MTL_EXTERN

// Metal
typedef id<CAMetalDrawable> CAMetalDrawableRef;
typedef id<MTLCommandBuffer> MTLCommandBufferRef;
typedef id<MTLCommandEncoder> MTLCommandEncoderRef;
typedef id<MTLCommandQueue> MTLCommandQueueRef;
typedef id<MTLDevice> MTLDeviceRef;
typedef id<MTLDrawable> MTLDrawableRef;
typedef id<MTLFunction> MTLFunctionRef;
typedef id<MTLLibrary> MTLLibraryRef;
typedef id<MTLRenderCommandEncoder> MTLRenderCommandEncoderRef;
typedef MTLRenderPassDescriptor *MTLRenderPassDescriptorRef;
typedef MTLRenderPipelineColorAttachmentDescriptor *MTLRenderPipelineColorAttachmentDescriptorRef;
typedef MTLRenderPipelineColorAttachmentDescriptorArray *MTLRenderPipelineColorAttachmentDescriptorArrayRef;
typedef MTLRenderPipelineDescriptor *MTLRenderPipelineDescriptorRef;
typedef id<MTLRenderPipelineState> MTLRenderPipelineStateRef;

// MetalKit
typedef MTKView *MTKViewRef;


#pragma mark - CAMetalDrawable
///////////////////////////////////////////////////
//              CAMetalDrawable                  //
///////////////////////////////////////////////////





#pragma mark - MTLCommandBuffer
///////////////////////////////////////////////////
//              MTLCommandBuffer                 //
///////////////////////////////////////////////////
// - create command encoders -
MTLRenderCommandEncoderRef MTLCommandBufferRenderCommandEncoderWithDescriptor( MTLCommandBufferRef ref, MTLRenderPassDescriptorRef descRef );

// - scheduling and executing commands -
void MTLCommandBufferCommit( MTLCommandBufferRef ref );

// - presenting a drawable -
void MTLCommandBufferPresentDrawable( MTLCommandBufferRef ref, MTLDrawableRef drawable );

// - status and timestamps -

// - strong reference -

// - identifying command buffer -
CFStringRef MTLCommandBufferLabel( MTLCommandBufferRef ref );
void MTLCommandBufferSetLabel( MTLCommandBufferRef ref, CFStringRef label );




#pragma mark - MTLCommandEncoder
///////////////////////////////////////////////////
//              MTLCommandEncoder                //
///////////////////////////////////////////////////
// - ending encoded commands -
void MTLCommandEncoderEndEncoding( MTLCommandEncoderRef ref );





#pragma mark - MTLCommandQueue
///////////////////////////////////////////////////
//               MTLCommandQueue                 //
///////////////////////////////////////////////////
// - creating command buffers -
MTLCommandBufferRef MTLCommandQueueCommandBuffer( MTLCommandQueueRef ref );








#pragma mark - MTLDevice
///////////////////////////////////////////////////
//                  MTLDevice                    //
///////////////////////////////////////////////////
// - functions -
//MTLDeviceRef mtlCreateSystemDefaultDevice( void );
CFArrayRef mtlCopyAllDevices( void );

// - physcial properties -
Boolean MTLDeviceIsHeadless( MTLDeviceRef ref );
Boolean MTLDeviceIsLowPower( MTLDeviceRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean MTLDeviceIsRemovable( MTLDeviceRef ref );
UInt64 MTLDeviceRegistryID( MTLDeviceRef ref );
#endif
CFStringRef MTLDeviceName( MTLDeviceRef ref );

// - feature sets -
Boolean MTLDeviceSupportsFeatureSet( MTLDeviceRef ref, MTLFeatureSet set );

// - command queues -
MTLCommandQueueRef MTLDeviceNewCommandQueue( MTLDeviceRef ref );

// - libraries -
MTLLibraryRef MTLDeviceNewDefaultLibrary( MTLDeviceRef ref );

// - render pipelines -
MTLRenderPipelineStateRef MTLDeviceNewRenderPipelineStateWithDescriptor( MTLDeviceRef ref, MTLRenderPipelineDescriptorRef descRef, ErrorRef *err );

// - compute pipelines -

// - resource and threadgroup memory -

// - standard buffers -

// - standard textures and samplers -

// - linear textures -

// - read-write textures -

// - argument buffers -

// - resource heaps and fences -

// - depth and stencil state -

// - programmable sample positions -

// - raster order groups -



#pragma mark - MTLDrawable
///////////////////////////////////////////////////
//                MTLDrawable                    //
///////////////////////////////////////////////////
// - presenting -
void MTLDrawablePresent( MTLDrawableRef ref );

// - identifying -





#pragma mark - MTLFunction
///////////////////////////////////////////////////
//                   MTLFunction                 //
///////////////////////////////////////////////////





#pragma mark - MTLLibrary
///////////////////////////////////////////////////
//                    MTLLibrary                 //
///////////////////////////////////////////////////
// - non-specialized funtion
MTLFunctionRef MTLLibraryNewFunctionWithName( MTLLibraryRef ref, CFStringRef name );




#pragma mark - MTLRenderCommandEncoder
///////////////////////////////////////////////////
//           MTLRenderCommandEncoder             //
///////////////////////////////////////////////////
// - graphics rendering state -
void MTLRenderCommandEncoderSetViewport( MTLRenderCommandEncoderRef ref, MTLViewport vp );

void MTLRenderCommandEncoderSetRenderPipelineState( MTLRenderCommandEncoderRef ref, MTLRenderPipelineStateRef pipelineRef );



#pragma mark - MTLRenderPassDescriptor
///////////////////////////////////////////////////
//       MTLRenderPassDescriptor                 //
///////////////////////////////////////////////////




#pragma mark - MTLRenderPipelineDescriptor
///////////////////////////////////////////////////
//       MTLRenderPipelineDescriptor             //
///////////////////////////////////////////////////

// -- MTLRenderPipelineColorAttachmentDescriptor --

MTLPixelFormat MTLRenderPipelineColorAttachmentDescriptorPixelFormat( MTLRenderPipelineColorAttachmentDescriptorRef ref );
void MTLRenderPipelineColorAttachmentDescriptorSetPixelFormat( MTLRenderPipelineColorAttachmentDescriptorRef ref, MTLPixelFormat format );







// -- MTLRenderPipelineColorAttachmentDescriptorArray --

MTLRenderPipelineColorAttachmentDescriptorRef MTLRenderPipelineColorAttachmentDescriptorArrayObject( MTLRenderPipelineColorAttachmentDescriptorArrayRef ref, NSUInteger index );
void MTLRenderPipelineColorAttachmentDescriptorArraySetObject( MTLRenderPipelineColorAttachmentDescriptorArrayRef ref, MTLRenderPipelineColorAttachmentDescriptorRef obj, NSUInteger index );



// -- MTLRenderPipelineDescriptor --

// - init -
MTLRenderPipelineDescriptorRef MTLRenderPipelineDescriptorInit( void );

// - graphics functions and associated data -
MTLFunctionRef MTLRenderPipelineDescriptorVertexFunction( MTLRenderPipelineDescriptorRef ref );
void MTLRenderPipelineDescriptorSetVertexFunction( MTLRenderPipelineDescriptorRef ref, MTLFunctionRef functionRef );
MTLFunctionRef MTLRenderPipelineDescriptorFragmentFunction( MTLRenderPipelineDescriptorRef ref );
void MTLRenderPipelineDescriptorSetFragmentFunction( MTLRenderPipelineDescriptorRef ref, MTLFunctionRef functionRef );

// - rendering pipeline state -
MTLRenderPipelineColorAttachmentDescriptorArrayRef MTLRenderPipelineDescriptorColorAttachments( MTLRenderPipelineDescriptorRef ref );

// - rasterization and visibility state -

// - tessellation state -

// - identifying properties -
CFStringRef MTLRenderPipelineDescriptorLabel( MTLRenderPipelineDescriptorRef ref );
void MTLRenderPipelineDescriptorSetLabel( MTLRenderPipelineDescriptorRef ref, CFStringRef label );









#pragma mark - MTLRenderPipelineState
///////////////////////////////////////////////////
//           MTLRenderPipelineState              //
///////////////////////////////////////////////////
// - properties -
MTLDeviceRef MTLRenderPipelineStateDevice( MTLRenderPipelineStateRef ref );
CFStringRef MTLRenderPipelineStateLabel( MTLRenderPipelineStateRef ref );










#pragma mark - MTKView
///////////////////////////////////////////////////
//                    MTKView                    //
///////////////////////////////////////////////////
// - init -
MTKViewRef MTKViewWithFrame( CGRect r, MTLDeviceRef device );

// - delegate -

// - device -
MTLDeviceRef MTKViewDevice( MTKViewRef ref );
void MTKViewSetDevice( MTKViewRef ref, MTLDeviceRef device );

// - configure render target properties -
MTLPixelFormat MTKViewColorPixelFormat( MTKViewRef ref );
void MTKViewSetColorPixelFormat( MTKViewRef ref, MTLPixelFormat format );

// - generate render target object -
MTLRenderPassDescriptorRef MTKViewCurrentRenderPassDescriptor( MTKViewRef ref );

// - configure and query frame rate -

// - control frame updates -

// - draw view contents -

// - manage render target -
CAMetalDrawableRef MTKViewCurrentDrawable( MTKViewRef ref );











#endif// #ifdef MTL_EXTERN
#endif// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
#endif// #if __LP64__

