/*
 Metal.m
 
 Bernie Wylde 20180312
 */

#import "Metal.h"

#if __LP64__
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
#ifdef MTL_EXTERN


#pragma mark - MTLCommandBuffer
///////////////////////////////////////////////////
//              MTLCommandBuffer                 //
///////////////////////////////////////////////////
// - create command encoders -
MTLRenderCommandEncoderRef MTLCommandBufferRenderCommandEncoderWithDescriptor( MTLCommandBufferRef ref, MTLRenderPassDescriptorRef descRef )
{ return [ref renderCommandEncoderWithDescriptor:descRef]; }

// - scheduling and executing commands -
void MTLCommandBufferCommit( MTLCommandBufferRef ref )
{ [ref commit]; }

// - presenting a drawable -
void MTLCommandBufferPresentDrawable( MTLCommandBufferRef ref, MTLDrawableRef drawable )
{ [ref presentDrawable:drawable]; }

// - status and timestamps -

// - strong reference -

// - identifying command buffer -
CFStringRef MTLCommandBufferLabel( MTLCommandBufferRef ref )
{ return (CFStringRef)[ref label]; }

void MTLCommandBufferSetLabel( MTLCommandBufferRef ref, CFStringRef label )
{ [ref setLabel:(NSString *)label]; }



#pragma mark - MTLCommandEncoder
///////////////////////////////////////////////////
//              MTLCommandEncoder                //
///////////////////////////////////////////////////
// - ending encoded commands -
void MTLCommandEncoderEndEncoding( MTLCommandEncoderRef ref )
{ [ref endEncoding]; }



#pragma mark - MTLCommandQueue
///////////////////////////////////////////////////
//               MTLCommandQueue                 //
///////////////////////////////////////////////////
// - creating command buffers -
MTLCommandBufferRef MTLCommandQueueCommandBuffer( MTLCommandQueueRef ref )
{ return [ref commandBuffer]; }





#pragma mark - MTLDevice
///////////////////////////////////////////////////
//                  MTLDevice                    //
///////////////////////////////////////////////////
// - functions -
//MTLDeviceRef mtlCreateSystemDefaultDevice( void )
//{ return MTLCreateSystemDefaultDevice(); }

CFArrayRef mtlCopyAllDevices( void )
{ return (CFArrayRef)MTLCopyAllDevices(); }

// - physcial properties -
Boolean MTLDeviceIsHeadless( MTLDeviceRef ref )
{ return [ref isHeadless]; }

Boolean MTLDeviceIsLowPower( MTLDeviceRef ref )
{ return [ref isLowPower]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean MTLDeviceIsRemovable( MTLDeviceRef ref )
{ return [ref isRemovable]; }

UInt64 MTLDeviceRegistryID( MTLDeviceRef ref )
{ return [ref registryID]; }
#endif

CFStringRef MTLDeviceName( MTLDeviceRef ref )
{ return (CFStringRef)[ref name]; }

// - feature sets -
Boolean MTLDeviceSupportsFeatureSet( MTLDeviceRef ref, MTLFeatureSet set )
{ return [ref supportsFeatureSet:set]; }

// - command queues -
MTLCommandQueueRef MTLDeviceNewCommandQueue( MTLDeviceRef ref )
{ return [ref newCommandQueue]; }

// - libraries -
MTLLibraryRef MTLDeviceNewDefaultLibrary( MTLDeviceRef ref )
{ return [ref newDefaultLibrary]; }

// - render pipelines -
MTLRenderPipelineStateRef MTLDeviceNewRenderPipelineStateWithDescriptor( MTLDeviceRef ref, MTLRenderPipelineDescriptorRef descRef, ErrorRef *err )
{ return [ref newRenderPipelineStateWithDescriptor:descRef error:err]; }

// - compute pipelines -

// - resource and threadgroup memory -

// - standard buffers -

// - standard textures and samplers -

// - linear textures -

// - read-write textures -

// - argument buffers -

// - resource heaps and fences -

// - depth and stencil state -

// - programmable sample positions -

// - raster order groups -



#pragma mark - MTLDrawable
///////////////////////////////////////////////////
//                MTLDrawable                    //
///////////////////////////////////////////////////
// - presenting -
void MTLDrawablePresent( MTLDrawableRef ref )
{ [ref present]; }

// - identifying -




#pragma mark - MTLFunction
///////////////////////////////////////////////////
//                   MTLFunction                 //
///////////////////////////////////////////////////





#pragma mark - MTLLibrary
///////////////////////////////////////////////////
//                    MTLLibrary                 //
///////////////////////////////////////////////////
// - non-specialized funtion
MTLFunctionRef MTLLibraryNewFunctionWithName( MTLLibraryRef ref, CFStringRef name )
{ return [ref newFunctionWithName:(NSString *)name]; }




#pragma mark - MTLRenderCommandEncoder
///////////////////////////////////////////////////
//           MTLRenderCommandEncoder             //
///////////////////////////////////////////////////
// - graphics rendering state -
void MTLRenderCommandEncoderSetViewport( MTLRenderCommandEncoderRef ref, MTLViewport vp )
{ [ref setViewport:vp]; }

void MTLRenderCommandEncoderSetRenderPipelineState( MTLRenderCommandEncoderRef ref, MTLRenderPipelineStateRef pipelineRef )
{ [ref setRenderPipelineState:pipelineRef]; }



#pragma mark - MTLRenderPassDescriptor
///////////////////////////////////////////////////
//       MTLRenderPassDescriptor                 //
///////////////////////////////////////////////////




#pragma mark - MTLRenderPipelineDescriptor
///////////////////////////////////////////////////
//       MTLRenderPipelineDescriptor             //
///////////////////////////////////////////////////

// -- MTLRenderPipelineColorAttachmentDescriptor --
MTLPixelFormat MTLRenderPipelineColorAttachmentDescriptorPixelFormat( MTLRenderPipelineColorAttachmentDescriptorRef ref )
{ return [ref pixelFormat]; }

void MTLRenderPipelineColorAttachmentDescriptorSetPixelFormat( MTLRenderPipelineColorAttachmentDescriptorRef ref, MTLPixelFormat format )
{ [ref setPixelFormat:format]; }




// -- MTLRenderPipelineColorAttachmentDescriptorArray --
MTLRenderPipelineColorAttachmentDescriptorRef MTLRenderPipelineColorAttachmentDescriptorArrayObject( MTLRenderPipelineColorAttachmentDescriptorArrayRef ref, NSUInteger index )
{ return [ref objectAtIndexedSubscript:index]; }

void MTLRenderPipelineColorAttachmentDescriptorArraySetObject( MTLRenderPipelineColorAttachmentDescriptorArrayRef ref, MTLRenderPipelineColorAttachmentDescriptorRef obj, NSUInteger index )
{ [ref setObject:obj atIndexedSubscript:index]; }



// -- MTLRenderPipelineDescriptor --

// - init -
MTLRenderPipelineDescriptorRef MTLRenderPipelineDescriptorInit( void )
{ return [[[MTLRenderPipelineDescriptor alloc] init] autorelease]; }

// - graphics functions and associated data -
MTLFunctionRef MTLRenderPipelineDescriptorVertexFunction( MTLRenderPipelineDescriptorRef ref )
{ return [ref vertexFunction]; }

void MTLRenderPipelineDescriptorSetVertexFunction( MTLRenderPipelineDescriptorRef ref, MTLFunctionRef functionRef )
{ [ref setVertexFunction:functionRef]; }

MTLFunctionRef MTLRenderPipelineDescriptorFragmentFunction( MTLRenderPipelineDescriptorRef ref )
{ return [ref fragmentFunction]; }

void MTLRenderPipelineDescriptorSetFragmentFunction( MTLRenderPipelineDescriptorRef ref, MTLFunctionRef functionRef )
{ [ref setFragmentFunction:functionRef]; }

// - rendering pipeline state -
MTLRenderPipelineColorAttachmentDescriptorArrayRef MTLRenderPipelineDescriptorColorAttachments( MTLRenderPipelineDescriptorRef ref )
{ return [ref colorAttachments]; }

// - rasterization and visibility state -

// - tessellation state -

// - identifying properties -
CFStringRef MTLRenderPipelineDescriptorLabel( MTLRenderPipelineDescriptorRef ref )
{ return (CFStringRef)[ref label]; }

void MTLRenderPipelineDescriptorSetLabel( MTLRenderPipelineDescriptorRef ref, CFStringRef label )
{ [ref setLabel:(NSString *)label]; }






#pragma mark - MTLRenderPipelineState
///////////////////////////////////////////////////
//           MTLRenderPipelineState              //
///////////////////////////////////////////////////
// - properties -
MTLDeviceRef MTLRenderPipelineStateDevice( MTLRenderPipelineStateRef ref )
{ return [ref device]; }

CFStringRef MTLRenderPipelineStateLabel( MTLRenderPipelineStateRef ref )
{ return (CFStringRef)[ref label]; }







#pragma mark - MTKView
///////////////////////////////////////////////////
//                    MTKView                    //
///////////////////////////////////////////////////
// - init -
MTKViewRef MTKViewWithFrame( CGRect r, MTLDeviceRef device )
{ return [[[MTKView alloc] initWithFrame:NSRectFromCGRect(r) device:device] autorelease]; }

// - delegate -

// - device -
MTLDeviceRef MTKViewDevice( MTKViewRef ref )
{ return [ref device]; }

void MTKViewSetDevice( MTKViewRef ref, MTLDeviceRef device )
{ [ref setDevice:device]; }

// - configure render target properties -
MTLPixelFormat MTKViewColorPixelFormat( MTKViewRef ref )
{ return [ref colorPixelFormat]; }

void MTKViewSetColorPixelFormat( MTKViewRef ref, MTLPixelFormat format )
{ [ref setColorPixelFormat:format]; }

// - generate render target object -
MTLRenderPassDescriptorRef MTKViewCurrentRenderPassDescriptor( MTKViewRef ref )
{return [ref currentRenderPassDescriptor]; }

// - configure and query frame rate -

// - control frame updates -

// - draw view contents -

// - manage render target -
CAMetalDrawableRef MTKViewCurrentDrawable( MTKViewRef ref )
{ return [ref currentDrawable]; }





#endif// #ifdef MTL_EXTERN
#endif// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
#endif// #if __LP64__



