/*
 PDFKit.h
 
 Bernie Wylde 20180330
 */

#import "CocoaUI.h"

typedef PDFAction *PDFActionRef;
typedef PDFActionGoTo *PDFActionGoToRef;
typedef PDFActionNamed *PDFActionNamedRef;
typedef PDFActionRemoteGoTo *PDFActionRemoteGoToRef;
typedef PDFActionResetForm *PDFActionResetFormRef;
typedef PDFActionURL *PDFActionURLRef;
typedef PDFAnnotation *PDFAnnotationRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
typedef PDFAppearanceCharacteristics *PDFAppearanceCharacteristicsRef;
#endif

typedef PDFBorder *PDFBorderRef;
typedef PDFDestination *PDFDestinationRef;
typedef PDFDocument *PDFDocumentRef;
typedef PDFOutline *PDFOutlineRef;
typedef PDFPage *PDFPageRef;
typedef PDFSelection *PDFSelectionRef;
typedef PDFThumbnailView *PDFThumbnailViewRef;
typedef PDFView *PDFViewRef;


#pragma mark - PDFAction
///////////////////////////////////////////////////
//                 PDFAction                     //
///////////////////////////////////////////////////
// - type -
CFStringRef PDFActionType( PDFActionRef ref );


#pragma mark - PDFActionGoTo
///////////////////////////////////////////////////
//                 PDFActionGoTo                 //
///////////////////////////////////////////////////
// - init -
PDFActionGoToRef PDFActionGotoWithDestination( PDFDestinationRef ref );

// - destination -
PDFDestinationRef PDFActionGotoDestination( PDFActionGoToRef ref );
void PDFActionGotoSetDestination( PDFActionGoToRef ref, PDFDestinationRef dest );


#pragma mark - PDFActionNamed
///////////////////////////////////////////////////
//                 PDFActionNamed                //
///////////////////////////////////////////////////
// - init -
PDFActionNamedRef PDFActionNamedWithName( NSInteger name );

// - name -
NSInteger PDFActionNamedname( PDFActionNamedRef ref );
void PDFActionNamedSetName( PDFActionNamedRef ref, NSInteger name );


#pragma mark - PDFActionRemoteGoTo
///////////////////////////////////////////////////
//           PDFActionRemoteGoTo                 //
///////////////////////////////////////////////////
// - init -
PDFActionRemoteGoToRef PDFActionRemoteGoToWithPageIndex( NSUInteger pageIndex, CGPoint pt, CFURLRef fileURL );

// - page index -
NSUInteger PDFActionRemoteGoToPageIndex( PDFActionRemoteGoToRef ref );
void PDFActionRemoteGoToSetPageIndex( PDFActionRemoteGoToRef ref, NSUInteger index );

// - point -
CGPoint PDFActionRemoteGoToPoint( PDFActionRemoteGoToRef ref );
void PDFActionRemoteGoToSetPoint( PDFActionRemoteGoToRef ref, CGPoint pt );

// - url -
CFURLRef PDFActionRemoteGoToURL( PDFActionRemoteGoToRef ref );
void PDFActionRemoteGoToSetURL( PDFActionRemoteGoToRef ref, CFURLRef url );


#pragma mark - PDFActionResetForm
///////////////////////////////////////////////////
//           PDFActionResetForm                  //
///////////////////////////////////////////////////
// - init -
PDFActionResetFormRef PDFActionResetFormInit( void );

// - fields -
CFArrayRef PDFActionResetFormFields( PDFActionResetFormRef ref );
void PDFActionResetFormSetFields( PDFActionResetFormRef ref, CFArrayRef fields );

// - fields are cleared -
Boolean PDFActionResetFormFieldsIncludedAreCleared( PDFActionResetFormRef ref );
void PDFActionResetFormSetFieldsIncludedAreCleared( PDFActionResetFormRef ref, Boolean flag );


#pragma mark - PDFActionURL
///////////////////////////////////////////////////
//                 PDFActionURL                  //
///////////////////////////////////////////////////
// - init -
PDFActionURLRef PDFActionURLWithURL( CFURLRef url );

// - url -
CFURLRef PDFActionURLURL( PDFActionURLRef ref );
void PDFActionURLSetURL( PDFActionURLRef ref, CFURLRef url );


#pragma mark - PDFAnnotation
///////////////////////////////////////////////////
//              PDFAnnotation                    //
///////////////////////////////////////////////////
// - init -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
PDFAnnotationRef PDFAnnotationWithBounds( CGRect bounds, CFStringRef type, CFDictionaryRef properties );
#endif

// - info -
PDFPageRef PDFAnnotationPage( PDFAnnotationRef ref );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
void PDFAnnotationSetPage( PDFAnnotationRef ref, PDFPageRef pg );
#endif

CFDateRef PDFAnnotationModificationDate( PDFAnnotationRef ref );
void PDFAnnotationSetModificationDate( PDFAnnotationRef ref, CFDateRef dt );
CFStringRef PDFAnnotationUserName( PDFAnnotationRef ref );
void PDFAnnotationSetUserName( PDFAnnotationRef ref, CFStringRef name );
CFStringRef PDFAnnotationType( PDFAnnotationRef ref );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void PDFAnnotationSetType( PDFAnnotationRef ref, CFStringRef type );
#endif

CFStringRef PDFAnnotationContents( PDFAnnotationRef ref );
void PDFAnnotationSetContents( PDFAnnotationRef ref, CFStringRef contents );

// - characteristics -
CGRect PDFAnnotationBounds( PDFAnnotationRef ref );
void PDFAnnotationSetBounds( PDFAnnotationRef ref, CGRect bounds );
PDFBorderRef PDFAnnotationBorder( PDFAnnotationRef ref );
void PDFAnnotationSetBorder( PDFAnnotationRef ref, PDFBorderRef border );
ColorRef PDFAnnotationColor( PDFAnnotationRef ref );
void PDFAnnotationSetColor( PDFAnnotationRef ref, ColorRef col );
Boolean PDFAnnotationHasAppearanceStream( PDFAnnotationRef ref );

// - drawing and output -
Boolean PDFAnnotationShouldDisplay( PDFAnnotationRef ref );
void PDFAnnotationSetShouldDisplay( PDFAnnotationRef ref, Boolean flag );
Boolean PDFAnnotationShouldPrint( PDFAnnotationRef ref );
void PDFAnnotationSetShouldPrint( PDFAnnotationRef ref, Boolean flag );

// - instance properties -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
PDFActionRef PDFAnnotationAction( PDFAnnotationRef ref );
void PDFAnnotationSetAction( PDFAnnotationRef ref, PDFActionRef action );
NSTextAlignment PDFAnnotationAlignment( PDFAnnotationRef ref );
void PDFAnnotationSetAlignment( PDFAnnotationRef ref, NSTextAlignment alignment );
Boolean PDFAnnotationAllowsToggleToOff( PDFAnnotationRef ref );
void PDFAnnotationSetAllowsToggleToOff( PDFAnnotationRef ref, Boolean flag );
CFDictionaryRef PDFAnnotationKeyValues( PDFAnnotationRef ref );
ColorRef PDFAnnotationBackgroundColor( PDFAnnotationRef ref );
void PDFAnnotationSetBackgroundColor( PDFAnnotationRef ref, ColorRef col );
PDFWidgetCellState PDFAnnotationButtonWidgetState( PDFAnnotationRef ref );
void PDFAnnotationSetButtonWidgetState( PDFAnnotationRef ref, PDFWidgetCellState state );
CFStringRef PDFAnnotationButtonWidgetStateString( PDFAnnotationRef ref );
void PDFAnnotationSetButtonWidgetStateString( PDFAnnotationRef ref, CFStringRef string );
CFStringRef PDFAnnotationCaption( PDFAnnotationRef ref );
void PDFAnnotationSetCaption( PDFAnnotationRef ref, CFStringRef caption );
CFArrayRef PDFAnnotationChoices( PDFAnnotationRef ref );
void PDFAnnotationSetChoices( PDFAnnotationRef ref, CFArrayRef choices );
PDFDestinationRef PDFAnnotationDestination( PDFAnnotationRef ref );
void PDFAnnotationSetDestination( PDFAnnotationRef ref, PDFDestinationRef dest );
PDFLineStyle PDFAnnotationEndLineStyle( PDFAnnotationRef ref );
void PDFAnnotationSetEndLineStyle( PDFAnnotationRef ref, PDFLineStyle style );
CGPoint PDFAnnotationEndPoint( PDFAnnotationRef ref );
void PDFAnnotationSetEndPoint( PDFAnnotationRef ref, CGPoint pt );
CFStringRef PDFAnnotationFieldName( PDFAnnotationRef ref );
void PDFAnnotationSetFieldName( PDFAnnotationRef ref, CFStringRef name );
FontRef PDFAnnotationFont( PDFAnnotationRef ref );
void PDFAnnotationSetFont( PDFAnnotationRef ref, FontRef font );
ColorRef PDFAnnotationFontColor( PDFAnnotationRef ref );
void PDFAnnotationSetFontColor( PDFAnnotationRef ref, ColorRef col );
Boolean PDFAnnotationHasComb( PDFAnnotationRef ref );
void PDFAnnotationSetComb( PDFAnnotationRef ref, Boolean flag );
PDFTextAnnotationIconType PDFAnnotationIconType( PDFAnnotationRef ref );
void PDFAnnotationSetIconType( PDFAnnotationRef ref, PDFTextAnnotationIconType type );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101302 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101302
ColorRef PDFAnnotationInteriorColor( PDFAnnotationRef ref );
void PDFAnnotationSetInteriorColor( PDFAnnotationRef ref, ColorRef col );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean PDFAnnotationIsHighlighted( PDFAnnotationRef ref );
void PDFAnnotationSetHighlighted( PDFAnnotationRef ref, Boolean flag );
Boolean PDFAnnotationIsListChoice( PDFAnnotationRef ref );
void PDFAnnotationSetListChoice( PDFAnnotationRef ref, Boolean flag );
Boolean PDFAnnotationIsMultiline( PDFAnnotationRef ref );
void PDFAnnotationSetMultiline( PDFAnnotationRef ref, Boolean flag );
Boolean PDFAnnotationIsOpen( PDFAnnotationRef ref );
void PDFAnnotationSetOpen( PDFAnnotationRef ref, Boolean flag );
Boolean PDFAnnotationIsPasswordField( PDFAnnotationRef ref );
Boolean PDFAnnotationIsReadOnly( PDFAnnotationRef ref );
void PDFAnnotationSetReadOnly( PDFAnnotationRef ref, Boolean flag );
PDFMarkupType PDFAnnotationMarkupType( PDFAnnotationRef ref );
void PDFAnnotationSetMarkupType( PDFAnnotationRef ref, PDFMarkupType type );
NSInteger PDFAnnotationMaximumLength( PDFAnnotationRef ref );
void PDFAnnotationSetMaximumLength( PDFAnnotationRef ref, NSInteger length );
CFArrayRef PDFAnnotationPaths( PDFAnnotationRef ref );
CFArrayRef PDFAnnotationQuadrilateralPoints( PDFAnnotationRef ref );
void PDFAnnotationSetQuadrilateralPoints( PDFAnnotationRef ref, CFArrayRef points );
Boolean PDFAnnotationRadiosInUnison( PDFAnnotationRef ref );
void PDFAnnotationSetRadiosInUnison( PDFAnnotationRef ref, Boolean flag );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101302 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101302
CFStringRef PDFAnnotationStampName( PDFAnnotationRef ref );
void PDFAnnotationSetStampName( PDFAnnotationRef ref, CFStringRef name );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
PDFLineStyle PDFAnnotationStartLineStyle( PDFAnnotationRef ref );
void PDFAnnotationSetStartLineStyle( PDFAnnotationRef ref, PDFLineStyle style );
CGPoint PDFAnnotationStartPoint( PDFAnnotationRef ref );
void PDFAnnotationSetStartPoint( PDFAnnotationRef ref, CGPoint pt );
CFURLRef PDFAnnotationURL( PDFAnnotationRef ref );
void PDFAnnotationSetURL( PDFAnnotationRef ref, CFURLRef url );
CFArrayRef PDFAnnotationValues( PDFAnnotationRef ref );
void PDFAnnotationSetValues( PDFAnnotationRef ref, CFArrayRef values );
PDFWidgetControlType PDFAnnotationWidgetControlType( PDFAnnotationRef ref );
void PDFAnnotationSetWidgetControlType( PDFAnnotationRef ref, PDFWidgetControlType type );
CFStringRef PDFAnnotationWidgetDefaultStringValue( PDFAnnotationRef ref );
void PDFAnnotationSetWidgetDefaultStringValue( PDFAnnotationRef ref, CFStringRef string );
CFStringRef PDFAnnotationWidgetFieldType( PDFAnnotationRef ref );
void PDFAnnotationSetWidgetFieldType( PDFAnnotationRef ref, CFStringRef type );
CFStringRef PDFAnnotationWidgetStringValue( PDFAnnotationRef ref );
void PDFAnnotationSetWidgetStringValue( PDFAnnotationRef ref, CFStringRef string );
#endif

// - instance methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void PDFAnnotationAddBezierPath( PDFAnnotationRef ref, BezierPathRef path );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
void PDFAnnotationDrawWithBox( PDFAnnotationRef ref, PDFDisplayBox bx, CGContextRef context );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void PDFAnnotationRemoveBezierPath( PDFAnnotationRef ref, BezierPathRef path );// macOS 10.13+
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
void PDFAnnotationRemoveValueForKey( PDFAnnotationRef ref, CFStringRef key );
void PDFAnnotationSetBooleanForKey( PDFAnnotationRef ref, Boolean flag, CFStringRef key );
void PDFAnnotationSetRectForKey( PDFAnnotationRef ref, CGRect r, CFStringRef key );
void PDFAnnotationSetValueForKey( PDFAnnotationRef ref, CFTypeRef value, CFStringRef key );
CFTypeRef PDFAnnotationValueForKey( PDFAnnotationRef ref, CFStringRef key );
#endif

// - type methods -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
PDFLineStyle PDFAnnotationLineStyleFromName( CFStringRef name );// macOS 10.13+
CFStringRef PDFAnnotationNameForLineStyle( PDFLineStyle style );// macOS 10.13+
#endif


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
#pragma mark - PDFAppearanceCharacteristics
///////////////////////////////////////////////////
//      PDFAppearanceCharacteristics             //
///////////////////////////////////////////////////
// - instance properties -
CFDictionaryRef PDFAppearanceCharacteristicsKeyValues( PDFAppearanceCharacteristicsRef ref );
ColorRef PDFAppearanceCharacteristicsBackgroundColor( PDFAppearanceCharacteristicsRef ref );
void PDFAppearanceCharacteristicsSetBackgroundColor( PDFAppearanceCharacteristicsRef ref, ColorRef col );
ColorRef PDFAppearanceCharacteristicsBorderColor( PDFAppearanceCharacteristicsRef ref );
void PDFAppearanceCharacteristicsSetBorderColor( PDFAppearanceCharacteristicsRef ref, ColorRef col );
CFStringRef PDFAppearanceCharacteristicsCaption( PDFAppearanceCharacteristicsRef ref );
void PDFAppearanceCharacteristicsSetCaption( PDFAppearanceCharacteristicsRef ref, CFStringRef caption );
PDFWidgetControlType PDFAppearanceCharacteristicsControlType( PDFAppearanceCharacteristicsRef ref );
void PDFAppearanceCharacteristicsSetControlType( PDFAppearanceCharacteristicsRef ref, PDFWidgetControlType type );
CFStringRef PDFAppearanceCharacteristicsDownCaption( PDFAppearanceCharacteristicsRef ref );
void PDFAppearanceCharacteristicsSetDownCaption( PDFAppearanceCharacteristicsRef ref, CFStringRef caption );
CFStringRef PDFAppearanceCharacteristicsRolloverCaption( PDFAppearanceCharacteristicsRef ref );
void PDFAppearanceCharacteristicsSetRolloverCaption( PDFAppearanceCharacteristicsRef ref, CFStringRef caption );
NSInteger PDFAppearanceCharacteristicsRotation( PDFAppearanceCharacteristicsRef ref );
void PDFAppearanceCharacteristicsSetRotation( PDFAppearanceCharacteristicsRef ref, NSInteger rotation );
#endif


#pragma mark - PDFBorder
///////////////////////////////////////////////////
//                 PDFBorder                     //
///////////////////////////////////////////////////
// - init -
PDFBorderRef PDFBorderInit( void );

// - styles and characteristics -
NSInteger PDFBorderstyle( PDFBorderRef ref );
void PDFBorderSetStyle( PDFBorderRef ref, NSInteger style );
CGFloat PDFBorderLineWidth( PDFBorderRef ref );
void PDFBorderSetLineWidth( PDFBorderRef ref, CGFloat lineWidth );
CFArrayRef PDFBorderDashPattern( PDFBorderRef ref );
void PDFBorderSetDashPattern( PDFBorderRef ref, CFArrayRef pattern );

// - drawing -
void PDFBorderDrawInRect( PDFBorderRef ref, CGRect r );

// - instance properties -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFDictionaryRef PDFBorderKeyValues( PDFBorderRef ref );
#endif


#pragma mark - PDFDestination
///////////////////////////////////////////////////
//               PDFDestination                  //
///////////////////////////////////////////////////
// - init -
PDFDestinationRef PDFDestinationWithPage( PDFPageRef pg, CGPoint pt );

// - pages and point -
PDFPageRef PDFDestinationPage( PDFDestinationRef dest );
CGPoint PDFDestinationPoint( PDFDestinationRef dest );

// - relative location -
NSComparisonResult PDFDestinationCompare( PDFDestinationRef dest1, PDFDestinationRef dest2 );

// - instance properties -
CGFloat PDFDestinationZoom( PDFDestinationRef dest );
void PDFDestinationSetZoom( PDFDestinationRef dest, CGFloat value );


#pragma mark - PDFDocument
///////////////////////////////////////////////////
//              PDFDocument                      //
///////////////////////////////////////////////////
// - init -
PDFDocumentRef PDFDocumentWithURL( CFURLRef url );
PDFDocumentRef PDFDocumentWithData( CFDataRef dta );
PDFDocumentRef PDFDocumentInit( void );

// - read and write -
// -- read operations --
// --- info ---
CFURLRef PDFDocumentURL( PDFDocumentRef ref );
NSInteger PDFDocumentMajorVersion( PDFDocumentRef ref );
NSInteger PDFDocumentMinorVersion( PDFDocumentRef ref );
CFStringRef PDFDocumentString( PDFDocumentRef ref );
PDFOutlineRef PDFDocumentOutlineItemForSelection( PDFDocumentRef docRef, PDFSelectionRef selRef );
PDFOutlineRef PDFDocumentOutlineRoot( PDFDocumentRef ref );
void PDFDocumentSetOutlineRoot( PDFDocumentRef docRef, PDFOutlineRef olRef );
CFDictionaryRef PDFDocumentAttributes( PDFDocumentRef ref );
void PDFDocumentSetAttributes( PDFDocumentRef ref, CFDictionaryRef attributes );
CGPDFDocumentRef PDFDocumentCGPDFRef( PDFDocumentRef ref );

// --- security ---
Boolean PDFDocumentIsEncrypted( PDFDocumentRef ref );
Boolean PDFDocumentIsLocked( PDFDocumentRef ref );
Boolean PDFDocumentUnlockWithPassword( PDFDocumentRef ref, CFStringRef password );
PDFDocumentPermissions PDFDocumentPermissionsStatus( PDFDocumentRef ref );

// ---- permissions properties ----
Boolean PDFDocumentAllowsCopying( PDFDocumentRef ref );
Boolean PDFDocumentAllowsPrinting( PDFDocumentRef ref );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean PDFDocumentAllowsCommenting( PDFDocumentRef ref );
Boolean PDFDocumentAllowsContentAccessibility( PDFDocumentRef ref );
Boolean PDFDocumentAllowsDocumentAssembly( PDFDocumentRef ref );
Boolean PDFDocumentAllowsDocumentChanges( PDFDocumentRef ref );
Boolean PDFDocumentAllowsFormFieldEntry( PDFDocumentRef ref );
#endif

// --- selections and searches ---
PDFSelectionRef PDFDocumentSelectionFromPageAtIndex( PDFDocumentRef ref, PDFPageRef startPage, NSUInteger startCharIndex, PDFPageRef endPage, NSUInteger endCharIndex );
PDFSelectionRef PDFDocumentSelectionFromPageAtPoint( PDFDocumentRef ref, PDFPageRef startPage, CGPoint startPt, PDFPageRef endPage, CGPoint endPt );
PDFSelectionRef PDFDocumentSelectionForEntireDocument( PDFDocumentRef ref );

// ---- search options ----
CFArrayRef PDFDocumentFindString( PDFDocumentRef ref, CFStringRef string, NSStringCompareOptions options );
//void PDFDocumentBeginFindString( PDFDocumentRef ref, CFStringRef string, NSStringCompareOptions options );
//void PDFDocumentBeginFindStrings( PDFDocumentRef ref, CFArrayRef strings, NSStringCompareOptions options );
PDFSelectionRef PDFDocumentFindStringFromSelection( PDFDocumentRef ref, CFStringRef string, PDFSelectionRef selection, NSStringCompareOptions options );
Boolean PDFDocumentIsFinding( PDFDocumentRef ref );
void PDFDocumentCancelFindString( PDFDocumentRef ref );

// --- pages ---
NSUInteger PDFDocumentPageCount( PDFDocumentRef ref );
PDFPageRef PDFDocumentPageAtIndex( PDFDocumentRef ref, NSUInteger index );
NSUInteger PDFDocumentIndexForPage( PDFDocumentRef ref, PDFPageRef pg );
void PDFDocumentInsertPageAtIndex( PDFDocumentRef ref, PDFPageRef pg, NSUInteger index );
void PDFDocumentRemovePageAtIndex( PDFDocumentRef ref, NSUInteger index );
void PDFDocumentExchangePageAtIndex( PDFDocumentRef ref, NSUInteger index1, NSUInteger index2 );
ClassRef PDFDocumentPageClass( PDFDocumentRef ref );

// -- write operations --
// --- data ---
Boolean PDFDocumentWriteToURL( PDFDocumentRef ref, CFURLRef url );
Boolean PDFDocumentWriteToURLWithOptions( PDFDocumentRef ref, CFURLRef url, CFDictionaryRef options );

// ---- data representations ----
CFDataRef PDFDocumentDataRepresentation( PDFDocumentRef ref );
CFDataRef PDFDocumentDataRepresentationWithOptions( PDFDocumentRef ref, CFDictionaryRef options );

// --- printing ---
PrintOperationRef PDFDocumentPrintOperationForPrintInfo( PDFDocumentRef ref, PrintInfoRef info, PDFPrintScalingMode scalingMode, Boolean autorotate );

// - delegate -


#pragma mark - PDFOutline
///////////////////////////////////////////////////
//                   PDFOutline                  //
///////////////////////////////////////////////////
// - init -
PDFOutlineRef PDFOutlineInit( void );

// - info -
PDFDocumentRef PDFOutlineDocument( PDFOutlineRef ref );
NSUInteger PDFOutlineNumberOfChildren( PDFOutlineRef ref );
PDFOutlineRef PDFOutlineParent( PDFOutlineRef ref );
PDFOutlineRef PDFOutlineChildAtIndex( PDFOutlineRef ref, NSUInteger index );
NSUInteger PDFOutlineIndex( PDFOutlineRef ref );

// - label -
CFStringRef PDFOutlineLabel( PDFOutlineRef ref );
void PDFOutlineSetLabel( PDFOutlineRef ref, CFStringRef label );

// - actions and destinations -
PDFDestinationRef PDFOutlineDestination( PDFOutlineRef ref );
void PDFOutlineSetDestination( PDFOutlineRef ref, PDFDestinationRef dest );
PDFActionRef PDFOutlineAction( PDFOutlineRef ref );
void PDFOutlineSetAction( PDFOutlineRef ref, PDFActionRef action );

// - hierarchy -
void PDFOutlineInsertChildAtIndex( PDFOutlineRef ref, PDFOutlineRef childRef, NSUInteger index );
void PDFOutlineRemoveFromParent( PDFOutlineRef ref );

// - disclosure -
Boolean PDFOutlineIsOpen( PDFOutlineRef ref );
void PDFOutlineSetIsOpen( PDFOutlineRef ref, Boolean flag );


#pragma mark - PDFPage
///////////////////////////////////////////////////
//                  PDFPage                      //
///////////////////////////////////////////////////
// -- init --
PDFPageRef PDFPageWithImage( ImageRef image );

// -- info --
PDFDocumentRef PDFPageDocument( PDFPageRef pg );
CFStringRef PDFPageLabel( PDFPageRef pg );
CGRect PDFPageBoundsForBox( PDFPageRef pg, PDFDisplayBox bx );
void PDFPageSetBoundsForBox( PDFPageRef pg, CGRect bounds, PDFDisplayBox bx );
NSInteger PDFPageRotation( PDFPageRef pg );
void PDFPageSetRotation( PDFPageRef pg, NSInteger rotation );

// -- annotations --
CFArrayRef PDFPageAnnotations( PDFPageRef pg );
Boolean PDFPageDisplaysAnnotations( PDFPageRef pg );
void PDFPageSetDisplaysAnnotations( PDFPageRef pg, Boolean flag );
void PDFPageAddAnnotation( PDFPageRef pg, PDFAnnotationRef annotation );
void PDFPageRemoveAnnotation( PDFPageRef pg, PDFAnnotationRef annotation );
PDFAnnotationRef PDFPageAnnotationAtPoint( PDFPageRef pg, CGPoint pt );

// -- textual content --
NSUInteger PDFPageNumberOfCharacters( PDFPageRef pg );
CFStringRef PDFPageString( PDFPageRef pg );
CFAttributedStringRef PDFPageAttributedString( PDFPageRef pg );
CGRect PDFPageCharacterBoundsAtIndex( PDFPageRef pg, NSInteger index );
NSInteger PDFPageCharacterIndexAtPoint( PDFPageRef pg, CGPoint pt );

// -- selections --
PDFSelectionRef PDFPageSelectionForRect( PDFPageRef pg, CGRect r );
PDFSelectionRef PDFPageSelectionForWordAtPoint( PDFPageRef pg, CGPoint pt );
PDFSelectionRef PDFPageSelectionForLineAtPoint( PDFPageRef pg, CGPoint pt );
PDFSelectionRef PDFPageSelectionFromPointToPoint( PDFPageRef pg, CGPoint startPt, CGPoint endPt );
PDFSelectionRef PDFPageSelectionForRange( PDFPageRef pg, CFRange range );

// -- instance properties --
CFDataRef PDFPageDataRepresentation( PDFPageRef pg );
CGPDFPageRef PDFPagePageRef( PDFPageRef pg );

// -- instance methods --
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
void PDFPageDrawWithBoxToContext( PDFPageRef pg, PDFDisplayBox bx, CGContextRef context );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
ImageRef PDFPageThumbnailOfSizeForBox( PDFPageRef pg, CGSize size, PDFDisplayBox bx );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
void PDFPageTransformContextForBox( PDFPageRef pg, CGContextRef context, PDFDisplayBox bx );
CGAffineTransform PDFPageTransformForBox( PDFPageRef pg, PDFDisplayBox bx );
#endif


#pragma mark - PDFSelection
///////////////////////////////////////////////////
//                PDFSelection                   //
///////////////////////////////////////////////////
// - init -
PDFSelectionRef PDFSelectionWithDocument( PDFDocumentRef doc );

// - info -
CFArrayRef PDFSelectionPages( PDFSelectionRef ref );
CFStringRef PDFSelectionString( PDFSelectionRef ref );
CFAttributedStringRef PDFSelectionAttributedString( PDFSelectionRef ref );
CGRect PDFSelectionBoundsForPage( PDFSelectionRef ref, PDFPageRef pg );
CFArrayRef PDFSelectionSelectionsByLine( PDFSelectionRef ref );
ColorRef PDFSelectionColor( PDFSelectionRef ref );
void PDFSelectionSetColor( PDFSelectionRef ref, ColorRef col );

// - modify -
void PDFSelectionAddSelection( PDFSelectionRef ref, PDFSelectionRef sel );
void PDFSelectionAddSelections( PDFSelectionRef ref, CFArrayRef selections );
void PDFSelectionExtendSelectionAtEnd( PDFSelectionRef ref, NSInteger succeed );
void PDFSelectionExtendSelectionAtStart( PDFSelectionRef ref, NSInteger precede );

// - drawing -
void PDFSelectionDrawForPage( PDFSelectionRef ref, PDFPageRef pg, Boolean active );
void PDFSelectionDrawForPageWithBox( PDFSelectionRef ref, PDFPageRef pg, PDFDisplayBox bx, Boolean active );

// - instance methods -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void PDFSelectionExtendSelectionForLineBoundaries( PDFSelectionRef ref );
#endif

NSUInteger PDFSelectionNumberOfTextRangesOnPage( PDFSelectionRef ref, PDFPageRef pg );
CFRange PDFSelectionRangeAtIndexOnPage( PDFSelectionRef ref, NSUInteger index, PDFPageRef pg );


#pragma mark - PDFThumbnailView
///////////////////////////////////////////////////
//         PDFThumbnailView                      //
///////////////////////////////////////////////////

@interface PDFThumbnailViewSC : PDFThumbnailView
@end


@interface PDFThumbnailView (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

PDFThumbnailViewRef PDFThumbnailViewWithTag( NSInteger tag );
BOOL PDFThumbnailViewExists( NSInteger tag );

// -- PDFView --
PDFViewRef PDFThumbnailViewPDFView( NSInteger tag );
void PDFThumbnailViewSetPDFView( NSInteger thumbTag, NSInteger pdfTag );

// -- size --
CGSize PDFThumbnailViewThumbnailSize( NSInteger tag );
void PDFThumbnailViewSetThumbnailSize( NSInteger tag, CGSize size );

// -- display --
NSUInteger PDFThumbnailViewMaxiumumNumberOfColumns( NSInteger tag );
void PDFThumbnailViewSetMaxiumumNumberOfColumns( NSInteger tag, NSUInteger columns );
FontRef PDFThumbnailViewLabelFont( NSInteger tag );
void PDFThumbnailViewSetLabelFont( NSInteger tag, FontRef font );
ColorRef PDFThumbnailViewBackgroundColor( NSInteger tag );
void PDFThumbnailViewSetBackgroundColor( NSInteger tag, ColorRef col );

// -- behaviour --
Boolean PDFThumbnailViewAllowsDragging( NSInteger tag );
void PDFThumbnailViewSetAllowsDragging( NSInteger tag, Boolean flag );
Boolean PDFThumbnailViewAllowsMultipleSelection( NSInteger tag );
void PDFThumbnailViewSetMultipleSelection( NSInteger tag, Boolean flag );
CFArrayRef PDFThumbnailViewSelectedPages( NSInteger tag );


// Init
PDFThumbnailViewRef PDFThumbnailViewInit( NSInteger inTag, CGRect inRect );

// - FB statement -
void CocoaPDFThumbnailView( NSInteger inTag, CGRect inRect, NSInteger inPDFViewTag, NSInteger inWndTag );



#pragma mark - PDFView
///////////////////////////////////////////////////
//                  PDFView                      //
///////////////////////////////////////////////////
@interface PDFViewSC : PDFView
@end

@interface PDFView (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

PDFViewRef PDFViewWithTag( NSInteger tag );
BOOL PDFViewExists( NSInteger tag );

// - document -
PDFDocumentRef PDFViewDocument( NSInteger tag );
void PDFViewSetDocument( NSInteger tag, PDFDocumentRef doc );

// - configurations -
// -- display mode --
PDFDisplayMode PDFViewDisplayMode( NSInteger tag );
void PDFViewSetDisplayMode( NSInteger tag, PDFDisplayMode displayMode );

// -- additional display configurations --
// --- display box ---
PDFDisplayBox PDFViewDisplayBox( NSInteger tag );
void PDFViewSetDisplayBox( NSInteger tag, PDFDisplayBox displayBox );

// --- page breaks ---
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean PDFViewDisplaysPageBreaks( NSInteger tag );
void PDFViewSetDisplaysPageBreaks( NSInteger tag, Boolean flag );
NSEdgeInsets PDFViewPageBreakMargins( NSInteger tag );
void PDFViewSetPageBreakMargins( NSInteger tag, NSEdgeInsets insets );
#endif

// --- display direction ---
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
PDFDisplayDirection PDFViewDisplayDirection( NSInteger tag );
void PDFViewSetDisplayDirection( NSInteger tag, PDFDisplayDirection direction );
Boolean PDFViewDisplaysRTL( NSInteger tag );
void PDFViewSetDisplaysRTL( NSInteger tag, Boolean flag );
#endif

// -- book display --
// --- display as book ---
Boolean PDFViewDisplaysAsBook( NSInteger tag );
void PDFViewSetDisplaysAsBook( NSInteger tag, Boolean flag );

// -- graphics properties --
// --- background color ---
ColorRef PDFViewBackgroundColor( NSInteger tag );
void PDFViewSetBackgroundColor( NSInteger tag, ColorRef col );

// - antialiasing -
PDFInterpolationQuality PDFViewInterpolationQuality( NSInteger tag );

// -- scaling the view --
CGFloat PDFViewScaleFactor( NSInteger tag );
void PDFViewSetScaleFactor( NSInteger tag, CGFloat factor );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CGFloat PDFViewScaleFactorForSizeToFit( NSInteger tag );
CGFloat PDFViewMaxScaleFactor( NSInteger tag );
void PDFViewSetMaxScaleFactor( NSInteger tag, CGFloat factor );
CGFloat PDFViewMinScaleFactor( NSInteger tag );
void PDFViewSetMinScaleFactor( NSInteger tag, CGFloat factor );
#endif

Boolean PDFViewAutoScales( NSInteger tag );
void PDFViewSetAutoScales( NSInteger tag, Boolean flag );
CGSize PDFViewRowSizeForPage( NSInteger tag, PDFPageRef pg );

// --- zoom operations ---
void PDFViewZoomIn( NSInteger tag );
Boolean PDFViewCanZoomIn( NSInteger tag );
void PDFViewZoomOut( NSInteger tag );
Boolean PDFViewCanZoomOut( NSInteger tag );

// -- rendering and printing the view --
void PDFViewPrintWithInfo( NSInteger tag, PrintInfoRef printInfo, Boolean autoRotate );
void PDFViewPrintWithInfoPageScaling( NSInteger tag, PrintInfoRef printInfo, Boolean autoRotate, PDFPrintScalingMode scale );

// -- specializing the view
ViewRef PDFViewDocumentView( NSInteger tag );
void PDFViewLayoutDocumentView( NSInteger tag );

// -- draw operations --
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
void PDFViewDrawPageToContext( NSInteger tag, PDFPageRef pg, CGContextRef context );
void PDFViewDrawPagePostToContext( NSInteger tag, PDFPageRef pg, CGContextRef context );
#endif

// -- document interactions --
// --- selections ---
PDFSelectionRef PDFViewCurrentSelection( NSInteger tag );
void PDFViewSetCurrentSelection( NSInteger tag, PDFSelectionRef selection, Boolean animate );
void PDFViewSelectAll( NSInteger tag );
void PDFViewClearSelection( NSInteger tag );
void PDFViewCopy( NSInteger tag );
void PDFViewScrollSelectionToVisible( NSInteger tag );
CFArrayRef PDFViewHighlightedSelections( NSInteger tag );

// --- annotation actions ---
void PDFViewAnnotationsChangedOnPage( NSInteger tag, PDFPageRef pg );

// --- link annotations ---
Boolean PDFViewEnableDataDetectors( NSInteger tag );

// --- convert page and view points ---
PDFPageRef PDFViewPageForPoint( NSInteger tag, CGPoint pt, Boolean nearest );
CGPoint PDFViewConvertPointToPage( NSInteger tag, CGPoint pt, PDFPageRef pg );
CGRect PDFViewConvertRectToPage( NSInteger tag, CGRect r, PDFPageRef pg );
CGPoint PDFViewConvertPointFromPage( NSInteger tag, CGPoint pt, PDFPageRef pg );
CGRect PDFViewConvertRectFromPage( NSInteger tag, CGRect r, PDFPageRef pg );

// --- mouse position and events ---
PDFAreaOfInterest PDFViewAreaOfInterestForMouse( NSInteger tag, CocoaEventRef evnt );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101100
PDFAreaOfInterest PDFViewAreaOfInterestForPoint( NSInteger tag, CGPoint pt );
#endif

void PDFViewSetCursorForAreaOfInterest( NSInteger tag, PDFAreaOfInterest area );
void PDFViewPerformAction( NSInteger tag, PDFActionRef action );

// --- drag operations
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean PDFViewAcceptsDraggedFiles( NSInteger tag );
void PDFViewSetAcceptsDraggedFiles( NSInteger tag, Boolean flag );
#endif

// -- navigating within document --
PDFPageRef PDFViewCurrentPage( NSInteger tag );
PDFDestinationRef PDFViewCurrentDestination( NSInteger tag );
CFArrayRef PDFViewVisiblePages( NSInteger tag );

// -- navigation --
Boolean PDFViewCanGoBack( NSInteger tag );
Boolean PDFViewCanGoForward( NSInteger tag );
Boolean PDFViewCanGoToFirstPage( NSInteger tag );
Boolean PDFViewCanGoToLastPage( NSInteger tag );
Boolean PDFViewCanGoToNextPage( NSInteger tag );
Boolean PDFViewCanGoToPreviousPage( NSInteger tag );

void PDFViewGoBack( NSInteger tag );
void PDFViewGoForward( NSInteger tag );
void PDFViewGoToFirstPage( NSInteger tag );
void PDFViewGoToLastPage( NSInteger tag );
void PDFViewGoToNextPage( NSInteger tag );
void PDFViewGoToPreviousPage( NSInteger tag );

void PDFViewGoToPage( NSInteger tag, PDFPageRef pg );
void PDFViewGoToDestination( NSInteger tag, PDFDestinationRef destination );
void PDFViewGoToSelection( NSInteger tag, PDFSelectionRef selection );
void PDFViewGoToRectOnPage( NSInteger tag, CGRect r, PDFPageRef pg );

// -- delegate --

// - convenience -
NSUInteger PDFViewCurrentPageIndex( NSInteger tag );
void PDFViewGoToPageAtIndex( NSInteger tag, NSUInteger index );


// Init
PDFViewRef PDFViewInit( NSInteger inTag, CGRect inRect );

// - FB statement -
void CocoaPDFView( NSInteger inTag, CGRect inRect, PDFDocumentRef inDocRef, NSInteger inWndTag );


