/*
 PDFKit.m
 
 Bernie Wylde 20180330
 */

#import "PDFKit.h"

#pragma mark - PDFAction
///////////////////////////////////////////////////
//                 PDFAction                     //
///////////////////////////////////////////////////
// - type -
CFStringRef PDFActionType( PDFActionRef ref )
{ return (CFStringRef)[ref type]; }


#pragma mark - PDFActionGoTo
///////////////////////////////////////////////////
//                 PDFActionGoTo                 //
///////////////////////////////////////////////////
// - init -
PDFActionGoToRef PDFActionGotoWithDestination( PDFDestinationRef ref )
{ return [[[PDFActionGoTo alloc] initWithDestination:ref] autorelease]; }

// - destination -
PDFDestinationRef PDFActionGotoDestination( PDFActionGoToRef ref )
{ return [ref destination]; }

void PDFActionGotoSetDestination( PDFActionGoToRef ref, PDFDestinationRef dest )
{ return [ref setDestination:dest]; }


#pragma mark - PDFActionNamed
///////////////////////////////////////////////////
//                 PDFActionNamed                //
///////////////////////////////////////////////////
// - init -
PDFActionNamedRef PDFActionNamedWithName( NSInteger name )
{ return [[[PDFActionNamed alloc] initWithName:name] autorelease]; }

// - name -
NSInteger PDFActionNamedname( PDFActionNamedRef ref )
{ return [ref name]; }

void PDFActionNamedSetName( PDFActionNamedRef ref, NSInteger name )
{ [ref setName:name]; }


#pragma mark - PDFActionRemoteGoTo
///////////////////////////////////////////////////
//           PDFActionRemoteGoTo                 //
///////////////////////////////////////////////////
// - init -
PDFActionRemoteGoToRef PDFActionRemoteGoToWithPageIndex( NSUInteger pageIndex, CGPoint pt, CFURLRef fileURL )
{ return [[[PDFActionRemoteGoTo alloc] initWithPageIndex:pageIndex atPoint:NSPointFromCGPoint(pt) fileURL:(NSURL *)fileURL] autorelease]; }

// - page index -
NSUInteger PDFActionRemoteGoToPageIndex( PDFActionRemoteGoToRef ref )
{ return [ref pageIndex]; }

void PDFActionRemoteGoToSetPageIndex( PDFActionRemoteGoToRef ref, NSUInteger index )
{ [ref setPageIndex:index]; }

// - point -
CGPoint PDFActionRemoteGoToPoint( PDFActionRemoteGoToRef ref )
{ return NSPointToCGPoint([ref point]); }

void PDFActionRemoteGoToSetPoint( PDFActionRemoteGoToRef ref, CGPoint pt )
{ [ref setPoint:NSPointFromCGPoint(pt)]; }

// - url -
CFURLRef PDFActionRemoteGoToURL( PDFActionRemoteGoToRef ref )
{ return (CFURLRef)[ref URL]; }

void PDFActionRemoteGoToSetURL( PDFActionRemoteGoToRef ref, CFURLRef url )
{ [ref setURL:(NSURL *)url]; }


#pragma mark - PDFActionResetForm
///////////////////////////////////////////////////
//           PDFActionResetForm                  //
///////////////////////////////////////////////////
// - init -
PDFActionResetFormRef PDFActionResetFormInit( void )
{ return [[[PDFActionResetForm alloc] init] autorelease]; }

// - fields -
CFArrayRef PDFActionResetFormFields( PDFActionResetFormRef ref )
{ return (CFArrayRef)[ref fields]; }

void PDFActionResetFormSetFields( PDFActionResetFormRef ref, CFArrayRef fields )
{ [ref setFields:(NSArray *)fields]; }

// - fields are cleared -
Boolean PDFActionResetFormFieldsIncludedAreCleared( PDFActionResetFormRef ref )
{ return [ref fieldsIncludedAreCleared]; }

void PDFActionResetFormSetFieldsIncludedAreCleared( PDFActionResetFormRef ref, Boolean flag )
{ [ref setFieldsIncludedAreCleared:flag]; }


#pragma mark - PDFActionURL
///////////////////////////////////////////////////
//                 PDFActionURL                  //
///////////////////////////////////////////////////
// - init -
PDFActionURLRef PDFActionURLWithURL( CFURLRef url )
{ return [[[PDFActionURL alloc] initWithURL:(NSURL *)url] autorelease]; }

// - url -
CFURLRef PDFActionURLURL( PDFActionURLRef ref )
{ return (CFURLRef)[ref URL]; }
    
void PDFActionURLSetURL( PDFActionURLRef ref, CFURLRef url )
{ [ref setURL:(NSURL *)url]; }


#pragma mark - PDFAnnotation
///////////////////////////////////////////////////
//              PDFAnnotation                    //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
// - init -
PDFAnnotationRef PDFAnnotationWithBounds( CGRect bounds, CFStringRef type, CFDictionaryRef properties )
{ return [[[PDFAnnotation alloc] initWithBounds:NSRectFromCGRect(bounds) forType:(NSString *)type withProperties:(NSDictionary *)properties] autorelease]; }
#endif

// - info -
PDFPageRef PDFAnnotationPage( PDFAnnotationRef ref )
{ return [ref page]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
void PDFAnnotationSetPage( PDFAnnotationRef ref, PDFPageRef pg )
{ [ref setPage:pg]; }
#endif

CFDateRef PDFAnnotationModificationDate( PDFAnnotationRef ref )
{ return (CFDateRef)[ref modificationDate]; }

void PDFAnnotationSetModificationDate( PDFAnnotationRef ref, CFDateRef dt )
{ [ref setModificationDate:(NSDate *)dt]; }

CFStringRef PDFAnnotationUserName( PDFAnnotationRef ref )
{ return (CFStringRef)[ref userName]; }

void PDFAnnotationSetUserName( PDFAnnotationRef ref, CFStringRef name )
{ [ref setUserName:(NSString *)name]; }

CFStringRef PDFAnnotationType( PDFAnnotationRef ref )
{ return (CFStringRef)[ref type]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void PDFAnnotationSetType( PDFAnnotationRef ref, CFStringRef type )
{ [ref setType:(NSString *)type]; }
#endif

CFStringRef PDFAnnotationContents( PDFAnnotationRef ref )
{ return (CFStringRef)[ref contents]; }

void PDFAnnotationSetContents( PDFAnnotationRef ref, CFStringRef contents )
{ [ref setContents:(NSString *)contents]; }

// - characteristics -
CGRect PDFAnnotationBounds( PDFAnnotationRef ref )
{ return NSRectToCGRect([ref bounds]); }

void PDFAnnotationSetBounds( PDFAnnotationRef ref, CGRect bounds )
{ [ref setBounds:NSRectFromCGRect(bounds)]; }

PDFBorderRef PDFAnnotationBorder( PDFAnnotationRef ref )
{ return [ref border]; }

void PDFAnnotationSetBorder( PDFAnnotationRef ref, PDFBorderRef border )
{ [ref setBorder:border]; }

ColorRef PDFAnnotationColor( PDFAnnotationRef ref )
{ return [ref color]; }

void PDFAnnotationSetColor( PDFAnnotationRef ref, ColorRef col )
{ [ref setColor:col]; }

Boolean PDFAnnotationHasAppearanceStream( PDFAnnotationRef ref )
{ return [ref hasAppearanceStream]; }

// - drawing and output -
Boolean PDFAnnotationShouldDisplay( PDFAnnotationRef ref )
{ return [ref shouldDisplay]; }

void PDFAnnotationSetShouldDisplay( PDFAnnotationRef ref, Boolean flag )
{ [ref setShouldDisplay:flag]; }

Boolean PDFAnnotationShouldPrint( PDFAnnotationRef ref )
{ return [ref shouldPrint]; }

void PDFAnnotationSetShouldPrint( PDFAnnotationRef ref, Boolean flag )
{ [ref setShouldPrint:flag]; }

// - instance properties -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
PDFActionRef PDFAnnotationAction( PDFAnnotationRef ref )// macOS 10.5+
{ return [ref action]; }

void PDFAnnotationSetAction( PDFAnnotationRef ref, PDFActionRef action )// macOS 10.5+
{ [ref setAction:action]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
NSTextAlignment PDFAnnotationAlignment( PDFAnnotationRef ref )
{ return [ref alignment]; }

void PDFAnnotationSetAlignment( PDFAnnotationRef ref, NSTextAlignment alignment )
{ [ref setAlignment:alignment]; }

Boolean PDFAnnotationAllowsToggleToOff( PDFAnnotationRef ref )
{ return [ref allowsToggleToOff]; }

void PDFAnnotationSetAllowsToggleToOff( PDFAnnotationRef ref, Boolean flag )
{ [ref setAllowsToggleToOff:flag]; }

CFDictionaryRef PDFAnnotationKeyValues( PDFAnnotationRef ref )
{ return (CFDictionaryRef)[ref annotationKeyValues]; }

ColorRef PDFAnnotationBackgroundColor( PDFAnnotationRef ref )
{ return [ref backgroundColor]; }

void PDFAnnotationSetBackgroundColor( PDFAnnotationRef ref, ColorRef col )
{ [ref setBackgroundColor:col]; }

PDFWidgetCellState PDFAnnotationButtonWidgetState( PDFAnnotationRef ref )
{ return [ref buttonWidgetState]; }

void PDFAnnotationSetButtonWidgetState( PDFAnnotationRef ref, PDFWidgetCellState state )
{ [ref setButtonWidgetState:state]; }

CFStringRef PDFAnnotationButtonWidgetStateString( PDFAnnotationRef ref )
{ return (CFStringRef)[ref buttonWidgetStateString]; }

void PDFAnnotationSetButtonWidgetStateString( PDFAnnotationRef ref, CFStringRef string )
{ [ref setButtonWidgetStateString:(NSString *)string]; }

CFStringRef PDFAnnotationCaption( PDFAnnotationRef ref )
{ return (CFStringRef)[ref caption]; }

void PDFAnnotationSetCaption( PDFAnnotationRef ref, CFStringRef caption )
{ [ref setCaption:(NSString *)caption]; }

CFArrayRef PDFAnnotationChoices( PDFAnnotationRef ref )
{ return (CFArrayRef)[ref choices]; }

void PDFAnnotationSetChoices( PDFAnnotationRef ref, CFArrayRef choices )
{ [ref setChoices:(NSArray *)choices]; }

PDFDestinationRef PDFAnnotationDestination( PDFAnnotationRef ref )
{ return [ref destination]; }

void PDFAnnotationSetDestination( PDFAnnotationRef ref, PDFDestinationRef dest )
{ [ref setDestination:dest]; }

PDFLineStyle PDFAnnotationEndLineStyle( PDFAnnotationRef ref )
{ return [ref endLineStyle]; }

void PDFAnnotationSetEndLineStyle( PDFAnnotationRef ref, PDFLineStyle style )
{ [ref setEndLineStyle:style]; }

CGPoint PDFAnnotationEndPoint( PDFAnnotationRef ref )
{ return NSPointToCGPoint([ref endPoint]); }

void PDFAnnotationSetEndPoint( PDFAnnotationRef ref, CGPoint pt )
{ [ref setEndPoint:NSPointFromCGPoint(pt)]; }

CFStringRef PDFAnnotationFieldName( PDFAnnotationRef ref )
{ return (CFStringRef)[ref fieldName]; }

void PDFAnnotationSetFieldName( PDFAnnotationRef ref, CFStringRef name )
{ [ref setFieldName:(NSString *)name]; }

FontRef PDFAnnotationFont( PDFAnnotationRef ref )
{ return [ref font]; }

void PDFAnnotationSetFont( PDFAnnotationRef ref, FontRef font )
{ [ref setFont:font]; }

ColorRef PDFAnnotationFontColor( PDFAnnotationRef ref )
{ return [ref fontColor]; }

void PDFAnnotationSetFontColor( PDFAnnotationRef ref, ColorRef col )
{ [ref setFontColor:col]; }

Boolean PDFAnnotationHasComb( PDFAnnotationRef ref )
{ return [ref hasComb]; }

void PDFAnnotationSetComb( PDFAnnotationRef ref, Boolean flag )
{ [ref setComb:flag]; }

PDFTextAnnotationIconType PDFAnnotationIconType( PDFAnnotationRef ref )
{ return [ref iconType]; }

void PDFAnnotationSetIconType( PDFAnnotationRef ref, PDFTextAnnotationIconType type )
{ [ref setIconType:type]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
ColorRef PDFAnnotationInteriorColor( PDFAnnotationRef ref )// macOS 10.13.2+
{ return [ref interiorColor]; }

void PDFAnnotationSetInteriorColor( PDFAnnotationRef ref, ColorRef col )// macOS 10.13.2+
{ [ref setInteriorColor:col]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean PDFAnnotationIsHighlighted( PDFAnnotationRef ref )
{ return [ref isHighlighted]; }

void PDFAnnotationSetHighlighted( PDFAnnotationRef ref, Boolean flag )
{ [ref setHighlighted:flag]; }

Boolean PDFAnnotationIsListChoice( PDFAnnotationRef ref )
{ return [ref isListChoice]; }

void PDFAnnotationSetListChoice( PDFAnnotationRef ref, Boolean flag )
{ [ref setListChoice:flag]; }

Boolean PDFAnnotationIsMultiline( PDFAnnotationRef ref )
{ return [ref isMultiline]; }

void PDFAnnotationSetMultiline( PDFAnnotationRef ref, Boolean flag )
{ [ref setMultiline:flag]; }

Boolean PDFAnnotationIsOpen( PDFAnnotationRef ref )
{ return [ref isOpen]; }

void PDFAnnotationSetOpen( PDFAnnotationRef ref, Boolean flag )
{ [ref setOpen:flag]; }

Boolean PDFAnnotationIsPasswordField( PDFAnnotationRef ref )
{ return [ref isPasswordField]; }

Boolean PDFAnnotationIsReadOnly( PDFAnnotationRef ref )
{ return [ref isReadOnly]; }

void PDFAnnotationSetReadOnly( PDFAnnotationRef ref, Boolean flag )
{ [ref setReadOnly:flag]; }

PDFMarkupType PDFAnnotationMarkupType( PDFAnnotationRef ref )
{ return [ref markupType]; }

void PDFAnnotationSetMarkupType( PDFAnnotationRef ref, PDFMarkupType type )
{ [ref setMarkupType:type]; }

NSInteger PDFAnnotationMaximumLength( PDFAnnotationRef ref )
{ return [ref maximumLength]; }

void PDFAnnotationSetMaximumLength( PDFAnnotationRef ref, NSInteger length )
{ [ref setMaximumLength:length]; }

CFArrayRef PDFAnnotationPaths( PDFAnnotationRef ref )
{ return (CFArrayRef)[ref paths]; }

CFArrayRef PDFAnnotationQuadrilateralPoints( PDFAnnotationRef ref )
{ return (CFArrayRef)[ref quadrilateralPoints]; }

void PDFAnnotationSetQuadrilateralPoints( PDFAnnotationRef ref, CFArrayRef points )
{ [ref setQuadrilateralPoints:(NSArray *)points]; }

Boolean PDFAnnotationRadiosInUnison( PDFAnnotationRef ref )
{ return [ref radiosInUnison]; }

void PDFAnnotationSetRadiosInUnison( PDFAnnotationRef ref, Boolean flag )
{ [ref setRadiosInUnison:flag]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFStringRef PDFAnnotationStampName( PDFAnnotationRef ref )// macOS 10.13.2+
{ return (CFStringRef)[ref stampName]; }

void PDFAnnotationSetStampName( PDFAnnotationRef ref, CFStringRef name )// macOS 10.13.2+
{ [ref setStampName:(NSString *)name]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
PDFLineStyle PDFAnnotationStartLineStyle( PDFAnnotationRef ref )
{ return [ref startLineStyle]; }

void PDFAnnotationSetStartLineStyle( PDFAnnotationRef ref, PDFLineStyle style )
{ [ref setStartLineStyle:style]; }

CGPoint PDFAnnotationStartPoint( PDFAnnotationRef ref )
{ return NSPointToCGPoint([ref startPoint]); }

void PDFAnnotationSetStartPoint( PDFAnnotationRef ref, CGPoint pt )
{ [ref setStartPoint:NSPointFromCGPoint(pt)]; }

CFURLRef PDFAnnotationURL( PDFAnnotationRef ref )
{ return (CFURLRef)[ref URL]; }

void PDFAnnotationSetURL( PDFAnnotationRef ref, CFURLRef url )
{ [ref setURL:(NSURL *)url]; }

CFArrayRef PDFAnnotationValues( PDFAnnotationRef ref )
{ return (CFArrayRef)[ref values]; }

void PDFAnnotationSetValues( PDFAnnotationRef ref, CFArrayRef values )
{ [ref setValues:(NSArray *)values]; }

PDFWidgetControlType PDFAnnotationWidgetControlType( PDFAnnotationRef ref )
{ return [ref widgetControlType]; }

void PDFAnnotationSetWidgetControlType( PDFAnnotationRef ref, PDFWidgetControlType type )
{ [ref setWidgetControlType:type]; }

CFStringRef PDFAnnotationWidgetDefaultStringValue( PDFAnnotationRef ref )
{ return (CFStringRef)[ref widgetDefaultStringValue]; }

void PDFAnnotationSetWidgetDefaultStringValue( PDFAnnotationRef ref, CFStringRef string )
{ [ref setWidgetDefaultStringValue:(NSString *)string]; }

CFStringRef PDFAnnotationWidgetFieldType( PDFAnnotationRef ref )
{ return (CFStringRef)[ref widgetFieldType]; }

void PDFAnnotationSetWidgetFieldType( PDFAnnotationRef ref, CFStringRef type )
{ [ref setWidgetFieldType:(NSString *)type]; }

CFStringRef PDFAnnotationWidgetStringValue( PDFAnnotationRef ref )
{ return (CFStringRef)[ref widgetStringValue]; }

void PDFAnnotationSetWidgetStringValue( PDFAnnotationRef ref, CFStringRef string )
{ [ref setWidgetStringValue:(NSString *)string]; }
#endif

// - instance methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void PDFAnnotationAddBezierPath( PDFAnnotationRef ref, BezierPathRef path )// macOS 10.13+
{ [ref addBezierPath:path]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
void PDFAnnotationDrawWithBox( PDFAnnotationRef ref, PDFDisplayBox bx, CGContextRef context )
{ [ref drawWithBox:bx inContext:context]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void PDFAnnotationRemoveBezierPath( PDFAnnotationRef ref, BezierPathRef path )// macOS 10.13+
{ [ref removeBezierPath:path]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
void PDFAnnotationRemoveValueForKey( PDFAnnotationRef ref, CFStringRef key )
{ [ref removeValueForAnnotationKey:(NSString *)key]; }

void PDFAnnotationSetBooleanForKey( PDFAnnotationRef ref, Boolean flag, CFStringRef key )
{ [ref setBoolean:flag forAnnotationKey:(NSString *)key]; }

void PDFAnnotationSetRectForKey( PDFAnnotationRef ref, CGRect r, CFStringRef key )
{ [ref setRect:NSRectFromCGRect(r) forAnnotationKey:(NSString *)key]; }

void PDFAnnotationSetValueForKey( PDFAnnotationRef ref, CFTypeRef value, CFStringRef key )
{ [ref setValue:(id)value forAnnotationKey:(NSString *)key]; }

CFTypeRef PDFAnnotationValueForKey( PDFAnnotationRef ref, CFStringRef key )
{ return (CFTypeRef)[ref valueForAnnotationKey:(NSString *)key]; }
#endif

// - type methods -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
PDFLineStyle PDFAnnotationLineStyleFromName( CFStringRef name )// macOS 10.13+
{ return [PDFAnnotation lineStyleFromName:(NSString *)name]; }

CFStringRef PDFAnnotationNameForLineStyle( PDFLineStyle style )// macOS 10.13+
{ return (CFStringRef)[PDFAnnotation nameForLineStyle:style]; }
#endif


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
#pragma mark - PDFAppearanceCharacteristics
///////////////////////////////////////////////////
//      PDFAppearanceCharacteristics             //
///////////////////////////////////////////////////
// - instance properties -
CFDictionaryRef PDFAppearanceCharacteristicsKeyValues( PDFAppearanceCharacteristicsRef ref )
{ return (CFDictionaryRef)[ref appearanceCharacteristicsKeyValues]; }

ColorRef PDFAppearanceCharacteristicsBackgroundColor( PDFAppearanceCharacteristicsRef ref )
{ return [ref backgroundColor]; }

void PDFAppearanceCharacteristicsSetBackgroundColor( PDFAppearanceCharacteristicsRef ref, ColorRef col )
{ [ref setBackgroundColor:col]; }

ColorRef PDFAppearanceCharacteristicsBorderColor( PDFAppearanceCharacteristicsRef ref )
{ return [ref borderColor]; }

void PDFAppearanceCharacteristicsSetBorderColor( PDFAppearanceCharacteristicsRef ref, ColorRef col )
{ [ref setBorderColor:col]; }

CFStringRef PDFAppearanceCharacteristicsCaption( PDFAppearanceCharacteristicsRef ref )
{ return (CFStringRef)[ref caption]; }

void PDFAppearanceCharacteristicsSetCaption( PDFAppearanceCharacteristicsRef ref, CFStringRef caption )
{ [ref setCaption:(NSString *)caption]; }

PDFWidgetControlType PDFAppearanceCharacteristicsControlType( PDFAppearanceCharacteristicsRef ref )
{ return [ref controlType]; }

void PDFAppearanceCharacteristicsSetControlType( PDFAppearanceCharacteristicsRef ref, PDFWidgetControlType type )
{ [ref setControlType:type]; }

CFStringRef PDFAppearanceCharacteristicsDownCaption( PDFAppearanceCharacteristicsRef ref )
{ return (CFStringRef)[ref downCaption]; }

void PDFAppearanceCharacteristicsSetDownCaption( PDFAppearanceCharacteristicsRef ref, CFStringRef caption )
{ [ref setDownCaption:(NSString *)caption]; }

CFStringRef PDFAppearanceCharacteristicsRolloverCaption( PDFAppearanceCharacteristicsRef ref )
{ return (CFStringRef)[ref rolloverCaption]; }

void PDFAppearanceCharacteristicsSetRolloverCaption( PDFAppearanceCharacteristicsRef ref, CFStringRef caption )
{ [ref setRolloverCaption:(NSString *)caption]; }

NSInteger PDFAppearanceCharacteristicsRotation( PDFAppearanceCharacteristicsRef ref )
{ return [ref rotation]; }

void PDFAppearanceCharacteristicsSetRotation( PDFAppearanceCharacteristicsRef ref, NSInteger rotation )
{ [ref setRotation:rotation]; }
#endif


#pragma mark - PDFBorder
///////////////////////////////////////////////////
//                 PDFBorder                     //
///////////////////////////////////////////////////
// - init -
PDFBorderRef PDFBorderInit( void )
{ return [[[PDFBorder alloc] init] autorelease]; }

// - styles and characteristics -
NSInteger PDFBorderstyle( PDFBorderRef ref )
{ return [ref style]; }

void PDFBorderSetStyle( PDFBorderRef ref, NSInteger style )
{ [ref setStyle:style]; }

CGFloat PDFBorderLineWidth( PDFBorderRef ref )
{ return [ref lineWidth]; }

void PDFBorderSetLineWidth( PDFBorderRef ref, CGFloat lineWidth )
{ [ref setLineWidth:lineWidth]; }

CFArrayRef PDFBorderDashPattern( PDFBorderRef ref )
{ return (CFArrayRef)[ref dashPattern]; }

void PDFBorderSetDashPattern( PDFBorderRef ref, CFArrayRef pattern )
{ [ref setDashPattern:(NSArray *)pattern]; }

// - drawing -
void PDFBorderDrawInRect( PDFBorderRef ref, CGRect r )
{ [ref drawInRect:NSRectFromCGRect(r)]; }

// - instance properties -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFDictionaryRef PDFBorderKeyValues( PDFBorderRef ref )
{ return (CFDictionaryRef)[ref borderKeyValues]; }
#endif


#pragma mark - PDFDestination
///////////////////////////////////////////////////
//               PDFDestination                  //
///////////////////////////////////////////////////
// - init -
PDFDestinationRef PDFDestinationWithPage( PDFPageRef pg, CGPoint pt )
{ return [[[PDFDestination alloc] initWithPage:pg atPoint:NSPointFromCGPoint(pt)] autorelease]; }

// - pages and point -
PDFPageRef PDFDestinationPage( PDFDestinationRef dest )
{ return [dest page]; }
          
CGPoint PDFDestinationPoint( PDFDestinationRef dest )
{ return NSPointToCGPoint([dest point]); }

// - relative location -
NSComparisonResult PDFDestinationCompare( PDFDestinationRef dest1, PDFDestinationRef dest2 )
{ return [dest1 compare:dest2]; }

// - instance properties -
CGFloat PDFDestinationZoom( PDFDestinationRef dest )
{ return [dest zoom]; }

void PDFDestinationSetZoom( PDFDestinationRef dest, CGFloat value )
{ [dest setZoom:value]; }


#pragma mark - PDFDocument
///////////////////////////////////////////////////
//              PDFDocument                      //
///////////////////////////////////////////////////
// - init -
PDFDocumentRef PDFDocumentWithURL( CFURLRef url )
{ return [[[PDFDocument alloc] initWithURL:(NSURL *)url] autorelease]; }

PDFDocumentRef PDFDocumentWithData( CFDataRef dta )
{ return [[[PDFDocument alloc] initWithData:(NSData *)dta] autorelease]; }

PDFDocumentRef PDFDocumentInit( void )
{ return [[[PDFDocument alloc] init] autorelease]; }

// - read and write -
// -- read operations --
// --- info ---
CFURLRef PDFDocumentURL( PDFDocumentRef ref )
{ return (CFURLRef)[ref documentURL]; }

NSInteger PDFDocumentMajorVersion( PDFDocumentRef ref )
{ return [ref majorVersion]; }

NSInteger PDFDocumentMinorVersion( PDFDocumentRef ref )
{ return [ref minorVersion]; }

CFStringRef PDFDocumentString( PDFDocumentRef ref )
{ return (CFStringRef)[ref string]; }

PDFOutlineRef PDFDocumentOutlineItemForSelection( PDFDocumentRef docRef, PDFSelectionRef selRef )
{ return [docRef outlineItemForSelection:selRef]; }

PDFOutlineRef PDFDocumentOutlineRoot( PDFDocumentRef ref )
{ return [ref outlineRoot]; }

void PDFDocumentSetOutlineRoot( PDFDocumentRef docRef, PDFOutlineRef olRef )
{ [docRef setOutlineRoot:olRef]; }

CFDictionaryRef PDFDocumentAttributes( PDFDocumentRef ref )
{ return (CFDictionaryRef)[ref documentAttributes]; }

void PDFDocumentSetAttributes( PDFDocumentRef ref, CFDictionaryRef attributes )
{ [ref setDocumentAttributes:(NSDictionary *)attributes]; }

CGPDFDocumentRef PDFDocumentCGPDFRef( PDFDocumentRef ref )
{ return [ref documentRef]; }

// --- security ---
Boolean PDFDocumentIsEncrypted( PDFDocumentRef ref )
{ return [ref isEncrypted]; }

Boolean PDFDocumentIsLocked( PDFDocumentRef ref )
{ return [ref isLocked]; }

Boolean PDFDocumentUnlockWithPassword( PDFDocumentRef ref, CFStringRef password )
{ return [ref unlockWithPassword:(NSString *)password]; }

PDFDocumentPermissions PDFDocumentPermissionsStatus( PDFDocumentRef ref )
{ return [ref permissionsStatus]; }

// ---- permissions properties ----
Boolean PDFDocumentAllowsCopying( PDFDocumentRef ref )
{ return [ref allowsCopying]; }

Boolean PDFDocumentAllowsPrinting( PDFDocumentRef ref )
{ return [ref allowsPrinting]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean PDFDocumentAllowsCommenting( PDFDocumentRef ref )
{ return [ref allowsCommenting]; }

Boolean PDFDocumentAllowsContentAccessibility( PDFDocumentRef ref )
{ return [ref allowsContentAccessibility]; }

Boolean PDFDocumentAllowsDocumentAssembly( PDFDocumentRef ref )
{ return [ref allowsDocumentAssembly]; }

Boolean PDFDocumentAllowsDocumentChanges( PDFDocumentRef ref )
{ return [ref allowsDocumentChanges]; }

Boolean PDFDocumentAllowsFormFieldEntry( PDFDocumentRef ref )
{ return [ref allowsFormFieldEntry]; }
#endif

// --- selections and searches ---
PDFSelectionRef PDFDocumentSelectionFromPageAtIndex( PDFDocumentRef ref, PDFPageRef startPage, NSUInteger startCharIndex, PDFPageRef endPage, NSUInteger endCharIndex )
{ return [ref selectionFromPage:startPage atCharacterIndex:startCharIndex toPage:endPage atCharacterIndex:endCharIndex]; }

PDFSelectionRef PDFDocumentSelectionFromPageAtPoint( PDFDocumentRef ref, PDFPageRef startPage, CGPoint startPt, PDFPageRef endPage, CGPoint endPt )
{ return [ref selectionFromPage:startPage atPoint:NSPointFromCGPoint(startPt) toPage:endPage atPoint:NSPointFromCGPoint(endPt)]; }

PDFSelectionRef PDFDocumentSelectionForEntireDocument( PDFDocumentRef ref )
{ return [ref selectionForEntireDocument]; }

// ---- search options ----
CFArrayRef PDFDocumentFindString( PDFDocumentRef ref, CFStringRef string, NSStringCompareOptions options )
{ return (CFArrayRef)[ref findString:(NSString *)string withOptions:options]; }

//void PDFDocumentBeginFindString( PDFDocumentRef ref, CFStringRef string, NSStringCompareOptions options );

//void PDFDocumentBeginFindStrings( PDFDocumentRef ref, CFArrayRef strings, NSStringCompareOptions options );

PDFSelectionRef PDFDocumentFindStringFromSelection( PDFDocumentRef ref, CFStringRef string, PDFSelectionRef selection, NSStringCompareOptions options )
{ return [ref findString:(NSString *)string fromSelection:selection withOptions:options]; }

Boolean PDFDocumentIsFinding( PDFDocumentRef ref )
{ return [ref isFinding]; }

void PDFDocumentCancelFindString( PDFDocumentRef ref )
{ [ref cancelFindString]; }

// --- pages ---
NSUInteger PDFDocumentPageCount( PDFDocumentRef ref )
{ return [ref pageCount]; }

PDFPageRef PDFDocumentPageAtIndex( PDFDocumentRef ref, NSUInteger index )
{ return [ref pageAtIndex:index]; }

NSUInteger PDFDocumentIndexForPage( PDFDocumentRef ref, PDFPageRef pg )
{ return [ref indexForPage:pg]; }

void PDFDocumentInsertPageAtIndex( PDFDocumentRef ref, PDFPageRef pg, NSUInteger index )
{ [ref insertPage:pg atIndex:index]; }

void PDFDocumentRemovePageAtIndex( PDFDocumentRef ref, NSUInteger index )
{ [ref removePageAtIndex:index]; }

void PDFDocumentExchangePageAtIndex( PDFDocumentRef ref, NSUInteger index1, NSUInteger index2 )
{ [ref exchangePageAtIndex:index1 withPageAtIndex:index2]; }

ClassRef PDFDocumentPageClass( PDFDocumentRef ref )
{ return [ref pageClass]; }

// -- write operations --
// --- data ---
Boolean PDFDocumentWriteToURL( PDFDocumentRef ref, CFURLRef url )
{ return [ref writeToURL:(NSURL *)url]; }

Boolean PDFDocumentWriteToURLWithOptions( PDFDocumentRef ref, CFURLRef url, CFDictionaryRef options )
{ return [ref writeToURL:(NSURL *)url withOptions:(NSDictionary *)options]; }

// ---- data representations ----
CFDataRef PDFDocumentDataRepresentation( PDFDocumentRef ref )
{ return (CFDataRef)[ref dataRepresentation]; }

CFDataRef PDFDocumentDataRepresentationWithOptions( PDFDocumentRef ref, CFDictionaryRef options )
{ return (CFDataRef)[ref dataRepresentationWithOptions:(NSDictionary *)options]; }

// --- printing ---
PrintOperationRef PDFDocumentPrintOperationForPrintInfo( PDFDocumentRef ref, PrintInfoRef info, PDFPrintScalingMode scalingMode, Boolean autorotate )
{ return [ref printOperationForPrintInfo:info scalingMode:scalingMode autoRotate:autorotate]; }

// - delegate -


#pragma mark - PDFOutline
///////////////////////////////////////////////////
//                   PDFOutline                  //
///////////////////////////////////////////////////
// - init -
PDFOutlineRef PDFOutlineInit( void )
{ return [[[PDFOutline alloc] init] autorelease]; }

// - info -
PDFDocumentRef PDFOutlineDocument( PDFOutlineRef ref )
{ return [ref document]; }

NSUInteger PDFOutlineNumberOfChildren( PDFOutlineRef ref )
{ return [ref numberOfChildren]; }

PDFOutlineRef PDFOutlineParent( PDFOutlineRef ref )
{ return [ref parent]; }

PDFOutlineRef PDFOutlineChildAtIndex( PDFOutlineRef ref, NSUInteger index )
{ return [ref childAtIndex:index]; }

NSUInteger PDFOutlineIndex( PDFOutlineRef ref )
{ return [ ref index]; }

// - label -
CFStringRef PDFOutlineLabel( PDFOutlineRef ref )
{ return (CFStringRef)[ref label]; }

void PDFOutlineSetLabel( PDFOutlineRef ref, CFStringRef label )
{ [ref setLabel:(NSString *)label]; }

// - actions and destinations -
PDFDestinationRef PDFOutlineDestination( PDFOutlineRef ref )
{ return [ref destination]; }

void PDFOutlineSetDestination( PDFOutlineRef ref, PDFDestinationRef dest )
{ [ref setDestination:dest]; }

PDFActionRef PDFOutlineAction( PDFOutlineRef ref )
{ return [ref action]; }

void PDFOutlineSetAction( PDFOutlineRef ref, PDFActionRef action )
{ [ref setAction:action]; }

// - hierarchy -
void PDFOutlineInsertChildAtIndex( PDFOutlineRef ref, PDFOutlineRef childRef, NSUInteger index )
{ [ref insertChild:childRef atIndex:index]; }

void PDFOutlineRemoveFromParent( PDFOutlineRef ref )
{ [ref removeFromParent]; }

// - disclosure -
Boolean PDFOutlineIsOpen( PDFOutlineRef ref )
{ return [ref isOpen]; }

void PDFOutlineSetIsOpen( PDFOutlineRef ref, Boolean flag )
{ return [ref setIsOpen:flag]; }


#pragma mark - PDFPage
///////////////////////////////////////////////////
//                  PDFPage                      //
///////////////////////////////////////////////////
// -- init --
PDFPageRef PDFPageWithImage( ImageRef image )
{ return [[[PDFPage alloc] initWithImage:image] autorelease]; }

// -- info --
PDFDocumentRef PDFPageDocument( PDFPageRef pg )
{ return [pg document]; }

CFStringRef PDFPageLabel( PDFPageRef pg )
{ return (CFStringRef)[pg label]; }

CGRect PDFPageBoundsForBox( PDFPageRef pg, PDFDisplayBox bx )
{ return NSRectToCGRect([pg boundsForBox:bx]); }

void PDFPageSetBoundsForBox( PDFPageRef pg, CGRect bounds, PDFDisplayBox bx )
{ [pg setBounds:NSRectFromCGRect(bounds) forBox:bx]; }

NSInteger PDFPageRotation( PDFPageRef pg )
{ return [pg rotation]; }

void PDFPageSetRotation( PDFPageRef pg, NSInteger rotation )
{ [pg setRotation:rotation]; }

// -- annotations --
CFArrayRef PDFPageAnnotations( PDFPageRef pg )
{ return (CFArrayRef)[pg annotations]; }

Boolean PDFPageDisplaysAnnotations( PDFPageRef pg )
{ return [pg displaysAnnotations]; }

void PDFPageSetDisplaysAnnotations( PDFPageRef pg, Boolean flag )
{ [pg setDisplaysAnnotations:flag]; }

void PDFPageAddAnnotation( PDFPageRef pg, PDFAnnotationRef annotation )
{ [pg addAnnotation:annotation]; }

void PDFPageRemoveAnnotation( PDFPageRef pg, PDFAnnotationRef annotation )
{ [pg removeAnnotation:annotation]; }

PDFAnnotationRef PDFPageAnnotationAtPoint( PDFPageRef pg, CGPoint pt )
{ return [pg annotationAtPoint:NSPointFromCGPoint(pt)]; }

// -- textual content --
NSUInteger PDFPageNumberOfCharacters( PDFPageRef pg )
{ return [pg numberOfCharacters]; }

CFStringRef PDFPageString( PDFPageRef pg )
{ return (CFStringRef)[pg string]; }

CFAttributedStringRef PDFPageAttributedString( PDFPageRef pg )
{ return ( CFAttributedStringRef)[pg attributedString]; }

CGRect PDFPageCharacterBoundsAtIndex( PDFPageRef pg, NSInteger index )
{ return NSRectToCGRect([pg characterBoundsAtIndex:index]); }

NSInteger PDFPageCharacterIndexAtPoint( PDFPageRef pg, CGPoint pt )
{ return [pg characterIndexAtPoint:NSPointFromCGPoint(pt)]; }

// -- selections --
PDFSelectionRef PDFPageSelectionForRect( PDFPageRef pg, CGRect r )
{ return [pg selectionForRect:NSRectFromCGRect(r)]; }

PDFSelectionRef PDFPageSelectionForWordAtPoint( PDFPageRef pg, CGPoint pt )
{ return [pg selectionForWordAtPoint:NSPointFromCGPoint(pt)]; }

PDFSelectionRef PDFPageSelectionForLineAtPoint( PDFPageRef pg, CGPoint pt )
{ return [pg selectionForLineAtPoint:NSPointFromCGPoint(pt)]; }

PDFSelectionRef PDFPageSelectionFromPointToPoint( PDFPageRef pg, CGPoint startPt, CGPoint endPt )
{ return [pg selectionFromPoint:NSPointFromCGPoint(startPt) toPoint:NSPointFromCGPoint(endPt)]; }

PDFSelectionRef PDFPageSelectionForRange( PDFPageRef pg, CFRange range )
{ return [pg selectionForRange:NSMakeRange(range.location,range.length)]; }

// -- instance properties --
CFDataRef PDFPageDataRepresentation( PDFPageRef pg )
{ return (CFDataRef)[pg dataRepresentation]; }

CGPDFPageRef PDFPagePageRef( PDFPageRef pg )
{ return [pg pageRef]; }

// -- instance methods --
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
void PDFPageDrawWithBoxToContext( PDFPageRef pg, PDFDisplayBox bx, CGContextRef context )
{ [pg drawWithBox:bx toContext:context]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
ImageRef PDFPageThumbnailOfSizeForBox( PDFPageRef pg, CGSize size, PDFDisplayBox bx )
{ return [pg thumbnailOfSize:NSSizeFromCGSize(size) forBox:bx]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
void PDFPageTransformContextForBox( PDFPageRef pg, CGContextRef context, PDFDisplayBox bx )
{ [pg transformContext:context forBox:bx]; }

CGAffineTransform PDFPageTransformForBox( PDFPageRef pg, PDFDisplayBox bx )
{ return [pg transformForBox:bx]; }
#endif


#pragma mark - PDFSelection
///////////////////////////////////////////////////
//                PDFSelection                   //
///////////////////////////////////////////////////
// - init -
PDFSelectionRef PDFSelectionWithDocument( PDFDocumentRef doc )
{ return [[[PDFSelection alloc] initWithDocument:doc] autorelease]; }

// - info -
CFArrayRef PDFSelectionPages( PDFSelectionRef ref )
{ return (CFArrayRef)[ref pages]; }

CFStringRef PDFSelectionString( PDFSelectionRef ref )
{ return (CFStringRef)[ref string]; }

CFAttributedStringRef PDFSelectionAttributedString( PDFSelectionRef ref )
{ return (CFAttributedStringRef)[ref attributedString]; }

CGRect PDFSelectionBoundsForPage( PDFSelectionRef ref, PDFPageRef pg )
{ return NSRectToCGRect([ref boundsForPage:pg]); }

CFArrayRef PDFSelectionSelectionsByLine( PDFSelectionRef ref )
{ return (CFArrayRef)[ref selectionsByLine]; }

ColorRef PDFSelectionColor( PDFSelectionRef ref )
{ return [ref color]; }

void PDFSelectionSetColor( PDFSelectionRef ref, ColorRef col )
{ [ref setColor:col]; }

// - modify -
void PDFSelectionAddSelection( PDFSelectionRef ref, PDFSelectionRef sel )
{ [ref addSelection:sel]; }

void PDFSelectionAddSelections( PDFSelectionRef ref, CFArrayRef selections )
{ [ref addSelections:(NSArray *)selections]; }

void PDFSelectionExtendSelectionAtEnd( PDFSelectionRef ref, NSInteger succeed )
{ [ref extendSelectionAtEnd:succeed]; }

void PDFSelectionExtendSelectionAtStart( PDFSelectionRef ref, NSInteger precede )
{ [ref extendSelectionAtStart:precede]; }

// - drawing -
void PDFSelectionDrawForPage( PDFSelectionRef ref, PDFPageRef pg, Boolean active )
{ [ref drawForPage:pg active:active]; }

void PDFSelectionDrawForPageWithBox( PDFSelectionRef ref, PDFPageRef pg, PDFDisplayBox bx, Boolean active )
{ [ref drawForPage:pg withBox:bx active:active]; }

// - instance methods -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void PDFSelectionExtendSelectionForLineBoundaries( PDFSelectionRef ref )
{ [ref extendSelectionForLineBoundaries]; }
#endif

NSUInteger PDFSelectionNumberOfTextRangesOnPage( PDFSelectionRef ref, PDFPageRef pg )
{ return [ref numberOfTextRangesOnPage:pg]; }

CFRange PDFSelectionRangeAtIndexOnPage( PDFSelectionRef ref, NSUInteger index, PDFPageRef pg )
{
    NSRange range = [ref rangeAtIndex:index onPage:pg];
    return CFRangeMake(range.location,range.length);
}


#pragma mark - PDFThumbnailView
///////////////////////////////////////////////////
//         PDFThumbnailView                      //
///////////////////////////////////////////////////
@implementation PDFThumbnailViewSC

@end

@implementation PDFThumbnailView (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

PDFThumbnailViewRef PDFThumbnailViewWithTag( NSInteger tag )
{
    PDFThumbnailView *view = (PDFThumbnailView *)CocoaViewWithTag( tag );
    if ( view ) {
        if ( [view isKindOfClass:[PDFThumbnailView class]] ) return view;
    }
    CocoaShowAlert( NSWarningAlertStyle, @"Error.", [NSString stringWithFormat:@"Can't find PDFThumbnailView with tag: %ld.",(long)tag] );
    return nil;
}

BOOL PDFThumbnailViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [PDFThumbnailView class] ); }


// -- PDFView --
PDFViewRef PDFThumbnailViewPDFView( NSInteger tag )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        return [view PDFView];
    }
    return nil;
}

void PDFThumbnailViewSetPDFView( NSInteger thumbTag, NSInteger pdfTag )
{
    PDFThumbnailViewRef thumbView = PDFThumbnailViewWithTag( thumbTag );
    if ( thumbView ) {
        PDFViewRef pdfView = PDFViewWithTag( pdfTag );
        if ( pdfView ) {
            [thumbView setPDFView:pdfView];
        }
    }
}

// -- size --
CGSize PDFThumbnailViewThumbnailSize( NSInteger tag )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        return NSSizeToCGSize( [view thumbnailSize] );
    }
    return CGSizeZero;
}

void PDFThumbnailViewSetThumbnailSize( NSInteger tag, CGSize size )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        [view setThumbnailSize:NSSizeFromCGSize(size)];
    }
}

// -- display --
NSUInteger PDFThumbnailViewMaxiumumNumberOfColumns( NSInteger tag )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        return [view maximumNumberOfColumns];
    }
    return 0;
}

void PDFThumbnailViewSetMaxiumumNumberOfColumns( NSInteger tag, NSUInteger columns )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        [view setMaximumNumberOfColumns:columns];
    }
}

FontRef PDFThumbnailViewLabelFont( NSInteger tag )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        return [view labelFont];
    }
    return nil;
}

void PDFThumbnailViewSetLabelFont( NSInteger tag, FontRef font )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        [view setLabelFont:font];
    }
}

ColorRef PDFThumbnailViewBackgroundColor( NSInteger tag )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        return [view backgroundColor];
    }
    return nil;
}

void PDFThumbnailViewSetBackgroundColor( NSInteger tag, ColorRef col )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        [view setBackgroundColor:col];
    }
}

// -- behaviour --
Boolean PDFThumbnailViewAllowsDragging( NSInteger tag )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        return [view allowsDragging];
    }
    return NO;
}

void PDFThumbnailViewSetAllowsDragging( NSInteger tag, Boolean flag )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        [view setAllowsDragging:flag];
    }
}

Boolean PDFThumbnailViewAllowsMultipleSelection( NSInteger tag )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        return [view allowsMultipleSelection];
    }
    return NO;
}

void PDFThumbnailViewSetMultipleSelection( NSInteger tag, Boolean flag )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        [view setAllowsMultipleSelection:flag];
    }
}

CFArrayRef PDFThumbnailViewSelectedPages( NSInteger tag )
{
    PDFThumbnailViewRef view = PDFThumbnailViewWithTag( tag );
    if ( view ) {
        return (CFArrayRef)[view selectedPages];
    }
    return nil;
}


// Init
PDFThumbnailViewRef PDFThumbnailViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) {
        rect = NSMakeRect( 20, 20, 75, 200 );
    }
    
    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    PDFThumbnailView *view;
    if ( subclass ) {
        view = [[[PDFThumbnailViewSC alloc] initWithFrame:rect] autorelease];
    } else {
        view = [[[PDFThumbnailView alloc] initWithFrame:rect] autorelease];
    }

    // tag
    [view setTag: inTag];
    
    return view;
}


// - FB statement -
void CocoaPDFThumbnailView( NSInteger inTag, CGRect inRect, NSInteger inPDFViewTag, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        
        PDFThumbnailView *view = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( view ) {   // already exists
            
            // make sure it's same view type
            if ( [view class] != [PDFThumbnailView class] ) return;
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [view setFrame:rect];
            
            // PDFView
            if ( inPDFViewTag ) {
                PDFView *pdfView = PDFViewWithTag( inPDFViewTag );
                if ( pdfView ) [view setPDFView:pdfView];
            }
            
        } else {
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) {
//                rect = NSMakeRect( 20, 20, 75, 200 );
//            }
            
            // init
//            if ( inSubclass ) {
//                view = [[PDFThumbnailViewSC alloc] initWithFrame:rect];
//            } else {
//                view = [[PDFThumbnailView alloc] initWithFrame:rect];
//            }
            
            view = PDFThumbnailViewInit( inTag, inRect );
            
            // default autoresizing
            [view setAutoresizingMask:NSViewMinYMargin];
            
            // tag
 //           [view setTag: inTag];
            
            // PDFView
            if ( inPDFViewTag ) {
                PDFView *pdfView = PDFViewWithTag( inPDFViewTag );
                if ( pdfView ) [view setPDFView:pdfView];
            }

            // wndTag
            [view setWndTag:inWndTag];
            
            // add to window/popover
            [contentView addSubview:view];
//            [view release];
        }
        
        [view setHidden:!show];
        
    } else {
        CocoaShowAlert( NSWarningAlertStyle, @"Window error.", @"Cocoa window does not exist." );
    }
}



#pragma mark - PDFView
///////////////////////////////////////////////////
//                  PDFView                      //
///////////////////////////////////////////////////
@implementation PDFViewSC

@end

@implementation PDFView (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

PDFViewRef PDFViewWithTag( NSInteger tag )
{
    PDFView *view = (PDFView *)CocoaViewWithTag( tag );
    if ( view ) {
        if ( [view isKindOfClass:[PDFView class]] ) return view;
    }
    CocoaShowAlert( NSWarningAlertStyle, @"Error.", [NSString stringWithFormat:@"Can't find PDFView with tag: %ld.",(long)tag] );
    return nil;
}

BOOL PDFViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [PDFView class] ); }

PDFDocumentRef PDFViewDocument( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) {
        return [view document];
    }
    return nil;
}

void PDFViewSetDocument( NSInteger tag, PDFDocumentRef doc )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) {
        [view setDocument:doc];
    }
}

// - configurations -
// -- display mode --
PDFDisplayMode PDFViewDisplayMode( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view displayMode];
    return 0;
}

void PDFViewSetDisplayMode( NSInteger tag, PDFDisplayMode displayMode )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setDisplayMode:displayMode];
}

// -- additional display configurations --
// --- display box ---
PDFDisplayBox PDFViewDisplayBox( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view displayBox];
    return 0;
}

void PDFViewSetDisplayBox( NSInteger tag, PDFDisplayBox displayBox )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setDisplayBox:displayBox];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
// --- page breaks ---
Boolean PDFViewDisplaysPageBreaks( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view displaysPageBreaks];
    return NO;
}

void PDFViewSetDisplaysPageBreaks( NSInteger tag, Boolean flag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setDisplaysPageBreaks:flag];
}

NSEdgeInsets PDFViewPageBreakMargins( NSInteger tag )
{
    NSEdgeInsets insets = {0,0,0,0};
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view pageBreakMargins];
    return insets;
}

void PDFViewSetPageBreakMargins( NSInteger tag, NSEdgeInsets insets )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setPageBreakMargins:insets];
}
#endif

// --- display direction ---
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
PDFDisplayDirection PDFViewDisplayDirection( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view displayDirection];
    return 0;
}

void PDFViewSetDisplayDirection( NSInteger tag, PDFDisplayDirection direction )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setDisplayDirection:direction];
}

Boolean PDFViewDisplaysRTL( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view displaysRTL];
    return NO;
}

void PDFViewSetDisplaysRTL( NSInteger tag, Boolean flag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setDisplaysRTL:flag];
}
#endif

// -- book display --
// --- display as book ---
Boolean PDFViewDisplaysAsBook( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view displaysAsBook];
    return NO;
}

void PDFViewSetDisplaysAsBook( NSInteger tag, Boolean flag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setDisplaysAsBook:flag];
}

// -- graphics properties --
// --- background color ---
ColorRef PDFViewBackgroundColor( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view backgroundColor];
    return nil;
}

void PDFViewSetBackgroundColor( NSInteger tag, ColorRef col )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setBackgroundColor:col];
}

// - antialiasing -
PDFInterpolationQuality PDFViewInterpolationQuality( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view interpolationQuality];
    return 0;
}

// -- scaling the view --
CGFloat PDFViewScaleFactor( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view scaleFactor];
    return 0.0;
}

void PDFViewSetScaleFactor( NSInteger tag, CGFloat factor )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setScaleFactor:factor];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CGFloat PDFViewScaleFactorForSizeToFit( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view scaleFactorForSizeToFit];
    return 0.0;
}

CGFloat PDFViewMaxScaleFactor( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view maxScaleFactor];
    return 0.0;
}

void PDFViewSetMaxScaleFactor( NSInteger tag, CGFloat factor )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setMaxScaleFactor:factor];
}

CGFloat PDFViewMinScaleFactor( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view minScaleFactor];
    return 0.0;
}

void PDFViewSetMinScaleFactor( NSInteger tag, CGFloat factor )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setMinScaleFactor:factor];
}
#endif

Boolean PDFViewAutoScales( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view autoScales];
    return NO;
}

void PDFViewSetAutoScales( NSInteger tag, Boolean flag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setAutoScales:flag];
}

CGSize PDFViewRowSizeForPage( NSInteger tag, PDFPageRef pg )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return NSSizeToCGSize([view rowSizeForPage:pg]);
    return CGSizeZero;
}

// --- zoom operations ---
void PDFViewZoomIn( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view zoomIn:nil];
}

Boolean PDFViewCanZoomIn( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view canZoomIn];
    return NO;
}

void PDFViewZoomOut( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view zoomOut:nil];
}

Boolean PDFViewCanZoomOut( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view canZoomOut];
    return NO;
}

// -- rendering and printing the view --
void PDFViewPrintWithInfo( NSInteger tag, PrintInfoRef printInfo, Boolean autoRotate )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view printWithInfo:printInfo autoRotate:autoRotate];
}

void PDFViewPrintWithInfoPageScaling( NSInteger tag, PrintInfoRef printInfo, Boolean autoRotate, PDFPrintScalingMode scale )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view printWithInfo:printInfo autoRotate:autoRotate pageScaling:scale];
}

// -- specializing the view
ViewRef PDFViewDocumentView( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view documentView];
    return nil;
}

void PDFViewLayoutDocumentView( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view layoutDocumentView];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
// -- draw operations --
void PDFViewDrawPageToContext( NSInteger tag, PDFPageRef pg, CGContextRef context )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view drawPage:pg toContext:context];
}

void PDFViewDrawPagePostToContext( NSInteger tag, PDFPageRef pg, CGContextRef context )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view drawPagePost:pg toContext:context];
}
#endif

// -- document interactions --
// --- selections ---
PDFSelectionRef PDFViewCurrentSelection( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view currentSelection];
    return nil;
}

void PDFViewSetCurrentSelection( NSInteger tag, PDFSelectionRef selection, Boolean animate )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setCurrentSelection:selection animate:animate];
}

void PDFViewSelectAll( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view selectAll:nil];
}

void PDFViewClearSelection( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view clearSelection];
}

void PDFViewCopy( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view copy:nil];
}

void PDFViewScrollSelectionToVisible( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view scrollSelectionToVisible:nil];
}

CFArrayRef PDFViewHighlightedSelections( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return (CFArrayRef)[view highlightedSelections];
    return nil;
}

// --- annotation actions ---
void PDFViewAnnotationsChangedOnPage( NSInteger tag, PDFPageRef pg )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view annotationsChangedOnPage:pg];
}

// --- link annotations ---
Boolean PDFViewEnableDataDetectors( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view enableDataDetectors];
    return NO;
}

// --- convert page and view points ---
PDFPageRef PDFViewPageForPoint( NSInteger tag, CGPoint pt, Boolean nearest )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view pageForPoint:NSPointFromCGPoint(pt) nearest:nearest];
    return nil;
}

CGPoint PDFViewConvertPointToPage( NSInteger tag, CGPoint pt, PDFPageRef pg )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return NSPointToCGPoint([view convertPoint:NSPointFromCGPoint(pt) toPage:pg]);
    return CGPointZero;
}

CGRect PDFViewConvertRectToPage( NSInteger tag, CGRect r, PDFPageRef pg )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return NSRectToCGRect([view convertRect:NSRectFromCGRect(r) toPage:pg]);
    return CGRectZero;
}

CGPoint PDFViewConvertPointFromPage( NSInteger tag, CGPoint pt, PDFPageRef pg )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return NSPointToCGPoint([view convertPoint:NSPointFromCGPoint(pt) fromPage:pg]);
    return CGPointZero;
}

CGRect PDFViewConvertRectFromPage( NSInteger tag, CGRect r, PDFPageRef pg )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return NSRectToCGRect([view convertRect:NSRectFromCGRect(r) fromPage:pg]);
    return CGRectZero;
}

// --- mouse position and events ---
PDFAreaOfInterest PDFViewAreaOfInterestForMouse( NSInteger tag, CocoaEventRef evnt )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view areaOfInterestForMouse:evnt];
    return 0;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101100
PDFAreaOfInterest PDFViewAreaOfInterestForPoint( NSInteger tag, CGPoint pt )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view areaOfInterestForPoint:NSPointFromCGPoint(pt)];
    return 0;
}
#endif

void PDFViewSetCursorForAreaOfInterest( NSInteger tag, PDFAreaOfInterest area )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view setCursorForAreaOfInterest:area];
}

void PDFViewPerformAction( NSInteger tag, PDFActionRef action )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view performAction:action];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
// --- drag operations
Boolean PDFViewAcceptsDraggedFiles( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view acceptsDraggedFiles];
    return NO;
}

void PDFViewSetAcceptsDraggedFiles( NSInteger tag, Boolean flag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view setAcceptsDraggedFiles:flag];
}
#endif

// -- navigating within document --
PDFPageRef PDFViewCurrentPage( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view currentPage];
    return nil;
}

PDFDestinationRef PDFViewCurrentDestination( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view currentDestination];
    return nil;
}

CFArrayRef PDFViewVisiblePages( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return (CFArrayRef)[view visiblePages];
    return nil;
}

// -- navigation --
Boolean PDFViewCanGoBack( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view canGoBack];
    return NO;
}

Boolean PDFViewCanGoForward( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view canGoForward];
    return NO;
}

Boolean PDFViewCanGoToFirstPage( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view canGoToFirstPage];
    return NO;
}

Boolean PDFViewCanGoToLastPage( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view canGoToLastPage];
    return NO;
}

Boolean PDFViewCanGoToNextPage( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view canGoToNextPage];
    return NO;
}

Boolean PDFViewCanGoToPreviousPage( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) return [view canGoToPreviousPage];
    return NO;
}

void PDFViewGoBack( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view goBack:nil];
}

void PDFViewGoForward( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view goForward:nil];
}

void PDFViewGoToFirstPage( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view goToFirstPage:nil];
}

void PDFViewGoToLastPage( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view goToLastPage:nil];
}

void PDFViewGoToNextPage( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view goToNextPage:nil];
}

void PDFViewGoToPreviousPage( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view goToPreviousPage:nil];
}

void PDFViewGoToPage( NSInteger tag, PDFPageRef pg )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view goToPage:pg];
}

void PDFViewGoToDestination( NSInteger tag, PDFDestinationRef destination )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view goToDestination:destination];
}

void PDFViewGoToSelection( NSInteger tag, PDFSelectionRef selection )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view goToSelection:selection];
}

void PDFViewGoToRectOnPage( NSInteger tag, CGRect r, PDFPageRef pg )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) [view goToRect:NSRectFromCGRect(r) onPage:pg];
}

// -- delegate --


// - convenience -
NSUInteger PDFViewCurrentPageIndex( NSInteger tag )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) {
        PDFDocument *doc = [view document];
        if ( doc ) {
            PDFPage *page = [view currentPage];
            if ( page ) return [doc indexForPage:page];
        }
    }
    return 0;
}

void PDFViewGoToPageAtIndex( NSInteger tag, NSUInteger index )
{
    PDFViewRef view = PDFViewWithTag( tag );
    if ( view ) {
        PDFDocument *doc = [view document];
        if ( doc ) {
            PDFPage *page = [doc pageAtIndex:index];
            if ( page ) [view goToPage:page];
        }
    }
}


// Init
PDFViewRef PDFViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) {
        rect = NSMakeRect( 20, 20, 300, 200 );
    }

    // subclass
    BOOL subclass = IsSubclassFlag();

    // init
    PDFView *view;
    if ( subclass ) {
        view = [[[PDFViewSC alloc] initWithFrame:rect] autorelease];
    } else {
        view = [[[PDFView alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [view setTag: inTag];

    return view;
}


// FB statement
void CocoaPDFView( NSInteger inTag, CGRect inRect, PDFDocumentRef inDocRef, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);

    NSRect rect = NSRectFromCGRect( inRect );

    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );

        PDFView *view = CocoaViewSubviewWithTag( contentView, inTag );

        if ( view ) {   // already exists
            
            // make sure it's same view type
            if ( [view class] != [PDFView class] ) return;

            // rect
            if ( !NSIsEmptyRect( rect ) ) [view setFrame:rect];
            
            // doc
            if ( inDocRef ) [view setDocument:inDocRef];
            
        } else {
            
            // rect
//            if ( NSIsEmptyRect( rect ) ) {
//                rect = NSMakeRect( 20, 20, 300, 200 );
//            }
            
            // init
//            if ( inSubclass ) {
//                view = [[PDFViewSC alloc] initWithFrame:rect];
//            } else {
//                view = [[PDFView alloc] initWithFrame:rect];
//            }
            
            view = PDFViewInit( inTag, inRect );
            
            // default autoresizing
            [view setAutoresizingMask:NSViewMinYMargin];
            
            // tag
//            [view setTag: inTag];
            
            // wndTag
            [view setWndTag:inWndTag];
            
            // add to window/popover
            [contentView addSubview:view];
//            [view release];
        }
        
        [view setHidden:!show];
        
    } else {
        CocoaShowAlert( NSWarningAlertStyle, @"Window error.", @"Cocoa window does not exist." );
    }
}


