/*
 SpriteKit.h
 
 Bernie Wylde 20190507
 */

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090

#import <SpriteKit/SpriteKit.h>
#import "CocoaUI.h"

#if __LP64__
#ifdef SK_EXPORT

typedef SKNode *SKNodeRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
typedef SK3DNode *SK3DNodeRef;
#endif

typedef SKAction *SKActionRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
typedef SKAttribute *SKAttributeRef;
typedef SKAttributeValue *SKAttributeValueRef;
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
typedef SKAudioNode *SKAudioNodeRef;
typedef SKCameraNode *SKCameraNodeRef;
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
typedef SKConstraint *SKConstraintRef;
#endif

typedef SKCropNode *SKCropNodeRef;
typedef SKEffectNode *SKEffectNodeRef;

typedef SKEmitterNode *SKEmitterNodeRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
typedef SKFieldNode *SKFieldNodeRef;
#endif

typedef SKKeyframeSequence *SKKeyframeSequenceRef;
typedef SKLabelNode *SKLabelNodeRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
typedef SKLightNode *SKLightNodeRef;
#endif

typedef SKTexture *SKTextureRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
typedef SKMutableTexture *SKMutableTextureRef;
#endif

typedef SKPhysicsBody *SKPhysicsBodyRef;
typedef SKPhysicsContact *SKPhysicsContactRef;
typedef SKPhysicsJoint *SKPhysicsJointRef;
typedef SKPhysicsJointFixed *SKPhysicsJointFixedRef;
typedef SKPhysicsJointLimit *SKPhysicsJointLimitRef;
typedef SKPhysicsJointPin *SKPhysicsJointPinRef;
typedef SKPhysicsJointSliding *SKPhysicsJointSlidingRef;
typedef SKPhysicsJointSpring *SKPhysicsJointSpringRef;
typedef SKPhysicsWorld *SKPhysicsWorldRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
typedef SKRange *SKRangeRef;
typedef SKReachConstraints *SKReachConstraintsRef;
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
typedef SKReferenceNode *SKReferenceNodeRef;
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
typedef SKRegion *SKRegionRef;
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
typedef SKRenderer *SKRendererRef;
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
typedef SKShader *SKShaderRef;
#endif

typedef SKScene *SKSceneRef;
typedef SKShapeNode *SKShapeNodeRef;
typedef SKSpriteNode *SKSpriteNodeRef;
typedef SKTextureAtlas *SKTextureAtlasRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
typedef SKTileDefinition *SKTileDefinitionRef;
typedef SKTileGroup *SKTileGroupRef;
typedef SKTileGroupRule *SKTileGroupRuleRef;
typedef SKTileMapNode *SKTileMapNodeRef;
typedef SKTileSet *SKTileSetRef;
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
typedef SKTransformNode *SKTransformNodeRef;
#endif

typedef SKTransition *SKTransitionRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
typedef SKUniform *SKUniformRef;
#endif

typedef SKVideoNode *SKVideoNodeRef;
typedef SKView *SKViewRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
typedef SKWarpGeometry *SKWarpGeometryRef;
typedef SKWarpGeometryGrid *SKWarpGeometryGridRef;
#endif


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SK3DNode
///////////////////////////////////////////////////
//                 SK3DNode                      //
///////////////////////////////////////////////////
// Create
SK3DNodeRef SK3DNodeWithViewportSize( CGSize size );
SK3DNodeRef SK3DNodeWithCoder( CoderRef coder );

// Configure
CGSize SK3DNodeViewportSize( SK3DNodeRef ref );
void SK3DNodeSetViewportSize( SK3DNodeRef ref, CGSize size );
Boolean SK3DNodeAutoenablesDefaultLighting( SK3DNodeRef ref );
void SK3DNodeSetAutoenablesDefaultLighting( SK3DNodeRef ref, Boolean flag );

// Animate
Boolean SK3DNodeIsPlaying( SK3DNodeRef ref );
void SK3DNodeSetPlaying( SK3DNodeRef ref, Boolean flag );
Boolean SK3DNodeLoops( SK3DNodeRef ref );
void SK3DNodeSetLoops( SK3DNodeRef ref, Boolean flag );
CFTimeInterval SK3DNodeSceneTime( SK3DNodeRef ref );
void SK3DNodeSetSceneTime( SK3DNodeRef ref, CFTimeInterval sceneTime );

// Project points and hit testing
CFArrayRef SK3DNodeHitTest( SK3DNodeRef ref, CGPoint pt, CFDictionaryRef options );
vector_float3 SK3DNodeProjectPoint( SK3DNodeRef ref, vector_float3 pt );
vector_float3 SK3DNodeUnprojectPoint( SK3DNodeRef ref, vector_float3 pt );

#endif


#pragma mark - SKAction
///////////////////////////////////////////////////
//                 SKAction                      //
///////////////////////////////////////////////////
typedef void (*SKActionCustomWithDurationCallbackType)(SKNode*,CGFloat,void*);
typedef float (*SKActionTimingFunctionCallbackType)(SKAction*,float,void*);

// Animate in linear path
SKActionRef SKActionMoveByXY( CGFloat deltaX, CGFloat deltaY, CFTimeInterval duration );
SKActionRef SKActionMoveByVector( CGVector delta, CFTimeInterval duration );
SKActionRef SKActionMoveToPoint( CGPoint location, CFTimeInterval duration );
SKActionRef SKActionMoveToX( CGFloat x, CFTimeInterval duration );
SKActionRef SKActionMoveToY( CGFloat y, CFTimeInterval duration );

// Animate in custom path
SKActionRef SKActionFollowPath( CGPathRef path, CFTimeInterval duration );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKActionRef SKActionFollowPathSpeed( CGPathRef path, CGFloat speed );
#endif
SKActionRef SKActionFollowPathAsOffset( CGPathRef path, Boolean asOffset, Boolean orientToPath, CFTimeInterval duration );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKActionRef SKActionFollowPathAsOffsetSpeed( CGPathRef path, Boolean asOffset, Boolean orientToPath, CGFloat speed );
#endif

// Animate rotation
SKActionRef SKActionRotateByAngle( CGFloat radians, CFTimeInterval duration );
SKActionRef SKActionRotateToAngle( CGFloat radians, CFTimeInterval duration );
SKActionRef SKActionRotateToAngleShortestUnitArc( CGFloat radians, CFTimeInterval duration, Boolean shortestUnitArc );

// Controlling speed
SKActionRef SKActionSpeedBy( CGFloat speed, CFTimeInterval duration );
SKActionRef SKActionSpeedTo( CGFloat speed, CFTimeInterval duration );

// Animate scaling
SKActionRef SKActionScaleBy( CGFloat scale, CFTimeInterval duration );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
SKActionRef SKActionScaleToSize( CGSize size, CFTimeInterval duration );
#endif
SKActionRef SKActionScaleTo( CGFloat scale, CFTimeInterval duration );
SKActionRef SKActionScaleXYBy( CGFloat xScale, CGFloat yScale, CFTimeInterval duration );
SKActionRef SKActionScaleXYTo( CGFloat xScale, CGFloat yScale, CFTimeInterval duration );
SKActionRef SKActionScaleXTo( CGFloat xScale, CFTimeInterval duration );
SKActionRef SKActionScaleYTo( CGFloat yScale, CFTimeInterval duration );

// Animate transparency
SKActionRef SKActionFadeIn( CFTimeInterval duration );
SKActionRef SKActionFadeOut( CFTimeInterval duration );
SKActionRef SKActionFadeAlphaBy( CGFloat factor, CFTimeInterval duration );
SKActionRef SKActionFadeAlphaTo( CGFloat alpha, CFTimeInterval duration );

// Animate texture
SKActionRef SKActionResizeByWidthAndHeight( CGFloat width, CGFloat height, CFTimeInterval duration );
SKActionRef SKActionResizeToHeight( CGFloat height, CFTimeInterval duration );
SKActionRef SKActionResizeToWidth( CGFloat width, CFTimeInterval duration );
SKActionRef SKActionResizeToWidthAndHeight( CGFloat width, CGFloat height, CFTimeInterval duration );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKActionRef SKActionSetTexture( SKTextureRef texture );
SKActionRef SKActionSetTextureResize( SKTextureRef texture, Boolean resize );
#endif
SKActionRef SKActionAnimateWithTextures( CFArrayRef textures, CFTimeInterval timePerFrame );
SKActionRef SKActionAnimateWithTexturesResize( CFArrayRef textures, CFTimeInterval timePerFrame, Boolean resize, Boolean restore );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKActionRef SKActionSetNormalTexture( SKTextureRef texture );
SKActionRef SKActionSetNormalTextureResize( SKTextureRef texture, Boolean resize );
SKActionRef SKActionAnimateWithNormalTextures( CFArrayRef textures, CFTimeInterval timePerFrame );
SKActionRef SKActionAnimateWithNormalTexturesResize( CFArrayRef textures, CFTimeInterval timePerFrame, Boolean resize, Boolean restore );
#endif
SKActionRef SKActionColorizeWithColor( ColorRef color, CGFloat colorBlendFactor, CFTimeInterval duration );
SKActionRef SKActionColorizeWithColorBlendFactor( CGFloat colorBlendFactor, CFTimeInterval duration );

// Animate properties of physics body
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKActionRef SKActionApplyForce( CGVector force, CFTimeInterval duration );
SKActionRef SKActionApplyTorque( CGFloat torque, CFTimeInterval duration );
SKActionRef SKActionApplyForceAtPoint( CGVector force, CGPoint pt, CFTimeInterval duration );
SKActionRef SKActionApplyImpulse( CGVector impulse, CFTimeInterval duration );
SKActionRef SKActionApplyAngularImpulse( CGFloat impulse, CFTimeInterval duration );
SKActionRef SKActionApplyImpulseAtPoint( CGVector impulse, CGPoint pt, CFTimeInterval duration );
SKActionRef SKActionChangeChargeTo( CGFloat v, CFTimeInterval duration );
SKActionRef SKActionChangeChargeBy( CGFloat v, CFTimeInterval duration );
SKActionRef SKActionChangeMassTo( CGFloat v, CFTimeInterval duration );
SKActionRef SKActionChangeMassBy( CGFloat v, CFTimeInterval duration );
#endif// 101100
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKActionRef SKActionStrengthTo( float strength, CFTimeInterval duration );
SKActionRef SKActionStrengthBy( float strength, CFTimeInterval duration );
SKActionRef SKActionFalloffTo( float falloff, CFTimeInterval duration );
SKActionRef SKActionFalloffBy( float falloff, CFTimeInterval duration );
#endif// 101000

// Reversing animation
SKActionRef SKActionReverseAction( SKActionRef ref );

// Animate warping
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
SKActionRef SKActionAnimateWithWarps( CFArrayRef warps, CFArrayRef times );
SKActionRef SKActionAnimateWithWarpsRestore( CFArrayRef warps, CFArrayRef times, Boolean restore );
SKActionRef SKActionWarpTo( SKWarpGeometryRef warp, CFTimeInterval duration );
#endif// 101200

// Controlling audio
SKActionRef SKActionPlaySoundFileNamed( CFStringRef name, Boolean waitForCompletion );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKActionRef SKActionPlay( void );
SKActionRef SKActionPause( void );
SKActionRef SKActionStop( void );
SKActionRef SKActionChangePlaybackRateTo( float v, CFTimeInterval duration );
SKActionRef SKActionChangePlaybackRateBy( float v, CFTimeInterval duration );
SKActionRef SKActionChangeVolumeTo( float v, CFTimeInterval duration );
SKActionRef SKActionChangeVolumeBy( float v, CFTimeInterval duration );
SKActionRef SKActionChangeObstructionTo( float v, CFTimeInterval duration );
SKActionRef SKActionChangeObstructionBy( float v, CFTimeInterval duration );
SKActionRef SKActionChangeOcclusionTo( float v, CFTimeInterval duration );
SKActionRef SKActionChangeOcclusionBy( float v, CFTimeInterval duration );
SKActionRef SKActionChangeReverbTo( float v, CFTimeInterval duration );
SKActionRef SKActionChangeReverbBy( float v, CFTimeInterval duration );
SKActionRef SKActionStereoPanTo( float v, CFTimeInterval duration );
SKActionRef SKActionStereoPanBy( float v, CFTimeInterval duration );
#endif// 101100

// Remove from scene
SKActionRef SKActionRemoveFromParent( void );

// Running on children
SKActionRef SKActionRunActionOnChildWithName( SKActionRef action, CFStringRef name );

// Chaining
SKActionRef SKActionGroup( CFArrayRef actions );
SKActionRef SKActionSequence( CFArrayRef actions );
SKActionRef SKActionRepeatAction( SKActionRef action, NSUInteger count );
SKActionRef SKActionRepeatActionForever( SKActionRef action );

// Delaying
SKActionRef SKActionWaitForDuration( CFTimeInterval duration );
SKActionRef SKActionWaitForDurationWithRange( CFTimeInterval duration, CFTimeInterval durationRange );

// Inverse kinematics
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKActionRef SKActionReachToDuration( CGPoint position, SKNodeRef rootNode, CFTimeInterval duration );
SKActionRef SKActionReachToVelocity( CGPoint position, SKNodeRef rootNode, CGFloat velocity );
SKActionRef SKActionReachToNodeDuration( SKNodeRef node, SKNodeRef rootNode, CFTimeInterval duration );
SKActionRef SKActionReachToNodeVelocity( SKNodeRef node, SKNodeRef rootNode, CGFloat velocity );
#endif// 101000

// Custom actions
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKActionRef SKActionNamed( CFStringRef name );
SKActionRef SKActionNamedDuration( CFStringRef name, CFTimeInterval duration );
SKActionRef SKActionNamedFromURL( CFStringRef name, CFURLRef url );
SKActionRef SKActionNamedFromURLDuration( CFStringRef name, CFURLRef url, CFTimeInterval duration );
#endif// 101100
SKActionRef SKActionCustomWithDuration( CFTimeInterval duration, SKActionCustomWithDurationCallbackType callback, void *userData );

// Visibility
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void SKActionUnhide( void );
void SKActionHide( void );
#endif// 101000

// Timing
CFTimeInterval SKActionDuration( SKActionRef ref );
void SKActionSetDuration( SKActionRef ref, CFTimeInterval duration );
SKActionTimingMode skActionTimingMode( SKActionRef ref );
void SKActionSetTimingMode( SKActionRef ref, SKActionTimingMode mode );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void skActionTimingFunction( SKActionRef ref, SKActionTimingFunctionCallbackType callback, void *userData );
#endif// 101000
CGFloat SKActionSpeed( SKActionRef ref );
void SKActionSetSpeed( SKActionRef ref, CGFloat speed );



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - SKAttribute
///////////////////////////////////////////////////
//                SKAttribute                    //
///////////////////////////////////////////////////

// Init
SKAttributeRef SKAttributeWithName( CFStringRef name, SKAttributeType type );

// Instance properties
CFStringRef SKAttributeName( SKAttributeRef ref );
SKAttributeType skAttributeType( SKAttributeRef ref );

// Initializers
SKAttributeValueRef SKAttributeValueInit( void );
SKAttributeValueRef SKAttributeValueWithFloat( float value );
SKAttributeValueRef SKAttributeValueWithVectorFloat2( vector_float2 value );
SKAttributeValueRef SKAttributeValueWithVectorFloat3( vector_float3 value );
SKAttributeValueRef SKAttributeValueWithVectorFloat4( vector_float4 value );

// Instance properties
float SKAttributeValueFloatValue( SKAttributeValueRef ref );
void SKAttributeValueSetFloatValue( SKAttributeValueRef ref, float value );
vector_float2 SKAttributeValueVectorFloat2Value( SKAttributeValueRef ref );
void SKAttributeValueSetVectorFloat2Value( SKAttributeValueRef ref, vector_float2 value );
vector_float3 SKAttributeValueVectorFloat3Value( SKAttributeValueRef ref );
void SKAttributeValueSetVectorFloat3Value( SKAttributeValueRef ref, vector_float3 value );
vector_float4 SKAttributeValueVectorFloat4Value( SKAttributeValueRef ref );
void SKAttributeValueSetVectorFloat4Value( SKAttributeValueRef ref, vector_float4 value );

#endif


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
#pragma mark - SKAudioNode
///////////////////////////////////////////////////
//                SKAudioNode                    //
///////////////////////////////////////////////////
// Init
SKAudioNodeRef SKAudioNodeWithFileNamed( CFStringRef name );
SKAudioNodeRef SKAudioNodeWithURL( CFURLRef url );
SKAudioNodeRef SKAudioNodeWithCoder( CoderRef coder );

// Configure
Boolean SKAudioNodeIsPositional( SKAudioNodeRef ref );
void SKAudioNodeSetPositional( SKAudioNodeRef ref, Boolean flag );
Boolean SKAudioNodeAutoplayLooped( SKAudioNodeRef ref );
void SKAudioNodeSetAutoplayLooped( SKAudioNodeRef ref, Boolean flag );

#endif


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
#pragma mark - SKCameraNode
///////////////////////////////////////////////////
//               SKCameraNode                    //
///////////////////////////////////////////////////
// Init
SKCameraNodeRef SKCameraNodeInit( void );

// Visibility
CFSetRef SKCameraNodeContainedNodeSet( SKCameraNodeRef ref );
Boolean SKCameraNodeContainsNode( SKCameraNodeRef ref, SKNodeRef node );

#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKConstraint
///////////////////////////////////////////////////
//               SKConstraint                    //
///////////////////////////////////////////////////

// Position
SKConstraintRef SKConstraintPositionXY( SKRangeRef xRange, SKRangeRef yRange );
SKConstraintRef SKConstraintPositionX( SKRangeRef range );
SKConstraintRef SKConstraintPositionY( SKRangeRef range );

// Orientation
SKConstraintRef SKConstraintOrientToNode( SKNodeRef node, SKRangeRef offsetRadians );
SKConstraintRef SKConstraintOrientToPoint( CGPoint pt, SKRangeRef offsetRadians );
SKConstraintRef SKConstraintOrientToPointInNode( CGPoint pt, SKNodeRef node, SKRangeRef offsetRadians );
SKConstraintRef SKConstraintZRotation( SKRangeRef zRange );

// Distance
SKConstraintRef SKConstraintDistanceToNode( SKRangeRef range, SKNodeRef node );
SKConstraintRef SKConstraintDistanceToPoint( SKRangeRef range, CGPoint pt );
SKConstraintRef SKConstraintDistanceToPointInNode( SKRangeRef range, CGPoint pt, SKNodeRef node );

// Coordinate system
SKNodeRef SKConstraintReferenceNode( SKConstraintRef ref );
void SKConstraintSetReferenceNode( SKConstraintRef ref, SKNodeRef node );

// Enable/disable
Boolean SKConstraintEnabled( SKConstraintRef ref );
void SKConstraintSetEnabled( SKConstraintRef ref, Boolean flag );

// Create
SKRangeRef SKRangeWithLowerLimitUpperLimit( CGFloat lowerLimit, CGFloat upperLimit );
SKRangeRef SKRangeWithValue( CGFloat value, CGFloat variance );
SKRangeRef SKRangeWithNoLimits( void );
SKRangeRef SKRangeWithLowerLimit( CGFloat lowerLimit );
SKRangeRef SKRangeWithUpperLimit( CGFloat upperLimit );
SKRangeRef SKRangeWithConstantValue( CGFloat value );

// Inspect
CGFloat SKRangeLowerLimit( SKRangeRef ref );
void SKRangeSetLowerLimit( SKRangeRef ref, CGFloat lowerLimit );
CGFloat SKRangeUpperLimit( SKRangeRef ref );
void SKRangeSetUpperLimit( SKRangeRef ref, CGFloat upperLimit );

#endif



#pragma mark - SKCropNode
///////////////////////////////////////////////////
//                 SKCropNode                    //
///////////////////////////////////////////////////

// Init
SKCropNodeRef SKCropNodeInit( void );

// Mask filter
SKNodeRef SKCropNodeMaskNode( SKCropNodeRef ref );
void SKCropNodeSetMaskNode( SKCropNodeRef ref, SKNodeRef node );



#pragma mark - SKEffectNode
///////////////////////////////////////////////////
//               SKEffectNode                    //
///////////////////////////////////////////////////
// Init
SKEffectNodeRef SKEffectNodeInit( void );

// Core image filters
CIFilterRef SKEffectNodeFilter( SKEffectNodeRef ref );
void SKEffectNodeSetFilter( SKEffectNodeRef ref, CIFilterRef filter );
Boolean SKEffectNodeShouldEnableEffects( SKEffectNodeRef ref );
void SKEffectNodeSetShouldEnableEffects( SKEffectNodeRef ref, Boolean flag );
Boolean SKEffectNodeShouldCenterFilter( SKEffectNodeRef ref );
void SKEffectNodeSetShouldCenterFilter( SKEffectNodeRef ref, Boolean flag );

// Shader
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKShaderRef SKEffectNodeShader( SKEffectNodeRef ref );
void SKEffectNodeSetShader( SKEffectNodeRef ref, SKShaderRef shader );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFDictionaryRef SKEffectNodeAttributeValues( SKEffectNodeRef ref );
void SKEffectNodeSetAttributeValues( SKEffectNodeRef ref, CFDictionaryRef values );
void SKEffectNodeSetValueForAttributeNamed( SKEffectNodeRef ref, SKAttributeValueRef value, CFStringRef name );
SKAttributeValueRef SKEffectNodeValueForAttributeNamed( SKEffectNodeRef ref, CFStringRef name );
#endif

// Flattening
Boolean SKEffectNodeShouldRasterize( SKEffectNodeRef ref );
void SKEffectNodeSetShouldRasterize( SKEffectNodeRef ref, Boolean flag );

// Alpha blending
SKBlendMode SKEffectNodeBlendMode( SKEffectNodeRef ref );
void SKEffectNodeSetBlendMode( SKEffectNodeRef ref, SKBlendMode mode );



#pragma mark - SKEmitterNode
///////////////////////////////////////////////////
//               SKEmitterNode                   //
///////////////////////////////////////////////////
// Init
SKEmitterNodeRef SKEmitterNodeInit( void );

// Choosing
SKNodeRef SKEmitterNodeTargetNode( SKEmitterNodeRef ref );
void SKEmitterNodeSetTargetNode( SKEmitterNodeRef ref, SKNodeRef node );

// Controlling when particles are created
void SKEmitterNodeAdvanceSimulationTime( SKEmitterNodeRef ref, CFTimeInterval sec );
void SKEmitterNodeResetSimulation( SKEmitterNodeRef ref );
CGFloat SKEmitterNodeParticleBirthRate( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleBirthRate( SKEmitterNodeRef ref, CGFloat birthRate );
NSUInteger SKEmitterNodeNumParticlesToEmit( SKEmitterNodeRef ref );
void SKEmitterNodeSetNumParticlesToEmit( SKEmitterNodeRef ref, NSUInteger num );

// Rendering order
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKParticleRenderOrder SKEmitterNodeParticleRenderOrder( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleRenderOrder( SKEmitterNodeRef ref, SKParticleRenderOrder order );
#endif

// Lifetime
CGFloat SKEmitterNodeParticleLifetime( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleLifetime( SKEmitterNodeRef ref, CGFloat lifetime );
CGFloat SKEmitterNodeParticleLifetimeRange( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleLifetimeRange( SKEmitterNodeRef ref, CGFloat range );

// Position
CGPoint SKEmitterNodeParticlePosition( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticlePosition( SKEmitterNodeRef ref, CGPoint position );
CGVector SKEmitterNodeParticlePositionRange( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticlePositionRange( SKEmitterNodeRef ref, CGVector range );
CGFloat SKEmitterNodeParticleZPosition( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleZPosition( SKEmitterNodeRef ref, CGFloat position );

// Velocity and acceleration
CGFloat SKEmitterNodeParticleSpeed( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleSpeed( SKEmitterNodeRef ref, CGFloat speed );
CGFloat SKEmitterNodeParticleSpeedRange( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleSpeedRange( SKEmitterNodeRef ref, CGFloat range );
CGFloat SKEmitterNodeEmissionAngle( SKEmitterNodeRef ref );
void SKEmitterNodeSetEmissionAngle( SKEmitterNodeRef ref, CGFloat angle );
CGFloat SKEmitterNodeEmissionAngleRange( SKEmitterNodeRef ref );
void SKEmitterNodeSetEmissionAngleRange( SKEmitterNodeRef ref, CGFloat range );
CGFloat SKEmitterNodeXAcceleration( SKEmitterNodeRef ref );
void SKEmitterNodeSetXAcceleration( SKEmitterNodeRef ref, CGFloat acceleration );
CGFloat SKEmitterNodeYAcceleration( SKEmitterNodeRef ref );
void SKEmitterNodeSetYAcceleration( SKEmitterNodeRef ref, CGFloat acceleration );

// Rotation
CGFloat SKEmitterNodeParticleRotation( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleRotation( SKEmitterNodeRef ref, CGFloat rotation );
CGFloat SKEmitterNodeParticleRotationRange( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleRotationRange( SKEmitterNodeRef ref, CGFloat range );
CGFloat SKEmitterNodeParticleRotationSpeed( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleRotationSpeed( SKEmitterNodeRef ref, CGFloat speed );

// Scaling
CGFloat SKEmitterNodeParticleScale( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleScale( SKEmitterNodeRef ref, CGFloat scale );
CGFloat SKEmitterNodeParticleScaleRange( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleScaleRange( SKEmitterNodeRef ref, CGFloat range );
CGFloat SKEmitterNodeParticleScaleSpeed( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleScaleSpeed( SKEmitterNodeRef ref, CGFloat speed );
SKKeyframeSequenceRef SKEmitterNodeParticleScaleSequence( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleScaleSequence( SKEmitterNodeRef ref, SKKeyframeSequenceRef sequence );

// Image and size
SKTextureRef SKEmitterNodeParticleTexture( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleTexture( SKEmitterNodeRef ref, SKTextureRef texture );
CGSize SKEmitterNodeParticleSize( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleSize( SKEmitterNodeRef ref, CGSize size );

// Color
SKKeyframeSequenceRef SKEmitterNodeParticleColorSequence( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorSequence( SKEmitterNodeRef ref, SKKeyframeSequenceRef sequence );
ColorRef SKEmitterNodeParticleColor( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColor( SKEmitterNodeRef ref, ColorRef col );
CGFloat SKEmitterNodeParticleColorAlphaRange( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorAlphaRange( SKEmitterNodeRef ref, CGFloat range );
CGFloat SKEmitterNodeParticleColorBlueRange( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorBlueRange( SKEmitterNodeRef ref, CGFloat range );
CGFloat SKEmitterNodeParticleColorGreenRange( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorGreenRange( SKEmitterNodeRef ref, CGFloat range );
CGFloat SKEmitterNodeParticleColorRedRange( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorRedRange( SKEmitterNodeRef ref, CGFloat range );
CGFloat SKEmitterNodeParticleColorAlphaSpeed( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorAlphaSpeed( SKEmitterNodeRef ref, CGFloat speed );
CGFloat SKEmitterNodeParticleColorBlueSpeed( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorBlueSpeed( SKEmitterNodeRef ref, CGFloat speed );
CGFloat SKEmitterNodeParticleColorGreenSpeed( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorGreenSpeed( SKEmitterNodeRef ref, CGFloat speed );
CGFloat SKEmitterNodeParticleColorRedSpeed( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorRedSpeed( SKEmitterNodeRef ref, CGFloat speed );

// Texture blend
SKKeyframeSequenceRef SKEmitterNodeParticleColorBlendFactorSequence( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorBlendFactorSequence( SKEmitterNodeRef ref, SKKeyframeSequenceRef sequence );
CGFloat SKEmitterNodeParticleColorBlendFactor( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorBlendFactor( SKEmitterNodeRef ref, CGFloat factor );
CGFloat SKEmitterNodeParticleColorBlendFactorRange( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorBlendFactorRange( SKEmitterNodeRef ref, CGFloat range );
CGFloat SKEmitterNodeParticleColorBlendFactorSpeed( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleColorBlendFactorSpeed( SKEmitterNodeRef ref, CGFloat speed );

// Framebuffer
SKBlendMode SKEmitterNodeParticleBlendMode( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleBlendMode( SKEmitterNodeRef ref, SKBlendMode mode );
SKKeyframeSequenceRef SKEmitterNodeParticleAlphaSequence( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleAlphaSequence( SKEmitterNodeRef ref, SKKeyframeSequenceRef sequence );
CGFloat SKEmitterNodeParticleAlpha( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleAlpha( SKEmitterNodeRef ref, CGFloat alpha );
CGFloat SKEmitterNodeParticleAlphaRange( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleAlphaRange( SKEmitterNodeRef ref, CGFloat range );
CGFloat SKEmitterNodeParticleAlphaSpeed( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleAlphaSpeed( SKEmitterNodeRef ref, CGFloat speed );

// Animating
SKActionRef SKEmitterNodeParticleAction( SKEmitterNodeRef ref );
void SKEmitterNodeSetParticleAction( SKEmitterNodeRef ref, SKActionRef action );

// Physics fields
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
UInt32 SKEmitterNodeFieldBitMask( SKEmitterNodeRef ref );
void SKEmitterNodeSetFieldBitMask( SKEmitterNodeRef ref, UInt32 mask );
#endif

// Drawing
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKShaderRef SKEmitterNodeShader( SKEmitterNodeRef ref );
void SKEmitterNodeSetShader( SKEmitterNodeRef ref, SKShaderRef shader );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFDictionaryRef SKEmitterNodeAttributeValues( SKEmitterNodeRef ref );
void SKEmitterNodeSetAttributeValues( SKEmitterNodeRef ref, CFDictionaryRef values );
void SKEmitterNodeSetValueForAttributeNamed( SKEmitterNodeRef ref, SKAttributeValueRef value, CFStringRef name );
SKAttributeValueRef SKEmitterNodeValueForAttributeNamed( SKEmitterNodeRef ref, CFStringRef name );
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKFieldNode
///////////////////////////////////////////////////
//                 SKFieldNode                   //
///////////////////////////////////////////////////
typedef vector_float3 (*SKFieldNodeCustomFieldCallbackType)(vector_float3,vector_float3,float,float,CFTimeInterval,void*);


// Create
SKFieldNodeRef SKFieldNodeDragField( void );
SKFieldNodeRef SKFieldNodeElectricField( void );
SKFieldNodeRef SKFieldNodeLinearGravityFieldWithVector( vector_float3 direction );
SKFieldNodeRef SKFieldNodeMagneticField( void );
SKFieldNodeRef SKFieldNodeNoiseFieldWithSmoothness( CGFloat smoothness, CGFloat animationSpeed );
SKFieldNodeRef SKFieldNodeRadialGravityField( void );
SKFieldNodeRef SKFieldNodeSpringField( void );
SKFieldNodeRef SKFieldNodeTurbulenceFieldWithSmoothness( CGFloat smoothness, CGFloat animationSpeed );
SKFieldNodeRef SKFieldNodeVelocityFieldWithTexture( SKTextureRef texture );
SKFieldNodeRef SKFieldNodeVelocityFieldWithVector( vector_float3 direction );
SKFieldNodeRef SKFieldNodeVortexField( void );
SKFieldNodeRef SKFieldNodeCustomFieldWithEvaluationCallback( SKFieldNodeCustomFieldCallbackType callback, void *userData );

// Physics bodies affected by field
Boolean SKFieldNodeIsEnabled( SKFieldNodeRef ref );
void SKFieldNodeSetEnabled( SKFieldNodeRef ref, Boolean flag );
Boolean SKFieldNodeIsExclusive( SKFieldNodeRef ref );
void SKFieldNodeSetExclusive( SKFieldNodeRef ref, Boolean flag );
SKRegionRef SKFieldNodeRegion( SKFieldNodeRef ref );
void SKFieldNodeSetRegion( SKFieldNodeRef ref, SKRegionRef region );
float SKFieldNodeMinimumRadius( SKFieldNodeRef ref );
void SKFieldNodeSetMinimumRadius( SKFieldNodeRef ref, float radius );
UInt32 SKFieldNodeCategoryBitMask( SKFieldNodeRef ref );
void SKFieldNodeSetCategoryBitMask( SKFieldNodeRef ref, UInt32 bitMask );
float SKFieldNodeStrength( SKFieldNodeRef ref );
void SKFieldNodeSetStrength( SKFieldNodeRef ref, float strength );
float SKFieldNodeFalloff( SKFieldNodeRef ref );
void SKFieldNodeSetFalloff( SKFieldNodeRef ref, float falloff );

// Other field properties
float SKFieldNodeAnimationSpeed( SKFieldNodeRef ref );
void SKFieldNodeSetAnimationSpeed( SKFieldNodeRef ref, float speed );
float SKFieldNodeAnimationSmoothness( SKFieldNodeRef ref );
void SKFieldNodeSetAnimationSmoothness( SKFieldNodeRef ref, float smoothness );
vector_float3 SKFieldNodeDirection( SKFieldNodeRef ref );
void SKFieldNodeSetDirection( SKFieldNodeRef ref, vector_float3 direction );
SKTextureRef SKFieldNodeTexture( SKFieldNodeRef ref );
void SKFieldNodeSetTexture( SKFieldNodeRef ref, SKTextureRef texture );


#endif



#pragma mark - SKKeyframeSequence
///////////////////////////////////////////////////
//              SKKeyframeSequence               //
///////////////////////////////////////////////////
// Init
SKKeyframeSequenceRef SKKeyframeSequenceWithKeyframeValues( CFArrayRef values, CFArrayRef times );
SKKeyframeSequenceRef SKKeyframeSequenceWithCapacity( NSUInteger numItems );
SKKeyframeSequenceRef SKKeyframeSequenceWithCoder( CoderRef coder );

// Building
void SKKeyframeSequenceAddKeyframeValue( SKKeyframeSequenceRef ref, CFTypeRef value, CGFloat time );
void SKKeyframeSequenceRemoveKeyframeAtIndex( SKKeyframeSequenceRef ref, NSUInteger index );
void SKKeyframeSequenceRemoveLastKeyframe( SKKeyframeSequenceRef ref );
void SKKeyframeSequenceSetKeyframeTimeForIndex( SKKeyframeSequenceRef ref, CGFloat time, NSUInteger index );
void SKKeyframeSequenceSetKeyframeValueForIndex( SKKeyframeSequenceRef ref, CFTypeRef value, NSUInteger index );
void SKKeyframeSequenceSetKeyframeValueAndTimeForIndex( SKKeyframeSequenceRef ref, CFTypeRef value, CGFloat time, NSUInteger index );

// Running
CFTypeRef SKKeyframeSequenceSampleAtTime( SKKeyframeSequenceRef ref, CGFloat time );

// Info
NSUInteger SKKeyframeSequenceCount( SKKeyframeSequenceRef ref );
CGFloat SKKeyframeSequenceKeyframeTimeForIndex( SKKeyframeSequenceRef ref, NSUInteger index );
CFTypeRef SKKeyframeSequenceKeyframeValueForIndex( SKKeyframeSequenceRef ref, NSUInteger index );

// Interpolation modifiers
SKInterpolationMode SKKeyframeSequenceInterpolationMode( SKKeyframeSequenceRef ref );
void SKKeyframeSequenceSetInterpolationMode( SKKeyframeSequenceRef ref, SKInterpolationMode mode );
SKRepeatMode SKKeyframeSequenceRepeatMode( SKKeyframeSequenceRef ref );
void SKKeyframeSequenceSetRepeatMode( SKKeyframeSequenceRef ref, SKRepeatMode mode );



#pragma mark - SKLabelNode
///////////////////////////////////////////////////
//                 SKLabelNode                   //
///////////////////////////////////////////////////
// Create
SKLabelNodeRef SKLabelNodeWithFontNamed( CFStringRef name );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKLabelNodeRef SKLabelNodeWithText( CFStringRef text );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
SKLabelNodeRef SKLabelNodeWithAttributedText( CFAttributedStringRef aText );
#endif

// Text
CFStringRef SKLabelNodeText( SKLabelNodeRef ref );
void SKLabelNodeSetText( SKLabelNodeRef ref, CFStringRef text );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFAttributedStringRef SKLabelNodeAttributedText( SKLabelNodeRef ref );
void SKLabelNodeSetAttributedText( SKLabelNodeRef ref, CFAttributedStringRef aText );
#endif

// Font
ColorRef SKLabelNodeFontColor( SKLabelNodeRef ref );
void SKLabelNodeSetFontColor( SKLabelNodeRef ref, ColorRef col );
CFStringRef SKLabelNodeFontName( SKLabelNodeRef ref );
void SKLabelNodeSetFontName( SKLabelNodeRef ref, CFStringRef name );
CGFloat SKLabelNodeFontSize( SKLabelNodeRef ref );
void SKLabelNodeSetFontSize( SKLabelNodeRef ref, CGFloat size );

// Alignment
SKLabelVerticalAlignmentMode SKLabelNodeVerticalAlignmentMode( SKLabelNodeRef ref );
void SKLabelNodeSetVerticalAlignmentMode( SKLabelNodeRef ref, SKLabelVerticalAlignmentMode mode );
SKLabelHorizontalAlignmentMode SKLabelNodeHorizontalAlignmentMode( SKLabelNodeRef ref );
void SKLabelNodeSetHorizontalAlignmentMode( SKLabelNodeRef ref, SKLabelHorizontalAlignmentMode mode );

// Line-break behavior
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CGFloat SKLabelNodePreferredMaxLayoutWidth( SKLabelNodeRef ref );
void SKLabelNodeSetPreferredMaxLayoutWidth( SKLabelNodeRef ref, CGFloat width );
NSLineBreakMode SKLabelNodeLineBreakMode( SKLabelNodeRef ref );
void SKLabelNodeSetLineBreakMode( SKLabelNodeRef ref, NSLineBreakMode mode );
NSInteger SKLabelNodeNumberOfLines( SKLabelNodeRef ref );
void SKLabelNodeSetNumberOfLines( SKLabelNodeRef ref, NSInteger lines );
#endif

// Colorizing
ColorRef SKLabelNodeColor( SKLabelNodeRef ref );
void SKLabelNodeSetColor( SKLabelNodeRef ref, ColorRef col );
CGFloat SKLabelNodeColorBlendFactor( SKLabelNodeRef ref );
void SKLabelNodeSetColorBlendFactor( SKLabelNodeRef ref, CGFloat factor );

// Alpha blending
SKBlendMode SKLabelNodeBlendMode( SKLabelNodeRef ref );
void SKLabelNodeSetBlendMode( SKLabelNodeRef ref, SKBlendMode mode );



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKLightNode
///////////////////////////////////////////////////
//                 SKLightNode                   //
///////////////////////////////////////////////////
// Init
SKLightNodeRef SKLightNodeInit( void );

// Active
Boolean SKLightNodeIsEnabled( SKLightNodeRef ref );
void SKLightNodeSetEnabled( SKLightNodeRef ref, Boolean flag );
UInt32 SKLightNodeCategoryBitMask( SKLightNodeRef ref );
void SKLightNodeSetCategoryBitMask( SKLightNodeRef ref, UInt32 bitMask );

// Lighting properties
ColorRef SKLightNodeAmbientColor( SKLightNodeRef ref );
void SKLightNodeSetAmbientColor( SKLightNodeRef ref, ColorRef col );
ColorRef SKLightNodeLightColor( SKLightNodeRef ref );
void SKLightNodeSetLightColor( SKLightNodeRef ref, ColorRef col );
ColorRef SKLightNodeShadowColor( SKLightNodeRef ref );
void SKLightNodeSetShadowColor( SKLightNodeRef ref, ColorRef col );
CGFloat SKLightNodeFalloff( SKLightNodeRef ref );
void SKLightNodeSetFalloff( SKLightNodeRef ref, CGFloat falloff );




#pragma mark - SKMutableTexture
///////////////////////////////////////////////////
//              SKMutableTexture                 //
///////////////////////////////////////////////////
typedef void (*SKMutableTextureModifyPixelDataCallbackType)(SKMutableTexture*,void*,NSUInteger,void*);

// Create
SKMutableTextureRef SKMutableTextureWithSize( CGSize size );
SKMutableTextureRef SKMutableTextureWithPixelFormat( CGSize size, UInt32 format );

// Modify
void SKMutableTextureModifyPixelData( SKMutableTextureRef ref, SKMutableTextureModifyPixelDataCallbackType callback, void *userData );

#endif



#pragma mark - SKNode
///////////////////////////////////////////////////
//                   SKNode                      //
///////////////////////////////////////////////////
typedef BOOL (*SKNodeEnumerateChildNodesWithNameCallbackType)(SKNode*,SKNode*,void*);
typedef void (*SKNodeRunActionCompletionCallbackType)(SKNode*,SKAction*,void*);


// Init
SKNodeRef SKNodeInit( void );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKNodeRef SKNodeWithFileNamed( CFStringRef name );
#endif
SKNodeRef SKNodeWithCoder( CoderRef coder );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
SKNodeRef SKNodeWithFileNamedSecurelyWithClasses( CFStringRef name, CFSetRef classes, ErrorRef *err );
#endif

// Position
CGPoint SKNodePosition( SKNodeRef ref );
void SKNodeSetPosition( SKNodeRef ref, CGPoint pt );

// Content size
CGRect SKNodeFrame( SKNodeRef ref );
CGRect SKNodeCalculateAccumulatedFrame( SKNodeRef ref );

// Draw order
CGFloat SKNodeZPosition( SKNodeRef ref );
void SKNodeSetZPosition( SKNodeRef ref, CGFloat position );

// Scale + rotate
CGFloat SKNodeZRotation( SKNodeRef ref );
void SKNodeSetZRotation( SKNodeRef ref, CGFloat rotation );
void SKNodeSetScale( SKNodeRef ref, CGFloat scale );
CGFloat SKNodeXScale( SKNodeRef ref );
void SKNodeSetXScale( SKNodeRef ref, CGFloat scale );
CGFloat SKNodeYScale( SKNodeRef ref );
void SKNodeSetYScale( SKNodeRef ref, CGFloat scale );

// Related nodes
SKSceneRef SKNodeScene( SKNodeRef ref );
SKNodeRef SKNodeParent( SKNodeRef ref );
CFArrayRef SKNodeChildren( SKNodeRef ref );

// Modify node tree
void SKNodeAddChild( SKNodeRef ref, SKNodeRef child );
void SKNodeInsertChild( SKNodeRef ref, SKNodeRef child, NSInteger index );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean SKNodeIsEqualToNode( SKNodeRef ref, SKNodeRef otherNode );
void SKNodeMoveToParent( SKNodeRef ref, SKNodeRef parent );
#endif
void SKNodeRemoveFromParent( SKNodeRef ref );
void SKNodeRemoveAllChildren( SKNodeRef ref );
void SKNodeRemoveChildrenInArray( SKNodeRef ref, CFArrayRef children );
Boolean SKNodeInParentHierarchy( SKNodeRef ref, SKNodeRef parent );

// Accessing by name
CFStringRef SKNodeName( SKNodeRef ref );
void SKNodeSetName( SKNodeRef ref, CFStringRef name );
SKNodeRef SKNodeChildNodeWithName( SKNodeRef ref, CFStringRef name );
void SKNodeEnumerateChildNodesWithName( SKNodeRef ref, CFStringRef name, SKNodeEnumerateChildNodesWithNameCallbackType callback, void *userData );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CFArrayRef SKNodeObjectForKeyedSubscript( SKNodeRef ref, CFStringRef name );
#endif

// Visibility
CGFloat SKNodeAlpha( SKNodeRef ref );
void SKNodeSetAlpha( SKNodeRef ref, CGFloat alpha );
Boolean SKNodeIsHidden( SKNodeRef ref );
void SKNodeSetHidden( SKNodeRef ref, Boolean flag );

// Running actions
void SKNodeRunAction( SKNodeRef ref, SKActionRef action );
void SKNodeRunActionCompletion( SKNodeRef ref, SKActionRef action, SKNodeRunActionCompletionCallbackType callback, void *userData );
void SKNodeRunActionWithKey( SKNodeRef ref, SKActionRef action, CFStringRef key );
CGFloat SKNodeSpeed( SKNodeRef ref );
void SKNodeSetSpeed( SKNodeRef ref, CGFloat speed );
Boolean SKNodeIsPaused( SKNodeRef ref );
void SKNodeSetPaused( SKNodeRef ref, Boolean flag );
SKActionRef SKNodeActionForKey( SKNodeRef ref, CFStringRef key );
Boolean SKNodeHasActions( SKNodeRef ref );
void SKNodeRemoveAllActions( SKNodeRef ref );
void SKNodeRemoveActionForKey( SKNodeRef ref, CFStringRef key );

// Physics behaviors
SKPhysicsBodyRef SKNodePhysicsBody( SKNodeRef ref );
void SKNodeSetPhysicsBody( SKNodeRef ref, SKPhysicsBodyRef body );

// Constraining
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CFArrayRef SKNodeConstraints( SKNodeRef ref );
void SKNodeSetConstraints( SKNodeRef ref, CFArrayRef constraints );
SKReachConstraintsRef SKNodeReachConstraints( SKNodeRef ref );
void SKNodeSetReachConstraints( SKNodeRef ref, SKReachConstraintsRef constraints );
#endif

// Collisions
Boolean SKNodeIntersectsNode( SKNodeRef ref, SKNodeRef otherNode );

// User input
Boolean SKNodeIsUserInteractionEnabled( SKNodeRef ref );
void SKNodeSetUserInteractionEnabled( SKNodeRef ref, Boolean flag );

// Hit testing
Boolean SKNodeContainsPoint( SKNodeRef ref, CGPoint pt );
SKNodeRef SKNodeNodeAtPoint( SKNodeRef ref, CGPoint pt );
CFArrayRef SKNodeNodesAtPoint( SKNodeRef ref, CGPoint pt );

// Converting between coordinate systems
CGPoint SKNodeConvertPointFromNode( SKNodeRef ref, CGPoint pt, SKNodeRef otherNode );
CGPoint SKNodeConvertPointToNode( SKNodeRef ref, CGPoint pt, SKNodeRef otherNode );

// Custom data
CFMutableDictionaryRef SKNodeUserData( SKNodeRef ref );
void SKNodeSetUserData( SKNodeRef ref, CFMutableDictionaryRef userData );

// Accessibility
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFArrayRef SKNodeAccessibilityChildren( SKNodeRef ref );
void SKNodeSetAccessibilityChildren( SKNodeRef ref, CFArrayRef children );
CGRect SKNodeAccessibilityFrame( SKNodeRef ref );
void SKNodeSetAccessibilityFrame( SKNodeRef ref, CGRect frame );
CFStringRef SKNodeAccessibilityHelp( SKNodeRef ref );
void SKNodeSetAccessibilityHelp( SKNodeRef ref, CFStringRef help );
CFStringRef SKNodeAccessibilityLabel( SKNodeRef ref );
void SKNodeSetAccessibilityLabel( SKNodeRef ref, CFStringRef label );
CFTypeRef SKNodeAccessibilityParent( SKNodeRef ref );
void SKNodeSetAccessibilityParent( SKNodeRef ref, CFTypeRef parent );
CFStringRef SKNodeAccessibilityRole( SKNodeRef ref );
void SKNodeSetAccessibilityRole( SKNodeRef ref, CFStringRef role );
CFStringRef SKNodeAccessibilityRoleDescription( SKNodeRef ref );
void SKNodeSetAccessibilityRoleDescription( SKNodeRef ref, CFStringRef description );
CFStringRef SKNodeAccessibilitySubrole( SKNodeRef ref );
void SKNodeSetAccessibilitySubrole( SKNodeRef ref, CFStringRef subrole );
Boolean SKNodeIsAccessibilityElement( SKNodeRef ref );
void SKNodeSetAccessibilityElement( SKNodeRef ref, Boolean flag );
Boolean SKNodeIsAccessibilityEnabled( SKNodeRef ref );
void SKNodeSetAccessibilityEnabled( SKNodeRef ref, Boolean flag );
CFTypeRef SKNodeAccessibilityHitTest( SKNodeRef ref, CGPoint pt );
#endif

// NSEvent (SKNodeEvent)
CGPoint SKNodeLocationInNode( SKNodeRef ref );


#pragma mark - SKPhysicsBody
///////////////////////////////////////////////////
//                SKPhysicsBody                  //
///////////////////////////////////////////////////

// Create from shape
SKPhysicsBodyRef SKPhysicsBodyWithCircle( CGFloat radius );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKPhysicsBodyRef SKPhysicsBodyWithCircleCenter( CGFloat radius, CGPoint pt );
#endif
SKPhysicsBodyRef SKPhysicsBodyWithRectangle( CGSize size );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKPhysicsBodyRef SKPhysicsBodyWithRectangleCenter( CGSize size, CGPoint pt );
#endif
SKPhysicsBodyRef SKPhysicsBodyWithPolygon( CGPathRef path );

// Create from texture
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKPhysicsBodyRef SKPhysicsBodyWithTexture( SKTextureRef texture, CGSize size );
SKPhysicsBodyRef SKPhysicsBodyWithTextureAlphaThreshold( SKTextureRef texture, float alphaThreshold, CGSize size );

// Create from collection of bodies
SKPhysicsBodyRef SKPhysicsBodyWithBodies( CFArrayRef bodies );
#endif

// Create edge-based
SKPhysicsBodyRef SKPhysicsBodyWithEdgeLoopFromRect( CGRect r );
SKPhysicsBodyRef SKPhysicsBodyWithEdgeFromPoint( CGPoint fromPt, CGPoint toPt );
SKPhysicsBodyRef SKPhysicsBodyWithEdgeLoopFromPath( CGPathRef path );
SKPhysicsBodyRef SKPhysicsBodyWithEdgeChainFromPath( CGPathRef path );

// Forces
Boolean SKPhysicsBodyAffectedByGravity( SKPhysicsBodyRef ref );
void SKPhysicsBodySetAffectedByGravity( SKPhysicsBodyRef ref, Boolean flag );
Boolean SKPhysicsBodyAllowsRotation( SKPhysicsBodyRef ref );
void SKPhysicsBodySetAllowsRotation( SKPhysicsBodyRef ref, Boolean flag );
Boolean SKPhysicsBodyIsDynamic( SKPhysicsBodyRef ref );
void SKPhysicsBodySetDynamic( SKPhysicsBodyRef ref, Boolean flag );

// Physical properties
CGFloat SKPhysicsBodyMass( SKPhysicsBodyRef ref );
void SKPhysicsBodySetMass( SKPhysicsBodyRef ref, CGFloat mass );
CGFloat SKPhysicsBodyDensity( SKPhysicsBodyRef ref );
void SKPhysicsBodySetDensity( SKPhysicsBodyRef ref, CGFloat density );
CGFloat SKPhysicsBodyArea( SKPhysicsBodyRef ref );
CGFloat SKPhysicsBodyFriction( SKPhysicsBodyRef ref );
void SKPhysicsBodySetFriction( SKPhysicsBodyRef ref, CGFloat friction );
CGFloat SKPhysicsBodyRestitution( SKPhysicsBodyRef ref );
void SKPhysicsBodySetRestitution( SKPhysicsBodyRef ref, CGFloat restitution );
CGFloat SKPhysicsBodyLinearDamping( SKPhysicsBodyRef ref );
void SKPhysicsBodySetLinearDamping( SKPhysicsBodyRef ref, CGFloat damping );
CGFloat SKPhysicsBodyAngularDamping( SKPhysicsBodyRef ref );
void SKPhysicsBodySetAngularDamping( SKPhysicsBodyRef ref, CGFloat damping );

// Collisions and contacts
UInt32 SKPhysicsBodyCategoryBitMask( SKPhysicsBodyRef ref );
void SKPhysicsBodySetCategoryBitMask( SKPhysicsBodyRef ref, UInt32 bitMask );
UInt32 SKPhysicsBodyCollisionBitMask( SKPhysicsBodyRef ref );
void SKPhysicsBodySetCollisionBitMask( SKPhysicsBodyRef ref, UInt32 bitMask );
Boolean SKPhysicsBodyUsesPreciseCollisionDetection( SKPhysicsBodyRef ref );
void SKPhysicsBodySetUsesPreciseCollisionDetection( SKPhysicsBodyRef ref, Boolean flag );
UInt32 SKPhysicsBodyContactTestBitMask( SKPhysicsBodyRef ref );
void SKPhysicsBodySetContactTestBitMask( SKPhysicsBodyRef ref, UInt32 bitMask );
CFArrayRef SKPhysicsBodyAllContactedBodies( SKPhysicsBodyRef ref );

// Forces and impulses
void SKPhysicsBodyApplyForce( SKPhysicsBodyRef ref, CGVector force );
void SKPhysicsBodyApplyTorque( SKPhysicsBodyRef ref, CGFloat torque );
void SKPhysicsBodyApplyForceAtPoint( SKPhysicsBodyRef ref, CGVector force, CGPoint pt );
void SKPhysicsBodyApplyImpulse( SKPhysicsBodyRef ref, CGVector impulse );
void SKPhysicsBodyApplyAngularImpulse( SKPhysicsBodyRef ref, CGFloat impulse );
void SKPhysicsBodyApplyImpulseAtPoint( SKPhysicsBodyRef ref, CGVector impulse, CGPoint pt );

// Position and velocity
CGVector SKPhysicsBodyVelocity( SKPhysicsBodyRef ref );
void SKPhysicsBodySetVelocity( SKPhysicsBodyRef ref, CGVector velocity );
CGFloat SKPhysicsBodyAngularVelocity( SKPhysicsBodyRef ref );
void SKPhysicsBodySetAngularVelocity( SKPhysicsBodyRef ref, CGFloat velocity );
Boolean SKPhysicsBodyIsResting( SKPhysicsBodyRef ref );
void SKPhysicsBodySetResting( SKPhysicsBodyRef ref, Boolean flag );

// Node
SKNodeRef SKPhysicsBodyNode( SKPhysicsBodyRef ref );

// Joints
CFArrayRef SKPhysicsBodyJoints( SKPhysicsBodyRef ref );

// Fiels
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
UInt32 SKPhysicsBodyFieldBitMask( SKPhysicsBodyRef ref );
void SKPhysicsBodySetFieldBitMask( SKPhysicsBodyRef ref, UInt32 bitMask );
CGFloat SKPhysicsBodyCharge( SKPhysicsBodyRef ref );
void SKPhysicsBodySetCharge( SKPhysicsBodyRef ref, CGFloat charge );

// Pinning to node parent
Boolean SKPhysicsBodyPinned( SKPhysicsBodyRef ref );
void SKPhysicsBodySetPinned( SKPhysicsBodyRef ref, Boolean flag );
#endif



#pragma mark - SKPhysicsContact
///////////////////////////////////////////////////
//              SKPhysicsContact                 //
///////////////////////////////////////////////////

// Properties
SKPhysicsBodyRef SKPhysicsContactBodyA( SKPhysicsContactRef ref );
SKPhysicsBodyRef SKPhysicsContactBodyB( SKPhysicsContactRef ref );
CGPoint SKPhysicsContactPoint( SKPhysicsContactRef ref );
CGFloat SKPhysicsContactCollisionImpulse( SKPhysicsContactRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGVector SKPhysicsContactNormal( SKPhysicsContactRef ref );
#endif



#pragma mark - SKPhysicsJoint
///////////////////////////////////////////////////
//                SKPhysicsJoint                 //
///////////////////////////////////////////////////

// Bodies
SKPhysicsBodyRef SKPhysicsJointBodyA( SKPhysicsJointRef ref );
void SKPhysicsJointSetBodyA( SKPhysicsJointRef ref, SKPhysicsBodyRef body );
SKPhysicsBodyRef SKPhysicsJointBodyB( SKPhysicsJointRef ref );
void SKPhysicsJointSetBodyB( SKPhysicsJointRef ref, SKPhysicsBodyRef body );

// Stress and speed
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGVector SKPhysicsJointReactionForce( SKPhysicsJointRef ref );
CGFloat SKPhysicsJointReactionTorque( SKPhysicsJointRef ref );
#endif


#pragma mark - SKPhysicsJointFixed
///////////////////////////////////////////////////
//           SKPhysicsJointFixed                 //
///////////////////////////////////////////////////
// Create
SKPhysicsJointFixedRef SKPhysicsJointFixedWithBodyAB( SKPhysicsBodyRef bodyA, SKPhysicsBodyRef bodyB, CGPoint anchor );


#pragma mark - SKPhysicsJointLimit
///////////////////////////////////////////////////
//             SKPhysicsJointLimit               //
///////////////////////////////////////////////////
// Create
SKPhysicsJointLimitRef SKPhysicsJointLimitWithBodyAB( SKPhysicsBodyRef bodyA, SKPhysicsBodyRef bodyB, CGPoint anchorA, CGPoint anchorB );

// Configure
CGFloat SKPhysicsJointLimitMaxLength( SKPhysicsJointLimitRef ref );
void SKPhysicsJointLimitSetMaxLength( SKPhysicsJointLimitRef ref, CGFloat length );


#pragma mark - SKPhysicsJointPin
///////////////////////////////////////////////////
//               SKPhysicsJointPin               //
///////////////////////////////////////////////////
// Create
SKPhysicsJointPinRef SKPhysicsJointPinWithBodyAB( SKPhysicsBodyRef bodyA, SKPhysicsBodyRef bodyB, CGPoint anchor );

// Configure
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGFloat SKPhysicsJointPinRotationSpeed( SKPhysicsJointPinRef ref );
void SKPhysicsJointPinSetRotationSpeed( SKPhysicsJointPinRef ref, CGFloat speed );
#endif
Boolean SKPhysicsJointPinShouldEnableLimits( SKPhysicsJointPinRef ref );
void SKPhysicsJointPinSetShouldEnableLimits( SKPhysicsJointPinRef ref, Boolean flag );
CGFloat SKPhysicsJointPinLowerAngleLimit( SKPhysicsJointPinRef ref );
void SKPhysicsJointPinSetLowerAngleLimit( SKPhysicsJointPinRef ref, CGFloat limit );
CGFloat SKPhysicsJoingPinUpperAngleLimit( SKPhysicsJointPinRef ref );
void SKPhysicsJointPinSetUpperAngleLimit( SKPhysicsJointPinRef ref, CGFloat limit );
CGFloat SKPhysicsJoingPinFrictionTorgue( SKPhysicsJointPinRef ref );
void SKPhysicsJointPinSetFrictionTorque( SKPhysicsJointPinRef ref, CGFloat torque );


#pragma mark - SKPhysicsJointSliding
///////////////////////////////////////////////////
//             SKPhysicsJointSliding             //
///////////////////////////////////////////////////
// Create
SKPhysicsJointSlidingRef SKPhysicsJointSlidingWithBodyAB( SKPhysicsBodyRef bodyA, SKPhysicsBodyRef bodyB, CGPoint anchor, CGVector axis );

// Configure
Boolean SKPhysicsJointSlidingShouldEnableLimits( SKPhysicsJointSlidingRef ref );
void SKPhysicsJointSlidingSetShouldEnableLimits( SKPhysicsJointSlidingRef ref, Boolean flag );
CGFloat SKPhysicsJointSlidingLowerDistanceLimit( SKPhysicsJointSlidingRef ref );
void SKPhysicsJointSlidingSetLowerDistanceLimit( SKPhysicsJointSlidingRef ref, CGFloat limit );
CGFloat SKPhysicsJointSlidingUpperDistanceLimit( SKPhysicsJointSlidingRef ref );
void SKPhysicsJointSlidingSetUpperDistanceLimit( SKPhysicsJointSlidingRef ref, CGFloat limit );



#pragma mark - SKPhysicsJointSpring
///////////////////////////////////////////////////
//             SKPhysicsJointSpring              //
///////////////////////////////////////////////////
// Create
SKPhysicsJointSpringRef SKPhysicsJointSpringWithBodyAB( SKPhysicsBodyRef bodyA, SKPhysicsBodyRef bodyB, CGPoint anchorA, CGPoint anchorB );

// Configure
CGFloat SKPhysicsJointSpringDamping( SKPhysicsJointSpringRef ref );
void SKPhysicsJointSpringSetDamping( SKPhysicsJointSpringRef ref, CGFloat damping );
CGFloat SKPhysicsJointSpringFrequency( SKPhysicsJointSpringRef ref );
void SKPhysicsJointSpringSetFrequency( SKPhysicsJointSpringRef ref, CGFloat frequency );





#pragma mark - SKPhysicsWorld
///////////////////////////////////////////////////
//                SKPhysicsWorld                 //
///////////////////////////////////////////////////
typedef BOOL (*SKPhysicsWorldEnumerateBodiesAlongRayCallbackType)(SKPhysicsWorld*,SKPhysicsBody*,CGPoint,CGVector,void*);
typedef BOOL (*SKPhysicsWorldEnumerateBodiesAtPointCallbackType)(SKPhysicsWorld*,SKPhysicsBody*,void*);
typedef BOOL (*SKPhysicsWorldEnumerateBodiesInRectCallbackType)(SKPhysicsWorld*,SKPhysicsBody*,void*);

typedef void (*SKPhysicsContactDelegateCallbackType)(long,SKPhysicsContact*,void*);

enum {
    skPhysicsContactDidBegin,
    skPhysicsContactDidEnd
};

@interface CocoaSKPhysicsContactDelegate : NSObject <SKPhysicsContactDelegate> {
    SKPhysicsContactDelegateCallbackType callback;
    void *userData;
}
@property (assign) SKPhysicsContactDelegateCallbackType callback;
@property (assign) void *userData;

- (id)initWithCallback:(void *)cb userData:(void*)ud;
@end

// Configure
CGVector SKPhysicsWorldGravity( SKPhysicsWorldRef ref );
void SKPhysicsWorldSetGravity( SKPhysicsWorldRef ref, CGVector gravity );
CGFloat SKPhysicsWorldSpeed( SKPhysicsWorldRef ref );
void SKPhysicsWorldSetSpeed( SKPhysicsWorldRef ref, CGFloat speed );

// Joints
void SKPhysicsWorldAddJoint( SKPhysicsWorldRef ref, SKPhysicsJointRef joint );
void SKPhysicsWorldRemoveAllJoints( SKPhysicsWorldRef ref );
void SKPhysicsWorldRemoveJoint( SKPhysicsWorldRef ref, SKPhysicsJointRef joint );

// Collisions
void SKPhysicsWorldSetContactDelegateCallback( SKPhysicsWorldRef ref, SKPhysicsContactDelegateCallbackType callback, void *userData );

// Searching scene for bodies
SKPhysicsBodyRef SKPhysicsWorldBodyAlongRay( SKPhysicsWorldRef ref, CGPoint startPt, CGPoint endPt );
SKPhysicsBodyRef SKPhysicsWorldBodyAtPoint( SKPhysicsWorldRef ref, CGPoint pt );
SKPhysicsBodyRef SKPhysicsWorldBodyInRect( SKPhysicsWorldRef ref, CGRect r );
void SKPhysicsWorldEnumerateBodiesAlongRay( SKPhysicsWorldRef ref, CGPoint startPt, CGPoint endPt, SKPhysicsWorldEnumerateBodiesAlongRayCallbackType callback, void *userData );
void SKPhysicsWorldEnumerateBodiesAtPoint( SKPhysicsWorldRef ref, CGPoint pt, SKPhysicsWorldEnumerateBodiesAtPointCallbackType callback, void *userData );
void SKPhysicsWorldEnumerateBodiesInRect( SKPhysicsWorldRef ref, CGRect r, SKPhysicsWorldEnumerateBodiesInRectCallbackType callback, void *userData );

// Sampling fields
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
vector_float3 SKPhysicsWorldSampleFields( SKPhysicsWorldRef ref, vector_float3 position );
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKReachConstraints
///////////////////////////////////////////////////
//             SKReachConstraints                //
///////////////////////////////////////////////////

// Create
SKReachConstraintsRef SKReachConstraintsWithAngleLimits( CGFloat lowerAngleLimit, CGFloat upperAngleLimit );

// Angle limits
CGFloat SKReachConstraintsLowerAngleLimit( SKReachConstraintsRef ref );
void SKReachConstraintsSetLowerAngleLimit( SKReachConstraintsRef ref, CGFloat angleLimit );
CGFloat SKReachConstraintsUpperAngleLimit( SKReachConstraintsRef ref );
void SKReachConstraintsSetUpperAngleLimit( SKReachConstraintsRef ref, CGFloat angleLimit );

#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
#pragma mark - SKReferenceNode
///////////////////////////////////////////////////
//               SKReferenceNode                 //
///////////////////////////////////////////////////

// Init
SKReferenceNodeRef SKReferenceNodeWithURL( CFURLRef url );
SKReferenceNodeRef SKReferenceNodeWithFileNamed( CFStringRef name );
SKReferenceNodeRef SKReferenceNodeWithCoder( CoderRef coder );

// Regenerating
void SKReferenceNodeResolve( SKReferenceNodeRef ref );

// Loading
void SKReferenceNodeDidLoad( SKReferenceNodeRef ref, SKNodeRef node );

#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKRegion
///////////////////////////////////////////////////
//                  SKRegion                     //
///////////////////////////////////////////////////

// Create
SKRegionRef SKRegionInfinite( void );
SKRegionRef SKRegionWithSize( CGSize size );
SKRegionRef SKRegionWithRadius( float radius );
SKRegionRef SKRegionWithPath( CGPathRef path );
SKRegionRef SKRegionInverse( SKRegionRef ref );
SKRegionRef SKRegionByDifferenceFromRegion( SKRegionRef ref, SKRegionRef otherRegion );
SKRegionRef SKRegionByIntersectionWithRegion( SKRegionRef ref, SKRegionRef otherRegion );
SKRegionRef SKRegionByUnionWithRegion( SKRegionRef ref, SKRegionRef otherRegion );

// Interacting
CGPathRef SKRegionPath( SKRegionRef ref );
Boolean SKRegionContainsPoint( SKRegionRef ref, CGPoint pt );

#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
#pragma mark - SKRenderer
///////////////////////////////////////////////////
//                 SKRenderer                    //
///////////////////////////////////////////////////

// First steps
SKSceneRef SKRendererScene( SKRendererRef ref );
void SKRendererSetScene( SKRendererRef ref, SKSceneRef scene );

// Scene update cycle
void SKRendererUpdateAtTime( SKRendererRef ref, CFTimeInterval currentTime );

// Performance related toggles
Boolean SKRendererIgnoresSiblingOrder( SKRendererRef ref );
void SKRendererSetIgnoresSiblingOrder( SKRendererRef ref, Boolean flag );
Boolean SKRendererShouldCullNonVisibleNodes( SKRendererRef ref );
void SKRendererSetShouldCullNonVisibleNodes( SKRendererRef ref, Boolean flag );

// Visual statistics for debugging
Boolean SKRendererShowsNodeCount( SKRendererRef ref );
void SKRendererSetShowsNodeCount( SKRendererRef ref, Boolean flag );
Boolean SKRendererShowsDrawCount( SKRendererRef ref );
void SKRendererSetShowsDrawCount( SKRendererRef ref, Boolean flag );
Boolean SKRendererShowsQuadCount( SKRendererRef ref );
void SKRendererSetShowsQuadCount( SKRendererRef ref, Boolean flag );
Boolean SKRendererShowsPhysics( SKRendererRef ref );
void SKRendererSetShowsPhysics( SKRendererRef ref, Boolean flag );
Boolean SKRendererShowsFields( SKRendererRef ref );
void SKRendererSetShowsFields( SKRendererRef ref, Boolean flag );

#endif


#pragma mark - SKScene
///////////////////////////////////////////////////
//                  SKScene                      //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
typedef void (*SKSceneDelegateCallbackType)(long,SKScene*,NSTimeInterval,void*);

enum {
    skSceneUpdate,
    skSceneDidEvaluateActions,
    skSceneDidSimulatePhysics,
    skSceneDidApplyConstraints,
    skSceneDidFinishUpdate
};

@interface CocoaSKSceneDelegate : NSObject <SKSceneDelegate> {
    SKSceneDelegateCallbackType callback;
    void *userData;
}
@property (assign) SKSceneDelegateCallbackType callback;
@property (assign) void *userData;

- (id)initWithCallback:(void *)cb userData:(void*)ud;
@end
#endif

// Create
SKSceneRef SKSceneWithSize( CGSize size );
CGSize SKSceneSize( SKSceneRef ref );
void SKSceneSetSize( SKSceneRef ref, CGSize size );

// Strech content to fit view
SKSceneScaleMode skSceneScaleMode( SKSceneRef ref );
void SKSceneSetScaleMode( SKSceneRef ref, SKSceneScaleMode scaleMode );

// Configure Viewport
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKCameraNodeRef SKSceneCamera( SKSceneRef ref );
void SKSceneSetCamera( SKSceneRef ref, SKCameraNodeRef camera );
#endif
CGPoint SKSceneAnchorPoint( SKSceneRef ref );
void SKSceneSetAnchorPoint( SKSceneRef ref, CGPoint pt );

// Configure delegate
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void SKSceneSetDelegateCallback( SKSceneRef ref, SKSceneDelegateCallbackType callback, void *userData );
#endif

// Background appearance
NSInteger SKSceneView( SKSceneRef ref );
ColorRef SKSceneBackgroundColor( SKSceneRef ref );
void SKSceneSetBackgroundColor( SKSceneRef ref, ColorRef col );

// Physics properties
SKPhysicsWorldRef SKScenePhysicsWorld( SKSceneRef ref );

// Positional audio
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKNodeRef SKSceneListener( SKSceneRef ref );
void SKSceneSetListener( SKSceneRef ref, SKNodeRef listener );
#endif

// Converting between coordinate systems
CGPoint SKSceneConvertPointFromView( SKSceneRef ref, CGPoint pt );
CGPoint SKSceneConvertPointToView( SKSceneRef ref, CGPoint pt );


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKShader
///////////////////////////////////////////////////
//                 SKShader                      //
///////////////////////////////////////////////////

// Create
SKShaderRef SKShaderInit( void );
SKShaderRef SKShaderWithFileNamed( CFStringRef name );
SKShaderRef SKShaderWithSource( CFStringRef source );
SKShaderRef SKShaderWithSourceUniforms( CFStringRef source, CFArrayRef uniforms );

// Uniform
void SKShaderAddUniform( SKShaderRef ref, SKUniformRef uniform );
void SKShaderRemoveUniformNamed( SKShaderRef ref, CFStringRef name );
CFArrayRef SKShaderUniforms( SKShaderRef ref );
void SKShaderSetUniforms( SKShaderRef ref, CFArrayRef uniforms );
SKUniformRef SKShaderUniformNamed( SKShaderRef ref, CFStringRef name );

// Attribute
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFArrayRef SKShaderAttributes( SKShaderRef ref );
void SKShaderSetAttributes( SKShaderRef ref, CFArrayRef attributes );
#endif

// Source code
CFStringRef SKShaderSource( SKShaderRef ref );
void SKShaderSetSource( SKShaderRef ref, CFStringRef source );

#endif


#pragma mark - SKShapeNode
///////////////////////////////////////////////////
//               SKShapeNode                     //
///////////////////////////////////////////////////

// Create from path
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKShapeNodeRef SKShapeNodeWithPath( CGPathRef path );
SKShapeNodeRef SKShapeNodeWithPathCentered( CGPathRef path, Boolean centered );
#endif
CGPathRef SKShapeNodePath( SKShapeNodeRef ref );
void SKShapeNodeSetPath( SKShapeNodeRef ref, CGPathRef path );

// Create from rectangle
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKShapeNodeRef SKShapeNodeWithRect( CGRect r );
SKShapeNodeRef SKShapeNodeWithRectOfSize( CGSize size );
SKShapeNodeRef SKShapeNodeWithRectCornerRadius( CGRect r, CGFloat radius );
SKShapeNodeRef SKShapeNodeWithRectOfSizeCornerRadius( CGSize size, CGFloat radius );

// Create circle shape
SKShapeNodeRef SKShapeNodeWithCircleOfRadius( CGFloat radius );

// Create ellipse shape
SKShapeNodeRef SKShapeNodeWithEllipseOfSize( CGSize size );
SKShapeNodeRef SKShapeNodeWithEllipseInRect( CGRect r );

// Create from array of points
SKShapeNodeRef SKShapeNodeWithPoints( CGPoint *size, long count );
SKShapeNodeRef SKShapeNodeWithSplinePoints( CGPoint *size, long count );
#endif

// Fill
ColorRef SKShapeNodeFillColor( SKShapeNodeRef ref );
void SKShapeNodeSetFillColor( SKShapeNodeRef ref, ColorRef col );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureRef SKShapeNodeFillTexture( SKShapeNodeRef ref );
void SKShapeNodeSetFillTexture( SKShapeNodeRef ref, SKTextureRef texture );
#endif

// Stroke
CGFloat SKShapeNodeLineWidth( SKShapeNodeRef ref );
void SKShapeNodeSetLineWidth( SKShapeNodeRef ref, CGFloat lineWidth );
ColorRef SKShapeNodeStrokeColor( SKShapeNodeRef ref );
void SKShapeNodeSetStrokeColor( SKShapeNodeRef ref, ColorRef col );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureRef SKShapeNodeStrokeTexture( SKShapeNodeRef ref );
void SKShapeNodeSetStrokeTexture( SKShapeNodeRef ref, SKTextureRef texture );
#endif
CGFloat SKShapeNodeGlowWidth( SKShapeNodeRef ref );
void SKShapeNodeSetGlowWidth( SKShapeNodeRef ref, CGFloat glowWidth );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGLineCap SKShapeNodeLineCap( SKShapeNodeRef ref );
void SKShapeNodeSetLineCap( SKShapeNodeRef ref, CGLineCap lineCap );
CGLineJoin SKShapeNodeLineJoin( SKShapeNodeRef ref );
void SKShapeNodeSetLineJoin( SKShapeNodeRef ref, CGLineJoin lineJoin );
CGFloat SKShapeNodeMiterLimit( SKShapeNodeRef ref );
void SKShapeNodeSetMiterLimit( SKShapeNodeRef ref, CGFloat miterLimit );
#endif
Boolean SKShapeNodeIsAntialiased( SKShapeNodeRef ref );
void SKShapeNodeSetAntialiased( SKShapeNodeRef ref, Boolean flag );

// Alpha blending
SKBlendMode SKShapeNodeBlendMode( SKShapeNodeRef ref );
void SKShapeNodeSetBlendMode( SKShapeNodeRef ref, SKBlendMode mode );

// Stroke length
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGFloat SKShapeNodeLineLength( SKShapeNodeRef ref );

// Customizing stroke or fill drawing
SKShaderRef SKShapeNodeStrokeShader( SKShapeNodeRef ref );
void SKShapeNodeSetStrokeShader( SKShapeNodeRef ref, SKShaderRef shader );
SKShaderRef SKShapeNodeFillShader( SKShapeNodeRef ref );
void SKShapeNodeSetFillShader( SKShapeNodeRef ref, SKShaderRef shader );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFDictionaryRef SKShapeNodeAttributeValues( SKShapeNodeRef ref );
void SKShapeNodeSetAttributeValues( SKShapeNodeRef ref, CFDictionaryRef values );
void SKShapeNodeSetValueForAttributeNamed( SKShapeNodeRef ref, SKAttributeValueRef value, CFStringRef key );
SKAttributeValueRef SKShapeNodeValueForAttributeNamed( SKShapeNodeRef ref, CFStringRef key );
#endif



#pragma mark - SKSpriteNode
///////////////////////////////////////////////////
//               SKSpriteNode                    //
///////////////////////////////////////////////////

// Create from filename
SKSpriteNodeRef SKSpriteNodeWithImageNamed( CFStringRef name );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKSpriteNodeRef SKSpriteNodeWithNormalMapped( CFStringRef name, Boolean normalMapped );
#endif

// Create from texture
SKSpriteNodeRef SKSpriteNodeWithColor( ColorRef col, CGSize size );
SKSpriteNodeRef SKSpriteNodeWithTexture( SKTextureRef texture );
SKSpriteNodeRef SKSpriteNodeWithTextureSize( SKTextureRef texture, CGSize size );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKSpriteNodeRef SKSpriteNodeWithTextureNormalMap( SKTextureRef texture, SKTextureRef normalMap );
#endif
SKTextureRef SKSpriteNodeTexture( SKSpriteNodeRef ref );
void SKSpriteNodeSetTexture( SKSpriteNodeRef ref, SKTextureRef texture );

// Create from archive
SKSpriteNodeRef SKSpriteNodeWithCoder( CoderRef coder );

// Size and position
CGSize SKSpriteNodeSize( SKSpriteNodeRef ref );
void SKSpriteNodeSetSize( SKSpriteNodeRef ref, CGSize size );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void SKSpriteNodeScaleToSize( SKSpriteNodeRef ref, CGSize size );
#endif
CGPoint SKSpriteNodeAnchorPoint( SKSpriteNodeRef ref );
void SKSpriteNodeSetAnchorPoint( SKSpriteNodeRef ref, CGPoint pt );

// Scaling in nine parts
CGRect SKSpriteNodeCenterRect( SKSpriteNodeRef ref );
void SKSpriteNodeSetCenterRect( SKSpriteNodeRef ref, CGRect r );

// Tint
ColorRef SKSpriteNodeColor( SKSpriteNodeRef ref );
void SKSpriteNodeSetColor( SKSpriteNodeRef ref, ColorRef col );
CGFloat SKSpriteNodeColorBlendFactor( SKSpriteNodeRef ref );
void SKSpriteNodeSetColorBlendFactor( SKSpriteNodeRef ref, CGFloat factor );

// Alpha blending
SKBlendMode SKSpriteNodeBlendMode( SKSpriteNodeRef ref );
void SKSpriteNodeSetBlendMode( SKSpriteNodeRef ref, SKBlendMode mode );

// Lighting
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
UInt32 SKSpriteNodeLightingBitMask( SKSpriteNodeRef ref );
void SKSpriteNodeSetLightingBitMask( SKSpriteNodeRef ref, UInt32 bitMask );
UInt32 SKSpriteNodeShadowedBitMask( SKSpriteNodeRef ref );
void SKSpriteNodeSetShadowedBitMask( SKSpriteNodeRef ref, UInt32 bitMask );
UInt32 SKSpriteNodeShadowCastBitMask( SKSpriteNodeRef ref );
void SKSpriteNodeSetShadowCastBitMask( SKSpriteNodeRef ref, UInt32 bitMask );
SKTextureRef SKSpriteNodeNormalTexture( SKSpriteNodeRef ref );
void SKSpriteNodeSetNormalTexture( SKSpriteNodeRef ref, SKTextureRef texture );

// Custom shader
SKShaderRef SKSpriteNodeShader( SKSpriteNodeRef ref );
void SKSpriteNodeSetShader( SKSpriteNodeRef ref, SKShaderRef shader );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFDictionaryRef SKSpriteNodeAttributeValues( SKSpriteNodeRef ref );
void SKSpriteNodeSetAttributeValues( SKSpriteNodeRef ref, CFDictionaryRef values );
void SKSpriteNodeSetValueForAttributeNamed( SKSpriteNodeRef ref, SKAttributeValueRef value, CFStringRef key );
SKAttributeValueRef SKSpriteNodeValueForAttributeNamed( SKSpriteNodeRef ref, CFStringRef key );
#endif



#pragma mark - SKTexture
///////////////////////////////////////////////////
//                  SKTexture                    //
///////////////////////////////////////////////////
typedef void (*SKTexturePreloadCallbackType)(SKTexture*,void*);
typedef void (*SKTexturePreloadTexturesCallbackType)(void*);

/*
 Initializers
 */

// Filename
SKTextureRef SKTextureWithImageNamed( CFStringRef name );

// Texture within texture
SKTextureRef SKTextureWithRectInTexture( CGRect r, SKTextureRef texture );

// Image
SKTextureRef SKTextureWithImage( ImageRef image );
SKTextureRef SKTextureWithCGImage( CGImageRef image );

// Effects
SKTextureRef SKTextureByApplyingCIFilter( SKTextureRef ref, CIFilterRef filter );

// Data
SKTextureRef SKTextureWithData( CFDataRef dta, CGSize size );
SKTextureRef SKTextureWithDataRowLength( CFDataRef dta, CGSize size, UInt32 rowLength, UInt32 alignment );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureRef SKTextureWithDataFlipped( CFDataRef dta, CGSize size, Boolean flipped );
#endif

// Normal map
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureRef SKTextureByGeneratingNormalMap( SKTextureRef ref );
SKTextureRef SKTextureByGeneratingNormalMapWithSmoothness( SKTextureRef ref, CGFloat smoothness, CGFloat contrast );
#endif

// Noise textures
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureRef SKTextureVectorNoise( CGFloat smoothness, CGSize size );
SKTextureRef SKTextureNoise( CGFloat smoothness, CGSize size, Boolean grayscale );
#endif

// ===============

// Size
CGSize SKTextureSize( SKTextureRef ref );
CGRect SKTextureRect( SKTextureRef ref );

// Scaling behavior
SKTextureFilteringMode skTextureFilteringMode( SKTextureRef ref );
void SKTextureSetFilteringMode( SKTextureRef ref, SKTextureFilteringMode mode );
Boolean SKTextureUsesMipmaps( SKTextureRef ref );
void SKTextureSetUsesMipmaps( SKTextureRef ref, Boolean flag );

// Image
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGImageRef SKTextureCGImage( SKTextureRef ref );
#endif

// Preload
void SKTexturePreload( SKTextureRef ref, SKTexturePreloadCallbackType callback, void *userData );
void SKTexturePreloadTextures( CFArrayRef textures, SKTexturePreloadTexturesCallbackType callback, void *userData );



#pragma mark - SKTextureAtlas
///////////////////////////////////////////////////
//                SKTextureAtlas                 //
///////////////////////////////////////////////////
typedef void (*SKTextureAtlasPreloadCallbackType)(SKTextureAtlas*,void*);
typedef void (*SKTextureAtlasPreloadAtlasesCallbackType)(void*);
typedef void (*SKTextureAtlasPreloadAtlasesNamedCallbackType)(NSError*,NSArray*,void*);

// Create
SKTextureAtlasRef SKTextureAtlasNamed( CFStringRef name );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureAtlasRef SKTextureAtlasWithDictionary( CFDictionaryRef dict );
#endif

// Textures
SKTextureRef SKTextureAtlasTextureNamed( SKTextureAtlasRef ref, CFStringRef name );

// Preload
void SKTextureAtlasPreload( SKTextureAtlasRef ref, SKTextureAtlasPreloadCallbackType callback, void *userData );
void SKTextureAtlasPreloadAtlases( CFArrayRef textureAtlases, SKTextureAtlasPreloadAtlasesCallbackType callback, void *userData );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void SKTextureAtlasPreloadNamed( CFArrayRef textureAtlasNames, SKTextureAtlasPreloadAtlasesNamedCallbackType callback, void *userData );
#endif

// Filenames
CFArrayRef SKTextureAtlasTextureNames( SKTextureAtlasRef ref );



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - SKTileDefinition
///////////////////////////////////////////////////
//              SKTileDefinition                 //
///////////////////////////////////////////////////

// Create
SKTileDefinitionRef SKTileDefinitionWithTexture( SKTextureRef texture );
SKTileDefinitionRef SKTileDefinitionWithNormalTexture( SKTextureRef texture, SKTextureRef normalTexture, CGSize size );
SKTileDefinitionRef SKTileDefinitionWithSize( SKTextureRef texture, CGSize size );
SKTileDefinitionRef SKTileDefinitionWithNormalTextures( CFArrayRef textures, CFArrayRef normalTextures, CGSize size, CGFloat timePerFrame );
SKTileDefinitionRef SKTileDefinitionWithTextures( CFArrayRef textures, CGSize size, CGFloat timePerFrame );

// Flip
Boolean SKTileDefinitionFlipHorizontally( SKTileDefinitionRef ref );
void SKTileDefinitionSetFlipHorizontally( SKTileDefinitionRef ref, Boolean flag );
Boolean SKTileDefinitionFlipVertically( SKTileDefinitionRef ref );
void SKTileDefinitionSetFlipVertically( SKTileDefinitionRef ref, Boolean flag );

// Rotate
SKTileDefinitionRotation skTileDefinitionRotation( SKTileDefinitionRef ref );
void SKTileDefinitionSetRotation( SKTileDefinitionRef ref, SKTileDefinitionRotation rotation );

// Animated tile properties
CFArrayRef SKTileDefinitionTextures( SKTileDefinitionRef ref );
void SKTileDefinitionSetTextures( SKTileDefinitionRef ref, CFArrayRef textures );
CFArrayRef SKTileDefinitionNormalTextures( SKTileDefinitionRef ref );
void SKTileDefinitionSetNormalTextures( SKTileDefinitionRef ref, CFArrayRef textures );
CGFloat SKTileDefinitionTimePerFrame( SKTileDefinitionRef ref );
void SKTileDefinitionSetTimePerFrame( SKTileDefinitionRef ref, CGFloat timePerFrame );

// Custom data
CFMutableDictionaryRef SKTileDefinitionUserData( SKTileDefinitionRef ref );
void SKTileDefinitionSetUserData( SKTileDefinitionRef ref, CFMutableDictionaryRef userData );

// Properties
CFStringRef SKTileDefinitionName( SKTileDefinitionRef ref );
void SKTileDefinitionSetName( SKTileDefinitionRef ref, CFStringRef name );
NSUInteger SKTileDefinitionPlacementWeight( SKTileDefinitionRef ref );
void SKTileDefinitionSetPlacementWeight( SKTileDefinitionRef ref, NSUInteger weight );
CGSize SKTileDefinitionSize( SKTileDefinitionRef ref );
void SKTileDefinitionSetSize( SKTileDefinitionRef ref, CGSize size );


#pragma mark - SKTileGroup
///////////////////////////////////////////////////
//                   SKTileGroup                 //
///////////////////////////////////////////////////

// Create
SKTileGroupRef SKTileGroupEmpty( void );
SKTileGroupRef SKTileGroupWithTileDefinition( SKTileDefinitionRef definition );
SKTileGroupRef SKTileGroupWithRules( CFArrayRef rules );

// Properties
CFStringRef SKTileGroupName( SKTileGroupRef ref );
void SKTileGroupSetName( SKTileGroupRef ref, CFStringRef name );
CFArrayRef SKTileGroupRules( SKTileGroupRef ref );
void SKTileGroupSetRules( SKTileGroupRef ref, CFArrayRef rules );


#pragma mark - SKTileGroupRule
///////////////////////////////////////////////////
//                 SKTileGroupRule               //
///////////////////////////////////////////////////

// Create
SKTileGroupRuleRef SKTileGroupRuleWithAdjacency( SKTileAdjacencyMask mask, CFArrayRef tileDefinitions );

// Properties
SKTileAdjacencyMask SKTileGroupRuleAdjacency( SKTileGroupRuleRef ref );
void SKTileGroupRuleSetAdjacency( SKTileGroupRuleRef ref, SKTileAdjacencyMask mask );
CFStringRef SKTileGroupRuleName( SKTileGroupRuleRef ref );
void SKTileGroupRuleSetName( SKTileGroupRuleRef ref, CFStringRef name );
CFArrayRef SKTileGroupRuleTileDefinitions( SKTileGroupRuleRef ref );
void SKTileGroupRuleSetTileDefinitions( SKTileGroupRuleRef ref, CFArrayRef definitions );


#pragma mark - SKTileMapNode
///////////////////////////////////////////////////
//                 SKTileMapNode                 //
///////////////////////////////////////////////////

// Create
SKTileMapNodeRef SKTileMapNodeWithTileSet( SKTileSetRef tileSet, NSUInteger columns, NSUInteger rows, CGSize tileSize );
SKTileMapNodeRef SKTileMapNodeWithTileSetFillWithTileGroup( SKTileSetRef tileSet, NSUInteger columns, NSUInteger rows, CGSize tileSize, SKTileGroupRef group );
SKTileMapNodeRef SKTileMapNodeWithTileSetFillWithTileGroupLayout( SKTileSetRef tileSet, NSUInteger columns, NSUInteger rows, CGSize tileSize, CFArrayRef tileGroupLayout );

// Contents
Boolean SKTileMapNodeEnableAutomapping( SKTileMapNodeRef ref );
void SKTileMapNodeSetEnableAutomapping( SKTileMapNodeRef ref, Boolean flag );
void SKTileMapNodeFillWithTileGroup( SKTileMapNodeRef ref, SKTileGroupRef tileGroup );
void SKTileMapNodeSetTileGroupAndDefinition( SKTileMapNodeRef ref, SKTileGroupRef tileGroup, SKTileDefinitionRef tileDefinition, NSUInteger column, NSUInteger row );
void SKTileMapNodeSetTileGroup( SKTileMapNodeRef ref, SKTileGroupRef tileGroup, NSUInteger column, NSUInteger row );

// Position
CGPoint SKTileMapNodeAnchorPoint( SKTileMapNodeRef ref );
void SKTileMapNodeSetAnchorPoint( SKTileMapNodeRef ref, CGPoint pt );

// Size
CGSize SKTileMapNodeTileSize( SKTileMapNodeRef ref );
void SKTileMapNodeSetTileSize( SKTileMapNodeRef ref, CGSize size );
SKTileSetRef SKTileMapNodeTileSet( SKTileMapNodeRef ref );
void SKTileMapNodeSetTileSet( SKTileMapNodeRef ref, SKTileSetRef tileSet );
NSUInteger SKTileMapNodeNumberOfColumns( SKTileMapNodeRef ref );
void SKTileMapNodeSetNumberOfColumns( SKTileMapNodeRef ref, NSUInteger numberOfColumns );
NSUInteger SKTileMapNodeNumberOfRows( SKTileMapNodeRef ref );
void SKTileMapNodeSetNumberOfRows( SKTileMapNodeRef ref, NSUInteger numberOfRows );

// Properties
CGPoint SKTileMapNodeCenterOfTile( SKTileMapNodeRef ref, NSUInteger column, NSUInteger row );
NSUInteger SKTileMapNodeTileColumnIndexFromPosition( SKTileMapNodeRef ref, CGPoint position );
SKTileDefinitionRef SKTileMapNodeTileDefinition( SKTileMapNodeRef ref, NSUInteger column, NSUInteger row );
SKTileGroupRef SKTileMapNodeTileGroup( SKTileMapNodeRef ref, NSUInteger column, NSUInteger row );
NSUInteger SKTileMapNodeTileRowIndexFromPosition( SKTileMapNodeRef ref, CGPoint position );
CGSize SKTileMapNodeMapSize( SKTileMapNodeRef ref );

// Tinting
ColorRef SKTileMapNodeColor( SKTileMapNodeRef ref );
void SKTileMapNodeSetColor( SKTileMapNodeRef ref, ColorRef col );
CGFloat SKTileMapNodeColorBlendFactor( SKTileMapNodeRef ref );
void SKTileMapNodeSetColorBlendFactor( SKTileMapNodeRef ref, CGFloat factor );

// Lighting
UInt32 SKTileMapNodeLightingBitMask( SKTileMapNodeRef ref );
void SKTileMapNodeSetLightingBitMask( SKTileMapNodeRef ref, UInt32 bitMask );

// Alpha
SKBlendMode SKTileMapNodeBlendMode( SKTileMapNodeRef ref );
void SKTileMapNodeSetBlendMode( SKTileMapNodeRef ref, SKBlendMode mode );

// Shader
SKShaderRef SKTileMapNodeShader( SKTileMapNodeRef ref );
void SKTileMapNodeSetShader( SKTileMapNodeRef ref, SKShaderRef shader );
CFDictionaryRef SKTileMapNodeAttributeValues( SKTileMapNodeRef ref );
void SKTileMapNodeSetAttributeValues( SKTileMapNodeRef ref, CFDictionaryRef values );
void SKTileMapNodeSetValueForAttributeNamed( SKTileMapNodeRef ref, SKAttributeValueRef value, CFStringRef key );
SKAttributeValueRef SKTileMapNodeValueForAttributeNamed( SKTileMapNodeRef ref, CFStringRef key );



#pragma mark - SKTileSet
///////////////////////////////////////////////////
//                   SKTileSet                   //
///////////////////////////////////////////////////

// Create
SKTileSetRef SKTileSetNamed( CFStringRef name );
SKTileSetRef SKTileSetFromURL( CFURLRef url );
SKTileSetRef SKTileSetWithTileGroups( CFArrayRef tileGroups );
SKTileSetRef SKTileSetWithTileGroupsType( CFArrayRef tileGroups, SKTileSetType type );

// Properties
SKTileGroupRef SKTileSetDefaultTileGroup( SKTileSetRef ref );
void SKTileSetSetDefaultTileGroup( SKTileSetRef ref, SKTileGroupRef group );
CGSize SKTileSetDefaultTileSize( SKTileSetRef ref );
void SKTileSetSetDefaultTileSize( SKTileSetRef ref, CGSize size );
CFStringRef SKTileSetName( SKTileSetRef ref );
void SKTileSetSetName( SKTileSetRef ref, CFStringRef name );
CFArrayRef SKTileSetTileGroups( SKTileSetRef ref );
void SKTileSetSetTileGroups( SKTileSetRef ref, CFArrayRef groups );
SKTileSetType skTileSetType( SKTileSetRef ref );
void SKTileSetSetType( SKTileSetRef ref, SKTileSetType type );

#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
#pragma mark - SKTransformNode
///////////////////////////////////////////////////
//                SKTransformNode                //
///////////////////////////////////////////////////

// Rotate child nodes
CGFloat SKTransformNodeXRotation( SKTransformNodeRef ref );
void SKTransformNodeSetXRotation( SKTransformNodeRef ref, CGFloat rotation );
CGFloat SKTransformNodeYRotation( SKTransformNodeRef ref );
void SKTransformNodeSetYRotation( SKTransformNodeRef ref, CGFloat rotation );

void SKTransformNodeSetEulerAngles( SKTransformNodeRef ref, vector_float3 euler );
void SKTransformNodeSetQuaternion( SKTransformNodeRef ref, simd_quatf quaternion );
void SKTransformNodeSetRotationMatrix( SKTransformNodeRef ref, matrix_float3x3 matrix );

// Current rotation
vector_float3 SKTransformNodeEulerAngles( SKTransformNodeRef ref );
simd_quatf SKTransformNodeQuaternion( SKTransformNodeRef ref );
matrix_float3x3 SKTransformNodeRotationMatrix( SKTransformNodeRef ref );
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - SKTransition
///////////////////////////////////////////////////
//                   SKTransition                //
///////////////////////////////////////////////////

// Create
SKTransitionRef SKTransitionCrossFadeWithDuration( CFTimeInterval duration );
SKTransitionRef SKTransitionDoorsCloseHorizontalWithDuration( CFTimeInterval duration );
SKTransitionRef SKTransitionDoorsCloseVerticalWithDuration( CFTimeInterval duration );
SKTransitionRef SKTransitionDoorsOpenHorizontalWithDuration( CFTimeInterval duration );
SKTransitionRef SKTransitionDoorsOpenVerticalWithDuration( CFTimeInterval duration );
SKTransitionRef SKTransitionDoorwayWithDuration( CFTimeInterval duration );
SKTransitionRef SKTransitionFadeWithColor( ColorRef col, CFTimeInterval duration );
SKTransitionRef SKTransitionFadeWithDuration( CFTimeInterval duration );
SKTransitionRef SKTransitionFlipHorizontalWithDuration( CFTimeInterval duration );
SKTransitionRef SKTransitionFlipVerticalWithDuration( CFTimeInterval duration );
SKTransitionRef SKTransitionMoveInWithDirection( SKTransitionDirection direction, CFTimeInterval duration );
SKTransitionRef SKTransitionPushWithDirection( SKTransitionDirection direction, CFTimeInterval duration );
SKTransitionRef SKTransitionRevealWithDirection( SKTransitionDirection direction, CFTimeInterval duration );
SKTransitionRef SKTransitionTransitionWithCIFilter( CIFilterRef filter, CFTimeInterval duration );

// Pausing
Boolean SKTransitionPausesIncomingScene( SKTransitionRef ref );
void SKTransitionSetPausesIncomingScene( SKTransitionRef ref, Boolean flag );
Boolean SKTransitionPausesOutgoingScene( SKTransitionRef ref );
void SKTransitionSetPausesOutgoingScene( SKTransitionRef ref, Boolean flag );

#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKUniform
///////////////////////////////////////////////////
//                   SKUniform                   //
///////////////////////////////////////////////////
// Create
SKUniformRef SKUniformWithName( CFStringRef name );
SKUniformRef SKUniformWithNameFloat( CFStringRef name, float n );
SKUniformRef SKUniformWithNameTexture( CFStringRef name, SKTextureRef texture );

// Info
CFStringRef SKUniformName( SKUniformRef ref );
SKUniformType skUniformType( SKUniformRef ref );

// Value
float SKUniformFloatValue( SKUniformRef ref );
void SKUniformSetFloatValue( SKUniformRef ref, float value );
SKTextureRef SKUniformTextureValue( SKUniformRef ref );
void SKUniformSetTextureValue( SKUniformRef ref, SKTextureRef texture );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
// Properties
matrix_float2x2 SKUniformMatrixFloat2x2Value( SKUniformRef ref );
matrix_float3x3 SKUniformMatrixFloat3x3Value( SKUniformRef ref );
matrix_float4x4 SKUniformMatrixFloat4x4Value( SKUniformRef ref );
vector_float2 SKUniformVectorFloat2Value( SKUniformRef ref );
vector_float3 SKUniformVectorFloat3Value( SKUniformRef ref );
vector_float4 SKUniformVectorFloat4Value( SKUniformRef ref );

// Type methods
SKUniformRef SKUniformWithNameVectorFloat2( CFStringRef name, vector_float2 value );
SKUniformRef SKUniformWithNameVectorFloat3( CFStringRef name, vector_float3 value );
SKUniformRef SKUniformWithNameVectorFloat4( CFStringRef name, vector_float4 value );
SKUniformRef SKUniformWithNameMatrixFloat2x2( CFStringRef name, matrix_float2x2 value );
SKUniformRef SKUniformWithNameMatrixFloat3x3( CFStringRef name, matrix_float3x3 value );
SKUniformRef SKUniformWithNameMatrixFloat4x4( CFStringRef name, matrix_float4x4 value );

#endif// 101200

#endif// 101000



#pragma mark - SKVideoNode
///////////////////////////////////////////////////
//                 SKVideoNode                   //
///////////////////////////////////////////////////

// Create
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKVideoNodeRef SKVideoNodeWithFileNamed( CFStringRef name );
SKVideoNodeRef SKVideoNodeWithURL( CFURLRef url );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKVideoNodeRef SKVideoNodeWithCoder( CoderRef coder );
#endif

// Visual properties
CGPoint SKVideoNodeAnchorPoint( SKVideoNodeRef ref );
void SKVideoNodeSetAnchorPoint( SKVideoNodeRef ref, CGPoint pt );
CGSize SKVideoNodeSize( SKVideoNodeRef ref );
void SKVideoNodeSetSize( SKVideoNodeRef ref, CGSize size );

// Playback
void SKVideoNodePlay( SKVideoNodeRef ref );
void SKVideoNodePause( SKVideoNodeRef ref );



#pragma mark - SKView
///////////////////////////////////////////////////
//                   SKView                      //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200

@interface SKViewSC : SKView
@end

// dialog events
enum {
    skViewShouldRenderAtTime
};

#pragma mark - text view delegate
@interface CocoaSKViewDelegate : NSObject <SKViewDelegate> {
    NSTimeInterval timeVal;
    BOOL boolVal;
}

@property (assign) NSTimeInterval timeVal;
@property (assign) BOOL boolVal;

@end

CocoaSKViewDelegate *CocoaSKViewDelegateObj( void );

BOOL SKViewEventBool( void );
void SKViewEventSetBool( BOOL flag );
CFTimeInterval SKViewEventTime( void );
void SKViewEventSetTime( CFTimeInterval ti );

#endif


@interface SKView (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

SKViewRef SKViewWithTag( NSInteger tag );
Boolean SKViewExists( NSInteger tag );

// Display scene
SKSceneRef SKViewScene( NSInteger tag );
void SKViewPresentScene( NSInteger tag, SKSceneRef scene );
void SKViewPresentSceneTransition( NSInteger tag, SKSceneRef scene, SKTransitionRef transition );

// Timing of rendering
Boolean SKViewIsPaused( NSInteger tag );
void SKViewSetPaused( NSInteger tag, Boolean flag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
NSInteger SKViewPreferredFramesPerSecond( NSInteger tag );
void SKViewSetPreferredFramesPerSecond( NSInteger tag, NSInteger framesPerSecond );
#endif

// Performance related toggles
Boolean SKViewIgnoresSiblingOrder( NSInteger tag );
void SKViewSetIgnoresSiblingOrder( NSInteger tag, Boolean flag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean SKViewShouldCullNonVisibleNodes( NSInteger tag );
void SKViewSetShouldCullNonVisibleNodes( NSInteger tag, Boolean flag );
Boolean SKViewAllowsTransparency( NSInteger tag );
void SKViewSetAllowsTransparency( NSInteger tag, Boolean flag );
#endif
Boolean SKViewIsAsynchronous( NSInteger tag );
void SKViewSetAsynchronous( NSInteger tag, Boolean flag );

// Visual stats for debugging
Boolean SKViewShowsFPS( NSInteger tag );
void SKViewSetShowsFPS( NSInteger tag, Boolean flag );
Boolean SKViewShowsNodeCount( NSInteger tag );
void SKViewSetShowsNodeCount( NSInteger tag, Boolean flag );
Boolean SKViewShowsDrawCount( NSInteger tag );
void SKViewSetShowsDrawCount( NSInteger tag, Boolean flag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean SKViewShowsQuadCount( NSInteger tag );
void SKViewSetShowsQuadCount( NSInteger tag, Boolean flag );
Boolean SKViewShowsPhysics( NSInteger tag );
void SKViewSetShowsPhysics( NSInteger tag, Boolean flag );
Boolean SKViewShowsFields( NSInteger tag );
void SKViewSetShowsFields( NSInteger tag, Boolean flag );
#endif

// Convert between view and scene coordinates
CGPoint SKViewConvertPointFromScene( NSInteger tag, CGPoint pt, SKSceneRef scene );
CGPoint SKViewConvertPointToScene( NSInteger tag, CGPoint pt, SKSceneRef scene );

// Snapshot nodes to texture
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureRef SKViewTextureFromNodeCrop( NSInteger tag, SKNodeRef node, CGRect crop );
#endif
SKTextureRef SKViewTextureFromNode( NSInteger tag, SKNodeRef node );


// Init
SKViewRef SKViewInit( NSInteger inTag, CGRect inRect );

// Statement
void CocoaSKView( NSInteger inTag, CGRect inRect, NSInteger inWndTag );





#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - SKWarpGeometry
///////////////////////////////////////////////////
//              SKWarpGeometry                   //
///////////////////////////////////////////////////




#pragma mark - SKWarpGeometryGrid
///////////////////////////////////////////////////
//            SKWarpGeometryGrid                 //
///////////////////////////////////////////////////
// Create
SKWarpGeometryGridRef SKWarpGeometryInit( void );
SKWarpGeometryGridRef SKWarpGeometryGridWithColumns( NSInteger columns, NSInteger rows );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
SKWarpGeometryGridRef SKWarpGeometryGridWithSourcePositions( NSInteger columns, NSInteger rows, vector_float2 *sourcePositions, vector_float2 *destPositions );
#endif
SKWarpGeometryGridRef SKWarpGeometryGridWithCoder( CoderRef coder );

// Size
NSInteger SKWarpGeometryNumberOfColumns( SKWarpGeometryGridRef ref );
NSInteger SKWarpGeometryNumberOfRows( SKWarpGeometryGridRef ref );
NSInteger SKWarpGeometryVertexCount( SKWarpGeometryGridRef ref );

// Vertices
vector_float2 SKWarpGeometryDestPositionAtIndex( SKWarpGeometryGridRef ref, NSInteger index );
vector_float2 SKWarpGeometrySourcePositionAtIndex( SKWarpGeometryGridRef ref, NSInteger index );
SKWarpGeometryGridRef SKWarpGeometryByRepacingDestPositions( SKWarpGeometryGridRef ref, vector_float2 *destPositions );
SKWarpGeometryGridRef SKWarpGeometryByRepacingSourcePositions( SKWarpGeometryGridRef ref, vector_float2 *sourcePositions );


#endif




#endif// #ifdef SK_EXPORT
#endif// #if __LP64__
#endif// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
