/*
 SpriteKit.m
 
 Bernie Wylde 20190507
 */

#import "SpriteKit.h"

#if __LP64__
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
#ifdef SK_EXPORT


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SK3DNode
///////////////////////////////////////////////////
//                 SK3DNode                      //
///////////////////////////////////////////////////
// Create
SK3DNodeRef SK3DNodeWithViewportSize( CGSize size )
{ return [SK3DNode nodeWithViewportSize:size]; }

SK3DNodeRef SK3DNodeWithCoder( CoderRef coder )
{ return [[[SK3DNode alloc] initWithCoder:coder] autorelease]; }

// Configure
CGSize SK3DNodeViewportSize( SK3DNodeRef ref )
{ return [ref viewportSize]; }

void SK3DNodeSetViewportSize( SK3DNodeRef ref, CGSize size )
{ [ref setViewportSize:size]; }

Boolean SK3DNodeAutoenablesDefaultLighting( SK3DNodeRef ref )
{ return [ref autoenablesDefaultLighting]; }

void SK3DNodeSetAutoenablesDefaultLighting( SK3DNodeRef ref, Boolean flag )
{ [ref setAutoenablesDefaultLighting:flag]; }

// Animate
Boolean SK3DNodeIsPlaying( SK3DNodeRef ref )
{ return [ref isPlaying]; }

void SK3DNodeSetPlaying( SK3DNodeRef ref, Boolean flag )
{ [ref setPlaying:flag]; }

Boolean SK3DNodeLoops( SK3DNodeRef ref )
{ return [ref loops]; }

void SK3DNodeSetLoops( SK3DNodeRef ref, Boolean flag )
{ [ref setLoops:flag]; }

CFTimeInterval SK3DNodeSceneTime( SK3DNodeRef ref )
{ return [ref sceneTime]; }

void SK3DNodeSetSceneTime( SK3DNodeRef ref, CFTimeInterval sceneTime )
{ [ref setSceneTime:sceneTime]; }

// Project points and hit testing
CFArrayRef SK3DNodeHitTest( SK3DNodeRef ref, CGPoint pt, CFDictionaryRef options )
{ return (CFArrayRef)[ref hitTest:pt options:(NSDictionary *)options]; }

vector_float3 SK3DNodeProjectPoint( SK3DNodeRef ref, vector_float3 pt )
{ return [ref projectPoint:pt]; }

vector_float3 SK3DNodeUnprojectPoint( SK3DNodeRef ref, vector_float3 pt )
{return [ref unprojectPoint:pt]; }

#endif



#pragma mark - SKAction
///////////////////////////////////////////////////
//                 SKAction                      //
///////////////////////////////////////////////////
// Animate in linear path
SKActionRef SKActionMoveByXY( CGFloat deltaX, CGFloat deltaY, CFTimeInterval duration )
{ return [SKAction moveByX:deltaX y:deltaY duration:duration]; }

SKActionRef SKActionMoveByVector( CGVector delta, CFTimeInterval duration )
{ return [SKAction moveBy:delta duration:duration]; }

SKActionRef SKActionMoveToPoint( CGPoint location, CFTimeInterval duration )
{ return [SKAction moveTo:location duration:duration]; }

SKActionRef SKActionMoveToX( CGFloat x, CFTimeInterval duration )
{ return [SKAction moveToX:x duration:duration]; }

SKActionRef SKActionMoveToY( CGFloat y, CFTimeInterval duration )
{ return [SKAction moveToY:y duration:duration]; }

// Animate in custom path
SKActionRef SKActionFollowPath( CGPathRef path, CFTimeInterval duration )
{ return [SKAction followPath:path duration:duration]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKActionRef SKActionFollowPathSpeed( CGPathRef path, CGFloat speed )
{ return [SKAction followPath:path speed:speed]; }
#endif

SKActionRef SKActionFollowPathAsOffset( CGPathRef path, Boolean asOffset, Boolean orientToPath, CFTimeInterval duration )
{ return [SKAction followPath:path asOffset:asOffset orientToPath:orientToPath duration:duration]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKActionRef SKActionFollowPathAsOffsetSpeed( CGPathRef path, Boolean asOffset, Boolean orientToPath, CGFloat speed )
{ return [SKAction followPath:path asOffset:asOffset orientToPath:orientToPath speed:speed]; }
#endif

// Animate rotation
SKActionRef SKActionRotateByAngle( CGFloat radians, CFTimeInterval duration )
{ return [SKAction rotateByAngle:radians duration:duration]; }

SKActionRef SKActionRotateToAngle( CGFloat radians, CFTimeInterval duration )
{ return [SKAction rotateToAngle:radians duration:duration]; }

SKActionRef SKActionRotateToAngleShortestUnitArc( CGFloat radians, CFTimeInterval duration, Boolean shortestUnitArc )
{ return [SKAction rotateToAngle:radians duration:duration shortestUnitArc:shortestUnitArc]; }

// Controlling speed
SKActionRef SKActionSpeedBy( CGFloat speed, CFTimeInterval duration )
{ return [SKAction speedBy:speed duration:duration]; }

SKActionRef SKActionSpeedTo( CGFloat speed, CFTimeInterval duration )
{ return [SKAction speedTo:speed duration:duration]; }

// Animate scaling
SKActionRef SKActionScaleBy( CGFloat scale, CFTimeInterval duration )
{ return [SKAction scaleBy:scale duration:duration]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
SKActionRef SKActionScaleToSize( CGSize size, CFTimeInterval duration )
{ return [SKAction scaleToSize:size duration:duration]; }
#endif

SKActionRef SKActionScaleTo( CGFloat scale, CFTimeInterval duration )
{ return [SKAction scaleTo:scale duration:duration]; }

SKActionRef SKActionScaleXYBy( CGFloat xScale, CGFloat yScale, CFTimeInterval duration )
{ return [SKAction scaleXBy:xScale y:yScale duration:duration]; }

SKActionRef SKActionScaleXYTo( CGFloat xScale, CGFloat yScale, CFTimeInterval duration )
{ return [SKAction scaleXTo:xScale y:yScale duration:duration]; }

SKActionRef SKActionScaleXTo( CGFloat xScale, CFTimeInterval duration )
{ return [SKAction scaleXTo:xScale duration:duration]; }

SKActionRef SKActionScaleYTo( CGFloat yScale, CFTimeInterval duration )
{ return [SKAction scaleYTo:yScale duration:duration]; }

// Animate transparency
SKActionRef SKActionFadeIn( CFTimeInterval duration )
{ return [SKAction fadeInWithDuration:duration]; }

SKActionRef SKActionFadeOut( CFTimeInterval duration )
{ return [SKAction fadeOutWithDuration:duration]; }

SKActionRef SKActionFadeAlphaBy( CGFloat factor, CFTimeInterval duration )
{ return [SKAction fadeAlphaBy:factor duration:duration]; }

SKActionRef SKActionFadeAlphaTo( CGFloat alpha, CFTimeInterval duration )
{ return [SKAction fadeAlphaTo:alpha duration:duration]; }

// Animate texture
SKActionRef SKActionResizeByWidthAndHeight( CGFloat width, CGFloat height, CFTimeInterval duration )
{ return [SKAction resizeByWidth:width height:height duration:duration]; }

SKActionRef SKActionResizeToHeight( CGFloat height, CFTimeInterval duration )
{ return [SKAction resizeToHeight:height duration:duration]; }

SKActionRef SKActionResizeToWidth( CGFloat width, CFTimeInterval duration )
{ return [SKAction resizeToWidth:width duration:duration]; }

SKActionRef SKActionResizeToWidthAndHeight( CGFloat width, CGFloat height, CFTimeInterval duration )
{ return [SKAction resizeToWidth:width height:height duration:duration]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKActionRef SKActionSetTexture( SKTextureRef texture )
{ return [SKAction setTexture:texture]; }

SKActionRef SKActionSetTextureResize( SKTextureRef texture, Boolean resize )
{ return [SKAction setTexture:texture resize:resize]; }
#endif

SKActionRef SKActionAnimateWithTextures( CFArrayRef textures, CFTimeInterval timePerFrame )
{ return [SKAction animateWithTextures:(NSArray *)textures timePerFrame:timePerFrame]; }

SKActionRef SKActionAnimateWithTexturesResize( CFArrayRef textures, CFTimeInterval timePerFrame, Boolean resize, Boolean restore )
{ return [SKAction animateWithTextures:(NSArray *)textures timePerFrame:timePerFrame resize:resize restore:restore]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKActionRef SKActionSetNormalTexture( SKTextureRef texture )
{ return [SKAction setNormalTexture:texture]; }

SKActionRef SKActionSetNormalTextureResize( SKTextureRef texture, Boolean resize )
{ return [SKAction setNormalTexture:texture resize:resize]; }

SKActionRef SKActionAnimateWithNormalTextures( CFArrayRef textures, CFTimeInterval timePerFrame )
{ return [SKAction animateWithNormalTextures:(NSArray *)textures timePerFrame:timePerFrame]; }

SKActionRef SKActionAnimateWithNormalTexturesResize( CFArrayRef textures, CFTimeInterval timePerFrame, Boolean resize, Boolean restore )
{ return [SKAction animateWithNormalTextures:(NSArray *)textures timePerFrame:timePerFrame resize:resize restore:restore]; }
#endif

SKActionRef SKActionColorizeWithColor( ColorRef color, CGFloat colorBlendFactor, CFTimeInterval duration )
{ return [SKAction colorizeWithColor:color colorBlendFactor:colorBlendFactor duration:duration]; }

SKActionRef SKActionColorizeWithColorBlendFactor( CGFloat colorBlendFactor, CFTimeInterval duration )
{ return [SKAction colorizeWithColorBlendFactor:colorBlendFactor duration:duration]; }

// Animate properties of physics body
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKActionRef SKActionApplyForce( CGVector force, CFTimeInterval duration )
{ return [SKAction applyForce:force duration:duration]; }

SKActionRef SKActionApplyTorque( CGFloat torque, CFTimeInterval duration )
{ return [SKAction applyTorque:torque duration:duration]; }

SKActionRef SKActionApplyForceAtPoint( CGVector force, CGPoint pt, CFTimeInterval duration )
{ return [SKAction applyForce:force atPoint:pt duration:duration]; }

SKActionRef SKActionApplyImpulse( CGVector impulse, CFTimeInterval duration )
{ return [SKAction applyImpulse:impulse duration:duration]; }

SKActionRef SKActionApplyAngularImpulse( CGFloat impulse, CFTimeInterval duration )
{ return [SKAction applyAngularImpulse:impulse duration:duration]; }

SKActionRef SKActionApplyImpulseAtPoint( CGVector impulse, CGPoint pt, CFTimeInterval duration )
{ return [SKAction applyImpulse:impulse atPoint:pt duration:duration]; }

SKActionRef SKActionChangeChargeTo( CGFloat v, CFTimeInterval duration )
{ return [SKAction changeChargeTo:v duration:duration]; }

SKActionRef SKActionChangeChargeBy( CGFloat v, CFTimeInterval duration )
{ return [SKAction changeChargeBy:v duration:duration]; }

SKActionRef SKActionChangeMassTo( CGFloat v, CFTimeInterval duration )
{ return [SKAction changeMassTo:v duration:duration]; }

SKActionRef SKActionChangeMassBy( CGFloat v, CFTimeInterval duration )
{ return [SKAction changeMassBy:v duration:duration]; }

#endif// 101100
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKActionRef SKActionStrengthTo( float strength, CFTimeInterval duration )
{ return [SKAction strengthTo:strength duration:duration]; }

SKActionRef SKActionStrengthBy( float strength, CFTimeInterval duration )
{ return [SKAction strengthBy:strength duration:duration]; }

SKActionRef SKActionFalloffTo( float falloff, CFTimeInterval duration )
{ return [SKAction falloffTo:falloff duration:duration]; }

SKActionRef SKActionFalloffBy( float falloff, CFTimeInterval duration )
{ return [SKAction falloffBy:falloff duration:duration]; }
#endif// 101000

// Reversing animation
SKActionRef SKActionReverseAction( SKActionRef ref )
{ return [ref reversedAction]; }

// Animate warping
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
SKActionRef SKActionAnimateWithWarps( CFArrayRef warps, CFArrayRef times )
{ return [SKAction animateWithWarps:(NSArray *)warps times:(NSArray *)times]; }

SKActionRef SKActionAnimateWithWarpsRestore( CFArrayRef warps, CFArrayRef times, Boolean restore )
{ return [SKAction animateWithWarps:(NSArray *)warps times:(NSArray *)times restore:restore]; }

SKActionRef SKActionWarpTo( SKWarpGeometryRef warp, CFTimeInterval duration )
{ return [SKAction warpTo:warp duration:duration]; }
#endif// 101200

// Controlling audio
SKActionRef SKActionPlaySoundFileNamed( CFStringRef name, Boolean waitForCompletion )
{ return [SKAction playSoundFileNamed:(NSString *)name waitForCompletion:waitForCompletion]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKActionRef SKActionPlay( void )
{ return [SKAction play]; }

SKActionRef SKActionPause( void )
{ return [SKAction pause]; }

SKActionRef SKActionStop( void )
{ return [SKAction stop]; }

SKActionRef SKActionChangePlaybackRateTo( float v, CFTimeInterval duration )
{ return [SKAction changePlaybackRateTo:v duration:duration]; }

SKActionRef SKActionChangePlaybackRateBy( float v, CFTimeInterval duration )
{ return [SKAction changePlaybackRateBy:v duration:duration]; }

SKActionRef SKActionChangeVolumeTo( float v, CFTimeInterval duration )
{ return [SKAction changeVolumeTo:v duration:duration]; }

SKActionRef SKActionChangeVolumeBy( float v, CFTimeInterval duration )
{ return [SKAction changeVolumeBy:v duration:duration]; }

SKActionRef SKActionChangeObstructionTo( float v, CFTimeInterval duration )
{ return [SKAction changeObstructionTo:v duration:duration]; }

SKActionRef SKActionChangeObstructionBy( float v, CFTimeInterval duration )
{ return [SKAction changeObstructionBy:v duration:duration]; }

SKActionRef SKActionChangeOcclusionTo( float v, CFTimeInterval duration )
{ return [SKAction changeOcclusionTo:v duration:duration]; }

SKActionRef SKActionChangeOcclusionBy( float v, CFTimeInterval duration )
{ return [SKAction changeOcclusionBy:v duration:duration]; }

SKActionRef SKActionChangeReverbTo( float v, CFTimeInterval duration )
{ return [SKAction changeReverbTo:v duration:duration]; }

SKActionRef SKActionChangeReverbBy( float v, CFTimeInterval duration )
{ return [SKAction changeReverbBy:v duration:duration]; }

SKActionRef SKActionStereoPanTo( float v, CFTimeInterval duration )
{ return [SKAction stereoPanTo:v duration:duration]; }

SKActionRef SKActionStereoPanBy( float v, CFTimeInterval duration )
{ return [SKAction stereoPanBy:v duration:duration]; }
#endif// 101100

// Remove from scene
SKActionRef SKActionRemoveFromParent( void )
{ return [SKAction removeFromParent]; }

// Running on children
SKActionRef SKActionRunActionOnChildWithName( SKActionRef action, CFStringRef name )
{ return [SKAction runAction:action onChildWithName:(NSString *)name]; }

// Chaining
SKActionRef SKActionGroup( CFArrayRef actions )
{ return [SKAction group:(NSArray *)actions]; }

SKActionRef SKActionSequence( CFArrayRef actions )
{ return [SKAction sequence:(NSArray *)actions]; }

SKActionRef SKActionRepeatAction( SKActionRef action, NSUInteger count )
{ return [SKAction repeatAction:action count:count]; }

SKActionRef SKActionRepeatActionForever( SKActionRef action )
{ return [SKAction repeatActionForever:action]; }

// Delaying
SKActionRef SKActionWaitForDuration( CFTimeInterval duration )
{ return [SKAction waitForDuration:duration]; }

SKActionRef SKActionWaitForDurationWithRange( CFTimeInterval duration, CFTimeInterval durationRange )
{ return [SKAction waitForDuration:duration withRange:durationRange]; }

// Inverse kinematics
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKActionRef SKActionReachToDuration( CGPoint position, SKNodeRef rootNode, CFTimeInterval duration )
{ return [SKAction reachTo:position rootNode:rootNode duration:duration]; }

SKActionRef SKActionReachToVelocity( CGPoint position, SKNodeRef rootNode, CGFloat velocity )
{ return [SKAction reachTo:position rootNode:rootNode velocity:velocity]; }

SKActionRef SKActionReachToNodeDuration( SKNodeRef node, SKNodeRef rootNode, CFTimeInterval duration )
{ return [SKAction reachToNode:node rootNode:rootNode duration:duration]; }

SKActionRef SKActionReachToNodeVelocity( SKNodeRef node, SKNodeRef rootNode, CGFloat velocity )
{ return [SKAction reachToNode:node rootNode:rootNode velocity:velocity]; }
#endif// 101000

// Custom actions
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKActionRef SKActionNamed( CFStringRef name )
{ return [SKAction actionNamed:(NSString *)name]; }

SKActionRef SKActionNamedDuration( CFStringRef name, CFTimeInterval duration )
{ return [SKAction actionNamed:(NSString *)name duration:duration]; }

SKActionRef SKActionNamedFromURL( CFStringRef name, CFURLRef url )
{ return [SKAction actionNamed:(NSString *)name fromURL:(NSURL *)url]; }

SKActionRef SKActionNamedFromURLDuration( CFStringRef name, CFURLRef url, CFTimeInterval duration )
{ return [SKAction actionNamed:(NSString *)name fromURL:(NSURL *)url duration:duration]; }
#endif// 101100

SKActionRef SKActionCustomWithDuration( CFTimeInterval duration, SKActionCustomWithDurationCallbackType callback, void *userData )
{
    return [SKAction customActionWithDuration:duration actionBlock:^(SKNode *node, CGFloat elapsedTime) {
        (*callback)(node,elapsedTime,userData);
    }];
}

// Visibility
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void SKActionUnhide( void )
{ [SKAction unhide]; }

void SKActionHide( void )
{ [SKAction hide]; }
#endif// 101000

// Timing
CFTimeInterval SKActionDuration( SKActionRef ref )
{ return [ref duration]; }

void SKActionSetDuration( SKActionRef ref, CFTimeInterval duration )
{ [ref setDuration:duration]; }

SKActionTimingMode skActionTimingMode( SKActionRef ref )
{ return [ref timingMode]; }

void SKActionSetTimingMode( SKActionRef ref, SKActionTimingMode mode )
{ [ref setTimingMode:mode]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void skActionTimingFunction( SKActionRef ref, SKActionTimingFunctionCallbackType callback, void *userData )
{
    ref.timingFunction = ^float(float time) {
        return (*callback)(ref,time,userData);
    };
}
#endif// 101000

CGFloat SKActionSpeed( SKActionRef ref )
{ return [ref speed]; }

void SKActionSetSpeed( SKActionRef ref, CGFloat speed )
{ [ref setSpeed:speed]; }



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - SKAttribute
///////////////////////////////////////////////////
//                SKAttribute                    //
///////////////////////////////////////////////////
// Init
SKAttributeRef SKAttributeWithName( CFStringRef name, SKAttributeType type )
{ return [SKAttribute attributeWithName:(NSString *)name type:type]; }

// Instance properties
CFStringRef SKAttributeName( SKAttributeRef ref )
{ return (CFStringRef)[ref name]; }

SKAttributeType skAttributeType( SKAttributeRef ref )
{ return [ref type]; }



#pragma mark - SKAttributeValue
///////////////////////////////////////////////////
//             SKAttributeValue                  //
///////////////////////////////////////////////////
// Initializers
SKAttributeValueRef SKAttributeValueInit( void )
{ return [[[SKAttributeValue alloc] init] autorelease]; }

SKAttributeValueRef SKAttributeValueWithFloat( float value )
{ return [SKAttributeValue valueWithFloat:value]; }

SKAttributeValueRef SKAttributeValueWithVectorFloat2( vector_float2 value )
{ return [SKAttributeValue valueWithVectorFloat2:value]; }

SKAttributeValueRef SKAttributeValueWithVectorFloat3( vector_float3 value )
{ return [SKAttributeValue valueWithVectorFloat3:value]; }

SKAttributeValueRef SKAttributeValueWithVectorFloat4( vector_float4 value )
{ return [SKAttributeValue valueWithVectorFloat4:value]; }

// Instance properties
float SKAttributeValueFloatValue( SKAttributeValueRef ref )
{ return [ref floatValue]; }

void SKAttributeValueSetFloatValue( SKAttributeValueRef ref, float value )
{ [ref setFloatValue:value]; }

vector_float2 SKAttributeValueVectorFloat2Value( SKAttributeValueRef ref )
{ return [ref vectorFloat2Value]; }

void SKAttributeValueSetVectorFloat2Value( SKAttributeValueRef ref, vector_float2 value )
{ [ref setVectorFloat2Value:value]; }

vector_float3 SKAttributeValueVectorFloat3Value( SKAttributeValueRef ref )
{ return [ref vectorFloat3Value]; }

void SKAttributeValueSetVectorFloat3Value( SKAttributeValueRef ref, vector_float3 value )
{ [ref setVectorFloat3Value:value]; }

vector_float4 SKAttributeValueVectorFloat4Value( SKAttributeValueRef ref )
{ return [ref vectorFloat4Value]; }

void SKAttributeValueSetVectorFloat4Value( SKAttributeValueRef ref, vector_float4 value )
{ [ref setVectorFloat4Value:value]; }
#endif


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
#pragma mark - SKAudioNode
///////////////////////////////////////////////////
//                SKAudioNode                    //
///////////////////////////////////////////////////
// Init
SKAudioNodeRef SKAudioNodeWithFileNamed( CFStringRef name )
{ return [[[SKAudioNode alloc] initWithFileNamed:(NSString *)name] autorelease]; }

SKAudioNodeRef SKAudioNodeWithURL( CFURLRef url )
{ return [[[SKAudioNode alloc] initWithURL:(NSURL *)url] autorelease]; }

SKAudioNodeRef SKAudioNodeWithCoder( CoderRef coder )
{ return [[[SKAudioNode alloc] initWithCoder:coder] autorelease]; }

// Configure
Boolean SKAudioNodeIsPositional( SKAudioNodeRef ref )
{ return [ref isPositional]; }

void SKAudioNodeSetPositional( SKAudioNodeRef ref, Boolean flag )
{ [ref setPositional:flag]; }

Boolean SKAudioNodeAutoplayLooped( SKAudioNodeRef ref )
{ return [ref autoplayLooped]; }

void SKAudioNodeSetAutoplayLooped( SKAudioNodeRef ref, Boolean flag )
{ [ref setAutoplayLooped:flag]; }
#endif


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
#pragma mark - SKCameraNode
///////////////////////////////////////////////////
//               SKCameraNode                    //
///////////////////////////////////////////////////
// Init
SKCameraNodeRef SKCameraNodeInit( void )
{ return [[[SKCameraNode alloc] init] autorelease]; }

// Visibility
CFSetRef SKCameraNodeContainedNodeSet( SKCameraNodeRef ref )
{ return (CFSetRef)[ref containedNodeSet]; }

Boolean SKCameraNodeContainsNode( SKCameraNodeRef ref, SKNodeRef node )
{ return [ref containsNode:node]; }
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKConstraint
///////////////////////////////////////////////////
//               SKConstraint                    //
///////////////////////////////////////////////////
// Position
SKConstraintRef SKConstraintPositionXY( SKRangeRef xRange, SKRangeRef yRange )
{ return [SKConstraint positionX:xRange Y:yRange]; }

SKConstraintRef SKConstraintPositionX( SKRangeRef range )
{ return [SKConstraint positionX:range]; }

SKConstraintRef SKConstraintPositionY( SKRangeRef range )
{ return [SKConstraint positionY:range]; }

// Orientation
SKConstraintRef SKConstraintOrientToNode( SKNodeRef node, SKRangeRef offsetRadians )
{ return [SKConstraint orientToNode:node offset:offsetRadians]; }

SKConstraintRef SKConstraintOrientToPoint( CGPoint pt, SKRangeRef offsetRadians )
{ return [SKConstraint orientToPoint:pt offset:offsetRadians]; }

SKConstraintRef SKConstraintOrientToPointInNode( CGPoint pt, SKNodeRef node, SKRangeRef offsetRadians )
{ return [SKConstraint orientToPoint:pt inNode:node offset:offsetRadians]; }

SKConstraintRef SKConstraintZRotation( SKRangeRef zRange )
{ return [SKConstraint zRotation:zRange]; }

// Distance
SKConstraintRef SKConstraintDistanceToNode( SKRangeRef range, SKNodeRef node )
{ return [SKConstraint distance:range toNode:node]; }

SKConstraintRef SKConstraintDistanceToPoint( SKRangeRef range, CGPoint pt )
{ return [SKConstraint distance:range toPoint:pt]; }

SKConstraintRef SKConstraintDistanceToPointInNode( SKRangeRef range, CGPoint pt, SKNodeRef node )
{ return [SKConstraint distance:range toPoint:pt inNode:node]; }

// Coordinate system
SKNodeRef SKConstraintReferenceNode( SKConstraintRef ref )
{ return [ref referenceNode]; }

void SKConstraintSetReferenceNode( SKConstraintRef ref, SKNodeRef node )
{ [ref setReferenceNode:node]; }

// Enable/disable
Boolean SKConstraintEnabled( SKConstraintRef ref )
{ return [ref enabled]; }

void SKConstraintSetEnabled( SKConstraintRef ref, Boolean flag )
{ [ref setEnabled:flag]; }
#endif


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKRange
///////////////////////////////////////////////////
//                 SKRange                       //
///////////////////////////////////////////////////
// Create
SKRangeRef SKRangeWithLowerLimitUpperLimit( CGFloat lowerLimit, CGFloat upperLimit )
{ return [SKRange rangeWithLowerLimit:lowerLimit upperLimit:upperLimit]; }

SKRangeRef SKRangeWithValue( CGFloat value, CGFloat variance )
{ return [SKRange rangeWithValue:value variance:variance]; }

SKRangeRef SKRangeWithNoLimits( void )
{ return [SKRange rangeWithNoLimits]; }

SKRangeRef SKRangeWithLowerLimit( CGFloat lowerLimit )
{ return [SKRange rangeWithLowerLimit:lowerLimit]; }

SKRangeRef SKRangeWithUpperLimit( CGFloat upperLimit )
{ return [SKRange rangeWithUpperLimit:upperLimit]; }

SKRangeRef SKRangeWithConstantValue( CGFloat value )
{ return [SKRange rangeWithConstantValue:value]; }

// Inspect
CGFloat SKRangeLowerLimit( SKRangeRef ref )
{ return [ref lowerLimit]; }

void SKRangeSetLowerLimit( SKRangeRef ref, CGFloat lowerLimit )
{ [ref setLowerLimit:lowerLimit]; }

CGFloat SKRangeUpperLimit( SKRangeRef ref )
{ return [ref upperLimit]; }

void SKRangeSetUpperLimit( SKRangeRef ref, CGFloat upperLimit )
{ [ref setUpperLimit:upperLimit]; }
#endif


#pragma mark - SKCropNode
///////////////////////////////////////////////////
//                 SKCropNode                    //
///////////////////////////////////////////////////
// Init
SKCropNodeRef SKCropNodeInit( void )
{ return [[[SKCropNode alloc] init] autorelease]; }

// Mask filter
SKNodeRef SKCropNodeMaskNode( SKCropNodeRef ref )
{ return [ref maskNode]; }

void SKCropNodeSetMaskNode( SKCropNodeRef ref, SKNodeRef node )
{ [ref setMaskNode:node]; }



#pragma mark - SKEffectNode
///////////////////////////////////////////////////
//               SKEffectNode                    //
///////////////////////////////////////////////////
// Init
SKEffectNodeRef SKEffectNodeInit( void )
{ return [[[SKEffectNode alloc] init] autorelease]; }

// Core image filters
CIFilterRef SKEffectNodeFilter( SKEffectNodeRef ref )
{ return [ref filter]; }

void SKEffectNodeSetFilter( SKEffectNodeRef ref, CIFilterRef filter )
{ [ref setFilter:filter]; }

Boolean SKEffectNodeShouldEnableEffects( SKEffectNodeRef ref )
{ return [ref shouldEnableEffects]; }

void SKEffectNodeSetShouldEnableEffects( SKEffectNodeRef ref, Boolean flag )
{ [ref setShouldEnableEffects:flag]; }

Boolean SKEffectNodeShouldCenterFilter( SKEffectNodeRef ref )
{ return [ref shouldCenterFilter]; }

void SKEffectNodeSetShouldCenterFilter( SKEffectNodeRef ref, Boolean flag )
{ [ref setShouldCenterFilter:flag]; }

// Shader
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKShaderRef SKEffectNodeShader( SKEffectNodeRef ref )
{ return [ref shader]; }

void SKEffectNodeSetShader( SKEffectNodeRef ref, SKShaderRef shader )
{ [ref setShader:shader]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFDictionaryRef SKEffectNodeAttributeValues( SKEffectNodeRef ref )
{ return (CFDictionaryRef)[ref attributeValues]; }

void SKEffectNodeSetAttributeValues( SKEffectNodeRef ref, CFDictionaryRef values )
{ [ref setAttributeValues:(NSDictionary *)values]; }

void SKEffectNodeSetValueForAttributeNamed( SKEffectNodeRef ref, SKAttributeValueRef value, CFStringRef name )
{ [ref setValue:value forAttributeNamed:(NSString *)name]; }

SKAttributeValueRef SKEffectNodeValueForAttributeNamed( SKEffectNodeRef ref, CFStringRef name )
{ return [ref valueForAttributeNamed:(NSString *)name]; }
#endif

// Flattening
Boolean SKEffectNodeShouldRasterize( SKEffectNodeRef ref )
{ return [ref shouldRasterize]; }

void SKEffectNodeSetShouldRasterize( SKEffectNodeRef ref, Boolean flag )
{ [ref setShouldRasterize:flag]; }

// Alpha blending
SKBlendMode SKEffectNodeBlendMode( SKEffectNodeRef ref )
{ return [ref blendMode]; }

void SKEffectNodeSetBlendMode( SKEffectNodeRef ref, SKBlendMode mode )
{ [ref setBlendMode:mode]; }



#pragma mark - SKEmitterNode
///////////////////////////////////////////////////
//               SKEmitterNode                   //
///////////////////////////////////////////////////
// Init
SKEmitterNodeRef SKEmitterNodeInit( void )
{ return [[[SKEmitterNode alloc] init] autorelease]; }

// Choosing
SKNodeRef SKEmitterNodeTargetNode( SKEmitterNodeRef ref )
{ return [ref targetNode]; }

void SKEmitterNodeSetTargetNode( SKEmitterNodeRef ref, SKNodeRef node )
{ [ref setTargetNode:node]; }

// Controlling when particles are created
void SKEmitterNodeAdvanceSimulationTime( SKEmitterNodeRef ref, CFTimeInterval sec )
{ [ref advanceSimulationTime:sec]; }

void SKEmitterNodeResetSimulation( SKEmitterNodeRef ref )
{ [ref resetSimulation]; }

CGFloat SKEmitterNodeParticleBirthRate( SKEmitterNodeRef ref )
{ return [ref particleBirthRate]; }

void SKEmitterNodeSetParticleBirthRate( SKEmitterNodeRef ref, CGFloat birthRate )
{ [ref setParticleBirthRate:birthRate]; }

NSUInteger SKEmitterNodeNumParticlesToEmit( SKEmitterNodeRef ref )
{ return [ref numParticlesToEmit]; }

void SKEmitterNodeSetNumParticlesToEmit( SKEmitterNodeRef ref, NSUInteger num )
{ [ref setNumParticlesToEmit:num]; }

// Rendering order
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKParticleRenderOrder SKEmitterNodeParticleRenderOrder( SKEmitterNodeRef ref )
{ return [ref particleRenderOrder]; }

void SKEmitterNodeSetParticleRenderOrder( SKEmitterNodeRef ref, SKParticleRenderOrder order )
{ [ref setParticleRenderOrder:order]; }
#endif

// Lifetime
CGFloat SKEmitterNodeParticleLifetime( SKEmitterNodeRef ref )
{ return [ref particleLifetime]; }

void SKEmitterNodeSetParticleLifetime( SKEmitterNodeRef ref, CGFloat lifetime )
{ [ref setParticleLifetime:lifetime]; }

CGFloat SKEmitterNodeParticleLifetimeRange( SKEmitterNodeRef ref )
{ return [ref particleLifetimeRange]; }

void SKEmitterNodeSetParticleLifetimeRange( SKEmitterNodeRef ref, CGFloat range )
{ [ref setParticleLifetimeRange:range]; }

// Position
CGPoint SKEmitterNodeParticlePosition( SKEmitterNodeRef ref )
{ return [ref particlePosition]; }

void SKEmitterNodeSetParticlePosition( SKEmitterNodeRef ref, CGPoint position )
{ [ref setParticlePosition:position]; }

CGVector SKEmitterNodeParticlePositionRange( SKEmitterNodeRef ref )
{ return [ref particlePositionRange]; }

void SKEmitterNodeSetParticlePositionRange( SKEmitterNodeRef ref, CGVector range )
{ [ref setParticlePositionRange:range]; }

CGFloat SKEmitterNodeParticleZPosition( SKEmitterNodeRef ref )
{ return [ref particleZPosition]; }

void SKEmitterNodeSetParticleZPosition( SKEmitterNodeRef ref, CGFloat position )
{ [ref setParticleZPosition:position]; }

// Velocity and acceleration
CGFloat SKEmitterNodeParticleSpeed( SKEmitterNodeRef ref )
{ return [ref particleSpeed]; }

void SKEmitterNodeSetParticleSpeed( SKEmitterNodeRef ref, CGFloat speed )
{ [ref setParticleSpeed:speed]; }

CGFloat SKEmitterNodeParticleSpeedRange( SKEmitterNodeRef ref )
{ return [ref particleSpeedRange]; }

void SKEmitterNodeSetParticleSpeedRange( SKEmitterNodeRef ref, CGFloat range )
{ [ref setParticleSpeedRange:range]; }

CGFloat SKEmitterNodeEmissionAngle( SKEmitterNodeRef ref )
{ return [ref emissionAngle]; }

void SKEmitterNodeSetEmissionAngle( SKEmitterNodeRef ref, CGFloat angle )
{ [ref setEmissionAngle:angle]; }

CGFloat SKEmitterNodeEmissionAngleRange( SKEmitterNodeRef ref )
{ return [ref emissionAngleRange]; }

void SKEmitterNodeSetEmissionAngleRange( SKEmitterNodeRef ref, CGFloat range )
{ [ref setEmissionAngleRange:range]; }

CGFloat SKEmitterNodeXAcceleration( SKEmitterNodeRef ref )
{ return [ref xAcceleration]; }

void SKEmitterNodeSetXAcceleration( SKEmitterNodeRef ref, CGFloat acceleration )
{ [ref setXAcceleration:acceleration]; }

CGFloat SKEmitterNodeYAcceleration( SKEmitterNodeRef ref )
{ return [ref yAcceleration]; }

void SKEmitterNodeSetYAcceleration( SKEmitterNodeRef ref, CGFloat acceleration )
{ [ref setYAcceleration:acceleration]; }

// Rotation
CGFloat SKEmitterNodeParticleRotation( SKEmitterNodeRef ref )
{ return [ref particleRotation]; }

void SKEmitterNodeSetParticleRotation( SKEmitterNodeRef ref, CGFloat rotation )
{ [ref setParticleRotation:rotation]; }

CGFloat SKEmitterNodeParticleRotationRange( SKEmitterNodeRef ref )
{ return [ref particleRotationRange]; }

void SKEmitterNodeSetParticleRotationRange( SKEmitterNodeRef ref, CGFloat range )
{ [ref setParticleRotationRange:range]; }

CGFloat SKEmitterNodeParticleRotationSpeed( SKEmitterNodeRef ref )
{ return [ref particleRotationSpeed]; }

void SKEmitterNodeSetParticleRotationSpeed( SKEmitterNodeRef ref, CGFloat speed )
{ [ref setParticleRotationSpeed:speed]; }

// Scaling
CGFloat SKEmitterNodeParticleScale( SKEmitterNodeRef ref )
{ return [ref particleScale]; }

void SKEmitterNodeSetParticleScale( SKEmitterNodeRef ref, CGFloat scale )
{ [ref setParticleScale:scale]; }

CGFloat SKEmitterNodeParticleScaleRange( SKEmitterNodeRef ref )
{ return [ref particleScaleRange]; }

void SKEmitterNodeSetParticleScaleRange( SKEmitterNodeRef ref, CGFloat range )
{ [ref setParticleScaleRange:range]; }

CGFloat SKEmitterNodeParticleScaleSpeed( SKEmitterNodeRef ref )
{ return [ref particleScaleSpeed]; }

void SKEmitterNodeSetParticleScaleSpeed( SKEmitterNodeRef ref, CGFloat speed )
{ [ref setParticleScaleSpeed:speed]; }

SKKeyframeSequenceRef SKEmitterNodeParticleScaleSequence( SKEmitterNodeRef ref )
{ return [ref particleScaleSequence]; }

void SKEmitterNodeSetParticleScaleSequence( SKEmitterNodeRef ref, SKKeyframeSequenceRef sequence )
{ [ref setParticleScaleSequence:sequence]; }

// Image and size
SKTextureRef SKEmitterNodeParticleTexture( SKEmitterNodeRef ref )
{ return [ref particleTexture]; }

void SKEmitterNodeSetParticleTexture( SKEmitterNodeRef ref, SKTextureRef texture )
{ [ref setParticleTexture:texture]; }

CGSize SKEmitterNodeParticleSize( SKEmitterNodeRef ref )
{ return [ref particleSize]; }

void SKEmitterNodeSetParticleSize( SKEmitterNodeRef ref, CGSize size )
{ [ref setParticleSize:size]; }

// Color
SKKeyframeSequenceRef SKEmitterNodeParticleColorSequence( SKEmitterNodeRef ref )
{ return [ref particleColorSequence]; }

void SKEmitterNodeSetParticleColorSequence( SKEmitterNodeRef ref, SKKeyframeSequenceRef sequence )
{ [ref setParticleColorSequence:sequence]; }

ColorRef SKEmitterNodeParticleColor( SKEmitterNodeRef ref )
{ return [ref particleColor]; }

void SKEmitterNodeSetParticleColor( SKEmitterNodeRef ref, ColorRef col )
{ [ref setParticleColor:col]; }

CGFloat SKEmitterNodeParticleColorAlphaRange( SKEmitterNodeRef ref )
{ return [ref particleColorAlphaRange]; }

void SKEmitterNodeSetParticleColorAlphaRange( SKEmitterNodeRef ref, CGFloat range )
{ [ref setParticleColorAlphaRange:range]; }

CGFloat SKEmitterNodeParticleColorBlueRange( SKEmitterNodeRef ref )
{ return [ref particleColorBlueRange]; }

void SKEmitterNodeSetParticleColorBlueRange( SKEmitterNodeRef ref, CGFloat range )
{ [ref setParticleColorBlueRange:range]; }

CGFloat SKEmitterNodeParticleColorGreenRange( SKEmitterNodeRef ref )
{ return [ref particleColorGreenRange]; }

void SKEmitterNodeSetParticleColorGreenRange( SKEmitterNodeRef ref, CGFloat range )
{ [ref setParticleColorGreenRange:range]; }

CGFloat SKEmitterNodeParticleColorRedRange( SKEmitterNodeRef ref )
{ return [ref particleColorRedRange]; }

void SKEmitterNodeSetParticleColorRedRange( SKEmitterNodeRef ref, CGFloat range )
{ [ref setParticleColorRedRange:range]; }

CGFloat SKEmitterNodeParticleColorAlphaSpeed( SKEmitterNodeRef ref )
{ return [ref particleColorAlphaSpeed]; }

void SKEmitterNodeSetParticleColorAlphaSpeed( SKEmitterNodeRef ref, CGFloat speed )
{ [ref setParticleColorAlphaSpeed:speed]; }

CGFloat SKEmitterNodeParticleColorBlueSpeed( SKEmitterNodeRef ref )
{ return [ref particleColorBlueSpeed]; }

void SKEmitterNodeSetParticleColorBlueSpeed( SKEmitterNodeRef ref, CGFloat speed )
{ [ref setParticleColorBlueSpeed:speed]; }

CGFloat SKEmitterNodeParticleColorGreenSpeed( SKEmitterNodeRef ref )
{ return [ref particleColorGreenSpeed]; }

void SKEmitterNodeSetParticleColorGreenSpeed( SKEmitterNodeRef ref, CGFloat speed )
{ [ref setParticleColorGreenSpeed:speed]; }

CGFloat SKEmitterNodeParticleColorRedSpeed( SKEmitterNodeRef ref )
{ return [ref particleColorRedSpeed]; }

void SKEmitterNodeSetParticleColorRedSpeed( SKEmitterNodeRef ref, CGFloat speed )
{ [ref setParticleColorRedSpeed:speed]; }

// Texture blend
SKKeyframeSequenceRef SKEmitterNodeParticleColorBlendFactorSequence( SKEmitterNodeRef ref )
{ return [ref particleColorBlendFactorSequence]; }

void SKEmitterNodeSetParticleColorBlendFactorSequence( SKEmitterNodeRef ref, SKKeyframeSequenceRef sequence )
{ [ref setParticleColorBlendFactorSequence:sequence]; }

CGFloat SKEmitterNodeParticleColorBlendFactor( SKEmitterNodeRef ref )
{ return [ref particleColorBlendFactor]; }

void SKEmitterNodeSetParticleColorBlendFactor( SKEmitterNodeRef ref, CGFloat factor )
{ [ref setParticleColorBlendFactor:factor]; }

CGFloat SKEmitterNodeParticleColorBlendFactorRange( SKEmitterNodeRef ref )
{ return [ref particleColorBlendFactorRange]; }

void SKEmitterNodeSetParticleColorBlendFactorRange( SKEmitterNodeRef ref, CGFloat range )
{ [ref setParticleColorBlendFactorRange:range]; }

CGFloat SKEmitterNodeParticleColorBlendFactorSpeed( SKEmitterNodeRef ref )
{ return [ref particleColorBlendFactorSpeed]; }

void SKEmitterNodeSetParticleColorBlendFactorSpeed( SKEmitterNodeRef ref, CGFloat speed )
{ [ref setParticleColorBlendFactorSpeed:speed]; }

// Framebuffer
SKBlendMode SKEmitterNodeParticleBlendMode( SKEmitterNodeRef ref )
{ return [ref particleBlendMode]; }

void SKEmitterNodeSetParticleBlendMode( SKEmitterNodeRef ref, SKBlendMode mode )
{ [ref setParticleBlendMode:mode]; }

SKKeyframeSequenceRef SKEmitterNodeParticleAlphaSequence( SKEmitterNodeRef ref )
{ return [ref particleAlphaSequence]; }

void SKEmitterNodeSetParticleAlphaSequence( SKEmitterNodeRef ref, SKKeyframeSequenceRef sequence )
{ [ref setParticleAlphaSequence:sequence]; }

CGFloat SKEmitterNodeParticleAlpha( SKEmitterNodeRef ref )
{ return [ref particleAlpha]; }

void SKEmitterNodeSetParticleAlpha( SKEmitterNodeRef ref, CGFloat alpha )
{ [ref setParticleAlpha:alpha]; }

CGFloat SKEmitterNodeParticleAlphaRange( SKEmitterNodeRef ref )
{ return [ref particleAlphaRange]; }

void SKEmitterNodeSetParticleAlphaRange( SKEmitterNodeRef ref, CGFloat range )
{ [ref setParticleAlphaRange:range]; }

CGFloat SKEmitterNodeParticleAlphaSpeed( SKEmitterNodeRef ref )
{ return [ref particleAlphaSpeed]; }

void SKEmitterNodeSetParticleAlphaSpeed( SKEmitterNodeRef ref, CGFloat speed )
{ [ref setParticleAlphaSpeed:speed]; }

// Animating
SKActionRef SKEmitterNodeParticleAction( SKEmitterNodeRef ref )
{ return [ref particleAction]; }

void SKEmitterNodeSetParticleAction( SKEmitterNodeRef ref, SKActionRef action )
{ [ref setParticleAction:action]; }

// Physics fields
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
UInt32 SKEmitterNodeFieldBitMask( SKEmitterNodeRef ref )
{ return [ref fieldBitMask]; }

void SKEmitterNodeSetFieldBitMask( SKEmitterNodeRef ref, UInt32 mask )
{ [ref setFieldBitMask:mask]; }
#endif

// Drawing
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKShaderRef SKEmitterNodeShader( SKEmitterNodeRef ref )
{ return [ref shader]; }

void SKEmitterNodeSetShader( SKEmitterNodeRef ref, SKShaderRef shader )
{ [ref setShader:shader]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFDictionaryRef SKEmitterNodeAttributeValues( SKEmitterNodeRef ref )
{ return (CFDictionaryRef)[ref attributeValues]; }

void SKEmitterNodeSetAttributeValues( SKEmitterNodeRef ref, CFDictionaryRef values )
{ [ref setAttributeValues:(NSDictionary *)values]; }

void SKEmitterNodeSetValueForAttributeNamed( SKEmitterNodeRef ref, SKAttributeValueRef value, CFStringRef name )
{ [ref setValue:value forAttributeNamed:(NSString *)name]; }

SKAttributeValueRef SKEmitterNodeValueForAttributeNamed( SKEmitterNodeRef ref, CFStringRef name )
{ return [ref valueForAttributeNamed:(NSString *)name]; }
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKFieldNode
///////////////////////////////////////////////////
//                 SKFieldNode                   //
///////////////////////////////////////////////////
// Create
SKFieldNodeRef SKFieldNodeDragField( void )
{ return [SKFieldNode dragField]; }

SKFieldNodeRef SKFieldNodeElectricField( void )
{ return [SKFieldNode electricField]; }

SKFieldNodeRef SKFieldNodeLinearGravityFieldWithVector( vector_float3 direction )
{ return [SKFieldNode linearGravityFieldWithVector:direction]; }

SKFieldNodeRef SKFieldNodeMagneticField( void )
{ return [SKFieldNode magneticField]; }

SKFieldNodeRef SKFieldNodeNoiseFieldWithSmoothness( CGFloat smoothness, CGFloat animationSpeed )
{ return [SKFieldNode noiseFieldWithSmoothness:smoothness animationSpeed:animationSpeed]; }

SKFieldNodeRef SKFieldNodeRadialGravityField( void )
{ return [SKFieldNode radialGravityField]; }

SKFieldNodeRef SKFieldNodeSpringField( void )
{ return [SKFieldNode springField]; }

SKFieldNodeRef SKFieldNodeTurbulenceFieldWithSmoothness( CGFloat smoothness, CGFloat animationSpeed )
{ return [SKFieldNode turbulenceFieldWithSmoothness:smoothness animationSpeed:animationSpeed]; }

SKFieldNodeRef SKFieldNodeVelocityFieldWithTexture( SKTextureRef texture )
{ return [SKFieldNode velocityFieldWithTexture:texture]; }

SKFieldNodeRef SKFieldNodeVelocityFieldWithVector( vector_float3 direction )
{ return [SKFieldNode velocityFieldWithVector:direction]; }

SKFieldNodeRef SKFieldNodeVortexField( void )
{ return [SKFieldNode vortexField]; }

SKFieldNodeRef SKFieldNodeCustomFieldWithEvaluationCallback( SKFieldNodeCustomFieldCallbackType callback, void *userData )
{
    return [SKFieldNode customFieldWithEvaluationBlock:^vector_float3(vector_float3 position, vector_float3 velocity, float mass, float charge, NSTimeInterval deltaTime) {
        return (*callback)(position,velocity,mass,charge,deltaTime,userData);
    }];
}

// Physics bodies affected by field
Boolean SKFieldNodeIsEnabled( SKFieldNodeRef ref )
{ return [ref isEnabled]; }

void SKFieldNodeSetEnabled( SKFieldNodeRef ref, Boolean flag )
{ [ref setEnabled:flag]; }

Boolean SKFieldNodeIsExclusive( SKFieldNodeRef ref )
{ return [ref isExclusive]; }

void SKFieldNodeSetExclusive( SKFieldNodeRef ref, Boolean flag )
{ [ref setExclusive:flag]; }

SKRegionRef SKFieldNodeRegion( SKFieldNodeRef ref )
{ return [ref region]; }

void SKFieldNodeSetRegion( SKFieldNodeRef ref, SKRegionRef region )
{ [ref setRegion:region]; }

float SKFieldNodeMinimumRadius( SKFieldNodeRef ref )
{ return [ref minimumRadius]; }

void SKFieldNodeSetMinimumRadius( SKFieldNodeRef ref, float radius )
{ [ref setMinimumRadius:radius]; }

UInt32 SKFieldNodeCategoryBitMask( SKFieldNodeRef ref )
{ return [ref categoryBitMask]; }

void SKFieldNodeSetCategoryBitMask( SKFieldNodeRef ref, UInt32 bitMask )
{ [ref setCategoryBitMask:bitMask]; }

float SKFieldNodeStrength( SKFieldNodeRef ref )
{ return [ref strength]; }

void SKFieldNodeSetStrength( SKFieldNodeRef ref, float strength )
{ [ref setStrength:strength]; }

float SKFieldNodeFalloff( SKFieldNodeRef ref )
{ return [ref falloff]; }

void SKFieldNodeSetFalloff( SKFieldNodeRef ref, float falloff )
{ [ref setFalloff:falloff]; }

// Other field properties
float SKFieldNodeAnimationSpeed( SKFieldNodeRef ref )
{ return [ref animationSpeed]; }

void SKFieldNodeSetAnimationSpeed( SKFieldNodeRef ref, float speed )
{ [ref setAnimationSpeed:speed]; }

float SKFieldNodeAnimationSmoothness( SKFieldNodeRef ref )
{ return [ref smoothness]; }

void SKFieldNodeSetAnimationSmoothness( SKFieldNodeRef ref, float smoothness )
{ [ref setSmoothness:smoothness]; }

vector_float3 SKFieldNodeDirection( SKFieldNodeRef ref )
{ return [ref direction]; }

void SKFieldNodeSetDirection( SKFieldNodeRef ref, vector_float3 direction )
{ [ref setDirection:direction]; }

SKTextureRef SKFieldNodeTexture( SKFieldNodeRef ref )
{ return [ref texture]; }

void SKFieldNodeSetTexture( SKFieldNodeRef ref, SKTextureRef texture )
{ [ref setTexture:texture]; }
#endif



#pragma mark - SKKeyframeSequence
///////////////////////////////////////////////////
//              SKKeyframeSequence               //
///////////////////////////////////////////////////
// Init
SKKeyframeSequenceRef SKKeyframeSequenceWithKeyframeValues( CFArrayRef values, CFArrayRef times )
{ return [[[SKKeyframeSequence alloc] initWithKeyframeValues:(NSArray *)values times:(NSArray *)times] autorelease]; }

SKKeyframeSequenceRef SKKeyframeSequenceWithCapacity( NSUInteger numItems )
{ return [[[SKKeyframeSequence alloc] initWithCapacity:numItems] autorelease]; }

SKKeyframeSequenceRef SKKeyframeSequenceWithCoder( CoderRef coder )
{ return [[[SKKeyframeSequence alloc] initWithCoder:coder] autorelease]; }

// Building
void SKKeyframeSequenceAddKeyframeValue( SKKeyframeSequenceRef ref, CFTypeRef value, CGFloat time )
{ [ref addKeyframeValue:(id)value time:time]; }

void SKKeyframeSequenceRemoveKeyframeAtIndex( SKKeyframeSequenceRef ref, NSUInteger index )
{ [ref removeKeyframeAtIndex:index]; }

void SKKeyframeSequenceRemoveLastKeyframe( SKKeyframeSequenceRef ref )
{ [ref removeLastKeyframe]; }

void SKKeyframeSequenceSetKeyframeTimeForIndex( SKKeyframeSequenceRef ref, CGFloat time, NSUInteger index )
{ [ref setKeyframeTime:time forIndex:index]; }

void SKKeyframeSequenceSetKeyframeValueForIndex( SKKeyframeSequenceRef ref, CFTypeRef value, NSUInteger index )
{ [ref setKeyframeValue:(id)value forIndex:index]; }

void SKKeyframeSequenceSetKeyframeValueAndTimeForIndex( SKKeyframeSequenceRef ref, CFTypeRef value, CGFloat time, NSUInteger index )
{ [ref setKeyframeValue:(id)value time:time forIndex:index]; }

// Running
CFTypeRef SKKeyframeSequenceSampleAtTime( SKKeyframeSequenceRef ref, CGFloat time )
{ return [ref sampleAtTime:time]; }

// Info
NSUInteger SKKeyframeSequenceCount( SKKeyframeSequenceRef ref )
{ return [ref count]; }

CGFloat SKKeyframeSequenceKeyframeTimeForIndex( SKKeyframeSequenceRef ref, NSUInteger index )
{ return [ref getKeyframeTimeForIndex:index]; }

CFTypeRef SKKeyframeSequenceKeyframeValueForIndex( SKKeyframeSequenceRef ref, NSUInteger index )
{ return (CFTypeRef)[ref getKeyframeValueForIndex:index]; }

// Interpolation modifiers
SKInterpolationMode SKKeyframeSequenceInterpolationMode( SKKeyframeSequenceRef ref )
{ return [ref interpolationMode]; }

void SKKeyframeSequenceSetInterpolationMode( SKKeyframeSequenceRef ref, SKInterpolationMode mode )
{ [ref setInterpolationMode:mode]; }

SKRepeatMode SKKeyframeSequenceRepeatMode( SKKeyframeSequenceRef ref )
{ return [ref repeatMode]; }

void SKKeyframeSequenceSetRepeatMode( SKKeyframeSequenceRef ref, SKRepeatMode mode )
{ [ref setRepeatMode:mode]; }



#pragma mark - SKLabelNode
///////////////////////////////////////////////////
//                 SKLabelNode                   //
///////////////////////////////////////////////////
// Create
SKLabelNodeRef SKLabelNodeWithFontNamed( CFStringRef name )
{ return [SKLabelNode labelNodeWithFontNamed:(NSString *)name]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKLabelNodeRef SKLabelNodeWithText( CFStringRef text )
{ return [SKLabelNode labelNodeWithText:(NSString *)text]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
SKLabelNodeRef SKLabelNodeWithAttributedText( CFAttributedStringRef aText )
{ return [SKLabelNode labelNodeWithAttributedText:(NSAttributedString *)aText]; }
#endif

// Text
CFStringRef SKLabelNodeText( SKLabelNodeRef ref )
{ return (CFStringRef)[ref text]; }

void SKLabelNodeSetText( SKLabelNodeRef ref, CFStringRef text )
{ [ref setText:(NSString *)text]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CFAttributedStringRef SKLabelNodeAttributedText( SKLabelNodeRef ref )
{ return (CFAttributedStringRef)[ref attributedText]; }

void SKLabelNodeSetAttributedText( SKLabelNodeRef ref, CFAttributedStringRef aText )
{ [ref setAttributedText:(NSAttributedString *)aText]; }
#endif

// Font
ColorRef SKLabelNodeFontColor( SKLabelNodeRef ref )
{ return [ref fontColor]; }

void SKLabelNodeSetFontColor( SKLabelNodeRef ref, ColorRef col )
{ [ref setFontColor:col]; }

CFStringRef SKLabelNodeFontName( SKLabelNodeRef ref )
{ return (CFStringRef)[ref fontName]; }

void SKLabelNodeSetFontName( SKLabelNodeRef ref, CFStringRef name )
{ [ref setFontName:(NSString *)name]; }

CGFloat SKLabelNodeFontSize( SKLabelNodeRef ref )
{ return [ref fontSize]; }

void SKLabelNodeSetFontSize( SKLabelNodeRef ref, CGFloat size )
{ [ref setFontSize:size]; }

// Alignment
SKLabelVerticalAlignmentMode SKLabelNodeVerticalAlignmentMode( SKLabelNodeRef ref )
{ return [ref verticalAlignmentMode]; }

void SKLabelNodeSetVerticalAlignmentMode( SKLabelNodeRef ref, SKLabelVerticalAlignmentMode mode )
{ [ref setVerticalAlignmentMode:mode]; }

SKLabelHorizontalAlignmentMode SKLabelNodeHorizontalAlignmentMode( SKLabelNodeRef ref )
{ return [ref horizontalAlignmentMode]; }

void SKLabelNodeSetHorizontalAlignmentMode( SKLabelNodeRef ref, SKLabelHorizontalAlignmentMode mode )
{ [ref setHorizontalAlignmentMode:mode]; }

// Line-break behavior
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
CGFloat SKLabelNodePreferredMaxLayoutWidth( SKLabelNodeRef ref )
{ return [ref preferredMaxLayoutWidth]; }
                                           
void SKLabelNodeSetPreferredMaxLayoutWidth( SKLabelNodeRef ref, CGFloat width )
{ [ref setPreferredMaxLayoutWidth:width]; }

NSLineBreakMode SKLabelNodeLineBreakMode( SKLabelNodeRef ref )
{ return [ref lineBreakMode]; }

void SKLabelNodeSetLineBreakMode( SKLabelNodeRef ref, NSLineBreakMode mode )
{ [ref setLineBreakMode:mode]; }

NSInteger SKLabelNodeNumberOfLines( SKLabelNodeRef ref )
{ return [ref numberOfLines]; }

void SKLabelNodeSetNumberOfLines( SKLabelNodeRef ref, NSInteger lines )
{ [ref setNumberOfLines:lines]; }
#endif

// Colorizing
ColorRef SKLabelNodeColor( SKLabelNodeRef ref )
{ return [ref color]; }

void SKLabelNodeSetColor( SKLabelNodeRef ref, ColorRef col )
{ [ref setColor:col]; }

CGFloat SKLabelNodeColorBlendFactor( SKLabelNodeRef ref )
{ return [ref colorBlendFactor]; }

void SKLabelNodeSetColorBlendFactor( SKLabelNodeRef ref, CGFloat factor )
{ [ref setColorBlendFactor:factor]; }

// Alpha blending
SKBlendMode SKLabelNodeBlendMode( SKLabelNodeRef ref )
{ return [ref blendMode]; }

void SKLabelNodeSetBlendMode( SKLabelNodeRef ref, SKBlendMode mode )
{ [ref setBlendMode:mode]; }



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKLightNode
///////////////////////////////////////////////////
//                 SKLightNode                   //
///////////////////////////////////////////////////
// Init
SKLightNodeRef SKLightNodeInit( void )
{ return [[[SKLightNode alloc] init] autorelease]; }

// Active
Boolean SKLightNodeIsEnabled( SKLightNodeRef ref )
{ return [ref isEnabled]; }

void SKLightNodeSetEnabled( SKLightNodeRef ref, Boolean flag )
{ [ref setEnabled:flag]; }

UInt32 SKLightNodeCategoryBitMask( SKLightNodeRef ref )
{ return [ref categoryBitMask]; }

void SKLightNodeSetCategoryBitMask( SKLightNodeRef ref, UInt32 bitMask )
{ [ref setCategoryBitMask:bitMask]; }

// Lighting properties
ColorRef SKLightNodeAmbientColor( SKLightNodeRef ref )
{ return [ref ambientColor]; }

void SKLightNodeSetAmbientColor( SKLightNodeRef ref, ColorRef col )
{ [ref setAmbientColor:col]; }

ColorRef SKLightNodeLightColor( SKLightNodeRef ref )
{ return [ref lightColor]; }

void SKLightNodeSetLightColor( SKLightNodeRef ref, ColorRef col )
{ [ref setLightColor:col]; }

ColorRef SKLightNodeShadowColor( SKLightNodeRef ref )
{ return [ref shadowColor]; }

void SKLightNodeSetShadowColor( SKLightNodeRef ref, ColorRef col )
{ [ref setShadowColor:col]; }

CGFloat SKLightNodeFalloff( SKLightNodeRef ref )
{ return [ref falloff]; }

void SKLightNodeSetFalloff( SKLightNodeRef ref, CGFloat falloff )
{ [ref setFalloff:falloff]; }




#pragma mark - SKMutableTexture
///////////////////////////////////////////////////
//              SKMutableTexture                 //
///////////////////////////////////////////////////
// Create
SKMutableTextureRef SKMutableTextureWithSize( CGSize size )
{ return [SKMutableTexture mutableTextureWithSize:size]; }

SKMutableTextureRef SKMutableTextureWithPixelFormat( CGSize size, UInt32 format )
{ return [[[SKMutableTexture alloc] initWithSize:size pixelFormat:format] autorelease]; }

// Modify
void SKMutableTextureModifyPixelData( SKMutableTextureRef ref, SKMutableTextureModifyPixelDataCallbackType callback, void *userData )
{
    [ref modifyPixelDataWithBlock:^(void *pixelData, size_t lengthInBytes) {
        (*callback)(ref,pixelData,lengthInBytes,userData);
    }];
}
#endif




#pragma mark - SKNode
///////////////////////////////////////////////////
//                   SKNode                      //
///////////////////////////////////////////////////
// First steps
SKNodeRef SKNodeInit( void )
{ return [SKNode node]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKNodeRef SKNodeWithFileNamed( CFStringRef name )
{ return [SKNode nodeWithFileNamed:(NSString *)name]; }
#endif

SKNodeRef SKNodeWithCoder( CoderRef coder )
{ return [[[SKNode alloc] initWithCoder:coder] autorelease]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
SKNodeRef SKNodeWithFileNamedSecurelyWithClasses( CFStringRef name, CFSetRef classes, ErrorRef *err )
{ return [SKNode nodeWithFileNamed:(NSString *)name securelyWithClasses:(NSSet *)classes andError:err]; }
#endif

// Position
CGPoint SKNodePosition( SKNodeRef ref )
{ return [ref position]; }

void SKNodeSetPosition( SKNodeRef ref, CGPoint pt )
{ [ref setPosition:pt]; }

// Content size
CGRect SKNodeFrame( SKNodeRef ref )
{ return [ref frame]; }

CGRect SKNodeCalculateAccumulatedFrame( SKNodeRef ref )
{ return [ref calculateAccumulatedFrame]; }

// Draw order
CGFloat SKNodeZPosition( SKNodeRef ref )
{ return [ref zPosition]; }

void SKNodeSetZPosition( SKNodeRef ref, CGFloat position )
{ [ref setZPosition:position]; }

// Scale + rotate
CGFloat SKNodeZRotation( SKNodeRef ref )
{ return [ref zRotation]; }

void SKNodeSetZRotation( SKNodeRef ref, CGFloat rotation )
{ [ref setZRotation:rotation]; }

void SKNodeSetScale( SKNodeRef ref, CGFloat scale )
{ [ref setScale:scale]; }

CGFloat SKNodeXScale( SKNodeRef ref )
{ return [ref xScale]; }

void SKNodeSetXScale( SKNodeRef ref, CGFloat scale )
{ [ref setXScale:scale]; }

CGFloat SKNodeYScale( SKNodeRef ref )
{ return [ref yScale]; }

void SKNodeSetYScale( SKNodeRef ref, CGFloat scale )
{ [ref setYScale:scale]; }

// Related nodes
SKSceneRef SKNodeScene( SKNodeRef ref )
{ return [ref scene]; }

SKNodeRef SKNodeParent( SKNodeRef ref )
{ return [ref parent]; }

CFArrayRef SKNodeChildren( SKNodeRef ref )
{ return (CFArrayRef)[ref children]; }

// Modify node tree
void SKNodeAddChild( SKNodeRef ref, SKNodeRef child )
{ [ref addChild:child]; }

void SKNodeInsertChild( SKNodeRef ref, SKNodeRef child, NSInteger index )
{ [ref insertChild:child atIndex:index]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean SKNodeIsEqualToNode( SKNodeRef ref, SKNodeRef otherNode )
{ return [ref isEqualToNode:otherNode]; }

void SKNodeMoveToParent( SKNodeRef ref, SKNodeRef parent )
{ [ref moveToParent:parent]; }
#endif

void SKNodeRemoveFromParent( SKNodeRef ref )
{ [ref removeFromParent]; }

void SKNodeRemoveAllChildren( SKNodeRef ref )
{ [ref removeAllChildren]; }

void SKNodeRemoveChildrenInArray( SKNodeRef ref, CFArrayRef children )
{ [ref removeChildrenInArray:(NSArray *)children]; }

Boolean SKNodeInParentHierarchy( SKNodeRef ref, SKNodeRef parent )
{ return [ref inParentHierarchy:parent]; }

// Accessing by name
CFStringRef SKNodeName( SKNodeRef ref )
{ return (CFStringRef)[ref name]; }

void SKNodeSetName( SKNodeRef ref, CFStringRef name )
{ [ref setName:(NSString *)name]; }

SKNodeRef SKNodeChildNodeWithName( SKNodeRef ref, CFStringRef name )
{ return [ref childNodeWithName:(NSString *)name]; }

void SKNodeEnumerateChildNodesWithName( SKNodeRef ref, CFStringRef name, SKNodeEnumerateChildNodesWithNameCallbackType callback, void *userData )
{
    [ref enumerateChildNodesWithName:(NSString *)name usingBlock:^(SKNode *node, BOOL *stop) {
        BOOL flag = (*callback)(ref,node,userData);
        if ( flag ) *stop = YES;
    }];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CFArrayRef SKNodeObjectForKeyedSubscript( SKNodeRef ref, CFStringRef name )
{ return (CFArrayRef)[ref objectForKeyedSubscript:(NSString *)name]; }
#endif

// Visibility
CGFloat SKNodeAlpha( SKNodeRef ref )
{ return [ref alpha]; }

void SKNodeSetAlpha( SKNodeRef ref, CGFloat alpha )
{ [ref setAlpha:alpha]; }

Boolean SKNodeIsHidden( SKNodeRef ref )
{ return [ref isHidden]; }

void SKNodeSetHidden( SKNodeRef ref, Boolean flag )
{ [ref setHidden:flag]; }

// Running actions
void SKNodeRunAction( SKNodeRef ref, SKActionRef action )
{ [ref runAction:action]; }

void SKNodeRunActionCompletion( SKNodeRef ref, SKActionRef action, SKNodeRunActionCompletionCallbackType callback, void *userData )
{
    [ref runAction:action completion:^(void) {
        (*callback)(ref,action,userData);
    }];
}

void SKNodeRunActionWithKey( SKNodeRef ref, SKActionRef action, CFStringRef key )
{ [ref runAction:action withKey:(NSString *)key]; }

CGFloat SKNodeSpeed( SKNodeRef ref )
{ return [ref speed]; }

void SKNodeSetSpeed( SKNodeRef ref, CGFloat speed )
{ [ref setSpeed:speed]; }

Boolean SKNodeIsPaused( SKNodeRef ref )
{ return [ref isPaused]; }

void SKNodeSetPaused( SKNodeRef ref, Boolean flag )
{ [ref setPaused:flag]; }

SKActionRef SKNodeActionForKey( SKNodeRef ref, CFStringRef key )
{ return [ref actionForKey:(NSString *)key]; }

Boolean SKNodeHasActions( SKNodeRef ref )
{ return [ref hasActions]; }

void SKNodeRemoveAllActions( SKNodeRef ref )
{ [ref removeAllActions]; }

void SKNodeRemoveActionForKey( SKNodeRef ref, CFStringRef key )
{ [ref removeActionForKey:(NSString *)key]; }

// Physics behaviors
SKPhysicsBodyRef SKNodePhysicsBody( SKNodeRef ref )
{ return [ref physicsBody]; }

void SKNodeSetPhysicsBody( SKNodeRef ref, SKPhysicsBodyRef body )
{ [ref setPhysicsBody:body]; }

// Constraining
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CFArrayRef SKNodeConstraints( SKNodeRef ref )
{ return (CFArrayRef)[ref constraints]; }

void SKNodeSetConstraints( SKNodeRef ref, CFArrayRef constraints )
{ [ref setConstraints:(NSArray *)constraints]; }

SKReachConstraintsRef SKNodeReachConstraints( SKNodeRef ref )
{ return [ref reachConstraints]; }

void SKNodeSetReachConstraints( SKNodeRef ref, SKReachConstraintsRef constraints )
{ [ref setReachConstraints:constraints]; }
#endif

// Collisions
Boolean SKNodeIntersectsNode( SKNodeRef ref, SKNodeRef otherNode )
{ return [ref intersectsNode:otherNode]; }

// User input
Boolean SKNodeIsUserInteractionEnabled( SKNodeRef ref )
{ return [ref isUserInteractionEnabled]; }

void SKNodeSetUserInteractionEnabled( SKNodeRef ref, Boolean flag )
{ [ref setUserInteractionEnabled:flag]; }

// Hit testing
Boolean SKNodeContainsPoint( SKNodeRef ref, CGPoint pt )
{ return [ref containsPoint:pt]; }

SKNodeRef SKNodeNodeAtPoint( SKNodeRef ref, CGPoint pt )
{ return [ref nodeAtPoint:pt]; }

CFArrayRef SKNodeNodesAtPoint( SKNodeRef ref, CGPoint pt )
{ return (CFArrayRef)[ref nodesAtPoint:pt]; }

// Converting between coordinate systems
CGPoint SKNodeConvertPointFromNode( SKNodeRef ref, CGPoint pt, SKNodeRef otherNode )
{ return [ref convertPoint:pt fromNode:otherNode]; }

CGPoint SKNodeConvertPointToNode( SKNodeRef ref, CGPoint pt, SKNodeRef otherNode )
{ return [ref convertPoint:pt toNode:otherNode]; }

// Custom data
CFMutableDictionaryRef SKNodeUserData( SKNodeRef ref )
{ return (CFMutableDictionaryRef)[ref userData]; }

void SKNodeSetUserData( SKNodeRef ref, CFMutableDictionaryRef userData )
{ [ref setUserData:(NSMutableDictionary *)userData]; }

// Accessibility
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFArrayRef SKNodeAccessibilityChildren( SKNodeRef ref )
{ return (CFArrayRef)[ref accessibilityChildren]; }

void SKNodeSetAccessibilityChildren( SKNodeRef ref, CFArrayRef children )
{ [ref setAccessibilityChildren:(NSArray *)children]; }

CGRect SKNodeAccessibilityFrame( SKNodeRef ref )
{ return [ref accessibilityFrame]; }

void SKNodeSetAccessibilityFrame( SKNodeRef ref, CGRect frame )
{ [ref setAccessibilityFrame:frame]; }

CFStringRef SKNodeAccessibilityHelp( SKNodeRef ref )
{ return (CFStringRef)[ref accessibilityHelp]; }

void SKNodeSetAccessibilityHelp( SKNodeRef ref, CFStringRef help )
{ [ref setAccessibilityHelp:(NSString *)help]; }

CFStringRef SKNodeAccessibilityLabel( SKNodeRef ref )
{ return (CFStringRef)[ref accessibilityLabel]; }

void SKNodeSetAccessibilityLabel( SKNodeRef ref, CFStringRef label )
{ [ref setAccessibilityHelp:(NSString *)label]; }

CFTypeRef SKNodeAccessibilityParent( SKNodeRef ref )
{ return (CFTypeRef)[ref accessibilityParent]; }

void SKNodeSetAccessibilityParent( SKNodeRef ref, CFTypeRef parent )
{ [ref setAccessibilityParent:(id)parent]; }

CFStringRef SKNodeAccessibilityRole( SKNodeRef ref )
{ return (CFStringRef)[ref accessibilityRole]; }

void SKNodeSetAccessibilityRole( SKNodeRef ref, CFStringRef role )
{ [ref setAccessibilityRole:(NSString *)role]; }

CFStringRef SKNodeAccessibilityRoleDescription( SKNodeRef ref )
{ return (CFStringRef)[ref accessibilityRoleDescription]; }

void SKNodeSetAccessibilityRoleDescription( SKNodeRef ref, CFStringRef description )
{ [ref setAccessibilityRoleDescription:(NSString *)description]; }

CFStringRef SKNodeAccessibilitySubrole( SKNodeRef ref )
{ return (CFStringRef)[ref accessibilitySubrole]; }

void SKNodeSetAccessibilitySubrole( SKNodeRef ref, CFStringRef subrole )
{ [ref setAccessibilitySubrole:(NSString *)subrole]; }

Boolean SKNodeIsAccessibilityElement( SKNodeRef ref )
{ return [ref isAccessibilityElement]; }

void SKNodeSetAccessibilityElement( SKNodeRef ref, Boolean flag )
{ [ref setAccessibilityElement:flag]; }

Boolean SKNodeIsAccessibilityEnabled( SKNodeRef ref )
{ return [ref isAccessibilityEnabled]; }

void SKNodeSetAccessibilityEnabled( SKNodeRef ref, Boolean flag )
{ [ref setAccessibilityEnabled:flag]; }

CFTypeRef SKNodeAccessibilityHitTest( SKNodeRef ref, CGPoint pt )
{ return (CFTypeRef)[ref accessibilityHitTest:pt]; }
#endif

#pragma mark - SKPhysicsBody
///////////////////////////////////////////////////
//                SKPhysicsBody                  //
///////////////////////////////////////////////////
// Create from shape
SKPhysicsBodyRef SKPhysicsBodyWithCircle( CGFloat radius )
{ return [SKPhysicsBody bodyWithCircleOfRadius:radius]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKPhysicsBodyRef SKPhysicsBodyWithCircleCenter( CGFloat radius, CGPoint pt )
{ return [SKPhysicsBody bodyWithCircleOfRadius:radius center:pt]; }
#endif

SKPhysicsBodyRef SKPhysicsBodyWithRectangle( CGSize size )
{ return [SKPhysicsBody bodyWithRectangleOfSize:size]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKPhysicsBodyRef SKPhysicsBodyWithRectangleCenter( CGSize size, CGPoint pt )
{ return [SKPhysicsBody bodyWithRectangleOfSize:size center:pt]; }
#endif
SKPhysicsBodyRef SKPhysicsBodyWithPolygon( CGPathRef path )
{ return [SKPhysicsBody bodyWithPolygonFromPath:path]; }

// Create from texture
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKPhysicsBodyRef SKPhysicsBodyWithTexture( SKTextureRef texture, CGSize size )
{ return [SKPhysicsBody bodyWithTexture:texture size:size]; }

SKPhysicsBodyRef SKPhysicsBodyWithTextureAlphaThreshold( SKTextureRef texture, float alphaThreshold, CGSize size )
{ return [SKPhysicsBody bodyWithTexture:texture alphaThreshold:alphaThreshold size:size]; }

// Create from collection of bodies
SKPhysicsBodyRef SKPhysicsBodyWithBodies( CFArrayRef bodies )
{ return [SKPhysicsBody bodyWithBodies:(NSArray *)bodies]; }
#endif

// Create edge-based
SKPhysicsBodyRef SKPhysicsBodyWithEdgeLoopFromRect( CGRect r )
{ return [SKPhysicsBody bodyWithEdgeLoopFromRect:r]; }

SKPhysicsBodyRef SKPhysicsBodyWithEdgeFromPoint( CGPoint fromPt, CGPoint toPt )
{ return [SKPhysicsBody bodyWithEdgeFromPoint:fromPt toPoint:toPt]; }

SKPhysicsBodyRef SKPhysicsBodyWithEdgeLoopFromPath( CGPathRef path )
{ return [SKPhysicsBody bodyWithEdgeLoopFromPath:path]; }

SKPhysicsBodyRef SKPhysicsBodyWithEdgeChainFromPath( CGPathRef path )
{ return [SKPhysicsBody bodyWithEdgeChainFromPath:path]; }

// Forces
Boolean SKPhysicsBodyAffectedByGravity( SKPhysicsBodyRef ref )
{ return [ref affectedByGravity]; }

void SKPhysicsBodySetAffectedByGravity( SKPhysicsBodyRef ref, Boolean flag )
{ [ref setAffectedByGravity:flag]; }

Boolean SKPhysicsBodyAllowsRotation( SKPhysicsBodyRef ref )
{ return [ref allowsRotation]; }

void SKPhysicsBodySetAllowsRotation( SKPhysicsBodyRef ref, Boolean flag )
{ [ref setAllowsRotation:flag]; }

Boolean SKPhysicsBodyIsDynamic( SKPhysicsBodyRef ref )
{ return [ref isDynamic]; }

void SKPhysicsBodySetDynamic( SKPhysicsBodyRef ref, Boolean flag )
{ [ref setDynamic:flag]; }

// Physical properties
CGFloat SKPhysicsBodyMass( SKPhysicsBodyRef ref )
{ return [ref mass]; }

void SKPhysicsBodySetMass( SKPhysicsBodyRef ref, CGFloat mass )
{ [ref setMass:mass]; }

CGFloat SKPhysicsBodyDensity( SKPhysicsBodyRef ref )
{ return [ref density]; }

void SKPhysicsBodySetDensity( SKPhysicsBodyRef ref, CGFloat density )
{ [ref setDensity:density]; }

CGFloat SKPhysicsBodyArea( SKPhysicsBodyRef ref )
{ return [ref area]; }

CGFloat SKPhysicsBodyFriction( SKPhysicsBodyRef ref )
{ return [ref friction]; }

void SKPhysicsBodySetFriction( SKPhysicsBodyRef ref, CGFloat friction )
{ [ref setFriction:friction]; }

CGFloat SKPhysicsBodyRestitution( SKPhysicsBodyRef ref )
{ return [ref restitution]; }

void SKPhysicsBodySetRestitution( SKPhysicsBodyRef ref, CGFloat restitution )
{ [ref setRestitution:restitution]; }

CGFloat SKPhysicsBodyLinearDamping( SKPhysicsBodyRef ref )
{ return [ref linearDamping]; }

void SKPhysicsBodySetLinearDamping( SKPhysicsBodyRef ref, CGFloat damping )
{ [ref setLinearDamping:damping]; }

CGFloat SKPhysicsBodyAngularDamping( SKPhysicsBodyRef ref )
{ return [ref angularDamping]; }

void SKPhysicsBodySetAngularDamping( SKPhysicsBodyRef ref, CGFloat damping )
{ [ref setAngularDamping:damping]; }

// Collisions and contacts
UInt32 SKPhysicsBodyCategoryBitMask( SKPhysicsBodyRef ref )
{ return [ref categoryBitMask]; }

void SKPhysicsBodySetCategoryBitMask( SKPhysicsBodyRef ref, UInt32 bitMask )
{ [ref setCategoryBitMask:bitMask]; }

UInt32 SKPhysicsBodyCollisionBitMask( SKPhysicsBodyRef ref )
{ return [ref collisionBitMask]; }

void SKPhysicsBodySetCollisionBitMask( SKPhysicsBodyRef ref, UInt32 bitMask )
{ [ref setCollisionBitMask:bitMask]; }

Boolean SKPhysicsBodyUsesPreciseCollisionDetection( SKPhysicsBodyRef ref )
{ return [ref usesPreciseCollisionDetection]; }

void SKPhysicsBodySetUsesPreciseCollisionDetection( SKPhysicsBodyRef ref, Boolean flag )
{ [ref setUsesPreciseCollisionDetection:flag]; }

UInt32 SKPhysicsBodyContactTestBitMask( SKPhysicsBodyRef ref )
{ return [ref contactTestBitMask]; }

void SKPhysicsBodySetContactTestBitMask( SKPhysicsBodyRef ref, UInt32 bitMask )
{ [ref setContactTestBitMask:bitMask]; }

CFArrayRef SKPhysicsBodyAllContactedBodies( SKPhysicsBodyRef ref )
{ return (CFArrayRef)[ref allContactedBodies]; }

// Forces and impulses
void SKPhysicsBodyApplyForce( SKPhysicsBodyRef ref, CGVector force )
{ [ref applyForce:force]; }

void SKPhysicsBodyApplyTorque( SKPhysicsBodyRef ref, CGFloat torque )
{ [ref applyTorque:torque]; }

void SKPhysicsBodyApplyForceAtPoint( SKPhysicsBodyRef ref, CGVector force, CGPoint pt )
{ [ref applyForce:force atPoint:pt]; }

void SKPhysicsBodyApplyImpulse( SKPhysicsBodyRef ref, CGVector impulse )
{ [ref applyImpulse:impulse]; }

void SKPhysicsBodyApplyAngularImpulse( SKPhysicsBodyRef ref, CGFloat impulse )
{ [ref applyAngularImpulse:impulse]; }

void SKPhysicsBodyApplyImpulseAtPoint( SKPhysicsBodyRef ref, CGVector impulse, CGPoint pt )
{ [ref applyImpulse:impulse atPoint:pt]; }

// Position and velocity
CGVector SKPhysicsBodyVelocity( SKPhysicsBodyRef ref )
{ return [ref velocity]; }

void SKPhysicsBodySetVelocity( SKPhysicsBodyRef ref, CGVector velocity )
{ [ref setVelocity:velocity]; }

CGFloat SKPhysicsBodyAngularVelocity( SKPhysicsBodyRef ref )
{ return [ref angularVelocity]; }

void SKPhysicsBodySetAngularVelocity( SKPhysicsBodyRef ref, CGFloat velocity )
{ [ref setAngularVelocity:velocity]; }

Boolean SKPhysicsBodyIsResting( SKPhysicsBodyRef ref )
{ return [ref isResting]; }

void SKPhysicsBodySetResting( SKPhysicsBodyRef ref, Boolean flag )
{ [ref setResting:flag]; }

// Node
SKNodeRef SKPhysicsBodyNode( SKPhysicsBodyRef ref )
{ return [ref node]; }

// Joints
CFArrayRef SKPhysicsBodyJoints( SKPhysicsBodyRef ref )
{ return (CFArrayRef)[ref joints]; }

// Fiels
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
UInt32 SKPhysicsBodyFieldBitMask( SKPhysicsBodyRef ref )
{ return [ref fieldBitMask]; }

void SKPhysicsBodySetFieldBitMask( SKPhysicsBodyRef ref, UInt32 bitMask )
{ [ref setFieldBitMask:bitMask]; }

CGFloat SKPhysicsBodyCharge( SKPhysicsBodyRef ref )
{ return [ref charge]; }

void SKPhysicsBodySetCharge( SKPhysicsBodyRef ref, CGFloat charge )
{ [ref setCharge:charge]; }

// Pinning to node parent
Boolean SKPhysicsBodyPinned( SKPhysicsBodyRef ref )
{ return [ref pinned]; }

void SKPhysicsBodySetPinned( SKPhysicsBodyRef ref, Boolean flag )
{ [ref setPinned:flag]; }
#endif

// NSEvent (SKNodeEvent)
CGPoint SKNodeLocationInNode( SKNodeRef ref )
{ return [[NSApp currentEvent] locationInNode:ref]; }


#pragma mark - SKPhysicsContact
///////////////////////////////////////////////////
//              SKPhysicsContact                 //
//////////////////////////////////////////////////
// Properties
SKPhysicsBodyRef SKPhysicsContactBodyA( SKPhysicsContactRef ref )
{ return [ref bodyA]; }

SKPhysicsBodyRef SKPhysicsContactBodyB( SKPhysicsContactRef ref )
{ return [ref bodyB]; }

CGPoint SKPhysicsContactPoint( SKPhysicsContactRef ref )
{ return [ref contactPoint]; }

CGFloat SKPhysicsContactCollisionImpulse( SKPhysicsContactRef ref )
{ return [ref collisionImpulse]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGVector SKPhysicsContactNormal( SKPhysicsContactRef ref )
{ return [ref contactNormal]; }
#endif



#pragma mark - SKPhysicsJoint
///////////////////////////////////////////////////
//                SKPhysicsJoint                 //
///////////////////////////////////////////////////
// Bodies
SKPhysicsBodyRef SKPhysicsJointBodyA( SKPhysicsJointRef ref )
{ return [ref bodyA]; }

void SKPhysicsJointSetBodyA( SKPhysicsJointRef ref, SKPhysicsBodyRef body )
{ [ref setBodyA:body]; }

SKPhysicsBodyRef SKPhysicsJointBodyB( SKPhysicsJointRef ref )
{ return [ref bodyB]; }

void SKPhysicsJointSetBodyB( SKPhysicsJointRef ref, SKPhysicsBodyRef body )
{ [ref setBodyB:body]; }

// Stress and speed
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGVector SKPhysicsJointReactionForce( SKPhysicsJointRef ref )
{ return [ref reactionForce]; }

CGFloat SKPhysicsJointReactionTorque( SKPhysicsJointRef ref )
{ return [ref reactionTorque]; }
#endif


#pragma mark - SKPhysicsJointFixed
///////////////////////////////////////////////////
//           SKPhysicsJointFixed                 //
///////////////////////////////////////////////////
// Create
SKPhysicsJointFixedRef SKPhysicsJointFixedWithBodyAB( SKPhysicsBodyRef bodyA, SKPhysicsBodyRef bodyB, CGPoint anchor )
{ return [SKPhysicsJointFixed jointWithBodyA:bodyA bodyB:bodyB anchor:anchor]; }


#pragma mark - SKPhysicsJointLimit
///////////////////////////////////////////////////
//             SKPhysicsJointLimit               //
///////////////////////////////////////////////////
// Create
SKPhysicsJointLimitRef SKPhysicsJointLimitWithBodyAB( SKPhysicsBodyRef bodyA, SKPhysicsBodyRef bodyB, CGPoint anchorA, CGPoint anchorB )
{ return [SKPhysicsJointLimit jointWithBodyA:bodyA bodyB:bodyB anchorA:anchorA anchorB:anchorB]; }


// Configure
CGFloat SKPhysicsJointLimitMaxLength( SKPhysicsJointLimitRef ref )
{ return [ref maxLength]; }

void SKPhysicsJointLimitSetMaxLength( SKPhysicsJointLimitRef ref, CGFloat length )
{ [ref setMaxLength:length]; }


#pragma mark - SKPhysicsJointPin
///////////////////////////////////////////////////
//               SKPhysicsJointPin               //
///////////////////////////////////////////////////
// Create
SKPhysicsJointPinRef SKPhysicsJointPinWithBodyAB( SKPhysicsBodyRef bodyA, SKPhysicsBodyRef bodyB, CGPoint anchor )
{ return [SKPhysicsJointPin jointWithBodyA:bodyA bodyB:bodyB anchor:anchor]; }

// Configure
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGFloat SKPhysicsJointPinRotationSpeed( SKPhysicsJointPinRef ref )
{ return [ref rotationSpeed]; }

void SKPhysicsJointPinSetRotationSpeed( SKPhysicsJointPinRef ref, CGFloat speed )
{ [ref setRotationSpeed:speed]; }
#endif

Boolean SKPhysicsJointPinShouldEnableLimits( SKPhysicsJointPinRef ref )
{ return [ref shouldEnableLimits]; }

void SKPhysicsJointPinSetShouldEnableLimits( SKPhysicsJointPinRef ref, Boolean flag )
{ [ref setShouldEnableLimits:flag]; }

CGFloat SKPhysicsJointPinLowerAngleLimit( SKPhysicsJointPinRef ref )
{ return [ref lowerAngleLimit]; }

void SKPhysicsJointPinSetLowerAngleLimit( SKPhysicsJointPinRef ref, CGFloat limit )
{ [ref setLowerAngleLimit:limit]; }

CGFloat SKPhysicsJoingPinUpperAngleLimit( SKPhysicsJointPinRef ref )
{ return [ref upperAngleLimit]; }

void SKPhysicsJointPinSetUpperAngleLimit( SKPhysicsJointPinRef ref, CGFloat limit )
{ [ref setUpperAngleLimit:limit]; }

CGFloat SKPhysicsJoingPinFrictionTorgue( SKPhysicsJointPinRef ref )
{ return [ref frictionTorque]; }

void SKPhysicsJointPinSetFrictionTorque( SKPhysicsJointPinRef ref, CGFloat torque )
{ [ref setFrictionTorque:torque]; }



#pragma mark - SKPhysicsJointSliding
///////////////////////////////////////////////////
//             SKPhysicsJointSliding             //
///////////////////////////////////////////////////
// Create
SKPhysicsJointSlidingRef SKPhysicsJointSlidingWithBodyAB( SKPhysicsBodyRef bodyA, SKPhysicsBodyRef bodyB, CGPoint anchor, CGVector axis )
{ return [SKPhysicsJointSliding jointWithBodyA:bodyA bodyB:bodyB anchor:anchor axis:axis]; }

// Configure
Boolean SKPhysicsJointSlidingShouldEnableLimits( SKPhysicsJointSlidingRef ref )
{ return [ref shouldEnableLimits]; }

void SKPhysicsJointSlidingSetShouldEnableLimits( SKPhysicsJointSlidingRef ref, Boolean flag )
{ [ref setShouldEnableLimits:flag]; }

CGFloat SKPhysicsJointSlidingLowerDistanceLimit( SKPhysicsJointSlidingRef ref )
{ return [ref lowerDistanceLimit]; }

void SKPhysicsJointSlidingSetLowerDistanceLimit( SKPhysicsJointSlidingRef ref, CGFloat limit )
{ [ref setLowerDistanceLimit:limit]; }

CGFloat SKPhysicsJointSlidingUpperDistanceLimit( SKPhysicsJointSlidingRef ref )
{ return [ref upperDistanceLimit]; }

void SKPhysicsJointSlidingSetUpperDistanceLimit( SKPhysicsJointSlidingRef ref, CGFloat limit )
{ [ref setUpperDistanceLimit:limit]; }




#pragma mark - SKPhysicsJointSpring
///////////////////////////////////////////////////
//             SKPhysicsJointSpring              //
///////////////////////////////////////////////////
// Create
SKPhysicsJointSpringRef SKPhysicsJointSpringWithBodyAB( SKPhysicsBodyRef bodyA, SKPhysicsBodyRef bodyB, CGPoint anchorA, CGPoint anchorB )
{ return [SKPhysicsJointSpring jointWithBodyA:bodyA bodyB:bodyB anchorA:anchorA anchorB:anchorB]; }

// Configure
CGFloat SKPhysicsJointSpringDamping( SKPhysicsJointSpringRef ref )
{ return [ref damping]; }

void SKPhysicsJointSpringSetDamping( SKPhysicsJointSpringRef ref, CGFloat damping )
{ [ref setDamping:damping]; }

CGFloat SKPhysicsJointSpringFrequency( SKPhysicsJointSpringRef ref )
{ return [ref frequency]; }

void SKPhysicsJointSpringSetFrequency( SKPhysicsJointSpringRef ref, CGFloat frequency )
{ [ref setFrequency:frequency]; }



#pragma mark - SKPhysicsWorld
///////////////////////////////////////////////////
//                SKPhysicsWorld                 //
///////////////////////////////////////////////////
@implementation CocoaSKPhysicsContactDelegate

@synthesize callback, userData;

- (id)initWithCallback:(void *)cb userData:(void*)ud {
    if ( self = [super init] ) {
        self.callback = cb;
        self.userData = ud;
    }
    return self;
}

- (void)didBeginContact:(SKPhysicsContact *)contact {
    if ( callback ) (*callback)( skPhysicsContactDidBegin, contact, userData );
}

- (void)didEndContact:(SKPhysicsContact *)contact {
    if ( callback ) (*callback)( skPhysicsContactDidEnd, contact, userData );
}

@end

// Configure
CGVector SKPhysicsWorldGravity( SKPhysicsWorldRef ref )
{ return [ref gravity]; }

void SKPhysicsWorldSetGravity( SKPhysicsWorldRef ref, CGVector gravity )
{ [ref setGravity:gravity]; }

CGFloat SKPhysicsWorldSpeed( SKPhysicsWorldRef ref )
{ return [ref speed]; }

void SKPhysicsWorldSetSpeed( SKPhysicsWorldRef ref, CGFloat speed )
{ [ref setSpeed:speed]; }

// Joints
void SKPhysicsWorldAddJoint( SKPhysicsWorldRef ref, SKPhysicsJointRef joint )
{ [ref addJoint:joint]; }

void SKPhysicsWorldRemoveAllJoints( SKPhysicsWorldRef ref )
{ [ref removeAllJoints]; }

void SKPhysicsWorldRemoveJoint( SKPhysicsWorldRef ref, SKPhysicsJointRef joint )
{ [ref removeJoint:joint]; }

// Collisions
void SKPhysicsWorldSetContactDelegateCallback( SKPhysicsWorldRef ref, SKPhysicsContactDelegateCallbackType callback, void *userData )
{
    CocoaSKPhysicsContactDelegate *delegate = [[CocoaSKPhysicsContactDelegate alloc] initWithCallback:callback userData:userData];
    [ref setContactDelegate:delegate];
}

// Searching scene for bodies
SKPhysicsBodyRef SKPhysicsWorldBodyAlongRay( SKPhysicsWorldRef ref, CGPoint startPt, CGPoint endPt )
{ return [ref bodyAlongRayStart:startPt end:endPt]; }

SKPhysicsBodyRef SKPhysicsWorldBodyAtPoint( SKPhysicsWorldRef ref, CGPoint pt )
{ return [ref bodyAtPoint:pt]; }

SKPhysicsBodyRef SKPhysicsWorldBodyInRect( SKPhysicsWorldRef ref, CGRect r )
{ return [ref bodyInRect:r]; }

void SKPhysicsWorldEnumerateBodiesAlongRay( SKPhysicsWorldRef ref, CGPoint startPt, CGPoint endPt, SKPhysicsWorldEnumerateBodiesAlongRayCallbackType callback, void *userData )
{
    [ref enumerateBodiesAlongRayStart:startPt end:endPt usingBlock:^(SKPhysicsBody *body, CGPoint point, CGVector normal, BOOL *stop) {
        BOOL flag = (*callback)(ref,body,point,normal,userData);
        if ( flag ) *stop = YES;
    }];
}

void SKPhysicsWorldEnumerateBodiesAtPoint( SKPhysicsWorldRef ref, CGPoint pt, SKPhysicsWorldEnumerateBodiesAtPointCallbackType callback, void *userData )
{
    [ref enumerateBodiesAtPoint:pt usingBlock:^(SKPhysicsBody *body, BOOL *stop) {
        BOOL flag = (*callback)(ref,body,userData);
        if ( flag ) *stop = YES;
    }];
}

void SKPhysicsWorldEnumerateBodiesInRect( SKPhysicsWorldRef ref, CGRect r, SKPhysicsWorldEnumerateBodiesInRectCallbackType callback, void *userData )
{
    [ref enumerateBodiesInRect:r usingBlock:^(SKPhysicsBody *body, BOOL *stop) {
        BOOL flag = (*callback)(ref,body,userData);
        if ( flag ) *stop = YES;
    }];
}

// Sampling fields
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
vector_float3 SKPhysicsWorldSampleFields( SKPhysicsWorldRef ref, vector_float3 position )
{ return [ref sampleFieldsAt:position]; }
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKReachConstraints
///////////////////////////////////////////////////
//             SKReachConstraints                //
///////////////////////////////////////////////////
// Create
SKReachConstraintsRef SKReachConstraintsWithAngleLimits( CGFloat lowerAngleLimit, CGFloat upperAngleLimit )
{ return [[[SKReachConstraints alloc] initWithLowerAngleLimit:lowerAngleLimit upperAngleLimit:upperAngleLimit] autorelease]; }

// Angle limits
CGFloat SKReachConstraintsLowerAngleLimit( SKReachConstraintsRef ref )
{ return [ref lowerAngleLimit]; }

void SKReachConstraintsSetLowerAngleLimit( SKReachConstraintsRef ref, CGFloat angleLimit )
{ [ref setLowerAngleLimit:angleLimit]; }

CGFloat SKReachConstraintsUpperAngleLimit( SKReachConstraintsRef ref )
{ return [ref upperAngleLimit]; }

void SKReachConstraintsSetUpperAngleLimit( SKReachConstraintsRef ref, CGFloat angleLimit )
{ [ref setUpperAngleLimit:angleLimit]; }
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
#pragma mark - SKReferenceNode
///////////////////////////////////////////////////
//               SKReferenceNode                 //
///////////////////////////////////////////////////
// Init
SKReferenceNodeRef SKReferenceNodeWithURL( CFURLRef url )
{ return [SKReferenceNode referenceNodeWithURL:(NSURL *)url]; }

SKReferenceNodeRef SKReferenceNodeWithFileNamed( CFStringRef name )
{ return [SKReferenceNode referenceNodeWithFileNamed:(NSString *)name]; }

SKReferenceNodeRef SKReferenceNodeWithCoder( CoderRef coder )
{ return [[[SKReferenceNode alloc] initWithCoder:coder] autorelease]; }

// Regenerating
void SKReferenceNodeResolve( SKReferenceNodeRef ref )
{ return [ref resolveReferenceNode]; }

// Loading
void SKReferenceNodeDidLoad( SKReferenceNodeRef ref, SKNodeRef node )
{ [ref didLoadReferenceNode:node]; }
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKRegion
///////////////////////////////////////////////////
//                  SKRegion                     //
///////////////////////////////////////////////////
// Create
SKRegionRef SKRegionInfinite( void )
{ return [SKRegion infiniteRegion]; }

SKRegionRef SKRegionWithSize( CGSize size )
{ return [[[SKRegion alloc] initWithSize:size] autorelease]; }

SKRegionRef SKRegionWithRadius( float radius )
{ return [[[SKRegion alloc] initWithRadius:radius] autorelease]; }

SKRegionRef SKRegionWithPath( CGPathRef path )
{ return [[[SKRegion alloc] initWithPath:path] autorelease]; }

SKRegionRef SKRegionInverse( SKRegionRef ref )
{ return [ref inverseRegion]; }

SKRegionRef SKRegionByDifferenceFromRegion( SKRegionRef ref, SKRegionRef otherRegion )
{ return [ref regionByDifferenceFromRegion:otherRegion]; }

SKRegionRef SKRegionByIntersectionWithRegion( SKRegionRef ref, SKRegionRef otherRegion )
{ return [ref regionByIntersectionWithRegion:otherRegion]; }

SKRegionRef SKRegionByUnionWithRegion( SKRegionRef ref, SKRegionRef otherRegion )
{ return [ref regionByUnionWithRegion:otherRegion]; }

// Interacting
CGPathRef SKRegionPath( SKRegionRef ref )
{ return [ref path]; }

Boolean SKRegionContainsPoint( SKRegionRef ref, CGPoint pt )
{ return [ref containsPoint:pt]; }
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
#pragma mark - SKRenderer
///////////////////////////////////////////////////
//                 SKRenderer                    //
///////////////////////////////////////////////////

// First steps
SKSceneRef SKRendererScene( SKRendererRef ref )
{ return [ref scene]; }

void SKRendererSetScene( SKRendererRef ref, SKSceneRef scene )
{ [ref setScene:scene]; }

// Scene update cycle
void SKRendererUpdateAtTime( SKRendererRef ref, CFTimeInterval currentTime )
{ [ref updateAtTime:currentTime]; }

// Performance related toggles
Boolean SKRendererIgnoresSiblingOrder( SKRendererRef ref )
{ return [ref ignoresSiblingOrder]; }

void SKRendererSetIgnoresSiblingOrder( SKRendererRef ref, Boolean flag )
{ [ref setIgnoresSiblingOrder:flag]; }

Boolean SKRendererShouldCullNonVisibleNodes( SKRendererRef ref )
{ return [ref shouldCullNonVisibleNodes]; }

void SKRendererSetShouldCullNonVisibleNodes( SKRendererRef ref, Boolean flag )
{ [ref setShouldCullNonVisibleNodes:flag]; }

// Visual statistics for debugging
Boolean SKRendererShowsNodeCount( SKRendererRef ref )
{ return [ref showsNodeCount]; }

void SKRendererSetShowsNodeCount( SKRendererRef ref, Boolean flag )
{ [ref setShowsNodeCount:flag]; }

Boolean SKRendererShowsDrawCount( SKRendererRef ref )
{ return [ref showsDrawCount]; }

void SKRendererSetShowsDrawCount( SKRendererRef ref, Boolean flag )
{ [ref setShowsDrawCount:flag]; }

Boolean SKRendererShowsQuadCount( SKRendererRef ref )
{ return [ref showsQuadCount]; }

void SKRendererSetShowsQuadCount( SKRendererRef ref, Boolean flag )
{ [ref setShowsQuadCount:flag]; }

Boolean SKRendererShowsPhysics( SKRendererRef ref )
{ return [ref showsPhysics]; }

void SKRendererSetShowsPhysics( SKRendererRef ref, Boolean flag )
{ [ref setShowsPhysics:flag]; }

Boolean SKRendererShowsFields( SKRendererRef ref )
{ return [ref showsFields]; }

void SKRendererSetShowsFields( SKRendererRef ref, Boolean flag )
{ [ref setShowsFields:flag]; }
#endif


#pragma mark - SKScene
///////////////////////////////////////////////////
//                  SKScene                      //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
@implementation CocoaSKSceneDelegate

@synthesize callback, userData;

- (id)initWithCallback:(void *)cb userData:(void*)ud {
    if ( self = [super init] ) {
        self.callback = cb;
        self.userData = ud;
    }
    return self;
}

- (void)update:(NSTimeInterval)currentTime forScene:(SKScene *)scene {
    if ( callback ) (*callback)( skSceneUpdate, scene, currentTime, userData );
}

- (void)didEvaluateActionsForScene:(SKScene *)scene {
    if ( callback ) (*callback)( skSceneDidEvaluateActions, scene, 0.0, userData );
}

- (void)didSimulatePhysicsForScene:(SKScene *)scene {
    if ( callback ) (*callback)( skSceneDidSimulatePhysics, scene, 0.0, userData );
}

- (void)didApplyConstraintsForScene:(SKScene *)scene {
    if ( callback ) (*callback)( skSceneDidApplyConstraints, scene, 0.0, userData );
}

- (void)didFinishUpdateForScene:(SKScene *)scene {
    if ( callback ) (*callback)( skSceneDidFinishUpdate, scene, 0.0, userData );
}
@end
#endif

// Create
SKSceneRef SKSceneWithSize( CGSize size )
{ return [SKScene sceneWithSize:size]; }

CGSize SKSceneSize( SKSceneRef ref );
void SKSceneSetSize( SKSceneRef ref, CGSize size );

// Strech content to fit view
SKSceneScaleMode skSceneScaleMode( SKSceneRef ref )
{ return [ref scaleMode]; }

void SKSceneSetScaleMode( SKSceneRef ref, SKSceneScaleMode scaleMode )
{ [ref setScaleMode:scaleMode]; }

// Configure Viewport
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKCameraNodeRef SKSceneCamera( SKSceneRef ref )
{ return [ref camera]; }

void SKSceneSetCamera( SKSceneRef ref, SKCameraNodeRef camera )
{ [ref setCamera:camera]; }
#endif

CGPoint SKSceneAnchorPoint( SKSceneRef ref )
{ return [ref anchorPoint]; }

void SKSceneSetAnchorPoint( SKSceneRef ref, CGPoint pt )
{ [ref setAnchorPoint:pt]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
// Configure delegate
void SKSceneSetDelegateCallback( SKSceneRef ref, SKSceneDelegateCallbackType callback, void *userData )
{
    CocoaSKSceneDelegate *delegate = [[CocoaSKSceneDelegate alloc] initWithCallback:callback userData:userData];
    [ref setDelegate:delegate];
}
#endif

// Background appearance
NSInteger SKSceneView( SKSceneRef ref )
{
    SKView *view = [ref view];
    if ( view ) return [view tag];
    return 0;
}

ColorRef SKSceneBackgroundColor( SKSceneRef ref )
{ return [ref backgroundColor]; }

void SKSceneSetBackgroundColor( SKSceneRef ref, ColorRef col )
{ [ref setBackgroundColor:col]; }

// Physics properties
SKPhysicsWorldRef SKScenePhysicsWorld( SKSceneRef ref )
{ return [ref physicsWorld]; }

// Positional audio
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKNodeRef SKSceneListener( SKSceneRef ref )
{ return [ref listener]; }

void SKSceneSetListener( SKSceneRef ref, SKNodeRef listener )
{ [ref setListener:listener]; }
#endif

// Converting between coordinate systems
CGPoint SKSceneConvertPointFromView( SKSceneRef ref, CGPoint pt )
{ return [ref convertPointFromView:pt]; }

CGPoint SKSceneConvertPointToView( SKSceneRef ref, CGPoint pt )
{ return [ref convertPointToView:pt]; }



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKShader
///////////////////////////////////////////////////
//                 SKShader                      //
///////////////////////////////////////////////////
// Create
SKShaderRef SKShaderInit( void )
{ return [SKShader shader]; }

SKShaderRef SKShaderWithFileNamed( CFStringRef name )
{ return [SKShader shaderWithFileNamed:(NSString *)name]; }

SKShaderRef SKShaderWithSource( CFStringRef source )
{ return [SKShader shaderWithSource:(NSString *)source]; }

SKShaderRef SKShaderWithSourceUniforms( CFStringRef source, CFArrayRef uniforms )
{ return [SKShader shaderWithSource:(NSString *)source uniforms:(NSArray *)uniforms]; }

// Uniform
void SKShaderAddUniform( SKShaderRef ref, SKUniformRef uniform )
{ [ref addUniform:uniform]; }

void SKShaderRemoveUniformNamed( SKShaderRef ref, CFStringRef name )
{ [ref removeUniformNamed:(NSString *)name]; }

CFArrayRef SKShaderUniforms( SKShaderRef ref )
{ return (CFArrayRef)[ref uniforms]; }

void SKShaderSetUniforms( SKShaderRef ref, CFArrayRef uniforms )
{ [ref setUniforms:(NSArray *)uniforms]; }

SKUniformRef SKShaderUniformNamed( SKShaderRef ref, CFStringRef name )
{ return [ref uniformNamed:(NSString *)name]; }

// Attribute
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFArrayRef SKShaderAttributes( SKShaderRef ref )
{ return (CFArrayRef)[ref attributes]; }

void SKShaderSetAttributes( SKShaderRef ref, CFArrayRef attributes )
{ [ref setAttributes:(NSArray *)attributes]; }
#endif

// Source code
CFStringRef SKShaderSource( SKShaderRef ref )
{ return (CFStringRef)[ref source]; }

void SKShaderSetSource( SKShaderRef ref, CFStringRef source )
{ [ref setSource:(NSString *)source]; }
#endif


#pragma mark - SKShapeNode
///////////////////////////////////////////////////
//               SKShapeNode                     //
///////////////////////////////////////////////////
// Create from path
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKShapeNodeRef SKShapeNodeWithPath( CGPathRef path )
{ return [SKShapeNode shapeNodeWithPath:path]; }

SKShapeNodeRef SKShapeNodeWithPathCentered( CGPathRef path, Boolean centered )
{ return [SKShapeNode shapeNodeWithPath:path centered:centered]; }

#endif
CGPathRef SKShapeNodePath( SKShapeNodeRef ref )
{ return [ref path]; }

void SKShapeNodeSetPath( SKShapeNodeRef ref, CGPathRef path )
{ [ref setPath:path]; }

// Create from rectangle
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKShapeNodeRef SKShapeNodeWithRect( CGRect r )
{ return [SKShapeNode shapeNodeWithRect:r]; }

SKShapeNodeRef SKShapeNodeWithRectOfSize( CGSize size )
{ return [SKShapeNode shapeNodeWithRectOfSize:size]; }

SKShapeNodeRef SKShapeNodeWithRectCornerRadius( CGRect r, CGFloat radius )
{ return [SKShapeNode shapeNodeWithRect:r cornerRadius:radius]; }

SKShapeNodeRef SKShapeNodeWithRectOfSizeCornerRadius( CGSize size, CGFloat radius )
{ return [SKShapeNode shapeNodeWithRectOfSize:size cornerRadius:radius]; }

// Create circle shape
SKShapeNodeRef SKShapeNodeWithCircleOfRadius( CGFloat radius )
{ return [SKShapeNode shapeNodeWithCircleOfRadius:radius]; }

// Create ellipse shape
SKShapeNodeRef SKShapeNodeWithEllipseOfSize( CGSize size )
{ return [SKShapeNode shapeNodeWithEllipseOfSize:size]; }

SKShapeNodeRef SKShapeNodeWithEllipseInRect( CGRect r )
{ return [SKShapeNode shapeNodeWithEllipseInRect:r]; }

// Create from array of points
SKShapeNodeRef SKShapeNodeWithPoints( CGPoint *size, long count )
{ return [SKShapeNode shapeNodeWithPoints:size count:count]; }

SKShapeNodeRef SKShapeNodeWithSplinePoints( CGPoint *size, long count )
{ return [SKShapeNode shapeNodeWithSplinePoints:size count:count]; }
#endif

// Fill
ColorRef SKShapeNodeFillColor( SKShapeNodeRef ref )
{ return [ref fillColor]; }

void SKShapeNodeSetFillColor( SKShapeNodeRef ref, ColorRef col )
{ [ref setFillColor:col]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureRef SKShapeNodeFillTexture( SKShapeNodeRef ref )
{ return [ref fillTexture]; }

void SKShapeNodeSetFillTexture( SKShapeNodeRef ref, SKTextureRef texture )
{ [ref setFillTexture:texture]; }
#endif

// Stroke
CGFloat SKShapeNodeLineWidth( SKShapeNodeRef ref )
{ return [ref lineWidth]; }

void SKShapeNodeSetLineWidth( SKShapeNodeRef ref, CGFloat lineWidth )
{ [ref setLineWidth:lineWidth]; }

ColorRef SKShapeNodeStrokeColor( SKShapeNodeRef ref )
{ return [ref strokeColor]; }

void SKShapeNodeSetStrokeColor( SKShapeNodeRef ref, ColorRef col )
{ [ref setStrokeColor:col]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureRef SKShapeNodeStrokeTexture( SKShapeNodeRef ref )
{ return [ref strokeTexture]; }

void SKShapeNodeSetStrokeTexture( SKShapeNodeRef ref, SKTextureRef texture )
{ [ref setStrokeTexture:texture]; }
#endif

CGFloat SKShapeNodeGlowWidth( SKShapeNodeRef ref )
{ return [ref glowWidth]; }

void SKShapeNodeSetGlowWidth( SKShapeNodeRef ref, CGFloat glowWidth )
{ [ref setGlowWidth:glowWidth]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGLineCap SKShapeNodeLineCap( SKShapeNodeRef ref )
{ return [ref lineCap]; }

void SKShapeNodeSetLineCap( SKShapeNodeRef ref, CGLineCap lineCap )
{ [ref setLineCap:lineCap]; }

CGLineJoin SKShapeNodeLineJoin( SKShapeNodeRef ref )
{ return [ref lineJoin]; }

void SKShapeNodeSetLineJoin( SKShapeNodeRef ref, CGLineJoin lineJoin )
{ [ref setLineJoin:lineJoin]; }

CGFloat SKShapeNodeMiterLimit( SKShapeNodeRef ref )
{ return [ref miterLimit]; }

void SKShapeNodeSetMiterLimit( SKShapeNodeRef ref, CGFloat miterLimit )
{ [ref setMiterLimit:miterLimit]; }
#endif

Boolean SKShapeNodeIsAntialiased( SKShapeNodeRef ref )
{ return [ref isAntialiased]; }

void SKShapeNodeSetAntialiased( SKShapeNodeRef ref, Boolean flag )
{ [ref setAntialiased:flag]; }

// Alpha blending
SKBlendMode SKShapeNodeBlendMode( SKShapeNodeRef ref )
{ return [ref blendMode]; }

void SKShapeNodeSetBlendMode( SKShapeNodeRef ref, SKBlendMode mode )
{ [ref setBlendMode:mode]; }

// Stroke length
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGFloat SKShapeNodeLineLength( SKShapeNodeRef ref )
{ return [ref lineLength]; }

// Customizing stroke or fill drawing
SKShaderRef SKShapeNodeStrokeShader( SKShapeNodeRef ref )
{ return [ref strokeShader]; }

void SKShapeNodeSetStrokeShader( SKShapeNodeRef ref, SKShaderRef shader )
{ [ref setStrokeShader:shader]; }

SKShaderRef SKShapeNodeFillShader( SKShapeNodeRef ref )
{ return [ref fillShader]; }

void SKShapeNodeSetFillShader( SKShapeNodeRef ref, SKShaderRef shader )
{ [ref setFillShader:shader]; }
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFDictionaryRef SKShapeNodeAttributeValues( SKShapeNodeRef ref )
{ return (CFDictionaryRef)[ref attributeValues]; }

void SKShapeNodeSetAttributeValues( SKShapeNodeRef ref, CFDictionaryRef values )
{ [ref setAttributeValues:(NSDictionary *)values]; }

void SKShapeNodeSetValueForAttributeNamed( SKShapeNodeRef ref, SKAttributeValueRef value, CFStringRef key )
{ [ref setValue:value forAttributeNamed:(NSString *)key]; }

SKAttributeValueRef SKShapeNodeValueForAttributeNamed( SKShapeNodeRef ref, CFStringRef key )
{ return [ref valueForAttributeNamed:(NSString *)key]; }
#endif




#pragma mark - SKSpriteNode
///////////////////////////////////////////////////
//               SKSpriteNode                    //
///////////////////////////////////////////////////
// Create
SKSpriteNodeRef SKSpriteNodeWithImageNamed( CFStringRef name )
{ return [[[SKSpriteNode alloc] initWithImageNamed:(NSString *)name] autorelease]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKSpriteNodeRef SKSpriteNodeWithNormalMapped( CFStringRef name, Boolean normalMapped )
{ return [SKSpriteNode spriteNodeWithImageNamed:(NSString *)name normalMapped:normalMapped]; }
#endif

// Create from texture
SKSpriteNodeRef SKSpriteNodeWithColor( ColorRef col, CGSize size )
{ return [SKSpriteNode spriteNodeWithColor:col size:size]; }

SKSpriteNodeRef SKSpriteNodeWithTexture( SKTextureRef texture )
{ return [SKSpriteNode spriteNodeWithTexture:texture]; }

SKSpriteNodeRef SKSpriteNodeWithTextureSize( SKTextureRef texture, CGSize size )
{ return [SKSpriteNode spriteNodeWithTexture:texture size:size]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKSpriteNodeRef SKSpriteNodeWithTextureNormalMap( SKTextureRef texture, SKTextureRef normalMap )
{ return [SKSpriteNode spriteNodeWithTexture:texture normalMap:normalMap]; }
#endif

SKTextureRef SKSpriteNodeTexture( SKSpriteNodeRef ref )
{ return [ref texture]; }

void SKSpriteNodeSetTexture( SKSpriteNodeRef ref, SKTextureRef texture )
{ [ref setTexture:texture]; }

// Create from archive
SKSpriteNodeRef SKSpriteNodeWithCoder( CoderRef coder )
{ return [[[SKSpriteNode alloc] initWithCoder:coder] autorelease]; }

// Size and position
CGSize SKSpriteNodeSize( SKSpriteNodeRef ref )
{ return [ref size]; }

void SKSpriteNodeSetSize( SKSpriteNodeRef ref, CGSize size )
{ [ref setSize:size]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
void SKSpriteNodeScaleToSize( SKSpriteNodeRef ref, CGSize size )
{ [ref scaleToSize:size]; }
#endif

CGPoint SKSpriteNodeAnchorPoint( SKSpriteNodeRef ref )
{ return [ref anchorPoint]; }

void SKSpriteNodeSetAnchorPoint( SKSpriteNodeRef ref, CGPoint pt )
{ [ref setAnchorPoint:pt]; }

// Scaling in nine parts
CGRect SKSpriteNodeCenterRect( SKSpriteNodeRef ref )
{ return [ref centerRect]; }

void SKSpriteNodeSetCenterRect( SKSpriteNodeRef ref, CGRect r )
{ [ref setCenterRect:r]; }

// Tint
ColorRef SKSpriteNodeColor( SKSpriteNodeRef ref )
{ return [ref color]; }

void SKSpriteNodeSetColor( SKSpriteNodeRef ref, ColorRef col )
{ [ref setColor:col]; }

CGFloat SKSpriteNodeColorBlendFactor( SKSpriteNodeRef ref )
{return [ref colorBlendFactor]; }

void SKSpriteNodeSetColorBlendFactor( SKSpriteNodeRef ref, CGFloat factor )
{ [ref setColorBlendFactor:factor]; }

// Alpha blending
SKBlendMode SKSpriteNodeBlendMode( SKSpriteNodeRef ref )
{ return [ref blendMode]; }

void SKSpriteNodeSetBlendMode( SKSpriteNodeRef ref, SKBlendMode mode )
{[ref setBlendMode:mode]; }

// Lighting
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
UInt32 SKSpriteNodeLightingBitMask( SKSpriteNodeRef ref )
{ return [ref lightingBitMask]; }

void SKSpriteNodeSetLightingBitMask( SKSpriteNodeRef ref, UInt32 bitMask )
{ [ref setLightingBitMask:bitMask]; }

UInt32 SKSpriteNodeShadowedBitMask( SKSpriteNodeRef ref )
{ return [ref shadowedBitMask]; }

void SKSpriteNodeSetShadowedBitMask( SKSpriteNodeRef ref, UInt32 bitMask )
{ [ref setShadowedBitMask:bitMask]; }

UInt32 SKSpriteNodeShadowCastBitMask( SKSpriteNodeRef ref )
{ return [ref shadowCastBitMask]; }

void SKSpriteNodeSetShadowCastBitMask( SKSpriteNodeRef ref, UInt32 bitMask )
{ [ref setShadowCastBitMask:bitMask]; }

SKTextureRef SKSpriteNodeNormalTexture( SKSpriteNodeRef ref )
{ return [ref normalTexture]; }

void SKSpriteNodeSetNormalTexture( SKSpriteNodeRef ref, SKTextureRef texture )
{ [ref setNormalTexture:texture]; }

// Custom shader
SKShaderRef SKSpriteNodeShader( SKSpriteNodeRef ref )
{ return [ref shader]; }

void SKSpriteNodeSetShader( SKSpriteNodeRef ref, SKShaderRef shader )
{ [ref setShader:shader]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFDictionaryRef SKSpriteNodeAttributeValues( SKSpriteNodeRef ref )
{ return (CFDictionaryRef)[ref attributeValues]; }

void SKSpriteNodeSetAttributeValues( SKSpriteNodeRef ref, CFDictionaryRef values )
{ [ref setAttributeValues:(NSDictionary *)values]; }

void SKSpriteNodeSetValueForAttributeNamed( SKSpriteNodeRef ref, SKAttributeValueRef value, CFStringRef key )
{ [ref setValue:value forAttributeNamed:(NSString *)key]; }

SKAttributeValueRef SKSpriteNodeValueForAttributeNamed( SKSpriteNodeRef ref, CFStringRef key )
{ return [ref valueForAttributeNamed:(NSString *)key]; }
#endif



#pragma mark - SKTexture
///////////////////////////////////////////////////
//                  SKTexture                    //
///////////////////////////////////////////////////
/*
 Initializers
 */

// Filename
SKTextureRef SKTextureWithImageNamed( CFStringRef name )
{ return [SKTexture textureWithImageNamed:(NSString *)name]; }

// Texture within texture
SKTextureRef SKTextureWithRectInTexture( CGRect r, SKTextureRef texture )
{ return [SKTexture textureWithRect:r inTexture:texture]; }

// Image
SKTextureRef SKTextureWithImage( ImageRef image )
{ return [SKTexture textureWithImage:image]; }

SKTextureRef SKTextureWithCGImage( CGImageRef image )
{ return [SKTexture textureWithCGImage:image]; }

// Effects
SKTextureRef SKTextureByApplyingCIFilter( SKTextureRef ref, CIFilterRef filter )
{ return [ref textureByApplyingCIFilter:filter]; }

// Data
SKTextureRef SKTextureWithData( CFDataRef dta, CGSize size )
{ return [SKTexture textureWithData:(NSData *)dta size:size]; }

SKTextureRef SKTextureWithDataRowLength( CFDataRef dta, CGSize size, UInt32 rowLength, UInt32 alignment )
{ return [SKTexture textureWithData:(NSData *)dta size:size rowLength:rowLength alignment:alignment]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
SKTextureRef SKTextureWithDataFlipped( CFDataRef dta, CGSize size, Boolean flipped )
{ return [SKTexture textureWithData:(NSData *)dta size:size flipped:flipped]; }
#endif

// Normal map
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureRef SKTextureByGeneratingNormalMap( SKTextureRef ref )
{ return [ref textureByGeneratingNormalMap]; }

SKTextureRef SKTextureByGeneratingNormalMapWithSmoothness( SKTextureRef ref, CGFloat smoothness, CGFloat contrast )
{ return [ref textureByGeneratingNormalMapWithSmoothness:smoothness contrast:contrast]; }
#endif

// Noise textures
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureRef SKTextureVectorNoise( CGFloat smoothness, CGSize size )
{ return [SKTexture textureVectorNoiseWithSmoothness:smoothness size:size]; }

SKTextureRef SKTextureNoise( CGFloat smoothness, CGSize size, Boolean grayscale )
{ return [SKTexture textureNoiseWithSmoothness:smoothness size:size grayscale:grayscale]; }
#endif

// ===============

// Size
CGSize SKTextureSize( SKTextureRef ref )
{ return [ref size]; }

CGRect SKTextureRect( SKTextureRef ref )
{ return [ref textureRect]; }

// Scaling behavior
SKTextureFilteringMode skTextureFilteringMode( SKTextureRef ref )
{ return [ref filteringMode]; }

void SKTextureSetFilteringMode( SKTextureRef ref, SKTextureFilteringMode mode )
{ [ref setFilteringMode:mode]; }

Boolean SKTextureUsesMipmaps( SKTextureRef ref )
{ return [ref usesMipmaps]; }

void SKTextureSetUsesMipmaps( SKTextureRef ref, Boolean flag )
{ [ref setUsesMipmaps:flag]; }

// Image
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGImageRef SKTextureCGImage( SKTextureRef ref )
{ return (CGImageRef)[(id)[ref CGImage] autorelease]; }
#endif

// Preload
void SKTexturePreload( SKTextureRef ref, SKTexturePreloadCallbackType callback, void *userData )
{
    [ref preloadWithCompletionHandler:^(void) {
        (*callback)(ref,userData);
    }];
}

void SKTexturePreloadTextures( CFArrayRef textures, SKTexturePreloadTexturesCallbackType callback, void *userData )
{
    [SKTexture preloadTextures:(NSArray *)textures withCompletionHandler:^(void) {
        (*callback)(userData);
    }];
}




#pragma mark - SKTextureAtlas
///////////////////////////////////////////////////
//                SKTextureAtlas                 //
///////////////////////////////////////////////////
// Create
SKTextureAtlasRef SKTextureAtlasNamed( CFStringRef name )
{ return [SKTextureAtlas atlasNamed:(NSString *)name]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureAtlasRef SKTextureAtlasWithDictionary( CFDictionaryRef dict )
{ return [SKTextureAtlas atlasWithDictionary:(NSDictionary *)dict]; }
#endif

// Textures
SKTextureRef SKTextureAtlasTextureNamed( SKTextureAtlasRef ref, CFStringRef name )
{ return [ref textureNamed:(NSString *)name]; }

// Preload
void SKTextureAtlasPreload( SKTextureAtlasRef ref, SKTextureAtlasPreloadCallbackType callback, void *userData )
{
    [ref preloadWithCompletionHandler:^(void) {
        (*callback)(ref,userData);
    }];
}

void SKTextureAtlasPreloadAtlases( CFArrayRef textureAtlases, SKTextureAtlasPreloadAtlasesCallbackType callback, void *userData )
{
    [SKTextureAtlas preloadTextureAtlases:(NSArray *)textureAtlases withCompletionHandler:^(void) {
        (*callback)(userData);
    }];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void SKTextureAtlasPreloadAtlasesNamed( CFArrayRef textureAtlasNames, SKTextureAtlasPreloadAtlasesNamedCallbackType callback, void *userData )
{
    [SKTextureAtlas preloadTextureAtlasesNamed:(NSArray *)textureAtlasNames withCompletionHandler:^(NSError *error, NSArray *foundAtlases) {
        (*callback)(error,foundAtlases,userData);
    }];
}
#endif

// Filenames
CFArrayRef SKTextureAtlasTextureNames( SKTextureAtlasRef ref )
{ return (CFArrayRef)[ref textureNames]; }



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - SKTileDefinition
///////////////////////////////////////////////////
//              SKTileDefinition                 //
///////////////////////////////////////////////////

// Create
SKTileDefinitionRef SKTileDefinitionWithTexture( SKTextureRef texture )
{ return [SKTileDefinition tileDefinitionWithTexture:texture]; }

SKTileDefinitionRef SKTileDefinitionWithNormalTexture( SKTextureRef texture, SKTextureRef normalTexture, CGSize size )
{ return [SKTileDefinition tileDefinitionWithTexture:texture normalTexture:normalTexture size:size]; }

SKTileDefinitionRef SKTileDefinitionWithSize( SKTextureRef texture, CGSize size )
{ return [SKTileDefinition tileDefinitionWithTexture:texture size:size]; }

SKTileDefinitionRef SKTileDefinitionWithNormalTextures( CFArrayRef textures, CFArrayRef normalTextures, CGSize size, CGFloat timePerFrame )
{ return [SKTileDefinition tileDefinitionWithTextures:(NSArray *)textures normalTextures:(NSArray *)normalTextures size:size timePerFrame:timePerFrame]; }

SKTileDefinitionRef SKTileDefinitionWithTextures( CFArrayRef textures, CGSize size, CGFloat timePerFrame )
{ return [SKTileDefinition tileDefinitionWithTextures:(NSArray *)textures size:size timePerFrame:timePerFrame]; }

// Flip
Boolean SKTileDefinitionFlipHorizontally( SKTileDefinitionRef ref )
{ return [ref flipHorizontally]; }

void SKTileDefinitionSetFlipHorizontally( SKTileDefinitionRef ref, Boolean flag )
{ [ref setFlipHorizontally:flag]; }

Boolean SKTileDefinitionFlipVertically( SKTileDefinitionRef ref )
{ return [ref flipVertically]; }

void SKTileDefinitionSetFlipVertically( SKTileDefinitionRef ref, Boolean flag )
{ [ref setFlipVertically:flag]; }

// Rotate
SKTileDefinitionRotation skTileDefinitionRotation( SKTileDefinitionRef ref )
{ return [ref rotation]; }

void SKTileDefinitionSetRotation( SKTileDefinitionRef ref, SKTileDefinitionRotation rotation )
{ [ref setRotation:rotation]; }

// Animated tile properties
CFArrayRef SKTileDefinitionTextures( SKTileDefinitionRef ref )
{ return (CFArrayRef)[ref textures]; }

void SKTileDefinitionSetTextures( SKTileDefinitionRef ref, CFArrayRef textures )
{ [ref setTextures:(NSArray *)textures]; }

CFArrayRef SKTileDefinitionNormalTextures( SKTileDefinitionRef ref )
{ return (CFArrayRef)[ref normalTextures]; }

void SKTileDefinitionSetNormalTextures( SKTileDefinitionRef ref, CFArrayRef textures )
{ [ref setNormalTextures:(NSArray *)textures]; }

CGFloat SKTileDefinitionTimePerFrame( SKTileDefinitionRef ref )
{ return [ref timePerFrame]; }

void SKTileDefinitionSetTimePerFrame( SKTileDefinitionRef ref, CGFloat timePerFrame )
{ [ref setTimePerFrame:timePerFrame]; }

// Custom data
CFMutableDictionaryRef SKTileDefinitionUserData( SKTileDefinitionRef ref )
{ return (CFMutableDictionaryRef)[ref userData]; }

void SKTileDefinitionSetUserData( SKTileDefinitionRef ref, CFMutableDictionaryRef userData )
{ [ref setUserData:(NSMutableDictionary *)userData]; }

// Properties
CFStringRef SKTileDefinitionName( SKTileDefinitionRef ref )
{ return (CFStringRef)[ref name]; }

void SKTileDefinitionSetName( SKTileDefinitionRef ref, CFStringRef name )
{ [ref setName:(NSString *)name]; }

NSUInteger SKTileDefinitionPlacementWeight( SKTileDefinitionRef ref )
{ return [ref placementWeight]; }

void SKTileDefinitionSetPlacementWeight( SKTileDefinitionRef ref, NSUInteger weight )
{ [ref setPlacementWeight:weight]; }

CGSize SKTileDefinitionSize( SKTileDefinitionRef ref )
{ return [ref size]; }

void SKTileDefinitionSetSize( SKTileDefinitionRef ref, CGSize size )
{ [ref setSize:size]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - SKTileGroup
///////////////////////////////////////////////////
//                   SKTileGroup                 //
///////////////////////////////////////////////////
// Create
SKTileGroupRef SKTileGroupEmpty( void )
{ return [SKTileGroup emptyTileGroup]; }

SKTileGroupRef SKTileGroupWithTileDefinition( SKTileDefinitionRef definition )
{ return [SKTileGroup tileGroupWithTileDefinition:definition]; }

SKTileGroupRef SKTileGroupWithRules( CFArrayRef rules )
{ return [SKTileGroup tileGroupWithRules:(NSArray *)rules]; }

// Properties
CFStringRef SKTileGroupName( SKTileGroupRef ref )
{ return (CFStringRef)[ref name]; }

void SKTileGroupSetName( SKTileGroupRef ref, CFStringRef name )
{ [ref setName:(NSString *)name]; }

CFArrayRef SKTileGroupRules( SKTileGroupRef ref )
{ return (CFArrayRef)[ref rules]; }

void SKTileGroupSetRules( SKTileGroupRef ref, CFArrayRef rules )
{ [ref setRules:(NSArray *)rules]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - SKTileGroupRule
///////////////////////////////////////////////////
//                 SKTileGroupRule               //
///////////////////////////////////////////////////
// Create
SKTileGroupRuleRef SKTileGroupRuleWithAdjacency( SKTileAdjacencyMask mask, CFArrayRef tileDefinitions )
{ return [SKTileGroupRule tileGroupRuleWithAdjacency:mask tileDefinitions:(NSArray *)tileDefinitions]; }

// Properties
SKTileAdjacencyMask SKTileGroupRuleAdjacency( SKTileGroupRuleRef ref )
{ return [ref adjacency]; }

void SKTileGroupRuleSetAdjacency( SKTileGroupRuleRef ref, SKTileAdjacencyMask mask )
{ [ref setAdjacency:mask]; }

CFStringRef SKTileGroupRuleName( SKTileGroupRuleRef ref )
{ return (CFStringRef)[ref name]; }

void SKTileGroupRuleSetName( SKTileGroupRuleRef ref, CFStringRef name )
{ [ref setName:(NSString *)name]; }

CFArrayRef SKTileGroupRuleTileDefinitions( SKTileGroupRuleRef ref )
{ return (CFArrayRef)[ref tileDefinitions]; }

void SKTileGroupRuleSetTileDefinitions( SKTileGroupRuleRef ref, CFArrayRef definitions )
{ [ref setTileDefinitions:(NSArray *)definitions]; }
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - SKTileMapNode
///////////////////////////////////////////////////
//                 SKTileMapNode                 //
///////////////////////////////////////////////////
// Create
SKTileMapNodeRef SKTileMapNodeWithTileSet( SKTileSetRef tileSet, NSUInteger columns, NSUInteger rows, CGSize tileSize )
{ return [SKTileMapNode tileMapNodeWithTileSet:tileSet columns:columns rows:rows tileSize:tileSize]; }

SKTileMapNodeRef SKTileMapNodeWithTileSetFillWithTileGroup( SKTileSetRef tileSet, NSUInteger columns, NSUInteger rows, CGSize tileSize, SKTileGroupRef group )
{ return [SKTileMapNode tileMapNodeWithTileSet:tileSet columns:columns rows:rows tileSize:tileSize fillWithTileGroup:group]; }

SKTileMapNodeRef SKTileMapNodeWithTileSetFillWithTileGroupLayout( SKTileSetRef tileSet, NSUInteger columns, NSUInteger rows, CGSize tileSize, CFArrayRef tileGroupLayout )
{ return [SKTileMapNode tileMapNodeWithTileSet:tileSet columns:columns rows:rows tileSize:tileSize tileGroupLayout:(NSArray *)tileGroupLayout]; }

// Contents
Boolean SKTileMapNodeEnableAutomapping( SKTileMapNodeRef ref )
{ return [ref enableAutomapping]; }

void SKTileMapNodeSetEnableAutomapping( SKTileMapNodeRef ref, Boolean flag )
{ [ref setEnableAutomapping:flag]; }

void SKTileMapNodeFillWithTileGroup( SKTileMapNodeRef ref, SKTileGroupRef tileGroup )
{ return [ref fillWithTileGroup:tileGroup]; }

void SKTileMapNodeSetTileGroupAndDefinition( SKTileMapNodeRef ref, SKTileGroupRef tileGroup, SKTileDefinitionRef tileDefinition, NSUInteger column, NSUInteger row )
{ [ref setTileGroup:tileGroup andTileDefinition:tileDefinition forColumn:column row:row]; }

void SKTileMapNodeSetTileGroup( SKTileMapNodeRef ref, SKTileGroupRef tileGroup, NSUInteger column, NSUInteger row )
{ [ref setTileGroup:tileGroup forColumn:column row:row]; }

// Position
CGPoint SKTileMapNodeAnchorPoint( SKTileMapNodeRef ref )
{ return [ref anchorPoint]; }

void SKTileMapNodeSetAnchorPoint( SKTileMapNodeRef ref, CGPoint pt )
{ [ref setAnchorPoint:pt]; }

// Size
CGSize SKTileMapNodeTileSize( SKTileMapNodeRef ref )
{ return [ref tileSize]; }

void SKTileMapNodeSetTileSize( SKTileMapNodeRef ref, CGSize size )
{ [ref setTileSize:size]; }

SKTileSetRef SKTileMapNodeTileSet( SKTileMapNodeRef ref )
{ return [ref tileSet]; }

void SKTileMapNodeSetTileSet( SKTileMapNodeRef ref, SKTileSetRef tileSet )
{ [ref setTileSet:tileSet]; }

NSUInteger SKTileMapNodeNumberOfColumns( SKTileMapNodeRef ref )
{ return [ref numberOfColumns]; }

void SKTileMapNodeSetNumberOfColumns( SKTileMapNodeRef ref, NSUInteger numberOfColumns )
{ [ref setNumberOfColumns:numberOfColumns]; }

NSUInteger SKTileMapNodeNumberOfRows( SKTileMapNodeRef ref )
{ return [ref numberOfRows]; }

void SKTileMapNodeSetNumberOfRows( SKTileMapNodeRef ref, NSUInteger numberOfRows )
{ [ref setNumberOfRows:numberOfRows]; }

// Properties
CGPoint SKTileMapNodeCenterOfTile( SKTileMapNodeRef ref, NSUInteger column, NSUInteger row )
{ return [ref centerOfTileAtColumn:column row:row]; }

NSUInteger SKTileMapNodeTileColumnIndexFromPosition( SKTileMapNodeRef ref, CGPoint position )
{ return [ref tileColumnIndexFromPosition:position]; }

SKTileDefinitionRef SKTileMapNodeTileDefinition( SKTileMapNodeRef ref, NSUInteger column, NSUInteger row )
{ return [ref tileDefinitionAtColumn:column row:row]; }

SKTileGroupRef SKTileMapNodeTileGroup( SKTileMapNodeRef ref, NSUInteger column, NSUInteger row )
{ return [ref tileGroupAtColumn:column row:row]; }

NSUInteger SKTileMapNodeTileRowIndexFromPosition( SKTileMapNodeRef ref, CGPoint position )
{ return [ref tileRowIndexFromPosition:position]; }

CGSize SKTileMapNodeMapSize( SKTileMapNodeRef ref )
{ return [ref mapSize]; }

// Tinting
ColorRef SKTileMapNodeColor( SKTileMapNodeRef ref )
{ return [ref color]; }

void SKTileMapNodeSetColor( SKTileMapNodeRef ref, ColorRef col )
{ [ref setColor:col]; }

CGFloat SKTileMapNodeColorBlendFactor( SKTileMapNodeRef ref )
{ return [ref colorBlendFactor]; }

void SKTileMapNodeSetColorBlendFactor( SKTileMapNodeRef ref, CGFloat factor )
{ [ref setColorBlendFactor:factor]; }

// Lighting
UInt32 SKTileMapNodeLightingBitMask( SKTileMapNodeRef ref )
{ return [ref lightingBitMask]; }

void SKTileMapNodeSetLightingBitMask( SKTileMapNodeRef ref, UInt32 bitMask )
{ [ref setLightingBitMask:bitMask]; }

// Alpha
SKBlendMode SKTileMapNodeBlendMode( SKTileMapNodeRef ref )
{ return [ref blendMode]; }

void SKTileMapNodeSetBlendMode( SKTileMapNodeRef ref, SKBlendMode mode )
{ [ref setBlendMode:mode]; }

// Shader
SKShaderRef SKTileMapNodeShader( SKTileMapNodeRef ref )
{ return [ref shader]; }

void SKTileMapNodeSetShader( SKTileMapNodeRef ref, SKShaderRef shader )
{ [ref setShader:shader]; }

CFDictionaryRef SKTileMapNodeAttributeValues( SKTileMapNodeRef ref )
{ return (CFDictionaryRef)[ref attributeValues]; }

void SKTileMapNodeSetAttributeValues( SKTileMapNodeRef ref, CFDictionaryRef values )
{ [ref setAttributeValues:(NSDictionary *)values]; }

void SKTileMapNodeSetValueForAttributeNamed( SKTileMapNodeRef ref, SKAttributeValueRef value, CFStringRef key )
{ [ref setValue:value forAttributeNamed:(NSString *)key]; }

SKAttributeValueRef SKTileMapNodeValueForAttributeNamed( SKTileMapNodeRef ref, CFStringRef key )
{ return [ref valueForAttributeNamed:(NSString *)key]; }
#endif


#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - SKTileSet
///////////////////////////////////////////////////
//                   SKTileSet                   //
///////////////////////////////////////////////////
// Create
SKTileSetRef SKTileSetNamed( CFStringRef name )
{ return [SKTileSet tileSetNamed:(NSString *)name]; }

SKTileSetRef SKTileSetFromURL( CFURLRef url )
{ return [SKTileSet tileSetFromURL:(NSURL *)url]; }

SKTileSetRef SKTileSetWithTileGroups( CFArrayRef tileGroups )
{ return [SKTileSet tileSetWithTileGroups:(NSArray *)tileGroups]; }

SKTileSetRef SKTileSetWithTileGroupsType( CFArrayRef tileGroups, SKTileSetType type )
{ return [SKTileSet tileSetWithTileGroups:(NSArray *)tileGroups tileSetType:type]; }

// Properties
SKTileGroupRef SKTileSetDefaultTileGroup( SKTileSetRef ref )
{ return [ref defaultTileGroup]; }

void SKTileSetSetDefaultTileGroup( SKTileSetRef ref, SKTileGroupRef group )
{ [ref setDefaultTileGroup:group]; }

CGSize SKTileSetDefaultTileSize( SKTileSetRef ref )
{ return [ref defaultTileSize]; }

void SKTileSetSetDefaultTileSize( SKTileSetRef ref, CGSize size )
{ [ref setDefaultTileSize:size]; }

CFStringRef SKTileSetName( SKTileSetRef ref )
{ return (CFStringRef)[ref name]; }

void SKTileSetSetName( SKTileSetRef ref, CFStringRef name )
{ [ref setName:(NSString *)name]; }

CFArrayRef SKTileSetTileGroups( SKTileSetRef ref )
{ return (CFArrayRef)[ref tileGroups]; }

void SKTileSetSetTileGroups( SKTileSetRef ref, CFArrayRef groups )
{ [ref setTileGroups:(NSArray *)groups]; }

SKTileSetType skTileSetType( SKTileSetRef ref )
{ return [ref type]; }

void SKTileSetSetType( SKTileSetRef ref, SKTileSetType type )
{ [ref setType:type]; }
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
#pragma mark - SKTransformNode
///////////////////////////////////////////////////
//                SKTransformNode                //
///////////////////////////////////////////////////

// Rotate
CGFloat SKTransformNodeXRotation( SKTransformNodeRef ref )
{ return [ref xRotation]; }

void SKTransformNodeSetXRotation( SKTransformNodeRef ref, CGFloat rotation )
{ [ref setXRotation:rotation]; }

CGFloat SKTransformNodeYRotation( SKTransformNodeRef ref )
{ return [ref yRotation]; }

void SKTransformNodeSetYRotation( SKTransformNodeRef ref, CGFloat rotation )
{ [ref setYRotation:rotation]; }

void SKTransformNodeSetEulerAngles( SKTransformNodeRef ref, vector_float3 euler )
{ [ref setEulerAngles:euler]; }

void SKTransformNodeSetQuaternion( SKTransformNodeRef ref, simd_quatf quaternion )
{ [ref setQuaternion:quaternion]; }

void SKTransformNodeSetRotationMatrix( SKTransformNodeRef ref, matrix_float3x3 matrix )
{ [ref setRotationMatrix:matrix]; }

// Current rotation
vector_float3 SKTransformNodeEulerAngles( SKTransformNodeRef ref )
{ return [ref eulerAngles]; }

simd_quatf SKTransformNodeQuaternion( SKTransformNodeRef ref )
{ return [ref quaternion]; }

matrix_float3x3 SKTransformNodeRotationMatrix( SKTransformNodeRef ref )
{ return [ref rotationMatrix]; }
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - SKTransition
///////////////////////////////////////////////////
//                   SKTransition                //
///////////////////////////////////////////////////
// Create
SKTransitionRef SKTransitionCrossFadeWithDuration( CFTimeInterval duration )
{ return [SKTransition crossFadeWithDuration:duration]; }

SKTransitionRef SKTransitionDoorsCloseHorizontalWithDuration( CFTimeInterval duration )
{ return [SKTransition doorsCloseHorizontalWithDuration:duration]; }

SKTransitionRef SKTransitionDoorsCloseVerticalWithDuration( CFTimeInterval duration )
{ return [SKTransition doorsCloseVerticalWithDuration:duration]; }

SKTransitionRef SKTransitionDoorsOpenHorizontalWithDuration( CFTimeInterval duration )
{ return [SKTransition doorsOpenHorizontalWithDuration:duration]; }

SKTransitionRef SKTransitionDoorsOpenVerticalWithDuration( CFTimeInterval duration )
{ return [SKTransition doorsOpenVerticalWithDuration:duration]; }

SKTransitionRef SKTransitionDoorwayWithDuration( CFTimeInterval duration )
{ return [SKTransition doorwayWithDuration:duration]; }

SKTransitionRef SKTransitionFadeWithColor( ColorRef col, CFTimeInterval duration )
{ return [SKTransition fadeWithColor:col duration:duration]; }

SKTransitionRef SKTransitionFadeWithDuration( CFTimeInterval duration )
{ return [SKTransition fadeWithDuration:duration]; }

SKTransitionRef SKTransitionFlipHorizontalWithDuration( CFTimeInterval duration )
{ return [SKTransition flipHorizontalWithDuration:duration]; }

SKTransitionRef SKTransitionFlipVerticalWithDuration( CFTimeInterval duration )
{ return [SKTransition flipVerticalWithDuration:duration]; }

SKTransitionRef SKTransitionMoveInWithDirection( SKTransitionDirection direction, CFTimeInterval duration )
{ return [SKTransition moveInWithDirection:direction duration:duration]; }

SKTransitionRef SKTransitionPushWithDirection( SKTransitionDirection direction, CFTimeInterval duration )
{ return [SKTransition pushWithDirection:direction duration:duration]; }

SKTransitionRef SKTransitionRevealWithDirection( SKTransitionDirection direction, CFTimeInterval duration )
{ return [SKTransition revealWithDirection:direction duration:duration]; }

SKTransitionRef SKTransitionTransitionWithCIFilter( CIFilterRef filter, CFTimeInterval duration )
{ return [SKTransition transitionWithCIFilter:filter duration:duration]; }

// Pausing
Boolean SKTransitionPausesIncomingScene( SKTransitionRef ref )
{ return [ref pausesIncomingScene]; }

void SKTransitionSetPausesIncomingScene( SKTransitionRef ref, Boolean flag )
{ [ref setPausesIncomingScene:flag]; }

Boolean SKTransitionPausesOutgoingScene( SKTransitionRef ref )
{ return [ref pausesOutgoingScene]; }

void SKTransitionSetPausesOutgoingScene( SKTransitionRef ref, Boolean flag )
{ [ref setPausesOutgoingScene:flag]; }
#endif



#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#pragma mark - SKUniform
///////////////////////////////////////////////////
//                   SKUniform                   //
///////////////////////////////////////////////////

// Create
SKUniformRef SKUniformWithName( CFStringRef name )
{ return [SKUniform uniformWithName:(NSString *)name]; }

SKUniformRef SKUniformWithNameFloat( CFStringRef name, float n )
{ return [SKUniform uniformWithName:(NSString *)name float:n]; }

SKUniformRef SKUniformWithNameTexture( CFStringRef name, SKTextureRef texture )
{ return [SKUniform uniformWithName:(NSString *)name texture:texture]; }

// Info
CFStringRef SKUniformName( SKUniformRef ref )
{ return (CFStringRef)[ref name]; }

SKUniformType skUniformType( SKUniformRef ref )
{ return [ref uniformType]; }

// Value
float SKUniformFloatValue( SKUniformRef ref )
{ return [ref floatValue]; }

void SKUniformSetFloatValue( SKUniformRef ref, float value )
{ [ref setFloatValue:value]; }

SKTextureRef SKUniformTextureValue( SKUniformRef ref )
{ return [ref textureValue]; }

void SKUniformSetTextureValue( SKUniformRef ref, SKTextureRef texture )
{ return [ref setTextureValue:texture]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
// Properties
matrix_float2x2 SKUniformMatrixFloat2x2Value( SKUniformRef ref )
{ return [ref matrixFloat2x2Value]; }

matrix_float3x3 SKUniformMatrixFloat3x3Value( SKUniformRef ref )
{ return [ref matrixFloat3x3Value]; }

matrix_float4x4 SKUniformMatrixFloat4x4Value( SKUniformRef ref )
{ return [ref matrixFloat4x4Value]; }

vector_float2 SKUniformVectorFloat2Value( SKUniformRef ref )
{ return [ref vectorFloat2Value]; }

vector_float3 SKUniformVectorFloat3Value( SKUniformRef ref )
{ return [ref vectorFloat3Value]; }

vector_float4 SKUniformVectorFloat4Value( SKUniformRef ref )
{ return [ref vectorFloat4Value]; }

// Type methods
SKUniformRef SKUniformWithNameMatrixFloat2x2( CFStringRef name, matrix_float2x2 value )
{ return [SKUniform uniformWithName:(NSString *)name matrixFloat2x2:value]; }

SKUniformRef SKUniformWithNameMatrixFloat3x3( CFStringRef name, matrix_float3x3 value )
{ return [SKUniform uniformWithName:(NSString *)name matrixFloat3x3:value]; }

SKUniformRef SKUniformWithNameMatrixFloat4x4( CFStringRef name, matrix_float4x4 value )
{ return [SKUniform uniformWithName:(NSString *)name matrixFloat4x4:value]; }

SKUniformRef SKUniformWithNameVectorFloat2( CFStringRef name, vector_float2 value )
{ return [SKUniform uniformWithName:(NSString *)name vectorFloat2:value]; }

SKUniformRef SKUniformWithNameVectorFloat3( CFStringRef name, vector_float3 value )
{ return [SKUniform uniformWithName:(NSString *)name vectorFloat3:value]; }

SKUniformRef SKUniformWithNameVectorFloat4( CFStringRef name, vector_float4 value )
{ return [SKUniform uniformWithName:(NSString *)name vectorFloat4:value]; }
#endif// 101200

#endif// 101000



#pragma mark - SKVideoNode
///////////////////////////////////////////////////
//                 SKVideoNode                   //
///////////////////////////////////////////////////

// Create
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
SKVideoNodeRef SKVideoNodeWithFileNamed( CFStringRef name )
{ return [SKVideoNode videoNodeWithFileNamed:(NSString *)name]; }

SKVideoNodeRef SKVideoNodeWithURL( CFURLRef url )
{ return [SKVideoNode videoNodeWithURL:(NSURL *)url]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKVideoNodeRef SKVideoNodeWithCoder( CoderRef coder )
{ return [[[SKVideoNode alloc] initWithCoder:coder] autorelease]; }
#endif

// Visual properties
CGPoint SKVideoNodeAnchorPoint( SKVideoNodeRef ref )
{ return [ref anchorPoint]; }

void SKVideoNodeSetAnchorPoint( SKVideoNodeRef ref, CGPoint pt )
{ [ref setAnchorPoint:pt]; }

CGSize SKVideoNodeSize( SKVideoNodeRef ref )
{ return [ref size]; }

void SKVideoNodeSetSize( SKVideoNodeRef ref, CGSize size )
{ [ref setSize:size]; }

// Playback
void SKVideoNodePlay( SKVideoNodeRef ref )
{ [ref play]; }

void SKVideoNodePause( SKVideoNodeRef ref )
{ [ref pause]; }







#pragma mark - SKView
///////////////////////////////////////////////////
//                     SKView                    //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200

@implementation SKViewSC
- (void)drawRect:(NSRect)dirtyRect {
    CocoaSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CocoaSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CocoaSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CocoaSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CocoaSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CocoaSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CocoaSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CocoaSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CocoaSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CocoaSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CocoaSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CocoaSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CocoaSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CocoaSubclassFlagsChanged( self, event );
}
@end

@implementation CocoaSKViewDelegate

@synthesize timeVal;
@synthesize boolVal;

- (BOOL)view:(SKView *)view shouldRenderAtTime:(NSTimeInterval)time {
    NSInteger wndTag = [view wndTag];
    CocoaOutputWindowSetTag( wndTag );
    self.timeVal = time;
    self.boolVal = YES;
    CallUserDefinedOnDialogFunction( skViewShouldRenderAtTime, [view tag], wndTag, nil );
    return self.boolVal;
}
@end

CocoaSKViewDelegate *CocoaSKViewDelegateObj( void )
{
    static CocoaSKViewDelegate *sCocoaSKViewDelegate = nil;
    if ( !sCocoaSKViewDelegate ) sCocoaSKViewDelegate = [[CocoaSKViewDelegate alloc] init];
    return sCocoaSKViewDelegate;
}

BOOL SKViewEventBool( void )
{
    CocoaSKViewDelegate *delegate = CocoaSKViewDelegateObj();
    if ( delegate ) return [delegate boolVal];
    return NO;
}

void SKViewEventSetBool( BOOL flag )
{
    CocoaSKViewDelegate *delegate = CocoaSKViewDelegateObj();
    if ( delegate ) [delegate setBoolVal:flag];
}

CFTimeInterval SKViewEventTime( void )
{
    CocoaSKViewDelegate *delegate = CocoaSKViewDelegateObj();
    if ( delegate ) return [delegate timeVal];
    return 0.0;
}

void SKViewEventSetTime( CFTimeInterval ti )
{
    CocoaSKViewDelegate *delegate = CocoaSKViewDelegateObj();
    if ( delegate ) [delegate setTimeVal:ti];
}
#endif


@implementation SKView (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

SKView *SKViewWithTag( NSInteger tag )
{
    SKView *view = (SKView *)CocoaViewWithTag( tag );
    if ( [view isKindOfClass:[SKView class]] ) return view;
    CocoaShowAlert( NSWarningAlertStyle, @"Error.", [NSString stringWithFormat:@"Can't find skview with tag: %ld.",(long)tag] );
    return nil;
}

Boolean SKViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [SKView class] ); }

// Display scene
SKSceneRef SKViewScene( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view scene];
    return nil;
}

void SKViewPresentScene( NSInteger tag, SKSceneRef scene )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view presentScene:scene];
}

void SKViewPresentSceneTransition( NSInteger tag, SKSceneRef scene, SKTransitionRef transition )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view presentScene:scene transition:transition];
}

// Timing of rendering
Boolean SKViewIsPaused( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view isPaused];
    return NO;
}

void SKViewSetPaused( NSInteger tag, Boolean flag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view setPaused:flag];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
NSInteger SKViewPreferredFramesPerSecond( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view preferredFramesPerSecond];
    return 0;
}

void SKViewSetPreferredFramesPerSecond( NSInteger tag, NSInteger framesPerSecond )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view setPreferredFramesPerSecond:framesPerSecond];
}
#endif

// Performance related toggles
Boolean SKViewIgnoresSiblingOrder( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view ignoresSiblingOrder];
    return NO;
}

void SKViewSetIgnoresSiblingOrder( NSInteger tag, Boolean flag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view setIgnoresSiblingOrder:flag];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean SKViewShouldCullNonVisibleNodes( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view shouldCullNonVisibleNodes];
    return NO;
}

void SKViewSetShouldCullNonVisibleNodes( NSInteger tag, Boolean flag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view setShouldCullNonVisibleNodes:flag];
}

Boolean SKViewAllowsTransparency( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view allowsTransparency];
    return NO;
}

void SKViewSetAllowsTransparency( NSInteger tag, Boolean flag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view setAllowsTransparency:flag];
}
#endif

Boolean SKViewIsAsynchronous( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view isAsynchronous];
    return NO;
}

void SKViewSetAsynchronous( NSInteger tag, Boolean flag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view setAsynchronous:flag];
}

// Visual stats for debugging
Boolean SKViewShowsFPS( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view showsFPS];
    return NO;
}

void SKViewSetShowsFPS( NSInteger tag, Boolean flag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view setShowsFPS:flag];
}

Boolean SKViewShowsNodeCount( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view showsNodeCount];
    return NO;
}

void SKViewSetShowsNodeCount( NSInteger tag, Boolean flag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view setShowsNodeCount:flag];
}

Boolean SKViewShowsDrawCount( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view showsDrawCount];
    return NO;
}

void SKViewSetShowsDrawCount( NSInteger tag, Boolean flag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view setShowsDrawCount:flag];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
Boolean SKViewShowsQuadCount( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view showsQuadCount];
    return NO;
}

void SKViewSetShowsQuadCount( NSInteger tag, Boolean flag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view setShowsQuadCount:flag];
}

Boolean SKViewShowsPhysics( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view showsPhysics];
    return NO;
}

void SKViewSetShowsPhysics( NSInteger tag, Boolean flag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view setShowsPhysics:flag];
}

Boolean SKViewShowsFields( NSInteger tag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view showsFields];
    return NO;
}

void SKViewSetShowsFields( NSInteger tag, Boolean flag )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) [view setShowsFields:flag];
}
#endif

// Convert between view and scene coordinates
CGPoint SKViewConvertPointFromScene( NSInteger tag, CGPoint pt, SKSceneRef scene )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view convertPoint:pt fromScene:scene];
    return CGPointZero;
}

CGPoint SKViewConvertPointToScene( NSInteger tag, CGPoint pt, SKSceneRef scene )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view convertPoint:pt toScene:scene];
    return CGPointZero;
}

// Snapshot nodes to texture
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
SKTextureRef SKViewTextureFromNodeCrop( NSInteger tag, SKNodeRef node, CGRect crop )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view textureFromNode:node crop:crop];
    return nil;
}
#endif

SKTextureRef SKViewTextureFromNode( NSInteger tag, SKNodeRef node )
{
    SKView *view = SKViewWithTag( tag );
    if ( view ) return [view textureFromNode:node];
    return nil;
}


// Init
SKViewRef SKViewInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 128, 128 );
    
    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // init
    SKView *view;
    if ( subclass ) {
        view = [[[SKViewSC alloc] initWithFrame:rect] autorelease];
    } else {
        view = [[[SKView alloc] initWithFrame:rect] autorelease];
    }
    
    // tag
    [view setTag:inTag];
    
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
    // delegate
    [view setDelegate:CocoaSKViewDelegateObj()];
#endif
    
    return view;
}


// Statement
void CocoaSKView( NSInteger inTag, CGRect inRect, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        SKView *view = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !view ) {
            view = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( view ) {    // already exists
            
            // make sure it's same view class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [view class], [SKView class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, view, [SKView class] ) ) {
                return;
            }

            // rect
            if ( !NSIsEmptyRect( rect ) ) [view setFrame:rect];
            
        } else {            // create
            // rect
            //            if ( NSIsEmptyRect( rect ) ) {
            //                rect = NSMakeRect( 20, 20, 128, 128 );
            //            }
            
            // init
            view = SKViewInit( inTag, inRect );
            
            //            if ( inSubclass ) {
            //                view = [[SKViewSC alloc] initWithFrame:rect];
            //            } else {
            //                view = [[SKView alloc] initWithFrame:rect];
            //            }
            
            // default autoresizing
            [(NSView *)view setAutoresizingMask:NSViewMinYMargin];
            
            // tag
            //            [view setTag:inTag];
            
            //#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
            //            // delegate
            //            [view setDelegate:CocoaSKViewDelegateObj()];
            //#endif
            
            // add to superview
            [contentView addSubview:view];
            
            // wndTag
            [view setWndTag:inWndTag];
            
            //            [view release];
        }
        [view setHidden:!show];
    } else {
        CocoaShowAlert( NSWarningAlertStyle, @"Window error.", [NSString stringWithFormat:@"Cocoa window %ld does not exist.",(long)inWndTag] );
    }
}





#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
#pragma mark - SKWarpGeometry
///////////////////////////////////////////////////
//              SKWarpGeometry                   //
///////////////////////////////////////////////////





#pragma mark - SKWarpGeometryGrid
///////////////////////////////////////////////////
//            SKWarpGeometryGrid                 //
///////////////////////////////////////////////////
// Create
SKWarpGeometryGridRef SKWarpGeometryInit( void )
{ return [SKWarpGeometryGrid grid]; }

SKWarpGeometryGridRef SKWarpGeometryGridWithColumns( NSInteger columns, NSInteger rows )
{ return [SKWarpGeometryGrid gridWithColumns:columns rows:rows]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
SKWarpGeometryGridRef SKWarpGeometryGridWithSourcePositions( NSInteger columns, NSInteger rows, vector_float2 *sourcePositions, vector_float2 *destPositions )
{ return [SKWarpGeometryGrid gridWithColumns:columns rows:rows sourcePositions:sourcePositions destPositions:destPositions]; }
#endif

SKWarpGeometryGridRef SKWarpGeometryGridWithCoder( CoderRef coder )
{ return [[[SKWarpGeometryGrid alloc] initWithCoder:coder] autorelease]; }

// Size
NSInteger SKWarpGeometryNumberOfColumns( SKWarpGeometryGridRef ref )
{ return [ref numberOfColumns]; }

NSInteger SKWarpGeometryNumberOfRows( SKWarpGeometryGridRef ref )
{ return [ref numberOfRows]; }

NSInteger SKWarpGeometryVertexCount( SKWarpGeometryGridRef ref )
{ return [ref vertexCount]; }

// Vertices
vector_float2 SKWarpGeometryDestPositionAtIndex( SKWarpGeometryGridRef ref, NSInteger index )
{ return [ref destPositionAtIndex:index]; }

vector_float2 SKWarpGeometrySourcePositionAtIndex( SKWarpGeometryGridRef ref, NSInteger index )
{ return [ref sourcePositionAtIndex:index]; }

SKWarpGeometryGridRef SKWarpGeometryByRepacingDestPositions( SKWarpGeometryGridRef ref, vector_float2 *destPositions )
{ return [ref gridByReplacingDestPositions:destPositions]; }

SKWarpGeometryGridRef SKWarpGeometryByRepacingSourcePositions( SKWarpGeometryGridRef ref, vector_float2 *sourcePositions )
{ return [ref gridByReplacingSourcePositions:sourcePositions]; }
#endif






#endif// #ifdef SK_EXPORT
#endif// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
#endif// #if __LP64__

