/*
 CocoaUI_UserNotifications.h
 
 Bernie Wylde 20181027
 */

#import <AppKit/AppKit.h>
#import <UserNotifications/UserNotifications.h>
#import "CocoaUI.h"

#if __LP64__
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400

typedef UNNotification *UNNotificationRef;
typedef UNNotificationAction *UNNotificationActionRef;
typedef UNTextInputNotificationAction *UNTextInputNotificationActionRef;
typedef UNNotificationAttachment *UNNotificationAttachmentRef;
typedef UNNotificationCategory *UNNotificationCategoryRef;
typedef UNNotificationContent *UNNotificationContentRef;
typedef UNMutableNotificationContent *UNMutableNotificationContentRef;
typedef UNNotificationRequest *UNNotificationRequestRef;
typedef UNNotificationResponse *UNNotificationResponseRef;
typedef UNTextInputNotificationResponse *UNTextInputNotificationResponseRef;
typedef UNNotificationServiceExtension *UNNotificationServiceExtensionRef;
typedef UNNotificationSettings *UNNotificationSettingsRef;
typedef UNNotificationSound *UNNotificationSoundRef;
typedef UNNotificationTrigger *UNNotificationTriggerRef;
typedef UNPushNotificationTrigger *UNPushNotificationTriggerRef;
typedef UNTimeIntervalNotificationTrigger *UNTimeIntervalNotificationTriggerRef;
typedef UNCalendarNotificationTrigger *UNCalendarNotificationTriggerRef;
typedef UNUserNotificationCenter *UNUserNotificationCenterRef;



#pragma mark - UNUserNotificationCenterDelegate
///////////////////////////////////////////////////
//       UNUserNotificationCenterDelegate        //
///////////////////////////////////////////////////
@interface CocoaUNUserNotificationCenterDelegate : NSObject <UNUserNotificationCenterDelegate>
@end

CocoaUNUserNotificationCenterDelegate *CocoaUNUserNotificationCenterDelegateGet( void );







#pragma mark - UNNotification
///////////////////////////////////////////////////
//                UNNotification                 //
///////////////////////////////////////////////////
// Details
UNNotificationRequestRef UNNotificationrequest( UNNotificationRef ref );
CFDateRef UNNotificationDate( UNNotificationRef ref );





#pragma mark - UNCalendarNotificationTrigger
///////////////////////////////////////////////////
//        UNCalendarNotificationTrigger          //
///////////////////////////////////////////////////
// Create
UNCalendarNotificationTriggerRef UNCalendarNotificationTriggerWithDateMatchingComponents( DateComponentsRef components, Boolean repeats );

// Info
CFDateRef UNCalendarNotificationTriggerNextTriggerDate( UNCalendarNotificationTriggerRef ref );
DateComponentsRef UNCalendarNotificationTriggerDateComponents( UNCalendarNotificationTriggerRef ref );




#pragma mark - UNMutableNotificationContent
///////////////////////////////////////////////////
//      UNMutableNotificationContent              //
///////////////////////////////////////////////////
// Init
UNMutableNotificationContentRef UNMutableNotificationContentInit( void );

// Data
void UNMutableNotificationContentSetTitle( UNMutableNotificationContentRef ref, CFStringRef title );
void UNMutableNotificationContentSetSubtitle( UNMutableNotificationContentRef ref, CFStringRef subtitle );
void UNMutableNotificationContentSetBody( UNMutableNotificationContentRef ref, CFStringRef body );
void UNMutableNotificationContentSetBadge( UNMutableNotificationContentRef ref, CFNumberRef badge );
void UNMutableNotificationContentSetSound( UNMutableNotificationContentRef ref, UNNotificationSoundRef snd );
void UNMutableNotificationContentSetUserInfo( UNMutableNotificationContentRef ref, CFDictionaryRef userInfo );
void UNMutableNotificationContentSetAttachments( UNMutableNotificationContentRef ref, CFArrayRef attachments );

// Summary
void UNMutableNotificationContentSetSummaryArgument( UNMutableNotificationContentRef ref, CFStringRef argument );
void UNMutableNotificationContentSetSummaryArgumentCount( UNMutableNotificationContentRef ref, NSUInteger count );

// Content
void UNMutableNotificationContentSetCategoryIdentifier( UNMutableNotificationContentRef ref, CFStringRef identifier );
void UNMutableNotificationContentSetThreadIdentifier( UNMutableNotificationContentRef ref, CFStringRef identifier );




#pragma mark - UNTimeIntervalNotificationTrigger
///////////////////////////////////////////////////
//   UNTimeIntervalNotificationTrigger          //
///////////////////////////////////////////////////
UNTimeIntervalNotificationTriggerRef UNTimeIntervalNotificationTriggerWithTimeInterval( CFTimeInterval ti, Boolean repeats );
CFDateRef UNTimeIntervalNotificationTriggerNextTriggerDate( UNTimeIntervalNotificationTriggerRef ref );
CFTimeInterval UNTimeIntervalNotificationTriggerTimeInterval( UNTimeIntervalNotificationTriggerRef ref );





#pragma mark - UNNotificationCategory
///////////////////////////////////////////////////
//          UNNotificationCategory               //
///////////////////////////////////////////////////
// Create
UNNotificationCategoryRef UNNotificationCategoryWithIdentifier( CFStringRef identifier, CFArrayRef actions, CFArrayRef intentIdentifiers, UNNotificationCategoryOptions options );
UNNotificationCategoryRef UNNotificationCategoryWithHiddenPreviews( CFStringRef identifier, CFArrayRef actions, CFArrayRef intentIdentifiers, CFStringRef hiddenPreviews, UNNotificationCategoryOptions options );
UNNotificationCategoryRef UNNotificationCategoryWithSummaryFormat( CFStringRef identifier, CFArrayRef actions, CFArrayRef intentIdentifiers, CFStringRef hiddenPreviews, CFStringRef summaryFormat, UNNotificationCategoryOptions options );

// Info
CFStringRef UNNotificationCategoryIdentifier( UNNotificationCategoryRef ref );
CFArrayRef UNNotificationCategoryActions( UNNotificationCategoryRef ref );
CFArrayRef UNNotificationCategoryIntentIdentifiers( UNNotificationCategoryRef ref );
CFStringRef UNNotificationCategoryHiddenPreviewsBodyPlaceholder( UNNotificationCategoryRef ref );
CFStringRef UNNotificationCategorySummaryFormat( UNNotificationCategoryRef ref );

// Options
UNNotificationCategoryOptions UNNotificationCategoryoptions( UNNotificationCategoryRef ref );



#pragma mark - UNNotificationRequest
///////////////////////////////////////////////////
//          UNNotificationRequest                //
///////////////////////////////////////////////////
// Create
UNNotificationRequestRef UNNotificationRequestWithIdentifier( CFStringRef identifier, UNNotificationContentRef content, UNNotificationTriggerRef trigger );

// Details
CFStringRef UNNotificationRequestIdentifier( UNNotificationRequestRef ref );
UNNotificationContentRef UNNotificationRequestContent( UNNotificationRequestRef ref );
UNNotificationTriggerRef UNNotificationRequestTrigger( UNNotificationRequestRef ref );



#pragma mark - UNNotificationResponse
///////////////////////////////////////////////////
//         UNNotificationResponse                //
///////////////////////////////////////////////////
// Info
CFStringRef UNNotificationResponseActionIdentifier( UNNotificationResponseRef ref );
UNNotificationRef UNNotificationResponseNotification( UNNotificationResponseRef ref );





#pragma mark - UNNotificationServiceExtension
///////////////////////////////////////////////////
//      UNNotificationServiceExtension           //
///////////////////////////////////////////////////
typedef void (*UNNotificationServiceExtensionNotificationHandlerType)(UNNotificationContent*,void*);

// Processing notifications
void UNNotificationServiceExtensionDidReceiveNotificationRequest( UNNotificationServiceExtensionRef ref, UNNotificationRequestRef request, UNNotificationServiceExtensionNotificationHandlerType handler, void *userData );




#pragma mark - UNNotificationSettings
///////////////////////////////////////////////////
//              UNNotificationSettings           //
///////////////////////////////////////////////////
// Authorization status
UNAuthorizationStatus UNNotificationSettingsAuthorizationStatus( UNNotificationSettingsRef ref );

// Device-specific settings
UNNotificationSetting UNNotificationSettingsNotificationCenterSetting( UNNotificationSettingsRef ref );
UNNotificationSetting UNNotificationSettingsLockScreenSetting( UNNotificationSettingsRef ref );
UNNotificationSetting UNNotificationSettingsAlertSetting( UNNotificationSettingsRef ref );
UNNotificationSetting UNNotificationSettingsBadgeSetting( UNNotificationSettingsRef ref );
UNNotificationSetting UNNotificationSettingsSoundSetting( UNNotificationSettingsRef ref );
UNNotificationSetting UNNotificationSettingsCriticalAlertSetting( UNNotificationSettingsRef ref );

// Interface settings
UNAlertStyle UNNotificationSettingsAlertStyle( UNNotificationSettingsRef ref );
UNShowPreviewsSetting UNNotificationSettingsShowPreviewsSetting( UNNotificationSettingsRef ref );
Boolean UNNotificationSettingsProvidesAppNotificationSettings( UNNotificationSettingsRef ref );




#pragma mark - UNNotificationSound
///////////////////////////////////////////////////
//              UNNotificationSound              //
///////////////////////////////////////////////////
// Notification sounds
UNNotificationSoundRef UNNotificationSoundDefault( void );
UNNotificationSoundRef UNNotificationSoundNamed( CFStringRef name );

// Critical sounds
UNNotificationSoundRef UNNotificationSoundDefaultCritical( void );
UNNotificationSoundRef UNNotificationSoundDefaultCriticalWithAudioVolume( float volume );
UNNotificationSoundRef UNNotificationSoundCriticalNamed( CFStringRef name );
UNNotificationSoundRef UNNotificationSoundCriticalNamedWithAudioVolume( CFStringRef name, float volume );



#pragma mark - UNUserNotificationCenter
///////////////////////////////////////////////////
//         UNUserNotificationCenter              //
///////////////////////////////////////////////////
typedef void (*UNNotificationCenterRequestAuthorizationCompletionHandlerType)(BOOL,NSError*,void*);
typedef void (*UNNotificationCenterGetNotificationSettingsCompletionHandlerType)(UNNotificationSettings*,void*);
typedef void (*UNNotificationCenterGetNotificationCategoriesCompletionHandlerType)(NSSet*,void*);
typedef void (*UNNotificationCenterAddNotificationRequestCompletionHandlerType)(NSError*,void*);
typedef void (*UNNotificationCenterGetPendingNotificationRequestsCompletionHandlerType)(NSArray*,void*);
typedef void (*UNNotificationCenterGetDeliveredNotificationsCompletionHandlerType)(NSArray*,void*);

// Get
UNUserNotificationCenterRef UNUserNotificationCenterCurrent( void );

// Settings and authorization
void UNUserNotificationCenterRequestAuthorization( UNUserNotificationCenterRef ref, UNAuthorizationOptions options, UNNotificationCenterRequestAuthorizationCompletionHandlerType completionHandler, void *userData );
void UNUserNotificationCenterGetNotificationSettings( UNUserNotificationCenterRef ref, UNNotificationCenterGetNotificationSettingsCompletionHandlerType completionHandler, void *userData );
Boolean UNUserNotificationCenterSupportsContentExtensions( UNUserNotificationCenterRef ref );

// Registering categories
void UNUserNotificationCenterSetNotificationCategories( UNUserNotificationCenterRef ref, CFSetRef categories );
void UNUserNotificationCenterGetNotificationCategories( UNUserNotificationCenterRef ref, UNNotificationCenterGetNotificationCategoriesCompletionHandlerType completionHandler, void *userData );

// Scheduling and canceling
void UNUserNotificationCenterAddNotificationRequest( UNUserNotificationCenterRef ref, UNNotificationRequestRef request, UNNotificationCenterAddNotificationRequestCompletionHandlerType completionHandler, void *userData );
void UNUserNotificationCenterGetPendingNotificationRequests( UNUserNotificationCenterRef ref, UNNotificationCenterGetPendingNotificationRequestsCompletionHandlerType completionHandler, void *userData );
void UNUserNotificationCenterRemovePendingNotificationRequests( UNUserNotificationCenterRef ref, CFArrayRef identifiers );
void UNUserNotificationCenterRemoveAllPendingNotificationRequests( UNUserNotificationCenterRef ref );

// Delivered notifications
void UNUserNotificationCenterGetDeliveredNotifications( UNUserNotificationCenterRef ref, UNNotificationCenterGetDeliveredNotificationsCompletionHandlerType completionHandler, void *userData );
void UNUserNotificationCenterRemoveDeliveredNotifications( UNUserNotificationCenterRef ref, CFArrayRef identifiers );
void UNUserNotificationCenterRemoveAllDeliveredNotifications( UNUserNotificationCenterRef ref );




#endif// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
#endif// #if __LP64__
