/*
 WebKit.h
 
 Bernie Wylde 20190221
 */

#import <WebKit/WebKit.h>
#import "CocoaUI.h"

#if __LP64__
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#ifdef WK_API_ENABLED

@interface WebViewSC : WKWebView
@end


typedef WKBackForwardList *WKBackForwardListRef;
typedef WKBackForwardListItem *WKBackForwardListItemRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
typedef WKContentRuleList *WKContentRuleListRef;
#endif

typedef WKFrameInfo *WKFrameInfoRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
typedef WKHTTPCookieStore *WKHTTPCookieStoreRef;
#endif

typedef WKNavigation *WKNavigationRef;
typedef WKNavigationAction *WKNavigationActionRef;
typedef WKNavigationResponse *WKNavigationResponseRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
typedef WKOpenPanelParameters *WKOpenPanelParametersRef;
#endif

typedef WKPreferences *WKPreferencesRef;
typedef WKProcessPool *WKProcessPoolRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
typedef WKSecurityOrigin *WKSecurityOriginRef;
#endif

typedef WKScriptMessage *WKScriptMessageRef;
typedef id<WKScriptMessageHandler> WKScriptMessageHandlerRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
typedef WKSnapshotConfiguration *WKSnapshotConfigurationRef;
typedef id<WKURLSchemeHandler> WKURLSchemeHandlerRef;
#endif

typedef WKUserContentController *WKUserContentControllerRef;
typedef WKUserScript *WKUserScriptRef;

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
typedef WKWebsiteDataRecord *WKWebsiteDataRecordRef;
typedef WKWebsiteDataStore *WKWebsiteDataStoreRef;
#endif

typedef WKWebView *WKWebViewRef;
typedef WKWebViewConfiguration *WKWebViewConfigurationRef;
typedef WKWindowFeatures *WKWindowFeaturesRef;


#pragma mark - WKBackForwardList
///////////////////////////////////////////////////
//             WKBackForwardList                 //
///////////////////////////////////////////////////
// List items
WKBackForwardListItemRef WKBackForwardListBackItem( WKBackForwardListRef ref );
WKBackForwardListItemRef WKBackForwardListCurrentItem( WKBackForwardListRef ref );
WKBackForwardListItemRef WKBackForwardListForwardItem( WKBackForwardListRef ref );
WKBackForwardListItemRef WKBackForwardListItemAtIndex( WKBackForwardListRef ref, NSInteger index );

// Sublists
CFArrayRef WKBackForwardListBackList( WKBackForwardListRef ref );
CFArrayRef WKBackForwardListForwardList( WKBackForwardListRef ref );


#pragma mark - WKBackForwardListItem
///////////////////////////////////////////////////
//           WKBackForwardListitem               //
///////////////////////////////////////////////////
// Info
CFURLRef WKBackForwardListItemURL( WKBackForwardListItemRef ref );
CFURLRef WKBackForwardListItemInitialURL( WKBackForwardListItemRef ref );
CFStringRef WKBackForwardListItemTitle( WKBackForwardListItemRef ref );


#pragma mark - WKContentRuleList
///////////////////////////////////////////////////
//             WKContentRuleList                 //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
// Instance properties
CFStringRef WKContentRuleListIdentifier( WKContentRuleListRef ref );
#endif


#pragma mark - WKFrameInfo
///////////////////////////////////////////////////
//                 WKFrameInfo                   //
///////////////////////////////////////////////////
// Info
Boolean WKFrameInfoIsMainFrame( WKFrameInfoRef ref );
URLRequestRef WKFrameInfoRequest( WKFrameInfoRef ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
WKSecurityOriginRef WKFrameInfoSecurityOrigin( WKFrameInfoRef ref );
#endif

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
WKWebViewRef WKFrameInfoWebView( WKFrameInfoRef ref );
#endif


#pragma mark - WKHTTPCookieStore
///////////////////////////////////////////////////
//             WKHTTPCookieStore                 //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300

#endif


#pragma mark - WKNavigation
///////////////////////////////////////////////////
//                 WKNavigation                  //
///////////////////////////////////////////////////


#pragma mark - WKNavigationAction
///////////////////////////////////////////////////
//              WKNavigationAction               //
///////////////////////////////////////////////////
// Info
URLRequestRef WKNavigationActionRequest( WKNavigationActionRef ref );
WKFrameInfoRef WKNavigationActionSourceFrame( WKNavigationActionRef ref );
WKFrameInfoRef WKNavigationActionTargetFrame( WKNavigationActionRef ref );

// User actions
NSInteger WKNavigationActionButtonNumber( WKNavigationActionRef ref );
NSEventModifierFlags WKNavigationActionModifierFlags( WKNavigationActionRef ref );
WKNavigationType WKNavigationActionNavigationType( WKNavigationActionRef ref );


#pragma mark - WKNavigationDelegate
///////////////////////////////////////////////////
//             WKNavigationDelegate              //
///////////////////////////////////////////////////
// dialog events
enum {
    webViewDidCommitNavigation = 3400,
    webViewDidStartProvisionalNavigation,
    webViewDidReceiveServerRedirectForProvisionalNavigation,
    webViewDidReceiveAuthenticationChallenge,
    webViewDidFailNavigation,
    webViewDidFailProvisionalNavigation,
    webViewDidFinishNavigation,
    webViewWebContentProcessDidTerminate,
    webViewDecidePolicyForNavigationAction
};

@interface CocoaWKNavigationDelegate : NSObject <WKNavigationDelegate>
@end

CocoaWKNavigationDelegate *CocoaWKNavigationDelegateObj( void );


#pragma mark - WKNavigationResponse
///////////////////////////////////////////////////
//             WKNavigationResponse              //
///////////////////////////////////////////////////
// Info
Boolean WKNavigationResponseCanShowMIMEType( WKNavigationResponseRef ref );
Boolean WKNavigationResponseIsForMainFrame( WKNavigationResponseRef ref );
URLResponseRef WKNavigationResponseResponse( WKNavigationResponseRef ref );


#pragma mark - WKOpenPanelParameters
///////////////////////////////////////////////////
//            WKOpenPanelParameters              //
///////////////////////////////////////////////////
// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean WKOpenPanelParametersAllowsMultipleSelection( WKOpenPanelParametersRef ref );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101304 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101304
Boolean WKOpenPanelParametersAllowsDirectories( WKOpenPanelParametersRef ref );
#endif


#pragma mark - WKPreferences
///////////////////////////////////////////////////
//                 WKPreferences                 //
///////////////////////////////////////////////////
// Rendering
CGFloat WKPreferencesMinimumFontSize( WKPreferencesRef ref );
void WKPreferencesSetMinimumFontSize( WKPreferencesRef ref, CGFloat size );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101203
Boolean WKPreferencesTabFocusesLinks( WKPreferencesRef ref );
void WKPreferencesSetTabFocusesLinks( WKPreferencesRef ref, Boolean flag );
#endif

// Java and JavaScript
Boolean WKPreferencesJavaEnabled( WKPreferencesRef ref );
void WKPreferencesSetJavaEnabled( WKPreferencesRef ref, Boolean flag );
Boolean WKPreferencesJavaScriptCanOpenWindowsAutomatically( WKPreferencesRef ref );
void WKPreferencesSetJavaScriptCanOpenWindowsAutomatically( WKPreferencesRef ref, Boolean flag );
Boolean WKPreferencesJavaScriptEnabled( WKPreferencesRef ref );
void WKPreferencesSetJavaScriptEnabled( WKPreferencesRef ref, Boolean flag );
Boolean WKPreferencesPlugInsEnabled( WKPreferencesRef ref );
void WKPreferencesSetPlugInsEnabled( WKPreferencesRef ref, Boolean flag );


#pragma mark - WKProcessPool
///////////////////////////////////////////////////
//                 WKProcessPool                 //
///////////////////////////////////////////////////


#pragma mark - WKScriptMessage
///////////////////////////////////////////////////
//               WKScriptMessage                 //
///////////////////////////////////////////////////
// Info
CFTypeRef WKScriptMessageBody( WKScriptMessageRef ref );
WKFrameInfoRef WKScriptMessageFrameInfo( WKScriptMessageRef ref );
CFStringRef WKScriptMessageName( WKScriptMessageRef ref );
WKWebViewRef WKScriptMessageWebView( WKScriptMessageRef ref );


#pragma mark - WKSecurityOrigin
///////////////////////////////////////////////////
//             WKSecurityOrigin                  //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
// Origins
CFStringRef WKSecurityOriginHost( WKSecurityOriginRef ref );
NSInteger WKSecurityOriginPort( WKSecurityOriginRef ref );
CFStringRef WKSecurityOriginProtocol( WKSecurityOriginRef ref );
#endif


#pragma mark - WKSnapshotConfiguration
///////////////////////////////////////////////////
//          WKSnapshotConfiguration              //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
// Init
WKSnapshotConfigurationRef WKSnapshotConfigurationWithRect( CGRect r, CFNumberRef wdth );

// Instance properties
CGRect WKSnapshotConfigurationRect( WKSnapshotConfigurationRef ref );
void WKSnapshotConfigurationSetRect( WKSnapshotConfigurationRef ref, CGRect r );
CFNumberRef WKSnapshotConfigurationSnapshotWidth( WKSnapshotConfigurationRef ref );
void WKSnapshotConfigurationSetSnapshotWidth( WKSnapshotConfigurationRef ref, CFNumberRef wdth );
#endif


#pragma mark - WKUIDelegate
///////////////////////////////////////////////////
//                WKUIDelegate                   //
///////////////////////////////////////////////////
// dialog events
enum {
    webViewCreateWebViewWithConfiguration = 3500,
    webViewRunJavaScriptAlertPanel,
    webViewRunJavaScriptConfirmPanel,
    webViewRunJavaScriptTextInputPanel,
    webViewDidClose,// macOS 10.11+
    webViewRunOpenPanel// macOS 10.12+
};

@interface CocoaWKUIDelegate : NSObject <WKUIDelegate>
@end

CocoaWKUIDelegate *CocoaWKUIDelegateObj( void );


#pragma mark - WKUserContentController
///////////////////////////////////////////////////
//         WKUserContentController               //
///////////////////////////////////////////////////
// Message handlers
void WKUserContentControllerAddScriptMessageHandler( WKUserContentControllerRef ref, WKScriptMessageHandlerRef handler, CFStringRef name );

// User scripts
void WKUserContentControllerAddUserScript( WKUserContentControllerRef ref, WKUserScriptRef script );
void WKUserContentControllerRemoveAllUserScripts( WKUserContentControllerRef ref );
void WKUserContentControllerRemoveScriptMessageHandlerForName( WKUserContentControllerRef ref, CFStringRef name );
CFArrayRef WKUserContentControllerUserScripts( WKUserContentControllerRef ref );

// Content rules
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void WKUserContentControllerAddContentRuleList( WKUserContentControllerRef ref, WKContentRuleListRef ruleList );
void WKUserContentControllerRemoveContentRuleList( WKUserContentControllerRef ref, WKContentRuleListRef ruleList );
void WKUserContentControllerRemoveAllContentRuleLists( WKUserContentControllerRef ref );
#endif

#pragma mark - WKUserScript
///////////////////////////////////////////////////
//              WKUserScript                     //
///////////////////////////////////////////////////
// Init
WKUserScriptRef WKUserScriptWithSource( CFStringRef source, WKUserScriptInjectionTime injectionTime, Boolean forMainFrameOnly );

// Info
CFStringRef WKUserScriptSource( WKUserScriptRef ref );
WKUserScriptInjectionTime wkUserScriptInjectionTime( WKUserScriptRef ref );
Boolean WKUserScriptIsForMainFrameOnly( WKUserScriptRef ref );


#pragma mark - WKWebsiteDataRecord
///////////////////////////////////////////////////
//              WKWebsiteDataRecord              //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
// Inspect
CFSetRef WKWebsiteDataRecordDataTypes( WKWebsiteDataRecordRef ref );
CFStringRef WKWebsiteDataRecordDisplayName( WKWebsiteDataRecordRef ref );
#endif


#pragma mark - WKWebsiteDataStore
///////////////////////////////////////////////////
//              WKWebsiteDataStore               //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
// Retrieve
WKWebsiteDataStoreRef WKWebsiteDataStoreDefaultDataSore( void );
WKWebsiteDataStoreRef WKWebsiteDataStoreNonPersistentDataStore( void );

// Properties
Boolean WKWebsiteDataStorePersistent( WKWebsiteDataStoreRef ref );

// Cookie store
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
WKHTTPCookieStoreRef WKWebsiteDataStoreHTTPCookieStore( WKWebsiteDataStoreRef ref );
#endif

// Retrieve data
CFSetRef WKWebsiteDataStoreAllWebsiteDataTypes( void );
#endif


#pragma mark - WKWebView
///////////////////////////////////////////////////
//                 WKWebView                     //
///////////////////////////////////////////////////
// callbacks
typedef void (*WKWebViewEvaluateJavaScriptCompletionHandlerType)(NSInteger,CFTypeRef,NSError*,void*);
typedef void (*WKWebViewTakeSnapshotCompletionHandlerType)(NSInteger,NSImage*,NSError*,void*);

@interface WKWebView (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

WKWebViewRef WKWebViewWithTag( NSInteger tag );
BOOL WKWebViewExists( NSInteger tag );

// Can load content
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean WKWebViewHandlesURLScheme( CFStringRef scheme );
#endif

// Init
WKWebViewConfigurationRef wkWebViewConfiguration( NSInteger tag );

// Info
CFStringRef WKWebViewTitle( NSInteger tag );
CFURLRef WKWebViewURL( NSInteger tag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFStringRef WKWebViewCustomUserAgent( NSInteger tag );
#endif

// Delegates

// Loading content
double WKWebViewEstimatedProgress( NSInteger tag );
Boolean WKWebViewHasOnlySecureContent( NSInteger tag );
WKNavigationRef WKWebViewLoadHTMLString( NSInteger tag, CFStringRef string, CFURLRef baseURL );
Boolean WKWebViewIsLoading( NSInteger tag );
WKNavigationRef WKWebViewReload( NSInteger tag );
WKNavigationRef WKWebViewReloadFromOrigin( NSInteger tag );
void WKWebViewStopLoading( NSInteger tag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
WKNavigationRef WKWebViewLoadData( NSInteger tag, CFDataRef dta, CFStringRef mimeType, CFStringRef charEncodingName, CFURLRef baseURL );
WKNavigationRef WKWebViewLoadFileURL( NSInteger tag, CFURLRef fileURL, CFURLRef readAccessURL );
#endif

// Scaling content
Boolean WKWebViewAllowsMagnification( NSInteger tag );
void WKWebViewSetAllowsMagnification( NSInteger tag, Boolean flag );
CGFloat WKWebViewMagnification( NSInteger tag );
void WKWebViewSetMagnification( NSInteger tag, CGFloat magnification );
void WKWebViewSetMagnificationCenteredAtPoint( NSInteger tag, CGFloat magnification, CGPoint pt );

// Navigating
Boolean WKWebViewAllowsBackForwardNavigationGestures( NSInteger tag );
void WKWebViewSetAllowsBackForwardNavigationGestures( NSInteger tag, Boolean flag );
WKBackForwardListRef WKWebViewBackForwardList( NSInteger tag );
Boolean WKWebViewCanGoBack( NSInteger tag );
Boolean WKWebViewCanGoForward( NSInteger tag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean WKWebViewAllowsLinkPreview( NSInteger tag );
void WKWebViewSetAllowsLinkPreview( NSInteger tag, Boolean flag );
#endif
WKNavigationRef WKWebViewGoBack( NSInteger tag );
WKNavigationRef WKWebViewGoForward( NSInteger tag );
WKNavigationRef WKWebViewGoToBackForwardListItem( NSInteger tag, WKBackForwardListItemRef item );
WKNavigationRef WKWebViewLoadRequest( NSInteger tag, URLRequestRef request );

// JavaScript
void WKWebViewEvaluateJavaScript( NSInteger tag, CFStringRef javaScriptString, WKWebViewEvaluateJavaScriptCompletionHandlerType fnAddress, void *userData );

// Snapshots
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void WKWebViewTakeSnapshot( NSInteger tag, WKSnapshotConfigurationRef configuration, WKWebViewTakeSnapshotCompletionHandlerType fnAddress, void *userData );
#endif

// Convenience
WKNavigationRef WKWebViewLoadURLString( NSInteger tag, CFStringRef string );

// Custom
void WKWebViewInstallNavigationDelegate( NSInteger tag );


#pragma mark - WKWebViewConfiguration
///////////////////////////////////////////////////
//           WKWebViewConfiguration              //
///////////////////////////////////////////////////
// Properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFStringRef WKWebViewConfigurationApplicationNameForUserAgent( WKWebViewConfigurationRef ref );
void WKWebViewConfigurationSetApplicationNameForUserAgent( WKWebViewConfigurationRef ref, CFStringRef name );
#endif
WKPreferencesRef WKWebViewConfigurationPreferences( WKWebViewConfigurationRef ref );
void WKWebViewConfigurationSetPreferences( WKWebViewConfigurationRef ref, WKPreferencesRef prefs );
WKProcessPoolRef WKWebViewConfigurationProcessPool( WKWebViewConfigurationRef ref );
void WKWebViewConfigurationSetProcessPool( WKWebViewConfigurationRef ref, WKProcessPoolRef pool );
WKUserContentControllerRef WKWebViewConfigurationUserContentController( WKWebViewConfigurationRef ref );
void WKWebViewConfigurationSetUserContentController( WKWebViewConfigurationRef ref, WKUserContentControllerRef controller );

// Rendering preferences
Boolean WKWebViewConfigurationSuppressesIncrementalRendering( WKWebViewConfigurationRef ref );
void WKWebViewConfigurationSetSuppressesIncrementalRendering( WKWebViewConfigurationRef ref, Boolean flag );

// Media playback preferences
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean WKWebViewConfigurationAllowsAirPlayForMediaPlayback( WKWebViewConfigurationRef ref );
void WKWebViewConfigurationSetAllowsAirPlayForMediaPlayback( WKWebViewConfigurationRef ref, Boolean flag );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
WKAudiovisualMediaTypes WKWebViewConfigurationMediaTypesRequiringUserActionForPlayback( WKWebViewConfigurationRef ref );
#endif

// UI directionality
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
WKUserInterfaceDirectionPolicy WKWebViewConfigurationUserInterfaceDirectionPolicy( WKWebViewConfigurationRef ref );
void WKWebViewConfigurationSetUserInterfaceDirectionPolicy( WKWebViewConfigurationRef ref, WKUserInterfaceDirectionPolicy policy );
#endif

// Add handlers for new URL schemes
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void WKWebViewConfigurationSetURLSchemeHandler( WKWebViewConfigurationRef ref, WKURLSchemeHandlerRef schemeHandler, CFStringRef scheme );
WKURLSchemeHandlerRef WKWebViewConfigurationURLSchemeHandler( WKWebViewConfigurationRef ref, CFStringRef scheme );
#endif


#pragma mark - WKWindowFeatures
///////////////////////////////////////////////////
//              WKWindowFeatures                 //
///////////////////////////////////////////////////
// Position and dimensions
CFNumberRef WKWindowFeaturesAllowsResizing( WKWindowFeaturesRef ref );
CFNumberRef WKWindowFeaturesHeight( WKWindowFeaturesRef ref );
CFNumberRef WKWindowFeaturesWidth( WKWindowFeaturesRef ref );
CFNumberRef WKWindowFeaturesX( WKWindowFeaturesRef ref );
CFNumberRef WKWindowFeaturesY( WKWindowFeaturesRef ref );

// Visibility
CFNumberRef WKWindowFeaturesMenuBarVisibility( WKWindowFeaturesRef ref );
CFNumberRef WKWindowFeaturesStatusBarVisibility( WKWindowFeaturesRef ref );
CFNumberRef WKWindowFeaturesToolbarsVisibility( WKWindowFeaturesRef ref );



// Init
WKWebViewRef WKWebViewInit( NSInteger inTag, CGRect inRect, WKWebViewConfigurationRef inConfig );

// Statement
void CocoaWebView( NSInteger inTag, CGRect inRect, WKWebViewConfigurationRef inConfig, NSInteger inWndTag );

#endif// #ifdef WK_API_ENABLED
#endif// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#endif// #if __LP64__
