/*
 WebKit.m
 
 Bernie Wylde 20190221
 */

#import "WebKit.h"

#if __LP64__
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#ifdef WK_API_ENABLED

@implementation WebViewSC
@end

#pragma mark - WKBackForwardList
///////////////////////////////////////////////////
//             WKBackForwardList                 //
///////////////////////////////////////////////////
// List items
WKBackForwardListItemRef WKBackForwardListBackItem( WKBackForwardListRef ref )
{ return [ref backItem]; }

WKBackForwardListItemRef WKBackForwardListCurrentItem( WKBackForwardListRef ref )
{ return [ref currentItem]; }

WKBackForwardListItemRef WKBackForwardListForwardItem( WKBackForwardListRef ref )
{ return [ref forwardItem]; }

WKBackForwardListItemRef WKBackForwardListItemAtIndex( WKBackForwardListRef ref, NSInteger index )
{ return [ref itemAtIndex:index]; }

// Sublists
CFArrayRef WKBackForwardListBackList( WKBackForwardListRef ref )
{ return (CFArrayRef)[ref backList]; }

CFArrayRef WKBackForwardListForwardList( WKBackForwardListRef ref )
{ return (CFArrayRef)[ref forwardList]; }


#pragma mark - WKBackForwardListItem
///////////////////////////////////////////////////
//           WKBackForwardListitem               //
///////////////////////////////////////////////////
// Info
CFURLRef WKBackForwardListItemURL( WKBackForwardListItemRef ref )
{ return (CFURLRef)[ref URL]; }

CFURLRef WKBackForwardListItemInitialURL( WKBackForwardListItemRef ref )
{ return (CFURLRef)[ref initialURL]; }

CFStringRef WKBackForwardListItemTitle( WKBackForwardListItemRef ref )
{ return (CFStringRef)[ref title]; }


#pragma mark - WKContentRuleList
///////////////////////////////////////////////////
//             WKContentRuleList                 //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
// Instance properties
CFStringRef WKContentRuleListIdentifier( WKContentRuleListRef ref )
{ return (CFStringRef)[ref identifier]; }
#endif


#pragma mark - WKFrameInfo
///////////////////////////////////////////////////
//                 WKFrameInfo                   //
///////////////////////////////////////////////////
// Info
Boolean WKFrameInfoIsMainFrame( WKFrameInfoRef ref )
{ return [ref isMainFrame]; }

URLRequestRef WKFrameInfoRequest( WKFrameInfoRef ref )
{ return [ref request]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
WKSecurityOriginRef WKFrameInfoSecurityOrigin( WKFrameInfoRef ref )
{ return [ref securityOrigin]; }
#endif

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
WKWebViewRef WKFrameInfoWebView( WKFrameInfoRef ref )
{ return [ref webView]; }
#endif


#pragma mark - WKNavigation
///////////////////////////////////////////////////
//                 WKNavigation                  //
///////////////////////////////////////////////////


#pragma mark - WKNavigationAction
///////////////////////////////////////////////////
//              WKNavigationAction               //
///////////////////////////////////////////////////
// Info
URLRequestRef WKNavigationActionRequest( WKNavigationActionRef ref )
{ return [ref request]; }

WKFrameInfoRef WKNavigationActionSourceFrame( WKNavigationActionRef ref )
{ return [ref sourceFrame]; }

WKFrameInfoRef WKNavigationActionTargetFrame( WKNavigationActionRef ref )
{ return [ref targetFrame]; }

// User actions
NSInteger WKNavigationActionButtonNumber( WKNavigationActionRef ref )
{ return [ref buttonNumber]; }

NSEventModifierFlags WKNavigationActionModifierFlags( WKNavigationActionRef ref )
{ return [ref modifierFlags]; }

WKNavigationType WKNavigationActionNavigationType( WKNavigationActionRef ref )
{ return [ref navigationType]; }


#pragma mark - WKNavigationDelegate
///////////////////////////////////////////////////
//             WKNavigationDelegate              //
///////////////////////////////////////////////////
@implementation CocoaWKNavigationDelegate

- (void)webView:(WKWebView *)webView forwardEvent:(NSInteger)ev object:(id)obj {
    CallUserDefinedOnDialogFunction( ev, [webView tag], [webView wndTag], obj );
}


- (void)webView:(WKWebView *)webView didCommitNavigation:(WKNavigation *)navigation {
    [self webView:webView forwardEvent:webViewDidCommitNavigation object:(id)navigation];
}

- (void)webView:(WKWebView *)webView didStartProvisionalNavigation:(WKNavigation *)navigation {
    [self webView:webView forwardEvent:webViewDidStartProvisionalNavigation object:(id)navigation];
}

- (void)webView:(WKWebView *)webView didReceiveServerRedirectForProvisionalNavigation:(WKNavigation *)navigation {
    [self webView:webView forwardEvent:webViewDidReceiveServerRedirectForProvisionalNavigation object:(id)navigation];
}

- (void)webView:(WKWebView *)webView didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge completionHandler:(void (^)(NSURLSessionAuthChallengeDisposition disposition, NSURLCredential *credential))completionHandler {
    // webViewDidReceiveAuthenticationChallenge
}

- (void)webView:(WKWebView *)webView didFailNavigation:(WKNavigation *)navigation withError:(NSError *)error {
    // set error
    [self webView:webView forwardEvent:webViewDidFailNavigation object:(id)navigation];
}

- (void)webView:(WKWebView *)webView didFailProvisionalNavigation:(WKNavigation *)navigation withError:(NSError *)error {
    // set error
    [self webView:webView forwardEvent:webViewDidFailProvisionalNavigation object:(id)navigation];
}

- (void)webView:(WKWebView *)webView didFinishNavigation:(WKNavigation *)navigation {
    [self webView:webView forwardEvent:webViewDidFinishNavigation object:(id)navigation];
}

- (void)webViewWebContentProcessDidTerminate:(WKWebView *)webView {
    [self webView:webView forwardEvent:webViewWebContentProcessDidTerminate object:nil];
}

- (void)webView:(WKWebView *)webView decidePolicyForNavigationAction:(WKNavigationAction *)navigationAction decisionHandler:(void (^)(WKNavigationActionPolicy))decisionHandler {
    // webViewDecidePolicyForNavigationAction
}
@end

CocoaWKNavigationDelegate *CocoaWKNavigationDelegateObj( void )
{
    static CocoaWKNavigationDelegate *sCocoaWKNavigationDelegate = nil;
    if ( !sCocoaWKNavigationDelegate ) sCocoaWKNavigationDelegate = [[CocoaWKNavigationDelegate alloc] init];
    return sCocoaWKNavigationDelegate;
}


#pragma mark - WKNavigationResponse
///////////////////////////////////////////////////
//             WKNavigationResponse              //
///////////////////////////////////////////////////
// Info
Boolean WKNavigationResponseCanShowMIMEType( WKNavigationResponseRef ref )
{ return [ref canShowMIMEType]; }

Boolean WKNavigationResponseIsForMainFrame( WKNavigationResponseRef ref )
{ return [ref isForMainFrame]; }

URLResponseRef WKNavigationResponseResponse( WKNavigationResponseRef ref )
{ return [ref response]; }


#pragma mark - WKOpenPanelParameters
///////////////////////////////////////////////////
//            WKOpenPanelParameters              //
///////////////////////////////////////////////////
// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
Boolean WKOpenPanelParametersAllowsMultipleSelection( WKOpenPanelParametersRef ref )
{ return [ref allowsMultipleSelection]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101304 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101304
Boolean WKOpenPanelParametersAllowsDirectories( WKOpenPanelParametersRef ref )
{ return [ref allowsDirectories]; }
#endif


#pragma mark - WKPreferences
///////////////////////////////////////////////////
//                 WKPreferences                 //
///////////////////////////////////////////////////
// Rendering
CGFloat WKPreferencesMinimumFontSize( WKPreferencesRef ref )
{ return [ref minimumFontSize]; }

void WKPreferencesSetMinimumFontSize( WKPreferencesRef ref, CGFloat size )
{ [ref setMinimumFontSize:size]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101203
Boolean WKPreferencesTabFocusesLinks( WKPreferencesRef ref )
{ return [ref tabFocusesLinks]; }

void WKPreferencesSetTabFocusesLinks( WKPreferencesRef ref, Boolean flag )
{ [ref setTabFocusesLinks:flag]; }
#endif

// Java and JavaScript
Boolean WKPreferencesJavaEnabled( WKPreferencesRef ref )
{ return [ref javaEnabled]; }

void WKPreferencesSetJavaEnabled( WKPreferencesRef ref, Boolean flag )
{ [ref setJavaEnabled:flag]; }

Boolean WKPreferencesJavaScriptCanOpenWindowsAutomatically( WKPreferencesRef ref )
{ return [ref javaScriptCanOpenWindowsAutomatically]; }

void WKPreferencesSetJavaScriptCanOpenWindowsAutomatically( WKPreferencesRef ref, Boolean flag )
{ [ref setJavaScriptCanOpenWindowsAutomatically:flag]; }

Boolean WKPreferencesJavaScriptEnabled( WKPreferencesRef ref )
{ return [ref javaScriptEnabled]; }

void WKPreferencesSetJavaScriptEnabled( WKPreferencesRef ref, Boolean flag )
{ [ref setJavaScriptEnabled:flag]; }

Boolean WKPreferencesPlugInsEnabled( WKPreferencesRef ref )
{ return [ref plugInsEnabled]; }

void WKPreferencesSetPlugInsEnabled( WKPreferencesRef ref, Boolean flag )
{ [ref setPlugInsEnabled:flag]; }


#pragma mark - WKScriptMessage
///////////////////////////////////////////////////
//               WKScriptMessage                 //
///////////////////////////////////////////////////
// Info
CFTypeRef WKScriptMessageBody( WKScriptMessageRef ref )
{ return (CFTypeRef)[ref body]; }

WKFrameInfoRef WKScriptMessageFrameInfo( WKScriptMessageRef ref )
{ return [ref frameInfo]; }

CFStringRef WKScriptMessageName( WKScriptMessageRef ref )
{ return (CFStringRef)[ref name]; }

WKWebViewRef WKScriptMessageWebView( WKScriptMessageRef ref )
{ return [ref webView]; }


#pragma mark - WKSecurityOrigin
///////////////////////////////////////////////////
//             WKSecurityOrigin                  //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
// Origins
CFStringRef WKSecurityOriginHost( WKSecurityOriginRef ref )
{ return (CFStringRef)[ref host]; }

NSInteger WKSecurityOriginPort( WKSecurityOriginRef ref )
{ return [ref port]; }

CFStringRef WKSecurityOriginProtocol( WKSecurityOriginRef ref )
{ return (CFStringRef)[ref protocol]; }
#endif


#pragma mark - WKSnapshotConfiguration
///////////////////////////////////////////////////
//          WKSnapshotConfiguration              //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
// Init
WKSnapshotConfigurationRef WKSnapshotConfigurationWithRect( CGRect r, CFNumberRef wdth )
{
    WKSnapshotConfiguration *ref = [[[WKSnapshotConfiguration alloc] init] autorelease];
    [ref setRect:r];
    if ( wdth ) [ref setSnapshotWidth:(NSNumber *)wdth];
    return ref;
}

// Instance properties
CGRect WKSnapshotConfigurationRect( WKSnapshotConfigurationRef ref )
{ return [ref rect]; }

void WKSnapshotConfigurationSetRect( WKSnapshotConfigurationRef ref, CGRect r )
{ [ref setRect:r]; }

CFNumberRef WKSnapshotConfigurationSnapshotWidth( WKSnapshotConfigurationRef ref )
{ return (CFNumberRef)[ref snapshotWidth]; }

void WKSnapshotConfigurationSetSnapshotWidth( WKSnapshotConfigurationRef ref, CFNumberRef wdth )
{ [ref setSnapshotWidth:(NSNumber *)wdth]; }
#endif


#pragma mark - WKUIDelegate
///////////////////////////////////////////////////
//                WKUIDelegate                   //
///////////////////////////////////////////////////
@implementation CocoaWKUIDelegate

- (void)webView:(WKWebView *)webView forwardEvent:(NSInteger)ev object:(id)obj {
    CallUserDefinedOnDialogFunction( ev, [webView tag], [webView wndTag], obj );
}


- (WKWebView *)webView:(WKWebView *)webView createWebViewWithConfiguration:(WKWebViewConfiguration *)configuration forNavigationAction:(WKNavigationAction *)navigationAction windowFeatures:(WKWindowFeatures *)windowFeatures {
    // set webView to nils
    [self webView:webView forwardEvent:webViewCreateWebViewWithConfiguration object:(id)configuration];
    // get webView and return
    return nil;
}

- (void)webView:(WKWebView *)webView runJavaScriptAlertPanelWithMessage:(NSString *)message initiatedByFrame:(WKFrameInfo *)frame completionHandler:(void (^)(void))completionHandler {
    [self webView:webView forwardEvent:webViewRunJavaScriptAlertPanel object:(id)message];
}

- (void)webView:(WKWebView *)webView runJavaScriptConfirmPanelWithMessage:(NSString *)message initiatedByFrame:(WKFrameInfo *)frame completionHandler:(void (^)(BOOL result))completionHandler {
    [self webView:webView forwardEvent:webViewRunJavaScriptConfirmPanel object:(id)message];
}

- (void)webView:(WKWebView *)webView runJavaScriptTextInputPanelWithPrompt:(NSString *)prompt defaultText:(NSString *)defaultText initiatedByFrame:(WKFrameInfo *)frame completionHandler:(void (^)(NSString *result))completionHandler {
    [self webView:webView forwardEvent:webViewRunJavaScriptTextInputPanel object:(id)prompt];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
- (void)webViewDidClose:(WKWebView *)webView {
    [self webView:webView forwardEvent:webViewDidClose object:nil];
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
- (void)webView:(WKWebView *)webView runOpenPanelWithParameters:(WKOpenPanelParameters *)parameters initiatedByFrame:(WKFrameInfo *)frame completionHandler:(void (^)(NSArray<NSURL *> *URLs))completionHandler {
    [self webView:webView forwardEvent:webViewRunOpenPanel object:(id)parameters];
}
#endif
@end

CocoaWKUIDelegate *CocoaWKUIDelegateObj( void )
{
    static CocoaWKUIDelegate *sCocoaWKUIDelegate = nil;
    if ( !sCocoaWKUIDelegate ) sCocoaWKUIDelegate = [[CocoaWKUIDelegate alloc] init];
    return sCocoaWKUIDelegate;
}



#pragma mark - WKUserContentController
///////////////////////////////////////////////////
//         WKUserContentController               //
///////////////////////////////////////////////////
// Message handlers
void WKUserContentControllerAddScriptMessageHandler( WKUserContentControllerRef ref, WKScriptMessageHandlerRef handler, CFStringRef name )
{ [ref addScriptMessageHandler:handler name:(NSString *)name]; }

// User scripts
void WKUserContentControllerAddUserScript( WKUserContentControllerRef ref, WKUserScriptRef script )
{ [ref addUserScript:script]; }

void WKUserContentControllerRemoveAllUserScripts( WKUserContentControllerRef ref )
{ [ref removeAllUserScripts]; }

void WKUserContentControllerRemoveScriptMessageHandlerForName( WKUserContentControllerRef ref, CFStringRef name )
{ [ref removeScriptMessageHandlerForName:(NSString *)name]; }

CFArrayRef WKUserContentControllerUserScripts( WKUserContentControllerRef ref )
{ return (CFArrayRef)[ref userScripts]; }

// Content rules
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void WKUserContentControllerAddContentRuleList( WKUserContentControllerRef ref, WKContentRuleListRef ruleList )
{ [ref addContentRuleList:ruleList]; }

void WKUserContentControllerRemoveContentRuleList( WKUserContentControllerRef ref, WKContentRuleListRef ruleList )
{ [ref removeContentRuleList:ruleList]; }

void WKUserContentControllerRemoveAllContentRuleLists( WKUserContentControllerRef ref )
{ [ref removeAllContentRuleLists]; }
#endif


#pragma mark - WKUserScript
///////////////////////////////////////////////////
//              WKUserScript                     //
///////////////////////////////////////////////////
// Init
WKUserScriptRef WKUserScriptWithSource( CFStringRef source, WKUserScriptInjectionTime injectionTime, Boolean forMainFrameOnly )
{ return [[[WKUserScript alloc] initWithSource:(NSString *)source injectionTime:injectionTime forMainFrameOnly:forMainFrameOnly] autorelease]; }

// Info
CFStringRef WKUserScriptSource( WKUserScriptRef ref )
{ return (CFStringRef)[ref source]; }

WKUserScriptInjectionTime wkUserScriptInjectionTime( WKUserScriptRef ref )
{ return [ref injectionTime]; }

Boolean WKUserScriptIsForMainFrameOnly( WKUserScriptRef ref )
{ return [ref isForMainFrameOnly]; }


#pragma mark - WKWebsiteDataRecord
///////////////////////////////////////////////////
//              WKWebsiteDataRecord              //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
// Inspect
CFSetRef WKWebsiteDataRecordDataTypes( WKWebsiteDataRecordRef ref )
{ return (CFSetRef)[ref dataTypes]; }

CFStringRef WKWebsiteDataRecordDisplayName( WKWebsiteDataRecordRef ref )
{ return (CFStringRef)[ref displayName]; }
#endif


#pragma mark - WKWebsiteDataStore
///////////////////////////////////////////////////
//              WKWebsiteDataStore               //
///////////////////////////////////////////////////
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
// Retrieve
WKWebsiteDataStoreRef WKWebsiteDataStoreDefaultDataSore( void )
{ return [WKWebsiteDataStore defaultDataStore]; }
                                                        
WKWebsiteDataStoreRef WKWebsiteDataStoreNonPersistentDataStore( void )
{ return [WKWebsiteDataStore nonPersistentDataStore]; }

// Properties
Boolean WKWebsiteDataStorePersistent( WKWebsiteDataStoreRef ref )
{ return [ref isPersistent]; }

// Cookie store
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
WKHTTPCookieStoreRef WKWebsiteDataStoreHTTPCookieStore( WKWebsiteDataStoreRef ref )
{ return [ref httpCookieStore]; }
#endif

// Retrieve data
CFSetRef WKWebsiteDataStoreAllWebsiteDataTypes( void );
#endif


#pragma mark - WKWebView
///////////////////////////////////////////////////
//                 WKWebView                     //
///////////////////////////////////////////////////
@implementation WKWebView (Additions)
@dynamic tagNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

WKWebViewRef WKWebViewWithTag( NSInteger tag )
{ return (WKWebView *)CocoaValidateViewKindOfClass( tag, [WKWebView class] ); }

BOOL WKWebViewExists( NSInteger tag )
{ return CocoaViewWithKindOfClassExists( tag, [WKWebView class] ); }


// Can load content
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
Boolean WKWebViewHandlesURLScheme( CFStringRef scheme )
{ return [WKWebView handlesURLScheme:(NSString *)scheme]; }
#endif

// Init
WKWebViewConfigurationRef wkWebViewConfiguration( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view configuration];
    return nil;
}

// Info
CFStringRef WKWebViewTitle( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return (CFStringRef)[view title];
    return nil;
}

CFURLRef WKWebViewURL( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return (CFURLRef)[view URL];
    return nil;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFStringRef WKWebViewCustomUserAgent( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return (CFStringRef)[view customUserAgent];
    return nil;
}
#endif

// Delegates

// Loading content
double WKWebViewEstimatedProgress( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view estimatedProgress];
    return 0.0;
}

Boolean WKWebViewHasOnlySecureContent( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view hasOnlySecureContent];
    return NO;
}

WKNavigationRef WKWebViewLoadHTMLString( NSInteger tag, CFStringRef string, CFURLRef baseURL )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view loadHTMLString:(NSString *)string baseURL:(NSURL *)baseURL];
    return nil;
}

Boolean WKWebViewIsLoading( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view isLoading];
    return NO;
}

WKNavigationRef WKWebViewReload( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view reload];
    return nil;
}

WKNavigationRef WKWebViewReloadFromOrigin( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view reloadFromOrigin];
    return nil;
}

void WKWebViewStopLoading( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view stopLoading];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
WKNavigationRef WKWebViewLoadData( NSInteger tag, CFDataRef dta, CFStringRef mimeType, CFStringRef charEncodingName, CFURLRef baseURL )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view loadData:(NSData *)dta MIMEType:(NSString *)mimeType characterEncodingName:(NSString *)charEncodingName baseURL:(NSURL *)baseURL];
    return nil;
}

WKNavigationRef WKWebViewLoadFileURL( NSInteger tag, CFURLRef fileURL, CFURLRef readAccessURL )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view loadFileURL:(NSURL *)fileURL allowingReadAccessToURL:(NSURL *)readAccessURL];
    return nil;
}
#endif

// Scaling content
Boolean WKWebViewAllowsMagnification( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view allowsMagnification];
    return NO;
}

void WKWebViewSetAllowsMagnification( NSInteger tag, Boolean flag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) [view setAllowsMagnification:flag];
}

CGFloat WKWebViewMagnification( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view magnification];
    return 0.0;
}

void WKWebViewSetMagnification( NSInteger tag, CGFloat magnification )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) [view setMagnification:magnification];
}

void WKWebViewSetMagnificationCenteredAtPoint( NSInteger tag, CGFloat magnification, CGPoint pt )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) [view setMagnification:magnification centeredAtPoint:pt];
}

// Navigating
Boolean WKWebViewAllowsBackForwardNavigationGestures( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view allowsBackForwardNavigationGestures];
    return NO;
}

void WKWebViewSetAllowsBackForwardNavigationGestures( NSInteger tag, Boolean flag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) [view setAllowsBackForwardNavigationGestures:flag];
}
WKBackForwardListRef WKWebViewBackForwardList( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view backForwardList];
    return nil;
}

Boolean WKWebViewCanGoBack( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view canGoBack];
    return NO;
}

Boolean WKWebViewCanGoForward( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view canGoForward];
    return NO;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean WKWebViewAllowsLinkPreview( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view allowsLinkPreview];
    return NO;
}

void WKWebViewSetAllowsLinkPreview( NSInteger tag, Boolean flag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) [view setAllowsLinkPreview:flag];
}

#endif
WKNavigationRef WKWebViewGoBack( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view goBack];
    return nil;
}

WKNavigationRef WKWebViewGoForward( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view goForward];
    return nil;
}

WKNavigationRef WKWebViewGoToBackForwardListItem( NSInteger tag, WKBackForwardListItemRef item )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view goToBackForwardListItem:item];
    return nil;
}

WKNavigationRef WKWebViewLoadRequest( NSInteger tag, URLRequestRef request )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) return [view loadRequest:request];
    return nil;
}

// JavaScript
void WKWebViewEvaluateJavaScript( NSInteger tag, CFStringRef javaScriptString, WKWebViewEvaluateJavaScriptCompletionHandlerType fnAddress, void *userData )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) {
        [view evaluateJavaScript:(NSString *)javaScriptString completionHandler:^(id result, NSError *error) {
            (*fnAddress)( tag, (CFTypeRef)result, error, userData );
        }];
    }
}

// Snapshots
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void WKWebViewTakeSnapshot( NSInteger tag, WKSnapshotConfigurationRef configuration, WKWebViewTakeSnapshotCompletionHandlerType fnAddress, void *userData )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) {
        [view takeSnapshotWithConfiguration:configuration completionHandler:^(NSImage *image, NSError *error) {
            (*fnAddress)( tag, image, error, userData );
        }];
    }
}
#endif
         
// Convenience
WKNavigationRef WKWebViewLoadURLString( NSInteger tag, CFStringRef string )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) {
        NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:(NSString *)string]];
        return [view loadRequest:request];
    }
    return nil;
}

// Custom
void WKWebViewInstallNavigationDelegate( NSInteger tag )
{
    WKWebView *view = WKWebViewWithTag( tag );
    if ( view ) {
        [view setNavigationDelegate:CocoaWKNavigationDelegateObj()];
        [view setUIDelegate:CocoaWKUIDelegateObj()];
    }
}


#pragma mark - WKWebViewConfiguration
///////////////////////////////////////////////////
//           WKWebViewConfiguration              //
///////////////////////////////////////////////////
// Properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFStringRef WKWebViewConfigurationApplicationNameForUserAgent( WKWebViewConfigurationRef ref )
{ return (CFStringRef)[ref applicationNameForUserAgent]; }

void WKWebViewConfigurationSetApplicationNameForUserAgent( WKWebViewConfigurationRef ref, CFStringRef name )
{ [ref setApplicationNameForUserAgent:(NSString *)name]; }
#endif

WKPreferencesRef WKWebViewConfigurationPreferences( WKWebViewConfigurationRef ref )
{ return [ref preferences]; }

void WKWebViewConfigurationSetPreferences( WKWebViewConfigurationRef ref, WKPreferencesRef prefs )
{ [ref setPreferences:prefs]; }

WKProcessPoolRef WKWebViewConfigurationProcessPool( WKWebViewConfigurationRef ref )
{ return [ref processPool]; }

void WKWebViewConfigurationSetProcessPool( WKWebViewConfigurationRef ref, WKProcessPoolRef pool )
{ [ref setProcessPool:pool]; }

WKUserContentControllerRef WKWebViewConfigurationUserContentController( WKWebViewConfigurationRef ref )
{ return [ref userContentController]; }

void WKWebViewConfigurationSetUserContentController( WKWebViewConfigurationRef ref, WKUserContentControllerRef controller )
{ [ref setUserContentController:controller]; }

// Rendering preferences
Boolean WKWebViewConfigurationSuppressesIncrementalRendering( WKWebViewConfigurationRef ref )
{ return [ref suppressesIncrementalRendering]; }

void WKWebViewConfigurationSetSuppressesIncrementalRendering( WKWebViewConfigurationRef ref, Boolean flag )
{ [ref setSuppressesIncrementalRendering:flag]; }

// Media playback preferences
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
Boolean WKWebViewConfigurationAllowsAirPlayForMediaPlayback( WKWebViewConfigurationRef ref )
{ return [ref allowsAirPlayForMediaPlayback]; }

void WKWebViewConfigurationSetAllowsAirPlayForMediaPlayback( WKWebViewConfigurationRef ref, Boolean flag )
{ [ref setAllowsAirPlayForMediaPlayback:flag]; }
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
WKAudiovisualMediaTypes WKWebViewConfigurationMediaTypesRequiringUserActionForPlayback( WKWebViewConfigurationRef ref )
{ return [ref mediaTypesRequiringUserActionForPlayback]; }
#endif

// UI directionality
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
WKUserInterfaceDirectionPolicy WKWebViewConfigurationUserInterfaceDirectionPolicy( WKWebViewConfigurationRef ref )
{ return [ref userInterfaceDirectionPolicy]; }

void WKWebViewConfigurationSetUserInterfaceDirectionPolicy( WKWebViewConfigurationRef ref, WKUserInterfaceDirectionPolicy policy )
{ [ref setUserInterfaceDirectionPolicy:policy]; }
#endif

// Add handlers for new URL schemes
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300
void WKWebViewConfigurationSetURLSchemeHandler( WKWebViewConfigurationRef ref, WKURLSchemeHandlerRef schemeHandler, CFStringRef scheme )
{ [ref setURLSchemeHandler:schemeHandler forURLScheme:(NSString *)scheme]; }

WKURLSchemeHandlerRef WKWebViewConfigurationURLSchemeHandler( WKWebViewConfigurationRef ref, CFStringRef scheme )
{ return [ref urlSchemeHandlerForURLScheme:(NSString *)scheme]; }
#endif


#pragma mark - WKWindowFeatures
///////////////////////////////////////////////////
//              WKWindowFeatures                 //
///////////////////////////////////////////////////
// Position and dimensions
CFNumberRef WKWindowFeaturesAllowsResizing( WKWindowFeaturesRef ref )
{ return (CFNumberRef)[ref allowsResizing]; }

CFNumberRef WKWindowFeaturesHeight( WKWindowFeaturesRef ref )
{ return (CFNumberRef)[ref height]; }

CFNumberRef WKWindowFeaturesWidth( WKWindowFeaturesRef ref )
{ return (CFNumberRef)[ref width]; }

CFNumberRef WKWindowFeaturesX( WKWindowFeaturesRef ref )
{ return (CFNumberRef)[ref x]; }

CFNumberRef WKWindowFeaturesY( WKWindowFeaturesRef ref )
{ return (CFNumberRef)[ref y]; }

// Visibility
CFNumberRef WKWindowFeaturesMenuBarVisibility( WKWindowFeaturesRef ref )
{ return (CFNumberRef)[ref menuBarVisibility]; }

CFNumberRef WKWindowFeaturesStatusBarVisibility( WKWindowFeaturesRef ref )
{ return (CFNumberRef)[ref statusBarVisibility]; }

CFNumberRef WKWindowFeaturesToolbarsVisibility( WKWindowFeaturesRef ref )
{ return (CFNumberRef)[ref toolbarsVisibility]; }


// Init
WKWebViewRef WKWebViewInit( NSInteger inTag, CGRect inRect, WKWebViewConfigurationRef inConfig )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) rect = NSMakeRect( 20, 20, 128, 128 );

    // subclass
    BOOL subclass = IsSubclassFlag();
    
    // configuration
    if ( !inConfig ) inConfig = [[[WKWebViewConfiguration alloc] init] autorelease];

    // init
    WKWebView *view;
    if ( subclass ) {
        view = [[[WebViewSC alloc] initWithFrame:rect configuration:inConfig] autorelease];
    } else {
        view = [[[WKWebView alloc] initWithFrame:rect configuration:inConfig] autorelease];
    }

    // tag
    [view setTag:inTag];
    
    return view;
}

// Statement
void CocoaWebView( NSInteger inTag, CGRect inRect, WKWebViewConfigurationRef inConfig, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CocoaTargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CocoaTargetContentView( w );
        WKWebView *view = CocoaViewSubviewWithTag( contentView, inTag );
        
        if ( !view ) {
            view = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( view ) {    // already exists
            
            // make sure it's same view class
//            if ( !CocoaValidateViewClass( inWndTag, inTag, [view class], [WKWebView class] ) ) {
//                return;
//            }
            
            if ( !CocoaValidateViewClass( inWndTag, inTag, view, [WKWebView class] ) ) {
                return;
            }

            // rect
            if ( !NSIsEmptyRect( rect ) ) [view setFrame:rect];
            
        } else {            // create
            // rect
//            if ( NSIsEmptyRect( rect ) ) {
//                rect = NSMakeRect( 20, 20, 128, 128 );
//            }
            
            // configuration
//            if ( !inConfig ) {
//                inConfig = [[[WKWebViewConfiguration alloc] init] autorelease];
//            }

            // init
//            if ( inSubclass ) {
//                view = [[WebViewSC alloc] initWithFrame:rect configuration:inConfig];
//            } else {
//                view = [[WKWebView alloc] initWithFrame:rect configuration:inConfig];
//            }
            view = WKWebViewInit( inTag, inRect, inConfig );
            
            // default autoresizing
            [(NSView *)view setAutoresizingMask:NSViewMinYMargin];

            // tag
//            [view setTag:inTag];
            
            // add to superview
            [contentView addSubview:view];
            
            // wndTag
            [view setWndTag:inWndTag];
            
//            [view release];
        }
        [view setHidden:!show];
    } else {
        CocoaShowAlert( NSWarningAlertStyle, @"Window error.", [NSString stringWithFormat:@"Cocoa window %ld does not exist.",(long)inWndTag] );
    }    
}

#endif// #ifdef WK_API_ENABLED
#endif// #if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
#endif// #if __LP64__

